#!/bin/sh
# Copyright (C) 2006-2013 OpenWrt.org
# Copyright (C) 2010 Vertical Communications

boot_hook_splice_start() {
	export -n PI_HOOK_SPLICE=1
}

boot_hook_splice_finish() {
	local hook
	for hook in $PI_STACK_LIST; do
		local v; eval "v=\${${hook}_splice:+\$${hook}_splice }$hook"
		export -n "${hook}=${v% }"
		export -n "${hook}_splice="
	done
	export -n PI_HOOK_SPLICE=
}

boot_hook_init() {
	local hook="${1}_hook"
	export -n "PI_STACK_LIST=${PI_STACK_LIST:+$PI_STACK_LIST }$hook"
	export -n "$hook="
}

boot_hook_add() {
	local hook="${1}_hook${PI_HOOK_SPLICE:+_splice}"
	local func="${2}"

	[ -n "$func" ] && {
		local v; eval "v=\$$hook"
		export -n "$hook=${v:+$v }$func"
	}
}

boot_hook_shift() {
	local hook="${1}_hook"
	local rvar="${2}"

	local v; eval "v=\$$hook"
	[ -n "$v" ] && {
		local first="${v%% *}"

		[ "$v" != "${v#* }" ] && \
			export -n "$hook=${v#* }" || \
			export -n "$hook="

		export -n "$rvar=$first"
		return 0
	}

	return 1
}

boot_run_hook() {
	local hook="$1"
	local func

	while boot_hook_shift "$hook" func; do
		local ran; eval "ran=\$PI_RAN_$func"
		[ -n "$ran" ] || {
			export -n "PI_RAN_$func=1"
			echo `cat /proc/uptime` "$1" "$2" $func>/dev/console
			$func "$1" "$2"
		}
	done
}

find_mtd_part() {
    local PART="$(grep "\"$1\"" /proc/mtd | awk -F: '{print $1}')"
    local PREFIX=/dev/mtdblock

    PART="${PART##mtd}"
    [ -d /de/mtdblock ] && PREFIX=/dev/mtdblock/
    echo "${PART:+$PREFIX$PART}"
}

find_mount_jffs2() {
    mkdir -p /tmp/userconfig
    echo "----mtdblock:$(find_mtd_part rootfs_data)----"
    mount "$(find_mtd_part rootfs_data)" /tmp/userconfig -t jffs2 -o sync
    mtd -qq unlock rootfs_data

}

# Linux Kernel 3.18开始主线正式引入overlayfs，并更名为overlay
check_overlayfs_name() {
	local KERNEL_VER_MAJOR=`uname -r |awk -F'.' '{print $1}'`
	local KERNEL_VER_MINOR=`uname -r |awk -F'.' '{print $2}'`
	
	if [[ ${KERNEL_VER_MAJOR} -lt 3 ]]; then
		echo "overlayfs"
	elif [[ ${KERNEL_VER_MAJOR} -eq 3 && ${KERNEL_VER_MAJOR} -lt 18 ]]; then
		echo "overlayfs"
	else
		echo "overlay"
	fi
}

pivot() { # <new_root> <old_root>
	/bin/mount -o noatime,move /proc $1/proc && \
	pivot_root $1 $1$2 && {
		/bin/mount -o noatime,move $2/dev /dev
		/bin/mount -o noatime,move $2/tmp /tmp
		/bin/mount -o noatime,move $2/sys /sys 2>&-
		/bin/mount -o noatime,move $2/overlay /overlay 2>&-
		return 0
	}
}

fopivot() { # <rw_root> <ro_root> <dupe?>
	/bin/mount -o noatime,lowerdir=/,upperdir=$1 -t overlayfs "overlayfs:$1" /mnt
	pivot /mnt $2
}

fopivot_new() { # <rw_root> <work_dir> <ro_root> <dupe?>
	/bin/mount -o noatime,lowerdir=/,upperdir=$1,workdir=$2 -t overlay "overlayfs:$1" /mnt
	pivot /mnt $3
}

ramoverlay() {
	mkdir -p /tmp/root
	/bin/mount -t tmpfs -o noatime,mode=0755 root /tmp/root
	
	local overlayfs_name=`check_overlayfs_name`
	if [[ ${overlayfs_name} == "overlay" ]]; then
		mkdir -p /tmp/root/root /tmp/root/work
		fopivot_new /tmp/root/root /tmp/root/work /rom 1
	else
		fopivot /tmp/root /rom 1
	fi
}

export_device_info_to_file() {
	local DEVICE_INFO_PATH="/tmp/device-info.json"

	device_info=`cat ${DEVICE_INFO_PATH}`

	model_name=`echo ${device_info} | sed 's/,/\n/g' | grep "model_name" | sed 's/:/\n/g'| sed '1d' | cut -d "\"" -f2`
	model_version=`echo ${device_info} | sed 's/,/\n/g' | grep "model_version" | sed 's/:/\n/g'| sed '1d' |sed 's/[}\" ]//g'`
	region=`echo ${device_info} | sed 's/,/\n/g' | grep "region" | sed 's/:/\n/g'| sed '1d' |sed 's/[}\" ]//g'`
	model_full_name=`echo ${device_info} | sed 's/,/\n/g' | grep "model_full_name" | sed 's/:/\n/g'| sed '1d' | cut -d "\"" -f2`
	mic_type=`echo ${device_info} | sed 's/,/\n/g' | grep "mic_type" | sed 's/:/\n/g'| sed '1d' | cut -d "\"" -f2`

	[ ! -n "$model_name" ] && {
		model_name="WVR450G"
	}
	[ ! -n "$model_version" ] && {
		model_version="1"
	}
	[ ! -n "$region" ] && {
		region="UN"
	}
	[ ! -n "$model_full_name" ] && {
		model_full_name="Omada Gigabit VPN Router"
	}

	mkdir -p /tmp/devinfo
	echo "$model_name" > /tmp/devinfo/model_name
	echo "$model_version" > /tmp/devinfo/model_version
	echo "$region" > /tmp/devinfo/region
	echo "$model_full_name" > /tmp/devinfo/model_full_name
	[ ! -z "$mic_type" ] && echo -n "$mic_type" > /tmp/devinfo/mic_type
}

do_load_profile() {
	local DEVICE_INFO_PATH="/tmp/device-info.json"
	local PROFILE_PATH="/etc/profile.d/"
	local PROFILE_NAME="profile"

	/usr/sbin/fwver_init

	if [ ! -e "${DEVICE_INFO_PATH}" ]; then
		echo "device-info.json does not exist, cannot load profile"
		exit 0
	fi

	export_device_info_to_file
	model_name=`cat /tmp/devinfo/model_name | sed 's/ /_/g'`
	region=`cat /tmp/devinfo/region`
	product_version=`cat /tmp/devinfo/model_version | awk -F"." '{print $1"_"substr($2,1,1)}' | sed 's/_0$//'`

	new_profile_name=`echo profile_${model_name}_${region}_v${product_version} | tr [A-Z] [a-z]`
	echo "new_profile_name: ${new_profile_name}" > /dev/console
	if [ ! -e "${PROFILE_PATH}${new_profile_name}" ]; then
		echo "profile: ${PROFILE_PATH}${new_profile_name} not found"
		exit 0
	fi
	mv ${PROFILE_PATH}${PROFILE_NAME}  ${PROFILE_PATH}${PROFILE_NAME}.bak
	cp ${PROFILE_PATH}${new_profile_name} ${PROFILE_PATH}${PROFILE_NAME}
}
