#!/bin/sh

. /lib/functions.sh

#修改配置文件总入口
pre_setting_config() {
    set_wan_poe_mss_auto
    qos_config_sync
	set_lan_ipv6_ra_auto
    pre_network_synchronous_ipv6group
}

qos_config_sync()
{
	lua /lib/qos_ctl/qos_config_sync.lua
}

set_wan_poe_mss_auto()
{
	mss_support=`uci get network.global.mss_support`
	[ "x${mss_support}" == "x1" ] && return
	major_wanipv4=`uci -c /etc/profile.d get profile.@ecs_mod_major_ver[0].wanIpv4`
	minor_wanipv4=`uci -c /etc/profile.d get profile.@ecs_mod_minor_ver[0].wanIpv4`
	version="${major_wanipv4}${minor_wanipv4}"
	[ $version -lt 13 ] && return
	uci set network.global.mss_support="1"
	local wan_nums=`uci get network.global.wan_numbers`
	for wan_id in ${wan_nums}; do
		local poe_enable=`uci get network.wan${wan_id}_poe.t_poe_enable`
		local poe_mss=`uci get network.wan${wan_id}_poe.mss_clamping_type`
		[ "x${poe_enable}" == "x1" ] && [ -z "${poe_mss}" ] && uci set network.wan${wan_id}_poe.mss_clamping_type="auto"
	done
	uci commit network
}

set_lan_ipv6_ra()
{
	ipv6_enable=`uci get vnetwork.$1.ipv6_enable`
	[ "$1" == "vwan" ] && return
	if [ "$ipv6_enable" == "on" ]; then
		ra_enable=`uci get vnetwork.$1.ra_enable`
		[ "$ra_enable" == "on" ] && return
		uci set vnetwork.$1.ra_enable="on"
		uci set vnetwork.$1.ra_preference="medium"
		uci set vnetwork.$1.ra_validLifetime="86400"
		uci set vnetwork.$1.ra_preferredLifetime="14400"
	fi
}

#解决从5.8出货版升级到5.13版本后，5.11新增的lan ipv6 ra相关配置不会自动下发的问题
set_lan_ipv6_ra_auto()
{
	major_network=`uci -c /etc/profile.d get profile.@ecs_mod_major_ver[0].network`
	minor_network=`uci -c /etc/profile.d get profile.@ecs_mod_minor_ver[0].network`
        if [ -z $major_network ] || [ -z $minor_network ]; then
        	return
    	fi
	version="${major_network}${minor_network}"
	[ $version -lt 12 ] && return
	config_load vnetwork
	config_foreach set_lan_ipv6_ra vinterface
	uci commit vnetwork
}

#解决5.8版本升级到5.13版本后，在5.8版本配置的lan network不会同步生成ipv6group的问题
pre_network_synchronous_ipv6group()
{
	. /lib/ipv6group/config.sh
	network_synchronous_ipv6group	
}
