#!/usr/bin/env bash
# Hacky way to make a cmakefile include so we can build the module properly.

echo -n "Checking prereqs..."
for a in cmake ruby perl grep sed python git; do
  if [ -z "$(command -v $a)" ]; then
    echo "ERROR."
    echo "You need ${a} to run this script"
    exit 1
  fi
  echo -n "${a}..."
done
echo "done"

BEAR=$(command -v bear)
echo "Checking for bear..."
if [[ -z ${BEAR} ]]; then
    if [[ ! -d /tmp/Bear ]]; then
        echo "Cloning bear..."
        (cd /tmp && git clone https://github.com/rizsotto/Bear)
    fi
    if [[ ! -f /tmp/bear/bin/bear ]]; then
        echo "Building bear..."
        (cd /tmp/Bear && cmake . -DCMAKE_INSTALL_PREFIX=/tmp/bear && make install)
    fi
    BEAR=/tmp/bear/bin/bear
fi
echo "Using bear = ${BEAR}"

if [[ ! -f /tmp/kernel-grok/generate_cmake ]]; then
    echo "Cloning kernel-grok..."
    (cd /tmp && git clone https://github.com/habemus-papadum/kernel-grok.git)
fi
echo "Building kernel module with bear prefix..."
make clean
${BEAR} make
echo "Creating cmake.include..."
KERNEL_SOURCE_DIR=$(grep '"directory"' < compile_commands.json | head -1 | awk -F '"' '{ print $4 }')
sed 's=file": ".*/speedtest.c"=file": "src/speedtest.c"=' < compile_commands.json \
  | sed 's=file": ".*/speedtest.mod.c"=file": "src/speedtest.mod.c"=' \
  | grep -E -v '"(-c|-MD|-o)"|speedtest-kmod' \
  | sed 's=directory": ".*"=directory": "SRCDIR_REPLACE"=' \
  | sed "s=SRCDIR_REPLACE=$(pwd)=" \
  | sed 's:-I\./:-I${KERNEL_SOURCE_DIR}/:g'\
  | sed 's:"\./:"${KERNEL_SOURCE_DIR}/:g' \
  | sed 's:-I\([a-z]\):-I${KERNEL_SOURCE_DIR}/\1:g'\
  | sed "s:${KERNEL_SOURCE_DIR}:\${KERNEL_SOURCE_DIR}:g"\
  | perl -0777 -pe 's/,(\s*])/\1/igs' > tmp.json
mv tmp.json compile_commands.json
/tmp/kernel-grok/generate_cmake
echo "set(KERNEL_SOURCE_DIR ${KERNEL_SOURCE_DIR})" > cmake.include
grep set_source_files < CMakeLists.txt | sed 's=\.\./*\$=\$=g' |sed 's=\.\.//=/=g' >> cmake.include

rm CMakeLists.txt compile_commands.json
ls -l cmake.include

