// This file is part of Speedtest kernel module
//
// Speedtest kernel module is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// Speedtest kernel module is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Speedtest kernel module. If not, see <https://www.gnu.org/licenses/>.
//
// This file has some helpful logging macros to keep consistent output.
#pragma once
#include <stdarg.h>

#include <linux/linkage.h>
#include <linux/printk.h>

#define __ST_LOG(level, fmt, ...) printk(level "Speedtest " fmt "\n", ##__VA_ARGS__)

#define LOG_EMERG(fmt, ...)   __ST_LOG(KERN_EMERG,   "(emrg) "fmt, ##__VA_ARGS__)
#define LOG_ALERT(fmt, ...)   __ST_LOG(KERN_ALERT,   "(alrt) "fmt, ##__VA_ARGS__)
#define LOG_CRIT(fmt, ...)    __ST_LOG(KERN_CRIT,    "(crit) "fmt, ##__VA_ARGS__)
#define LOG_ERROR(fmt, ...)   __ST_LOG(KERN_ERR,     "(err)  "fmt, ##__VA_ARGS__)
#define LOG_WARNING(fmt, ...) __ST_LOG(KERN_WARNING, "(warn) "fmt, ##__VA_ARGS__)
#define LOG_NOTICE(fmt, ...)  __ST_LOG(KERN_NOTICE,  "(note) " fmt, ##__VA_ARGS__)
#define LOG_INFO(fmt, ...)    __ST_LOG(KERN_INFO,    "(info) " fmt, ##__VA_ARGS__ )

#if ENABLE_DEBUG_LOG || ENABLE_TRACE_LOG
#define LOG_DEBUG(fmt, ...)   __ST_LOG(KERN_DEBUG,   "(dbg)  " fmt, ##__VA_ARGS__)
#else
#define LOG_DEBUG(fmt, ...) do {} while(false)
#endif
#if ENABLE_TRACE_LOG
#define LOG_TRACE(fmt, ...)   __ST_LOG(KERN_DEBUG,   "(trce) " fmt, ##__VA_ARGS__)
#else
#define LOG_TRACE(fmt, ...) do {} while(false)
#endif
