// This file is part of Speedtest kernel module
//
// Speedtest kernel module is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// Speedtest kernel module is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Speedtest kernel module. If not, see <https://www.gnu.org/licenses/>.
//

#pragma once

#include <linux/types.h>
#include <asm/ioctl.h>


/// Version number, check with an ioctl to make sure the client is compatible with the kernel
/// change when there's an API change either in the request or how api's work (ioctl/read)
#define SPEEDTEST_API_LEVEL 101
/// Used for communicating with the module
#define SPEEDTEST_IOCTL_BASE 0xddU
/// Optionally send a command, read bytes.
#define SPEEDTEST_IOCTL_SOCKET_RECV _IOWR(SPEEDTEST_IOCTL_BASE, 1U, unsigned int)
/// Used to retrieve the API version to avoid using the API if it's not compatible.
#define SPEEDTEST_IOCTL_QUERY_API_LEVEL _IOWR(SPEEDTEST_IOCTL_BASE, 2U, unsigned int)

struct socketReceiveRequest {
    /// Request sent at beginning of each receive.
    char commandString[256];
    /// Length of command. If zero, only read rxBytes bytes, and exit. Otherwise the send is repeated
    /// after rxBytes have been read.
    size_t commandLen;
    /// Number of bytes to expect per sent command.
    ssize_t rxBytes;
    /// Flags used for recv - note that MSG_TRUNC is always used.
    int flags;
    /// Block sizes used for recv.
    size_t blockSize;
};

struct schedulerPriority {
    int priority;
    int scheduler; // 0 = normal
};

struct speedtestKernelRequest
{
    // Request details - anonymous union to show that
    // future requests would share the same space.
    union {
        struct socketReceiveRequest recv;
    };
    // File descriptor used for request.
    int fd;
    // Kernel thread scheduler type and priority
    struct schedulerPriority sched;
};
