#!/bin/sh /etc/rc.common
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file     nat.init
# brief
# author   Chen chen
# version  1.0.0
# date     05June15
# histry   arg 1.0.0, 05June15, Chen chen, Create the file.

. /lib/zone/zone_api.sh
. /lib/functions/network.sh

fw_load_vpn()
{
	# load ipsec and l2tp
	iptables -w -N ipsec_forward_rule
	iptables -w -A forwarding_rule -j ipsec_forward_rule

	# used for l2tp and pptp vpn
	iptables -w -N ppp_forward_rule
	iptables -w -A forwarding_rule -j ppp_forward_rule

	# used for gre vpn
	iptables -w -N gre_forward_rule
	iptables -w -A forwarding_rule -j gre_forward_rule

	iptables -w -N local_service_rule
	iptables -w -A local_service_rule -p esp -j ACCEPT
	iptables -w -A input_rule -j local_service_rule

	iptables -w -t mangle -A INPUT -m policy --dir in --pol ipsec --proto ah  -j ipsecmark
	iptables -w -t mangle -A INPUT -m policy --dir in --pol ipsec --proto esp -j ipsecmark

	iptables -w -N zone_vpn_forward
	iptables -w -A forward -j zone_vpn_forward

	iptables -w -t raw -N gre_preservice_rule
	iptables -w -t nat -N gre_service_rule
	iptables -w -t raw -A PREROUTING -j gre_preservice_rule
	iptables -w -t nat -A POSTROUTING -j gre_service_rule

	#used for sslvpn resource acl rules
	iptables  -w  -N sslvpn_resource_acl
    iptables -w -A input_rule  -j sslvpn_resource_acl
    iptables -w -A forwarding_rule   -j sslvpn_resource_acl

	#used for openvpn
	#bind when wan create
	iptables -w -N zone_wan_openvpn

	iptables -w -N openvpn_forwarding_rule
	iptables -w -A forwarding_rule -j openvpn_forwarding_rule

	iptables -w -t nat -N openvpn_prerouting_rule
	iptables -w -t nat -A PREROUTING -j openvpn_prerouting_rule

	iptables -w -t nat -N openvpn_postrouting_rule
	iptables -w -t nat -I POSTROUTING -j openvpn_postrouting_rule

	#used for wireguard
	iptables -w -N wireguard_forwarding_rule
	iptables -w -A forwarding_rule -j wireguard_forwarding_rule

	iptables -w -t nat -N wireguard_prerouting_rule
	iptables -w -t nat -A PREROUTING -j wireguard_prerouting_rule

	iptables -w -t nat -N wireguard_postrouting_rule

	iptables -w -N wireguard_input_rule
	iptables -w -A input_rule -j wireguard_input_rule

	iptables -w -t raw -N wireguard_preservice_rule
	iptables -w -t nat -N wireguard_service_rule
	iptables -w -t raw -A PREROUTING -j wireguard_preservice_rule
	iptables -w -t nat -A POSTROUTING -j wireguard_service_rule

	
	#set forwarding mark for vpn/ip rule: 0x20000
	iptables -w -t mangle -N VPN_FORWARD_RULE
	iptables -w -t mangle -A PREROUTING -j VPN_FORWARD_RULE
	iptables -w -t mangle -N VPN_FORWARD_MARK
	iptables -w -t mangle -N VPN_ROUTING_MARK
	iptables -w -t mangle -A VPN_FORWARD_RULE -j VPN_FORWARD_MARK
	iptables -w -t mangle -A FORWARD -j VPN_ROUTING_MARK
	iptables -w -t mangle -A VPN_FORWARD_RULE -m mark --mark 0x20000/0x20000 -j RETURN

	#set forwarding mark for openvpn/ip rule: 0x20000
	iptables -w -t mangle -N OPENVPN_FORWARD_RULE
	iptables -w -t mangle -A PREROUTING -j OPENVPN_FORWARD_RULE
	iptables -w -t mangle -N OPENVPN_FORWARD_MARK
	iptables -w -t mangle -N OPENVPN_ROUTING_MARK
	iptables -w -t mangle -A OPENVPN_FORWARD_RULE -j OPENVPN_FORWARD_MARK
	iptables -w -t mangle -A FORWARD -j OPENVPN_ROUTING_MARK
	iptables -w -t mangle -A OPENVPN_FORWARD_RULE -m mark --mark 0x20000/0x20000 -j RETURN

	#set forwarding mark for wireguard/ip rule: 0x20000
	iptables -w -t mangle -N WIREGUARD_FORWARD_RULE
	iptables -w -t mangle -A PREROUTING -j WIREGUARD_FORWARD_RULE
	iptables -w -t mangle -N WIREGUARD_FORWARD_MARK
	iptables -w -t mangle -N WIREGUARD_ROUTING_MARK
	iptables -w -t mangle -A WIREGUARD_FORWARD_RULE -j WIREGUARD_FORWARD_MARK
	iptables -w -t mangle -A FORWARD -j WIREGUARD_ROUTING_MARK
	iptables -w -t mangle -A WIREGUARD_FORWARD_RULE -m mark --mark 0x20000/0x20000 -j RETURN

	iptables -w -t mangle -I OUTPUT -m conntrack --ctdir ORIGINAL -m set --match-set IPGROUP_LAN dst  -m comment --comment "for ldap in full mode" -j MARK --set-mark 0x0/0x20000

	#for dmz bypass rule
	iptables -w -t nat -N dmz_prerouting_rule
}

fw_load_manage_vlan()
{
	local mvlan=$(uci get network.lan.manage_vid)

	config_load remote_mngt
	set_mngt_vlan() {
		local iface=$(uci get remote_mngt.$1.iface)
		[ "$iface" = "lan" ] && {
			local vlan=$(uci get remote_mngt.$1.mvlan)
			[ "$vlan" != "$2" ] && {
				uci set remote_mngt.$1.mvlan=$2
				uci commit
			}
		}
	}

	[ -n "$mvlan" -a "$(vlanctrl vlan_dump ${mvlan})" = "{}" ]  && mvlan=""

	config_foreach set_mngt_vlan mngt_iface "$mvlan"

	/etc/init.d/remote_mngt restart
}

fw_load_sfe()
{
	if ! iptables -w -t mangle -C sfe_mark -j CONNEXTMARK --set-mark 0x8/0x8 2>/dev/null; then
		iptables -w -t mangle -N sfe_mark
		iptables -w -t mangle -A sfe_mark -j EXTMARK --or-mark 0x8
		iptables -w -t mangle -A sfe_mark -m connextmark --mark 0x8/0x8 -j RETURN
		iptables -w -t mangle -A sfe_mark -j CONNEXTMARK --set-mark 0x8/0x8 
	fi
	if ! iptables -w -t mangle -C hwnat_mark -j CONNEXTMARK --set-mark 0x4/0x4 2>/dev/null; then
		iptables -w -t mangle -N hwnat_mark
		iptables -w -t mangle -A hwnat_mark -j EXTMARK --or-mark 0x4
		iptables -w -t mangle -A hwnat_mark -m connextmark --mark 0x4/0x4 -j RETURN
		iptables -w -t mangle -A hwnat_mark -j CONNEXTMARK --set-mark 0x4/0x4 
		iptables -w -t mangle -I sfe_mark 1 -j hwnat_mark
	fi
	iptables -w -t mangle -I FORWARD 1 -j CONNEXTMARK --set-mark 0x0/0xc

        if ! ip6tables -w -t mangle -C sfe_mark -j CONNEXTMARK --set-mark 0x8/0x8 2>/dev/null; then
                ip6tables -w -t mangle -N sfe_mark
                ip6tables -w -t mangle -A sfe_mark -j EXTMARK --or-mark 0x8
                ip6tables -w -t mangle -A sfe_mark -m connextmark --mark 0x8/0x8 -j RETURN
                ip6tables -w -t mangle -A sfe_mark -j CONNEXTMARK --set-mark 0x8/0x8
        fi
        if ! ip6tables -w -t mangle -C hwnat_mark -j CONNEXTMARK --set-mark 0x4/0x4 2>/dev/null; then
                ip6tables -w -t mangle -N hwnat_mark
                ip6tables -w -t mangle -A hwnat_mark -j EXTMARK --or-mark 0x4
                ip6tables -w -t mangle -A hwnat_mark -m connextmark --mark 0x4/0x4 -j RETURN
                ip6tables -w -t mangle -A hwnat_mark -j CONNEXTMARK --set-mark 0x4/0x4
                ip6tables -w -t mangle -I sfe_mark 1 -j hwnat_mark
        fi
        ip6tables -w -t mangle -I FORWARD 1 -j CONNEXTMARK --set-mark 0x0/0xc
}

MANAGE_DEVS_LOCK=/tmp/.manage_devs.lock

fw_add_passthrough_manage_devs()
{
	local manage_devs_chain="manage_devs"

	{
		flock -x 63
		iptables -w -I ${manage_devs_chain} 1 $@ -j RETURN
		flock -u 63
	} 63<>${MANAGE_DEVS_LOCK}
}

fw_del_passthrough_manage_devs()
{
	local manage_devs_chain="manage_devs"

	{
		flock -x 63
		iptables -w -D ${manage_devs_chain} $@ -j RETURN
		flock -u 63
	} 63<>${MANAGE_DEVS_LOCK}
}

fw_del_passthrough_manage_devs_by_label()
{
	local manage_devs_chain="manage_devs"
	local label1="$1"
	local label2="$2"
	local nums=""

	{
		flock -x 63

		nums=$(iptables -w -vnL ${manage_devs_chain} --line-numbers | grep "\b${label1}\b" | grep "\b${label2}\b" | cut -d ' ' -f 1)
		while [ -n "${nums}" ]; do
			for num in $nums;
			do
				iptables -w -D ${manage_devs_chain} ${num}
				break; # just delete the first once.
			done
			nums=$(iptables -w -vnL ${manage_devs_chain} --line-numbers | grep "\b${label1}\b" | grep "\b${label2}\b" | cut -d ' ' -f 1)
		done
	
		flock -u 63
	} 63<>${MANAGE_DEVS_LOCK}
}

fw_install_passthrough_icmp()
{
	local action="$1"
	local ifname="$2"
        local srcitf="$3"

	network_get_ipaddr srcip ${srcitf}
	case "${action}" in
		"add" )
			fw_add_passthrough_manage_devs -o ${ifname} -s ${srcip} -p icmp
		;;
		"del" )
			fw_del_passthrough_manage_devs_by_label ${ifname} "icmp"
		;;
	esac
}

fw_install_passthrough_dns()
{
	local action="$1"
	local ifname="$2"
	local srcitf="$3"

	network_get_ipaddr srcip ${srcitf}
	case "${action}" in
		"add" )
			fw_add_passthrough_manage_devs -o ${ifname} -s ${srcip} -p udp --dport 53
		;;
		"del" )
			fw_del_passthrough_manage_devs_by_label ${ifname} "dpt:53"
		;;
	esac

	# online check
	fw_install_passthrough_icmp "$@"
}

fw_event_manage_dev()
{
	local manage_devs_chain="manage_devs"
	local manage_devs="${1%%:*}"
	local updown="${1##*:}"

	{
		flock -x 63

		if ! iptables -w -C ${manage_devs_chain} -j DROP 2>/dev/null; then
			iptables -w -N ${manage_devs_chain}
			iptables -w -A output -j ${manage_devs_chain}
			iptables -w -A ${manage_devs_chain} -p tcp --dport 1723 -j RETURN
			iptables -w -A ${manage_devs_chain} -p udp -m multiport --dports 1701,67,20001 -j RETURN
			iptables -w -A ${manage_devs_chain} -j DROP
		fi

		case "${updown}" in
			"up")
				# setup new rule
				if ! iptables -w -C ${manage_devs_chain} -o ${manage_devs} -j RETURN 2>/dev/null; then
					iptables -w -I ${manage_devs_chain} 1 -o ${manage_devs} -j RETURN
				fi
				;;
			"down")
				# clean up old rule
				iptables -w -D ${manage_devs_chain} -o ${manage_devs} -j RETURN 2>/dev/null
				;;
			*)
				;;
		esac
		flock -u 63
	} 63<>${MANAGE_DEVS_LOCK}

	true ;
}

fw_load_manage_devs()
{
	network_flush_cache
	for zone in $(zone_get_zonelist) ; do
		local ifaces=$(zone_get_wan_ifaces $zone)

		for iface in ${ifaces} ; do
			if network_is_up ${iface}; then
				local device=$(zone_get_device_byif $iface)
				fw_event_manage_dev "${device}:up"
			fi
		done
	done

	if network_is_up "loopback"; then
		fw_event_manage_dev "lo:up"
	fi
}

# default netstack: IPV4/IPV6
fw_dual_ipt()
{
	iptables $@
	ip6tables $@
}

# vnet netstack: IPV4/IPV6
fw_dual_vnet_ipt()
{	
	vnete iptables $@
	vnete ip6tables $@
}

# default netstack & vnet netstack: IPV4/IPV6
fw_dual_anet_ipt()
{	
	anete iptables $@
	anete ip6tables $@
}

fw_load_service_default()
{
	local support_ipv6=`uci -c /etc/profile.d/ get profile.@global[0].support_ipv6`
	fw_dual_ipt -w -N service_default
	fw_dual_ipt -w -A service_default -j reject
	#fw_dual_ipt -w -A input_default -p tcp -m multiport --dports 22,23,53,80,443,1723,1900,2026,8080,20002 -j service_default
	fw_dual_ipt -w -A input_default -p tcp -j service_default
	if [ "1" == "$support_ipv6" ]; then
		fw_dual_ipt -w -A input_default -p udp -j service_default
	else
	# fw_dual_ipt -w -A input_default -p udp -m multiport --dports 53,67,137,138,161,500,1701,1900,4500,20002 -j service_default
		iptables -w -A input_default -p udp -j service_default
		ip6tables -w -A input_default -p udp -m multiport --dports 53,67,137,138,161,500,1701,1900,4500,20002 -j service_default
	fi
}

#0x10000
fw_load_forwarding_mark()
{
	mark_chain="GENERAL_FORWARD_MARK"
	forwarding_mark=$((1<<16))
	forwarding_excl_management_mark=$((1<<12))

	if ! iptables -w -C FORWARD -j $mark_chain 2>/dev/null; then	
		fw_dual_ipt -t mangle -w -N $mark_chain
		fw_dual_ipt -t mangle -w -I FORWARD -j $mark_chain
		fw_dual_ipt -t mangle -w -I $mark_chain 1 -j CONNMARK --or-mark ${forwarding_excl_management_mark}
		fw_dual_ipt -t mangle -w -I $mark_chain 1 -j MARK --or-mark ${forwarding_excl_management_mark}
		fw_dual_ipt -t mangle -w -I $mark_chain 1 -m set --match-set VMe src -j RETURN
		fw_dual_ipt -t mangle -w -I $mark_chain 1 -m set --match-set VMe dst -j RETURN
		fw_dual_ipt -t mangle -w -I $mark_chain 1 -j CONNMARK --or-mark ${forwarding_mark}
		fw_dual_ipt -t mangle -w -I $mark_chain 1 -j MARK --or-mark ${forwarding_mark}
	fi
}

#-j MARK 0x800
fw_load_management_mark()
{
	management_mark=$((1<<11))
	fw_dual_ipt -t raw -w -A PREROUTING -m set --match-set Me dst -j MARK --or-mark ${management_mark}
	fw_dual_vnet_ipt -t raw -w -A PREROUTING -m set --match-set VMe dst -j MARK --or-mark ${management_mark}
	fw_dual_ipt -t raw -w -A OUTPUT -m set --match-set Me src -j MARK --or-mark ${management_mark}
	fw_dual_vnet_ipt -t raw -w -A OUTPUT -m set --match-set VMe src -j MARK --or-mark ${management_mark}
}
