#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/list.h>
#include <linux/mii.h>
#include <linux/phy.h>
#include <linux/of.h>
#include <linux/of_mdio.h>
#include <linux/delay.h>
#include <linux/switch.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/workqueue.h>


#include "mvsw61xx.h"
#include "Amethyst_switch.h"
#undef IN
#define IN
#undef OUT
#define OUT
#undef INOUT
#define INOUT
#define MSD_SMI_ACCESS_LOOP        1000U
#define MSD_SMI_BUSY                0x8000U
#define MSD_SMI_MODE                0x1000U
#define MSD_SMI_MODE_BIT            12U
#define MSD_SMI_FUNC_BIT            13U
#define MSD_SMI_FUNC_SIZE            2U
#define MSD_SMI_OP_BIT              10U
#define MSD_SMI_OP_SIZE              2U
#define MSD_SMI_DEV_ADDR_BIT         5U
#define MSD_SMI_DEV_ADDR_SIZE        5U
#define MSD_SMI_REG_ADDR_BIT         0U
#define MSD_SMI_REG_ADDR_SIZE        5U

#define MSD_SMI_CLAUSE45            0U
#define MSD_SMI_CLAUSE22            1U

#define MSD_SMI_WRITE_ADDR          0x00U
#define MSD_SMI_WRITE               0x01U
#define MSD_SMI_READ_22             0x02U
#define MSD_SMI_READ_45             0x03U
#define MSD_SMI_READ_INC            0x02U

#define MSD_SMI_INTERNAL			0x0U
#define MSD_SMI_EXTERNAL			0x1U
#define MSD_SMI_SMISETUP			0x2U



#define SW_DEBUG KERN_DEBUG
MSD_STATUS Amethyst_gstatsGetPortAllCounters
(
    struct switch_dev *dev,
    MSD_LPORT port,
    AMETHYST_MSD_STATS_COUNTER_SET    *statsCounterSet
);
MSD_STATUS Amethyst_gstatsFlushPort
(
    struct switch_dev *dev,
    MSD_LPORT      port
);

char* msdDisplayStatus(MSD_STATUS status)
{
	char * str;
	switch (status)
	{
	case MSD_OK:
		str = "MSD_OK";
		break;
	case MSD_FAIL:
		str = "MSD_FAIL";
		break;
	case MSD_BAD_PARAM:
		str = "MSD_BAD_PARAM";
		break;
	case MSD_NO_SUCH:
		str = "MSD_NO_SUCH";
		break;
	case MSD_NOT_SUPPORTED:
		str = "MSD_NOT_SUPPORTED";
		break;
	case MSD_ALREADY_EXIST:
		str = "MSD_ALREADY_EXIST";
		break;
	case MSD_BAD_CPU_PORT:
		str = "MSD_BAD_CPU_PORT";
		break;
    case MSD_FEATURE_NOT_ENABLE:
        str = "MSD_FEATURE_NOT_ENABLE";
        break;
	default:
		str = "UnknowStatus";
		break;
	}
	return str;
}

/*****************************************************************************
* msdSetSMIPhyReg
*
* DESCRIPTION:
*       This function indirectly write internal PHY register through SMI PHY command.
*
* INPUTS:
*       devAddr - The PHY address.
*       regAddr - The register address.
*       data - data to be written
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK   - on success
*       MSD_FAIL - on error
*
* COMMENTS:
*       None.
*
*******************************************************************************/
MSD_STATUS Amethyst_msdSetSMIPhyReg
(
    IN struct switch_dev *dev,
    IN  u8    devAddr,
    IN  u8    regAddr,
    IN u16   data
)
{
    unsigned int timeOut;
    u16 smiReg;

    MSD_DBG(SW_DEBUG "(LOG AMETHYST SMI PHY C22 RW): devAddr 0x%02x, regAddr 0x%02x, \n",
        devAddr, regAddr);
    MSD_DBG(SW_DEBUG "data 0x%04x.\n", data);

    /* first check that it is not busy */
    timeOut = MSD_SMI_ACCESS_LOOP; /* initialize the loop count */

    do 
    {
        if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, &smiReg) != MSD_OK)
        {
            MSD_DBG_ERROR(SW_DEBUG "read AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
            return MSD_FAIL;
        }
        if(timeOut-- < 1 ) 
        {
            MSD_DBG_INFO(SW_DEBUG "read AMETHYST_QD_REG_SMI_PHY_CMD register Timed Out\n");
            return MSD_FAIL;
        }
    } while (smiReg & MSD_SMI_BUSY);


    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_DATA, data) != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "write AMETHYST_QD_REG_SMI_PHY_DATA register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }
    smiReg = MSD_SMI_BUSY | (devAddr << MSD_SMI_DEV_ADDR_BIT) | (MSD_SMI_WRITE << MSD_SMI_OP_BIT) | 
            (regAddr << MSD_SMI_REG_ADDR_BIT) | (MSD_SMI_CLAUSE22 << MSD_SMI_MODE_BIT);

    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, smiReg) != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "write AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }

    return MSD_OK;
}

/*****************************************************************************
* msdGetSMIPhyReg
*
* DESCRIPTION:
*       This function indirectly read internal PHY register through SMI PHY command.
*
* INPUTS:
*       devAddr  - The PHY address to be read.
*       regAddr  - The register address to read.
*
* OUTPUTS:
*       data     - The storage where register date to be saved.
*
* RETURNS:
*       MSD_OK   - on success
*       MSD_FAIL - on error
*
* COMMENTS:
*       None.
*
*******************************************************************************/
MSD_STATUS Amethyst_msdGetSMIPhyReg
(
    IN struct switch_dev *dev,
    IN  u8    devAddr,
    IN  u8    regAddr,
    OUT  u16   *data
)
{
    unsigned int timeOut; 
    u16 smiReg;



    /* first check that it is not busy */
    timeOut = MSD_SMI_ACCESS_LOOP; /* initialize the loop count */

    do 
    {
        if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, &smiReg) != MSD_OK)
        {
            MSD_DBG_ERROR(SW_DEBUG "read AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
            return MSD_FAIL;
        }
        if(timeOut-- < 1 ) 
        {
            MSD_DBG_ERROR(SW_DEBUG "Read AMETHYST_QD_REG_SMI_PHY_CMD Timed Out\n");
            return MSD_FAIL;
        }
    } while (smiReg & MSD_SMI_BUSY);

    smiReg =  MSD_SMI_BUSY | (devAddr << MSD_SMI_DEV_ADDR_BIT) | (MSD_SMI_READ_22 << MSD_SMI_OP_BIT) | 
            (regAddr << MSD_SMI_REG_ADDR_BIT) | (MSD_SMI_CLAUSE22 << MSD_SMI_MODE_BIT);

    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, smiReg) != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "write AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }
	
    timeOut = MSD_SMI_ACCESS_LOOP; /* initialize the loop count */

    do 
    {
        if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, &smiReg) != MSD_OK)
        {
            MSD_DBG_ERROR(SW_DEBUG "read AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
            return MSD_FAIL;
        }
        if(timeOut-- < 1 ) 
        {
            MSD_DBG_ERROR(SW_DEBUG "read TOPAZ_QD_REG_SMI_PHY_DATA register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
            return MSD_FAIL;
        }
    } while (smiReg & MSD_SMI_BUSY);

    if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_DATA, &smiReg) != MSD_OK)
    {
        MSD_DBG_INFO(SW_DEBUG "Reading Phy register Failed\n");
        return MSD_FAIL;
    }
    *data = smiReg;

    MSD_DBG(SW_DEBUG "(LOG AMETHYST SMI PHY C22 RW): devAddr 0x%02x, regAddr 0x%02x, \n",
        devAddr, regAddr);
    MSD_DBG(SW_DEBUG "data 0x%04x.\n", *data);

    return MSD_OK;
}

MSD_STATUS Amethyst_msdReadPagedPhyReg
(
    IN  struct switch_dev *dev,
    IN  u8    portNum,
    IN  u8    pageNum,
    IN  u8    regAddr,
    OUT u16   *data
)
{
	MSD_STATUS retVal;
	retVal = Amethyst_msdSetSMIPhyReg(dev, portNum, 22, pageNum);
	if(retVal != MSD_OK)
		return retVal;
	retVal = Amethyst_msdGetSMIPhyReg(dev, portNum, regAddr, data);
	return retVal;
}

MSD_STATUS Amethyst_msdWritePagedPhyReg
(
    IN  struct switch_dev *dev,
    IN  u8    portNum,
    IN  u8    pageNum,
    IN  u8    regAddr,
    IN  u16   data
)
{
	
	MSD_STATUS retVal;
	retVal = Amethyst_msdSetSMIPhyReg(dev, portNum, 22, pageNum);
	if(retVal != MSD_OK)
		return retVal;
	retVal = Amethyst_msdSetSMIPhyReg(dev, portNum, regAddr, data);
	return retVal;
}

/*****************************************************************************
* msdGetSMIPhyRegField
*
* DESCRIPTION:
*       This function indirectly read internal PHY specified register field through SMI PHY command.
*
* INPUTS:
*       devAddr     - The PHY address to be read.
*       regAddr     - The register address to read.
*       fieldOffset - The field start bit index. (0 - 15)
*       fieldLength - Number of bits to write.
*
* OUTPUTS:
*       data       - The storage where register date to be saved.
*
* RETURNS:
*       MSD_TRUE   - on success
*       MSD_FALSE  - on error
*
* COMMENTS:
*       None.
*
*******************************************************************************/
MSD_STATUS Amethyst_msdGetSMIPhyRegField
(
    IN  struct switch_dev *dev,
    IN  u8    devAddr,
    IN  u8    regAddr,
    IN  u8    fieldOffset,
    IN  u8    fieldLength,
    OUT  u16   *data
)
{
    u16 mask;            /* Bits mask to be read */
    u16 tmpData;
    MSD_STATUS   retVal;

    retVal =  Amethyst_msdGetSMIPhyReg(dev,devAddr,regAddr,&tmpData);
    if (retVal != MSD_OK)
        return retVal;
        
    MSD_CALC_MASK(fieldOffset,fieldLength,mask);

    tmpData = (tmpData & mask) >> fieldOffset;
    *data = tmpData;
    MSD_DBG_INFO(SW_DEBUG "Read from deviceAddr(%d): regAddr 0x%x, \n",
              devAddr,regAddr);
    MSD_DBG_INFO(SW_DEBUG "fOff %d, fLen %d, data 0x%x.\n",fieldOffset,fieldLength,*data);

    return MSD_OK;
}
/*****************************************************************************
* msdSetSMIPhyRegField
*
* DESCRIPTION:
*       This function indirectly write internal PHY specified register field through SMI PHY command.
*
* INPUTS:
*       devAddr     - The PHY address to be read.
*       regAddr     - The register address to read.
*       fieldOffset - The field start bit index. (0 - 15)
*       fieldLength - Number of bits to write.
*		data		- register date to be written.
*
* OUTPUTS:
*		None
*
* RETURNS:
*       MSD_TRUE   - on success
*       MSD_FALSE  - on error
*
* COMMENTS:
*       None.
*
*******************************************************************************/
MSD_STATUS Amethyst_msdSetSMIPhyRegField
(
    IN struct switch_dev *dev,
    IN  u8    devAddr,
    IN  u8    regAddr,
    IN  u8    fieldOffset,
    IN  u8    fieldLength,
    IN u16    data
)
{
    u16 mask;
   	u16 tmpData;
    MSD_STATUS   retVal;

    retVal =  Amethyst_msdGetSMIPhyReg(dev,devAddr,regAddr,&tmpData);
    if(retVal != MSD_OK)
    {
        return retVal;
    }

    MSD_CALC_MASK(fieldOffset,fieldLength,mask);

    /* Set the desired bits to 0.                       */
    tmpData &= ~mask;
    /* Set the given data into the above reset bits.    */
    tmpData |= ((data << fieldOffset) & mask);
    MSD_DBG_INFO(SW_DEBUG "Write to devAddr(%d): regAddr 0x%x, \n",
              devAddr,regAddr);
    MSD_DBG_INFO(SW_DEBUG "fieldOff %d, fieldLen %d, data 0x%x.\n",fieldOffset,
              fieldLength,data);

    retVal = Amethyst_msdSetSMIPhyReg(dev,devAddr,regAddr,tmpData);

    return retVal;
}

MSD_STATUS Amethyst_msdSetPagedPhyRegField
(
    IN  struct switch_dev *dev,
    IN  u8    portNum,
    IN  u8    pageNum,
    IN  u8    regAddr,
    IN  u8    fieldOffset,
    IN  u8    fieldLength,
    IN  u16   data
)	
{
	MSD_STATUS retVal;
	retVal = Amethyst_msdSetSMIPhyReg(dev, portNum, 22, pageNum);
	if(retVal != MSD_OK)
		return retVal;
	retVal = Amethyst_msdSetSMIPhyRegField(dev, portNum, regAddr, fieldOffset, fieldLength, data);
	return retVal;
}	

MSD_STATUS Amethyst_msdGetPagedPhyRegField
(
    IN  struct switch_dev *dev,
    IN  u8    portNum,
    IN  u8    pageNum,
    IN  u8    regAddr,
    IN  u8    fieldOffset,
    IN  u8    fieldLength,
    IN  u16   *data
)	
{
	MSD_STATUS retVal;
	retVal = Amethyst_msdSetSMIPhyReg(dev, portNum, 22, pageNum);
	if(retVal != MSD_OK)
		return retVal;
	retVal = Amethyst_msdGetSMIPhyRegField(dev, portNum, regAddr, fieldOffset, fieldLength, data);
	return retVal;
}

MSD_STATUS Amethyst_msdSetSMIPhyXMDIOReg
(
    IN  struct switch_dev *dev,
    IN  u8    portNum,
    IN  u8    devAddr,
    IN  u16   regAddr,
    IN  u16   data
)
{
    MSD_STATUS retVal;
    u16 tmpData;
    u8 hwPort;

    hwPort = portNum;

    /* Set MMD access control address is Address | devAddr */
    tmpData = MSD_PHY_MMD_ADDR | devAddr;
    retVal = Amethyst_msdWritePagedPhyReg(dev, hwPort, 0, 13, tmpData);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("ERROR to Set MMD access control address is Address | devAddr.\n");
        return retVal;
    }

    /* Set MMD access Data is regAddr */
    tmpData = regAddr;
    retVal = Amethyst_msdWritePagedPhyReg(dev, hwPort, 0, 14, tmpData);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("ERROR to Set MMD access Data is regAddr.\n");
        return retVal;
    }

    /* Set MMD access control address is Data no inc | devAddr */
    tmpData = MSD_PHY_MMD_DATA_NO_INC | devAddr;
    retVal = Amethyst_msdWritePagedPhyReg(dev, hwPort, 0, 13, tmpData);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("ERROR to Set MMD access control address is Data no inc | devAddr.\n");
        return retVal;
    }

    /* Write MMD access Data */
    retVal = Amethyst_msdWritePagedPhyReg(dev, hwPort, 0, 14, data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("ERROR to Write MMD access Data.\n");
        return retVal;
    }

    return MSD_OK;
}

/* ##############################   port MIB   #####################################  */

int mvsw61xx_reset_port_mib(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{
	int portNum = val->port_vlan;
	Amethyst_gstatsFlushPort(dev, portNum);
	return MSD_OK;
}
char statistic_buf[4096] = {0};
int mvsw61xx_get_port_mib(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{
	MSD_STATUS status;
	AMETHYST_MSD_STATS_COUNTER_SET statsCounter;
	int portNum = val->port_vlan;
	u64 ifInOctetsLo,ifInOctetsHi,ifInOctets;
	u64 ifOutOctetsLo,ifOutOctetsHi,ifOutOctets;
	int len = 0;

	memset(&statsCounter, 0, sizeof(AMETHYST_MSD_STATS_COUNTER_SET));
	if ((status = Amethyst_gstatsGetPortAllCounters(dev, portNum, &statsCounter)) != MSD_OK)
	{
		return status;
	}
	memset(statistic_buf,0,4096);
	len += snprintf(statistic_buf + len, sizeof(statistic_buf) - len, 
                    "Port %d MIB counters\n", portNum);
	/* Bank 0 */
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","InGoodOctetsLo", statsCounter.InGoodOctetsLo);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","InGoodOctetsHi", statsCounter.InGoodOctetsHi);
	ifInOctetsLo = statsCounter.InGoodOctetsLo;
	ifInOctetsHi = statsCounter.InGoodOctetsHi;
	ifInOctets = (ifInOctetsHi << 32) + ifInOctetsLo;
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %llu\n","ifInOctets", ifInOctets);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","InBadOctets", statsCounter.InBadOctets);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","OutFCSErr", statsCounter.OutFCSErr);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","ifInUcastPkts" ,statsCounter.InUnicasts);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","Deferred" ,statsCounter.Deferred);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","ifInBroadcastPkts" ,statsCounter.InBroadcasts);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","ifInMulticastPkts" ,statsCounter.InMulticasts);
	/*
	Histogram Counters : Rx Only, Tx Only, or both Rx and Tx
	(refer to Histogram Mode)
	*/
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","etherStatsPkts64Octets" ,statsCounter.Octets64);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","etherStatsPkts65to127Octets" ,statsCounter.Octets127);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","etherStatsPkts128to255Octets" ,statsCounter.Octets255);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","etherStatsPkts256to511Octets" ,statsCounter.Octets511);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","etherStatsPkts512to1023Octets" ,statsCounter.Octets1023);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","etherStatsPkts1024toMaxOctets" ,statsCounter.OctetsMax);

	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","OutOctetsLo" ,statsCounter.OutOctetsLo);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","OutOctetsHi" ,statsCounter.OutOctetsHi);
	ifOutOctetsLo = statsCounter.OutOctetsLo;
	ifOutOctetsHi = statsCounter.OutOctetsHi;
	ifOutOctets = (ifOutOctetsHi << 32) + ifOutOctetsLo;
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %llu\n","ifOutOctets" ,ifOutOctets);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","ifOutUcastPkts" ,statsCounter.OutUnicasts);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","Excessive" ,statsCounter.Excessive);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","ifOutMulticastPkts" ,statsCounter.OutMulticasts);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","ifOutBrocastPkts" ,statsCounter.OutBroadcasts);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","Single" ,statsCounter.Single);

	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","dot3OutPauseFrames" ,statsCounter.OutPause);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","dot3InPauseFrames" ,statsCounter.InPause);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","Multiple" ,statsCounter.Multiple);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","etherStatsUndersizePkts" ,statsCounter.InUndersize);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","InFragments" ,statsCounter.InFragments);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","etherStatsOversizePkts" ,statsCounter.InOversize);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","InJabber" ,statsCounter.InJabber);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","InRxErr" ,statsCounter.InRxErr);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","InFCSErr" ,statsCounter.InFCSErr);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","Collisions" ,statsCounter.Collisions);
	len += snprintf(statistic_buf + len,sizeof(statistic_buf) -len,"%-36s: %u\n","Late" ,statsCounter.Late);

	val->value.s = statistic_buf;
	val->len = len;

        return 0;
}

static MSD_STATUS Amethyst_statsReadCounter
(
    struct switch_dev             *dev,
    u16         port,
    u32         counter,
    u32         *statsData
)
{
    MSD_STATUS   retVal;         /* Functions return value.            */
    u16      data;/* Data to be set into the  register. */ 
    u16    counter3_2;     /* Counter Register Bytes 3 & 2       */
    u16    counter1_0;     /* Counter Register Bytes 1 & 0       */

	
    data = 1;
    while(data == 1)
    {
        retVal = msdGetAnyRegField(dev,  AMETHYST_GLOBAL1_DEV_ADDR,AMETHYST_QD_REG_STATS_OPERATION,15,1,&data);
        if(retVal != MSD_OK)
        {
            return retVal;
        }
	}
	
    data = (u16)((1 << 15) | (Amethyst_STATS_READ_COUNTER << 12) | port | (counter&0x1f) );
	if (counter & AMETHYST_MSD_TYPE_BANK)
	{
		data |= (1<<10);
	}

    retVal = msdSetAnyReg(dev,  AMETHYST_GLOBAL1_DEV_ADDR,AMETHYST_QD_REG_STATS_OPERATION,data);
    if(retVal != MSD_OK)
    {
        return retVal;
    }

    data = 1;
    while(data == 1)
    {
        retVal = msdGetAnyRegField(dev,  AMETHYST_GLOBAL1_DEV_ADDR,AMETHYST_QD_REG_STATS_OPERATION,15,1,&data);
        if(retVal != MSD_OK)
        {
            return retVal;
        }
	}

	retVal = msdGetAnyReg(dev,  AMETHYST_GLOBAL1_DEV_ADDR, AMETHYST_QD_REG_STATS_COUNTER3_2, &counter3_2);
    if(retVal != MSD_OK)
    {
        return retVal;
    }

	retVal = msdGetAnyReg(dev,  AMETHYST_GLOBAL1_DEV_ADDR, AMETHYST_QD_REG_STATS_COUNTER1_0, &counter1_0);
    if(retVal != MSD_OK)
    {
        return retVal;
    }

    *statsData = (counter3_2 << 16) | counter1_0;

    return MSD_OK;

}


static MSD_STATUS Amethyst_statsOperationPerform
(
    struct switch_dev             *dev,
    AMETHYST_MSD_STATS_OPERATION   statsOp,
    u8                port,
    AMETHYST_MSD_STATS_COUNTERS    counter,
    void              *statsData
)
{
    MSD_STATUS       retVal;         /* Functions return value.      */
    u16          data;			/* Data to be set into the      */
    u32	tmpCounter;
    u32	startCounter;
    u32	lastCounter;
    u16	portNum;
	u8 	bank;

	portNum = (port + 1) << 5;

    /* Wait until the stats in ready. */
    data = 1;
    while(data == 1)
    {
        retVal = msdGetAnyRegField(dev,  AMETHYST_GLOBAL1_DEV_ADDR,AMETHYST_QD_REG_STATS_OPERATION,15,1,&data);
        if(retVal != MSD_OK)
        {
            return retVal;
        }
    }

	retVal = msdGetAnyReg(dev,  AMETHYST_GLOBAL1_DEV_ADDR,AMETHYST_QD_REG_STATS_OPERATION,&data);
	if(retVal != MSD_OK)
	{
		return retVal;
	}
	data &= 0xfff;
    /* Set the STAT Operation register */
    switch (statsOp)
    {
        case Amethyst_STATS_FLUSH_ALL:
            data |= (1 << 15) | (Amethyst_STATS_FLUSH_ALL << 12);
            retVal = msdSetAnyReg(dev,  AMETHYST_GLOBAL1_DEV_ADDR,AMETHYST_QD_REG_STATS_OPERATION,data);

            return retVal;

        case Amethyst_STATS_FLUSH_PORT:
			data &= 0xc1f;
            data |= (1 << 15) | (Amethyst_STATS_FLUSH_PORT << 12) | portNum ;
            retVal = msdSetAnyReg(dev,  AMETHYST_GLOBAL1_DEV_ADDR,AMETHYST_QD_REG_STATS_OPERATION,data);
			
            return retVal;

		case Amethyst_STATS_READ_COUNTER:
            retVal = Amethyst_statsReadCounter(dev, portNum, (MSD_U32)counter, (MSD_U32*)statsData);
            if(retVal != MSD_OK)
            {
                return retVal;
            }
            break;

        case Amethyst_STATS_READ_ALL:
			{
				int bankSize = 2;
				for(bank=0; bank<bankSize; bank++)
				{
					lastCounter = (bank == 0) ? (MSD_U32)Amethyst_STATS_Late : (MSD_U32)Amethyst_STATS_OutMGMT;
					startCounter = (bank==0)?(MSD_U32)Amethyst_STATS_InGoodOctetsLo : (MSD_U32)Amethyst_STATS_InDiscards;

					if(bank==1)
						statsData = (MSD_U32 *)statsData + Amethyst_STATS_Late +1;
					for(tmpCounter=startCounter; tmpCounter<=lastCounter; tmpCounter++)
					{
						retVal = Amethyst_statsReadCounter(dev, portNum, tmpCounter,((MSD_U32*)statsData+tmpCounter-startCounter));
						if(retVal != MSD_OK)
						{
							return retVal;
						}
					}
				}
			}
			break;
			
        default:
            return MSD_FAIL;
    }
    return MSD_OK;
}

/*******************************************************************************
* Amethyst_gstatsFlushPort
*
* DESCRIPTION:
*       Flush All RMON counters for a given port.
*
* INPUTS:
*       port - the logical port number.
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK - on success,
*       MSD_FAIL - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*       None
*
*******************************************************************************/
MSD_STATUS Amethyst_gstatsFlushPort
(
    struct switch_dev *dev,
    MSD_LPORT      port
)
{
    MSD_STATUS    retVal;
    u8        hwPort;         /* physical port number         */

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gstatsFlushPort Called.\n");

    /* translate logical port to physical port */
	hwPort = port;

    retVal = Amethyst_statsOperationPerform(dev,Amethyst_STATS_FLUSH_PORT,hwPort,0,NULL);
    if(retVal != MSD_OK)
    {
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_statsOperationPerform FLUSH_PORT returned: %d.\n", retVal);
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gstatsFlushPort Exit.\n");
    return MSD_OK;

}

/*******************************************************************************
* Amethyst_gstatsGetPortCounter
*
* DESCRIPTION:
*        This routine Amethyst_gets a specific counter of the given port
*
* INPUTS:
*        port - the logical port number.
*        counter - the counter which will be read
*
* OUTPUTS:
*        statsData - points to 32bit data storage for the MIB counter
*
* RETURNS:
*       MSD_OK - on success,
*       MSD_FAIL - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*        None
*
*******************************************************************************/
MSD_STATUS Amethyst_gstatsGetPortCounter
(
    struct switch_dev *dev,
    MSD_LPORT        port,
    AMETHYST_MSD_STATS_COUNTERS    counter,
    MSD_U32            *statsData
)
{
    MSD_STATUS    retVal;
    u8        hwPort;         /* physical port number         */

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gstatsGetPortCounter Called.\n");

    /* translate logical port to physical port */
    hwPort = port;

    if (counter > AMETHYST_MSD_TYPE_BANK + 0x1F || counter < 0 || (counter > 0x1f && counter < AMETHYST_MSD_TYPE_BANK))
    {
        MSD_DBG_ERROR(SW_DEBUG "Bad counter: %d.\n", (int)counter);
        return MSD_BAD_PARAM;
    }

	retVal = Amethyst_statsOperationPerform(dev, Amethyst_STATS_READ_COUNTER, hwPort, counter, (void*)statsData);
    if(retVal != MSD_OK)
    {
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_statsOperationPerform READ_COUNTER returned: %d.\n", retVal);
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gstatsGetPortCounter Exit.\n");
    return MSD_OK;

}


/*******************************************************************************
* Amethyst_gstatsGetPortAllCounters
*
* DESCRIPTION:
*       This routine Amethyst_gets all RMON counters of the given port
*
* INPUTS:
*       port - the logical port number.
*
* OUTPUTS:
*       statsCounterSet - points to AMETHYST_MSD_STATS_COUNTER_SET for the MIB counters
*
* RETURNS:
*       MSD_OK - on success,
*       MSD_FAIL - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*       None
*
*******************************************************************************/
MSD_STATUS Amethyst_gstatsGetPortAllCounters
(
    struct switch_dev *dev,
    MSD_LPORT port,
    AMETHYST_MSD_STATS_COUNTER_SET    *statsCounterSet
)
{
    MSD_STATUS    retVal;
    u8        hwPort;         /* physical port number         */

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gstatsGetPortAllCounters Called.\n");

    /* translate logical port to physical port */
    hwPort = port;

    retVal = Amethyst_statsOperationPerform(dev,Amethyst_STATS_READ_ALL,hwPort,0,(void*)statsCounterSet);
    if(retVal != MSD_OK)
    {
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_statsOperationPerform READ_ALL returned: %d.\n", retVal);
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gstatsGetPortAllCounters Exit.\n");
    return MSD_OK;

}
/* ##############################  end port MIB   #####################################  */

/* ##############################  port IRL   #####################################  */


static MSD_STATUS Amethyst_pirlOperationPerform
(
    struct switch_dev *dev,
    IN    AMETHYST_MSD_PIRL_OPERATION   pirlOp,
    IN    MSD_U32              irlPort,
    IN    MSD_U32              irlRes,
    IN    MSD_U32              irlReg,
    INOUT AMETHYST_MSD_PIRL_OP_DATA     *opData
)
{
    MSD_STATUS       retVal;    /* Functions return value */
    u16          data;     /* temporary Data storage */


    /* Wait until the pirl in ready. */
    data = 1;
    while(data == 1)
    {
		retVal = msdGetAnyRegField(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_COMMAND, 15, 1, &data);
        if(retVal != MSD_OK)
        {
            return retVal;
        }
    }

    /* Set the PIRL Operation register */
    switch (pirlOp)
    {
        case Amethyst_PIRL_INIT_ALL_RESOURCE:
            data = (1 << 15) | (Amethyst_PIRL_INIT_ALL_RESOURCE << 13);
			retVal = msdSetAnyReg(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_COMMAND, data);
            if(retVal != MSD_OK)
            {
                return retVal;
            }
            break;
        case Amethyst_PIRL_INIT_RESOURCE:
            data = (u16)((1 << 15) | (Amethyst_PIRL_INIT_RESOURCE << 13) | (irlPort&0x1f)<<8 | 
                    ((irlRes&0x7)<< 5));
			printk(KERN_ERR"port %d res %d\n",irlPort,irlRes);
			retVal = msdSetAnyReg(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_COMMAND, data);
            if(retVal != MSD_OK)
            {
                return retVal;
            }
            break;

        case Amethyst_PIRL_WRITE_RESOURCE:
            data = (u16)(opData->irlData);
			retVal = msdSetAnyReg(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_DATA, data);
            if(retVal != MSD_OK)
            {
                return retVal;
            }

            data = (u16)((1 << 15) | (Amethyst_PIRL_WRITE_RESOURCE << 13) | (irlPort&0x1f)<<8 | 
                    ((irlRes&0x7) << 5) | (irlReg & 0xF));
			retVal = msdSetAnyReg(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_COMMAND, data);
            if(retVal != MSD_OK)
            {
                return retVal;
            }
            break;

        case Amethyst_PIRL_READ_RESOURCE:
            data = (u16)((1 << 15) | (Amethyst_PIRL_READ_RESOURCE << 13) | (irlPort&0x1f)<<8 | 
                    ((irlRes&0x7) << 5) | (irlReg & 0xF));
			retVal = msdSetAnyReg(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_COMMAND, data);
            if(retVal != MSD_OK)
            {
                return retVal;
            }

            data = 1;
            while(data == 1)
            {
				retVal = msdGetAnyRegField(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_COMMAND, 15, 1, &data);
                if(retVal != MSD_OK)
                {
                    return retVal;
                }
            }

			retVal = msdGetAnyReg(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_DATA, &data);
            opData->irlData = (MSD_U32)data;
            if(retVal != MSD_OK)
            {
                return retVal;
            }
            return retVal;

        default:
            
            return MSD_FAIL;
    }

    /* Wait until the pirl in ready. */

    data = 1;
    while(data == 1)
    {
		retVal = msdGetAnyRegField(dev,  AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_IGR_RATE_COMMAND, 15, 1, &data);
        if(retVal != MSD_OK)
        {
            return retVal;
        }
    }

    return retVal;
}



/*******************************************************************************
* Amethyst_gpirlCustomSetup_sr2c
*
* DESCRIPTION:
*       This function Calculate CBS/EBS/BI/BRFGrn/BRFYel/Delta according to user 
*       specific tarAmethyst_get rate, tarAmethyst_get burst size and countMode.
*
* INPUTS:
*       tAmethyst_gtRate	- tarAmethyst_get rate(units is fps when countMode = AMETHYST_MSD_PIRL_COUNT_FRAME,
*				  otherwise units is bps)
*       tAmethyst_gtBstSize	- tarAmethyst_get burst size(units is Byte)
*       customSetup - Constant and countMode in customSetup as input parameter
*
* OUTPUTS:
*       customSetup - customer setup including CBS/EBS/BI/BRFGrn/BRFYel/Delta/countMode
*
* RETURNS:
*       MSD_OK - on success,
*       MSD_FAIL - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*		We have the following contrain in calculate the final parameters:
*			CBS >= BktIncrement * tAmethyst_gtBstSize
*
*******************************************************************************/
MSD_STATUS Amethyst_gpirlCustomSetup_sr2c
(
    IN  MSD_U32	   	tAmethyst_gtRate,
    IN  MSD_U32	   	tAmethyst_gtBstSize,
	INOUT AMETHYST_MSD_PIRL_CUSTOM_RATE_LIMIT  *customSetup
)
{

    u64 IRL_constant = 500000000;
	u64 BktInc = 4000;/* BktInc 0..4095  */
	u64 speedGrade = (IRL_constant/1000)/BktInc;
	u64 tgRate = 0;
    AMETHYST_MSD_PIRL_CUSTOM_RATE_LIMIT Setup;
    

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gpirlCustomSetup Called.\n");

    if (tAmethyst_gtBstSize < 1600 || tAmethyst_gtRate == 0)
    {
        MSD_DBG_ERROR(SW_DEBUG "Bad tgtBstSize Or tgtRate, tgtBstSize: %u, tgtRate: %u.\n", tAmethyst_gtBstSize, tAmethyst_gtRate);
        return MSD_BAD_PARAM;
    }

	MSD_DBG_INFO(SW_DEBUG "speedGrade %llu", speedGrade);
    memset(&Setup, 0, sizeof(AMETHYST_MSD_PIRL_CUSTOM_RATE_LIMIT));

	if (customSetup->countMode == AMETHYST_MSD_PIRL_COUNT_FRAME)
	{
		IRL_constant /= 8;
	}

	if(tAmethyst_gtRate < speedGrade)
	{
		tAmethyst_gtRate = speedGrade;
	}
	else if(tAmethyst_gtRate%speedGrade <= speedGrade/2)
	{
		tAmethyst_gtRate = (tAmethyst_gtRate/speedGrade)*speedGrade;
	}
	else
	{
		tAmethyst_gtRate = ((tAmethyst_gtRate/speedGrade)+1) * speedGrade;
	}
	MSD_DBG_INFO(SW_DEBUG "tAmethyst_gtRate %d", tAmethyst_gtRate);
	tgRate = tAmethyst_gtRate * 1000;
    Setup.bktRateFactorGrn = (u32)((tgRate * BktInc) / IRL_constant);

    customSetup->isValid = MSD_TRUE;
    customSetup->cbsLimit = BktInc * tAmethyst_gtBstSize;
    customSetup->bktIncrement = BktInc;
    customSetup->bktRateFactorGrn = Setup.bktRateFactorGrn;

    customSetup->bktRateFactorYel = 0;
    customSetup->ebsLimit = 0;

    customSetup->Delta = 0;

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gpirlCustomSetup Exit,cbsLimit %d bktIncrement %d bktRateFactorGrn %d.\n",customSetup->cbsLimit,
		customSetup->bktIncrement,customSetup->bktRateFactorGrn);
    return MSD_OK;
}


/*******************************************************************************
* Amethyst_gpirlCustomSetup_sr2c
*
* DESCRIPTION:
*       This function Calculate CBS/EBS/BI/BRFGrn/BRFYel/Delta according to user 
*       specific tarAmethyst_get rate, tarAmethyst_get burst size and countMode.
*
* INPUTS:
*       tAmethyst_gtRate	- tarAmethyst_get rate(units is fps when countMode = AMETHYST_MSD_PIRL_COUNT_FRAME,
*				  otherwise units is kbps)
*       tAmethyst_gtBstSize	- tarAmethyst_get burst size(units is Byte)
*       customSetup - Constant and countMode in customSetup as input parameter
*
* OUTPUTS:
*       customSetup - customer setup including CBS/EBS/BI/BRFGrn/BRFYel/Delta/countMode
*
* RETURNS:
*       MSD_OK - on success,
*       MSD_FAIL - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*		We have the following contrain in calculate the final parameters:
*			CBS >= BktIncrement * tAmethyst_gtBstSize
*
*******************************************************************************/
MSD_STATUS Amethyst_gpirlCustomSetup_sr2cIntf
(
IN  MSD_U32	   	tgtRate,
IN  MSD_U32	   	tgtBstSize,
IN  MSD_PIRL_COUNT_MODE countMode,
OUT MSD_PIRL_CUSTOM_RATE_LIMIT  *customSetup
)
{
	AMETHYST_MSD_PIRL_CUSTOM_RATE_LIMIT setup;
	MSD_STATUS       retVal;

	if (NULL == customSetup)
	{
		MSD_DBG_ERROR(SW_DEBUG "Input param MSD_PIRL_CUSTOM_RATE_LIMIT in Amethyst_gpirlCustomSetup_sr2cIntf is NULL. \n");
		return MSD_BAD_PARAM;
	}

	setup.isValid = customSetup->isValid;
	setup.ebsLimit = customSetup->ebsLimit;
	setup.cbsLimit = customSetup->cbsLimit;
	setup.bktIncrement = customSetup->bktIncrement;
	setup.bktRateFactorGrn = customSetup->bktRateFactorGrn;
	setup.bktRateFactorYel = customSetup->bktRateFactorYel;
	setup.countMode = countMode;

	retVal = Amethyst_gpirlCustomSetup_sr2c( tgtRate, tgtBstSize, &setup);
	if (MSD_OK != retVal)
		return retVal;

	memset((void*)customSetup, 0, sizeof(MSD_PIRL_CUSTOM_RATE_LIMIT));

	customSetup->isValid = setup.isValid;
	customSetup->ebsLimit = setup.ebsLimit;
	customSetup->cbsLimit = setup.cbsLimit;
	customSetup->bktIncrement = setup.bktIncrement;
	customSetup->bktRateFactorGrn = setup.bktRateFactorGrn;
	customSetup->countMode = setup.countMode;
	customSetup->bktRateFactorYel = setup.bktRateFactorYel;

	return retVal;
}


/*******************************************************************************
* msdIRLCustomSetup
*
* DESCRIPTION:
*       This function calculates CBS/EBS/BI/BRFGrn/BRFYel/Delta according to user
*       specific target rate, target burst size and countMode.
*
* INPUTS:
*       devNum  - physical device number
*       tgtRate	- target rate(units is kfps when countMode = MSD_PIRL_COUNT_FRAME,
*				  otherwise units is kbps)
*       tgtBstSize	- target burst size(units is Byte)
*       countMode - MSD_PIRL_COUNT_MODE
*       customSetup - countMode in customSetup as input parameter
*
* OUTPUTS:
*       customSetup - MSD_PIRL_CUSTOM_RATE_LIMIT
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*       MSD_NOT_SUPPORTED - device not support
*
* COMMENTS:
*		We have the following contrain in calculate the final parameters:
*			CBS >= BktIncrement * tgtBstSize
*       for Topaz:
*		    As the input for structure 'MSD_PIRL_CUSTOM_RATE_LIMIT', Constant can be
*           0 or any value from {7812500, 125000000, 62500000, 31250000, 15625000, 
*           3906250, 1953750, 976250}. If Constant is 0, this API will automatically
*           calculate the best IRL Constant which means Delta between target Rate
*           and real Rate is small enough or 0. However, since Constant is Global
*           for all Resource of all ports, user can specify fixed Constant to 
*           gurantee that the Constant is consistent across all existing resources.
*
*******************************************************************************/
MSD_STATUS msdIRLCustomSetup
(struct switch_dev *dev,u32  tgtRate,u32  tgtBstSize,MSD_PIRL_COUNT_MODE  countMode,MSD_PIRL_CUSTOM_RATE_LIMIT  *customSetup)
{
	MSD_STATUS retVal;

	retVal = Amethyst_gpirlCustomSetup_sr2cIntf(tgtRate, tgtBstSize, countMode, customSetup);

	return retVal;
}

/*******************************************************************************
* msdIRLReset
*
* DESCRIPTION:
*       This routine initializes all PIRL Resources for all ports.
*
* INPUTS:
*       devNum  - physical devie number
*
* OUTPUTS:
*       None
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*       MSD_NOT_SUPPORTED - device not support
*
* COMMENTS:
*       None
*
*******************************************************************************/
MSD_STATUS Amethyst_gpirlInitialize
(
    IN  struct switch_dev *dev,
    IN MSD_LPORT port,
    IN MSD_U32 res
)
{
    MSD_STATUS       retVal;    /* Functions return value */
    AMETHYST_MSD_PIRL_OPERATION    op;
	AMETHYST_MSD_PIRL_OP_DATA        opData = {0};

    MSD_DBG_INFO(SW_DEBUG"Amethyst_gpirlInitialize Called.\n");

    op = Amethyst_PIRL_WRITE_RESOURCE;

    retVal = Amethyst_pirlOperationPerform(dev, op, port, res, 0, &opData);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "Amethyst_pirlOperationPerform Amethyst_PIRL_INIT_ALL_RESOURCE returned: %d.\n",retVal);
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gpirlInitialize Exit.\n");
    return retVal;

}

/*******************************************************************************
* Amethyst_gpirlWriteResource
*
* DESCRIPTION:
*       This function writes Resource bucket parameter to the given resource of port
*
* INPUTS:
*       irlPort	- tarAmethyst_get port
*		irlRes  - resource unit to be accessed
*       pirlData - IRL Resource data
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK - on success,
*       MSD_FAIL - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*       None
*
*******************************************************************************/
MSD_STATUS Amethyst_gpirlWriteResource
(
    IN  struct switch_dev *dev,
    IN  MSD_LPORT    	irlPort,
	IN  MSD_U32        	irlRes,
    IN  AMETHYST_MSD_PIRL_DATA    *pirlData
)
{
    MSD_STATUS       retVal;            /* Functions return value */
    u16          data[9];     /* temporary Data storage */
    AMETHYST_MSD_PIRL_OPERATION    op;
    AMETHYST_MSD_PIRL_OP_DATA        opData;
	u8			port;
    int                i;

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gpirlWriteResource Called.\n");

    op = Amethyst_PIRL_WRITE_RESOURCE;
	port = irlPort;

	if(irlRes > 7)
	{
		MSD_DBG_ERROR(SW_DEBUG "Bad irlRes: %u.\n", (unsigned int)irlRes);
        return MSD_BAD_PARAM;
	}

    /* reg0 data */
    data[0] = (u16)(pirlData->bktTypeMask & 0xFFFF);        /* Bit[15:0] : Traffic Type   */

    /* reg1 data */
	data[1] = (u16)((((u16)pirlData->samplingMode?1:0) << 14) |	/* Bit[14] : sampling Mode */
				(((u16)pirlData->colorAware?1:0) << 13) |			/* Bit[13] : Color Aware */
				(((u16)pirlData->accountGrnOverflow?1:0) << 12) |    /* Bit[12] : AcctForGrnOvflow */
				(((u16)pirlData->accountQConf?1:0) << 11) |		/* Bit[11] : AcctForQConf */
				(((u16)pirlData->accountFiltered?1:0) << 10) |   /* Bit[10] : AcctForFiltered */
				(((u16)pirlData->priAndPt?1:0) << 9) |			/* Bit[9] : Priority And Packet Type */
				(((u16)pirlData->useFPri?1:0) << 8) |			/* Bit[8] : Frame Priority vs Queue Priority */
				((u16)(pirlData->priSelect&0xFF))); 					/* Bit[7:0] : Priority Select */

    /* reg2 data */
	data[2] = (u16)((((u16)pirlData->customSetup.countMode)<<14) |
				(((u16)pirlData->tcamFlows?1:0)<<13) |
				((u16)((pirlData->customSetup.bktIncrement)&0x1FFF)));

    /* reg3 data */
	data[3] = (u16)(pirlData->customSetup.bktRateFactorGrn & 0xFFFF);    /* Bit[15:0] : Bucket Rate Factor Green */

    /* reg4 data */
    data[4] = (u16)(pirlData->customSetup.cbsLimit & 0xFFFF);   /* Bit[15:0] : CBS Limit[15:0] */

    /* reg5 data */
    data[5] = (u16)((pirlData->customSetup.cbsLimit >> 16) & 0xFF);   /* Bit[7:0] : CBS Limit[23:16] */
	
    /* reg6 data */
	data[6] = (u16)(pirlData->customSetup.bktRateFactorYel & 0xFFFF);		/* Bit[15:0] : Bucket Rate Factor Yellow */

	/* reg7 data */
	data[7] = (u16)(pirlData->customSetup.ebsLimit & 0xFFFF);	/* Bit[15:0] : EBS Limit[15:0] */

	/* reg8 data */
	data[8] = (u16)(((pirlData->fcPriority&0x7) << 13) |
				((pirlData->fcMode?1:0) << 12) |
				((pirlData->actionMode?1:0) << 11) |
				((pirlData->customSetup.ebsLimit >> 16) & 0xFF));	

    for(i=0; i<9; i++)
    {
        opData.irlData = data[i];

        retVal = Amethyst_pirlOperationPerform(dev, op, port, irlRes, i, &opData);
        if (retVal != MSD_OK)
        {
            MSD_DBG_ERROR(SW_DEBUG "Amethyst_pirlOperationPerform Amethyst_PIRL_WRITE_RESOURCE returned: %d.\n", retVal);
            return retVal;
        }
    }

    MSD_DBG_INFO( SW_DEBUG "Amethyst_gpirlWriteResource Exit.\n");
    return MSD_OK;  
}


/*******************************************************************************
* Amethyst_gpirlWriteResource
*
* DESCRIPTION:
*       This function writes Resource bucket parameter to the given resource of port
*
* INPUTS:
*       irlPort	- tarAmethyst_get port
*		irlRes  - resource unit to be accessed
*       pirlData - IRL Resource data
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK - on success,
*       MSD_FAIL - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*       None
*
*******************************************************************************/
MSD_STATUS Amethyst_gpirlWriteResourceIntf
(
IN  struct switch_dev *dev,
IN  MSD_LPORT    	irlPort,
IN  MSD_U32        	irlRes,
IN  MSD_PIRL_DATA    *pirlData
)
{
	AMETHYST_MSD_PIRL_DATA pirl;

	if (NULL == pirlData)
	{
		MSD_DBG_ERROR(SW_DEBUG "Input param MSD_PIRL_DATA in Amethyst_gpirlWriteResourceIntf is NULL. \n");
		return MSD_BAD_PARAM;
	}

	pirl.bktTypeMask = pirlData->bktTypeMask;
	pirl.tcamFlows = pirlData->tcamFlows;
	pirl.priAndPt = pirlData->priAndPt;
	pirl.useFPri = pirlData->useFPri;
	pirl.priSelect = pirlData->priSelect;
	pirl.colorAware = pirlData->colorAware;
	pirl.accountGrnOverflow = pirlData->accountGrnOverflow;
	pirl.accountQConf = pirlData->accountQConf;
	pirl.accountFiltered = pirlData->accountFiltered;
	pirl.samplingMode = pirlData->samplingMode;
	pirl.actionMode = pirlData->actionMode;
	pirl.fcMode = pirlData->fcMode;
	pirl.fcPriority = pirlData->fcPriority;
	pirl.customSetup.isValid = pirlData->customSetup.isValid;
	pirl.customSetup.ebsLimit = pirlData->customSetup.ebsLimit;
	pirl.customSetup.cbsLimit = pirlData->customSetup.cbsLimit;
	pirl.customSetup.bktIncrement = pirlData->customSetup.bktIncrement;
	pirl.customSetup.bktRateFactorGrn = pirlData->customSetup.bktRateFactorGrn;
	pirl.customSetup.bktRateFactorYel = pirlData->customSetup.bktRateFactorYel;

	pirl.customSetup.countMode = pirlData->customSetup.countMode;

	return Amethyst_gpirlWriteResource(dev, irlPort, irlRes, &pirl);

}
#define PKT_TYPE_UNKNOWN_UNI 	(0)
#define PKT_TYPE_BC 			(1)
#define PKT_TYPE_MC				(2)
#define PKT_TYPE_INGRESS		(3)
#define PKT_TYPE_BC_MC			(4)

MSD_STATUS Amethyst_msdPIRL_set(struct switch_dev *dev,MSD_LPORT port, MSD_U32 irlRes,MSD_U32 speed)
{
	MSD_STATUS status;

	MSD_U32 tgtRate;
	MSD_U32 tgtBstSize;
    MSD_PIRL_CUSTOM_RATE_LIMIT customSetup;
    MSD_PIRL_COUNT_MODE countMode;

	MSD_PIRL_DATA pirlData;

	MSD_LPORT irlPort = port;

	/*
	*    Initialize all IRL Resources for all ports.
	*/
	if ((status = Amethyst_gpirlInitialize(dev,irlPort,irlRes)) != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "msdIRLReset returned fail.\n");
		return status;
	}

	/*
	*    Calculate CBS/EBS/BI/BRF according to user specific target rate, target burst size.
	*    Get customSetup for next step (to initialize pirl data)
	*/
	tgtRate = speed; /*rate limit target is 1Mbps*/
	tgtBstSize = 3000; /*target burst size should be larger than maximum frame size, here is 3000 byte*/

	memset(&customSetup, 0, sizeof(MSD_PIRL_CUSTOM_RATE_LIMIT)); /* Set all bit to 0 by default */
	countMode = MSD_PIRL_COUNT_ALL_LAYER2;           /* Rate limit for Layer 2 bit rate*/
	if ((status = msdIRLCustomSetup(dev, tgtRate, tgtBstSize, countMode, &customSetup)) != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG"msdIRLCustomSetup returned fail.\n");
		return status;
	}

	/* Initialize pirl data */
	memset(&pirlData, 0, sizeof(MSD_PIRL_DATA)); /* Set all bit to 0 by default */
	pirlData.priAndPt = MSD_FALSE ;				  /* rate limit from frame that matches both PktType and specified priority*/
	if(irlRes == PKT_TYPE_UNKNOWN_UNI)
	{
		pirlData.bktTypeMask = 1 << 0;                /* rate limit for unknown unicast frame */
	}
	else if(irlRes == PKT_TYPE_BC)
	{
		pirlData.bktTypeMask = 1 << 2;
	}
	else if(irlRes == PKT_TYPE_MC)
	{
		pirlData.bktTypeMask = 1 << 1;
	}
	else if(irlRes == PKT_TYPE_INGRESS)
	{
		pirlData.bktTypeMask = 0xffff;
	}
	else if(irlRes == PKT_TYPE_BC_MC)
	{
		pirlData.bktTypeMask = (1 << 1)|(1 << 2);
	}
    pirlData.actionMode = MSD_PIRL_ACTION_DROP;       /* discard packets if over rate */
	pirlData.accountQConf = MSD_TRUE;

	pirlData.customSetup.isValid = customSetup.isValid;
	pirlData.customSetup.ebsLimit = customSetup.ebsLimit;
	pirlData.customSetup.cbsLimit = customSetup.cbsLimit;
	pirlData.customSetup.bktIncrement = customSetup.bktIncrement;
	pirlData.customSetup.bktRateFactorGrn = customSetup.bktRateFactorGrn;
	pirlData.customSetup.bktRateFactorYel = customSetup.bktRateFactorYel;
	pirlData.customSetup.IRLConstant = customSetup.IRLConstant;
	pirlData.customSetup.countMode = customSetup.countMode;
	MSD_DBG_ERROR(SW_DEBUG "pirlData ebsLimit %d cbsLimit %d bktIncrement %d bktRateFactorGrn %d bktRateFactorYel %d IRLConstant %d countMode %d.\n",
		pirlData.customSetup.ebsLimit,pirlData.customSetup.cbsLimit,pirlData.customSetup.bktIncrement,pirlData.customSetup.bktRateFactorGrn,
		pirlData.customSetup.bktRateFactorYel,pirlData.customSetup.IRLConstant,pirlData.customSetup.countMode);
	/* Writes resource bucket parameters to the given resource of the port. */
	//irlPort = 1;
	//irlRes = 0;
	if ((status = Amethyst_gpirlWriteResourceIntf(dev, irlPort, irlRes, &pirlData)) != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "msdIRLResourceWrite returned fail.\n");
		return status;
	}

	MSD_DBG_INFO(SW_DEBUG "sample_msdPIRL_UnknownUC_FPRI sample code success.\n");
	return MSD_OK;
}



int mvsw61xx_set_port_bc_sc(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{

	int portNum = val->port_vlan;
	int speed = val->value.i;
	if(0 == speed)
	{
		Amethyst_gpirlInitialize(dev,portNum,PKT_TYPE_BC);
	}
	else
	{
		Amethyst_msdPIRL_set(dev,portNum,PKT_TYPE_BC,speed);
	}
        return 0;
}

int mvsw61xx_set_port_mc_sc(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{

	int portNum = val->port_vlan;
	int speed = val->value.i;
	if(0 == speed)
	{
		Amethyst_gpirlInitialize(dev,portNum,PKT_TYPE_MC);
	}
	else
	{
		Amethyst_msdPIRL_set(dev,portNum,PKT_TYPE_MC,speed);
	}
        return 0;
}


int mvsw61xx_set_port_ingress(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{

	int portNum = val->port_vlan;
	int speed = val->value.i;
	if(0 == speed)
	{
		Amethyst_gpirlInitialize(dev,portNum,PKT_TYPE_INGRESS);
	}
	else
	{
		Amethyst_msdPIRL_set(dev,portNum,PKT_TYPE_INGRESS,speed);
	}
        return 0;
}


int mvsw61xx_set_port_bc_mc_sc(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{

	int portNum = val->port_vlan;
	int speed = val->value.i;
	if(0 == speed)
	{
		Amethyst_gpirlInitialize(dev,portNum,PKT_TYPE_BC_MC);
	}
	else
	{
		Amethyst_msdPIRL_set(dev,portNum,PKT_TYPE_BC_MC,speed);
	}
	return 0;
}

/*###################################### end IRL  ##############################################*/



/*******************************************************************************
* Amethyst_grcSetELimitMode
*
* DESCRIPTION:
*       This routine sets Egress Rate Limit counting mode.
*       The supported modes are as follows:
*            AMETHYST_MSD_ELIMIT_FRAME -
*                Count the number of frames
*            AMETHYST_MSD_ELIMIT_LAYER1 -
*                Count all Layer 1 bytes: 
*                Preamble (8bytes) + Frame's DA to CRC + IFG (12bytes)
*            AMETHYST_MSD_ELIMIT_LAYER2 -
*                Count all Layer 2 bytes: Frame's DA to CRC
*            AMETHYST_MSD_ELIMIT_LAYER3 -
*                Count all Layer 1 bytes: 
*                Frame's DA to CRC - 18 - 4 (if frame is tagged)
*
* INPUTS:
*       port - logical port number
*       mode - AMETHYST_MSD_ELIMIT_MODE enum type
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*		None. 
*
*******************************************************************************/
MSD_STATUS Amethyst_grcSetELimitMode
(
    IN  struct switch_dev *dev,
    IN  MSD_LPORT			port,
    IN  AMETHYST_MSD_ELIMIT_MODE      mode
)
{
    u16            data;
    MSD_STATUS       retVal;         /* Functions return value.      */
    u8           hwPort;        /* Physical port.               */

    MSD_DBG_INFO(("Amethyst_grcSetELimitMode Called.\n"));

    hwPort = port;

    data = (u16)mode & 0x3;

    /* Set the Elimit mode.            */
    retVal = msdSetAnyRegField(dev, hwPort, AMETHYST_QD_REG_EGRESS_RATE_CTRL2, 14, 2, data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "ERROR to read AMETHYST_QD_REG_EGRESS_RATE_CTRL2 Register.\n");
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_grcSetELimitMode Exit.\n");
    return MSD_OK;
}

					   
/*******************************************************************************
* Amethyst_grcSetEgressRate
*
* DESCRIPTION:
*       This routine sets the port's egress data limit.
*        
*
* INPUTS:
*       port      - logical port number.
*       rateType  - egress data rate limit (MSD_ERATE_TYPE union type). 
*                    union type is used to support multiple devices with the
*                    different formats of egress rate.
*                    MSD_ERATE_TYPE has the following fields:
*                        kbRate - rate in kbps that should used with the AMETHYST_MSD_ELIMIT_MODE of 
*                                AMETHYST_MSD_ELIMIT_LAYER1,
*                                AMETHYST_MSD_ELIMIT_LAYER2, or 
*                                AMETHYST_MSD_ELIMIT_LAYER3 (see Amethyst_grcSetELimitMode)
*                            64kbps ~ 1Mbps    : increments of 64kbps,
*                            1Mbps ~ 100Mbps   : increments of 1Mbps, and
*                            100Mbps ~ 1000Mbps: increments of 10Mbps
*                            1Gbps ~ 5Gbps: increments of 100Mbps
*                            Therefore, the valid values are:
*                                64, 128, 192, 256, 320, 384,..., 960,
*                                1000, 2000, 3000, 4000, ..., 100000,
*                                110000, 120000, 130000, ..., 1000000
*                                1100000, 1200000, 1300000, ..., 5000000.
*                        fRate - frame per second that should used with AMETHYST_MSD_ELIMIT_MODE of 
*                                AMETHYST_MSD_PIRL_ELIMIT_FRAME
*                            Valid values are between 3815 and 14880000
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*		None.
*
*******************************************************************************/
MSD_STATUS Amethyst_grcSetEgressRate
(
    IN struct switch_dev *dev,
    IN MSD_LPORT        port,
    IN AMETHYST_MSD_ELIMIT_MODE mode,
    IN MSD_U32	rate
)
{
    MSD_STATUS    retVal;         /* Functions return value.      */
    u16        data;
    MSD_U32        eDec;
    u8        hwPort;        /* Physical port.               */

    MSD_DBG_INFO(("Amethyst_grcSetEgressRate Called.\n"));

    hwPort = port;

    switch (mode)
    {
    case AMETHYST_MSD_ELIMIT_FRAME:
    case AMETHYST_MSD_ELIMIT_LAYER1:
    case AMETHYST_MSD_ELIMIT_LAYER2:
    case AMETHYST_MSD_ELIMIT_LAYER3:
        break;
    default:
        MSD_DBG_ERROR(("Failed (Bad Mode).\n"));
        return MSD_BAD_PARAM;
    }

    if((retVal = Amethyst_grcSetELimitMode(dev,port,mode)) != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "Amethyst_grcSetELimitMode returned: %d.\n",retVal);
        return retVal;
    }

    if (mode == AMETHYST_MSD_ELIMIT_FRAME)    
    {
        if (rate == 0) /* disable egress rate limit */
        {
            eDec = 0;
            data = 0;
        }
        else if((rate < 3815)  || (rate > 14880000))
        {
            MSD_DBG_ERROR(("Failed (Bad Rate).\n"));
            return MSD_BAD_PARAM;
        }
        else
        {
            eDec = 1;
            data = (u16)AMETHYST_MSD_GET_RATE_LIMIT_PER_FRAME(rate,eDec);
        }
    }
    else
    {
        if(rate == 0)
        {
            eDec = 0;
        }
        else if(rate < 1000)    /* less than 1Mbps */
        {
            /* it should be divided by 64 */
            if (rate % 64)
            {
                if(rate < 64)
                {
					rate = 64;
				}
				else if(rate % 64 < 32)
				{
					rate = (rate/64)*64;
				}
				else
				{
					rate = ((rate / 64) + 1) * 64;
				}
            }
            eDec = rate/64;
        }
        else if(rate <= 100000)    /* less than or equal to 100Mbps */
        {
            /* it should be divided by 1000 */
            if (rate % 1000)
            {
                if(rate % 1000 < 500)
				{
					rate = (rate/1000)*1000;
				}
				else
				{
					rate = ((rate / 1000) + 1) * 1000;
				}
            }
            eDec = rate/1000;
        }
        else if(rate <= 1000000)    /* less than or equal to 1000Mbps */
        {
            /* it should be divided by 10000 */
            if(rate % 1000000 < 500000)
				{
					rate = (rate/1000000)*1000000;
				}
				else
				{
					rate = ((rate / 1000000) + 1) * 1000000;
				}
            eDec = rate/10000;
        }
        else if(rate <= 10000000)    /* less than or equal to 10Gbps */
        {
            /* it should be divided by 100000 */
           if(rate % 10000000 < 5000000)
				{
					rate = (rate/10000000)*10000000;
				}
				else
				{
					rate = ((rate / 10000000) + 1) * 10000000;
				}
            eDec = rate/100000;
        }
        else
        {
            MSD_DBG_ERROR(SW_DEBUG "Failed (Bad Rate).\n");
            return MSD_BAD_PARAM;
        }

        if(rate == 0)
        {
            data = 0;
        }
        else
        {
            data = (u16)AMETHYST_MSD_GET_RATE_LIMIT_PER_BYTE(rate,eDec);
        }
    }

	retVal = msdSetAnyRegField(dev,  hwPort, AMETHYST_QD_REG_EGRESS_RATE_CTRL, 0, 7, (u16)eDec);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "ERROR to write AMETHYST_QD_REG_EGRESS_RATE_CTRL Register.\n");
        return retVal;
    }

	retVal = msdSetAnyRegField(dev,  hwPort, AMETHYST_QD_REG_EGRESS_RATE_CTRL2, 0, 14, (u16)data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "ERROR to write AMETHYST_QD_REG_EGRESS_RATE_CTRL2 Register.\n");
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_grcSetEgressRate Exit.\n");
    return MSD_OK;
}

int mvsw61xx_set_port_egress(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{

	int portNum = val->port_vlan;
	int speed = val->value.i;

	Amethyst_grcSetEgressRate(dev,portNum,AMETHYST_MSD_ELIMIT_LAYER2,speed);
        return 0;
}

/*######################################## port mirror ##############################################*/


/*******************************************************************************
* Amethyst_gpolicyGetMgmtCtrl
*
* DESCRIPTION:
*       Gets policy and mgmt control data.
*
* INPUTS:
*       point - Pointer to the Monitor and mgmt control register.
*           The Pointer bits are used to access the Index registers as follows:
*              0x00 to 0x0F: Polcy Trap/Mirror/Discard Policy Selections
*              0x10 to 0x1F: Policy Configurations
*              0x20 to 0x2F: Reserved DA's to CPU & Alt CPU Enables
*              0x30 to 0x3F: Trap/Mirror/CPU Port Settings
*
* OUTPUTS:
*       data  - Octet Data of the Policy Control Register referenced by the
*               by the point above.
*
* RETURNS:
*       MSD_OK      - on success.
*       MSD_FAIL    - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*       MSD_NOT_SUPPORTED - device not supported
*
* COMMENTS:
*       None
*
*******************************************************************************/
MSD_STATUS Amethyst_gpolicyGetMgmtCtrl
(
IN struct switch_dev *dev,
IN MSD_LPORT	port,
IN u8		pointer,
OUT u8		*data
)
{
	MSD_STATUS    retVal;         /* Functions return value.      */
	u8        hwPort;         /* the physical port number     */
	u8		 phyAddr;
	u16   count, tmpData;
	MSD_DBG_INFO(SW_DEBUG "Amethyst_gpolicyGetMgmtCtrl Called.\n");

	if (pointer >= 64)
	{
		MSD_DBG_INFO(SW_DEBUG "MSD_BAD_PARAM (pointer: %u). It should be within [0, 63].\n", pointer);
		return MSD_BAD_PARAM;
	}

	/* translate LPORT to hardware port */
	hwPort = port;
	phyAddr = hwPort;

	count = 5;
	tmpData = 1;
	while (tmpData == 1)
	{
		retVal = msdGetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_POLICY_CONTROL, 15, 1, &tmpData);
		if (retVal != MSD_OK)
		{
			return retVal;
		}
		if ((count--) == 0)
		{
			return MSD_FAIL;
		}
	}

	tmpData = (u16)(((pointer & 0x3F) << 8) | 0);
	retVal = msdSetAnyReg(dev, phyAddr, AMETHYST_QD_REG_POLICY_CONTROL, tmpData);
	if (retVal != MSD_OK)
	{
		MSD_DBG_INFO(SW_DEBUG "Failed.\n");
		return retVal;
	}
	retVal = msdGetAnyReg(dev, phyAddr, AMETHYST_QD_REG_POLICY_CONTROL, &tmpData);
	if (retVal != MSD_OK)
	{
		return retVal;
	}
	*data = (u8)(tmpData & 0xff);

	MSD_DBG_INFO(SW_DEBUG "Amethyst_gpolicyGetMgmtCtrl Exit.\n");
	return retVal;
}
/*******************************************************************************
* Amethyst_gpolicySetMgmtCtrl
*
* DESCRIPTION:
*       Sets policy and mgmt control data.
*
* INPUTS:
*       point - Pointer to the Monitor and mgmt control register.
*           The Pointer bits are used to access the Index registers as follows:
*              0x00 to 0x0F: Polcy Trap/Mirror/Discard Policy Selections
*              0x10 to 0x1F: Policy Configurations
*              0x20 to 0x2F: Reserved DA's to CPU & Alt CPU Enables
*              0x30 to 0x3F: Trap/Mirror/CPU Port Settings
*       data  - Octet Data of the Policy Control Register referenced by the
*               by the point above.
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK      - on success.
*       MSD_FAIL    - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*       MSD_NOT_SUPPORTED - device not supported
*
* COMMENTS:
*       None
*
*******************************************************************************/
MSD_STATUS Amethyst_gpolicySetMgmtCtrl
(
IN struct switch_dev *dev,
IN MSD_LPORT	port,
IN u8		pointer,
IN u8		data
)
{
	MSD_STATUS    retVal;         /* Functions return value.      */
	u8        hwPort;         /* the physical port number     */
	u8		 phyAddr;
	u16	tmpData;
	int  count = 0x10;
   
	MSD_DBG_INFO(SW_DEBUG "Amethyst_gpolicySetMgmtCtrl Called.\n");

	if (pointer >= 64)
	{
		MSD_DBG_INFO(SW_DEBUG "MSD_BAD_PARAM (pointer: %u). It should be within [0, 63].\n", pointer);
		return MSD_BAD_PARAM;
	}

	/* translate LPORT to hardware port */
	hwPort = port;
	phyAddr = hwPort;

	tmpData = 1;
	while (tmpData == 1)
	{
		retVal = msdGetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_POLICY_CONTROL, 15, 1, &tmpData);
		if (retVal != MSD_OK)
		{
			return retVal;
		}
		if ((count--) == 0)
		{
			return MSD_FAIL;
		}
	}

	tmpData = (u16)((2 << 14) | ((pointer & 0x3F) << 8) | (u8)data);
	retVal = msdSetAnyReg(dev, phyAddr, AMETHYST_QD_REG_POLICY_CONTROL, tmpData);
	if (retVal != MSD_OK)
	{
		MSD_DBG_INFO(SW_DEBUG "Failed.\n");
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG "Amethyst_gpolicySetMgmtCtrl Exit.\n");
	return retVal;
}


/*******************************************************************************
* Amethyst_gpolicySetIngressMonitorDest
*
* DESCRIPTION:
*       This routine sets Ingress Monitor Destination Port. Frames that are
*       targeted toward an Ingress Monitor Destination go out the port number
*       indicated in these bits. This includes frames received on a Marvell Tag port
*       with the Ingress Monitor type, and frames received on a Network port that
*       is enabled to be the Ingress Monitor Source Port.
*       If the Ingress Monitor Destination Port resides in this device these bits
*       should point to the Network port where these frames are to egress. If the
*       Ingress Monitor Destination Port resides in another device these bits
*       should point to the Marvell Tag port in this device that is used to get
*       to the device that contains the Ingress Monitor Destination Port.
*
* INPUTS:
*       port	- the logical port number.
*		imd		- The ingress monitor destination port.
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK      - on success.
*       MSD_FAIL    - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*       MSD_NOT_SUPPORTED - device not supported
*
* COMMENTS:
*       None.
*
*******************************************************************************/
MSD_STATUS Amethyst_gpolicySetIngressMonitorDest
(
IN struct switch_dev *dev,
IN MSD_LPORT	port,
IN MSD_LPORT    imd
)
{
	MSD_STATUS       retVal;         /* Functions return value.      */
	u8           hwPort;         /* the physical port number     */
	u8			data;

	MSD_DBG_INFO(SW_DEBUG "Amethyst_gpolicySetIngressMonitorDest Called.\n");

	/* translate LPORT to hardware port */
	hwPort = imd;

	retVal = Amethyst_gpolicyGetMgmtCtrl(dev, port, 0x30, &data);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_gpolicyGetMgmtCtrl returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	data = ((data & 0xE0) | hwPort);
	retVal = Amethyst_gpolicySetMgmtCtrl(dev, port, 0x30, (u8)data);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_gpolicySetMgmtCtrl returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}


	MSD_DBG_INFO(SW_DEBUG "Amethyst_gpolicySetIngressMonitorDest Exit.\n");
	return retVal;
}

/*******************************************************************************
* Amethyst_gprtSetIngressMonitorSource
*
* DESCRIPTION:
*       When this be is cleared to a zero, normal network switching occurs.
*       When this bit is set to a one, any frame that ingresses in this port will
*       also be sent to the IngressMonitorDest Port
*
* INPUTS:
*       port - the logical port number.
*       mode - MSD_TRUE to set IngressMonitorSource, MSD_FALSE otherwise
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*		None.
*
*******************************************************************************/
MSD_STATUS Amethyst_gprtSetIngressMonitorSource
(
    struct switch_dev *dev,
    IN MSD_LPORT     port,
    IN MSD_BOOL        mode
)
{
    u16          data;           
    MSD_STATUS       retVal;         /* Functions return value.      */
    u8           hwPort;         /* the physical port number     */
	u8			phyAddr;

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtSetIngressMonitorSource Called.\n");

    /* translate LPORT to hardware port */
    hwPort = port;
	phyAddr = hwPort;
    /* translate BOOL to binary */
    MSD_BOOL_2_BIT(mode, data);

    /* Set IngressMonitorSource. */
    retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PORT_CONTROL2, 4, 1, data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "Set IngressMonitorSource bit error\n");
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtSetIngressMonitorSource Exit.\n");
    return retVal;
}

/*******************************************************************************
* Amethyst_gsysSetEgressMonitorDest
*
* DESCRIPTION:
*       This routine sets Egress Monitor Destination Port. Frames that are 
*       tarAmethyst_geted toward an Egress Monitor Destination go out the port number 
*       indicated in these bits. This includes frames received on a Marvell Tag port
*       with the Egress Monitor type, and frames transmitted on a Network port that 
*       is enabled to be the Egress Monitor Source Port.
*       If the Egress Monitor Destination Port resides in this device these bits 
*       should point to the Network port where these frames are to egress. If the 
*       Egress Monitor Destination Port resides in another device these bits 
*       should point to the Marvell Tag port in this device that is used to Amethyst_get 
*       to the device that contains the Egress Monitor Destination Port.
*
* INPUTS:
*       port  - the logical port number.If port = 0x1F, EMD is disabled
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK - on success,
*       MSD_FAIL - on error.
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*       None.
*
*******************************************************************************/
MSD_STATUS Amethyst_gsysSetEgressMonitorDest
(
    IN struct switch_dev *dev,
    IN MSD_LPORT        port
)
{
    MSD_STATUS       retVal;         /* Functions return value.      */
    u8           hwPort;         /* the physical port number     */

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gsysSetEgressMonitorDest Called.\n");

    /* translate LPORT to hardware port */
	hwPort = port;


	retVal = msdSetAnyRegField(dev, AMETHYST_GLOBAL2_DEV_ADDR, AMETHYST_QD_REG_MANAGEMENT, 0, 5, (u8)hwPort);
    if(retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_gsysSetMonitorMgmtCtrl returned: %s.\n", msdDisplayStatus(retVal));
	    return retVal;
	}
	
    MSD_DBG_INFO(SW_DEBUG "Amethyst_gsysSetEgressMonitorDest Exit.\n");
    return retVal;
}

/*******************************************************************************
* Amethyst_gprtSetEgressMonitorSource
*
* DESCRIPTION:
*       When this bit is cleared to a zero, normal network switching occurs.
*       When this bit is set to a one, any frame that egresses out this port will
*       also be sent to the EgressMonitorDest Port
*
* INPUTS:
*       port - the logical port number.
*       mode - MSD_TRUE to set EgressMonitorSource, MSD_FALSE otherwise
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*		None.
*
*******************************************************************************/
MSD_STATUS Amethyst_gprtSetEgressMonitorSource
(
	struct switch_dev *dev,
	IN MSD_LPORT     port,
 	IN MSD_BOOL        mode
)
{
    u16          data;           
    MSD_STATUS       retVal;         /* Functions return value.      */
    u8           hwPort;         /* the physical port number     */
	u8			phyAddr;

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtSetEgressMonitorSource Called.\n");

    /* translate LPORT to hardware port */
    hwPort = port;
	phyAddr = hwPort;
    /* translate BOOL to binary */
    MSD_BOOL_2_BIT(mode, data);

    /* Set EgressMonitorSource. */
	retVal = msdSetAnyRegField(dev,  phyAddr, AMETHYST_QD_REG_PORT_CONTROL2, 5, 1, data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "Set EgressMonitorSource bit error\n");
        return retVal;
    }
    
    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtSetEgressMonitorSource Exit.\n");
    return retVal;
}


int mvsw61xx_set_port_mirrors(struct switch_dev *dev, const struct switch_attr *attr, struct switch_val *val)
{
    u32 ret = 0;
    u32 port = val->port_vlan;
    u32 logicalPort = 0;
    u32 rx_portmask = dev->ports + 1;
    u32 tx_portmask = dev->ports + 1;
    struct switch_port *ports = val->value.ports;
    int i;

	logicalPort = port;

	for(i = 0;i<11;i++)
	{
		Amethyst_gprtSetEgressMonitorSource(dev,i,MSD_FALSE);
		Amethyst_gprtSetIngressMonitorSource(dev,i,MSD_FALSE);
	}

	if(port == 9)
	{
		/* if trans CPU port,turn off all mirrors  */
		return 0;
	}

	for (i = 0; i < val->len; i ++)
	{
		MSD_DBG_INFO(SW_DEBUG "ports[%d].id = %d, flags = %x", i, ports[i].id, ports[i].flags);
		if (ports[i].id == port)
		{
			MSD_DBG_INFO(SW_DEBUG "%s:%d: invalid port = %d, dev->ports = %d, mirroring_port = %d\r\n", __func__, __LINE__, ports[i].id, dev->ports, port);
			return MSD_BAD_PARAM;
		}

		if (ports[i].flags & (1 << SWITCH_PORT_FLAG_TAGGED))
		{
			MSD_DBG_INFO(SW_DEBUG "Add port %d to tx portmask", ports[i].id);
			tx_portmask = ports[i].id;
		}
		else
		{
			MSD_DBG_INFO(SW_DEBUG "Add port %d to rx portmask", ports[i].id);
			rx_portmask = ports[i].id;
		}

		if(tx_portmask <= dev->ports)
		{
			Amethyst_gprtSetEgressMonitorSource(dev,tx_portmask,MSD_TRUE);
			Amethyst_gsysSetEgressMonitorDest(dev,logicalPort);
		}
		else
		{
			Amethyst_gprtSetEgressMonitorSource(dev,tx_portmask,MSD_FALSE);
		}

		if(rx_portmask <= dev->ports)
		{
			Amethyst_gprtSetIngressMonitorSource(dev,rx_portmask,MSD_TRUE);
			Amethyst_gpolicySetIngressMonitorDest(dev, rx_portmask ,logicalPort);
		}
		else
		{
			Amethyst_gprtSetIngressMonitorSource(dev,rx_portmask,MSD_FALSE);
		}
	}

	return ret;
}   

/*#################################### end mirrors  ##########################################################*/


/*######################################### speed & duplex & FC  #############################################*/

/*******************************************************************************
* Amethyst_gprtSetForceSpeed
*
* DESCRIPTION:
*       This routine forces switch MAC speed.
*
* INPUTS:
*       port - the logical port number.
*       mode - AMETHYST_MSD_PORT_FORCED_SPEED_MODE (10, 100, 200, 1000, 2.5g, 10g,or No Speed Force)
*
* OUTPUTS:
*       None
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*		None.
*
*******************************************************************************/
MSD_STATUS Amethyst_gprtSetForceSpeed
(
IN struct switch_dev *dev,
IN MSD_LPORT     port,
IN AMETHYST_MSD_PORT_FORCED_SPEED_MODE  mode
)
{
    MSD_STATUS       retVal;
    u8           hwPort;
    u8			phyAddr;
    u16			data1;
    u16			data2;
    u16			linkStatus;

    MSD_DBG_INFO(("Amethyst_gprtSetForceSpeed Called.\n"));

    /* translate LPORT to hardware port */
    hwPort = port;
    phyAddr = hwPort;
    /* Set the ForceSpeed bit.  */
    if (mode == Amethyst_PORT_DO_NOT_FORCE_SPEED)
    {
		/*Get the force link status and then force link down*/
		if ((retVal = msdGetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 4, 2, &linkStatus)) != MSD_OK)
		{
			MSD_DBG_ERROR(SW_DEBUG "Get link status Failed\n");
			return retVal;
		}
		if ((retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 4, 2, 1)) != MSD_OK)
		{
			MSD_DBG_ERROR(SW_DEBUG "Force Link down failed\n");
			return retVal;
		}

        if ((retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 13, 1, 0)) != MSD_OK)
        {
            MSD_DBG_ERROR(SW_DEBUG "Set Force Speed bit error\n");
            return retVal;
        }
    }
    else
    {
        switch (mode)
        {
        case Amethyst_PORT_FORCE_SPEED_10M:
            data1 = 0;
            data2 = 0;
            break;
        case Amethyst_PORT_FORCE_SPEED_100M:
            data1 = 0;
            data2 = 1;
            break;
        case Amethyst_PORT_FORCE_SPEED_200M:
            if (port != 0)
            {
                MSD_DBG_ERROR(SW_DEBUG "Failed (Bad Port), Only port 0 support 200M.\n");
				return MSD_NOT_SUPPORTED;
            }
            data1 = 1;
            data2 = 1;
            break;
        case Amethyst_PORT_FORCE_SPEED_1000M:
            data1 = 0;
            data2 = 2;
            break;
        case Amethyst_PORT_FORCE_SPEED_2_5G:
			if ((port != 0) && (port != 9) && (port != 10))
            {
                MSD_DBG_ERROR(SW_DEBUG "Failed (Bad Port), Only port 0, 9, 10 support 2.5G.\n");
				return MSD_NOT_SUPPORTED;
            }
            data1 = 1;
            data2 = 2;
            break;
		case Amethyst_PORT_FORCE_SPEED_5G:
			if ((port != 0) && (port != 9) && (port != 10))
			{
				MSD_DBG_ERROR(SW_DEBUG "Failed (Bad Port), Only port 0, 9, 10 support 5G.\n");
				return MSD_NOT_SUPPORTED;
			}
			data1 = 1;
			data2 = 3;
			break;
        case Amethyst_PORT_FORCE_SPEED_10G:
			if ((port != 0) && (port != 9) && (port != 10))
            {
                MSD_DBG_ERROR(SW_DEBUG "Failed (Bad Port), Only port 0, 9, 10 support 10G.\n");
				return MSD_NOT_SUPPORTED;
            }
            data1 = 0;
            data2 = 3;
            break;
        default:
            MSD_DBG_ERROR(SW_DEBUG "Failed (Bad Speed).\n");
            return MSD_BAD_PARAM;
        }

		/*Get the force link status and then force link down*/
		if ((retVal = msdGetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 4, 2, &linkStatus)) != MSD_OK)
		{
			MSD_DBG_ERROR(SW_DEBUG "Get link status Failed\n");
			return retVal;
		}
		if ((retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 4, 2, 1)) != MSD_OK)
		{
			MSD_DBG_ERROR(SW_DEBUG "Force Link down failed\n");
			return retVal;
		}

		/* Set the ForceSpeed bit.  */
		if ((retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 13, 1, 1)) != MSD_OK)
		{
			MSD_DBG_ERROR(SW_DEBUG "Set Force Speed bit error\n");
			return retVal;
		}

        if ((retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 12, 1, data1)) != MSD_OK)
        {
            MSD_DBG_ERROR(SW_DEBUG "Set Alternate Speed bit error\n");
            return retVal;
        }
        if ((retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 0, 2, data2)) != MSD_OK)
        {
            MSD_DBG_ERROR(SW_DEBUG "Set Speed bit error\n");
            return retVal;
        }
    }

    /*Set back the force link status*/
    if ((retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 4, 2, linkStatus)) != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "Set back force link status error\n");
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtSetForceSpeed Called.\n");
    return retVal;
}

MSD_STATUS Amethyst_gprtSetDuplex
(
    IN  struct switch_dev *dev,
    IN  MSD_LPORT  port,
    OUT AMETHYST_MSD_PORT_FORCE_DUPLEX_MODE   mode
)
{
    u16          data;
    MSD_STATUS       retVal;         /* Functions return value.      */
    u8           hwPort;         /* the physical port number     */
    u8			phyAddr;

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtSetDuplex Called.\n");
    /* translate LPORT to hardware port */
    hwPort = MSD_LPORT_2_PORT(port);
    phyAddr = AMETHYST_MSD_CALC_SMI_DEV_ADDR(dev, hwPort);

    switch (mode)
    {
    case AMETHYST_MSD_DUPLEX_HALF:
        data = 0x1;
        break;
    case AMETHYST_MSD_DUPLEX_FULL:
        data = 0x3;
        break;
    case AMETHYST_MSD_DUPLEX_AUTO:
        data = 0x0;
        break;
    default:
        MSD_DBG_ERROR(SW_DEBUG "Failed (Bad mode).\n");
        return MSD_BAD_PARAM;
        break;
    }

    retVal = msdSetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_PHY_CONTROL, 2, 2, data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "ERROR to read AMETHYST_QD_REG_PORT_STATUS Register.\n");
        return retVal;
    }

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtSetDuplex Exit.\n");
    return MSD_OK;
}

/*******************************************************************************
* msdPhyReset
*
* DESCRIPTION:
*       This function performs softreset and waits until reset completion.
*
* INPUTS:
*       portNum     - Port number to write the register for.
*       u16Data     - data should be written into Phy control register.
*                      if this value is 0xFF, normal operation occcurs (read, 
*                      update, and write back.)
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK   - on success
*       MSD_FAIL - on error
*
* COMMENTS:
*		None.
*
*******************************************************************************/
MSD_STATUS Amethyst_msdPhyReset
(
    IN struct switch_dev *dev,
    IN  MSD_U8        portNum,
	IN  MSD_U8		 pageNum,
    IN  MSD_U16       u16Data
)
{
    MSD_U16 tmpData = 0;
    MSD_STATUS   retVal;
    MSD_U32 retryCount;
    MSD_BOOL    pd = MSD_FALSE;

    /*if((retVal=msdGetSMIPhyReg(dev,portNum,0,&tmpData)) 
           != MSD_OK)*/
	if((retVal=Amethyst_msdReadPagedPhyReg(dev, portNum, pageNum, 0,&tmpData)) 
           != MSD_OK)
    {
		MSD_DBG_INFO("Reading Register failed\n");
		return retVal;
    }

    if (tmpData & 0x800)
    {
        pd = MSD_TRUE;
    }

    if (u16Data != 0xFF)
    {
        tmpData = u16Data;
    }

    /* Set the desired bits to 0. */
    if (pd)
    {
        tmpData |= 0x800;
    }
    else
    {
	  if(((tmpData&0x4000)==0)||(u16Data==0xFF)) /* setting loopback do not set reset */
        tmpData |= 0x8000;
    }

    /*if((retVal=msdSetSMIPhyReg(dev,portNum,0,tmpData)) 
        != MSD_OK)*/
	if((retVal=Amethyst_msdWritePagedPhyReg(dev, portNum, pageNum, 0,tmpData)) 
        != MSD_OK)
    {
        MSD_DBG_INFO("Writing to register failed\n");
        return retVal;
    }

    if (pd)
    {
        return MSD_OK;
    }

    for (retryCount = 0x1000; retryCount > 0; retryCount--)
    {
        /*if((retVal=msdGetSMIPhyReg(dev,portNum,0,&tmpData)) != MSD_OK)*/
		if((retVal=Amethyst_msdReadPagedPhyReg(dev, portNum, pageNum, 0,&tmpData)) != MSD_OK)
        {
            MSD_DBG_INFO("Reading register failed\n");
            return retVal;
        }
        if ((tmpData & 0x8000) == 0)
            break;
    }

    if (retryCount == 0)
    {
        MSD_DBG_INFO("Reset bit is not cleared\n");
        return MSD_FAIL;
    }

    return MSD_OK;
}

/*
* This routine set Auto-Negotiation Ad Register for Copper
*/
static
MSD_STATUS Amethyst_gigCopperSetAutoMode
(
	IN struct switch_dev *dev,
	IN u8 hwPort,
	IN AMETHYST_MSD_PHY_AUTO_MODE mode
)
{
	u16             u16Data, u16Data1;

	if (Amethyst_msdReadPagedPhyReg(dev, hwPort, 0, AMETHYST_QD_PHY_AUTONEGO_AD_REG, &u16Data) != MSD_OK)
	{
		return MSD_FAIL;
	}

	/* Mask out all auto mode related bits. */
	u16Data &= ~AMETHYST_QD_PHY_MODE_AUTO_AUTO;

	if (Amethyst_msdReadPagedPhyReg(dev, hwPort, 0, AMETHYST_QD_PHY_AUTONEGO_1000AD_REG, &u16Data1) != MSD_OK)
	{
		return MSD_FAIL;
	}

	/* Mask out all auto mode related bits. */
	u16Data1 &= ~(AMETHYST_QD_GIGPHY_1000T_FULL | AMETHYST_QD_GIGPHY_1000T_HALF);

	switch (mode)
	{
		case AMETHYST_SPEED_AUTO_DUPLEX_AUTO:
			u16Data |= AMETHYST_QD_PHY_MODE_AUTO_AUTO;
		case AMETHYST_SPEED_1000_DUPLEX_AUTO:
			u16Data1 |= AMETHYST_QD_GIGPHY_1000T_FULL | AMETHYST_QD_GIGPHY_1000T_HALF;
			break;
		case AMETHYST_SPEED_AUTO_DUPLEX_FULL:
			u16Data |= AMETHYST_QD_PHY_MODE_AUTO_FULL;
			u16Data1 |= AMETHYST_QD_GIGPHY_1000T_FULL;
			break;
		case AMETHYST_SPEED_1000_DUPLEX_FULL:
			u16Data1 |= AMETHYST_QD_GIGPHY_1000T_FULL;
			break;
		case AMETHYST_SPEED_1000_DUPLEX_HALF:
			u16Data1 |= AMETHYST_QD_GIGPHY_1000T_HALF;
			break;
		case AMETHYST_SPEED_AUTO_DUPLEX_HALF:
			u16Data |= AMETHYST_QD_PHY_MODE_AUTO_HALF;
			u16Data1 |= AMETHYST_QD_GIGPHY_1000T_HALF;
			break;
		case AMETHYST_SPEED_100_DUPLEX_AUTO:
			u16Data |= AMETHYST_QD_PHY_MODE_100_AUTO;
			break;
		case AMETHYST_SPEED_10_DUPLEX_AUTO:
			u16Data |= AMETHYST_QD_PHY_MODE_10_AUTO;
			break;
		case AMETHYST_SPEED_100_DUPLEX_FULL:
			u16Data |= AMETHYST_QD_PHY_100_FULL;
			break;
		case AMETHYST_SPEED_100_DUPLEX_HALF:
			u16Data |= AMETHYST_QD_PHY_100_HALF;
			break;
		case AMETHYST_SPEED_10_DUPLEX_FULL:
			u16Data |= AMETHYST_QD_PHY_10_FULL;
			break;
		case AMETHYST_SPEED_10_DUPLEX_HALF:
			u16Data |= AMETHYST_QD_PHY_10_HALF;
			break;
		default:
			return MSD_BAD_PARAM;
	}

	/* Write to Phy AutoNegotiation Advertisement Register.  */
	if (Amethyst_msdWritePagedPhyReg(dev, hwPort, 0, AMETHYST_QD_PHY_AUTONEGO_AD_REG, u16Data) != MSD_OK)
	{
        return MSD_FAIL;
	}

	/* Write to Phy AutoNegotiation 1000B Advertisement Register.  */
	if (Amethyst_msdWritePagedPhyReg(dev, hwPort, 0, AMETHYST_QD_PHY_AUTONEGO_1000AD_REG, u16Data1) != MSD_OK)
	{
        return MSD_FAIL;
	}

	return MSD_OK;
}

/*
* This routine sets Auto Mode and Reset the phy
*/
static
MSD_STATUS phySetAutoMode
(
	IN struct switch_dev *dev,
	IN u8 hwPort,
	IN AMETHYST_MSD_PHY_AUTO_MODE mode
)
{
	MSD_U16         u16Data;
	MSD_STATUS    status;

	if ((status = Amethyst_gigCopperSetAutoMode(dev, hwPort, mode)) != MSD_OK)
	{
		return status;
	}

	/* Read to Phy Control Register.  */
	if (Amethyst_msdReadPagedPhyReg(dev, hwPort, 0, AMETHYST_QD_PHY_CONTROL_REG, &u16Data) != MSD_OK)
		return MSD_FAIL;

	u16Data |= AMETHYST_QD_PHY_AUTONEGO;

	/* Write to Phy Control Register.  */
	if (Amethyst_msdWritePagedPhyReg(dev, hwPort, 0, AMETHYST_QD_PHY_CONTROL_REG, u16Data) != MSD_OK)
		return MSD_FAIL;


	return Amethyst_msdPhyReset(dev, hwPort, 0, 0xFF);
}

/*******************************************************************************
* Amethyst_gphyReset
*
* DESCRIPTION:
*       This routine preforms PHY reset.
*       After reset, phy will be in Autonegotiation mode.
*
* INPUTS:
* 		hwPort - The physical port number
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK   - on success
*       MSD_FAIL - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
* 		data sheet register 0.15 - Reset
* 		data sheet register 0.13 - Speed(LSB)
* 		data sheet register 0.12 - Autonegotiation
* 		data sheet register 0.8  - Duplex Mode
*		data sheet register 0.6  - Speed(MSB)
* 
*       If DUT is in power down or loopback mode, hwPort will link down, 
*       in this case, no need to do software reset to restart AN.
*       When hwPort recover from link down, AN will restart automatically.
*
*******************************************************************************/
MSD_STATUS Amethyst_gphyReset
(
	IN struct switch_dev *dev,
	IN u8 hwPort
)
{
    MSD_STATUS       retVal;         /* Functions return value.      */
	MSD_U16  pageReg;

    MSD_DBG_INFO("Amethyst_gphySetPhyReset Called.\n");
	
	if ((retVal = Amethyst_msdGetSMIPhyRegField(dev, hwPort, 22, 0, 8, &pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR("read Phy Page Register return %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    /* set Auto Negotiation AD Register */
	retVal = phySetAutoMode(dev, hwPort, AMETHYST_SPEED_AUTO_DUPLEX_AUTO);
    if(retVal != MSD_OK)
    {
        MSD_DBG_ERROR("Amethyst_phySetAutoMode AMETHYST_SPEED_AUTO_DUPLEX_AUTO returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
    }

	if ((retVal = Amethyst_msdSetSMIPhyRegField(dev, hwPort, 22, 0, 8, pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR("write back Phy Page Register %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    MSD_DBG_INFO("Amethyst_gphyReset Exit.\n");
    return retVal;
}


/*******************************************************************************
* Amethyst_gphySetPortSpeed
*
* DESCRIPTION:
*       Sets speed for a specific physical port. This function will keep the duplex 
*       mode and loopback mode to the previous value, but disable others, such as 
*       Autonegotiation.
*
* INPUTS:
*       hwPort -  The physical port number, unless SERDES device is accessed
*               The physical address, if SERDES device is accessed
*       speed - port speed.
*               AMETHYST_PHY_SPEED_10_MBPS for 10Mbps
*               AMETHYST_PHY_SPEED_100_MBPS for 100Mbps
*               AMETHYST_PHY_SPEED_1000_MBPS for 1000Mbps
*		
* OUTPUTS:
* 		None.
*
* RETURNS:
* 		MSD_OK - on success
* 		MSD_FAIL - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
* 		data sheet register 0.13 - Speed Selection (LSB)
* 		data sheet register 0.6  - Speed Selection (MSB)
*
*       If DUT is in power down or loopback mode, port will link down, 
*       in this case, no need to do software reset to force take effect .
*       When port recover from link down, configure will take effect automatically.
*
*******************************************************************************/
MSD_STATUS Amethyst_gphySetPortSpeed
(
	IN struct switch_dev *dev,
	IN u8 hwPort,
	IN AMETHYST_MSD_PHY_SPEED speed
)
{
    MSD_U16             u16Data;
    MSD_STATUS        retVal;
	MSD_U16  pageReg;

    MSD_DBG_INFO("Amethyst_gphySetPortSpeed Called.\n");
	
	if ((retVal = Amethyst_msdGetSMIPhyRegField(dev, hwPort, 22, 0, 8, &pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR("read Phy Page Register %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    if(Amethyst_msdReadPagedPhyReg(dev,hwPort,0,AMETHYST_QD_PHY_CONTROL_REG,&u16Data) != MSD_OK)
    {
        MSD_DBG_ERROR("read Phy Control Register %s.\n", msdDisplayStatus(retVal));
        return MSD_FAIL;
    }

    switch(speed)
    {
        case AMETHYST_PHY_SPEED_10_MBPS:
            u16Data = u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_DUPLEX);
            break;
        case AMETHYST_PHY_SPEED_100_MBPS:
            u16Data = (u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_DUPLEX)) | AMETHYST_QD_PHY_SPEED;
            break;
        case AMETHYST_PHY_SPEED_1000_MBPS:
            u16Data = (u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_DUPLEX)) | AMETHYST_QD_PHY_SPEED_MSB;
            break;
        default:
            MSD_DBG_ERROR("Bad speed: %u.\n", speed);
            return MSD_BAD_PARAM;
    }

	retVal = Amethyst_msdPhyReset(dev, hwPort, 0, u16Data);
	if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("Amethyst_hwPhyReset returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}
    
	if ((retVal = Amethyst_msdSetSMIPhyRegField(dev, hwPort, 22, 0, 8, pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR("write back Phy Page Register %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    MSD_DBG_INFO("Amethyst_gphySetPortSpeed Exit.\n");
    return retVal;
}

/*******************************************************************************
* Amethyst_gphySetPortDuplexMode
*
* DESCRIPTION:
*		Sets duplex mode for a specific physical port. This function will keep 
*		the speed and loopback mode to the previous value, but disable others,
*		such as Autonegotiation.
*
* INPUTS:
*		hwPort -  The physical port number, unless SERDES device is accessed
*                The physical address, if SERDES device is accessed
*		dMode - dulpex mode
*
* OUTPUTS:
*		None.
*
* RETURNS:
*		MSD_OK  - on success
*		MSD_FAIL - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
* 		data sheet register 0.8 - Duplex Mode
*
*       If DUT is in power down or loopback mode, port will link down, 
*       in this case, no need to do software reset to force take effect .
*       When port recover from link down, configure will take effect automatically.
*
*******************************************************************************/
MSD_STATUS Amethyst_gphySetPortDuplexMode
(
	IN struct switch_dev *dev,
	IN u8 hwPort,
	IN MSD_BOOL   dMode
)
{
    MSD_U16             u16Data;
    MSD_STATUS        retVal;
	MSD_U16  pageReg;

    MSD_DBG_INFO("Amethyst_gphySetPortDuplexMode Called.\n");
	
	if ((retVal = Amethyst_msdGetSMIPhyRegField(dev, hwPort, 22, 0, 8, &pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR("read Phy Page Register %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    if(Amethyst_msdReadPagedPhyReg(dev,hwPort,0,AMETHYST_QD_PHY_CONTROL_REG,&u16Data) != MSD_OK)
    {
        MSD_DBG_ERROR("read Phy Control Register %s.\n", msdDisplayStatus(retVal));
        return MSD_FAIL;
    }

    if(dMode)
    {
        u16Data = (u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_SPEED | AMETHYST_QD_PHY_SPEED_MSB)) | AMETHYST_QD_PHY_DUPLEX;
    }
    else
    {
        u16Data = u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_SPEED | AMETHYST_QD_PHY_SPEED_MSB);
    }

    /* Write to Phy Control Register.  */
	retVal = Amethyst_msdPhyReset(dev, hwPort, 0, u16Data);
	if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("Amethyst_msdPhyReset returned: %s.\n", msdDisplayStatus(retVal)); 
		return retVal;
	}

	if ((retVal = Amethyst_msdSetSMIPhyRegField(dev, hwPort, 22, 0, 8, pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR("write back Phy Page Register %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    MSD_DBG_INFO("Amethyst_gphySetPortDuplexMode Exit.\n");
    return retVal;
}

/*******************************************************************************
* Amethyst_gphySetPortDuplexMode
*
* DESCRIPTION:
*		Sets duplex mode for a specific physical port. This function will keep 
*		the speed and loopback mode to the previous value, but disable others,
*		such as Autonegotiation.
*
* INPUTS:
*		hwPort -  The physical port number, unless SERDES device is accessed
*                The physical address, if SERDES device is accessed
*       speed - port speed.
*               AMETHYST_PHY_SPEED_10_MBPS for 10Mbps
*               AMETHYST_PHY_SPEED_100_MBPS for 100Mbps
*               AMETHYST_PHY_SPEED_1000_MBPS for 1000Mbps
*		dMode - dulpex mode
*
* OUTPUTS:
*		None.
*
* RETURNS:
*		MSD_OK  - on success
*		MSD_FAIL - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
* 		data sheet register 0.13 - Speed Selection (LSB)
* 		data sheet register 0.6  - Speed Selection (MSB)
* 		data sheet register 0.8 - Duplex Mode
*
*       If DUT is in power down or loopback mode, port will link down, 
*       in this case, no need to do software reset to force take effect .
*       When port recover from link down, configure will take effect automatically.
*
*******************************************************************************/
MSD_STATUS Amethyst_gphySetPortSpeedAndDuplexMode
(
	IN struct switch_dev *dev,
	IN u8 hwPort,
	IN AMETHYST_MSD_PHY_SPEED speed,
	IN MSD_BOOL   dMode
)
{
    MSD_U16             u16Data;
    MSD_STATUS        retVal;
	MSD_U16  pageReg;

    MSD_DBG_INFO("Amethyst_gphySetPortSpeedAndDuplexMode Called.\n");
	
	if ((retVal = Amethyst_msdGetSMIPhyRegField(dev, hwPort, 22, 0, 8, &pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR("read Phy Page Register %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    if(Amethyst_msdReadPagedPhyReg(dev,hwPort,0,AMETHYST_QD_PHY_CONTROL_REG,&u16Data) != MSD_OK)
    {
        MSD_DBG_ERROR("read Phy Control Register %s.\n", msdDisplayStatus(retVal));
        return MSD_FAIL;
    }

    switch(speed)
    {
        case AMETHYST_PHY_SPEED_10_MBPS:
            u16Data = u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_DUPLEX);
            break;
        case AMETHYST_PHY_SPEED_100_MBPS:
            u16Data = (u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_DUPLEX)) | AMETHYST_QD_PHY_SPEED;
            break;
        case AMETHYST_PHY_SPEED_1000_MBPS:
            u16Data = (u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_DUPLEX)) | AMETHYST_QD_PHY_SPEED_MSB;
            break;
        default:
            MSD_DBG_ERROR("Bad speed: %u.\n", speed);
            return MSD_BAD_PARAM;
    }

    if(dMode)
    {
        u16Data = (u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_SPEED | AMETHYST_QD_PHY_SPEED_MSB)) | AMETHYST_QD_PHY_DUPLEX;
    }
    else
    {
        u16Data = u16Data & (AMETHYST_QD_PHY_LOOPBACK | AMETHYST_QD_PHY_SPEED | AMETHYST_QD_PHY_SPEED_MSB);
    }

    /* Write to Phy Control Register.  */
	retVal = Amethyst_msdPhyReset(dev, hwPort, 0, u16Data);
	if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("Amethyst_msdPhyReset returned: %s.\n", msdDisplayStatus(retVal)); 
		return retVal;
	}

	if ((retVal = Amethyst_msdSetSMIPhyRegField(dev, hwPort, 22, 0, 8, pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR("write back Phy Page Register %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    MSD_DBG_INFO("Amethyst_gphySetPortSpeedAndDuplexMode Exit.\n");
    return retVal;
}

int mvsw61xx_set_port_speed_duplex(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{

	int portNum = val->port_vlan;
	int mod = val->value.i;
	if(portNum > 0 && portNum < 9)
	{
		if(0 == mod)/* AUTO  */
		{
			Amethyst_gphyReset(dev, portNum);
			//Amethyst_gigCopperSetAutoMode(dev, portNum,AMETHYST_SPEED_AUTO_DUPLEX_AUTO);
			//Amethyst_gprtSetDuplex(dev, portNum, AMETHYST_MSD_DUPLEX_AUTO);
			//Amethyst_gprtSetForceSpeed(dev, portNum, Amethyst_PORT_DO_NOT_FORCE_SPEED);
		}
		else if(1 == mod)/* 10MH  */
		{
			Amethyst_gphySetPortSpeedAndDuplexMode(dev, portNum, AMETHYST_PHY_SPEED_10_MBPS, MSD_FALSE);
			//Amethyst_gigCopperSetAutoMode(dev, portNum,AMETHYST_SPEED_10_DUPLEX_HALF);
			//Amethyst_gprtSetDuplex(dev, portNum, AMETHYST_MSD_DUPLEX_HALF);
			//Amethyst_gprtSetForceSpeed(dev, portNum, Amethyst_PORT_FORCE_SPEED_10M);
		}
		else if(2 == mod)/* 10MF  */
		{
			Amethyst_gphySetPortSpeedAndDuplexMode(dev, portNum, AMETHYST_PHY_SPEED_10_MBPS, MSD_TRUE);
			//Amethyst_gigCopperSetAutoMode(dev, portNum,AMETHYST_SPEED_10_DUPLEX_FULL);
			//Amethyst_gprtSetDuplex(dev, portNum, AMETHYST_MSD_DUPLEX_FULL);
			//Amethyst_gprtSetForceSpeed(dev, portNum, Amethyst_PORT_FORCE_SPEED_10M);
		}
		else if(3 == mod)/* 100MH  */
		{
			Amethyst_gphySetPortSpeedAndDuplexMode(dev, portNum, AMETHYST_PHY_SPEED_100_MBPS, MSD_FALSE);
			//Amethyst_gigCopperSetAutoMode(dev, portNum,AMETHYST_SPEED_100_DUPLEX_HALF);
			//Amethyst_gprtSetDuplex(dev, portNum, AMETHYST_MSD_DUPLEX_HALF);
			//Amethyst_gprtSetForceSpeed(dev, portNum, Amethyst_PORT_FORCE_SPEED_100M);
		}
		else if(4 == mod)/* 100MF  */
		{
			Amethyst_gphySetPortSpeedAndDuplexMode(dev, portNum, AMETHYST_PHY_SPEED_100_MBPS, MSD_TRUE);
			//Amethyst_gigCopperSetAutoMode(dev, portNum,AMETHYST_SPEED_100_DUPLEX_FULL);
			//Amethyst_gprtSetDuplex(dev, portNum, AMETHYST_MSD_DUPLEX_FULL);
			//Amethyst_gprtSetForceSpeed(dev, portNum, Amethyst_PORT_FORCE_SPEED_100M);
		}
		else if(5 == mod)/* 1000MF  */
		{
			Amethyst_gphySetPortSpeedAndDuplexMode(dev, portNum, AMETHYST_PHY_SPEED_1000_MBPS, MSD_TRUE);
			//Amethyst_gigCopperSetAutoMode(dev, portNum,AMETHYST_SPEED_1000_DUPLEX_FULL);
			//Amethyst_gprtSetDuplex(dev, portNum, AMETHYST_MSD_DUPLEX_FULL);
			//Amethyst_gprtSetForceSpeed(dev, portNum, Amethyst_PORT_FORCE_SPEED_1000M);
		}
	}
	else
	{
		if(5 == mod)
		{
			Amethyst_gprtSetForceSpeed(dev, portNum,Amethyst_PORT_FORCE_SPEED_1000M);
			/* relate with the errata describe in release not, 4.7 of 4.8 */
			if(10 == portNum || 0 == portNum)
			{
				/* P0/P10 1000Base-X setting */
				/* Force EEE off */
				msdSetAnyReg(dev, portNum, 0x1, 0x103);
				/* Power Down SERDES core */
				Amethyst_msdSetSMIC45PhyReg(dev, 0x4, portNum, 0x2000, 0x1940);
				/* Change C_MODE */
				msdSetAnyReg(dev, portNum, 0x0, 0x9);
				/* Errata 4.8 */
				Amethyst_msdSetSMIC45PhyReg(dev, 0x4, portNum, 0xF074, 0x6000);
				/* Errata 4.7 */
				Amethyst_msdSetSMIC45PhyReg(dev, 0x4, portNum, 0x2004, 0x20);
				/* Power Up SERDES core */
				Amethyst_msdSetSMIC45PhyReg(dev, 0x4, portNum, 0x2000, 0x9140);
			}
		}
		else if(6 == mod)
		{
			Amethyst_gprtSetForceSpeed(dev, portNum,Amethyst_PORT_FORCE_SPEED_10G);
		}
	}
        return 0;
}


/*******************************************************************************
* gprtPortPowerDown
*
* DESCRIPTION:
*        Enable/disable (power down) on specific logical port.
*        Phy configuration remains unchanged after Power down.
*
* INPUTS:
*       port -  The logical port number, unless SERDES device is accessed
*               The physical address, if SERDES device is accessed
*       state - MSD_TRUE: power down
*                MSD_FALSE: normal operation
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*
* COMMENTS:
*       data sheet register 0.11 - Power Down
*
*******************************************************************************/
MSD_STATUS Amethyst_gphyPortPowerDown
(
	IN struct switch_dev *dev,
	IN MSD_LPORT  port,
	IN MSD_BOOL   state
)
{
    MSD_STATUS       retVal;         /* Functions return value.      */
    u8           hwPort;         /* the physical port number     */
    u16          u16Data;
	u16  pageReg;

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtPortPowerDown Called.state %d\n",state);
    
    /* translate LPORT to hardware port */
	hwPort = port;

	if ((retVal = Amethyst_msdGetSMIPhyRegField(dev, hwPort, 22, 0, 8, &pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR(("Not able to read Phy Page Register.\n"));
		return retVal;
	}

    MSD_BOOL_2_BIT(state,u16Data);

    if((retVal=Amethyst_msdSetPagedPhyRegField(dev,hwPort,0,AMETHYST_QD_PHY_CONTROL_REG,11,1,u16Data)) != MSD_OK)
    {
        MSD_DBG_ERROR(SW_DEBUG "read Phy Control Register %s.\n", msdDisplayStatus(retVal));
        return retVal;
    }

	if ((retVal = Amethyst_msdSetSMIPhyRegField(dev, hwPort, 22, 0, 8, pageReg)) != MSD_OK)
	{
        MSD_DBG_ERROR(SW_DEBUG "write back Phy Page Register %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

    MSD_DBG_INFO(SW_DEBUG "Amethyst_gprtPortPowerDown Exit.\n");
    return MSD_OK;
}

int mvsw61xx_set_port_state(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{

	int portNum = val->port_vlan;
	int state = val->value.i;
	if(state == 0)
	{
		Amethyst_gphyPortPowerDown(dev, portNum ,MSD_TRUE);
	}
	else
	{
		Amethyst_gphyPortPowerDown(dev, portNum ,MSD_FALSE);
	}
	return MSD_OK;
}

MSD_STATUS Amethyst_gphySetEEE
(
	IN struct switch_dev *dev,
	IN MSD_LPORT portNum,
	IN MSD_BOOL en
)
{
    MSD_STATUS retVal;
    u8 hwPort;
    u16 data;

	MSD_DBG_INFO(("Amethyst_gphySetEEE Called.\n"));

    /* translate LPORT to hardware port */
    hwPort = portNum;
    switch (en)
    {
        case MSD_TRUE:
            data = 0x06;
            break;
        case MSD_FALSE:
            data = 0;
            break;
        default:
            data = 0x06;
            break;
    }

    /*enable/Disable the EEE for 100Base-T and 1000Base-T*/
    retVal = Amethyst_msdSetSMIPhyXMDIOReg(dev, (MSD_U8)hwPort, 7, 60, data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("enable/Disable the EEE for 100Base-T and 1000Base-T.\n");
        return retVal;
    }
    /*Software reset*/
    retVal = Amethyst_msdReadPagedPhyReg(dev, (MSD_U8)hwPort, 0, 0, &data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("get status.\n");
        return retVal;
    }
    data = data | 0x8000;
    retVal = Amethyst_msdWritePagedPhyReg(dev, (MSD_U8)hwPort, 0, 0, data);
    if (retVal != MSD_OK)
    {
        MSD_DBG_ERROR("Software reset.\n");
        return retVal;
    }

	MSD_DBG_INFO(("Amethyst_gphySetEEE Exit.\n"));

    return retVal;
}

MSD_STATUS Amethyst_gphySetFlowControlEnable
(
IN struct switch_dev *dev,
IN MSD_LPORT portNum,
IN MSD_BOOL en
)
{
	MSD_STATUS retVal;
	u8 hwPort;
	u16 data, tmpdata;

	MSD_DBG_INFO(SW_DEBUG "Amethyst_gphySetFlowControlEnable Called.\n");

	/* translate LPORT to hardware port */
	hwPort = portNum;
	switch (en)
	{
	case MSD_TRUE:
		data = 0x1;
		break;
	case MSD_FALSE:
		data = 0x0;
		break;
	default:
		data = 0x1;
		break;
	}

	/*enable/Disable the flow control page 0 reg 4 bit 10*/
	retVal = Amethyst_msdReadPagedPhyReg(dev, (u8)hwPort, 0, 4, &tmpdata);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "enable/Disable the flow control.\n");
		return retVal;
	}

	data = (tmpdata & 0xFBFF) | (data << 10);

	retVal = Amethyst_msdWritePagedPhyReg(dev, (u8)hwPort, 0, 4, data);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "enable/Disable flow control.\n");
		return retVal;
	}
	/*Software reset*/
	retVal = Amethyst_msdReadPagedPhyReg(dev, (u8)hwPort, 0, 0, &data);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "get status.\n");
		return retVal;
	}
	data = data | 0x8000;
	retVal = Amethyst_msdWritePagedPhyReg(dev, (u8)hwPort, 0, 0, data);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "Software reset.\n");
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG "Amethyst_gphySetFlowControlEnable Exit.\n");

	return retVal;
}

int mvsw61xx_set_port_flow_control(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{

	int portNum = val->port_vlan;
	int state = val->value.i;
	if(state == 1)
	{
		Amethyst_gphySetFlowControlEnable(dev, portNum ,MSD_TRUE);
	}
	else
	{
		Amethyst_gphySetFlowControlEnable(dev, portNum ,MSD_FALSE);
	}
	return MSD_OK;
}
/*######################################################  LED ###################################*/

static MSD_STATUS Amethyst_LedCtrl_Write
(
IN	struct switch_dev *dev,
IN  MSD_LPORT  portNum,
IN	u8	pointer,
IN	u16	data
)
{
    MSD_STATUS    retVal;         /* Functions return value.      */
    u8        hwPort;         /* the physical port number     */
    u8		 phyAddr;
    u16   count, tmpData;

    /* translate LPORT to hardware port */
    hwPort = MSD_LPORT_2_PORT(portNum);
    phyAddr = AMETHYST_MSD_CALC_SMI_DEV_ADDR(dev, hwPort);

    count = 10;
    tmpData = 1;
    while (tmpData == 1)
    {
        retVal = msdGetAnyRegField(dev, phyAddr, AMETHYST_QD_REG_LED_CONTROL, 15, 1, &tmpData);
        if (retVal != MSD_OK)
        {
            return retVal;
        }
        if ((count--) == 0)
        {
            return MSD_FAIL;
        }
    }

    tmpData = (u16)((1 << 15) | ((pointer & 0x7) << 12) | (data & 0x7FF));

    retVal = msdSetAnyReg(dev, phyAddr, AMETHYST_QD_REG_LED_CONTROL, tmpData);
    if (retVal != MSD_OK)
    {
        return retVal;
    }

    return retVal;
}

static MSD_STATUS Amethyst_LedCtrl_Read
(
IN	struct switch_dev *dev,
IN  MSD_LPORT  portNum,
IN	u8	pointer,
OUT	u16	*data
)
{
    MSD_STATUS    retVal;         /* Functions return value.      */
    u8           hwPort;         /* the physical port number     */
    u8			phyAddr;
    u16            tmpData;
    int count = 0x10;

    /* translate LPORT to hardware port */
    hwPort = MSD_LPORT_2_PORT(portNum);
    phyAddr = AMETHYST_MSD_CALC_SMI_DEV_ADDR(dev, hwPort);

    do {
        retVal = msdGetAnyReg(dev, phyAddr, AMETHYST_QD_REG_LED_CONTROL, &tmpData);
        if (retVal != MSD_OK)
        {
            return retVal;
        }
        if ((count--) == 0)
        {
            return MSD_FAIL;
        }
    } while (tmpData & 0x8000);

    tmpData = (u16)(((pointer & 0x7) << 12) | 0);
    retVal = msdSetAnyReg(dev, phyAddr, AMETHYST_QD_REG_LED_CONTROL, tmpData);
    if (retVal != MSD_OK)
    {
        return retVal;
    }

    retVal = msdGetAnyReg(dev, phyAddr, AMETHYST_QD_REG_LED_CONTROL, &tmpData);
    if (retVal != MSD_OK)
    {
        return retVal;
    }

    *data = (u16)(tmpData & 0x7ff);

    return retVal;
}

/*******************************************************************************
*   msdLedModeSet
*
* DESCRIPTION:
*       This routine is to set led mode
*		For Amethyst :
*		Port 1 - 8 :
*			0000 :  Link/Act/Speed by Blink Rate (off = no link, on = link, blink = activity, blink speed = link speed)
*			0001 :  100/Gig Link/Act (off = no link, on = 100 or Gig link, blink = activity)
*			0010 :  Gig Link/Act (off = no link, on = Gig link, blink = activity)
*			0010 :  Link/Act (off = no link, on = link, blink = activity)
*			0100 :  Special LED 3
*			0101 :  Special LED 4
*			0110 :  Duplex/Collision (off = half duplex, on = full duplex, blink = col)
*			0111 :  10/Gig Link/Act (off = no link, on = 10 or Gig link, blink = activity)
*			1000 :  Link (off = no link, on = link)
*			1001 :  10 Link (off = no link, on = 10 link)
*			1010 :  10 Link/Act (off = no link, on = 10 link, blink = activity)
*			1011 :  100/Gig Link (off = no link, on = 100 or Gig link)
*			1100 :  PTP Act (blink on = PTP activity)
*			1101 :  Force Blink
*			1110 :  Force Off
*			1111 :  Force On
*
* INPUTS:
*       devNum  - physical devie number
*       portNum - logical port number
*       ledNum  - LED Index
*       mode  - LED mode
*
* OUTPUTS:
*       None.
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*       MSD_NOT_SUPPORTED - device not support
*
* COMMENTS:
*       For Amethyst and Amethyst:
*       If IMP boots, all LEDs are deactive. It's the IMP software's responsibility
*       to determine when to turn on the LEDs by clearing these bits (Global LED Control,
*       index 0x00, bit[3:0], Skip Columns) to zero. Then we can call this API to
*       configure LED mode.
*
*******************************************************************************/
MSD_STATUS   Amethyst_gledModeSet
(
IN  struct switch_dev *dev,
IN  MSD_LPORT portNum,
IN  u8    ledNum,
IN  u8    mode
)
{
	MSD_STATUS retVal;
	u8 pointer;
	u8 foff;
	u8 flen;
	u16 val;
	u16 tmp;

	MSD_DBG_INFO(SW_DEBUG "Amethyst_gledModeSet Called.\n");

	if (portNum > 10 || portNum == 0)
	{
		MSD_DBG_ERROR(SW_DEBUG "Bad Port: %u. It should be within [1,10]\n", (unsigned int)portNum);
		return MSD_BAD_PARAM;
	}
	if (mode > 0xf)
	{
		MSD_DBG_ERROR(SW_DEBUG "Bad Mode: %u. It should be within [0,15]\n", (unsigned int)portNum);
		return MSD_BAD_PARAM;
	}
	switch (ledNum)
	{
	case 0:
		pointer = 0;
		foff = 0;
		flen = 4;
		val = mode;
		break;
	case 1:
		pointer = 0;
		foff = 4;
		flen = 4;
		val = mode;
		break;
	default:
		MSD_DBG_ERROR(SW_DEBUG "Bad LED: %u. It should be within [0,1]\n", (unsigned int)ledNum);
		return MSD_BAD_PARAM;
	}

	retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &tmp);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	MSD_BF_SET(tmp, val, foff, flen);

	retVal = Amethyst_LedCtrl_Write(dev, portNum, pointer, tmp);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_LedCtrl_Write returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG "Amethyst_gledModeSet Exit.\n");
	return MSD_OK;
}
/*******************************************************************************
*   msdLedModeGet
*
* DESCRIPTION:
*       This routine is to get led mode
*		For Amethyst :
*		LED mode (Port 1 - 8) :
*			0000 :  Link/Act/Speed by Blink Rate (off = no link, on = link, blink = activity, blink speed = link speed)
*			0001 :  100/Gig Link/Act (off = no link, on = 100 or Gig link, blink = activity)
*			0010 :  Gig Link/Act (off = no link, on = Gig link, blink = activity)
*			0010 :  Link/Act (off = no link, on = link, blink = activity)
*			0100 :  Special LED 3
*			0101 :  Special LED 4
*			0110 :  Duplex/Collision (off = half duplex, on = full duplex, blink = col)
*			0111 :  10/Gig Link/Act (off = no link, on = 10 or Gig link, blink = activity)
*			1000 :  Link (off = no link, on = link)
*			1001 :  10 Link (off = no link, on = 10 link)
*			1010 :  10 Link/Act (off = no link, on = 10 link, blink = activity)
*			1011 :  100/Gig Link (off = no link, on = 100 or Gig link)
*			1100 :  PTP Act (blink on = PTP activity)
*			1101 :  Force Blink
*			1110 :  Force Off
*			1111 :  Force On
*
* INPUTS:
*       devNum  - physical devie number
*       portNum - logical port number
*       ledNum  - LED Index
*
* OUTPUTS:
*       mode - Led mode.
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*       MSD_NOT_SUPPORTED - device not support
*
* COMMENTS:
*       For Amethyst and Amethyst:
*       If IMP boots, all LEDs are deactive. It's the IMP software's responsibility
*       to determine when to turn on the LEDs by clearing these bits (Global LED Control,
*       index 0x00, bit[3:0], Skip Columns) to zero. Then we can call this API to
*       configure LED mode.
*
*******************************************************************************/
MSD_STATUS   Amethyst_gledModeGet
(
IN  struct switch_dev *dev,
IN  MSD_LPORT portNum,
IN  u8    ledNum,
OUT  u8    *mode
)
{
	MSD_STATUS retVal;
	u16 tmp;
	
	MSD_DBG_INFO(SW_DEBUG "Amethyst_gledModeGet Called.\n");

	if (portNum > 10 || portNum == 0)
	{
		MSD_DBG_ERROR(SW_DEBUG "Bad Port: %u. It should be within [1,10]\n", (unsigned int)portNum);
		return MSD_BAD_PARAM;
	}

	retVal = Amethyst_LedCtrl_Read(dev, portNum, 0, &tmp);

	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG "Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	switch (ledNum)
	{
	case 0:
		*mode = tmp & 0x0f;
		break;
	case 1:
		*mode = (tmp & 0xf0) >> 4;
		break;
	default:
		MSD_DBG_ERROR(SW_DEBUG "Bad LED: %u. It should be within [0,1]\n", (unsigned int)ledNum);
		return MSD_BAD_PARAM;
	}
	MSD_DBG_INFO(SW_DEBUG "Amethyst_gledModeGet Exit.\n");
	return MSD_OK;
}

/*******************************************************************************
* Amethyst_msdLedDump
*
* DESCRIPTION:
*       This routine is to get all led infomation
*
*
* INPUTS:
*       devNum  - physical devie number
*
* OUTPUTS:
*       None
*
* RETURNS:
*       MSD_OK  - on success
*       MSD_FAIL  - on error
*       MSD_BAD_PARAM - if invalid parameter is given
*       MSD_NOT_SUPPORTED - device not support
*
* COMMENTS:
*       For Peridot and Topaz:
*       If IMP boots, all LEDs are deactive. It's the IMP software's responsibility
*       to determine when to turn on the LEDs by clearing these bits (Global LED Control,
*       index 0x00, bit[3:0], Skip Columns) to zero. Then we can call this API to
*       configure LED mode.
*
*******************************************************************************/
MSD_STATUS Amethyst_gledDump
(
IN  struct switch_dev *dev
)
{
	MSD_STATUS retVal;
	u8 portNum = 0, pointer = 0;
	u16 data0, data6;
	MSD_DBG_INFO(SW_DEBUG "Amethyst_gledDump Called.\n");

	/* Port0 global control*/
	MSD_DBG_INFO(SW_DEBUG"\n--------- Port 0 Global Control ----------\n");

	pointer = 0;
	retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &data0);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG"Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG"DirectLEDS      0x%-7x\n", (data0 & 0x10) >> 4);
	MSD_DBG_INFO(SW_DEBUG"SkipColumns     0x%-7x\n", (data0 & 0xF));

	pointer = 1;
	retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &data0);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG"Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG"LAN Link/Act    0x%-7x\n", data0);

	pointer = 2;
	retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &data0);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG"Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG"WAN Link/Act    0x%-7x\n", data0);

	pointer = 3;
	retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &data0);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG"Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG"CPU Link/Act    0x%-7x\n", data0);

	pointer = 4;
	retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &data0);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG"Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG"PTP Activity    0x%-7x\n", data0);

	pointer = 6;
	retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &data0);
	if (retVal != MSD_OK)
	{
		MSD_DBG_ERROR(SW_DEBUG"Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
		return retVal;
	}

	MSD_DBG_INFO(SW_DEBUG"Pulse Stretch   0x%-7x\n", ((data0 & 0x70) >> 4));
	MSD_DBG_INFO(SW_DEBUG"Blink Rate      0x%-7x\n", (data0 & 0x7));

	MSD_DBG_INFO(SW_DEBUG"------------------------------------------\n");
	/* Print out led field*/
	MSD_DBG_INFO(SW_DEBUG"\n----------------------------------------------------------------\n");
	MSD_DBG_INFO(SW_DEBUG"     LED1Select    LED0Select    PulseStretch    BlinkRate\n");
	MSD_DBG_INFO(SW_DEBUG"----------------------------------------------------------------\n");
	for (portNum = 1; portNum <= 10; portNum++)
	{
		pointer = 0;
		retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &data0);
		if (retVal != MSD_OK)
		{
			MSD_DBG_ERROR(SW_DEBUG"Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
			return retVal;
		}

		pointer = 6;
		retVal = Amethyst_LedCtrl_Read(dev, portNum, pointer, &data6);
		if (retVal != MSD_OK)
		{
			MSD_DBG_ERROR(SW_DEBUG"Amethyst_LedCtrl_Read returned: %s.\n", msdDisplayStatus(retVal));
			return retVal;
		}

		MSD_DBG_INFO(SW_DEBUG"P%X     0x%-7x      0x% -7x     0x%-7x      0x%-7x \n", portNum, ((data0 & 0xF0) >> 4), (data0 & 0xF), ((data6 & 0x70) >> 4), (data6 & 0x7));
	}


	MSD_DBG_INFO(SW_DEBUG"Amethyst_gledDump Exit.\n");
	return MSD_OK;
}

int mvsw61xx_set_port_led(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val)
{
	int i = 0;
	for(i = 1;i < 9 ;i++)
	{
	Amethyst_gledModeSet(dev, i , 0,2);
	Amethyst_gledModeSet(dev, i , 1,2);
	}
	Amethyst_gledDump(dev);
	return MSD_OK;
}

int mvsw61xx_init_port_led(struct switch_dev *dev)
{
	int i = 0;
	for(i = 1;i < 9 ;i++)
	{
		Amethyst_gledModeSet(dev, i , 0, 2);
		Amethyst_gledModeSet(dev, i , 1, 2);
	}
	Amethyst_gledModeSet(dev, 9 , 0, 9);
	Amethyst_gledModeSet(dev, 9 , 1, 0);
	return MSD_OK;
}
MSD_STATUS Amethyst_msdSetSMIC45PhyReg
(
    struct switch_dev *dev,
    IN u8    devAddr,
    IN u8    phyAddr,
    IN u16   regAddr,
    IN u16   data
)
{
    unsigned int timeOut; 
    u16 smiReg;

    MSD_DBG("(LOG AMETHYST SMI PHY C45 RW): devAddr 0x%02x, phyAddr 0x%02x, regAddr 0x%02x, \n",
        devAddr, phyAddr, regAddr);
    MSD_DBG("data 0x%04x.\n", data);

    /* first check that it is not busy */
    timeOut = MSD_SMI_ACCESS_LOOP; /* initialize the loop count */

    do 
    {
        if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, &smiReg) != MSD_OK)
        {
            MSD_DBG_ERROR("read AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
            return MSD_FAIL;
        }
        if(timeOut-- < 1 ) 
        {
            MSD_DBG_ERROR(("read SMI Phy AMETHYST_QD_REG_SMI_PHY_CMD register Timed Out\n"));
            return MSD_FAIL;
        }
    } while (smiReg & MSD_SMI_BUSY);


    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_DATA, regAddr) != MSD_OK)
    {
        MSD_DBG_ERROR("Write AMETHYST_QD_REG_SMI_PHY_DATA register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }
    smiReg = MSD_SMI_BUSY | (phyAddr << MSD_SMI_DEV_ADDR_BIT) | (MSD_SMI_WRITE_ADDR << MSD_SMI_OP_BIT) | 
            (devAddr << MSD_SMI_REG_ADDR_BIT) | (MSD_SMI_CLAUSE45 << MSD_SMI_MODE_BIT);

    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, smiReg) != MSD_OK)
    {
        MSD_DBG_ERROR("Write AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }

    timeOut = MSD_SMI_ACCESS_LOOP; /* initialize the loop count */

    do 
    {
        if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, &smiReg) != MSD_OK)
        {
            MSD_DBG_ERROR("read AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
            return MSD_FAIL;
        }
        if(timeOut-- < 1 ) 
        {
            MSD_DBG_INFO("read SMI Phy AMETHYST_QD_REG_SMI_PHY_CMD register Timed Out\n");
            return MSD_FAIL;
        }
    } while (smiReg & MSD_SMI_BUSY);


    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_DATA, data) != MSD_OK)
    {
        MSD_DBG_ERROR("write AMETHYST_QD_REG_SMI_PHY_DATA register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }
    smiReg = MSD_SMI_BUSY | (phyAddr << MSD_SMI_DEV_ADDR_BIT) | (MSD_SMI_WRITE << MSD_SMI_OP_BIT) | 
            (devAddr << MSD_SMI_REG_ADDR_BIT) | (MSD_SMI_CLAUSE45 << MSD_SMI_MODE_BIT);

    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, smiReg) != MSD_OK)
    {
        MSD_DBG_ERROR("write AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }

	return MSD_OK;
}

MSD_STATUS Amethyst_msdGetSMIC45PhyReg
(
    struct switch_dev *dev,
    IN  u8     devAddr,
    IN  u8     phyAddr,
    IN  u16    regAddr,
    OUT u16    *data
)
{
    unsigned int timeOut;
    u16 smiReg;
	u16 tmp;

    MSD_DBG_INFO(("Read Phy register while PPU Enabled\n"));

     /* first check that it is not busy */
    timeOut = MSD_SMI_ACCESS_LOOP; /* initialize the loop count */

    do 
    {
        if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, &smiReg) != MSD_OK)
        {
            MSD_DBG_ERROR("read AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
            return MSD_FAIL;
        }
        if(timeOut-- < 1 ) 
        {
            MSD_DBG_INFO("read AMETHYST_QD_REG_SMI_PHY_CMD register Timed Out\n");
            return MSD_FAIL;
        }
    } while (smiReg & MSD_SMI_BUSY);


    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_DATA, regAddr) != MSD_OK)
    {
        MSD_DBG_ERROR("write AMETHYST_QD_REG_SMI_PHY_DATA register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }
    smiReg = MSD_SMI_BUSY | (phyAddr << MSD_SMI_DEV_ADDR_BIT) | (MSD_SMI_WRITE_ADDR << MSD_SMI_OP_BIT) | 
            (devAddr << MSD_SMI_REG_ADDR_BIT) | (MSD_SMI_CLAUSE45 << MSD_SMI_MODE_BIT);

    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, smiReg) != MSD_OK)
    {
        MSD_DBG_ERROR("write AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }

    smiReg = MSD_SMI_BUSY | (phyAddr << MSD_SMI_DEV_ADDR_BIT) | (MSD_SMI_READ_45 << MSD_SMI_OP_BIT) | 
            (devAddr << MSD_SMI_REG_ADDR_BIT) | (MSD_SMI_CLAUSE45 << MSD_SMI_MODE_BIT);

    if(msdSetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, smiReg) != MSD_OK)
    {
        MSD_DBG_ERROR("Write TOPAZ_QD_REG_SMI_PHY_CMD register Failed\n");
        return MSD_FAIL;
    }

    timeOut = MSD_SMI_ACCESS_LOOP; /* initialize the loop count */

    do 
    {
        if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_CMD, &smiReg) != MSD_OK)
        {
            MSD_DBG_ERROR("read AMETHYST_QD_REG_SMI_PHY_CMD register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
            return MSD_FAIL;
        }
        if(timeOut-- < 1 ) 
        {
            MSD_DBG_ERROR("read AMETHYST_QD_REG_SMI_PHY_CMD register Timed Out\n");
            return MSD_FAIL;
        }
    } while (smiReg & MSD_SMI_BUSY);


    if(msdGetAnyReg(dev, AMETHYST_GLOBAL2_DEV_ADDR,AMETHYST_QD_REG_SMI_PHY_DATA, &tmp) != MSD_OK)
    {
        MSD_DBG_ERROR("read TOPAZ_QD_REG_SMI_PHY_DATA register returned: %s.\n", msdDisplayStatus(MSD_FAIL));
        return MSD_FAIL;
    }
	
    *data = tmp;

    MSD_DBG("(LOG AMETHYST SMI PHY C45 RR): devAddr 0x%02x, phyAddr 0x%02x, regAddr 0x%02x, \n",
        devAddr, phyAddr, regAddr);
    MSD_DBG("data 0x%04x.\n", *data);

    return MSD_OK;
}


int mvsw61xx_set_10g_kr(struct switch_dev *dev)
{
	u16 data;
	u16 mask;
		{/* SFP+ 2 10G-baseR port10  */
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x8093,0xcb5a);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x8171,0x7088);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x80c9,0x311a);
	
	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,10,0x80a2,&data);
	mask = 0x80;
	data = 0x8000|(data&mask);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x80a2,data);

	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,10,0x80a9,&data);
	mask = 0xf;
	data = 0x0 |(data&mask);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x80a9,data);

	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,10,0x80a3,&data);
	mask = 0xf8ff;
	data = 0x0|(data&mask);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x80a3,data);

	Amethyst_msdGetSMIC45PhyReg(dev,0x4,10,0xf002,&data);
	mask = 0x7fff;
	data = 0x8000|(data&mask);
	Amethyst_msdSetSMIC45PhyReg(dev,0x4,10,0xf002,data);



	data = 0x1<<11|0x1c<<5|0x9<<0;
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x8101,data);
	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,10,0x8101,&data);

	data = 0x1<<11|0x2f<<5|6<<0;
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x8105,data);
	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,10,0x8105,&data);

	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,10,0x8104,data);
	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,10,0x8104,&data);

	msdSetAnyReg(dev, 10, 0x1, 0x103);
	Amethyst_msdSetSMIC45PhyReg(dev, 0x4, 10, 0x2000, 0x1940);
	msdSetAnyReg(dev, 10, 0x0, 0x9);
	Amethyst_msdSetSMIC45PhyReg(dev, 0x4, 10, 0xF074, 0x6000);
	Amethyst_msdSetSMIC45PhyReg(dev, 0x4, 10, 0x2004, 0x20);
	Amethyst_msdSetSMIC45PhyReg(dev, 0x4, 10, 0x2000, 0x9140);

	
}
	{
		/* sfp port0 1000base-x */
	data = 0x1<<11|0x2f<<5|6<<0;
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,0,0x8105,data);
	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,0,0x8105,&data);


	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,0,0x8104,data);
	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,0,0x8104,&data);

	msdSetAnyReg(dev, 0, 0x1, 0x103);
	Amethyst_msdSetSMIC45PhyReg(dev, 0x4, 0, 0x2000, 0x1940);
	msdSetAnyReg(dev, 0, 0x0, 0x9);
	Amethyst_msdSetSMIC45PhyReg(dev, 0x4, 0, 0xF074, 0x6000);
	Amethyst_msdSetSMIC45PhyReg(dev, 0x4, 0, 0x2004, 0x20);
	Amethyst_msdSetSMIC45PhyReg(dev, 0x4, 0, 0x2000, 0x9140);
	}

	{
		/* switch CPU port9  */
	
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,9,0x8093,0xcb5a);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,9,0x8171,0x7088);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,9,0x80c9,0x311a);
	
	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,9,0x80a2,&data);
	mask = 0x80;
	data = 0x8000|(data&mask);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,9,0x80a2,data);

	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,9,0x80a9,&data);
	mask = 0xf;
	data = 0x0 |(data&mask);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,9,0x80a9,data);

	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,9,0x80a3,&data);
	mask = 0xf8ff;
	data = 0x0|(data&mask);
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,9,0x80a3,data);

	Amethyst_msdGetSMIC45PhyReg(dev,0x4,9,0xf002,&data);
	mask = 0x7fff;
	data = 0x8000|(data&mask);
	Amethyst_msdSetSMIC45PhyReg(dev,0x4,9,0xf002,data);

	Amethyst_msdGetSMIC45PhyReg(dev,0x1e,9,0x8101,&data);

	data = 0x1<<11|0x22<<5|0x6<<0;
	Amethyst_msdSetSMIC45PhyReg(dev,0x1e,9,0x8101,data);
	}

	return 0;
}
