#ifndef __AMETHYST_SWITCH_H
#define __AMETHYST_SWITCH_H

#define MSD_OK              (0x00)   /* Operation succeeded                   */
#define MSD_FAIL            (0x01)   /* Operation failed                    */
#define MSD_BAD_PARAM       (0x04)   /* Illegal parameter in function called  */
#define MSD_BUSY            (0x05)   /* Last operation busy */
#define MSD_NO_SUCH         (0x0D)   /* No such item                    */
#define MSD_NOT_SUPPORTED   (0x10)   /* This request is not support           */
#define MSD_ALREADY_EXIST   (0x1B)   /* Tried to create existing item         */
#define MSD_BAD_CPU_PORT    (0x20)   /* Input CPU Port is not valid physical port number */
#define MSD_FEATURE_NOT_ENABLE  (0x40) /*The feature not been enabled when do operation*/

#define MSD_DBG_INFO printk
#define MSD_DBG_ERROR printk
#define MSD_DBG printk

typedef enum {
MSD_FALSE = 0,
MSD_TRUE = 1
} MSD_BOOL;

typedef u8 MSD_U8;
typedef u16 MSD_U16;
typedef u32 MSD_U32;
typedef MSD_U32 MSD_LPORT;
typedef int MSD_STATUS;
/* The following macro converts a boolean   */
/* value to a binary one (of 1 bit).        */
/* MSD_FALSE --> 0                           */
/* MSD_TRUE --> 1                            */
#define MSD_BOOL_2_BIT(boolVal,binVal)                                  \
            (binVal) = (((boolVal) == MSD_FALSE) ? 0 : 1)
#define MSD_LPORT_2_PORT(_lport)      (u8)((_lport) & 0xff)
	/* Convert port number to smi device address */
#define AMETHYST_MSD_CALC_SMI_DEV_ADDR(_dev, _portNum)  \
	(0 + (_portNum))

/*
 *  typedef: struct MSD_STATS_COUNTERS
 *
 *  Description: MIB counter
 *     
 */
#define AMETHYST_MSD_TYPE_BANK 0x400
typedef enum
{
	/* Bank 0 */
    Amethyst_STATS_InGoodOctetsLo = 0,
    Amethyst_STATS_InGoodOctetsHi,
    Amethyst_STATS_InBadOctets,
    
    Amethyst_STATS_OutFCSErr,
    Amethyst_STATS_InUnicasts,
    Amethyst_STATS_Deferred,             /* offset 5 */
    Amethyst_STATS_InBroadcasts,
    Amethyst_STATS_InMulticasts,
    Amethyst_STATS_64Octets,
    Amethyst_STATS_127Octets,
    Amethyst_STATS_255Octets,            /* offset 10 */
    Amethyst_STATS_511Octets,
    Amethyst_STATS_1023Octets,
    Amethyst_STATS_MaxOctets,
    Amethyst_STATS_OutOctetsLo,
    Amethyst_STATS_OutOctetsHi,
    Amethyst_STATS_OutUnicasts,          /* offset 16 */
    Amethyst_STATS_Excessive,
    Amethyst_STATS_OutMulticasts,
    Amethyst_STATS_OutBroadcasts,
    Amethyst_STATS_Single,
    Amethyst_STATS_OutPause,
    Amethyst_STATS_InPause,
    Amethyst_STATS_Multiple,
    Amethyst_STATS_InUndersize,          /* offset 24 */
    Amethyst_STATS_InFragments,
    Amethyst_STATS_InOversize,
    Amethyst_STATS_InJabber,
    Amethyst_STATS_InRxErr,
    Amethyst_STATS_InFCSErr,
    Amethyst_STATS_Collisions,
    Amethyst_STATS_Late,                 /* offset 31 */
	/* Bank 1 */
    Amethyst_STATS_InDiscards      = AMETHYST_MSD_TYPE_BANK+0x00,
    Amethyst_STATS_InFiltered      = AMETHYST_MSD_TYPE_BANK+0x01,
    Amethyst_STATS_InAccepted      = AMETHYST_MSD_TYPE_BANK+0x02,
    Amethyst_STATS_InBadAccepted   = AMETHYST_MSD_TYPE_BANK+0x03,
    Amethyst_STATS_InGoodAvbClassA = AMETHYST_MSD_TYPE_BANK+0x04,
    Amethyst_STATS_InGoodAvbClassB = AMETHYST_MSD_TYPE_BANK+0x05,
    Amethyst_STATS_InBadAvbClassA  = AMETHYST_MSD_TYPE_BANK+0x06,
    Amethyst_STATS_InBadAvbClassB  = AMETHYST_MSD_TYPE_BANK+0x07,
    Amethyst_STATS_TCAMCounter0    = AMETHYST_MSD_TYPE_BANK+0x08,
    Amethyst_STATS_TCAMCounter1    = AMETHYST_MSD_TYPE_BANK+0x09,
    Amethyst_STATS_TCAMCounter2    = AMETHYST_MSD_TYPE_BANK+0x0a,
    Amethyst_STATS_TCAMCounter3    = AMETHYST_MSD_TYPE_BANK+0x0b,
    Amethyst_STATS_InDroppedAvbA   = AMETHYST_MSD_TYPE_BANK+0x0c,
    Amethyst_STATS_InDroppedAvbB   = AMETHYST_MSD_TYPE_BANK+0x0d, 
    Amethyst_STATS_InDaUnknown     = AMETHYST_MSD_TYPE_BANK+0x0e,
    Amethyst_STATS_InMGMT          = AMETHYST_MSD_TYPE_BANK+0x0f,
    Amethyst_STATS_OutQueue0       = AMETHYST_MSD_TYPE_BANK+0x10,
    Amethyst_STATS_OutQueue1       = AMETHYST_MSD_TYPE_BANK+0x11,
    Amethyst_STATS_OutQueue2       = AMETHYST_MSD_TYPE_BANK+0x12,
    Amethyst_STATS_OutQueue3       = AMETHYST_MSD_TYPE_BANK+0x13,
    Amethyst_STATS_OutQueue4       = AMETHYST_MSD_TYPE_BANK+0x14,
    Amethyst_STATS_OutQueue5       = AMETHYST_MSD_TYPE_BANK+0x15,
    Amethyst_STATS_OutQueue6       = AMETHYST_MSD_TYPE_BANK+0x16,
    Amethyst_STATS_OutQueue7       = AMETHYST_MSD_TYPE_BANK+0x17,
    Amethyst_STATS_OutCutThrough   = AMETHYST_MSD_TYPE_BANK+0x18,
	Amethyst_STATS_InBadQbv		   = AMETHYST_MSD_TYPE_BANK+0x19,
    Amethyst_STATS_OutOctetsA      = AMETHYST_MSD_TYPE_BANK+0x1a,
    Amethyst_STATS_OutOctetsB      = AMETHYST_MSD_TYPE_BANK+0x1b,
    Amethyst_STATS_OutYel          = AMETHYST_MSD_TYPE_BANK+0x1c,
    Amethyst_STATS_OutDroppedYel   = AMETHYST_MSD_TYPE_BANK+0x1d, 
    Amethyst_STATS_OutDiscards     = AMETHYST_MSD_TYPE_BANK+0x1e, 
    Amethyst_STATS_OutMGMT         = AMETHYST_MSD_TYPE_BANK+0x1f

} AMETHYST_MSD_STATS_COUNTERS;

/*
 *  typedef: struct AMETHYST_MSD_STATS_COUNTER_SET
 *
 *  Description: MIB Counter Set
 *     
 */
typedef struct _AMETHYST_MSD_STATS_COUNTER_SET
{
	/* Bank 0 */
    u32    InGoodOctetsLo;     /* offset 0 */
    u32    InGoodOctetsHi;     /* offset 1 */
    u32    InBadOctets;        /* offset 2 */
    u32    OutFCSErr;          /* offset 3 */
    u32    InUnicasts;         /* offset 4 */
    u32    Deferred;           /* offset 5 */
    u32    InBroadcasts;       /* offset 6 */
    u32    InMulticasts;       /* offset 7 */
    /* 
        Histogram Counters : Rx Only, Tx Only, or both Rx and Tx 
        (refer to Histogram Mode) 
    */
    u32    Octets64;         /* 64 Octets, offset 8 */
    u32    Octets127;        /* 65 to 127 Octets, offset 9 */
    u32    Octets255;        /* 128 to 255 Octets, offset 10 */
    u32    Octets511;        /* 256 to 511 Octets, offset 11 */
    u32    Octets1023;       /* 512 to 1023 Octets, offset 12 */
    u32    OctetsMax;        /* 1024 to Max Octets, offset 13 */
    u32    OutOctetsLo;      /* offset 14 */
    u32    OutOctetsHi;      /* offset 15 */
    u32    OutUnicasts;      /* offset 16 */
    u32    Excessive;        /* offset 17 */
    u32    OutMulticasts;    /* offset 18 */
    u32    OutBroadcasts;    /* offset 19 */
    u32    Single;           /* offset 20 */

    u32    OutPause;         /* offset 21 */
    u32    InPause;          /* offset 22 */
    u32    Multiple;         /* offset 23 */
    u32    InUndersize;        /* offset 24 */
    u32    InFragments;        /* offset 25 */
    u32    InOversize;         /* offset 26 */
    u32    InJabber;           /* offset 27 */
    u32    InRxErr;          /* offset 28 */
    u32    InFCSErr;         /* offset 29 */
    u32    Collisions;       /* offset 30 */
    u32    Late;             /* offset 31 */
	/* Bank 1 */
    u32    InDiscards;       /* offset 0x00 */
    u32    InFiltered;       /* offset 0x01 */
    u32    InAccepted;       /* offset 0x02 */
    u32    InBadAccepted;    /* offset 0x03 */
    u32    InGoodAvbClassA;  /* offset 0x04 */
    u32    InGoodAvbClassB;  /* offset 0x05 */
    u32    InBadAvbClassA ;  /* offset 0x06 */
    u32    InBadAvbClassB ;  /* offset 0x07 */
    u32    TCAMCounter0;     /* offset 0x08 */
    u32    TCAMCounter1;     /* offset 0x09 */
    u32    TCAMCounter2;     /* offset 0x0a */
    u32    TCAMCounter3;     /* offset 0x0b */
    u32    InDroppedAvbA;    /* offset 0x0c */
    u32    InDroppedAvbB;    /* offset 0x0d */
    u32    InDaUnknown ;     /* offset 0x0e */
    u32    InMGMT;           /* offset 0x0f */
    u32    OutQueue0;        /* offset 0x10 */
    u32    OutQueue1;        /* offset 0x11 */
    u32    OutQueue2;        /* offset 0x12 */
    u32    OutQueue3;        /* offset 0x13 */
    u32    OutQueue4;        /* offset 0x14 */
    u32    OutQueue5;        /* offset 0x15 */
    u32    OutQueue6;        /* offset 0x16 */
    u32    OutQueue7;        /* offset 0x17 */
    u32    OutCutThrough;    /* offset 0x18 */
	u32    InBadQbv;		 /* offset 0x19 */
    u32    OutOctetsA;       /* offset 0x1a */
    u32    OutOctetsB;       /* offset 0x1b */
    u32    OutYel;           /* offset 0x1c */
    u32    OutDroppedYel;    /* offset 0x1d */
    u32    OutDiscards;      /* offset 0x1e */
    u32    OutMGMT;          /* offset 0x1f */

}AMETHYST_MSD_STATS_COUNTER_SET;

/*
 *    Type definition for MIB counter operation
*/
typedef enum 
{
	Amethyst_STATS_FLUSH_ALL = 1,            /* Flush all counters for all ports */
	Amethyst_STATS_FLUSH_PORT = 2,           /* Flush all counters for a port */
	Amethyst_STATS_READ_COUNTER = 4,         /* Read a specific counter from a port */
	Amethyst_STATS_READ_ALL = 5,              /* Read all counters from a port */
} AMETHYST_MSD_STATS_OPERATION;

/****************************************************************************/
/*																			*/
/* Block: Switch Registers													*/
/*																			*/
/****************************************************************************/

/* Define Switch SMI Device Address */
#define AMETHYST_PORT_START_ADDR	0x0	    /* Start Port Address */
#define AMETHYST_GLOBAL1_DEV_ADDR	0x1B	/* Global 1 Device Address */
#define AMETHYST_GLOBAL2_DEV_ADDR	0x1C	/* Global 2 Device Address */
#define AMETHYST_TCAM_DEV_ADDR		0x1F	/* TCAM Device Address */

/* Switch Per Port Registers */
#define AMETHYST_QD_REG_PORT_STATUS          0x0	    /* Port Status Register */
#define AMETHYST_QD_REG_PHY_CONTROL          0x1		/* Physical Control Register */
#define AMETHYST_QD_REG_LIMIT_PAUSE_CONTROL  0x2		/* Flow Control Register */
#define AMETHYST_QD_REG_SWITCH_ID            0x3		/* Switch Identifier Register */
#define AMETHYST_QD_REG_PORT_CONTROL         0x4		/* Port Control Register */
#define AMETHYST_QD_REG_PORT_CONTROL1        0x5		/* Port Control 1 Register */
#define AMETHYST_QD_REG_PORT_VLAN_MAP        0x6		/* Port based VLAN Map Register */
#define AMETHYST_QD_REG_PVID                 0x7		/* Port VLAN ID & Priority Register */
#define AMETHYST_QD_REG_PORT_CONTROL2        0x8		/* Port Control 2 Register */
#define AMETHYST_QD_REG_EGRESS_RATE_CTRL     0x9		/* Egress Rate Control Register */
#define AMETHYST_QD_REG_EGRESS_RATE_CTRL2    0xA		/* Egress Rate Control 2 Register */
#define AMETHYST_QD_REG_PAV                  0xB		/* Port Association Vector Register */
#define AMETHYST_QD_REG_PORT_ATU_CONTROL     0xC		/* Port ATU Control Register */
#define AMETHYST_QD_REG_PRI_OVERRIDE         0xD		/* Override Register */
#define AMETHYST_QD_REG_POLICY_CONTROL       0xE		/* Policy & MGMT Control Register */
#define AMETHYST_QD_REG_EXTENDED_PORT_CTRL_CMD	0x10	/*Extended Port Control Operation Register*/
#define AMETHYST_QD_REG_EXTENDED_PORT_CTRL_DATA 0x11	/*Extended Port Control Data Register*/
#define AMETHYST_QD_REG_LED_CONTROL          0x16		/* LED Control Register */
#define AMETHYST_QD_REG_IP_PRI_MAPPING_TBL   0x17		/* IP Priority Mapping Table Register */
#define AMETHYST_QD_REG_IEEE_PRI_MAPPING_TBL 0x18		/* IEEE Priority Mapping Table Register */
#define AMETHYST_QD_REG_PORT_CONTROL3		0x19		/* Port Control 3 Register */
#define AMETHYST_QD_REG_Q_COUNTER           0x1B		/* Queue Counter Register */
#define AMETHYST_QD_REG_Q_CONTROL           0x1C		/* Port Queue Control Register */
#define AMETHYST_QD_REG_CT_CONTROL          0x1E		/* Cut Through Control Register */
#define AMETHYST_QD_REG_DBG_COUNTER         0x1F		/* Debug Counters Register */

/* Switch Global Registers */
#define AMETHYST_QD_REG_GLOBAL_STATUS       0x0		/* Global Status Register */
#define AMETHYST_QD_REG_ATU_FID_REG        	0x1		/* ATU FID Register */
#define AMETHYST_QD_REG_VTU_FID_REG        	0x2		/* VTU FID Register */
#define AMETHYST_QD_REG_STU_SID_REG        	0x3		/* VTU SID Register */
#define AMETHYST_QD_REG_GLOBAL_CONTROL      0x4		/* Global Control Register */
#define AMETHYST_QD_REG_VTU_OPERATION      	0x5		/* VTU Operation Register */
#define AMETHYST_QD_REG_VTU_VID_REG        	0x6		/* VTU VID Register */
#define AMETHYST_QD_REG_VTU_DATA1_REG      	0x7		/* VTU/STU Data Register Ports 0 to 7 */
#define AMETHYST_QD_REG_VTU_DATA2_REG      	0x8		/* VTU/STU Data Register Ports 8 to 10 and Priority */
#define AMETHYST_QD_REG_ATU_CTRL_REG        0xA		/* ATU Control Register */
#define AMETHYST_QD_REG_ATU_OPERATION      	0xB		/* ATU Operation Register */
#define AMETHYST_QD_REG_ATU_DATA_REG       	0xC		/* ATU Data Register */
#define AMETHYST_QD_REG_ATU_MAC_BASE       	0xD		/* ATU MAC Address Register Base */
#define AMETHYST_QD_REG_MONITOR_MGMT_CTRL    0x1A    /* Monitor & MGMT Control Register */
#define AMETHYST_QD_REG_TOTAL_FREE_COUNTER   0x1B    /* Total Free Counter Register */
#define AMETHYST_QD_REG_GLOBAL_CONTROL2      0x1C    /* Global Control 2 Register */
#define AMETHYST_QD_REG_STATS_OPERATION    	0x1D    /* Stats Operation Register */
#define AMETHYST_QD_REG_STATS_COUNTER3_2   	0x1E    /* Stats Counter Register Bytes 3 & 2 */
#define AMETHYST_QD_REG_STATS_COUNTER1_0   	0x1F    /* Stats Counter Register Bytes 1 & 0 */

/*Multi Chip Directly access Registers*/
#define AMETHYST_QD_REG_TOP_LEVEL_INTERRUPT         0x4
#define AMETHYST_QD_REG_ATU_FID_REG_MULTICHIP     	0x9
#define AMETHYST_QD_REG_IGR_RATE_COMMAND_MULTICHIP  0x14
#define AMETHYST_QD_REG_IGR_RATE_DATA_MULTICHIP   	0x15

/* Switch Global 2 Registers */
#define AMETHYST_QD_REG_INT_SOURCE     		0x0		/* Interrupt Source Register */
#define AMETHYST_QD_REG_INT_MASK       		0x1		/* Interrupt Mask Register */
#define AMETHYST_QD_REG_FLOWCTRL_DELAY    	0x4		/* Flow Control Delay Register */
#define AMETHYST_QD_REG_MANAGEMENT        	0x5		/* Switch Management Register */
#define AMETHYST_QD_REG_ROUTING_TBL       	0x6		/* Device Mapping Table Register */
#define AMETHYST_QD_REG_TRUNK_MASK_TBL    	0x7		/* Trunk Mask Table Register */
#define AMETHYST_QD_REG_TRUNK_ROUTING     	0x8		/* Trunk Mapping Register */	
#define AMETHYST_QD_REG_IGR_RATE_COMMAND    0x9		/* Ingress Rate Command Register */	
#define AMETHYST_QD_REG_IGR_RATE_DATA   	0xA		/* Ingress Rate Data Register */	
#define AMETHYST_QD_REG_PVT_ADDR            0xB		/* Cross Chip Port VLAN Addr Register */	
#define AMETHYST_QD_REG_PVT_DATA            0xC		/* Cross Chip Port VLAN Data Register */	
#define AMETHYST_QD_REG_SWITCH_MAC          0xD		/* Switch MAC/WoL/WoF Register */	
#define AMETHYST_QD_REG_ATU_STATS           0xE		/* ATU Stats Register */	
#define AMETHYST_QD_REG_PRIORITY_OVERRIDE 	0xF		/* Priority Override Table Register */	
#define AMETHYST_QD_REG_ENERGE_MNT        	0x12	/* Energy Management Register */
#define AMETHYST_QD_REG_IMP_COMM_DBG		0x13	/* IMP Comm/Debug Register */
#define AMETHYST_QD_REG_EEPROM_COMMAND    	0x14	/* EEPROM Command Register */	
#define AMETHYST_QD_REG_EEPROM_DATA      	0x15	/* EEPROM Address Register */	
#define AMETHYST_QD_REG_AVB_COMMAND     	0x16	/* AVB Command Register */	
#define AMETHYST_QD_REG_AVB_DATA          	0x17	/* AVB Data Register */
#define AMETHYST_QD_REG_SMI_PHY_CMD       	0x18	/* SMI PHY Command Register */	
#define AMETHYST_QD_REG_SMI_PHY_DATA      	0x19	/* SMI PHY Data Register */	
#define AMETHYST_QD_REG_SCRATCH_MISC      	0x1A	/* Scratch & Misc Register */	
#define AMETHYST_QD_REG_WD_CONTROL        	0x1B	/* Watch Dog Control Register */	
#define AMETHYST_QD_REG_QOS_WEIGHT        	0x1C	/* QoS Weights Register */	
#define AMETHYST_QD_REG_MISC				0x1D	/* Misc Register */	
#define AMETHYST_QD_REG_CUT_THROU_CTRL    	0x1F	/* Cut Through Control Register */	

/* TCAM Registers - Page 0 */
#define AMETHYST_QD_REG_TCAM_OPERATION         0x0		/* TCAM Operation Register */
#define AMETHYST_QD_REG_TCAM_P0_KEYS_1         0x2		/* TCAM Key Register 1 */
#define AMETHYST_QD_REG_TCAM_P0_KEYS_2         0x3		/* TCAM Key Register 2 */
#define AMETHYST_QD_REG_TCAM_P0_KEYS_3         0x4		/* TCAM Key Register 3 */
#define AMETHYST_QD_REG_TCAM_P0_KEYS_4         0x5		/* TCAM Key Register 4 */
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_1   0x6		/* TCAM Match Data Register 1 */
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_2   0x7		/* TCAM Match Data Register 2 */
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_3   0x8
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_4   0x9
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_5   0xa
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_6   0xb
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_7   0xc
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_8   0xd
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_9   0xe
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_10  0xf
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_11  0x10
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_12  0x11
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_13  0x12
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_14  0x13
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_15  0x14
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_16  0x15
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_17  0x16
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_18  0x17
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_19  0x18
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_20  0x19
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_21  0x1a
#define AMETHYST_QD_REG_TCAM_P0_MATCH_DATA_22  0x1b

/* TCAM Registers - Page 1 */
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_23   0x2		/* TCAM Match Data Register 23 */
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_24   0x3		/* TCAM Match Data Register 24 */
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_25   0x4
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_26   0x5
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_27   0x6
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_28   0x7
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_29   0x8
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_30   0x9
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_31   0xa
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_32   0xb
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_33   0xc
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_34   0xd
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_35   0xe
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_36   0xf
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_37   0x10
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_38   0x11
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_39   0x12
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_40   0x13
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_41   0x14
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_42   0x15
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_43   0x16
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_44   0x17
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_45   0x18
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_46   0x19
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_47   0x1a
#define AMETHYST_QD_REG_TCAM_P1_MATCH_DATA_48   0x1b

/* TCAM Registers - Page 2 */
#define AMETHYST_QD_REG_TCAM_P2_ACTION_1        0x2		/* TCAM Action 1 Register */
#define AMETHYST_QD_REG_TCAM_P2_ACTION_2        0x3		/* TCAM Action 2 Register */
#define AMETHYST_QD_REG_TCAM_P2_ACTION_3        0x4		/* TCAM Action 3 Register */
#define AMETHYST_QD_REG_TCAM_P2_ACTION_5        0x6		/* TCAM Action 5 Register */
#define AMETHYST_QD_REG_TCAM_P2_ACTION_6        0x7		/* TCAM Action 6 Register */
#define AMETHYST_QD_REG_TCAM_P2_ACTION_7        0x8		/* TCAM Action 7 Register */
#define AMETHYST_QD_REG_TCAM_P2_DEBUG_PORT      0x1C		/* TCAM Debug Register */
#define AMETHYST_QD_REG_TCAM_P2_ALL_HIT         0x1F		/* TCAM Debug Register 2 */

/* TCAM Registers - Page 3 */
#define AMETHYST_QD_REG_TCAM_EGR_PORT           0x1		/* Egress TCAM Port Register */
#define AMETHYST_QD_REG_TCAM_EGR_ACTION_1       0x2		/* Egress TCAM Action 1 Register */
#define AMETHYST_QD_REG_TCAM_EGR_ACTION_2       0x3		/* Egress TCAM Action 2 Register */
#define AMETHYST_QD_REG_TCAM_EGR_ACTION_3       0x4		/* Egress TCAM Action 3 Register */

/****************************************************************************/
/* Exported PIRL Types			 			                                */
/****************************************************************************/
/*
*  typedef: enum MSD_PIRL_COUNT_MODE
*
*  Description: Enumeration of the port egress rate limit counting mode.
*
*  Fields:
*      MSD_PIRL_COUNT_FRAME -
*                Count the number of frames
*      MSD_PIRL_COUNT_ALL_LAYER1 -
*                Count all Layer 1 bytes:
*                Preamble (8bytes) + Frame's DA to CRC + IFG (12bytes)
*      MSD_PIRL_COUNT_ALL_LAYER2 -
*                Count all Layer 2 bytes: Frame's DA to CRC
*      MSD_PIRL_COUNT_ALL_LAYER3 -
*                Count all Layer 3 bytes:
*                Frame's DA to CRC - 18 - 4 (if frame is tagged)
*/
typedef enum
{
	MSD_PIRL_COUNT_FRAME = 0,
	MSD_PIRL_COUNT_ALL_LAYER1,
	MSD_PIRL_COUNT_ALL_LAYER2,
	MSD_PIRL_COUNT_ALL_LAYER3
} MSD_PIRL_COUNT_MODE;

/*
*  typedef: struct MSD_PIRL_CUSTOM_RATE_LIMIT
*
*  Description:  This structure includes all the necessary parameters used to set port
*                ingress rate limit. It's the output of API 'gpirlCustomSetup' based on users'
*                target rate and target burst allocation. And also, this structure can be used
*                in API 'gpirlWriteResource' to configure IRL port resource. You can check IRL
*                sample code for detail usage. If the real Ingress Rate Limit is too much off
*                from the target rate, please contact FAE and gets the correct parameters.
*
*  Fields:
*        isValid         - Reserved for future use
*        ebsLimit        - Excess Burst Size limit on the Yellow bucket ( 0 ~ 0xFFFFFF)
*                          In Single Rate Two Color Mode, EBSLimit must be cleared to zero.
*        cbsLimit        - Committed Burst Size limit on the Green bucket
*        bktIncrement    - the amount of tokens that need to be added for each
*                          byte of packet information.
*        bktRateFactorGrn   - the amount of tokens that need to be decremented for each rate
*                             resource decrement on the Green Bucket.
*                             The rate for Green is:
*                                  Rate in bps: Calculated Igress Rate(bps) = (500,000,000 * BRFGrn)/BktInc
*                                  Rate if fps: Calculated Igress Rate(fps) = (62,000,000 * BRFGrn)/BktInc
*        bktRateFactorYel   - the amount of tokens that need to be decremented for each rate
*                             resource decrement on the Yellow Bucket.
*                             In Single Rate Two Color Mode, BRFYel must be cleared to zero.
*                             In Single Rate Three Color Mode, BRFYel must be equal to BRFGrn. Both must be non-zero.
*                                  The rate for (Green + Yellow) is:
*                                      Rate in bps: Calculated Igress Rate(bps) = (500,000,000 * BRFYel)/BktInc
*                                      Rate if fps: Calculated Igress Rate(fps) = (62,000,000 * BRFTel)/BktInc
*							   In Dual Rate Three Color Mode, BRFYel must be greater than BRFGrn
*                                  The Yellow rate limit is:
*                                      Rate in bps: Calculated Igress Rate(bps) = (500,000,000 * (BRFYel-BRFGrn)/BktInc
*                                      Rate if fps: Calculated Igress Rate(fps) = (62,000,000 * (BRFYel-BRFGrn)/BktInc
*        IRLConstant      - internal coefficient
*        countMode	      - bytes to be counted for accounting
*                          MSD_PIRL2_COUNT_FRAME,
*                          MSD_PIRL2_COUNT_ALL_LAYER1,
*                          MSD_PIRL2_COUNT_ALL_LAYER2, or
*                          MSD_PIRL2_COUNT_ALL_LAYER3
*/
typedef struct
{
	int       isValid;
	u32        ebsLimit;
	u32        cbsLimit;
	u32        bktIncrement;
	u32        bktRateFactorGrn;
	u32        bktRateFactorYel;
	/*topaz*/
	MSD_U32		  IRLConstant;

	/*Count Mode*/
	MSD_PIRL_COUNT_MODE       countMode;

} MSD_PIRL_CUSTOM_RATE_LIMIT;
/****************************************************************************/
/* Exported PIRL Types			 			                                   */
/****************************************************************************/

/*
 *  typedef: enum AMETHYST_MSD_PIRL_COUNT_MODE
 *
 *  Description: Enumeration of the port egress rate limit counting mode.
 *
 *  Enumerations:
 *      AMETHYST_MSD_PIRL_COUNT_FRAME -
 *                Count the number of frames
 *      AMETHYST_MSD_PIRL_COUNT_ALL_LAYER1 -
 *                Count all Layer 1 bytes:
 *                Preamble (8bytes) + Frame's DA to CRC + IFG (12bytes)
 *      AMETHYST_MSD_PIRL_COUNT_ALL_LAYER2 -
 *                Count all Layer 2 bytes: Frame's DA to CRC
 *      AMETHYST_MSD_PIRL_COUNT_ALL_LAYER3 -
 *                Count all Layer 3 bytes:
 *                Frame's DA to CRC - 18 - 4 (if frame is tagged)
 */
typedef enum
{
    AMETHYST_MSD_PIRL_COUNT_FRAME = 0,
    AMETHYST_MSD_PIRL_COUNT_ALL_LAYER1,
    AMETHYST_MSD_PIRL_COUNT_ALL_LAYER2,
    AMETHYST_MSD_PIRL_COUNT_ALL_LAYER3
} AMETHYST_MSD_PIRL_COUNT_MODE;

/*
 *  typedef: struct AMETHYST_MSD_PIRL_CUSTOM_RATE_LIMIT
 *
 *  Description:  This structure includes all the necessary parameters used to set port 
 *                ingress rate limit. It's the output of API 'Amethyst_gpirlCustomSetup' based on users' 
 *                target rate and target burst allocation. And also, this structure can be used 
 *                in API 'Amethyst_gpirlWriteResource' to configure IRL port resource. You can check IRL 
 *                sample code for detail usage. If the real Ingress Rate Limit is too much off 
 *                from the target rate, please contact FAE and gets the correct parameters.
 *
 *        isValid         - Reserved for future use
 *        ebsLimit        - Excess Burst Size limit on the Yellow bucket ( 0 ~ 0xFFFFFF)
 *                          In Single Rate Two Color Mode, EBSLimit must be cleared to zero.
 *        cbsLimit        - Committed Burst Size limit on the Green bucket
 *        bktIncrement    - the amount of tokens that need to be added for each 
 *                          byte of packet information.
 *        bktRateFactorGrn   - the amount of tokens that need to be decremented for each rate 
 *                             resource decrement on the Green Bucket.
 *                             The rate for Green is:
 *                                  Rate in bps: Calculated Igress Rate(bps) = (500,000,000 * BRFGrn)/BktInc
 *                                  Rate if fps: Calculated Igress Rate(fps) = (62,000,000 * BRFGrn)/BktInc
 *        bktRateFactorYel   - the amount of tokens that need to be decremented for each rate 
 *                             resource decrement on the Yellow Bucket.
 *                             In Single Rate Two Color Mode, BRFYel must be cleared to zero.
 *                             In Single Rate Three Color Mode, BRFYel must be equal to BRFGrn. Both must be non-zero.
 *                                  The rate for (Green + Yellow) is:
 *                                      Rate in bps: Calculated Igress Rate(bps) = (500,000,000 * BRFYel)/BktInc
 *                                      Rate if fps: Calculated Igress Rate(fps) = (62,000,000 * BRFTel)/BktInc						
 *							   In Dual Rate Three Color Mode, BRFYel must be greater than BRFGrn
 *                                  The Yellow rate limit is:
 *                                      Rate in bps: Calculated Igress Rate(bps) = (500,000,000 * (BRFYel-BRFGrn)/BktInc
 *                                      Rate if fps: Calculated Igress Rate(fps) = (62,000,000 * (BRFYel-BRFGrn)/BktInc		
 *        Delta           - the error ratio between Calculated Rate and Target Rate((Rate - TgtRate)/TgtRate)
 *        countMode	      - bytes to be counted for accounting
 *                          MSD_PIRL2_COUNT_FRAME,
 *                          MSD_PIRL2_COUNT_ALL_LAYER1,
 *                          MSD_PIRL2_COUNT_ALL_LAYER2, or
 *                          MSD_PIRL2_COUNT_ALL_LAYER3
*/
typedef struct
{
    int        isValid;
    u32        ebsLimit;
    u32        cbsLimit;
    u32        bktIncrement;
    u32        bktRateFactorGrn;
    u32        bktRateFactorYel;
    u32        Delta;

    /*Count Mode*/
    AMETHYST_MSD_PIRL_COUNT_MODE       countMode;

} AMETHYST_MSD_PIRL_CUSTOM_RATE_LIMIT;
/*
*  typedef: enum MSD_PIRL_FC_DEASSERT
*
*  Description: Enumeration of the port flow control de-assertion mode on PIRL.
*
*  Fields:
*      MSD_PIRL_FC_DEASSERT_EMPTY -
*                De-assert when the ingress rate resource has become empty
*      MSD_PIRL_FC_DEASSERT_CBS_LIMIT -
*                De-assert when the ingress rate resource has enough room as
*                specified by Limit.
*/
typedef enum
{
	MSD_PIRL_FC_DEASSERT_EMPTY = 0,
	MSD_PIRL_FC_DEASSERT_LIMIT
} MSD_PIRL_FC_DEASSERT;

/*
* Typedef: enum MSD_PIRL_ACTION
*
* Description: Defines the Action that should be taken when
*        there there are not enough tokens to accept the entire incoming frame
*
* Fields:
*        MSD_PIRL_ACTION_DROP - drop packets
*        MSD_PIRL_ACTION_FC - end flow control packet
*/
typedef enum
{
	MSD_PIRL_ACTION_DROP = 0x0,
	MSD_PIRL_ACTION_FC = 0x1
} MSD_PIRL_ACTION;

/*
*  typedef: struct MSD_PIRL_DATA
*
*  Description: data structure for PIRL Bucket programing that is resource based.
*
*  Fields:
*      bktTypeMask	- various traffic type to be counted
*      tcamFlows	- indicate to account for tcam flows 0 to 3, but not TCP/UDP data
*      priAndPt    - determine the incoming frames that get rate limited using
*                    this ingress rate resource.
*                         MSD_TRUE - typeMask And priMask
*                         MSD_FALSE - typeMask Or priMask
*      useFPri     - used to determine what priority type this resource is to account for
*                    via the Pri Select bits below as follows:
*                          MSD_TRUE  - use the FPri assigned to the frame
*                          MSD_FALSE - use the QPri assigned to the frame
*      priSelect   - priority bit mask that each bit indicates one of the eight
*                    priorities. Setting each one of these bits indicates
*                    that this particular rate resource is slated to account for
*                    incoming frames with the enabled bits' priority.
*      colorAware  - set to color aware mode or color blind mode
*      accountGrnOverflow	- Green&Yellow buckets are connected when in Coloar Aware mode.
*      accountQConf    - account discarded frames due to queue congestion
*      accountFiltered - account filtered frames. Account for All
*      samplingMode    - sample one out of so many frames/bytes for a stream of frames
*      actionMode      - action should be taken when there are not enough tokens
*                        to accept the entire incoming frame
*                                MSD_PIRL_ACTION_DROP - drop frame
*                                MSD_PIRL_ACTION_FC - send flow control mode.
*      fcMode         - flow control de-assertion mode when actionMode is
*                       set to MSD_PIRL_ACTION_FC.
*                                MSD_PIRL_FC_DEASSERT_EMPTY -
*                                    De-assert when the ingress rate resource has
*                                    become empty.
*                                MSD_PIRL_FC_DEASSERT_CBS_LIMIT -
*                                    De-assert when the ingress rate resource has
*                                    enough room as specified by the EBSLimit.
*      fcPriority     - When the ingress port associated with this resource is enabled
*                          for Priority Flow Control and if this resource is configured to
*                          use FC Action, then this register is used to indicate the PFC
*                          priority that is to be used on the Priority flow Control frames
*                          generated due to this resource.
*
*/
typedef struct
{
	/* frame types to be accounted*/
	u16 			bktTypeMask;
	MSD_BOOL		tcamFlows;
	MSD_BOOL		priAndPt;
	MSD_BOOL		useFPri;
	u8				priSelect;

	/*color aware or color blind mode*/
	MSD_BOOL		colorAware;

	/*condition for account*/
	MSD_BOOL		accountGrnOverflow;
	MSD_BOOL		accountQConf;
	MSD_BOOL		accountFiltered;

	/*condition for account*/
	MSD_BOOL		samplingMode;

	/*flow control*/
	MSD_PIRL_ACTION    actionMode;
	MSD_PIRL_FC_DEASSERT		fcMode;

	u8	    		fcPriority;	/* used when port PFC mode enabled */

	/*customer setup*/
	MSD_PIRL_CUSTOM_RATE_LIMIT customSetup;
} MSD_PIRL_DATA;
/*
 * Typedef: enum AMETHYST_MSD_PIRL_OPERATION
 *
 * Description: Defines the PIRL (Port Ingress Rate Limit) Operation type
 *
 * Fields:
 *      Amethyst_PIRL_INIT_ALL_RESOURCE - Initialize all resources to the inital state
 *      Amethyst_PIRL_INIT_RESOURCE     - Initialize selected resources to the inital state
 *      Amethyst_PIRL_WRITE_RESOURCE    - Write to the selected resource/register
 *      Amethyst_PIRL_READ_RESOURCE     - Read from the selected resource/register
 */
typedef enum
{
	Amethyst_PIRL_READ_RESOURCE          = 0x0,
    Amethyst_PIRL_INIT_ALL_RESOURCE      = 0x1,
    Amethyst_PIRL_INIT_RESOURCE          = 0x2,
    Amethyst_PIRL_WRITE_RESOURCE         = 0x3
} AMETHYST_MSD_PIRL_OPERATION;
/*
 *  typedef: struct AMETHYST_MSD_PIRL_OP_DATA
 *
 *  Description: data required by PIRL Operation
 *
 *  Fields:
 *      irlUnit   - Ingress Rate Limit Unit that defines one of IRL resources.
 *      irlReg    - Ingress Rate Limit Register.
 *      irlData   - Ingress Rate Limit Data.
 */
typedef struct
{
    MSD_U32    irlUnit;
    MSD_U32    irlReg;
    MSD_U32    irlData;
} AMETHYST_MSD_PIRL_OP_DATA;

/*
 *  typedef: enum AMETHYST_MSD_PIRL_FC_DEASSERT
 *
 *  Description: Enumeration of the port flow control de-assertion mode on PIRL.
 *
 *  Enumerations:
 *      AMETHYST_MSD_PIRL_FC_DEASSERT_EMPTY -
 *                De-assert when the ingress rate resource has become empty
 *        AMETHYST_MSD_PIRL_FC_DEASSERT_CBS_LIMIT -
 *                De-assert when the ingress rate resource has enough room as
 *                specified by the CBSLimit.
 */
typedef enum
{
    AMETHYST_MSD_PIRL_FC_DEASSERT_EMPTY = 0,
    AMETHYST_MSD_PIRL_FC_DEASSERT_EBS_LIMIT
} AMETHYST_MSD_PIRL_FC_DEASSERT;

/*
 * Typedef: enum AMETHYST_MSD_PIRL_ACTION
 *
 * Description: Defines the Action that should be taken when
 *        there there are not enough tokens to accept the entire incoming frame
 *
 * Fields:
 *        Amethyst_PIRL_ACTION_DROP - drop packets
 *        Amethyst_PIRL_ACTION_FC - end flow control packet
 */
typedef enum
{
    Amethyst_PIRL_ACTION_DROP = 0x0,
    Amethyst_PIRL_ACTION_FC     = 0x1
} AMETHYST_MSD_PIRL_ACTION;

/*
 *  typedef: struct AMETHYST_MSD_PIRL_DATA
 *
 *  Description: data structure for PIRL Bucket programing that is resource based.
 *
 *  Fields:
 *      bktTypeMask	- various traffic type to be counted
 *      tcamFlows	- indicate to account for tcam flows 0 to 3, but not TCP/UDP data                 
 *      priAndPt    - determine the incoming frames that get rate limited using
 *                    this ingress rate resource.
 *                         MSD_TRUE - typeMask And priMask
 *                         MSD_FALSE - typeMask Or priMask         
 *      useFPri     - used to determine what priority type this resource is to account for
 *                    via the Pri Select bits below as follows:
 *                          MSD_TRUE  - use the FPri assigned to the frame
 *                          MSD_FALSE - use the QPri assigned to the frame
 *      priSelect   - priority bit mask that each bit indicates one of the eight
 *                    priorities. Setting each one of these bits indicates
 *                    that this particular rate resource is slated to account for
 *                    incoming frames with the enabled bits' priority.
 *      colorAware  - set to color aware mode or color blind mode
 *      accountGrnOverflow	- Green&Yellow buckets are connected when in Coloar Aware mode.
 *      accountQConf    - account discarded frames due to queue congestion
 *      accountFiltered - account filtered frames. Account for All 
 *      samplingMode    - sample one out of so many frames/bytes for a stream of frames
 *      actionMode      - action should be taken when there are not enough tokens
 *                        to accept the entire incoming frame
 *                                Amethyst_PIRL_ACTION_DROP - drop frame
 *                                Amethyst_PIRL_ACTION_FC - send flow control mode.
 *      fcMode         - flow control de-assertion mode when actionMode is
 *                       set to Amethyst_PIRL_ACTION_FC.
 *                                AMETHYST_MSD_PIRL_FC_DEASSERT_EMPTY -
 *                                    De-assert when the ingress rate resource has 
 *                                    become empty.
 *                                AMETHYST_MSD_PIRL_FC_DEASSERT_CBS_LIMIT -
 *                                    De-assert when the ingress rate resource has 
 *                                    enough room as specified by the EBSLimit.     
 *      fcPriority     - When the ingress port associated with this resource is enabled
 *                          for Priority Flow Control and if this resource is configured to 
 *                          use FC Action, then this register is used to indicate the PFC 
 *                          priority that is to be used on the Priority flow Control frames 
 *                          generated due to this resource.
 *                                
 */
typedef struct
{
    /* frame types to be accounted*/
    u16 		bktTypeMask;
    MSD_BOOL		tcamFlows;
    MSD_BOOL		priAndPt;
    MSD_BOOL		useFPri;
    u8			priSelect;

    /*color aware or color blind mode*/
    MSD_BOOL		colorAware;

    /*condition for account*/
    MSD_BOOL		accountGrnOverflow;
    MSD_BOOL		accountQConf;
    MSD_BOOL		accountFiltered;

    /*condition for account*/
    MSD_BOOL		samplingMode;
	   
	/*flow control*/
    AMETHYST_MSD_PIRL_ACTION    actionMode;
    AMETHYST_MSD_PIRL_FC_DEASSERT		fcMode;
    u8	    fcPriority;	/* used when port PFC mode enabled */
	
    /*customer setup*/
    AMETHYST_MSD_PIRL_CUSTOM_RATE_LIMIT customSetup;

} AMETHYST_MSD_PIRL_DATA;

/*
 *  typedef: enum AMETHYST_MSD_ELIMIT_MODE
 *
 *  Description: Enumeration of the port egress rate limit counting mode.
 *
 *  Enumerations:
 *      AMETHYST_MSD_ELIMIT_FRAME -
 *                Count the number of frames
 *      AMETHYST_MSD_ELIMIT_LAYER1 -
 *                Count all Layer 1 bytes: 
 *                Preamble (8bytes) + Frame's DA to CRC + IFG (12bytes)
 *      AMETHYST_MSD_ELIMIT_LAYER2 -
 *                Count all Layer 2 bytes: Frame's DA to CRC
 *      AMETHYST_MSD_ELIMIT_LAYER3 -
 *                Count all Layer 3 bytes: 
 *                Frame's DA to CRC - 18 - 4 (if frame is tagged)
 */
typedef enum
{
    AMETHYST_MSD_ELIMIT_FRAME = 0,
    AMETHYST_MSD_ELIMIT_LAYER1,
    AMETHYST_MSD_ELIMIT_LAYER2,
    AMETHYST_MSD_ELIMIT_LAYER3
} AMETHYST_MSD_ELIMIT_MODE;
#define AMETHYST_MSD_GET_RATE_LIMIT_PER_FRAME(_frames, _dec)    \
        ((_frames)?(1000000000 / (16 * (_frames)) + ((1000000000 % (16 * (_frames)) > (16*(_frames)/2))?1:0)):0)

#define AMETHYST_MSD_GET_RATE_LIMIT_PER_BYTE(_kbps, _dec)    \
        ((_kbps)?((8000000*(_dec)) / (16 * (_kbps)) + ((8000000*(_dec)) % (16 * (_kbps))?1:0)):0)


/* Definition for the Port Duplex Mode */
typedef enum
{
    AMETHYST_MSD_DUPLEX_HALF,
    AMETHYST_MSD_DUPLEX_FULL,
    AMETHYST_MSD_DUPLEX_AUTO
} AMETHYST_MSD_PORT_FORCE_DUPLEX_MODE;

/* Definition for full duplex flow control mode */
typedef enum
{
	Amethyst_PORT_FC_TX_RX_ENABLED,
	Amethyst_PORT_RX_ONLY,
	Amethyst_PORT_TX_ONLY,
	Amethyst_PORT_PFC_ENABLED
} AMETHYST_MSD_PORT_FC_MODE;

/* Definition for the forced Port Speed */
typedef enum
{
    Amethyst_PORT_FORCE_SPEED_10M = 0x0,
    Amethyst_PORT_FORCE_SPEED_100M,
    Amethyst_PORT_FORCE_SPEED_200M,  
    Amethyst_PORT_FORCE_SPEED_1000M,
	Amethyst_PORT_FORCE_SPEED_2_5G,
	Amethyst_PORT_FORCE_SPEED_5G,
    Amethyst_PORT_FORCE_SPEED_10G,     
    Amethyst_PORT_DO_NOT_FORCE_SPEED
} AMETHYST_MSD_PORT_FORCED_SPEED_MODE;

typedef enum
{
    AMETHYST_PORT_FORCE_LINK_UP,
    AMETHYST_PORT_FORCE_LINK_DOWN,
    AMETHYST_PORT_DO_NOT_FORCE_LINK
} AMETHYST_MSD_PORT_FORCE_LINK_MODE;
/*
* typedef: enum AMETHYST_MSD_PHY_AUTO_MODE
*
* Description: Enumeration of Autonegotiation mode.
*    Auto for both speed and duplex.
*    Auto for speed only and Full duplex.
*    Auto for speed only and Half duplex. (1000Mbps is not supported)
*    Auto for duplex only and speed 1000Mbps.
*    Auto for duplex only and speed 100Mbps.
*    Auto for duplex only and speed 10Mbps.
*    1000Mbps Full duplex.
*    100Mbps Full duplex.
*    100Mbps Half duplex.
*    10Mbps Full duplex.
*    10Mbps Half duplex.
*/

typedef enum
{
    AMETHYST_SPEED_AUTO_DUPLEX_AUTO,
    AMETHYST_SPEED_1000_DUPLEX_AUTO,
    AMETHYST_SPEED_100_DUPLEX_AUTO,
    AMETHYST_SPEED_10_DUPLEX_AUTO,
    AMETHYST_SPEED_AUTO_DUPLEX_FULL,
    AMETHYST_SPEED_AUTO_DUPLEX_HALF,
    AMETHYST_SPEED_1000_DUPLEX_FULL,
    AMETHYST_SPEED_1000_DUPLEX_HALF,
    AMETHYST_SPEED_100_DUPLEX_FULL,
    AMETHYST_SPEED_100_DUPLEX_HALF,
    AMETHYST_SPEED_10_DUPLEX_FULL,
    AMETHYST_SPEED_10_DUPLEX_HALF
}AMETHYST_MSD_PHY_AUTO_MODE;

/*
* typedef: enum AMETHYST_MSD_PHY_SPEED
*
* Description: Enumeration of Phy Speed
*
* Enumerations:
*    AMETHYST_PHY_SPEED_10_MBPS   - 10Mbps
*    AMETHYST_PHY_SPEED_100_MBPS    - 100Mbps
*    AMETHYST_PHY_SPEED_1000_MBPS - 1000Mbps
*/
typedef enum
{
    AMETHYST_PHY_SPEED_10_MBPS,
    AMETHYST_PHY_SPEED_100_MBPS,
    AMETHYST_PHY_SPEED_1000_MBPS
} AMETHYST_MSD_PHY_SPEED;

/****************************************************************************/
/*																			*/
/* Block: INTERNAL PHY Registers											*/
/*																			*/
/****************************************************************************/

/* Internal Copper PHY registers*/
#define AMETHYST_QD_PHY_CONTROL_REG          0
#define AMETHYST_QD_PHY_AUTONEGO_AD_REG      4
#define AMETHYST_QD_PHY_AUTONEGO_1000AD_REG  9
#define AMETHYST_QD_PHY_SPEC_CONTROL_REG		16
#define AMETHYST_QD_PHY_PAGE_ANY_REG			22

/* Definitions for VCT registers */
#define AMETHYST_QD_REG_MDI0_VCT_STATUS     16
#define AMETHYST_QD_REG_MDI1_VCT_STATUS     17
#define AMETHYST_QD_REG_MDI2_VCT_STATUS     18
#define AMETHYST_QD_REG_MDI3_VCT_STATUS     19
#define AMETHYST_QD_REG_ADV_VCT_CONTROL_5   23
#define AMETHYST_QD_REG_ADV_VCT_SMPL_DIST   24
#define AMETHYST_QD_REG_ADV_VCT_CONTROL_8   20
#define AMETHYST_QD_REG_PAIR_SKEW_STATUS    20
#define AMETHYST_QD_REG_PAIR_SWAP_STATUS    21

/* Bit Definition for QD_PHY_CONTROL_REG */
#define AMETHYST_QD_PHY_RESET            0x8000
#define AMETHYST_QD_PHY_LOOPBACK         0x4000
#define AMETHYST_QD_PHY_SPEED            0x2000
#define AMETHYST_QD_PHY_AUTONEGO         0x1000
#define AMETHYST_QD_PHY_POWER            0x800
#define AMETHYST_QD_PHY_ISOLATE          0x400
#define AMETHYST_QD_PHY_RESTART_AUTONEGO 0x200
#define AMETHYST_QD_PHY_DUPLEX           0x100
#define AMETHYST_QD_PHY_SPEED_MSB        0x40

/* Bit Definition for QD_PHY_AUTONEGO_AD_REG */
#define AMETHYST_QD_PHY_NEXTPAGE           0x8000
#define AMETHYST_QD_PHY_REMOTEFAULT        0x4000
#define AMETHYST_QD_PHY_PAUSE              0x400
#define AMETHYST_QD_PHY_100_FULL           0x100
#define AMETHYST_QD_PHY_100_HALF           0x80
#define AMETHYST_QD_PHY_10_FULL            0x40
#define AMETHYST_QD_PHY_10_HALF            0x20

#define AMETHYST_QD_PHY_MODE_AUTO_AUTO    (AMETHYST_QD_PHY_100_FULL | AMETHYST_QD_PHY_100_HALF | AMETHYST_QD_PHY_10_FULL | AMETHYST_QD_PHY_10_HALF)
#define AMETHYST_QD_PHY_MODE_100_AUTO     (AMETHYST_QD_PHY_100_FULL | AMETHYST_QD_PHY_100_HALF)
#define AMETHYST_QD_PHY_MODE_10_AUTO      (AMETHYST_QD_PHY_10_FULL | AMETHYST_QD_PHY_10_HALF)
#define AMETHYST_QD_PHY_MODE_AUTO_FULL    (AMETHYST_QD_PHY_100_FULL | AMETHYST_QD_PHY_10_FULL)
#define AMETHYST_QD_PHY_MODE_AUTO_HALF    (AMETHYST_QD_PHY_100_HALF | AMETHYST_QD_PHY_10_HALF)

#define AMETHYST_QD_PHY_MODE_100_FULL    AMETHYST_QD_PHY_100_FULL
#define AMETHYST_QD_PHY_MODE_100_HALF    AMETHYST_QD_PHY_100_HALF
#define AMETHYST_QD_PHY_MODE_10_FULL        AMETHYST_QD_PHY_10_FULL    
#define AMETHYST_QD_PHY_MODE_10_HALF        AMETHYST_QD_PHY_10_HALF    

/* Gigabit Phy related definition */
#define AMETHYST_QD_GIGPHY_1000X_FULL_CAP    0x8
#define AMETHYST_QD_GIGPHY_1000X_HALF_CAP    0x4
#define AMETHYST_QD_GIGPHY_1000T_FULL_CAP    0x2
#define AMETHYST_QD_GIGPHY_1000T_HALF_CAP    0x1

#define AMETHYST_QD_GIGPHY_1000X_CAP        (AMETHYST_QD_GIGPHY_1000X_FULL_CAP|AMETHYST_QD_GIGPHY_1000X_HALF_CAP)
#define AMETHYST_QD_GIGPHY_1000T_CAP        (AMETHYST_QD_GIGPHY_1000T_FULL_CAP|AMETHYST_QD_GIGPHY_1000T_HALF_CAP)

#define AMETHYST_QD_GIGPHY_1000X_FULL        0x20
#define AMETHYST_QD_GIGPHY_1000X_HALF        0x40

#define AMETHYST_QD_GIGPHY_1000T_FULL        0x200
#define AMETHYST_QD_GIGPHY_1000T_HALF        0x100
/****************************************************************************/
/* Internal functions.                                                  */
/****************************************************************************/
#define MSD_BIT(n)                  ( (1) << (n) )
#define MSD_BIT_SET(y, mask)        ( (y) |=  (mask) )
#define MSD_BIT_CLEAR(y, mask)      ( (y) &= ~(mask) )
#define MSD_BIT_FLIP(y, mask)       ( (y) ^=  (mask) )

/* ! Create a bitmask of length \a len. */
#define MSD_BIT_MASK(len)           ( MSD_BIT(len)-1 )
/* ! Create a bitfield mask of length \a starting at bit \a start. */
#define MSD_BF_MASK(start, len)     ( MSD_BIT_MASK(len)<<(start) )
/* ! Prepare a bitmask for insertion or combining. */
#define MSD_BF_PREP(x, start, len)  ( ((x)&MSD_BIT_MASK(len)) << (start) )
/* ! Extract a bitfield of length \a len starting at bit \a start from \a y. */
#define MSD_BF_GET(y, start, len)   ( ((y)>>(start)) & MSD_BIT_MASK(len) )
/* ! Insert a new bitfield value \a x into \a y. */
#define MSD_BF_SET(y, x, start, len)    \
((y) = ((y) & ~MSD_BF_MASK(start, len)) | MSD_BF_PREP(x, start, len))


int mvsw61xx_reset_port_mib(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_get_port_mib(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_set_port_bc_sc(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_set_port_mc_sc(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_set_port_ingress(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_set_port_bc_mc_sc(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_set_port_egress(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);

int mvsw61xx_set_port_mirrors(struct switch_dev *dev,
						const struct switch_attr *attr, 
						struct switch_val *val);
int mvsw61xx_set_port_speed_duplex(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);

int mvsw61xx_set_port_state(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_set_port_flow_control(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_set_port_led(struct switch_dev *dev,
                       const struct switch_attr *attr,
                       struct switch_val *val);
int mvsw61xx_init_port_led(struct switch_dev *dev);
int mvsw61xx_set_10g_kr(struct switch_dev *dev);

MSD_STATUS Amethyst_gphyPortPowerDown(struct switch_dev *dev,
                                            MSD_LPORT  port,
                                            MSD_BOOL   state);

#define   MSD_PHY_MMD_ADDR          0x0000U
#define   MSD_PHY_MMD_DATA_NO_INC   0x4000U

MSD_STATUS Amethyst_gphySetEEE(struct switch_dev *dev, MSD_LPORT portNum, MSD_BOOL en);

MSD_STATUS Amethyst_msdSetSMIC45PhyReg(struct switch_dev *dev, u8 devAddr, u8 phyAddr, u16 regAddr, u16 data);

#endif
