/** @file      browser_classification_events.h
  * @brief     Browser classification event definitions.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef BROWSER_CLASSIFICATION_EVENTS_H
#define BROWSER_CLASSIFICATION_EVENTS_H

/* Includes *********************************************************************/
#include "common_event_defs.h"

/* Event Groups *****************************************************************/
/**
 * @brief Public define for the assigned metadata group of the browser classification event.
 */
#define PACE2_BROWSER_CLASSIFICATION_EVENT_GROUP PACE2_ENGINE_META_DATA_GROUP

/* Public Types *****************************************************************/

/**
 * @brief Browser strings
 */
#define PACE2_BROWSER_NAMES \
    "Not yet detected",     \
    "Not detected",         \
    "Other",                \
    "Chrome",               \
    "Firefox",              \
    "Microsoft Edge",       \
    "Chromium"

/**
 * These are the currently supported platforms for the classification
 */
typedef enum {
    /**
     * @brief PACE2_BROWSER_NOT_YET_DETECTED indicates that the classification is
     *        still active but no decision has been made so far. More packets must
     *        be analyzed.
     */
    PACE2_BROWSER_NOT_YET_DETECTED = 0,

    /**
     * @brief PACE2_BROWSER_NOT_DETECTED indicates that the classification is finished
     *        and the platform could not be found for the current flow.
     */
    PACE2_BROWSER_NOT_DETECTED,

    /**
     * @brief PACE2_BROWSER_OTHER is used when the platform information could be found
     *        but the actual platform is not known to PACE.
     */
    PACE2_BROWSER_OTHER,
    PACE2_BROWSER_CHROME,
    PACE2_BROWSER_FIREFOX,
    PACE2_BROWSER_MICROSOFT_EDGE,
    PACE2_BROWSER_CHROMIUM,

    PACE2_BROWSER_TYPES_COUNT
} PACE2_browser_type;

/**
 * @brief browser classification metadata flags.
 */
enum PACE2_browser_data_flags {
    PACE2_BROWSER_TYPE = 0,                  /**< Browser type flag. */
    PACE2_BROWSER_VERSION_STRING,            /**< Browser version string flag. */

    PACE2_BROWSER_DATA_FLAGS_COUNT           /**< Number of implemented browser classification data flags. */
};

/* Event Structure **************************************************************/

/**
 * @brief Data structure for browser classification events.
 */
typedef struct {
    /** Global event type, has to be @ref PACE2_BROWSER_CLASSIFICATION_EVENT for this structure. */
    PACE2_EVENT_COMMON_FIELDS;
    PACE2_bitmask meta_data_mask;             /**< Bitmask, which states the available browser metadata. */
    PACE2_browser_type browser_type;          /**< Variable of the @ref PACE2_BROWSER_TYPE flag. */
    PACE2_byte_buffer browser_version_string; /**< Structure of the @ref PACE2_BROWSER_VERSION_STRING flag. */
} PACE2_browser_classification_event;

#endif /* BROWSER_CLASSIFICATION_EVENTS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
