/** @file      class_generic_events.h
  * @brief     Generic class event definitions.
  * @details   @copydoc doc_class_generic
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef CLASS_GENERIC_EVENTS_H
#define CLASS_GENERIC_EVENTS_H

/* Includes *********************************************************************/
#include "common_event_defs.h"

/* Event Groups *****************************************************************/
/**
 * @brief Public define for the assigned metadata group of the generic class event.
 */
#define PACE2_CLASS_GENERIC_EVENT_GROUP PACE2_CLASS_PROTOCOL_META_DATA_GROUP

/* Event Types ******************************************************************/

/**
 * @brief Types for generic class events.
 */
typedef enum {
    PACE2_CLASS_GENERIC_UPDATE,                 /**< Update event for Advanced Meta Data Events that does not affect
                                                     any Class Event of any Class decoder. It is used as a part of the
                                                     event correlation feature only.
                                                     See @ref PACE2_config_event_correlation. */
    PACE2_CLASS_GENERIC_NEW_FLOW,               /**< Informs that a new flow was added to a session in the class
                                                     decoder. */
    PACE2_NUMBER_OF_CLASS_GENERIC_EVENT_TYPES   /**< Number of implemented event types. */
} PACE2_class_generic_event_type;

/* Event Structures *************************************************************/

#ifndef PACE_KERNEL_LIB

/**
 * @brief Data structure for the new correlation between flows and session_ids in class decoders.
 */
struct PACE2_class_generic_new_flow {
    u64 session_id; /**< Id of the session. */
    u64 flow_id;    /**< Id of the correlated flow. */
};

/**
 * @brief Data structure for generic class events.
 */
typedef struct {
    /** Global event type, has to be @ref PACE2_CLASS_GENERIC_EVENT for this structure. */
    PACE2_EVENT_COMMON_FIELDS;
    PACE2_class_generic_event_type meta_data_type;         /**< Generic event type. */
    PACE2_CLASS_META_DATA_EVENT_EXTRA_FIELDS;
    union {
        struct PACE2_class_generic_new_flow new_flow;      /**< Contains data for a @ref PACE2_CLASS_GENERIC_NEW_FLOW
                                                                event. */
    } event_data;
} PACE2_class_generic_event;

#endif /* PACE_KERNEL_LIB */

#endif /* CLASS_GENERIC_EVENTS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
