/** @file      common_event_defs.h
  * @brief     Common event structure field definitions.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef COMMON_EVENT_DEFS_H
#define COMMON_EVENT_DEFS_H

/* Includes *********************************************************************/
#include "../pace2_event_types.h"

#ifndef PACE_KERNEL_LIB
#include "ipd_charset.h"
#endif

/* Public Macros ****************************************************************/

/** Macro for the common structure fields of all events. */
#define PACE2_EVENT_COMMON_FIELDS \
    PACE2_event_type type; \
    /** Pointer to the flow user data of the currently processed packet.
     *  Exception is in stage5. Here the pointer points to the currently
     *  processed timed out flow. */ \
    void *flow_user_data

/** Macro for extra structure fields of a class event. */
#define PACE2_CLASS_META_DATA_EVENT_EXTRA_FIELDS PACE2_timestamp timestamp; /*!< Timestamp of the packet leading to this event. */      \
                                                 /** ID of the packet leading to this event. */     \
                                                 u64 external_packet_id;                            \
                                                 /** List of AME identifier. */                     \
                                                 pace2_id_list id_list

/** Macro for extra structure fields of a advanced event. */
#define PACE2_ADVANCED_META_DATA_EVENT_EXTRA_FIELDS PACE2_timestamp timestamp;  /*!< Timestamp of the packet leading to this event. */  \
                                                    /** ID of the packet leading to this event. */  \
                                                    u64 external_packet_id;                         \
                                                    /** ID of the flow of this event. */            \
                                                    u64 flow_id;                                    \
                                                    /** ID of the advanced metadata event. */      \
                                                    u64 event_id

#endif /* COMMON_EVENT_DEFS_H */
