/** @file      imf_events.h
  * @brief     Help structures for Mail Protocols with IMF content.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef IMF_EVENTS_H
#define IMF_EVENTS_H

#include "ipq_config.h"

#ifndef PACE_KERNEL_LIB

#include "ipd_charset.h"

/* Event Structures *************************************************************/

/** IMF address. */
struct ipd_imf_address {

    /** Actual email address. */
    struct ipd_charset_string email;

    /** Complete text of the address as it appeared in the IMF header field. */
    struct ipd_charset_string full;
};

struct ipd_imf_content_type {

    /** Section number as used in IMAP, e.g. "1.2" for the second sub-part of the first part of the mail
     * message; empty for the Content-Type line in the message header. */
    struct ipd_charset_string part;

    /** Complete text of the Content-Type field value. */
    struct ipd_charset_string full;

    /** MIME media type. */
    struct ipd_charset_string type;

    /** MIME media subtype. */
    struct ipd_charset_string subtype;

    /** Value of the "boundary" parameter (for multipart types). */
    struct ipd_charset_string boundary;

    /** Value of the "charset" parameter, if available. */
    struct ipd_charset_string charset;
};


/** String associated with a specific MIME part. */
struct ipd_imf_part_data {

    /** Section number as used in IMAP, e.g. "1.2" for the second sub-part of the first part of the mail
     * message; empty for non-MIME mails. */
    struct ipd_charset_string part;

    /** Data block, if the content is base64 encoded it will decoded in the class decoder. */
    struct ipd_charset_string data;
};

/** Missing data length associated with a specific MIME part. */
struct ipd_imf_missing_data {

    /** Section number as used in IMAP, e.g. "1.2" for the second sub-part of the first part of the mail
     * message; empty for non-MIME mails. */
    struct ipd_charset_string part;

    /** Length of the missing data. */
    u64 length;
};

#endif /* PACE_KERNEL_LIB */

#endif /* IMF_EVENTS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
