/** @file      javascript_events.h
  * @brief     JavaScript event definitions.
  * @details   @copydoc doc_javascript
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef JAVASCRIPT_EVENTS_H
#define JAVASCRIPT_EVENTS_H

/* Event Groups *****************************************************************/
/**
 * @brief Public define for the assigned metadata group of the advanced JavaScript event.
 */
#define PACE2_ADVANCED_JAVASCRIPT_EVENT_GROUP PACE2_ADVANCED_PROTOCOL_META_DATA_GROUP

/* Includes *********************************************************************/
#include "common_event_defs.h"

/* Event Types ******************************************************************/

/**
 * @brief Types for advanced JavaScript events.
 */
typedef enum {
    PACE2_ADVANCED_JAVASCRIPT_TRANSFER,         /**< JavaScript transfer. Returns an pace2_javascript_transfer * */

    PACE2_NUMBER_OF_ADVANCED_JAVASCRIPT_EVENT_TYPES         /**< Number of implemented event types. */
} PACE2_advanced_javascript_event_type;

#ifndef PACE_KERNEL_LIB

/**
 * @brief Enumeration for the transaction direction.
 */
typedef enum {
    PACE2_JAVASCRIPT_TRANSACTION_DIRECTION_REQUEST, /**< Transaction request. */
    PACE2_JAVASCRIPT_TRANSACTION_DIRECTION_RESPONSE /**< Transaction response. */
} PACE2_javascript_transaction_direction_type;

/**
 * @brief Enumeration for the JavaScript type..
 */
typedef enum {
    PACE2_JAVASCRIPT_SCRIPT_TYPE_INLINE,    /**< JavaScript included inline in an HTML file. */
    PACE2_JAVASCRIPT_SCRIPT_TYPE_FILE       /**< JavaScript provided in a separate file. */
} PACE2_javascript_script_type;

/**
 * @brief JavaScript transfer structure.
 */
struct pace2_javascript_transfer {
    u64 http_session_id;                                                /**< ID of the ipd_class_http_session it belongs to. */
    u64 http_transaction_id;                                            /**< ID of the ipd_class_http_transaction it belongs to. */
    PACE2_javascript_transaction_direction_type transaction_direction;  /**< Transaction direction. */
    PACE2_javascript_script_type script_type;                           /**< Type of the script (inline or in a separate file). */
    struct ipd_charset_string url;                                      /**< URL of the file in which the JavaScript is contained. */
    u64 data_offset;                                                    /**< Offset of the current chunk in the data. */
    struct ipd_charset_string data;                                     /**< Data with encoding. */
    u8 finished;                                                        /**< Flag indicating that no further data will be thrown for this transaction. */
};

/**
 * @brief Data structure for advanced JavaScript events.
 */
typedef struct {
    /** Global event type, has to be @ref PACE2_ADVANCED_JAVASCRIPT_EVENT for this structure. */
    PACE2_EVENT_COMMON_FIELDS;
    PACE2_advanced_javascript_event_type meta_data_type;    /**< JavaScript event subtype. */
    PACE2_ADVANCED_META_DATA_EVENT_EXTRA_FIELDS;
    union {
        struct pace2_javascript_transfer transfer;          /**< Contains data for a @ref PACE2_ADVANCED_JAVASCRIPT_TRANSFER event. */
    } event_data;
} PACE2_advanced_javascript_event;

#endif /* PACE_KERNEL_LIB */

#endif // JAVASCRIPT_EVENTS_H

/*********************************************************************************
 ** EOF
 ********************************************************************************/
