/** @file      l2tp_events.h
  * @brief     L2TP event definitions.
  * @details   @copydoc doc_l2tp
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef L2TP_EVENTS_H
#define L2TP_EVENTS_H

/* Event Groups *****************************************************************/
/**
 * @brief Public define for the assigned metadata group of the advanced L2TP event.
 */
#define PACE2_ADVANCED_L2TP_EVENT_GROUP PACE2_ADVANCED_PROTOCOL_META_DATA_GROUP

/* Includes *********************************************************************/
#include "common_event_defs.h"

/* Event Types ******************************************************************/

/**
 * @brief Types for advanced L2TP events.
 */
typedef enum {

    /** Sent for each L2TP tunnel that is set up. */
    PACE2_ADVANCED_L2TP_TUNNEL_SETUP,

    /** Sent for each L2TP tunnel that is shut down. */
    PACE2_ADVANCED_L2TP_TUNNEL_SHUTDOWN,

    /** Sent for each IP packet that is decoded from a L2TP tunnel. */
    PACE2_ADVANCED_L2TP_IP_PACKET,

    /** Number of implemented event types. */
    PACE2_NUMBER_OF_ADVANCED_L2TP_EVENT_TYPES

} PACE2_advanced_l2tp_event_type;

/* Event Structures *************************************************************/

#ifndef PACE_KERNEL_LIB

/**
 * @brief Data structure for advanced L2TP events.
 */
typedef struct {

    /** Global event type, has to be @ref PACE2_ADVANCED_L2TP_EVENT for this structure. */
    PACE2_EVENT_COMMON_FIELDS;

    /** L2TP event subtype. */
    PACE2_advanced_l2tp_event_type meta_data_type;

    PACE2_ADVANCED_META_DATA_EVENT_EXTRA_FIELDS;

    /** Named union which contains the data of the sub-event types. */
    union {

        /** Contains data for a @ref PACE2_ADVANCED_L2TP_TUNNEL_SETUP event. */
        u64 setup;

        /** Contains data for a @ref PACE2_ADVANCED_L2TP_TUNNEL_SHUTDOWN event. */
        u64 shutdown;

        /** Contains data for a @ref PACE2_ADVANCED_L2TP_IP_PACKET event. */
        struct ipd_tunnelled_ip_packet ip_packet;

    } event_data;

} PACE2_advanced_l2tp_event;

#endif

#endif /* L2TP_EVENTS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
