/** @file      pptp_events.h
  * @brief     PPTP event definitions.
  * @details   @copydoc doc_pptp
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PPTP_EVENTS_H
#define PPTP_EVENTS_H

/* Includes *********************************************************************/
#include "common_event_defs.h"

/* Event Groups *****************************************************************/
/**
 * @brief Public define for the assigned metadata group of the advanced PPTP event.
 */
#define PACE2_ADVANCED_PPTP_EVENT_GROUP PACE2_ADVANCED_PROTOCOL_META_DATA_GROUP

/* Event Types ******************************************************************/

/**
 * @brief Types for advanced PPTP events.
 */
typedef enum {

    /** Sent for each PPTP tunnel that is set up. */
    PACE2_ADVANCED_PPTP_TUNNEL_SETUP,

    /** Sent for each PPTP tunnel that is shut down. */
    PACE2_ADVANCED_PPTP_TUNNEL_SHUTDOWN,

    /** Sent for each IP packet that is decoded from a PPTP tunnel. */
    PACE2_ADVANCED_PPTP_IP_PACKET,

    /** Number of implemented event types. */
    PACE2_NUMBER_OF_ADVANCED_PPTP_EVENT_TYPES

} PACE2_advanced_PPTP_event_type;

/* Event Structures *************************************************************/

#ifndef PACE_KERNEL_LIB

/**
 * @brief Data structure for advanced PPTP events.
 */
typedef struct {

    /** Global event type, has to be @ref PACE2_ADVANCED_HTTP_EVENT for this structure. */
    PACE2_EVENT_COMMON_FIELDS;

    /** PPTP event subtype. */
    PACE2_advanced_PPTP_event_type meta_data_type;

    PACE2_ADVANCED_META_DATA_EVENT_EXTRA_FIELDS;

    /** Named union which contains the data of the sub-event types. */
    union {

        /** Contains data for a @ref PACE2_ADVANCED_PPTP_TUNNEL_SETUP event. */
        struct ipd_tunnelled_ip_packet ip_packet;

        /** Contains data for a @ref PACE2_ADVANCED_PPTP_TUNNEL_SHUTDOWN event. */
        u64 setup;

        /** Contains data for a @ref PACE2_ADVANCED_PPTP_IP_PACKET event. */
        u64 shutdown;

    } event_data;

} PACE2_advanced_PPTP_event;

#endif

#endif /* PPTP_EVENTS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
