/** @file      quic_events.h
  * @brief     QUIC event definitions
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_INCLUDE_IPOQUE_EVENT_TYPES_QUIC_EVENTS_H_
#define PACE2_INCLUDE_IPOQUE_EVENT_TYPES_QUIC_EVENTS_H_

/* Includes *********************************************************************/
#include "common_event_defs.h"

/* Event Groups *****************************************************************/
/**
 * @brief Public define for the assigned metadata group of the basic QUIC Client Hello event.
 */
#define PACE2_BASIC_QUIC_CLIENT_HELLO_EVENT_GROUP PACE2_BASIC_PROTOCOL_META_DATA_GROUP

/* Event Types ******************************************************************/

/**
 * @brief Types for QUIC Client Hello lines.
 */
typedef enum {
    PACE2_QUIC_CLIENT_HELLO_SNI = 0,            /**< QUIC Client Hello sni Tag. */
    PACE2_QUIC_CLIENT_HELLO_UAID,               /**< QUIC Client Hello uaid Tag. */
    PACE2_QUIC_CLIENT_HELLO_STK,                /**< QUIC Client Hello stk Tag. */
    PACE2_QUIC_CLIENT_HELLO_CCS,                /**< QUIC Client Hello ccs Tag. */
    PACE2_QUIC_CLIENT_HELLO_SCID,               /**< QUIC Client Hello scid Tag. */

    PACE2_QUIC_CLIENT_HELLO_CID,                /**< QUIC Client Hello Client ID. */
    PACE2_QUIC_CLIENT_HELLO_VERSION,            /**< QUIC Client Hello Client Version. */
    PACE2_QUIC_CLIENT_HELLO_DCID,               /**< QUIC Client Hello dcid tag. */

    PACE2_NUMBER_OF_QUIC_CLIENT_HELLO_TAGS      /**< Number of implemented client hello lines. */
} PACE2_QUIC_client_hello_tag;

/**
 * @brief Data structure of a QUIC Client Hello metadata event.
 */
typedef struct {
    /** Global event type, has to be @ref PACE2_BASIC_QUIC_CLIENT_HELLO_EVENT for this structure. */
    PACE2_EVENT_COMMON_FIELDS;
    PACE2_bitmask meta_data_mask;               /**< Bitmask, which states the available QUIC client hello metadata */
    struct {
        PACE2_byte_buffer content;              /**< Client hello line content. */
    } line[PACE2_NUMBER_OF_QUIC_CLIENT_HELLO_TAGS];  /**< Data structure for a event of type
                                                     @ref PACE2_BASIC_QUIC_CLIENT_HELLO_EVENT. */
} PACE2_basic_QUIC_client_hello_event;

#endif /* PACE2_INCLUDE_IPOQUE_EVENT_TYPES_QUIC_EVENTS_H_ */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
