/** @file      radius_events.h
  * @brief     RADIUS event definitions.
  * @details   @copydoc doc_radius
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef RADIUS_EVENTS_H
#define RADIUS_EVENTS_H

/* Includes *********************************************************************/
#include "common_event_defs.h"

/* Event Groups *****************************************************************/
/**
 * @brief Public define for the assigned metadata group of the advanced radius event.
 */
#define PACE2_ADVANCED_RADIUS_EVENT_GROUP PACE2_ADVANCED_PROTOCOL_META_DATA_GROUP

/* Event Types ******************************************************************/

/**
 * @brief Types for advanced radius events.
 */
typedef enum {

    /** Event throwing general information. */
    PACE2_ADVANCED_RADIUS_GENERAL_PACKET_INFORMATION,

    /** Event throwing the attributes of an packet. */
    PACE2_ADVANCED_RADIUS_ATTRIBUTE,

    /** Number of implemented event types. */
    PACE2_NUMBER_OF_ADVANCED_RADIUS_EVENT_TYPES

} PACE2_advanced_Radius_event_type;

/* Event Structures *************************************************************/

#ifndef PACE_KERNEL_LIB

/** Enumerates the type of packet. */
enum ipd_radius_packet_code {
    IPD_RADIUS_PACKET_CODE_ACCESS_REQUEST = 1,
    IPD_RADIUS_PACKET_CODE_ACCESS_ACCEPT = 2,
    IPD_RADIUS_PACKET_CODE_ACCESS_REJECT = 3,
    IPD_RADIUS_PACKET_CODE_ACCOUNTING_REQUEST = 4,
    IPD_RADIUS_PACKET_CODE_ACCOUNTING_RESPONSE = 5,
    IPD_RADIUS_PACKET_CODE_ACCESS_CHALLENGE = 11,
    IPD_RADIUS_PACKET_CODE_STATUS_SERVER = 12,
    IPD_RADIUS_PACKET_CODE_STATUS_CLIENT = 13,
};

/** RADIUS attribute types (RFC 2865 and 2866). */
enum ipd_radius_attributes {

    /** User-Name. */
    IPD_RADIUS_ATTRIBUTES_USER_NAME = 1,

    /** User-Password. */
    IPD_RADIUS_ATTRIBUTES_PASSWORD = 2,

    /** CHAP-Password. */
    IPD_RADIUS_ATTRIBUTES_CHAP_PASSWORD = 3,

    /** NAS-IP-Address. */
    IPD_RADIUS_ATTRIBUTES_NAS_IP_ADDRESS = 4,

    /** NAS-Port. */
    IPD_RADIUS_ATTRIBUTES_NAS_PORT = 5,

    /** Service-Type. */
    IPD_RADIUS_ATTRIBUTES_SERVICE_TYPE = 6,

    /** Framed-Protocol. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_PROTOCOL = 7,

    /** Framed-IP-Address. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_IP_ADDRESS = 8,

    /** Framed-IP-Netmask. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_IP_NETMASK = 9,

    /** Framed-Routing. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_ROUTING = 10,

    /** Filter-ID. */
    IPD_RADIUS_ATTRIBUTES_FILTER_ID = 11,

    /** Framed-MTU. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_MTU = 12,

    /** Framed-Compression. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_COMPRESSION = 13,

    /** Login-IP-Host. */
    IPD_RADIUS_ATTRIBUTES_LOGIN_IP_HOST = 14,

    /** Login-Service. */
    IPD_RADIUS_ATTRIBUTES_LOGIN_SERVICE = 15,

    /** Login-TCP-Port. */
    IPD_RADIUS_ATTRIBUTES_LOGIN_TCP_PORT = 16,

    /** Reply-Message. */
    IPD_RADIUS_ATTRIBUTES_REPLY_MESSAGE = 18,

    /** Callback-Number. */
    IPD_RADIUS_ATTRIBUTES_CALLBACK_NUMBER = 19,

    /** Callback-ID. */
    IPD_RADIUS_ATTRIBUTES_CALLBACK_ID = 20,

    /** Framed-Route. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_ROUTE = 22,

    /** Framed-IPX-Network. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_IPX_NETWORK = 23,

    /** State. */
    IPD_RADIUS_ATTRIBUTES_STATE = 24,

    /** Class. */
    IPD_RADIUS_ATTRIBUTES_CLASS = 25,

    /** Vendor-Specific. */
    IPD_RADIUS_ATTRIBUTES_VENDOR_SPECIFIC = 26,

    /** Session-Timeout. */
    IPD_RADIUS_ATTRIBUTES_SESSION_TIMEOUT = 27,

    /** Idle-Timeout. */
    IPD_RADIUS_ATTRIBUTES_IDLE_TIMEOUT = 28,

    /** Termination-Action. */
    IPD_RADIUS_ATTRIBUTES_TERMINATION_ACTION = 29,

    /** Called-Station-ID. */
    IPD_RADIUS_ATTRIBUTES_CALLED_STATION_ID = 30,

    /** Calling-Station-ID. */
    IPD_RADIUS_ATTRIBUTES_CALLING_STATION_ID = 31,

    /** NAS-Identifier. */
    IPD_RADIUS_ATTRIBUTES_NAS_IDENTIFIER = 32,

    /** Proxy-State. */
    IPD_RADIUS_ATTRIBUTES_PROXY_STATE = 33,

    /** Login-LAT-Service. */
    IPD_RADIUS_ATTRIBUTES_LOGIN_LAT_SERVICE = 34,

    /** Login-LAT-Node. */
    IPD_RADIUS_ATTRIBUTES_LOGIN_LAT_NODE = 35,

    /** Login-LAT-Group. */
    IPD_RADIUS_ATTRIBUTES_LOGIN_LAT_GROUP = 36,

    /** Framed-AppleTalk-Link. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_APPLETALK_LINK = 37,

    /** Framed-AppleTalk-Network. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_APPLETALK_NETWORK = 38,

    /** Framed-AppleTalk-Zone. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_APPLETALK_ZONE = 39,

    /** Acct-Status-Type. */
    IPD_RADIUS_ATTRIBUTES_STATUS_TYPE = 40,

    /** Acct-Delay-Time. */
    IPD_RADIUS_ATTRIBUTES_DELAY_TIME = 41,

    /** Acct-Input-Octets. */
    IPD_RADIUS_ATTRIBUTES_INPUT_OCTETS = 42,

    /** Acct-Output-Octets. */
    IPD_RADIUS_ATTRIBUTES_OUTPUT_OCTETS = 43,

    /** Acct-Session-ID. */
    IPD_RADIUS_ATTRIBUTES_SESSION_ID = 44,

    /** Acct-Authentic. */
    IPD_RADIUS_ATTRIBUTES_AUTHENTIC = 45,

    /** Acct-Session-Time. */
    IPD_RADIUS_ATTRIBUTES_SESSION_TIME = 46,

    /** Acct-Input-Packets. */
    IPD_RADIUS_ATTRIBUTES_INPUT_PACKETS = 47,

    /** Acct-Output-Packets. */
    IPD_RADIUS_ATTRIBUTES_OUTPUT_PACKETS = 48,

    /** Acct-Terminate-Cause. */
    IPD_RADIUS_ATTRIBUTES_TERMINATE_CAUSE = 49,

    /** Acct-Multi-Session-ID. */
    IPD_RADIUS_ATTRIBUTES_MULTI_SESSION_ID = 50,

    /** Acct-Link-Count. */
    IPD_RADIUS_ATTRIBUTES_LINK_COUNT = 51,

    /** CHAP-Challenge. */
    IPD_RADIUS_ATTRIBUTES_CHAP_CHALLENGE = 60,

    /** NAS-Port-Type. */
    IPD_RADIUS_ATTRIBUTES_NAS_PORT_TYPE = 61,

    /** Port-Limit. */
    IPD_RADIUS_ATTRIBUTES_PORT_LIMIT = 62,

    /** Login-LAT-Port. */
    IPD_RADIUS_ATTRIBUTES_LOGIN_LAT_PORT = 63,

    /** Framed-IPv6-Prefix. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_IPV6_PREFIX = 97,

    /** Framed-IPv6-Route. */
    IPD_RADIUS_ATTRIBUTES_FRAMED_IPV6_ROUTE = 99,

    /** Delegated-IPv6-Prefix. */
    IPD_RADIUS_ATTRIBUTES_DELEGATED_IPV6_PREFIX = 123,
};

/** Enumerates the status types packets can have. */
enum ipd_radius_status_types {
    IPD_RADIUS_STATUS_TYPE_START = 1,
    IPD_RADIUS_STATUS_TYPE_STOP = 2,
    IPD_RADIUS_STATUS_TYPE_ACCOUNTING_ON = 7,
    IPD_RADIUS_STATUS_TYPE_ACCOUNTING_OFF = 8,
};

/** Contains the Information send with every Packet. */
struct ipd_radius_general_packet_information {

    /** Reason why the packet is send - enumerated in ipd_radius_packet_code. */
    u8 code;

    /** Packet identifier - part of the MD5 - authenticator. */
    u8 pkg_id;

    /** Contains the general packet length. */
    u32 pkg_len;

    /** Struct contains the MD5 authenticator which is unique to a packet (so duplicates can be detected). */
    struct ipd_charset_string md5_authenticator;
};

/** Enumeration for the content type of a radius attribute. */
enum ipd_radius_content_type {

    /** Signals that the content type of the radius attribute is unknown. Only data is set. */
    IPD_RADIUS_TYPE_UNKNOWN,

    /** Signals that the content type of the radius attribute is a IP.
     * The fields content.ip and data are set. */
    IPD_RADIUS_TYPE_IP,

    /** Signals that the content type of the radius attribute is a String.
     * The fields content.string and data are set. */
    IPD_RADIUS_TYPE_TEXT,

    /** Signals that the content type of the radius attribute is a Integer.
     * The fields content.integer and data are set. */
    IPD_RADIUS_TYPE_INTEGER,

    /** Signals that the content type of the radius attribute is a IPv6-Prefix.
     * The fields content.ipv6_prefix and data are set. */
    IPD_RADIUS_TYPE_IPV6_PREFIX,
};

struct ipd_ipv6_prefix {

    /** The IPv6 content in network byte order. */
    struct in6_addr prefix;

    u8 prefix_length;
};

/** Contains Information which are possibly send with a packet. */
struct ipd_radius_attribute {

    /** Type of attribute - enumerated in ipd_radius_attributes. */
    u8 type;

    /** Length of Attribute payload (length of attribute without the type and length). */
    u64 length;

    /** The type of the attribute content. */
    enum ipd_radius_content_type content_type;

    union {

        /** The integer content (host byte order). */
        u32 integer;

        /** The string content. */
        struct ipd_charset_string string;

        /** The IP content in network byte order. */
        struct in_addr ip;

        /** The IPv6 prefix length and the IPv6 prefix content in network byte order. */
        struct ipd_ipv6_prefix ipv6_prefix;
    } content;

    /** Struct provide space for different options like Password or unknown content. */
    struct ipd_charset_string data;

    /** Struct contains the MD5 authenticator which is unique to a packet (so duplicates can be detected). */
    struct ipd_charset_string md5_authenticator;
};

/**
 * @brief Data structure for advanced radius events.
 */
typedef struct {

    /** Global event type, has to be @ref PACE2_ADVANCED_RADIUS_EVENT for this structure. */
    PACE2_EVENT_COMMON_FIELDS;

    /** Radius event subtype. */
    PACE2_advanced_Radius_event_type meta_data_type;

    PACE2_ADVANCED_META_DATA_EVENT_EXTRA_FIELDS;

    /** Named union which contains the data of the sub-event types. */
    union {

        /** Contains data for a @ref PACE2_ADVANCED_RADIUS_GENERAL_PACKET_INFORMATION event. */
        struct ipd_radius_general_packet_information gpi;

        /** Contains data for a @ref PACE2_ADVANCED_RADIUS_ATTRIBUTE event. */
        struct ipd_radius_attribute attribute;

    } event_data;

} PACE2_advanced_radius_event;

#endif /* PACE_KERNEL_LIB */

#endif /* RADIUS_EVENTS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
