/** @file      ipq_basic_def.h
  * @brief     PACE basic type definitions.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef __IPQ_BASIC_DEF_H__
#define __IPQ_BASIC_DEF_H__

#include <stddef.h>

#include "ipq_config.h"

/* We make sure that the standard byte-order definitions are available. Recent gcc, clang or icc compilers provide
 * them by default.
 */
#if !defined(__BYTE_ORDER__)
# if !defined(__BYTE_ORDER) && !defined(BYTE_ORDER)
   /* We need system specific includes or guessing. */
#  if defined(__sun)
#   include <sys/byteorder.h>
#  elif defined(__FreeBSD__) || defined(__ANDROID__)
#   include <sys/endian.h>
#  elif defined( __linux__)
#   include <endian.h>
#  elif defined(__unix__)
#   include <sys/endian.h>
#  endif
# endif /* !defined(__BYTE_ORDER) && !defined(BYTE_ORDER) */

# if defined(__BYTE_ORDER)
#  define __ORDER_LITTLE_ENDIAN__ __LITTLE_ENDIAN
#  define __ORDER_BIG_ENDIAN__ __BIG_ENDIAN
#  define __BYTE_ORDER__ __BYTE_ORDER
# elif defined(BYTE_ORDER)
#  define __ORDER_LITTLE_ENDIAN__ LITTLE_ENDIAN
#  define __ORDER_BIG_ENDIAN__ BIG_ENDIAN
#  define __BYTE_ORDER__ BYTE_ORDER
# elif defined(_WIN32)
#  define __ORDER_LITTLE_ENDIAN__ 1234
#  define __ORDER_BIG_ENDIAN__ 4321
#  define __BYTE_ORDER__ __ORDER_LITTLE_ENDIAN__
# endif
#endif /* !defined(__BYTE_ORDER__) */

#ifdef __cplusplus
extern "C" {
#endif

/* define u64, u32, here
 * this is used for compatibility restrictions on different platforms
 *
 */
#ifndef IPOQUE_INTEGER_TYPEDEFS_DONE
#ifndef __KERNEL__
    typedef unsigned long long u64;
    typedef unsigned int u32;
    typedef unsigned short u16;
    typedef unsigned char u8;
#endif
#define IPOQUE_INTEGER_TYPEDEFS_DONE
#endif

/* WIN32 definitions and redefinitions
 * to enable compilation with the VisualC compiler
 */
#ifdef WIN32
    typedef u8 uint8_t;
    typedef u16 uint16_t;
    typedef u32 uint32_t;
    typedef u64 uint64_t;

#ifdef _MSC_VER
#define __attribute__(arg)
#endif  /* _MSC_VER */

#endif  /* WIN32 */

/* generic timestamp counter size */
#define IPOQUE_TIMESTAMP_COUNTER_SIZE		u32
#ifdef __cplusplus
}
#endif
#endif
