/** @file      ipq_metadata_public.h
  * @brief     PACE metadata definition.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef __IPQ_METADATA_PUBLIC_H__
#define __IPQ_METADATA_PUBLIC_H__

#ifndef __IPOQUE_API_INCLUDE_FILE__
#error CANNOT INCLUDE THIS .H FILE, INCLUDE IPQ_API.H
#endif

#ifdef __cplusplus
extern "C" {
#endif

enum dissector_metadata_type {
    DISSECTOR_IP = 0,               /*!< Dissector for IP metadata */
    DISSECTOR_TCP,                  /*!< Dissector for TCP metadata */
    DISSECTOR_H264,                 /*!< Dissector for H264 metadata */
    DISSECTOR_AMR,                  /*!< Dissector for AMR metadata */
    DISSECTOR_RTP,                  /*!< Dissector for RTP metadata */
    DISSECTOR_ID3,                  /*!< Dissector for ID3 metadata */
    DISSECTOR_MP3,                  /*!< Dissector for MP3 metadata */
    DISSECTOR_HTTP,                 /*!< Dissector for HTTP metadata */
    DISSECTOR_MP4,                  /*!< Dissector for MP4 metadata */
    DISSECTOR_RTCP,                 /*!< Dissector for RTCP metadata */
    DISSECTOR_H263,                 /*!< Dissector for H263 metadata */
    DISSECTOR_EVS,                  /*!< Dissector for EVS metadata */
    DISSECTOR_ZRTP,                 /*!< Dissector for ZRTP metadata */
    DISSECTOR_STUN,                 /*!< Dissector for STUN metadata */
    DISSECTOR_ICMP,                 /*!< Dissector for ICMP metadata */
    DISSECTOR_QUIC,                 /*!< Dissector for QUIC metadata */

    METADATA_DISSECTOR_COUNT
};

enum metadata_status {
    METADATA_DISSECTOR_SUCCESS = 0,
    METADATA_DISSECTOR_MEMORY_ALLOCATION_FAILED,
    METADATA_DISSECTOR_INVALID_ARGUMENTS,
    METADATA_DISSECTOR_NO_DATA,
    METADATA_DISSECTOR_NOT_ENABLED,
    METADATA_DISSECTOR_MISSING_DEPENDENCY,
    METADATA_DISSECTOR_FAILURE
};

#define CODEC_LIST(M) M(UNKNOWN, UNKNOWN) \
                      M(H263,    VIDEO)   \
                      M(H264,    VIDEO)   \
                      M(AMR,     AUDIO)   \
                      M(PCMU,    AUDIO)   \
                      M(GSM,     AUDIO)   \
                      M(G723,    AUDIO)   \
                      M(DVI4,    AUDIO)   \
                      M(LPC,     AUDIO)   \
                      M(PCMA,    AUDIO)   \
                      M(G722,    AUDIO)   \
                      M(L16,     AUDIO)   \
                      M(QCELP,   AUDIO)   \
                      M(CN,      AUDIO)   \
                      M(MPA,     AUDIO)   \
                      M(G728,    AUDIO)   \
                      M(G729,    AUDIO)   \
                      M(CELB,    VIDEO)   \
                      M(H261,    VIDEO)   \
                      M(MPV,     VIDEO)   \
                      M(JPEG,    VIDEO)   \
                      M(NV,      VIDEO)   \
                      M(AMR_WB,  AUDIO)   \
                      M(AAC,     AUDIO)   \
                      M(MP3,     AUDIO)   \
                      M(MP4,     AUDIO)   \
                      M(EVS,     AUDIO)

#define STREAM_TYPE_LIST(M) M(UNKNOWN)              \
                            M(AUDIO)                \
                            M(VIDEO)                \
                            M(AUDIO_VIDEO)          \
                            M(VIDEO_FRAGMENT)       \
                            M(AUDIO_FRAGMENT)

#define STREAM_TRANSFER_TYPE_LIST(M) M(UNKNOWN) \
                                     M(DASH)    \
                                     M(MP4)     \
                                     M(MP2T)    \
                                     M(MMS)     \
                                     M(HDS)


#define LIST_TO_STREAM_TYPE_ENUM(NAME) STREAM_TYPE_ ## NAME,
#define LIST_TO_CODEC_ENUM(NAME, ...) CODEC_ ## NAME,
#define LIST_TO_STR(NAME, ...) #NAME,
#define LIST_TO_STREAM_TRANSFER_TYPE_ENUM(NAME) STREAM_TRANSFER_TYPE_ ## NAME,

enum stream_transfer_type {
    STREAM_TRANSFER_TYPE_LIST(LIST_TO_STREAM_TRANSFER_TYPE_ENUM)

    NUMBER_OF_STREAM_TRANSFER_TYPES
};

enum stream_media_type {
    STREAM_TYPE_LIST(LIST_TO_STREAM_TYPE_ENUM)

    NUMBER_OF_STREAM_TYPES
};

enum codec_type {
    CODEC_LIST(LIST_TO_CODEC_ENUM)

    NUMBER_OF_CODECS
};

/** 
 * @brief Enum of available session description item types
 */
enum sdes_item_type {
    SDES_END = 0,
    SDES_CNAME,
    SDES_NAME,
    SDES_EMAIL,
    SDES_PHONE,
    SDES_LOC,
    SDES_TOOL,
    SDES_NOTE,
    SDES_PRIV,

    SDES_TYPE_COUNT
};

/**
 * @brief TCP_CONNECTION_STATE enum which is used for transition of connection states.
 */
enum TCP_CONNECTION_STATE {
    TCP_CONNECTION_STATE_LISTEN,
    TCP_CONNECTION_STATE_SYN_SENT,
    TCP_CONNECTION_STATE_SYN_RECEIVED,
    TCP_CONNECTION_STATE_ESTABLISHED,
    TCP_CONNECTION_STATE_FIN_WAIT_1,
    TCP_CONNECTION_STATE_FIN_WAIT_2,
    TCP_CONNECTION_STATE_CLOSE_WAIT,
    TCP_CONNECTION_STATE_CLOSING,
    TCP_CONNECTION_STATE_LAST_ACK,
    TCP_CONNECTION_STATE_TIME_WAIT,
    TCP_CONNECTION_STATE_CLOSED
};

struct metadata_string {
    u8 const * ptr;
    u16 len;
};

struct rtcp_report {
    u32 identifier;                         /*!< Identifier */
    u32 packets_lost;                       /*!< Amount of packets lost */
    u32 interarrival_jitter;                /*!< Jitter in RTP timestamp units */
    u32 last_sr_timestamp;                  /*!< Last SR timestamp */
    u32 delay_since_last_sr_timestamp;      /*!< Delay since last SR timestamp */
    u32 rtt;                                /*!< Round Trip Time */
    u16 sequence_num_cycles;                /*!< Count of sequence number wrap-arounds */
    u16 sequence_num_max;                   /*!< Highest seen sequence number */
    u8 fraction_lost;                       /*!< Fraction lost */
};

#define RTCP_SENDER_REPORT                              (1 << 0)
#define RTCP_RECEIVER_REPORT                            (1 << 1)
#define RTCP_SOURCE_DESCRIPTION                         (1 << 2)
#define RTCP_GOOD_BYE                                   (1 << 3)
#define RTCP_PICTURE_LOSS_INDICATION                    (1 << 4)
#define RTCP_REFERENCE_PICTURE_SELECTION_INDICATION     (1 << 5)
#define RTCP_APPLICATION_SPECIFIC                       (1 << 6)
#define RTCP_NEGATIVE_ACKNOWLEDGEMENT                   (1 << 7)
#define RTCP_MAX_BITRATE_REQUEST                        (1 << 8)
#define RTCP_MAX_BITRATE_NOTIFICATION                   (1 << 9)
#define RTCP_RTT_REPORT                                 (1 << 10)
#define RTCP_EXTENDED_REPORT                            (1 << 11)

enum pace_evs_mode {
    PACE_EVS_NONE = 0,
    PACE_EVS_2000,
    PACE_EVS_2400,
    PACE_EVS_2800,
    PACE_EVS_6600,
    PACE_EVS_7200,
    PACE_EVS_8000,
    PACE_EVS_8850,
    PACE_EVS_9600,
    PACE_EVS_12650,
    PACE_EVS_13200,
    PACE_EVS_14250,
    PACE_EVS_15850,
    PACE_EVS_16400,
    PACE_EVS_18250,
    PACE_EVS_19850,
    PACE_EVS_23050,
    PACE_EVS_23850,
    PACE_EVS_24400,
    PACE_EVS_32000,
    PACE_EVS_48000,
    PACE_EVS_64000,
    PACE_EVS_96000,
    PACE_EVS_128000,
    PACE_EVS_RESERVED,
    PACE_EVS_SPEECH_LOST,
    PACE_EVS_NO_DATA,

    IPOQUE_NUMBER_OF_EVS_MODES
};

enum pace_evs_bw {
    PACE_EVS_NO_BAND = 0,
    PACE_EVS_NARROW_BAND,
    PACE_EVS_AMR_IO_BAND,
    PACE_EVS_WIDE_BAND,
    PACE_EVS_SUPER_WIDE_BAND,
    PACE_EVS_FULL_BAND,
    PACE_EVS_RESERVED_BAND,
    PACE_EVS_NO_REQ_BAND,

    IPOQUE_NUMBER_OF_EVS_BANDS
};

enum pace_evs_type {
    PACE_EVS_AMR_WB_IO = 0,
    PACE_EVS_PRIMARY,

    IPOQUE_NUMBER_OF_EVS_TYPES
};

enum pace_evs_header_type {
    PACE_EVS_INVALID = 0,
    PACE_EVS_COMPACT_HEADER,
    PACE_EVS_FULL_HEADER,

    IPOQUE_NUMBER_OF_EVS_HEADER_TYPES
};

#define PACE_EVS_MAX_SUPPORTED_TOC 10

struct pace_evs_cmr {
    enum pace_evs_mode mode;                     /*!< Mode requested by CMR */
    enum pace_evs_bw bw;                         /*!< Bandwidth requested by CMR */
    u32 bitrate;                                 /*!< Requested bitrate */
};

struct pace_evs_packet_info {
    union {
        struct {
            struct {
                enum pace_evs_type type;      /*!< AMR-WB IO or EVS Primary */
                enum pace_evs_mode mode;      /*!< Enum representation of current EVS Mode/Bitrate */
                u32 bitrate;                  /*!< Bitrate in bits per second (dependent on EVS Mode) */

                u8 amr_q_bit;                 /*!< Whether the AMR quality bit is set or not */
            } toc[PACE_EVS_MAX_SUPPORTED_TOC];

            struct pace_evs_cmr cmr;          /*!< CMR */

            u8 toc_count;                     /*!< Count of available TOC's */
        } full;

        struct {
            struct pace_evs_cmr cmr;          /*!< CMR */
            enum pace_evs_type type;          /*!< AMR-WB IO or EVS Primary */
            enum pace_evs_mode mode;          /*!< Enum representation of current EVS Mode/Bitrate */
            u32 bitrate;                      /*!< Bitrate in bits per second (dependent on EVS Mode) */
        } compact;
    } header;

    enum pace_evs_header_type header_type;    /*!< Header format of this packet */
};

#define PACE_STUN_MAX_REPORTED_ATTRIBUTES 24  /*!< Maximum number of attributes that can be found in one pcap */
/**
 * @brief The pace_stun_message_class enum is used for values for the message class
 */
enum pace_stun_message_class
{
    PACE_STUN_MESSAGE_REQUEST = 0x00,
    PACE_STUN_MESSAGE_INDICATION = 0x01,
    PACE_STUN_MESSAGE_SUCCESS_RESPONSE = 0x02,
    PACE_STUN_MESSAGE_ERROR_RESPONSE = 0x03
};

/**
 * @brief The pace_stun_message_method enum is used for values for the message method
 */
enum pace_stun_message_method
{
    PACE_STUN_MESSAGE_RESERVED = 0x00,
    PACE_STUN_MESSAGE_BINDING = 0x01,
    PACE_STUN_MESSAGE_ALLOCATE = 0x03,
    PACE_STUN_MESSAGE_REFRESH = 0x04,
    PACE_STUN_MESSAGE_SEND = 0x06,
    PACE_STUN_MESSAGE_DATA = 0x07,
    PACE_STUN_MESSAGE_CREATE_PERMISSION = 0x08,
    PACE_STUN_MESSAGE_CHANNEL_BIND = 0x09,
    PACE_STUN_MESSAGE_CONNECT = 0x0a,
    PACE_STUN_MESSAGE_CONNECTION_BIND = 0x0b,
    PACE_STUN_MESSAGE_CONNECTION_ATTEMPT = 0x0c,
    PACE_STUN_MESSAGE_GOOG_PING = 0x80
};

/**
 * @brief The pace_stun_attribute_type enum is used for values for the attribute type
 */
enum pace_stun_attribute_type {
    PACE_STUN_ATTRIBUTE_MAPPED_ADDRESS = 0x0001,
    PACE_STUN_ATTRIBUTE_RESPONSE_ADDRESS = 0x0002,
    PACE_STUN_ATTRIBUTE_SOURCE_ADDRESS = 0x0004,
    PACE_STUN_ATTRIBUTE_CHANGED_ADDRESS = 0x0005,
    PACE_STUN_ATTRIBUTE_USERNAME = 0x0006,
    PACE_STUN_ATTRIBUTE_MESSAGE_INTEGRITY = 0x0008,
    PACE_STUN_ATTRIBUTE_ERROR_CODE = 0x0009,
    PACE_STUN_ATTRIBUTE_UNKNOWN_ATTRIBUTES = 0x000a,
    PACE_STUN_ATTRIBUTE_REFLECTED_FROM = 0x000b,
    PACE_STUN_ATTRIBUTE_CHANNEL_NUMBER = 0x000c,
    PACE_STUN_ATTRIBUTE_LIFETIME = 0x000d,
    PACE_STUN_ATTRIBUTE_XOR_PEER_ADDRESS = 0x0012,
    PACE_STUN_ATTRIBUTE_DATA = 0x0013,
    PACE_STUN_ATTRIBUTE_REALM = 0x0014,
    PACE_STUN_ATTRIBUTE_NONCE = 0x0015,
    PACE_STUN_ATTRIBUTE_XOR_RELAYED_ADDRESS = 0x0016,
    PACE_STUN_ATTRIBUTE_REQUESTED_ADDRESS_FAMILY = 0x0017,
    PACE_STUN_ATTRIBUTE_EVEN_PORT = 0x0018,
    PACE_STUN_ATTRIBUTE_REQUESTED_TRANSPORT = 0x0019,
    PACE_STUN_ATTRIBUTE_DONT_FRAGMENT = 0x001a,
    PACE_STUN_ATTRIBUTE_PASSWORD_ALGORITHM = 0x001d,
    PACE_STUN_ATTRIBUTE_USERHASH = 0x001e,
    PACE_STUN_ATTRIBUTE_XOR_MAPPED_ADDRESS_1 = 0x0020,
    PACE_STUN_ATTRIBUTE_XOR_MAPPED_ADDRESS_2 = 0x8020,
    PACE_STUN_ATTRIBUTE_RESERVATION_TOKEN = 0x0022,
    PACE_STUN_ATTRIBUTE_PRIORITY = 0x0024,
    PACE_STUN_ATTRIBUTE_USE_CANDIDATE = 0x0025,
    PACE_STUN_ATTRIBUTE_PADDING = 0x0026,
    PACE_STUN_ATTRIBUTE_RESPONSE_PORT = 0x0027,
    PACE_STUN_ATTRIBUTE_CONNECTION_ID = 0x002a,
    PACE_STUN_ATTRIBUTE_ADDITIONAL_ADDRESS_FAMILY = 0x8000,
    PACE_STUN_ATTRIBUTE_ADDRESS_ERROR_CODE = 0x8001,
    PACE_STUN_ATTRIBUTE_PASSWORD_ALGORITHMS = 0x8002,
    PACE_STUN_ATTRIBUTE_ALTERNATE_DOMAIN = 0x8003,
    PACE_STUN_ATTRIBUTE_ICMP = 0x8004,
    PACE_STUN_ATTRIBUTE_SOFTWARE = 0x8022,
    PACE_STUN_ATTRIBUTE_ALTERNATE_SERVER = 0x8023,
    PACE_STUN_ATTRIBUTE_TRANSACTION_TRANSMIT_COUNTER = 0x8025,
    PACE_STUN_ATTRIBUTE_CACHE_TIMEOUT = 0x8027,
    PACE_STUN_ATTRIBUTE_FINGERPRINT = 0x8028,
    PACE_STUN_ATTRIBUTE_ICE_CONTROLLED = 0x8029,
    PACE_STUN_ATTRIBUTE_ICE_CONTROLLING = 0x802a,
    PACE_STUN_ATTRIBUTE_RESPONSE_ORIGIN = 0x802b,
    PACE_STUN_ATTRIBUTE_OTHER_ADDRESS = 0x802c
};

/**
 * @brief The pace_stun_address_attributes_types enum is used for values for the address attribute type
 */
enum pace_stun_address_attributes_types {
    PACE_STUN_ADDRESS_ATTRIBUTE_MAPPED_ADDRESS = 0,    /*!< Transport address of the client. */
    PACE_STUN_ADDRESS_ATTRIBUTE_RESPONSE_ADDRESS,      /*!< Indicates where Binding request response should be sent. */
    PACE_STUN_ADDRESS_ATTRIBUTE_SOURCE_ADDRESS,        /*!< Address where Binding response sent from. */
    PACE_STUN_ADDRESS_ATTRIBUTE_CHANGED_ADDRESS,       /*!< Address where responses would've been sent if change flags set. */
    PACE_STUN_ADDRESS_ATTRIBUTE_XOR_MAPPED_ADDRESS,    /*!< Transport address obfuscated with XOR function. */
    PACE_STUN_ADDRESS_ATTRIBUTE_RESPONSE_ORIGIN,       /*!< Transport address where response sent from. */
    PACE_STUN_ADDRESS_ATTRIBUTE_OTHER_ADDRESS,         /*!< Address that would be used if change flags set. */
    PACE_STUN_ADDRESS_ATTRIBUTE_ALTERNATE_SERVER,      /*!< Alternate transport address for client to try. */
    PACE_STUN_ADDRESS_ATTRIBUTE_REFLECTED_FROM,        /*!< Contains address of source of request. */
    PACE_STUN_ADDRESS_ATTRIBUTE_XOR_PEER_ADDRESS,      /*!< Contains address of source of request. */
    PACE_STUN_ADDRESS_ATTRIBUTE_XOR_RELAYED_ADDRESS,   /*!< Contains address of source of request. */

    PACE_NUMBER_OF_ADDRESS_ATTRIBUTES_TYPES            /*!< Number of address attributes types. */
};

/**
 * @brief The pace_icmp_types enum is used for values for the icmp type
 */
enum pace_icmp_types {
    PACE_ICMP_ECHO_REPLY = 0x00,       /*!< Echo Reply. */
    PACE_ICMP_DEST_UNREACHABLE = 0x03, /*!< Destination Unreachable. */
    PACE_ICMP_REDIRECT = 0x05,         /*!< Redirect. */
    PACE_ICMP_ECHO_REQUEST = 0x08,     /*!< Echo Request. */
    PACE_ICMP_ROUTER_ADVERT = 0x09,    /*!< Router Advertisement. */
    PACE_ICMP_ROUTER_SOLICIT = 0x0a,   /*!< Router Solicitation. */
    PACE_ICMP_TIME_EXCEEDED = 0x0b,    /*!< Time Exceeded. */
    PACE_ICMP_PARAM_PROBLEM = 0x0c,    /*!< Parameter Problem. */
    PACE_ICMP_TIMESTAMP = 0x0d,        /*!< Timestamp. */
    PACE_ICMP_TIMESTAMP_REPLY = 0x0e,  /*!< Timestamp Reply. */
    PACE_ICMP_PHOTURIS = 0x28,         /*!< Photuris. */
    PACE_ICMP_MOBILITY_PROT = 0x29,    /*!< ICMP messages utilized by experimental mobility protocols e.g Seamoby. */
    PACE_ICMP_EXT_ECHO_REQUEST = 0x2a, /*!< Extended Echo Request. */
    PACE_ICMP_EXT_ECHO_REPLY = 0x2b,   /*!< Extended Echo Reply. */
    PACE_ICMP_EXP_VALUES_1 = 0xfd,     /*!< RFC3692-style Experiment 1. */
    PACE_ICMP_EXP_VALUES_2 = 0xfe      /*!< RFC3692-style Experiment 2. */
};

/**
 * @brief Extended report block. (RFC 3611)
 */
struct rtcp_extended_block {
    u8 block_type;     /*!< Block type. */
    u16 type_specific; /*!< Type specific data */
    u16 block_length;  /*!< Block length */
#define RTCP_WORD_COUNT_MAX 8
    union {
        /**
         * @brief Provide generic data access for all other block types.
         */
        struct {
            /**
             * Words in network byte order. Number of blocks is specified by @ref rtcp_extended_block::block_length,
             * maximum 8, additional blocks will not be reported.
             */
            u32 block_content[RTCP_WORD_COUNT_MAX];
        } generic;
#define RTCP_BT_RRTRB 0x04
        /**
         * @brief Receiver Reference Time Report Block
         */
        struct {
            struct {
                u32 seconds;  /*!< Seconds part of the timestamp */
                u32 fraction; /*!< Fraction part of the timestamp */
            } ntp_timestamp;
        } rrtrp;
#define RTCP_BT_DLRR 0x05
        /**
         * @brief DLRR Report Block
         */
        struct {
            struct {
                u32 identifier;          /*!< Identifier */
                u32 last_rr_timestamp;   /*!< Last RR timestamp */
                u32 delay_since_last_rr; /*!< Delay since the last RR */
                u32 padding;             /*!< Padding */
#define RTCP_SOURCE_COUNT_MAX 4
            } sources[RTCP_SOURCE_COUNT_MAX];
            u8 source_count;             /*!< Count of sub blocks. */
        } dlrr;
#define RTCP_BT_STATISTIC_SUMMARY 0x06
        /**
         * @brief Statistics Summary Report Block
         */
        struct {
            u32 identifier;     /*!< Identifier */
            u16 begin_seq;      /*!< First sequence number. */
            u16 end_seq;        /*!< Last sequence number. */
            u32 lost_packets;   /*!< Lost packets. */
            u32 dup_packets;    /*!< Duplicate packets. */
            u32 min_jitter;     /*!< Min jitter. */
            u32 max_jitter;     /*!< Max jitter. */
            u32 mean_jitter;    /*!< Mean jitter. */
            u32 dev_jitter;     /*!< Dev jitter. */
            u16 min_ttl_or_hl;  /*!< Minimum of TTL or Hop Limit */
            u16 max_ttl_or_hl;  /*!< Maximum of TTL or Hop Limit */
            u16 mean_ttl_or_hl; /*!< Mean of TTL or Hop Limit */
            u16 dev_ttl_or_hl;  /*!< Standard deviation of TTL or Hop Limit */
        } stat_summary;
#define RTCP_BT_VOIP_METRICS 0x07
        /**
         * @brief VoIP Metrics Report Block
         */
        struct {
            u32 identifier;               /*!< Identifier */
            u8 loss_rate;                 /*!< Loss rate */
            u8 discard_rate;              /*!< Discard rate */
            u8 burst_density;             /*!< Burst density */
            u8 gap_density;               /*!< Gap density */
            u16 burst_duration;           /*!< Burst duration */
            u16 gap_duration;             /*!< Gap duration */
            u16 round_trip_delay;         /*!< Round trip delay */
            u16 end_system_delay;         /*!< End system delay */
            u8 signal_level;              /*!< Signal level */
            u8 noise_level;               /*!< Noise level */
            u8 residual_echo_return_loss; /*!< Resigual echo return loss */
            u8 g_min;                     /*!< G min */
            u8 r_factor;                  /*!< R factor */
            u8 ext_r_factor;              /*!< ext. R factor */
            u8 mos_lq;                    /*!< MOS-LQ */
            u8 mos_cq;                    /*!< MOS-CQ */
            u8 rx_config;                 /*!< RX config */
            u8 reserved;                  /*!< reserved */
            u16 jb_nominal;               /*!< JB nominal */
            u16 jb_maximum;               /*!< JB maximum */
            u16 jb_abs_max;               /*!< JB abs max */
        } voip_metrics;
    };
};

union dissector_result {
    struct {
        struct {
            u16 min;                /*!< Minimum packet size per flow */
            u16 max;                /*!< Maximum packet size per flow */
            u16 avg;                /*!< Average packet size per flow */
        } packet_size[2];

        u32 invalid_checksums[2];   /*!< Total count of invalid IP checksum per flow and direction */

        struct {
            u64 min;                /*!< Minimum throughput in Bytes/s per flow */
            u64 max;                /*!< Maximum throughput in Bytes/s per flow */
            u64 avg;                /*!< Average throughput in Bytes/s per flow */
        } throughput[2];

        u64 throughput_interval[2];    /*!< Throughput in Bytes/s
                                            The interval is the time between one get_metadata_ip()
                                            function call and the next one. */
        u32 interval_duration[2];      /*!< Interval duration of throughput_interval in s or ms */
        u64 interval_duration_hres[2]; /*!< Interval duration of throughput_interval in ticks. */

        u64 packets[2];                /*!< Number of packets per direction */
        u64 bytes[2];                  /*!< Bytes per direction */
    } ip;

    struct {

        u32 ooo_packets;            /*!< Total count of out-of-order packets per flow, the value can overflow */
        u32 retransmissions;        /*!< Total count of retransmitted packets per flow, the value can overflow */
        u32 invalid_checksums[2];   /*!< Total count of packets with an invalid TCP checksum per flow and direction,
                                         the value can overflow */

        struct {
            u32 min;                /*!< Minimum TCP window size per flow and direction */
            u32 max;                /*!< Maximum TCP window size per flow and direction */
            u32 avg;                /*!< Average TCP window size per flow and direction */
            u32 cur;                /*!< Current TCP window size of the packet */
        } window_size[2];

        struct {
            u8 value:4;             /*!< Current window scale value per flow and direction */
        } window_scale[2];

        struct {
            u16 min;                /*!< Minimum RTT per flow and direction. [ms] */
            u16 max;                /*!< Maximum RTT per flow and direction. [ms] */
            u16 avg;                /*!< Average RTT per flow and direction. [ms] */
            u16 cur;                /*!< Current RTT of the packet */
        } round_trip_time
#if defined(clang) || (__GNUC__ > 4 || (__GNUC__ == 4 && (__GNUC_MINOR__ > 4)))
        __attribute__ ((deprecated("This struct is deprecated, use round_trip_time_dir instead.")));
#else
        __attribute__ ((deprecated));
#endif

        struct {
            u16 min;                /*!< Minimum TTT per flow. [ms] */
            u16 max;                /*!< Maximum TTT per flow. [ms] */
            u16 avg;                /*!< Average TTT per flow. [ms] */
            u16 cur;                /*!< Current TTT of the packet. [ms] */
            u64 min_hres;           /*!< Minimum TTT per flow in ticks. */
            u64 max_hres;           /*!< Maximum TTT per flow in ticks. */
            u64 avg_hres;           /*!< Average TTT per flow in ticks. */
            u64 cur_hres;           /*!< Current TTT of the packet in ticks. */
            u16 transaction_count;  /*!< Number of total transactions per flow */
            u8 started:1;           /*!< Indicates a running transaction */
            u8 stopped:1;           /*!< Indicates that a transaction is over */
        } total_transaction_time;

        struct {
            u16 min;                /*!< Minimum server delay per flow. [ms] */
            u16 max;                /*!< Maximum server delay per flow. [ms] */
            u16 avg;                /*!< Average server delay per flow. [ms] */
            u16 cur;                /*!< Current server delay of the packet. [ms] */
            u64 min_hres;           /*!< Minimum server delay per flow in ticks. */
            u64 max_hres;           /*!< Maximum server delay per flow in ticks. */
            u64 avg_hres;           /*!< Average server delay per flow in ticks. */
            u64 cur_hres;           /*!< Current server delay of the packet in ticks. */
            u16 transaction_count;  /*!< Number of total server delays per flow */
            u8 started:1;           /*!< Indicates a running server delay */
            u8 stopped:1;           /*!< Indicates that the server delay has finished */
        } server_delay;

        u64 network_limit;          /*!< Theoretical throughput of this flow without considering the TCP window,
                                         given in Bytes/s */
        u64 throughput_max[2];      /*!< Maximum possible TCP throughput, given in Bytes/s */

        u16 max_segment_size;           /*!< Maximum segment size */
        u32 window_update_packets[2];   /*!< Number of window updates that occurred, the value can overflow */
        u32 zero_window_packets[2];     /*!< Number of packets with a window size of zero, the value can overflow */
        u32 timeout_retransmissions[2]; /*!< Number of retransmissions due to timeout, the value can overflow */
        u32 fast_retransmissions[2];    /*!< Number of fast retransmissions triggered by dup acks,
                                             the value can overflow */
        u32 ramp_up_time[2];            /*!< Time in milliseconds from the flow start to the first peak in window size.
                                             Measured only after the first payload packet.
                                             Value remains at 0 in case the window size never increases. */
        u64 ramp_up_time_hres[2];       /*!< Ticks from the flow start to the first peak in window size.
                                             Measured only after the first payload packet.
                                             Value remains at 0 in case the window size never increases. */
        u32 ramp_up_drop_time[2];       /*!< Time in milliseconds from the flow start to the first decrease in window size.
                                             Measured only after the first payload packet.
                                             Value remains at 0 in case the window size never decreases. */
        u64 ramp_up_drop_time_hres[2];  /*!< Ticks from the flow start to the first decrease in window size.
                                             Measured only after the first payload packet.
                                             Value remains at 0 in case the window size never decreases. */
        u32 out_of_order[2];            /*!< Number of out of order packets that occurred per direction.
                                             Measured only after the handshake.
                                             the value can overflow */

        struct {
            u16 min;                /*!< Minimum RTT per flow and direction. [ms] */
            u16 max;                /*!< Maximum RTT per flow and direction. [ms] */
            u16 avg;                /*!< Average RTT per flow and direction. [ms] */
            u16 cur;                /*!< Current RTT of the packet. [ms] */
        } round_trip_time_dir[2];
        struct {
            u64 min;                /*!< Minimum RTT per flow and direction in ticks.*/
            u64 max;                /*!< Maximum RTT per flow and direction in ticks. */
            u64 avg;                /*!< Average RTT per flow and direction in ticks. */
            u64 cur;                /*!< Current RTT of the packet in ticks. */
        } round_trip_time_dir_hres[2];
        enum TCP_CONNECTION_STATE connection_state_client;
        enum TCP_CONNECTION_STATE connection_state_server;
        struct {
            u32 min;                /*!< Minimum Jitter measurement. [ms^2] */
            u32 max;                /*!< Maximum Jitter measurement. [ms^2] */
            u32 var;                /*!< Jitter measurement as the variance of the RTT. [ms^2] */
        } jitter[2];
    } tcp;


    struct {
        struct {
            u8 numerator;
            u8 divisor;
        } aspect_ratio[2];                      /*!< aspect ratio, i.e 4:3 */

        u32 bitrate[2];                         /*!< bitrate in bits per second */
        u32 frame_count[2];                     /*!< total number of frames */
        u32 iframe_count[2];                    /*!< number of i-frames */
        u16 picture_width[2];                   /*!< picture width in pixels */
        u16 picture_height[2];                  /*!< picture height in pixels */
        u8 frame_rate[2];                       /*!< frame rate in frames per second */
        u8 avg_frame_rate[2];                   /*!< average frame rate in frames per second */

        struct {
            u8 is_set;                          /*!< defines if the video codec profile is present or not */
            u8 profile_idc;                     /*!< video codec profile_idc */
            u8 level_id;                        /*!< video codec level_id */

            struct {
                u8 set0_flag : 1;
                u8 set1_flag : 1;
                u8 set2_flag : 1;
                u8 set3_flag : 1;
                u8 set4_flag : 1;
                u8 set5_flag : 1;
                u8 reserved_bit1 : 1;
                u8 reserved_bit2 : 1;
            } constraint;                       /*!< video codec constraint */

        } video_codec[2];                       /*!< information about the video codec */

    } h264;

    struct {
        const char *codec_str[2];               /*!< detected codec as string */
        const char *stream_type_str[2];         /*!< detected stream type as string */
        enum codec_type codec[2];               /*!< detected codec as enum value */
        enum stream_media_type stream_type[2];  /*!< detected stream type as enum value */

        struct {
            u32 timestamp;                      /*!< timestamp in host byte order */
            u32 ssrc;                           /*!< ssrc in host byte order */
            u16 sequence_number;                /*!< sequence number in host byte order */
            u8 payload_type;                    /*!< payload type */
            u8 padding_length;                  /*!< length of the padding in bytes if present */
            u8 extensions_used:1;               /*!< extension flag  */
            u8 marker_set:1;                    /*!< marker flag */
            u8 padding_used:1;                  /*!< padding flag  */
            u8 csrc_count:4;                    /*!< csrc count */
        } packet;                               /*!< fields in this struct are set for the last processed packet */

        struct {
            u32 packets_sent[2];                /*!< number of packets sent, for each direction */
            u32 packets_lost[2];                /*!< number of packets lost, for each direction */
            u32 current_jitter[2];              /*!< jitter, for each direction */
            u32 current_frequency[2];           /*!< frequency used for jitter calculation, for each direction */
            u32 packets_overhead[2];            /*!< number of overhead packets for each direction */
            u32 current_jitter_ms[2];           /*!< RTP timestamp in millisamples. ( [ms] = [mS] / Hz ) */
        } statistics;

    } rtp;

    struct {
        u32 bitrate[2];                         /*!< bitrate in bits per second */
        u8 dtx_silence_active[2];               /*!< whether dtx silence is currently active or not */
        u8 mode[2];                             /*!< detected AMR mode */
    } amr;

    struct {
        struct {
            struct metadata_string talb;       /*!< Album/Movie/Show title */
            struct metadata_string tpe1;       /*!< Lead performer(s)/Soloist(s) */
            struct metadata_string tpe2;       /*!< Band/orchestra/accompaniment */
            struct metadata_string tpe3;       /*!< Conductor/performer refinement */
            struct metadata_string tpe4;       /*!< Interpreted, remixed, or otherwise modified by */
            struct metadata_string trck;       /*!< Track number/Position in set */
            struct metadata_string tit1;       /*!< Content group description */
            struct metadata_string tit2;       /*!< Title/songname/content description */
            struct metadata_string tit3;       /*!< Subtitle/Description refinement */
            struct metadata_string tcon;       /*!< Content type */
        } frames;
        u8 version;                            /*!< ID3v2 version */
    } id3;

    struct {
        const char * codec_str[2];             /*!< detected codec as string */
        enum codec_type codec[2];              /*!< detected codec as enum value */
        struct {
            u32 bitrate;                       /*!< bitrate in bits per second */
            u16 sampling_rate;                 /*!< Sampling rate frequency index */
            u8 channel_mode:2;                 /*!< Channel mode */
            u8 version:2;                      /*!< MPEG Audio version ID */
            u8 copyright:1;                    /*!< Copyright*/
            u8 original:1;                     /*!< Original media */
        } audio[2];

    } mp3;

    struct {
        struct {
            enum pace_http_content_type type;   /*!< the container format which could be identified */
            char const * type_str;              /*!< detected type as string */
            u8 done;                            /*!< tracks the status of the http dissector - 0: still ongoing; 1: done */
        } content;
    } http;

    struct {

        struct {
            const char *codec_str;              /*!< detected codec as string */
            enum codec_type codec;              /*!< detected codec as enum value */
            u32 bitrate;                        /*!< bitrate of the video stream */
            u16 width;                          /*!< width of the video in pixels */
            u16 height;                         /*!< width of the video in pixels */
            u16 color_depth;                    /*!< color depth of the video in bit */
            struct {
                u8 numerator;
                u8 divisor;
            } aspect;                           /*!< aspect ratio, i.e 4:3 */
            u8 fps;                             /*!< calculated frames per second */
        } video[2];

        struct {
            const char *codec_str;              /*!< detected codec as string */
            enum codec_type codec;              /*!< detected codec as enum value */
            u32 sampling_rate;                  /*!< sampling rate of the audio stream */
            u32 bitrate;                        /*!< bitrate of the audio stream in bits per second */
            u16 channels;                       /*!< number of audio channels */
        } audio[2];

        u64 duration[2];                             /*!< the current duration of the stream in seconds */
        const char *stream_type_str[2];              /*!< detected stream type as string */
        enum stream_media_type type[2];              /*!< detected stream type as enum value */
        enum stream_transfer_type transfer_type[2];  /*!< detected stream transfer type as enum value */
    } mp4;

    struct {
        struct {
            struct rtcp_report report[3];           /*!< Report entries */

            struct {
                u32 seconds;                        /*!< Seconds part of the timestamp */
                u32 fraction;                       /*!< Fraction part of the timestamp */
            } ntp_timestamp;

            u32 rtp_timestamp;                      /*!< RTP timestamp */
            u32 sender_ssrc;                        /*!< Sender SSRC */
            u32 sender_packet_count;                /*!< Packet count */
            u32 sender_octet_count;                 /*!< Byte count */
            u8 report_count;                        /*!< Count of parsed reports */
        } sender_report;

        struct {
            struct {
                struct {
                    enum sdes_item_type type;       /*!< Item type */
                    const u8 *type_str;             /*!< Item type as null terminated string */
                    struct metadata_string content; /*!< Pointer to the content and its length */
                } item[8];

                u32 identifier;                     /*!< Identifier */
                u8 item_count;                      /*!< Count of parsed items */
            } chunk[3];

            u8 chunk_count;                         /*!< Count of parsed chunks */
        } source_description;

        struct {
            struct rtcp_report report[3];           /*!< Report entries */

            u32 sender_ssrc;                        /*!< Sender SSRC */
            u8 report_count;                        /*!< Count of parsed reports */
        } receiver_report;

        struct {
            u32 identifier;                         /*!< Identifier */
            const u8 *name;                         /*!< Name, always 4 byte */
            const u8 *data_ptr;                     /*!< Pointer to application data */
            u16 data_len;                           /*!< Length of application data */
            u8 subtype;                             /*!< Subtype id */
        } application_specific;

        struct {
            struct metadata_string text;            /*!< Text message */
            u32 identifier;                         /*!< Identifier */
            u32 additional_identifier[20];          /*!< Additional identifiers */
            u8 additional_identifier_count;         /*!< Number of additional identifiers */
        } good_bye;

        struct {
            u32 sender_ssrc;                        /*!< Sender SSRC */
            u32 media_source_ssrc;                  /*!< Source SSRC */
            u32 feedback_control_information;       /*!< FCI */
        } reference_picture_selection_indication;

        struct {
            u32 sender_ssrc;                        /*!< Sender SSRC */
            u32 media_source_ssrc;                  /*!< Source SSRC */
            u16 pid;                                /*!< Number of first lost frame */
            u16 blp;                                /*!< Lost frame bitmask */
        } negative_acknowledgement;

        struct {
            struct {
                u64 max_bit_rate;                   /*!< Maximum bitrate, calculated from exp and mantissa */
                u32 ssrc;                           /*!< SSRC */
                u32 mantissa;                       /*!< Mantissa part */
                u16 exponent;                       /*!< Exponent part */
                u16 overhead;                       /*!< Overhead */
            } item[3];

            u32 sender_ssrc;                        /*!< Sender SSRC */
            u32 media_source_ssrc;                  /*!< Source SSRC */
            u8 item_count;                          /*!< Count of parsed items */
        } max_bitrate_request;

        struct {
            struct {
                u64 max_bit_rate;                   /*!< Maximum bitrate, calculated from exp and mantissa */
                u32 ssrc;                           /*!< SSRC */
                u32 mantissa;                       /*!< Mantissa part */
                u16 exponent;                       /*!< Exponent part */
                u16 overhead;                       /*!< Overhead */
            } item[3];

            u32 sender_ssrc;                        /*!< Sender SSRC */
            u32 media_source_ssrc;                  /*!< Source SSRC */
            u8 item_count;                          /*!< Count of parsed items */
        } max_bitrate_notification;

        struct {
            u32 sender_ssrc;                        /*!< Sender SSRC */
            u32 media_source_ssrc;                  /*!< Source SSRC */
        } picture_loss_indication;

        struct {
            struct {
                u32 sender_ssrc;                    /*!< Sender SSRC */
                u32 rtt_cur;                        /*!< Round trip time in milliseconds of the source identified by 'sender_ssrc' */
                u32 rtt_min;                        /*!< Minimum round trip time in milliseconds of the source identified by 'sender_ssrc' */
                u32 rtt_max;                        /*!< Maximum round trip time in milliseconds of the source identified by 'sender_ssrc' */
                u32 rtt_avg;                        /*!< Round trip time in milliseconds of the source identified by 'sender_ssrc' */
            } report_block[3];
            u8 report_count;                        /*!< Count of parsed reports */
        } rtt;

        u64 bitmask;                                /*!< Bitmask of available structures */

        struct {
            u32 sender_ssrc;                                             /*!< Sender SSRC */
#define RTCP_EXTENTED_BLOCKS_MAX 3
            struct rtcp_extended_block blocks[RTCP_EXTENTED_BLOCKS_MAX]; /*!< Extended block */
            u8 block_count;                                              /*!< Count of parsed blocks */
        } extended_report;
    } rtcp;

    struct {
        struct {
            u8 numerator;
            u8 divisor;
        } aspect_ratio[2];                      /*!< aspect ratio, i.e 4:3 */

        u32 avg_bitrate[2];                     /*!< average bitrate in bits per second */
        u32 frame_count[2];                     /*!< total number of frames */
        u32 iframe_count[2];                    /*!< number of i-frames */
        u16 picture_width[2];                   /*!< picture width in pixels */
        u16 picture_height[2];                  /*!< picture height in pixels */
        u8 avg_frame_rate[2];                   /*!< average frame rate in frames per second */
        u8 mode[2];                             /*!< Mode of the H.263 payload header */
    } h263;

    struct {
        struct {
            u32 count;                              /*!< Count of speech frames seen */
            u32 evs_primary;                        /*!< Count of speech frames in EVS Primary mode */
            u32 amr_wb_io;                          /*!< Count of speech frames in AMR-WB IO mode */
            u32 dtx;                                /*!< Count of DTX frames */
        } frames[2];

        struct {
            u32 full_header;                        /*!< Count of full header packets */
            u32 compact_header;                     /* Count of compact header packets */
        } packets[2];

        struct pace_evs_cmr cmr[2];                 /*!< CMR */

        const struct pace_evs_packet_info *evs_packet;   /*!< EVS packet data, NULL if not available. */

        u32 bitrate[2];                             /*!< Bitrate in bits per second (dependent on EVS Mode) */

        enum pace_evs_type type[2];               /*!< AMR-WB IO or EVS Primary */
        enum pace_evs_mode mode[2];               /*!< Enum representation of current EVS Mode/Bitrate */
        enum pace_evs_header_type header_type[2]; /*!< Header type of the last packet */

        u8 dtx_silence_active[2];                   /*!< Whether dtx silence is currently active or not */
    } evs;

    struct {
        u32 hash_algos[7];       /*!< supported hash algorithms */
        u32 cipher_algos[7];     /*!< supported cipher algorithms */
        u32 auth_types[7];       /*!< supported SRTP auth tag types */
        u32 key_types[7];        /*!< supported key agreement types */
        u32 sas_types[7];        /*!< supported SAS types */
        u32 zid[3];              /*!< unique ZRTP endpoint id in host byte order */
        char cid[16];            /*!< client identifier string */
        char version[4];         /*!< version string of ZRTP protocol */
        u8 hash_algos_count:3;   /*!< count of supported hash algorithms */
        u8 cipher_algos_count:3; /*!< count of supported cipher algorithms */
        u8 auth_types_count:3;   /*!< count of supported SRTP auth tag types */
        u8 key_types_count:3;    /*!< count of supported key agreement types */
        u8 sas_types_count:3;    /*!< count of supported SAS types */
        u8 signature_flag:1;     /*!< endpoint is able to parse and verify digital signatures */
        u8 mitm_flag:1;          /*!< endpoint is capable of sending an SASrelay message */
        u8 passive_flag:1;       /*!< endpoint is configured to never send a Commit message */
    } zrtp;

    struct {
        struct {
            u8 m_class:2;                        /*!< Class from method type */
            u16 m_method:12;                     /*!< Method from method type */
        } message_type;

        struct {
            struct {
                enum pace_stun_attribute_type type;  /*!< Attribute Type */
                struct metadata_string value;        /*!< Attribute Value */
            } attribute[PACE_STUN_MAX_REPORTED_ATTRIBUTES]; /*!< Array of Attributes */
            u8 count;                                /*!< number of Attributes */
        } attributes;

        struct {
            struct pace_stun_address {
                u32 ip;                               /*!< IP for mapped address (IPv4 in current implementation)*/
                u16 addr_port;                        /*!< Port for mapped address */
                u8 family;                            /*!< 0x01:IPv4 ; 0x02:IPv6 */
            } address_attribute[PACE_NUMBER_OF_ADDRESS_ATTRIBUTES_TYPES]; /* Array of address attributes types */
            u8 count;                                 /*!< number of address attributes found */
        } address_attributes;

        struct {
            struct {
                u32 part_1;
                u32 part_2;
            } array[5];
            u8 count:3;
        } reservation_tokens_attribute;           /*!< Tokens that uniquely identifies a relayed transport address */

        struct {
            u16 array[4];
            u8 count:3;
        } unknown_attributes;                     /*!< Attributes not understood by server up to 4 */

        struct {
           u16 code;
           struct metadata_string reason_phrase;
        } error_attribute;                        /*!< Error */

        struct metadata_string message_transaction_id;  /*!< The transaction ID correlates requests and responses */

        u16 message_length;                             /*!< Message length */
        u8 flags_count:3;                               /*!< Count of flags set in packet */
        u8 use_candidate_flag:1;                        /*!< Indicates candidate pair checked be used for transmission */
        u8 even_port_flag:1;                            /*!< Client request that the port address be even */

        struct {
            u8 req;
            u8 resp;
        } transaction_transmit_counter_attribute;       /*!< TRANSACTION_TRANSMIT_COUNTER Attribute */
    } stun;

    struct {
        u32 unused;            /*!< Unused field */
        u32 orig_time;         /*!< Originate Timestamp */
        u32 recv_time;         /*!< Receive Timestamp */
        u32 trans_time;        /*!< Transmit Timestamp */
        u16 checksum;          /*!< Checksum */
        u16 identifier;        /*!< Identifier */
        u16 seq_number;        /*!< Sequence Number */
        u16 lifetime;          /*!< Lifetime */
        u16 data_len;          /*!< Length of remaining data */
        u8 type;               /*!< Type */
        u8 code;               /*!< Code */
        u8 num_addresses;      /*!< Number of Addresses */
        u8 octet_pointer;      /*!< Identifies the octet where an error was detected */
        u8 address_entry_size; /*!< Address entry size */
        u8 ext_reserved_bits;  /*!< For reserved bits */
        u8 gate_address[4];    /*!< Gateway Address */
        const u8 * data_ptr;   /*!< Pointer to remaining data */
    } icmp;

    struct {
        struct {
            u8 handshake_rtt_complete:1; /*!< Indicates whether the handshake RTT measurement has completed. */
            struct {
                u32 observer_server;     /*!< Delay between the server's and client's initial packets. */
                u32 client_observer;     /*!< Delay between the server's and client's first handshake packets. */
                u32 value;               /*!< Total handshake RTT value.*/
            } handshake;
        } rtt;
    } quic;
};

enum throughput_resolution {
    METADATA_DISSECTOR_UNDEFINED = 0,
    METADATA_DISSECTOR_SECONDS,
    METADATA_DISSECTOR_MILLISECONDS,
};

struct dissector_config {
    struct {
        u8 packet_size_enabled;
        u8 throughput_enabled;
        u8 checksum_enabled;
        u8 enabled;
    } ip;

    struct {
        u8 enabled;
        u8 checksum_enabled;
        u8 transaction_timer_enabled;
        u8 server_delay_enabled;
        u8 connection_state_enabled;
    } tcp;

    struct {
        u8 enabled;
    } h264;

    struct {
        u8 enabled;
    } amr;

    struct {
        u8 enabled;
    } rtp;

    struct {
        u8 enabled;
    } id3;

    struct {
        u8 enabled;
    } mp3;

    struct {
        u8 enabled;
    } http;

    struct {
        u8 enabled;
    } mp4;

    struct {
        u8 enabled;
    } rtcp;

    struct {
        u8 enabled;
    } h263;

    struct {
        u8 enabled;
    } evs;

    struct {
        u8 enabled;
    } zrtp;

    struct {
        u8 enabled;
    } stun;

    struct {
        u8 enabled;
    } icmp;

    struct {
        u8 enabled;
    } quic;

    enum throughput_resolution ip_throughput_resolution;

    u8 rtp_performance_enabled;
};


#ifdef __cplusplus
}
#endif
#endif /* __IPQ_METADATA_PUBLIC_H__ */
