/** @file      ipq_timeorderedhash.h
  * @brief     PACE timeordered hashtable definition.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef __IPOQUE_API_INCLUDE_FILE__
#error CANNOT INCLUDE THIS .H FILE, INCLUDE IPQ_API.H
#endif

#ifndef __IPQ_TIMEORDEREDHASH_H__
#define __IPQ_TIMEORDEREDHASH_H__

#ifdef __cplusplus
extern "C" {
#endif

	/* obsolete define, this feature is always available. This defines is only here to not break old code */
#define IPOQUE_TOH_MULTI_TIMEOUT

#if defined(clang) || (__GNUC__ > 4 || (__GNUC__ == 4 && (__GNUC_MINOR__ > 4)))
#define IPOQUE_DEPRECATE_TOH                                                                                           \
    __attribute__((deprecated(                                                                                         \
        "The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.")))
#else
#define IPOQUE_DEPRECATE_TOH __attribute__((deprecated()))
#endif

/** basic 5 tuple which is used for connection tracking
 */

	typedef struct ipoque_unique_flow_struct {
		u32 lower_ip;
					/**< the lower ip address of the two ip addresses */
		u32 upper_ip;
					/**< the upper ip address of the two ip addresses */
		u16 lower_port;
					/**< the port of the lower ip address */
		u16 upper_port;
					/**< the port of the upper ip address */
		u32 protocol;
					/**< the protocol, for alignment reason, protocol is 32 bit, not 8 bit */
	} ipoque_unique_flow_struct_t;

/** macro to compare 2 IPv6 addresses with each other to identify the "smaller" IPv6 address  */
#if defined(__ANDROID__)  || defined(__arm__) && !defined(__LP64__)

#define __get_unaligned_2(__p)                                  \
        (__p[0] | __p[1] << 8)

#define __get_unaligned_4(__p)                                  \
        (__p[0] | __p[1] << 8 | __p[2] << 16 | __p[3] << 24)

#define get_unaligned(ptr)                                      \
        ({                                                      \
                __typeof__(*(ptr)) __v;                         \
                u8 *__p = (u8 *)(ptr);                          \
                switch (sizeof(*(ptr))) {                       \
                case 1: __v = *(ptr);                   break;  \
                case 2: __v = __get_unaligned_2(__p);   break;  \
                case 4: __v = __get_unaligned_4(__p);   break;  \
                case 8: {                                       \
                                unsigned int __v1, __v2;        \
                                __v2 = __get_unaligned_4((__p+4)); \
                                __v1 = __get_unaligned_4(__p);  \
                                __v = ((unsigned long long)__v2 << 32 | __v1);  \
                        }                                       \
                        break;                                  \
                default: break;  \
                }                                               \
                __v;                                            \
        })




#undef IPOQUE_COMPARE_IPV6_ADDRESS_STRUCTS
#define IPOQUE_COMPARE_IPV6_ADDRESS_STRUCTS(x,y)  \
     (  get_unaligned((uint64_t*)(x)) < get_unaligned((uint64_t*)(y)) ||        \
            (get_unaligned((uint64_t*)x) == get_unaligned((uint64_t*)y) &&          \
                    get_unaligned( ((uint64_t*)x)+1) < get_unaligned( ((uint64_t*)y)+1 )) )

#else
    #undef IPOQUE_COMPARE_IPV6_ADDRESS_STRUCTS
    #define IPOQUE_COMPARE_IPV6_ADDRESS_STRUCTS(x,y)  \
      ((((u64 *)(x))[0]) < (    ((u64 *)(y)) [0]) || ( (((u64 *)(x))[0]) == (((u64 *)(y))[0]) && (((u64 *)(x))[1]) < (((u64 *)(y))[1])) )
#endif

/** definition for the structure of the timeorderedhash table
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 */
typedef IPOQUE_DEPRECATE_TOH struct timeorderedhash timeorderedhash_t;

/** definition of the infinite timeout */
#define IPQ_TOH_TIMEOUT_INFINITE	0xFFFFFFFF



/** Time ordered hash create function
 * This function creates a time ordered hash. The Hash table will be allocated by this call. As long as no hash resize is allowed, the hash table allocates the memory only by this call.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param memory the allocated amount of bytes which can be used by the hash create function. It is possible that a small amount more bytes are allocated.
 * To determine the amount of memory needed for a certain number of elements use ipoque_to_calculate_hash_size.
 * When the memory given is below the size of an element the allocation will fail and the function returns NULL.
 * To remove the hash table, ipoque_to_hash_destroy() has to be called.
 * @param UserBufferSize size of data which is stored per key. May be 0 if only the existence of an entry is of interest.
 * @param UniqueBufferSize size of the key, must be a multiple of 4
 * @param timeout the timeout for every element, can be IPQ_TOH_TIMEOUT_INFINITE for infinite timeouts
 * @param timeout_callback callback function. Function will be called for every element removed from the hash table because of timeout, overflow, or explicit removal.
 *        The parameters are the last access time of the element, a pointer to the key, the data and a global user pointer. Can be NULL
 * @param timeout_userptr The userptr which is passed to the callback function
 * @param ipq_malloc Allocator function, including the allocation user-pointer, not NULL
 * @param ipq_free Freeing function with the allocation user-pinter, not NULL
 * @param allocation_userptr user pointer for the allocation and free function.
 * @return The initialized pointer to the hash table. It returns NULL for an error ( for example a failed memory allocation )
 */
IPOQUE_DEPRECATE_TOH
	struct timeorderedhash *ipoque_to_hash_create2(unsigned long memory,
												   u32 UserBufferSize,
												   u32 UniqueBufferSize,
												   IPOQUE_TIMESTAMP_COUNTER_SIZE timeout, void (*timeout_callback)
												    (IPOQUE_TIMESTAMP_COUNTER_SIZE lasttime, u8 * unique_buffer,
													 u8 * user_data, void *userptr), void *timeout_userptr,
												   void *(*ipq_malloc) (unsigned long size, void *userptr),
												   void (*ipq_free) (void *ptr, void *userptr),
												   void *allocation_userptr);

    /** Time ordered hash create function
     * This function creates a time ordered hash. The Hash table will be allocated by this call. As long as no hash resize is allowed, the hash table allocates the memory only by this call.
     * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
     * @param memory the allocated amount of bytes which can be used by the hash create function. It is possible that a small amount more bytes are allocated.
     * To determine the amount of memory needed for a certain number of elements use ipoque_to_calculate_hash_size.
     * When the memory given is below the size of an element the allocation will fail and the function returns NULL.
     * To remove the hash table, ipoque_to_hash_destroy() has to be called.
     * @param UserBufferSize size of data which is stored per key. May be 0 if only the existence of an entry is of interest.
     * @param UniqueBufferSize size of the key, must be a multiple of 4
     * @param timeout the timeout for every element, can be IPQ_TOH_TIMEOUT_INFINITE for infinite timeouts
     * @param timeout_callback callback function. Function will be called for every element removed from the hash table because of timeout, overflow, or explicit removal.
     *        The parameters are the last access time of the element, a pointer to the key, the data and a global user pointer. Can be NULL
     * @param timeout_userptr The userptr which is passed to the callback function
     * @param pace2_malloc Allocator function, including the allocation user-pointer, not NULL
     * @param pace2_free Freeing function with the allocation user-pinter, not NULL
     * @param thread_ID thread ID passed to the allocation and free functions
     * @param allocation_userptr user pointer for the allocation and free function.
     * @return The initialized pointer to the hash table. It returns NULL for an error ( for example a failed memory allocation )
     */
IPOQUE_DEPRECATE_TOH
    struct timeorderedhash *pace2_toh_create(unsigned long memory,
                                             u32 UserBufferSize,
                                             u32 UniqueBufferSize,
                                             IPOQUE_TIMESTAMP_COUNTER_SIZE timeout,
                                             void (*timeout_callback)
                                             (IPOQUE_TIMESTAMP_COUNTER_SIZE lasttime, u8 * unique_buffer,
                                              u8 * user_data, void *userptr),
                                             void *timeout_userptr,
                                             pace2_malloc_t pace2_malloc,
                                             pace2_free_t pace2_free,
                                             int thread_ID,
                                             void *allocation_userptr);

/**
 * hash table lookup
 * @param toh the pointer to the hash table, not NULL
 * @param unique the pointer to the key, not NULL
 * @param last_access_ts a pointer to a variable where the successful lookup should store the last insert call for this element, not NULL
 * @return returns the pointer to the user data, NULL if the lookup has failed
 */
	void *ipoque_to_hash_lookup_ts(struct timeorderedhash *toh, const u8 * unique,
								   IPOQUE_TIMESTAMP_COUNTER_SIZE * last_access_ts);

/**
 * hash table lookup, wrapper for lookups without using the access time stamp
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the pointer to the hash table, not NULL
 * @param unique the pointer to the key, not NULL
 * @return returns the pointer to the user data, NULL if the lookup has failed
 */

IPOQUE_DEPRECATE_TOH
	static inline void *ipoque_to_hash_lookup(struct timeorderedhash *toh, const u8 * unique) {
		IPOQUE_TIMESTAMP_COUNTER_SIZE ts;
		 return ipoque_to_hash_lookup_ts(toh, unique, &ts);
	}

/**
 * update timestamp for an element retrieved by a lookup call
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the pointer to the hash table, not NULL
 * @param user_data pointer to user data
 * @return 0 on success, !0 on error
 */
IPOQUE_DEPRECATE_TOH u32 ipoque_to_hash_update_ts_of_element(struct timeorderedhash *toh, void *user_data);

/**
 * hash table insert/update operation
 *
 * In default insertion mode this operation cannot fail because if the
 * hash table is full, the oldest element is deleted.
 *
 * In case of an insert of an already inserted element its timestamp
 * will get updated with value of the last call of
 * ipoque_to_hash_set_timestamp().
 *
 * The call may return NULL if the insertion mode is set to
 * INSERT_FAILS_IF_FULL and not space is available in the hash table
 * (and the optional resize fails).
 *
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the pointer to the hash table, not NULL
 * @param unique the pointer to the key, not NULL
 * @param new_entry pointer to an u8, or ignored if NULL. This will be set to != 0 when the insert has created a new entry in the hash table.
 * @return returns the pointer to the user data.
 * @see ipoque_to_hash_set_insertion_mode
 */
IPOQUE_DEPRECATE_TOH void *ipoque_to_hash_insert(struct timeorderedhash *toh, const u8 * unique, u8 * new_entry);

/** removes one element from the hash table. If a callback is initialized, it will be called during this call.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the pointer to the hash table, not NULL
 * @param unique the pointer to the key, not NULL
 */
IPOQUE_DEPRECATE_TOH void ipoque_to_hash_delete(struct timeorderedhash *toh, const u8 * unique);



/** frees all memory which has been allocated for a hash table
 * It does NOT clean up all elements, please use ipoque_to_hash_clear if required
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the pointer to the hash table, can be NULL
 * @see ipoque_to_hash_clear
 */
IPOQUE_DEPRECATE_TOH void ipoque_to_hash_destroy2(struct timeorderedhash *toh);


/** set the time stamp of the hash table. It should be called before any insert operation with the timestamp of the inserted element.
 * Be aware that the time stamp MUST grow monotonic. The time stamp can wrap around.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @param timestamp The current time stamp of the hash table.
 * @see ipoque_to_hash_clear
 */
IPOQUE_DEPRECATE_TOH
	void ipoque_to_hash_set_timestamp(struct timeorderedhash *toh, IPOQUE_TIMESTAMP_COUNTER_SIZE timestamp);

/** set or change the timeout of a timeout queue
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, NULL returns
 * @param timeout The current time stamp of the hash table. Can be IPQ_TOH_TIMEOUT_INFINITE for infinite timeouts.
 * @param queue the affected queue. Multi queues must be enabled if the queue is not 0
 * @return returns 0 for success or not 0 for an error
 */
	u8 ipoque_to_hash_set_timeout_ext(struct timeorderedhash *toh, IPOQUE_TIMESTAMP_COUNTER_SIZE timeout, u32 queue);

/** get the timestamp of a timeout queue
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, NULL returns 0
 * @param queue the affected queue. Multi queues must be enabled if the queue is not 0
 * @return returns the timeout.
 */
	IPOQUE_TIMESTAMP_COUNTER_SIZE ipoque_to_hash_get_timeout_ext(struct timeorderedhash *toh, u32 queue);

/** set or change the timeout of the default timeout queue
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, NULL returns 0
 * @param timestamp The current time stamp of the hash table. Can be IPQ_TOH_TIMEOUT_INFINITE for infinite timeouts.
 * @return returns 0 for success or not 0 for an error
 */
IPOQUE_DEPRECATE_TOH
	static inline u8 ipoque_to_hash_set_timeout(struct timeorderedhash *toh, IPOQUE_TIMESTAMP_COUNTER_SIZE timestamp) {
		return ipoque_to_hash_set_timeout_ext(toh, timestamp, 0);
	}

/** get the timestamp of the default timeout queue
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, NULL returns 0
 * @return returns the timeout.
 */
IPOQUE_DEPRECATE_TOH
	static inline IPOQUE_TIMESTAMP_COUNTER_SIZE ipoque_to_hash_get_timeout(struct timeorderedhash *toh) {
		return ipoque_to_hash_get_timeout_ext(toh, 0);
	}


/** set the callback function called for removed elements due to timeout, overflow, or explicit removal.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @param timeout_callback callback function. The parameters are the last access time of the element, a pointer to the key, the data and a global user pointer. Can be NULL
 * @param userptr The user defined pointer which is passed to the callback function
 */
IPOQUE_DEPRECATE_TOH
	void ipoque_to_hash_set_timeout_callback(struct timeorderedhash *toh, void (*timeout_callback)
											  (IPOQUE_TIMESTAMP_COUNTER_SIZE
											   lasttime, u8 * unique_buffer, u8 * user_data, void *userptr),
											 void *userptr);


/** enum listing reasons why an item was removed from the toh
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 */
	typedef enum toh_removal_reason {
		TOH_DELETED,			/**< item was removed due to requested delete */
		TOH_TIMEDOUT,			/**< item was removed due to timeout */
		TOH_OVERFLOWED			/**< item was removed as oldest entry of a full toh */
	} toh_removal_reason_t;

/** get the reason for the last remove operation. It can be a timeout, a delete or an hash table overflow. The result is undefined if called before any element has been removed.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @return returns the timeout reason
 */
IPOQUE_DEPRECATE_TOH enum toh_removal_reason ipoque_to_hash_get_last_removal_reason(struct timeorderedhash *toh);

/** Delete all entries from the hash table. If a callback is given, it will be called for every element.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 */
IPOQUE_DEPRECATE_TOH void ipoque_to_hash_clear(struct timeorderedhash *toh);

/** dump the hash table for each inserted element
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @param callback callback function this dump. It is called for each element. If callback return != 0, the element will be deleted (nothing happens otherwise), not NULL
 * @param user_data A user defined pointer that is passed to the callback function
 */
IPOQUE_DEPRECATE_TOH
	void ipoque_to_hash_foreach(struct timeorderedhash *toh,
								u8(*callback) (u8 * unique_buffer,
											   u8 * user_buffer,
											   IPOQUE_TIMESTAMP_COUNTER_SIZE
											   last_timestamp, void *user_data), void *user_data);
/** Function to activate an automatic hash resize operation. This will automatically resize the hash table when the hash table is full.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @param ipq_realloc unused, deprecated
 * @see ipoque_to_hash_create2
 */
IPOQUE_DEPRECATE_TOH
	void ipoque_to_hash_enable_hash_resize(struct timeorderedhash *toh,
										   void *(*ipq_realloc) (void *ptr, unsigned long size, void *userptr));

/** If hash resize is enabled, this function ensures that the given
 * number of elements can be inserted. The hash table will be resized
 * if necessary or, if the resize fails, the oldest elements are
 * removed until enough space is available.
 *
 * If hash resize is disabled, this function does nothing.
 *
 * Since reallocations might move the memory area used for the user
 * data, pointers might be invalidated during multiple inserts.  For
 * example, consider a function which does two inserts into a hash
 * table and uses both pointer afterwards. In this case, this function
 * must be called before the two inserts with an argument of "2".
 *
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @param min_free_elements number of elements which must be available before the next resize operation
 */
IPOQUE_DEPRECATE_TOH void ipoque_to_hash_check_hash_resize(struct timeorderedhash *toh, u32 min_free_elements);

/**
 * this function does an insert operation on a hash table with the 5 tuple without tunnel decapsulation.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the hash table
 * @param iphdr pointer to the ipv4 header
 * @param l3len accessible bytes from the iphdr
 * @return returns 0 for success and != 0 if an error happened
 */
IPOQUE_DEPRECATE_TOH void *ipoque_get_current_flow(struct timeorderedhash *toh, const void *iphdr, u16 l3len);

/** helper function to calculate the hash size by a known number of elements and buffer sizes
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param number_of_elements number of elements
 * @param user_buffer_size size of the key element
 * @param unique_buffer_size size of the data per key element
 * @return returns the amount of required memory
 */
IPOQUE_DEPRECATE_TOH
    unsigned long ipoque_to_calculate_hash_size(u32 number_of_elements, u32 user_buffer_size, u32 unique_buffer_size);


/** enables multi timeout handling, this function must be called before any insert operation !
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the hash table, NULL allowed
 * @param queues the number of used queues, must be >= 1. If > 1, the hash table must have been created with ipoque_to_hash_create2(...)
 * @return returns 0 for success, != 0 for error.
 */
IPOQUE_DEPRECATE_TOH u8 ipoque_to_hash_enable_multiple_timeout(struct timeorderedhash *toh, u32 queues);


/**
 * Get the queue of the last insert operation
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the hash table, NULL allowed
 * @return returns the queue of the last insert operation ( NOT lookup)
 * */
IPOQUE_DEPRECATE_TOH u32 ipoque_to_hash_get_timeout_queue(struct timeorderedhash *toh);

/** sets the queue of the last insert operation ( NOT lookup),
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the hash table, NULL allowed
 * @param queue new queue for this element
 * @return returns 0 for success, != 0 for failure */
IPOQUE_DEPRECATE_TOH u8 ipoque_to_hash_set_timeout_queue(struct timeorderedhash *toh, u32 queue);

/** Old hash table dump function.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 */
IPOQUE_DEPRECATE_TOH
	u32 ipoque_to_hash_dump(struct timeorderedhash *, u32, IPOQUE_TIMESTAMP_COUNTER_SIZE **, u8 **, u8 **);

/** memory boundary checks for the user memory. This function can be called after an insert operation to check whether the code has written more bytes than allowed.
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh the hash table
 */
#ifdef IPQ_MEMCHECK_HASH_USER_BUFFER
IPOQUE_DEPRECATE_TOH void toh_debug_check_last_accessed_memory(struct timeorderedhash *toh);
#else
IPOQUE_DEPRECATE_TOH
    static inline void toh_debug_check_last_accessed_memory(struct timeorderedhash *toh __attribute__ ((unused)))
    {}
#endif

/** @deprecated Deprecated function, automatic hash table health check is done already */
IPOQUE_DEPRECATE_TOH
	static inline void ipoque_to_hash_check_fill_percentage(struct timeorderedhash *toh
															__attribute__ ((unused)), u8 percentage
															__attribute__ ((unused)))
    {}

/** Deprecated macro, do not use anymore */
#define TOH_FILL(HSize, UBSize, UsBSize) \
        (((HSize) / ((UBSize) + (UsBSize))) / 2)

/** Deprecated macro, do not use anymore */
#define TOH2TOH(HElements, DElements, UBSize, UsBSize) \
        ((HElements) * sizeof(u8*) + (DElements) * ((UBsize) + (UsBSize)))

/** Deprecated macro, do not use anymore */
#define TOH2TOH2(HSize, UBSize, UsBSize) \
        (((HSize) - ((UBSize) - (UsBSize)) * (TOH_FILL)) / sizeof(u8*))


/** returns the number of available elements for a hash table
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @return The number of available elements
 */
IPOQUE_DEPRECATE_TOH u32 ipoque_to_hash_number_of_elements(struct timeorderedhash *toh);

/** returns the number of used elements for a hash table
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @return The number of used elements
 */
IPOQUE_DEPRECATE_TOH u32 ipoque_to_hash_used_elements(struct timeorderedhash *toh);

	/**
	 * returns the maximum number of elements used in the hash table
     * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
	 * @param toh The pointer to the hash table, not NULL
	 * @return The number of maximum used elements
	 */
IPOQUE_DEPRECATE_TOH u32 ipoque_to_hash_maximum_number_of_used_elements(struct timeorderedhash *toh);


#define IPOQUE_TOH_QUEUE_LEN(toh) ipoque_to_hash_used_elements(toh)
#define IPOQUE_TOH_ELEMENTS(toh) ipoque_to_hash_number_of_elements(toh)

/** returns the raw insert pointer from the last insert operation. This can be used to retrieve the last accessed unique and the last accessed user data pointer
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @return the raw insert pointer
 */
IPOQUE_DEPRECATE_TOH void *ipoque_to_hash_get_last_raw_insert_pointer(struct timeorderedhash *toh);

#define IPOQUE_TOH_GET_NEWEST_FLOW_ID(toh) ipoque_to_hash_get_last_raw_insert_pointer(toh)


/** returns the pointer to the unique data from a raw pointer
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @param ptr the pointer from e.g. ipoque_to_hash_get_last_raw_insert_pointer, not NULL
 * @return pointer to the unique data
 */

IPOQUE_DEPRECATE_TOH void *ipoque_to_hash_raw_to_unique(struct timeorderedhash *toh, void *ptr);

/** returns the pointer to the user data from a raw pointer
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @param ptr the pointer from e.g. ipoque_to_hash_get_last_raw_insert_pointer, not NULL
 * @return pointer to the user data
 */
IPOQUE_DEPRECATE_TOH void *ipoque_to_hash_raw_to_user(struct timeorderedhash *toh, void *ptr);

/** @deprecated Deprecated macro.
                Instead, use the function @ref ipoque_to_hash_raw_to_unique. */
#define IPOQUE_TOH_FLOW_ID_TO_UNIQUE(toh,ptr) ipoque_to_hash_raw_to_unique(toh,ptr)

/** @deprecated Deprecated macro.
                Instead, use the function @ref ipoque_to_hash_raw_to_user. */
#define IPOQUE_TOH_FLOW_ID_TO_USER(toh,ptr) ipoque_to_hash_raw_to_user(toh,ptr)

/** returns the size of the key element as initialized from ipoque_to_hash_create2
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @return the key element size
 * @see ipoque_to_hash_create2
 */
IPOQUE_DEPRECATE_TOH u32 ipoque_to_hash_get_unique_buffer_size(struct timeorderedhash *toh);

/** returns the size of the user data as initialized from ipoque_to_hash_create2
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, not NULL
 * @return the user data size per element
 * @see ipoque_to_hash_create2
 */
IPOQUE_DEPRECATE_TOH u32 ipoque_to_hash_get_user_buffer_size(struct timeorderedhash *toh);

/** set the userptr for the timeout callback
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, just returns when toh is NULL
 * @param new_value The new userptr, NULL allowed
 */
IPOQUE_DEPRECATE_TOH void ipoque_to_hash_set_timeout_userptr(struct timeorderedhash *toh, void *new_value);

/** returns the userptr for the timeout callback
 * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
 * @param toh The pointer to the hash table, just returns when toh is NULL
 * @return the current userptr
 */
IPOQUE_DEPRECATE_TOH void *ipoque_to_hash_get_timeout_userptr(struct timeorderedhash *toh);

	/**
	 * this function returns the timestamp of the oldest entry in the hash table.
	 *
     * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
	 * @param toh The pointer to the hash table
	 * @return the timestamp of the oldest entry or 0 if there is no entry.
	 */
IPOQUE_DEPRECATE_TOH
	IPOQUE_TIMESTAMP_COUNTER_SIZE ipoque_to_hash_get_timestamp_of_oldest_entry(struct timeorderedhash *toh);


	/**
	 * this function returns a pointer to the unique key for the given user data pointer
	 *
     * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
	 * @param toh The pointer to the initialized hash table
	 * @param user The pointer to the user data as returned by insert/lookup. user must be a valid element,
	               the pointer will not be validated.
	 * @return the pointer to the unique key
	 */
IPOQUE_DEPRECATE_TOH
	const void *ipoque_to_hash_get_unique_key_from_user_data(struct timeorderedhash *toh, const void *user);

    /**
     * this function returns the last timestamp update for the table entry.
     *
     * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
     * @param toh The pointer to the hash table
     * @param user The pointer to the user data as returned by insert/lookup. user must be a valid element,
                   the pointer will not be validated.
     * @return last update timestamp
     */
IPOQUE_DEPRECATE_TOH
    IPOQUE_TIMESTAMP_COUNTER_SIZE ipoque_to_hash_get_last_timestamp_for_user_data(struct timeorderedhash *toh, const void *user);


	enum toh_insertion_mode {
		/** if the hash table is full and cannot be resized to store
			more entries, the oldest element is removed to make space
			for the insert. This is the default behavior/ */
		REMOVE_OLDEST_IF_FULL = 0,

		/** An insert operation may fail if not space is available,
			that means old entries are only removed in case of
			timeouts */
		INSERT_FAILS_IF_FULL = 1
	};

	/**
	 * this function sets the mode of the insert operation. By
	 * default, inserts are always successful and will remove the
	 * oldest entry if not enough space is available in the hash
	 * table.  When setting the mode to INSERT_FAILS_IF_FULL, an
	 * insert will fail, no old elements are removed * (except when
	 * timeouts occur)
	 *
     * @deprecated The PACE TOH API has been deprecated and will be removed. Please use the PACE 2 PHT API instead.
	 * @param toh The pointer to the initialized hash table.
	 * @param mode the insertion mode, either REMOVE_OLDEST_IF_FULL or INSERT_FAILS_IF_FULL.
	 */
IPOQUE_DEPRECATE_TOH
	void ipoque_to_hash_set_insertion_mode(struct timeorderedhash *toh,
										   enum toh_insertion_mode mode);

#undef IPOQUE_DEPRECATE_TOH

#ifdef __cplusplus
}
#endif
#endif /* __IPQ_TIMEORDEREDHASH_H__ */
