/** @file      ipq_wrapper.h
  * @brief     PACE wrapper definition.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef __IPOQUE_API_INCLUDE_FILE__
#error CANNOT INCLUDE THIS .H FILE, INCLUDE IPQ_API.H
#endif

#ifndef IPQ_WRAPPER_H
#define IPQ_WRAPPER_H

#ifdef __cplusplus
extern "C" {
#endif

typedef void *(*pace_malloc_t)( unsigned long size );
typedef void (*pace_free_t)( void *ptr );

typedef void *(*pace_ext_malloc_t)( unsigned long size,
                                    void *user_ptr );
typedef void (*pace_ext_free_t)( void *ptr,
                                 void *user_ptr );

typedef void *(*pace_realloc_t)( void *ptr, unsigned long size,
                                 void *memory_userptr );

/**
 * @brief Wrapper function for memory allocation requests.
 * @param[in] size number of byte the requested memory shall hold.
 * @param[in] thread_ID index of the thread requesting memory. May be used for in NUMA setups, to allow the
 *                      application to provide different memory areas depending on the thread that requests memory.
 * @param[in] user_ptr holds the pointer to user data specified in the config as memory_callback_user_ptr.
 * @param[in] scope reserved for future use.
 * @return Pointer to memory of at least size bytes or NULL for error or out-of-memory.
 */
typedef void *(*pace2_malloc_t)( u64 size,
                                 int thread_ID,
                                 void *user_ptr,
                                 int scope );

/**
 * @brief Wrapper function for requests to free allocated memory.
 * @param[in] ptr pointer to the memory area that should be freed
 * @param[in] thread_ID index of the thread requesting memory. May be used for in NUMA setups, to allow the
 *                      application to provide different memory areas depending on the thread that requests memory.
 * @param[in] user_ptr holds the pointer to user data specified in the config as memory_callback_user_ptr.
 * @param[in] scope reserved for future use.
 */
typedef void (*pace2_free_t)( void *ptr,
                              int thread_ID,
                              void *user_ptr,
                              int scope );

/**
 * @brief Wrapper function for memory resizing requests.
 * @param[in] ptr pointer to the memory area that should be resized
 * @param[in] size number of byte the requested memory shall hold.
 * @param[in] thread_ID index of the thread requesting memory. May be used for in NUMA setups, to allow the
 *                      application to provide different memory areas depending on the thread that requests memory.
 * @param[in] user_ptr holds the pointer to user data specified in the config as memory_callback_user_ptr.
 * @param[in] scope reserved for future use.
 * @return Pointer to memory of at least size bytes or NULL for error or out-of-memory.
 */
typedef void *(*pace2_realloc_t)( void *ptr,
                                  u64 size,
                                  int thread_ID,
                                  void *user_ptr,
                                  int scope );

typedef void *(*pace2_dlopen_t)( const char *filename, int flag, void *userptr );
typedef void *(*pace2_dlsym_t)( void *handle, const char *symbol, void *userptr );
typedef int (*pace2_dlclose_t)( void *handle, void *userptr );

#ifdef __cplusplus
}
#endif
#endif /* IPQ_WRAPPER_H */
