/** @file      pace2.h
  * @brief     PACE 2 general include file.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef IPOQUE_PACE2_H
#define IPOQUE_PACE2_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes *********************************************************************/
/* PACE specific includes */
#include <ipq_api.h>

/* Generic PACE2 includes */
#include "pace2_version.h"
#include "pace2_basic_def.h"
#include "pace2_packet_stack.h"
#include "pace2_event_bitmask.h"
#include "pace2_event_policy.h"
#include "pace2_frame_buffer.h"
#include "pace2_config.h"
#include "pace2_config_dict.h"
#include "pace2_classification_results.h"
#include "pace2_dynamic_upgrade.h"
#include "pace2_pa_tags.h"
#include "pace2_pa_statuses.h"
#include "pace2_application_categories.h"

/* Stage processing specific includes */
#include "pace2_s1_preparing.h"
#include "pace2_s2_reordering.h"
#include "pace2_s3_classification.h"
#include "pace2_s4_decoding.h"
#include "pace2_s5_timeout.h"
#include "pace2_processing_states.h"

/* Machine Learning specific includes */
#include "pace2_ml.h"
#include "pace2_ml_class.h"

/* Event specific includes */
#include "pace2_event_types.h"
#include "pace2_events.h"
#include "pace2_status_csi.h"
#include "pace2_status_classification.h"
#include "pace2_status_dhcp.h"
#include "pace2_status_dns.h"
#include "pace2_status_fastpath.h"
#include "pace2_status_rtcp.h"
#include "pace2_status_rtp.h"
#include "pace2_status_sip.h"
#include "pace2_status_sit.h"
#include "pace2_status_tcp.h"
#include "pace2_serialization.h"

/* CPI specific includes */
#include "pace2_cpi_classification.h"
#include "pace2_cpi_preparing.h"

/* hash table specific include */
#include "pace2_pht.h"

#include "pace2_generic_func.h"
#include "pace2_debug.h"
#include "pace2_plugins.h"
#include "pace2_pif.h"

#ifndef PACE_KERNEL_LIB

/** @deprecated Deprecated. Do not use. */
#define IPOQUE_DECODER_VERSION IPOQUE_PACE_VERSION
/** @deprecated Deprecated. Do not use. */
#define IPOQUE_DECODER_VERSION_NUMBER IPOQUE_PACE_VERSION_NUMBER

/* The following headers are not necessarily needed by PACE 2 but currently still kept for compatibility. */
#include "ipd_postproc_protocols.h"

#endif /* PACE_KERNEL_LIB */

#ifdef __cplusplus
}
#endif
#endif /* IPOQUE_PACE2_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
