/** @file      pace2_attributes.h
  * @brief     PACE 2 attributes definitions
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_ATTRIBUTES_H
#define PACE2_ATTRIBUTES_H

/* Includes *********************************************************************/
#include "pace2_basic_def.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public Types *****************************************************************/

/**
 * @brief Enum defining all available application attributes.
**/
typedef enum pace2_application_attributes {
    PACE2_APPLICATION_ATTRIBUTE_NONE = 0,
    PACE2_APPLICATION_ATTRIBUTE_ENCRYPTED,
    PACE2_APPLICATION_ATTRIBUTE_AUDIO,
    PACE2_APPLICATION_ATTRIBUTE_LANDLINECALL,
    PACE2_APPLICATION_ATTRIBUTE_VIDEO,
    PACE2_APPLICATION_ATTRIBUTE_FILE_TRANSFER,
    PACE2_APPLICATION_ATTRIBUTE_WEB,
    PACE2_APPLICATION_ATTRIBUTE_CHAT,
    PACE2_APPLICATION_ATTRIBUTE_MAIL,
    PACE2_APPLICATION_ATTRIBUTE_STREAM,
    PACE2_APPLICATION_ATTRIBUTE_OS_ANDROID,
    PACE2_APPLICATION_ATTRIBUTE_OS_IOS,
    PACE2_APPLICATION_ATTRIBUTE_OS_WINDOWS_MOBILE,
    PACE2_APPLICATION_ATTRIBUTE_OS_BLACKBERRY,
    PACE2_APPLICATION_ATTRIBUTE_PICTURE,
    PACE2_APPLICATION_ATTRIBUTE_DDL,
    PACE2_APPLICATION_ATTRIBUTE_GOOGLE,
    PACE2_APPLICATION_ATTRIBUTE_MSEXCHANGE_OUTLOOK_WEB_ACCESS,
    PACE2_APPLICATION_ATTRIBUTE_AMAZON_CLOUD,
    PACE2_APPLICATION_ATTRIBUTE_APACHE,
    PACE2_APPLICATION_ATTRIBUTE_MYSQL_SERVER,
    PACE2_APPLICATION_ATTRIBUTE_MARIADB_SERVER,
    PACE2_APPLICATION_ATTRIBUTE_NTLM,
    PACE2_APPLICATION_ATTRIBUTE_OS_WINDOWS,
    PACE2_APPLICATION_ATTRIBUTE_OS_CHROME,
    PACE2_APPLICATION_ATTRIBUTE_AKAMAI_CLOUD,
    PACE2_APPLICATION_ATTRIBUTE_DOX,
    PACE2_APPLICATION_ATTRIBUTE_RCS,
    PACE2_APPLICATION_ATTRIBUTE_CONTROL_FLOW,

    PACE2_APPLICATION_ATTRIBUTES_COUNT

} PACE2_application_attributes;

typedef PACE2_DEPRECATED("Application attribute is marked as deprecated and should not be used anymore.") PACE2_application_attributes PACE2_deprecated_application_attributes;


/**
 * @brief Struct representing all available application attributes as a bitmask.
 *
 * @deprecated This structure is deprecated. Please use the functionality provided by
 *             @ref pace2_get_number_of_application_attributes_of_application and
 *             @ref pace2_get_number_of_application_attributes_of_protocol and
 *             @ref pace2_get_application_attribute_of_application_by_index and
 *             @ref pace2_get_application_attribute_of_protocol_by_index.
**/
typedef struct {
#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 none:1;                 //!< none
    u32 encrypted:1;            //!< encrypted
    u32 audio:1;                //!< audio
    u32 out:1;                  //!< out
    u32 video:1;                //!< video
    u32 file_transfer:1;        //!< file-transfer
    u32 web:1;                  //!< web
    u32 chat:1;                 //!< chat
    u32 mail:1;                 //!< mail
    u32 stream:1;               //!< stream
    u32 android:1;              //!< android
    u32 ios:1;                  //!< ios
    u32 windows_mobile:1;       //!< windows-mobile
    u32 blackberry:1;           //!< blackberry
    u32 picture:1;              //!< picture
    u32 ddl:1;                  //!< ddl
    u32 google:1;               //!< google
    u32 outlook_web_access:1;   //!< outlook_web_access
    u32 amazon_cloud:1;         //!< amazon-cloud
    u32 apache:1;               //!< apache
    u32 mysql_server:1;         //!< mysql-server
    u32 mariadb_server:1;       //!< mariadb-server
    u32 ntlm:1;                 //!< ntlm
    u32 microsoft_windows:1;    //!< microsoft-windows
    u32 chrome:1;               //!< chrome
    u32 akamai_cloud:1;         //!< akamai-cloud
    u32 dox:1;                  //!< dox
    u32 rcs:1;                  //!< rcs
    u32 control_flow:1;         //!< control-flow
    u32 padding_1:(64 - 32 - PACE2_APPLICATION_ATTRIBUTES_COUNT);
    u32 padding_2:32;
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 padding_2:32;
    u32 padding_1:(64 - 32 - PACE2_APPLICATION_ATTRIBUTES_COUNT);
    u32 control_flow:1;         //!< control-flow
    u32 rcs:1;                  //!< rcs
    u32 dox:1;                  //!< dox
    u32 akamai_cloud:1;         //!< akamai-cloud
    u32 chrome:1;               //!< chrome
    u32 microsoft_windows:1;    //!< microsoft-windows
    u32 ntlm:1;                 //!< ntlm
    u32 mariadb_server:1;       //!< mariadb-server
    u32 mysql_server:1;         //!< mysql-server
    u32 apache:1;               //!< apache
    u32 amazon_cloud:1;         //!< amazon-cloud
    u32 outlook_web_access:1;   //!< outlook_web_access
    u32 google:1;               //!< google
    u32 ddl:1;                  //!< ddl
    u32 picture:1;              //!< picture
    u32 blackberry:1;           //!< blackberry
    u32 windows_mobile:1;       //!< windows-mobile
    u32 ios:1;                  //!< ios
    u32 android:1;              //!< android
    u32 stream:1;               //!< stream
    u32 mail:1;                 //!< mail
    u32 chat:1;                 //!< chat
    u32 web:1;                  //!< web
    u32 file_transfer:1;        //!< file-transfer
    u32 video:1;                //!< video
    u32 out:1;                  //!< out
    u32 audio:1;                //!< audio
    u32 encrypted:1;            //!< encrypted
    u32 none:1;                 //!< none
#endif

} PACE2_application_attributes_struct;

/**
 * List of strings, each identifying the appropriate application attribute.
 *
 * Items do not change between releases. Only for new attributes strings are added.
 */
#define PACE2_APPLICATION_ATTRIBUTES_STRINGS \
    "none",\
    "encrypted",\
    "audio",\
    "out",\
    "video",\
    "file-transfer",\
    "web",\
    "chat",\
    "mail",\
    "stream",\
    "android",\
    "ios",\
    "windows-mobile",\
    "blackberry",\
    "picture",\
    "ddl",\
    "google",\
    "outlook_web_access",\
    "amazon-cloud",\
    "apache",\
    "mysql-server",\
    "mariadb-server",\
    "ntlm",\
    "microsoft-windows",\
    "chrome",\
    "akamai-cloud",\
    "dox",\
    "rcs",\
    "control-flow"

/**
 * List of strings, each describing the appropriate application attribute.
 *
 * Items may be changed between releases.
 */
#define PACE2_APPLICATION_ATTRIBUTES_DESCRIPTION \
    "Application has no attribute.",\
    "Indicates that the current connection is encrypted traffic.",\
    "Indicates that the current connection is an audio or voice signal.",\
    "Indicates that the current connection is a landline call, e.g. a call to a home phone.",\
    "Indicates that the current connection is a video signal.",\
    "Indicates that the current connection is a file transfer.",\
    "Indicates that the current connection is a surf the Internet session.",\
    "Indicates that the current connection is a chat session.",\
    "Indicates that the current connection is mail traffic.",\
    "Indicates that the current connection is a continues unidirectional stream of audio and / or video.",\
    "Indicates that the client side uses the operating system Android.",\
    "Indicates that the client side uses the operating system iOS.",\
    "Indicates that the client side uses the operating system Windows Mobile.",\
    "Indicates that the client side uses the operating system Blackberry.",\
    "Indicates that the current connection transfers pictures.",\
    "Indicates that the current connection is a Direct Download Hoster.",\
    "Indicates that the current connection is a Google service.",\
    "Indicates that the current connection uses the Microsoft Exchange Outlook Web Access as authentication mechanism.",\
    "Indicates that the current connection is a service of Amazon Cloud.",\
    "Indicates that the server side is an Apache server.",\
    "Indicates that the server side is a MySQL database server.",\
    "Indicates that the server side is a MariaDB database server.",\
    "Current connection uses NTLM as authentication mechanism.",\
    "Indicates that the client side is the operating system Microsoft Windows.",\
    "Indicates that the client side is the operating system Chrome.",\
    "Indicates that the current connection is a service of Akamai Cloud.",\
    "Indicates that the current connection is DoT (DNS over TLS), DoH (DNS over HTTPS) or DoQ (DNS over QUIC).",\
    "Indicates that the current connection is RCS (Rich Communication Services).",\
    "Indicates that the current connection is used for timestamp correlation."

/**
 * @brief Enum defining all available protocol attributes.
**/
typedef enum pace2_protocol_attributes {
    PACE2_PROTOCOL_ATTRIBUTE_NONE = 0,
    PACE2_PROTOCOL_ATTRIBUTE_RES_1,
    PACE2_PROTOCOL_ATTRIBUTE_IPSEC_ISAKMP,
    PACE2_PROTOCOL_ATTRIBUTE_FTP_CONTROL,
    PACE2_PROTOCOL_ATTRIBUTE_FTP_DATA,
    PACE2_PROTOCOL_ATTRIBUTE_GTP_UP,
    PACE2_PROTOCOL_ATTRIBUTE_GTP_CP,
    PACE2_PROTOCOL_ATTRIBUTE_GTP_PRIME,
    PACE2_PROTOCOL_ATTRIBUTE_GTP_V1,
    PACE2_PROTOCOL_ATTRIBUTE_GTP_V2,
    PACE2_PROTOCOL_ATTRIBUTE_SOCKS_V4,
    PACE2_PROTOCOL_ATTRIBUTE_SOCKS_V5,
    PACE2_PROTOCOL_ATTRIBUTE_RES_12,
    PACE2_PROTOCOL_ATTRIBUTE_UTP,
    PACE2_PROTOCOL_ATTRIBUTE_DIRECTCONNECT_HUB,
    PACE2_PROTOCOL_ATTRIBUTE_DIRECTCONNECT_PEER,
    PACE2_PROTOCOL_ATTRIBUTE_DIRECTCONNECT_ADC_PEER,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_SPDY,
    PACE2_PROTOCOL_ATTRIBUTE_TELNET_TN3270,
    PACE2_PROTOCOL_ATTRIBUTE_TCP_WO_HANDSHAKE,
    PACE2_PROTOCOL_ATTRIBUTE_RES_20,
    PACE2_PROTOCOL_ATTRIBUTE_RES_21,
    PACE2_PROTOCOL_ATTRIBUTE_RES_22,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_HTTP2,
    PACE2_PROTOCOL_ATTRIBUTE_HTTP_CONNECT,
    PACE2_PROTOCOL_ATTRIBUTE_TUNNEL,
    PACE2_PROTOCOL_ATTRIBUTE_TLS_V1_3,
    PACE2_PROTOCOL_ATTRIBUTE_DFS_R,
    PACE2_PROTOCOL_ATTRIBUTE_RAS,
    PACE2_PROTOCOL_ATTRIBUTE_ESNI,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_HTTP,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_STUN,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_WEBRTC,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_FTP,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_IMAP,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_POP3,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_MANAGESIEVE,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_COAP,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_XMPP,
    PACE2_PROTOCOL_ATTRIBUTE_IANA_PORT_SMTPS,
    PACE2_PROTOCOL_ATTRIBUTE_IANA_PORT_IMAPS,
    PACE2_PROTOCOL_ATTRIBUTE_IANA_PORT_POP3S,
    PACE2_PROTOCOL_ATTRIBUTE_IANA_PORT_DOMAIN_S,
    PACE2_PROTOCOL_ATTRIBUTE_QUIC_GQUIC,
    PACE2_PROTOCOL_ATTRIBUTE_QUIC_IETF,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_HTTP3,
    PACE2_PROTOCOL_ATTRIBUTE_QUIC_MVFST,
    PACE2_PROTOCOL_ATTRIBUTE_TLS_V1_3_SESSION_RESUMPTION,
    PACE2_PROTOCOL_ATTRIBUTE_NETBIOS_NAME_SERVICE,
    PACE2_PROTOCOL_ATTRIBUTE_NETBIOS_DATAGRAM_SERVICE,
    PACE2_PROTOCOL_ATTRIBUTE_NETBIOS_SESSION_SERVICE,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_DOQ,
    PACE2_PROTOCOL_ATTRIBUTE_HLS,
    PACE2_PROTOCOL_ATTRIBUTE_DASH,
    PACE2_PROTOCOL_ATTRIBUTE_NDP,
    PACE2_PROTOCOL_ATTRIBUTE_PLAIN_ESP,
    PACE2_PROTOCOL_ATTRIBUTE_ENCRYPTED_ESP,
    PACE2_PROTOCOL_ATTRIBUTE_ANOMALOUS,
    PACE2_PROTOCOL_ATTRIBUTE_SMB_CIFS,
    PACE2_PROTOCOL_ATTRIBUTE_SMB_V2,
    PACE2_PROTOCOL_ATTRIBUTE_SMB_V3,
    PACE2_PROTOCOL_ATTRIBUTE_PROFINET_IO_CM,
    PACE2_PROTOCOL_ATTRIBUTE_CIP_CM,
    PACE2_PROTOCOL_ATTRIBUTE_CIP_IO,
    PACE2_PROTOCOL_ATTRIBUTE_PROFINET_DCP,
    PACE2_PROTOCOL_ATTRIBUTE_PROFINET_RT,
    PACE2_PROTOCOL_ATTRIBUTE_DHT,
    PACE2_PROTOCOL_ATTRIBUTE_ECH,
    PACE2_PROTOCOL_ATTRIBUTE_IANA_PORT_LDAPS,
    PACE2_PROTOCOL_ATTRIBUTE_RFB_V3_3,
    PACE2_PROTOCOL_ATTRIBUTE_RFB_V3_7,
    PACE2_PROTOCOL_ATTRIBUTE_RFB_V3_8,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V4_0,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V5_0_TO_8_1,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_0,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_1,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_2,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_3,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_4,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_5,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_6,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_7,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_8,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_9,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_10,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_V10_11,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_UDP_V1,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_UDP_V2,
    PACE2_PROTOCOL_ATTRIBUTE_RDP_UDP_V3,
    PACE2_PROTOCOL_ATTRIBUTE_MQTT_V3_1,
    PACE2_PROTOCOL_ATTRIBUTE_MQTT_V3_1_1,
    PACE2_PROTOCOL_ATTRIBUTE_MQTT_V5_0,
    PACE2_PROTOCOL_ATTRIBUTE_SNMP_V1,
    PACE2_PROTOCOL_ATTRIBUTE_SNMP_V2C,
    PACE2_PROTOCOL_ATTRIBUTE_SNMP_V2,
    PACE2_PROTOCOL_ATTRIBUTE_SNMP_V3,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_SMB,
    PACE2_PROTOCOL_ATTRIBUTE_ALPN_TDS,
    PACE2_PROTOCOL_ATTRIBUTE_QUIC_IETF_V2,

    PACE2_PROTOCOL_ATTRIBUTES_COUNT

} PACE2_protocol_attributes;

typedef PACE2_DEPRECATED("Protocol attribute is marked as deprecated and should not be used anymore.") PACE2_protocol_attributes PACE2_deprecated_protocol_attributes;
#define PACE2_PROTOCOL_ATTRIBUTE_IPSEC_PAYLOAD ((PACE2_deprecated_protocol_attributes) PACE2_PROTOCOL_ATTRIBUTE_RES_1)
#define PACE2_PROTOCOL_ATTRIBUTE_MEDIA ((PACE2_deprecated_protocol_attributes) PACE2_PROTOCOL_ATTRIBUTE_RES_12)
#define PACE2_PROTOCOL_ATTRIBUTE_SPDY ((PACE2_deprecated_protocol_attributes) PACE2_PROTOCOL_ATTRIBUTE_ALPN_SPDY)
#define PACE2_PROTOCOL_ATTRIBUTE_IEC61850_GOOSE ((PACE2_deprecated_protocol_attributes) PACE2_PROTOCOL_ATTRIBUTE_RES_20)
#define PACE2_PROTOCOL_ATTRIBUTE_IEC61850_MMS ((PACE2_deprecated_protocol_attributes) PACE2_PROTOCOL_ATTRIBUTE_RES_21)
#define PACE2_PROTOCOL_ATTRIBUTE_IEC61850_SV ((PACE2_deprecated_protocol_attributes) PACE2_PROTOCOL_ATTRIBUTE_RES_22)
#define PACE2_PROTOCOL_ATTRIBUTE_HTTP2 ((PACE2_deprecated_protocol_attributes) PACE2_PROTOCOL_ATTRIBUTE_ALPN_HTTP2)

/**
 * @brief Struct representing all available protocol attributes as a bitmask.
 *
 * @deprecated This structure is deprecated. Please use the functionality provided by
 *             @ref pace2_get_number_of_protocol_attributes_of_protocol and
 *             @ref pace2_get_protocol_attribute_of_protocol_by_index.
**/
typedef struct {
#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 none:1;             //!< none
    PACE2_DEPRECATED("Protocol attribute ipsec_payload is marked as deprecated and should not be used anymore.")
    u32 ipsec_payload:1;    //!< ipsec_payload
    u32 ipsec_isakmp:1;     //!< ipsec_isakmp
    u32 ftp_control:1;      //!< ftp_control
    u32 ftp_data:1;         //!< ftp_data
    u32 gtp_up:1;           //!< gtp_up
    u32 gtp_cp:1;           //!< gtp_cp
    u32 gtp_prime:1;        //!< gtp_prime
    u32 gtp_v1:1;           //!< gtp_v1
    u32 gtp_v2:1;           //!< gtp_v2
    u32 socks_v4:1;         //!< socks_v4
    u32 socks_v5:1;         //!< socks_v5
    PACE2_DEPRECATED("Protocol attribute media is marked as deprecated and should not be used anymore.")
    u32 media:1;            //!< media
    u32 utp:1;              //!< uTP
    u32 hub:1;              //!< hub
    u32 peer:1;             //!< peer
    u32 adc_peer:1;         //!< adc-peer
    u32 spdy:1;             //!< spdy
    u32 tn3270:1;           //!< tn3270
    u32 tcp_wo_hs:1;        //!< tcp_wo_hs
    PACE2_DEPRECATED("Protocol attribute iec-61850-goose is marked as deprecated and should not be used anymore.")
    u32 iec_61850_goose:1;  //!< iec-61850-goose
    PACE2_DEPRECATED("Protocol attribute iec-61850-mms is marked as deprecated and should not be used anymore.")
    u32 iec_61850_mms:1;    //!< iec-61850-mms
    PACE2_DEPRECATED("Protocol attribute iec-61850-smv is marked as deprecated and should not be used anymore.")
    u32 iec_61850_smv:1;    //!< iec-61850-smv
    u32 http2:1;            //!< http2
    u32 http_connect:1;     //!< http-connect
    u32 tunnel:1;           //!< tunnel
    u32 tls_v1_3:1;         //!< tls_v1_3
    u32 dfs_r:1;            //!< dfs-r
    u32 ras:1;              //!< ras
    u32 esni:1;             //!< esni
    u32 http:1;             //!< http
    u32 stun:1;             //!< stun
    u32 webrtc:1;           //!< webrtc
    u32 ftp:1;              //!< ftp
    u32 imap:1;             //!< imap
    u32 pop3:1;             //!< pop3
    u32 managesieve:1;      //!< managesieve
    u32 coap:1;             //!< coap
    u32 xmpp:1;             //!< xmpp
    u32 smtps:1;            //!< smtps
    u32 imaps:1;            //!< imaps
    u32 pop3s:1;            //!< pop3s
    u32 domain_s:1;         //!< domain-s
    u32 gquic:1;            //!< gquic
    u32 ietf:1;             //!< ietf
    u32 http3:1;            //!< http3
    u32 mvfst:1;            //!< mvfst
    u32 session_resumption:1;//!< session_resumption
    u32 name_service:1;     //!< name_service
    u32 datagram_service:1; //!< datagram_service
    u32 session_service:1;  //!< session_service
    u32 doq:1;              //!< doq
    u32 hls:1;              //!< hls
    u32 dash:1;             //!< dash
    u32 ndp:1;              //!< ndp
    u32 plain_esp:1;        //!< plain_esp
    u32 encrypted_esp:1;    //!< encrypted_esp
    u32 anomalous:1;        //!< anomalous
    u32 smb_cifs:1;         //!< smb-cifs
    u32 smb_v2:1;           //!< smb-v2
    u32 smb_v3:1;           //!< smb-v3
    u32 io_cm:1;            //!< io-cm
    u32 cip_cm:1;           //!< cip-cm
    u32 cip_io:1;           //!< cip-io
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 cip_io:1;           //!< cip-io
    u32 cip_cm:1;           //!< cip-cm
    u32 io_cm:1;            //!< io-cm
    u32 smb_v3:1;           //!< smb-v3
    u32 smb_v2:1;           //!< smb-v2
    u32 smb_cifs:1;         //!< smb-cifs
    u32 anomalous:1;        //!< anomalous
    u32 encrypted_esp:1;    //!< encrypted_esp
    u32 plain_esp:1;        //!< plain_esp
    u32 ndp:1;              //!< ndp
    u32 dash:1;             //!< dash
    u32 hls:1;              //!< hls
    u32 doq:1;              //!< doq
    u32 session_service:1;  //!< session_service
    u32 datagram_service:1; //!< datagram_service
    u32 name_service:1;     //!< name_service
    u32 session_resumption:1;//!< session_resumption
    u32 mvfst:1;            //!< mvfst
    u32 http3:1;            //!< http3
    u32 ietf:1;             //!< ietf
    u32 gquic:1;            //!< gquic
    u32 domain_s:1;         //!< domain-s
    u32 pop3s:1;            //!< pop3s
    u32 imaps:1;            //!< imaps
    u32 smtps:1;            //!< smtps
    u32 xmpp:1;             //!< xmpp
    u32 coap:1;             //!< coap
    u32 managesieve:1;      //!< managesieve
    u32 pop3:1;             //!< pop3
    u32 imap:1;             //!< imap
    u32 ftp:1;              //!< ftp
    u32 webrtc:1;           //!< webrtc
    u32 stun:1;             //!< stun
    u32 http:1;             //!< http
    u32 esni:1;             //!< esni
    u32 ras:1;              //!< ras
    u32 dfs_r:1;            //!< dfs-r
    u32 tls_v1_3:1;         //!< tls_v1_3
    u32 tunnel:1;           //!< tunnel
    u32 http_connect:1;     //!< http-connect
    u32 http2:1;            //!< http2
    PACE2_DEPRECATED("Protocol attribute iec-61850-smv is marked as deprecated and should not be used anymore.")
    u32 iec_61850_smv:1;    //!< iec-61850-smv
    PACE2_DEPRECATED("Protocol attribute iec-61850-mms is marked as deprecated and should not be used anymore.")
    u32 iec_61850_mms:1;    //!< iec-61850-mms
    PACE2_DEPRECATED("Protocol attribute iec-61850-goose is marked as deprecated and should not be used anymore.")
    u32 iec_61850_goose:1;  //!< iec-61850-goose
    u32 tcp_wo_hs:1;        //!< tcp_wo_hs
    u32 tn3270:1;           //!< tn3270
    u32 spdy:1;             //!< spdy
    u32 adc_peer:1;         //!< adc-peer
    u32 peer:1;             //!< peer
    u32 hub:1;              //!< hub
    u32 utp:1;              //!< uTP
    PACE2_DEPRECATED("Protocol attribute media is marked as deprecated and should not be used anymore.")
    u32 media:1;            //!< media
    u32 socks_v5:1;         //!< socks_v5
    u32 socks_v4:1;         //!< socks_v4
    u32 gtp_v2:1;           //!< gtp_v2
    u32 gtp_v1:1;           //!< gtp_v1
    u32 gtp_prime:1;        //!< gtp_prime
    u32 gtp_cp:1;           //!< gtp_cp
    u32 gtp_up:1;           //!< gtp_up
    u32 ftp_data:1;         //!< ftp_data
    u32 ftp_control:1;      //!< ftp_control
    u32 ipsec_isakmp:1;     //!< ipsec_isakmp
    PACE2_DEPRECATED("Protocol attribute ipsec_payload is marked as deprecated and should not be used anymore.")
    u32 ipsec_payload:1;    //!< ipsec_payload
    u32 none:1;             //!< none
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 dcp:1;              //!< dcp
    u32 rt:1;               //!< rt
    u32 dht:1;              //!< dht
    u32 ech:1;              //!< ech
    u32 ldaps:1;            //!< ldaps
    u32 rfb_v3_3:1;         //!< rfb_v3_3
    u32 rfb_v3_7:1;         //!< rfb_v3_7
    u32 rfb_v3_8:1;         //!< rfb_v3_8
    u32 rdp_v4_0:1;         //!< rdp_v4_0
    u32 rdp_v5_0_to_8_1:1;  //!< rdp_v5_0_to_8_1
    u32 rdp_v10_0:1;        //!< rdp_v10_0
    u32 rdp_v10_1:1;        //!< rdp_v10_1
    u32 rdp_v10_2:1;        //!< rdp_v10_2
    u32 rdp_v10_3:1;        //!< rdp_v10_3
    u32 rdp_v10_4:1;        //!< rdp_v10_4
    u32 rdp_v10_5:1;        //!< rdp_v10_5
    u32 rdp_v10_6:1;        //!< rdp_v10_6
    u32 rdp_v10_7:1;        //!< rdp_v10_7
    u32 rdp_v10_8:1;        //!< rdp_v10_8
    u32 rdp_v10_9:1;        //!< rdp_v10_9
    u32 rdp_v10_10:1;       //!< rdp_v10_10
    u32 rdp_v10_11:1;       //!< rdp_v10_11
    u32 rdp_udp_v1:1;       //!< rdp_udp_v1
    u32 rdp_udp_v2:1;       //!< rdp_udp_v2
    u32 rdp_udp_v3:1;       //!< rdp_udp_v3
    u32 mqtt_v3_1:1;        //!< mqtt_v3_1
    u32 mqtt_v3_1_1:1;      //!< mqtt_v3_1_1
    u32 mqtt_v5_0:1;        //!< mqtt_v5_0
    u32 snmp_v1:1;          //!< snmp_v1
    u32 snmp_v2c:1;         //!< snmp_v2c
    u32 snmp_v2:1;          //!< snmp_v2
    u32 snmp_v3:1;          //!< snmp_v3
    u32 smb:1;              //!< smb
    u32 tds:1;              //!< tds
    u32 ietf_v2:1;          //!< ietf_v2
    u32 padding:(128 - PACE2_PROTOCOL_ATTRIBUTES_COUNT);
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 padding:(128 - PACE2_PROTOCOL_ATTRIBUTES_COUNT);
    u32 ietf_v2:1;          //!< ietf_v2
    u32 tds:1;              //!< tds
    u32 smb:1;              //!< smb
    u32 snmp_v3:1;          //!< snmp_v3
    u32 snmp_v2:1;          //!< snmp_v2
    u32 snmp_v2c:1;         //!< snmp_v2c
    u32 snmp_v1:1;          //!< snmp_v1
    u32 mqtt_v5_0:1;        //!< mqtt_v5_0
    u32 mqtt_v3_1_1:1;      //!< mqtt_v3_1_1
    u32 mqtt_v3_1:1;        //!< mqtt_v3_1
    u32 rdp_udp_v3:1;       //!< rdp_udp_v3
    u32 rdp_udp_v2:1;       //!< rdp_udp_v2
    u32 rdp_udp_v1:1;       //!< rdp_udp_v1
    u32 rdp_v10_11:1;       //!< rdp_v10_11
    u32 rdp_v10_10:1;       //!< rdp_v10_10
    u32 rdp_v10_9:1;        //!< rdp_v10_9
    u32 rdp_v10_8:1;        //!< rdp_v10_8
    u32 rdp_v10_7:1;        //!< rdp_v10_7
    u32 rdp_v10_6:1;        //!< rdp_v10_6
    u32 rdp_v10_5:1;        //!< rdp_v10_5
    u32 rdp_v10_4:1;        //!< rdp_v10_4
    u32 rdp_v10_3:1;        //!< rdp_v10_3
    u32 rdp_v10_2:1;        //!< rdp_v10_2
    u32 rdp_v10_1:1;        //!< rdp_v10_1
    u32 rdp_v10_0:1;        //!< rdp_v10_0
    u32 rdp_v5_0_to_8_1:1;  //!< rdp_v5_0_to_8_1
    u32 rdp_v4_0:1;         //!< rdp_v4_0
    u32 rfb_v3_8:1;         //!< rfb_v3_8
    u32 rfb_v3_7:1;         //!< rfb_v3_7
    u32 rfb_v3_3:1;         //!< rfb_v3_3
    u32 ldaps:1;            //!< ldaps
    u32 ech:1;              //!< ech
    u32 dht:1;              //!< dht
    u32 rt:1;               //!< rt
    u32 dcp:1;              //!< dcp
#endif

} PACE2_protocol_attributes_struct;

/**
 * List of strings, each identifying the appropriate protocol attribute.
 *
 * Items do not change between releases. Only for new attributes strings are added.
 */
#define PACE2_PROTOCOL_ATTRIBUTES_STRINGS \
    "none",\
    "ipsec_payload",\
    "ipsec_isakmp",\
    "ftp_control",\
    "ftp_data",\
    "gtp_up",\
    "gtp_cp",\
    "gtp_prime",\
    "gtp_v1",\
    "gtp_v2",\
    "socks_v4",\
    "socks_v5",\
    "media",\
    "uTP",\
    "hub",\
    "peer",\
    "adc-peer",\
    "spdy",\
    "tn3270",\
    "tcp_wo_hs",\
    "iec-61850-goose",\
    "iec-61850-mms",\
    "iec-61850-smv",\
    "http2",\
    "http-connect",\
    "tunnel",\
    "tls_v1_3",\
    "dfs-r",\
    "ras",\
    "esni",\
    "http",\
    "stun",\
    "webrtc",\
    "ftp",\
    "imap",\
    "pop3",\
    "managesieve",\
    "coap",\
    "xmpp",\
    "smtps",\
    "imaps",\
    "pop3s",\
    "domain-s",\
    "gquic",\
    "ietf",\
    "http3",\
    "mvfst",\
    "session_resumption",\
    "name_service",\
    "datagram_service",\
    "session_service",\
    "doq",\
    "hls",\
    "dash",\
    "ndp",\
    "plain_esp",\
    "encrypted_esp",\
    "anomalous",\
    "smb-cifs",\
    "smb-v2",\
    "smb-v3",\
    "io-cm",\
    "cip-cm",\
    "cip-io",\
    "dcp",\
    "rt",\
    "dht",\
    "ech",\
    "ldaps",\
    "rfb_v3_3",\
    "rfb_v3_7",\
    "rfb_v3_8",\
    "rdp_v4_0",\
    "rdp_v5_0_to_8_1",\
    "rdp_v10_0",\
    "rdp_v10_1",\
    "rdp_v10_2",\
    "rdp_v10_3",\
    "rdp_v10_4",\
    "rdp_v10_5",\
    "rdp_v10_6",\
    "rdp_v10_7",\
    "rdp_v10_8",\
    "rdp_v10_9",\
    "rdp_v10_10",\
    "rdp_v10_11",\
    "rdp_udp_v1",\
    "rdp_udp_v2",\
    "rdp_udp_v3",\
    "mqtt_v3_1",\
    "mqtt_v3_1_1",\
    "mqtt_v5_0",\
    "snmp_v1",\
    "snmp_v2c",\
    "snmp_v2",\
    "snmp_v3",\
    "smb",\
    "tds",\
    "ietf_v2"

/**
 * List of strings, each describing the appropriate protocol attribute.
 *
 * Items may be changed between releases.
 */
#define PACE2_PROTOCOL_ATTRIBUTES_DESCRIPTION \
    "Protocol has no attribute.",\
    "Protocol attribute IPSec payload is marked as deprecated and should not be used anymore. The ESP plain or ESP encrypted attributes should be used instead.",\
    "Indicates that the current connection is IPSEC isakmp traffic.",\
    "Indicates that the current connection is FTP control traffic.",\
    "Indicates that the current connection is FTP data traffic.",\
    "Indicates that the current connection is GTP user plane traffic.",\
    "Indicates that the current connection is GTP control plane traffic.",\
    "Indicates that the current connection is GTP prime traffic.",\
    "Indicates that the current connection is GTP v1 traffic.",\
    "Indicates that the current connection is GTP v2 traffic.",\
    "Indicates that the current connection is SOCKS v4 traffic.",\
    "Indicates that the current connection is SOCKS v5 traffic.",\
    "Protocol attribute media is deprecated and should not be used anymore.",\
    "Indicates that the current connection is uTP traffic.",\
    "Indicates that the current connection is Direct Connect Hub traffic.",\
    "Indicates that the current connection is Direct Connect peer traffic.",\
    "Indicates that the current connection is Direct Connect Advanced direct connect peer traffic.",\
    "Indicates that the current connection is SPDY traffic.",\
    "Indicates that the current connection is Telnet TN3270 traffic.",\
    "Indicates that the current connection is TCP traffic for which no handshake has been observed. It is set if no part of the TCP handshake has been seen, i.e. when the SYN-flag is missing in the TCP flow as well as the SYN/ACK-flag. The attribute is not set for TCP segments without payload.",\
    "The Protocol attribute iec-61850-goose is marked as deprecated and should not be used anymore. Instead the Layer 3 protocol IEC 61850 GOOSE should be used.",\
    "Indicates that the current connection is IEC 61850-8-1 Manufacturing Messaging Specification traffic.",\
    "The Protocol attribute iec-61850-smv is marked as deprecated and should not be used anymore. Instead the Layer 3 protocol IEC 61850 Sampled Values should be used.",\
    "Indicates that the current connection is HTTP2 traffic. The attribute can be set for the SSL and Zero layer 7 protocol.",\
    "Indicates that the current connection is HTTP-Connect traffic.",\
    "Indicates that the current connection is tunnel traffic.",\
    "Indicates that the current connection is TLS traffic with version 1.3.",\
    "Specifies the SD Microsoft Distributed File System Replication Protocol.",\
    "Specifies the Registration, Admission and Status protocol for Endpoint-Gatekeeper and H.323 Gatekeeper communication.",\
    "Indicates that the current connection contains an TLS client hello with encrypted SNI extension.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated HTTP traffic via ALPN.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated STUN traffic via ALPN.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated WebRTC traffic via ALPN.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated FTP traffic via ALPN.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated IMAP traffic via ALPN.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated POP3 traffic via ALPN.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated Managesieve traffic via ALPN.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated COAP traffic via ALPN.",\
    "Indicates that the current connection contains an TLS/DTLS handshake with negotiated XMPP traffic via ALPN.",\
    "Indicates that the current connection contains SMTP data over an encrypted TLS channel on port 465. As described in RFC 8314.",\
    "Indicates that the current connection contains IMAP data over an encrypted TLS channel on port 993. As described in RFC 8314.",\
    "Indicates that the current connection contains POP3 data over an encrypted TLS channel on port 995. As described in RFC 8314.",\
    "Indicates that the current connection contains DNS data over an encrypted TLS channel on port 853. As described in RFC 7858 and RFC 8094.",\
    "Indicates that the current connection is Google QUIC traffic.",\
    "Indicates that the current connection is QUIC IETF traffic.",\
    "Indicates that the current connection contains an TLS handshake with negotiated HTTP/3 traffic via ALPN. More information can be found inside the QUIC HTTP draft",\
    "Indicates that the current connection is Facebook MVFST QUIC traffic.",\
    "Indicates that the current connection is a TLS 1.3 session resumption.",\
    "Indicates that the current connection contains Netbios Name Service.",\
    "Indicates that the current connection contains Netbios Datagram Service.",\
    "Indicates that the current connection contains Netbios Session Service.",\
    "Indicates that the current connection contains an QUIC handshake with negotiated DNS traffic via ALPN.",\
    "Indicates that the current connection is used for streaming media content over HTTP with HTTP Live Streaming (HLS).",\
    "Indicates that the current connection is used for streaming media content over HTTP with DASH (Dynamic Adaptive Streaming over HTTP).",\
    "The Network Discovery Protocol (NDP) operates at the data link layer and is used for the address resolution in IPv6 networks.",\
    "Indicates that the current connection is using a plain text encapsulating security payload (esp) header.",\
    "Indicates that the current connection is using an encrypted encapsulating security payload (esp) header.",\
    "The anomalous attribute indicates an invalid or unusual protocol payload, e.g. for invalid NTP or ICMP traffic, that does not follow the protocol specification.",\
    "Indicates that the current connection is SMB (Server Message Block) or CIFS (Common Internet File System) traffic.",\
    "Indicates that the current connection is SMB (Server Message Block) v2 traffic.",\
    "Indicates that the current connection is SMB (Server Message Block) v3 traffic.",\
    "Indicates that the current connection is Profinet IO CM.",\
    "Indicates that the current connection is CIP CM (Common Industrial Protocol Connection Manager).",\
    "Indicates that the current connection is CIP I/O.",\
    "Indicates that the current connection is Profinet Discovery and Configuration Protocol.",\
    "Indicates that the current connection is Profinet Real-Time.",\
    "Indicates that the current connection is BitTorrent DHT (Distributed Hash Table) traffic.",\
    "Indicates that the current connection contains a TLS Encrypted Client Hello Extension.",\
    "Indicates that the current connection may contains LDAP data over an encrypted TLS channel on port 636.",\
    "Indicates that the current connection uses the RFB (remote framebuffer) protocol version 3.3. This protocol attribute is also set if an unknown version is reported by the server or client, as described in RFC-6143.",\
    "Indicates that the current connection uses the RFB (remote framebuffer) protocol version 3.7.",\
    "Indicates that the current connection uses the RFB (remote framebuffer) protocol version 3.8.",\
    "Indicates that the current connection uses the RDP protocol version 4.0 (0x00080001).",\
    "Indicates that the current connection uses the RDP protocol version 5.0, 5.1, 5.2, 6.0, 6.1, 7.0, 7.1, 8.0 or 8.1 (0x00080004).",\
    "Indicates that the current connection uses the RDP protocol version 10.0 (0x00080005).",\
    "Indicates that the current connection uses the RDP protocol version 10.1 (0x00080006).",\
    "Indicates that the current connection uses the RDP protocol version 10.2 (0x00080007).",\
    "Indicates that the current connection uses the RDP protocol version 10.3 (0x00080008).",\
    "Indicates that the current connection uses the RDP protocol version 10.4 (0x00080009).",\
    "Indicates that the current connection uses the RDP protocol version 10.5 (0x0008000A).",\
    "Indicates that the current connection uses the RDP protocol version 10.6 (0x0008000B).",\
    "Indicates that the current connection uses the RDP protocol version 10.7 (0x0008000C).",\
    "Indicates that the current connection uses the RDP protocol version 10.8 (0x0008000D).",\
    "Indicates that the current connection uses the RDP protocol version 10.9 (0x0008000E).",\
    "Indicates that the current connection uses the RDP protocol version 10.10 (0x0008000F).",\
    "Indicates that the current connection uses the RDP protocol version 10.11 (0x00080010).",\
    "Indicates that the current connection uses the RDP UDP protocol version 1.",\
    "Indicates that the current connection uses the RDP UDP protocol version 2.",\
    "Indicates that the current connection uses the RDP UDP protocol version 3.",\
    "Indicates that the current connection uses the MQTT protocol version 3.1.",\
    "Indicates that the current connection uses the MQTT protocol version 3.1.1.",\
    "Indicates that the current connection uses the MQTT protocol version 5.0.",\
    "Indicates that the current connection uses the SNMP protocol version 1.",\
    "Indicates that the current connection uses the SNMP protocol version 2c.",\
    "Indicates that the current connection uses the SNMP protocol version 2 (SNMPv2u and SNMPv2* as in RFC 3411 section 6.2).",\
    "Indicates that the current connection uses the SNMP protocol version 3.",\
    "Indicates that the current connection contains a QUIC handshake with negotiated SMB traffic via ALPN.",\
    "Indicates that the current connection contains a TLS handshake with negotiated TDS traffic via ALPN.",\
    "Indicates that the current connection is QUIC IETF Version 2 traffic based on DRAFT or RFC 9369."

/* Public Prototypes ************************************************************/


/**
 * @brief This function provides the string for the given application attribute.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   attribute_id            The application attribute ID.
 * @returns     A char pointer to the application attribute string or NULL when one of the input parameters is invalid.
 */
const char * pace2_get_application_attribute_string(PACE2_module * const pace2_module, int const thread_ID,
                                                    PACE2_application_attributes const attribute_id);


/**
 * @brief This function provides the string for the given protocol attribute.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   attribute_id            The protocol attribute ID.
 * @returns     A char pointer to the protocol attribute string or NULL if one of the input parameters is invalid.
 */
const char * pace2_get_protocol_attribute_string(PACE2_module * const pace2_module, int const thread_ID,
                                                 PACE2_protocol_attributes const attribute_id);

/**
 * @brief This function provides the description for the given application attribute.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   attribute_id            The application attribute ID.
 * @returns     The application attribute description or NULL if one of the input parameters is invalid.
 */
const char * pace2_get_application_attribute_description(PACE2_module * const pace2_module, int const thread_ID,
                                                         PACE2_application_attributes const attribute_id);

/**
 * @brief This function provides the description for the given protocol attribute.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   attribute_id            The protocol attribute ID.
 * @returns     The protocol attribute description or NULL if one of the input parameters is invalid.
 */
const char * pace2_get_protocol_attribute_description(PACE2_module * const pace2_module, int const thread_ID,
                                                      PACE2_protocol_attributes const attribute_id);

#ifdef __cplusplus
}
#endif
#endif /* PACE2_ATTRIBUTES_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
