/** @file      pace2_debug.h
  * @brief     PACE 2 debug advanced event definition.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_DEBUG_H
#define PACE2_DEBUG_H

/* Includes *********************************************************************/
#include "ipq_config.h"

#ifndef PACE_KERNEL_LIB

#include <stdio.h>

#include "pace2_basic_def.h"
#include "pace2_event_types.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public Prototypes ************************************************************/

/** Helper function to write an advanced/class event to a file pointer for debugging reasons.
 * @param[in]   f       A FILE pointer with write access. stderr or stdout can be used too.
 * @param[in]   event   Pointer to the event to write.
 */
void pace2_debug_advanced_event(FILE * f, PACE2_event const * const event);

/**
 * @brief Exemplary default implementation of @ref pace2_logging_callback
 *
 * This function can be used as default implementation if the user does not provide his own logging callback function.
 *
 * @param[in]   thread_ID   The thread ID of the thread, from which this callback was called.
 * @param[in]   level       The kind of information for which this callback was called.
 *                          See also @ref PACE2_log_level.
 * @param[in]   user_ptr    A pointer to user defined data. It is ignored by this default implementation.
 * @param[in]   format      Is a format string like in printf, which describes the PLUG-IN information.
 * @param[in]   args        The set of variable arguments, which are passed to this function. Contains the values
 *                          for the format string like for printf.
 */
void pace2_default_logging_callback_impl(u32 thread_ID, PACE2_log_level level,
                                         void * user_ptr, const char *format, va_list args);

typedef struct pace2_logging_filters {
    u16 filter_count;           /**< Number of component filters. */
    char ** filter_list;        /**< List of allowed components. */
} PACE2_logging_filters;

/**
 * @brief Exemplary implementation of @ref pace2_logging_callback
 *
 * This function can be used as a logging callback function.
 * The user pointer has to be of type @ref PACE2_logging_filters.
 * If no filters are provided or the list is empty, all logging messages will be printed.
 *
 * @param[in]   thread_ID   The thread ID of the thread, from which this callback was called.
 * @param[in]   level       The kind of information for which this callback was called.
 *                          See also @ref PACE2_log_level.
 * @param[in]   user_ptr    A pointer to user defined data. It is ignored by this default implementation.
 * @param[in]   format      Is a format string like in printf.
 * @param[in]   args        The set of variable arguments, which are passed to this function. Contains the values
 *                          for the format string like for printf.
 */
void pace2_logging_callback_filter_impl(u32 thread_ID, PACE2_log_level level,
                                        void * user_ptr, const char * format, va_list args);

#ifdef __cplusplus
}
#endif
#endif /* PACE_KERNEL_LIB */
#endif /* PACE2_DEBUG_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
