/** @file      pace2_events.h
  * @brief     Global PACE 2 event definitions.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_EVENTS_H
#define PACE2_EVENTS_H

/* Includes *********************************************************************/
#include "pace2_basic_def.h"
#include "pace2_event_bitmask.h"

/* Begin of Event Includes */
#include "event_types/http_events.h"
#include "event_types/http2_events.h"
#include "event_types/citrix_events.h"
#include "event_types/dhcp_events.h"
#include "event_types/os_events.h"
#include "event_types/nat_events.h"
#include "event_types/sip_events.h"
#include "event_types/ssl_events.h"
#include "event_types/sit_events.h"
#include "event_types/csi_events.h"
#include "event_types/rtp_events.h"
#include "event_types/rtcp_events.h"
#include "event_types/tcp_events.h"
#include "event_types/classification_events.h"
#include "event_types/fastpath_events.h"
#include "event_types/diameter_events.h"
#include "event_types/dns_events.h"
#include "event_types/ftp_events.h"
#include "event_types/gtp_events.h"
#include "event_types/radius_events.h"
#include "event_types/icmpv6_events.h"
#include "event_types/pop3_events.h"
#include "event_types/nntp_events.h"
#include "event_types/imap_events.h"
#include "event_types/smtp_events.h"
#include "event_types/wsp_events.h"
#include "event_types/mail_events.h"
#include "event_types/pptp_events.h"
#include "event_types/jabber_events.h"
#include "event_types/generic_events.h"
#include "event_types/license_events.h"
#include "event_types/scada_events.h"
#include "event_types/irc_events.h"
#include "event_types/cdd_events.h"
#include "event_types/h323_events.h"
#include "event_types/socks_events.h"
#include "event_types/flute_events.h"
#include "event_types/hi5_events.h"
#include "event_types/dissector_metadata_events.h"
#include "event_types/telnet_events.h"
#include "event_types/mmse_events.h"
#include "event_types/l2tp_events.h"
#include "event_types/yahoo_msgr_events.h"
#include "event_types/mpeg_events.h"
#include "event_types/oscar_events.h"
#include "event_types/icq_webchat_events.h"
#include "event_types/rtsp_events.h"
#include "event_types/bittorrent_events.h"
#include "event_types/class_generic_events.h"
#include "event_types/quic_events.h"
#include "event_types/chat_events.h"
#include "event_types/edonkey_events.h"
#include "event_types/ssl_events.h"
#include "event_types/ocsp_events.h"
#include "event_types/javascript_events.h"
#include "event_types/q931_events.h"
#include "event_types/smb_events.h"
#include "event_types/modbus_events.h"
#include "event_types/zero_events.h"
#include "event_types/voip_events.h"
#include "event_types/plugin_events.h"
#include "event_types/ssh_events.h"
#include "event_types/mqtt_events.h"
#include "event_types/netbios_events.h"
#include "event_types/mdns_events.h"
#include "event_types/ml_events.h"
#include "event_types/browser_classification_events.h"

/* End of Event Includes */

#ifdef __cplusplus
extern "C" {
#endif

/* Event Structures *************************************************************/

/**
 * @brief Global event structure definition.
 */
union pace2_event {
    /** structure containing just the header fields common to all events */
    struct {
        /** Global event type, see @ref PACE2_event_type for the possible values */
        PACE2_EVENT_COMMON_FIELDS;
    } header;

    PACE2_no_event no_event;                                        /*!< Placeholder structure. */
    PACE2_flow_started flow_started;                                /*!< Flow started event data. */
    PACE2_flow_dropped flow_dropped;                                /*!< Flow dropped event data. */
    PACE2_subscriber_started subscriber_started;                    /*!< Subscriber started event data. */
    PACE2_subscriber_dropped subscriber_dropped;                    /*!< Subscriber dropped event data. */
    PACE2_basic_HTTP_request_event http_basic_request_meta_data;    /*!< Event data of a basic HTTP request event. */
    PACE2_basic_HTTP_response_event http_basic_response_meta_data;  /*!< Event data of a basic HTTP response event. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_HTTP_event http_advanced_meta_data;              /*!< Advanced HTTP event data. */
    PACE2_class_HTTP_event http_class_meta_data;                    /*!< HTTP class event data. */
#endif
    PACE2_citrix_event citrix_meta_data;                            /*!< Event data of a Citrix metadata event. */
    PACE2_basic_DHCP_event dhcp_basic_meta_data;                    /*!< Event data of a basic DHCP event. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_DHCP_event dhcp_advanced_meta_data;              /*!< Advanced DHCP event data. */
#endif
    PACE2_OS_event os_meta_data;                                    /*!< Event data of a OS detection metadata event. */
    PACE2_NAT_event nat_meta_data;                                  /*!< Event data of a NAT detection metadata event. */
    PACE2_basic_SIP_event sip_basic_meta_data;                      /*!< Event data of a basic SIP event. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_SIP_event sip_advanced_meta_data;                /*!< Advanced SIP event data. */
    PACE2_class_VOIP_event voip_class_meta_data;                    /*!< VOIP class event data. */
#endif
    PACE2_SSL_client_hello_event ssl_client_hello_meta_data;        /*!< Event data of a basic SSL client hello event. */
    PACE2_SSL_server_hello_event ssl_server_hello_meta_data;        /*!< Event data of a basic SSL server hello event. */
    PACE2_SIT_event sit_meta_data;                                  /*!< Event data of a SSL session ID tracker status event. */
    PACE2_csi_packet_dir_event csi_packet_dir_meta_data;            /*!< Event data of a packet direction event. */
    PACE2_csi_host_type_event csi_host_type_meta_data;              /*!< Event data of a host type event. */
    /**
     * RTP performance report event data.
     *
     * @deprecated The rtp_meta_data event is deprecated and will be removed completely in the future. The information
     *             provided by this event is moved to RTP dissector result 'dissector_result.rtp.statistics' in the
     *             'PACE2_dissector_metadata_event'.
     */
    PACE2_RTP_event rtp_meta_data;
    PACE2_RTCP_event rtcp_meta_data;                                /*!< RTCP report event data. */
    PACE2_TCP_event tcp_meta_data;                                  /*!< TCP status event data. */
    PACE2_classification_status_event classification_status_data;   /*!< Classification status event data. */
    PACE2_fastpath_event fastpath_meta_data;                        /*!< Fastpath status event data. */

#ifndef PACE_KERNEL_LIB
    PACE2_advanced_Diameter_event diameter_advanced_meta_data;      /*!< Advanced Diameter event data. */
    PACE2_advanced_DNS_event dns_advanced_meta_data;                /*!< Advanced DNS event data. */
    PACE2_advanced_FTP_event ftp_advanced_meta_data;                /*!< Advanced FTP event data. */
    PACE2_advanced_radius_event radius_advanced_meta_data;          /*!< Advanced radius event data. */
    PACE2_advanced_icmpv6_event icmpv6_advanced_meta_data;          /*!< Advanced ICMPv6 event data. */
    PACE2_advanced_pop3_event pop3_advanced_meta_data;              /*!< Advanced POP3 event data. */
    PACE2_advanced_imap_event imap_advanced_meta_data;              /*!< Advanced IMAP event data. */
    PACE2_advanced_smtp_event smtp_advanced_meta_data;              /*!< Advanced SMTP event data. */
    PACE2_advanced_GTP_event gtp_advanced_meta_data;                /*!< Advanced GTP event data. */
    PACE2_advanced_wsp_event wsp_advanced_meta_data;                /*!< Advanced WAP and WSP event data. */
    PACE2_advanced_jabber_event jabber_advanced_meta_data;          /*!< Advanced unencrypted Jabber event data. */
    PACE2_advanced_PPTP_event pptp_advanced_meta_data;              /*!< Advanced PPTP event data. */
    PACE2_advanced_ssh_event ssh_advanced_meta_data;                /*!< Advanced SSH event data. */
    PACE2_class_MAIL_event mail_class_meta_data;                    /*!< Class Mail event data. */
#endif
    PACE2_classification_result_event classification_result_data;   /*!< Classification result event data. */
    PACE2_cdc_result_event cdc_data;                                /*!< Custom Defined Classification result event data. */
    PACE2_license_event license_event;                              /*!< License exceeded event data. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_irc_event irc_advanced_meta_data;                /*!< Advanced IRC event data. */
#endif
    PACE2_flow_info flow_info;                                      /*!< Flow info event data. */
    PACE2_flow_process flow_process;                                /*!< Flow process event data. */
    PACE2_subscriber_process subscriber_process;                    /*!< Subscriber process event data. */
#ifndef PACE_KERNEL_LIB
    PACE2_cdd_event cdd_meta_data;                                  /*!< Custom defined decoder event data. */
    PACE2_advanced_h323_event h323_advanced_meta_data;              /*!< Advanced H323 event data. */
    PACE2_advanced_socks_event socks_advanced_meta_data;            /*!< Advanced SOCKS event data. */
#endif
    PACE2_TCP_started_event tcp_started_meta_data;                  /*!< TCP start event data. */
    PACE2_TCP_closed_event tcp_dropped_meta_data;                   /*!< TCP finished event data. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_hi5_event hi5_advanced_meta_data;                /*!< Advanced Hi5 event data. */
#endif
    PACE2_SSL_dns_alt_names_event ssl_dns_alt_names_meta_data;      /*!< Event data of a basic SSL DNS Alt Names event. */
    PACE2_dissector_metadata_event dissector_metadata;              /*!< Dissector metadata event data. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_telnet_event telnet_advanced_meta_data;          /*!< Advanced Telnet event data. */
    PACE2_advanced_mmse_event mmse_advanced_meta_data;              /*!< Advanced MMSE event data. */
    PACE2_advanced_l2tp_event l2tp_advanced_meta_data;              /*!< Advanced L2TP event data. */
    PACE2_advanced_yahoo_msgr_event yahoo_msgr_advanced_meta_data;  /*!< Advanced Yahoo Messenger event data. */
    PACE2_advanced_rtp_event rtp_advanced_meta_data;                /*!< Advanced RTP event data. */
    PACE2_advanced_nntp_event nntp_advanced_meta_data;              /*!< Advanced NNTP event data. */
    PACE2_advanced_flute_event flute_advanced_meta_data;            /*!< Advanced FLUTE event data. */
    PACE2_advanced_oscar_event oscar_advanced_meta_data;            /*!< Advanced Oscar event data. */
#endif
    PACE2_SSL_session_ticket_event ssl_session_ticket_meta_data;    /*!< Event data of a basic SSL session ticket event. */
    PACE2_advanced_TCP_event tcp_advanced_meta_data;                /*!< Advanced TCP event data. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_icq_webchat_event icq_webchat_advanced_meta_data;/*!< Advanced ICQ webchat event data. */
    PACE2_advanced_mpeg_event mpeg_advanced_meta_data;              /*!< Advanced MPEG2 event data. */
    PACE2_advanced_mqtt_event mqtt_advanced_meta_data;              /*!< Advanced MQTT event data. */
    PACE2_advanced_rtsp_event rtsp_advanced_meta_data;              /*!< Advanced RTSP event data. */
    PACE2_advanced_bittorrent_event bittorrent_advanced_meta_data;  /*!< Advanced BitTorrent event data. */
    PACE2_class_generic_event generic_class_meta_data;              /*!< Generic class event data. */
    PACE2_class_FTP_event ftp_class_meta_data;                      /*!< FTP class event data. */
    PACE2_class_CHAT_event chat_class_meta_data;                    /*!< Class Chat event data. */
#endif
    PACE2_basic_HTTP_parsed_request_event http_parsed_request_meta_data; /*!< HTTP parsed request event. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_edonkey_event edonkey_advanced_meta_data;         /*!< Advanced eDonkey event data. */
    PACE2_advanced_SSL_event ssl_advanced_meta_data;                 /*!< Advanced SSL event data. */
    PACE2_class_SSL_event class_ssl_meta_data;                       /*!< Class SSL event data. */
    PACE2_advanced_OCSP_event ocsp_advanced_meta_data;               /*!< Advanced OCSP event data. */
    PACE2_advanced_javascript_event javascript_advanced_meta_data;   /*!< Advanced JavaScript event data. */
    PACE2_advanced_Scada_event scada_advanced_meta_data;             /*!< Advanced SCADA event data. */
    PACE2_advanced_smb_event smb_advanced_meta_data;                 /*!< Advanced SMB event data. */
#endif
    PACE2_events_dropped dropped_events_data;                        /*!< Dropped Events event data. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_q931_event q931_advanced_meta_data;               /*!< Advanced Q.931 event data. */
    PACE2_advanced_modbus_event modbus_advanced_meta_data;           /*!< Advanced Modbus event data. */
#endif
    PACE2_ZERO_sni_event zero_sni_meta_data;                         /*!< Event data of a ZERO sni event. */
    PACE2_basic_QUIC_client_hello_event quic_client_hello_meta_data; /*!< Event data of a QUIC client hello event. */
    PACE2_plugin_event plugin_event;                                 /*!< Custom defined PLUG-IN event data. */
    PACE2_basic_DNS_event dns_basic_meta_data;                       /*!< Basic DNS event data. */
    PACE2_basic_HTTP2_event http2_basic_meta_data;                   /*!< Basic HTTP/2 event data. */
#ifndef PACE_KERNEL_LIB
    PACE2_advanced_HTTP2_event http2_advanced_meta_data;             /*!< Advanced HTTP/2 event data. */
#endif
    PACE2_basic_netbios_event netbios_basic_meta_data;               /*!< Basic NetBIOS event data. */
    PACE2_basic_mdns_event mdns_basic_meta_data;                     /*!< Basic mDNS event data. */
    PACE2_pht_usage_notification_event pht_threshold_notification_event; /*!< Hashtable usage notification event data.*/
    PACE2_ml_classification_event ml_classification_meta_data;       /*!< Machine learning classification event data. */
    PACE2_ml_deregister_event ml_deregister_meta_data;               /*!< Machine learning deregister event data. */
    PACE2_browser_classification_event browser_classification_meta_data; /*!< Event data of a browser classification
                                                                              metadata event. */
};

/* Public Prototypes ************************************************************/

/**
 * @brief Get the corresponding event group of a given event.
 *
 * @param[in]   event_type      Event type for which the group should be returned.
 * @param[out]  event_group     Group of the given event type.
 * @retval      PACE2_SUCCESS       @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_PARAM_MISSING @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval      PACE2_FAILURE       Event type is unknown.
 */
PACE2_return_state pace2_get_group_of_event(PACE2_event_type const event_type,
                                            PACE2_event_groups * const event_group);

/**
 * @brief Get the corresponding events of a given event group.
 *
 * @param[in]   event_group     Event group for which the events should be returned.
 * @param[out]  event_types     Bitmask which indicates all events of the given group.
 * @retval      PACE2_SUCCESS       @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_PARAM_MISSING @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval      PACE2_FAILURE       Event group is unknown.
 */
PACE2_return_state pace2_get_events_of_group(PACE2_event_groups const event_group,
                                             PACE2_event_bitmask * const event_types);

/**
 * @brief Get the size of an event of the given type.
 *
 * This function operates on a static internal mapping between the event type and its event size. The result is
 * independent of the internal state of pace2 and could return different values exclusively after a dynamic upgrade if
 * the feature is available.
 *
 * It is recommended to use the size returned by this function before copying any event using e.g. @p memcpy.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgraded variant.
 *
 * @param[in]   pace2_module    Constant pointer to the PACE 2 module
 *                              Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID       The ID of the thread currently processing.
 *                              Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   event_type      Event type for which the size should be retrieved.
 * @retval      > 0             The resulting structure size of this specific event type.
 * @retval      0               In case of an unknown event type.
 */
uint32_t pace2_get_event_size(PACE2_module const * pace2_module, int thread_ID, PACE2_event_type event_type);

#ifdef __cplusplus
}
#endif
#endif /* PACE2_EVENTS_H */
