/** @file      pace2_processing_states.h
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_PROCESSING_STATES_H
#define PACE2_PROCESSING_STATES_H

#include "pace2_basic_def.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Definition for basic and detailed information about a processing state.
 *
 * This structure is used to get processing state strings as a parameter for
 * @ref pace2_get_processing_state_info.
 */
typedef struct pace2_processing_state_info {
    char const * name;         /**< Internal Name of the processing state (only valid C names). */
    char const * display_name; /**< User-friendly name of this processing state. */
    char const * description;  /**< Full-text description of this processing state. */
} PACE2_processing_state_info;

/**
 * @brief Get the number of available processing states.
 *
 * @param[in]   pace2_module        Constant pointer to the PACE 2 module.
 * @param[out]  number_of_states    Pointer for the resulting number of processing states.
 *
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_INVALID_MODULE_POINTER     @copydoc PACE2_return_state::PACE2_INVALID_MODULE_POINTER
 */
PACE2_return_state pace2_get_number_of_processing_states(PACE2_module * const pace2_module,
                                                         u32 * const number_of_states);

/**
 * @brief Get the processing state information for a specific state.
 *
 * @param[in]   pace2_module        Constant pointer to the PACE 2 module.
 * @param[in]   state_index         Array Index of processing state information.
 * @param[out]  info                Pointer to processing state information.
 *
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_INVALID_MODULE_POINTER     @copydoc PACE2_return_state::PACE2_INVALID_MODULE_POINTER
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 * @retval PACE2_POINTER_NULL               @copydoc PACE2_return_state::PACE2_POINTER_NULL
 */
PACE2_return_state pace2_get_processing_state_info(PACE2_module * const pace2_module,
                                                   u32 const state_index,
                                                   PACE2_processing_state_info const ** const info);

#ifdef __cplusplus
}
#endif
#endif /* PACE2_PROCESSING_STATES_H */
