/** @file      pace2_protocols.h
  * @brief     PACE 2 protocol definitions
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_PROTOCOLS_H
#define PACE2_PROTOCOLS_H

/* Includes *********************************************************************/
#include "pace2_basic_def.h"
#include "pace2_attributes_bitmask.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public Types *****************************************************************/

/**
 * @brief Enum defining all available protocols.
**/
typedef enum pace2_protocols {
    PACE2_PROTOCOL_UNKNOWN = 0,
    PACE2_PROTOCOL_BITTORRENT,
    PACE2_PROTOCOL_EDONKEY,
    PACE2_PROTOCOL_GNUTELLA,
    PACE2_PROTOCOL_WINMX,
    PACE2_PROTOCOL_DIRECTCONNECT,
    PACE2_PROTOCOL_APPLEJUICE,
    PACE2_PROTOCOL_SOULSEEK,
    PACE2_PROTOCOL_ARES,
    PACE2_PROTOCOL_FREENET,
    PACE2_PROTOCOL_RES_10,
    PACE2_PROTOCOL_XDCC,
    PACE2_PROTOCOL_SKYPE,
    PACE2_PROTOCOL_SIP,
    PACE2_PROTOCOL_RES_14,
    PACE2_PROTOCOL_YAHOO,
    PACE2_PROTOCOL_OSCAR,
    PACE2_PROTOCOL_IRC,
    PACE2_PROTOCOL_XMPP,
    PACE2_PROTOCOL_HTTP,
    PACE2_PROTOCOL_POP,
    PACE2_PROTOCOL_SMTP,
    PACE2_PROTOCOL_IMAP,
    PACE2_PROTOCOL_RES_23,
    PACE2_PROTOCOL_FTP,
    PACE2_PROTOCOL_DNS,
    PACE2_PROTOCOL_RTSP,
    PACE2_PROTOCOL_FILETOPIA,
    PACE2_PROTOCOL_RES_28,
    PACE2_PROTOCOL_IMESH,
    PACE2_PROTOCOL_SSL,
    PACE2_PROTOCOL_FLASH,
    PACE2_PROTOCOL_MMS,
    PACE2_PROTOCOL_MPEG,
    PACE2_PROTOCOL_QUICKTIME,
    PACE2_PROTOCOL_RES_35,
    PACE2_PROTOCOL_OPENVPN,
    PACE2_PROTOCOL_IPSEC,
    PACE2_PROTOCOL_GRE,
    PACE2_PROTOCOL_NOT_DETECTED,
    PACE2_PROTOCOL_H323,
    PACE2_PROTOCOL_WINDOWSMEDIA,
    PACE2_PROTOCOL_REALMEDIA,
    PACE2_PROTOCOL_TVANTS,
    PACE2_PROTOCOL_RES_44,
    PACE2_PROTOCOL_RES_45,
    PACE2_PROTOCOL_RES_46,
    PACE2_PROTOCOL_PPLIVE,
    PACE2_PROTOCOL_IAX,
    PACE2_PROTOCOL_VTUN,
    PACE2_PROTOCOL_MGCP,
    PACE2_PROTOCOL_GADUGADU,
    PACE2_PROTOCOL_ZATTOO,
    PACE2_PROTOCOL_WINNY,
    PACE2_PROTOCOL_QQ,
    PACE2_PROTOCOL_QQLIVE,
    PACE2_PROTOCOL_RES_56,
    PACE2_PROTOCOL_SSH,
    PACE2_PROTOCOL_POPO,
    PACE2_PROTOCOL_THUNDER,
    PACE2_PROTOCOL_UUSEE,
    PACE2_PROTOCOL_VNC,
    PACE2_PROTOCOL_ICMP,
    PACE2_PROTOCOL_DHCP,
    PACE2_PROTOCOL_IGMP,
    PACE2_PROTOCOL_EGP,
    PACE2_PROTOCOL_RES_66,
    PACE2_PROTOCOL_RES_67,
    PACE2_PROTOCOL_XBOX,
    PACE2_PROTOCOL_SOFTETHER,
    PACE2_PROTOCOL_SCTP,
    PACE2_PROTOCOL_SMB,
    PACE2_PROTOCOL_TELNET,
    PACE2_PROTOCOL_NTP,
    PACE2_PROTOCOL_NFS,
    PACE2_PROTOCOL_RTP,
    PACE2_PROTOCOL_TOR,
    PACE2_PROTOCOL_SSDP,
    PACE2_PROTOCOL_WORLDOFWARCRAFT,
    PACE2_PROTOCOL_AVI,
    PACE2_PROTOCOL_VPN_X,
    PACE2_PROTOCOL_HAMACHI_VPN,
    PACE2_PROTOCOL_SKINNY,
    PACE2_PROTOCOL_RES_83,
    PACE2_PROTOCOL_OGG,
    PACE2_PROTOCOL_PALTALK,
    PACE2_PROTOCOL_MOVE,
    PACE2_PROTOCOL_POSTGRES,
    PACE2_PROTOCOL_OSPF,
    PACE2_PROTOCOL_BGP,
    PACE2_PROTOCOL_QUAKE,
    PACE2_PROTOCOL_SECONDLIFE,
    PACE2_PROTOCOL_MYSQL,
    PACE2_PROTOCOL_BATTLEFIELD,
    PACE2_PROTOCOL_RES_94,
    PACE2_PROTOCOL_RDP,
    PACE2_PROTOCOL_SNMP,
    PACE2_PROTOCOL_KONTIKI,
    PACE2_PROTOCOL_VEOHTV,
    PACE2_PROTOCOL_KERBEROS,
    PACE2_PROTOCOL_IP_IN_IP,
    PACE2_PROTOCOL_GTP,
    PACE2_PROTOCOL_RADIUS,
    PACE2_PROTOCOL_STUN,
    PACE2_PROTOCOL_SYSLOG,
    PACE2_PROTOCOL_TDS,
    PACE2_PROTOCOL_NETBIOS,
    PACE2_PROTOCOL_MDNS,
    PACE2_PROTOCOL_IPP,
    PACE2_PROTOCOL_LDAP,
    PACE2_PROTOCOL_WARCRAFT3,
    PACE2_PROTOCOL_XDMCP,
    PACE2_PROTOCOL_RES_112,
    PACE2_PROTOCOL_RES_113,
    PACE2_PROTOCOL_MAPI,
    PACE2_PROTOCOL_L2TP,
    PACE2_PROTOCOL_ISAKMP,
    PACE2_PROTOCOL_RTCP,
    PACE2_PROTOCOL_TFTP,
    PACE2_PROTOCOL_RES_119,
    PACE2_PROTOCOL_PPP,
    PACE2_PROTOCOL_PPTP,
    PACE2_PROTOCOL_STEALTHNET,
    PACE2_PROTOCOL_SAP,
    PACE2_PROTOCOL_ICMPV6,
    PACE2_PROTOCOL_YOURFREEDOM,
    PACE2_PROTOCOL_VOIPSWITCH_VOIPTUNNEL,
    PACE2_PROTOCOL_DHCPV6,
    PACE2_PROTOCOL_MEEBO,
    PACE2_PROTOCOL_FUNSHION,
    PACE2_PROTOCOL_AFP,
    PACE2_PROTOCOL_AIMINI,
    PACE2_PROTOCOL_RES_132,
    PACE2_PROTOCOL_FLORENSIA,
    PACE2_PROTOCOL_TEAMSPEAK,
    PACE2_PROTOCOL_MAPLESTORY,
    PACE2_PROTOCOL_RES_136,
    PACE2_PROTOCOL_RES_137,
    PACE2_PROTOCOL_DOFUS,
    PACE2_PROTOCOL_RES_139,
    PACE2_PROTOCOL_WORLD_OF_KUNG_FU,
    PACE2_PROTOCOL_FIESTA,
    PACE2_PROTOCOL_SPLASHFIGHTER,
    PACE2_PROTOCOL_CROSSFIRE,
    PACE2_PROTOCOL_WAP_WSP,
    PACE2_PROTOCOL_WAP_WTP_WSP,
    PACE2_PROTOCOL_WAP_WTLS,
    PACE2_PROTOCOL_RES_147,
    PACE2_PROTOCOL_GUILDWARS,
    PACE2_PROTOCOL_ARMAGETRON,
    PACE2_PROTOCOL_RIM_PROPRIETARY,
    PACE2_PROTOCOL_RFACTOR,
    PACE2_PROTOCOL_RDT,
    PACE2_PROTOCOL_TEAMVIEWER,
    PACE2_PROTOCOL_GAMEKIT,
    PACE2_PROTOCOL_NETMOTION,
    PACE2_PROTOCOL_TEREDO,
    PACE2_PROTOCOL_SPOTIFY,
    PACE2_PROTOCOL_RES_158,
    PACE2_PROTOCOL_ULTRASURF,
    PACE2_PROTOCOL_VIBER,
    PACE2_PROTOCOL_TANGO,
    PACE2_PROTOCOL_RES_162,
    PACE2_PROTOCOL_JBK3K,
    PACE2_PROTOCOL_OPERAMINI,
    PACE2_PROTOCOL_BOLT,
    PACE2_PROTOCOL_RES_166,
    PACE2_PROTOCOL_WHATSAPP,
    PACE2_PROTOCOL_WEBEX,
    PACE2_PROTOCOL_RES_169,
    PACE2_PROTOCOL_IMO,
    PACE2_PROTOCOL_CITRIX_GOTO,
    PACE2_PROTOCOL_COMODOUNITE,
    PACE2_PROTOCOL_RES_173,
    PACE2_PROTOCOL_VENTRILO,
    PACE2_PROTOCOL_RES_175,
    PACE2_PROTOCOL_WEBSOCKET,
    PACE2_PROTOCOL_EBUDDY,
    PACE2_PROTOCOL_SSTP,
    PACE2_PROTOCOL_RES_179,
    PACE2_PROTOCOL_RES_180,
    PACE2_PROTOCOL_LDP,
    PACE2_PROTOCOL_RES_182,
    PACE2_PROTOCOL_MSRP,
    PACE2_PROTOCOL_RES_184,
    PACE2_PROTOCOL_SOCKS,
    PACE2_PROTOCOL_RES_186,
    PACE2_PROTOCOL_RES_187,
    PACE2_PROTOCOL_RES_188,
    PACE2_PROTOCOL_RES_189,
    PACE2_PROTOCOL_RES_190,
    PACE2_PROTOCOL_PDPROXY,
    PACE2_PROTOCOL_RES_192,
    PACE2_PROTOCOL_RES_193,
    PACE2_PROTOCOL_FIX,
    PACE2_PROTOCOL_ORACLE_DB,
    PACE2_PROTOCOL_RSVP,
    PACE2_PROTOCOL_LOTUS_NOTES,
    PACE2_PROTOCOL_POISON_IVY,
    PACE2_PROTOCOL_NETFLOW,
    PACE2_PROTOCOL_WECHAT,
    PACE2_PROTOCOL_SOAP,
    PACE2_PROTOCOL_KIK_MESSENGER,
    PACE2_PROTOCOL_MPLUS,
    PACE2_PROTOCOL_LINE,
    PACE2_PROTOCOL_IPERF,
    PACE2_PROTOCOL_DIAMETER,
    PACE2_PROTOCOL_HIKE_MESSENGER,
    PACE2_PROTOCOL_MQTT,
    PACE2_PROTOCOL_DCE_RPC,
    PACE2_PROTOCOL_IDRIVE,
    PACE2_PROTOCOL_DOTA2,
    PACE2_PROTOCOL_QVOD,
    PACE2_PROTOCOL_RES_213,
    PACE2_PROTOCOL_MSNAV,
    PACE2_PROTOCOL_TIBIA,
    PACE2_PROTOCOL_KAKAO,
    PACE2_PROTOCOL_DESTINY,
    PACE2_PROTOCOL_VYPR_VPN,
    PACE2_PROTOCOL_TACACS,
    PACE2_PROTOCOL_THREEMA,
    PACE2_PROTOCOL_NTLM,
    PACE2_PROTOCOL_RES_222,
    PACE2_PROTOCOL_WHOIS,
    PACE2_PROTOCOL_LAYER3_IPV4,
    PACE2_PROTOCOL_LAYER3_IPV6,
    PACE2_PROTOCOL_LAYER4_TCP,
    PACE2_PROTOCOL_LAYER4_UDP,
    PACE2_PROTOCOL_RTMP,
    PACE2_PROTOCOL_NNTP,
    PACE2_PROTOCOL_FLUTE,
    PACE2_PROTOCOL_MEGACO,
    PACE2_PROTOCOL_RES_232,
    PACE2_PROTOCOL_QUIC,
    PACE2_PROTOCOL_FUZE,
    PACE2_PROTOCOL_RES_235,
    PACE2_PROTOCOL_RES_236,
    PACE2_PROTOCOL_SPEEDTEST,
    PACE2_PROTOCOL_IEC61850,
    PACE2_PROTOCOL_SCADA,
    PACE2_PROTOCOL_SHOUTCAST,
    PACE2_PROTOCOL_CASSANDRA_QUERY_LANGUAGE,
    PACE2_PROTOCOL_CASSANDRA_INTERNODE_COMMUNICATION,
    PACE2_PROTOCOL_THRIFT,
    PACE2_PROTOCOL_PLAYSTATION,
    PACE2_PROTOCOL_SUPERCELL,
    PACE2_PROTOCOL_RES_246,
    PACE2_PROTOCOL_T3,
    PACE2_PROTOCOL_SRTP,
    PACE2_PROTOCOL_SRTCP,
    PACE2_PROTOCOL_LEAGUE_OF_LEGENDS,
    PACE2_PROTOCOL_FACEBOOK,
    PACE2_PROTOCOL_BATTLENET,
    PACE2_PROTOCOL_WOWZA,
    PACE2_PROTOCOL_ZALO,
    PACE2_PROTOCOL_GUILDWARS2,
    PACE2_PROTOCOL_RES_256,
    PACE2_PROTOCOL_EA_ORIGIN,
    PACE2_PROTOCOL_RES_258,
    PACE2_PROTOCOL_ICECAST,
    PACE2_PROTOCOL_RES_260,
    PACE2_PROTOCOL_GOOD,
    PACE2_PROTOCOL_PATH_OF_EXILE,
    PACE2_PROTOCOL_PC_OVER_IP,
    PACE2_PROTOCOL_OCSP,
    PACE2_PROTOCOL_MODBUS,
    PACE2_PROTOCOL_FASP,
    PACE2_PROTOCOL_ICA,
    PACE2_PROTOCOL_OPC_UA,
    PACE2_PROTOCOL_OPENFLOW,
    PACE2_PROTOCOL_IPMI,
    PACE2_PROTOCOL_AMQP,
    PACE2_PROTOCOL_S7COMM,
    PACE2_PROTOCOL_OFF,
    PACE2_PROTOCOL_RSYNC,
    PACE2_PROTOCOL_ZEROMQ,
    PACE2_PROTOCOL_DNP3,
    PACE2_PROTOCOL_BACNET,
    PACE2_PROTOCOL_NINTENDO_NETWORK,
    PACE2_PROTOCOL_PPFILM,
    PACE2_PROTOCOL_BITCOIN,
    PACE2_PROTOCOL_LETV,
    PACE2_PROTOCOL_PSIPHON,
    PACE2_PROTOCOL_ZERO,
    PACE2_PROTOCOL_SYNOLOGY,
    PACE2_PROTOCOL_COAP,
    PACE2_PROTOCOL_LINEAGE_2,
    PACE2_PROTOCOL_IPFIX,
    PACE2_PROTOCOL_MINECRAFT,
    PACE2_PROTOCOL_STRATUM_MINING,
    PACE2_PROTOCOL_LOCAL_PEER_DISCOVERY,
    PACE2_PROTOCOL_RES_291,
    PACE2_PROTOCOL_RES_292,
    PACE2_PROTOCOL_RES_293,
    PACE2_PROTOCOL_IQIYI,
    PACE2_PROTOCOL_RES_295,
    PACE2_PROTOCOL_VPN_UNLIMITED,
    PACE2_PROTOCOL_EXPRESSVPN,
    PACE2_PROTOCOL_TUNNELGURU,
    PACE2_PROTOCOL_BOTIM,
    PACE2_PROTOCOL_HI_VPN,
    PACE2_PROTOCOL_RES_301,
    PACE2_PROTOCOL_RES_302,
    PACE2_PROTOCOL_YEECALL,
    PACE2_PROTOCOL_RES_304,
    PACE2_PROTOCOL_RES_305,
    PACE2_PROTOCOL_WIREGUARD,
    PACE2_PROTOCOL_RES_307,
    PACE2_PROTOCOL_FAST_SECURE_VPN,
    PACE2_PROTOCOL_RES_309,
    PACE2_PROTOCOL_RES_310,
    PACE2_PROTOCOL_APPVPN,
    PACE2_PROTOCOL_X_VPN,
    PACE2_PROTOCOL_DROID_VPN,
    PACE2_PROTOCOL_DTLS,
    PACE2_PROTOCOL_SMART_VEEPN,
    PACE2_PROTOCOL_APEX_LEGENDS,
    PACE2_PROTOCOL_VPN_365,
    PACE2_PROTOCOL_RES_318,
    PACE2_PROTOCOL_RES_319,
    PACE2_PROTOCOL_ACT_VPN,
    PACE2_PROTOCOL_RES_321,
    PACE2_PROTOCOL_MTPROTO,
    PACE2_PROTOCOL_SHADOWSOCKS,
    PACE2_PROTOCOL_RES_324,
    PACE2_PROTOCOL_ACTMOBILE_SERVICES,
    PACE2_PROTOCOL_ZRTP,
    PACE2_PROTOCOL_PUBG_MOBILE,
    PACE2_PROTOCOL_GAMELOFT_SERVICES,
    PACE2_PROTOCOL_WICKR,
    PACE2_PROTOCOL_ANCHORFREE_SERVICES,
    PACE2_PROTOCOL_RES_331,
    PACE2_PROTOCOL_UNREAL_ENGINE,
    PACE2_PROTOCOL_ZOOM,
    PACE2_PROTOCOL_RES_334,
    PACE2_PROTOCOL_RES_335,
    PACE2_PROTOCOL_IGG,
    PACE2_PROTOCOL_MUDFISH,
    PACE2_PROTOCOL_RES_338,
    PACE2_PROTOCOL_HATCH_CLOUD_GAMING,
    PACE2_PROTOCOL_SOURCE_ENGINE,
    PACE2_PROTOCOL_RES_341,
    PACE2_PROTOCOL_YOUKU_TUDOU,
    PACE2_PROTOCOL_PDANET,
    PACE2_PROTOCOL_HTTP2,
    PACE2_PROTOCOL_TOTOK,
    PACE2_PROTOCOL_BIGO_SERVICES,
    PACE2_PROTOCOL_WEBM,
    PACE2_PROTOCOL_RES_348,
    PACE2_PROTOCOL_IPVANISH,
    PACE2_PROTOCOL_MICROSOFT_SERVICES,
    PACE2_PROTOCOL_RIP,
    PACE2_PROTOCOL_PRIVATEVPN,
    PACE2_PROTOCOL_SPONGYCASTLE,
    PACE2_PROTOCOL_SURFSHARK_VPN,
    PACE2_PROTOCOL_SAPLING_GROWTH_VPN,
    PACE2_PROTOCOL_IEC13818,
    PACE2_PROTOCOL_TENCENT,
    PACE2_PROTOCOL_ULTRAVPN,
    PACE2_PROTOCOL_WECHAT_WORK,
    PACE2_PROTOCOL_ALIBABA,
    PACE2_PROTOCOL_ISIS,
    PACE2_PROTOCOL_RAKNET,
    PACE2_PROTOCOL_INNOVATIVE_CONNECTING,
    PACE2_PROTOCOL_VIPNET,
    PACE2_PROTOCOL_RESP,
    PACE2_PROTOCOL_NOE,
    PACE2_PROTOCOL_AGORA,
    PACE2_PROTOCOL_MIOP,
    PACE2_PROTOCOL_GIOP,
    PACE2_PROTOCOL_ELEX_TECHNOLOGY,
    PACE2_PROTOCOL_JSON_RPC,
    PACE2_PROTOCOL_ZIOP,
    PACE2_PROTOCOL_UFTP,
    PACE2_PROTOCOL_MOONTON,
    PACE2_PROTOCOL_RTPS,
    PACE2_PROTOCOL_ETHEREUM,
    PACE2_PROTOCOL_RES_377,
    PACE2_PROTOCOL_SFLOW,
    PACE2_PROTOCOL_COLLECTD,
    PACE2_PROTOCOL_HL7,
    PACE2_PROTOCOL_AMONG_US,
    PACE2_PROTOCOL_ETHERNET_IP,
    PACE2_PROTOCOL_DNSCRYPT,
    PACE2_PROTOCOL_ONC_RPC,
    PACE2_PROTOCOL_VXI_11,
    PACE2_PROTOCOL_HISLIP,
    PACE2_PROTOCOL_IEEE_C37118,
    PACE2_PROTOCOL_IEC_62056,
    PACE2_PROTOCOL_IEEE_1703,
    PACE2_PROTOCOL_GAIJIN_ENTERTAINMENT,
    PACE2_PROTOCOL_MICOWORLD_SERVICES,
    PACE2_PROTOCOL_PHOTON,
    PACE2_PROTOCOL_NINJA_KIWI_SERVICES,
    PACE2_PROTOCOL_KASPERSKY_SERVICES,
    PACE2_PROTOCOL_CALL_OF_DUTY_MOBILE,
    PACE2_PROTOCOL_ACTIVISION_SERVICES,
    PACE2_PROTOCOL_CAMSCANNER,
    PACE2_PROTOCOL_LOL_WILD_RIFT,
    PACE2_PROTOCOL_TASE_2,
    PACE2_PROTOCOL_RES_400,
    PACE2_PROTOCOL_LITECOIN,
    PACE2_PROTOCOL_PORTMAP,
    PACE2_PROTOCOL_HAGO,
    PACE2_PROTOCOL_MIHOYO_GAMES,
    PACE2_PROTOCOL_RES_405,
    PACE2_PROTOCOL_SUPERVPN,
    PACE2_PROTOCOL_ALLJOYN,
    PACE2_PROTOCOL_NOON_VPN,
    PACE2_PROTOCOL_RIOT_GAMES,
    PACE2_PROTOCOL_GARENA_FREE_FIRE,
    PACE2_PROTOCOL_STP,
    PACE2_PROTOCOL_NORD_VPN,
    PACE2_PROTOCOL_LIGHTSAIL_VPN,
    PACE2_PROTOCOL_LLDP,
    PACE2_PROTOCOL_LLC,
    PACE2_PROTOCOL_DINGTALK,
    PACE2_PROTOCOL_MANUFACTURING_MESSAGING_SPECIFICATION,
    PACE2_PROTOCOL_STOMP,
    PACE2_PROTOCOL_ETHERNET_POWERLINK,
    PACE2_PROTOCOL_UBITUS,
    PACE2_PROTOCOL_ETHERCAT,
    PACE2_PROTOCOL_ARP,
    PACE2_PROTOCOL_PROFINET,
    PACE2_PROTOCOL_OUROBOROS,
    PACE2_PROTOCOL_EIGRP,
    PACE2_PROTOCOL_CIP,
    PACE2_PROTOCOL_CDP,
    PACE2_PROTOCOL_VALORANT,
    PACE2_PROTOCOL_VTP,
    PACE2_PROTOCOL_MMSE,
    PACE2_PROTOCOL_DTP,
    PACE2_PROTOCOL_NVIDIA,
    PACE2_PROTOCOL_UBISOFT,
    PACE2_PROTOCOL_VRRP,
    PACE2_PROTOCOL_PPPOE,
    PACE2_PROTOCOL_VXLAN,
    PACE2_PROTOCOL_CLOUDFLARE,
    PACE2_PROTOCOL_FINS,
    PACE2_PROTOCOL_PFCP,
    PACE2_PROTOCOL_LOST_ARK,
    PACE2_PROTOCOL_POLKADOT,
    PACE2_PROTOCOL_MUMBLE,
    PACE2_PROTOCOL_NARAKA_BLADEPOINT,
    PACE2_PROTOCOL_FINAL_FANTASY_XIV,
    PACE2_PROTOCOL_ETHERNET,
    PACE2_PROTOCOL_NHRP,
    PACE2_PROTOCOL_PINDUODUO,
    PACE2_PROTOCOL_CISCO_METADATA,
    PACE2_PROTOCOL_IEEE_8021Q,
    PACE2_PROTOCOL_IEEE_8021AD,
    PACE2_PROTOCOL_MPLS,
    PACE2_PROTOCOL_FALL_GUYS,
    PACE2_PROTOCOL_RES_453,
    PACE2_PROTOCOL_IEC_61850_GOOSE,
    PACE2_PROTOCOL_IEC_61850_SV,
    PACE2_PROTOCOL_IEEE_8021AH,
    PACE2_PROTOCOL_NETEASE,
    PACE2_PROTOCOL_H245,
    PACE2_PROTOCOL_H225,
    PACE2_PROTOCOL_T120,
    PACE2_PROTOCOL_HIREZ_SERVICES,
    PACE2_PROTOCOL_CRYPTIC_STUDIOS,
    PACE2_PROTOCOL_CCP_GAMES,
    PACE2_PROTOCOL_WARGAMING_NET,
    PACE2_PROTOCOL_KAFKA,
    PACE2_PROTOCOL_DIDI_SERVICES,
    PACE2_PROTOCOL_V4,
    PACE2_PROTOCOL_PTP,
    PACE2_PROTOCOL_ESCAPE_FROM_TARKOV,
    PACE2_PROTOCOL_ZANGI,
    PACE2_PROTOCOL_NOMACHINE_NX,

    PACE2_PROTOCOL_COUNT

} PACE2_protocols;

typedef PACE2_DEPRECATED("Protocol is marked as deprecated and should not be used anymore.") PACE2_protocols PACE2_deprecated_protocols;
#define PACE2_PROTOCOL_MUTE ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_10)
#define PACE2_PROTOCOL_MSN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_14)
#define PACE2_PROTOCOL_USENET ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_23)
#define PACE2_PROTOCOL_MANOLITO ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_28)
#define PACE2_PROTOCOL_PANDO ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_35)
#define PACE2_PROTOCOL_SOPCAST ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_44)
#define PACE2_PROTOCOL_TVUPLAYER ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_45)
#define PACE2_PROTOCOL_PPSTREAM ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_46)
#define PACE2_PROTOCOL_FEIDIAN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_56)
#define PACE2_PROTOCOL_STEAM ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_66)
#define PACE2_PROTOCOL_HALFLIFE2 ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_67)
#define PACE2_PROTOCOL_CITRIX ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_83)
#define PACE2_PROTOCOL_PCANYWHERE ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_94)
#define PACE2_PROTOCOL_OOVOO ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_112)
#define PACE2_PROTOCOL_FRING ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_113)
#define PACE2_PROTOCOL_MSSQL ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_119)
#define PACE2_PROTOCOL_CLUBPENGUIN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_132)
#define PACE2_PROTOCOL_OSH ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_136)
#define PACE2_PROTOCOL_PS3 ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_137)
#define PACE2_PROTOCOL_WII ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_139)
#define PACE2_PROTOCOL_MULTIMEDIA_MESSAGING ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_147)
#define PACE2_PROTOCOL_IMPLUS ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_158)
#define PACE2_PROTOCOL_IPLAYER ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_162)
#define PACE2_PROTOCOL_SCYDO ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_166)
#define PACE2_PROTOCOL_MOJO ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_169)
#define PACE2_PROTOCOL_GOOBER ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_173)
#define PACE2_PROTOCOL_MYPEOPLE ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_175)
#define PACE2_PROTOCOL_ADOBE_CONNECT ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_179)
#define PACE2_PROTOCOL_JAP ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_180)
#define PACE2_PROTOCOL_WUALA ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_182)
#define PACE2_PROTOCOL_LYNC ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_184)
#define PACE2_PROTOCOL_AUDIOGALAXY ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_186)
#define PACE2_PROTOCOL_MIG33 ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_187)
#define PACE2_PROTOCOL_RHAPSODY ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_188)
#define PACE2_PROTOCOL_SUDAPHONE ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_189)
#define PACE2_PROTOCOL_WEBQQ ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_190)
#define PACE2_PROTOCOL_SILVERLIGHT ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_192)
#define PACE2_PROTOCOL_CYBERGHOST ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_193)
#define PACE2_PROTOCOL_MSEXCHANGE ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_213)
#define PACE2_PROTOCOL_TELEGRAM ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_222)
#define PACE2_PROTOCOL_TWITCH ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_232)
#define PACE2_PROTOCOL_AOWE ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_235)
#define PACE2_PROTOCOL_DOOOK ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_236)
#define PACE2_PROTOCOL_SKYPE_FOR_BUSINESS ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_246)
#define PACE2_PROTOCOL_CRASHPLAN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_256)
#define PACE2_PROTOCOL_FASTTRACK ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_258)
#define PACE2_PROTOCOL_CHATCUBE ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_260)
#define PACE2_PROTOCOL_ZERO_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_291)
#define PACE2_PROTOCOL_TURBO_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_292)
#define PACE2_PROTOCOL_VPN_MASTER ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_293)
#define PACE2_PROTOCOL_SNAP_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_295)
#define PACE2_PROTOCOL_SPEED_VPN_MAD ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_301)
#define PACE2_PROTOCOL_DASH_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_302)
#define PACE2_PROTOCOL_ATOM_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_304)
#define PACE2_PROTOCOL_FORTNITE ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_305)
#define PACE2_PROTOCOL_AMAZE_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_307)
#define PACE2_PROTOCOL_ZPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_309)
#define PACE2_PROTOCOL_WORLD_OF_TANKS ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_310)
#define PACE2_PROTOCOL_COUNTERSTRIKE_GO ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_318)
#define PACE2_PROTOCOL_SUPERVPN_BY_CHENG ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_319)
#define PACE2_PROTOCOL_HOTSPOT_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_321)
#define PACE2_PROTOCOL_ASPHALT9 ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_324)
#define PACE2_PROTOCOL_VPN_PROXY_MASTER_UNLIMITED ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_331)
#define PACE2_PROTOCOL_VPN_PROXY_MASTER ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_334)
#define PACE2_PROTOCOL_DEAD_RIVALS ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_335)
#define PACE2_PROTOCOL_SOMA ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_338)
#define PACE2_PROTOCOL_STREAMCRAFT ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_341)
#define PACE2_PROTOCOL_HIDE_ME_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_348)
#define PACE2_PROTOCOL_ZYNN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_377)
#define PACE2_PROTOCOL_RILEY_CILLIAN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_400)
#define PACE2_PROTOCOL_SNAIL_VPN ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_405)
#define PACE2_PROTOCOL_ROGUE_COMPANY ((PACE2_deprecated_protocols) PACE2_PROTOCOL_RES_453)

/**
 * @brief Struct representing all available protocols as a bitmask.
**/
typedef struct {
#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 ukn:1;                    //!< unknown
    u32 bit:1;                    //!< BitTorrent
    u32 edk:1;                    //!< eDonkey
    u32 gnu:1;                    //!< Gnutella
    u32 winmx:1;                  //!< WinMX
    u32 directconnect:1;          //!< DirectConnect
    u32 apple:1;                  //!< AppleJuice
    u32 soul:1;                   //!< Soulseek
    u32 ares:1;                   //!< Ares
    u32 freenet:1;                //!< Freenet
    PACE2_DEPRECATED("Protocol Mute is marked as deprecated and should not be used anymore.")
    u32 mute:1;                   //!< Mute
    u32 xdcc:1;                   //!< XDCC
    u32 skype:1;                  //!< Skype
    u32 sip:1;                    //!< SIP
    PACE2_DEPRECATED("Protocol MSN is marked as deprecated and should not be used anymore.")
    u32 msn:1;                    //!< MSN
    u32 yahoo:1;                  //!< Yahoo
    u32 oscar:1;                  //!< Oscar
    u32 irc:1;                    //!< IRC
    u32 xmpp:1;                   //!< XMPP
    u32 http:1;                   //!< HTTP
    u32 pop:1;                    //!< POP
    u32 smtp:1;                   //!< SMTP
    u32 imap:1;                   //!< IMAP
    PACE2_DEPRECATED("Protocol Usenet is marked as deprecated and should not be used anymore.")
    u32 usenet:1;                 //!< Usenet
    u32 ftp:1;                    //!< FTP
    u32 dns:1;                    //!< DNS
    u32 rtsp:1;                   //!< RTSP
    u32 filetopia:1;              //!< Filetopia
    PACE2_DEPRECATED("Protocol Manolito is marked as deprecated and should not be used anymore.")
    u32 manolito:1;               //!< Manolito
    u32 imesh:1;                  //!< iMesh
    u32 ssl:1;                    //!< SSL
    u32 flash:1;                  //!< Flash
    u32 mms:1;                    //!< MMS
    u32 mpeg:1;                   //!< MPEG
    u32 quicktime:1;              //!< QuickTime
    PACE2_DEPRECATED("Protocol Pando is marked as deprecated and should not be used anymore.")
    u32 pando:1;                  //!< Pando
    u32 openvpn:1;                //!< OpenVPN
    u32 ipsec:1;                  //!< IPsec
    u32 gre:1;                    //!< GRE
    u32 notdetect:1;              //!< undetected
    u32 h323:1;                   //!< H323
    u32 windowsmedia:1;           //!< WindowsMedia
    u32 realmedia:1;              //!< RealMedia
    u32 tvants:1;                 //!< TvAnts
    PACE2_DEPRECATED("Protocol SopCast is marked as deprecated and should not be used anymore.")
    u32 sopcast:1;                //!< SopCast
    PACE2_DEPRECATED("Protocol TVUPlayer is marked as deprecated and should not be used anymore.")
    u32 tvuplayer:1;              //!< TVUPlayer
    PACE2_DEPRECATED("Protocol PPStream is marked as deprecated and should not be used anymore.")
    u32 ppstream:1;               //!< PPStream
    u32 pplive:1;                 //!< PPLive
    u32 iax:1;                    //!< IAX
    u32 vtun:1;                   //!< VTun
    u32 mgcp:1;                   //!< MGCP
    u32 gadugadu:1;               //!< Gadu-Gadu
    u32 zattoo:1;                 //!< Zattoo
    u32 winny:1;                  //!< WinNY
    u32 qq:1;                     //!< QQ
    u32 qqlive:1;                 //!< QQLive
    PACE2_DEPRECATED("Protocol Feidian is marked as deprecated and should not be used anymore.")
    u32 feidian:1;                //!< Feidian
    u32 ssh:1;                    //!< SSH
    u32 popo:1;                   //!< PoPo
    u32 thunder:1;                //!< Thunder/Webthunder
    u32 uusee:1;                  //!< UUSee
    u32 vnc:1;                    //!< VNC
    u32 icmp:1;                   //!< ICMP
    u32 dhcp:1;                   //!< DHCP
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 dhcp:1;                   //!< DHCP
    u32 icmp:1;                   //!< ICMP
    u32 vnc:1;                    //!< VNC
    u32 uusee:1;                  //!< UUSee
    u32 thunder:1;                //!< Thunder/Webthunder
    u32 popo:1;                   //!< PoPo
    u32 ssh:1;                    //!< SSH
    PACE2_DEPRECATED("Protocol Feidian is marked as deprecated and should not be used anymore.")
    u32 feidian:1;                //!< Feidian
    u32 qqlive:1;                 //!< QQLive
    u32 qq:1;                     //!< QQ
    u32 winny:1;                  //!< WinNY
    u32 zattoo:1;                 //!< Zattoo
    u32 gadugadu:1;               //!< Gadu-Gadu
    u32 mgcp:1;                   //!< MGCP
    u32 vtun:1;                   //!< VTun
    u32 iax:1;                    //!< IAX
    u32 pplive:1;                 //!< PPLive
    PACE2_DEPRECATED("Protocol PPStream is marked as deprecated and should not be used anymore.")
    u32 ppstream:1;               //!< PPStream
    PACE2_DEPRECATED("Protocol TVUPlayer is marked as deprecated and should not be used anymore.")
    u32 tvuplayer:1;              //!< TVUPlayer
    PACE2_DEPRECATED("Protocol SopCast is marked as deprecated and should not be used anymore.")
    u32 sopcast:1;                //!< SopCast
    u32 tvants:1;                 //!< TvAnts
    u32 realmedia:1;              //!< RealMedia
    u32 windowsmedia:1;           //!< WindowsMedia
    u32 h323:1;                   //!< H323
    u32 notdetect:1;              //!< undetected
    u32 gre:1;                    //!< GRE
    u32 ipsec:1;                  //!< IPsec
    u32 openvpn:1;                //!< OpenVPN
    PACE2_DEPRECATED("Protocol Pando is marked as deprecated and should not be used anymore.")
    u32 pando:1;                  //!< Pando
    u32 quicktime:1;              //!< QuickTime
    u32 mpeg:1;                   //!< MPEG
    u32 mms:1;                    //!< MMS
    u32 flash:1;                  //!< Flash
    u32 ssl:1;                    //!< SSL
    u32 imesh:1;                  //!< iMesh
    PACE2_DEPRECATED("Protocol Manolito is marked as deprecated and should not be used anymore.")
    u32 manolito:1;               //!< Manolito
    u32 filetopia:1;              //!< Filetopia
    u32 rtsp:1;                   //!< RTSP
    u32 dns:1;                    //!< DNS
    u32 ftp:1;                    //!< FTP
    PACE2_DEPRECATED("Protocol Usenet is marked as deprecated and should not be used anymore.")
    u32 usenet:1;                 //!< Usenet
    u32 imap:1;                   //!< IMAP
    u32 smtp:1;                   //!< SMTP
    u32 pop:1;                    //!< POP
    u32 http:1;                   //!< HTTP
    u32 xmpp:1;                   //!< XMPP
    u32 irc:1;                    //!< IRC
    u32 oscar:1;                  //!< Oscar
    u32 yahoo:1;                  //!< Yahoo
    PACE2_DEPRECATED("Protocol MSN is marked as deprecated and should not be used anymore.")
    u32 msn:1;                    //!< MSN
    u32 sip:1;                    //!< SIP
    u32 skype:1;                  //!< Skype
    u32 xdcc:1;                   //!< XDCC
    PACE2_DEPRECATED("Protocol Mute is marked as deprecated and should not be used anymore.")
    u32 mute:1;                   //!< Mute
    u32 freenet:1;                //!< Freenet
    u32 ares:1;                   //!< Ares
    u32 soul:1;                   //!< Soulseek
    u32 apple:1;                  //!< AppleJuice
    u32 directconnect:1;          //!< DirectConnect
    u32 winmx:1;                  //!< WinMX
    u32 gnu:1;                    //!< Gnutella
    u32 edk:1;                    //!< eDonkey
    u32 bit:1;                    //!< BitTorrent
    u32 ukn:1;                    //!< unknown
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 igmp:1;                   //!< IGMP
    u32 egp:1;                    //!< EGP
    PACE2_DEPRECATED("Protocol Steam is marked as deprecated and should not be used anymore.")
    u32 steam:1;                  //!< Steam
    PACE2_DEPRECATED("Protocol Half-Life 2 is marked as deprecated and should not be used anymore.")
    u32 hl2:1;                    //!< Half-Life 2
    u32 xbox:1;                   //!< XBOX
    u32 softether:1;              //!< SoftEthernet
    u32 sctp:1;                   //!< SCTP
    u32 smb:1;                    //!< SMB/CIFS
    u32 telnet:1;                 //!< Telnet
    u32 ntp:1;                    //!< NTP
    u32 nfs:1;                    //!< NFS
    u32 rtp:1;                    //!< RTP
    u32 tor:1;                    //!< Tor
    u32 ssdp:1;                   //!< SSDP
    u32 worldofwarcraft:1;        //!< World of Warcraft
    u32 avi:1;                    //!< AVI
    u32 vpnx:1;                   //!< VPN-X
    u32 hamachi:1;                //!< HamachiVPN
    u32 skinny:1;                 //!< Skinny
    PACE2_DEPRECATED("Protocol Citrix is marked as deprecated and should not be used anymore.")
    u32 citrix:1;                 //!< Citrix
    u32 ogg:1;                    //!< OGG
    u32 paltalk:1;                //!< Paltalk
    u32 move:1;                   //!< Move
    u32 postgres:1;               //!< PostgreSQL
    u32 ospf:1;                   //!< OSPF
    u32 bgp:1;                    //!< BGP
    u32 quake:1;                  //!< Quake
    u32 secondlife:1;             //!< Second Life
    u32 mysql:1;                  //!< MySQL
    u32 battlefield:1;            //!< Battlefield
    PACE2_DEPRECATED("Protocol PCAnywhere is marked as deprecated and should not be used anymore.")
    u32 pcanywhere:1;             //!< PCAnywhere
    u32 rdp:1;                    //!< RDP
    u32 snmp:1;                   //!< SNMP
    u32 kontiki:1;                //!< Kontiki
    u32 veohtv:1;                 //!< VeohTV
    u32 kerberos:1;               //!< Kerberos
    u32 ipip:1;                   //!< IP in IP
    u32 gtp:1;                    //!< GTP
    u32 radius:1;                 //!< RADIUS
    u32 stun:1;                   //!< STUN
    u32 syslog:1;                 //!< Syslog
    u32 tds:1;                    //!< TDS
    u32 netbios:1;                //!< NetBIOS
    u32 mdns:1;                   //!< MulticastDNS
    u32 ipp:1;                    //!< IPP
    u32 ldap:1;                   //!< LDAP
    u32 warcraft3:1;              //!< Warcraft 3
    u32 xdmcp:1;                  //!< XDMCP
    PACE2_DEPRECATED("Protocol ooVoo is marked as deprecated and should not be used anymore.")
    u32 oovoo:1;                  //!< ooVoo
    PACE2_DEPRECATED("Protocol Fring is marked as deprecated and should not be used anymore.")
    u32 fring:1;                  //!< Fring
    u32 mapi:1;                   //!< Mapi
    u32 l2tp:1;                   //!< L2TP
    u32 isakmp:1;                 //!< ISAKMP
    u32 rtcp:1;                   //!< RTCP
    u32 tftp:1;                   //!< TFTP
    PACE2_DEPRECATED("Protocol MSSQL is marked as deprecated and should not be used anymore.")
    u32 mssql:1;                  //!< MSSQL
    u32 ppp:1;                    //!< PPP
    u32 pptp:1;                   //!< PPTP
    u32 stealthnet:1;             //!< StealthNet
    u32 sap:1;                    //!< SAP
    u32 icmpv6:1;                 //!< ICMPv6
    u32 yourfreedom:1;            //!< YourFreedom Tunnel
    u32 voipswitchvoiptunnel:1;   //!< VoipSwitch VoIP Tunnel
    u32 dhcpv6:1;                 //!< DHCPv6
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 dhcpv6:1;                 //!< DHCPv6
    u32 voipswitchvoiptunnel:1;   //!< VoipSwitch VoIP Tunnel
    u32 yourfreedom:1;            //!< YourFreedom Tunnel
    u32 icmpv6:1;                 //!< ICMPv6
    u32 sap:1;                    //!< SAP
    u32 stealthnet:1;             //!< StealthNet
    u32 pptp:1;                   //!< PPTP
    u32 ppp:1;                    //!< PPP
    PACE2_DEPRECATED("Protocol MSSQL is marked as deprecated and should not be used anymore.")
    u32 mssql:1;                  //!< MSSQL
    u32 tftp:1;                   //!< TFTP
    u32 rtcp:1;                   //!< RTCP
    u32 isakmp:1;                 //!< ISAKMP
    u32 l2tp:1;                   //!< L2TP
    u32 mapi:1;                   //!< Mapi
    PACE2_DEPRECATED("Protocol Fring is marked as deprecated and should not be used anymore.")
    u32 fring:1;                  //!< Fring
    PACE2_DEPRECATED("Protocol ooVoo is marked as deprecated and should not be used anymore.")
    u32 oovoo:1;                  //!< ooVoo
    u32 xdmcp:1;                  //!< XDMCP
    u32 warcraft3:1;              //!< Warcraft 3
    u32 ldap:1;                   //!< LDAP
    u32 ipp:1;                    //!< IPP
    u32 mdns:1;                   //!< MulticastDNS
    u32 netbios:1;                //!< NetBIOS
    u32 tds:1;                    //!< TDS
    u32 syslog:1;                 //!< Syslog
    u32 stun:1;                   //!< STUN
    u32 radius:1;                 //!< RADIUS
    u32 gtp:1;                    //!< GTP
    u32 ipip:1;                   //!< IP in IP
    u32 kerberos:1;               //!< Kerberos
    u32 veohtv:1;                 //!< VeohTV
    u32 kontiki:1;                //!< Kontiki
    u32 snmp:1;                   //!< SNMP
    u32 rdp:1;                    //!< RDP
    PACE2_DEPRECATED("Protocol PCAnywhere is marked as deprecated and should not be used anymore.")
    u32 pcanywhere:1;             //!< PCAnywhere
    u32 battlefield:1;            //!< Battlefield
    u32 mysql:1;                  //!< MySQL
    u32 secondlife:1;             //!< Second Life
    u32 quake:1;                  //!< Quake
    u32 bgp:1;                    //!< BGP
    u32 ospf:1;                   //!< OSPF
    u32 postgres:1;               //!< PostgreSQL
    u32 move:1;                   //!< Move
    u32 paltalk:1;                //!< Paltalk
    u32 ogg:1;                    //!< OGG
    PACE2_DEPRECATED("Protocol Citrix is marked as deprecated and should not be used anymore.")
    u32 citrix:1;                 //!< Citrix
    u32 skinny:1;                 //!< Skinny
    u32 hamachi:1;                //!< HamachiVPN
    u32 vpnx:1;                   //!< VPN-X
    u32 avi:1;                    //!< AVI
    u32 worldofwarcraft:1;        //!< World of Warcraft
    u32 ssdp:1;                   //!< SSDP
    u32 tor:1;                    //!< Tor
    u32 rtp:1;                    //!< RTP
    u32 nfs:1;                    //!< NFS
    u32 ntp:1;                    //!< NTP
    u32 telnet:1;                 //!< Telnet
    u32 smb:1;                    //!< SMB/CIFS
    u32 sctp:1;                   //!< SCTP
    u32 softether:1;              //!< SoftEthernet
    u32 xbox:1;                   //!< XBOX
    PACE2_DEPRECATED("Protocol Half-Life 2 is marked as deprecated and should not be used anymore.")
    u32 hl2:1;                    //!< Half-Life 2
    PACE2_DEPRECATED("Protocol Steam is marked as deprecated and should not be used anymore.")
    u32 steam:1;                  //!< Steam
    u32 egp:1;                    //!< EGP
    u32 igmp:1;                   //!< IGMP
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 meebo:1;                  //!< Meebo
    u32 funshion:1;               //!< Funshion
    u32 afp:1;                    //!< AFP
    u32 aimini:1;                 //!< Aimini
    PACE2_DEPRECATED("Protocol ClubPenguin is marked as deprecated and should not be used anymore.")
    u32 clubpenguin:1;            //!< ClubPenguin
    u32 florensia:1;              //!< Florensia
    u32 teamspeak:1;              //!< TeamSpeak
    u32 maplestory:1;             //!< MapleStory
    PACE2_DEPRECATED("Protocol Octoshape is marked as deprecated and should not be used anymore.")
    u32 osh:1;                    //!< Octoshape
    PACE2_DEPRECATED("Protocol PS3 is marked as deprecated and should not be used anymore.")
    u32 ps3:1;                    //!< PS3
    u32 dofus:1;                  //!< Dofus
    PACE2_DEPRECATED("Protocol WII is marked as deprecated and should not be used anymore.")
    u32 wii:1;                    //!< WII
    u32 wokf:1;                   //!< World of Kung Fu
    u32 fiesta:1;                 //!< Fiesta
    u32 splashfighter:1;          //!< SplashFighter
    u32 crossfire:1;              //!< CrossFire
    u32 wapwsp:1;                 //!< WAP-WSP
    u32 wapwtpwsp:1;              //!< WAP-WTP-WSP
    u32 wapwtls:1;                //!< WAP-WTLS
    PACE2_DEPRECATED("Protocol MultimediaMessaging is marked as deprecated and should not be used anymore.")
    u32 multimediamessaging:1;    //!< MultimediaMessaging
    u32 guildwars:1;              //!< Guildwars
    u32 armagetron:1;             //!< Armagetron
    u32 rim_prop:1;               //!< RIM proprietary
    u32 rfactor:1;                //!< rFactor
    u32 rdt:1;                    //!< RealDataTransport
    u32 teamviewer:1;             //!< TeamViewer
    u32 gamekit:1;                //!< GameKit
    u32 netmotion:1;              //!< NetMotion
    u32 teredo:1;                 //!< Teredo
    u32 spotify:1;                //!< Spotify
    PACE2_DEPRECATED("Protocol IMplus is marked as deprecated and should not be used anymore.")
    u32 implus:1;                 //!< IMplus
    u32 ultrasurf:1;              //!< UltraSurf
    u32 viber:1;                  //!< Viber
    u32 tango:1;                  //!< Tango
    PACE2_DEPRECATED("Protocol iPlayer is marked as deprecated and should not be used anymore.")
    u32 iplayer:1;                //!< iPlayer
    u32 jbk3k:1;                  //!< JBK3000
    u32 operamini:1;              //!< Opera Mini
    u32 bolt:1;                   //!< Bolt
    PACE2_DEPRECATED("Protocol Scydo is marked as deprecated and should not be used anymore.")
    u32 scydo:1;                  //!< Scydo
    u32 whatsapp:1;               //!< WhatsApp
    u32 webex:1;                  //!< Webex
    PACE2_DEPRECATED("Protocol Mojo is marked as deprecated and should not be used anymore.")
    u32 mojo:1;                   //!< Mojo
    u32 imo:1;                    //!< IMO
    u32 citrixgoto:1;             //!< CitrixGoTo
    u32 comodounite:1;            //!< ComodoUnite
    PACE2_DEPRECATED("Protocol Goober is marked as deprecated and should not be used anymore.")
    u32 goober:1;                 //!< Goober
    u32 ventrilo:1;               //!< Ventrilo
    PACE2_DEPRECATED("Protocol MyPeople is marked as deprecated and should not be used anymore.")
    u32 mypeople:1;               //!< MyPeople
    u32 websocket:1;              //!< WebSocket
    u32 ebuddy:1;                 //!< eBuddy
    u32 sstp:1;                   //!< SSTP
    PACE2_DEPRECATED("Protocol AdobeConnect is marked as deprecated and should not be used anymore.")
    u32 adobeconnect:1;           //!< AdobeConnect
    PACE2_DEPRECATED("Protocol JAP is marked as deprecated and should not be used anymore.")
    u32 jap:1;                    //!< JAP
    u32 ldp:1;                    //!< LDP
    PACE2_DEPRECATED("Protocol Wuala is marked as deprecated and should not be used anymore.")
    u32 wuala:1;                  //!< Wuala
    u32 msrp:1;                   //!< MSRP
    PACE2_DEPRECATED("Protocol Lync is marked as deprecated and should not be used anymore.")
    u32 lync:1;                   //!< Lync
    u32 socks:1;                  //!< Socks
    PACE2_DEPRECATED("Protocol Audiogalaxy is marked as deprecated and should not be used anymore.")
    u32 audiogalaxy:1;            //!< Audiogalaxy
    PACE2_DEPRECATED("Protocol Mig33 is marked as deprecated and should not be used anymore.")
    u32 mig33:1;                  //!< Mig33
    PACE2_DEPRECATED("Protocol Rhapsody is marked as deprecated and should not be used anymore.")
    u32 rhapsody:1;               //!< Rhapsody
    PACE2_DEPRECATED("Protocol Sudaphone is marked as deprecated and should not be used anymore.")
    u32 sudaphone:1;              //!< Sudaphone
    PACE2_DEPRECATED("Protocol WebQQ is marked as deprecated and should not be used anymore.")
    u32 webqq:1;                  //!< WebQQ
    u32 pdproxy:1;                //!< PDProxy
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 pdproxy:1;                //!< PDProxy
    PACE2_DEPRECATED("Protocol WebQQ is marked as deprecated and should not be used anymore.")
    u32 webqq:1;                  //!< WebQQ
    PACE2_DEPRECATED("Protocol Sudaphone is marked as deprecated and should not be used anymore.")
    u32 sudaphone:1;              //!< Sudaphone
    PACE2_DEPRECATED("Protocol Rhapsody is marked as deprecated and should not be used anymore.")
    u32 rhapsody:1;               //!< Rhapsody
    PACE2_DEPRECATED("Protocol Mig33 is marked as deprecated and should not be used anymore.")
    u32 mig33:1;                  //!< Mig33
    PACE2_DEPRECATED("Protocol Audiogalaxy is marked as deprecated and should not be used anymore.")
    u32 audiogalaxy:1;            //!< Audiogalaxy
    u32 socks:1;                  //!< Socks
    PACE2_DEPRECATED("Protocol Lync is marked as deprecated and should not be used anymore.")
    u32 lync:1;                   //!< Lync
    u32 msrp:1;                   //!< MSRP
    PACE2_DEPRECATED("Protocol Wuala is marked as deprecated and should not be used anymore.")
    u32 wuala:1;                  //!< Wuala
    u32 ldp:1;                    //!< LDP
    PACE2_DEPRECATED("Protocol JAP is marked as deprecated and should not be used anymore.")
    u32 jap:1;                    //!< JAP
    PACE2_DEPRECATED("Protocol AdobeConnect is marked as deprecated and should not be used anymore.")
    u32 adobeconnect:1;           //!< AdobeConnect
    u32 sstp:1;                   //!< SSTP
    u32 ebuddy:1;                 //!< eBuddy
    u32 websocket:1;              //!< WebSocket
    PACE2_DEPRECATED("Protocol MyPeople is marked as deprecated and should not be used anymore.")
    u32 mypeople:1;               //!< MyPeople
    u32 ventrilo:1;               //!< Ventrilo
    PACE2_DEPRECATED("Protocol Goober is marked as deprecated and should not be used anymore.")
    u32 goober:1;                 //!< Goober
    u32 comodounite:1;            //!< ComodoUnite
    u32 citrixgoto:1;             //!< CitrixGoTo
    u32 imo:1;                    //!< IMO
    PACE2_DEPRECATED("Protocol Mojo is marked as deprecated and should not be used anymore.")
    u32 mojo:1;                   //!< Mojo
    u32 webex:1;                  //!< Webex
    u32 whatsapp:1;               //!< WhatsApp
    PACE2_DEPRECATED("Protocol Scydo is marked as deprecated and should not be used anymore.")
    u32 scydo:1;                  //!< Scydo
    u32 bolt:1;                   //!< Bolt
    u32 operamini:1;              //!< Opera Mini
    u32 jbk3k:1;                  //!< JBK3000
    PACE2_DEPRECATED("Protocol iPlayer is marked as deprecated and should not be used anymore.")
    u32 iplayer:1;                //!< iPlayer
    u32 tango:1;                  //!< Tango
    u32 viber:1;                  //!< Viber
    u32 ultrasurf:1;              //!< UltraSurf
    PACE2_DEPRECATED("Protocol IMplus is marked as deprecated and should not be used anymore.")
    u32 implus:1;                 //!< IMplus
    u32 spotify:1;                //!< Spotify
    u32 teredo:1;                 //!< Teredo
    u32 netmotion:1;              //!< NetMotion
    u32 gamekit:1;                //!< GameKit
    u32 teamviewer:1;             //!< TeamViewer
    u32 rdt:1;                    //!< RealDataTransport
    u32 rfactor:1;                //!< rFactor
    u32 rim_prop:1;               //!< RIM proprietary
    u32 armagetron:1;             //!< Armagetron
    u32 guildwars:1;              //!< Guildwars
    PACE2_DEPRECATED("Protocol MultimediaMessaging is marked as deprecated and should not be used anymore.")
    u32 multimediamessaging:1;    //!< MultimediaMessaging
    u32 wapwtls:1;                //!< WAP-WTLS
    u32 wapwtpwsp:1;              //!< WAP-WTP-WSP
    u32 wapwsp:1;                 //!< WAP-WSP
    u32 crossfire:1;              //!< CrossFire
    u32 splashfighter:1;          //!< SplashFighter
    u32 fiesta:1;                 //!< Fiesta
    u32 wokf:1;                   //!< World of Kung Fu
    PACE2_DEPRECATED("Protocol WII is marked as deprecated and should not be used anymore.")
    u32 wii:1;                    //!< WII
    u32 dofus:1;                  //!< Dofus
    PACE2_DEPRECATED("Protocol PS3 is marked as deprecated and should not be used anymore.")
    u32 ps3:1;                    //!< PS3
    PACE2_DEPRECATED("Protocol Octoshape is marked as deprecated and should not be used anymore.")
    u32 osh:1;                    //!< Octoshape
    u32 maplestory:1;             //!< MapleStory
    u32 teamspeak:1;              //!< TeamSpeak
    u32 florensia:1;              //!< Florensia
    PACE2_DEPRECATED("Protocol ClubPenguin is marked as deprecated and should not be used anymore.")
    u32 clubpenguin:1;            //!< ClubPenguin
    u32 aimini:1;                 //!< Aimini
    u32 afp:1;                    //!< AFP
    u32 funshion:1;               //!< Funshion
    u32 meebo:1;                  //!< Meebo
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    PACE2_DEPRECATED("Protocol Silverlight is marked as deprecated and should not be used anymore.")
    u32 silverlight:1;            //!< Silverlight
    PACE2_DEPRECATED("Protocol CyberGhost is marked as deprecated and should not be used anymore.")
    u32 cyberghost:1;             //!< CyberGhost
    u32 fix:1;                    //!< FIX
    u32 oracledb:1;               //!< Oracle Database
    u32 rsvp:1;                   //!< RSVP
    u32 lotusnotes:1;             //!< Lotus Notes
    u32 poisonivy:1;              //!< Poison Ivy
    u32 netflow:1;                //!< NetFlow
    u32 wechat:1;                 //!< WeChat
    u32 soap:1;                   //!< SOAP
    u32 kik:1;                    //!< Kik Messenger
    u32 mplus:1;                  //!< MPlus
    u32 line:1;                   //!< Line
    u32 iperf:1;                  //!< Iperf
    u32 diameter:1;               //!< Diameter
    u32 hike:1;                   //!< Hike Messenger
    u32 mqtt:1;                   //!< MQTT
    u32 dce_rpc:1;                //!< DCE RPC
    u32 idrive:1;                 //!< IDrive
    u32 dota2:1;                  //!< Dota 2
    u32 qvod:1;                   //!< QVOD
    PACE2_DEPRECATED("Protocol Microsoft Exchange is marked as deprecated and should not be used anymore.")
    u32 msexchange:1;             //!< Microsoft Exchange
    u32 msnav:1;                  //!< Microsoft Dynamics NAV
    u32 tibia:1;                  //!< Tibia
    u32 kakao:1;                  //!< Kakao
    u32 destiny:1;                //!< Destiny
    u32 vyprvpn:1;                //!< VyprVPN
    u32 tacacs:1;                 //!< TACACS
    u32 threema:1;                //!< Threema
    u32 ntlm:1;                   //!< NTLM
    PACE2_DEPRECATED("Protocol Telegram is marked as deprecated and should not be used anymore.")
    u32 telegram:1;               //!< Telegram
    u32 whois:1;                  //!< WHOIS
    u32 ipv4:1;                   //!< IPv4
    u32 ipv6:1;                   //!< IPv6
    u32 tcp:1;                    //!< TCP
    u32 udp:1;                    //!< UDP
    u32 rtmp:1;                   //!< RTMP
    u32 nntp:1;                   //!< NNTP
    u32 flute:1;                  //!< FLUTE
    u32 megaco:1;                 //!< Megaco
    PACE2_DEPRECATED("Protocol Twitch is marked as deprecated and should not be used anymore.")
    u32 twitch:1;                 //!< Twitch
    u32 quic:1;                   //!< QUIC
    u32 fuze:1;                   //!< Fuze Meeting
    PACE2_DEPRECATED("Protocol Age of Warring Empire is marked as deprecated and should not be used anymore.")
    u32 aowe:1;                   //!< Age of Warring Empire
    PACE2_DEPRECATED("Protocol Doook is marked as deprecated and should not be used anymore.")
    u32 doook:1;                  //!< Doook
    u32 speedtest:1;              //!< Speedtest
    u32 iec_61850:1;              //!< IEC 61850
    u32 scada:1;                  //!< SCADA
    u32 shoutcast:1;              //!< Shoutcast
    u32 cassandra_query_language:1;//!< Cassandra Query Language
    u32 cassandra_internode_communication:1;//!< Cassandra Internode Communication
    u32 thrift:1;                 //!< Thrift
    u32 playstation:1;            //!< PlayStation Network
    u32 supercell:1;              //!< Supercell
    PACE2_DEPRECATED("Protocol Skype for Business is marked as deprecated and should not be used anymore.")
    u32 skype_for_business:1;     //!< Skype for Business
    u32 t3:1;                     //!< T3
    u32 srtp:1;                   //!< SRTP
    u32 srtcp:1;                  //!< SRTCP
    u32 league_of_legends:1;      //!< League of Legends
    u32 facebook:1;               //!< Facebook
    u32 battlenet:1;              //!< BattleNet
    u32 wowza:1;                  //!< Wowza
    u32 zalo:1;                   //!< Zalo
    u32 guildwars2:1;             //!< Guild Wars 2
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 guildwars2:1;             //!< Guild Wars 2
    u32 zalo:1;                   //!< Zalo
    u32 wowza:1;                  //!< Wowza
    u32 battlenet:1;              //!< BattleNet
    u32 facebook:1;               //!< Facebook
    u32 league_of_legends:1;      //!< League of Legends
    u32 srtcp:1;                  //!< SRTCP
    u32 srtp:1;                   //!< SRTP
    u32 t3:1;                     //!< T3
    PACE2_DEPRECATED("Protocol Skype for Business is marked as deprecated and should not be used anymore.")
    u32 skype_for_business:1;     //!< Skype for Business
    u32 supercell:1;              //!< Supercell
    u32 playstation:1;            //!< PlayStation Network
    u32 thrift:1;                 //!< Thrift
    u32 cassandra_internode_communication:1;//!< Cassandra Internode Communication
    u32 cassandra_query_language:1;//!< Cassandra Query Language
    u32 shoutcast:1;              //!< Shoutcast
    u32 scada:1;                  //!< SCADA
    u32 iec_61850:1;              //!< IEC 61850
    u32 speedtest:1;              //!< Speedtest
    PACE2_DEPRECATED("Protocol Doook is marked as deprecated and should not be used anymore.")
    u32 doook:1;                  //!< Doook
    PACE2_DEPRECATED("Protocol Age of Warring Empire is marked as deprecated and should not be used anymore.")
    u32 aowe:1;                   //!< Age of Warring Empire
    u32 fuze:1;                   //!< Fuze Meeting
    u32 quic:1;                   //!< QUIC
    PACE2_DEPRECATED("Protocol Twitch is marked as deprecated and should not be used anymore.")
    u32 twitch:1;                 //!< Twitch
    u32 megaco:1;                 //!< Megaco
    u32 flute:1;                  //!< FLUTE
    u32 nntp:1;                   //!< NNTP
    u32 rtmp:1;                   //!< RTMP
    u32 udp:1;                    //!< UDP
    u32 tcp:1;                    //!< TCP
    u32 ipv6:1;                   //!< IPv6
    u32 ipv4:1;                   //!< IPv4
    u32 whois:1;                  //!< WHOIS
    PACE2_DEPRECATED("Protocol Telegram is marked as deprecated and should not be used anymore.")
    u32 telegram:1;               //!< Telegram
    u32 ntlm:1;                   //!< NTLM
    u32 threema:1;                //!< Threema
    u32 tacacs:1;                 //!< TACACS
    u32 vyprvpn:1;                //!< VyprVPN
    u32 destiny:1;                //!< Destiny
    u32 kakao:1;                  //!< Kakao
    u32 tibia:1;                  //!< Tibia
    u32 msnav:1;                  //!< Microsoft Dynamics NAV
    PACE2_DEPRECATED("Protocol Microsoft Exchange is marked as deprecated and should not be used anymore.")
    u32 msexchange:1;             //!< Microsoft Exchange
    u32 qvod:1;                   //!< QVOD
    u32 dota2:1;                  //!< Dota 2
    u32 idrive:1;                 //!< IDrive
    u32 dce_rpc:1;                //!< DCE RPC
    u32 mqtt:1;                   //!< MQTT
    u32 hike:1;                   //!< Hike Messenger
    u32 diameter:1;               //!< Diameter
    u32 iperf:1;                  //!< Iperf
    u32 line:1;                   //!< Line
    u32 mplus:1;                  //!< MPlus
    u32 kik:1;                    //!< Kik Messenger
    u32 soap:1;                   //!< SOAP
    u32 wechat:1;                 //!< WeChat
    u32 netflow:1;                //!< NetFlow
    u32 poisonivy:1;              //!< Poison Ivy
    u32 lotusnotes:1;             //!< Lotus Notes
    u32 rsvp:1;                   //!< RSVP
    u32 oracledb:1;               //!< Oracle Database
    u32 fix:1;                    //!< FIX
    PACE2_DEPRECATED("Protocol CyberGhost is marked as deprecated and should not be used anymore.")
    u32 cyberghost:1;             //!< CyberGhost
    PACE2_DEPRECATED("Protocol Silverlight is marked as deprecated and should not be used anymore.")
    u32 silverlight:1;            //!< Silverlight
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    PACE2_DEPRECATED("Protocol CrashPlan is marked as deprecated and should not be used anymore.")
    u32 crashplan:1;              //!< CrashPlan
    u32 ea_origin:1;              //!< EA Origin
    PACE2_DEPRECATED("Protocol Fasttrack is marked as deprecated and should not be used anymore.")
    u32 fasttrack:1;              //!< Fasttrack
    u32 icecast:1;                //!< Icecast
    PACE2_DEPRECATED("Protocol ChatCube is marked as deprecated and should not be used anymore.")
    u32 chatcube:1;               //!< ChatCube
    u32 good:1;                   //!< Good
    u32 path_of_exile:1;          //!< Path of Exile
    u32 pc_over_ip:1;             //!< PCoIP
    u32 ocsp:1;                   //!< OCSP
    u32 modbus:1;                 //!< Modbus
    u32 fasp:1;                   //!< FASP
    u32 ica:1;                    //!< ICA
    u32 opc_ua:1;                 //!< OPC UA
    u32 openflow:1;               //!< OpenFlow
    u32 ipmi:1;                   //!< IPMI
    u32 amqp:1;                   //!< AMQP
    u32 s7comm:1;                 //!< S7comm
    u32 off:1;                    //!< OFF
    u32 rsync:1;                  //!< Rsync
    u32 zeromq:1;                 //!< ZeroMQ
    u32 dnp3:1;                   //!< DNP3
    u32 bacnet:1;                 //!< BACNET
    u32 nintendo_network:1;       //!< Nintendo Network
    u32 ppfilm:1;                 //!< PPFilm
    u32 bitcoin:1;                //!< Bitcoin
    u32 letv:1;                   //!< LeTV
    u32 psiphon:1;                //!< Psiphon
    u32 zero:1;                   //!< Zero
    u32 synology:1;               //!< Synology
    u32 coap:1;                   //!< CoAP
    u32 lineage_2:1;              //!< Lineage 2
    u32 ipfix:1;                  //!< IPFIX
    u32 minecraft:1;              //!< Minecraft
    u32 stratum_mining:1;         //!< Stratum Mining
    u32 lpd:1;                    //!< Local Peer Discovery
    PACE2_DEPRECATED("Protocol Zero VPN is marked as deprecated and should not be used anymore.")
    u32 zero_vpn:1;               //!< Zero VPN
    PACE2_DEPRECATED("Protocol Turbo VPN is marked as deprecated and should not be used anymore.")
    u32 turbo_vpn:1;              //!< Turbo VPN
    PACE2_DEPRECATED("Protocol VPN Master is marked as deprecated and should not be used anymore.")
    u32 vpn_master:1;             //!< VPN Master
    u32 iqiyi:1;                  //!< Iqiyi
    PACE2_DEPRECATED("Protocol Snap VPN is marked as deprecated and should not be used anymore.")
    u32 snap_vpn:1;               //!< Snap VPN
    u32 vpn_unlimited:1;          //!< VPN Unlimited
    u32 express_vpn:1;            //!< ExpressVPN
    u32 tunnelguru:1;             //!< TunnelGuru
    u32 botim:1;                  //!< BOTIM
    u32 hi_vpn:1;                 //!< Hi VPN
    PACE2_DEPRECATED("Protocol Speed VPN MobileAppDev is marked as deprecated and should not be used anymore.")
    u32 speed_vpn_mad:1;          //!< Speed VPN MobileAppDev
    PACE2_DEPRECATED("Protocol Dash VPN is marked as deprecated and should not be used anymore.")
    u32 dash_vpn:1;               //!< Dash VPN
    u32 yeecall:1;                //!< YeeCall
    PACE2_DEPRECATED("Protocol Atom VPN is marked as deprecated and should not be used anymore.")
    u32 atom_vpn:1;               //!< Atom VPN
    PACE2_DEPRECATED("Protocol Fortnite is marked as deprecated and should not be used anymore.")
    u32 fortnite:1;               //!< Fortnite
    u32 wireguard:1;              //!< WireGuard
    PACE2_DEPRECATED("Protocol Amaze VPN is marked as deprecated and should not be used anymore.")
    u32 amaze_vpn:1;              //!< Amaze VPN
    u32 fast_secure_vpn:1;        //!< Fast Secure VPN
    PACE2_DEPRECATED("Protocol ZPN is marked as deprecated and should not be used anymore.")
    u32 zpn:1;                    //!< ZPN
    PACE2_DEPRECATED("Protocol World of Tanks is marked as deprecated and should not be used anymore.")
    u32 world_of_tanks:1;         //!< World of Tanks
    u32 appvpn:1;                 //!< AppVPN
    u32 x_vpn:1;                  //!< X-VPN
    u32 droid_vpn:1;              //!< Droid VPN
    u32 dtls:1;                   //!< DTLS
    u32 smart_veepn:1;            //!< Smart VeePN
    u32 apex_legends:1;           //!< Apex Legends
    u32 vpn_365:1;                //!< VPN 365
    PACE2_DEPRECATED("Protocol CounterStrikeGO is marked as deprecated and should not be used anymore.")
    u32 counterstrike_go:1;       //!< CounterStrikeGO
    PACE2_DEPRECATED("Protocol SuperVPN by Cheng is marked as deprecated and should not be used anymore.")
    u32 supervpn_by_cheng:1;      //!< SuperVPN by Cheng
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    PACE2_DEPRECATED("Protocol SuperVPN by Cheng is marked as deprecated and should not be used anymore.")
    u32 supervpn_by_cheng:1;      //!< SuperVPN by Cheng
    PACE2_DEPRECATED("Protocol CounterStrikeGO is marked as deprecated and should not be used anymore.")
    u32 counterstrike_go:1;       //!< CounterStrikeGO
    u32 vpn_365:1;                //!< VPN 365
    u32 apex_legends:1;           //!< Apex Legends
    u32 smart_veepn:1;            //!< Smart VeePN
    u32 dtls:1;                   //!< DTLS
    u32 droid_vpn:1;              //!< Droid VPN
    u32 x_vpn:1;                  //!< X-VPN
    u32 appvpn:1;                 //!< AppVPN
    PACE2_DEPRECATED("Protocol World of Tanks is marked as deprecated and should not be used anymore.")
    u32 world_of_tanks:1;         //!< World of Tanks
    PACE2_DEPRECATED("Protocol ZPN is marked as deprecated and should not be used anymore.")
    u32 zpn:1;                    //!< ZPN
    u32 fast_secure_vpn:1;        //!< Fast Secure VPN
    PACE2_DEPRECATED("Protocol Amaze VPN is marked as deprecated and should not be used anymore.")
    u32 amaze_vpn:1;              //!< Amaze VPN
    u32 wireguard:1;              //!< WireGuard
    PACE2_DEPRECATED("Protocol Fortnite is marked as deprecated and should not be used anymore.")
    u32 fortnite:1;               //!< Fortnite
    PACE2_DEPRECATED("Protocol Atom VPN is marked as deprecated and should not be used anymore.")
    u32 atom_vpn:1;               //!< Atom VPN
    u32 yeecall:1;                //!< YeeCall
    PACE2_DEPRECATED("Protocol Dash VPN is marked as deprecated and should not be used anymore.")
    u32 dash_vpn:1;               //!< Dash VPN
    PACE2_DEPRECATED("Protocol Speed VPN MobileAppDev is marked as deprecated and should not be used anymore.")
    u32 speed_vpn_mad:1;          //!< Speed VPN MobileAppDev
    u32 hi_vpn:1;                 //!< Hi VPN
    u32 botim:1;                  //!< BOTIM
    u32 tunnelguru:1;             //!< TunnelGuru
    u32 express_vpn:1;            //!< ExpressVPN
    u32 vpn_unlimited:1;          //!< VPN Unlimited
    PACE2_DEPRECATED("Protocol Snap VPN is marked as deprecated and should not be used anymore.")
    u32 snap_vpn:1;               //!< Snap VPN
    u32 iqiyi:1;                  //!< Iqiyi
    PACE2_DEPRECATED("Protocol VPN Master is marked as deprecated and should not be used anymore.")
    u32 vpn_master:1;             //!< VPN Master
    PACE2_DEPRECATED("Protocol Turbo VPN is marked as deprecated and should not be used anymore.")
    u32 turbo_vpn:1;              //!< Turbo VPN
    PACE2_DEPRECATED("Protocol Zero VPN is marked as deprecated and should not be used anymore.")
    u32 zero_vpn:1;               //!< Zero VPN
    u32 lpd:1;                    //!< Local Peer Discovery
    u32 stratum_mining:1;         //!< Stratum Mining
    u32 minecraft:1;              //!< Minecraft
    u32 ipfix:1;                  //!< IPFIX
    u32 lineage_2:1;              //!< Lineage 2
    u32 coap:1;                   //!< CoAP
    u32 synology:1;               //!< Synology
    u32 zero:1;                   //!< Zero
    u32 psiphon:1;                //!< Psiphon
    u32 letv:1;                   //!< LeTV
    u32 bitcoin:1;                //!< Bitcoin
    u32 ppfilm:1;                 //!< PPFilm
    u32 nintendo_network:1;       //!< Nintendo Network
    u32 bacnet:1;                 //!< BACNET
    u32 dnp3:1;                   //!< DNP3
    u32 zeromq:1;                 //!< ZeroMQ
    u32 rsync:1;                  //!< Rsync
    u32 off:1;                    //!< OFF
    u32 s7comm:1;                 //!< S7comm
    u32 amqp:1;                   //!< AMQP
    u32 ipmi:1;                   //!< IPMI
    u32 openflow:1;               //!< OpenFlow
    u32 opc_ua:1;                 //!< OPC UA
    u32 ica:1;                    //!< ICA
    u32 fasp:1;                   //!< FASP
    u32 modbus:1;                 //!< Modbus
    u32 ocsp:1;                   //!< OCSP
    u32 pc_over_ip:1;             //!< PCoIP
    u32 path_of_exile:1;          //!< Path of Exile
    u32 good:1;                   //!< Good
    PACE2_DEPRECATED("Protocol ChatCube is marked as deprecated and should not be used anymore.")
    u32 chatcube:1;               //!< ChatCube
    u32 icecast:1;                //!< Icecast
    PACE2_DEPRECATED("Protocol Fasttrack is marked as deprecated and should not be used anymore.")
    u32 fasttrack:1;              //!< Fasttrack
    u32 ea_origin:1;              //!< EA Origin
    PACE2_DEPRECATED("Protocol CrashPlan is marked as deprecated and should not be used anymore.")
    u32 crashplan:1;              //!< CrashPlan
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 act_vpn:1;                //!< Act VPN
    PACE2_DEPRECATED("Protocol Hotspot VPN is marked as deprecated and should not be used anymore.")
    u32 hotspot_vpn:1;            //!< Hotspot VPN
    u32 mtproto:1;                //!< MTProto
    u32 shadowsocks:1;            //!< Shadowsocks
    PACE2_DEPRECATED("Protocol Asphalt9 is marked as deprecated and should not be used anymore.")
    u32 asphalt9:1;               //!< Asphalt9
    u32 actmobile_services:1;     //!< Actmobile Services
    u32 zrtp:1;                   //!< ZRTP
    u32 pubg_mobile:1;            //!< PUBG Mobile
    u32 gameloft_services:1;      //!< Gameloft Services
    u32 wickr:1;                  //!< Wickr
    u32 anchorfree_services:1;    //!< Anchorfree Services
    PACE2_DEPRECATED("Protocol VPN Proxy Master Unlimited is marked as deprecated and should not be used anymore.")
    u32 vpn_proxy_master_unlimited:1;//!< VPN Proxy Master Unlimited
    u32 unreal_engine:1;          //!< Unreal Engine
    u32 zoom:1;                   //!< Zoom
    PACE2_DEPRECATED("Protocol VPN Proxy Master is marked as deprecated and should not be used anymore.")
    u32 vpn_proxy_master:1;       //!< VPN Proxy Master
    PACE2_DEPRECATED("Protocol Dead Rivals is marked as deprecated and should not be used anymore.")
    u32 dead_rivals:1;            //!< Dead Rivals
    u32 igg:1;                    //!< IGG
    u32 mudfish:1;                //!< Mudfish
    PACE2_DEPRECATED("Protocol SOMA is marked as deprecated and should not be used anymore.")
    u32 soma:1;                   //!< SOMA
    u32 hatch_cloud_gaming:1;     //!< Hatch Cloud Gaming
    u32 source_engine:1;          //!< Source Engine
    PACE2_DEPRECATED("Protocol Streamcraft is marked as deprecated and should not be used anymore.")
    u32 streamcraft:1;            //!< Streamcraft
    u32 youku_tudou:1;            //!< Youku Tudou
    u32 pdanet:1;                 //!< PdaNet
    u32 http2:1;                  //!< HTTP/2
    u32 totok:1;                  //!< ToTok
    u32 bigo_services:1;          //!< BIGO Services
    u32 webm:1;                   //!< WebM
    PACE2_DEPRECATED("Protocol HideMe VPN is marked as deprecated and should not be used anymore.")
    u32 hide_me_vpn:1;            //!< HideMe VPN
    u32 ipvanish:1;               //!< IPVanish
    u32 microsoft_services:1;     //!< Microsoft Services
    u32 rip:1;                    //!< Routing Information Protocol
    u32 privatevpn:1;             //!< PrivateVPN
    u32 spongycastle:1;           //!< SpongyCastle
    u32 surfshark_vpn:1;          //!< Surfshark VPN
    u32 sapling_growth_vpn:1;     //!< Sapling Growth VPN Services
    u32 iec_13818:1;              //!< IEC 13818
    u32 tencent:1;                //!< Tencent
    u32 ultravpn:1;               //!< UltraVPN
    u32 wechat_work:1;            //!< WeChat Work
    u32 alibaba:1;                //!< Alibaba
    u32 isis:1;                   //!< ISIS
    u32 raknet:1;                 //!< RakNet
    u32 innovative_connecting:1;  //!< Innovative Connecting
    u32 vipnet:1;                 //!< ViPNet
    u32 resp:1;                   //!< RESP
    u32 noe:1;                    //!< New Office Environment
    u32 agora:1;                  //!< Agora
    u32 miop:1;                   //!< MIOP
    u32 giop:1;                   //!< GIOP
    u32 elex_technology:1;        //!< Elex Technology
    u32 json_rpc:1;               //!< JSON-RPC
    u32 ziop:1;                   //!< ZIOP
    u32 uftp:1;                   //!< UFTP
    u32 moonton:1;                //!< Moonton
    u32 rtps:1;                   //!< RTPS
    u32 ethereum:1;               //!< Ethereum
    PACE2_DEPRECATED("Protocol Zynn is marked as deprecated and should not be used anymore.")
    u32 zynn:1;                   //!< Zynn
    u32 sflow:1;                  //!< sFlow
    u32 collectd:1;               //!< collectd
    u32 hl7:1;                    //!< HL7
    u32 among_us:1;               //!< Among Us
    u32 ethernetip:1;             //!< EtherNetIP
    u32 dnscrypt:1;               //!< DNSCrypt
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 dnscrypt:1;               //!< DNSCrypt
    u32 ethernetip:1;             //!< EtherNetIP
    u32 among_us:1;               //!< Among Us
    u32 hl7:1;                    //!< HL7
    u32 collectd:1;               //!< collectd
    u32 sflow:1;                  //!< sFlow
    PACE2_DEPRECATED("Protocol Zynn is marked as deprecated and should not be used anymore.")
    u32 zynn:1;                   //!< Zynn
    u32 ethereum:1;               //!< Ethereum
    u32 rtps:1;                   //!< RTPS
    u32 moonton:1;                //!< Moonton
    u32 uftp:1;                   //!< UFTP
    u32 ziop:1;                   //!< ZIOP
    u32 json_rpc:1;               //!< JSON-RPC
    u32 elex_technology:1;        //!< Elex Technology
    u32 giop:1;                   //!< GIOP
    u32 miop:1;                   //!< MIOP
    u32 agora:1;                  //!< Agora
    u32 noe:1;                    //!< New Office Environment
    u32 resp:1;                   //!< RESP
    u32 vipnet:1;                 //!< ViPNet
    u32 innovative_connecting:1;  //!< Innovative Connecting
    u32 raknet:1;                 //!< RakNet
    u32 isis:1;                   //!< ISIS
    u32 alibaba:1;                //!< Alibaba
    u32 wechat_work:1;            //!< WeChat Work
    u32 ultravpn:1;               //!< UltraVPN
    u32 tencent:1;                //!< Tencent
    u32 iec_13818:1;              //!< IEC 13818
    u32 sapling_growth_vpn:1;     //!< Sapling Growth VPN Services
    u32 surfshark_vpn:1;          //!< Surfshark VPN
    u32 spongycastle:1;           //!< SpongyCastle
    u32 privatevpn:1;             //!< PrivateVPN
    u32 rip:1;                    //!< Routing Information Protocol
    u32 microsoft_services:1;     //!< Microsoft Services
    u32 ipvanish:1;               //!< IPVanish
    PACE2_DEPRECATED("Protocol HideMe VPN is marked as deprecated and should not be used anymore.")
    u32 hide_me_vpn:1;            //!< HideMe VPN
    u32 webm:1;                   //!< WebM
    u32 bigo_services:1;          //!< BIGO Services
    u32 totok:1;                  //!< ToTok
    u32 http2:1;                  //!< HTTP/2
    u32 pdanet:1;                 //!< PdaNet
    u32 youku_tudou:1;            //!< Youku Tudou
    PACE2_DEPRECATED("Protocol Streamcraft is marked as deprecated and should not be used anymore.")
    u32 streamcraft:1;            //!< Streamcraft
    u32 source_engine:1;          //!< Source Engine
    u32 hatch_cloud_gaming:1;     //!< Hatch Cloud Gaming
    PACE2_DEPRECATED("Protocol SOMA is marked as deprecated and should not be used anymore.")
    u32 soma:1;                   //!< SOMA
    u32 mudfish:1;                //!< Mudfish
    u32 igg:1;                    //!< IGG
    PACE2_DEPRECATED("Protocol Dead Rivals is marked as deprecated and should not be used anymore.")
    u32 dead_rivals:1;            //!< Dead Rivals
    PACE2_DEPRECATED("Protocol VPN Proxy Master is marked as deprecated and should not be used anymore.")
    u32 vpn_proxy_master:1;       //!< VPN Proxy Master
    u32 zoom:1;                   //!< Zoom
    u32 unreal_engine:1;          //!< Unreal Engine
    PACE2_DEPRECATED("Protocol VPN Proxy Master Unlimited is marked as deprecated and should not be used anymore.")
    u32 vpn_proxy_master_unlimited:1;//!< VPN Proxy Master Unlimited
    u32 anchorfree_services:1;    //!< Anchorfree Services
    u32 wickr:1;                  //!< Wickr
    u32 gameloft_services:1;      //!< Gameloft Services
    u32 pubg_mobile:1;            //!< PUBG Mobile
    u32 zrtp:1;                   //!< ZRTP
    u32 actmobile_services:1;     //!< Actmobile Services
    PACE2_DEPRECATED("Protocol Asphalt9 is marked as deprecated and should not be used anymore.")
    u32 asphalt9:1;               //!< Asphalt9
    u32 shadowsocks:1;            //!< Shadowsocks
    u32 mtproto:1;                //!< MTProto
    PACE2_DEPRECATED("Protocol Hotspot VPN is marked as deprecated and should not be used anymore.")
    u32 hotspot_vpn:1;            //!< Hotspot VPN
    u32 act_vpn:1;                //!< Act VPN
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 onc_rpc:1;                //!< ONC-RPC
    u32 vxi_11:1;                 //!< VXI-11
    u32 hislip:1;                 //!< HiSLIP
    u32 ieee_c37118:1;            //!< IEEE C37118
    u32 iec_62056:1;              //!< IEC 62056
    u32 ieee_1703:1;              //!< IEEE 1703
    u32 gaijin_entertainment:1;   //!< Gaijin Entertainment
    u32 micoworld_services:1;     //!< MicoWorld Services
    u32 photon:1;                 //!< Photon
    u32 ninja_kiwi_services:1;    //!< Ninja Kiwi Services
    u32 kaspersky_services:1;     //!< Kaspersky Services
    u32 call_of_duty_mobile:1;    //!< Call of Duty Mobile
    u32 activision_services:1;    //!< Activision Services
    u32 camscanner:1;             //!< CamScanner
    u32 lol_wild_rift:1;          //!< LoL Wild Rift
    u32 tase2:1;                  //!< TASE 2
    PACE2_DEPRECATED("Protocol Riley Cillian is marked as deprecated and should not be used anymore.")
    u32 riley_cillian:1;          //!< Riley Cillian
    u32 litecoin:1;               //!< Litecoin
    u32 portmap:1;                //!< Portmap
    u32 hago:1;                   //!< Hago
    u32 mihoyo_games:1;           //!< miHoYo Games
    PACE2_DEPRECATED("Protocol Snail VPN is marked as deprecated and should not be used anymore.")
    u32 snail_vpn:1;              //!< Snail VPN
    u32 supervpn:1;               //!< SuperVPN
    u32 alljoyn:1;                //!< AllJoyn
    u32 noon_vpn:1;               //!< Noon VPN
    u32 riot_games:1;             //!< Riot Games
    u32 garena_free_fire:1;       //!< Garena Free Fire
    u32 stp:1;                    //!< STP
    u32 nord_vpn:1;               //!< Nord VPN
    u32 lightsail_vpn:1;          //!< Lightsail VPN
    u32 lldp:1;                   //!< LLDP
    u32 llc:1;                    //!< LLC
    u32 dingtalk:1;               //!< DingTalk
    u32 manufacturing_messaging_spec:1;//!< Manufacturing Messaging Specification
    u32 stomp:1;                  //!< STOMP
    u32 ethernet_powerlink:1;     //!< Ethernet Powerlink
    u32 ubitus:1;                 //!< Ubitus
    u32 ethercat:1;               //!< EtherCAT
    u32 arp:1;                    //!< ARP
    u32 profinet:1;               //!< Profinet
    u32 ouroboros:1;              //!< Ouroboros
    u32 eigrp:1;                  //!< EIGRP
    u32 cip:1;                    //!< Common Industrial Protocol
    u32 cdp:1;                    //!< Cisco Discovery Protocol
    u32 valorant:1;               //!< Valorant
    u32 vtp:1;                    //!< VLAN Trunking Protocol
    u32 mmse:1;                   //!< Multimedia Messaging Service Encapsulation
    u32 dtp:1;                    //!< Dynamic Trunking Protocol
    u32 nvidia:1;                 //!< Nvidia
    u32 ubisoft:1;                //!< Ubisoft
    u32 vrrp:1;                   //!< Virtual Router Redundancy Protocol
    u32 pppoe:1;                  //!< PPPoE
    u32 vxlan:1;                  //!< VXLAN
    u32 cloudflare:1;             //!< Cloudflare
    u32 fins:1;                   //!< FINS
    u32 pfcp:1;                   //!< Packet Forwarding Control Protocol
    u32 lost_ark:1;               //!< Lost Ark
    u32 polkadot:1;               //!< Polkadot
    u32 mumble:1;                 //!< Mumble
    u32 naraka_bladepoint:1;      //!< Naraka Bladepoint
    u32 final_fantasy_xiv:1;      //!< Final Fantasy XIV
    u32 ethernet:1;               //!< Ethernet
    u32 nhrp:1;                   //!< Next Hop Resolution Protocol
    u32 pinduoduo:1;              //!< Pinduoduo
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 pinduoduo:1;              //!< Pinduoduo
    u32 nhrp:1;                   //!< Next Hop Resolution Protocol
    u32 ethernet:1;               //!< Ethernet
    u32 final_fantasy_xiv:1;      //!< Final Fantasy XIV
    u32 naraka_bladepoint:1;      //!< Naraka Bladepoint
    u32 mumble:1;                 //!< Mumble
    u32 polkadot:1;               //!< Polkadot
    u32 lost_ark:1;               //!< Lost Ark
    u32 pfcp:1;                   //!< Packet Forwarding Control Protocol
    u32 fins:1;                   //!< FINS
    u32 cloudflare:1;             //!< Cloudflare
    u32 vxlan:1;                  //!< VXLAN
    u32 pppoe:1;                  //!< PPPoE
    u32 vrrp:1;                   //!< Virtual Router Redundancy Protocol
    u32 ubisoft:1;                //!< Ubisoft
    u32 nvidia:1;                 //!< Nvidia
    u32 dtp:1;                    //!< Dynamic Trunking Protocol
    u32 mmse:1;                   //!< Multimedia Messaging Service Encapsulation
    u32 vtp:1;                    //!< VLAN Trunking Protocol
    u32 valorant:1;               //!< Valorant
    u32 cdp:1;                    //!< Cisco Discovery Protocol
    u32 cip:1;                    //!< Common Industrial Protocol
    u32 eigrp:1;                  //!< EIGRP
    u32 ouroboros:1;              //!< Ouroboros
    u32 profinet:1;               //!< Profinet
    u32 arp:1;                    //!< ARP
    u32 ethercat:1;               //!< EtherCAT
    u32 ubitus:1;                 //!< Ubitus
    u32 ethernet_powerlink:1;     //!< Ethernet Powerlink
    u32 stomp:1;                  //!< STOMP
    u32 manufacturing_messaging_spec:1;//!< Manufacturing Messaging Specification
    u32 dingtalk:1;               //!< DingTalk
    u32 llc:1;                    //!< LLC
    u32 lldp:1;                   //!< LLDP
    u32 lightsail_vpn:1;          //!< Lightsail VPN
    u32 nord_vpn:1;               //!< Nord VPN
    u32 stp:1;                    //!< STP
    u32 garena_free_fire:1;       //!< Garena Free Fire
    u32 riot_games:1;             //!< Riot Games
    u32 noon_vpn:1;               //!< Noon VPN
    u32 alljoyn:1;                //!< AllJoyn
    u32 supervpn:1;               //!< SuperVPN
    PACE2_DEPRECATED("Protocol Snail VPN is marked as deprecated and should not be used anymore.")
    u32 snail_vpn:1;              //!< Snail VPN
    u32 mihoyo_games:1;           //!< miHoYo Games
    u32 hago:1;                   //!< Hago
    u32 portmap:1;                //!< Portmap
    u32 litecoin:1;               //!< Litecoin
    PACE2_DEPRECATED("Protocol Riley Cillian is marked as deprecated and should not be used anymore.")
    u32 riley_cillian:1;          //!< Riley Cillian
    u32 tase2:1;                  //!< TASE 2
    u32 lol_wild_rift:1;          //!< LoL Wild Rift
    u32 camscanner:1;             //!< CamScanner
    u32 activision_services:1;    //!< Activision Services
    u32 call_of_duty_mobile:1;    //!< Call of Duty Mobile
    u32 kaspersky_services:1;     //!< Kaspersky Services
    u32 ninja_kiwi_services:1;    //!< Ninja Kiwi Services
    u32 photon:1;                 //!< Photon
    u32 micoworld_services:1;     //!< MicoWorld Services
    u32 gaijin_entertainment:1;   //!< Gaijin Entertainment
    u32 ieee_1703:1;              //!< IEEE 1703
    u32 iec_62056:1;              //!< IEC 62056
    u32 ieee_c37118:1;            //!< IEEE C37118
    u32 hislip:1;                 //!< HiSLIP
    u32 vxi_11:1;                 //!< VXI-11
    u32 onc_rpc:1;                //!< ONC-RPC
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 cisco_metadata:1;         //!< Cisco Metadata
    u32 ieee_8021q:1;             //!< IEEE 8021Q
    u32 ieee_8021ad:1;            //!< IEEE 8021AD
    u32 mpls:1;                   //!< MPLS
    u32 fall_guys:1;              //!< Fall Guys
    PACE2_DEPRECATED("Protocol Rogue Company is marked as deprecated and should not be used anymore.")
    u32 rogue_company:1;          //!< Rogue Company
    u32 iec_61850_goose:1;        //!< IEC 61850 GOOSE
    u32 iec_61850_sv:1;           //!< IEC 61850 Sampled Values
    u32 ieee_8021ah:1;            //!< IEEE 8021AH
    u32 netease:1;                //!< NetEase
    u32 h245:1;                   //!< H245
    u32 h225:1;                   //!< H225
    u32 t120:1;                   //!< T120
    u32 hi_rez_services:1;        //!< Hi-Rez Services
    u32 cryptic_studios:1;        //!< Cryptic Studios
    u32 ccp_games:1;              //!< CCP Games
    u32 wargaming_net:1;          //!< Wargaming-Net
    u32 kafka:1;                  //!< Kafka
    u32 didi_services:1;          //!< DiDi Services
    u32 v4:1;                     //!< V4
    u32 ptp:1;                    //!< PTP
    u32 escape_from_tarkov:1;     //!< Escape from Tarkov
    u32 zangi:1;                  //!< Zangi
    u32 nomachine_nx:1;           //!< NoMachine NX
    u32 padding_1:(512 - 32 - PACE2_PROTOCOL_COUNT);
    u32 padding_2:32;
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 padding_2:32;
    u32 padding_1:(512 - 32 - PACE2_PROTOCOL_COUNT);
    u32 nomachine_nx:1;           //!< NoMachine NX
    u32 zangi:1;                  //!< Zangi
    u32 escape_from_tarkov:1;     //!< Escape from Tarkov
    u32 ptp:1;                    //!< PTP
    u32 v4:1;                     //!< V4
    u32 didi_services:1;          //!< DiDi Services
    u32 kafka:1;                  //!< Kafka
    u32 wargaming_net:1;          //!< Wargaming-Net
    u32 ccp_games:1;              //!< CCP Games
    u32 cryptic_studios:1;        //!< Cryptic Studios
    u32 hi_rez_services:1;        //!< Hi-Rez Services
    u32 t120:1;                   //!< T120
    u32 h225:1;                   //!< H225
    u32 h245:1;                   //!< H245
    u32 netease:1;                //!< NetEase
    u32 ieee_8021ah:1;            //!< IEEE 8021AH
    u32 iec_61850_sv:1;           //!< IEC 61850 Sampled Values
    u32 iec_61850_goose:1;        //!< IEC 61850 GOOSE
    PACE2_DEPRECATED("Protocol Rogue Company is marked as deprecated and should not be used anymore.")
    u32 rogue_company:1;          //!< Rogue Company
    u32 fall_guys:1;              //!< Fall Guys
    u32 mpls:1;                   //!< MPLS
    u32 ieee_8021ad:1;            //!< IEEE 8021AD
    u32 ieee_8021q:1;             //!< IEEE 8021Q
    u32 cisco_metadata:1;         //!< Cisco Metadata
#endif

} PACE2_protocols_struct;

/**
 * List of strings, each identifying the appropriate protocol.
 *
 * Items do not change between releases. Only for new protocols strings are added.
 */
#define PACE2_PROTOCOLS_SHORT_STRS \
    "ukn",\
    "bit",\
    "edk",\
    "gnu",\
    "winmx",\
    "directconnect",\
    "apple",\
    "soul",\
    "ares",\
    "freenet",\
    "mute",\
    "xdcc",\
    "skype",\
    "sip",\
    "msn",\
    "yahoo",\
    "oscar",\
    "irc",\
    "xmpp",\
    "http",\
    "pop",\
    "smtp",\
    "imap",\
    "usenet",\
    "ftp",\
    "dns",\
    "rtsp",\
    "filetopia",\
    "manolito",\
    "imesh",\
    "ssl",\
    "flash",\
    "mms",\
    "mpeg",\
    "quicktime",\
    "pando",\
    "openvpn",\
    "ipsec",\
    "gre",\
    "notdetect",\
    "h323",\
    "windowsmedia",\
    "realmedia",\
    "tvants",\
    "sopcast",\
    "tvuplayer",\
    "ppstream",\
    "pplive",\
    "iax",\
    "vtun",\
    "mgcp",\
    "gadugadu",\
    "zattoo",\
    "winny",\
    "qq",\
    "qqlive",\
    "feidian",\
    "ssh",\
    "popo",\
    "thunder",\
    "uusee",\
    "vnc",\
    "icmp",\
    "dhcp",\
    "igmp",\
    "egp",\
    "steam",\
    "hl2",\
    "xbox",\
    "softether",\
    "sctp",\
    "smb",\
    "telnet",\
    "ntp",\
    "nfs",\
    "rtp",\
    "tor",\
    "ssdp",\
    "worldofwarcraft",\
    "avi",\
    "vpnx",\
    "hamachi",\
    "skinny",\
    "citrix",\
    "ogg",\
    "paltalk",\
    "move",\
    "postgres",\
    "ospf",\
    "bgp",\
    "quake",\
    "secondlife",\
    "mysql",\
    "battlefield",\
    "pcanywhere",\
    "rdp",\
    "snmp",\
    "kontiki",\
    "veohtv",\
    "kerberos",\
    "ipip",\
    "gtp",\
    "radius",\
    "stun",\
    "syslog",\
    "tds",\
    "netbios",\
    "mdns",\
    "ipp",\
    "ldap",\
    "warcraft3",\
    "xdmcp",\
    "oovoo",\
    "fring",\
    "mapi",\
    "l2tp",\
    "isakmp",\
    "rtcp",\
    "tftp",\
    "mssql",\
    "ppp",\
    "pptp",\
    "stealthnet",\
    "sap",\
    "icmpv6",\
    "yourfreedom",\
    "voipswitchvoiptunnel",\
    "dhcpv6",\
    "meebo",\
    "funshion",\
    "afp",\
    "aimini",\
    "clubpenguin",\
    "florensia",\
    "teamspeak",\
    "maplestory",\
    "osh",\
    "ps3",\
    "dofus",\
    "wii",\
    "wokf",\
    "fiesta",\
    "splashfighter",\
    "crossfire",\
    "wapwsp",\
    "wapwtpwsp",\
    "wapwtls",\
    "multimediamessaging",\
    "guildwars",\
    "armagetron",\
    "rim-prop",\
    "rfactor",\
    "rdt",\
    "teamviewer",\
    "gamekit",\
    "netmotion",\
    "teredo",\
    "spotify",\
    "implus",\
    "ultrasurf",\
    "viber",\
    "tango",\
    "iplayer",\
    "jbk3k",\
    "operamini",\
    "bolt",\
    "scydo",\
    "whatsapp",\
    "webex",\
    "mojo",\
    "imo",\
    "citrixgoto",\
    "comodounite",\
    "goober",\
    "ventrilo",\
    "mypeople",\
    "websocket",\
    "ebuddy",\
    "sstp",\
    "adobeconnect",\
    "jap",\
    "ldp",\
    "wuala",\
    "msrp",\
    "lync",\
    "socks",\
    "audiogalaxy",\
    "mig33",\
    "rhapsody",\
    "sudaphone",\
    "webqq",\
    "pdproxy",\
    "silverlight",\
    "cyberghost",\
    "fix",\
    "oracledb",\
    "rsvp",\
    "lotusnotes",\
    "poisonivy",\
    "netflow",\
    "wechat",\
    "soap",\
    "kik",\
    "mplus",\
    "line",\
    "iperf",\
    "diameter",\
    "hike",\
    "mqtt",\
    "dce-rpc",\
    "idrive",\
    "dota2",\
    "qvod",\
    "msexchange",\
    "msnav",\
    "tibia",\
    "kakao",\
    "destiny",\
    "vyprvpn",\
    "tacacs",\
    "threema",\
    "ntlm",\
    "telegram",\
    "whois",\
    "ipv4",\
    "ipv6",\
    "tcp",\
    "udp",\
    "rtmp",\
    "nntp",\
    "flute",\
    "megaco",\
    "twitch",\
    "quic",\
    "fuze",\
    "aowe",\
    "doook",\
    "speedtest",\
    "iec-61850",\
    "scada",\
    "shoutcast",\
    "cql",\
    "cic",\
    "thrift",\
    "playstation",\
    "supercell",\
    "skype-for-business",\
    "t3",\
    "srtp",\
    "srtcp",\
    "league-of-legends",\
    "facebook",\
    "battle-net",\
    "wowza",\
    "zalo",\
    "guildwars2",\
    "crashplan",\
    "ea-origin",\
    "fasttrack",\
    "icecast",\
    "chatcube",\
    "good",\
    "path-of-exile",\
    "pcoip",\
    "ocsp",\
    "modbus",\
    "fasp",\
    "ica",\
    "opc-ua",\
    "openflow",\
    "ipmi",\
    "amqp",\
    "s7comm",\
    "off",\
    "rsync",\
    "zeromq",\
    "dnp3",\
    "bacnet",\
    "nintendo-network",\
    "ppfilm",\
    "bitcoin",\
    "letv",\
    "psiphon",\
    "zero",\
    "synology",\
    "coap",\
    "lineage-2",\
    "ipfix",\
    "minecraft",\
    "stratum-mining",\
    "lpd",\
    "zero-vpn",\
    "turbo-vpn",\
    "vpn-master",\
    "iqiyi",\
    "snap-vpn",\
    "vpn-unlimited",\
    "express-vpn",\
    "tunnelguru",\
    "botim",\
    "hi-vpn",\
    "speed-vpn-mad",\
    "dash-vpn",\
    "yeecall",\
    "atom-vpn",\
    "fortnite",\
    "wireguard",\
    "amaze-vpn",\
    "fast-secure-vpn",\
    "zpn",\
    "world-of-tanks",\
    "appvpn",\
    "x-vpn",\
    "droid-vpn",\
    "dtls",\
    "smart-veepn",\
    "apex-legends",\
    "vpn-365",\
    "counterstrike-go",\
    "supervpn-by-cheng",\
    "act-vpn",\
    "hotspot-vpn",\
    "mtproto",\
    "shadowsocks",\
    "asphalt9",\
    "actmobile-services",\
    "zrtp",\
    "pubg-mobile",\
    "gameloft-services",\
    "wickr",\
    "anchorfree-services",\
    "vpn-proxy-master-unlimited",\
    "unreal-engine",\
    "zoom",\
    "vpn-proxy-master",\
    "dead-rivals",\
    "igg",\
    "mudfish",\
    "soma",\
    "hatch-cloud-gaming",\
    "source-engine",\
    "streamcraft",\
    "youku-tudou",\
    "pdanet",\
    "http2",\
    "totok",\
    "bigo-services",\
    "webm",\
    "hide-me-vpn",\
    "ipvanish",\
    "microsoft-services",\
    "rip",\
    "privatevpn",\
    "spongycastle",\
    "surfshark-vpn",\
    "sapling-growth-vpn",\
    "iec-13818",\
    "tencent",\
    "ultravpn",\
    "wechat-work",\
    "alibaba",\
    "isis",\
    "raknet",\
    "innovative-connecting",\
    "vipnet",\
    "resp",\
    "noe",\
    "agora",\
    "miop",\
    "giop",\
    "elex-technology",\
    "json-rpc",\
    "ziop",\
    "uftp",\
    "moonton",\
    "rtps",\
    "ethereum",\
    "zynn",\
    "sflow",\
    "collectd",\
    "hl7",\
    "among-us",\
    "ethernetip",\
    "dnscrypt",\
    "onc-rpc",\
    "vxi-11",\
    "hislip",\
    "ieee-c37118",\
    "iec-62056",\
    "ieee-1703",\
    "gaijin-entertainment",\
    "micoworld-services",\
    "photon",\
    "ninja-kiwi-services",\
    "kaspersky-services",\
    "call-of-duty-mobile",\
    "activision-services",\
    "camscanner",\
    "lol-wild-rift",\
    "tase2",\
    "riley-cillian",\
    "litecoin",\
    "portmap",\
    "hago",\
    "mihoyo-games",\
    "snail-vpn",\
    "supervpn",\
    "alljoyn",\
    "noon-vpn",\
    "riot-games",\
    "garena-free-fire",\
    "stp",\
    "nord-vpn",\
    "lightsail-vpn",\
    "lldp",\
    "llc",\
    "dingtalk",\
    "manufacturing-messaging-spec",\
    "stomp",\
    "ethernet-powerlink",\
    "ubitus",\
    "ethercat",\
    "arp",\
    "profinet",\
    "ouroboros",\
    "eigrp",\
    "cip",\
    "cdp",\
    "valorant",\
    "vtp",\
    "mmse",\
    "dtp",\
    "nvidia",\
    "ubisoft",\
    "vrrp",\
    "pppoe",\
    "vxlan",\
    "cloudflare",\
    "fins",\
    "pfcp",\
    "lost-ark",\
    "polkadot",\
    "mumble",\
    "naraka-bladepoint",\
    "final-fantasy-xiv",\
    "ethernet",\
    "nhrp",\
    "pinduoduo",\
    "cisco-metadata",\
    "ieee-8021q",\
    "ieee-8021ad",\
    "mpls",\
    "fall-guys",\
    "rogue-company",\
    "iec-61850-goose",\
    "iec-61850-sv",\
    "ieee-8021ah",\
    "netease",\
    "h245",\
    "h225",\
    "t120",\
    "hi-rez-services",\
    "cryptic-studios",\
    "ccp-games",\
    "wargaming-net",\
    "kafka",\
    "didi-services",\
    "v4",\
    "ptp",\
    "escape-from-tarkov",\
    "zangi",\
    "nomachine-nx"

/**
 * List of strings, each identifying the appropriate protocol in a more human readable way.
 *
 * Items may be changed between releases.
 */
#define PACE2_PROTOCOLS_LONG_STRS \
    "unknown",\
    "BitTorrent",\
    "eDonkey",\
    "Gnutella",\
    "WinMX",\
    "DirectConnect",\
    "AppleJuice",\
    "Soulseek",\
    "Ares",\
    "Freenet",\
    "Mute",\
    "XDCC",\
    "Skype",\
    "SIP",\
    "MSN",\
    "Yahoo",\
    "Oscar",\
    "IRC",\
    "XMPP",\
    "HTTP",\
    "POP",\
    "SMTP",\
    "IMAP",\
    "Usenet",\
    "FTP",\
    "DNS",\
    "RTSP",\
    "Filetopia",\
    "Manolito",\
    "iMesh",\
    "SSL",\
    "Flash",\
    "MMS",\
    "MPEG",\
    "QuickTime",\
    "Pando",\
    "OpenVPN",\
    "IPsec",\
    "GRE",\
    "undetected",\
    "H323",\
    "WindowsMedia",\
    "RealMedia",\
    "TvAnts",\
    "SopCast",\
    "TVUPlayer",\
    "PPStream",\
    "PPLive",\
    "IAX",\
    "VTun",\
    "MGCP",\
    "Gadu-Gadu",\
    "Zattoo",\
    "WinNY",\
    "QQ",\
    "QQLive",\
    "Feidian",\
    "SSH",\
    "PoPo",\
    "Thunder/Webthunder",\
    "UUSee",\
    "VNC",\
    "ICMP",\
    "DHCP",\
    "IGMP",\
    "EGP",\
    "Steam",\
    "Half-Life 2",\
    "XBOX",\
    "SoftEthernet",\
    "SCTP",\
    "SMB/CIFS",\
    "Telnet",\
    "NTP",\
    "NFS",\
    "RTP",\
    "Tor",\
    "SSDP",\
    "World of Warcraft",\
    "AVI",\
    "VPN-X",\
    "HamachiVPN",\
    "Skinny",\
    "Citrix",\
    "OGG",\
    "Paltalk",\
    "Move",\
    "PostgreSQL",\
    "OSPF",\
    "BGP",\
    "Quake",\
    "Second Life",\
    "MySQL",\
    "Battlefield",\
    "PCAnywhere",\
    "RDP",\
    "SNMP",\
    "Kontiki",\
    "VeohTV",\
    "Kerberos",\
    "IP in IP",\
    "GTP",\
    "RADIUS",\
    "STUN",\
    "Syslog",\
    "TDS",\
    "NetBIOS",\
    "MulticastDNS",\
    "IPP",\
    "LDAP",\
    "Warcraft 3",\
    "XDMCP",\
    "ooVoo",\
    "Fring",\
    "Mapi",\
    "L2TP",\
    "ISAKMP",\
    "RTCP",\
    "TFTP",\
    "MSSQL",\
    "PPP",\
    "PPTP",\
    "StealthNet",\
    "SAP",\
    "ICMPv6",\
    "YourFreedom Tunnel",\
    "VoipSwitch VoIP Tunnel",\
    "DHCPv6",\
    "Meebo",\
    "Funshion",\
    "AFP",\
    "Aimini",\
    "ClubPenguin",\
    "Florensia",\
    "TeamSpeak",\
    "MapleStory",\
    "Octoshape",\
    "PS3",\
    "Dofus",\
    "WII",\
    "World of Kung Fu",\
    "Fiesta",\
    "SplashFighter",\
    "CrossFire",\
    "WAP-WSP",\
    "WAP-WTP-WSP",\
    "WAP-WTLS",\
    "MultimediaMessaging",\
    "Guildwars",\
    "Armagetron",\
    "RIM proprietary",\
    "rFactor",\
    "RealDataTransport",\
    "TeamViewer",\
    "GameKit",\
    "NetMotion",\
    "Teredo",\
    "Spotify",\
    "IMplus",\
    "UltraSurf",\
    "Viber",\
    "Tango",\
    "iPlayer",\
    "JBK3000",\
    "Opera Mini",\
    "Bolt",\
    "Scydo",\
    "WhatsApp",\
    "Webex",\
    "Mojo",\
    "IMO",\
    "CitrixGoTo",\
    "ComodoUnite",\
    "Goober",\
    "Ventrilo",\
    "MyPeople",\
    "WebSocket",\
    "eBuddy",\
    "SSTP",\
    "AdobeConnect",\
    "JAP",\
    "LDP",\
    "Wuala",\
    "MSRP",\
    "Lync",\
    "Socks",\
    "Audiogalaxy",\
    "Mig33",\
    "Rhapsody",\
    "Sudaphone",\
    "WebQQ",\
    "PDProxy",\
    "Silverlight",\
    "CyberGhost",\
    "FIX",\
    "Oracle Database",\
    "RSVP",\
    "Lotus Notes",\
    "Poison Ivy",\
    "NetFlow",\
    "WeChat",\
    "SOAP",\
    "Kik Messenger",\
    "MPlus",\
    "Line",\
    "Iperf",\
    "Diameter",\
    "Hike Messenger",\
    "MQTT",\
    "DCE RPC",\
    "IDrive",\
    "Dota 2",\
    "QVOD",\
    "Microsoft Exchange",\
    "Microsoft Dynamics NAV",\
    "Tibia",\
    "Kakao",\
    "Destiny",\
    "VyprVPN",\
    "TACACS",\
    "Threema",\
    "NTLM",\
    "Telegram",\
    "WHOIS",\
    "IPv4",\
    "IPv6",\
    "TCP",\
    "UDP",\
    "RTMP",\
    "NNTP",\
    "FLUTE",\
    "Megaco",\
    "Twitch",\
    "QUIC",\
    "Fuze Meeting",\
    "Age of Warring Empire",\
    "Doook",\
    "Speedtest",\
    "IEC 61850",\
    "SCADA",\
    "Shoutcast",\
    "Cassandra Query Language",\
    "Cassandra Internode Communication",\
    "Thrift",\
    "PlayStation Network",\
    "Supercell",\
    "Skype for Business",\
    "T3",\
    "SRTP",\
    "SRTCP",\
    "League of Legends",\
    "Facebook",\
    "BattleNet",\
    "Wowza",\
    "Zalo",\
    "Guild Wars 2",\
    "CrashPlan",\
    "EA Origin",\
    "Fasttrack",\
    "Icecast",\
    "ChatCube",\
    "Good",\
    "Path of Exile",\
    "PCoIP",\
    "OCSP",\
    "Modbus",\
    "FASP",\
    "ICA",\
    "OPC UA",\
    "OpenFlow",\
    "IPMI",\
    "AMQP",\
    "S7comm",\
    "OFF",\
    "Rsync",\
    "ZeroMQ",\
    "DNP3",\
    "BACNET",\
    "Nintendo Network",\
    "PPFilm",\
    "Bitcoin",\
    "LeTV",\
    "Psiphon",\
    "Zero",\
    "Synology",\
    "CoAP",\
    "Lineage 2",\
    "IPFIX",\
    "Minecraft",\
    "Stratum Mining",\
    "Local Peer Discovery",\
    "Zero VPN",\
    "Turbo VPN",\
    "VPN Master",\
    "Iqiyi",\
    "Snap VPN",\
    "VPN Unlimited",\
    "ExpressVPN",\
    "TunnelGuru",\
    "BOTIM",\
    "Hi VPN",\
    "Speed VPN MobileAppDev",\
    "Dash VPN",\
    "YeeCall",\
    "Atom VPN",\
    "Fortnite",\
    "WireGuard",\
    "Amaze VPN",\
    "Fast Secure VPN",\
    "ZPN",\
    "World of Tanks",\
    "AppVPN",\
    "X-VPN",\
    "Droid VPN",\
    "DTLS",\
    "Smart VeePN",\
    "Apex Legends",\
    "VPN 365",\
    "CounterStrikeGO",\
    "SuperVPN by Cheng",\
    "Act VPN",\
    "Hotspot VPN",\
    "MTProto",\
    "Shadowsocks",\
    "Asphalt9",\
    "Actmobile Services",\
    "ZRTP",\
    "PUBG Mobile",\
    "Gameloft Services",\
    "Wickr",\
    "Anchorfree Services",\
    "VPN Proxy Master Unlimited",\
    "Unreal Engine",\
    "Zoom",\
    "VPN Proxy Master",\
    "Dead Rivals",\
    "IGG",\
    "Mudfish",\
    "SOMA",\
    "Hatch Cloud Gaming",\
    "Source Engine",\
    "Streamcraft",\
    "Youku Tudou",\
    "PdaNet",\
    "HTTP/2",\
    "ToTok",\
    "BIGO Services",\
    "WebM",\
    "HideMe VPN",\
    "IPVanish",\
    "Microsoft Services",\
    "Routing Information Protocol",\
    "PrivateVPN",\
    "SpongyCastle",\
    "Surfshark VPN",\
    "Sapling Growth VPN Services",\
    "IEC 13818",\
    "Tencent",\
    "UltraVPN",\
    "WeChat Work",\
    "Alibaba",\
    "ISIS",\
    "RakNet",\
    "Innovative Connecting",\
    "ViPNet",\
    "RESP",\
    "New Office Environment",\
    "Agora",\
    "MIOP",\
    "GIOP",\
    "Elex Technology",\
    "JSON-RPC",\
    "ZIOP",\
    "UFTP",\
    "Moonton",\
    "RTPS",\
    "Ethereum",\
    "Zynn",\
    "sFlow",\
    "collectd",\
    "HL7",\
    "Among Us",\
    "EtherNetIP",\
    "DNSCrypt",\
    "ONC-RPC",\
    "VXI-11",\
    "HiSLIP",\
    "IEEE C37118",\
    "IEC 62056",\
    "IEEE 1703",\
    "Gaijin Entertainment",\
    "MicoWorld Services",\
    "Photon",\
    "Ninja Kiwi Services",\
    "Kaspersky Services",\
    "Call of Duty Mobile",\
    "Activision Services",\
    "CamScanner",\
    "LoL Wild Rift",\
    "TASE 2",\
    "Riley Cillian",\
    "Litecoin",\
    "Portmap",\
    "Hago",\
    "miHoYo Games",\
    "Snail VPN",\
    "SuperVPN",\
    "AllJoyn",\
    "Noon VPN",\
    "Riot Games",\
    "Garena Free Fire",\
    "STP",\
    "Nord VPN",\
    "Lightsail VPN",\
    "LLDP",\
    "LLC",\
    "DingTalk",\
    "Manufacturing Messaging Specification",\
    "STOMP",\
    "Ethernet Powerlink",\
    "Ubitus",\
    "EtherCAT",\
    "ARP",\
    "Profinet",\
    "Ouroboros",\
    "EIGRP",\
    "Common Industrial Protocol",\
    "Cisco Discovery Protocol",\
    "Valorant",\
    "VLAN Trunking Protocol",\
    "Multimedia Messaging Service Encapsulation",\
    "Dynamic Trunking Protocol",\
    "Nvidia",\
    "Ubisoft",\
    "Virtual Router Redundancy Protocol",\
    "PPPoE",\
    "VXLAN",\
    "Cloudflare",\
    "FINS",\
    "Packet Forwarding Control Protocol",\
    "Lost Ark",\
    "Polkadot",\
    "Mumble",\
    "Naraka Bladepoint",\
    "Final Fantasy XIV",\
    "Ethernet",\
    "Next Hop Resolution Protocol",\
    "Pinduoduo",\
    "Cisco Metadata",\
    "IEEE 8021Q",\
    "IEEE 8021AD",\
    "MPLS",\
    "Fall Guys",\
    "Rogue Company",\
    "IEC 61850 GOOSE",\
    "IEC 61850 Sampled Values",\
    "IEEE 8021AH",\
    "NetEase",\
    "H245",\
    "H225",\
    "T120",\
    "Hi-Rez Services",\
    "Cryptic Studios",\
    "CCP Games",\
    "Wargaming-Net",\
    "Kafka",\
    "DiDi Services",\
    "V4",\
    "PTP",\
    "Escape from Tarkov",\
    "Zangi",\
    "NoMachine NX"

/**
 * List of strings, each describing the appropriate protocol.
 *
 * Items may be changed between releases.
 */
#define PACE2_PROTOCOLS_DESCRIPTIONS \
    "All traffic that is not classified yet is labeled as unknown.",\
    "BitTorrent is the protocol of the corresponding P2P network.",\
    "eDonkey (also known as the eDonkey2000 network or eD2k) is the protocol of the corresponding P2P network.",\
    "Gnutella is the protocol of the corresponding P2P network.",\
    "WinMX is the protocol of the corresponding P2P network.",\
    "DirectConnect is the protocol of the corresponding P2P network.",\
    "Applejuice is the protocol of the corresponding P2P network.",\
    "Soulseek is the protocol of the corresponding P2P file-sharing network and application.",\
    "Ares is the protocol of the corresponding P2P network.",\
    "Freenet is the protocol of the corresponding P2P network.",\
    "The protocol MUTE was deprecated and should not be used anymore.",\
    "XDCC is a protocol used by popular instant messaging applications.",\
    "Skype is the protocol of the corresponding instant messaging, voice, video and file transfer application.",\
    "SIP is voice call traffic initiated over the session initiation protocol.",\
    "MSN is deprecated and should not be used anymore.",\
    "Yahoo is the protocol of the corresponding instant messaging application with support for chat, voice calls and file transfer.",\
    "Oscar is the protocol used by AOL Messenger applications.",\
    "IRC is a protocol used by a number of popular instant messaging applications.",\
    "XMPP (Extensible Messaging and Presence Protocol, formerly named Jabber) is an open, XML-based protocol.",\
    "HTTP is a protocol used to access web servers. It is also the transport protocol for a number of streaming applications.",\
    "POP is a protocol used to fetch emails from servers.",\
    "SMTP is a protocol used to transmit emails.",\
    "IMAP is a protocol used to remotely access emails.",\
    "Protocol Usenet is deprecated and should not be used anymore.",\
    "FTP (File Transfer Protocol) is a protocol to upload or download files from FTP servers.",\
    "DNS (Domain Name System) is a protocol to resolve domain names to IP addresses.",\
    "RTSP is a protocol used to transport audio and video streams.",\
    "Filetopia is the protocol of the corresponding free, Windows-based client and networking tool allowing users to organize files, texts and voice recordings as well as chat and instant messages.",\
    "Manolito protocol was deprecated and should not be used anymore.",\
    "iMesh is the protocol of the corresponding P2P network.",\
    "SSL/TLS is a network protocol used to provide secure communication over IP networks.",\
    "Flash is a protocol for streaming content over HTTP.",\
    "MMS is a proprietary network streaming protocol from Microsoft.",\
    "MPEG is a protocol for streaming content over HTTP.",\
    "QuickTime is a protocol for streaming content over HTTP.",\
    "The Pando protocol was deprecated and should not be used anymore.",\
    "OpenVPN is the protocol of a free and open source virtual private network.",\
    "IPSec is a protocol suite for securing Internet Protocol (IP) communications by authenticating and encrypting each IP packet of a communication session.",\
    "GRE is a protocol used to tunnel usually non-IP protocols over IP. Examples are PPTP, IPv6, IPX or AppleTalk.",\
    "Traffic that cannot be classified by PACE is labeled as undetected.",\
    "H323 is the protocol of the corresponding traffic of H.323 initiated calls.",\
    "WindowsMedia is the Windows Media Streaming protocol.",\
    "RealMedia is the protocol of the RealNetworks RealMedia streaming application and browser plugin.",\
    "TvAnts is the protocol of a P2P TV streaming network of the same name.",\
    "SopCast was deprecated and should not be used anymore.",\
    "TVUPlayer is deprecated and should not be used anymore, because TVU Networks shut down the TVUPlayer service on February 25, 2013.",\
    "PPStream is the protocol of the corresponding Chinese peer-to-peer streaming video network software.",\
    "PPLive is the protocol of the corresponding peer-to-peer streaming video network.",\
    "IAX is the protocol of the open source PBX Asterisk.",\
    "VTun is a networking application which can set up Virtual Tunnels over TCP/IP networks.",\
    "MGCP is a protocol used for communication between voice gateways.",\
    "Gadu-Gadu is the protocol of the corresponding popular Polnish instant messaging application.",\
    "Zattoo is a proprietary peer-to-peer Internet Protocol Television system ('P2PTV').",\
    "WinNY is the protocol of the P2P network with the same name.",\
    "QQ is an instant messaging application popular in Asia.",\
    "QQLive is the protocol of the corresponding P2PTV software and a service provided by the Tencent Corporation of China.",\
    "Feidian (Boiling Point) was deprecated and should not be used anymore.",\
    "SSH is a protocol used to provide secure, encrypted communication over IP networks.",\
    "PoPo is the protocol of the corresponding instant messaging application.",\
    "Thunder is the protocol of the corresponding P2P file sharing application.",\
    "UUSee is the protocol of the corresponding P2P network.",\
    "VNC is the protocol of a corresponding application to access the graphical user interface/desktop of remote systems.",\
    "ICMP is a protocol used to exchange status information and error messages over IP networks.",\
    "DHCP is a protocol used to dynamically configure systems in IP networks.",\
    "IGMP is a protocol used to manage multicast groups.",\
    "EGP is an Internet routing protocol.",\
    "The protocol Steam was deprecated and should not be used anymore. Instead the Source Engine protocol will cover the related traffic.",\
    "Half-Life 2 is deprecated and should not be used anymore. Instead, please use the Source Engine protocol.",\
    "XBOX is the protocol of the corresponding traffic of the XBOX video game console.",\
    "SoftEthernet is the protocol of a multi-protocol VPN software.",\
    "SCTP is a reliable, connection-oriented protocol at layer 4 of the OSI model.",\
    "CIFS/SMB is a protocol usually used to provide file access to Microsoft Windows systems.",\
    "Telnet (teletype network) is a network protocol used on the Internet or local area networks to provide a bidirectional interactive communications facility.",\
    "NTP is a protocol used to synchronise the time of computer systems.",\
    "NFS is a protocol used to share files via a network.",\
    "RTP is a standard protocol used to transport audio and video streams over IP networks.",\
    "Tor is a protocol used to provide secure communication and anonymity over IP networks.",\
    "SSDP is used to discover UPnP devices.",\
    "World of Warcraft is the protocol of the corresponding multiplayer online game.",\
    "AVI is the protocol of AVI video streams over HTTP.",\
    "VPN-X is the protocol used by the vpnx tunnel application.",\
    "Hamachi is the protocol for the corresponding zero-configuration virtual private network (VPN) shareware application.",\
    "SCCP (Skinny Call Control Protocol or short Skinny) is a proprietary network terminal control protocol.",\
    "The protocol Citrix is deprecated and should not be used anymore. It was renamed into ICA.",\
    "OGG Vorbis is the protocol for the corresponding open-source streaming technology.",\
    "Paltalk is an Internet chat service which allows users to communicate via instant messaging, voice and video chat.",\
    "Move is a streaming protocol over HTTP.",\
    "PostgreSQL is the protocol for corresponding database traffic.",\
    "OSPF is a routing protocol developed by the Internet Engineering Task Force.",\
    "BGP is a routing protocol connecting autonomous systems (AS).",\
    "Quake is a protocol which is used for communication between systems during multiplayer games.",\
    "Second Life is the protocol for the corresponding virtual world application.",\
    "MySQL is the protocol for the traffic of the MySQL database.",\
    "Battlefield is the protocol for the corresponding online game.",\
    "The protocol PCAnywhere was deprecated and should not be used anymore, because Symantec discontinued the product in May 2014.",\
    "RDP is a proprietary Microsoft protocol which is used to remotely connect to Windows systems and control the desktop.",\
    "SNMP is used for monitoring and managing network components such as switches or routers.",\
    "Kontiki is the protocol of the content delivery platform with the same name.",\
    "Veoh TV is the protocol for the corresponding application of the Veoh Internet television service.",\
    "Kerberos is a computer network authentication protocol, which allows nodes communicating over a non-secure network to prove their identity to one another in a secure manner. It is also a suite of free software published by the Massachusetts Institute of Technology (MIT).",\
    "IP in IP is an IP tunneling protocol that encapsulates one IP packet in another IP packet.",\
    "GTP (GPRS Tunneling Protocol) is a group of IP-based communication protocols used to carry General Packet Radio Service (GPRS) within GSM and UMTS networks.",\
    "RADIUS (Remote Authentication Dial In User Service) is a networking protocol that provides centralized Authentication, Authorization and Accounting (AAA) management for computers to connect to and use a network service.",\
    "STUN is an Internet standards-track suite of methods, including a network protocol, used in NAT traversal for applications of real-time voice, video, messaging, and other interactive IP communications.",\
    "Syslog is a standard for forwarding log messages in an Internet Protocol (IP) computer network.",\
    "TDS (Tabular Data Stream) is an application layer protocol used to transfer data between a database server and a client.",\
    "NetBIOS is an acronym for Network Basic Input/Output System. It provides services related to the session layer of the OSI model allowing applications on separate computers to communicate over a local area network.",\
    "MDNS is part of zero configuration networking (zeroconf) and is used for the automatic resolution and distribution of computer hostnames (multicast DNS).",\
    "IPP (Internet Printing Protocol) is a standard network protocol for remote printing as well as for managing print jobs, media size and resolution.",\
    "LDAP (Lightweight Directory Access Protocol) is an application protocol for querying and modifying directory services running over TCP/IP.",\
    "Warcraft 3 is the protocol of the corresponding multiplayer online game.",\
    "XDMCP is a Display Manager Control Protocol for the X window system.",\
    "ooVoo is a VoIP protocol.",\
    "The Fring protocol was deprecated and should not be used anymore.",\
    "MAPI/RPC is a proprietary protocol that Microsoft Outlook uses to communicate with Microsoft Exchange.",\
    "L2TP is a protocol used to tunnel PPP frames over IP networks.",\
    "ISAKMP is a protocol used to establish Security Associations (SA) and cryptographic keys on the Internet. The protocol is defined by RFC 2408.",\
    "RTCP (The RTP Control Protocol) is a sister protocol of the Real-time Transport Protocol (RTP). Its basic functionality and packet structure is defined in the RTP specification RFC 3550.",\
    "TFTP (Trivial File Transfer Protocol) is a file transfer protocol with the functionality of a very basic File Transfer Protocol (FTP).",\
    "MSSQL is the protocol for the corresponding database application. This protocol is deprecated and will be detected as TDS.",\
    "PPP (Point-to-Point Protocol) is a data link protocol commonly used to establish a direct connection between two networking nodes. It can provide connection authentication, transmission encryption privacy and compression.",\
    "PPTP (Point-to-Point Tunneling Protocol) is a protocol for the implementation of virtual private networks (VPN). PPTP uses a control channel over TCP and a GRE tunnel operating to encapsulate PPP packets.",\
    "StealthNet is the protocol of the corresponding P2P application.",\
    "SAP is the protocol used between SAP server(s) and SAP clients.",\
    "ICMPv6 is ICMP for IPv6. It is used to exchange status information and error messages over IP network.",\
    "YourFreedom is a tunnel protocol.",\
    "VoipSwitch or VoipTunnel is a tunnel protocol for VoIP.",\
    "DHCPv6 is the DHCP for IPv6.",\
    "Meebo is the protocol for the corresponding instant messenger.",\
    "Funshion is a streaming protocol.",\
    "AFP (Apple Filing Protocol) is a network protocol that offers file services for Mac OS X and original Mac OS.",\
    "Aimini is a P2P protocol.",\
    "The protocol ClubPenguin is deprecated and should not be used anymore.",\
    "Florensia is the protocol of the corresponding multiplayer online game.",\
    "TeamSpeak is the protocol of the corresponding VoIP application.",\
    "MapleStory is the protocol of the corresponding multiplayer online role-playing game.",\
    "The protocol Octoshape was deprecated and should not be used anymore.",\
    "PS3 detection is deprecated. The traffic is now handled by the PlayStation Network classification.",\
    "Dofus is the protocol of the corresponding multiplayer online role-playing game.",\
    "This protocol has been deprecated and should not be used any more. Its traffic is now covered by the protocol Nintendo Network.",\
    "World of Kung Fu is the protocol of the corresponding multiplayer online role-playing game.",\
    "Fiesta is the protocol of the corresponding multiplayer online role-playing game.",\
    "SplashFighter is the protocol of the corresponding multiplayer online game.",\
    "Crossfire is the protocol of the corresponding open source, cooperative multiplayer graphical RPG and adventure game.",\
    "WAP-WSP Wireless Application Protocol using the Wireless Session Protocol is used to deliver Internet contents to devices with slower Internet connections like mobile phones.",\
    "WAP-WTP-WSP Wireless Application Protocol using the Wireless Transport Protocol and Wireless Session Protocol is used to deliver Internet contents to devices with slower Internet connections like mobile phones.",\
    "WAP-WTLS Wireless Application Protocol using the Wireless Transport Layer Security is used to securely deliver Internet contents to devices with slower Internet connections like mobile phones.",\
    "Removed. Reason: The name of the protocol was not fitting. Instead the protocol Multimedia Messaging Service Encapsulation (MMSE) should be used.",\
    "Guildwars is the protocol of the corresponding multiplayer online role-playing game.",\
    "Armagetron Advanced is the protocol of the corresponding multiplayer game.",\
    "RIM is a proprietary protocol used by BlackBerry.",\
    "rFactor is the protocol of the corresponding computer racing simulator.",\
    "RDT is an audio/video transport protocol from RealNetworks.",\
    "TeamViewer is a remote administration tool.",\
    "GameKit is the protocol of the corresponding Apple framework for multiplayer/social games.",\
    "NetMotion is the protocol of the corresponding mobile VPN solution.",\
    "Teredo is an IPv6 in IPv4/UDP tunnel protocol.",\
    "Spotify is a music listening service.",\
    "IMplus was a protocol used by the IMplus messenger application. Nowadays, IMplus is just an aggregator for other messenger apps and the protocol is not used anymore.",\
    "UltraSurf is a proxy solution for anonymous browsing.",\
    "Viber is the protocol of the corresponding VoIP application.",\
    "Tango is the protocol of the corresponding VoIP application.",\
    "The iPlayer protocol was deprecated and should not be used anymore.",\
    "JBK3000 is the protocol of the corresponding network traffic. It sends measurement data over LAN from an automatic tester which is used to measure various types of flammable liquid petroleum products.",\
    "Opera Mini is the protocol used by the mobile web browser of the same name from Opera.",\
    "Bolt is the protocol used by the mobile web browser Bolt for smartphones.",\
    "The Scydo protocol was deprecated and should not be used anymore.",\
    "WhatsApp is the protocol of the corresponding cross-platform mobile messaging app.",\
    "Webex is Cisco's web conferencing and collaboration solution protocol.",\
    "Mojo was the proprietary protocol of the corresponding P2P software, a tool to share iTunes libraries. The software did not receive an update since 2009, therefore the protocol was deprecated.",\
    "IMO is a web-based service that allows users to hold text, voice, and video chats on multiple instant messaging protocols.",\
    "CitrixGoTo is the protocol of the corresponding Citrix GoTo applications.",\
    "ComodoUnite is the protocol of the corresponding VPN client for windows computers.",\
    "The protocol Goober is deprecated and should not be used anymore. It was renamed into ChatCube in 2014.",\
    "Ventrilo is a proprietary VoIP software which includes text chat.",\
    "MyPeople is the proprietary protocol used by the Korean VoIP and IM application MyPeople.",\
    "WebSockets is an XMLHttpRequest replacement for HTTP.",\
    "eBuddy is a web and mobile multi messenger.",\
    "SSTP is the Secure Socket Tunneling Protocol, a VPN over SSL.",\
    "The protocol Adobe Connect was deprecated and should not be used anymore.",\
    "JAP (JonDonym anonymous proxy servers) was deprecated and should not be used anymore.",\
    "LDP is used to label servers to propagate labeling information, e.g. used for routing.",\
    "Wuala was deprecated and should not be used anymore. The service was shutdown in August 2015.",\
    "MSRP is a protocol used for transmitting instant messages, files and sharing images within a communication session.",\
    "The protocol Lync is deprecated. Its traffic is now handled by the Skype for Business classification.",\
    "Socks is a proxy protocol for tunneling traffic via Socks proxy server.",\
    "Audiogalaxy was deprecated and should not be used anymore.",\
    "Mig33 is the protocol of the corresponding mobile social network. It supports IM, blogs and minigames.",\
    "The protocol Rhapsody was deprecated and should not be used anymore.",\
    "The protocol Sudaphone is deprecated and should not be used anymore.",\
    "WebQQ was deprecated and should not be used anymore. The service was shutdown in January 2019.",\
    "PDProxy is the protocol of the corresponding tunneling software.",\
    "Silverlight is deprecated and should not be used anymore.",\
    "CyberGhost is the protocol of the corresponding tunneling software.",\
    "FIX (Financial Information eXchange) is a protocol used for financial and trade related transactions.",\
    "Oracle Database is the protocol of the corresponding database management system.",\
    "RSVP is a protocol to reserve resources across a network.",\
    "Lotus Notes is the protocol of the groupware solution HCL Notes and Domino.",\
    "Poison Ivy is the protocol of the corresponding remote administration tool that is mostly used as a Trojan attached to other executables.",\
    "NetFlow is a network reporting protocol developed by Cisco.",\
    "WeChat is the protocol of the corresponding voice and text messaging application.",\
    "SOAP is a protocol for exchanging structured information. It is based on XML.",\
    "Kik Messenger is the protocol of the corresponding IM application.",\
    "MPlus is the protocol of the corresponding messaging application.",\
    "Line is a free VoIP and messaging application.",\
    "Iperf is the protocol of the corresponding tool to measure network performance by generating traffic.",\
    "Diameter is an authentication, accounting and authorization protocol for network services. It was designed to replace RADIUS.",\
    "Hike Messenger is the protocol of the corresponding IM application.",\
    "MQTT is a lightweight messaging protocol for small sensors and mobile devices.",\
    "DCE/RPC is the remote procedure call system developed for the Distributed Computing Environment (DCE).",\
    "IDrive is the protocol of the corresponding cloud based storage and backup service.",\
    "Dota 2 is the protocol of the corresponding online multiplayer game.",\
    "QVoD is an all-in-one media player.",\
    "Microsoft Exchange is a groupware server to manage mails, contacts and calendars.",\
    "Microsoft Dynamics NAV is an enterprise resource planning software product.",\
    "Tibia is the protocol of the corresponding online role playing game developed by CipSoft.",\
    "Kakao is the protocol of the corresponding Kakao Talk application and underlying platform.",\
    "Destiny is the protocol for a multiplayer online first-person shooter (Destiny or Destiny2) developed by Bungie.",\
    "VyprVPN is the protocol of the corresponding personal VPN solution.",\
    "TACACS provides access control for routers, network access servers and other networked computing devices via one or more centralized servers.",\
    "Threema is a proprietary protocol used by the encrypted instant messaging application with the same name.",\
    "NTLM (NT LAN Manager) is a Microsoft security protocol used for authentication mechanisms.",\
    "Telegram is the protocol of the corresponding encrypted instant messaging application.",\
    "WHOIS is a protocol for querying registered users of an Internet resource.",\
    "IPv4 is a Layer 3 protocol.",\
    "IPv6 is a Layer 3 protocol.",\
    "TCP is a Layer 4 protocol.",\
    "UDP is a Layer 4 protocol.",\
    "RTMP (Real Time Messaging Protocol) is a proprietary protocol for streaming audio and video by Flash player.",\
    "NNTP (Network News Transfer Protocol) is the protocol used to transfer messages with Usenet.",\
    "FLUTE is a protocol used to deliver files over the Internet or unidirectional systems from one or more senders to one or more receivers.",\
    "Megaco (Media Gateway Control Protocol) is a protocol for controlling media gateways.",\
    "The protocol Twitch was deprecated and should not be used anymore.",\
    "QUIC is a multiplexed stream transport protocol over UDP.",\
    "Fuze Meeting is the protocol of the corresponding free online meeting service.",\
    "The Age of Warring Empire protocol was deprecated and should not be used anymore. Instead, please use the Elex Technology protocol.",\
    "The protocol Doook was deprecated and should not be used anymore.",\
    "Speedtest is an Internet connection quality measurement tool from Ookla.",\
    "IEC 61850 is a standard for the design of electrical substations automation.",\
    "SCADA (IEC 60870-5-104) is a control and data transfer protocol used in industrial applications.",\
    "Shoutcast is the protocol of the corresponding media streaming server software developed by Nullsoft.",\
    "CQL (Cassandra Query Language) is the client protocol used to access Cassandra NoSQL databases.",\
    "CIC (Cassandra Internode Communication) is the protocol used for Cassandra server to server traffic.",\
    "Thrift is a protocol for scalable, interoperational services.",\
    "PlayStation Network is the protocol of the corresponding traffic of PlayStation video game consoles such as PS Vita, PS3, PS4 or PS5.",\
    "Supercell is the protocol of the corresponding traffic from Supercell games.",\
    "The protocol Skype for Business was deprecated and should not be used anymore.",\
    "T3 is a proprietary protocol for RMI communication with Oracle WebLogic Server.",\
    "Secure RTP is a security profile for RTP that adds confidentiality, message authentication, and replay protection to that protocol.",\
    "Secure RTCP is a sister protocol of the Secure Real-time Transport Protocol (SRTP). Its basic functionality and packet structure is defined in the SRTP specification RFC 3711.",\
    "League of Legends is a multiplayer game developed and published by Riot Games.",\
    "Facebook is a UDP-based proprietary protocol used by the Facebook application for producing/uploading live-streams.",\
    "BattleNet is a proprietary protocol used by Blizzard Entertainment for all BattleNet Games.",\
    "Wowza Streaming Engine is a streaming media server software which allows to stream different formats. They offer also a client application for Android and iOS named GoCoder.",\
    "Zalo is a Vietnamese mobile messaging app.",\
    "Guild Wars 2 is a multiplayer online role-playing game developed by ArenaNet and published by NCsoft.",\
    "The CrashPlan protocol was deprecated and should not be used anymore.",\
    "EA Origin is an online gaming, digital distribution and digital rights management platform developed by Electronic Arts.",\
    "Fasttrack was deprecated and should not be used anymore.",\
    "Icecast is an open source streaming project.",\
    "The ChatCube protocol was deprecated and should not be used anymore.",\
    "Good Technology makes products for managing and securing mobile devices in a business environment.",\
    "Path of Exile is a free to play online action role-playing game.",\
    "PC over IP is a remote display protocol.",\
    "OCSP (Online Certificate Status Protocol) is an Internet protocol used for obtaining the revocation status of an X.509 digital certificate.",\
    "Modbus is a serial communication protocol. It is an industrial standard communication protocol.",\
    "FASP (Fast and Secure Protocol) is a network protocol optimizing data traffic and developed by Aspera Inc.",\
    "ICA (Independent Computing Architecture) is a protocol for an application server system designed by Citrix Systems.",\
    "OPC UA (Open Platform Communications - Unified Architecture) is an industrial M2M communication protocol for interoperability developed by the OPC Foundation.",\
    "OpenFlow is a communications protocol that gives access to the forwarding plane of a network switch or router over the network.",\
    "Intelligent Platform Management Interface (IPMI) is a standardized interface for monitoring of server-hardware.",\
    "AMQP (Advanced Message Queuing Protocol) is an open standard for middleware systems. It's used to transmit business messages or instructions between different applications or organizations.",\
    "S7comm (S7 Communication) is a Siemens proprietary protocol that runs between programmable logic controllers (PLCs) of the Siemens S7-300/400 family.",\
    "OFF is the protocol of the p2p file sharing application with the same name.",\
    "Rsync is an utility for efficiently transferring and synchronizing files across computer systems.",\
    "ZeroMQ is a high-performance asynchronous messaging library, aimed at use in distributed or concurrent applications.",\
    "DNP3 (Distributed Network Protocol) is a set of communications protocols used between components in process automation systems.",\
    "BACNET (Building Automation and Control Networks) is a network protocol for building automation.",\
    "Nintendo Network is an online service provided by Nintendo. It provides online functionality for the Wii U, Nintendo 3DS and their games.",\
    "PPFilm is the UDP-Protocol used by the application PPFilm.",\
    "Bitcoin is an encrypted digital currency.",\
    "LeTV is a proprietary protocol used by the streaming application with the same name.",\
    "Psiphon is an open-source tool that uses a combination of secure communication and obfuscation technologies (VPN, SSH, and HTTP Proxy). It is specifically designed for Internet censorship circumvention.",\
    "Zero is a protocol inspired by QUIC and developed by the Facebook Company.",\
    "Synology provides network attached storage drives with the ability to synchronize with several online clouds.",\
    "CoAP (Constrained Application Protocol) is a web transfer protocol for IoT, respectively embedded systems.",\
    "Lineage 2 is the game server protocol of Lineage 2, a popular MMORPG from NCSoft.",\
    "IPFIX is a network reporting protocol and successor of netflow, developed by Cisco. It provides flow-based IP traffic measurements.",\
    "Minecraft is a popular game from the development studio Mojang and is written in Java.",\
    "Stratum Mining is a protocol for bitcoin mining.",\
    "Local Peer Discovery is an extension to the BitTorrent protocol, developed to minimize traffic when discovering local peers. This protocol is specified in the BEP-14 proposal.",\
    "Zero VPN was a proprietary protocol used by the Zero VPN app. The backend services are not reachable anymore, therefore the protocol was deprecated.",\
    "Turbo VPN was deprecated and should not be used anymore. Instead the protocol Innovative Connecting should be used.",\
    "VPN Master was deprecated and should not be used anymore. Instead the protocol Innovative Connecting should be used.",\
    "Iqiyi is the protocol of the corresponding Chinese peer-to-peer streaming video network software. It was formerly known as PPStream.",\
    "Snap VPN was deprecated and should not be used anymore. Instead the protocol Innovative Connecting should be used.",\
    "VPN Unlimited is a proprietary protocol used by the VPN Unlimited application.",\
    "ExpressVPN is a proprietary protocol used by the Express VPN application.",\
    "TunnelGuru is a proprietary protocol used by the VPN applications provided by the company TunnelGuru.",\
    "BOTIM is a protocol used by the VoIP app also called BOTIM.",\
    "Hi VPN is a proprietary protocol used by the VPN application Hi VPN provided by the company Hi Security Lab.",\
    "Speed VPN MobileAppDev is deprecated and should not be used anymore.",\
    "Dash VPN is a protocol used by application Dash VPN.",\
    "YeeCall is a proprietary protocol used by the YeeCall app.",\
    "The Atom VPN protocol was deprecated and should not be used anymore.",\
    "The Fortnite protocol is deprecated and should not be used anymore. Instead, the traffic will be covered by the Unreal Engine protocol.",\
    "WireGuard is an encryption protocol similar to IPsec or OpenVPN.",\
    "Amaze VPN is a proprietary protocol used by the Amaze VPN app.",\
    "Fast Secure VPN is a proprietary protocol used by the Fast Secure VPN app.",\
    "ZPN was deprecated and should not be used anymore.",\
    "Removed. Reason: The protocol is not specific for the application World Of Tanks. Instead, all games by the company Wargaming.net are using the same protocol. See instead: Wargaming-Net protocol.",\
    "AppVPN is a proprietary protocol used by the AppVPN application.",\
    "X-VPN is a proprietary protocol used by the X-VPN app.",\
    "Droid VPN is a proprietary protocol used by the Droid VPN application.",\
    "DTLS is the UDP version of the known TLS protocol.",\
    "Smart VeePN is a proprietary protocol developed and used by VeePN Inc.",\
    "Apex Legends is the proprietary in-game protocol of the video game Apex Legends.",\
    "VPN 365 is a proprietary protocol used by the VPN 365 app.",\
    "The CounterStrikeGO protocol is deprecated. Instead, the protocol Source Engine should be used.",\
    "Removed. Reason: The SuperVPN by Cheng application never actually used its own proprietary protocol, but instead used OpenVPN.",\
    "Act VPN is a proprietary protocol used by the Act VPN application.",\
    "The Hotspot VPN protocol is deprecated and should not be used anymore. Instead, the traffic will be covered by the Sapling Growth VPN Services protocol.",\
    "MTProto (MTProto Mobile Protocol) is the proprietary protocol used by the application Telegram. It's designed for access to a server API from applications running on mobile devices.",\
    "Shadowsocks is an encrypted proxy based on SOCKS5.",\
    "Asphalt9 was deprecated and should not be used anymore. Instead use the protocol Gameloft-Servies.",\
    "Actmobile Services is a protocol used by the application Actmobile Services.",\
    "ZRTP is a cryptographic key-agreement protocol to negotiate the keys for encryption between two end points in a VoIP phone telephony call based on RTP.",\
    "PUBG Mobile is a proprietary protocol used by the PUBG Mobile application.",\
    "Gameloft Services is a proprietary protocol used by the Gameloft-Services application.",\
    "Wickr is a proprietary protocol used by the Wickr application.",\
    "Anchorfree Services is the proprietary protocol used by some of the applications provided by Anchorfree Services.",\
    "The VPN Proxy Master Unlimited protocol is deprecated and should not be used anymore. Instead, the traffic will be covered by the Sapling Growth VPN Services protocol.",\
    "Unreal Engine is the proprietary protocol used by the Unreal Engine application.",\
    "Zoom is a proprietary protocol used by the Zoom video conferencing application.",\
    "VPN Proxy Master was deprecated and should not be used anymore. Instead the protocol Innovative Connecting should be used.",\
    "Dead Rivals was deprecated and should not be used anymore. Instead use the protocol Gameloft-Services.",\
    "IGG (I Got Games) Inc. is a proprietary protocol used by the IGG gaming applications.",\
    "Mudfish is a proprietary protocol used by the Mudfish VPN application.",\
    "The protocol SOMA was deprecated and should not be used anymore.",\
    "Hatch Cloud Gaming is a proprietary protocol used by the Hatch Cloud Gaming application.",\
    "Source Engine is the protocol of the game engine with the same name.",\
    "The protocol Streamcraft was deprecated and should not be used anymore.",\
    "Youku Tudou is a proprietary protocol used by the application with the same name.",\
    "PdaNet is a proprietary protocol used by the application with the same name.",\
    "HTTP/2 is a major revision of the HTTP network protocol used to communicate with web servers.",\
    "ToTok is a proprietary protocol used by the application with the same name.",\
    "BIGO Services is a proprietary protocol used by the application with the same name.",\
    "WebM is a protocol for streaming content over HTTP.",\
    "The protocol HideMe VPN was deprecated and should not be used anymore. The protocol SpongyCastle should be used instead",\
    "IPVanish is a proprietary protocol used by the IPVanish application.",\
    "Microsoft Services is a proprietary protocol used by different Microsoft applications such as Microsoft Teams and Skype for establishing VoIP-calls.",\
    "The Routing Information Protocol (RIP) is a distance vector routing protocol used for automatic routing table creation.",\
    "PrivateVPN is a proprietary protocol used by the PrivateVPN application.",\
    "SpongyCastle is a proprietary protocol used by PrivateVPN and Hide.me VPN applications.",\
    "Surfshark VPN is a proprietary protocol used by the application with the same name.",\
    "Sapling Growth Technology Co. Ltd. provides several VPN applications for mobile platforms.",\
    "IEC 13818 is a standard for the transmission of audio and video data.",\
    "Tencent is a proprietary protocol used by the Tencent Services application.",\
    "UltraVPN is a proprietary protocol used by the Wirelab IO VPN services.",\
    "WeChat Work is a proprietary protocol used by the application with the same name.",\
    "Alibaba is a proprietary protocol used by the application with the same name.",\
    "ISIS (Intermediate System to Intermediate System Protocol) is a layer 3 routing protocol, designed for use within an administrative domain or network.",\
    "The RakNet protocol is used by gaming applications for real time communication.",\
    "Innovative Connecting is a proprietary protocol used by several applications provided by this company, like Turbo VPN, Snap VPN and VPN Proxy Master.",\
    "ViPNet is a proprietary protocol created by the company Infotecs GmbH.",\
    "RESP (REdis Serialization Protocol) is a simple human-readable, request-response protocol originally designed for Redis.",\
    "New Office Environment is a proprietary telephony signaling protocol provided by Alcatel",\
    "Agora is a propriatary protocol for real-time audio and video streaming and chat.",\
    "MIOP (Unreliable Multicast Inter-ORB Protocol) is used to deliver GPIO (General Inter-ORB Protocol) requests and fragmet messages via multicast.",\
    "GIOP (General Inter-ORB Protocol) is used to communicate between Object Request Brokern (ORB).",\
    "Elex Technology is a proprietary protocol used by different gaming applications developed by the Chinese company Elex Technology.",\
    "JSON-RPC is a simple remote procedure call protocol based on the JSON data format.",\
    "ZIOP is the compressed variant of the GIOP protocol to prevent bandwidth issues.",\
    "UFTP is a protocol used for encrypted UDP based file transfer with multicast.",\
    "Moonton is a proprietary protocol used by different gaming applications developed by the Chinese company Moonton.",\
    "RTPS (Real-Time Publish-Subscribe) Wire Protocol is used for industrial automation and is designed to run on an unreliable transport mechanism, such as UDP/IP.",\
    "Ethereum is a proprietary protocol used by services running on top of the Ethereum blockchain platform.",\
    "The protocol Zynn was deprecated and should not be used anymore.",\
    "sFlow is a proprietary protocol used to monitoring high-speed switched and routed networks.",\
    "collectd is a daemon which collects system and application performance metrics periodically and provides mechanisms to store the data in a variety of ways.",\
    "HL7 protocol provides a framework (and related standards) for the exchange, integration, sharing, and retrieval of electronic health information.",\
    "Among Us is the proprietary protocol used by the application with the same name.",\
    "EtherNetIP is an industrial network to adapt common industrial protocols to standard Ethernet.",\
    "DNSCrypt is a protocol that authenticates and encrypts communications between a DNSCrypt client/proxy and a DNSCrypt resolver to enhance network security.",\
    "ONC-RPC is a remote procedure call protocol originally developed by Sun Microsystems.",\
    "VXI-11 is a protocol for the remote control of instruments such as oscilloscopes.",\
    "HiSLIP (High-Speed LAN Instrument Protocol) is the successor of the VXI-11 remote control protocol.",\
    "The IEEE C37.118 (Synchrophasor) protocol is used in phasor networks for synchronized real-time measurements between different Phasor Measurement Units (PMU).",\
    "The IEC 62056 (Device Language Message Specification/Companion Specification for Energy Metering) protocol is used to exchange data with a wide range of smart metering devices.",\
    "The IEEE 1703 (ANSI C12.22) protocol is used by electric power suppliers in smart grids for the exchange of smart meter data tables.",\
    "Gaijin Entertainment is a proprietary protocol used by different gaming applications developed by Gaijin Entertainment.",\
    "MicoWorld Services is a proprietary protocol used by different social networking applications developed by the Chinese company MicoWorld Limited.",\
    "Photon is a proprietary protocol for multiplayer games such as Golf Clash.",\
    "Ninja Kiwi Services is a proprietary protocol used by different gaming applications developed by Ninja Kiwi.",\
    "Kaspersky Services is a proprietary protocol used by different security and VPN applications developed by the company Kaspersky Lab.",\
    "Call of Duty Mobile is the proprietary protocol of the corresponding multiplayer online game.",\
    "Activision Services is a proprietary protocol used by different gaming applications developed by Activision Publishing, Inc..",\
    "CamScanner is the proprietary protocol used by the application with the same name.",\
    "LoL Wild Rift is the proprietary protocol of the corresponding multiplayer online game.",\
    "The TASE.2 (Telecontrol Application Service Element) is based on Manufacturing Messaging Specification and is used for the remote control and data exchange between control centers of power plants.",\
    "The protocol Riley Cillian was deprecated and should not be used anymore. Instead the Agora protocol will cover the related traffic.",\
    "Litecoin is a cryptographic protocol that is used in P2P networks with the same name.",\
    "Portmap is a service that maps ONC-RPC services and network ports.",\
    "Hago is the proprietary protocol used by the application with the same name.",\
    "miHoYo Games is a proprietary protocol used by different gaming applications developed by miHoYo Co., Ltd..",\
    "Snail VPN was deprecated and should not be used anymore.",\
    "SuperVPN is the proprietary protocol used by the application with the same name.",\
    "AllJoyn is an IoT protocol that enables devices to interact at the app-to-app level.",\
    "Noon VPN is a proprietary protocol used by the application with the same name.",\
    "Riot Games is a proprietary protocol used by different gaming applications developed by Riot Games, Inc.",\
    "Garena Free Fire is the proprietary protocol of the corresponding multiplayer online game.",\
    "STP (Spanning Tree Protocol) is a protocol used in networks of Layer 2 bridges to ensure loop-free topology for any bridged Ethernet LAN.",\
    "Nord VPN is a proprietary protocol used by the application with the same name.",\
    "Lightsail VPN is a proprietary protocol used by the application with the same name.",\
    "LLDP (Link Layer Discovery Protocol) is a data link layer protocol that allows devices to advertise device information to adjacent devices for reachability.",\
    "LLC (Logical Link Control) is a protocol used in networks of Layer 2 as mechanism for multiplexing, on networks using IEEE 802.2 LLC.",\
    "DingTalk is the proprietary protocol used by the application with the same name.",\
    "The Manufacturing Messaging Specification protocol as a standard is used to exchange data in distributed automation environments.",\
    "STOMP (Simple/Streaming Text Orientated Messaging Protocol) is a text-based interoperable messaging protocol designed to talk with message brokers.",\
    "Ethernet Powerlink is a real-time protocol for standard Ethernet.",\
    "The Ubitus is a proprietary protocol used by different cloud gaming applications developed by Ubitus K.K..",\
    "EtherCAT is an Ethernet-based protocol suitable for both hard and soft real-time computing requirements in automation technology.",\
    "ARP (Address Resolution Protocol) is a network infrastructure protocol used to find link layer addresses to corresponding internet layer addresses.",\
    "Profinet is an industry technical standard for data communication over Industrial Ethernet.",\
    "Ouroboros is a proof-of-stake blockchain consensus protocol which is used by the Cardano blockchain platform.",\
    "EIGRP (Enhanced Interior Gateway Routing Protocol) is a Cisco developed distance-vector routing protocol. It was first developed as proprietary protocol and was published as RFC 7868 in 2016.",\
    "Common Industrial Protocol is a protocol for industrial automation applications.",\
    "CDP (Cisco Discovery Protocol) is a proprietary protocol developed by Cisco Systems. It is a Data Link Layer service that shares information about Cisco devices.",\
    "Valorant is a proprietary protocol used by the application with the same name.",\
    "VTP (VLAN Trunking Protocol) is a proprietary protocol developed by Cisco Systems. It carries VLAN information to all the switches in a VTP domain.",\
    "The Multimedia Messaging Service Encapsulation (MMSE) protocol is used by the Multimedia Messaging Service, that allows users to exchange messages including multimedia content with their mobile phones.",\
    "DTP (Dynamic Trunking Protocol) is a proprietary protocol developed by Cisco Systems. It negotiates the trunking on a link between switches or the type of the used trunking encapsulation.",\
    "Nvidia is a proprietary protocol used by the Nvidia Geforce NOW application.",\
    "Ubisoft is a proprietary protocol used by different games of the Ubisoft company.",\
    "VRRP (Virtual Router Redundancy Protocol) is an open standard (RFC 5798) that provides automatic assignment of available IP routers to related hosts. RFC 5798 is published by the IETF.",\
    "PPPoE (PPP over Ethernet) is a network protocol that encapsulates PPP to provide this information over Ethernet.",\
    "VXLAN (Virtual Extensible LAN) is a network virtualization technology that encapsulates layer 2 Ethernet frames within layer 4 UDP datagrams.",\
    "Cloudflare is a proprietary protocol used by the products and services of the company with the same name.",\
    "FINS (Factory Interface Network Service) is a protocol for sending messages between controllers on OMRON networks.",\
    "Packet Forwarding Control Protocol (PFCP) is a 3GPP protocol used on the Sx/N4 interface between the control plane and the user plane function to formalize the interaction between different types of functional elements of the mobile core.",\
    "Lost Ark is a proprietary protocol of the corresponding multiplayer online video game.",\
    "Polkadot is a protocol which allows different blockchains to exchange messages with each other.",\
    "Mumble is a proprietary protocol used by the application with the same name.",\
    "Naraka Bladepoint is the proprietary protocol of the corresponding multiplayer online game.",\
    "Final Fantasy XIV is the proprietary protocol of the corresponding multiplayer online video game.",\
    "Ethernet is a protocol for wired computer networks standardized as IEEE 802.3.",\
    "NHRP (Next Hop Resolution Protocol) is an open standard configuration technology that routes data on a distributed network by discovering the best routing path between endpoints.",\
    "Pinduoduo is the proprietary protocol used by the application with the same name.",\
    "Cisco Metadata is a protocol used to carry the Scalable Group Tags (SGT) as inline metadata at the Layer 2 level.",\
    "IEEE 8021Q is a networking standard that adds support for virtual local area networking (VLANs) on Ethernet networks.",\
    "IEEE 8021AD is a networking standard that adds support for virtual local area networking (VLANs) on Ethernet networks.",\
    "MPLS (Multiprotocol Label Switching) is a routing technique based on labels that can encapsulate packets of various network protocols.",\
    "Fall Guys is the proprietary protocol of the corresponding multiplayer online game.",\
    "Removed. Reason: The protocol is not specific for the game Rogue Company. Instead, all games by the company Hi-Rez are using the same protocol. See instead: Hi-Rez Services protocol.",\
    "IEC 61850 GOOSE (Generic Object Oriented Substation Event) is a standard protocol used for information exchange between Intelligent Electronic Devices (IEDs) over Ethernet.",\
    "IEC 61850 Sampled Values (SV) is a standard protocol used for information exchange between Merging Units and Intelligent Electronic Devices (IEDs) over Ethernet.",\
    "IEEE 8021AH is a networking standard for routing over a provider's network allowing interconnection of multiple provider bridge networks.",\
    "NetEase is a proprietary protocol used by different games of the NetEase company.",\
    "H245 is a control channel protocol used within H323 communication sessions.",\
    "H225 is a control channel protocol used within H323 communication sessions.",\
    "T120 is a suite of teleconferencing protocols used within H323 communication sessions.",\
    "Hi-Rez Services is a proprietary protocol used by various video games by the corporation Hi-Rez.",\
    "Cryptic Studios is a proprietary protocol used by different games developed by Cryptic Studios.",\
    "CCP Games is a proprietary protocol used by different gaming applications developed by the company CCP Games.",\
    "Wargaming-Net is a proprietary protocol used by various video games by the corporation Wargaming.net.",\
    "Kafka is a stream-processing and message broker system by Apache.",\
    "DiDi Services is a proprietary protocol used by various applications by the Chinese company DiDi.",\
    "V4 is a proprietary protocol used by the application with the same name.",\
    "PTP (Precision Time Protocol) is a protocol used to synchronize clocks throughout a computer network.",\
    "Escape from Tarkov is a proprietary protocol used by the application with the same name.",\
    "Zangi is a proprietary protocol used by the application Zangi Messenger, mainly to transfer voice and video call data.",\
    "NoMachine NX is a protocol for remote access, desktop sharing, virtual desktop and file transfer between computers."

/* Public Prototypes ************************************************************/

#include "pace2_applications.h"

/**
 * @fn PACE2_return_state pace2_get_application_attributes_of_protocol(PACE2_module * const pace2_module, int const
 * thread_ID, PACE2_protocols const protocol_id, PACE2_application_attributes_mask * const application_attributes)
 *
 * @brief This function provides the used application attributes for the given protocol.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @deprecated The usage of this function is deprecated. Please use
 *             @ref pace2_get_number_of_application_attributes_of_protocol and
 *             @ref pace2_get_application_attribute_of_protocol_by_index instead.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   protocol_id             The protocol ID.
 * @param[out]  application_attributes  Pointer to bitmask of type @ref PACE2_application_attributes_bitmask
 *                                      "PACE2_application_attributes_bitmask".
 *
 * @retval      PACE2_SUCCESS           @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_PARAM_MISSING     @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval      PACE2_FAILURE           @copydoc PACE2_return_state::PACE2_FAILURE
 *                                      The given protocol id does not exist.
 */
PACE2_DEPRECATED(
    "Deprecated, Please use pace2_get_number_of_application_attributes_of_protocol and "
    "pace2_get_application_attribute_of_protocol_by_index instead.")
PACE2_return_state pace2_get_application_attributes_of_protocol(
    PACE2_module * const pace2_module,
    int const thread_ID,
    PACE2_protocols const protocol_id,
    PACE2_application_attributes_mask * const application_attributes);

/**
 * @fn PACE2_return_state pace2_get_protocol_attributes_of_protocol(PACE2_module * const pace2_module, int const
 * thread_ID, PACE2_protocols const protocol_id, PACE2_protocol_attributes_mask * const protocol_attributes)
 *
 * @brief This function provides the used protocol attributes for the given protocol.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @deprecated The usage of this function is deprecated. Please use
 *             @ref pace2_get_number_of_protocol_attributes_of_protocol and
 *             @ref pace2_get_protocol_attribute_of_protocol_by_index instead.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   protocol_id             The protocol ID.
 * @param[out]  protocol_attributes     Pointer to bitmask of type @ref PACE2_protocol_attributes_bitmask
 *                                      "PACE2_protocol_attributes_bitmask".
 *
 * @retval      PACE2_SUCCESS           @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_PARAM_MISSING     @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval      PACE2_FAILURE           @copydoc PACE2_return_state::PACE2_FAILURE
 *                                      The given protocol id does not exist.
 */
PACE2_DEPRECATED(
    "Deprecated, Please use pace2_get_number_of_protocol_attributes_of_protocol and "
    "pace2_get_protocol_attribute_of_protocol_by_index instead.")
PACE2_return_state pace2_get_protocol_attributes_of_protocol(
    PACE2_module * const pace2_module,
    int const thread_ID,
    PACE2_protocols const protocol_id,
    PACE2_protocol_attributes_mask * const protocol_attributes);

/**
 * @brief This function provides the short string for the given protocol.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   protocol_id             The protocol ID.
 * @returns     A char pointer to the protocol short string or NULL when one of the input parameters in invalid.
 */
const char * pace2_get_protocol_short_string(PACE2_module * const pace2_module, int const thread_ID,
                                             PACE2_protocols const protocol_id);

/**
 * @brief This function provides the long string for the given protocol.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   protocol_id             The protocol ID.
 * @returns     A char pointer to the protocol long string or NULL when one of the input parameters is invalid.
 */
const char * pace2_get_protocol_long_string(PACE2_module * const pace2_module, int const thread_ID,
                                            PACE2_protocols const protocol_id);

/**
 * @brief This function provides the description for the given protocol.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   protocol_id             The protocol ID.
 * @returns     A char pointer to the protocol description or NULL when one of the input parameters is invalid.
 */
const char * pace2_get_protocol_description(PACE2_module * const pace2_module, int const thread_ID,
                                            PACE2_protocols const protocol_id);

/**
 * @brief This function provides the used protocols by the given application.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   application_id          The application ID.
 * @param[out]  protocols               Pointer to protocols of type @ref PACE2_protocols "PACE2_protocols".
 * @param[out]  number_of_protocols     Pointer to number_of_protocols of type @ref u32.
 * @retval      PACE2_SUCCESS           @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_FAILURE           @copydoc PACE2_return_state::PACE2_FAILURE
 *                                      The given application id does not exist.
 */
PACE2_return_state pace2_get_used_protocols_of_application(PACE2_module * const pace2_module,
                                                           int const thread_ID,
                                                           PACE2_applications const application_id,
                                                           PACE2_protocols const ** protocols,
                                                           u32 * number_of_protocols);

/**
 * @brief This function provides the used layer 7 protocols by the given application.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   application_id          The application ID.
 * @param[out]  protocols               Pointer to protocols of type @ref PACE2_protocols "PACE2_protocols".
 * @param[out]  number_of_protocols     Pointer to number_of_protocols of type @ref u32.
 * @retval      PACE2_SUCCESS           @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_FAILURE           @copydoc PACE2_return_state::PACE2_FAILURE
 *                                      The given application id does not exist.
 */
PACE2_return_state pace2_get_used_l7_protocols_of_application(PACE2_module * const pace2_module,
                                                              int const thread_ID,
                                                              PACE2_applications const application_id,
                                                              PACE2_protocols const ** protocols,
                                                              u32 * number_of_protocols);

/**
 * @brief This function provides the used OSI layers of the given protocol.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls its upgrade variant.
 *
 * @param[in]   pace2_module                Constant pointer to the PACE 2 module.
 *                                          Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID                   The ID of the thread currently processing.
 *                                          Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   protocol_id                 The protocol ID.
 * @param[out]  osi_layers                  Pointer to OSI layers of type @ref PACE2_result_protocol_layer_indexes.
 * @param[out]  number_of_used_osi_layers   Pointer to number_of_used_osi_layers of type @ref u8.
 * @retval      PACE2_SUCCESS               @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_FAILURE               @copydoc PACE2_return_state::PACE2_FAILURE
 *                                          The given protocol id does not exist.
 */
PACE2_return_state pace2_get_osi_layers_of_protocol(PACE2_module * const pace2_module,
                                                    int const thread_ID,
                                                    PACE2_protocols const protocol_id,
                                                    PACE2_result_protocol_layer_indexes const ** osi_layers,
                                                    u8 * number_of_used_osi_layers);

/**
 * @brief This function provides the information if a protocol is already excluded or not.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version. This function should be used instead of the exldued bitmask inside
 * @ref PACE2_CLASSIFICATION_RESULT . The exclude bitmask inside @ref PACE2_CLASSIFICATION_RESULT
 * does not contain information about new protocols if the bitmask growth and might result in invalid access.
 *
 * @param[in]   pace2_module                    Constant pointer to the PACE 2 module.
 * @param[in]   thread_ID                       The ID of the thread currently processing.
 * @param[in]   flow                            Pointer to the PACE 2 flow.
 * @param[in]   protocol_id                     The protocol ID.
 * @param[out]  is_excluded                     Pointer to is_excluded of type @ref u8. Must not be NULL.
 *                                              Value 0 means the given protocol id is not yet excluded.
 *                                              Value 1 means the given protocol id is excluded.
 *                                              Value 2 is the default initialized value. The protocol is either not yet
 *                                              excluded or excluded. The PACE2_return_state should be checked for more
 *                                              error information.
 * @retval      PACE2_SUCCESS                   @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_FAILURE                   @copydoc PACE2_return_state::PACE2_FAILURE
 *                                              The given protocol id does not exist.
 * @retval      PACE2_POINTER_NULL              @copydoc PACE2_return_state::PACE2_POINTER_NULL
 *                                              The flow pointer is NULL.
 * @retval      PACE2_THREAD_ID_INVALID         @copydoc PACE2_return_state::PACE2_THREAD_ID_INVALID
 * @retval      PACE2_INVALID_MODULE_POINTER    @copydoc PACE2_return_state::PACE2_INVALID_MODULE_POINTER
 * @retval      PACE2_PARAM_MISSING             @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 */
PACE2_return_state pace2_is_protocol_excluded(PACE2_module * const pace2_module,
                                              int const thread_ID,
                                              void * const flow,
                                              PACE2_protocols const protocol_id,
                                              u8 * is_excluded);

/**
 * @brief This function provides the number of available application attributes for the given protocol ID.
 *
 * @param[in]  pace2_module         Constant pointer to the PACE 2 module
 *                                  Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]  thread_ID            The ID of the thread currently processing.
 *                                  Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]  protocol_id          The protocol ID.
 * @param[out] number_of_attributes The number of available application attributes for @p protocol_id.
 *
 * @retval     PACE2_SUCCESS        @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval     PACE2_FAILURE        @copydoc PACE2_return_state::PACE2_FAILURE
 */
PACE2_return_state pace2_get_number_of_application_attributes_of_protocol(PACE2_module * const pace2_module,
                                                                          int const thread_ID,
                                                                          PACE2_protocols const protocol_id,
                                                                          u16 * const number_of_attributes);

/**
 * @brief This function provides the number of available protocol attributes for the given protocol ID.
 *
 * @param[in]  pace2_module         Constant pointer to the PACE 2 module
 *                                  Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]  thread_ID            The ID of the thread currently processing.
 *                                  Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]  protocol_id          The protocol ID.
 * @param[out] number_of_attributes The number of available protocol attributes for @p protocol_id.
 *
 * @retval     PACE2_SUCCESS        @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval     PACE2_FAILURE        @copydoc PACE2_return_state::PACE2_FAILURE
 */
PACE2_return_state pace2_get_number_of_protocol_attributes_of_protocol(PACE2_module * const pace2_module,
                                                                       int const thread_ID,
                                                                       PACE2_protocols const protocol_id,
                                                                       u16 * const number_of_attributes);

/**
 * @brief This function can be used to iterate over the number of application attributes for a specific protocol. The
 *        number of attributes for this protocol has to be determined with
 *        @ref pace2_get_number_of_application_attributes_of_protocol beforehand.
 *
 * @param[in] pace2_module      Constant pointer to the PACE 2 module.
 *                              Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in] thread_ID         The ID of the thread currently processing.
 *                              Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in] protocol_id       The protocol ID.
 * @param[in] attribute_index   Index of the attribute.
 *
 * @return The attribute of type enum pace2_application_attributes for the given @p protocol_id and
 *         @p attribute_index on success. @ref PACE2_APPLICATION_ATTRIBUTE_NONE otherwise and in all error cases.
 */
enum pace2_application_attributes pace2_get_application_attribute_of_protocol_by_index(
    PACE2_module * const pace2_module, int const thread_ID, PACE2_protocols const protocol_id, u16 attribute_index);

/**
 * @brief This function can be used to iterate over the number of protocol attributes for a specific protocol. The
 *        number of attributes for this protocol has to be determined with
 *        @ref pace2_get_number_of_protocol_attributes_of_protocol beforehand.
 *
 * @param[in] pace2_module      Constant pointer to the PACE 2 module.
 *                              Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in] thread_ID         The ID of the thread currently processing.
 *                              Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in] protocol_id       The protocol ID.
 * @param[in] attribute_index   Index of the attribute.
 *
 * @return The attribute of type enum pace2_protocol_attributes for the given @p protocol_id and
 *         @p attribute_index on success. @ref PACE2_PROTOCOL_ATTRIBUTE_NONE otherwise and in all error cases.
 */
enum pace2_protocol_attributes pace2_get_protocol_attribute_of_protocol_by_index(PACE2_module * const pace2_module,
                                                                                 int const thread_ID,
                                                                                 PACE2_protocols const protocol_id,
                                                                                 u16 attribute_index);

#ifdef __cplusplus
}
#endif
#endif /* PACE2_PROTOCOLS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
