/**********************************************************************************************************************/
/**
 **
 ** @file       pace2_stail_queue.h
 ** @brief      This file defines singly-linked tail queue structures which are taken from sys/queue.h
 **             This port / renaming was done to make the STAILQ defines available for non BSD
 **             builds e.g. windows.
 **             The code in this file is taken from sys/queue.h with the following original copyright notice:
 **
 ** Copyright (c) 1991, 1993
 **  The Regents of the University of California.  All rights reserved.
 **  COPYRIGHT (C) 2014 - 2022 BY IPOQUE GMBH
 **
 ** Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that the following conditions
 ** are met:
 ** 1. Redistributions of source code must retain the above copyright
 **    notice, this list of conditions and the following disclaimer.
 ** 2. Redistributions in binary form must reproduce the above copyright
 **    notice, this list of conditions and the following disclaimer in the
 **    documentation and/or other materials provided with the distribution.
 ** 3. Neither the name of the University nor the names of its contributors
 **    may be used to endorse or promote products derived from this software
 **    without specific prior written permission.
 **
 ** THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 ** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ** ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 ** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 ** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 ** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 ** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 ** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 ** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 ** SUCH DAMAGE.
 **
 **  @(#)queue.h 8.5 (Berkeley) 8/20/94
 **/
/**********************************************************************************************************************/

#ifndef PACE2_STAIL_QUEUE_H_
#define PACE2_STAIL_QUEUE_H_

/*
 * Singly-linked Tail queue declarations.
 */
#define PACE2_STAILQ_HEAD(name, type)                                                  \
struct name {                                                                          \
        struct type *stqh_first; /* first element */                                   \
        struct type **stqh_last; /* addr of last next element */                       \
}

#define PACE2_STAILQ_ENTRY(type)                                                       \
struct {                                                                               \
        struct type *stqe_next; /* next element */                                     \
}

#define PACE2_STAILQ_EMPTY(head)      ((head)->stqh_first == NULL)

#define PACE2_STAILQ_FIRST(head)      ((head)->stqh_first)

#define PACE2_STAILQ_INIT(head) do {                                                   \
        PACE2_STAILQ_FIRST((head)) = NULL;                                             \
        (head)->stqh_last = &PACE2_STAILQ_FIRST((head));                               \
} while (0)

#define PACE2_STAILQ_INSERT_TAIL(head, elm, field) do {                                \
        PACE2_STAILQ_NEXT((elm), field) = NULL;                                        \
        *(head)->stqh_last = (elm);                                                    \
        (head)->stqh_last = &PACE2_STAILQ_NEXT((elm), field);                          \
} while (0)

#define PACE2_STAILQ_NEXT(elm, field) ((elm)->field.stqe_next)

#define PACE2_STAILQ_REMOVE_HEAD(head, field) do {                                     \
        if ((PACE2_STAILQ_FIRST((head)) =                                              \
             PACE2_STAILQ_NEXT(PACE2_STAILQ_FIRST((head)), field)) == NULL)            \
                (head)->stqh_last = &PACE2_STAILQ_FIRST((head));                       \
} while (0)

#define PACE2_STAILQ_FOREACH(var, head, field)                                         \
        for ((var) = ((head)->stqh_first);                                             \
                (var);                                                                 \
                (var) = ((var)->field.stqe_next))

#endif /* PACE2_STAIL_QUEUE_H_ */
