/** @file      pace2_status_dns.h
  * @brief     PACE 2 module wrapper for DNS information functions.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_STATUS_DNS_H
#define PACE2_STATUS_DNS_H

/* Includes *********************************************************************/
#include "pace2_basic_def.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public Types *****************************************************************/

/**
 * @brief Return state of DNS information functions.
 */
typedef enum {
    PACE2_DNS_SUCCESS,                  /**< Function returned successfully, returned data is valid. */
    PACE2_DNS_BAD_PARAM,                /**< Function parameter invalid. */
    PACE2_DNS_NO_L7_PAYLOAD,            /**< No Layer 7 payload found. */
    PACE2_DNS_PARSING_FAILED,           /**< Parsing failed. */
} PACE2_DNS_return_state;

/* Public Prototypes ************************************************************/

/**
 * @brief Get human readable string representation of a DNS domain name from Layer 7 data.
 *
 * @param[in]   pd              Pointer to the packet descriptor.
 * @param[in]   content_ptr     Pointer to the DNS domain name inside the Layer 7 payload.
 * @param[in]   max_pointers    Maximum number of pointers to follow.
 * @param[out]  dst             Pointer to the already allocated destination buffer.
 * @param[out]  dst_len         Length of the dst buffer / Maximum length that will be written to dst.
 * @param[out]  len             Pointer to the length of the domain name / amount of characters written to dst.
 *
 * @retval PACE2_DNS_SUCCESS            @copydoc PACE2_DNS_return_state::PACE2_DNS_SUCCESS
 * @retval PACE2_DNS_BAD_PARAM          @copydoc PACE2_DNS_return_state::PACE2_DNS_BAD_PARAM
 * @retval PACE2_DNS_NO_L7_PAYLOAD      @copydoc PACE2_DNS_return_state::PACE2_DNS_NO_L7_PAYLOAD
 * @retval PACE2_DNS_PARSING_FAILED     @copydoc PACE2_DNS_return_state::PACE2_DNS_PARSING_FAILED
 */
PACE2_DNS_return_state pace2_dns_parse_name(PACE2_packet_descriptor const * const pd,
                                            u8 const * const content_ptr,
                                            u32 const max_pointers,
                                            u8 * const dst,
                                            u32 const dst_len,
                                            u32 * const len);

#ifdef __cplusplus
}
#endif
#endif /* PACE2_STATUS_DNS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
