/** @file      pace2_status_rtp.h
  * @brief     PACE 2 module wrapper for RTP performance report functions.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_STATUS_RTP_H
#define PACE2_STATUS_RTP_H

/* Includes *********************************************************************/
#include "pace2_basic_def.h"
#include "event_types/rtp_events.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public Types *****************************************************************/

/**
 * @brief Return state of RTP performance report functions.
 *
 * @deprecated This enum is deprecated and will be removed in the future with the removal of all RTP functions in this
 *             file.
 */
typedef enum {
    PACE2_RTP_SUCCESS,                  /**< Function returned successfully, returned data is valid. */
    PACE2_RTP_PARAM_MISSING,            /**< Function parameter is missing. */
    PACE2_RTP_STATS_UNAVAILABLE,        /**< RTP statistics are disabled. */
    PACE2_RTP_UNKNOWN_ERROR,            /**< An unknown error occurred. */
    PACE2_RTP_FAILURE,                  /**< The given thread ID is invalid or the thread instance is not initialized. */
} PACE2_RTP_return_state;

/* Public Prototypes ************************************************************/

/**
 * @brief Get the flow statistics for the RTP flow of the given packet descriptor.
 *
 * @deprecated This function is deprecated and will be removed in the future. The RTP performance statistics are
 *             available in the RTP dissector metadata event. The RTP performance statistics can also be retrieved with
 *             'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.
 *
 * @param[in]   pace2_module      Pointer to the PACE 2 module.
 * @param[in]   thread_ID         Identifier of a thread.
 * @param[in]   packet_descriptor Pointer to the pace2 packet descriptor.
 * @param[out]  rtp_event         Pointer to a variable, which is used to store the received data.
 *                                This variable is a event structure of type @ref PACE2_RTP_event .
 * @retval PACE2_RTP_SUCCESS            @copydoc PACE2_RTP_return_state::PACE2_RTP_SUCCESS
 * @retval PACE2_RTP_PARAM_MISSING      @copydoc PACE2_RTP_return_state::PACE2_RTP_PARAM_MISSING
 * @retval PACE2_RTP_STATS_UNAVAILABLE  @copydoc PACE2_RTP_return_state::PACE2_RTP_STATS_UNAVAILABLE
 * @retval PACE2_RTP_UNKNOWN_ERROR      @copydoc PACE2_RTP_return_state::PACE2_RTP_UNKNOWN_ERROR
 * @retval PACE2_RTP_FAILURE            @copydoc PACE2_RTP_return_state::PACE2_RTP_FAILURE
 */
PACE2_DEPRECATED(
    "This function is deprecated and will be removed in the future. The RTP performance statistics are available in "
    "the RTP dissector metadata event. The RTP performance statistics can also be retrieved with "
    "'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.")
PACE2_RTP_return_state pace2_rtp_get_flow_stats(PACE2_module * const pace2_module,
                                                int const thread_ID,
                                                PACE2_packet_descriptor * const packet_descriptor,
                                                PACE2_RTP_event * const rtp_event);

/**
 * @brief Reset packet counters for the RTP flow of the given packet descriptor.
 *
 * @deprecated This function is deprecated and will be removed in the future. The RTP performance statistics are
 *             available in the RTP dissector metadata event. The RTP performance statistics can also be retrieved with
 *             'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.
 *
 * @param[in]   pace2_module      Pointer to the PACE 2 module.
 * @param[in]   thread_ID         Identifier of a thread.
 * @param[in]   packet_descriptor Pointer to the pace2 packet descriptor of the flow.
 * @retval PACE2_RTP_SUCCESS            @copydoc PACE2_RTP_return_state::PACE2_RTP_SUCCESS
 * @retval PACE2_RTP_PARAM_MISSING      @copydoc PACE2_RTP_return_state::PACE2_RTP_PARAM_MISSING
 * @retval PACE2_RTP_STATS_UNAVAILABLE  @copydoc PACE2_RTP_return_state::PACE2_RTP_STATS_UNAVAILABLE
 * @retval PACE2_RTP_UNKNOWN_ERROR      @copydoc PACE2_RTP_return_state::PACE2_RTP_UNKNOWN_ERROR
 * @retval PACE2_RTP_FAILURE            @copydoc PACE2_RTP_return_state::PACE2_RTP_FAILURE
 */
PACE2_DEPRECATED(
    "This function is deprecated and will be removed in the future. The RTP performance statistics are available in "
    "the RTP dissector metadata event. The RTP performance statistics can also be retrieved with"
    "'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.")
PACE2_RTP_return_state pace2_rtp_reset_flow_stats(PACE2_module * const pace2_module,
                                                  int const thread_ID,
                                                  PACE2_packet_descriptor * const packet_descriptor);

/**
 * @brief Reset internal sequence number state for the RTP flow of the given packet descriptor.
 *
 * @deprecated This function is deprecated and will be removed in the future. The RTP performance statistics are
 *             available in the RTP dissector metadata event. The RTP performance statistics can also be retrieved with
 *             'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.
 *
 * @param[in]   pace2_module      Pointer to the PACE 2 module.
 * @param[in]   thread_ID         Identifier of a thread.
 * @param[in]   packet_descriptor Pointer to the pace2 packet descriptor of the flow.
 * @retval PACE2_RTP_SUCCESS            @copydoc PACE2_RTP_return_state::PACE2_RTP_SUCCESS
 * @retval PACE2_RTP_PARAM_MISSING      @copydoc PACE2_RTP_return_state::PACE2_RTP_PARAM_MISSING
 * @retval PACE2_RTP_STATS_UNAVAILABLE  @copydoc PACE2_RTP_return_state::PACE2_RTP_STATS_UNAVAILABLE
 * @retval PACE2_RTP_UNKNOWN_ERROR      @copydoc PACE2_RTP_return_state::PACE2_RTP_UNKNOWN_ERROR
 * @retval PACE2_RTP_FAILURE            @copydoc PACE2_RTP_return_state::PACE2_RTP_FAILURE
 */
PACE2_DEPRECATED(
    "This function is deprecated and will be removed in the future. The RTP performance statistics are available in "
    "the RTP dissector metadata event.")
PACE2_RTP_return_state pace2_rtp_reset_flow_sequence_num(PACE2_module * const pace2_module,
                                                         int const thread_ID,
                                                         PACE2_packet_descriptor * const packet_descriptor);

/**
 * @brief Get the flow statistics for the given RTP flow.
 *
 * @deprecated This function is deprecated and will be removed in the future. The RTP performance statistics are
 *             available in the RTP dissector metadata event. The RTP performance statistics can also be retrieved with
 *             'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.
 *
 * @param[in]   pace2_module      Pointer to the PACE 2 module.
 * @param[in]   thread_ID         Identifier of a thread.
 * @param[in]   flow              Pointer to the pace2 flow.
 * @param[out]  rtp_event         Pointer to a variable, which is used to store the received data.
 *                                This variable is a event structure of type @ref PACE2_RTP_event .
 * @retval PACE2_RTP_SUCCESS            @copydoc PACE2_RTP_return_state::PACE2_RTP_SUCCESS
 * @retval PACE2_RTP_PARAM_MISSING      @copydoc PACE2_RTP_return_state::PACE2_RTP_PARAM_MISSING
 * @retval PACE2_RTP_STATS_UNAVAILABLE  @copydoc PACE2_RTP_return_state::PACE2_RTP_STATS_UNAVAILABLE
 * @retval PACE2_RTP_UNKNOWN_ERROR      @copydoc PACE2_RTP_return_state::PACE2_RTP_UNKNOWN_ERROR
 * @retval PACE2_RTP_FAILURE            @copydoc PACE2_RTP_return_state::PACE2_RTP_FAILURE
 */
PACE2_DEPRECATED(
    "This function is deprecated and will be removed in the future. The RTP performance statistics are available in "
    "the RTP dissector metadata event. The RTP performance statistics can also be retrieved with"
    "'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.")
PACE2_RTP_return_state pace2_rtp_get_flow_stats_for_flow(PACE2_module * const pace2_module,
                                                         int const thread_ID,
                                                         PACE2_flow_data * const flow,
                                                         PACE2_RTP_event * const rtp_event);

/**
 * @brief Reset packet counters for the given RTP flow.
 *
 * @deprecated This function is deprecated and will be removed in the future. The RTP performance statistics are
 *             available in the RTP dissector metadata event. The RTP performance statistics can also be retrieved with
 *             'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.
 *
 * @param[in]   pace2_module      Pointer to the PACE 2 module.
 * @param[in]   thread_ID         Identifier of a thread.
 * @param[in]   flow              Pointer to the pace2 flow.
 * @retval PACE2_RTP_SUCCESS            @copydoc PACE2_RTP_return_state::PACE2_RTP_SUCCESS
 * @retval PACE2_RTP_PARAM_MISSING      @copydoc PACE2_RTP_return_state::PACE2_RTP_PARAM_MISSING
 * @retval PACE2_RTP_STATS_UNAVAILABLE  @copydoc PACE2_RTP_return_state::PACE2_RTP_STATS_UNAVAILABLE
 * @retval PACE2_RTP_UNKNOWN_ERROR      @copydoc PACE2_RTP_return_state::PACE2_RTP_UNKNOWN_ERROR
 * @retval PACE2_RTP_FAILURE            @copydoc PACE2_RTP_return_state::PACE2_RTP_FAILURE
 */
PACE2_DEPRECATED(
    "This function is deprecated and will be removed in the future. The RTP performance statistics are available in "
    "the RTP dissector metadata event. The RTP performance statistics can also be retrieved with"
    "'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.")
PACE2_RTP_return_state pace2_rtp_reset_flow_stats_for_flow(PACE2_module * const pace2_module,
                                                           int const thread_ID,
                                                           PACE2_flow_data * const flow);

/**
 * @brief Reset internal sequence number state for the given RTP flow.
 *
 * @deprecated This function is deprecated and will be removed in the future. The RTP performance statistics are
 *             available in the RTP dissector metadata event. The RTP performance statistics can also be retrieved with
 *             'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.
 *
 * @param[in]   pace2_module      Pointer to the PACE 2 module.
 * @param[in]   thread_ID         Identifier of a thread.
 * @param[in]   flow              Pointer to the pace2 flow.
 * @retval PACE2_RTP_SUCCESS            @copydoc PACE2_RTP_return_state::PACE2_RTP_SUCCESS
 * @retval PACE2_RTP_PARAM_MISSING      @copydoc PACE2_RTP_return_state::PACE2_RTP_PARAM_MISSING
 * @retval PACE2_RTP_STATS_UNAVAILABLE  @copydoc PACE2_RTP_return_state::PACE2_RTP_STATS_UNAVAILABLE
 * @retval PACE2_RTP_UNKNOWN_ERROR      @copydoc PACE2_RTP_return_state::PACE2_RTP_UNKNOWN_ERROR
 * @retval PACE2_RTP_FAILURE            @copydoc PACE2_RTP_return_state::PACE2_RTP_FAILURE
 */
PACE2_DEPRECATED(
    "This function is deprecated and will be removed in the future. The RTP performance statistics are available in "
    "the RTP dissector metadata event. The RTP performance statistics can also be retrieved with"
    "'pace2_cpi_get_dissector_metadata' and can be reset with 'pace2_cpi_reset_metadata_dissector'.")
PACE2_RTP_return_state pace2_rtp_reset_flow_sequence_num_for_flow(PACE2_module * const pace2_module,
                                                                  int const thread_ID,
                                                                  PACE2_flow_data * const flow);

#ifdef __cplusplus
}
#endif
#endif /* PACE2_STATUS_RTP_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
