/** @file      pace2_status_sip.h
  * @brief     PACE 2 module wrapper for the SIP protocol status functions.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_STATUS_SIP_H
#define PACE2_STATUS_SIP_H

/* Includes *********************************************************************/
#include "pace2_basic_def.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public Types *****************************************************************/

/**
 * List of strings, each identifying the appropriate RTP payload name.
 * It is comparable to the protocol name list of wireshark.
 */
#define PACE2_RTP_PAYLOAD_DESCRIPTION \
    "ITU-T G.711 PCMU",\
    "USA Federal Standard FS-1016",\
    "ITU-T G.721",\
    "GSM 06.10",\
    "ITU-T G.723",\
    "DVI4 8000 samples/s",\
    "DVI4 16000 samples/s",\
    "Experimental linear predictive encoding from Xerox PARC",\
    "ITU-T G.711 PCMA",\
    "ITU-T G.722",\
    "16-bit uncompressed audio, stereo",\
    "16-bit uncompressed audio, monaural",\
    "Qualcomm Code Excited Linear Predictive coding",\
    "Comfort noise",\
    "MPEG-I/II Audio",\
    "ITU-T G.728",\
    "DVI4 11025 samples/s",\
    "DVI4 22050 samples/s",\
    "ITU-T G.729",\
    "Comfort noise (old)",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Sun CellB video encoding",\
    "JPEG-compressed video",\
    "Unassigned",\
    "'nv' program",\
    "Unassigned",\
    "Unassigned",\
    "ITU-T H.261",\
    "MPEG-I/II Video",\
    "MPEG-II transport streams",\
    "ITU-T H.263",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Reserved for RTCP conflict avoidance",\
    "Reserved for RTCP conflict avoidance",\
    "Reserved for RTCP conflict avoidance",\
    "Reserved for RTCP conflict avoidance",\
    "Reserved for RTCP conflict avoidance",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "Unassigned",\
    "DynamicRTP-Type-96",\
    "DynamicRTP-Type-97",\
    "DynamicRTP-Type-98",\
    "DynamicRTP-Type-99",\
    "DynamicRTP-Type-100",\
    "DynamicRTP-Type-101",\
    "DynamicRTP-Type-102",\
    "DynamicRTP-Type-103",\
    "DynamicRTP-Type-104",\
    "DynamicRTP-Type-105",\
    "DynamicRTP-Type-106",\
    "DynamicRTP-Type-107",\
    "DynamicRTP-Type-108",\
    "DynamicRTP-Type-109",\
    "DynamicRTP-Type-110",\
    "DynamicRTP-Type-111",\
    "DynamicRTP-Type-112",\
    "DynamicRTP-Type-113",\
    "DynamicRTP-Type-114",\
    "DynamicRTP-Type-115",\
    "DynamicRTP-Type-116",\
    "DynamicRTP-Type-117",\
    "DynamicRTP-Type-118",\
    "DynamicRTP-Type-119",\
    "DynamicRTP-Type-120",\
    "DynamicRTP-Type-121",\
    "DynamicRTP-Type-122",\
    "DynamicRTP-Type-123",\
    "DynamicRTP-Type-124",\
    "DynamicRTP-Type-125",\
    "DynamicRTP-Type-126",\
    "DynamicRTP-Type-127"

/* Public Prototypes ************************************************************/
/**
 * @brief This function provides the protocol name for a given media format integer.
 *
 * The function does not support the dynamic upgrade feature.
 *
 * @param[in]   pace2_module                    Constant pointer to the PACE 2 module.
 * @param[in]   thread_ID                       The ID of the thread currently processing.
 * @param[in]   media_format                    The format type as integer for the related protocol name
 * @returns     The media format as string or NULL when one of the input parameters is invalid.
 *              It returns the static string from the related list.
 */
const char * pace2_sip_get_static_rtp_payload_format_name(PACE2_module * const pace2_module,
                                                          int const thread_ID,
                                                          u8 media_format);

#ifdef __cplusplus
}
#endif
#endif /* PACE2_STATUS_SIP_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
