# ${DEVICE} = agl0.4094
# ${ACTION} = ifup/ifdown
# ${INTERFACE} = wan1_eth

. /lib/gre/gre_common.sh

[ -z "${INTERFACE}" -a -z "${DEVICE}" ] && exit 1
[ "${ACTION}" != "ifdown" -a "${ACTION}" != "ifup" ] && exit 1

gre_each_connection()
{
    local zone
    local effect_iface
    local status
    local name
    local remote_ip
    local local_gre_ip
    local remote_gre_ip
    local local_subnet
    local remote_subnet

    config_get zone "$1" bindif
	[ -z "$zone" ] && return	
	effect_iface=$(zone_get_effect_main_wan_ifaces ${zone})
	[ "$INTERFACE" != "$effect_iface" ] && return

    case "$ACTION" in
		ifup)
            config_get name "$1" id
            name="X_gre_$name"

            config_get remote_ip "$1" remote_ip
            config_get local_gre_ip "$1" local_gre_ip
            config_get remote_gre_ip "$1" remote_gre_ip
            config_get local_subnet "$1" local_subnet
            config_get remote_subnet "$1" remote_subnet
            config_get ipsec_encryption "$1" ipsec_encryption

            [ -n "$(ip a | grep ${name})" ] && return

            [ -z "$name" -o -z "$remote_ip" -o -z "$local_gre_ip" -o -z "$remote_gre_ip" -o -z "$local_subnet" -o -z "$remote_subnet" ] && echo "Params Miss." > /dev/console && return    
            
            config_get status "$1" status
            [ "$status" != "on" -a "$status" != "off" ] && return
            [ "$status" = "on" ] && {
                gre_create $name $zone $remote_ip $local_subnet $remote_subnet $local_gre_ip $remote_gre_ip $ipsec_encryption
            }
            set_gre_link $name $status $remote_subnet $local_gre_ip
            ;;
        ifdown)
            config_get name "$1" id
            name="X_gre_$name"

            config_get remote_ip "$1" remote_ip
            [ -z "$(ip a | grep ${name})" ] && return

            [ -z "$name" -o -z "$remote_ip" ] && echo "Params Miss." > /dev/console && return

            gre_delete $name $remote_ip
            ;;
    esac
    
    

}

#echo "${DEVICE}-${INTERFACE} ${ACTION}" > /dev/console

config_load gre
config_foreach gre_each_connection conn
