#!/bin/sh
#author: cuibinglin
#brief: dealing with events from interface
#env: $INTERRFACE $ACTION $DEVICE

. /lib/functions.sh
. /lib/zone/zone_api.sh
. /lib/firewall/fw.sh

[ -z "${INTERFACE}" -a -z "${DEVICE}" ] && exit 1
[ "${ACTION}" != "ifdown" -a "${ACTION}" != "ifup" ] && exit 1


fw_check() {
	local ret=$(iptables -w -t $1 -C $2 $3 2>&1)
	[ -n "$ret" ] && return 0||return 1
}


ipsec_each_connection(){
	local zone
	local status
	local effect_iface
	local indevice
	local iface
	local ipsec_service_chain=ipsec_service_rule

	config_get zone "$1" local_binding
	[ -z "$zone" ] && return
	config_get status "$1" status
	[ "$status" != "enable" ] && return	
	effect_iface=$(zone_get_effect_main_wan_ifaces ${zone})
	[ "$INTERFACE" != "$effect_iface" ] && return
	
	case "$ACTION" in
		ifup)
		    indevice=$(zone_get_device_byif $INTERFACE)
			for iface in $indevice;do
			[ -n "$iface" -a "$iface" != "loopback" ] && {
				local rule="-i ${iface} -p udp -m multiport --dports 500,4500"
				fw_check "filter" "$ipsec_service_chain" "$rule -j ACCEPT"
				[ x$? == x0 ] && {
					fw add 4 f $ipsec_service_chain ACCEPT $ { "$rule" }
				}
			}
			done
		;;
		ifdown)
			[ -n "$DEVICE" -a "$DEVICE" != "loopback" ] && {
				local rule="-i ${DEVICE} -p udp -m multiport --dports 500,4500"
				fw_check "filter" "$ipsec_service_chain" "$rule -j ACCEPT"
				[ x$? == x1 ] && {
					fw del 4 f $ipsec_service_chain ACCEPT $ { "$rule" }
				}
			}

		;;
	esac

}
config_load ipsec
config_foreach ipsec_each_connection connection

