#!/bin/sh
#when nat restart, all nat rules about pppox should be setted again
#author: wangdechu, 20150616

. /lib/functions.sh
. /lib/firewall/fw.sh
. /lib/pppox/pppox-default-variables.sh

response_nat()
{
	get_param_and_del_ct()
	{
		config_get netmode $1 "netmode"
		config_get remotesubnet $1 "remotesubnet"
		config_get local_network $1 "local_network"
		config_get workmode $1 "workmode"

		for rnet in ${remotesubnet}; do
			local r_netmask=$(ipcalc -m ${rnet} | sed 's/NETMASK=//g')
			local r_ipaddr=$(ipcalc -n ${rnet} | sed 's/NETWORK=//g')
			if [ -z "$local_network" ]; then
				#echo "conntrack -D --dst ${r_ipaddr} --mask-dst ${r_netmask}" > /dev/console
				[ -x /usr/sbin/conntrack ] && conntrack -D --dst ${r_ipaddr} --mask-dst ${r_netmask} 
				[ -x /usr/sbin/conntrack ] && conntrack -D --src ${r_ipaddr} --mask-src ${r_netmask}
			else
				for lnet in ${local_network}; do
					local l_netmask=$(uci get vnetwork."${lnet}".netmask)
					local l_ipaddr=$(uci get vnetwork."${lnet}".ipaddr)
					#echo "conntrack -D --src ${l_ipaddr} --mask-src ${l_netmask} --dst ${r_ipaddr} --mask-dst ${r_netmask}" > /dev/console
					#echo "conntrack -D --src ${r_ipaddr} --mask-src ${r_netmask} --dst ${l_ipaddr} --mask-dst ${l_netmask}" > /dev/console
					[ -x /usr/sbin/conntrack ] && conntrack -D --src ${l_ipaddr} --mask-src ${l_netmask} --dst ${r_ipaddr} --mask-dst ${r_netmask} 
					[ -x /usr/sbin/conntrack ] && conntrack -D --src ${r_ipaddr} --mask-src ${r_netmask} --dst ${l_ipaddr} --mask-dst ${l_netmask}
				done
			fi
		done
	}

	do_setnat()
	{
		[ ${workmode} = "nat" ] && {
			iptables -w -t nat -C postrouting_rule_multinat -o ${ifname} -j MASQUERADE 2>/dev/null
			[ "$?" != "0" ] && fw add 4 nat postrouting_rule_multinat MASQUERADE $ { -o ${ifname} }
		}
	}

	#traverse all tunnel to set nat
	for dir in `ls ${pppox_ppppath}/pid 2>/dev/null`; do
		username=`uci -c ${pppox_ppppath}/pid get ${dir}.@info[0].username 2>/dev/null`
		isserver=`uci -c ${pppox_ppppath}/pid get ${dir}.@info[0].isserver 2>/dev/null`
		configpath=`uci -c ${pppox_ppppath}/pid get ${dir}.@info[0].configpath 2>/dev/null`
		ifname=`uci -c ${pppox_ppppath}/pid get ${dir}.@info[0].ifname 2>/dev/null`
		
		[ -z "${configpath}" ] && continue
		#configpath like "/tmp/l2tp/server/server1", note: no "config"
		path=${configpath}/config
		UCI_CONFIG_DIR=${path}
		
		if [ "${isserver}" = "1" ]; then
	#workmode for server must be route
	#		configname=${pppox_if_config}
	#		config_load ${configname}
	#		get_param ${username}
	#		config_foreach config_clear
			workmode=route
			[ -x /usr/sbin/conntrack ] && conntrack -D -m 0x1000
		elif [ "${isserver}" = "0" ]; then
			configname=${pppox_configname}
			
			config_load ${configname}
			config_foreach get_param_and_del_ct
			config_foreach config_clear
			netmode=lan2lan
		else
			return 1
		fi
		
		do_setnat

	done

	#clean HWNAT, SFE and contract for old connect
	sleep 1
	[ -x /usr/sbin/xfwdm ] && xfwdm -F
	[ -e "/etc/init.d/hwnat" ] && hwnat -y
	[ -e /sys/kernel/debug/ecm/ecm_db/defunct_all ] && echo 1 > /sys/kernel/debug/ecm/ecm_db/defunct_all	
}

case "$MODULE" in
	NAPT)
		echo "Got an NAPT hotplug clean=\"${CLEAN}\" "
	if [ "${CLEAN}" == "on" ]
	then	
		response_nat
	fi
	;;
esac
