#!/bin/sh
# This script is executed as part of the hotplug event with
# HOTPLUG_TYPE=iface, triggered by various scripts when an interface
# is configured (ACTION=ifup) or deconfigured (ACTION=ifdown).  The
# interface is available as INTERFACE, the real device as DEVICE.

. /lib/functions.sh
. /lib/zone/zone_api.sh

[ -n "$VNET_BOOTING" ] && exit 0

[ -z "$INTERFACE" -o "$INTERFACE" = "vwan" ] && exit 0

local ZONE="$(zone_get_zone_byif $INTERFACE)"

delinfs=

local cid=$(uci get vnetwork."${INTERFACE}".cid)
local system=$(uci get vnetwork."${INTERFACE}".system)
local netmask=$(uci get vnetwork."${INTERFACE}".netmask)
local ipaddr=$(uci get vnetwork."${INTERFACE}".ipaddr)
local network=$(ipcalc -n ${ipaddr} ${netmask} | cut -d '=' -f2)
local prefix=$(ipcalc -p ${ipaddr} ${netmask} | cut -d '=' -f2)
local group="@""${ZONE}"

case ${ACTION} in
	add)
		lua /usr/lib/lua/ipgroup/lanhook.lua "${network}/${prefix}" add
		ipset add "IPGROUP_LAN" "${network}/${prefix}" -exist

		ipset create "${group}" hash:net -exist
		ipset flush "${group}"
		ipset add "${group}" "${network}/${prefix}" -exist
	;;
	delete)
		ipset del "IPGROUP_LAN" "${network}/${prefix}" -exist
		lua /usr/lib/lua/ipgroup/lanhook.lua "${network}/${prefix}" del

		ipset destroy "${group}" -exist
	;;
	*)
	;;
esac
