#!/bin/sh
# This script is executed as part of the hotplug event with
# HOTPLUG_TYPE=iface, triggered by various scripts when an interface
# is configured (ACTION=ifup) or deconfigured (ACTION=ifdown).  The
# interface is available as INTERFACE, the real device as DEVICE.

. /lib/functions.sh
. /lib/zone/zone_api.sh

[ -n "$VNET_BOOTING" ] && exit 0

[ -z "$INTERFACE" -o "$INTERFACE" = "vwan" ] && exit 0

local ZONE="$(zone_get_zone_byif $INTERFACE)"

delinfs=

[ "x${ACTION}" = "xipv6_add" ] && ACTION="add"
[ "x${ACTION}" = "xipv6_delete" ] && ACTION="delete"

local prefix=$(uci get vnetwork.${INTERFACE}.ipv6_prefixlen)
local network=$(cat /tmp/.vnet_${INTERFACE}_ipv6_addr)
[ -z "$prefix"] && [ -z "$network" ] && exit 0
[ -z "$prefix" ] && prefix=$(cat /tmp/.vnet_${INTERFACE}_ipv6_addr_len)
case ${ACTION} in
	add)
		[ -z "$prefix" ] || [ -z "$network" ] && exit 0
		lua /usr/lib/lua/ipv6group/lanhook.lua "${network}/${prefix}" add
		ipset add "ipv6_IPV6GROUP_LAN" "${network}/${prefix}" -exist
	;;
	delete)
		ipset del "ipv6_IPV6GROUP_LAN" "${network}/${prefix}" -exist
		lua /usr/lib/lua/ipv6group/lanhook.lua "${network}/${prefix}" del
	;;
	commit)
		ipset flush "ipv6_IPV6GROUP_LAN" -exist
                [ ${#network} -eq 0 ] && network="fe80::0"
				[ ${#prefix} -eq 0 ] && prefix="64"
		ipset add "ipv6_IPV6GROUP_LAN" "${network}/${prefix}" -exist
		lua /usr/lib/lua/ipv6group/lanhook.lua "${network}/${prefix}" replace
	;;
	*)
	;;
esac
