#!/bin/sh
. /lib/functions.sh
. /lib/zone/zone_api.sh

[ -n "$VNET_BOOTING" ] && exit 0

[ -z "$INTERFACE" -o "$INTERFACE" = "vwan" ] && exit 0

VPN_FWMARK=0x20000
VPN_FORWARD_TABLE=VPN_FORWARD_MARK

local netmask=$(uci get vnetwork."${INTERFACE}".netmask)
local ipaddr=$(uci get vnetwork."${INTERFACE}".ipaddr)
local network=$(ipcalc -n ${ipaddr} ${netmask} | cut -d '=' -f2)
local prefix=$(ipcalc -p ${ipaddr} ${netmask} | cut -d '=' -f2)
local cidr="${network}/${prefix}"

# $1:section $2:config $3:zone
update_local_network()
{
    local needupdate=0
    local local_network_zones
    config_get local_network_zones $1 local_network_zone

    for local_network_zone in ${local_network_zones}
    do
        if [ "$local_network_zone" == "$3" ] ; then
            needupdate=1
            break
        fi
    done
    [ $needupdate == 0 ] && return 0

    local local_networks
    for local_network_zone in ${local_network_zones}
    do
        local_network=$(zone_get_ifaces "$local_network_zone")
        if [ -n "$local_network" ]; then
            if [ -n "$local_networks" ]; then
                local_networks=${local_networks}" "${local_network}
            else
                local_networks=${local_network}
            fi
        fi
    done
    uci_set "$2" "$1" local_network "${local_networks}"
}

case ${ACTION} in
        add)
			iptables -w -t mangle -nL ${VPN_FORWARD_TABLE} --line-numbers | grep "\b${INTERFACE}\b" | cut -d ' ' -f 1 > /dev/console
			nums=$(iptables -w -t mangle -nL ${VPN_FORWARD_TABLE} --line-numbers | grep "\b${INTERFACE}\b" | cut -d ' ' -f 1)
			for num in $nums;
			do

				echo "num=${num}" > /dev/console
				
				local src=$(iptables -w -t mangle -nvL ${VPN_FORWARD_TABLE} ${num} | awk '{print $8}')
				local des=$(iptables -w -t mangle -nvL ${VPN_FORWARD_TABLE} ${num} | awk '{print $9}')
				local str=$(iptables -w -t mangle -nvL ${VPN_FORWARD_TABLE} ${num} | awk '{print $11}')
				
				iptables -w -t mangle -D ${VPN_FORWARD_TABLE} ${num}
				iptables -w -t mangle -I ${VPN_FORWARD_TABLE} ${num} -i br-lan -s ${cidr} -d ${des} -m comment --comment "${str}" -j MARK --or-mark ${VPN_FWMARK}
			done
			
			nums=$(iptables -w -nL ppp_forward_rule --line-numbers | grep "${INTERFACE}" | cut -d ' ' -f 1)	
			for num in $nums;
			do
				echo "num=${num}" > /dev/console
				local indev=$(iptables -w  -nvL ppp_forward_rule ${num} | awk '{print $6}')
				local outdev=$(iptables -w  -nvL ppp_forward_rule ${num} | awk '{print $7}')
				local des=$(iptables -w  -nvL ppp_forward_rule ${num} | awk '{print $9}')
				local str=$(iptables -w  -nvL ppp_forward_rule ${num} | awk '{print $11}')
						
				if [ "${str:0:12}" == "vnet=NETWORK" ]; then
					iptables -w -D ppp_forward_rule ${num}
					iptables -w -I ppp_forward_rule ${num} -i ${indev} -o ${outdev} -s ${cidr} -d ${des} -m comment --comment "${str}" -j ACCEPT
				elif [ "${str:0:28}" == "receive_local_subnet_NETWORK" ]; then
					iptables -w -D ppp_forward_rule ${num}
					iptables -w -I ppp_forward_rule ${num} -i ${indev} -o ${outdev} -s 0.0.0.0/0 -d ${cidr} -m comment --comment "${str}" -j ACCEPT
				fi
			done
			
        ;;
        delete)
			nums=$(iptables -w -t mangle -nL ${VPN_FORWARD_TABLE} --line-numbers | grep "\b${INTERFACE}\b" | cut -d ' ' -f 1)
			for num in $nums;
			do
				local src=$(iptables -w -t mangle -nvL ${VPN_FORWARD_TABLE} ${num} | awk '{print $8}')
				local des=$(iptables -w -t mangle -nvL ${VPN_FORWARD_TABLE} ${num} | awk '{print $9}')
				local str=$(iptables -w -t mangle -nvL ${VPN_FORWARD_TABLE} ${num} | awk '{print $11}')
				
				iptables -w -t mangle -D ${VPN_FORWARD_TABLE} ${num}	
				iptables -w -t mangle -I ${VPN_FORWARD_TABLE} ${num} -i br-lan -s ${cidr} -d ${des} -m comment --comment "${str}" -j RETURN

			done    
        ;;
        commit)
            local zone=${ZACTION%%=*}
            [ -n "$zone" ] || return 0

            config_load l2tp-server
            config_foreach update_local_network lns l2tp-server $zone
            uci commit l2tp-server

            config_load pptp-server
            config_foreach update_local_network pns pptp-server $zone
            uci commit pptp-server
        ;;
        *)
        ;;
esac

