#!/bin/sh

. /lib/functions/network.sh
. /lib/zone/zone_api.sh
. /lib/nat/nat_common.sh

local HOTP_LOG="/tmp/.nat/nat_wanhook.log"

mkdir -p /tmp/.nat
if [[ "`ls -l $HOTP_LOG | awk '{print $5}'`" -gt 50000 ]]; then
	echo -n "" > $HOTP_LOG
fi
HP_LOG()
{
	echo -e "`date +%Y%m%d-%H:%M:%S` $@" >> $HOTP_LOG
}

local needSave=0

_delete_rule()
{
	local cfg="$1"
	local iface="$2"
	local interface
	local name

	interface=$(uci_get nat "$cfg" interface)
	name=$(uci_get nat "$cfg" name)
	HP_LOG "cfg: $cfg, name: $name, iface: $iface, interface: $interface"
	if [[ -n "$interface" && "$interface" == "$iface" ]]; then
		uci delete nat.$cfg
		needSave=1
	fi
}

_add_rule()
{
	local ipaddr="$1"
	local mask="$2"
	local iface="$3"
	local naptRuleName="NAT_LAN_$iface"
	local enable="on"
	local sysrule="1"
	#local mask=${${1}#*/}
	#local ipaddr=${${1}%/*}

	HP_LOG "naptRuleName: $naptRuleName, ipaddr: $ipaddr, mask: $mask"
	uci add nat rule_napt
	uci set nat.@rule_napt[-1].name=${naptRuleName}
	uci set nat.@rule_napt[-1].enable=${enable}
	uci set nat.@rule_napt[-1].mask=${mask}
	uci set nat.@rule_napt[-1].ipaddr=${ipaddr}
	uci set nat.@rule_napt[-1].interface=${iface}
	uci set nat.@rule_napt[-1].sysrule=${sysrule}
	needSave=1
}

case ${ACTION} in
	DELETE)
		if [[ -n "${interfaces}" ]]; then
			config_load nat
			HP_LOG "interfaces: $interfaces, action: $ACTION"
			interfaces=${interfaces//,/ }
			for element in $interfaces; do
				HP_LOG "element: $element"
				config_foreach _delete_rule rule_napt $element
				config_foreach _delete_rule rule_onenat $element
				config_foreach _delete_rule rule_vs $element
				config_foreach _delete_rule rule_dmz $element
				config_foreach _delete_rule rule_pt $element
			done
			if [[ "$needSave" -eq 1 ]]; then
				uci commit nat
				/lib/cfg_save/config.sh save
			fi
			#/etc/init.d/nat restart
		fi
	;;
	ADD)
		if [[ -n "${interfaces}" ]]; then
			#echo "sleep 20s................" >> /tmp/nat_wanhook.log
			#sleep 20;
			#echo "sleep 20s finished" >> /tmp/nat_wanhook.log
			HP_LOG "interfaces: $interfaces, action: $ACTION"

			local ipaddr=$(nat_napt_lan_ipaddr)	
			local netmask=$(nat_napt_lan_netmask)
			local prefix=
			local subnet=

			if [[ -z "$ipaddr" || -z "$netmask" ]]; then
				HP_LOG "lan subnet is null error"
				return
			else
				HP_LOG "ipaddr: $ipaddr, netmask: $netmask"
				prefix=$(ipcalc -p $ipaddr $netmask)
				prefix=${prefix#*=}
				subnet=$(ipcalc -n $ipaddr $netmask)
				subnet=${subnet#*=}
				HP_LOG "prefix: $prefix, subnet: $subnet"
			fi

		    interfaces=${interfaces//,/ }
			for element in $interfaces; do
				HP_LOG "element: $element"
				_add_rule $subnet $prefix $element
			done
			if [[ "$needSave" -eq 1 ]]; then
				uci commit nat
				/lib/cfg_save/config.sh save
			fi
			#/etc/init.d/nat restart
		fi
	;;
	*)
	;;
esac
