local sys  = require "luci.sys"
local json = require 'luci.json'
local utl = require "luci.util"
local dbg  = require 'luci.tools.debug'
local form = require "luci.tools.form"
local uci  = require "luci.model.uci"
local error_print = require "luci.tools.error_print"
local ifs = require "luci.model.interfaces".ifs()
--local snmp = require 'luci.controller.admin.snmp'
local accesslist = require 'luci.controller.admin.access_ctl'
local ipgroup = require "luci.controller.admin.ipgroup"
local time = require "luci.controller.admin.time_mngt"
local lan = require "luci.controller.admin.dhcps"
local uci_r = uci.cursor()
--缓存UCI目录，才cli进程中创建
--local CLI_CFG_PATH = "/tmp/access_config"
local CFG_PATH = "/etc/config"

--[[local filename = CLI_CFG_PATH.."/cli_access"
local file = io.open(filename,"r")
if file == nil then
    --创建/tmp/cli_config/cli_snmpd缓存UCI
    sys.exec("touch " .. filename)
else
    file:close()
end--]]
function get_ipgroup()
	local list = ipgroup.get_ipgroup_list_common()
	--dbg.dumptable(list)
	local list_ipGroup = {}
	for k,v in ipairs(list) do
		table.insert(list_ipGroup,v.name)
	end
	return list_ipGroup
end

function get_lan_list()
	local lan_list = {}
	uci_r:foreach("vnetwork","vinterface",
		function(section)
			if uci_r:get("vnetwork", section[".name"], "name") then
				table.insert(lan_list,uci_r:get("vnetwork", section[".name"], "name"))
			end
		end
	)
	return lan_list
end

function get_if_all()
    local interface = {}
    local res = ifs.get_ifs("pure")
    for k,v in ipairs(res.normal) do
        table.insert(interface,v)
    end

    if res.vpn then
        for k,v in ipairs(res.vpn) do
            table.insert(interface,v)
        end
    end
    return interface
end

function get_time_range()
	local time_list = {}
	uci_r:foreach("time_mngt","entry",
		function(section)
			if uci_r:get("time_mngt", section[".name"], "entry_name") then
				table.insert(time_list,uci_r:get("time_mngt", section[".name"], "entry_name"))
			end
		end
	)
	return time_list
end

function check_rule_id_set(ruleid)
	local num = getcount();
	if tonumber(ruleid) > num or tonumber(ruleid) < 0 then
		return "FAILED"
	end
	return "SUCCESS"
end

function check_rule_id_add(ruleid)
	local num = getcount();
	if tonumber(ruleid) > num+1 or tonumber(ruleid) < 0 then
		return "FAILED"
	end
	return "SUCCESS"
end

function mysplit(inputstr, sep)
	if sep == nil then
		sep = "%s"
	end
	local t={} ; i=1
	for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
		table.insert(t,str)
		--t[i] = str
		--i = i + 1
	end
	return t
end

function getindex(name)
	--uci_r:set_confdir(CFG_PATH)
	local count = 0
	local res = -1
	uci_r:foreach("access_ctl","rule_acl_inner",
		function(section)
			if name == uci_r:get("access_ctl", section[".name"], "name") then
				res = count
			end
			count = count+1
		end
	)
	return res
end

function add_deny_acl_rule(access_list_num, rule_id, service_type, direction, sip, dip, effective_time)
	local result = "SUCCESS"
	local http_form = {}
	local input = {}
	local tmp_config = {}
	local num = getcount()
	--uci_r:set_confdir(CFG_PATH)
	--direction = mysplit(direction, ",")
	tmp_config.flag = uci_r:get("access_ctl", access_list_num, "flag") or ""
	if tmp_config.flag == "" then
		if "FAILED" == check_rule_id_add(rule_id) then
			dbg.print_tty("Error: Invalid parameter rule.Please input 0 - "..num+1)
			return "FAILED"
		end
		if rule_id == "1" and getcount() == 0 then
			rule_id = ""
		end
		if rule_id == "0" then
			rule_id = ""
		else
			rule_id = tostring(rule_id)
		end
		if string.sub(sip, 1, 1) == "!" then 
			sip = string.sub(sip, 2).."_REV"
		end
		if string.sub(dip, 1, 1) == "!" then 
			dip = string.sub(dip, 2).."_REV"
		end
		if direction[1] == "VNET" then
			http_form.data = json.encode(
			{
				["method"] = "add",
				["params"] = 
				{
					["index"] = getcount(),
					["old"] = "add",
					["new"] = 
					{
						["name"] = access_list_num,
						["policy"] = "DROP",
						["service"] = service_type,
						["iptype"] = "ipv4",
						["zone"] = direction,
						["srcIf"] = sip,
						["destIf"] = dip,
						["states"] = {"new","established","invalid","related"},
						["time"] = effective_time,
						["position"] = rule_id,
						["flag"] = "1",
						["user"] = "1",	
					},
					["key"] = "add",    
				}
			})
		else
			http_form.data = json.encode(
			{
				["method"] = "add",
				["params"] = 
				{
					["index"] = getcount(),
					["old"] = "add",
					["new"] = 
					{
						["name"] = access_list_num,
						["policy"] = "DROP",
						["service"] = service_type,
						["iptype"] = "ipv4",
						["zone"] = direction,
						["src"] = sip,
						["dest"] = dip,
						["states"] = {"new","established","invalid","related"},
						["time"] = effective_time,
						["position"] = rule_id,
						["flag"] = "1",
						["user"] = "1",	
					},
					["key"] = "add",    
				}
			})
		end
		http_form.form = "acl_inner"
		--dbg.dumptable(http_form)
		ret = accesslist.dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
			dbg.print_tty(error_print.check_error(ret))
			return "FAILED"
	    end
	
		return result
	else
		if "FAILED" == check_rule_id_set(rule_id) then
			dbg.print_tty("Error: Invalid parameter rule.Please input 0 - "..num)
			return "FAILED"
		end
		if rule_id == "0" then
			rule_id = ""
		else
			rule_id = tostring(rule_id)
		end
		if string.sub(sip, 1, 1) == "!" then 
			sip = string.sub(sip, 2).."_REV"
		end
		if string.sub(dip, 1, 1) == "!" then 
			dip = string.sub(dip, 2).."_REV"
		end
		if direction[1] == "VNET" then
			http_form.data = json.encode(
			{
				["method"] = "set",
				["params"] = 
				{
					["index"] = getindex(access_list_num),
					["old"] = 
					{
						["name"] = access_list_num,
						["policy"] = uci_r:get("access_ctl", access_list_num, "policy"),
						["service"] = uci_r:get("access_ctl", access_list_num, "service"),
						["iptype"] = "ipv4",
						["zone"] = uci_r:get("access_ctl", access_list_num, "zone"),
						["src"] = uci_r:get("access_ctl", access_list_num, "src") or "",
						["dest"] = uci_r:get("access_ctl", access_list_num, "dest") or "",
						["srcIf"] = uci_r:get("access_ctl", access_list_num, "srcIf") or "",
						["destIf"] = uci_r:get("access_ctl", access_list_num, "destIf") or "",
						["t_label"] = uci_r:get("access_ctl", access_list_num, "zone"),
						["time"] = uci_r:get("access_ctl", access_list_num, "time"),
						["flag"] = "1",
						["user"] = "1",
						["states"] = {"new","established","invalid","related"},
					},
					["new"] = 
					{
						["name"] = access_list_num,
						["policy"] = "DROP",
						["service"] = service_type,
						["iptype"] = "ipv4",
						["zone"] = direction[1],
						["srcIf"] = sip,
						["destIf"] = dip,
						["states"] = {"new","established","invalid","related"},
						["time"] = effective_time,
						["positon"] = rule_id,
						["flag"] = "1",
						["user"] = "1",
					},
					["key"] = "add",    
				}
			})
		else
			http_form.data = json.encode(
			{
				["method"] = "set",
				["params"] = 
				{
					["index"] = getindex(access_list_num),
					["old"] = 
					{
						["name"] = access_list_num,
						["policy"] = uci_r:get("access_ctl", access_list_num, "policy"),
						["service"] = uci_r:get("access_ctl", access_list_num, "service"),
						["iptype"] = "ipv4",
						["zone"] = uci_r:get("access_ctl", access_list_num, "zone"),
						["src"] = uci_r:get("access_ctl", access_list_num, "src") or "",
						["dest"] = uci_r:get("access_ctl", access_list_num, "dest") or "",
						["srcIf"] = uci_r:get("access_ctl", access_list_num, "srcIf") or "",
						["destIf"] = uci_r:get("access_ctl", access_list_num, "destIf") or "",
						["t_label"] = uci_r:get("access_ctl", access_list_num, "zone"),
						["time"] = uci_r:get("access_ctl", access_list_num, "time"),
						["flag"] = "1",
						["user"] = "1",
						["states"] = {"new","established","invalid","related"},
					},
					["new"] = 
					{
						["name"] = access_list_num,
						["policy"] = "DROP",
						["service"] = service_type,
						["iptype"] = "ipv4",
						["zone"] = direction,
						["src"] = sip,
						["dest"] = dip,
						["states"] = {"new","established","invalid","related"},
						["time"] = effective_time,
						["position"] = rule_id,
						["flag"] = "1",
						["user"] = "1",
					},
					["key"] = "add",    
				}
			})
		end
		http_form.form = "acl_inner"
		--dbg.dumptable(http_form)
		ret = accesslist.dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
			dbg.print_tty(error_print.check_error(ret))
			return "FAILED"
	    end
	
		return result
	end
	
end



function delete_acl_rule(access_list_num)
	local result = "SUCCESS"
	local http_form = {}
	local input = {}
	local tmp_config = {}
	local index = getindex(access_list_num)
	--uci_r:set_confdir(CFG_PATH)
	http_form.data = json.encode(
	{
		["method"] = "delete",
		["params"] = 
		{
			["key"] = "key-1",
			["index"] = tostring(index),
		}
	})
	http_form.form = "acl_inner"
	--dbg.dumptable(http_form)
	ret = accesslist.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
	return result
end



function add_permit_acl_rule(access_list_num, rule_id, service_type, direction, sip, dip, effective_time)
	local result = "SUCCESS"
	local http_form = {}
	local input = {}
	local tmp_config = {}
	local num = getcount()
	--uci_r:set_confdir(CFG_PATH)
	--direction = mysplit(direction, ",")
	tmp_config.flag = uci_r:get("access_ctl", access_list_num, "flag") or ""
	if tmp_config.flag == "" then
		if "FAILED" == check_rule_id_add(rule_id) then
			dbg.print_tty("Error: Invalid parameter rule.Please input 0 - "..num+1)
			return "FAILED"
		end
		if rule_id == "1" and getcount() == 0 then
			rule_id = ""
		end
		if rule_id == "0" then
			rule_id = ""
		else
			rule_id = tostring(rule_id)
		end
		if string.sub(sip, 1, 1) == "!" then 
			sip = string.sub(sip, 2).."_REV"
		end
		if string.sub(dip, 1, 1) == "!" then 
			dip = string.sub(dip, 2).."_REV"
		end
		if direction[1] == "VNET" then
			http_form.data = json.encode(
			{
				["method"] = "add",
				["params"] = 
				{
					["index"] = getcount(),
					["old"] = "add",
					["new"] = 
					{
						["name"] = access_list_num,
						["policy"] = "ACCEPT",
						["service"] = service_type,
						["iptype"] = "ipv4",
						["zone"] = direction,
						["srcIf"] = sip,
						["destIf"] = dip,
						["states"] = {"new","established","related"},
						["time"] = effective_time,
						["position"] = rule_id,
						["flag"] = "1",
						["user"] = "1",	
					},
					["key"] = "add",    
				}
			})
		else
			http_form.data = json.encode(
			{
				["method"] = "add",
				["params"] = 
				{
					["index"] = getcount(),
					["old"] = "add",
					["new"] = 
					{
						["name"] = access_list_num,
						["policy"] = "ACCEPT",
						["service"] = service_type,
						["iptype"] = "ipv4",
						["zone"] = direction,
						["src"] = sip,
						["dest"] = dip,
						["states"] = {"new","established","related"},
						["time"] = effective_time,
						["position"] = rule_id,
						["flag"] = "1",
						["user"] = "1",	
					},
					["key"] = "add",    
				}
			})
		end
		http_form.form = "acl_inner"
		dbg.dumptable(http_form)
		ret = accesslist.dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
			dbg.print_tty(error_print.check_error(ret))
			return "FAILED"
	    end
	
		return result
	else
		if "FAILED" == check_rule_id_set(rule_id) then
			dbg.print_tty("Error: Invalid parameter rule.Please input 0 - "..num)
			return "FAILED"
		end
		if rule_id == "0" then
			rule_id = ""
		else
			rule_id = tostring(rule_id)
		end
		if string.sub(sip, 1, 1) == "!" then 
			sip = string.sub(sip, 2).."_REV"
		end
		if string.sub(dip, 1, 1) == "!" then 
			dip = string.sub(dip, 2).."_REV"
		end
		if direction[1] == "VNET" then
			--print(getindex(access_list_num)-1)
			http_form.data = json.encode(
			{
				["method"] = "set",
				["params"] = 
				{
					["index"] = getindex(access_list_num),
					["old"] = 
					{
						["name"] = access_list_num,
						["policy"] = uci_r:get("access_ctl", access_list_num, "policy"),
						["service"] = uci_r:get("access_ctl", access_list_num, "service"),
						["iptype"] = "ipv4",
						["zone"] = uci_r:get("access_ctl", access_list_num, "zone"),
						["src"] = uci_r:get("access_ctl", access_list_num, "src") or "",
						["dest"] = uci_r:get("access_ctl", access_list_num, "dest") or "",
						["srcIf"] = uci_r:get("access_ctl", access_list_num, "srcIf") or "",
						["destIf"] = uci_r:get("access_ctl", access_list_num, "destIf") or "",
						["t_label"] = uci_r:get("access_ctl", access_list_num, "zone"),
						["time"] = uci_r:get("access_ctl", access_list_num, "time"),
						["flag"] = "1",
						["user"] = "1",
						["states"] = {"new","established","related"},
					},
					["new"] = 
					{
						["name"] = access_list_num,
						["policy"] = "ACCEPT",
						["service"] = service_type,
						["iptype"] = "ipv4",
						["zone"] = direction[1],
						["srcIf"] = sip,
						["destIf"] = dip,
						["states"] = {"new","established","related"},
						["time"] = effective_time,
						["position"] = rule_id,
						["flag"] = "1",
						["user"] = "1",
					},
					["key"] = "add",    
				}
			})
		else
			http_form.data = json.encode(
			{
				["method"] = "set",
				["params"] = 
				{
					["index"] = getindex(access_list_num),
					["old"] = 
					{
						["name"] = access_list_num,
						["policy"] = uci_r:get("access_ctl", access_list_num, "policy"),
						["service"] = uci_r:get("access_ctl", access_list_num, "service"),
						["iptype"] = "ipv4",
						["zone"] = uci_r:get("access_ctl", access_list_num, "zone"),
						["src"] = uci_r:get("access_ctl", access_list_num, "src") or "",
						["dest"] = uci_r:get("access_ctl", access_list_num, "dest") or "",
						["srcIf"] = uci_r:get("access_ctl", access_list_num, "srcIf") or "",
						["destIf"] = uci_r:get("access_ctl", access_list_num, "destIf") or "",
						["t_label"] = uci_r:get("access_ctl", access_list_num, "zone"),
						["time"] = uci_r:get("access_ctl", access_list_num, "time"),
						["flag"] = "1",
						["user"] = "1",
						["states"] = {"new","established","related"},
					},
					["new"] = 
					{
						["name"] = access_list_num,
						["policy"] = "ACCEPT",
						["service"] = service_type,
						["iptype"] = "ipv4",
						["zone"] = direction,
						["src"] = sip,
						["dest"] = dip,
						["states"] = {"new","established","related"},
						["time"] = effective_time,
						["position"] = rule_id,
						["flag"] = "1",
						["user"] = "1",
					},
					["key"] = "add",    
				}
			})
		end
		http_form.form = "acl_inner"
		--dbg.dumptable(http_form)
		ret = accesslist.dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
			dbg.print_tty(error_print.check_error(ret))
			return "FAILED"
	    end
	
		return result
	end
	
end

function show_access_ctl()
	local state = "SUCCESS"
	local count = 0
	local policy = ""
	local srcIf = ""
	local destIf = ""
	local name = ""
	uci_r:foreach("access_ctl","rule_acl_inner",
	function(section)
		if uci_r:get("access_ctl",section[".name"],"service")==nil  then
			state = "FAILED"
		else
			count = count + 1
			if uci_r:get("access_ctl",section[".name"],"policy") == "DROP" then
				policy = "Deny"
			else
				policy = "Permit"
			end
			name = section[".name"]
			dbg.print_tty("---------------------------------------------------------------------")
			dbg.print_tty("name:           "..section[".name"])
			dbg.print_tty("service-type:   "..uci_r:get("access_ctl",section[".name"],"service"))   
			dbg.print_tty("time-range:     "..uci_r:get("access_ctl",section[".name"],"time"))
			dbg.print_tty("policy:         "..policy)
			zone_name = uci_r:get_list("access_ctl",section[".name"],"zone")
			if zone_name[1] == "VNET" then 
				uci_r:foreach("vnetwork", "vinterface",
				function(section)
					if uci_r:get("vnetwork", section[".name"], "zone") then
						if (uci_r:get("access_ctl",name,"srcIf") == uci_r:get("vnetwork", section[".name"], "zone")) then
							srcIf = uci_r:get("vnetwork", section[".name"], "name")
						elseif (uci_r:get("access_ctl",name,"srcIf") == (uci_r:get("vnetwork", section[".name"], "zone").."_REV")) then
							srcIf = "!"..uci_r:get("vnetwork", section[".name"], "name")
						end
						if (uci_r:get("access_ctl",name,"destIf") == uci_r:get("vnetwork", section[".name"], "zone")) then
							destIf = uci_r:get("vnetwork", section[".name"], "name")
						elseif (uci_r:get("access_ctl",name,"destIf") == (uci_r:get("vnetwork", section[".name"], "zone").."_REV")) then
							destIf = "!"..uci_r:get("vnetwork", section[".name"], "name")
						end
					end
				end
				)
				dbg.print_tty("sip-address:    "..srcIf or "")
				dbg.print_tty("dip-address:    "..destIf or "")
			else
				if (string.sub((uci_r:get("access_ctl",section[".name"],"src")),-3) == "REV") then
					dbg.print_tty("sip-address:    ".."!"..string.sub((uci_r:get("access_ctl",section[".name"],"src")),1,-5) or "")
				else
					dbg.print_tty("sip-address:    "..(uci_r:get("access_ctl",section[".name"],"src") or ""))
				end	
				if (string.sub((uci_r:get("access_ctl",section[".name"],"dest")),-3) == "REV") then
					dbg.print_tty("dip-address:    ".."!"..string.sub((uci_r:get("access_ctl",section[".name"],"dest")),1,-5) or "")
				else
					dbg.print_tty("dip-address:    "..(uci_r:get("access_ctl",section[".name"],"dest") or ""))
				end
			end
			local zone_list = get_if_all()
			if zone_name[1] == "VNET" then
				zone_name[1] = "LAN-LAN"
			end
			for k,v in ipairs(zone_name) do
				for j,m in ipairs(zone_list) do
					if v == zone_list[j].t_name then
						zone_name[k] = string.gsub(zone_list[j].t_label, " ", "_")
					end
				end
			end
			dbg.print_tty("direction:      "..table.concat(zone_name,","))
		end
	end
	)
	if count ~= 0 then
		dbg.print_tty("---------------------------------------------------------------------")
	end
	return state
end

function getcount()
	local count=0
	--uci_r:set_confdir(CFG_PATH)
	uci_r:foreach("access_ctl","rule_acl_inner",
        function(section)
            count = count + 1
		end
	)
	return count
end

function checkservice(service_type)
	--uci_r:set_confdir(CFG_PATH)
	local flag = uci_r:get("service", service_type, "flag") or ""
	if flag == "" then
		return "FAILED"
	else
		return "SUCCESS"	
	end
end

function checkdirection(direction)
	local interface = get_if_all()
	dbg.dumptable(direction)
	if (#direction == 0) then
		return "FAILED"
	end
	if (#direction == 1) then
		if direction[1] == "LAN-LAN" or direction[1] == "ALL" then
			if direction[1] == "LAN-LAN" then
				direction[1] = "VNET"
			end
			return "SUCCESS"
		end
		for k,v in ipairs(interface) do
			if direction[1] == v.t_label then
				direction[1] = v.t_name
				return "SUCCESS"
			end
		end
		return "FAILED"
	else 
		for k,v in ipairs(direction) do
			local tmp_flag = 0
			for j,m in ipairs(interface) do
				if direction[k] == m.t_label then
					direction[k] = m.t_name
					tmp_flag = 1
				end
			end
			if tmp_flag == 0 then
				return "FAILED"
			end
		end
		return "SUCCESS"
	end
end

--[[function convertdirection(direction)
	if direction == "LANTOLAN" then
		direction = "VNET"
	elseif direction == "LANTOWAN" then
		direction = "LAN"
	elseif direction == "WANIN" then
		direction = "WAN1"
	elseif direction == "SFPWANIN" then
		direction = "WAN2"
	elseif direction == "LANTOWAN,WANIN,SFPWANIN" then
		direction = "LAN,WAN1,WAN2"
	elseif direction == "LANTOWAN,WANIN" then
		direction = "LAN,WAN1"
	elseif direction == "LANTOWAN,SFPWANIN" then
		direction = "LAN,WAN2"
	elseif direction == "WANIN,SFPWANIN" then
		direction = "WAN1,WAN2"
	end
	return direction
end--]]

function checkaddr(direction, addr)
	local res = "FAILED"
	--uci_r:set_confdir(CFG_PATH)
	if direction[1] == "VNET" then
		--[[if addr == "LAN" or addr == "!LAN" then
			return "SUCCESS"
		end]]--
		if string.sub(addr, 1, 1) == "!" then 
			addr = string.sub(addr, 2)
		end
		--print(addr)
		uci_r:foreach("vnetwork","vinterface",
			function(section)
				--print(addr == uci_r:get("ipgroup", section[".name"], "name"))
				if addr == uci_r:get("vnetwork", section[".name"], "name") then
					res = "SUCCESS"
				end
			end
		)
		--print("-----")
		return res	
	else
		if addr == "IPGROUP_ANY" or addr == "!IPGROUP_ANY" or  addr == "IPGROUP_LAN" or addr == "!IPGROUP_LAN" or addr == "Me" or addr == "!Me" then
			return "SUCCESS"
		end
		if string.sub(addr, 1, 1) == "!" then 
			addr = string.sub(addr, 2)
		end
		--print(addr)
		uci_r:foreach("ipgroup","rule_ipgroup",
			function(section)
				--print(addr == uci_r:get("ipgroup", section[".name"], "name"))
				if addr == uci_r:get("ipgroup", section[".name"], "name") then
					res = "SUCCESS"
				end
			end
		)
		--print("-----")
		return res
	end
end

function checktime(effective_time)
	--uci_r:set_confdir(CFG_PATH)
	local count = 0
	uci_r:foreach("time_mngt","entry",
		function(section)
			if effective_time == uci_r:get("time_mngt", section[".name"], "entry_name") then
				count = 1
			end
		end
	)
	if count == 1 then
		return "SUCCESS"
	else
		return "FAILED"
	end
end
function get_if()
    local interface = {}
    local res = ifs.get_ifs("pure")
    for k,v in ipairs(res.normal) do
		v.t_label = string.gsub(v.t_label, " ", "_")
		table.insert(interface,v.t_label)
    end

    if res.vpn then
        for k,v in ipairs(res.vpn) do
			v.t_label = string.gsub(v.t_label, " ", "_")
            table.insert(interface,v.t_label)
        end
    end

    return interface
end



local result = "SUCCESS"
local cmd = arg[1]
--dbg.print_tty(cmd)
if arg[1] == "access-list ip rule deny" then
	local access_list_num = arg[2]
	local rule_id = arg[3]  
	local service_type = arg[4]
	local direction = string.gsub(arg[5], "_"," ")
	local sip = arg[6]
	local dip = arg[7]
	local effective_time = arg[8]
	--[[if tonumber(rule_id) > 1000 then
		dbg.print_tty("id error")
		result = "FAILED"]]--
	direction = mysplit(direction,",")
	if "FAILED" == checkservice(service_type) then
		dbg.print_tty("Error: Invalid deny parameter [1] service-type. Such as ALL or FTP.")
		result = "FAILED"
	elseif "FAILED" == checkdirection(direction) then
		local interface = get_if()
		dbg.print_tty("Error: Invalid deny parameter [2] direction. Such as "..table.concat(interface, ",")..",LAN-LAN,ALL.")
		result = "FAILED"
	elseif "FAILED" == checkaddr(direction, sip) or  "FAILED" == checkaddr(direction, dip) then
		local ipgroup_list = get_ipgroup()
		local lan_list = get_lan_list()
		if (direction[1] == "VNET") then
			dbg.print_tty("Error: Invalid deny parameter [3/4] sip or dip. Such as "..table.concat(lan_list, ",")..".")
		else
			dbg.print_tty("Error: Invalid deny parameter [3/4] sip or dip. Such as "..table.concat(ipgroup_list, ",")..".")
		end
		result = "FAILED"
	elseif "FAILED" == checktime(effective_time) then
		local time_list = get_time_range()
		dbg.print_tty("Error: Invalid deny parameter [5] time-range. Such as "..table.concat(time_list, ",")..".")
		result = "FAILED"
	else		
		--direction = convertdirection(direction)
		result = add_deny_acl_rule(access_list_num, rule_id, service_type, direction, sip, dip, effective_time)
	end
end

if arg[1] == "access-list ip rule permit" then
	local access_list_num = arg[2]
	local rule_id = arg[3]
	local service_type = arg[4]
	local direction = string.gsub(arg[5], "_"," ")
	local sip = arg[6]
	local dip = arg[7]
	local effective_time = arg[8]
	
	--[[if tonumber(rule_id) > 10000 then
		dbg.print_tty("id error")
		result = "FAILED"]]--
	direction = mysplit(direction,",")
	if "FAILED" == checkservice(service_type) then
		dbg.print_tty("Error: Invalid permit parameter [1] service-type. Such as ALL or FTP.")
		result = "FAILED"
	elseif "FAILED" == checkdirection(direction) then
		local interface = get_if()
		dbg.print_tty("Error: Invalid permit parameter [2] direction. Such as "..table.concat(interface, ",")..",LAN-LAN,ALL.")
		result = "FAILED"
	elseif "FAILED" == checkaddr(direction, sip) or  "FAILED" == checkaddr(direction, dip) then
		local ipgroup_list = get_ipgroup()
		local lan_list = get_lan_list()
		if (direction[1] == "VNET") then
			dbg.print_tty("Error: Invalid permit parameter [3/4] sip or dip. Such as "..table.concat(lan_list, ",")..".")
		else
			dbg.print_tty("Error: Invalid permit parameter [3/4] sip or dip. Such as "..table.concat(ipgroup_list, ",")..".")
		end
		result = "FAILED"
	elseif "FAILED" == checktime(effective_time) then
		local time_list = get_time_range()
		dbg.print_tty("Error: Invalid permit parameter [5] time-range. Such as "..table.concat(time_list, ",")..".")
		result = "FAILED"
	else	
		--direction = convertdirection(direction)
		result = add_permit_acl_rule(access_list_num, rule_id, service_type, direction, sip, dip, effective_time)
	end
end

if arg[1] == "no access-list ip" then
	local access_list_num = arg[2]
	--uci_r:set_confdir(CFG_PATH)
	local flag = uci_r:get("access_ctl", access_list_num, "flag") or ""
	if flag == "" then
		dbg.print_tty("Error: Invalid acl name.")
		result = "FAILED"
	else
		result = delete_acl_rule(access_list_num)
	end
end

if arg[1] == "show access-list" then
	result = show_access_ctl()
	--dbg.print_tty(result)
end

--io.stdout:write( '<<S>>' .. result .. '<<E>>' )












