local sys  = require "luci.sys"
local json = require 'luci.json'
local utl = require "luci.util"
local dbg  = require ('luci.tools.debug')
local form = require "luci.tools.form"
local uci  = require "luci.model.uci"
local ospf = require 'luci.controller.admin.ospf'
local error_print = require "luci.tools.error_print"

local uci_r = uci.cursor()
local CLI_CFG_PATH = "/tmp/cli_config"
local CFG_PATH = "/etc/config"

local filename = CLI_CFG_PATH.."/cli_ospf"
local file = io.open(filename,"r")
if file == nil then
    sys.exec("touch " .. filename)
else
    file:close()
end

OSPF_INTERFACE = nil
OSPF_INDEX = "0"
OSPF_AUTO_TYPE_NONE = "0"
OSPF_AUTO_TYPE_SIMPLE = "1"
OSPF_AUTO_TYPE_MD5 = "2"
OSPF_ENABLE = "1"
OSPF_DISABLE = "0"
OSPF_BROADCAST = "0"
OSPF_POINT_TO_POINT = "1"
OSPF_RET_RE = "0"
OSPF_AREA_ID = "0"
UINT32_MAX = "4294967295"
MULTICAST_ADD_PREFIX = 224
LOOP_ADD_PREFIX = 127
ALL_ZERO_ADD = "0.0.0.0" 

--ospf {enable|disable}
function set_enabled_ospf(enabled)
    local result  = "SUCCESS"
    local http_form = {}
    local tmp_config = {}
    local get_http_form = {}

    uci_r:set_confdir(CFG_PATH) 

    local router_ospf_en
    if enabled == "enable" then
        --若ospf进程已开启则直接返回
        local ospf_status = uci_r:get("ospf", "global", "status")
        if ospf_status == "on" then
            return result
        end

        router_ospf_en = "on"
    else
        --若ospf进程已关闭则直接返回
        local ospf_status = uci_r:get("ospf", "global", "status")
        if ospf_status == "off" then
            return result
        end

        router_ospf_en = "off"
    end

    tmp_config.sys_auto = "on"
    tmp_config.sys_area_id = OSPF_AREA_ID

    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["status"] = router_ospf_en,
            ["auto"] = tmp_config.sys_auto,
            ["area_id"] = tmp_config.sys_area_id,
        }

    })
    http_form.form = "global"
    --(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end

    --get_http_form
	get_http_form.data = json.encode(
        {
            ["method"] = "get",
            ["params"] = {}
    
        })
    get_http_form.form = "interface"
    ret = ospf.dispatch(get_http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end

    return result
end

--area-id {areaid}
function set_area_id(areaid)
    local http_form = {}
    local tmp_config = {}
    local max_areaid = UINT32_MAX 

    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end 
    if tonumber(areaid) < 0 or #max_areaid < #areaid or (#max_areaid == #areaid and tonumber(max_areaid) < tonumber(areaid)) then
        dbg.print_tty("Error: Invalid parameter " .. areaid)
        return "FAILED"
    end
    uci_r:set_confdir(CFG_PATH)
    tmp_config.sys_auto = uci_r:get("ospf", "global", "auto")
    tmp_config.sys_area_id = areaid
    if tmp_config.sys_auto == "off" then
        tmp_config.sys_router_id = uci_r:get("ospf", "global", "router_id")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["params"] = 
                {
                    ["status"] = tmp_config.sys_status,
                    ["auto"] = tmp_config.sys_auto,
                    ["router_id"] = tmp_config.sys_router_id,
                    ["area_id"] = tmp_config.sys_area_id,
                }
        
            })
    else
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["params"] = 
            {
                ["status"] = tmp_config.sys_status,
                ["auto"] = tmp_config.sys_auto,
                ["area_id"] = tmp_config.sys_area_id,
            }

        })
    end
    http_form.form = "global"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--router-id {routerid}
function set_router_id(auto_enabled, routerid)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 


    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    tmp_config.sys_auto = auto_enabled
    tmp_config.sys_area_id = uci_r:get("ospf", "global", "area_id") or OSPF_AREA_ID
    tmp_config.sys_router_id = routerid
    if auto_enabled == "off" then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["params"] = 
            {
                ["status"] = tmp_config.sys_status,
                ["auto"] = tmp_config.sys_auto,
                ["router_id"] = tmp_config.sys_router_id,
                ["area_id"] = tmp_config.sys_area_id,
            }

        })
    elseif auto_enabled == "on" then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["params"] = 
            {
                ["status"] = tmp_config.sys_status,
                ["auto"] = tmp_config.sys_auto,
                ["area_id"] = tmp_config.sys_area_id,
            }

        })
    end
    http_form.form = "global"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--default-information originate
function set_default_information_originate()
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 


    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    tmp_config.sys_originate = "enable"
    tmp_config.sys_always = uci_r:get("ospf", "advertise_cfg", "always")
    tmp_config.sys_metric = uci_r:get("ospf", "advertise_cfg", "metric")
    tmp_config.sys_metric_type = uci_r:get("ospf", "advertise_cfg", "metric_type")
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["originate"] = tmp_config.sys_originate,
            ["always"] = tmp_config.sys_always,
            ["metric"] = tmp_config.sys_metric,
            ["metric_type"] = tmp_config.sys_metric_type,
        }

    })
    http_form.form = "advertise_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

function set_no_default_information_originate()
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 


    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    tmp_config.sys_originate = "disable"
    tmp_config.sys_always = uci_r:get("ospf", "advertise_cfg", "always")
    tmp_config.sys_metric = uci_r:get("ospf", "advertise_cfg", "metric")
    tmp_config.sys_metric_type = uci_r:get("ospf", "advertise_cfg", "metric_type")
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["originate"] = tmp_config.sys_originate,
            ["always"] = tmp_config.sys_always,
            ["metric"] = tmp_config.sys_metric,
            ["metric_type"] = tmp_config.sys_metric_type,
        }

    })
    http_form.form = "advertise_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

function set_default_information_originate_always(always)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 


    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    tmp_config.sys_originate = uci_r:get("ospf", "advertise_cfg", "originate")
    if tmp_config.sys_originate == "disable" then
        dbg.print_tty("Error: Default route originate is not enabled.")
        return "FAILED"
    end
    if always == "enable" then
        tmp_config.sys_always = "enable"
    else
        tmp_config.sys_always = "disable"
    end

    tmp_config.sys_metric = uci_r:get("ospf", "advertise_cfg", "metric")
    tmp_config.sys_metric_type = uci_r:get("ospf", "advertise_cfg", "metric_type")
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["originate"] = tmp_config.sys_originate,
            ["always"] = tmp_config.sys_always,
            ["metric"] = tmp_config.sys_metric,
            ["metric_type"] = tmp_config.sys_metric_type,
        }

    })
    http_form.form = "advertise_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end  
function set_default_information_originate_metric(metric_cost)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    tmp_config.sys_originate = uci_r:get("ospf", "advertise_cfg", "originate")
    if tmp_config.sys_originate == "disable" then
        dbg.print_tty("Error: Default route originate is not enabled.")
        return "FAILED"
    end
    tmp_config.sys_always = uci_r:get("ospf", "advertise_cfg", "always")
    tmp_config.sys_metric = metric_cost
    tmp_config.sys_metric_type = uci_r:get("ospf", "advertise_cfg", "metric_type")
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["originate"] = tmp_config.sys_originate,
            ["always"] = tmp_config.sys_always,
            ["metric"] = tmp_config.sys_metric,
            ["metric_type"] = tmp_config.sys_metric_type,
        }

    })
    http_form.form = "advertise_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end 
function set_default_information_originate_metric_type(metric_type)  
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 


    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    tmp_config.sys_originate = uci_r:get("ospf", "advertise_cfg", "originate")
    if tmp_config.sys_originate == "disable" then
        dbg.print_tty("Error: Default route originate is not enabled.")
        return "FAILED"
    end
    tmp_config.sys_always = uci_r:get("ospf", "advertise_cfg", "always")
    tmp_config.sys_metric = uci_r:get("ospf", "advertise_cfg", "metric")
    tmp_config.sys_metric_type = metric_type
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["originate"] = tmp_config.sys_originate,
            ["always"] = tmp_config.sys_always,
            ["metric"] = tmp_config.sys_metric,
            ["metric_type"] = tmp_config.sys_metric_type,
        }

    })
    http_form.form = "advertise_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end   

--compatible rfc1583
function set_compatible_rfc1583()
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    tmp_config.sys_rfc_compatible = "enable"

    tmp_config.sys_distance = uci_r:get("ospf", "ospf_cfg", "distance") 
    tmp_config.sys_spf_delay = uci_r:get("ospf", "ospf_cfg", "spf_delay") 
    tmp_config.sys_spf_hold_init = uci_r:get("ospf", "ospf_cfg", "spf_hold_init") 
    tmp_config.sys_spf_hold_max = uci_r:get("ospf", "ospf_cfg", "spf_hold_max") 
    tmp_config.sys_max_path = uci_r:get("ospf", "ospf_cfg", "max_path")
    tmp_config.sys_passive_def = uci_r:get("ospf", "ospf_cfg", "passive_def")

    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["distance"] = tmp_config.sys_distance,
            ["rfc_compatible"] = tmp_config.sys_rfc_compatible,
            ["spf_delay"] = tmp_config.sys_spf_delay,
            ["spf_hold_init"] = tmp_config.sys_spf_hold_init,
            ["spf_hold_max"] = tmp_config.sys_spf_hold_max,
            ["max_path"] = tmp_config.sys_max_path,
            ["passive_def"] = tmp_config.sys_passive_def,
        }

    })
    http_form.form = "ospf_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--no compatible_rfc1583
function set_no_compatible_rfc1583()
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    tmp_config.sys_rfc_compatible = "disable"

    tmp_config.sys_distance = uci_r:get("ospf", "ospf_cfg", "distance")
    tmp_config.sys_spf_delay = uci_r:get("ospf", "ospf_cfg", "spf_delay")
    tmp_config.sys_spf_hold_init = uci_r:get("ospf", "ospf_cfg", "spf_hold_init")
    tmp_config.sys_spf_hold_max = uci_r:get("ospf", "ospf_cfg", "spf_hold_max")
    tmp_config.sys_max_path = uci_r:get("ospf", "ospf_cfg", "max_path")
    tmp_config.sys_passive_def = uci_r:get("ospf", "ospf_cfg", "passive_def")

    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["distance"] = tmp_config.sys_distance,
            ["rfc_compatible"] = tmp_config.sys_rfc_compatible,
            ["spf_delay"] = tmp_config.sys_spf_delay,
            ["spf_hold_init"] = tmp_config.sys_spf_hold_init,
            ["spf_hold_max"] = tmp_config.sys_spf_hold_max,
            ["max_path"] = tmp_config.sys_max_path,
            ["passive_def"] = tmp_config.sys_passive_def,
        }

    })
    http_form.form = "ospf_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--distance
function set_distance(distance)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    tmp_config.sys_distance = distance

    tmp_config.sys_rfc_compatible = uci_r:get("ospf", "ospf_cfg", "rfc_compatible")
    tmp_config.sys_spf_delay = uci_r:get("ospf", "ospf_cfg", "spf_delay")
    tmp_config.sys_spf_hold_init = uci_r:get("ospf", "ospf_cfg", "spf_hold_init")
    tmp_config.sys_spf_hold_max = uci_r:get("ospf", "ospf_cfg", "spf_hold_max")
    tmp_config.sys_max_path = uci_r:get("ospf", "ospf_cfg", "max_path")
    tmp_config.sys_passive_def = uci_r:get("ospf", "ospf_cfg", "passive_def")

    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["distance"] = tmp_config.sys_distance,
            ["rfc_compatible"] = tmp_config.sys_rfc_compatible,
            ["spf_delay"] = tmp_config.sys_spf_delay,
            ["spf_hold_init"] = tmp_config.sys_spf_hold_init,
            ["spf_hold_max"] = tmp_config.sys_spf_hold_max,
            ["max_path"] = tmp_config.sys_max_path,
            ["passive_def"] = tmp_config.sys_passive_def,
        }

    })
    http_form.form = "ospf_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--timers throttle spf{spf_delaytime, spf_init_holdtime, spf_max_holdtime}
function set_spf_times6(spf_delaytime, spf_init_holdtime, spf_max_holdtime)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    tmp_config.sys_distance = uci_r:get("ospf", "ospf_cfg", "distance")
    tmp_config.sys_rfc_compatible = uci_r:get("ospf", "ospf_cfg", "rfc_compatible")

    tmp_config.sys_spf_delay = spf_delaytime
    tmp_config.sys_spf_hold_init = spf_init_holdtime
    tmp_config.sys_spf_hold_max = spf_max_holdtime

    tmp_config.sys_max_path = uci_r:get("ospf", "ospf_cfg", "max_path")
    tmp_config.sys_passive_def = uci_r:get("ospf", "ospf_cfg", "passive_def")

    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["distance"] = tmp_config.sys_distance,
            ["rfc_compatible"] = tmp_config.sys_rfc_compatible,
            ["spf_delay"] = tmp_config.sys_spf_delay,
            ["spf_hold_init"] = tmp_config.sys_spf_hold_init,
            ["spf_hold_max"] = tmp_config.sys_spf_hold_max,
            ["max_path"] = tmp_config.sys_max_path,
            ["passive_def"] = tmp_config.sys_passive_def,
        }

    })
    http_form.form = "ospf_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--maximum-paths
function set_max_path(max_paths)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    tmp_config.sys_distance = uci_r:get("ospf", "ospf_cfg", "distance")
    tmp_config.sys_rfc_compatible = uci_r:get("ospf", "ospf_cfg", "rfc_compatible")
    tmp_config.sys_spf_delay = uci_r:get("ospf", "ospf_cfg", "spf_delay")
    tmp_config.sys_spf_hold_init = uci_r:get("ospf", "ospf_cfg", "spf_hold_init")
    tmp_config.sys_spf_hold_max = uci_r:get("ospf", "ospf_cfg", "spf_hold_max")

    tmp_config.sys_max_path = max_paths

    tmp_config.sys_passive_def = uci_r:get("ospf", "ospf_cfg", "passive_def")

    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["distance"] = tmp_config.sys_distance,
            ["rfc_compatible"] = tmp_config.sys_rfc_compatible,
            ["spf_delay"] = tmp_config.sys_spf_delay,
            ["spf_hold_init"] = tmp_config.sys_spf_hold_init,
            ["spf_hold_max"] = tmp_config.sys_spf_hold_max,
            ["max_path"] = tmp_config.sys_max_path,
            ["passive_def"] = tmp_config.sys_passive_def,
        }

    })
    http_form.form = "ospf_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--passive-interface default
function set_passive_def(enabled)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    tmp_config.sys_distance = uci_r:get("ospf", "ospf_cfg", "distance")
    tmp_config.sys_rfc_compatible = uci_r:get("ospf", "ospf_cfg", "rfc_compatible")
    tmp_config.sys_spf_delay = uci_r:get("ospf", "ospf_cfg", "spf_delay")
    tmp_config.sys_spf_hold_init = uci_r:get("ospf", "ospf_cfg", "spf_hold_init")
    tmp_config.sys_spf_hold_max = uci_r:get("ospf", "ospf_cfg", "spf_hold_max")
    tmp_config.sys_max_path = uci_r:get("ospf", "ospf_cfg", "max_path")

    if enabled == "enable" then
        tmp_config.sys_passive_def = "enable"
    else
        tmp_config.sys_passive_def = "disable"
    end
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["distance"] = tmp_config.sys_distance,
            ["rfc_compatible"] = tmp_config.sys_rfc_compatible,
            ["spf_delay"] = tmp_config.sys_spf_delay,
            ["spf_hold_init"] = tmp_config.sys_spf_hold_init,
            ["spf_hold_max"] = tmp_config.sys_spf_hold_max,
            ["max_path"] = tmp_config.sys_max_path,
            ["passive_def"] = tmp_config.sys_passive_def,
        }

    })
    http_form.form = "ospf_cfg"
    --dbg.dumptable(http_form)
    
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--ip ospf
--get index id
function get_interface_index()
    local count = 0
    uci_r:foreach("ospf", "interface", 
    function(section)
        if section[".name"] == OSPF_INTERFACE then
            OSPF_INDEX = tostring(count)
        end
        count = count + 1;
    end
    )
    return OSPF_INDEX
end

--ip ospf priority
function set_ip_ospf_priority(pri)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_new_priority = pri
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_new_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_new_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
            tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
            tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
            http_form.data = json.encode(
                {
                    ["method"] = "set",
                    ["config_source"] = "cli",
                    ["params"] = 
                    {
                        ["index"] = tmp_config.sys_index,
                        ["old"] =
                        {
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["new"] =
                        { 
                            ["interface"] = tmp_config.sys_interface,
                            ["priority"] = tmp_config.sys_new_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["interface"] = tmp_config.sys_interface
                    }
                })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--ip ospf hello-interval
function set_ip_ospf_hello_interval(hello_interval)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_new_hello = hello_interval
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_new_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_new_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
            tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
            tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
            http_form.data = json.encode(
                {
                    ["method"] = "set",
                    ["config_source"] = "cli",
                    ["params"] = 
                    {
                        ["index"] = tmp_config.sys_index,
                        ["old"] =
                        {
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["new"] =
                        { 
                            ["interface"] = tmp_config.sys_interface,
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_new_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["interface"] = tmp_config.sys_interface
                    }
                })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end 

--ip ospf dead-interval
function set_ip_ospf_dead_interval(dead_interval)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_new_dead = dead_interval
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_new_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_new_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
            tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
            tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
            http_form.data = json.encode(
                {
                    ["method"] = "set",
                    ["config_source"] = "cli",
                    ["params"] = 
                    {
                        ["index"] = tmp_config.sys_index,
                        ["old"] =
                        {
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["new"] =
                        { 
                            ["interface"] = tmp_config.sys_interface,
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_new_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["interface"] = tmp_config.sys_interface
                    }
                })

    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

function set_ip_ospf_transmit_delay(transmit_delay)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_new_transmit = transmit_delay
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_new_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_new_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
            tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
            tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
            http_form.data = json.encode(
                {
                    ["method"] = "set",
                    ["config_source"] = "cli",
                    ["params"] = 
                    {
                        ["index"] = tmp_config.sys_index,
                        ["old"] =
                        {
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["new"] =
                        { 
                            ["interface"] = tmp_config.sys_interface,
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_new_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["interface"] = tmp_config.sys_interface
                    }
                })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--ip ospf retransmit-interval
function set_ip_ospf_retransmit_interval(retransmit_interval)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_new_retransmit = retransmit_interval
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_new_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_new_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
            tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
            tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
            http_form.data = json.encode(
                {
                    ["method"] = "set",
                    ["config_source"] = "cli",
                    ["params"] = 
                    {
                        ["index"] = tmp_config.sys_index,
                        ["old"] =
                        {
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["new"] =
                        { 
                            ["interface"] = tmp_config.sys_interface,
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_new_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["interface"] = tmp_config.sys_interface
                    }
                })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end

--ip ospf cost
function set_ip_ospf_cost(cost)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_new_cost = cost
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_new_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_new_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
            tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
            tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
            http_form.data = json.encode(
                {
                    ["method"] = "set",
                    ["config_source"] = "cli",
                    ["params"] = 
                    {
                        ["index"] = tmp_config.sys_index,
                        ["old"] =
                        {
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["new"] =
                        { 
                            ["interface"] = tmp_config.sys_interface,
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_new_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["interface"] = tmp_config.sys_interface
                    }
                })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end 

--ip ospf network
function set_ip_ospf_network(network)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    if network == "Broadcast" then
        tmp_config.sys_new_network_type = OSPF_BROADCAST
    else
        tmp_config.sys_new_network_type = OSPF_POINT_TO_POINT
    end

    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_new_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_new_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
            tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
            tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
            http_form.data = json.encode(
                {
                    ["method"] = "set",
                    ["config_source"] = "cli",
                    ["params"] = 
                    {
                        ["index"] = tmp_config.sys_index,
                        ["old"] =
                        {
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["new"] =
                        { 
                            ["interface"] = tmp_config.sys_interface,
                            ["priority"] = tmp_config.sys_priority,
                            ["retransmit"] = tmp_config.sys_retransmit,
                            ["hello"] = tmp_config.sys_hello,
                            ["dead"] = tmp_config.sys_dead,
                            ["transmit"] = tmp_config.sys_transmit,
                            ["cost"] = tmp_config.sys_cost,
                            ["network_type"] = tmp_config.sys_new_network_type,
                            ["passive"] = tmp_config.sys_passive,
                            ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                            ["auto_type"] = tmp_config.sys_auto_type,
                            ["md5_key_id"] = tmp_config.sys_md5_key_id,
                            ["md5_key"] = tmp_config.sys_md5_key,
                            ["state"] = tmp_config.sys_state,
                            ["dr"] = tmp_config.sys_dr,
                            ["bdr"] = tmp_config.sys_bdr,
                        },
                        ["interface"] = tmp_config.sys_interface
                    }
                })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end 

--ip ospf passive
function set_ip_ospf_passive(passive)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    if passive == "enable" then
        tmp_config.sys_new_passive = OSPF_ENABLE
    else
        tmp_config.sys_new_passive = OSPF_DISABLE
    end
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_new_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_new_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
        tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
        tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["md5_key_id"] = tmp_config.sys_md5_key_id,
                        ["md5_key"] = tmp_config.sys_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_new_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["md5_key_id"] = tmp_config.sys_md5_key_id,
                        ["md5_key"] = tmp_config.sys_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end   

--ip ospf mtu-ignore
function set_ip_ospf_mtu_ignore(mtu_ignore)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    if mtu_ignore == "enable" then
        tmp_config.sys_new_mtu_ignore = OSPF_ENABLE
    else
        tmp_config.sys_new_mtu_ignore = OSPF_DISABLE
    end
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_new_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_new_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
        tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
        tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["md5_key_id"] = tmp_config.sys_md5_key_id,
                        ["md5_key"] = tmp_config.sys_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_new_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["md5_key_id"] = tmp_config.sys_md5_key_id,
                        ["md5_key"] = tmp_config.sys_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end 

--ip ospf authentication
function set_ip_ospf_authentication_None()
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    tmp_config.sys_new_auto_type = OSPF_AUTO_TYPE_NONE   
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_new_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })

    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_new_auto_type,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
        tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
        tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["md5_key_id"] = tmp_config.sys_md5_key_id,
                        ["md5_key"] = tmp_config.sys_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_new_auto_type,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end 

function set_ip_ospf_authentication_simple(simple_key)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    tmp_config.sys_new_auto_type = OSPF_AUTO_TYPE_SIMPLE   
    tmp_config.sys_new_simple_key = simple_key
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_new_auto_type,
                    ["simple_key"] = tmp_config.sys_new_simple_key,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })

    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_new_auto_type,
                        ["simple_key"] = tmp_config.sys_new_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
        tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
        tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["md5_key_id"] = tmp_config.sys_md5_key_id,
                        ["md5_key"] = tmp_config.sys_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_new_auto_type,
                        ["simple_key"] = tmp_config.sys_new_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end  

function set_ip_ospf_authentication_md5(md5_key_id, md5_key)
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    if OSPF_INTERFACE == nil then
        dbg.print_tty("Error: Please input vlan id first.")
        return "FAILD"
    end
    tmp_config.sys_index = get_interface_index()
    tmp_config.sys_new_auto_type = OSPF_AUTO_TYPE_MD5   
    tmp_config.sys_new_md5_key_id = md5_key_id
    tmp_config.sys_new_md5_key = md5_key
    --old
    tmp_config.sys_priority = uci_r:get("ospf", OSPF_INTERFACE, "priority")
    tmp_config.sys_retransmit = uci_r:get("ospf", OSPF_INTERFACE, "retransmit")
    tmp_config.sys_hello = uci_r:get("ospf", OSPF_INTERFACE, "hello")
    tmp_config.sys_dead = uci_r:get("ospf", OSPF_INTERFACE, "dead")
    tmp_config.sys_transmit = uci_r:get("ospf", OSPF_INTERFACE, "transmit")
    tmp_config.sys_cost = uci_r:get("ospf", OSPF_INTERFACE, "cost")
    tmp_config.sys_network_type = uci_r:get("ospf", OSPF_INTERFACE, "network_type")
    tmp_config.sys_passive = uci_r:get("ospf", OSPF_INTERFACE, "passive")
    tmp_config.sys_mtu_ignore = uci_r:get("ospf", OSPF_INTERFACE, "mtu_ignore")
    tmp_config.sys_auto_type = uci_r:get("ospf", OSPF_INTERFACE, "auto_type")
    tmp_config.sys_state = uci_r:get("ospf", OSPF_INTERFACE, "state")
    tmp_config.sys_dr = ""
    tmp_config.sys_bdr = ""
    --new
    tmp_config.sys_interface = uci_r:get("ospf", OSPF_INTERFACE, "interface")
    
    if tmp_config.sys_auto_type == OSPF_AUTO_TYPE_NONE then
        http_form.data = json.encode(
        {
            ["method"] = "set",
            ["config_source"] = "cli",
            ["params"] = 
            {
                ["index"] = tmp_config.sys_index,
                ["old"] =
                {
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_auto_type,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["new"] =
                { 
                    ["interface"] = tmp_config.sys_interface,
                    ["priority"] = tmp_config.sys_priority,
                    ["retransmit"] = tmp_config.sys_retransmit,
                    ["hello"] = tmp_config.sys_hello,
                    ["dead"] = tmp_config.sys_dead,
                    ["transmit"] = tmp_config.sys_transmit,
                    ["cost"] = tmp_config.sys_cost,
                    ["network_type"] = tmp_config.sys_network_type,
                    ["passive"] = tmp_config.sys_passive,
                    ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                    ["auto_type"] = tmp_config.sys_new_auto_type,
                    ["md5_key_id"] = tmp_config.sys_new_md5_key_id,
                    ["md5_key"] = tmp_config.sys_new_md5_key,
                    ["state"] = tmp_config.sys_state,
                    ["dr"] = tmp_config.sys_dr,
                    ["bdr"] = tmp_config.sys_bdr,
                },
                ["interface"] = tmp_config.sys_interface
            }
        })

    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_SIMPLE then
        tmp_config.sys_simple_key = uci_r:get("ospf", OSPF_INTERFACE, "simple_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["simple_key"] = tmp_config.sys_simple_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_new_auto_type,
                        ["md5_key_id"] = tmp_config.sys_new_md5_key_id,
                        ["md5_key"] = tmp_config.sys_new_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    elseif tmp_config.sys_auto_type == OSPF_AUTO_TYPE_MD5 then
        tmp_config.sys_md5_key_id = uci_r:get("ospf", OSPF_INTERFACE, "md5_key_id")
        tmp_config.sys_md5_key = uci_r:get("ospf", OSPF_INTERFACE, "md5_key")
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["config_source"] = "cli",
                ["params"] = 
                {
                    ["index"] = tmp_config.sys_index,
                    ["old"] =
                    {
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_auto_type,
                        ["md5_key_id"] = tmp_config.sys_md5_key_id,
                        ["md5_key"] = tmp_config.sys_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["new"] =
                    { 
                        ["interface"] = tmp_config.sys_interface,
                        ["priority"] = tmp_config.sys_priority,
                        ["retransmit"] = tmp_config.sys_retransmit,
                        ["hello"] = tmp_config.sys_hello,
                        ["dead"] = tmp_config.sys_dead,
                        ["transmit"] = tmp_config.sys_transmit,
                        ["cost"] = tmp_config.sys_cost,
                        ["network_type"] = tmp_config.sys_network_type,
                        ["passive"] = tmp_config.sys_passive,
                        ["mtu_ignore"] = tmp_config.sys_mtu_ignore,
                        ["auto_type"] = tmp_config.sys_new_auto_type,
                        ["md5_key_id"] = tmp_config.sys_new_md5_key_id,
                        ["md5_key"] = tmp_config.sys_new_md5_key,
                        ["state"] = tmp_config.sys_state,
                        ["dr"] = tmp_config.sys_dr,
                        ["bdr"] = tmp_config.sys_bdr,
                    },
                    ["interface"] = tmp_config.sys_interface
                }
            })
    end
    http_form.form = "interface"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return "SUCCESS"
end   

--show ip ospf datebase
function set_show_ospf_datebase()
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    --data: {"method":"get","params":{}}
    http_form.data = json.encode(
        {
            ["method"] = "get",
            ["params"] = {}
        })
    http_form.form = "lsdb"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    --print_lua_table_database(ret.result)
	ret = ret.result
	dbg.print_tty(string.format("%-15s", "Area ID") .. string.format("%-20s", "Advertising Router") .. string.format("%-15s", "LSA Type") 
		.. string.format("%-20s", "Link State ID") .. string.format("%-15s", "Age") .. string.format("%-15s", "Sequence") 
		.. string.format("%-15s", "Checksum") .. string.format("%-15s", "Options"))
	
	for _,v in ipairs(ret) do
		dbg.print_tty(string.format("%-15s", v.area_id) .. string.format("%-20s", v.router_id) .. string.format("%-15s", v.lsa_type)
			 .. string.format("%-20s", v.link_id) .. string.format("%-15s", v.age) .. string.format("%-15s", v.seq)
			  .. string.format("%-15s", v.chk_sum) .. string.format("%-15s", v.option))
	end
	
    return "SUCCESS"
end  

--show ip ospf neighbor
function set_show_ospf_neighbor()
    local http_form = {}
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    --data: {"method":"get","params":{}}
    http_form.data = json.encode(
        {
            ["method"] = "get",
            ["params"] = {}
        })
    http_form.form = "neighbor"
    --dbg.dumptable(http_form) 
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
	ret = ret.result
	dbg.print_tty(string.format("%-15s", "Interface") .. string.format("%-20s", "Neighbor IP Address") .. string.format("%-15s", "Router ID")
		 .. string.format("%-18s", "Area ID") .. string.format("%-20s", "Options") .. string.format("%-15s", "Router Priority")
		  .. string.format("%-15s", "State") .. string.format("%-16s", "Events") .. string.format("%-30s", "Retransmission Queue length")
		 .. string.format("%-16s", "Dead Time"))
	
	for _,v in ipairs(ret) do
		dbg.print_tty(string.format("%-15s", v.interface) .. string.format("%-20s", v.neighbor_ip) .. string.format("%-15s", v.router_id)
			 .. string.format("%-18s", v.area_id) .. string.format("%-20s", v.option) .. string.format("%-15s", v.priority)
			  .. string.format("%-15s", v.dead) .. string.format("%-16s", v.event) .. string.format("%-30s", v.retrans_quelen)
			 .. string.format("%-16s", v.dead1))
	end
		 
    return "SUCCESS"
end

--show ip ospf
function set_show_ospf()
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    dbg.print_tty("---------------------------------------------------------------------")
    dbg.print_tty("OSPF Global Config")
    dbg.print_tty("status:                 "..tmp_config.sys_status)
    tmp_config.sys_auto = uci_r:get("ospf", "global", "auto")
    tmp_config.sys_area_id = uci_r:get("ospf", "global", "area_id")
    dbg.print_tty("auto:                   "..tmp_config.sys_auto)
    dbg.print_tty("area_id:                "..tmp_config.sys_area_id)
    if tmp_config.sys_auto == "off" then
        tmp_config.sys_router_id = uci_r:get("ospf", "global", "router_id")
        dbg.print_tty("router_id:              "..tmp_config.sys_router_id)
    end
    
    dbg.print_tty("---------------------------------------------------------------------")
    dbg.print_tty("OSPF Config")
    tmp_config.sys_distance = uci_r:get("ospf", "ospf_cfg", "distance")
    tmp_config.sys_rfc_compatible = uci_r:get("ospf", "ospf_cfg", "rfc_compatible")
    tmp_config.sys_max_path = uci_r:get("ospf", "ospf_cfg", "max_path")
    tmp_config.sys_spf_delay = uci_r:get("ospf", "ospf_cfg", "spf_delay")
    tmp_config.sys_spf_hold_init = uci_r:get("ospf", "ospf_cfg", "spf_hold_init")
    tmp_config.sys_spf_hold_max = uci_r:get("ospf", "ospf_cfg", "spf_hold_max")
    tmp_config.sys_passive_def = uci_r:get("ospf", "ospf_cfg", "passive_def")
    --ssh_cli dbg.print_tty
    dbg.print_tty("Distance:               "..tmp_config.sys_distance)
    dbg.print_tty("RFC 1583 Compatibility: "..tmp_config.sys_rfc_compatible)
    dbg.print_tty("SPF Delay Time:         "..tmp_config.sys_spf_delay)
    dbg.print_tty("SPF Hold Init Time:     "..tmp_config.sys_spf_hold_init)
    dbg.print_tty("SPF Hold Max Time:      "..tmp_config.sys_spf_hold_max)
    dbg.print_tty("Maximum Paths:          "..tmp_config.sys_max_path)
    dbg.print_tty("Passive Default:        "..tmp_config.sys_passive_def)
    
    return "SUCCESS"
end

--show ip ospf interface
function set_show_ospf_interface()
    local tmp_config = {}
    local http_form = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
    --get_http_form
	http_form.data = json.encode(
        {
            ["method"] = "get",
            ["params"] = {}
    
        })
    http_form.form = "interface"
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= OSPF_RET_RE) then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end

    ret = ret.result
	dbg.print_tty(string.format("%-15s", "Interface") .. string.format("%-18s", "IP Address/Mask") .. string.format("%-12s", "Working")
		 .. string.format("%-18s", "Router Priority") .. string.format("%-20s", "Retransmit Interval") .. string.format("%-15s", "Hello Interval")
		  .. string.format("%-15s", "Dead Interval") .. string.format("%-16s", "Transmit Delay") .. string.format("%-10s", "Cost")
		 .. string.format("%-16s", "Network Type") .. string.format("%-16s", "Passive Mode") .. string.format("%-16s", "MTU Ignore")
		 .. string.format("%-20s", "Authentication Type"))
    for _,v in ipairs(ret) do
        if v.network_type == OSPF_BROADCAST then
            v.network_type = "Broadcast"
        else
            v.network_type = "point-to-point"
        end
        
        if v.passive == OSPF_ENABLE then
            v.passive = "Enable"
        else
            v.passive = "Disable"
        end
        
        if v.mtu_ignore == OSPF_ENABLE then
            v.mtu_ignore = "Enable"
        else
            v.mtu_ignore = "Disable"
        end
        
        if v.auto_type == OSPF_AUTO_TYPE_NONE then
            v.auto_type = "None"
        elseif v.auto_type == OSPF_AUTO_TYPE_SIMPLE then
            v.auto_type = "Simple"
        elseif v.auto_type == OSPF_AUTO_TYPE_MD5 then
            v.auto_type = "MD5"
        end
    
        dbg.print_tty(string.format("%-15s", v.interface) .. string.format("%-18s", v.ip_mask) .. string.format("%-12s", v.working)
                .. string.format("%-18s", v.priority) .. string.format("%-20s", v.retransmit) .. string.format("%-15s", v.hello)
                .. string.format("%-15s", v.dead) .. string.format("%-16s", v.transmit) .. string.format("%-10s", v.cost)
                .. string.format("%-16s", v.network_type) .. string.format("%-16s", v.passive) .. string.format("%-16s", v.mtu_ignore)
                .. string.format("%-20s", v.auto_type))
    end

    return "SUCCESS"
end

--show network
function set_show_network()
    local tmp_config = {}

    uci_r:set_confdir(CFG_PATH) 
    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end
	dbg.print_tty(string.format("%-20s", "IP_Address") .. string.format("%-20s", "Wildcard Mask") .. string.format("%-20s", "Area ID"))
	uci_r:foreach("ospf", "network", 
    function(section)
		local name = section[".name"]
		tmp_config.sys_ip = uci_r:get("ospf", name, "ip")
		tmp_config.sys_mask = uci_r:get("ospf", name, "wildcard_mask")
		tmp_config.sys_area_id = uci_r:get("ospf", name, "area_id")


		dbg.print_tty(string.format("%-20s", tmp_config.sys_ip) .. string.format("%-20s", tmp_config.sys_mask) 
        .. string.format("%-15s", tmp_config.sys_area_id))
	end
	)
    return "SUCCESS"
end

--network
local function set_network(method, ip_address,mask)
    local result = "SUCCESS"
    local http_form = {}
    local tmp_config = {}
    local index = 0

    uci_r:set_confdir(CFG_PATH) 

    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled")
        return "FAILED"
    end

    --对添加network时的特殊ip做限制
    local a1, a2, a3, a4 = ip_address:match("^(%d+)%.(%d+)%.(%d+)%.(%d+)$")
    if ip_address == ALL_ZERO_ADD then 
        dbg.print_tty("Error: The IP address cannot be 0.0.0.0.")
        return "FAILED"
    end
    if tonumber(a1) == LOOP_ADD_PREFIX then
        dbg.print_tty("Error: The IP address cannot be a loopback address.")
        return "FAILED"
    end
    if tonumber(a1) >= MULTICAST_ADD_PREFIX then
        dbg.print_tty("Error: The IP address cannot be a class D address.")
        return "FAILED"
    end

    uci_r:foreach("ospf", "network",
    function(section)
        index = index + 1
    end
    )
    http_form.form = "network"
    http_form.data = json.encode(
    {
        ["method"] = method,
        ["params"] = 
        {   
            ["index"] = tostring(index),
            ["old"] = method,
            ["new"] = 
            {
                ["ip"] = ip_address,
                ["wildcard_mask"] = mask,

            },
            ["key"] = method,
        }
    })
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        result = "FAILED"
    end

    return result
end

local function set_no_network(method, ip_address)
    local result = "SUCCESS"
    local http_form = {}
    local tmp_config = {}
    local index = 0
    local delete_index = -1

    uci_r:set_confdir(CFG_PATH) 

    tmp_config.sys_status = uci_r:get("ospf", "global", "status")
    if tmp_config.sys_status == "off" then
        dbg.print_tty("Error: OSPF is not enabled.")
        return "FAILED"
    end

    local a1, a2, a3, a4 = ip_address:match("^(%d+)%.(%d+)%.(%d+)%.(%d+)$")
    uci_r:foreach("ospf", "network",
    function(section)
        local b1, b2, b3, b4 = section.ip:match("^(%d+)%.(%d+)%.(%d+)%.(%d+)$")
        if a1 == b1 and a2 == b2 and a3 == b3 then
            delete_index = index
        end
        index = index + 1
    end
    )
    if delete_index == -1 then
        dbg.print_tty("Error: Please input correct ip to delete, the network table is shown below.")
        set_show_network()
        return "FAILED"
    end
    http_form.form = "network"
    http_form.data = json.encode(
    {
        ["method"] = method,
        ["params"] = 
        {   
            ["index"] = tostring(delete_index),
        }
    })
    ret = ospf.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        result = "FAILED"
    end

    return result
end



--脚本总入口
--global variable

--ospf {enable|disable}
if arg[1] == "router ospf" then
    local enabled = "enable"
    result = set_enabled_ospf(enabled)
elseif arg[1] == "no router ospf" then
    local enabled = "disable"
    result = set_enabled_ospf(enabled)
--router-id {routerid}
elseif arg[1] == "router-id" then
    local auto_enabled = "off"
    local routerid = arg[2]
    result = set_router_id(auto_enabled, routerid)
elseif arg[1] == "no router-id" then
    local auto_enabled = "on"
    local routerid = nil
    result = set_router_id(auto_enabled, routerid)
--area-id {areaid}
elseif arg[1] == "area-id" then
    local areaid = arg[2]
    result = set_area_id(areaid)
--default-information originate
elseif arg[1] == "default-information originate enable" then

    result = set_default_information_originate()
elseif arg[1] == "no default-information originate" then

    result = set_no_default_information_originate()
elseif arg[1] == "default-information originate always" then
    local always = arg[2]
    result = set_default_information_originate_always(always)
elseif arg[1] == "default-information originate metric" then
    local metric_cost = arg[2]
    result = set_default_information_originate_metric(metric_cost)
elseif arg[1] == "default-information originate metric-type" then
    local metric_type = arg[2]
    result = set_default_information_originate_metric_type(metric_type)
--compatible rfc1583
elseif arg[1] == "compatible rfc1583" then
    result = set_compatible_rfc1583()
--no compatible rfc1583
elseif arg[1] == "no compatible rfc1583" then
    result = set_no_compatible_rfc1583()
--distance
elseif arg[1] == "ospf-distance" then
    local distance = arg[2]
    result = set_distance(distance)
--timers throttle spf{spf_delaytime, spf_init_holdtime, spf_max_holdtime}
elseif arg[1] == "timers throttle spf" then
    local spf_delaytime = arg[2]
    local spf_init_holdtime = arg[3]
    local spf_max_holdtime = arg[4]
    result = set_spf_times6(spf_delaytime, spf_init_holdtime, spf_max_holdtime)
--maximum-paths
elseif arg[1] == "maximum-paths" then
    local max_paths = arg[2]
    result = set_max_path(max_paths)
--passive-interface default
elseif arg[1] == "passive-interface default" then
    local enabled = "enable"
    result = set_passive_def(enabled)
elseif arg[1] == "no passive-interface default" then
    local enabled = "disable"
    result = set_passive_def(enabled)
--ip ospf priority
elseif arg[1] == "ip ospf priority" then
    local pri = arg[2]
    OSPF_INTERFACE = arg[3]
    result = set_ip_ospf_priority(pri)
--ip ospf hello-interval
elseif arg[1] == "ip ospf hello-interval" then
    local hello_interval = arg[2]
    OSPF_INTERFACE = arg[3]
    result = set_ip_ospf_hello_interval(hello_interval)
--ip ospf dead-interval
elseif arg[1] == "ip ospf dead-interval" then
    local dead_interval = arg[2]
    OSPF_INTERFACE = arg[3]
    result = set_ip_ospf_dead_interval(dead_interval)
--ip ospf transmit-delay
elseif arg[1] == "ip ospf transmit-delay" then
    local transmit_delay = arg[2]
    OSPF_INTERFACE = arg[3]
    result = set_ip_ospf_transmit_delay(transmit_delay)
--ip ospf retransmit-interval
elseif arg[1] == "ip ospf retransmit-interval" then
    local retransmit_interval = arg[2]
    OSPF_INTERFACE = arg[3]
    result = set_ip_ospf_retransmit_interval(retransmit_interval)
--ip ospf cost
elseif arg[1] == "ip ospf cost" then
    local cost = arg[2]
    OSPF_INTERFACE = arg[3]
    result = set_ip_ospf_cost(cost)
--ip ospf network
elseif arg[1] == "ip ospf network" then
    local network = arg[2]
    OSPF_INTERFACE = arg[3]
    if network == "Broadcast" or network == "point-to-point" then
        result = set_ip_ospf_network(network)
    else
        dbg.print_tty("Error: The parameter is not allowed.")
        dbg.print_tty("Network type,ex: Broadcast or point-to-point.")
    end
--ip ospf passive
elseif arg[1] == "ip ospf passive" then
    local passive = arg[2]
    OSPF_INTERFACE = arg[3]
    if passive == "enable" or passive == "disable" then
        result = set_ip_ospf_passive(passive)
    else
        dbg.print_tty("Error: The parameter is not allowed.")
        dbg.print_tty("Interface passive-mode,ex: enable or disable.")
    end
--ip ospf mtu-ignore
elseif arg[1] == "ip ospf mtu-ignore" then
    local  mtu_ignore = arg[2]
    OSPF_INTERFACE = arg[3]
    if mtu_ignore == "enable" or mtu_ignore == "disable" then
        result = set_ip_ospf_mtu_ignore(mtu_ignore)
    else
        dbg.print_tty("Error: The parameter is not allowed.")
        dbg.print_tty("Interface mtu-ignore,ex: enable or disable.")
    end
--ip ospf authentication
elseif arg[1] == "ip ospf authentication None" then
    OSPF_INTERFACE = arg[2]
    result = set_ip_ospf_authentication_None()
elseif arg[1] == "ip ospf authentication Simple" then
    local simple_key = arg[2]
    OSPF_INTERFACE = arg[3]
    if string.len(simple_key) < 9 and string.len(simple_key) > 0 then 
        result = set_ip_ospf_authentication_simple(simple_key)
    else
        dbg.print_tty("Error: The parameter is not allowed.")
        dbg.print_tty("Simple key of this interface, a combination of 1-8 digits and characters ex: 12345.")
    end
elseif arg[1] == "ip ospf authentication MD5" then
    local md5_key_id = arg[2]
    local md5_key = arg[3]
    OSPF_INTERFACE = arg[4]
    if string.len(md5_key) < 9 and string.len(md5_key) > 0 then
        result = set_ip_ospf_authentication_md5(md5_key_id, md5_key)
    else
        dbg.print_tty("Error: The parameter is not allowed.")
        dbg.print_tty("MD5 key of this interface, a combination of 1-8 digits and characters ex: 12345.")
    end
--show ip ospf database
elseif arg[1] == "show ip ospf database" then
    result = set_show_ospf_datebase()
--show ip ospf neighbor
elseif arg[1] == "show ip ospf neighbor" then
    result = set_show_ospf_neighbor()
--show ip ospf
elseif arg[1] == "show ip ospf" then
    result = set_show_ospf()
--show ip ospf interface
elseif arg[1] == "show ip ospf interface" then
    result = set_show_ospf_interface()
--show network
elseif arg[1] == "show network" then
    result = set_show_network()
--network
elseif arg[1] == "ospf-network" then
    local ip_address = arg[2]
    local mask = arg[3]
    local method = "add"
    result = set_network(method, ip_address, mask)
elseif arg[1] == "no ospf-network" then
    local ip_address = arg[2]
    local method = "delete"
    result = set_no_network(method, ip_address)
end

--io.stdout:write(result)





