local uci = require "luci.model.uci"
local sys = require "luci.sys"
local form = require "luci.tools.form"
local dbg = require "luci.tools.debug"
local json = require 'luci.json'
local utl = require "luci.util"
local dhcps = require "luci.controller.admin.dhcps"
local error_print = require "luci.tools.error_print"
local dtype = require "luci.tools.datatypes"

local uci_r = uci.cursor()

local CLI_DHCP_CFGPATH =  "/tmp/cli_config/cli_dhcp"

local CLI_CFG_PATH = "/tmp/cli_config"
local CFG_PATH = "/etc/config"

local DHCP_SUCCESS_STATUS = "SUCCESS"
local DHCP_FAIL_STATUS = "FAILED"

local filename = CLI_DHCP_CFGPATH 
local file = io.open(filename,"r")
if file == nil then
    sys.exec("touch " .. filename)
else
    file:close()
end

function tablecopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local newObject = {}
        lookup_table[object] = newObject
        for key, value in pairs(object) do
            newObject[_copy(key)] = _copy(value)
        end
        return setmetatable(newObject, getmetatable(object))
    end
    return _copy(object)
end

function get_client()
    local http_form = {}
    local ret
    http_form.data = json.encode(
        {
            ["method"] = "get",
            ["params"] = {}
        }
    )
    http_form.form = "client"
    ret = dhcps.dhcp_dispatch(http_form)

    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
 	end

    return ret
end

function dhcp_enable()
	local http_form={}
	local ret = get_config()
	local network={}
	local all_dhcp = ret.result 
	http_form.form = "lan"
	local count = table.getn(all_dhcp)
	if count == 0 then
		all_dhcp = {all_dhcp}
		count = 1
	end
	for i=1,count
	do
		network = all_dhcp[i]
		local new = tablecopy(network)
		new.enable = "on"
		new.dhcp_mode = "dhcp_server"
		new.relay_enable = "off"
		http_form.data = json.encode(
			{
				["method"] = "set",
				["params"] = 
				{
					["old"] = network,
					["new"] = new,
					["vlanid"] = network.vlanid
				}
			}
		)
		ret = dhcps.dhcp_dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
        	dbg.print_tty(error_print.check_error(ret))
        	return "FAILED"
 		end
	end
    return DHCP_SUCCESS_STATUS
end  
 
function dhcp_disable()
    local http_form={}
    local ret = get_config()
    local all_dhcp = ret.result 
    local old
    local count = table.getn(all_dhcp)
    dbg(type(all_dhcp))
    dbg(count)
    dbg.dumptable(all_dhcp)
    http_form.form = "lan"
    if count == 0 then
        all_dhcp = {all_dhcp}
        dbg.dumptable(all_dhcp)
        count = 1
    end
    for i=1,count do
        old = all_dhcp[i]
        local new = tablecopy(old)
        new.enable = "off"
        http_form.data = json.encode(
            {
                ["method"] = "set",
                ["params"] = 
                {
                    ["old"] = old,
                    ["new"] = new,
                    ["vlanid"] = old.vlanid
                }
            }
        )
        ret = dhcps.dhcp_dispatch(http_form)
        if (not ret) or (ret.error_code ~= "0") then
        	dbg.print_tty(error_print.check_error(ret))
        	return "FAILED"
 		end
    end
    return DHCP_SUCCESS_STATUS
end 

--@0 not found
--@1 found in /tmp/cli_config/dhcp
--@2 found in /etc/config/vnetwork

function show_dhcp_relay()

    local http_form={}
    local ret = get_config()
    local all_dhcp = ret.result 
    local count = table.getn(all_dhcp)
    http_form.form = "lan"
    if count == 0 then
        all_dhcp = {all_dhcp}
        count = 1
    end
    for i=1,count
    do
		local lan_name = all_dhcp[i].name
		dbg.print_tty("---------------------------------------------------------------------")
        dbg.print_tty("name:"..lan_name)
        local relay_enable_config = all_dhcp[i].relay_enable
        if relay_enable_config == "off" then
            dbg.print_tty("\t".."relay_enable:"..relay_enable_config)
        end

        if relay_enable_config == "on" then
            local relay_server_ipaddr_config = all_dhcp[i].relay_server_ipaddr
            dbg.print_tty("\t".."relay_enable:"..relay_enable_config)
            dbg.print_tty("\t".."relay_server_ipaddr:"..relay_server_ipaddr_config)
        end
	end
	dbg.print_tty("---------------------------------------------------------------------")
    return DHCP_SUCCESS_STATUS
end

function show_ip_dhcp_server_pool()

    local http_form={}
    local ret = get_config()
    local all_dhcp = ret.result 
    local count = table.getn(all_dhcp)
    http_form.form = "lan"
    if count == 0 then
        all_dhcp = {all_dhcp}
        count = 1
    end
    for i=1,count
    do
        local lan_name = all_dhcp[i].name
        local lan_ipaddr_start = all_dhcp[i].ipaddr_start
        local lan_ipaddr_end = all_dhcp[i].ipaddr_end
        local lan_leasetime = all_dhcp[i].leasetime
        local lan_dhcp_mode = all_dhcp[i].dhcp_mode or ""
		local lan_enable = all_dhcp[i].enable
	
		if lan_dhcp_mode == "dhcp_server" or "" then
			lan_dhcp_mode="dhcp_server"
			dbg.print_tty("---------------------------------------------------------------------")
            dbg.print_tty("name:"..lan_name)
            dbg.print_tty("\t".."status dhcp_mode:"..lan_dhcp_mode)
            dbg.print_tty("\t".."status enable:"..lan_enable)
            dbg.print_tty("\t".."status ipaddr_start:"..lan_ipaddr_start)
            dbg.print_tty("\t".."status ipaddr_end:"..lan_ipaddr_end)
            dbg.print_tty("\t".."status leasetime:"..lan_leasetime)
		else
			dbg.print_tty("---------------------------------------------------------------------")
            dbg.print_tty("name: "..lan_name)
            dbg.print_tty("\t".."status dhcp_mode:"..lan_dhcp_mode)
            dbg.print_tty("\t".."status enable:"..lan_enable)
        end
	end
	dbg.print_tty("---------------------------------------------------------------------")
    return DHCP_SUCCESS_STATUS
end

function show_ip_dhcp_server_status()
    local http_form={}
    local ret = get_config()
    local all_dhcp = ret.result 
    local count = table.getn(all_dhcp)
    http_form.form = "lan"
    if count == 0 then
        all_dhcp = {all_dhcp}
        count = 1
    end
    for i=1,count
    do
        local lan_name = all_dhcp[i].name
		local lan_dhcp_enable = all_dhcp[i].enable
		dbg.print_tty("---------------------------------------------------------------------")
        dbg.print_tty("name:"..lan_name)
        if lan_dhcp_enable == "on" then
            dbg.print_tty("\t".."enable:"..lan_dhcp_enable)
            local lan_status = "DHCP server is enable"
            dbg.print_tty("\t"..lan_status)
        end
        if lan_dhcp_enable == "off" then
            dbg.print_tty("\t".."enable:"..lan_dhcp_enable)
            local lan_status = "DHCP server is disable"
            dbg.print_tty("\t"..lan_status)
		end
	end
	dbg.print_tty("---------------------------------------------------------------------")
    return DHCP_SUCCESS_STATUS
end

function show_dhcp_server_client_list()
    local http_form_client={}
    local ret_client = get_client()
    local all_client = ret_client.result 
    http_form_client.form = "client"
    local count = table.getn(all_client)
    if count ~= 0 then
        for _, network in ipairs(all_client)
        do
            local client_bind = network.bind or "0"
            local client_interface = network.interface or ""
            local client_name = network.name or ""
            local client_mac_addr = network.macaddr or ""
            local client_assigned_ip_addr = network.ipaddr or ""
			local client_leasetime = network.leasetime or ""
			dbg.print_tty("---------------------------------------------------------------------")
            dbg.print_tty("client name:"..client_name)
            dbg.print_tty("\t".."bind:"..client_bind)
            dbg.print_tty("\t".."interface:"..client_interface)
            dbg.print_tty("\t".."macaddr:"..client_mac_addr)
            dbg.print_tty("\t".."ipaddr:"..client_assigned_ip_addr)
            dbg.print_tty("\t".."leasetime:"..client_leasetime)
		end
		dbg.print_tty("---------------------------------------------------------------------")
    else
        dbg.print_tty("no client list")
    end
    return DHCP_SUCCESS_STATUS
end

function show_dhcp_server()
    local http_form={}
    local ret = get_config()
    local all_dhcp = ret.result 
    local count = table.getn(all_dhcp)
    http_form.form = "lan"
    if count == 0 then
        all_dhcp = {all_dhcp}
        count = 1
    end
	local dhcp_server_web_cfg = {}

    for i=1,count
    do
        dhcp_server_web_cfg = {
            {["name"] = all_dhcp[i].name or ""},
            {["ipaddr"] = all_dhcp[i].ipaddr or ""},
            {["netmask"] = all_dhcp[i].netmask or ""},
            {["dhcp_mode"] = all_dhcp[i].dhcp_mode or ""},
            {["enable"] = all_dhcp[i].enable or ""},
            {["relay_enable"] = all_dhcp[i].relay_enable or ""},
            {["vlan_id"] = all_dhcp[i].vlanid or ""},
            {["ipaddr_start"] = all_dhcp[i].ipaddr_start or ""},
            {["ipaddr_end"] = all_dhcp[i].ipaddr_end or ""},
            {["leasetime"] = all_dhcp[i].leasetime or ""},
            {["gateway"] = all_dhcp[i].gateway or ""},
            {["domain"] = all_dhcp[i].domain or ""},
            {["option60"] = all_dhcp[i].option60 or ""},
            {["option66"] = all_dhcp[i].option66 or ""},
            {["option67"] = all_dhcp[i].option67 or ""},
            {["option138"] = all_dhcp[i].option138 or ""},
            {["option150"] = all_dhcp[i].option150 or ""},
            {["option159"] = all_dhcp[i].option159 or ""},
            {["option160"] = all_dhcp[i].option160 or ""},
            {["option176"] = all_dhcp[i].option176 or ""},
            {["option242"] = all_dhcp[i].option242 or ""},
            {["pri_dns"] = all_dhcp[i].pri_dns or ""},
            {["snd_dns"] = all_dhcp[i].snd_dns or ""}
		}
		dbg.print_tty("---------------------------------------------------------------------")
        local prompted_msg = "DHCP server configuration information"
        dbg.print_tty(prompted_msg..":"..all_dhcp[i].name)
        
        table.foreach(dhcp_server_web_cfg, 
            function(i, v)
                table.foreach(v, 
                    function(j, w)
                        if w ~= "" then
                            dbg.print_tty("\t".."option "..j.." "..w)
                        end
                    end
                )  
            end
		)
	end
	dbg.print_tty("---------------------------------------------------------------------")
    -- 设置UCI配置为当前目录
    local tmp_cli_dhcp_config = {}
    uci_r:set_confdir(CLI_CFG_PATH)
    local line = 0
    local section_name = {}
    local filename = "cli_dhcp"
    local cfg_dhcp_paras_name ="dhcp_paras"
    uci_r:foreach(filename, cfg_dhcp_paras_name,
        function(section)
            section_name[line] = section[".name"]
            line = line+1
        end
    )
    for i = 0, #section_name do
        if section_name[i] ~= nil then
            tmp_cli_dhcp_config = {
                {["name"] = uci_r:get(filename, section_name[i], "name") or ""},
                {["ipaddr"] = uci_r:get(filename, section_name[i], "ipaddr") or ""},
                {["netmask"] = uci_r:get(filename, section_name[i], "netmask") or ""},
                {["lease_time"] = uci_r:get(filename, section_name[i], "lease_time") or ""},
                {["default_gateway"] = uci_r:get(filename, section_name[i], "default_gateway") or ""},
                {["domain_name"] = uci_r:get(filename, section_name[i], "domain_name") or ""},
                {["option60"] = uci_r:get(filename, section_name[i], "option60") or ""},
                {["option66"] = uci_r:get(filename, section_name[i], "option66") or ""},
                {["option67"] = uci_r:get(filename, section_name[i], "option67") or ""},
                {["option138"] = uci_r:get(filename, section_name[i], "option138") or ""},
                {["option150"] = uci_r:get(filename, section_name[i], "option150") or ""},
                {["option159"] = uci_r:get(filename, section_name[i], "option159") or ""},
                {["option160"] = uci_r:get(filename, section_name[i], "option160") or ""},
                {["option176"] = uci_r:get(filename, section_name[i], "option176") or ""},
                {["option242"] = uci_r:get(filename, section_name[i], "option242") or ""},
                {["primary_dns"] = uci_r:get(filename, section_name[i], "primary_dns") or ""},
                {["secondary_dns"] = uci_r:get(filename, section_name[i], "secondary_dns") or ""},
                {["vlanid"] = uci_r:get(filename, section_name[i], "vlanid") or ""}
			}
            local prompted_msg_cfg = "Cache DHCP server configuration information"
            dbg.print_tty(prompted_msg_cfg..":"..section_name[i])
            table.foreach(tmp_cli_dhcp_config, 
                function(i, v)
                    table.foreach(v, 
                        function(j, w)
                            if w ~= "" then
                                dbg.print_tty("\t".."option "..j.." "..w)
                            end
                        end
                    )
                end
          )
        dbg.print_tty("---------------------------------------------------------------------")
        end   
	end
    return DHCP_SUCCESS_STATUS
end


function get_config()	--GUI
	uci_r:set_confdir(CFG_PATH) --设置路径
	local http_form = {}
	http_form.data = json.encode(	--获取参数
	{
		["method"] = "get",		--获取
		["params"] ={
		}
	})
	http_form.form = "lan"  
	local ret = dhcps.dhcp_dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
 	end
	return ret
end

function para_check(option, option_value)	--判断参数在哪个位置，0：网页端、缓存文件均不存在，1：存在在网页端，2：存在在缓存文件
	local flag = 0
	local exist_para = {}
	local ret = get_config()
	local i = 0
	if ret ~= "FAILED" then
		ret = ret.result
		if (#ret) == 0 then
			ret = {ret}
		end
		for key,value in ipairs(ret) do
			exist_para[i] = value[option]
			i = i + 1
		end	
	end
	for i = 0, #exist_para do	--判断输入的参数是否在网页端存�?
		if option_value == exist_para[i] then	--存在在网页端
			flag = 1
			--return flag
		end
	end
	
	uci_r:set_confdir(CLI_CFG_PATH)	--判断输入的参数是否在缓存文件存在
	uci_r:foreach("cli_dhcp", "dhcp_paras",
	function(section)
		if section[option] == option_value then
			if flag == 1 then
				flag = 3		--同时在网页端和缓存文�?
			else
				flag = 2		--只在缓存文件
			end
			return flag
		end
	end
	)
	return flag
end

function set_dhcp_parm(cmd, para)
	local Result = "SUCCESS"
	
	if para and not dtype.checktabletype(para) then
		return false
	end
	--对已存在的name和vlanid和ip进行获取
	local name_location = 0	--判断当前name是否存在,0不存在，1存在网页端，2存在在缓存文件中
	uci_r:set_confdir(CLI_CFG_PATH)
	if cmd == "ip dhcp server pool" then
		name_location = para_check("name",para["name"])	--获取参数位置
		if name_location == 1 then	--存在在网页端
			uci_r:set_confdir(CLI_CFG_PATH)
			uci_r:set("cli_dhcp", "pool_name", "pool_name")
			uci_r:set("cli_dhcp", "pool_name", "current_name", para["name"])
			uci_r:commit("cli_dhcp")
		end
		if name_location == 2 then
			uci_r:set_confdir(CLI_CFG_PATH)
			uci_r:set("cli_dhcp", "pool_name", "current_name", para["name"])	--current_name设置成当前name
			uci_r:commit("cli_dhcp")
		end
		if name_location == 0 then
			uci_r:set_confdir(CLI_CFG_PATH)
			uci_r:set("cli_dhcp", "pool_name", "pool_name")
			uci_r:set("cli_dhcp", "pool_name", "current_name", para["name"])
			uci_r:set("cli_dhcp", para["name"], "dhcp_paras")
			uci_r:set("cli_dhcp", para["name"], "name", para["name"])
			uci_r:commit("cli_dhcp")
		end
	else
		para["name"] = uci_r:get("cli_dhcp", "pool_name", "current_name") or "none"
		name_location = para_check("name",para["name"])	--获取参数位置
	end
	if name_location == 0 or name_location == 2 then		--name同时不存在或仅存在缓存文件?
		if cmd == "ip-mask" then
			local ipaddr_location = para_check("ipaddr", para["ipaddr"])
			if ipaddr_location ~= 0 then
					dbg.print_tty("Error: The IP address of the LAN port and that of the other WAN/LAN port cannot be in the same subnet.")
				return "FAILED"
			end
			local pool_name = uci_r:get("cli_dhcp", "pool_name", "current_name") or "none"
			uci_r:set("cli_dhcp", pool_name, "ipaddr", para["ipaddr"])
			uci_r:set("cli_dhcp", pool_name, "netmask", para["netmask"])
			uci_r:commit("cli_dhcp")
			if "true" == judge_parms(pool_name) then
				add_dhcp(pool_name)
				uci_r:set_confdir(CLI_CFG_PATH)	--删除一定要加个路径，不然不成功
				uci_r:delete("cli_dhcp", pool_name)	--添加清除缓存文件条目
				uci_r:commit("cli_dhcp")
			end
		end
		if cmd == "vlan-id" then
			local vlanid_location = para_check("vlanid", para["vlanid"])
			if vlanid_location ~= 0 then
				dbg.print_tty("Error: The current VLAN has been used. Please enter again.")
				return "FAILED"
			end
			local pool_name = uci_r:get("cli_dhcp", "pool_name", "current_name") or "none"
			uci_r:set("cli_dhcp", pool_name, "vlanid", para["vlanid"])
			uci_r:commit("cli_dhcp")
			if "true" == judge_parms(pool_name) then	--4个必要条件满足就新建最简单的lan
				add_dhcp(pool_name)
				uci_r:set_confdir(CLI_CFG_PATH)
				uci_r:delete("cli_dhcp", pool_name)	--添加清除缓存文件条目
				uci_r:commit("cli_dhcp")
			end
		end
		local pool_name = uci_r:get("cli_dhcp", "pool_name", "current_name") or "none"
		if cmd == "lease" then	--添加lease_time参数
			uci_r:set("cli_dhcp", pool_name, "lease_time", para["lease_time"])
		end
		
		if cmd == "default-gateway" then
			uci_r:set("cli_dhcp", pool_name, "default_gateway", para["default_gateway"])
		end
		if cmd == "no default-gateway" then
			uci_r:set("cli_dhcp", pool_name, "default_gateway", "")
		end
		
		if cmd == "domain-name" then
			uci_r:set("cli_dhcp", pool_name, "domain_name", para["domain_name"])
		end
		if cmd == "no domain-name" then
			uci_r:set("cli_dhcp", pool_name, "domain_name", "")
		end
		
		if cmd == "dns-server dns1" then
			uci_r:set("cli_dhcp", pool_name, "primary_dns", para["primary_dns"])
		end
		if cmd == "dns-server dns2" then
			uci_r:set("cli_dhcp", pool_name, "secondary_dns", para["secondary_dns"])
		end
		if cmd == "no dns-server" then
			uci_r:set("cli_dhcp", pool_name, "primary_dns", "")
			uci_r:set("cli_dhcp", pool_name, "secondary_dns", "")
		end
		if cmd == "option138 ip-address" then
			uci_r:set("cli_dhcp", pool_name, "option_no", "138")
			uci_r:set("cli_dhcp", pool_name, "option138", para["option_ip"])
		end
		if cmd == "option" then
			uci_r:set("cli_dhcp", pool_name, "option_no", para["option_no"])
			uci_r:set("cli_dhcp", pool_name, "option"..para["option_no"], para["option_value"])
		end

		if cmd == "no option" then
			uci_r:set("cli_dhcp", pool_name, "option"..para["option_no"], "")
		end
		
		if cmd == "ip dhcp relay helper-address" then
			uci_r:set("cli_dhcp", pool_name, "dhcp_mode", "dhcp_relay")
			uci_r:set("cli_dhcp", pool_name, "relay_enable", "on")
			uci_r:set("cli_dhcp", pool_name, "hp_addr", para["hp_addr"])
		end
		if cmd == "no ip dhcp relay helper-address" then
			uci_r:get("cli_dhcp", pool_name, "dhcp_status", "off")	--将dhcp关闭
			uci_r:set("cli_dhcp", pool_name, "hp_addr", para["hp_addr"])
		end
		
		uci_r:commit("cli_dhcp")
	end
	if name_location == 1 or name_location == 3 then		--name存在网页端就修改配置
		if cmd ~= "ip dhcp server pool" then
			Result = modified_dhcp(cmd, para)
		end
	end
	
	return Result
end

function judge_parms(current_name)	--判断是否满足必要�?个条�?
	uci_r:set_confdir(CLI_CFG_PATH)
	local ipaddr = uci_r:get("cli_dhcp", current_name, "ipaddr") or "none"
	local netmask = uci_r:get("cli_dhcp", current_name, "netmask") or "none"
	local vlanid = uci_r:get("cli_dhcp", current_name, "vlanid") or "none" 
	if ipaddr ~= "none" and netmask ~= "none" and vlanid ~= "none" then	--三个必要条件满足
		if para_check("ipaddr", ipaddr) == 3 then	--防止网页端修改?
			dbg.print_tty("Error: The IP address of the LAN port and that of the other WAN/LAN port cannot be in the same subnet.")
			return "false"
		end
		if para_check("vlanid", vlanid) == 3 then
			dbg.print_tty("Error: The current VLAN has been used. Please enter again.")
			return "false"
		end
		return "true"
	end
	return "false"
end

function add_dhcp(current_name)
	local Result = "SUCCESS"
	
	local ret = get_config()
	if ret ~= "FAILED" then
		ret = ret.result	--读取结果
	end
	local nums = #ret	--当前lan的个数?
	--dbg.print_tty(nums + 1)
	
	uci_r:set_confdir(CLI_CFG_PATH)
	local pool_name = current_name
	local addr = uci_r:get("cli_dhcp", pool_name, "ipaddr") or "none"
	local mask = uci_r:get("cli_dhcp", pool_name, "netmask") or "none"
	local vlan_id = uci_r:get("cli_dhcp", pool_name, "vlanid") or "none"
	
	local lease_time = uci_r:get("cli_dhcp", pool_name, "lease_time") or "120"
	local dhcp_status = uci_r:get("cli_dhcp", pool_name, "dhcp_status") or "on"
	local relay_enable = uci_r:get("cli_dhcp", pool_name, "relay_enable") or "off"
	local dhcp_mode = uci_r:get("cli_dhcp", pool_name, "dhcp_mode") or "dhcp_server"
	
	local default_gateway = uci_r:get("cli_dhcp", pool_name, "default_gateway") or ""
	local domain_name = uci_r:get("cli_dhcp", pool_name, "domain_name") or ""
	local primary_dns = uci_r:get("cli_dhcp", pool_name, "primary_dns") or ""
	local secondary_dns = uci_r:get("cli_dhcp", pool_name, "secondary_dns") or ""
	local option60 = uci_r:get("cli_dhcp", pool_name, "option60") or ""
	local option66 = uci_r:get("cli_dhcp", pool_name, "option66") or ""
	local option67 = uci_r:get("cli_dhcp", pool_name, "option67") or ""
	local option138 = uci_r:get("cli_dhcp", pool_name, "option138") or ""
	local option150 = uci_r:get("cli_dhcp", pool_name, "option150") or ""
	local option159 = uci_r:get("cli_dhcp", pool_name, "option159") or ""
	local option160 = uci_r:get("cli_dhcp", pool_name, "option160") or ""
	local option176 = uci_r:get("cli_dhcp", pool_name, "option176") or ""
	local option242 = uci_r:get("cli_dhcp", pool_name, "option242") or ""
	local hp_addr = uci_r:get("cli_dhcp", pool_name, "hp_addr") or ""
	
	http_form = {}	--tmp_macaddr
	if relay_enable == "off" then
		http_form.data = json.encode(	--新建一个最简单的dhcp server模式使能的lan
		{
			["method"] = "add",		
			["params"] ={
				["index"] = tonumber(nums + 1),
				["old"] = "add",
				["new"] = {
					["default"] = "0",
					["name"] = pool_name,
					["ipaddr"] = addr,
					["netmask"] = mask,
					["vlanid"] = vlan_id,
					["dhcp_mode"] = dhcp_mode,
					["enable"] = dhcp_status,
					["leasetime"] = lease_time,
					["gateway"] = default_gateway,
					["domain"] = domain_name,
					["pri_dns"] = primary_dns,
					["snd_dns"] = secondary_dns,
					["option60"] = option60,
					["option66"] = option66,
					["option67"] = option67,
					["option138"] = option138,
					["option150"] = option150,
					["option159"] = option159,
					["option160"] = option160,
					["option176"] = option176,
					["option242"] = option242,
					["relay_enable"] = relay_enable
				},
				["vlanid"] = "add"
			}
		})
	else
		http_form.data = json.encode(	--新建一个最简单的dhcp relay模式使能的lan
		{
			["method"] = "add",		
			["params"] ={
				["index"] = tonumber(nums + 1),
				["old"] = "add",
				["new"] = {
					["default"] = "0",
					["name"] = pool_name,
					["ipaddr"] = addr,
					["netmask"] = mask,
					["vlanid"] = vlan_id,
					["dhcp_mode"] = dhcp_mode,
					["enable"] = dhcp_status,
					["leasetime"] = "120",
					["gateway"] = "",
					["domain"] = "",
					["pri_dns"] = "",
					["snd_dns"] = "",
					["option60"] = "",
					["option66"] = "",
					["option67"] = "",
					["option138"] = "",
					["option150"] = "",
					["option159"] = "",
					["option160"] = "",
					["option176"] = "",
					["option242"] = "",
					["relay_enable"] = relay_enable,
					["relay_server_ipaddr"] = hp_addr
				},
				["vlanid"] = "add"
			}
		})
	end
	http_form.form = "lan"  					--1
	uci_r:set_confdir(CFG_PATH)
	ret = dhcps.dhcp_dispatch(http_form)	--新建lan
	if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty("Error:wrong vlan-id or ip-mask")
        return "FAILED"
	end
	return Result
end

function del_dhcp(current_name)
	local Result = "SUCCESS"
	local ret = get_config()
	if ret ~= "FAILED" then
		ret = ret.result	--读取结果
	end
	local key_lan = "none"
	local index_lan = 0
	local default = 0
	
	if (#ret) == 0 then
		ret = {ret}
	end
	for key,value in ipairs(ret) do
		index_lan = index_lan + 1
		if value.name == current_name then
			key_lan = value.vlanid
			default = value.default
			break
		end
	end	
	
	if 1 == default then	--默认lan判断，默认lan不能被删除?
		dbg.print_tty("Error: The default lan cannot be deleted.")
		return "FAILED"
	end
	
	http_form = {}
	http_form.data = json.encode(	--删除lan
	{
		["method"] = "delete",	--删除
		["params"] ={
			["key"] = key_lan,
			["index"] = tostring(index_lan)
		}
	})
	http_form.form = "lan" 
	
	ret = dhcps.dhcp_dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
	end
	return Result
end

function modified_dhcp(cmd, para)
	local Result = "SUCCESS"
	uci_r:set_confdir(CLI_CFG_PATH) --设置工作路径
	local pool_name = uci_r:get("cli_dhcp", "pool_name", "current_name") or "none"
	
	local tmp_index = 0
	local tmp_dhcp_mode, tmp_dhcp_mode_new = "", ""
	local tmp_ip, tmp_ip_new = "", ""
	
	local tmp_relay_ip, tmp_relay_ip_new = "", ""  --tmp_relay_en为on的时候才生效?
	
	local tmp_mask, tmp_mask_new = "", ""
	local tmp_vlanid, tmp_vlanid_new = "", ""
	local tmp_ip_start, tmp_ip_start_new = "", ""
	local tmp_maxleases, tmp_maxleases_new = "", ""
	local tmp_leasetime, tmp_leasetime_new = "", ""
	local tmp_name, tmp_name_new = "", ""
	local tmp_relay_en, tmp_relay_en_new = "", ""
	local tmp_ip_end, tmp_ip_end_new = "", ""
	local tmp_mode, tmp_mode_new = "", ""
	local tmp_enable, tmp_enable_new = "", ""
	local tmp_default, tmp_default_new = "", ""
	
	local tmp_def_gateway, tmp_def_gateway_new = "", ""
	local tmp_domainname, tmp_domainname_new = "", ""
	local tmp_primarydns, tmp_primarydns_new = "", ""
	local tmp_seconddns, tmp_seconddns_new = "", ""
	local option = {} 
	local option_new = {}
	
	local ret = get_config()
	if ret ~= "FAILED" then
		ret = ret.result	--读取结果
	end
	if (#ret) == 0 then
		ret = {ret}
	end
	for key,value in ipairs(ret) do
		if value.name == pool_name then
			tmp_dhcp_mode, tmp_dhcp_mode_new = value.dhcp_mode or "", value.dhcp_mode or ""
			tmp_ip, tmp_ip_new = value.ipaddr or "", value.ipaddr or ""
			tmp_relay_ip, tmp_relay_ip_new = value.relay_server_ipaddr or "", value.relay_server_ipaddr or ""
			--[[if "on" == value.relay_enable then
				tmp_relay_ip = value.relay_server_ipaddr
			end]]--
			tmp_mask, tmp_mask_new = value.netmask or "", value.netmask or ""
			tmp_vlanid, tmp_vlanid_new = value.vlanid or "", value.vlanid or ""
			tmp_ip_start, tmp_ip_start_new = value.ipaddr_start or "", value.ipaddr_start or ""
			tmp_maxleases, tmp_maxleases_new = value.maxleases or "", value.maxleases or ""
			tmp_leasetime, tmp_leasetime_new = value.leasetime or "", value.leasetime or ""
			tmp_name, tmp_name_new = value.name or "", value.name or ""
			tmp_relay_en, tmp_relay_en_new = value.relay_enable or "", value.relay_enable or ""
			tmp_ip_end, tmp_ip_end_new = value.ipaddr_end or "", value.ipaddr_end or ""
			tmp_mode, tmp_mode_new = value.mode or "", value.mode or ""
			tmp_enable, tmp_enable_new = value.enable or "", value.enable or ""
			tmp_default, tmp_default_new = value.default or "", value.default or ""
			
			tmp_def_gateway, tmp_def_gateway_new = value.gateway or "", value.gateway or ""
			tmp_domainname, tmp_domainname_new = value.domain or "", value.domain or ""
			tmp_primarydns, tmp_primarydns_new = value.pri_dns or "", value.pri_dns or ""
			tmp_seconddns, tmp_seconddns_new = value.snd_dns or "", value.snd_dns or ""
			option["option60"], option_new["option60"] = value.option60 or "", value.option60 or ""
			option["option66"], option_new["option66"] = value.option66 or "", value.option66 or ""
			option["option67"], option_new["option67"] = value.option67 or "", value.option67 or ""
			option["option138"], option_new["option138"] = value.option138 or "", value.option138 or ""
			option["option150"], option_new["option150"] = value.option150 or "", value.option150 or ""
			option["option159"], option_new["option159"] = value.option159 or "", value.option159 or ""
			option["option160"], option_new["option160"] = value.option160 or "", value.option160 or ""
			option["option176"], option_new["option176"] = value.option176 or "", value.option176 or ""
			option["option242"], option_new["option242"] = value.option242 or "", value.option242 or ""
			break
		end
		tmp_index = tmp_index + 1
	end
	
	if cmd == "ip-mask" then
		local ipaddr_location = para_check("ipaddr", para["ipaddr"])
		if ipaddr_location ~= 0 then
			dbg.print_tty("Error: The IP address of the LAN port and that of the other WAN/LAN port cannot be in the same subnet.")
			return "FAILED"
		end
		
		tmp_ip_new = para["ipaddr"]
		tmp_mask_new = para["netmask"]
	end
	if cmd == "vlan-id" then
		local vlanid_location = para_check("vlanid", para["vlanid"])
		if vlanid_location ~= 0 then
			dbg.print_tty("Error: The current VLAN has been used. Please enter again.")
			return "FAILED"
		end
		tmp_vlanid_new = para["vlanid"]
	end
	if tmp_relay_en == "" or tmp_relay_en == "off"  then  --dhcp_relay模式不能修改这些参数
		if cmd == "lease" then	--添加lease_time参数
			tmp_leasetime_new = para["lease_time"]
		end
		
		if cmd == "default-gateway" then
			tmp_def_gateway_new = para["default_gateway"]
		end
		if cmd == "no default-gateway" then
			tmp_def_gateway_new = ""
		end
		
		if cmd == "domain-name" then
			tmp_domainname_new = para["domain_name"]
		end
		if cmd == "no domain-name" then
			tmp_domainname_new = ""
		end
		
		if cmd == "dns-server dns1" then
			tmp_primarydns_new = para["primary_dns"]
		end
		if cmd == "dns-server dns2" then
			tmp_seconddns_new = para["secondary_dns"]
		end
		if cmd == "no dns-server" then
			tmp_primarydns_new = ""
			tmp_seconddns_new = ""
		end
		
		if cmd == "option138 ip-address" then
			local option_name = "option138"
			option_new[option_name] = para["option_ip"]
		end

		if cmd == "option" then
			local option_name = "option"..para["option_no"]
			option_new[option_name] = para["option_value"]
		end

		if cmd == "no option" then
			local option_name = "option"..para["option_no"]
			option_new[option_name] = ""
		end
	end
	
	if cmd == "ip dhcp relay helper-address" then
		tmp_dhcp_mode_new = "dhcp_relay"
		tmp_relay_en_new = "on"
		tmp_relay_ip_new = para["hp_addr"]
	end
	if cmd == "no ip dhcp relay helper-address" then
		tmp_relay_en_new = "off"
		tmp_relay_ip_new = ""
	end
	
	uci_r:set_confdir(CFG_PATH)
	http_form = {}
	http_form.data = json.encode(	
	{
		["method"] = "set",		--设置修改的参数
		["params"] ={
			["index"] = tmp_index,
			["old"] = {
				["default"] = tmp_default,
				["name"] = tmp_name,
				["ipaddr"] = tmp_ip,
				["netmask"] = tmp_mask,
				["enable"] = tmp_enable,
				["ipaddr_start"] = tmp_ip_start,
				["ipaddr_end"] = tmp_ip_end,
				["leasetime"] = tmp_leasetime,
				["gateway"] = tmp_def_gateway,
				["domain"] = tmp_domainname,
				["pri_dns"] = tmp_primarydns,
				["snd_dns"] = tmp_seconddns,
				["option60"] = option["option60"],
				["option66"] = option["option66"],
				["option67"] = option["option67"],
				["option138"] = option["option138"],
				["option150"] = option["option150"],
				["option159"] = option["option159"],
				["option160"] = option["option160"],
				["option176"] = option["option176"],
				["option242"] = option["option242"],
				["dhcp_mode"] = tmp_dhcp_mode,
				["relay_enable"] = tmp_relay_en,
				["relay_server_ipaddr"] = tmp_relay_ip,
				["vlanid"] = tmp_vlanid
			},
			["new"] = {
				["default"] = tmp_default_new,
				["name"] = tmp_name_new,
				["ipaddr"] = tmp_ip_new,
				["netmask"] = tmp_mask_new,
				["enable"] = tmp_enable_new,
				["vlanid"] = tmp_vlanid_new,
				["dhcp_mode"] = tmp_dhcp_mode_new,
				["enable"] = tmp_enable_new,
				["ipaddr_start"] = tmp_ip_start_new,
				["ipaddr_end"] = tmp_ip_end_new,
				["leasetime"] = tmp_leasetime_new,
				["gateway"] = tmp_def_gateway_new,
				["domain"] = tmp_domainname_new,
				["pri_dns"] = tmp_primarydns_new,
				["snd_dns"] = tmp_seconddns_new,
				["option60"] = option_new["option60"],
				["option66"] = option_new["option66"],
				["option67"] = option_new["option67"],
				["option138"] = option_new["option138"],
				["option150"] = option_new["option150"],
				["option159"] = option_new["option159"],
				["option160"] = option_new["option160"],
				["option176"] = option_new["option176"],
				["option242"] = option_new["option242"],
				["relay_enable"] = tmp_relay_en_new,
				["relay_server_ipaddr"] = tmp_relay_ip_new
			},
			["vlanid"] = tmp_vlanid
		}
	})
	http_form.form = "lan" 
	local ret = dhcps.dhcp_dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty("Error:wrong vlan-id or ip-mask")
        return "FAILED"
	end
	return Result
end

function check_option(option_no) --检查输入的option是否合格
	if 60 == tonumber(option_no) or 66 == tonumber(option_no) or 67 == tonumber(option_no) or
		138 == tonumber(option_no) or 150 == tonumber(option_no) or 159 == tonumber(option_no) or
		160 == tonumber(option_no) or 176 == tonumber(option_no) or 242 == tonumber(option_no) then
		return "true"
	end
	return "false"
end

function IPv4_addr_check(IPv4_address) --检查输入的ip范围是否正确
	if not IPv4_address then
		return false
	end

    if tostring(IPv4_address) == "0.0.0.0" then
        dbg.print_tty("Error: The address cannot be 0.0.0.0.")
        return false
    end
    if tostring(IPv4_address) == "255.255.255.255" then
        dbg.print_tty("Error: The address cannot be 255.255.255.255.")
        return false
    end

    local IPv4_netwk_num = string.match(tostring(IPv4_address), '%d+') --取ip第一段值
    if tonumber(IPv4_netwk_num) == 127 then
        dbg.print_tty("Error: The address cannot be a loopback address.")
        return false
    elseif tonumber(IPv4_netwk_num) >= 224 and tonumber(IPv4_netwk_num) <= 239 then
        dbg.print_tty("Error: The address cannot be a Class D address.")
        return false
    elseif tonumber(IPv4_netwk_num) >= 240 and tonumber(IPv4_netwk_num) <= 254 then
        dbg.print_tty("Error: The address cannot be a Class E address.")
        return false
    elseif tonumber(IPv4_netwk_num) == 255 then
        dbg.print_tty("Error: The address cannot start with 255.")
        return false
    end

    return true
end

function IPv4_addr_broadcast_check(IPv4_address, flag) --检查输入的ip是否为广播地址或网络地址
	if not IPv4_address then
		return false
	end
	local ipstring_end = nil
	for s in string.gmatch(IPv4_address, '%d+') do 
		ipstring_end = s
	end
	if tostring(ipstring_end) == "0" or tostring(ipstring_end) == "255" then
		if flag == 0 then
			dbg.print_tty("Error: Invaild IP address.")
		else
        	dbg.print_tty("Error: The entered IP address is a broadcast address or a network address.")
		end
		return false
    end
	return true
end


function Subnet_Mask_check(Subnet_Mask)
	if not Subnet_Mask then
		return false
	end
	local SubMask = tostring(Subnet_Mask)
	if SubMask =="255.0.0.0" or SubMask =="255.128.0.0" or SubMask =="255.192.0.0" or
		SubMask =="255.224.0.0" or SubMask =="255.240.0.0" or SubMask =="255.248.0.0" or
		SubMask =="255.252.0.0" or SubMask =="255.254.0.0" or SubMask =="255.255.0.0" or
		SubMask =="255.255.128.0" or SubMask =="255.255.192.0" or SubMask =="255.255.224.0" or
		SubMask =="255.255.240.0" or SubMask =="255.255.248.0" or SubMask =="255.255.252.0" or
		SubMask =="255.255.254.0" or SubMask =="255.255.255.0" or SubMask =="255.255.255.128" or
		SubMask =="255.255.255.192" or SubMask =="255.255.255.224" or SubMask =="255.255.255.240" or
		SubMask =="255.255.255.248" or SubMask =="255.255.255.252" then
		
		return true
	end
	return false
end

local Result = "SUCCESS"
local cmd = arg[1]
local para = {}

uci_r:set_confdir(CFG_PATH)
if arg[1] == "ip dhcp server pool" then
    para["name"] = arg[2]
	if para_check("name", para["name"]) == 3 then	--NAME在网页端，也在缓存文件中，不允许进shell再操作
		uci_r:set_confdir(CLI_CFG_PATH)	--删除一定要加个路径，不然不成功
		uci_r:delete("cli_dhcp", para["name"])	--添加清除缓存文件条目
		uci_r:commit("cli_dhcp")
	end
	Result = set_dhcp_parm(cmd, para)
elseif arg[1] == "no ip dhcp server pool" then
	local pool_name = arg[2]
	if para_check("name", pool_name) ~= 1 then	--NAME在网页端，也在缓存文件中，不允许进shell再操作
		dbg.print_tty("Error: Invalid pool name.")
		Result = "FAILED"
	else
		Result = del_dhcp(pool_name)
	end
elseif arg[1] == "ip-mask" then
	para["ipaddr"] = arg[2]
	para["netmask"] = arg[3]
	if not IPv4_addr_check(para["ipaddr"]) or not IPv4_addr_broadcast_check(para["ipaddr"], 0) then
		return "FAILED"
	elseif tostring(para["netmask"]) == "0.0.0.0" then
		dbg.print_tty("Error: Subnet mask cannot be 0.0.0.0.")
		return "FAILED"
	elseif tostring(para["netmask"]) == "255.255.255.255" then
		dbg.print_tty("Error: Subnet mask cannot be 255.255.255.255.")
		return "FAILED"
	elseif not Subnet_Mask_check(para["netmask"]) then
		dbg.print_tty("Error: Invaild subnet mask format.")
		return "FAILED"
	else
		Result = set_dhcp_parm(cmd, para)
	end
elseif arg[1] == "vlan-id" then
	para["vlanid"] = arg[2]
	Result = set_dhcp_parm(cmd, para)

elseif arg[1] == "lease" then
	para["lease_time"] = arg[2]
	if tonumber(para["lease_time"]) < 1 or tonumber(para["lease_time"]) > 2880 then
		dbg.print_tty("Error: Invalid number. Number should be between 1 and 2880.")
	else
		Result = set_dhcp_parm(cmd, para)
	end

elseif arg[1] == "default-gateway" then
if IPv4_addr_check(arg[2]) and IPv4_addr_broadcast_check(arg[2], 1) then
		para["default_gateway"] = arg[2]
		Result = set_dhcp_parm(cmd, para)
	end

elseif arg[1] == "no default-gateway" then
	Result = set_dhcp_parm(cmd, para)


elseif arg[1] == "domain-name" then
	para["domain_name"] = arg[2]
	Result = set_dhcp_parm(cmd, para)

elseif arg[1] == "no domain-name" then
	Result = set_dhcp_parm(cmd, para)

 
elseif arg[1] == "dns-server dns1" then
	if IPv4_addr_check(arg[2]) and IPv4_addr_broadcast_check(arg[2], 1) then
		para["primary_dns"] = arg[2]
		Result = set_dhcp_parm(cmd, para)
	end
elseif arg[1] == "dns-server dns2" then
	if IPv4_addr_check(arg[2]) and IPv4_addr_broadcast_check(arg[2], 1) then
		para["secondary_dns"] = arg[2]
		Result = set_dhcp_parm(cmd, para)
	end
elseif arg[1] == "no dns-server" then
	Result = set_dhcp_parm(cmd, para)


elseif arg[1] == "option138 ip-address" then
	para["option_ip"] = arg[2]
	if not IPv4_addr_check(para["option_ip"]) then
		return "FAILED"
	else
		Result = set_dhcp_parm(cmd, para)
	end

elseif arg[1] == "option" then
	para["option_no"] = arg[2]
	para["option_value"] = arg[3]
	if check_option(para["option_no"]) == "true" and tonumber(para["option_no"]) ~= 138 then
		Result = set_dhcp_parm(cmd, para)
	else
		dbg.print_tty("Error: Insert wrong option.")
	end

elseif arg[1] == "no option" then
	para["option_no"] = arg[2]
	if check_option(para["option_no"]) == "true" then
		Result = set_dhcp_parm(cmd, para)
	else
		dbg.print_tty("Error: Insert wrong option.")
	end

elseif arg[1] == "ip dhcp relay helper-address" then
	para["hp_addr"] = arg[2]
	Result = set_dhcp_parm(cmd, para)

elseif arg[1] == "no ip dhcp relay helper-address" then
	Result = set_dhcp_parm(cmd, para)
	
elseif arg[1] == "service dhcp server" then		--liu
	Result = dhcp_enable()
elseif arg[1] == "no service dhcp server" then
	Result = dhcp_disable()

elseif arg[1] == "show dhcp server" then
	Result = show_dhcp_server()

elseif arg[1] == "show dhcp relay" then
	Result = show_dhcp_relay()

elseif arg[1] == "show ip dhcp server pool"	then
	Result = show_ip_dhcp_server_pool()
	
elseif arg[1] == "show ip dhcp server status" then
	Result = show_ip_dhcp_server_status()

elseif arg[1] == "show dhcp server client-list" then
	Result = show_dhcp_server_client_list()

else
	dbg.print_tty("Error: Invalid command.")
end

io.stdout:write(Result)




