local sys  = require "luci.sys"
local json = require "luci.json"
local utl = require  "luci.util"
local dbg  = require "luci.tools.debug"
local form = require "luci.tools.form"
local error_print = require "luci.tools.error_print"
local uci  = require "luci.model.uci"
local administration = require "luci.controller.admin.administration"

local uci_r = uci.cursor()
local CFG_PATH = "/etc/config"

function set_http_settings(key_name, value)
    local result = "SUCCESS"
    local http_form = {}
    local input = {}
    local old_data = administration.management_get()

    uci_r:set_confdir(CFG_PATH)
    input.http_port      = old_data.http_port or ""
    input.redirect_https = old_data.redirect_https or "off"
    input.https_port     = old_data.https_port or ""
    input.enable_https   = old_data.enable_https or "off"
    input.web_timeout    = old_data.web_timeout or ""

    input[key_name] = value

    if input.http_port == input.https_port then
        if key_name == "http_port" then
            dbg.print_tty("Error: Conflict with the port number of HTTPS.")
        elseif key_name == "https_port" then
            dbg.print_tty("Error: Conflict with the port number of HTTP.")
        end

        result = "FAILED"
        return result
    end
    
    if key_name == "enable_https" and value == "off" and input.redirect_https == "on" then
        input.redirect_https = "off"
    end

    if key_name == "redirect_https" and value == "on" and input.enable_https == "off" then
        input.enable_https = "on"
    end

    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["http_port"] = input.http_port,
            ["redirect_https"] = input.redirect_https,
            ["https_port"] = input.https_port,
            ["enable_https"] = input.enable_https,
            ["web_timeout"] = input.web_timeout
        }
    })

    http_form.form = "management"
    ret = administration.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end

    return result
end

function show_http_settings()
    local old_data = administration.management_get()
    http_port      = old_data.http_port or ""
    redirect_https = old_data.redirect_https or "off"
    https_port     = old_data.https_port or ""
    enable_https   = old_data.enable_https or "off"
    web_timeout    = old_data.web_timeout or ""

    dbg.print_tty("http_port:                 "..http_port)
    dbg.print_tty("redirect_https:            "..redirect_https)
    dbg.print_tty("https_port:                "..https_port)
    dbg.print_tty("enable_https:              "..enable_https)
    dbg.print_tty("web_timeout:               "..web_timeout)

    return "SUCCESS"
end

--arg[1]为cli命令，其它为命令附带的参数
local result = "SUCCESS"
local cmd = arg[1]

if not cmd then
    return "FAILED"
end

if cmd == "ip http port" then
    local port = arg[2]
    if not port then
        return "FAILED"
    end

    result = set_http_settings("http_port", port)
elseif cmd == "ip http server redirect-to-https" then
    result = set_http_settings("redirect_https", "on")

elseif cmd == "no ip http server redirect-to-https" then
    result = set_http_settings("redirect_https", "off")

elseif cmd == "ip http secure-port" then
    local port = arg[2]
    if not port then
        return "FAILED"
    end

    result = set_http_settings("https_port", port)

elseif cmd == "ip http secure-server" then
    result = set_http_settings("enable_https", "on")

elseif cmd == "no ip http secure-server" then
    result = set_http_settings("enable_https", "off")

elseif cmd == "ip http session timeout" then
    local web_timeout = arg[2]
    if not web_timeout then
        return "FAILED"
    end

    result = set_http_settings("web_timeout", web_timeout)

elseif cmd == "show ip http configuration" then
    result = show_http_settings()
end

io.stdout:write(result)
