-- External reference definition
local sys  = require "luci.sys"
local json = require "luci.json"
local utl = require "luci.util"
local dbg  = require "luci.tools.debug"
local form = require "luci.tools.form"
local uci  = require "luci.model.uci"
local vlan_setting = require "luci.controller.admin.vlan_setting"
local vmod = require "luci.model.vlan"
local arplist = require "luci.controller.admin.arplist"
--缓存目录
-- Variable declaration
local CLI_CFG_PATH = "/tmp/cli_config"
local interface_config_file_name = CLI_CFG_PATH.."/interface_form.txt"
local port_config_file_name = CLI_CFG_PATH.."/port_form.txt"
local file = io.open(interface_config_file_name, "r")
local cmd = arg[1]
local uci_r = uci.cursor()
local CFG_PATH = "/etc/config"
local INTERFACE_SUCCESS="SUCCESS"
local error_print = require "luci.tools.error_print"
local INTERFACE_ERROR={
	[0] = "NO COMMAND INPUT",
	[1] = "NO SUCH COMMAND AT PRESENT",
	[2] = "INSUFFICIENT PARAMETERS",
	[3] = "TOO MUCH PARAMETERS",
	[4] = "FORMAT ERROR OF PARA",
	[5] = "GET CONFIG FAILED",
	[6] = "DUPLICATE NAME",
	[7] = "CRUD ERROR"
}
local line=""
local LINE_NUM=99
for i=1,LINE_NUM do
	line=line.."-"
end
-- Function declaration
function check_vlan_id(id)
    local result = "SUCCESS"
    local vconfigs = vmod.call_vlan("read_vlan_config", tostring(id))
    if vconfigs and not vconfigs.vlan then
        dbg.print_tty("Error: The vlan ID does not exist.")
        result = "FAILED"
        return result
    end
    local interface_vlan_config_file = io.open(interface_config_file_name, "r")
    if interface_vlan_config_file == nil then
        --文件不存在，则创建
        sys.exec("touch " .. interface_config_file_name)
    else
        interface_vlan_config_file:close()
    end
    --写入config
    local wr = assert(io.open(interface_config_file_name, "w"))
    vconfigs = json.encode(vconfigs)
    wr:write(vconfigs)
    wr:close()
    return result
end

function check_port_id(port) 
    local result = "SUCCESS"
    local port_http_form ={}
    local total = 0

    port_http_form.data = json.encode(
        {
            ["method"] = "get",
        }
    ) 
    port_http_form.form = "portsettinglist"
    dbg.dumptable(port_http_form)
    local ret = vlan_setting.dispatch(port_http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    for k,v in pairs(ret.result) do
        total = total + 1
        if v["port"] == port then
            local port_vlan_config_file = io.open(port_config_file_name, "r")
            if port_vlan_config_file == nil then
             --文件不存在，则创建
            sys.exec("touch " .. port_config_file_name)
            else
                port_vlan_config_file:close()
            end
            --写入config
            local wr = assert(io.open(port_config_file_name, "w"))
            pconfig = json.encode(v)
            wr:write(pconfig)
            wr:close()
            return result
        end
    end
    dbg.print_tty("Error: The port does not exist.")
    return "FAILED"
end

function get_vlans(name)
    local ret="0"
    uci_r:foreach("vnetwork","vinterface",
		function(section)
			if uci_r:get("vnetwork", section[".name"], "ifname") == name then
                ret = uci_r:get("vnetwork", section[".name"], "vlans")
			end
		end
	)
    return ret
end

function show_arp()
	http_form = {}
	http_form.form = "getlist"
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] = 
        	{
           
        	}
	})
	uci_r:set_confdir(CFG_PATH)
	local ret = arplist.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		return INTERFACE_ERROR[5]
	end
	local ret=ret.result
	dbg.print_tty(line)
	dbg.print_tty(string.format("%-14s","Interface")..string.format("%-16s","IP Address")..string.format("%-23s","MACAdress")       		..string.format("%-10s","Type")..string.format("%-10s","AGE"))
	
	for k1,v1 in pairs(ret) do
		dbg.print_tty(line)
		local vlan_id = utl.split(v1.interface, '.')[2] or get_vlans(v1.interface)
		v1.interface="vlan"..vlan_id.." "
		if tonumber(v1.status)==1 then
			v1.status="Local"
		end
		if tonumber(v1.status)==0 then
			v1.status="Dynamic"
		end	
		dbg.print_tty(string.format("%-14s",v1.interface)..string.format("%-16s",v1.ipaddr)..string.format("%-23s",v1.mac)
		..string.format("%-10s",v1.status)..string.format("%-10s","N/A"))						 
	end
	dbg.print_tty(line)
	return INTERFACE_SUCCESS
end

function set_desc(desc, no)
    local result = "SUCCESS"
    local http_form = {}
    local vconfigs = {}
    local new = 
    {
        ["name"] = "",
        ["if_onoff1"] = "0",
        
        ["if_onoff2"] = "0",
        ["tag_type2"] = "t",
        ["if_onoff3"] = "0",
        ["tag_type3"] = "t",
        ["if_onoff4"] = "0",
        ["tag_type4"] = "t",
        ["if_onoff5"] = "0",
        ["tag_type5"] = "t",
        ["desc"] = "",
    }
    local old = {}
    local interface_vlan_config_file = io.open(interface_config_file_name, "r")

    if interface_vlan_config_file == nil then
        result =  "FAILED"
        return result
    else
        interface_vlan_config_file:close()
    end

    local re = io.open(interface_config_file_name, "r")
    local readjson = re:read("*a")
    local read_config = json.decode(readjson)
    local vlanid = read_config["vlan"]
    vconfigs = vmod.call_vlan("read_vlan_config", vlanid)
    local ports_status = string.split(vconfigs["ports"]," ")

    new["if_onoff1"] = "0"
    new["name"] = vconfigs["name"]
    new["desc"] = desc
    for k, v in ipairs(ports_status) do
        local if_onoff = ""
        local tag_type = ""
        local port_num = string.match(v,"%d+")
        local port_tag = string.match(v,"%a")
        if port_num ~= nil then
            if_onoff = "if_onoff"..port_num
            new[if_onoff] = "1"
        end
        if port_tag ~= nil then
            tag_type = "tag_type"..port_num
            new[tag_type] = "t"
        else
            tag_type = "tag_type"..port_num
            new[tag_type] = "u"
        end
    end
    old =
    {
        ["name"] = vconfigs["name"],
        ["ports"] = vconfigs["ports"],
        ["desc"] = vconfigs["desc"],
        ["users"] = vconfigs["users"],
    }
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {

            ["index"] = "1",
            ["old"] = old,
            ["new"] = new,
            ["vlan"] = vconfigs["vlan"],
        }
    })
    http_form.form = "vlansettinglist"
    dbg.dumptable(http_form)
    local ret = vlan_setting.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    return result
end

    --创建缓存文件夹
if file == nil then
    sys.exec("touch " .. interface_config_file_name)
else
    file:close()
end
file = io.open(port_config_file_name, "r")
if file == nil then
    sys.exec("touch " .. interface_config_file_name)
else
    file:close()
end

if arg[1] == "interface vlan" then
    local vlanid = arg[2]
    result = check_vlan_id(vlanid)
elseif arg[1] == "interface switchport" then
    local port = arg[2]
    result = check_port_id(port)
elseif arg[1] == "description" then
    local desc = arg[2]
    result = set_desc(desc,"")
elseif arg[1] == "no description" then
    result = set_desc("","no")
elseif arg[1] == "show arp" then
    result = show_arp()	
end

io.stdout:write(result)

