-- External reference definition
local sys  = require "luci.sys"
local json = require "luci.json"
local interfaces = require "luci.model.interfaces"
local userconfig = require "luci.model.userconfig"
local lan  = require "luci.controller.admin.lan"
local IPPOOL = require "luci.controller.admin.ippool"
local utl = require "luci.util"
local dbg = require "luci.tools.debug"
local err = require "luci.tools.error"
local form = require "luci.tools.form"
local uci  = require "luci.model.uci"
local ippoolref = require "luci.model.reference"
local interface = require "luci.controller.admin.interface"
local nat = require "luci.controller.admin.nat" 
local error_print = require "luci.tools.error_print"
local dtype = require "luci.tools.datatypes"
-- Variable declaration
local CFG_PATH = "/etc/config/"
local CLI_CFG_PATH = "/tmp/cli_config"
local filename = CLI_CFG_PATH .."/nat_cli"
local uci_r = uci.cursor()
uci_r:set_confdir(CLI_CFG_PATH)
local file = io.open(filename,"r")
if file == nil then
    sys.exec("touch " .. filename)
else
    file:close()
end
local filename = CLI_CFG_PATH .."/nat_cli_one"
local uci_r = uci.cursor()
uci_r:set_confdir(CLI_CFG_PATH)
local file = io.open(filename,"r")
if file == nil then
    sys.exec("touch " .. filename)
else
    file:close()
end
local line=""
local LINE_NUM=99
for i=1,LINE_NUM do
	line=line.."-"
end
local change_flag
local NAT_ERROR={
	[0] = "Error: No command input.",
	[1] = "Error: No such command at present.",
	[2] = "Error: Get alg failed.",
	[3] = "Error: Insufficient parameters.",
	[4] = "Error: Too much parameters.",
	[5] = "Error: Add virtual server failed.",
	[6] = "Error: Para format error.",
	[7] = "Error: Name error in cache.",
	[8] = "Error: Number out of bounds.",
	[9] = "Error: Set nat server failed,the port or name may conflict.",
	[10] = "Error: Crud error."
}
local NAT_PROMPT={
	[0] = "PROMPT: This virtual name have exist,you can edit parameters information.",
	[1] = "PROMPT: Creat a virtual name success,you can add parameters now.",
	[2] = "PROMPT: Virtual server have submitted autoamtically.",
	[3] = "PROMPT: Please don't submit identical content.",
	[4] = "PROMPT: Nat one-to-one have submitted autoamtically.",
}
local NAT_SUCCESS="SUCCESS"

-- Function declaration
function print_lua_table(lua_table, indent)
       indent = indent or 0
       len = 0
       for k, v in pairs(lua_table) do
              if type(k) == "string" then
                     len = 23 - string.len(k)
                     k = string.format("%s", k)
              end
              local szSuffix = string.rep(" ", len)
              --local szPrefix = string.rep(" ", indent)
              if type(v) == "table" then
                     szSuffix = ":"
                     formatting = k..szSuffix
              else
                     formatting = k..":"..szSuffix
              end
              if type(v) == "table" then
                     dbg.print_tty("table "..formatting)
                     print_lua_table(v, indent + 1)
                     --dbg.dbg.print_tty_tty(szPrefix.."},")
              else
                     local szValue = ""
                     if type(v) == "string" then
                            szValue = string.format("%s", v)
                     else
                            szValue = tostring(v)
                     end
                     dbg.print_tty(formatting..szValue)
              end
       end
end

function show_nat_virtual_server()
	http_form = {}
	http_form.form = "vs"
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] = 
        	{
           
        	}
	})
	uci_r:set_confdir(CFG_PATH)
	local ret = nat.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(NAT_ERROR[2])
		return NAT_ERROR[2]
	end
	ret=ret.result
	dbg.print_tty(line)
	dbg.print_tty(string.format("%-7s","ID")..string.format("%-10s","Name")
	..string.format("%-16s","Interface")..string.format("%-16s","Exteranl Port")
	..string.format("%-16s","Internal Port")..string.format("%-23s","Internal Server Ip")
	..string.format("%-8s","Protocol"))
	for k, rt in ipairs(ret) do
		dbg.print_tty(line)
		if type(rt.interface) =="table" then
			local tmp="["
			for i=1,#(rt.interface) do
				tmp=tmp..(rt.interface)[i]
				if i~=#(rt.interface) then
					tmp=tmp..","
				end
			end
			tmp=tmp.."]"
			rt.interface=tmp
		end
		dbg.print_tty(string.format("%-7s",k)..string.format("%-10s",rt.name)
		..string.format("%-16s",rt.interface)..string.format("%-16s",rt.external_port)
		..string.format("%-16s",rt.internal_port)..string.format("%-23s",rt.ipaddr)
		..string.format("%-8s",rt.protocol))
	end
	dbg.print_tty(line)
	return NAT_SUCCESS
end

function get_nat_alg()
	http_form={}
	http_form.form = "alg"
	http_form.data = json.encode(
	{
		["method"] = "get",
	})
	uci_r:set_confdir(CFG_PATH)
	local ret = nat.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(NAT_ERROR[2])
		return NAT_ERROR[2]
	end
	return ret.result
end

function show_nat_alg()
	local ret=get_nat_alg()
	dbg.print_tty(line)
	dbg.print_tty("FTP ALG:\t\t"..ret.ftp)
	dbg.print_tty("H.323 ALG:\t\t"..ret.h323)
	dbg.print_tty("PPTP ALG:\t\t"..ret.pptp)
	dbg.print_tty("SIP ALG:\t\t"..ret.sip)
	dbg.print_tty("IPsec ALG:\t\t"..ret.ipsec)
	dbg.print_tty(line)
	return NAT_SUCCESS
end

function set_nat_alg(nat_data)
	for k,rt in pairs(nat_data) do
		if (rt~="on") and (rt~="off") then
			dbg.print_tty("Error: Please input correct para,on or off.")
			return NAT_ERROR[6]
		end
	end
	http_form={}
	http_form.form = "alg"
	http_form.data = json.encode(
	{
		["method"] = "set",
		["params"] =
		{
			["ftp"]=nat_data[1],
			["tftp"]="off",
			["h323"]=nat_data[2],
			["pptp"]=nat_data[3],
			["sip"]=nat_data[4],
			["ipsec"]=nat_data[5],
		}
	})
	uci_r:set_confdir(CFG_PATH)
	local ret = nat.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(NAT_ERROR[2])
		return NAT_ERROR[2]
    	end
	return NAT_SUCCESS
end

function nat_alg(alg_data)
	local ret = get_nat_alg()
	local nat_data
	if alg_data=="ftp" then
		nat_data={"on",ret.h323,ret.pptp,ret.sip,ret.ipsec}
	elseif alg_data=="h323" then
		nat_data={ret.ftp,"on",ret.pptp,ret.sip,ret.ipsec}
	elseif alg_data=="pptp" then
		nat_data={ret.ftp,ret.h323,"on",ret.sip,ret.ipsec}
	elseif alg_data=="sip" then
		nat_data={ret.ftp,ret.h323,ret.pptp,"on",ret.ipsec}
	elseif alg_data=="ipsec" then
		nat_data={ret.ftp,ret.h323,ret.pptp,ret.sip,"on"}
	else 
		dbg.print_tty("Error: Please input correct para,ftp h323 pptp sip or ipsec.")
		return NAT_ERROR[6]
	end
	set_nat_alg(nat_data)
	return NAT_SUCCESS
end
	
function no_nat_alg(no_alg_data)
	local ret = get_nat_alg()
	local nat_data
	if no_alg_data=="ftp" then
		nat_data={"off",ret.h323,ret.pptp,ret.sip,ret.ipsec}
	elseif no_alg_data=="h323" then
		nat_data={ret.ftp,"off",ret.pptp,ret.sip,ret.ipsec}
	elseif no_alg_data=="pptp" then
		nat_data={ret.ftp,ret.h323,"off",ret.sip,ret.ipsec}
	elseif no_alg_data=="sip" then
		nat_data={ret.ftp,ret.h323,ret.pptp,"off",ret.ipsec}
	elseif no_alg_data=="ipsec" then
		nat_data={ret.ftp,ret.h323,ret.pptp,ret.sip,"off"}
	else 
		dbg.print_tty("Error: Please input correct para,ftp h323 pptp sip or ipsec.")
		return NAT_ERROR[6]
	end
	set_nat_alg(nat_data)
	return NAT_SUCCESS
end

function check_name_virtual(selected_name)
	uci_r:set_confdir(CLI_CFG_PATH)
	local ret=uci_r:get("nat_cli",selected_name)
	if ret==nil then
		dbg.print_tty("Error: Please input the existing name.")
		return true 
	end
	return false
end

function check_parameters_virtual(selected_name)
	uci_r:set_confdir(CLI_CFG_PATH)
	local get1=uci_r:get("nat_cli",selected_name,"vlan_id") 
	local get2=uci_r:get("nat_cli",selected_name,"external_port")  
	local get3=uci_r:get("nat_cli",selected_name,"internal_port")  
	local get4=uci_r:get("nat_cli",selected_name,"server_ip") 
	local get5=uci_r:get("nat_cli",selected_name,"protocol")
	if get1 and get2 and get3 and get4 and get5 then
		return get1,get2,get3,get4,get5
	end
	return false
end

function find_index_of_section_virtual(selected_name)
	local index=-1
	local max_index=-1
	uci_r:set_confdir(CFG_PATH)
      uci_r:foreach("nat", "rule_vs", function (section)
		max_index=max_index+1
		if section.name == selected_name then
			index=max_index
		end
	end)
	return index,max_index
end

function nat_virtual_server(name)
	local index,max_index=find_index_of_section_virtual(name)
	if index>=0 then
		dbg.print_tty(NAT_PROMPT[0])
	else	
		dbg.print_tty(NAT_PROMPT[1])
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	if name and not dtype.safestring(name) then
		return false
	end
	uci_r:set("nat_cli","current_name","current_name")
	uci_r:set("nat_cli","current_name","current_name",name)
	uci_r:set("nat_cli",name,"rule_vs")
	uci_r:set("nat_cli",name,"name",name)
	uci_r:commit("nat_cli")
	return NAT_SUCCESS 
end

function check_vlan_id(vlan_id)
	uci_r:set_confdir(CFG_PATH)
	if vlan_id=="0" then
		local sfp = string.sub(uci_r:get_profile("phy1","pname"),1,3)
		if sfp=="SFP" then
			local sfp_wan_name="WAN"..uci_r:get_profile("phy1","pnumber")
			return sfp_wan_name
		else
			dbg.print_tty("Error: Vlan id can not be 0.")
			return false
		end
	end
	local dev_name = uci_r:get("network","global","wan_device")
	local ret=false
	uci_r:foreach("zone", "zone", function(section)
		if section.dev_eth ~= nil and string.match(section[".name"], "^WAN") then
			local vlan_id_find,find=string.gsub(section.dev_eth,dev_name..".","")
			if find==1 then
				if vlan_id_find==vlan_id then
					ret=section[".name"]
				end
			end
		end
	end)
	if not ret then
		dbg.print_tty("Error: Vlan id do not exist.")
	end
	return ret
end

function vlan_id_split(vlan_id)
	vlan_id=vlan_id..","
	local res={}
	local tmp=""
	for i=1,#vlan_id do
		if string.sub(vlan_id,i,i) ~="," then
			tmp=tmp..string.sub(vlan_id,i,i)
		else
			res[#res+1]=tmp
			tmp=""
		end
	end
	return res
end

function add_interface_name(selected_name,vlan_id)
	vlan_id=vlan_id_split(vlan_id)
	for i=1,#vlan_id do
		local vlan_id_change=check_vlan_id(vlan_id[i])
		if not vlan_id_change then
			return NAT_ERROR[10]
		end
		vlan_id[i]=vlan_id_change
	end

	if check_name_virtual(selected_name) then
		return NAT_ERROR[7]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	if vlanid and not dtype.uinteger(vlanid) then
		return false
	end
	uci_r:set("nat_cli",selected_name,"vlan_id",vlan_id)
	uci_r:commit("nat_cli")
	
	if check_parameters_virtual(selected_name) then
		local vlan_id,external_port,internal_port,server_ip,protocol=check_parameters_virtual(selected_name)
		local index,max_index=find_index_of_section_virtual(selected_name)
		if index>=0 then --selected_name exist
			change_flag=true
			add_virtual_server(index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		else
			change_flag=false
			add_virtual_server(max_index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		end
	end
	return NAT_SUCCESS 
end

function add_external_port(selected_name,external_port)
	if check_name_virtual(selected_name) then
		return NAT_ERROR[7]
	end
	if tonumber(external_port)<1 or tonumber(external_port)>65535 then
		dbg.print_tty("Error: The port is between 1~65535.")
		return NAT_ERROR[8]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	uci_r:set("nat_cli",selected_name,"external_port",external_port)
	uci_r:commit("nat_cli")
	
	if check_parameters_virtual(selected_name) then
		local vlan_id,external_port,internal_port,server_ip,protocol=check_parameters_virtual(selected_name)
		local index,max_index=find_index_of_section_virtual(selected_name)
		if index>=0 then 
			change_flag=true
			add_virtual_server(index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		else
			change_flag=false
			add_virtual_server(max_index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		end
	end
	return NAT_SUCCESS 
end

function add_internal_port(selected_name,internal_port)
	if tonumber(internal_port)<1 or tonumber(internal_port)>65535 then
		dbg.print_tty("Error: The port is between 1~65535.")
		return NAT_ERROR[8]
	end
	if check_name_virtual(selected_name) then
		return NAT_ERROR[7]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	uci_r:set("nat_cli",selected_name,"internal_port",internal_port)
	uci_r:commit("nat_cli")
	local vlan_id,external_port,internal_port,server_ip,protocol=check_parameters_virtual(selected_name)
	
	if check_parameters_virtual(selected_name) then
		local vlan_id,external_port,internal_port,server_ip,protocol=check_parameters_virtual(selected_name)
		local index,max_index=find_index_of_section_virtual(selected_name)
		if index>=0 then 
			change_flag=true
			add_virtual_server(index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		else
			change_flag=false
			add_virtual_server(max_index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		end
	end
	return NAT_SUCCESS 
end

function add_server_ip(selected_name,server_ip)
	if check_name_virtual(selected_name) then
		return NAT_ERROR[7]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	if server_ip and not dtype.ipaddr(server_ip) then
		return false
	end
	uci_r:set("nat_cli",selected_name,"server_ip",server_ip)
	uci_r:commit("nat_cli")
	local vlan_id,external_port,internal_port,server_ip,protocol=check_parameters_virtual(selected_name)
	
	if check_parameters_virtual(selected_name) then
		local vlan_id,external_port,internal_port,server_ip,protocol=check_parameters_virtual(selected_name)
		local index,max_index=find_index_of_section_virtual(selected_name)
		if index>=0 then 
			change_flag=true
			add_virtual_server(index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		else
			change_flag=false
			add_virtual_server(max_index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		end
	end
	return NAT_SUCCESS 
end

function add_protocol(selected_name,protocol)
	if protocol~="ALL" and protocol~="TCP" and protocol~="UDP" then
		dbg.print_tty("Error: Please input correct protocol,ALL TCP UDP.")
		return NAT_ERROR[6]
	end
	
	if check_name_virtual(selected_name) then
		return NAT_ERROR[7]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	uci_r:set("nat_cli",selected_name,"protocol",protocol)
	uci_r:commit("nat_cli")
	
	local vlan_id,external_port,internal_port,server_ip,protocol=check_parameters_virtual(selected_name)
	if check_parameters_virtual(selected_name) then
		local vlan_id,external_port,internal_port,server_ip,protocol=check_parameters_virtual(selected_name)
		local index,max_index=find_index_of_section_virtual(selected_name)
		if index>=0 then 
			change_flag=true
			add_virtual_server(index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		else
			change_flag=false
			add_virtual_server(max_index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
		end
	end
	return NAT_SUCCESS 
end

function add_virtual_server(index,selected_name,vlan_id,external_port,internal_port,server_ip,protocol)
	uci_r:set_confdir(CFG_PATH)
	http_form={}
	http_form.form="vs"
	if change_flag then
		local old = {}
		uci_r:set_confdir(CFG_PATH)
		uci_r:foreach("nat", "rule_vs", function (section)
			if section.name == selected_name then
				old=section			
			end
		end)
		http_form.data=json.encode(
		{
			["method"] = "set",
			["params"]=
			{
				["index"]=index,
				["old"]=
				{
					["name"]=old.name,
					["interface"]=old.interface,
					["external_port"]=old.external_port.."",
					["internal_port"]=old.internal_port.."",
					["ipaddr"]=old.ipaddr.."",
					["protocol"]=old.protocol.."",
					["enable"]=old.enable,
					["t_label"]=old.t_label,
				},
				["new"]=
				{
					["name"]=selected_name,
					["interface"]=vlan_id,
					["external_port"]=external_port.."",
					["internal_port"]=internal_port.."",
					["ipaddr"]=server_ip.."",
					["protocol"]=protocol.."",
					["enable"]="on",
				},
				["key"]="key-"..index
			}
		})
	else
		http_form.data=json.encode(
		{
			["method"] = "add",
			["params"]=
			{
				["index"]=index,
				["old"]="add",
				["new"]=
				{
					["name"]=selected_name,
					["interface"]=vlan_id,
					["external_port"]=external_port.."",
					["internal_port"]=internal_port.."",
					["ipaddr"]=server_ip.."",
					["protocol"]=protocol.."",
					["enable"]="on",
				},
				["key"]="add"
			}
		})
	end
	local ret = nat.dispatch(http_form)
	if (ret.error_code == "1003") then
		dbg.print_tty(NAT_PROMPT[3])
	end
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(NAT_ERROR[9])
		return NAT_ERROR[9]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	dbg.print_tty(NAT_PROMPT[2])
	return NAT_SUCCESS
end

function no_nat_virtual_server(selected_name)
	local index,max_index=find_index_of_section_virtual(selected_name)
	if index<0 then
		dbg.print_tty("Error: No such name.")
		return NAT_ERROR[6]
	end
	uci_r:set_confdir(CFG_PATH)
	http_form={}
	http_form.form="vs"
	http_form.data=json.encode(
	{
		["method"] = "delete",
		["params"]=
		{
			["key"]="key-"..index,
			["index"]=index.."",
		}
	})
	local ret = nat.dispatch(http_form)
	return NAT_SUCCESS
end

function show_nat_one_to_one()
	local result= NAT_SUCCESS
	http_form = {}
	http_form.form = "one_nat"
	http_form.data = json.encode(
 	{
		["method"] = "get",
		["params"] = 
        	{
           
        	}
	})
	uci_r:set_confdir(CFG_PATH)
	local ret = nat.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(NAT_ERROR[2])
		return NAT_ERROR[2]
	end
	ret=ret.result
	dbg.print_tty(line)
	dbg.print_tty(string.format("%-7s","ID")..string.format("%-10s","Name")
	..string.format("%-13s","Interface")..string.format("%-16s","Original IP")
	..string.format("%-16s","Translated Ip")..string.format("%-17s","DMZ Forwarding")
	..string.format("%-13s","Description")..string.format("%-6s","Status"))
	for k, rt in ipairs(ret) do
		dbg.print_tty(line)
		if type(rt.interface) =="table" then
			local tmp="["
			for i=1,#(rt.interface) do
				tmp=tmp..(rt.interface)[i]
				if i~=#(rt.interface) then
					tmp=tmp..","
				end
			end
			tmp=tmp.."]"
			rt.interface=tmp
		end
		dbg.print_tty(string.format("%-7s",k)..string.format("%-10s",rt.name)
		..string.format("%-13s",rt.interface)..string.format("%-16s",rt.internal_ip)
		..string.format("%-16s",rt.external_ip)..string.format("%-17s",rt.dmz)
		..string.format("%-13s",rt.comment or "")..string.format("%-6s",rt.enable))
	end
	dbg.print_tty(line)
	return result
end

function check_name_one(selected_name)
	uci_r:set_confdir(CLI_CFG_PATH)
	local ret=uci_r:get("nat_cli_one",selected_name)
	if ret==nil then
		dbg.print_tty("Error: Please input the existing name.")
		return true 
	end
	return false
end

function find_index_of_section_one_to_one(selected_name)
	local index=-1
	local max_index=-1
	uci_r:set_confdir(CFG_PATH)
	uci_r:foreach("nat", "rule_onenat", function (section)
		max_index=max_index+1
		if section.name == selected_name then
			index=max_index
		end
	end)
	return index,max_index
end

function check_parameters_one(selected_name)
	uci_r:set_confdir(CLI_CFG_PATH)
	local get1=uci_r:get("nat_cli_one",selected_name,"vlan_id") 
	local get2=uci_r:get("nat_cli_one",selected_name,"original_ip")  
	local get3=uci_r:get("nat_cli_one",selected_name,"translated_ip")  
	local get4=uci_r:get("nat_cli_one",selected_name,"dmz") 
	local get5=uci_r:get("nat_cli_one",selected_name,"description")
	if get1 and get2 and get3 and get4 and get5 then
		return get1,get2,get3,get4,get5
	end
	return false
end

function no_nat_one_to_one(selected_name)
	local index,max_index=find_index_of_section_one_to_one(selected_name)
	if index<0 then
		dbg.print_tty("Error: No such name.")
		return NAT_ERROR[6]
	end
	uci_r:set_confdir(CFG_PATH)
	http_form={}
	http_form.form="one_nat"
	http_form.data=json.encode(
	{
		["method"] = "delete",
		["params"]=
		{
			["key"]="key-"..index,
			["index"]=index.."",
		}
	})
	local ret = nat.dispatch(http_form)
	return NAT_SUCCESS
end

function nat_one_to_one_server(name)
	local index,max_index=find_index_of_section_one_to_one(name)
	if index>=0 then
		dbg.print_tty(NAT_PROMPT[0])
	else	
		dbg.print_tty(NAT_PROMPT[1])
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	if name and not dtype.safestring(name) then
		return false
	end
	uci_r:set("nat_cli_one","current_name","current_name")
	uci_r:set("nat_cli_one","current_name","current_name",name)
	uci_r:set("nat_cli_one",name,"rule_onenat")
	uci_r:set("nat_cli_one",name,"name",name)
	uci_r:commit("nat_cli_one")
	return NAT_SUCCESS 
end

function add_one_to_one_server(index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
	if translated_ip==original_ip then
		dbg.print_tty("Error: Original IP and Translated IP should be different.")
		return NAT_ERROR[6]
	end
	uci_r:set_confdir(CFG_PATH)
	http_form={}
	http_form.form="one_nat"
	if change_flag then
		local old = {}
		uci_r:foreach("nat", "rule_onenat", function (section)
			if section.name == selected_name then
				old=section
			end
		end)
		http_form.data=json.encode(
		{
			["method"] = "set",
			["params"]=
			{
				["index"]=index,
				["old"]=
				{
					["name"]=old.name,
					["interface"]=old.interface,
					["internal_ip"]=old.internal_ip.."",
					["external_ip"]=old.external_ip.."",
					["dmz"]=old.dmz.."",
					["comment"]=old.comment.."",
					["enable"]=old.enable,
					["t_label"]=old.t_label,
				},
				["new"]=
				{
					["name"]=selected_name,
					["interface"]=vlan_id,
					["internal_ip"]=original_ip.."",
					["external_ip"]=translated_ip.."",
					["dmz"]=dmz.."",
					["comment"]=description.."",
					["enable"]="on",
				},
				["key"]="key-"..index
			}
		})
	else
		http_form.data=json.encode(
		{
			["method"] = "add",
			["params"]=
			{
				["index"]=index,
				["old"]="add",
				["new"]=
				{
					["name"]=selected_name,
					["interface"]=vlan_id,
					["internal_ip"]=original_ip.."",
					["external_ip"]=translated_ip.."",
					["dmz"]=dmz.."",
					["comment"]=description.."",
					["enable"]="on",
				},
				["key"]="add"
			}
		})
	end
	local ret = nat.dispatch(http_form)
	if (ret.error_code == "1003") then
		dbg.print_tty(NAT_PROMPT[3])
	end
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(NAT_ERROR[9])
		return NAT_ERROR[9]
	end
	dbg.print_tty(NAT_PROMPT[4])
	uci_r:set_confdir(CLI_CFG_PATH)
	return NAT_SUCCESS
end

function check_vlan_static(check_vlan_static)
	uci_r:set_confdir(CFG_PATH)
	local proto = uci_r:get("network", check_vlan_static, "proto")
	if proto=="static" then
		return true
	end
	dbg.print_tty("Error: Please set Network to static ip,then set nat one-to-one.")
	return false
end

function check_vlan_id_one(vlan_id)
	uci_r:set_confdir(CFG_PATH)
	if vlan_id=="0" then
		local sfp = string.sub(uci_r:get_profile("phy1","pname"),1,3)
		if sfp=="SFP" then
			local sfp_wan_name="WAN"..uci_r:get_profile("phy1","pnumber")
			local id=string.gsub(sfp_wan_name,"WAN","wan")
			if not check_vlan_static(id.."_eth") then
				return false
			end
			return sfp_wan_name
		else
			dbg.print_tty("Error: Vlan id can not be 0.")
			return false
		end
	end
	local dev_name = uci_r:get("network","global","wan_device")
	local ret=false
	uci_r:foreach("zone", "zone", function(section)
		if section.dev_eth ~= nil then
			local vlan_id_find,find=string.gsub(section.dev_eth,dev_name..".","")
			if find==1 then
				if vlan_id_find==vlan_id then
					ret=section[".name"]
				end
			end
		end
	end)
	if not ret then
		dbg.print_tty("Error: Vlan id do not exist.")
		return false
	else
		local id=string.gsub(ret,"WAN","wan")
		if not check_vlan_static(id.."_eth") then
			return false
		end
	end
	return ret
end

function add_interface_name_one(selected_name,vlan_id)
	vlan_id=vlan_id_split(vlan_id)
	for i=1,#vlan_id do
		local vlan_id_change=check_vlan_id_one(vlan_id[i])
		if not vlan_id_change then
			return NAT_ERROR[10]
		end
		vlan_id[i]=vlan_id_change
	end
	
	if check_name_one(selected_name) then
		return NAT_ERROR[7]
	end

	uci_r:set_confdir(CLI_CFG_PATH)
	if vlanid and not dtype.uinteger(vlanid) then
		return false
	end
	uci_r:set("nat_cli_one",selected_name,"vlan_id",vlan_id)
	uci_r:commit("nat_cli_one")

	if check_parameters_one(selected_name) then
		local vlan_id,original_ip,translated_ip,dmz,description=check_parameters_one(selected_name)
		local index,max_index=find_index_of_section_one_to_one(selected_name)
		if index>=0 then
			change_flag=true
			add_one_to_one_server(index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		else
			change_flag=false
			add_one_to_one_server(max_index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		end
	end
	return NAT_SUCCESS 
end
 
function add_original_ip(selected_name,original_ip)
	if check_name_one(selected_name) then
		return NAT_ERROR[7]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	if original_ip and not dtype.ipaddr(original_ip) then
		return false
	end
	uci_r:set("nat_cli_one",selected_name,"original_ip",original_ip)
	uci_r:commit("nat_cli_one")

	if check_parameters_one(selected_name) then
		local vlan_id,original_ip,translated_ip,dmz,description=check_parameters_one(selected_name)
		local index,max_index=find_index_of_section_one_to_one(selected_name)
		if index>=0 then
			change_flag=true
			add_one_to_one_server(index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		else
			change_flag=false
			add_one_to_one_server(max_index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		end
	end
	return NAT_SUCCESS 
end

function add_translated_ip(selected_name,translated_ip)
	if check_name_one(selected_name) then
		return NAT_ERROR[7]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	if translated_ip and not dtype.ipaddr(translated_ip) then
		return false
	end
	uci_r:set("nat_cli_one",selected_name,"translated_ip",translated_ip)
	uci_r:commit("nat_cli_one")

	if check_parameters_one(selected_name) then
		local vlan_id,original_ip,translated_ip,dmz,description=check_parameters_one(selected_name)
		local index,max_index=find_index_of_section_one_to_one(selected_name)
		if index>=0 then
			change_flag=true
			add_one_to_one_server(index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		else
			change_flag=false
			add_one_to_one_server(max_index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		end
	end
	return NAT_SUCCESS 
end
  
function add_dmz(selected_name,dmz)
	if dmz~="on" and dmz~="off" then
		dbg.print_tty("Error: Please input correct para,on or off.")
		return NAT_ERROR[6]
	end
	if check_name_one(selected_name) then
		return NAT_ERROR[7]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	uci_r:set("nat_cli_one",selected_name,"dmz",dmz)
	uci_r:commit("nat_cli_one")

	if check_parameters_one(selected_name) then
		local vlan_id,original_ip,translated_ip,dmz,description=check_parameters_one(selected_name)
		local index,max_index=find_index_of_section_one_to_one(selected_name)
		if index>=0 then
			change_flag=true
			add_one_to_one_server(index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		else
			change_flag=false
			add_one_to_one_server(max_index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		end
	end
	return NAT_SUCCESS 
end

function add_description(selected_name,description)
	if check_name_one(selected_name) then
		return NAT_ERROR[7]
	end
	uci_r:set_confdir(CLI_CFG_PATH)
	uci_r:set("nat_cli_one",selected_name,"description",description)
	uci_r:commit("nat_cli_one")

	if check_parameters_one(selected_name) then
		local vlan_id,original_ip,translated_ip,dmz,description=check_parameters_one(selected_name)
		local index,max_index=find_index_of_section_one_to_one(selected_name)
		if index>=0 then
			change_flag=true
			add_one_to_one_server(index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		else
			change_flag=false
			add_one_to_one_server(max_index,selected_name,vlan_id,original_ip,translated_ip,dmz,description)
		end
	end
	return NAT_SUCCESS 
end

function get_nat_vs_current_name()
	uci_r:set_confdir(CLI_CFG_PATH)
	return uci_r:get("nat_cli","current_name","current_name")
end

function get_nat_oto_current_name()
	uci_r:set_confdir(CLI_CFG_PATH)
	return uci_r:get("nat_cli_one","current_name","current_name")
end

function get_ahead_ip(ip)
	local res=""
	for token in string.gmatch(ip, ".") do
		if token~="." then
			res=res..token
		else
			return res
		end
	end
end

function check_ip(ip)
	local res=get_ahead_ip(ip)
	res=tonumber(res)
	if res==0 or res==127 or res >=224 then
		return false
	end
	return true
end
--Entrance
function set_nat_cfg(arg)
	local cmd = arg[1]
	if not cmd then
		return NAT_ERROR[0]
	elseif arg[1] == "show nat one-to-one" then
		if arg[2] then
			return NAT_ERROR[4]
		end
		return show_nat_one_to_one()
	elseif arg[1] == "show nat virtual-server" then
		if arg[2] then
			return NAT_ERROR[4]
		end
		return show_nat_virtual_server()
	elseif arg[1] == "nat alg" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local alg_data=arg[2]
		return nat_alg(alg_data)
	elseif arg[1] == "no nat alg" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local no_alg_data=arg[2]
		return no_nat_alg(no_alg_data)
	elseif arg[1] == "show nat alg" then
		if arg[2] then
			return NAT_ERROR[4]
		end
		return show_nat_alg()
	elseif arg[1]=="nat virtual-server" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local name=arg[2]
		return nat_virtual_server(name)
	elseif arg[1]=="interface virtual-server" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_vs_current_name()
		local vlan_id =arg[2]
		return add_interface_name(selected_name,vlan_id)
	elseif arg[1]=="external port" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_vs_current_name()
		local external_port =arg[2]
		return add_external_port(selected_name,external_port)
	elseif arg[1]=="internal port" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_vs_current_name()
		local internal_port =arg[2]
		return add_internal_port(selected_name,internal_port)
	elseif arg[1]=="internal server-ip" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_vs_current_name()
		local server_ip =arg[2]
		if not check_ip(server_ip) then
			dbg.print_tty("Error: The address cannot start with 0 or 127 or a Class D address(224~255).")
			return NAT_ERROR[10]
		end
		return add_server_ip(selected_name,server_ip)
	elseif arg[1]=="protocol" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_vs_current_name()
		local protocol =arg[2]
		return add_protocol(selected_name,protocol)
	elseif arg[1]=="no nat virtual-server" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = arg[2]
		return no_nat_virtual_server(selected_name)
	elseif arg[1]=="no nat one-to-one" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = arg[2]
		return no_nat_one_to_one(selected_name)
	elseif arg[1]=="nat one-to-one" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local name=arg[2]
		return nat_one_to_one_server(name)
	elseif arg[1]=="interface one-to-one" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_oto_current_name()
		local vlan_id =arg[2]
		return add_interface_name_one(selected_name,vlan_id)
	elseif arg[1]=="original-ip" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_oto_current_name()
		local original_ip =arg[2]
		if not check_ip(original_ip) then
			dbg.print_tty("Error: The address cannot start with 0 or 127 or a Class D address(224~255).")
			return NAT_ERROR[10]
		end
		return add_original_ip(selected_name,original_ip)
	elseif arg[1]=="translated-ip" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_oto_current_name()
		local translated_ip =arg[2]
		if not check_ip(translated_ip) then
			dbg.print_tty("Error: The address cannot start with 0 or 127 or a Class D address(224~255).")
			return NAT_ERROR[10]
		end
		return add_translated_ip(selected_name,translated_ip)
	elseif arg[1]=="dmz forwarding" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_oto_current_name()
		local dmz =arg[2]
		return add_dmz(selected_name,dmz)
	elseif arg[1]=="description-one" then
		if not arg[2] then
			return NAT_ERROR[3]
		end
		if arg[3] then
			return NAT_ERROR[4]
		end
		local selected_name = get_nat_oto_current_name()
		local description =arg[2]
		return add_description(selected_name,description)
	end

	return NAT_ERROR[1]
end

local result = NAT_SUCCESS
result=set_nat_cfg(arg)
io.stdout:write(result)
