-- External reference definition
local sys  = require "luci.sys"
local json = require "luci.json"
local interfaces = require "luci.model.interfaces"
local userconfig = require "luci.model.userconfig"
local lan  = require "luci.controller.admin.lan"
local IPSEC = require "luci.controller.admin.ipsec"
local IPPOOL = require "luci.controller.admin.ippool"
local utl = require "luci.util"
local dbg  = require "luci.tools.debug"
local err = require "luci.tools.error"
local form = require "luci.tools.form"
local uci  = require "luci.model.uci"
local ippoolref = require "luci.model.reference"
local routing = require "luci.controller.admin.routing" 
local error_print = require "luci.tools.error_print"
local ip=require "luci.ip"
require "nixio"
local bit  = nixio.bit
-- Variable declaration
local uci_r = uci.cursor()
local CLI_CFG_PATH = "/tmp/cli_config"
local CFG_PATH = "/etc/config"
local ROUTING_SUCCESS="SUCCESS"
local ROUTING_ERROR={
	[0] = "Error:No command input",
	[1] = "Error:No such command at present",
	[2] = "Error:Insufficient parameters",
	[3] = "Error:Too much parameters",
	[4] = "Error:Format error of para",
	[5] = "Error:Get config failed",
	[6] = "Error:Duplicate name",
	[7] = "Error:Crud error"
}
local line=""
local LINE_NUM=99
for i=1,LINE_NUM do
	line=line.."-"
end
-- Function declaration
function print_lua_table(lua_table, indent)
       indent = indent or 0
       len = 0
       for k, v in pairs(lua_table) do
              if type(k) == "string" then
                     len = 23 - string.len(k)
                     k = string.format("%s", k)
              end
              local szSuffix = string.rep(" ", len)
              --local szPrefix = string.rep(" ", indent)
              if type(v) == "table" then
                     szSuffix = ":"
                     formatting = k..szSuffix
              else
                     formatting = k..":"..szSuffix
              end
              if type(v) == "table" then
                     --dbg.print_tty(formatting)
                     print_lua_table(v, indent + 1)
                     --dbg.dbg.print_tty_tty(szPrefix.."},")
              else
                     local szValue = ""
                     if type(v) == "string" then
                            szValue = string.format("%s", v)
                     else
                            szValue = tostring(v)
                     end
                     dbg.print_tty(formatting..szValue)
              end
       end
end

function masklen_to_wildmask(masklen)
	local wild_mask=""
	if masklen < 1 or masklen > 32 then
		dbg.print_tty("Error: Mask is invalid.")
		return wild_mask
	end
	if (masklen >= 24) then
		local wild_one_bit_len = masklen-24
		local shift_len = 32 - masklen
		--第一次移位获取将对应位数置1，第二次移位将对应位数置0
		local tmp = tostring(bit.lshift(bit.lshift(1, wild_one_bit_len) - 1, shift_len))
		wild_mask = "255.255.255." .. tmp
	elseif (masklen >= 16) then
		local wild_one_bit_len = masklen-16
		local shift_len = 24 - masklen
		local tmp = tostring(bit.lshift(bit.lshift(1, wild_one_bit_len) - 1, shift_len))
		wild_mask = "255.255." .. tmp .. ".0"
	elseif (masklen >= 8) then
		local wild_one_bit_len = masklen-8
		local shift_len = 16 - masklen
		local tmp = tostring(bit.lshift(bit.lshift(1, wild_one_bit_len) - 1, shift_len))
		wild_mask = "255."..tmp .. ".0.0"
	else
		local shift_len = 8 - masklen
		local tmp = tostring(bit.lshift(bit.lshift(1, masklen) - 1, shift_len))
		wild_mask = tmp .. ".0.0.0"
	end
	return wild_mask
end

function show_ip_route_static()
	local index = 1
	http_form = {}
	http_form.form = "static_routing"
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] = 
        	{
           
        	}
	})
	uci_r:set_confdir(CFG_PATH)	
	local ret = routing.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty("No data")
		return ROUTING_ERROR[5]
	end

	ret=ret.result
	dbg.print_tty(line)
	dbg.print_tty(string.format("%-7s","Index")..string.format("%-16s","Name")
	..string.format("%-16s","Destination_IP")..string.format("%-16s","Subnet_Mask")
	..string.format("%-16s","Next_Hop")..string.format("%-12s","Interface")..string.format("%-8s","Metric")
	..string.format("%-10s","Status")..string.format("%-6s","Key"))
	for k, rt in ipairs(ret) do
		if rt.enable=="on" then
			rt.enable="Enabled"
		else
			rt.enable="Disabled"
		end
		if type(rt.target) == "table" then
			for i, v in ipairs(rt.target) do
				local ip = utl.split(v, "/")
				if ip[1] and ip[2] then
					local mask = masklen_to_wildmask(tonumber(ip[2]))
					dbg.print_tty(line)
					dbg.print_tty(string.format("%-7s",index)..string.format("%-16s",rt.name)
					..string.format("%-16s",ip[1])..string.format("%-16s",mask)
					..string.format("%-16s",rt.gateway)..string.format("%-12s",rt.interface)..string.format("%-8s",rt.metric)
					..string.format("%-10s",rt.enable)..string.format("%-6s",rt.key))
					index = index + 1
				end
			end
		else
			dbg.print_tty(line)
			dbg.print_tty(string.format("%-7s",index)..string.format("%-16s",rt.name)
			..string.format("%-16s",rt.target)..string.format("%-16s",rt.netmask)
			..string.format("%-16s",rt.gateway)..string.format("%-12s",rt.interface)..string.format("%-8s",rt.metric)
			..string.format("%-10s",rt.enable)..string.format("%-6s",rt.key))
			index = index + 1
		end
	end
	dbg.print_tty(line)
	return ROUTING_SUCCESS
end

function check_interface_vlan_id(vlan_id)
	if vlan_id=="0" then
		local sfp = string.sub(uci_r:get_profile("phy1","pname"),1,3)
		if sfp=="SFP" then
			local sfp_wan_name="WAN"..uci_r:get_profile("phy1","pnumber")
			return sfp_wan_name
		else
			dbg.print_tty("Error: Vlan id can not be 0.")
			return false
		end
	end
	uci_r:set_confdir(CFG_PATH)
	local vlan_id_change =uci_r:get("vnetwork", "NETWORK"..vlan_id,"zone") 
	if vlan_id_change then
		return vlan_id_change
	end
	local wan_name = uci_r:get("network","global","wan_device")
	local ret=false
	uci_r:foreach("zone", "zone", function(section)
		if section.dev_eth ~= nil then
			local vlan_id_find_wan,find_wan=string.gsub(section.dev_eth,wan_name..".","")
			if find_wan==1 then
				if vlan_id_find_wan==vlan_id then
					ret=section[".name"]
				end
			end
		end
	end)
	if not ret then
		dbg.print_tty("Error: Vlan id do not exist.")
	end
	return ret
end

function ip_route(add_data)
	local change_flag=false
	local get_ret=get_ip_route_result()
	local index=get_ip_route_num()
	local old
	for k, rt in ipairs(get_ret) do
		if add_data[1]==rt.name then
			change_flag=true
			old=rt
			index=k-1
		end
	end
	local interface=check_interface_vlan_id(add_data[5])
	if not interface then
		return ROUTING_ERROR[7]
	end
	http_form = {}
	http_form.form = "static_routing"
	if change_flag then
		http_form.data = json.encode(
		{
			["method"] = "set",
			["params"] = 
				{
				["index"]=index,
				["old"]={
					["name"]=old.name,
					["target"]=old.target,
					["netmask"]=old.netmask,
					["gateway"]=old.gateway,
					["interface"]=old.interface,
					["metric"]=old.metric,
					["note"]=old.note,
					["reachability"]=old.reachability,
					["enable"]=old.enable,
					["t_label"]=old.t_label,
				},
					["new"]=
				{
					["name"]=add_data[1],
					["target"]=add_data[2],
					["netmask"]=add_data[3],
					["gateway"]=add_data[4],
					["interface"]=interface,
					["metric"]=add_data[6],
					["note"]="",
					["enable"]="on",
				},
				["key"]=old.key,
				}
		})
	else
		http_form.data = json.encode(
		{
			["method"] = "add",
			["params"] = 
				{
				["index"]=index,
				["old"]="add",
					["new"]=
				{
					["name"]=add_data[1],
					["target"]=add_data[2],
					["netmask"]=add_data[3],
					["gateway"]=add_data[4],
					["interface"]=interface,
					["metric"]=add_data[6],
					["note"]="",
					["enable"]="on",
				},
				["key"]="add",
				}
		})
	end
	local ret = routing.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty("Error: The route must be different.")
		return ROUTING_ERROR[7]
	end
	return ROUTING_SUCCESS
end

function no_ip_route(index)
	http_form = {}
	http_form.form = "static_routing"
	http_form.data = json.encode(
	{
		["method"] = "delete",
		["params"] = 
        	{
			["key"]="",
			["index"]=(tonumber(index)-1).."",
        	}
	})
	uci_r:set_confdir(CFG_PATH)
	
	local ret = routing.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty("Error: Crud Error.")
		return ROUTING_ERROR[7]
	end
	return ROUTING_SUCCESS
end

function get_ip_route_result()
	http_form = {}
	http_form.form = "static_routing"
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] = 
        	{
           
        	}
	})
	local ret = routing.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty("GET_TABLE_FAILED")
		return "GET_TABLE_FAILED"
      end
	ret=ret.result
	return ret
end

function get_ip_route_num()
	local count=0
	local ret=get_ip_route_result()
	for k, rt in ipairs(ret) do
		count=count+1;
	end
	return count
end

function get_ahead_ip(ip)
	local res=""
	for token in string.gmatch(ip, ".") do
		if token~="." then
			res=res..token
		else
			return res
		end
	end
end

function check_ip(ip)
	local res=get_ahead_ip(ip)
	res=tonumber(res)
	if res==0 or res==127 or res >=224 then
		return false
	end
	return true
end

function byte2bin(n)
	local t = {}
	for i=7,0,-1 do
		t[#t+1] = math.floor(n / 2^i)
		n = n % 2^i
	end
	return table.concat(t)
end

function ip_10to2(ip)
	ip=ip.."."
	local res=""
	local tmp=""
	for token in string.gmatch(ip, ".") do
		if token ~="." then
			tmp=tmp..token
		else
			res=res..byte2bin(tmp)
			tmp=""
		end
	end
	return res
end

function check_ip_mask(ip)
	local arr=ip_10to2(ip)
	local flag=true
	for token in string.gmatch(arr, ".") do 
		if flag then
			if token=="0" then
				flag=false	
			end
		else
			if token=="1" then
				return false
			end
			
		end
	end
	return true
end

function ip_change_2_10(ip)
	local res=0
	for i=1,#ip do
		res=res+tonumber(string.sub(ip,i,i))*2^(8-i)
	end
	return res..""
end

function ip_change_by_mask(ip,mask)
	ip=ip.."."
	local ip2=""
	local tmp=""
	for token in string.gmatch(ip, ".") do
		if token =="." then
			ip2=ip2..byte2bin(tmp)
			ip2=ip2..token
			tmp=""
		else 
			tmp=tmp..token
		end
	end
	
	mask=mask.."."
	local mask2=""
	tmp=""
	for token in string.gmatch(mask, ".") do
		if token =="." then
			mask2=mask2..byte2bin(tmp)
			mask2=mask2..token
			tmp=""
		else 
			tmp=tmp..token
		end
	end
	
	local mask_and_ip=""
	for i=1,#mask2 do
		if string.sub(mask2,i,i) ~="." then
			tmp=string.sub(ip2,i,i)*string.sub(mask2,i,i)
			mask_and_ip=mask_and_ip..tmp
		else
			mask_and_ip=mask_and_ip.."."
		end
	end
	
	tmp=""
	res=""
	for i=1,#mask_and_ip do
		if string.sub(mask_and_ip,i,i) ~="." then
			tmp=tmp..string.sub(mask_and_ip,i,i)
		else
			res=res..ip_change_2_10(tmp)
			if i~=#mask_and_ip then
				res=res.."."
			end
			tmp=""
		end
	end
	return res
end

--Entrance
function set_routing_cfg(arg)
	local cmd = arg[1]
	if not cmd then
		return ROUTING_ERROR[0]
	elseif arg[1] == "show ip route static" then
		if arg[2] then
			return ROUTING_ERROR[3]
		end
		return show_ip_route_static()
	elseif arg[1]=="ip route" then
	
		if not arg[2] then
			return ROUTING_ERROR[2]
		end
		
		if arg[8] then
			return ROUTING_ERROR[3]
		end
		
		local name=arg[2]
		local target=arg[3]
		local netmask=arg[4]
		local gateway=arg[5]
		target=ip_change_by_mask(target,netmask)
		if not check_ip(target) or  not check_ip(gateway) then 
			dbg.print_tty("Error: The address cannot start with 0 or 127 or a Class D address(224~255).")
			return ROUTING_ERROR[4]
		elseif not check_ip_mask(netmask) then
			dbg.print_tty("Error: Invalid subnet mask format.")
			return ROUTING_ERROR[4]
		end

		local interface=arg[6]
		local metric=arg[7]
		if tonumber(metric)<0 or tonumber(metric)>15 then
			dbg.print_tty("Error: Please input metric between 0 and 15.")
			return ROUTING_ERROR[4]
		end
		add_data={name,target,netmask,gateway,interface,metric}
		return ip_route(add_data)
	elseif arg[1] == "no ip route" then
		if not arg[2] then
			return ROUTING_ERROR[2]
		end
		if arg[3] then
			return ROUTING_ERROR[3]
		end
		local count=get_ip_route_num()
		if(count==0)then
			dbg.print_tty("Error: The ip route table is empty.")
			return ROUTING_ERROR[5]
		end
		local index=arg[2]
		if tonumber(index)<1 or tonumber(index)>count then
			dbg.print_tty("Error: Please input index between 1 and "..count..".")
			return ROUTING_ERROR[4]
		end
		return no_ip_route(index)
	end
	return ROUTING_ERROR[1]
end

local result = ROUTING_SUCCESS
result=set_routing_cfg(arg)
io.stdout:write(result)