--头文件
local sys  = require "luci.sys"
local json = require 'luci.json'
local utl = require "luci.util"
local dbg  = require 'luci.tools.debug'
local form = require "luci.tools.form"
local uci  = require "luci.model.uci"
local if_wan = require "luci.controller.admin.interface_wan"
local iface = require "luci.controller.admin.interface"
local vlan_st = require "luci.controller.admin.vlan_setting" 
local dhp = require "luci.controller.admin.dhcps"
local mac_st = require "luci.controller.admin.mac_setting"
local error_print = require "luci.tools.error_print"
local ifs  = require "luci.model.interfaces"

local uci_r = uci.cursor()   --uci初始化
local CFG_PATH = "/etc/config"      --系统配置文件路径
uci_r:set_confdir(CFG_PATH)    --设置当前工作路径


--校验wan口，排查是否包含iptv口
function check_wan_port(wan_port)
	local wan_ports
	_, _, wan_ports = ifs.get_lan_wan_ports(ifs.wan_numbers())
	if wan_ports then
		for p in string.gmatch(wan_ports, "%d+") do
			if string.match(wan_port, p) then
				return p
			end
		end
	end
	return wan_port
end

function show_interface_cfg(cmd, port_or_id)
	local Result = "SUCCESS"
	
	local http_form = {}

	if cmd == "show interface vlan" then
		local vlan_id = port_or_id
		http_form.data = json.encode(
		{
			["method"] = "get",
			["params"] = 
			{
			}
		})
		http_form.form = "vlansettinglist"
		--dbg.dumptable(http_form)
		-- 调用web对应的lua脚本
		local ret = vlan_st.dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
			dbg.print_tty(error_print.check_error(ret))
			return "FAILED"
	 	end
	
		
		ret = ret.result	--读取结果
		for key,value in ipairs(ret) do
			if vlan_id == value.vlan then
				dbg.print_tty("     vlan"..vlan_id.." ports:"..value.ports)	--输出当前vlan包含哪些端口
				if "wan" == value.users then
					local wan_port = value.ports
					wan_port = check_wan_port(wan_port)
					dbg.print_tty("     Vlan type.................."..value.users)
					Result = get_interface_wan_cfg(wan_port)
				elseif "lan" == value.users then
					dbg.print_tty("     Vlan type.................."..value.users)
			
					Result = get_interface_lan_cfg(vlan_id)
					
				else
					dbg.print_tty("     Vlan type..................".."lan")
					dbg.print_tty("     Routing interface status...".."DOWN")
					break;
				end
				break;
			end
		end
	else			--"cmd == show interface switchport"
		local port = port_or_id
		
		http_form.data = json.encode(
		{
			["method"] = "get"
		})
		http_form.form = "wanmode" 					--1
		
		local ret = if_wan.dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
			dbg.print_tty(error_print.check_error(ret))
			return "FAILED"
	 	end
		local poffset = tonumber(uci_r:get_profile("switch0_user", "poffset") or "0")

		ret = ret.result	--读取结果
		ret = ret.wan_names
		for key,value in ipairs(ret) do
			if tostring(port) == tostring(tonumber(value.logical) - poffset) then
				dbg.print_tty("     Port name.................."..value.name)	--显示port名称
				break
			end
		end
		
		http_form = {}
		http_form.data = json.encode(
		{
			["method"] = "get"
		})
		http_form.form = "portsettinglist"
		
		-- 调用web对应的lua脚本
		local ret = vlan_st.dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
			dbg.print_tty(error_print.check_error(ret))
			return "FAILED"
	 	end
		
		ret = ret.result
		for key,value in ipairs(ret) do
			if value.port == port then
				local tmp_vlans = value.vlans
				dbg.print_tty("     Belonged vlan.............."..table.concat(tmp_vlans, ' ')) --属于哪些vlan组
				dbg.print_tty("     Pvid......................."..value.pvid)	--显示当前端口的pvid
				for index,vlan_id in ipairs(tmp_vlans) do --每一个vlan id进行显示
					if nil ~= string.find(vlan_id, "t") then
						vlan_id = string.sub(vlan_id, 1, "-2")
					end
					dbg.print_tty(" Vlan"..vlan_id.." config")
					http_form = {}
					http_form.data = json.encode(
					{
						["method"] = "get",
						["params"] = 
						{
						}
					})
					http_form.form = "vlansettinglist"
					
					-- 调用web对应的lua脚本
					local tmp_ret = vlan_st.dispatch(http_form)
					if (not tmp_ret) or (tmp_ret.error_code ~= "0") then
						dbg.print_tty(error_print.check_error(tmp_ret))
						return "FAILED"
					end
					
					tmp_ret = tmp_ret.result	--读取结果
					for key,value in ipairs(tmp_ret) do
						if vlan_id == value.vlan then
							if "wan" == value.users then
								local wan_port = value.ports
								wan_port = check_wan_port(wan_port)
								dbg.print_tty("     Vlan type.................."..value.users)
								Result = get_interface_wan_cfg(wan_port)
							elseif "lan" == value.users then
								dbg.print_tty("     Vlan type.................."..value.users)
						
								Result = get_interface_lan_cfg(vlan_id)
								
							else
								dbg.print_tty("     Vlan type..................".."lan")
								dbg.print_tty("     Routing interface status...".."DOWN")
								break;
							end
							break;
						end
					end
				end
			end
		end
	end
	
	return Result
end


function get_interface_wan_cfg(wan_port)
    local Result = "SUCCESS"
	local http_form
	
	local tmp_status = "DOWN"
	local tmp_host_name = "none"
	local tmp_proto = "none"
	local tmp_uplink = "none"
	local tmp_downlink = "none"
	local tmp_bandwidth = "none"
	local tmp_mtu = "none"
	local tmp_ip = "none"
	local tmp_mask = "none"
	local tmp_def_gateway = "0.0.0.0"
	local tmp_primary_dns = "0.0.0.0"
	local tmp_secondary_dns = "0.0.0.0"
	local tmp_macaddr = "none"
	local tmp_name = "none"
	local tmp_main_proto = "none"
	--第二连接数据
	local tmp_sec_status = "DOWN"
	local tmp_sec_proto = "none"
	local tmp_sec_ip = "none"
	local tmp_sec_mask = "none"
	local tmp_sec_def_gateway = "0.0.0.0"
	local tmp_sec_primary_dns = "0.0.0.0"
	local tmp_sec_secondary_dns = "0.0.0.0"
	
	local tmp_wan_mode = "none"
	
	http_form = {}	--tmp_name
	http_form.data = json.encode(
	{
		["method"] = "get"
	})
	http_form.form = "wanmode" 					--1
	
	local ret = if_wan.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(error_print.check_error(ret))
		return "FAILED"
	end
	ret = ret.result	--读取结果
	ret = ret.wan_names
	for key,value in ipairs(ret) do
		if wan_port == value.logical then
			tmp_name = value.name or "none"
			break
		end
	end
	
	http_form = {} 
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] = 
		{
			["wan_id"] = wan_port
		}
	})
	http_form.form = "wanconfig"  				--2
	ret = if_wan.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(error_print.check_error(ret))
		return "FAILED"
	end
	ret = ret.result	--读取结果
	tmp_proto = ret.proto or "none"
	tmp_host_name = ret.hostname or "none"
	tmp_bandwidth = ret.bandwidth or "none"
	tmp_uplink = ret.uplink or "none"
	tmp_downlink = ret.downlink or "none"
	tmp_mtu = ret.mtu or "none"
	tmp_sec_proto = ret.second_proto or "none"
	http_form = {}
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] = 
		{
			["wan_id"] = wan_port,
			["proto"] = tmp_proto
		}
	})
	http_form.form = "status"  						--4
	
	ret = if_wan.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(error_print.check_error(ret))
		return "FAILED"
	end
	ret = ret.result	--读取结果
	
	if "true" == tostring(ret.t_isup) then
		tmp_status = "UP" or "DOWN"
		tmp_ip = ret.ipaddr or "none"
		tmp_mask = ret.netmask or "none"
		tmp_def_gateway = ret.gateway or "0.0.0.0"
		tmp_primary_dns = ret.dns1 or "0.0.0.0"
	end
	
	if tmp_sec_proto ~= "none" then		--有第二连接
		tmp_main_proto = tmp_proto
		
		http_form = {}
		http_form.data = json.encode(
		{
			["method"] = "get",
			["params"] = 
			{
				["wan_id"] = wan_port,
				["proto"] = tmp_sec_proto,
				["main_proto"] = tmp_main_proto
			}
		})
		http_form.form = "status"  						--4 请求第二连接数据
		
		ret = if_wan.dispatch(http_form)
		if (not ret) or (ret.error_code ~= "0") then
			dbg.print_tty(error_print.check_error(ret))
			return "FAILED"
		end
		ret = ret.result	--读取结果
		
		if "true" == tostring(ret.t_isup) then
			tmp_sec_status = "UP" or "DOWN"
			tmp_sec_ip = ret.ipaddr or "none"
			tmp_sec_mask = ret.netmask or "none"
			tmp_sec_def_gateway = ret.gateway or "0.0.0.0"
			tmp_sec_primary_dns = ret.dns1 or "0.0.0.0"
		end	
	end
	
	http_form = {}
	http_form.data = json.encode(
	{
		["method"] = "get"
	})
	http_form.form = "wanmode" 						--6
	ret = if_wan.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(error_print.check_error(ret))
		return "FAILED"
	end
	ret = ret.result
	tmp_wan_mode = ret.wanmode or "none"
	
	http_form = {}
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] = 
		{
			["wanmode"] = tonumber(tmp_wan_mode)
		}
	})
	http_form.form = "macconfig"  					--7
	ret = mac_st.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(error_print.check_error(ret))
		return "FAILED"
	end
	ret = ret.result
	local mac_wan = "wan"..wan_port.."_mac"
	tmp_macaddr = ret[mac_wan] or "none"
	
	--显示信息
	dbg.print_tty("     Routing Interface Status..."..tmp_status)
	dbg.print_tty("     Primary IP Address........."..tmp_ip.."/"..tmp_mask)  --显示ip地址和掩码信息
	dbg.print_tty("     MAC Address................"..tmp_macaddr)
	dbg.print_tty("     Proto......................"..tmp_proto)
	dbg.print_tty("     Mtu........................"..tmp_mtu)
	dbg.print_tty("     Bandwidth.................."..tmp_bandwidth.."kbps")
	dbg.print_tty("     Uplink....................."..tmp_uplink.."kbps")
	dbg.print_tty("     Downlink..................."..tmp_downlink.."kbps")
	dbg.print_tty("     Default Gateway............"..tmp_def_gateway)
	dbg.print_tty("     Primary DNS................"..tmp_primary_dns)
	dbg.print_tty("     Secondary DNS.............."..tmp_secondary_dns)
	
	if tmp_sec_proto ~= "none" then			--第二连接不为空，则显示数据
		dbg.print_tty(" Secondary Connection")
		dbg.print_tty("     Routing Interface Status..."..tmp_sec_status)
		dbg.print_tty("     Primary IP Address........."..tmp_sec_ip.."/"..tmp_sec_mask)
		dbg.print_tty("     Proto......................"..tmp_sec_proto)
		dbg.print_tty("     Default Gateway............"..tmp_sec_def_gateway)
		dbg.print_tty("     Primary DNS................"..tmp_sec_primary_dns)
		dbg.print_tty("     Secondary DNS.............."..tmp_sec_secondary_dns)
	end
	
    return Result
end

function get_interface_lan_cfg(vlan_id)
	local Result = "SUCCESS"
	
	local tmp_dhcp_mode = "none"
	local tmp_ip = "none"
	local tmp_mask = "none"
	local tmp_lease_time = "none"
	local tmp_dhp_enable = "off"
	local tmp_relay_on = "off"
	local tmp_ipaddr_start = "none"
	local tmp_ipaddr_end = "none"

	local tmp_status = "UP"
	local tmp_proto = "static"
	local tmp_macaddr = "none"
	local tmp_mtu = "1500"
	local tmp_wan_mode = "none"
	
	
	local http_form = {}	--tmp_macaddr
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] ={
		}
	})
	http_form.form = "lan"  					--1
	
	local ret = dhp.dhcp_dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(error_print.check_error(ret))
		return "FAILED"
	end
	ret = ret.result	--读取结果
	if (#ret) == 0 then
		ret = {ret}
	end
	for key,value in ipairs(ret) do
		if vlan_id == value.vlanid then
			tmp_dhcp_mode = value.dhcp_mode or "none"
			tmp_dhp_enable = value.enable or "off"
			tmp_ip = value.ipaddr or "none"
			tmp_mask = value.netmask or "none"
			tmp_ipaddr_start = value.ipaddr_start or "none"
			tmp_ipaddr_end = value.ipaddr_end or "none"
			tmp_lease_time = value.leasetime or "none"
			tmp_relay_on = value.relay_enable or "off"
			break
		end
	end
	
	http_form = {}
	http_form.data = json.encode(
	{
		["method"] = "get"
	})
	http_form.form = "wanmode" 						--6
	ret = if_wan.dispatch(http_form)
	
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(error_print.check_error(ret))
		return "FAILED"
	end
	ret = ret.result
	tmp_wan_mode = ret.wanmode or "none"
	
	http_form = {}
	http_form.data = json.encode(
	{
		["method"] = "get",
		["params"] = 
		{
			["wanmode"] = tonumber(tmp_wan_mode)
		}
	})
	http_form.form = "macconfig"  					--7
	ret = mac_st.dispatch(http_form)
	if (not ret) or (ret.error_code ~= "0") then
		dbg.print_tty(error_print.check_error(ret))
		return "FAILED"
	end
	ret = ret.result
	local mac_lan = "lan_mac"
	tmp_macaddr = ret[mac_lan] or "none"
	
	--显示信息
	dbg.print_tty("     Routing Interface Status..."..tmp_status)
	dbg.print_tty("     Proto......................"..tmp_proto)
	dbg.print_tty("     Dhcp mode.................."..tmp_dhcp_mode)
	dbg.print_tty("     Dhcp enable................"..tmp_dhp_enable)
	dbg.print_tty("     Primary IP address:........"..tmp_ip.."/"..tmp_mask)
	dbg.print_tty("     Dhcp start................."..tmp_ipaddr_start)
	dbg.print_tty("     Dhcp end..................."..tmp_ipaddr_end)
	dbg.print_tty("     Dhcp release time.........."..tmp_lease_time.."min")
	dbg.print_tty("     Dhcp relay enable.........."..tmp_relay_on)
	
	dbg.print_tty("     Macaddr...................."..tmp_macaddr)
	dbg.print_tty("     Mtu........................"..tmp_mtu)
	
	return Result
end

local function port_in_range(port_number, list)

	return string.match(list, "^" .. port_number .. " ") or
		string.match(list, " " .. port_number .. " ") or
		string.match(list, " " .. port_number .. "$") or
		string.match(list, "^" .. port_number .. "$")
end

local Result = "SUCCESS"
local cmd = arg[1]
local list = uci_r:get_profile("switch0_user", "pport")
--dbg.print_tty(cmd)  --print是打印到ssh窗口
if arg[1] == "show interface vlan" then
    local vlan_id = arg[2]
    Result = show_interface_cfg(cmd, vlan_id)
end
if arg[1] == "show interface switchport" then
	if port_in_range(arg[2], list) then
		local port = arg[2]
		Result = show_interface_cfg(cmd, port)
	else
		dbg.print_tty("Error: Port number does not exist.")
		return "FAILED"
	end
end
io.stdout:write(Result)
