local sys  = require "luci.sys"
local json = require 'luci.json'
local utl = require "luci.util"
local dbg  = require ('luci.tools.debug')
local form = require "luci.tools.form"
local uci  = require "luci.model.uci"
local snmp = require 'luci.controller.admin.snmp'
local error_print = require "luci.tools.error_print"

local uci_r = uci.cursor()
local CFG_PATH = "/etc/config"

function set_snmp_v3_account(username, password, snmp_v3_en)
    local result = "SUCCESS"
    local http_form = {}
    local tmp_config = {}
    local old_data = snmp.get_snmp()

    uci_r:set_confdir(CFG_PATH)
    tmp_config.snmp_v1_v2_en = old_data.snmp_v1_v2_en or "off"
    if tmp_config.snmp_v1_v2_en == "on" then
        tmp_config.sys_contact = old_data.sys_contact or ""
        tmp_config.sys_name = old_data.sys_name or ""
        tmp_config.sys_location = old_data.sys_location or ""
        tmp_config.get_community = old_data.get_community or ""
        tmp_config.get_source = old_data.get_source or ""
    end
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {

            ["snmp_v1_v2_en"] = tmp_config.snmp_v1_v2_en,
            ["snmp_v3_en"] = snmp_v3_en,
            ["username"] = username,
            ["password"] = password,
            ["sys_contact"] = tmp_config.sys_contact,
            ["sys_name"] = tmp_config.sys_name,
            ["sys_location"] = tmp_config.sys_location,
            ["get_community"] = tmp_config.get_community,
            ["get_source"] = tmp_config.get_source,
        }
    })

    http_form.form = "snmp"
    ret = snmp.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        result = "FAILED"
    end

    return result
end

function set_snmp_v1_v2(key_name, value)
    local result = "SUCCESS"
    local http_form = {}
    local input = {}
    local tmp_config = {}
    local old_data = snmp.get_snmp()
    uci_r:set_confdir(CFG_PATH)
    if string.sub(value,1,3) == "127" then
        dbg.print_tty("Error: The address cannot be a loopback address.")
        return "FAILED"
    end
    if string.sub(value,1,3) == "255" then
        dbg.print_tty("Error: The address cannot start with 255.")
        return "FAILED"
    end
    local snmp_v1_v2_en = old_data.snmp_v1_v2_en or "off"
    if snmp_v1_v2_en == "off" and key_name ~= "snmp_v1_v2_en" then
        dbg.print_tty("Error: Please enable snmp-server v1&v2 first.")
        return "FAILED"
    end

    tmp_config.sys_contact = old_data.sys_contact or ""
    tmp_config.sys_name = old_data.sys_name or ""
    tmp_config.sys_location = old_data.sys_location or ""
    tmp_config.get_community = old_data.get_community or ""
    tmp_config.get_source = old_data.get_source or ""
    tmp_config.snmp_v3_en = old_data.snmp_v3_en or "off"

    if tmp_config.snmp_v3_en == "on" then
        tmp_config.username = old_data.username or nil
        tmp_config.password = old_data.password or nil
    end

    input[key_name] = value
    if snmp_v1_v2_en == "on" and key_name == "get_community" and value == "" then
        input.get_community = "public"
    end

    if snmp_v1_v2_en == "on" and key_name == "get_source" and value == "" then
        input.get_source = "0.0.0.0"
    end

    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["snmp_v1_v2_en"] = input.snmp_v1_v2_en or snmp_v1_v2_en,
            ["sys_contact"] = input.sys_contact or tmp_config.sys_contact,
            ["sys_name"] = input.sys_name or tmp_config.sys_name,
            ["sys_location"] = input.sys_location or tmp_config.sys_location,
            ["get_community"] = input.get_community or tmp_config.get_community,
            ["get_source"] = input.get_source or tmp_config.get_source,
            ["snmp_v3_en"] = tmp_config.snmp_v3_en,
            ["username"] = tmp_config.username,
            ["password"] = tmp_config.password,
        }
    })

    http_form.form = "snmp"
    ret = snmp.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        result = "FAILED"
    end

    return result
end

function show_snmp_settings()
    local old_data = snmp.get_snmp()
    local snmp_v1_v2_en = old_data.snmp_v1_v2_en or "off"

    dbg.print_tty("SNMPv1-v2c:          "..snmp_v1_v2_en)
    if snmp_v1_v2_en ==  "on" then
        dbg.print_tty("Contact:             "..(old_data.sys_contact or ""))
        dbg.print_tty("Device Name:         "..(old_data.sys_name or ""))
        dbg.print_tty("Location:            "..(old_data.sys_location or ""))
        dbg.print_tty("Get Community:       "..(old_data.get_community or ""))
        dbg.print_tty("Get Trusted Host:    "..(old_data.get_source or ""))
    end

    local snmp_v3_en = old_data.snmp_v3_en or "off"
    dbg.print_tty("SNMPv3:              "..snmp_v3_en)
    if snmp_v3_en == "on" then
        local username = old_data.username or nil
        local password = old_data.password or nil
        dbg.print_tty("Username:            "..username)
        dbg.print_tty("Password:            "..password)
    end

    return "SUCCESS"

end

--arg[1]为cli命令，其它为命令附带的参数
local result = "SUCCESS"
local cmd = arg[1]

if not cmd then
    return "FAILED"
end

if cmd == "snmp-server v3 username password enable" then
    local username = arg[2]
    local password = arg[3]
    result = set_snmp_v3_account(username, password, "on")

elseif cmd == "no snmp-server v3 enable" then
    result = set_snmp_v3_account(nil, nil, "off")

elseif cmd == "snmp-server v1-v2c enable" then
    result = set_snmp_v1_v2("snmp_v1_v2_en", "on")

elseif cmd == "no snmp-server v1-v2c enable" then
    result = set_snmp_v1_v2("snmp_v1_v2_en", "off")

elseif cmd == "snmp-server v1-v2c contact" then
    local contact = arg[2]
    result = set_snmp_v1_v2("sys_contact", contact)

elseif cmd == "snmp-server v1-v2c host" then
    local host = arg[2]
    result = set_snmp_v1_v2("get_source", host)

elseif cmd == "snmp-server v1-v2c device-name" then
    local device_name = arg[2]
    result = set_snmp_v1_v2("sys_name", device_name)

elseif cmd == "snmp-server v1-v2c location" then
    local location = arg[2]
    result = set_snmp_v1_v2("sys_location", location)

elseif cmd == "snmp-server v1-v2c community" then
    local community = arg[2]
    result = set_snmp_v1_v2("get_community", community)

elseif cmd == "no snmp-server v1-v2c contact" then
    result = set_snmp_v1_v2("sys_contact", "")

elseif cmd == "no snmp-server v1-v2c host" then
    result = set_snmp_v1_v2("get_source", "")

elseif cmd == "no snmp-server v1-v2c device-name" then
    result = set_snmp_v1_v2("sys_name", "")

elseif cmd == "no snmp-server v1-v2c location" then
    result = set_snmp_v1_v2("sys_location", "")

elseif cmd == "no snmp-server v1-v2c community" then
    result = set_snmp_v1_v2("get_community", "")

elseif cmd == "show snmp-server" then
    result = show_snmp_settings()
end

io.stdout:write(result)
