local sys  = require "luci.sys"
local json = require "luci.json"
local utl  = require "luci.util"
local dbg  = require "luci.tools.debug"
local form = require "luci.tools.form"
local error_print = require "luci.tools.error_print"
local uci  = require "luci.model.uci"
local diagnose = require "luci.controller.admin.diagnose"

local uci_r = uci.cursor()
local CFG_PATH = "/etc/config"

function set_ssh_server(value)
    local result = "SUCCESS"
    local http_form = {}
    local input = {}
    
    uci_r:set_confdir(CFG_PATH)
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] = 
        {
            ["ssh_port_switch"] = value,
            ["ssh_port"] = "22",
            ["ssh_l3_enable"] = "on",
            ["cli_flag"] = (value == "off") and "0" or "1",
        }
    })
    http_form.form = "setting"
    ret = diagnose.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end

    return result
end

function show_ssh_settings()
    uci_r:set_confdir(CFG_PATH)
    local old_data = diagnose.ssh_get()

    ssh_port_switch = old_data.ssh_port_switch or "off"
    ssh_port        = old_data.ssh_port or "22"
    ssh_l3_enable   = old_data.ssh_l3_enable or "on"

    dbg.print_tty("ssh server:          "..ssh_port_switch)
    dbg.print_tty("ssh port:            "..ssh_port)
    dbg.print_tty("ssh_l3 enable:       "..ssh_l3_enable)

    return "SUCCESS"
end

--arg[1]为cli命令，其它为命令附带的参数
local result = "SUCCESS"
local cmd = arg[1]

if not cmd then
    return "FAILED"
end

if cmd == "ssh server" then
    result = set_ssh_server("on")
elseif cmd == "no ssh server" then
    result = set_ssh_server("off")
elseif cmd == "show ssh configuration" then
    result = show_ssh_settings()
end

io.stdout:write(result)
