--外部引用定义
local sys  = require "luci.sys"
local ctl = require "luci.model.controller"
local json = require 'luci.json'
local utl = require "luci.util"
local uci = require "luci.model.uci"
local dbg  = require "luci.tools.debug"
local form = require "luci.tools.form"
local err = require "luci.tools.error"
local vlan_setting = require "luci.controller.admin.vlan_setting"
local vmod = require "luci.model.vlan"
local error_print = require "luci.tools.error_print"
local uci_r = uci.cursor()

--变量定义
local CLI_CFG_PATH = "/tmp/cli_config"
local CFG_PATH = "/etc/config"
local VLAN_FAIL_STATUS = "FAILED"
local VLAN_SUCCESS_STAUS = "SUCCESS"
local filename = CLI_CFG_PATH .."/vlan_cli"
local vlanfilename = "/tmp/vlan_form.txt"
local portfilename = "/tmp/port_form.txt"
local port_config_file_name = CLI_CFG_PATH.."/port_form.txt"

--函数定义
function show_vlan_summary()
    local result = "SUCCESS"
    local http_form = {}
    local num_static = 0
    local num_dynamic = 0
    local total = 0
    http_form.data = json.encode(
    {
        ["method"] = "get",
        ["params"] = {},
    })
    http_form.form = "vlansettinglist"
    dbg.dumptable(http_form)
    local ret = vlan_setting.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    dbg.print_tty("Number of existed VLANS: "..ret.others.totalNum)
    --当前项目没有动态VLANS
    dbg.print_tty("Number of static VLANS: "..ret.others.totalNum)
    return result
end

function show_vlan()
    local result = "SUCCESS"
    local http_form = {}
    local id_blank = 6
    local name_blank = 10
    local vlan_blank = 9
    local port_blank = 36
    local line_num = 72
    local blank_long = ""
    local line = ""
    for i = 1, port_blank do
        blank_long = blank_long.." "
    end
    for i = 1, line_num do
        line = line.."-"
    end
    http_form.data = json.encode(
    {
        ["method"] = "get",
        ["params"] = {},
    })
    http_form.form = "vlansettinglist"
    dbg.dumptable(http_form)
    local ret = vlan_setting.dispatch(http_form)
    if (not ret) or (ret.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(ret))
        return "FAILED"
    end
    dbg.print_tty(line)
    dbg.print_tty("ID"..string.sub(blank_long, 1, id_blank-#("id")).."VLAN ID"..string.sub(blank_long, 1, vlan_blank-#("VLAN ID")).."NAME"..string.sub(blank_long, 1, name_blank-#("NAME")).."Ports"..string.sub(blank_long, 1, port_blank-#("ports")).."Description")
    dbg.print_tty(line)
    --行打印
    for k,v in pairs(ret.result) do
        local id = tostring(k)
        local name = v["name"]
        local vlan = v["vlan"]
        local ports = ""
        local port_status = string.split(v["ports"], " ")
        for _,v in pairs(port_status) do
            ports = ports..string.match(v,"%d+").."("
            if string.match(v,"%a") == nil then
                ports = ports.."UNTAG) "
            else
                ports = ports.."TAG) "
            end
        end
        local desc = v["desc"]
        dbg.print_tty(id..string.sub(blank_long, 1, id_blank-#(id))..vlan..string.sub(blank_long, 1, vlan_blank-#(vlan))..name..string.sub(blank_long, 1, name_blank-#(name))..ports..string.sub(blank_long, 1, port_blank-#(ports))..desc)
        dbg.print_tty(line)
    end
    return result
end

function create_single_vlan(vlanid)
    local http_form = {}
    local http_table = {
        ["method"] = "add",
        ["params"] = 
        {
            ["new"] = "",
        }
    }
    local new_params_vlan = {
        ["vlan"] = tonumber(vlanid), 
        ["name"] = "", 
        ["desc"] = ""
    }
    local pconfigs = vlan_setting.get_port_setting_group()
    for _, oldp in pairs(pconfigs) do
        if oldp.group[1] == "glan" then
            new_params_vlan["if_onoff"..tostring(oldp.port)] = "1"
            new_params_vlan["tag_type"..tostring(oldp.port)] = "t"
        else
            new_params_vlan["if_onoff"..tostring(oldp.port)] = "0"
        end
    end
    http_table.params.new = new_params_vlan
    http_form.data = json.encode(http_table)
    http_form.form = "vlansettinglist"
    dbg.dumptable(http_form)
    local res = vlan_setting.dispatch(http_form)
    if (not res) or (res.error_code ~= "0")then
        dbg.print_tty(error_print.check_error(res))
        return VLAN_FAIL_STATUS
    end
    return VLAN_SUCCESS_STAUS
end

function create_vlan(section_name)
    local http_form = {}
    local vlanids = string.split(section_name, '-')
    --单独创建
    if #vlanids == 1 then
        if (tonumber(vlanids[1]) < 1) or (tonumber(vlanids[1]) > 4094) then
            dbg.print_tty("Error: Failed to add the vlan.")
            return VLAN_FAIL_STATUS
        end
        create_single_vlan(vlanids[1])
    --批量创建
    else
        local id_min = tonumber(vlanids[1])
        local id_max = tonumber(vlanids[2])
        if id_min > id_max then
            dbg.print_tty("Error: Failed to add the vlan.")
            return VLAN_FAIL_STATUS
        end
        if (id_min < 1) or (id_max > 4094) then
            dbg.print_tty("Error: Failed to add the vlan.")
            return VLAN_FAIL_STATUS
        end
        for i = id_min, id_max do
            create_single_vlan(i)
        end
    end
    return VLAN_SUCCESS_STAUS
end
--删除vlan条目
function delete_vlan(section_name)
    local http_form = {}
    http_form.data = json.encode(
    {
        ["method"] = "delete",
        ["params"] =
        {
            ["key"] = section_name
        }
    })
    http_form.form = "vlansettinglist"
    local res = vlan_setting.dispatch(http_form)
    if (not res) or (res.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(res))
        return VLAN_FAIL_STATUS
    end
    return VLAN_SUCCESS_STAUS
end

--进入vlan视图
function enter_vlan_mode(section_name)
    --判断该vlan是否存在
    local vconfigs = vmod.call_vlan("read_vlan_config", tostring(section_name))
    if vconfigs and not vconfigs.vlan then
        dbg.print_tty("Error: Failed to enter the vlan mode.")
        return VLAN_FAIL_STATUS
    end
    local vlanfile = io.open(vlanfilename,"r")
    if vlanfile == nil then
        sys.exec("touch " .. vlanfilename)
    else
        vlanfile:close()
    end
    local test = assert(io.open(vlanfilename, "w"))
    local vlan_form = json.encode(vconfigs)
    test:write(vlan_form)
    test:close()
    return VLAN_SUCCESS_STAUS
end

--vlan视图命令，为vlan创建名字
function create_name_for_vlan(section_name)
    local test = io.open(vlanfilename, "r")
    local readjson = test:read("*a")
    local vlan_form = json.decode(readjson) 
    test:close()
    --判断该vlan是否存在
    local vconfig = vmod.call_vlan("read_vlan_config", vlan_form["vlan"])
    if vconfig and not vconfig.vlan then
        dbg.print_tty("Error: Failed to name the vlan.")
        return VLAN_FAIL_STATUS
    end
    local http_table = {
        ["method"] = "set",
        ["params"] =
        {
            ["vlan"] = vlan_form["vlan"],
            ["old"] = vconfig,
        }
    }
    local params = http_table["params"]
    local newvlan = {
        ["mod"] = vconfig["mod"] ,
        ["name"] = section_name, 
        ["prio"] = vconfig["prio"] ,
        ["users"] = vconfig["users"] ,
        ["vlan"] = vlan_form["vlan"],
        ["desc"] =vconfig["desc"] , 
        ["ports"] = vconfig["ports"] ,
    }
    local ports_cur = string.split(vconfig["ports"], ' ')
    local port_tag_cur
    for _, port_cur in ipairs(ports_cur) do
        port_tag_cur = string.match(port_cur, "(%d+)t")
        if port_tag_cur then
            newvlan["if_onoff"..tostring(port_tag_cur)] = '1'
            newvlan["tag_type"..tostring(port_tag_cur)] = 't'
        else
            newvlan["if_onoff"..tostring(port_cur)] = '1'
            newvlan["tag_type"..tostring(port_cur)] = 'u'
        end
    end
    http_table.params.new = newvlan
    local http_form = {}
    http_form.data = json.encode(http_table)
    http_form.form = "vlansettinglist"
    local res = vlan_setting.dispatch(http_form)
    if (not res) or (res.error_code ~= "0")then
        dbg.print_tty(error_print.check_error(res))
        return VLAN_FAIL_STATUS
    end
    return VLAN_SUCCESS_STAUS
end

--vlan视图命令，删除名字
function delete_name_for_vlan()
    local test = io.open(vlanfilename, "r")
    local readjson = test:read("*a")
    local vlan_form = json.decode(readjson) 
    test:close()
    local vconfig = vmod.call_vlan("read_vlan_config", vlan_form["vlan"])
    if vconfig and not vconfig.vlan then
        dbg.print_tty("Error: Failed to delete the name.")
        return VLAN_FAIL_STATUS
    end
    local http_table = {
        ["method"] = "set",
        ["params"] =
        {
            ["vlan"] = vlan_form["vlan"],
            ["old"] = vconfig,
        }
    }
    local params = http_table["params"]
    local newvlan = {
        ["mod"] = vconfig["mod"] ,
        ["name"] = "", 
        ["prio"] = vconfig["prio"] ,
        ["users"] = vconfig["users"] ,
        ["vlan"] = vlan_form["vlan"],
        ["desc"] =vconfig["desc"] , 
        ["ports"] = vconfig["ports"] ,
    }
    local ports_cur = string.split(vconfig["ports"], ' ')
    local port_tag_cur
    for _, port_cur in ipairs(ports_cur) do
        port_tag_cur = string.match(port_cur, "(%d+)t")
        if port_tag_cur then
            newvlan["if_onoff"..tostring(port_tag_cur)] = '1'
            newvlan["tag_type"..tostring(port_tag_cur)] = 't'
        else
            newvlan["if_onoff"..tostring(port_cur)] = '1'
            newvlan["tag_type"..tostring(port_cur)] = 'u'
        end
    end
    http_table.params.new = newvlan
    local http_form = {}
    http_form.data = json.encode(http_table)
    http_form.form = "vlansettinglist"
    local res = vlan_setting.dispatch(http_form)
    if (not res) or (res.error_code ~= "0") then
        dbg.print_tty(error_print.check_error(res))
        return VLAN_FAIL_STATUS
    end
    return VLAN_SUCCESS_STAUS
end
--从vlan视图进入port视图
function enter_port_mode_by_vlan(section_name)
    local test = io.open(vlanfilename, "r")
    local readjson = test:read("*a")
    local vlan_form = json.decode(readjson) 
    test:close()
    local vconfig = vmod.call_vlan("read_vlan_config", vlan_form["vlan"])
    if vconfig and not vconfig.vlan then
        dbg.print_tty("Error: Failed to enter the port mode.")
        return VLAN_FAIL_STATUS
    end
    local ports_cur = string.split(vconfig["ports"], ' ')
    local port_tag_cur
    for _, port_cur in ipairs(ports_cur) do
        port_tag_cur = string.match(port_cur, "(%d+)t")
        if tostring(section_name) == port_cur or tostring(section_name) == port_tag_cur then
            local pconfigs = vmod.call_vlan("read_port_config", tostring(section_name))
            local portfile = io.open(portfilename,"r")
            if portfile == nil then
                sys.exec("touch " .. portfilename)
            else
                portfile:close()
            end
            local test = assert(io.open(portfilename, "w"))
            local port_form = json.encode(pconfigs)
            test:write(port_form)
            test:close()
            return VLAN_SUCCESS_STAUS
        end
    end
    dbg.print_tty("Error: Failed to enter the port mode.")
    return VLAN_FAIL_STATUS
end

--port视图命令，修改port接收帧格式
function switchport_acceptable_frame(section_name)
    local http_form = {}
    local test_vlan = io.open(vlanfilename, "r")
    local readjson = test_vlan:read("*a")
    local vlan_form = json.decode(readjson) 
    test_vlan:close()
    local test_port = io.open(portfilename, "r")
    local readjson = test_port:read("*a")
    local port_form = json.decode(readjson) 
    test_port:close()
    local vconfig = vmod.call_vlan("read_vlan_config", vlan_form["vlan"])
    local pconfig = vmod.call_vlan("read_port_config", port_form["port"])
    local http_table = {
        ["method"] = "set",
        ["params"] =
        {
            ["vlan"] = vlan_form["vlan"],
            ["old"] = vconfig,
        }
    }
    local params = http_table["params"]
    local newvlan = {
        ["mod"] = vconfig["mod"] ,
        ["name"] = vconfig["name"], 
        ["prio"] = vconfig["prio"] ,
        ["users"] = vconfig["users"] ,
        ["vlan"] = vlan_form["vlan"],
        ["desc"] =vconfig["desc"] , 
        ["ports"] = vconfig["ports"] ,
    }
    local ports_cur = string.split(vconfig["ports"], ' ')
    local port_tag_cur
    for _, port_cur in ipairs(ports_cur) do
        port_tag_cur = string.match(port_cur, "(%d+)t")
        if port_tag_cur then
            newvlan["if_onoff"..tostring(port_tag_cur)] = '1'
            newvlan["tag_type"..tostring(port_tag_cur)] = 't'
        else
            newvlan["if_onoff"..tostring(port_cur)] = '1'
            newvlan["tag_type"..tostring(port_cur)] = 'u'
        end
    end
    if section_name == "untagged" then
        newvlan["tag_type"..port_form["port"]] = "u"
        http_table.params.new = newvlan
        http_form.data = json.encode(http_table)
        http_form.form = "vlansettinglist"
        local res = vlan_setting.dispatch(http_form)
        if (not res) or (res.error_code ~= "0") then
            dbg.print_tty(error_print.check_error(res))
            return VLAN_FAIL_STATUS
        end
        return VLAN_SUCCESS_STAUS
    elseif section_name == "tagged" then
        newvlan["tag_type"..port_form["port"]] = "t"
        http_table.params.new = newvlan
        http_form.data = json.encode(http_table)
        http_form.form = "vlansettinglist"
        local res = vlan_setting.dispatch(http_form)
        if (not res) or (res.error_code ~= "0") then
            dbg.print_tty(error_print.check_error(res))
            return VLAN_FAIL_STATUS
        end
        return VLAN_SUCCESS_STAUS
    end
end

--interface视图命令，修改port pvid
function switchport_pvid(section_name)
    local http_form = {}
    local test = io.open(port_config_file_name, "r")
    if test == nil then 
        dbg.print_tty("Error: Failed to set the pvid.")
        return VLAN_FAIL_STATUS
    end
    local readjson = test:read("*a")
    local port_form = json.decode(readjson) 
    test:close()
    local pconfig = vmod.call_vlan("read_port_config", port_form["port"])
    http_form.data = json.encode(
    {
        ["method"] = "set",
        ["params"] =
        {
            ["newp"] = 
            {
                ["port"] = pconfig["port"],
                ["type"] = pconfig["type"],
                ["tag"] = pconfig["tag"],
                ["pvid"] = tostring(section_name),
            }
        }
    })
    http_form.form = "portsettinglist"
    local res = vlan_setting.dispatch(http_form)
    if (not res) or (res.error_code ~= "0") then 
        dbg.print_tty(error_print.check_error(res))
        return VLAN_FAIL_STATUS
    end
    return VLAN_SUCCESS_STAUS
end

--代码逻辑调用
local file = io.open(filename,"r")
local vlanid_cur = 0
if file == nil then
    sys.exec("touch " .. filename)
else
    file:close()
end
local vlanfile = io.open(vlanfilename,"r")
if vlanfile == nil then
    sys.exec("touch " .. vlanfilename)
else
    vlanfile:close()
end
local portfile = io.open(portfilename,"r")
if portfile == nil then
    sys.exec("touch " .. portfilename)
else
    portfile:close()
end

local cmd = arg[1]
if not cmd then
    return VLAN_FAIL_STATUS
end
if arg[1] == "vlan" then
    local section_name = arg[2]
    result = create_vlan(section_name)
elseif arg[1] == "no vlan" then
    local section_name = arg[2]
    result = delete_vlan(section_name)
elseif arg[1] == "router vlan" then
    local section_name = arg[2]
    result = enter_vlan_mode(section_name)
elseif arg[1] == "name" then
    local section_name = arg[2]
    result = create_name_for_vlan(section_name)
elseif arg[1] == "no name" then
    result = delete_name_for_vlan()
elseif arg[1] == "router port" then
    local section_name = arg[2]
    result = enter_port_mode_by_vlan(section_name)
elseif arg[1] == "switchport acceptable frame" then
    local section_name = arg[2]
    result = switchport_acceptable_frame(section_name)
elseif arg[1] == "switchport pvid" then
    local section_name = arg[2]
    result = switchport_pvid(section_name)
elseif arg[1] == "show vlan summary" then
    result = show_vlan_summary()
elseif arg[1] == "show vlan" then
    result = show_vlan()
end

if result == VLAN_SUCCESS_STAUS then
    uci_r:commit("vlan_cli")
end
io.stdout:write(result)

