#!/bin/sh

dhcps_option_add()
{
	local section="$1"
	local value="$2"
	local configfile="$3"
	echo -e "\toption $section '$value'" >> $configfile
}

dhcps_list_add()
{
	local list="$1"
	local value="$2"
	local configfile="$3"
	echo "	list $list '$value'" >> $configfile
}

build_dhcps_config()
{
	local config="$1"
	config_get dhcp_on "$config" dhcp_on
	[ -z "$dhcp_on" ] && {
		return 0
	}
	
	local CONFIGFILE="$2"

	config_get netmask "$config" netmask ''
	config_get dhcp_start "$config" dhcp_start ''
	config_get dhcp_end "$config" dhcp_end ''
	config_get dhcp_release_time "$config" dhcp_release_time ''
	config_get dhcp_gateway "$config" dhcp_gateway ''
	config_get dhcp_dns "$config" dhcp_dns
	config_get dhcp_controller "$config" dhcp_controller ''
	config_get ifname "$config" ifname
	config_get dhcp_relay_on "$config" dhcp_relay_on
	config_get dhcp_relay_server "$config" dhcp_relay_server
	config_get zone "$config" zone

 	[ -f $CONFIGFILE ] && {
		sed -i -e "/config vinterface '$config'/,/option ifname '$ifname'/d" $CONFIGFILE
	}
	echo -e "config vinterface '$config'" >> $CONFIGFILE
	
	if [ $dhcp_on = "0" ];then
		dhcps_option_add ignore '1' $CONFIGFILE
	else
		dhcps_option_add ignore '0' $CONFIGFILE
	fi
	
	if [ -n "$dhcp_gateway" ];then
		dhcps_list_add dhcp_option "3,$dhcp_gateway" $CONFIGFILE
	fi
	
	if [ -n "$dhcp_dns" ];then
		dhcps_list_add dhcp_option "6,$dhcp_dns" $CONFIGFILE
	fi
	
	ret=`lua /lib/dhcps/get_option.lua "$config"`
	if [ -n "$ret" ];then
		for i in $ret
		do
			index=${i:11}
			config_get dhcp_option "$config" "$i" ''
			if [ -n "$dhcp_option" ];then
				dhcps_list_add dhcp_option "$index,$dhcp_option" $CONFIGFILE
			fi
		done
	fi


	if [ -z "$dhcp_start" ];then
		dhcps_option_add dhcp_start '0.0.0.0' $CONFIGFILE
	else
		dhcps_option_add dhcp_start $dhcp_start $CONFIGFILE
	fi
	
	if [ -z "$dhcp_end" ];then
		dhcps_option_add dhcp_end '0.0.0.0' $CONFIGFILE
	else
		dhcps_option_add dhcp_end $dhcp_end $CONFIGFILE
	fi
	
	if [ -z "$netmask" ];then
		dhcps_option_add netmask '255.255.255.0' $CONFIGFILE
	else
		dhcps_option_add netmask $netmask $CONFIGFILE
	fi
	
	if [ -z "$dhcp_release_time" ];then
		dhcps_option_add dhcp_release_time '120m' $CONFIGFILE
	else
		dhcps_option_add dhcp_release_time $dhcp_release_time $CONFIGFILE
	fi
	
	dhcps_option_add dynamicdhcp '1' $CONFIGFILE
	dhcps_option_add addrtype 'iprange' $CONFIGFILE
	dhcps_option_add addrtype_v2 'ippool' $CONFIGFILE
	dhcps_option_add interface 'lan' $CONFIGFILE
	if [ -n "$dhcp_relay_on" ];then
		dhcps_option_add dhcp_relay_on $dhcp_relay_on $CONFIGFILE
	fi
	if [ -n "$dhcp_relay_server" ];then
		dhcps_option_add dhcp_relay_server $dhcp_relay_server $CONFIGFILE
	fi
	if [ -n "$zone" ];then
		dhcps_option_add zone $zone $CONFIGFILE
	fi
	# it should be set last so that we could override the config
	dhcps_option_add ifname $ifname $CONFIGFILE

	chmod +x $CONFIGFILE
	
}

build_dhcps_dnsmasq()
{
	local CONFIGFILE="$1"
	sed -i "/config dnsmasq/,/option nonwildcard*/d" $CONFIGFILE
	
	echo "config dnsmasq" >> $CONFIGFILE
	
	dhcps_option_add domainneeded '1' $CONFIGFILE
	dhcps_option_add port '0' $CONFIGFILE
	dhcps_option_add boguspriv '1' $CONFIGFILE
	dhcps_option_add filterwin2k '0' $CONFIGFILE
	dhcps_option_add localise_queries '1' $CONFIGFILE
	dhcps_option_add rebind_protection '1' $CONFIGFILE
	dhcps_option_add rebind_localhost '1' $CONFIGFILE
	dhcps_option_add local '/lan/' $CONFIGFILE
	dhcps_option_add domain 'lan' $CONFIGFILE
	dhcps_option_add expandhosts '1' $CONFIGFILE
	dhcps_option_add nonegcache '0' $CONFIGFILE
	dhcps_option_add authoritative '1' $CONFIGFILE
	dhcps_option_add readethers '1' $CONFIGFILE
	dhcps_option_add leasefile '/tmp/dhcp.leases' $CONFIGFILE
	dhcps_option_add resolvfile '/tmp/resolv.conf.auto' $CONFIGFILE
	dhcps_option_add nonwildcard '1' $CONFIGFILE

}

clean_dhcps_config()
{
	local config="$1"
	
	local CONFIGFILE="$2"
	
 	[ -f $CONFIGFILE ] && {
		config_load dhcp
		config_get ifname "$config" ifname
		sed -i -e "/config vinterface '$config'/,/option ifname '$ifname'/d" $CONFIGFILE
		chmod +x $CONFIGFILE
	}
	
}
