#!/bin/sh

local ETC_DIR="/etc/unbound"
local VAR_DIR="/var/unbound/"
local CONF_BACK="/etc/unbound/unbound.conf.back"
local CONF_VAR="/var/unbound/unbound.conf"
local CA_BUNDLE_PATH="/etc/ssl/certs/ca-certificates.crt"
local dnsproxy_online_file="/tmp/online/dnsproxy_state"
local CTL_CMD="/usr/sbin/unbound-control"
local PID_VAR="/var/unbound/unbound.pid"
local port1=5301
local port2=5302

update_cache()
{
    local switch="$1"
    local ttl="$2"
    [ ! -n "$switch" ] &&  switch="$(uci get dnsproxySecurity.dnsproxy.cache_switch)"
    [ ! -n "$ttl" ] && ttl="$(uci get dnsproxySecurity.dnsproxy.cache_ttl)"
    [ -n "$switch" -a "$switch" == "off" ] && {
        sed -i '/.*cache-max-ttl:*/c\        cache-max-ttl: 0' $CONF_VAR
        sed -i '/.*cache-min-ttl:*/c\        cache-min-ttl: 0' $CONF_VAR
        return
    }
    [ -n "$switch" -a "$switch" == "on" ] && {
        sed -i '/.*cache-max-ttl:*/c\        cache-max-ttl: 86400' $CONF_VAR
    }

    if [ -n "$ttl" -a "$ttl" != "0" ] ;then
        sed -i "/.*cache-min-ttl:*/c\        cache-min-ttl: ${ttl}" $CONF_VAR
        sed -i "/.*cache-max-ttl:*/c\        cache-max-ttl: ${ttl}" $CONF_VAR
    else
        sed -i '/.*cache-min-ttl:*/c\        cache-min-ttl: 0' $CONF_VAR
    fi
}


build_unbound_conf() {
    [ ! -d "$VAR_DIR" ] && mkdir -p ${VAR_DIR}
    rm -f $VAR_DIR/*
    cp -r $ETC_DIR/* $VAR_DIR
    cp -r $CA_BUNDLE_PATH $VAR_DIR
    cp $CONF_BACK $CONF_VAR

    #更新cache配置
    update_cache

    #检查 doh 开关
    switch=`uci get dnsproxySecurity.dnsproxy.doh_switch`
    [ "$switch" = "on" ] && {
        echo "enable unbound for doh."
        local servers=`uci get dnsproxySecurity.dnsproxy.server`

        local i=0
        local tmp
        for server in $servers
        do
            i=`expr $i + 1`
            [ $i -gt 2 ] && break
            tmp=`eval echo '$'"port$i"`
            sed -i "/.*forward-zone*/a\        forward-addr: 169.254.11.21@${tmp}" $CONF_VAR
            #sed -i '/.*module-config*/c\        module-config: "iterator"' $CONF_VAR
            echo RunCmd="https-dns-proxy -d -u nobody -g nogroup -a 169.254.11.21 -p $tmp -r $server -b 8.8.8.8,8.8.4.4"
            https-dns-proxy -d -u nobody -g nogroup -4 -x -a 169.254.11.21 -p $tmp -r $server -b 8.8.8.8,8.8.4.4
            #https-dns-proxy -d -u nobody -g nogroup -a 169.254.11.21 -p 53 -r https://dns.google/dns-query -b 8.8.8.8,8.8.4.4
        done
        return
    }

    #检查 dnssec 开关
    local switch=`uci get dnsproxySecurity.dnsproxy.dnssec_switch`
    [ "$switch" = "on" ] && {
        cp /etc/unbound/root.key /var/unbound/root.key
        sed -i '/.*auto-trust-anchor-file*/c\        auto-trust-anchor-file: "/var/unbound/root.key"' $CONF_VAR
        local bogus=`uci get dnsproxySecurity.dnsproxy.bogus_accept`
        [ "$bogus" = "pass" ] &&{
            sed -i '/.*val-permissive-mode*/c\        val-permissive-mode: yes' $CONF_VAR
        }
        local servers=`uci get dnsproxySecurity.dnsproxy.server`
        for server in $servers
        do
            sed -i "/.*forward-zone*/a\        forward-addr: ${server}" $CONF_VAR
        done
        return
    }

    #检查 dot 开关
    switch=`uci get dnsproxySecurity.dnsproxy.dot_switch`
    [ "$switch" = "on" ] && {
        [ -n "$real_outgoingIF" ] && {
            local real_outgoingIP=""
        }
        #sed -i '/.*module-config*/c\        module-config: "iterator"' $CONF_VAR
        sed -i '/.*tls-cert-bundle*/c\        tls-cert-bundle:"/var/unbound/ca-certificates.crt"' $CONF_VAR
        sed -i '/.*forward-tls-upstream*/c\        forward-tls-upstream: yes' $CONF_VAR
        #sed -i '/.*module-config*/c\        module-config: "iterator"' $CONF_VAR
        local servers=`uci get dnsproxySecurity.dnsproxy.server`
        for server in $servers
        do
            sed -i "/.*forward-zone*/a\        forward-addr: ${server}@853" $CONF_VAR
        done
        return
    }
    #更新dns server
    [ -f "/etc/resolv.conf" ] && {
        local dns_t=`cat /etc/resolv.conf  | grep nameserver | grep '[\.]' |awk '{print $2}'` #only support ipv4
        for server in ${dns_t} ;do
            sed -i "/.*forward-zone*/a\        forward-addr: ${server}" $CONF_VAR
        done
    }
    return

}


int_to_ip()
{
local IP=$1
local H1=$(($IP & 0x000000ff))
local H2=$((($IP & 0x0000ff00) >> 8))
local L1=$((($IP & 0x00ff0000) >> 16))
local L2=$((($IP & 0xff000000) >> 24))
echo "$H1.$H2.$L1.$L2"

}

add_dns_server()
{
    local iface="$1"
    shift
    local dns_t="$@"
    #echo "${dns_t}" >/dev/console
        
    for server in ${dns_t};do
        [ "$server" == "0" ] && continue
        local addr=`int_to_ip ${server}`
        sed -i "/.*forward-zone*/a\        forward-addr: ${addr}" $CONF_VAR
        local vpn_if=`echo ${iface}|grep '^ppp-'`
        if [ -n "$vpn_if" ];then
            echo "$vpn_if is vpn iface" >/dev/console
            vpn_if=${vpn_if:4}
            local real_if=`ifconfig |grep -w "lt\-${vpn_if}"|awk '{print $1}'`
            if [ -n "$real_if" ] ;then
                ip route add $addr dev $real_if
            else
                real_if=`ifconfig |grep -w "pt\-${vpn_if}"|awk '{print $1}'`
                [ -n "$real_if" ] && ip route add $addr dev $real_if
            fi
        else
            vpn_if=`echo ${iface}|grep '^tun'`
            if [ -n "$vpn_if" ];then
                echo "$vpn_if is vpn iface" >/dev/console
                ip route add $addr dev $vpn_if
            fi
        fi
        
    done

#    /etc/init.d/dnsproxy_security restart
}

del_all_dns_server()
{
    sed -i '/forward-addr:/d' $CONF_VAR

#    /etc/init.d/dnsproxy_security restart
}

del_dns_server()
{
    local iface="$1"
    shift
    local dns_t="$@"
    for server in ${dns_t};do
        [ "$server" == "0" ] && continue
        local addr=`int_to_ip ${server}`
        sed -i "/forward-addr: ${addr}/d" $CONF_VAR
        local vpn_if=`echo ${iface}|grep '^ppp-'`
        if [ -n "$vpn_if" ];then
            echo "$vpn_if is vpn iface" >/dev/console
            vpn_if=${vpn_if:4}
            local real_if=`ifconfig |grep -w "lt\-${vpn_if}"|awk '{print $1}'`
            if [ -n "$real_if" ] ;then
                ip route delete $addr dev $real_if
            else
                real_if=`ifconfig |grep -w "pt\-${vpn_if}"|awk '{print $1}'`
                [ -n "$real_if" ] && ip route delete $addr dev $real_if
            fi
        fi
    done

#    /etc/init.d/dnsproxy_security restart
}

flush_cache()
{
    $CTL_CMD -c $CONF_VAR flush_zone .
}

dump_v4()
{
    $CTL_CMD -c $CONF_VAR dump_tp_ip4_cache
}

dump_v6()
{
    $CTL_CMD -c $CONF_VAR dump_tp_ip6_cache
}

dns_reload()
{
    echo "reload dnsproxy_security.">/dev/console

    if [ -f "$PID_VAR" ]; then
        kill -9 `cat ${PID_VAR}` && rm ${PID_VAR}
    else
        pids=`pgrep unbound`
        for pid in $pids
        do
            kill -9 $pid
        done
    fi

    unbound -c $CONF_VAR
	[ "$?" -ne 0 ] && build_unbound_conf && unbound -c $CONF_VAR

}
