#!/bin/sh

. /lib/functions.sh
. /lib/firewall/fw.sh
. /lib/firewall/core_tpfirewall.sh
. /lib/functions/network.sh
. /lib/zone/zone_api.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
#GREVPN使用和PPTP/L2TP相同的自定义规则链
VPN_RULE_NUM=25
VPN_FWMARK=0x20000
VPN_FORWARD_TABLE=VPN_FORWARD_MARK
VPN_ROUTING_TABLE=VPN_ROUTING_MARK
VPN_INPUT_RULE=gre_input_rule
GRE_PRESERVICE_RULE=gre_preservice_rule
GRE_SERVICE_RULE=gre_service_rule


add_gre_service()
{
	local ifconfigall=$(ifconfig $TUNNEL_NAME)
	local iplocal=${ifconfigall#*inet addr:}
	iplocal=${iplocal%% *}
	[ -z "$iplocal" ] && return
	iptables -w -t raw -I $GRE_PRESERVICE_RULE -i $TUNNEL_NAME -p udp --dport 53 -d $1 -j CT --notrack -m comment --comment "if=${TUNNEL_NAME}"
	iptables -w -t raw -I $GRE_PRESERVICE_RULE -i $TUNNEL_NAME -p tcp --dport 53 -d $1 -j CT --notrack -m comment --comment "if=${TUNNEL_NAME}"
	iptables -w -t nat -A $GRE_SERVICE_RULE -o $TUNNEL_NAME -p udp --sport 53 -s $iplocal -j SNAT --to-source $1 -m comment --comment "if=${TUNNEL_NAME}"
	iptables -w -t nat -A $GRE_SERVICE_RULE -o $TUNNEL_NAME -p tcp --sport 53 -s $iplocal -j SNAT --to-source $1 -m comment --comment "if=${TUNNEL_NAME}"
}

find_lanip()
{
	local section=$1
	local system=`uci get vnetwork.$section.system`
	[ "$system" == "1" ] && lanip=`uci get vnetwork.$section.ipaddr`
}


do_service_gre_up()
{
	#echo "dns1: $dns1 dns2 $dns2" > /dev/console
	local lanip="169.254.11.22"
	config_load vnetwork
	config_foreach find_lanip vinterface
	[ "$lanip" == "169.254.11.22" ] && return
	[ -n "$lanip" ] && add_gre_service "$lanip" 
}

do_service_gre_down()
{
	nums=$(iptables -w -t nat -nvL $GRE_SERVICE_RULE --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t nat -D $GRE_SERVICE_RULE ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL $GRE_SERVICE_RULE --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
	done

	nums=$(iptables -w -t raw -nvL $GRE_PRESERVICE_RULE --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
		while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t raw -D $GRE_PRESERVICE_RULE ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t raw -nvL $GRE_PRESERVICE_RULE --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
	done
}

get_mss_for_vpn_to_internet()
{
    local WAN_NAME=$1
    local encryption=$2

    local effect_iface=$(zone_get_effect_devices $WAN_NAME)
    local ifconfigall=$(ifconfig $effect_iface)
    local gre_mtu=1400
    local mtu=${ifconfigall#*MTU:}
    mtu=${mtu%% *}
    local mss

    if [ -z ${mtu} ]; then
        echo "1088" && return
    fi

    if [ $mtu -ge $gre_mtu ]; then
        let mss=$gre_mtu
    else
        let mss=$mtu
    fi

    let mss=$mss-80

    if [ "$encryption" = "encrypted" ]; then
        let mss=${mss}-60
    fi

    echo "$mss"  # 作为函数返回值，不能注释
}

route_table_init()
{
	ret=`ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		ip rule add pref $VPN_RULE_NUM fwmark ${VPN_FWMARK}/${VPN_FWMARK} table $VPN_RULE_NUM
	fi
}

route_table_create()
{  
    TUNNEL_NAME=$1
    remote_ip=$2
    local_subnet=$3
    remote_subnet=$4
    local_gre_ip=$5
    cur_mss=$6

    #add iptables rules
    iptables -w -A gre_forward_rule -i br-lan -o ${TUNNEL_NAME} -s ${local_subnet} -m comment --comment "if=${TUNNEL_NAME}" -j ACCEPT
    iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -i br-lan -s ${local_subnet} -d ${remote_subnet} -m comment --comment "if=${TUNNEL_NAME}" -j MARK --or-mark ${VPN_FWMARK}
	iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -i br-lan -s ${local_subnet} -d ${remote_subnet} -m comment --comment "if=${TUNNEL_NAME}" -j CONNMARK --or-mark ${VPN_FWMARK}
	iptables -w -t mangle -A ${VPN_ROUTING_TABLE} -i ${TUNNEL_NAME} -s ${remote_subnet} -m set --match-set $(zone_allwan_group) dst -m comment --comment "if=${TUNNEL_NAME}" -j CONNMARK --or-mark ${VPN_FWMARK}
	iptables -w -t mangle -A ${VPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY  -m comment --comment "if=${TUNNEL_NAME}" -m connmark --mark ${VPN_FWMARK}/${VPN_FWMARK} -j MARK --or-mark ${VPN_FWMARK}
    iptables -w -t filter -I manage_devs 1 -o ${TUNNEL_NAME} -j RETURN

    #vpn to internet
    iptables -w -t nat -A postrouting_rule -s ${remote_subnet} -o br-lan -m comment --comment "if=${TUNNEL_NAME}" -j RETURN
    iptables -w -t nat -A postrouting_rule ! -o ${TUNNEL_NAME} -s ${remote_subnet} -m comment --comment "if=${TUNNEL_NAME}" -j MASQUERADE
    local chain=zone_vpn_${TUNNEL_NAME}_MSSFIX
    local rule="-p tcp -m tcp --tcp-flags SYN,RST SYN"
    local target_size="TCPMSS --set-mss ${cur_mss}"
    local target_mtu="TCPMSS --clamp-mss-to-pmtu"
    fw add 4 m ${chain}
    fw add 4 m zone_MSSFIX ${chain} ^ { -o ${TUNNEL_NAME} }
    fw add 4 m zone_MSSFIX ${chain} ^ { -i ${TUNNEL_NAME} }
    fw add 4 m ${chain} "$target_size" $ { "$rule" }
    fw add 4 m ${chain} "$target_mtu" $ { "$rule" }
    do_service_gre_up
    [ -n "$TUNNEL_NAME" ] && /lib/dnsproxy/api.sh add "$TUNNEL_NAME"
}

route_table_delete()
{  
    TUNNEL_NAME=$1
    remote_ip=$2

    #del iptables rules
    num=$(iptables -w -nL gre_forward_rule --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
    iptables -w -D gre_forward_rule ${num}

    tables="${VPN_FORWARD_TABLE} ${VPN_ROUTING_TABLE}"
    for tbl in $tables
    do
        nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
        while [ -n "${nums}" ]; do
            for num in $nums;
            do
                iptables -w -t mangle -D ${tbl} ${num}
                break; # just delete the first once.
            done
            nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
        done
    done

    #vpn to internet
    nums=$(iptables -w -t nat -nL postrouting_rule --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
    while [ -n "${nums}" ]; do
        for num in $nums;
        do
            iptables -w -t nat -D postrouting_rule ${num}
            break; # just delete the first once.
        done
        nums=$(iptables -w -t nat -nL postrouting_rule --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
    done
    local chain=zone_vpn_${TUNNEL_NAME}_MSSFIX
    fw flush 4 m ${chain}
    fw del 4 m zone_MSSFIX ${chain} ^ { -o ${TUNNEL_NAME} }
    fw del 4 m zone_MSSFIX ${chain} ^ { -i ${TUNNEL_NAME} }
    fw del 4 m ${chain}

    num=$(iptables -w -t filter -nvL manage_devs --line-numbers | grep "\b${TUNNEL_NAME}\b" | cut -d ' ' -f 1)
    iptables -w -t filter -D manage_devs ${num}
    do_service_gre_down
    [ -n "$TUNNEL_NAME" ] && /lib/dnsproxy/api.sh delete "$TUNNEL_NAME"
}

#input like "WAN1"
get_ipaddr_by_wan_name()
{
    wan_name=$1
    ipaddr=""
    local effect_iface=$(zone_get_effect_main_wan_ifaces ${wan_name})
	network_get_ipaddr ipaddr ${effect_iface}
    echo "$ipaddr"  # 作为函数返回值，不能注释
}

gre_create()
{
    TUNNEL_NAME=$1
    WAN_NAME=$2
    remote_ip=$3
    local_subnet=$4
    remote_subnet=$5
    local_gre_ip=$6
    remote_gre_ip=$7
    encryption=$8
    
    local_ip=`get_ipaddr_by_wan_name ${WAN_NAME}`
    mss=`get_mss_for_vpn_to_internet ${WAN_NAME} ${encryption}`

    if [ -z ${local_ip} ]; then
        echo "There is no avaliable ipaddr for ${WAN_NAME}." > /dev/console && return 
    fi

    ip tunnel add ${TUNNEL_NAME} mode gre local ${local_ip} remote ${remote_ip} ttl 255
    ip addr add ${local_gre_ip} peer ${remote_gre_ip} dev ${TUNNEL_NAME}
    ip link set ${TUNNEL_NAME} mtu 1400
    route_table_create ${TUNNEL_NAME} ${remote_ip} ${local_subnet} ${remote_subnet} ${local_gre_ip} ${mss}
    zone_add_vgroup "${TUNNEL_NAME}" "${WAN_NAME}"
}

gre_delete()
{
    TUNNEL_NAME=$1
    remote_ip=$2

    ip tunnel del ${TUNNEL_NAME}
    route_table_delete ${TUNNEL_NAME} ${remote_ip}
    zone_delete_vgroup "${TUNNEL_NAME}"
}

set_gre_link()
{
    TUNNEL_NAME=$1
    STATUS=$2
    remote_subnet=$3
    local_gre_ip=$4

    if [ -z "$(ip a | grep ${TUNNEL_NAME})" ];then
        echo "no such a tunnel" > /dev/console && return 
    fi
    case "${STATUS}" in
		on)
            ip link set dev ${TUNNEL_NAME} up
            #add route rules
            local remote_subnet_network=`ipcalc -n ${remote_subnet} | cut -d "=" -f 2` 
            local remote_subnet_prefix=`echo ${remote_subnet} | cut -d "/" -f 2`
            local remote="${remote_subnet_network}/${remote_subnet_prefix}"
            ip route add ${remote} via ${local_gre_ip} dev ${TUNNEL_NAME} metric 0 table $VPN_RULE_NUM
        ;;
        off)
            ip link set dev ${TUNNEL_NAME} down
        ;;
    esac
    
}
