#!/bin/sh /etc/rc.common
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file     ipv6group config.sh
# brief    
# author   Chen chen
# version  1.0.0
# date     11June15
# histry   arg 1.0.0, 08June15, Chen chen, Create the file.

. /lib/functions.sh
. /lib/vnet/vnet_init.sh

ipv6group_config_append() { # <package>
	CONFIG_APPEND=1 config_load "$@"
	unset CONFIG_APPEND
}

ipv6group_config_get_section() { # <config> <prefix> <type> <name> <default> ...
	local config=$1
	local prefix=$2
	shift 2

	[ -n "$config" ] || return 1
	[ -n "$prefix" ] && {
        prefix="${prefix}_"
    }

	#local enabled
	#config_get_bool enabled "$config" enabled 1
	#[ $enabled -eq 1 ] || return 1

	[ "$1" == '{' ] && shift
	while [ $# -ge 3 ]; do
		local type=$1
		local name=$2
		local dflt=$3
		shift 3
		# TODO: Move handling of defaults to /lib/functions.sh
		# and get replace the case block with the following 
		# two lines:
		# type=${type#string}
		# config_get${type:+_${type}} "${prefix}${name}" "$config" "$name" "$dflt" || return
		case "$type" in
			string)
				local tmp
				config_get tmp "$config" "$name" || return
				[ -z "$tmp" ] && tmp=$dflt
				export ${NO_EXPORT:+-n} -- "${prefix}${name}=${tmp}"
				continue
			;;
			boolean)
				type=bool
			;;
		esac;
		
		local cmd=${prefix}config_get_${type}
		type $cmd > /dev/null || {
			cmd=config_get_${type} 
		}
		type $cmd > /dev/null || {
			echo "config type $type (for $name) not supported" >&2
			return 1
		}
		$cmd "${prefix}${name}" "$config" "$name" "$dflt" || return
	done
}

set_an_ipv6group()
{
	local config="$1"
	local cid label description user
	local flag name type scope scope_mask comment uniq_id
	local ipv6_enable ipv6_proto ipv6_addr ipv6_prefixtype ipv6_prefixlen ipv6_prefix system_flag
	user="network"
	config_get cid "$config" cid ''
	config_get description "$config" name ''
	config_get label "$config" zone "$description"
	config_get ipv6_enable "$config" ipv6_enable ''
	config_get ipv6_proto "$config" ipv6_proto ''

	flag="$user"
	name="$cid@NETWORK_1"
	type="mask"
	comment="$description"

	scope="::/128"
	if [ "$ipv6_enable" == "on" ]; then
		if [ "$ipv6_proto" == "dhcpv6" ]; then
			config_get ipv6_addr "$config" ipv6_addr ''
			config_get ipv6_prefixlen "$config" ipv6_prefixlen ''
			[ -n "$ipv6_addr" ] && [ -n "$ipv6_prefixlen" ] && {
				scope="${ipv6_addr}/${ipv6_prefixlen}"
			}
		elif [ "$ipv6_proto" == "slaac" ] || [ "$ipv6_proto" == "rdnss" ]; then
			#config_get ipv6_prefixtype "$config" ipv6_prefixtype ''
			config_get ipv6_prefix "$config" ipv6_prefix ''
			config_get ipv6_prefixlen "$config" ipv6_prefixlen ''
			[ -n "$ipv6_prefix" ] && [ -n "$ipv6_prefixlen" ] && {
				scope="${ipv6_prefix}/${ipv6_prefixlen}"
			}
			config_get ipv6_addr "$config" ipv6_addr
			[ -z "$ipv6_addr" ] && calc_vnet_ipv6_addr ${config} ipv6_addr 1
		else
			scope="::/128"
		fi
	else
		scope="::/128"
	fi

	#将ipv6addr添加至IPV6_LAN
	config_get system_flag "$config" system
	[ -n "$ipv6_addr" ] && [ -n "$ipv6_prefixlen" ]  && [ "$system_flag" == "0" ] && uci add_list ipv6group.@rule_ipv6scope[0].scope="${ipv6_addr}/${ipv6_prefixlen}"

	#添加ipv6scope配置文件
	local scopename="${cid}_NETWORK_1_ipv6scope"

	uci set ipv6group.$scopename="rule_ipv6scope"
	uci set ipv6group.$scopename.type="$type"
	uci set ipv6group.$scopename.name="$name"
	uci set ipv6group.$scopename.scope="$scope"
	uci set ipv6group.$scopename.comment="$comment"
	uci set ipv6group.$scopename.flag="$flag"
	#添加ipv6group配置文件
	local ipv6group_name="${cid}_NETWORK_1_ipv6group"
	local rule_scope="$cid@NETWORK_1"
	uniq_id="$cid@NETWORK"
	name="$uniq_id"

	uci set ipv6group.$ipv6group_name="rule_ipv6group"
	uci add_list ipv6group.$ipv6group_name.rule_scope="$rule_scope"
	uci set ipv6group.$ipv6group_name.name="$name"
	uci set ipv6group.$ipv6group_name.label="$label"
	uci set ipv6group.$ipv6group_name.flag="$flag"
	uci set ipv6group.$ipv6group_name.ref="0"
	uci set ipv6group.$ipv6group_name.uniq_id="$uniq_id"
	uci set ipv6group.$ipv6group_name.comment="$comment"
	#添加view_relation配置文件
	local relation_name="${cid}_NETWORK_1_relation"
	local group="$name"
	uci set ipv6group.$relation_name="view_relation"
	uci set ipv6group.$relation_name.group="$group"
	uci add_list ipv6group.$relation_name.rule_scope="$rule_scope"

	uci commit ipv6group
}

ipv6group_exist()
{
	local config="$1"
	local zone_name="$2"
	local label
	local flag
	config_get label "$config" label ''
	config_get flag "$config" flag ''
	[ -z "$flag" ] && return
	[ "$flag" == "network" ] && {
		[ -z "$label" ] && return
		if [ "$zone_name" == "$label" ]; then
			is_exit="true"
		fi
	}

}

vnetwork_ipv6group_exist()
{
	local config="$1"
	local zone_name 
	is_exit="false"

	config_get zone_name "$config" zone ''

	[ -z "$zone_name" ] && return

	config_load ipv6group 
	config_foreach ipv6group_exist rule_ipv6group $zone_name 
	[ "$is_exit" == "false" ] && {
		#network对应的ipv6 group不存在，需要同步生成
		set_an_ipv6group $config
	}
	

}
network_synchronous_ipv6group() {
	support_ipv6group_cfg=`uci get ipv6group.global.network_ipv6group`
	[ "$support_ipv6group_cfg" == "1" ] && return
	#通过network版本号判断是否进行了版本升级
	major_network=`uci -c /etc/profile.d get profile.@ecs_mod_major_ver[0].network`
	minor_network=`uci -c /etc/profile.d get profile.@ecs_mod_minor_ver[0].network`
	version="${major_network}${minor_network}"
	[ $version -lt 12 ] && return
	uci set ipv6group.global="cfg_ipv6Group"
	uci set ipv6group.global.network_ipv6group="1"

	#5.8版本中存在错误配置文件/64，将会影响ipv6group的ipset的创建，在升级时将其删除
	uci del_list ipv6group.@rule_ipv6scope[0].scope=/64

	ipv6_scope=$(uci get ipv6group.@rule_ipv6scope[0].scope 2>/dev/null)
	[ -n "$ipv6_scope" ] && {
		uci delete ipv6group.@rule_ipv6scope[0].scope
		IFS=' '
		local flag="true"
		set -f
		for element in $ipv6_scope
		do
			if [ "$element" == "fe80::0/64" ]; then
				flag="false"
			fi
			uci add_list ipv6group.@rule_ipv6scope[0].scope=$element
		done
		set +f
		if [ "$flag" == "true" ]; then
			uci add_list ipv6group.@rule_ipv6scope[0].scope="fe80::0/64"
		fi
	}

	uci commit ipv6group
	#修改配置文件总入口
	config_load vnetwork
	config_foreach vnetwork_ipv6group_exist vinterface
	/lib/cfg_save/config.sh "modify"

}

change_lanv6_ipgroup_scope() {
	local wan_iface="$1"
	local lan_iface="$2"
	local action="$3"

	# echo "===========change_lanv6_ipgroup_scope $action, $wan_iface, $lan_iface" >> /tmp/ipv6_loggg
	[ -z "$lan_iface" -o -z "$wan_iface" -o -z "action" ] && return
	
	local cid name ipv6_proto ipv6_prefixlen ipv6_prefix ipv6_addr ipv6_enable
 	local old_scope new_scope old_prefix all_scope del_scope
	
	cid=$(uci get vnetwork.${lan_iface}.cid)
	ipv6_proto=$(uci get vnetwork.${lan_iface}.ipv6_proto)
	ipv6_addr=$(uci get vnetwork.${lan_iface}.ipv6_addr)
	ipv6_prefix=$(uci get vnetwork.${lan_iface}.ipv6_prefix)
	ipv6_prefixlen=$(uci get vnetwork.${lan_iface}.ipv6_prefixlen)
	ipv6_enable=$(uci get vnetwork.${lan_iface}.ipv6_enable)
	
	[ -z "$cid" ] && return
	name="${cid}_NETWORK_1"
	# echo "cid: $cid, prefix: $ipv6_prefix" >> /tmp/ipv6_loggg

	[ "x$action" == "xifdown" ] && {
		# del old scope from all lan rule_ipv6scope
		old_scope=$(uci get ipv6group.${name}_ipv6scope.scope)
		old_prefix=$(echo ${old_scope} | sed 's/::.*//')
		# echo "old: $old_scope, prefix: $old_prefix" >> /tmp/ipv6_loggg
		all_scope=$(uci get ipv6group.@rule_ipv6scope[0].scope)
		for scp in $all_scope
		do
			# echo "for $scp" >> /tmp/ipv6_loggg
			local match=$(expr match "${scp}" "${old_prefix}")
			[ $match != 0 ] && uci del_list ipv6group.@rule_ipv6scope[0].scope="${scp}" # && echo "del $scp" >> /tmp/ipv6_loggg
		done

		# del old scope from cid lan rule_ipv6scope
		uci set ipv6group.${name}_ipv6scope.scope="::/128"
	}

	[ "x$action" == "xifup" ] && {
		# set new lanv6 ipgroup
		new_scope="::/128"
		if [ "$ipv6_enable" == "on" ]; then
			if [ "$ipv6_proto" == "slaac" ] || [ "$ipv6_proto" == "rdnss" ]; then
				[ -z "$ipv6_prefix" ] && ipv6_prefix=$(cat /tmp/dhcp6c/prefix.info.${wan_iface})
				[ -z "$ipv6_prefixlen" ] && ipv6_prefixlen=$(cat /tmp/dhcp6c/prefixlen.info.${wan_iface})
				[ -n "$ipv6_prefix" ] && [ -n "$ipv6_prefixlen" ] && new_scope="${ipv6_prefix}/${ipv6_prefixlen}"
				# echo "set new_scope" >> /tmp/ipv6_loggg
			fi
		fi
		# echo "new: $new_scope" >> /tmp/ipv6_loggg
		uci set ipv6group.${name}_ipv6scope.scope="${new_scope}"

		[ -z "$ipv6_addr" ] && calc_vnet_ipv6_addr ${lan_iface} ipv6_addr 1
		# echo "new ip : $ipv6_addr" >> /tmp/ipv6_loggg
		[ "$new_scope" != "::/128" ] && [ -n "$ipv6_addr" ] && [ -n "$ipv6_prefixlen" ] && uci add_list ipv6group.@rule_ipv6scope[0].scope="${ipv6_addr}/${ipv6_prefixlen}"
	}

	uci commit ipv6group
	. /lib/ipv6group/core_ipv6group.sh; ipv6group_load
}
