#!/bin/sh

SEARCH="/usr/bin/ldapsearch"
CONFIG_DIR="/etc/openldap"
CONFIG_FILE="/etc/openldap/ldap.conf"
NET_TIMEOUT=3    #ldap nginx expiretime=30(~45)s, ldap profile max_count=8. Thus set timeout to 3s.

. /lib/ldap-tplink/char_conv.sh
ssl_handle()
{
    local cmd=$1
    local line=`cat $CONFIG_FILE | grep "TLS_REQCERT"`
    [ ! -f $CONFIG_FILE ] && mkdir -p $CONFIG_DIR && touch $CONFIG_FILE
    if [ $cmd == "enable" ];then
        if [ ${#line} -ne 0 ] ;then
            sed "s/TLS_REQCERT .\+/TLS_REQCERT ALLOW/g" -i $CONFIG_FILE
        else
            echo "TLS_REQCERT ALLOW" >> $CONFIG_FILE
        fi
    else
        [ ${#line} -ne 0 ] && sed "/TLS_REQCERT .\+/d" -i $CONFIG_FILE
    fi
}


auth()
{
    local uid=$1
    uid=$(char_conv "$uid" "uid")
    local uid_ori=${uid}
    local passwd=$2
    local name_t=`uci get ldap.setting.portal`
    [ ${#name_t} -eq 0 ] && return
    for name in $name_t; do
        local status=`uci get ldap.${name}.status`
        [ "$status" == "off" ] && continue 
        local host=`uci get ldap.${name}.host`
        local port=`uci get ldap.${name}.port`
        local ssl=`uci get ldap.${name}.ssl`
        local mtype=`uci get ldap.${name}.type`
        local dn=`uci get ldap.${name}.dn`
        local dn_passwd=`uci get ldap.${name}.password`
        local base=`uci get ldap.${name}.base`
        local cn=`uci get ldap.${name}.cn`
        local group=`uci get ldap.${name}.group`
        local filter=`uci get ldap.${name}.filter`
        local ldap_cmd="-H ldaps://"
        local ssl_cmd="-Z"
        local user=""
        uid=${uid_ori}
        if [ "$mtype" != "simple" ];then
            local new_uid=$(char_conv "$uid" "filter")
            echo "new_uid: $new_uid" > /dev/console
            [ $ssl != "enable" ] && ssl_cmd="" && ldap_cmd="-H ldap://"
            local filter_cmd="${cn}=${new_uid}"
            local group_filter=""
            local cnt=""
            local cnt2=""
            [ ${#filter} -ne 0 ] && filter_cmd="(&(${filter_cmd})(${filter}))"
            [ ! -d /tmp/.ldap ] && mkdir -p /tmp/.ldap
            if [ "$mtype" == "anonymous" ];then
                ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" ${ssl_cmd} -l 5 -o ldif-wrap="no" -o nettimeout=${NET_TIMEOUT} "${filter_cmd}" dn >/tmp/.ldap/auth_log
                [ $? -ne 0 ] && continue
                cnt=`cat /tmp/.ldap/auth_log |fgrep "dn: ${cn}=${uid},"`
                cnt=${cnt//\\/\\\\}
                if [ ${#cnt} -gt 0 ];then
                    echo "$cnt" >/tmp/.ldap/auth_log
                    while read line
                    do
                        line=${line#*dn: }
                        local new_line=$(char_conv "$line" "group_filter")
                        if [ ${#group} -ne 0 ]; then
                            local group_uid=$(char_conv "$uid" "group_filter")
                            group_filter="(|(|(&(objectclass=posixGroup)(memberUid=$group_uid))(&(objectcategory=group)(member=${new_line})))(&(objectclass=posixGroup)(uniqueMember=${new_line})))" 
                            ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${group}" ${ssl_cmd} -l 5 -o ldif-wrap="no" -o nettimeout=${NET_TIMEOUT} "${group_filter}" dn >/tmp/.ldap/auth_log2
                            [ $? -ne 0 ] && continue
                            cnt2=`cat /tmp/.ldap/auth_log2 |grep -w "dn:"`
                            [ ${#cnt2} -gt 0 ] && ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${line}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
                            [ $? -eq 0 ] && echo "success" && return
                        else
                            ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${line}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
                            [ $? -eq 0 ] && echo "success" && return
                        fi
                    done < "/tmp/.ldap/auth_log"
                fi
            else
                ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${dn}" -w "${dn_passwd}" ${ssl_cmd} -l 5 -o ldif-wrap="no" -o nettimeout=${NET_TIMEOUT} "${filter_cmd}" dn >/tmp/.ldap/auth_log
                [ $? -ne 0 ] && continue
                cnt=`cat /tmp/.ldap/auth_log |fgrep "dn: ${cn}=${uid},"`
                cnt=${cnt//\\/\\\\}
                if [ ${#cnt} -gt 0 ];then
                    echo "$cnt" >/tmp/.ldap/auth_log
                    while read line
                    do
                        line=${line#*dn: }
                        local new_line=$(char_conv "$line" "group_filter")
                        if [ ${#group} -ne 0 ]; then
                            local group_uid=$(char_conv "$uid" "group_filter")
                            group_filter="(|(|(&(objectclass=posixGroup)(memberUid=$group_uid))(&(objectcategory=group)(member=${new_line})))(&(objectclass=posixGroup)(uniqueMember=${new_line})))" 
                            ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${group}" -D "${dn}" -w "${dn_passwd}" ${ssl_cmd} -l 5 -o ldif-wrap="no" -o nettimeout=${NET_TIMEOUT} "${group_filter}" dn >/tmp/.ldap/auth_log2
                            [ $? -ne 0 ] && continue
                            cnt2=`cat /tmp/.ldap/auth_log2 |grep -w "dn:"`
                            [ ${#cnt2} -gt 0 ] && ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${line}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
                            [ $? -eq 0 ] && echo "success" && return
                        else
                            ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${line}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
                            [ $? -eq 0 ] && echo "success" && return
                        fi
                    done < "/tmp/.ldap/auth_log"
                fi
            fi
        else
            [ "$ssl" != "enable" ] && ssl_cmd="" && ldap_cmd="-H ldap://"
            user="${cn}=${uid},${base}"
            ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "$base" -D "${user}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
            [ $? -eq 0 ] && echo "success" && return
        fi
    done
    echo "fail"
}


vpn_ldap_auth()
{
    local uid=$1
    uid=$(char_conv "$uid" "uid")
    local uid_ori=${uid}
    local passwd=$2
    local name=$3
    local status=`uci get ldap.${name}.status`
    [ "$status" == "off" ] && return 1
    local host=`uci get ldap.${name}.host`
    local port=`uci get ldap.${name}.port`
    local ssl=`uci get ldap.${name}.ssl`
    local mtype=`uci get ldap.${name}.type`
    local dn=`uci get ldap.${name}.dn`
    local dn_passwd=`uci get ldap.${name}.password`

    local base=`uci get ldap.${name}.base`
    local cn=`uci get ldap.${name}.cn`
    local group=`uci get ldap.${name}.group`
    local filter=`uci get ldap.${name}.filter`
    local ldap_cmd="-H ldaps://"
    local ssl_cmd="-Z"
    local user=""
    uid=${uid_ori}
    if [ "$mtype" != "simple" ];then
        local new_uid=$(char_conv "$uid" "filter")
        echo "new_uid: $new_uid" > /dev/console
        [ $ssl != "enable" ] && ssl_cmd="" && ldap_cmd="-H ldap://"
        local filter_cmd="${cn}=${new_uid}"
        local group_filter=""
        local cnt=""
        local cnt2=""
        [ ${#filter} -ne 0 ] && filter_cmd="(&(${filter_cmd})(${filter}))"
        [ ! -d /tmp/.ldap ] && mkdir -p /tmp/.ldap
        if [ "$mtype" == "anonymous" ];then
            ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" ${ssl_cmd} -l 5 -o ldif-wrap="no" -o nettimeout=${NET_TIMEOUT} "${filter_cmd}" dn >/tmp/.ldap/auth_log
            [ $? -ne 0 ] && return 1
            cnt=`cat /tmp/.ldap/auth_log |fgrep "dn: ${cn}=${uid},"`
            cnt=${cnt//\\/\\\\}
            if [ ${#cnt} -gt 0 ];then
                echo "$cnt" >/tmp/.ldap/auth_log
                while read line
                do
                    line=${line#*dn: }
                    local new_line=$(char_conv "$line" "group_filter")
                    if [ ${#group} -ne 0 ]; then
                        local group_uid=$(char_conv "$uid" "group_filter")
                        group_filter="(|(|(&(objectclass=posixGroup)(memberUid=$group_uid))(&(objectcategory=group)(member=${new_line})))(&(objectclass=posixGroup)(uniqueMember=${new_line})))" 
                        ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${group}" ${ssl_cmd} -l 5 -o ldif-wrap="no" -o nettimeout=${NET_TIMEOUT} "${group_filter}" dn >/tmp/.ldap/auth_log2
                        [ $? -ne 0 ] && return 1
                        cnt2=`cat /tmp/.ldap/auth_log2 |grep -w "dn:"`
                        [ ${#cnt2} -gt 0 ] && ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${line}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
                        [ $? -eq 0 ] && echo "success" && return 0
                    else
                        ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${line}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
                        [ $? -eq 0 ] && echo "success" && return 0
                    fi
                done < "/tmp/.ldap/auth_log"
            fi
        else
            ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${dn}" -w "${dn_passwd}" ${ssl_cmd} -l 5 -o ldif-wrap="no" -o nettimeout=${NET_TIMEOUT} "${filter_cmd}" dn >/tmp/.ldap/auth_log
            [ $? -ne 0 ] && return 1
            cnt=`cat /tmp/.ldap/auth_log |fgrep "dn: ${cn}=${uid},"`
            cnt=${cnt//\\/\\\\}
            if [ ${#cnt} -gt 0 ];then
                echo "$cnt" >/tmp/.ldap/auth_log
                while read line
                do
                    line=${line#*dn: }
                    local new_line=$(char_conv "$line" "group_filter")
                    if [ ${#group} -ne 0 ]; then
                        local group_uid=$(char_conv "$uid" "group_filter")
                        group_filter="(|(|(&(objectclass=posixGroup)(memberUid=$group_uid))(&(objectcategory=group)(member=${new_line})))(&(objectclass=posixGroup)(uniqueMember=${new_line})))" 
                        ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${group}" -D "${dn}" -w "${dn_passwd}" ${ssl_cmd} -l 5 -o ldif-wrap="no" -o nettimeout=${NET_TIMEOUT} "${group_filter}" dn >/tmp/.ldap/auth_log2
                        [ $? -ne 0 ] && return 1
                        cnt2=`cat /tmp/.ldap/auth_log2 |grep -w "dn:"`
                        [ ${#cnt2} -gt 0 ] && ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${line}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
                        [ $? -eq 0 ] && echo "success" && return 0
                    else
                        ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "${base}" -D "${line}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
                        [ $? -eq 0 ] && echo "success" && return 0
                    fi
                done < "/tmp/.ldap/auth_log"
            fi
        fi
    else
        [ "$ssl" != "enable" ] && ssl_cmd="" && ldap_cmd="-H ldap://"
        user="${cn}=${uid},${base}"
        ${SEARCH} ${ldap_cmd}"${host}":${port} -x -b "$base" -D "${user}" -w "${passwd}" ${ssl_cmd} -l 5 -o nettimeout=${NET_TIMEOUT} objectclass=result
        [ $? -eq 0 ] && echo "success" && return 0
    fi
    return 1
}


