#
# Copyright (C) 2008-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# 
# brief        mac filter module core_global shell scripts
# author     Huang Zhenwei
# version    1.0.0 
# date         29Apr2015
# history     29Apr2015    Huang Zhenwei    Create the file

#
# use "-C" to check if the rule exists
# return: 1 rule exists;
#            0 rule not exists;
#
. /lib/zone/zone_api.sh

fw_check() {
	local ret=$(vnete iptables -w -t $1 -C $2 $3 2>&1)
	[ -n "$ret" ] && return 0||return 1
}

fw_config_get_global() {
	fw_config_get_section "$1" global { \
		string enable "off" \
		string filter_mode "black" \
		string interfaces "" \
	} || return
}

fw_config_get_white_list() {
	fw_config_get_section "$1" white_list { \
		string name "" \
		string mac "" \
	} || return
}

fw_config_get_black_list() {
	fw_config_get_section "$1" black_list { \
		string name "" \
		string mac "" \
	} || return
}

fw_config_get_mac_list() {
	fw_config_get_section "$1" mac_list { \
		string name "" \
		string mac "" \
	} || return
}

fw_config_get_mac_address_list() {
	fw_config_get_section "$1" macfilters { \
		string id "" \
		string type "" \
		string macAddrs "" \
	} || return
}

fw_config_get_mac_group_list() {
	fw_config_get_section "$1" macfilters { \
		string id "" \
		string type "" \
		string macGroups "" \
	} || return
}

fw_load_all() {
	fw_config_once fw_load_global global
}

fw_exit_all() {
	vnete fw flush 4 f mac_filtering
	fw_check "filter" "INPUT" "-j mac_filtering"
	while [ x$? == x1 ]; do
		vnete fw del 4 f INPUT mac_filtering
		fw_check "filter" "INPUT" "-j mac_filtering"
	done

	fw_check "filter" "forwarding_rule" "-j mac_filtering"
	while [ x$? == x1 ]; do
		vnete fw del 4 f forwarding_rule mac_filtering
		fw_check "filter" "forwarding_rule" "-j mac_filtering"
	done

	vnete fw del 4 f mac_filtering

	vnete ipset destroy mac_set
	config_foreach fw_clear_mac_group_list macfilters
}

fw_add_input_chain() {
	fw_check "filter" "INPUT" "-j mac_filtering"
	[ x$? == x0 ] && {
		fw_check "filter" "INPUT" "-j mac_drop"
		if [ x$? == x0 ];then
			vnete fw add 4 f INPUT mac_filtering 1
		else
			#fw add 4 f forwarding_rule mac_filtering 3
			local num=$(vnete iptables -w -nL INPUT --line-number | grep "\bmac_drop\b" | awk '{print $1}')
			num=$(($num+1))
			vnete fw add 4 f INPUT mac_filtering $num
		fi
	}
}

fw_load_global() {
	fw_config_get_global $1
	case $global_enable in
		on )
		redirect_dest=`uci get vnetwork.vwan.ippeer`
		vnete ipset create mac_set hash:mac -exist
		vnete fw add 4 f mac_filtering
		fw_check "filter" "forwarding_rule" "-j mac_filtering"
		[ x$? == x0 ] && {
			fw_check "filter" "forwarding_rule" "-j mac_drop"
			if [ x$? == x0 ];then
				vnete fw add 4 f forwarding_rule mac_filtering 1
			else
				num=$(vnete iptables -w -nL forwarding_rule --line-number | grep "\bmac_drop\b" | awk '{print $1}')
				num=$(($num+1))
				vnete fw add 4 f forwarding_rule mac_filtering $num
			fi
		}

		case $global_filter_mode in
		 	black )
			vnete ipset flush mac_set
			config_foreach fw_load_black_list black_list
			# config_foreach fw_load_mac_list mac_list
			local tmp_interface=""
			if [ x"$global_interfaces" == 'x' ];then  #interface is empty
				#local rule="-m set --match-set mac_set src"
				#fw_check "filter" "mac_filtering" "$rule -j DROP"
				#[ x$? == x0 ] && fw add 4 f mac_filtering DROP { "$rule" }
				return
			else
				for tmp_interface in $global_interfaces;do
					if [ "$tmp_interface"x = "ALL"x ];then
						local rule="-m set --match-set mac_set src"
						fw_check "filter" "mac_filtering" "$rule -j DROP"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering DROP { "$rule" }

						fw_add_input_chain
					else
						local rule="-m set --match-set mac_set src -o veth1 ! -d $redirect_dest"
						fw_check "filter" "mac_filtering" "$rule -j DROP"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering DROP { "$rule" }
					fi
				done
			fi
			for tmp_interface in $global_interfaces;do
				fw_check "filter" "mac_filtering" "-j RETURN"
				[ x$? == x0 ] && vnete fw add 4 f mac_filtering RETURN
			done
		 		;;
		 	white )
			vnete ipset flush mac_set
			config_foreach fw_load_white_list white_list
			# config_foreach fw_load_mac_list mac_list
			local tmp_interface
			if [ x"$global_interfaces" == 'x' ];then
				#local rule="-m set --match-set mac_set src"
				#fw_check "filter" "mac_filtering" "$rule -j RETURN"
				#[ x$? == x0 ] && fw add 4 f mac_filtering RETURN { "$rule" }
				return
			else
				for tmp_interface in $global_interfaces;do
					if [ "$tmp_interface"x = "ALL"x ];then
						local rule="-m set --match-set mac_set src"
						fw_check "filter" "mac_filtering" "$rule -j RETURN"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering RETURN { "$rule" }

						fw_add_input_chain

						rule="! -i veth1"
						fw_check "filter" "mac_filtering" "$rule -j DROP"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering DROP { "$rule" }
					else
						local rule="-m set --match-set mac_set src -o veth1 ! -d $redirect_dest"
						fw_check "filter" "mac_filtering" "$rule -j RETURN"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering RETURN { "$rule" }

						rule="! -i veth1 ! -d $redirect_dest -o veth1"
						fw_check "filter" "mac_filtering" "$rule -j DROP"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering DROP { "$rule" }
					fi
				done
			fi
		 		;;
			white_controller )
			vnete ipset flush mac_set
			local tmp_interface
			if [ x"$global_interfaces" == 'x' ];then
				return
			else
				for tmp_interface in $global_interfaces;do
					if [ "$tmp_interface"x = "ALL"x ];then
						local rule="-m set --match-set mac_set src"
						fw_check "filter" "mac_filtering" "$rule -j RETURN"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering RETURN { "$rule" }

						fw_add_input_chain

						rule="! -i veth1"
						fw_check "filter" "mac_filtering" "$rule -j DROP"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering DROP { "$rule" }
					else
						local rule="-m set --match-set mac_set src -o veth1 ! -d $redirect_dest"
						fw_check "filter" "mac_filtering" "$rule -j RETURN"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering RETURN { "$rule" }

						rule="! -i veth1 ! -d $redirect_dest -o veth1"
						fw_check "filter" "mac_filtering" "$rule -j DROP"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering DROP { "$rule" }
					fi
				done
			fi

			#load mac_list
			config_foreach fw_load_mac_address_list macfilters
			config_foreach fw_load_mac_group_list macfilters
				;;
			black_controller )
			vnete ipset flush mac_set

			local tmp_interface=""
			if [ x"$global_interfaces" == 'x' ];then  #interface is empty
				#local rule="-m set --match-set mac_set src"
				#fw_check "filter" "mac_filtering" "$rule -j DROP"
				#[ x$? == x0 ] && fw add 4 f mac_filtering DROP { "$rule" }
				return
			else
				for tmp_interface in $global_interfaces;do
					if [ "$tmp_interface"x = "ALL"x ];then
						local rule="-m set --match-set mac_set src"
						fw_check "filter" "mac_filtering" "$rule -j DROP"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering DROP { "$rule" }

						fw_add_input_chain
					else
						local rule="-m set --match-set mac_set src -o veth1 ! -d $redirect_dest"
						fw_check "filter" "mac_filtering" "$rule -j DROP"
						[ x$? == x0 ] && vnete fw add 4 f mac_filtering DROP { "$rule" }
					fi
				done
			fi
			for tmp_interface in $global_interfaces;do
				fw_check "filter" "mac_filtering" "-j RETURN"
				[ x$? == x0 ] && vnete fw add 4 f mac_filtering RETURN
			done
			#load mac_list
			config_foreach fw_load_mac_address_list macfilters
			config_foreach fw_load_mac_group_list macfilters
				;;
		esac

		#conntrack -F
		#syslog $LOG_INF_FUNCTION_ENABLE
		#syslog $LOG_NTC_FLUSH_CT_SUCCESS
			;;
		off )
		#syslog $LOG_INF_FUNCTION_DISABLE
			;;
	esac
	
}

fw_load_white_list() {
	fw_config_get_white_list $1
	local mac=$(echo $white_list_mac | tr [a-z] [A-Z])
	vnete ipset add mac_set "${mac//-/:}"
	#syslog $ACCESS_CONTROL_LOG_DBG_WHITE_LIST_ADD "${mac//-/:}"
}

fw_load_black_list() {
	fw_config_get_black_list $1
	local mac=$(echo $black_list_mac | tr [a-z] [A-Z])
	vnete ipset add mac_set "${mac//-/:}"
	#syslog $ACCESS_CONTROL_LOG_DBG_BLACK_LIST_ADD "${mac//-/:}"
}

fw_load_mac_list() {
	fw_config_get_mac_list $1
	local mac=$(echo $mac_list_mac | tr [a-z] [A-Z])
	vnete ipset add mac_set "${mac//-/:}" -exist
	#syslog $ACCESS_CONTROL_LOG_DBG_BLACK_LIST_ADD "${mac//-/:}"
}

fw_load_mac_address_list() {
	fw_config_get_mac_address_list $1
	local macs=$(echo $macfilters_macAddrs | tr [a-z] [A-Z])
	if [ x"$macs" == 'x' ];then
		return
	else
		for mac in $macs;do
			vnete ipset add mac_set "${mac//-/:}" -exist
		done
	fi
}

fw_load_mac_group_list() {
	fw_config_get_mac_group_list $1
	local macIds=$(echo $macfilters_macGroups | tr [a-z] [A-Z])
	if [ x"$macIds" == 'x' ];then
		return
	else
		redirect_dest=`uci get vnetwork.vwan.ippeer`
		ipset_mac=$macfilters_id"@macgroup_merged"
		vnete ipset create $ipset_mac "list:set"
		rule="vnete iptables -w -I mac_filtering 1"
		if [ $global_filter_mode == "white_controller" ];then
			if [ "$global_interfaces"x = "ALL"x ];then
				$rule -m set --match-set $ipset_mac src -j RETURN
			else
				$rule -o veth1 ! -d $redirect_dest -m set --match-set $ipset_mac src -j RETURN
			fi
		elif [ $global_filter_mode == "black_controller" ];then
			if [ "$global_interfaces"x = "ALL"x ];then
				$rule -m set --match-set $ipset_mac src -j DROP
			else
				$rule -o veth1 ! -d $redirect_dest -m set --match-set $ipset_mac src -j DROP
			fi
		else
			return
		fi
		for macId in $macIds;do
		#iptables
			local macGroup=$macId"@macgroup"
			vnete ipset add $ipset_mac $macGroup
		done
	fi
}

fw_clear_mac_group_list() {
	fw_config_get_mac_group_list $1
	local ruleIds=$(echo $macfilters_id)
	if [ x"$ruleIds" == 'x' ];then
		return
	else
		if [ x"$macfilters_type" != 'x1' ];then
            return
        fi
		vnete ipset destroy $ruleIds"@macgroup_merged"
	fi
}