#!/bin/sh 
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file      nat_dmz_bypass.sh
# brief    
# author    Xie zhiyong
# version   1.1.0
# date      14Aug23

#para add/del tcp/udp ports (wan_name)
#example nat_dmz_bypass.sh add tcp 80,443 WAN2 


method=$1
protocol=$2
port=$3
wan=$4
local dmz_pre_rule="dmz_prerouting_rule"
ipsec_dmz_reload()
{
	local section=$1
	local local_binding=`uci get ipsec.$section.local_binding`
    local ipsec_status=`uci get ipsec.$section.status`
	if [ -n "$local_binding" ] && [ -n "$ipsec_status" ] && [ "x$ipsec_status" = "xenable" ]; then
        local devices=$(zone_get_effect_devices $local_binding)
        for iface in $devices;do
			[ -n "$iface" -a "$iface" != "loopback" ] && {
                if ! iptables -w -t nat -C $dmz_pre_rule -i $iface -p udp -m multiport --dport 500,4500 -m comment --comment "ipsec_dmz" -j RETURN 2>/dev/null; then
					iptables -w -t nat  -I $dmz_pre_rule -i $iface -p udp -m multiport --dport 500,4500 -m comment --comment "ipsec_dmz" -j RETURN
                fi
            }
		done
    fi
}
pptp_dmz_reload()
{
    local section=$1
	local local_binding=`uci get pptp-server.$section.bindif`
    local pptp_status=`uci get pptp-server.$section.enable`
	if [ -n "$local_binding" ] && [ -n "$pptp_status" ] && [ "x$pptp_status" = "xon" ]; then
        local devices=$(zone_get_effect_devices $local_binding)
        for iface in $devices;do
			[ -n "$iface" -a "$iface" != "loopback" ] && {
                if ! iptables -w -t nat -C $dmz_pre_rule -i $iface -p tcp -m multiport --dport 1723 -m comment --comment "pptp_dmz" -j RETURN 2>/dev/null; then
					iptables -w -t nat  -I $dmz_pre_rule -i $iface -p tcp -m multiport --dport 1723 -m comment --comment "pptp_dmz" -j RETURN
                fi
            }
		done
    fi
}
l2tp_dmz_reload()
{
    local section=$1
	local local_binding=`uci get l2tp-server.$section.bindif`
    local l2tp_status=`uci get l2tp-server.$section.enable`
	if [ -n "$local_binding" ] && [ -n "$l2tp_status" ] && [ "x$l2tp_status" = "xyes" ]; then
        local devices=$(zone_get_effect_devices $local_binding)
        for iface in $devices;do
			[ -n "$iface" -a "$iface" != "loopback" ] && {
                if ! iptables -w -t nat -C $dmz_pre_rule -i $iface -p udp -m multiport --dport 1701 -m comment --comment "l2tp_dmz" -j RETURN 2>/dev/null; then
					iptables -w -t nat  -I $dmz_pre_rule -i $iface -p udp -m multiport --dport 1701 -m comment --comment "l2tp_dmz" -j RETURN
                fi
            }
		done
    fi
}
ovpn_dmz_reload()
{
	local section=$1
	local local_binding=`uci get openvpn.$section.wan`
    local ovpn_status=`uci get openvpn.$section.enabled`
    local ovpn_port=`uci get openvpn.$section.port`
    local ovpn_proto=`uci get openvpn.$section.proto`
	if [ -n "$local_binding" ] && [ -n "$ovpn_status" ] && [ "x$ovpn_status" = "xon" ]; then
        local devices=$(zone_get_effect_devices $local_binding)
        for iface in $devices;do
			[ -n "$iface" -a "$iface" != "loopback" ] && {
                if ! iptables -w -t nat -C $dmz_pre_rule -i $iface -p $ovpn_proto -m multiport --dport $ovpn_port -m comment --comment "ovpn_dmz" -j RETURN 2>/dev/null; then
					iptables -w -t nat  -I $dmz_pre_rule -i $iface -p $ovpn_proto -m multiport --dport $ovpn_port -m comment --comment "ovpn_dmz" -j RETURN
                fi
            }
		done
    fi
}
sslvpn_dmz_reload()
{
	local section=$1
	local local_binding=`uci get ssl_vpn_config.$section.service_if`
    local sslvpn_status=`uci get ssl_vpn_config.$section.enable`
    local sslvpn_port=`uci get ssl_vpn_config.$section.port`
	if [ -n "$local_binding" ] && [ -n "$sslvpn_status" ] && [ "x$sslvpn_status" = "xon" ]; then
        local devices=$(zone_get_effect_devices $local_binding)
        for iface in $devices;do
			[ -n "$iface" -a "$iface" != "loopback" ] && {
                if ! iptables -w -t nat -C $dmz_pre_rule -i $iface -p udp -m multiport --dport $sslvpn_port -m comment --comment "sslvpn_dmz" -j RETURN 2>/dev/null; then
					iptables -w -t nat  -I $dmz_pre_rule -i $iface -p udp -m multiport --dport $sslvpn_port -m comment --comment "sslvpn_dmz" -j RETURN
                fi
            }
		done
    fi
}

if [ -n "$method" ] && [ "x$method" = "xipsec_reload" ]; then
	nums=$(iptables -w  -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bipsec_dmz\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w  -t nat -D dmz_prerouting_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bipsec_dmz\b" | cut -d ' ' -f 1)
	done
    config_load 'ipsec'	
    config_foreach ipsec_dmz_reload connection
fi

if [ -n "$method" ] && [ "x$method" = "xpptp_reload" ]; then
	nums=$(iptables -w  -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bpptp_dmz\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w  -t nat -D dmz_prerouting_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bpptp_dmz\b" | cut -d ' ' -f 1)
	done
    config_load 'pptp-server'	
    config_foreach pptp_dmz_reload pns
fi

if [ -n "$method" ] && [ "x$method" = "xl2tp_reload" ]; then
	nums=$(iptables -w  -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bl2tp_dmz\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w  -t nat -D dmz_prerouting_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bl2tp_dmz\b" | cut -d ' ' -f 1)
	done
    config_load 'l2tp-server'	
    config_foreach l2tp_dmz_reload lns
fi

if [ -n "$method" ] && [ "x$method" = "xovpn_reload" ]; then
	nums=$(iptables -w  -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bovpn_dmz\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w  -t nat -D dmz_prerouting_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bovpn_dmz\b" | cut -d ' ' -f 1)
	done
    config_load 'openvpn'	
    config_foreach ovpn_dmz_reload server
fi

if [ -n "$method" ] && [ "x$method" = "xsslvpn_reload" ]; then
	nums=$(iptables -w  -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bsslvpn_dmz\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w  -t nat -D dmz_prerouting_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL dmz_prerouting_rule --line-numbers | grep "\bsslvpn_dmz\b" | cut -d ' ' -f 1)
	done
    config_load 'ssl_vpn_config'	
    config_foreach sslvpn_dmz_reload global
fi

if [ -n "$method" ] && [ -n "$protocol" ] && [ -n "$port" ]; then
    if [ "x$method" = "xadd" ]; then
        if [ -n "$wan" ] && [ "x$wan" != "xall_wan" ]; then
            local devices=$(zone_get_effect_devices $wan)
            for iface in $devices;do
				[ -n "$iface" -a "$iface" != "loopback" ] && {
                    if ! iptables -w -t nat -C $dmz_pre_rule -i $iface -p $protocol -m multiport --dport $port -j RETURN 2>/dev/null; then
					    iptables -w -t nat  -I $dmz_pre_rule -i $iface -p $protocol -m multiport --dport $port -j RETURN
                    fi
                }
			done
        else
            iptables -w -t nat -I $dmz_pre_rule -p $protocol -m multiport --dport $port -j RETURN
        fi
    fi
    if [ "x$method" = "xdel" ]; then
        if [ -n "$wan" ] && [ "x$wan" != "xall_wan" ]; then
            local devices=$(zone_get_effect_devices $wan)
            for iface in $devices;do
				[ -n "$iface" -a "$iface" != "loopback" ] && {
					iptables -w -t nat  -D $dmz_pre_rule -i $iface -p $protocol -m multiport --dport $port -j RETURN
                }
			done
        else
            iptables -w -t nat -D $dmz_pre_rule -p $protocol -m multiport --dport $port -j RETURN
        fi
    fi
fi