#!/bin/sh

[ -x /usr/sbin/pppd ] || exit 0

[ -n "$INCLUDE_ONLY" ] || {
	. /lib/functions.sh
	. ../netifd-proto.sh
	init_proto "$@"
}

pppv6_generic_init_config() {
	proto_config_add_string "username"
	proto_config_add_string "password"
	proto_config_add_string "keepalive"
	proto_config_add_string "pppd_options"
	proto_config_add_boolean "authfail"
	proto_config_add_int "mru"
	proto_config_add_boolean "ipv4"
	proto_config_add_string "lanif"
	proto_config_add_string "pppshare"
	proto_config_add_string "parent"
}

pppv6_generic_setup() {
	local config="$1"; shift
	local ifname="$1"; shift

	json_get_vars ipv4 keepalive username password pppd_options ip_mode ip_config dns_mode pppshare parent

	[ -n "$mru" ] || json_get_var mru mru

	local ipv4arg=""
	[ "$ipv4" == "1" ] && ipv4arg="ipv4"

	local interval="${keepalive##*[, ]}"
	[ "$interval" != "$keepalive" ] || interval=5

	[ "$ip_config" == "specified" ] && json_get_var ipaddr ipaddr

	# ipv4 & ipv6 share the same ppp session,
	# pseudo pppd for reading the ipv6 variables.
	local proc=/usr/sbin/pppd
	[ -n "$pppshare" -a -n "$parent" ] && proc=/usr/sbin/pppd-share

	local dnsarg=""
#	[ "$dns_mode" == "static" ] && dnsarg=""

	local zonex=`echo "${config%_v6}" | tr '[a-z]' '[A-Z]'`
	zonex=$zonex"v6"

	echo "0" > /proc/sys/net/ipv6/conf/$ifname/accept_ra

	proto_run_command "$config" "$proc" \
		nodetach ifname "${proto:-ppp}-$config" \
		ipparam "$config" \
		file /etc/ppp/options.default \
		${keepalive:+lcp-echo-interval $interval lcp-echo-failure ${keepalive%%[, ]*}} \
		defaultroute noaccomp nopcomp +ipv6 \
		${dnsarg:+"$dnsarg"} \
		${ipv4arg:+"$ipv4arg"} \
		persist maxfail 1 ${ipaddr:+"$ipaddr:"} \
		${username:+user "$username"} \
		${password:+password "$password"} \
		ipv6-up-script /lib/netifd/pppv6-up \
		ipv6-down-script /lib/netifd/ppp-down \
		${mru:+mtu $mru mru $mru} \
		${zonex:+zonex $zonex} \
		$pppd_options "$@"
}

pppv6_generic_teardown() {
	local interface="$1"
	local ifname="$2"

	rm /tmp/dhcp6c/prefix.info
	rm /tmp/dhcp6c/prefix.info.${interface}
	rm /tmp/dhcp6c/prefixlen.info.${interface}

	case "$ERROR" in
		11|19)
			proto_notify_error "$interface" AUTH_FAILED
			json_get_var authfail authfail
			if [ "${authfail:-0}" -gt 0 ]; then
				proto_block_restart "$interface"
			fi
		;;
		2)
			proto_notify_error "$interface" INVALID_OPTIONS
			proto_block_restart "$interface"
		;;
	esac

	uci_set dhcp6c $interface enabled '0'         
	uci_commit dhcp6c
	/etc/init.d/dhcp6c stop $interface

	proto_kill_command "$interface" 15
}

proto_pppoev6_init_config() {
	pppv6_generic_init_config
	proto_config_add_string "ac"
	proto_config_add_string "service"
	proto_config_add_string "dns"
	proto_config_add_string "ip_mode"
	proto_config_add_string "ip_config"
	proto_config_add_string "dns_mode"
	proto_config_add_string "ipaddr"
	proto_config_add_string "ip6addr"
}

proto_pppoev6_setup() {
	local config="$1"
	local iface="$2"

	for module in slhc ppp_generic pppox pppoe; do
		/sbin/insmod $module 2>&- >&-
	done

	json_get_var ac ac
	json_get_var service service

	pppv6_generic_setup "$config" "$iface" \
		plugin rp-pppoe.so "nic-$iface" \
		${ac:+rp_pppoe_ac "$ac"} \
		tunnel-proto pppoev6 \
		${service:+rp_pppoe_service "$service"}
	
	#update V zone groups
	local log_wan=`echo "${config%_v6}" |tr '[a-z]' '[A-Z]'`
	. /lib/zone/zone_api.sh
	zone_add_vgroup "pppoev6-${config}" "${log_wan}"
}

proto_pppoev6_teardown() {
	pppv6_generic_teardown "$@"
	
	#update V zone groups
	local log_wan=`echo "${1%_v6}" |tr '[a-z]' '[A-Z]'`
	. /lib/zone/zone_api.sh
	zone_delete_vgroup "pppoev6-${1}" "${log_wan}"
}

[ -n "$INCLUDE_ONLY" ] || {
	add_protocol pppoev6
}
