#!/bin/sh

ONLINE_STATE_DIR="/tmp/online/"
ONLINE_STATE_FILE="/tmp/online/state"
ONLINE_DEV_FILE="/tmp/online/zone"
ONLINE_LATENCY_FILE="/tmp/online/latency"
ONLINE_BLOCK_FILE="/tmp/online/block"
ONLINE_MODE_ID=26
#MSG(13901, WNG, "[OMADA]%255 The online detection result of [WAN%1] is offline")
#MSG(13902, WNG, "[OMADA]%255 The online detection result of [WAN%1] is online")
LINKBACKUP_MODE_ID=29
#MSG(14235, NTC, "[OMADA]%255 Link Backup %1 took effect.")
#MSG(14236, NTC, "[OMADA]%255 Link Backup %1 was down.")
UP_LOG_ID=14235
DOWM_LOG_ID=14236

. /lib/zone/zone_api.sh
. /lib/functions/network.sh 
. /lib/functions.sh 

change_default_route()
{
	local dev="$1"
	local default=`ip route | awk 'FNR==1 {print $1}'`
	local dev_1=`ip route | awk 'FNR==1 {print $5}'`
	[ "$default" == "default" ] && [ "$dev_1" == "$dev" ] && {
		local dev_on=`grep "=on.*" "$ONLINE_STATE_FILE" | awk 'FNR==1' | awk -F "=" '{print $1}'` 
		[ -n "$dev_on" ] && {
			local gw=`ip route | grep default | grep $dev_on | awk -F ' ' '{printf $3}' `
			route del default gw $gw $dev_on
			route add default gw $gw $dev_on
		}
	}
}

online_set_state()
{
	local dev="$1"
	#[ -n "$key" ] || { echo "set online state failed! can not get interface by device ${1}" > /dev/console }
	local cont="on"
	if [ "$2" = "0" ]; then
		cont="off"
		change_default_route $1 $2
	fi
	
	[ ! -e $ONLINE_STATE_DIR ] && mkdir -p $ONLINE_STATE_DIR
	[ ! -f $ONLINE_STATE_FILE ] && touch $ONLINE_STATE_FILE

	[ -n "$3" ] && {
		local device=""
		local i=""
		global_wanmode="$(uci get network.global.wanmode)"
		for i in `seq 1 $global_wanmode`
		do
			network_get_device device wan${i}_eth
			[ "$device" == "$dev" ] && break
			network_get_device device wan${i}_poe
			[ "$device" == "$dev" ] && break
		done
		[ "$device" == "$dev" ] && echo -e "$3" > ${ONLINE_LATENCY_FILE}${i}
		echo -e "$3" > ${ONLINE_LATENCY_FILE}_${dev}
	}

	local iface=`zone_get_iface_bydev $dev`
	local wan_num=`zone_get_zone_byif $iface`
	grep "^${dev}=.*" "$ONLINE_STATE_FILE" 2>&1 >/dev/null || {
		echo -e "${dev}=${cont}" >> "$ONLINE_STATE_FILE" 2>/dev/null
		if [ "$cont" = "on" ]; then
			logger_reg  $ONLINE_MODE_ID 13902 $wan_num
		elif [ "$cont" = "off" ]; then
			logger_reg  $ONLINE_MODE_ID 13901 $wan_num
		fi 
 		return 0
	}

	local old=""
	old=`grep "^${dev}=.*" "$ONLINE_STATE_FILE"`
	if [ "$old" != "${dev}=${cont}" ]; then
		if [ "$cont" = "on" ]; then
			logger_reg  $ONLINE_MODE_ID 13902 $wan_num
		elif [ "$cont" = "off" ]; then
			logger_reg  $ONLINE_MODE_ID 13901 $wan_num
		fi 
	fi
	sed -i "s/^${dev}=.*/${dev}=${cont}/" $ONLINE_STATE_FILE	
}

online_del_state()
{
	#[ -n "$1" ] || { echo "delete online state failed! no device ${1}" > /dev/console }
	[ -z ${1} ] && return
	local dev="$1"
	#[ -n "$key" ] || { echo "delete online state failed! can not get interface by device ${1}" > /dev/console }
	grep "^${dev}=.*" $ONLINE_STATE_FILE 2>&1 >/dev/null && sed "/^${dev}=.*/d" -i $ONLINE_STATE_FILE
}

online_clear_state()
{
	[ ! -e $ONLINE_STATE_DIR ] && return 
	[ ! -f $ONLINE_STATE_FILE ] && return 
	echo >$ONLINE_STATE_FILE 
}

add_online_check()
{
	[ -n "$1" ] || return
	local zone_or_iface="$1"
	for cnt in $(seq 1 300); do
		dev=`zone_get_effect_devices $zone_or_iface`
		if [ -n "$dev" ]; then
			break
		fi	
		sleep 1
	done
	#[ -n "$dev" ] || continue

	ubus call online add_info "{\"dev\":\"$dev\"}"
	add_online_dev_state ${zone_or_iface} ${dev}
}

add_online_dev_state()
{
	[ -z ${1} ] && return
	[ -z ${2} ] && return 
	local zone_or_iface="$1"
	local dev="$2"

	[ ! -e $ONLINE_STATE_DIR ] && mkdir -p $ONLINE_STATE_DIR
	[ ! -f $ONLINE_DEV_FILE ] && touch $ONLINE_DEV_FILE

	grep "^${zone_or_iface}=.*" "$ONLINE_DEV_FILE" 2>&1 >/dev/null || {
		echo -e "${zone_or_iface}=${dev}" >> "$ONLINE_DEV_FILE" 2>/dev/null
		return 0
	}
	sed -i "s/^${zone_or_iface}=.*/${zone_or_iface}=${dev}/" $ONLINE_DEV_FILE

}

del_online_check()
{
	[ -z "$1" ] && return
	local zone_or_iface="$1"
	local dev="`cat $ONLINE_DEV_FILE |grep "\<${zone_or_iface}\>" |cut -d '=' -f 2`"
	[ -z "$dev" ] && return

	ubus call online delete_info "{\"dev\":\"$dev\"}"
	del_online_dev_state ${zone_or_iface}
}

del_online_dev_state()
{
	[ -z ${1} ] && return
	local zone_or_iface="$1"
	grep "^${zone_or_iface}=.*" $ONLINE_DEV_FILE 2>&1 >/dev/null && sed "/^${zone_or_iface}=.*/d" -i $ONLINE_DEV_FILE
}

flush_online_check()
{
	. /usr/share/libubox/jshn.sh

	jdata=`ubus call online get_info`
	json_load "$jdata"
	json_get_keys jkeys
	for jkey in $jkeys
	do
		json_select "$jkey"
		json_get_var jval dev
		ubus call online delete_info "{\"dev\":\"$jval\"}"
		json_select ..
	done
}

update_online_manual_check()
{
	[ -n "$1" ] || [ -n "$2" ] || [ -n "$3" ] || [ -n "$4" ] || return

	. /lib/zone/zone_api.sh
	dev=`zone_get_effect_devices $1`
	[ -n "$dev" ] || return

	ubus call online update_manual_info "{\"dev\":\"$dev\",\"mode\":\"$2\",\"manual_gw\":\"$3\",\"manual_dns\":\"$4\"}"
}

#$1: ifaces
del_ifaces_online_check()
{
	[ -n "$1" ] || return

	for i in $1;do
		del_online_check $i
	done
}

#$1: interval
update_online_interval()
{
	[ -z "$1" ] && return
	ubus call online update_interval "{\"interval\":$1}"
}

cmpslave()
{
        config_get slave_iface $1 slave_iface
        local slave_dev=`zone_get_devices $slave_iface`
        # shell内调用for循环效率较低，不推荐；但此处slave_dev最多包含两个值，影响较小
        for sdev in $slave_dev
        do
                if [ "$sdev" == "$dev" ]; then
                        if [ "$blocked" == "no" ]; then
                                ubus call online refresh "{\"dev\":\"$dev\"}"
                                logger_reg $LINKBACKUP_MODE_ID $DOWM_LOG_ID "Backup" $slave_iface
                        elif [ "$blocked" == "yes" ]; then
                                ubus call online refresh "{\"dev\":\"$dev\"}"
                                logger_reg $LINKBACKUP_MODE_ID $UP_LOG_ID "Backup" $slave_iface
                        fi
                        slave_flag=1
                fi
        done
}

#$1: zone name
#$2: block status
#return old status
update_online_dev_blocked()
{
	. /lib/zone/zone_api.sh
	devs=`zone_get_devices $1`
        slave_flag=0

	blocked="no"
	for dev in $devs
	do
		if [ -e ${ONLINE_BLOCK_FILE}"_${dev}" ];
		then
			blocked="yes"
		fi
	done

	if [ "$2" != "0" ];
	then
		for dev in $devs
		do
			echo -n "$1" > ${ONLINE_BLOCK_FILE}"_${dev}"
			if [ "$blocked" == "no" ]; then 
				config_load line_backup
				config_foreach cmpslave rule
				if [ "$slave_flag" == "0" ]; then
					ubus call online refresh "{\"dev\":\"$dev\"}"
					logger_reg $LINKBACKUP_MODE_ID $DOWM_LOG_ID "Primary" $1
				else
					slave_flag=0
				fi
			fi
		done
	else
		for devfile in `ls ${ONLINE_BLOCK_FILE}_*`
		do
			if [ "$1" == "`cat ${devfile}`" ];
			then
				rm -f ${devfile}
				dev="${devfile##${ONLINE_BLOCK_FILE}_}"
				iface=`zone_get_iface_bydev $dev`
				if [ "x$iface" != "x" -a "$blocked" == "yes" ]; then
					config_load line_backup
					config_foreach cmpslave rule
					if [ "$slave_flag" == "0" ]; then
						ubus call online refresh "{\"dev\":\"$dev\"}"
						logger_reg $LINKBACKUP_MODE_ID $UP_LOG_ID "Primary" $1
					else
						slave_flag=0
					fi
				fi
			fi
		done
	fi
}
