local dbg = require "luci.tools.debug"
local sys = require "luci.sys"
local utl = require "luci.util"
local uci = require "luci.model.uci"
local uci_r = uci.cursor()

module("pppox_functions", package.seeall)

local user_config = "user-secrets"
local pppox_l2tptype = "l2tp"
local pppox_pptptype = "pptp"
local pppox_pppoetype = "pppoe"
local pppox_path = "/tmp/pppox/"
local pppox_l2tp_path = pppox_path .. "l2tp/"
local pppox_pptp_path = pppox_path .. "pptp/"
local pppox_pppoe_path= pppox_path .. "pppoe/"
local pppox_l2tp_server_path = "/tmp/l2tp/server/"
local pppox_pptp_server_path = "/tmp/pptp/server/"
local pppox_if_config = "if-config"
local pppox_user_secrets = "user-secrets"
local pppox_profile = "-profile"
local pppox_profilex = "-profilex"

local initl2tp = false
local initpptp = false
local initpppoe = false

local function to_ascii(str)
	local len = string.len(str)
	local res = ""
	
	for i = 1, len do
		res = res .. string.format("%X", string.byte(str, i, i))
	end
	return res
end

local function  _echo_info(path, server_path, username, password, localip, ippool, ms_dns, ms_dns1, ms_dns2, maxsessions, netmode, remotesubnet, serverlist, servername)
	local password1=string.gsub(password, "\\", "\\\\")
	password1=string.gsub(password1, "\'", "\\\'")
	local username1=string.gsub(username, "\\", "\\\\")
	username1=string.gsub(username1, "\'", "\\\'")
	
	local username2=to_ascii(username)
	
	local filepath = path .. pppox_user_secrets
	local ff
	ff = io.open(filepath, "a")
	ff:write("'" .. username1 .. "' * '" .. password1 .. "' *\n")
	ff:close()
	
	filepath = path .. username2 .. pppox_profile
	ff = io.open(filepath, "a")
	ff:write(localip .. ":\nippool " .. ippool .. "\n")
	if ms_dns then
		ff:write("ms-dns " .. "'" ..  ms_dns  .. "'" .. "\n")
	end
	if ms_dns1 then
		ff:write("ms-dns " .. "'" ..  ms_dns1  .. "'" .. "\n")
	end
	if ms_dns2 then
		ff:write("ms-dns " .. "'" ..  ms_dns2  .. "'" .. "\n")
	end
	ff:close()
	
	filepath = path .. username2 .. pppox_profilex
	ff = io.open(filepath, "a")
	ff:write("maxsessions " ..  maxsessions .. "\n")
	ff:close()
	filepath = path .. pppox_if_config
	ff = io.open(filepath, "a")
	ff:write("config if " .. "'" .. username2 .. "'" .. "\n\toption netmode " .. "'" .. netmode .. "'" .. 
			"\n\toption remotesubnet " .. "'" .. remotesubnet .. "'" .. "\n")
	ff:close()

	if servername == nil then
		if serverlist == nil then
			local server_cmd = "ls \"" .. server_path .. "\" | grep -v server-global | grep -v wan | grep -v maxsessions 2>/dev/null"
			local server_output = io.popen(server_cmd)
			for line in server_output:lines() do
				local servers = utl.split(line, ' ')
				for _, v in ipairs(servers) do
					filepath = server_path .. v .. "/config/" .. pppox_user_secrets
					--dbg("filepath "..filepath)
					local ff = io.open(filepath, "a")
					ff:write("'" .. username1 .. "' * '" .. password1 .. "' *\n")
					ff:close()
					
					filepath = server_path .. v .. "/config/" .. username2 .. pppox_profile
					ff = io.open(filepath, "a")
					--ff:write(localip .. ":\nippool " .. ippool .. "\nms-dns " .. ms_dns .. "\n")
					ff:write(localip .. ":\nippool " .. ippool .. "\n")
					if ms_dns then
						ff:write("ms-dns " .. "'" ..  ms_dns  .. "'" .. "\n")
					end
					if ms_dns1 then
						ff:write("ms-dns " .. "'" ..  ms_dns1  .. "'" .. "\n")
					end
					if ms_dns2 then
						ff:write("ms-dns " .. "'" ..  ms_dns2  .. "'" .. "\n")
					end
					ff:close()
					
					filepath = server_path .. v .. "/config/" .. username2 .. pppox_profilex
					ff = io.open(filepath, "a")
					ff:write("maxsessions " ..  maxsessions .. "\n")
					ff:close()
					
					filepath = server_path .. v .. "/config/" .. pppox_if_config
					ff = io.open(filepath, "a")
					ff:write("config if " .. "'" .. username2 .. "'" .. "\n\toption netmode " .. "'" .. netmode .. "'" .. 
							"\n\toption remotesubnet " .. "'" .. remotesubnet .. "'" .. "\n")
					ff:close()
				end
			end
		else
			local serverlist = utl.split(serverlist, ' ')
			serverlist = (type(serverlist)=="table") and serverlist or {serverlist}
			for _, v in ipairs(serverlist) do
				local pptp_match = false
				local l2tp_match = false
				local tmp_table = utl.split(v, "/", nil, true)
				serverlist_type = table.remove(tmp_table, 1)
				serverlist_id = table.remove(tmp_table, 1)
				serverlist_pool = table.remove(tmp_table, 1)
				serverlist_local = table.concat(tmp_table, "/")
				serverlist_local = string.gsub(serverlist_local,"-"," ")
				if (serverlist_type == pppox_pptptype) and (initpptp and (server_path == pppox_pptp_server_path))then
					pptp_match = true
				end
				if (serverlist_type == pppox_l2tptype) and (initl2tp and (server_path == pppox_l2tp_server_path))then
					l2tp_match = true
				end
				if pptp_match or l2tp_match then
					filepath = server_path .. serverlist_id .. "/config/" .. pppox_user_secrets
					local ff = io.open(filepath, "a")
					ff:write("'" .. username1 .. "' * '" .. password1 .. "' *\n")
					ff:close()
					
					filepath = server_path .. serverlist_id .. "/config/" .. username2 .. pppox_profile
					ff = io.open(filepath, "a")
					--ff:write(localip .. ":\nippool " .. serverlist_pool .. "\nms-dns " .. ms_dns .. "\n")
					ff:write(localip .. ":\nippool " .. serverlist_pool .. "\n")
					if ms_dns then
						ff:write("ms-dns " .. "'" ..  ms_dns  .. "'" .. "\n")
					end
					if ms_dns1 then
						ff:write("ms-dns " .. "'" ..  ms_dns1  .. "'" .. "\n")
					end
					if ms_dns2 then
						ff:write("ms-dns " .. "'" ..  ms_dns2  .. "'" .. "\n")
					end
					ff:close()
					
					filepath = server_path .. serverlist_id .. "/config/" .. username2 .. pppox_profilex
					ff = io.open(filepath, "a")
					ff:write("maxsessions " ..  maxsessions .. "\n")
					ff:close()
					
					filepath = server_path .. serverlist_id .. "/config/" .. pppox_if_config
					ff = io.open(filepath, "a")
					ff:write("config if " .. "'" .. username2 .. "'" .. "\n\toption netmode " .. "'" .. netmode .. "'" .. 
							"\n\toption remotesubnet " .. "'" .. remotesubnet .. "'" .. "\n\toption local_network " .. "'" .. serverlist_local .. "'" .. "\n")
					ff:close()
				end
			end
		end
	else
		if serverlist == nil then
			filepath = server_path .. servername .. "/config/" .. pppox_user_secrets
			local ff = io.open(filepath, "a")
			ff:write("'" .. username1 .. "' * '" .. password1 .. "' *\n")
			ff:close()
			
			filepath = server_path .. servername .. "/config/" .. username2 .. pppox_profile
			ff = io.open(filepath, "a")
			--ff:write(localip .. ":\nippool " .. ippool .. "\nms-dns " .. ms_dns .. "\n")
			ff:write(localip .. ":\nippool " .. ippool .. "\n")
			if ms_dns then
				ff:write("ms-dns " .. "'" ..  ms_dns  .. "'" .. "\n")
			end
			if ms_dns1 then
				ff:write("ms-dns " .. "'" ..  ms_dns1  .. "'" .. "\n")
			end
			if ms_dns2 then
				ff:write("ms-dns " .. "'" ..  ms_dns2  .. "'" .. "\n")
			end
			ff:close()
			
			filepath = server_path .. servername .. "/config/" .. username2 .. pppox_profilex
			ff = io.open(filepath, "a")
			ff:write("maxsessions " ..  maxsessions .. "\n")
			ff:close()
			
			filepath = server_path .. servername .. "/config/" .. pppox_if_config
			ff = io.open(filepath, "a")
			ff:write("config if " .. "'" .. username2 .. "'" .. "\n\toption netmode " .. "'" .. netmode .. "'" .. 
					"\n\toption remotesubnet " .. "'" .. remotesubnet .. "'" .. "\n")
			ff:close()
		else
			local serverlist = utl.split(serverlist, ' ')
			serverlist = (type(serverlist)=="table") and serverlist or {serverlist}
			for _, v in ipairs(serverlist) do
				local pptp_match = false
				local l2tp_match = false
				local pppox_match = false
				local tmp_table = utl.split(v, "/", nil, true)
				serverlist_type = table.remove(tmp_table, 1)
				serverlist_id = table.remove(tmp_table, 1)
				serverlist_pool = table.remove(tmp_table, 1)
				serverlist_local = table.concat(tmp_table, "/")
				serverlist_local = string.gsub(serverlist_local,"-"," ")
				if (serverlist_type == pppox_pptptype) and (initpptp and (server_path == pppox_pptp_server_path))then
					pptp_match = true
				end
				if (serverlist_type == pppox_l2tptype) and (initl2tp and (server_path == pppox_l2tp_server_path))then
					l2tp_match = true
				end
				if pptp_match or l2tp_match then
					pppox_match = true
				end
				if  pppox_match and serverlist_id == servername then
					filepath = server_path .. servername .. "/config/" .. pppox_user_secrets
					local ff = io.open(filepath, "a")
					ff:write("'" .. username1 .. "' * '" .. password1 .. "' *\n")
					ff:close()
					
					filepath = server_path .. servername .. "/config/" .. username2 .. pppox_profile
					ff = io.open(filepath, "a")
					--ff:write(localip .. ":\nippool " .. serverlist_pool .. "\nms-dns " .. ms_dns .. "\n")
					ff:write(localip .. ":\nippool " .. serverlist_pool .. "\n")
					if ms_dns then
						ff:write("ms-dns " .. "'" ..  ms_dns  .. "'" .. "\n")
					end
					if ms_dns1 then
						ff:write("ms-dns " .. "'" ..  ms_dns1  .. "'" .. "\n")
					end
					if ms_dns2 then
						ff:write("ms-dns " .. "'" ..  ms_dns2  .. "'" .. "\n")
					end
					ff:close()
					
					filepath = server_path .. servername .. "/config/" .. username2 .. pppox_profilex
					ff = io.open(filepath, "a")
					ff:write("maxsessions " ..  maxsessions .. "\n")
					ff:close()
					
					filepath = server_path .. servername .. "/config/" .. pppox_if_config
					ff = io.open(filepath, "a")
					ff:write("config if " .. "'" .. username2 .. "'" .. "\n\toption netmode " .. "'" .. netmode .. "'" .. 
							"\n\toption remotesubnet " .. "'" .. remotesubnet .. "'" .. "\n\toption local_network " .. "'" ..  serverlist_local .. "'" .. "\n")
					ff:close()
				end
			end
		end
	end
	return true
end


local function _load_one_user(section, servername)
	if section.type == pppox_l2tptype and initl2tp then
		_echo_info(pppox_l2tp_path, pppox_l2tp_server_path, section.username, section.password, section.localip, 
			section.ippool, section.ms_dns, section.ms_dns1, section.ms_dns2, section.maxsessions, section.netmode, section.remotesubnet, section.serverlist, servername)
	elseif section.type == pppox_pptptype and initpptp then
		_echo_info(pppox_pptp_path, pppox_pptp_server_path, section.username, section.password, section.localip, 
			section.ippool, section.ms_dns, section.ms_dns1, section.ms_dns2, section.maxsessions, section.netmode, section.remotesubnet, section.serverlist, servername)
	--[[
	elseif	section.type == pppox_pppoetype and initpppoe then
		_echo_info(pppox_pppoe_path, section.username, section.password, section.localip, 
			section.ippool, section.ms_dns, section.maxsessions, section.netmode, section.remotesubnet)
	]]--
	elseif	section.type == "auto"  then
		if initl2tp then
			_echo_info(pppox_l2tp_path, pppox_l2tp_server_path, section.username, section.password, section.localip, 
			section.ippool, section.ms_dns, section.ms_dns1, section.ms_dns2, section.maxsessions, section.netmode, section.remotesubnet, section.serverlist, servername)
		end
		if initpptp then
			_echo_info(pppox_pptp_path, pppox_pptp_server_path, section.username, section.password, section.localip, 
			section.ippool, section.ms_dns, section.ms_dns1, section.ms_dns2, section.maxsessions, section.netmode, section.remotesubnet, section.serverlist, servername)
		end
		--[[ 
		if initpppoe then
			_echo_info(pppox_pppoe_path, section.username, section.password, section.localip, 
			section.ippool, section.ms_dns, section.maxsessions, section.netmode, section.remotesubnet)
		end
		]]--
	end
	
	return true
end

function pppox_load_user(servername)
	uci_r:foreach(user_config, "user", function(section)
			_load_one_user(section, servername)
            return true
        end
        )
	return true
end

if arg[1] ~= "0" then
	initl2tp = true
end

if arg[2] ~= "0" then
	initpptp = true
end

-- servername: like server1, maybe empty
servername = arg[3]
--[[
if arg[3] ~= "0" then
	initpppoe = true
end
]]--

pppox_load_user(servername)
