#!/bin/sh
#Copyright (C) 2015 tplink.com

. /lib/zone/zone_conf.sh

local IPT4=iptables
local IPT6=ip6tables
local TABLE=mangle
local IPT4_PREFIX="$IPT4 -w -t $TABLE"
local IPT6_PREFIX="$IPT6 -w -t $TABLE"
local QOS_CHAIN="qosctl_default"
local QOS_RULE_CHAIN="qosctl_rule_mark"
local class1_mark="0x2000/0x6000"
local class2_mark="0x4000/0x6000"
local class3_mark="0x6000/0x6000"
local class4_mark="0x0/0x0"
local class1_ormark="0x2000"
local class2_ormark="0x4000"
local class3_ormark="0x6000"
local class4_ormark="0x0"
local clean_mark="0x0/0xe000"
local prio_connmark="0x8000"
local prio_mark="0x8000"
local wan1_ormark="0x40000"
local wan2_ormark="0x80000"
local wan3_ormark="0xc0000"
local wan4_ormark="0x20000000"
local wan1_mark="0x40000/0xc0000"
local wan2_mark="0x80000/0xc0000"
local wan3_mark="0xc0000/0xc0000"
local wan4_mark="0x20000000/0x20000000"
local QOS_HWNAT_CHAIN="hwnat_mark"
local THIS_MODULE="qos_ctl"
local QOS_CONFIG_DIR="/etc/config"
local QOS_UCI="${QOS_CONFIG_DIR}/${THIS_MODULE}"
local QOS_TMP_DIR="/tmp/.qos_ctl"
local QOS_FILE_IP4="$QOS_TMP_DIR/ip4_rule"
local QOS_FILE_IP6="$QOS_TMP_DIR/ip6_rule"
local QOS_FILE_IP4_BW="$QOS_TMP_DIR/ip4_rule_bw"
local QOS_FILE_IP6_BW="$QOS_TMP_DIR/ip6_rule_bw"
local QOS_STAT_READY="$QOS_TMP_DIR/ready"
local QOS_STAT_UPDATE="$QOS_TMP_DIR/need_update"
local QOS_STAT_BUSY="$QOS_TMP_DIR/busy"
local QOS_IPT_BAK_TMP="/tmp/.qos_ctl/.mangle_ipt.bak"
local QOS_IPT_TMP="/tmp/.qos_ctl/.mangle_ipt.tmp"
local QOS_IPT_RES="/tmp/.qos_ctl/.mangle_ipt.res"
local SLEEP_TIME=5
local IPT4_RESTORE="iptables-restore -w -n"
local IPT6_RESTORE="ip6tables-restore -w -n"
local IPT4_SAVE="iptables-save -t $TABLE"
local IPT6_SAVE="ip6tables-save -t $TABLE"

#qos_iptables_add add/insert 4/6 params
qos_iptables_add()
{
    local OpMode=$1
    local IpVersion=$2
    local Params1=$3
    local Params2=$4
    local OpString="-A" #默认为添加
    local IpTablesCom=$IPT4_PREFIX  #默认为ipv4
    local IpTablesRes=$IPT4_RESTORE
    local IpTablesSave=$IPT4_SAVE

    #确定添加方式
    case $OpMode in
    "add")
        OpString="-A"
        ;;
    "insert")
        OpString="-I"
        ;;
    esac

    #确定调用版本
    case $IpVersion in
    "4")
        IpTablesCom=$IPT4_PREFIX
        IpTablesRes=$IPT4_RESTORE
        IpTablesSave=$IPT4_SAVE
        ;;
    "6")
        IpTablesCom=$IPT6_PREFIX
        IpTablesRes=$IPT6_RESTORE
        IpTablesSave=$IPT6_SAVE
        ;;
    esac

    #检查条目是否已存在，不存在才添加
    if ! $IpTablesCom -C $Params1 2> /dev/null ;then
        #$IpTablesCom $OpString $Params2
        #$IpTablesCom $OpString $Params1
        $IpTablesSave > $QOS_IPT_BAK_TMP
        
        cat /tmp/.qos_ctl/.mangle_ipt.bak | grep "\-A qosctl_rule_mark" > $QOS_IPT_TMP
	echo "${OpString} $Params1" >> $QOS_IPT_TMP
        echo "${OpString} $Params2" >> $QOS_IPT_TMP
        echo "*mangle" > $QOS_IPT_RES
	for i in `cat /tmp/.qos_ctl/.mangle_ipt.tmp |awk -F ':' '{print $2}'|sort -n|awk -F '"' '{print $1}'`
	do
	        cat $QOS_IPT_TMP | grep -w "idx:$i" >> $QOS_IPT_RES
	done
	echo "COMMIT" >> $QOS_IPT_RES
        $IpTablesCom -F ${QOS_RULE_CHAIN}
        $IpTablesRes < $QOS_IPT_RES
    else
        echo "##rule exists: $IpTablesCom $OpString $Params" > /dev/console
    fi
}

qos_wanmax()
{
    local switch_wanmax="$(uci -c /etc/profile.d get profile.@switch0_user[0].pwanmax)"
    local phy_wanmax="$(uci -c /etc/profile.d get profile.@phy1[0].pwanmax)"
    local phy_name="$(uci -c /etc/profile.d get profile.@phy1[0].pname)"
    if [ "$(echo ${phy_name}|grep WAN)" != "" ];then
        echo "$(($switch_wanmax+$phy_wanmax))"
    else
        echo "${switch_wanmax}"
    fi
}

qos_wanlist_get()
{
    local wanlist=""
    local WANMAX="$(qos_wanmax)"
    for i in `seq ${WANMAX}` ;do
        wanlist="${wanlist} WAN${i}"
    done
    echo "${wanlist}"
}

qos_chainlist_get()
{
    local chain_list=""
    local WANMAX="$(qos_wanmax)"
    for i in `seq ${WANMAX}` ;do
        local up_chain="qosctl_WAN${i}_up"
        local down_chain="qosctl_WAN${i}_down"
        chain_list="${chain_list} ${up_chain} ${down_chain}"
    done
    echo "${chain_list}"
}

qos_rulelist_get()
{
    local list=`cat $QOS_UCI | grep "option id "|sed "s/'//g" |awk '{print $3}' NR==1 `
    echo $list
}

fastpath_sw_real_on()
{
        local sfe_switch=`uci -q get sfe.setting.switch`
        [ "$sfe_switch" == "on" ]
}

fastpath_hw_real_on()
{
        local nss_switch=`uci -q get ecm.global.enable`
        local hwnat_switch=`uci -q get hwnat.global.enabled`
        [ "$nss_switch" == "on" -o "$hwnat_switch" == "1" ]
}

qos_nf_sfe_toggle()
{
    local chain="$QOS_CHAIN"

    # should clean hwnat/sfe connections after iptables rules be renewed
    echo qos_nf_sfe_toggle >> /dev/console

    #conntrack -U -m $clean_mark 2>/dev/null
    #conntrack -f 'ipv6' -U -m $clean_mark 2>/dev/null
    #sleep $SLEEP_TIME
    if fastpath_sw_real_on;then
                xfwdm -F
    fi
    if fastpath_hw_real_on; then
    #    echo 1 > /sys/kernel/debug/ecm/ecm_db/defunct_all
        [ -e "/etc/init.d/hwnat" ] && hwnat -y
    fi
    conntrack -F && echo "conntrack clear">/dev/console
    if fastpath_sw_real_on; then
        xfwdm -F
    fi
}


# flush all qos chains
#   unmount the target to qos-chains
qos_nf_destroy()
{
    local chain_list="$QOS_CHAIN $(qos_chainlist_get) $QOS_RULE_CHAIN"
    for chain in $chain_list;do
    # qos single iface chain flush
        local ln=`$IPT4_PREFIX -L FORWARD --line | grep -w "$chain" | sort -nr | awk '{print $1}'`
        [ ${#ln} -ne 0 ] && {
            for sub_ln in ${ln};do
                $IPT4_PREFIX -D FORWARD $sub_ln
            done
        }
        ln=`$IPT6_PREFIX -L FORWARD --line | grep -w "$chain" | sort -nr | awk '{print $1}'`
        [ ${#ln} -ne 0 ] && {
            for sub_ln in ${ln};do
                $IPT6_PREFIX -D FORWARD $sub_ln
            done
        }
        $IPT4_PREFIX -F $chain 2>/dev/null
        $IPT4_PREFIX -X $chain 2>/dev/null
        $IPT6_PREFIX -F $chain 2>/dev/null
        $IPT6_PREFIX -X $chain 2>/dev/null
    done
}

# # create the qos chain named src_to_dst, if exist, return
qos_nf_chain_create()
{

    local chain_pare="$QOS_CHAIN"

# create qos_default target to
    
    $IPT4_PREFIX -N $chain_pare 2>/dev/null
    $IPT4_PREFIX -N $QOS_RULE_CHAIN 2>/dev/null
    $IPT6_PREFIX -N $chain_pare 2>/dev/null
    $IPT6_PREFIX -N $QOS_RULE_CHAIN 2>/dev/null
    local ln=`$IPT4_PREFIX  -L FORWARD --line | awk '/zone.*MSSFIX/ {print $1}' | awk 'NR==1'`
    [ ${#ln} -ne 0 ] && plus="-I FORWARD $ln" || plus="-A FORWARD"
    $IPT4_PREFIX $plus -j $chain_pare

    ln=`$IPT6_PREFIX  -L FORWARD --line | awk '/zone.*MSSFIX/ {print $1}' | awk 'NR==1'`
    [ ${#ln} -ne 0 ] && plus="-I FORWARD $ln" || plus="-A FORWARD"
    $IPT6_PREFIX $plus -j $chain_pare
    
    local chain_list="$(qos_chainlist_get)" 
    for chain in $chain_list;do
        $IPT4_PREFIX -N $chain 2>/dev/null   
        $IPT6_PREFIX -N $chain 2>/dev/null   
    done
    # Init status flags
    local WANMAX="$(qos_wanmax)"
    for i in `seq ${WANMAX}` ;do
        uci set qos_ctl.setting.update_bw${i}="0"
    done
    uci set qos_ctl.setting.qos_stat_ready_bw="1"
    uci set qos_ctl.setting.qos_stat_busy_bw="0"
    uci commit qos_ctl
    return 0
}



# qos_ctl_default:
# -o WAN1_ZONE -j qos_WAN1
# -i WAN1_ZONE -j qos_WAN1
# qos_WAN1:
# -j hwnat_mark
# ret: 0 OK
qos_nf_update_bandwidth()
{   
    local iface_list=$(qos_wanlist_get)
    local is_init="0"
    if [ -n "$1" ] ;then
         iface_list="$1"
    else
        is_init="1"
    fi
    local chain_pare=$QOS_CHAIN
    local IPT_LIST="$IPT4 $IPT6"
    local count=1
    local zone_lan=$(zone_group LAN)
    local ifupdate=0
    local WANMAX=$(qos_wanmax)

    while [ $ifupdate -gt 0 ] || [ $count -eq 1 ]; do    
        if [ "$(uci get qos_ctl.setting.update_bw${ifupdate})" == "1" ];then
            uci set qos_ctl.setting.update_bw${ifupdate}="0"    
            iface_list="WAN${ifupdate}"
            uci commit qos_ctl
        fi
        ifupdate=$((0))

        uci set qos_ctl.setting.qos_stat_busy_bw="1"
        uci set qos_ctl.setting.qos_stat_ready_bw="0" 2>/dev/null
        uci commit qos_ctl

        for iface in $iface_list ;do
            local chain_if="qosctl_$iface"
            local zone_if=$(zone_vgname ${iface})
            local QOS_FILE_IP4_BW=$QOS_FILE_IP4_BW
            local QOS_FILE_IP6_BW=$QOS_FILE_IP6_BW
            echo "*mangle" > $QOS_FILE_IP4_BW
            echo "*mangle" > $QOS_FILE_IP6_BW
            local status=`uci -q get qos_ctl.$iface.status`
            for IPT in $IPT_LIST;do
                [ $IPT = $IPT4 ] && QOS_FILE=$QOS_FILE_IP4_BW || QOS_FILE=$QOS_FILE_IP6_BW
                local IPT_PREFIX="$IPT -w -t $TABLE"
                local IPT_RESTORE="${IPT}-restore -w -n"
                local ln=`$IPT_PREFIX -L FORWARD --line | grep -w "$chain_pare" | awk '{print $1}' NR==1`
                [ ${#ln} -eq 0 ] && return
                ln=`$IPT_PREFIX -L $chain_pare --line | grep "${chain_if}_*" | awk '{print $1}' NR==1`
                [ ${#ln} -ne 0 ] && {
                    local start_ln=0
                    local count=0
                    for i in ${ln};do
                        [ $count -eq 0 ] && start_ln=$i && count=$(($count+1))
                        $IPT_PREFIX -D $chain_pare $start_ln
                    done
                }
                $IPT_PREFIX -F ${chain_if}_up 2>/dev/null
                $IPT_PREFIX -F ${chain_if}_down 2>/dev/null
                if [ ${#status} -eq 0 ]||[ $status != "enable" ];then
                    echo "status disable,return!"
                else
                    local direction=`uci -q get qos_ctl.$iface.direction`
                    echo "-A ${chain_if}_up -j $QOS_HWNAT_CHAIN" >> $QOS_FILE
                    echo "-A ${chain_if}_down -j $QOS_HWNAT_CHAIN" >> $QOS_FILE
                    if [ $direction = "out" -o $direction = "both" ];then
                        echo "-A ${chain_if}_up -j $QOS_RULE_CHAIN" >> $QOS_FILE
                    fi
                    if [ $direction = "in" -o $direction = "both" ];then
                        echo "-A ${chain_if}_down -j $QOS_RULE_CHAIN" >> $QOS_FILE
                    fi
                    local wan_mark=""
                    local wan_ormark=""
                    [ $iface = "WAN1" ] && wan_mark=$wan1_mark && wan_ormark=$wan1_ormark
                    [ $iface = "WAN2" ] && wan_mark=$wan2_mark && wan_ormark=$wan2_ormark
                    [ $iface = "WAN3" ] && wan_mark=$wan3_mark && wan_ormark=$wan3_ormark
                    [ $iface = "WAN4" ] && wan_mark=$wan4_mark && wan_ormark=$wan4_ormark
                
                    echo "-A ${chain_if}_up -j CONNMARK --or-mark $wan_ormark" >> $QOS_FILE
                    echo "-A ${chain_if}_down -j CONNMARK --or-mark $wan_ormark" >> $QOS_FILE
                    local uplink_tol=`uci -q get qos_ctl.$iface.outbound`
                    local downlink_tol=`uci -q get qos_ctl.$iface.inbound`
                    local udp_status=`uci -q get qos_ctl.$iface.udpEnable`
                    local udp=`uci -q get qos_ctl.$iface.udp`
                    local ack=`uci -q get qos_ctl.$iface.ack`
                    local class1=`uci -q get qos_ctl.$iface.class1`  #percentance
                    local class2=`uci -q get qos_ctl.$iface.class2`
                    local class3=`uci -q get qos_ctl.$iface.class3`
                    local class4=`uci -q get qos_ctl.$iface.others`
#### dscp  
                    local tag1_status=`uci -q get qos_ctl.setting.status1`
                    local tag1_str=""
                    [ ${tag1_status} == "enable" ] && {
                        local tag1=`uci -q get qos_ctl.setting.tag1`
                        [ ${#tag1} -ne 0 ] && tag1_str=" --set-dscp ${tag1}"
                    }
                    local tag2_status=`uci -q get qos_ctl.setting.status2`
                    local tag2_str=""
                    [ ${tag2_status} == "enable" ] && { 
                        local tag2=`uci -q get qos_ctl.setting.tag2`
                        [ ${#tag2} -ne 0 ] && tag2_str=" --set-dscp ${tag2}"
                    }
                    local tag3_status=`uci -q get qos_ctl.setting.status3`
                    local tag3_str=""
                    [ ${tag3_status} == "enable" ] && {
                        local tag3=`uci -q get qos_ctl.setting.tag3`
                        [ ${#tag3} -ne 0 ] && tag3_str=" --set-dscp ${tag3}"
                    }
                    local tag4_status=`uci -q get qos_ctl.setting.status4`
                    local tag4_str=""
                    [ ${tag4_status} == "enable" ] && {
                        local tag4=`uci -q get qos_ctl.setting.tag4`
                        [ ${#tag4} -ne 0 ] && tag4_str=" --set-dscp ${tag4}"
                    }
    
##### direction
                    ln=`$IPT_PREFIX -L $chain_pare --line | grep -w "${chain_if}_up" | awk '{print $1}'`
                    [ ${#ln} -eq 0 ] && {
                    $IPT_PREFIX -A ${chain_pare} -m set --match-set ${zone_if} dst -m set --match-set ${zone_lan} src -j ${chain_if}_up 
                    }
                    ln=`$IPT_PREFIX -L $chain_pare --line | grep -w "${chain_if}_down" | awk '{print $1}'`
                    [ ${#ln} -eq 0 ] && {
                    $IPT_PREFIX -A ${chain_pare} -m set --match-set ${zone_if} src -m set --match-set ${zone_lan} dst -j ${chain_if}_down 
                    }
#### parse 
                    if [ $direction = "in" -o $direction = "both" ];then
###### udp & class
                    local class_list="class1 class2 class3 class4"
                        local udp_str=""
                        for class in $class_list;do
                            local class_str=""
                        local class_mark=""
                            local tag_str=""
                            if [ $class = "class1" ];then
                                class_str=$class1
                            class_mark=" -m connmark --mark $class1_mark"
                                tag_str=$tag1_str
                                [ ${udp_status} == "enable" ] && {
                                    local udp_rate=$((${class_str}*${udp}*${downlink_tol}/10000))
                                    [ $(($udp_rate)) -lt 100 ] && udp_rate=100
                                    [ $(($udp_rate)) -gt 10000000 ] && udp_rate=10000000
                                    udp_str="--udp-rate $udp_rate"
                                }
                            elif [ $class = "class2" ];then
                                class_str=$class2
                            class_mark=" -m connmark --mark $class2_mark"
                                tag_str=$tag2_str
                                [ ${udp_status} == "enable" ] && {
                                    local udp_rate=$((${class_str}*${udp}*${downlink_tol}/10000))
                                    [ $(($udp_rate)) -lt 100 ] && udp_rate=100
                                    [ $(($udp_rate)) -gt 10000000 ] && udp_rate=10000000
                                    udp_str="--udp-rate $udp_rate"
                                }
                            elif [ $class = "class3" ];then
                                class_str=$class3
                            class_mark=" -m connmark --mark $class3_mark"
                                tag_str=$tag3_str
                                [ ${udp_status} == "enable" ] && {
                                    local udp_rate=$((${class_str}*${udp}*${downlink_tol}/10000))
                                    [ $(($udp_rate)) -lt 100 ] && udp_rate=100
                                    [ $(($udp_rate)) -gt 10000000 ] && udp_rate=10000000
                                    udp_str="--udp-rate $udp_rate"
                                }
                            else
                                class_str=$class4
                            class_mark=" -m connmark --mark $class4_mark"
                                tag_str=$tag4_str
                                [ ${udp_status} == "enable" ] && {
                                    local udp_rate=$((${class_str}*${udp}*${downlink_tol}/10000))
                                    [ $(($udp_rate)) -lt 100 ] && udp_rate=100
                                    [ $(($udp_rate)) -gt 10000000 ] && udp_rate=10000000
                                    udp_str="--udp-rate $udp_rate"
                                }
                            fi
                            [ ${#class_str} -ne 0 ] && {
                                local downlink_str=$((${class_str}*${downlink_tol}/100))
                                [ $(($downlink_str)) -lt 100 ] && downlink_str=100
                                [ $(($downlink_str)) -gt 10000000 ] && downlink_str=10000000
                                echo "-A ${chain_if}_down $class_mark -m qoslimit --rate $downlink_str ${udp_str} --info ${class}_rule \
                                    --type share --prioconnmark $prio_connmark --priomark $prio_mark ${tag_str} -j DROP" >> $QOS_FILE
                                echo "-A ${chain_if}_down $class_mark -j RETURN" >> $QOS_FILE
                                
                            }  
   
                         done
                     fi
                     if [ $direction = "out" ] || [ $direction = "both" ];then
#### ack
                         [ ${ack} = "enable" ] && {
                             echo "-A ${chain_if}_up -p tcp --tcp-flags ALL ACK -m conntrack --ctstate ESTABLISHED -j MARK --or-mark ${prio_mark}" >> $QOS_FILE
                         }
##### udp & class
                         local class_list="class1 class2 class3 class4"
                         local udp_str=""
                         for class in $class_list;do
                             class_str=""
                         class_mark=""
                             local tag_str=""
                             if [ $class = "class1" ];then
                                 class_str=$class1
                         class_mark=" -m connmark --mark $class1_mark"
                                 tag_str=$tag1_str
                                 [ ${udp_status} == "enable" ] && {
                                    local udp_rate=$((${class_str}*${udp}*${downlink_tol}/10000))
                                    [ $(($udp_rate)) -lt 100 ] && udp_rate=100
                                    [ $(($udp_rate)) -gt 10000000 ] && udp_rate=10000000
                                    udp_str="--udp-rate $udp_rate"
                                }
                             elif [ $class = "class2" ];then
                                 class_str=$class2
                                 class_mark=" -m connmark --mark $class2_mark"
                                 tag_str=$tag2_str
                                 [ ${udp_status} == "enable" ] && {
                                    local udp_rate=$((${class_str}*${udp}*${downlink_tol}/10000))
                                    [ $(($udp_rate)) -lt 100 ] && udp_rate=100
                                    [ $(($udp_rate)) -gt 10000000 ] && udp_rate=10000000
                                    udp_str="--udp-rate $udp_rate"
                                }
                             elif [ $class = "class3" ];then
                                 class_str=$class3
                             class_mark=" -m connmark --mark $class3_mark"
                                 tag_str=$tag3_str
                                 [ ${udp_status} == "enable" ] && {
                                    local udp_rate=$((${class_str}*${udp}*${downlink_tol}/10000))
                                    [ $(($udp_rate)) -lt 100 ] && udp_rate=100
                                    [ $(($udp_rate)) -gt 10000000 ] && udp_rate=10000000
                                    udp_str="--udp-rate $udp_rate"
                                }
                             else 
                                 class_str=$class4
                             class_mark=" -m connmark --mark $class4_mark"
                                 tag_str=$tag4_str
                                 [ ${udp_status} == "enable" ] && {
                                    local udp_rate=$((${class_str}*${udp}*${downlink_tol}/10000))
                                    [ $(($udp_rate)) -lt 100 ] && udp_rate=100
                                    [ $(($udp_rate)) -gt 10000000 ] && udp_rate=10000000
                                    udp_str="--udp-rate $udp_rate"
                                }
                             fi
                             [ ${#class_str} -ne 0 ] && {
                                 local uplink_str=$((${class_str}*${uplink_tol}/100))
                                 [ $(($uplink_str)) -lt 100 ] && uplink_str=100
                                 [ $(($uplink_str)) -gt 10000000 ] && uplink_str=10000000
                                 echo "-A ${chain_if}_up ${class_mark} -m qoslimit --rate $uplink_str ${udp_str} --info ${class}_rule \
                                      --type share --prioconnmark $prio_connmark --priomark $prio_mark ${tag_str} -j DROP" >> $QOS_FILE
                                 echo "-A ${chain_if}_up $class_mark -j RETURN" >> $QOS_FILE
                             }
                         done
                     fi
                 fi
                 echo "COMMIT" >> $QOS_FILE
                 $IPT_RESTORE < $QOS_FILE
             done
         done
         [ "$is_init" == "0" ] && qos_nf_sfe_toggle
         count=$(($count+1))
         if [ "$(uci get qos_ctl.setting.qos_stat_busy_bw)" == "1" ] ;then
            uci set qos_ctl.setting.qos_stat_busy_bw="0"  
            uci set qos_ctl.setting.qos_stat_ready_bw="1" 
            uci commit qos_ctl
         fi
        for i in `seq ${WANMAX}` ;do
            [ "$(uci get qos_ctl.setting.update_bw${i})" == "1" ] && ifupdate=$((i)) && break
        done        
    done
    return 0
}

qos_nf_ipt_idx_check()
{
    local ipt_idx=${1}
    local qos_ip=${2}
    local ipt_cmd="iptables"
    [ "${qos_ip}" != "ipv4" ] && ipt_cmd="ip6tables"
    local ipt_num=`${ipt_cmd} -w -t mangle -nvL $QOS_RULE_CHAIN | wc -l`
    let ipt_num-=2
    let ipt_num/=2
    let ipt_num+=1
    let ipt_num*=2
    let ipt_num-=1
    let ipt_idx*=2
    let ipt_idx-=1
    #echo "ipt_num:${ipt_num}, ipt_idx:${ipt_idx}" > /dev/console
    if [ ${ipt_num} -lt ${ipt_idx} ];then
        echo ${ipt_num}
    else
        echo ${ipt_idx}
    fi
}

qos_nf_add_rule_directly()
{
    local param=$1
    local count=1
    local id=`uci -q get qos_ctl.rule$param.id`
    [ $id != $param ] && echo "id error.exit!" && return 0
    local status=`uci -q get qos_ctl.rule$param.status`
    [ $status = "disable" ] && return 0
    local class=`uci -q get qos_ctl.rule$param.class`
    local ip=`uci -q get qos_ctl.rule$param.ip`
    local src_ip=`uci -q get qos_ctl.rule$param.src_ip`
    local dst_ip=`uci -q get qos_ctl.rule$param.dst_ip`
    local dscp=`uci -q get qos_ctl.rule$param.dscp`
    local src_str="",dst_str="",dscp_str=""
    local ipt_idx=`uci -q get qos_ctl.rule$param.ipt_idx`
    
    if [ $src_ip = "any" ];then
        src_str=""
    else
        if [ $ip = "ipv4" ];then
            src_str="-m set --match-set $src_ip src "
        else
            src_str="-m set --match-set ipv6_$src_ip src "
        fi
    fi
    if [ $dst_ip = "any" ];then
        dst_str=""
    else
        if [ $ip = "ipv4" ];then
            dst_str="-m set --match-set $dst_ip dst "
        else
            dst_str="-m set --match-set ipv6_$dst_ip dst "
        fi
    fi
    if [ $dscp = "any" ];then
        dscp_str=""
    else
        dscp_str="-m dscp --dscp $dscp "
    fi
    local service=`uci -q get qos_ctl.rule$param.service`
    local service_str=""
    local service6_str=""
    [ $service != "ALL" ] && {
        service_str="-m set --match-set ${service}_src src -m set --match-set ${service}_dst dst"
        service6_str="-m set --match-set ${service}_src6 src -m set --match-set ${service}_dst6 dst"
    }
    local chain=$QOS_RULE_CHAIN
    local class_mark=0
    local class_ormark=0
    if [ $class = "class1" ];then
        class_mark=$class1_mark
        class_ormark=$class1_ormark
    elif [ $class = "class2" ];then
        class_mark=$class2_mark
        class_ormark=$class2_ormark
    elif [ $class = "class3" ];then
        class_mark=$class3_mark
        class_ormark=$class3_ormark
    else
        class_mark=$class4_mark
        class_ormark=$class4_ormark
    fi
    {
        flock -x 175
        #ipt_idx=`qos_nf_ipt_idx_check ${ipt_idx} ${ip}`
        #echo "####[`date`] ipt_idx:${ipt_idx}" > /dev/console
        if [ $ip = "ipv4" ];then
            qos_iptables_add "add" "4" "$chain -m comment --comment \"idx:`expr ${ipt_idx} \* 2 - 1`\" ${src_str} ${dst_str} ${dscp_str} ${service_str} -j CONNMARK --or-mark $class_ormark" "$chain -m comment --comment \"idx:`expr ${ipt_idx} \* 2`\" -m connmark --mark $class_mark -j RETURN" 
            #qos_iptables_add "add" "4" "$chain  -m connmark --mark $class_mark -j RETURN"
        else
            qos_iptables_add "add" "6" "$chain -m comment --comment \"idx:`expr ${ipt_idx} \* 2 - 1`\" ${src_str} ${dst_str} ${dscp_str} ${service6_str} -j CONNMARK --or-mark $class_ormark" "$chain -m comment --comment \"idx:`expr ${ipt_idx} \* 2`\" -m connmark --mark $class_mark -j RETURN"
            #qos_iptables_add "add" "6" "$chain  -m connmark --mark $class_mark -j RETURN"
        fi
        flock -u 175
    } 175<>/tmp/.qos_ctl/qos_ipt_lock
    sleep $SLEEP_TIME
}

qos_nf_add_rule_to_restore()
{
    local param=$1
    echo $1
    local id=`uci -q get qos_ctl.rule$param.id`
    [ $id != $param ] && echo "id error.exit!" && return 0
    local status=`uci -q get qos_ctl.rule$param.status`
    [ $status = "disable" ] && return 0
    local class=`uci -q get qos_ctl.rule$param.class`
    local ip=`uci -q get qos_ctl.rule$param.ip`
    local src_ip=`uci -q get qos_ctl.rule$param.src_ip`
    local dst_ip=`uci -q get qos_ctl.rule$param.dst_ip`
    local dscp=`uci -q get qos_ctl.rule$param.dscp`
    local ipt_idx=`uci -q get qos_ctl.rule$param.ipt_idx`
    local src_str="",dst_str="",dscp_str=""
 
    if [ $src_ip = "any" ];then
        src_str=""
    else
        if [ $ip = "ipv4" ];then
            src_str="-m set --match-set $src_ip src "
        else
            src_str="-m set --match-set ipv6_$src_ip src "
        fi
    fi
    if [ $dst_ip = "any" ];then
        dst_str=""
    else
        if [ $ip = "ipv4" ];then
            dst_str="-m set --match-set $dst_ip dst "
        else
            dst_str="-m set --match-set ipv6_$dst_ip dst "
        fi
    fi
    if [ $dscp = "any" ];then
        dscp_str=""
    else
        dscp_str="-m dscp --dscp $dscp "
    fi
    local service=`uci -q get qos_ctl.rule$param.service`
    local service_str=""
    local service6_str=""
    [ $service != "ALL" ] && {
        service_str="-m set --match-set ${service}_src src -m set --match-set ${service}_dst dst"
        service6_str="-m set --match-set ${service}_src6 src -m set --match-set ${service}_dst6 dst"
    }
    local chain=$QOS_RULE_CHAIN
    local class_mark=0
    local class_ormark=0
    if [ $class = "class1" ];then
        class_mark=$class1_mark
        class_ormark=$class1_ormark
    elif [ $class = "class2" ];then
        class_mark=$class2_mark
        class_ormark=$class2_ormark
    elif [ $class = "class3" ];then
        class_mark=$class3_mark
        class_ormark=$class3_ormark
    else
        class_mark=$class4_mark
        class_ormark=$class4_ormark
    fi
    if [ $ip = "ipv4" ];then
        echo "-A $chain -m comment --comment idx:`expr ${ipt_idx} \* 2 - 1` ${src_str} ${dst_str} ${dscp_str} ${service_str} -j CONNMARK --or-mark $class_ormark"  >> $QOS_FILE_IP4
        echo "-A $chain -m comment --comment idx:`expr ${ipt_idx} \* 2` -m connmark --mark $class_mark -j RETURN"  >> $QOS_FILE_IP4
    else
        echo "-A $chain -m comment --comment idx:`expr ${ipt_idx} \* 2 - 1` ${src_str} ${dst_str} ${dscp_str} ${service6_str} -j CONNMARK --or-mark $class_ormark" >> $QOS_FILE_IP6
        echo "-A $chain -m comment --comment idx:`expr ${ipt_idx} \* 2` -m connmark --mark $class_mark -j RETURN" >> $QOS_FILE_IP6
    fi
}

# delete the item defined by the nfmark from nf qos chain
    # nfmark is get from mark.data with arg rule_name
qos_nf_update_rule()
{
    
    local chain=$QOS_RULE_CHAIN
    local count=1
    local is_init="0"
    [ -n "$1" -a "$1" == "init" ] && is_init="1"
    while [ -e $QOS_STAT_UPDATE ] || [ $count -eq 1 ];do  
        [ -e $QOS_STAT_UPDATE ] && rm $QOS_STAT_UPDATE 
        if [ -e $QOS_STAT_READY ] || [ ! -e $QOS_STAT_BUSY ]; then
            rm $QOS_STAT_READY 2> /dev/null
            touch $QOS_STAT_BUSY
            $IPT4_PREFIX -F $chain
            $IPT6_PREFIX -F $chain
            rm $QOS_FILE_IP4 2>/dev/null
            rm $QOS_FILE_IP6 2>/dev/null
            echo "*mangle" > $QOS_FILE_IP4
            echo "*mangle" > $QOS_FILE_IP6
            local rule_list=$(qos_rulelist_get)
            for rule in $rule_list ;do
                qos_nf_add_rule_to_restore $rule
            done
            echo "COMMIT" >> $QOS_FILE_IP4
            echo "COMMIT" >> $QOS_FILE_IP6
            $IPT4_RESTORE < $QOS_FILE_IP4
            $IPT6_RESTORE < $QOS_FILE_IP6
            [ "$is_init" == "0" ] && qos_nf_sfe_toggle
            [ -e $QOS_STAT_BUSY ] && rm $QOS_STAT_BUSY && touch $QOS_STAT_READY
            count=$(($count+1))
        fi
    done
    return 0
}

qos_nf_voip_update()
{
    local file=
    local voip=`uci -q get qos_ctl.setting.voip`
    local voip_status=`uci -q get qos_ctl.setting.voip_status`
    local chain=$QOS_CHAIN
    local ln=`$IPT4_PREFIX -L ${chain} --line | grep -w "udp"| awk '{print $1}' NR==1`
    local start_ln=0
    local count=0
    for i in ${ln};do
        [ $count -eq 0 ] && start_ln=$i && count=$(($count+1))
        $IPT4_PREFIX -D $chain $start_ln
    done
    ln=`$IPT6_PREFIX -L ${chain} --line | grep -w "udp"| awk '{print $1}' NR==1`
    start_ln=0
    count=0
    for i in ${ln};do
        [ $count -eq 0 ] && start_ln=$i && count=$(($count+1))
        $IPT6_PREFIX -D $chain $start_ln
    done
    [ ${voip_status} = "enable" ] && [ ${#voip} -ne 0 ] && {
        $IPT4_PREFIX -I ${chain} -p udp --dport $voip -j MARK --or-mark $prio_connmark
        $IPT4_PREFIX -I ${chain} 2 -p udp -m mark --mark $prio_connmark/$prio_connmark -j CONNMARK --or-mark $prio_connmark
        $IPT6_PREFIX -I ${chain} -p udp --dport $voip -j MARK --or-mark $prio_connmark
        $IPT6_PREFIX -I ${chain} 2 -p udp -m mark --mark $prio_connmark/$prio_connmark -j CONNMARK --or-mark $prio_connmark
    }
    return 0
}

##################################################################################
# single $0 sh debug mode
if [ ! -d $QOS_TMP_DIR ];then
    mkdir $QOS_TMP_DIR
fi
if [ $1 = "init" ];then
    qos_nf_destroy 
    qos_nf_chain_create
    qos_nf_update_bandwidth
    qos_nf_update_rule "init"
    qos_nf_voip_update
elif [ $1 = "destroy" ];then
    echo "destroy !!!"
    qos_nf_destroy
    qos_nf_sfe_toggle 
elif [ $1 = "add_rule" ];then
    qos_nf_add_rule_directly $2
    qos_nf_sfe_toggle
elif [ $1 = "update_bandwidth" ]; then
{
    flock -x -n 171
    if [ $? -eq 1 ];then
        local WANMAX="$(qos_wanmax)"
        for i in `seq ${WANMAX}` ;do
            if [ "$2" == "WAN${i}" ];then
                [ "$(uci get qos_ctl.setting.update_bw${i})" == "1" ] || uci set qos_ctl.setting.update_bw${i}="1"
                uci commit qos_ctl
                echo "block!!!"
                exit 1
            fi    
        done
        
    else 
        qos_nf_update_bandwidth $2
    fi
    flock -u 171
} 171<>/tmp/.qos_ctl/qos_lock_bw
elif [ $1 = "update_rule" ]
then
{
     flock -x -n 170
   if [ $? -eq 1 ];then
       [ -e $QOS_STAT_UPDATE ] || touch $QOS_STAT_UPDATE
       exit 1 
   else
       qos_nf_update_rule
   fi
   flock -u 170
} 170<>/tmp/.qos_ctl/qos_lock
elif [ $1 = "update_voip" ];then
    qos_nf_voip_update
    qos_nf_sfe_toggle 
elif [ $1 = "toggle" ];then
    qos_nf_sfe_toggle
else
    echo "Invalid params!"
fi
