#!/bin/sh 
# Copyright(c) 2011-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file     service type core.sh
# brief    
# author   Wang Wenjing
# version  1.0.0
# date     1July22
# histry   arg 1.0.0, 1July22, Wang Wenjing, Create the file.

THIS_MODULE="service"
CONFIG_DIR="/etc/config/service"

service_rulelist_get()
{
    local list=`cat ${CONFIG_DIR}|grep "config service "|sed "s/'//g" |awk '{print $3}' NR==1`
    echo "$list"
}

service_add()
{
    local service=$1
    echo "add service:$service" > /dev/console
    ipset create ${service}_src hash:proto,port -exist
    ipset flush ${service}_src
    ipset create ${service}_dst hash:proto,port -exist
    ipset flush ${service}_dst

    ipset create ${service}_src6 hash:proto,port family inet6 -exist
    ipset flush ${service}_src6
    ipset create ${service}_dst6 hash:proto,port family inet6 -exist
    ipset flush ${service}_dst6

    local sport=`uci -q get service.${service}.sport`
    local dport=`uci -q get service.${service}.dport`
    local proto=`uci -q get service.${service}.proto`
    local proto_list=$proto
    [ $proto = "tcp-udp" ] && proto_list="tcp udp"
    for proto in $proto_list;do
        echo "proto:$proto"
        [ $proto != "tcp" -a $proto != "udp" -a $proto != "6" -a $proto != "17" ] && sport="0" && dport="0"
        if [ $proto = "icmp" ];then
            local type=`uci -q get service.${service}.type`
            local code=`uci -q get service.${service}.code`
            ipset add ${service}_src $proto:$type/$code -exist
            ipset add ${service}_src6 $proto:$type/$code -exist
        else
            ipset add ${service}_src $proto:$sport -exist
            ipset add ${service}_src6 $proto:$sport -exist
        fi
        ##### dst port
        if [ $proto = "icmp" ];then
            local type=`uci -q get service.${service}.type`
            local code=`uci -q get service.${service}.code`
            ipset add ${service}_dst $proto:$type/$code -exist
            ipset add ${service}_dst6 $proto:$type/$code -exist
        else
            ipset add ${service}_dst $proto:$dport -exist
            ipset add ${service}_dst6 $proto:$dport -exist
        fi
    done
    return 0
}

service_update()
{
    local old_service=$1
    local new_service=$2
    service_delete $old_service
    service_add $new_service
}

service_delete()
{
    local service=$1
    ipset destroy ${service}_src -exist 2>/dev/null
    ipset destroy ${service}_src6 -exist 2>/dev/null
    ##### dst port
    ipset destroy ${service}_dst  -exist 2>/dev/null
    ipset destroy ${service}_dst6  -exist 2>/dev/null
    return 0
}

service_destroy()
{
    local service_list=$(service_rulelist_get)
    for service in ${service_list};do
        echo "service:$service"
        service_delete $service
    done
    return 0
}

service_type_load_default_set()
{
    local service_list=$(service_rulelist_get)
    for service in ${service_list};do
        echo $service
        service_add $service
    done
    return 0
}

#####################################
if [ $1 = "add" ];then
    service_add $2
elif [ $1 = "del" ];then
    service_delete $2
elif [ $1 = "update" ];then
    service_update $2 $3
elif [ $1 = "flush" ];then
    service_destroy
else
    echo "Invalid param!!"
fi
