# Copyight(c) 2008-2015 Shenzhen TP-LINK Technologies Co.Ltd.
# file     zone_conf.sh
# bief     Config the zone info
# author   Yuan Fengjia
# vesion   1.0.0
# history  1.0.0, 13May15, Yuanfengjia, Ceate the file

. /usr/share/libubox/jshn.sh
. /lib/net_share.sh

GROUP_ZONES="LAN VPN "$(global_wan_name_list)
GROUP_VZONES="$(global_wan_name_list)"   #group of zones which includes the vpn interfaces of each WAN

# Notice: max length of ipset name is 32 chars.
_zone_gname() {
	short_names="${@//WAN/W}"
	short_names="${short_names//LAN/L}"
	short_names="${short_names//VPN/V}"
	echo -n "Z_${short_names// /}"
}

_zone_vgname() {
    short_vnames="${@//WAN/W}"
    short_vnames="${short_vnames//LAN/L}"
    short_vnames="${short_vnames//VPN/V}"
    echo -n "ZV_${short_vnames// /}"    #ZV means zone with VPN
}

zone_vgname() {
    _zone_vgname $@
}

_zone_create_group() {
    gname="$(_zone_gname $1)"
    if [ "$1" == "${1// /}" ]; then
        ipset create "$gname" hash:iface
    else
        ipset create "$gname" list:set
        [ $? -eq 0 ] && {
            for zone in ${1}
            do
                ipset add "$gname" "$(_zone_gname $zone)"
            done
        }
    fi
    echo -n "$gname"
}

_zone_create_vgroup() {
    vgname="$(_zone_vgname $1)"
    if [ "$1" == "${1// /}" ]; then
        ipset create "$vgname" hash:iface
        ipset flush "$vgname"
	devs=$(uci_get_state zone $1 effect_devs)
        for dev in $devs
        do	
            ipset add "$vgname" "$dev"
            ipset_add_for_poe_parent "$vgname" "$dev"
        done
    else
        ipset create "$vgname" list:set
        ipset flush "$vgname"
                [ $? -eq 0 ] && {
                        for zone in ${1}
                        do
                                ipset add "$vgname" "$(_zone_vgname $zone)"
                        done
                }
    fi
        echo -n "$vgname"
}

zone_create_group() {
    # create wan group
    _zone_create_group "" 1>/dev/null 2>&1

    for wname in ${GROUP_ZONES}
    do
        _zone_create_group "$wname" 1>/dev/null 2>&1
    done

    for wname in ${GROUP_VZONES}
    do
         _zone_create_vgroup "$wname" 1>/dev/null 2>&1
    done	
	# create all wan group
    _zone_create_group "$(global_wan_name_list)"
    _zone_create_vgroup "$(global_wan_name_list)"
}

#$1:VPN iface name $2:WAN
zone_add_vgroup() {
    [ ! -n "$1" -o ! -n "$2" ] && return
    local vgroup="$(_zone_vgname $2)"
    ipset add "$vgroup" "$1" 
}

zone_delete_vgroup() {
    local wan_list="${GROUP_VZONES}"
    [ -n "$2" ] && wan_list="$2"
    [ ! -n "$1" ] && return
    for wan in $wan_list ;do
        local vgroup="$(_zone_vgname $wan)"
        ipset del "$vgroup" "$1" --exist
    done
}

zone_rebuild_group() {
    echo $GROUP_ZONES | grep -vqE "(^| )${1}( |$)" && return
    
    #create zone group
    gname="$(_zone_gname $1)"

    ipset flush "$gname"

    devs=$(uci_get_state zone $1 effect_devs)

    for dev in $devs
    do
        ipset add "$gname" "$dev"
    done
}

#Used for Bug 730739 - ipv4 pppoe拨号，ipv6静态拨号，会导致相应WAN的QoS规则对ipv6流量不生效
#对于pppoe等拨号生成的poe网卡，将其原生网卡一并添加至ipset之中
ipset_add_for_poe_parent() {
    local vgname=$1
    local dev=$2
    local postfix=${dev#*_}
    if [ "$postfix"x = "poe"x ]; then
		local dev_wan_poe=${dev#*-}
        local poe_parent=$(uci get network.$dev_wan_poe.parent)
        local parent_ifname=$(uci get network.$poe_parent.ifname)
        if [ $parent_ifname ]; then
            ipset add "$vgname" "$parent_ifname"
        fi
	fi
}

zone_rebuild_vgroup() {
    echo $GROUP_VZONES | grep -vqE "(^| )${1}( |$)" && return
    #create zone group
    vgname="$(_zone_vgname $1)"

    local v6_devs=`ipset list $vgname | grep v6`
    ipset flush "$vgname"
    if [ -n "$v6_devs" ]; then
        for v6_dev in $v6_devs
        do
            ipset add "$vgname" "$v6_dev"
        done
    fi

    devs=$(uci_get_state zone $1 effect_devs)

    for dev in $devs
    do
        ipset add "$vgname" "$dev"
        ipset_add_for_poe_parent "$vgname" "$dev"
    done
}

zone_group() {
    local zones=""

    for gzone in $GROUP_ZONES
    do
        for zone in $@
        do
            if [ "$gzone" == "$zone" ]; then
				if [ -z "$zones" ]; then
                    zones=$gzone
                else
                    zones=$zones" "$gzone
                fi
            fi
        done
    done

    [ -n "$zones" ] && {
		_zone_create_group "$zones"
	}
}

zone_vgroup() {
    local zones=""

    for vgzone in $GROUP_VZONES
    do
        for zone in $@
        do
            if [ "$vgzone" == "$zone" ]; then
                if [ -z "$zones" ]; then
                    zones=$vgzone
                else
                    zones=$zones" "$vgzone
                fi
           fi
        done
    done

    [ -n "$zones" ] && {
        _zone_create_vgroup "$zones"
    }
}


zone_allwan_group() {
	zone_group "$(global_wan_name_list)"
}

_config_vpn_iface() {
    local vpn_iface=""
    local name="" 
    local count=""
    
    local iface=""
    local device="" 
    name=$(uci_get zone "$1")
    [ -n "$name"  ] && {
        for i in $(uci_get_state zone VPN ifaces); do
            local j=${i#}
            append iface $j
        done
        
        local tmp=$(uci_get zone "$1" iface)
        append iface $tmp
        #echo "ifaces=$iface"
        uci_toggle_state zone VPN ifaces "$iface"
        uci_toggle_state zone VPN effect_ifs "$iface"
        
        for m in $(uci_get_state zone VPN devices); do
            local n=${m#}                          
            append device $n                           
        done
        #echo "devices_old=$device"
        tmp=""
        tmp=$(uci_get zone "$1" dev)
        append device $tmp
        #echo "devices=$device"                                         
        uci_toggle_state zone VPN devices "$device"
        uci_toggle_state zone VPN effect_devs "$device"
        
        iface=""
        device=""
        iface=$(uci_get zone "$1" iface)
        device=$(uci_get zone "$1" dev)
        #echo "iface=$iface"
        uci_set_state zone $iface "" ${iface}_state
        uci_set_state zone $iface devname $device
        device=${device/-/_}
 
        uci_set_state zone $device "" ${device}_state
        uci_set_state zone $device ifname $iface
    }
}

_config_normal_iface() {
    local normal_iface=""
    local name="" 
    local count=""
    
    local iface=""
    local device="" 
    name=$(uci_get zone "$1")
    [ -n "$name"  ] && {
        for i in $(uci_get_state zone NORMAL ifaces); do
            local j=${i#}
            append iface $j
        done
        
        local tmp=$(uci_get zone "$1" iface)
        append iface $tmp
        #echo "ifaces=$iface"
        uci_toggle_state zone NORMAL ifaces "$iface"
        uci_toggle_state zone NORMAL effect_ifs "$iface"
        
        for m in $(uci_get_state zone NORMAL devices); do
            local n=${m#}                          
            append device $n                           
        done
        tmp=""
        tmp=$(uci_get zone "$1" dev)
        append device $tmp
        #echo "devices=$device"                                         
        uci_toggle_state zone NORMAL devices "$device"
        uci_toggle_state zone NORMAL effect_devs "$device"
        
        iface=""
        device=""
        iface=$(uci_get zone "$1" iface)
        device=$(uci_get zone "$1" dev)
        #echo "iface=$iface"
        uci_set_state zone $iface "" ${iface}_state
        uci_set_state zone $iface devname $device
        device=${device//./_}
        device=${device//-/_}
 
        uci_set_state zone $device "" ${device}_state
        uci_set_state zone $device ifname $iface
    }
}

_zone_state_config() {
    local zone_name=$1
    
    local ifname_eth
    local ifname_x
    local devname_eth
    local devname_x
    local zone_old
    local proto

    #echo "zone_name=$zone_name"
    [ -n "$zone_name" ] && {
        #echo "set zones"
        for zs in $(uci_get_state zone core zones); do
                local i=${zs#*}
                append zone_old $i
        done
                                                               

        append zone_old $zone_name  
 
        #echo "zone_old=$zone_old"
        uci_toggle_state zone core zones "$zone_old"
        
        #echo "set interfaces"
        ifname_eth=$(uci_get zone "$zone_name" if_eth)
        #echo "ifname_eth=$ifname_eth"
        uci_set_state zone $zone_name ifname_eth "$ifname_eth"
        
        ifname_x=$(uci_get zone "$zone_name" if_1)
        #echo "ifname_x=$ifname_x"
        [ -n "$ifname_x" ] && {
        	uci_set_state zone $zone_name ifname_1 "$ifname_x"
        }
        
        ifname_x=$(uci_get zone "$zone_name" if_2)
        #echo "ifname_x=$ifname_x"
        [ -n "$ifname_x" ] && {
        	uci_set_state zone $zone_name ifname_2 "$ifname_x"
        }
        
        #echo "set devices"
        devname_eth=$(uci_get zone "$zone_name" dev_eth)
        #echo "devname_eth=$devname_eth"
        uci_set_state zone $zone_name devname_eth "$devname_eth"
        
        devname_x=$(uci_get zone "$zone_name" dev_1)
        #echo "devname_x=$devname_x"
        [ -n "$devname_x" ] && {
        	uci_set_state zone $zone_name devname_1 "$devname_x"
        }
        
        devname_x=$(uci_get zone "$zone_name" dev_2)
        #echo "devname_x=$devname_x"
        [ -n "$devname_x" ] && {
        	uci_set_state zone $zone_name devname_2 "$devname_x"
        }
        #get the proto
        proto=$(uci_get network "$ifname_eth" proto)
        [ "none" == "$proto" ] && {
          ifname_eth=""
          devname_eth=""
        }
        
        #echo "set effect interfaces"
        ifname_x=$(uci_get zone "$zone_name" if_1)
        append ifname_eth $ifname_x
        #echo "effect_ifs=$ifname_eth"
        uci_set_state zone $zone_name effect_ifs "$ifname_eth"
        
        #echo "set effect devices"
        devname_x=$(uci_get zone "$zone_name" dev_1)
        append devname_eth $devname_x
        #echo "effect_devs=$devname_eth"
        uci_set_state zone $zone_name effect_devs "$devname_eth"
        
        #echo "set interfaces devices map"
        ifname_eth=$(uci_get zone "$zone_name" if_eth)
        devname_eth=$(uci_get zone "$zone_name" dev_eth)
        #echo "ifname_eth=$ifname_eth,devname_eth=$devname_eth"
        uci_set_state zone $ifname_eth "" ${ifname_eth}_state
        uci_set_state zone $ifname_eth devname "$devname_eth"
        
        #echo "devname_eth=$devname_eth"
        devname_eth=${devname_eth//./_}
        devname_eth=${devname_eth//-/_}
        #echo "devname_eth=$devname_eth" 
        uci_set_state zone $devname_eth "" ${devname_eth}_state
        uci_set_state zone $devname_eth ifname "$ifname_eth" 
        
        ifname_x=""
        devname_x=""
        ifname_x=$(uci_get zone "$zone_name" if_1)
        devname_x=$(uci_get zone "$zone_name" dev_1)
        #echo "ifname_x=$ifname_x,devname_x=$devname_x"
        [ -n "$ifname_x" ] && [ -n "$devname_x" ] && {
            uci_set_state zone $ifname_x "" ${ifname_x}_state
            uci_set_state zone $ifname_x devname "$devname_x"
            
            devname_x=${devname_x//./_}
            devname_x=${devname_x//-/_}
            uci_set_state zone $devname_x "" ${devname_x}_state
            uci_set_state zone $devname_x ifname "$ifname_x" 
        }
        
        ifname_x=""
        devname_x=""
        ifname_x=$(uci_get zone "$zone_name" if_2)
        devname_x=$(uci_get zone "$zone_name" dev_2)
        #echo "ifname_x=$ifname_x,devname_x=$devname_x"
        [ -n "$ifname_x" ] && [ -n "$devname_x" ] && {
            uci_set_state zone $ifname_x "" ${ifname_x}_state
            uci_set_state zone $ifname_x devname "$devname_x"
            
            devname_x=${devname_x//./_}
            devname_x=${devname_x//-/_}                                                                                      
            uci_set_state zone $devname_x "" ${devname_x}_state                                                             
            uci_set_state zone $devname_x ifname "$ifname_x" 
        }   
        
        #VPN
        [ "$zone_name" == "VPN" ] && {
            #config_foreach _config_vpn_iface
            local vpn_iface
            local count
            
            config_get vpn_iface "$zone_name" iface
            #echo "vpn_iface=$vpn_iface"
            let count=0
            for i in $vpn_iface
            do 
                let count=$count+1
                #echo "i=$i"
                #config_foreach _config_vpn_iface iface $i $count
                _config_vpn_iface $i
            done       
        } 
		
		#NORMAL:for multi interface product such as TL-ER7520G
		[ "$zone_name" == "NORMAL" ] && {
            local normal_iface
            local count
            
            config_get normal_iface "$zone_name" iface
            #echo "normal_iface=$normal_iface"
            let count=0
            for i in $normal_iface
            do 
                let count=$count+1
                #echo "i=$i"
                _config_normal_iface $i
            done       
        }
	zone_rebuild_group "$zone_name"
#        zone_rebuild_vgroup "$zone_name"
    }

}

zone_config() {
    config_load zone
    config_foreach _zone_state_config zone
    
    local zones=$(uci_get_state zone core zones)
    #echo "zones=$zones"
    uci_toggle_state zone core zones "$zones"
}

