# Copyright(c) 2011-2013 Shenzhen TP-LINK Technologies Co.Ltd.
#
# /usr/lib/ddns/dynamic_dns_customddns.sh
# Written by Guo <suyuhang@tp-link.com.cn>, 06/29 2012
#
# This script is used to parse dyndns server return code and convert
# to TP New UI status code

. /lib/zone/zone_api.sh
. /usr/lib/ddns/dynamic_dns_log.sh

CUSTOMDDNS_RETCODE_GOOD=8001
CUSTOMDDNS_RETCODE_ERR=8002
CUSTOMDDNS_RETCODE_UNKNOWN=8003

CUSTOMDDNS_RETCODE_NOCHG=9000
CUSTOMDDNS_RETCODE_NOHOST=9001      # The hostname specified does not exist
CUSTOMDDNS_RETCODE_NOTFQDN=9002     # The hostname specified is not a fully-qualified domain name
CUSTOMDDNS_RETCODE_NOT_YOURS=9003   # The hostname specified exists, but not under the username specified
CUSTOMDDNS_RETCODE_ABUSE=9004       # The hostname specified is blocked for update abuse
CUSTOMDDNS_RETCODE_NUMHOST=9005     # Too many or too few hosts found

CUSTOMDDNS_RETCODE_BADAUTH=9101     # The username or password specified are incorrect

# An option available only to credited users was specified, but the user is
# not a credited user. If mulitple hosts were specified, only a single !donator
# will be returned
CUSTOMDDNS_RETCODE_NOT_DONATOR=9102
# The sycodestem parameter given is not valid. Valid system parameters:dyndns,statdns and custom
CUSTOMDDNS_RETCODE_BADSYS=9201      
# The user agent that was sent has been blocked for not flollowing these specifications or no user agent was specified
CUSTOMDDNS_RETCODE_BADAGENT=9202

CUSTOMDDNS_RETCODE_DNSERR=9301      # DNS error encountered
CUSTOMDDNS_RETCODE_911=9302         # There is a serious problem on our side, such as a database or DNS server failure 
CUSTOMDDNS_RETCODE_MAX_SIZE=512

# CUSTOMDDNS state
CUSTOMDDNS_STATE_RUN_OK=0     # ok
CUSTOMDDNS_STATE_STOP=1       # stop
CUSTOMDDNS_STATE_CONNECTING=2 # connecting
CUSTOMDDNS_STATE_BADAUTH=3    # badauth
CUSTOMDDNS_STATE_BADHOST=4    # badhost     

CUSTOMDDNS_HTTP_ERR_PATH=/tmp/customddns.error
CUSTOMDDNS_NSLOOKUP_TMO=10

customddns_retrieve_prog()
{
    # MIPS:"ez-update-3.0.10 mips-mips-linux-gnu [daemon] (by Angus Mackay)"
    # ARM:?
    local user_agent=
    
    if [ -z "$user_agent" ]; then
        customddns_prog="/usr/bin/curl"
    else
        customddns_prog="/usr/bin/curl -U ${user_agent}"
    fi
    
    echo "$customddns_prog"
}

customddns_state_init()
{
    uci_set_state ddns "customddns_${1}" "" ddns_state
    uci_set_state ddns "customddns_${1}" state $customddns_STATE_CONNECTING
}

customddns_update()
{
    local prog="$1"
    local url="$2"
    local iface="$3"
    local devname=$(echo $iface | awk '{print toupper($0)}')
    local ifname=$(zone_get_effect_main_wan_devices $devname)

    local host_ip=$(dnslookup members.customddns.org 127.0.0.1 | head -n 1)
    if [ -n "${host_ip}" ]
    then
        url=$(echo "$url" | sed s/members.customddns.org/$host_ip/g)
    fi

    #echo if_name=$ifname > /dev/console
    customddns_result=1
    
    if [ -n "$prog" -a -n "$url" ]
    then
        request=$( ${prog} -k --interface ${ifname} "${url}" 2>"/tmp/${iface}_customddns.error" )
        if [ $? -gt 0 ]; then
            customddns_result=1
        else
            customddns_result=0        
        fi
    fi
    
    echo $customddns_result
}

customddns_parse_retcode()
{
    local service_id="$1"
    local iface_name="$2"
    local http_res=$(cat "/tmp/${iface_name}_customddns.error"|grep -o "curl")

    customddns_code=''
    if [ -n "$http_res" -a "$http_res" == "curl" ]; then
        # Only check 4xx http error code
        local http_code=$(cat "/tmp/${iface_name}_customddns.error" | grep -m 1 "curl")
        customddns_code="Offline"
        ddns_syslog $CUSTOM_DDNS_RERCODE "customddns" "$iface_name" "$http_code" "$3"
    else
        local result=$(cat /var/run/dynamic_dns/${service_id}_${iface_name}.retcode |tr "\n\r" " ")
        if [ ${#result} -gt ${CUSTOMDDNS_RETCODE_MAX_SIZE} ]; then
            result="Error! The retcode is too large, please check your configuration!"
        fi
        customddns_code=$result
        ddns_syslog $CUSTOM_DDNS_RERCODE "customddns" "$iface_name" "$result" "$3"
    fi

    # After parsing, then remove the temp http response code file
    rm -f "/tmp/${iface_name}_customddns.error"

    echo "$customddns_code"
}

customddns_set_state()
{
    local code="$1"

    [ -z "$code" -o '' -eq "$code" ] && {
        uci_toggle_state ddns "customddns_${2}" state $CUSTOMDDNS_STATE_STOP
        ddns_syslog $CUSTOM_DDNS_LOG_NTC_LOGIN_FAIL "customddns" "$2" "$3"
        return    
    }
    uci_toggle_state ddns "customddns_${2}" state "$code"

    
    return    
}

customddns_check()
{
    return 1          
}

customddns_retry_interval()
{
    customddns_retry_int=
    local s=$(uci_get_state ddns "customddns_${1}" state)

    local chk_int chk_unit chk_int_sec
    config_get chk_int "customddns_${1}" check_interval
    config_get chk_unit "customddns_${1}" check_unit

    [ "$chk_int" != "never" ] && {
        case "$check_unit" in
	        "days" )
		        chk_int_sec=$(($chk_int*60*60*24))
		    ;;
	        "hours" )
		        chk_int_sec=$(($chk_int*60*60))
		    ;;
	        "minutes" )
		        chk_int_sec=$(($chk_int*60))
		    ;;
	        "seconds" )
		        chk_int_sec=$chk_int
		    ;;
	        * )
		        #default is seconds
		        chk_int_sec=$chk_int
		    ;;
        esac
    }
        
    if [ "$s" -eq $CUSTOMDDNS_STATE_BADHOST ]; then
        customddns_retry_int=${chk_int_sec:-"never"}
    elif [ "$s" -eq $CUSTOMDDNS_STATE_BADAUTH ]; then
        customddns_retry_int=${chk_int_sec:-"never"}
    elif [ "$s" -eq $CUSTOMDDNS_STATE_STOP ]; then
        customddns_retry_int=$((60*1)) # One minutes
    else
        customddns_retry_int=$((60*1)) # Default, One minutes
    fi

    echo "$customddns_retry_int"
}

