. /lib/functions.sh

#VARS
NETE="dnete"
DPI_MODULE="dpi_core"
DPI_APP_DATABASE="/tmp/dpi_app_database.csv"
DPI_TAG_DATABASE="/tmp/dpi_tag_database.csv"
DPI_DATABASE_FLASH="/tmp/userconfig/dpi.db"
DPI_DATABASE_RAM="/tmp/dpi.db"
DPI_DB_APP_VOLUME_TABLE="dpi_log_app_volume"
DPI_DB_APP_BLOCK_TABLE="dpi_log_app_block"
FILD_DPI_RUNNING="/tmp/dpi_running"
FILE_PID_PATH="/tmp/dpi_engine_pid"
FILE_FLOWLOG_PATH="/tmp/dpi_flow_log_a"
FILE_BLOCKLOG_PATH="/tmp/dpi_app_log"
DPI_RESTRICTION_TABLE="filter"
DPI_HOOK_TABLE="mangle"

CONTROLLER_ORI_CSID="c21f969b5f03d33d43e04f8f136e7682"

# qos mark
local class1_mark="0x2000/0x6000"
local class2_mark="0x4000/0x6000"
local class3_mark="0x6000/0x6000"
local class4_mark="0x0/0x0"
local class1_ormark="0x2000"
local class2_ormark="0x4000"
local class3_ormark="0x6000"
local class4_ormark="0x0"

# table|dpi_app|dpi_app|2|CREATE TABLE dpi_app("appid" TEXT,"appname" TEXT,"tagid" TEXT,"tag" TEXT,"description" TEXT)
# table|dpi_tag|dpi_tag|352|CREATE TABLE dpi_tag("tagid" TEXT,"tag" TEXT)
# table|dpi_log_app_volume|dpi_log_app_volume|355|CREATE TABLE dpi_log_app_volume (id text,name text,volume text)
# table|dpi_log_app_block|dpi_log_app_block|356|CREATE TABLE dpi_log_app_block (id text,name text,count text)

#API

#$1: timeobj 
#$2: ruleid
add_timeobj()
{
    if [ -n "$1" -a -n "$2" ]; then
		. /lib/time_mngt/timeobj_api.sh

		timeobj_api_add "$1" "dpi" $2
		timeobj_api_commit "dpi"
	fi
}

del_timeobj()
{
	if [ -n "$1" -a -n "$2" ]; then
		. /lib/time_mngt/timeobj_api.sh

		timeobj_api_delete "$1" "dpi" $2
		timeobj_api_commit "dpi"
	fi
}

#dpi_set_rule rule_id
dpi_set_rule()
{
    local rule_id=$1
    [ -n "$rule_id" ] && {
        local schedule=`uci get dpi.rule$rule_id.schedule`
        if [ -n "$schedule" -a "$schedule" == "Any" ];then
            dpi_control -s $rule_id
        elif [ -n "$schedule" -a "$schedule" != "Any" ];then
            add_timeobj $schedule $rule_id
        fi
    }
}

#dpi_del_rule rule_id
dpi_del_rule()
{
    local rule_id=$1
    [ -n "$rule_id" ] && {
        dpi_control -d $rule_id

        local schedule=`uci get dpi.rule$rule_id.schedule`
        [ -n "$schedule" -a "$schedule" != "Any" ] && {
            del_timeobj $schedule $rule_id
        }
    }
}

#dpi_set_restriction restriction_id
dpi_set_restriction()
{
    local restriction_id=$1

    [ -n "$restriction_id" ] && {
        local network=`uci get dpi.restriction$restriction_id.network`
        [ -z "$network" ] && return
        local ifname=`uci get vnetwork.$network.ifname`
        local ipgroup="@"`uci get vnetwork.$network.zone`
        [ -z "$ifname" -o -z "$ipgroup" ] && return
        $NETE iptables -w -t $DPI_RESTRICTION_TABLE -N DPI_RESTRICTION_$restriction_id
        $NETE iptables -w -t $DPI_RESTRICTION_TABLE -A DPI_RESTRICTION -m set --match-set $ipgroup src -j DPI_RESTRICTION_$restriction_id -m comment --comment "DPI_RESTRICTION_$restriction_id"

        local filter_id=`uci get dpi.restriction$restriction_id.filter_id`
        [ -n "$filter_id" ] && {
            local rule_id_list=`uci get dpi.filter$filter_id.rule_id`
            [ -n "$rule_id_list" ] && {
                rule_id_list="$rule_id_list "
                for id in $rule_id_list
                do
                    local action=`uci get dpi.rule$id.action`
                    local schedule=`uci get dpi.rule$id.schedule`

                    #[ "$schedule" == "any" -o "$schedule" == "0" ] && {
                        [ "$action" == "1" ] && {
                            $NETE iptables -w -t $DPI_RESTRICTION_TABLE -A DPI_RESTRICTION_$restriction_id -m dpi-filter --filter-id ${filter_id} --rule-id $id -j DROP
                        }
                        [ "$action" == "2" ] && {
                            local qos_class=`uci get dpi.rule$id.qos_class`
                            local class_ormark=""
                            local class_mark=""
                            if [ "$qos_class" == "class1" -o "$qos_class" == "1" ];then
                                class_mark=$class1_mark
                                class_ormark=$class1_ormark;
                            elif [ "$qos_class" == "class2" -o "$qos_class" == "2" ];then
                                class_mark=$class2_mark
                                class_ormark=$class2_ormark;
                            elif [ "$qos_class" == "class3" -o "$qos_class" == "3" ];then
                                class_mark=$class3_mark
                                class_ormark=$class3_ormark;
                            elif [ "$qos_class" == "other" -o "$qos_class" == "0" -o "$qos_class" == "4" ];then
                                class_mark=$class4_mark
                                class_ormark=$class4_ormark;
                            else
                                echo "qos params error." > /dev/console
                            fi

                            [ -n "$class_ormark" -a -n "$class_mark" ] && {
                                $NETE iptables -w -t $DPI_RESTRICTION_TABLE -A DPI_RESTRICTION_$restriction_id -m dpi-filter --filter-id ${filter_id} --rule-id $id -j CONNMARK --or-mark $class_ormark
                                $NETE iptables -w -t $DPI_RESTRICTION_TABLE -A DPI_RESTRICTION_$restriction_id -m connmark --mark $class_mark -j RETURN
                            }
                        }
                    #}

                done
            }
        }
    }
}

#dpi_del_restriction restriction_id
dpi_del_restriction()
{
    local restriction_id=$1
    [ -n "$restriction_id" ] && {
        $NETE iptables -w -t $DPI_RESTRICTION_TABLE -F DPI_RESTRICTION_$restriction_id
        local line_num=`$NETE iptables -w -t $DPI_RESTRICTION_TABLE -vnL DPI_RESTRICTION --line-num | grep DPI_RESTRICTION_$restriction_id`
        line_num=`echo $line_num | awk '{print $1}'`
        [ -n "$line_num" ] && {
            $NETE iptables -w -t $DPI_RESTRICTION_TABLE -D DPI_RESTRICTION $line_num
            $NETE iptables -w -t $DPI_RESTRICTION_TABLE -X DPI_RESTRICTION_$restriction_id
        }
    }
}

#dpi_reload_restriction restriction_id
dpi_reload_restriction()
{
    local restriction_id=$1
    [ -n "$restriction_id" ] && {
        dpi_del_restriction $restriction_id
        dpi_set_restriction $restriction_id
    }
}

update_app_database()
{
    [ -f "$DPI_DATABASE_RAM" ] && return
    dpi_database
    [ -f "$DPI_APP_DATABASE" ] && [ -f "$DPI_TAG_DATABASE" ] && {
        echo -e ".separator "\;"\n.import $DPI_APP_DATABASE dpi_app" | sqlite3 $DPI_DATABASE_RAM
        echo -e ".separator "\;"\n.import $DPI_TAG_DATABASE dpi_tag" | sqlite3 $DPI_DATABASE_RAM
        echo -e "create table $DPI_DB_APP_VOLUME_TABLE (id text,volume text);" | sqlite3 $DPI_DATABASE_RAM
        echo -e "create table $DPI_DB_APP_BLOCK_TABLE (id text,count text);" | sqlite3 $DPI_DATABASE_RAM
    }

    cp $DPI_DATABASE_RAM $DPI_DATABASE_FLASH
}

dpi_set_rule_foreach()
{
    local rule="$1"
    config_get id "$rule" id
    dpi_set_rule $id    
}

dpi_del_rule_foreach()
{
    local rule="$1"
    config_get id "$rule" id
    dpi_del_rule $id
}

dpi_ruleset_init()
{
    config_load dpi
    config_foreach dpi_set_rule_foreach rule
}

dpi_ruleset_exit()
{
    config_load dpi
    config_foreach dpi_del_rule_foreach rule
}

dpi_set_restriction_foreach()
{
    local restrcition="$1"
    config_get id "$restrcition" id
    config_get network "$restrcition" network
    config_get filter_id "$restrcition" filter_id

    local ifname=`uci get vnetwork.$network.ifname`
    local ip_group=`uci get vnetwork.$network.zone`

    [ -n "$ifname" -a -n "$ip_group" ] && {
        dpi_set_restriction $id
    }
}


dpi_del_restriction_foreach()
{
    local restrcition="$1"
    config_get id "$restrcition" id

    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -F DPI_RESTRICTION_$id
    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -X DPI_RESTRICTION_$id

}

dpi_iptables_init()
{
    local flag=`$NETE iptables -w -t $DPI_RESTRICTION_TABLE -vnL FORWARD | grep DPI_RESTRICTION`
    [ -n "$flag" ] && {
        $NETE iptables -w -t $DPI_RESTRICTION_TABLE -F DPI_RESTRICTION
        $NETE iptables -w -t $DPI_RESTRICTION_TABLE -D FORWARD -j DPI_RESTRICTION
        $NETE iptables -w -t $DPI_RESTRICTION_TABLE -X DPI_RESTRICTION
        config_load dpi
        config_foreach dpi_del_restriction_foreach restriction
    }
    
    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -N DPI_RESTRICTION
    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -I FORWARD -j DPI_RESTRICTION
    $NETE iptables -w -t $DPI_HOOK_TABLE -A FORWARD -j dpi-hook

    if ! $NETE iptables -w -t $DPI_RESTRICTION_TABLE -C hwnat_mark -j CONNEXTMARK --set-mark 0x4/0x4 2>/dev/null; then
		iptables -w -t $DPI_RESTRICTION_TABLE -N hwnat_mark
		iptables -w -t $DPI_RESTRICTION_TABLE -A hwnat_mark -j EXTMARK --or-mark 0x4
		iptables -w -t $DPI_RESTRICTION_TABLE -A hwnat_mark -m connextmark --mark 0x4/0x4 -j RETURN
		iptables -w -t $DPI_RESTRICTION_TABLE -A hwnat_mark -j CONNEXTMARK --set-mark 0x4/0x4 
		iptables -w -t $DPI_RESTRICTION_TABLE -I DPI_RESTRICTION 1 -j hwnat_mark
	fi
    config_load dpi
    config_foreach dpi_set_restriction_foreach restriction
}

dpi_iptables_exit()
{
    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -F hwnat_mark
    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -D DPI_RESTRICTION -j hwnat_mark
    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -X hwnat_mark

    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -F DPI_RESTRICTION
    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -D FORWARD -j DPI_RESTRICTION
    $NETE iptables -w -t $DPI_RESTRICTION_TABLE -X DPI_RESTRICTION

    config_load dpi
    config_foreach dpi_del_restriction_foreach restriction

    $NETE iptables -w -t $DPI_HOOK_TABLE -D FORWARD -j dpi-hook
}

dpi_engine_init()
{
    local num_cpu=`uci get dpi.dpi.nb_cpu`
    local log_switch=`uci get dpi.dpi.log_switch`
    local work_mode=`uci get dpi.dpi.mode`
    local params_log=""
    local params_controller=""
    local params_mode=""

    [ -f "$FILE_PID_PATH" ] && {
        local pid=`cat $FILE_PID_PATH`
        [ -n "$pid" ] && {
            kill -SIGINT $pid
        }
    }
    local pid_t=`ps | grep dpi_engine | grep -v grep | awk '{print $1}'`
    [ -n "$pid_t" ] && {
        kill -SIGINT $pid_t
    }

    [ -n "$log_switch" ] && {
        [ "$log_switch" == "on" -o "$log_switch" == "1" ] && {
            params_log="-d"
        }
    }

    [ -n "$work_mode" ] && {
        [ "$work_mode" == "1" ] && {
            params_mode="-m 1"
        }
    }

    #判断是否处于controller收养状态
    local tmp=`cat /etc/config/controller.conf  | grep csId`
    tmp=`echo $tmp | sed s/[[:space:]]//g`
    tmp=`echo $tmp | sed s/\"csId\":\"//g`
    tmp=`echo $tmp | sed s/\",//g`
    [ -n $tmp ] && {
        [ ! "$tmp" == $CONTROLLER_ORI_CSID ] && params_controller="-o"
    }

    $NETE dpi_engine -c $num_cpu $params_log $params_controller $params_mode 1>/dev/console&
    echo "$NETE dpi_engine -c $num_cpu $params_log $params_controller $params_mode 1>/dev/console&"
}

dpi_engine_exit()
{
    [ -f "$FILE_PID_PATH" ] && {
        local pid=`cat $FILE_PID_PATH`
        [ -n "$pid" ] && {
            kill -SIGINT $pid
            return
        }
    }

    local pid_t=`ps | grep dpi_engine | grep -v grep | awk '{print $1}'`
    [ -n "$pid_t" ] && {
        kill -SIGINT $pid_t
    }
}

dpi_prepare_controller_inform()
{
    rm -f /tmp/dpi_flow_log_a
    rm -f /tmp/dpi_app_log
    
    [ ! -f "$FILE_PID_PATH" ] && echo 0 && return
    pid=`cat $FILE_PID_PATH`

    [ ! -n "$pid" ] && echo 0 && return

    cat /proc/dpi_ip_app_stat > /tmp/dpi_flow_log_a
    cat /proc/dpi_app_block > /tmp/dpi_app_log

    if [ -f "/tmp/dpi_flow_log_a" -a -f "/tmp/dpi_app_log" ];then
        echo 1
    else
        echo 0
    fi
}