
local sys  = require "luci.sys"
local json = require 'luci.json'
local dbg = require ('luci.tools.debug')
local sqlite3 = require 'luasql.sqlite3'

local err = require "luci.tools.error"
local dt_m = require "luci.tools.datatypes"

local DPI_DB_PATH = "/tmp/dpi.db"
local DPI_DB_APP_VOLUME_TABLE = "dpi_log_app_volume"
local DPI_DB_APP_BLOCK_TABLE = "dpi_log_app_block"

local DPI_DB_APP_LOG_BYTES_FILE = "/proc/dpi_app_stat"
local DPI_DB_APP_LOG_BLOCKS_FILE = "/proc/dpi_app_block"

local dbenv  = sqlite3.sqlite3()
local dbcon = dbenv:connect(DPI_DB_PATH)

function update_table(table_name, key, value, update_key, update_value)
    if not table_name or not key or not value or not update_key or not update_value then
        return
    end

    if not (dt_m.safestring(table_name)
            and dt_m.safestring(key)
            and dt_m.safestring(value)
            and dt_m.safestring(update_key)
            and dt_m.safestring(update_value)) then
        return false,err.ERR_COM_INVALID_PARAM
    end

    value = tostring(value)
    local sql = string.format([[select * from %s where %s='%s']], table_name, key, value)
    cur = assert(dbcon:execute(sql))
    row = cur:fetch ({}, "a")

    if tonumber(row) ~= 0 and row ~= nil then
		dbcon:execute(string.format([[update %s set %s=%s+%s where %s='%s']], table_name, update_key, update_key, update_value, key, value))
	else
        dbcon:execute(string.format([[insert into %s(id,%s) values('%s','%s')]], table_name, update_key, value, update_value))
    end

	cur:close()
    
end

local cmd = arg[1]
if cmd == "volume" then
    sys.exec("cat /proc/dpi_app_stat > /tmp/dpi_tmp_stat")
    local file = io.open("/tmp/dpi_tmp_stat", "r")
    for line in file:lines() do
        --dbg.printf(line)
        local log = json.decode(line)
        --dbg.printf(log, log.app)
        if log then
            update_table(DPI_DB_APP_VOLUME_TABLE, "id", tostring(log.id), "volume", tostring(log.bytes))
        end
    end
    file:close()

elseif cmd == "block" then
    sys.exec("cat /proc/dpi_app_block > /tmp/dpi_tmp_block")
    local file = io.open("/tmp/dpi_tmp_block", "r")
    for line in file:lines() do
        --dbg.printf(line)
        local log = json.decode(line)
        --dbg.printf(log, log.app)
        if log then
            update_table(DPI_DB_APP_BLOCK_TABLE, "id", tostring(log.id), "count", tostring(log.count))
        end
    end
    file:close()
elseif cmd == "clear" then
    dbcon:execute(string.format([[delete from %s]], DPI_DB_APP_VOLUME_TABLE))
    dbcon:execute(string.format([[delete from %s]], DPI_DB_APP_BLOCK_TABLE))
else
    dbg.printf("dpi_log_database.lua, params error.\n")
end

dbcon:close()
dbenv:close()