#!/bin/sh

local uVid uPid uMa uPr uSe
local idV idP

export MODEMLIB=/usr/lib/modem
export MODEMTMP=/tmp/modem
export BUSFILE=/proc/bus/usb/devices
export MODEMCFG=/etc/config/modem
export NETWORKCFG=/etc/config/network

#. ${MODEMLIB}/modem_scan.sh
. ${MODEMLIB}/usbmodem_log.sh
. /lib/functions.sh

local modeswitch="/usr/bin/modeswitch"

log() {
	echo "==>handle card" "$@" >> /dev/console
}

sanitize() {
	# 将每行结尾的空字符删除，将其他空字符替换成下划线
	sed -e 's/[[:space:]]\+$//; s/[[:space:]]\+/_/g' "$@"
}

find_usb_attrs() {
	local usb_dir="/sys$DEVPATH"
	echo "[1] usb_dir: $usb_dir" >> /dev/console
	[ -f "$usb_dir/idVendor" ] || usb_dir="${usb_dir%/*}"
	echo "[2] usb_dir: $usb_dir" >> /dev/console

	uVid=$(cat "$usb_dir/idVendor")
	uPid=$(cat "$usb_dir/idProduct")
	uMa=$(sanitize "$usb_dir/manufacturer")
	uPr=$(sanitize "$usb_dir/product")
	uSe=$(sanitize "$usb_dir/serial")
}

display_top() {
	log "*****************************************************************"
	log "*"
}

display_bottom() {
	log "*****************************************************************"
}


display() {
	local line1=$1
	log "* $line1"
	log "*"
}

# USB_LTE_LED needs to twinkle when USB Modem is plugged in
# USB_STORAGE_LED just needs to light on/off
usb_lte_led_twinkle() {
	echo "timer" > /sys/class/leds/USB_LTE_LED/trigger
	echo 200 > /sys/class/leds/USB_LTE_LED/delay_on
	echo 200 > /sys/class/leds/USB_LTE_LED/delay_off
}

usb_lte_led_stop_twinkle() {
	echo "none" > /sys/class/leds/USB_LTE_LED/trigger
	echo 1 > /sys/class/leds/USB_LTE_LED/brightness
}

# $1: 0-off, 1-on
usb_lte_led_set() {
	echo $1 > /sys/class/leds/USB_LTE_LED/brightness
}

# $1: 0-off, 1-on
usb_storage_led_set() {
	echo $1 > /sys/class/leds/USB_STORAGE_LED/brightness
}

# $1: section name
# $2: on/off
# $3: name (USB Modem / USB Storage / ...)
usb_pwr_set() {
	local target=$3
	local name=$(uci get system.$1.name)
	if [ -z "$target" -o "$name" == "$target" ]; then
		local value=$(uci get system.$1.value$2)
		local gpio=$(uci get system.$1.gpio)
		echo $value > /sys/class/gpio/gpio$gpio/value
	fi
}

# $1: on/off
# $2: name (USB Modem / USB Storage / ...)
usb_pwr_ctrl() {
	config_load system
	config_foreach usb_pwr_set usb $1 $2
}

#
# delay until httpd done
#
bootdelay() {
	# dnsmasq starts after uhttpd
	# check dnsmasq to make sure uhttpd done
	if [ ! -f /tmp/run/dnsmasq.pid ]; then
		log "Delay for boot up"
		# sleep 10
		while [ ! -f /tmp/run/dnsmasq.pid ]; do
			sleep 1
		done
		sleep 10
	fi
}

wait_proc_mount() {
	local count=0
	while [ ! -f $BUSFILE ]; do
		count=$(($count+1))
		if [ $count -gt 5 ];then
			log "before mode-switch,no procfs!!!!!!"
			syslog $LOG_MODESWITCH_S "before mode-switch,no procfs.times_sum=5"
			break
		fi
		syslog $LOG_MODESWITCH_S "before mode-switch,check procfs times=$count."
		log "before mode-switch,check procfs times=$count."
		sleep 2
	done
	syslog $LOG_MODESWITCH_S "end check procfs times_sum=$count."
}

check_success_switch()
{
	usb_dir="/sys$DEVPATH"
	idV="$(sanitize "$usb_dir/idVendor")"
	idP="$(sanitize "$usb_dir/idProduct")"
	
	cat $BUSFILE > ${MODEMTMP}/checkswitchmode
	${MODEMLIB}/check_switchmode.lua $idV $idP
	local retval=$?
	rm -f ${MODEMTMP}/checkswitchmode
	return $retval
}

fuzzy_switch()
{
	file_19d2=`find /etc/usb_modeswitch.d/ -name "19d2*" -type f | sort` 
	file_12d1=`find /etc/usb_modeswitch.d/ -name "12d1*" -type f | sort`
	file_others=`find /etc/usb_modeswitch.d/ -name "ffff*" -type f | sort`
	local vid
	local pid
	if [ $# -lt 2 ];then
		return 0
	else
		vid=$1
		pid=$2
	fi
	if [ "$vid" = "19d2" ];then
		for file in $file_19d2
		do 
		    syslog $LOG_MODESWITCH_S "fuzzy switch with $file."
			usb_modeswitch -v $vid -p $pid -c $file
			sleep 8
			check_success_switch $vid $pid
			local ret=$?
			if [ $ret = 1 ];then
				return 1
			fi
		done
	elif [ "$vid" = "12d1" ];then
		for file in $file_12d1
		do 
			syslog $LOG_MODESWITCH_S "fuzzy switch with $file."
			syslog "usb_modeswitch -v $vid -p $pid -c $file"
			usb_modeswitch -v $vid -p $pid -c $file
			sleep 8
			check_success_switch $vid $pid
			local ret=$?
			if [ $ret = 1 ];then
				return 1
			fi
		done
	else
		for file in $file_others
		do 
			syslog $LOG_MODESWITCH_S "fuzzy switch with $file."
			usb_modeswitch -v $vid -p $pid -c $file
			sleep 8
			check_success_switch $vid $pid
			local ret=$?
			if [ $ret = 1 ];then
				return 1
			fi
		done		
	fi
	return 0
}

# ugly code for some modem can't switch when first boot
# $1: name (USB Modem / USB Storage / ... , if nil, reboot all usb)
reinit_when_first_boot()
{
	if [ ! -f  "/tmp/reinit_usb_modem"  ];then
		echo "Reinit USB Modem when first boot---" >/dev/console
		syslog $LOG_MODESWITCH_S "Reinit USB Modem when first boot---"
		echo 2 > /tmp/reinit_usb_modem
		usb_pwr_ctrl off $1
		sleep 5
		usb_pwr_ctrl on $1
		echo 1 > /tmp/reinit_usb_modem
		rm -f ${MODEMTMP}/modem_handling
		rm -f ${MODEMTMP}/modem_handled
		exit 0
	fi
}

handle_modem_plugout()
{
	uci set modem.modemconf.modemstatus=0
	uci set modem.modemisp.ispstatus=0
	uci set modem.modemisp.pinlock=0
	uci set modem.modemisp.pincode=""
	uci set modem.modemisp.modem_signal=32
	local setisp=$(uci get modem.modemisp.setisp)
	if [ x$setisp = "xselect" ];then
		uci set modem.modemisp.setisp="auto"
	fi
	
	uci commit modem
	rm -rf ${MODEMTMP}
}

if [ ! -d "${MODEMTMP}" ]; then
	mkdir -p "$MODEMTMP"
fi

if [ -z "${USBPORT}" ]; then
	USBPORT="USB1"
fi

echo "${USBPORT}" >/tmp/modem/USBPORTNUM

if [ ! -f "$BUSFILE" ];then
	mount --bind /dev/bus /proc/bus
	ln -s /sys/kernel/debug/usb/devices $BUSFILE
fi

#
# Add Modem and connect
#
# "5-1"、"6-1"、"7-1"、"8-1"分别是USB驱动分配给"LTE口-2.0"、"LTE口-3.0"、"STORAGE口-2.0"、
# "STORAGE口-3.0"的编号。当插入的是存储设备，又会在编号下分出数个子编号代表接入的设备，
# 比如"5-1"下"5-1.1"、"5-1.2"。此处代码只允许LTE口接入设备， 因而if条件判断"5-1"、"6-1"可进入。
# 并且会继续排除hub设备和单独的USB设备。ER605v2暂不支持USB3.0，所以只有一个设备编号"1-2"
if [ "$ACTION" = add ]; then
	local usb_lte_dev_num=`uci -c /etc/profile.d get profile.@usbshare[0].usb_lte_dev_num`
	local comp=$(echo $usb_lte_dev_num |grep "${DEVICENAME}")

	if [ "$comp" == "" ] ; then
		log "not a modem pluged, exit"
		exit 0
	fi

	if [ -z $BUSNUM ]; then
		exit 0
	fi

	if echo $DEVICENAME | grep -q ":" ; then
		exit 0
	fi
	
	find_usb_attrs

	local usb_interface_create=`cat proc/net/dev |grep usb0 |wc -l`
	local dev_product=`echo $uPr|grep iPhone|wc -l`
	if [ -f "/usr/sbin/usbmuxd" ] && [ $usb_interface_create -eq 0 ] && [ $dev_product -ne 0 ]; then
		/etc/init.d/usbmuxd reload
	fi

	if [ -z $uMa ]; then
		log "Ignoring Unnamed Hub"
		exit 0
	fi

	UPR=${uPr}
	CT=`echo $UPR | tr '[A-Z]' '[a-z]'`
	if echo $CT | grep -q "hub" ; then
		log "Ignoring Named Hub"
		exit 0
	fi

	if [ $uVid = 1d6b ]; then
		log "Ignoring Linux Hub"
		exit 0
	fi
	
	# Only light USB LED when it's not a hub
	echo 1 > /tmp/modem/USB_PLUGIN
	local led_enable=$(uci get switch.led_set.enable)
	if [ "$led_enable" = "on" ]; then
		usb_lte_led_set 1
	fi

	#check whether the handle_card_process.sh process is abnormally exited...
	ps -w | grep "[h]andle_card_process"
	if [ "$?" = "1" ];then
		log "handle_card_process.sh process not exist, create a new one"
		/usr/lib/modem/handle_card_process.sh &
	fi
    
	wait_proc_mount
    modem_printer $uVid $uPid
    if [ "$?" = "0" ];then
        log "Ignoring printer"
        exit 0
    fi

    if [ "$uVid" = "0930" -a "$uPid" = "6545" ];then
		log "storage"
		exit 0
	fi
	bootdelay
	syslog $LOG_CARD_PLUGIN
	if [ -f ${MODEMTMP}/modem_handled ]; then
		log "Modem is handled"
		syslog $LOG_HANDLED_TWICE
		exit 0
	elif [ -f ${MODEMTMP}/modem_handling ]; then
		log "modem is handling"
		syslog $LOG_HANDLEING_TWICE
		exit 0
	fi
	
	led_enable=$(uci get switch.led_set.enable)
	if [ "$led_enable" = "on" ];then
		usb_lte_led_twinkle ${USBPORT}
	fi
	
	log "Add : $DEVICENAME: Manufacturer=${uMa:-?} Product=${uPr:-?} Serial=${uSe:-?} $uVid $uPid"
	syslog $LOG_MODEM_INFO_ORI "$uVid" "$uPid" "${uMa:-?}" "${uPr:-?}" "${uSe:-?}"
	idV=$uVid
	idP=$uPid

	FILEN=$uVid:$uPid
	FILEM=$uVid:
	display_top; display "Start of Modem Detection and Connection Information" 
	display "Product=${uPr:-?} $uVid $uPid"; display_bottom

	uci set modem.modeminfo.defaultvid=$uVid
	uci set modem.modeminfo.defaultpid=$uPid
	uci set modem.modeminfo.modem_type=$uPr
	
	uci set modem.modemisp.ispstatus='0'
	uci commit modem

	cat ${BUSFILE} > ${MODEMTMP}/prembim
	${MODEMLIB}/mbimfind.lua $uVid $uPid 
	local retval=$?
	rm -f ${MODEMTMP}/prembim
	awk -f $MODEMLIB/log_awk $BUSFILE
	if [ $retval -eq 11 ]; then
		display_top; display "Found MBIM Modem at $DEVICENAME"; display_bottom
		echo 2 >/sys/bus/usb/devices/$DEVICENAME/bConfigurationValue
		syslog $LOG_IS_MBIM
		uci set modem.modeminfo.proto="mbim"
		uci commit modem
	else
		check_success_switch
		local befval=$?
		
		# For HUAWEI new model which will modeswitch itself, we wait until itself to finish. (Include: E3372/E8372/...)
		local timeWait=0
		while [ $befval == 0 -a $uVid == "12d1" -a $uPid == "1f01" -a $timeWait -lt 8 ]
		do
			sleep 1
			check_success_switch
			befval=$?
			timeWait=`expr $timeWait + 1`
			log "timeWait: $timeWait"
		done
		if [ $uVid == "12d1" -a $uPid == "1f01" ];then
			ox=`modem_handle -a >> /dev/console`
		fi
		
		if [ $befval = 0 ]; then
		syslog $LOG_MODESWITCH_S "start switch mode of modem."
		if grep "$FILEN" /etc/usb-mode.json > /dev/null ; then
			#logx -p $$ 292 5
			echo "1" > ${MODEMTMP}/modem_handling
			uci set modem.modemconf.modemstatus=1
			uci commit modem

                        syslog $LOG_MODESWITCH_S "modeswitch with modeswtich $idV $idP."
			modeswitch $idV $idP  >> /dev/console
                        sleep 10

			check_success_switch
			local switch_ret=$?
			if [ $switch_ret = 0 ]; then
				syslog $LOG_MODESWITCH_S "modeswitch with specific pid fail, start usbmode switch."
				usbmode -s >> /dev/console
				sleep 8 
			fi

		elif grep "$FILEM" /etc/usb-mode.json > /dev/null; then
			#logx -p $$ 292 6
				syslog $LOG_MODESWITCH_S "no switch message with usb-mode,start fuzzy switch."
			if [ "$uVid" = "04e8" -o "$uVid" = "04E8" ];then
				log "vid=04e8 stroge device."
				if [ "$led_enable" = "on" ]; then
					usb_lte_led_stop_twinkle ${USBPORT}
				fi
				exit 0
			fi
			echo "1" > ${MODEMTMP}/modem_handling
			uci set modem.modemconf.modemstatus=1
			uci commit modem
			fuzzy_switch $uVid $uPid
				#sleep 10
		else
			syslog $LOG_MODESWITCH_S "fuzzy switch fail,this device does not have a switch data file ."
			display_top; display "This device does not have a switch data file" 
			display "Product=${uPr:-?} $uVid $uPid"; display_bottom
			
			reinit_when_first_boot
			if [ "$led_enable" = "on" ]; then
				usb_lte_led_stop_twinkle ${USBPORT}
			fi
			exit 0
		fi
		else
			log  "this device dont modeswitch."
			syslog $LOG_MODESWITCH_S "this device dont modeswitch."
			echo "1" > ${MODEMTMP}/modem_handling
			uci set modem.modemconf.modemstatus=1
			uci commit modem
		fi
	fi
	#sleep 10
	check_success_switch
	local retval=$?
	if [ $retval = 0 ]; then
		log "Cannot switch this modem"
		rm -f ${MODEMTMP}/modem_handling
		uci set modem.modemconf.modemstatus=3
		uci commit modem
		#logx -p $$ 292 7
		syslog $LOG_N_MODESWITCH_FAIL
		#syslog $LOG_N_MODESWITCH_FAIL
		reinit_when_first_boot
		if [ "$led_enable" = "on" ]; then
			usb_lte_led_stop_twinkle ${USBPORT}
		fi
		exit 0
	fi

	echo $DEVPATH > /tmp/modem_dev_path
	echo $DEVPATH >> /dev/console

	# USBPORT not used
	PORTFILE="/tmp/modem/USBPORTNUM"
	
	# uci delete network.mobile
	uci commit network
	/etc/init.d/network reload
    sleep 2

	uci set modem.modeminfo.targetvid=$idV
	uci set modem.modeminfo.targetpid=$idP
	uci commit modem
	#logx -p $$ 292 8 "$idV" "$idP"
	syslog $LOG_MODESWITCH_SUCCEED "$idV" "$idP"
	#syslog $LOG_N_MODESWITCH_SUCCEED
	awk -f $MODEMLIB/log_awk $BUSFILE
	display_top; display "Switched to : $idV:$idP"; display_bottom

	if [ $idV = 2357 -a $idP = 9000 ]; then
		sleep 10
	fi

	if [ "$idV" = 1bbb -a "$idP" = 022c ]; then
		if [ -d "/lib/print_server" ]; then
			. /lib/print_server/printer_driver.sh
			stop_printer_driver
			syslog "print_server stop."
		fi
	fi
	ox=`modem_handle -a >> /dev/console`
	sleep 2

	if [ "$idV" = 1bbb -a "$idP" = 022c ]; then	
		if [ -f "/etc/init.d/print_server" ]; then
			/etc/init.d/print_server  start
			syslog "print_server start."
		fi
	fi
	if [ $ox -ne 0 ]; then
		log "modem_handle failed"
		syslog $LOG_HANDLE_MODEM_FAIL
		rm -f ${MODEMTMP}/modem_handling
		uci set modem.modemconf.modemstatus=3
		uci commit modem
		reinit_when_first_boot
		usb_lte_led_stop_twinkle ${USBPORT}
		exit 0
	fi
	
	#ugly code,for switch mode fail case...
	if [ $(uci get modem.modeminfo.targetpid) = $(uci get modem.modeminfo.defaultpid) ];then
		reinit_when_first_boot
	fi
	
	syslog $LOG_HANDLE_MODEM_SUCCEED
	awk -f $MODEMLIB/log_awk $BUSFILE

	echo "######################################################" >> /dev/console
	echo "################before transfer control to bottom half" >> /dev/console
	
	sem_notifier
	echo "################after transfer control to bottom half" >> /dev/console
	usb_lte_led_stop_twinkle ${USBPORT}
fi

#
# Remove Modem
#
if [ "$ACTION" = remove ]; then
	usbshare cfg -o /etc/config/sharecfg

	local usb_lte_dev_num=`uci -c /etc/profile.d get profile.@usbshare[0].usb_lte_dev_num`
	local comp=$(echo $usb_lte_dev_num |grep "${DEVICENAME}")

	if [ "$comp" == "" ] ; then
		log "do nothing"
	else
		usb_lte_led_set 0
		echo 0 > /tmp/modem/USB_PLUGIN
	fi
	
	if [ -z $BUSNUM ]; then
		log "non busnum"
		exit 0
	fi

	if echo $DEVICENAME | grep -q ":" ; then
		exit 0
	fi
	
	if [ -d "/sys$DEVPATH" ]; then
		log "devpath exist"
		exit 0
	fi

	record_dev_path=`cat /tmp/modem_dev_path`

	echo $record_dev_path >> /dev/console
	echo $DEVPATH >> /dev/console

	if [ x"$record_dev_path" != x"$DEVPATH" ]; then
		log "not a modem unpluged"
		#handle_modem_plugout
		exit 0
	fi

	rm /tmp/modem_dev_path
        rm -f /tmp/3g4gclick

	syslog $LOG_REMOVE_CARD
	modem_handle -r >> /dev/console
	
	local usb_ifs=$(uci get modem.modeminfo.ifs)
	ifdown ${usb_ifs}
	# uci delete network.wan2_eth
	# uci commit network
	# /lib/cfg_save/config.sh modify
    #uci_commit_flash network
	# /etc/init.d/network reload
	killall -9 usbmode
	killall -9 usb_modeswitch
	killall -9 modem_handle
	killall -9 unlock_pin.sh
	killall -9 getisp.sh
	display_top; display "Remove : $DEVICENAME : Modem"; display_bottom

	handle_modem_plugout
fi

