#!/bin/sh

local uVid uPid uMa uPr uSe
local idV idP

export MODEMLIB=/usr/lib/modem
export MODEMTMP=/tmp/modem
export BUSFILE=/proc/bus/usb/devices
export MODEMCFG=/etc/config/modem
export NETWORKCFG=/etc/config/network

. ${MODEMLIB}/modem_scan.sh
. ${MODEMLIB}/usbmodem_log.sh

log() {
	echo "==>handle card" "$@" >> /dev/console
}

# $1: section name
# $2: on/off
# $3: name (USB Modem / USB Storage / ...)
usb_pwr_set() {
	local target=$3
	local name=$(uci get system.$1.name)
	if [ -z "$target" -o "$name" == "$target" ]; then
		local value=$(uci get system.$1.value$2)
		local gpio=$(uci get system.$1.gpio)
		echo $value > /sys/class/gpio/gpio$gpio/value
	fi
}

# $1: on/off
# $2: name (USB Modem / USB Storage / ...)
usb_pwr_ctrl() {
	config_load system
	config_foreach usb_pwr_set usb $1 $2
}

reinit_when_first_boot()
{
	if [ ! -f  "/tmp/reinit_usb_modem"  ];then
		echo "Reinit USB Modem when first boot---" >/dev/console
		syslog $LOG_MODESWITCH_S "Reinit USB Modem when first boot---"
		usb_pwr_ctrl off
		sleep 5
		usb_pwr_ctrl on
		echo 1 > /tmp/reinit_usb_modem
		rm -f ${MODEMTMP}/modem_handling
		rm -f ${MODEMTMP}/modem_handled
		# exit 0
	fi
}

while :
do
	echo "############################### top half stand by" >> /dev/console

	sem_receiver

	echo "###############################taken over contorl from bottom half" >> /dev/console
	
	proto=$(uci get modem.modeminfo.proto)

	idV=$(uci get modem.modeminfo.targetvid)
	idP=$(uci get modem.modeminfo.targetpid)
	defVid=$(uci get modem.modeminfo.defaultvid)
	defPid=$(uci get modem.modeminfo.defaultpid)

	if [ $proto = "ncm" ] && [ $idV = "12d1" ] && [ $idP = "1506" ] && [ $defVid = "12d1" ] && [ $defPid = "14fe" ];then
		proto="dhcp"
		uci set modem.modeminfo.proto="dhcp"
		uci set modem.modeminfo.ifname="usb0"
		echo "########handle_process set proto = dhcp" > /dev/console
	fi

	case ${proto} in
	"3g" | "ncm" )
		if [ -e /dev/ttyUSB0 -o -e /dev/ttyACM0 ];then
			log "device OK"
		else
			log "wait device"
			sleep 10
		fi
		if [ $proto = "ncm" ];then
			if [ $idV = "12d1" ] && [ $idP = "1506" ];then
				echo "this is E3276 send reset cmd" >> /dev/console
				resetTTY=$(uci get modem.modeminfo.cport)
				comgt -d "/dev/$resetTTY" -s /etc/gcom/reset.gcom
			fi

		fi
		
		if [ $proto = "3g" ];then
			if [ $idV = "19d2" ] && [ $idP = "0079" ];then
				echo "this is ZTE A356" >> /dev/console
				COMMAND="AT+CFUN=1" gcom -d /dev/ttyUSB1 -s /etc/gcom/runcommand.gcom		
			fi
		fi

		cport=$(uci get modem.modeminfo.cport)

		if [ "${cport}" = "UNSURE_TTY" ]; then
		    
			local found=0
			modem_scan_allport
			local retval=$?
			
			if [ "$retval" = 1 ]; then
				log "modem scan allport fail"
				
				syslog $LOG_SEARCHPORT_FAIL
				
				rm -f ${MODEMTMP}/modem_handling
				
				uci set modem.modemconf.modemstatus=3
				
				uci commit modem
				reinit_when_first_boot				
				exit 0
			fi

			 modem_scan_cport
			 retval=$?
			
			if [ "$retval" = 1 ]; then
				log "modem scan cport fail"
				syslog $LOG_SCAN_CPORT_FAIL 
				rm -f ${MODEMTMP}/modem_handling
				uci set modem.modemconf.modemstatus=3
				uci commit modem
				reinit_when_first_boot
				exit 0
			fi
		fi

		cport=$(uci get modem.modeminfo.cport)
		if [ "${cport}" = "UNSURE_TTY" -o "${cport}" = "NO_TTY" ]; then
			log "no cport found"
		else
			${MODEMLIB}/unlock_pin.sh "query" ${proto} ${cport}
			local pin_lock=$(uci get modem.modemisp.pinlock)
			local setisp_flag=$(uci get modem.modemisp.setisp)
			if [ $pin_lock = 0 -o $pin_lock = 4 ] && [ x$setisp_flag = "xauto" ]; then				
				${MODEMLIB}/getisp.sh
			fi
		fi
		;;
	"qmi" )
		if [ -e /dev/cdc-wdm0 ];then
			log "device OK"
		else
			log "wait device"
			sleep 10
		fi
		${MODEMLIB}/unlock_pin.sh "query"
		local pin_lock=$(uci get modem.modemisp.pinlock)
		local setisp_flag=$(uci get modem.modemisp.setisp)
		if [ $pin_lock = 0 -o $pin_lock = 4 ] && [ x$setisp_flag = "xauto" ]; then
			log "start get isp"
			${MODEMLIB}/getisp.sh
		fi

		;;
	"mbim" )
		;;
	"dhcp" )
		log "dhcp modem delay."
		sleep 10
		;;
	esac

	local usb_ifs=$(uci get modem.modeminfo.ifs)
	local usb_ifs6=$(uci get modem.modeminfo.ifs6)
	
	uci set network.${usb_ifs}=interface
	
	uci set network.${usb_ifs}.proto=$proto  
	
	uci set network.${usb_ifs}.ifname=$(uci get modem.modeminfo.ifname)
	
	uci set network.${usb_ifs}.apn=$(uci get modem.modemisp.apn)
	
	uci set network.${usb_ifs}.dialnumber=$(uci get modem.modemisp.dial_num)
	
	uci set network.${usb_ifs}.username=$(uci get modem.modemisp.username)
	
	uci set network.${usb_ifs}.password=$(uci get modem.modemisp.password)
	
	uci set network.${usb_ifs}.auth=$(uci get modem.modemisp.authentype)
	
	uci set network.${usb_ifs}.conn_mode=$(uci get modem.modemconf.connectmode)
	
	uci set network.${usb_ifs}.connectable="1"
	
	uci set network.${usb_ifs}.hostname=""
	
	uci set network.${usb_ifs}.broadcast="1"
	
	uci set network.${usb_ifs}.auto="0"
	
	local dns_manual=$(uci get modem.modemconf.manualdns)
	
	if [ x$dns_manual = x"on" ];then
		uci set network.${usb_ifs}.dns_mode="static"
		uci set network.${usb_ifs}.peerdns="0"
		local pdns=$(uci get modem.modemconf.primarydns)
		local sdns=$(uci get modem.modemconf.seconddns)
		uci set network.${usb_ifs}.dns="$pdns $sdns"
	else
		uci set network.${usb_ifs}.dns_mode="dynamic"
		uci set network.${usb_ifs}.peerdns="1"
	fi
	
	case ${proto} in
	"3g" )
		dport=$(uci get modem.modeminfo.dport)
		uci set network.${usb_ifs}.device="/dev/${dport}"
        if [ $dport = "UNSURE_TTY" ];then
           uci set network.${usb_ifs}.testTTY="1"
           uci set network.${usb_ifs}.nextTryTTY=`head -1 /tmp/modem/ttylist` 
        fi
		;;
	"ncm" )
		cport=$(uci get modem.modeminfo.cport)
		uci set network.${usb_ifs}.device="/dev/${cport}"
		;;
	"qmi" | "mbim")
		uci set network.${usb_ifs}.device="/dev/cdc-wdm0"
		;;
	esac
	uci commit network
	awk -f $MODEMLIB/log_awk $NETWORKCFG
	echo "1" > ${MODEMTMP}/modem_handled
	rm -f ${MODEMTMP}/modem_handling
	uci set modem.modemconf.modemstatus=2
	uci commit modem
	
	awk -f $MODEMLIB/log_awk $MODEMCFG
	
	local conn_mode=$(uci get modem.modemconf.connectmode)
	[ $conn_mode != "manually" ] && {
	local modem_st=$(uci get modem.modemconf.modemstatus)
	local pin_st=$(uci get modem.modemisp.pinlock)
	local pdp_type=$(uci get modem.modemisp.pdp_type)
	if [ $modem_st = 2 ] && [ $pin_st = 0 -o $pin_st = 4 ];then
		log "start network reload"
		syslog $LOG_START_CONNECTING
		/etc/init.d/network reload
		sleep 3	
	fi

	log "ifconfig usb0 up"
	ifconfig usb0 up
	sleep 3
	log "ifup ${usb_ifs}"
	ifup ${usb_ifs}
	
	[ "$pdp_type" != "IP" -a "$proto" == "dhcp" ] && {
		log "ifup ${usb_ifs6}"
		ifup {$usb_ifs6}
	}
	}
done

