. /lib/functions.sh

local portal_status=0

get_portal_enabled_from_vnetwork(){
	config_get portal $1 portal
	config_get httpsRedirect $1 httpsRedirect

	if [ "${portal}" == "1" -a "$portal_status" == "0" ];then
		portal_status=1
	fi	

	if [ "${portal}" == "1" -a "${httpsRedirect}" == "1" ];then
		portal_status=2
	fi
}

get_portal_enabled_from_ssid_list(){
	config_get portalEnable $1 portalEnable
	config_get httpsRedirectEnable $1 httpsRedirectEnable

	if [ "${portalEnable}" == "1" -a "$portal_status" == "0" ];then
		portal_status=1
	fi

	if [ "${portalEnable}" == "1" -a "${httpsRedirectEnable}" == "1" ];then
		portal_status=2
	fi
}

get_portal_https_enable()
{
	config_load vnetwork
	config_foreach  get_portal_enabled_from_vnetwork vinterface
	config_load ssid_list
	config_foreach  get_portal_enabled_from_ssid_list rule_ssid_info

	if [ "$portal_status" == "1" ];then
		echo "enable"
	elif [ "$portal_status" == "2" ];then
		echo "https"
	else
		echo "disable"
	fi
}

portal_start()
{
	local isCsMng=`cat /etc/config/controller.conf | grep csId | grep c21f969b5f03d33d43e04f8f136e7682 | wc -l`
	if [ ${isCsMng} -le 0 ] ; then
		portal_mgmt csstat 1
	fi

	pid=$(ps | grep "portal_mgmt_mon" | grep -v grep | awk '{print $1}')
	if [ "$pid" == "" ];then
		/usr/lib/portal_mgmt/portal_mgmt_monitor.sh&
	fi

}

portal_nginx_start()
{
	mkdir -p /var/log/nginx
	mkdir -p /var/lib/nginx
	mkdir -p /usr/logs
	
	cnt=`ps | grep "nginx" | grep -v grep | wc -l`
	if [ $cnt -le 0 ] ; then
		vnete $NGINX_BIN
	fi
	
	# match mark bit 28(0x10000000, mask 0x10000000), redirect to nginx listen port 4433
	if ! vnete iptables -t nat -w -C PREROUTING -m mark --mark 0x10000000/0x10000000 -p tcp --dport 443 -j REDIRECT --to-ports 4433 2>/dev/null; then
		vnete iptables -t nat -w -I PREROUTING -m mark --mark 0x10000000/0x10000000 -p tcp --dport 443 -j REDIRECT --to-ports 4433
	fi
    #fix security bugs, reject connections to localhost:4433
	if ! vnete iptables -t raw -w -C PREROUTING -m set --match-set VMe dst -p tcp --dport 4433 -j DROP 2>/dev/null; then
    	vnete iptables -t raw -w -I PREROUTING -m set --match-set VMe dst -p tcp --dport 4433 -j DROP
	fi
    #vnete iptables -t filter -w -I INPUT -m mark ! --mark 0x10000000/0x10000000 -p tcp --dport 4433 -j REJECT
}

portal_monitor_stop()
{
	pid=$(ps | grep "portal_mgmt_mon" | grep -v grep | awk '{print $1}')
	[ "$pid" != "" ] && kill -9 $pid
}

portal_nginx_stop()
{
    #fix security bugs, reject connections to localhost:4433
    #vnete iptables -t filter -w -D INPUT -m mark ! --mark 0x10000000/0x10000000 -p tcp --dport 4433 -j REJECT
	vnete iptables -t raw -w -D PREROUTING -m set --match-set VMe dst -p tcp --dport 4433 -j DROP 2>/dev/null
	vnete iptables -t nat -w -D PREROUTING -m mark --mark 0x10000000/0x10000000 -p tcp --dport 443 -j REDIRECT --to-ports 4433 2>/dev/null
	vnete $NGINX_BIN -s stop	
}

try_to_start_daemon()
{
	portal_https=`get_portal_https_enable`
	if [ "$portal_https" == "enable" ];then
		echo "no need to start nginx"
		portal_start
	elif [ "$portal_https" == "https" ];then
		portal_start
		portal_nginx_start
	else
		echo "to do nothing"
	fi
}

try_to_stop_daemon()
{
	portal_https=`get_portal_https_enable`
	if [ "$portal_https" == "disable" ];then
		portal_monitor_stop
		portal_nginx_stop
	elif [ "$portal_https" == "enable" ];then
		portal_nginx_stop
	else
		echo "to do nothing"
	fi
}
