#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/firewall/core_tpfirewall.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK
IFNAME=$1

if [ ! -n "$7" ] ;then
        echo "no config file name"
        CONFIGFILENAME=${IFNAME}
else
        CONFIGFILENAME=$7
fi

openvpn_client_down_hook()
{
	local indexname=
	indexname=$(echo $CONFIGFILENAME | tr -cd "[0-9]")
	echo "openvpn_client_down $IFNAME $indexname" >  /dev/console
	local dns_len=0
	local dns_1=
	local dns_2=

	config_load 'openvpn'	
	config_get dns_len "client$indexname" dns_len
	config_get wan "client$indexname" wan
        #update V zone groups
        . /lib/zone/zone_api.sh
        zone_delete_vgroup "$IFNAME"

    #fixme:dns没有值
	for i in seq $dns
	do
		if [ $i == 1 ]; then
			dns_1=${nameserver}
		fi
		if [ $i == 2 ]; then
			dns_2=${nameserver}
		fi
		uci set openvpn.client${indexname}.dns$i=""
	done
	if [ -z ${dns_2} ]; then
		dns_2=${dns_1}
	fi
	# prevent the recurrence of the same interface
	uci set openvpn.client${indexname}.dev=""
	uci set openvpn.client${indexname}.uptime=""
	uci set openvpn.client${indexname}.vlocalip=""
	uci set openvpn.client${indexname}.vremoteip=""
	uci commit openvpn
	
	nums=$(iptables -w -nvL openvpn_forwarding_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -D openvpn_forwarding_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -nvL openvpn_forwarding_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

	nums=$(iptables -w  -nvL zone_wan_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w  -D zone_wan_openvpn ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -nvL zone_wan_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done
	
	nums=$(iptables -w -t nat -nvL postrouting_rule_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t nat -D postrouting_rule_openvpn ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL postrouting_rule_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done
	
	nbptc down ${IFNAME} "ovpn"

	tables="${OPENVPN_FORWARD_TABLE} ${OPENVPN_ROUTING_TABLE}"
	for tbl in $tables
	do
		nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${dev}\b" | cut -d ' ' -f 1)
		while [ -n "${nums}" ]; do
			for num in $nums;
			do
				iptables -w -t mangle -D ${tbl} ${num}
				break; # just delete the first once.
			done
			nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${dev}\b" | cut -d ' ' -f 1)
		done
	done

	#CHAIN:manage_devs
	fw_event_manage_dev "${IFNAME}:down"

	add_del_dnsserver "del" ${IFNAME} ${dns_1} ${dns_2}

    local conf_line=
    let conf_line=$(grep -n "${IFNAME}" /tmp/resolv.conf.auto | cut -d ":" -f 1)
    if [ -n "${conf_line}" ]; then
        {
            flock -x 199
            if [ -n "${conf_line}" ]; then
                let conf_line2=${conf_line}+2
                sed -i "${conf_line},${conf_line2}d" /tmp/resolv.conf.auto
            fi
        } 199> /tmp/resolv.conf.l2tp.pptp.lock
    fi
	
	[ -x /etc/init.d/dnsproxy ] && /etc/init.d/dnsproxy restart

	exit 0
}

openvpn_client_down_hook

