#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh

VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK

local indexname=
local local_network=

indexname=$(echo $config | tr -cd "[0-9]")
#echo "openvpn_client_routeup $dev $indexname" >  /dev/console

config_load 'openvpn'
config_get local_network "client$indexname" local_network

if [ $redirect_gateway -eq 0 ]; then

	local netnum=0
	for i in $(seq 1 129)
	do
		local tempnet=$(eval echo '$'route_network_${i})
		if [ -n "$tempnet" ]; then
			netnum=$(($netnum+1))
		else
			break
		fi
	done

	for i in $(seq 1 $netnum)
	do
		local remote_network=
		local remote_net=
		local remote_mask=

		remote_net=$(eval echo '$'route_network_${i})
		remote_mask=$(eval echo '$'route_netmask_${i})
		remote_mask=$(ipcalc -p ${remote_net} ${remote_mask} | cut -d '=' -f2)
		remote_network=${remote_net}/${remote_mask}

		for local in ${local_network}
		do
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -i br-lan -s ${local} -d ${remote_network} -m comment --comment "tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -i br-lan -s ${local} -d ${remote_network} -m comment --comment "tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_ROUTING_TABLE} -i "${dev}" -s ${remote_network} -m set --match-set $(zone_allwan_group) dst -m comment --comment "vnet=${local},tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} -m comment --comment "vnet=${local},tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
		done
	done
else
	for local in ${local_network}
	do
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -i br-lan -s ${local} -d "0.0.0.0/0" -m comment --comment "tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -i br-lan -s ${local} -d "0.0.0.0/0" -m comment --comment "tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_ROUTING_TABLE} -i "${dev}" -s "0.0.0.0/0" -m set --match-set $(zone_allwan_group) dst -m comment --comment "vnet=${local},tun=${dev}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} -m comment --comment "vnet=${local},tun=${dev}" -j MARK --or-mark ${OPENVPN_FWMARK}
	done
fi
exit 0
