#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/firewall/core_tpfirewall.sh
. /lib/nat/nat_core.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK
IFNAME=$1
VLOCALIP=$4
VREMOTEIP=$route_vpn_gateway
if [ ! -n "$7" ] ;then
        echo "no config file name"
        CONFIGFILENAME=${IFNAME}
else
        CONFIGFILENAME=$7
fi

# init variables
i=1
fopt=
nns=0
nl='
'

while true; do
  eval fopt=\$foreign_option_${i}
  [ -z "${fopt}" ] && break

  case ${fopt} in
		dhcp-option\ DNS\ *)
           nns=$((nns + 1))
           if [ $nns -le 3 ]; then
             dns="${dns}${dns:+$nl} ${fopt#dhcp-option DNS }"
           else
             printf "%s\n" "Too many nameservers - ignoring after third" >&2
           fi
           ;;
        *)
           printf "%s\n" "Unknown option \"${fopt}\" - ignored" >&2
           ;;
	esac
  i=$((i + 1))
done



route_table_init()
{
	ret=`ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		iptables -w -t mangle -F ${OPENVPN_FORWARD_TABLE}
		iptables -w -t mangle -F ${OPENVPN_ROUTING_TABLE}
		ip rule add pref $VPN_RULE_NUM fwmark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} table $VPN_RULE_NUM
	fi
}

openvpn_client_up_hook()
{
	local indexname=
    local proto=
    local port=
	local local_network=
	local dns_num=1
	#local wans=
	local dns_1=
	local dns_2=
    local line=1
    local null_file=1

	indexname=$(echo $CONFIGFILENAME | tr -cd "[0-9]")
	echo "openvpn_client_up $IFNAME $indexname" >  /dev/console

    if [ -n "${dns}" ]; then
        {
            flock -x 199
            test -s /tmp/resolv.conf.auto && null_file=0
            #sed不能操作空文件，sed直接操作resolv.conf会破坏软连接，于是空文件采用echo到resolv.conf.auto，非空sed操作/tmp/resolv.conf.auto
            [ $null_file == 0 ] && sed -i "${line}i #Interface ${IFNAME}" /tmp/resolv.conf.auto || echo "#Interface ${IFNAME}" >>/tmp/resolv.conf
            for nameserver in $dns 
            do
                if [ $dns_num == 1 ]; then
                    dns_1=${nameserver}
                fi
                if [ $dns_num == 2 ]; then
                    dns_2=${nameserver}
                fi
                let line+=1
                [ $null_file == 0 ] && sed -i "${line}i nameserver ${nameserver}" /tmp/resolv.conf.auto || echo "nameserver ${nameserver}" >>/tmp/resolv.conf
                uci set openvpn.client${indexname}.dns$dns_num=${nameserver}
                let dns_num+=1 
            done
            if [ -z ${dns_2} ]; then
                let line+=1
                [ $null_file == 0 ] && sed -i "${line}i nameserver ${dns_1}" /tmp/resolv.conf.auto || echo "nameserver ${dns_1}" >>/tmp/resolv.conf
                dns_2=${dns_1}
            fi
        } 199> /tmp/resolv.conf.l2tp.pptp.lock
    fi

	
	uci set openvpn.client${indexname}.dns_len=$nns
	uci set openvpn.client${indexname}.dev=${IFNAME}
	uci set openvpn.client${indexname}.uptime=$(date +%s)
	uci set openvpn.client${indexname}.vlocalip=${VLOCALIP}
	uci set openvpn.client${indexname}.vremoteip=${VREMOTEIP}
	uci commit openvpn
	
	config_load 'openvpn'	
	config_get port "client$indexname" port
	config_get proto "client$indexname" proto
	config_get local_network "client$indexname" local_network
	config_get wans "client$indexname" wan

	#zone_wan_openvpn forword
	for wan in ${wans} 
	do 
		devices=$(zone_get_wan_devices $wan)
		for device in ${devices}
		do
			echo "zone_wan_openvpn ${wan} ${device}" > /dev/console
			fw add 4 f zone_wan_openvpn ACCEPT { "-i $device -p $proto -m $proto --dport $port -m comment --comment $IFNAME" }
		done
	done

	#INPUT
	#for wan in ${wans} 
	#do 
	#	device=$(zone_get_wan_devices $wan)
	#	fw add 4 f zone_wan_openvpn ACCEPT { "-i $device -p $proto -m $proto --dport $port" }
	#done
	
	#update V zone groups
	. /lib/zone/zone_api.sh
	for wan in ${wans}
        do
		zone_add_vgroup "$IFNAME" "$wan"
        done


	#FORWORDING
    fw add 4 f openvpn_forwarding_rule ACCEPT { "-o $IFNAME -m conntrack --ctstate RELATED,ESTABLISHED" }
	for local in ${local_network} 
	do 
		fw add 4 f openvpn_forwarding_rule ACCEPT { "-i br-lan -o $IFNAME -s $local" }
		fw add 4 f openvpn_forwarding_rule ACCEPT { "-i $IFNAME -o br-lan -d $local" }
		nat_postrouting_add openvpn "-A" "-s $local -o $IFNAME -j MASQUERADE" 
	done
	
	add_del_dnsserver "add" ${IFNAME} ${dns_1} ${dns_2}
	[ -x /etc/init.d/dnsproxy ] && /etc/init.d/dnsproxy restart

	#CHAIN:manage_devs
	fw_event_manage_dev "${IFNAME}:up"

	nbptc up ${IFNAME} "ovpn"
	exit 0
}

route_table_init
openvpn_client_up_hook