#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh

VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK

IFNAME=$1

openvpn_server_down_hook()
{
	local indexname=
	indexname=$(echo $IFNAME | tr -cd "[0-9]")
	echo "openvpn_server_down $IFNAME" >  /dev/console

	uci set openvpn.server${indexname}.vlocalip=""
	uci commit openvpn

	#update V zone groups
	. /lib/zone/zone_api.sh
	zone_delete_vgroup "$IFNAME"
	/lib/nat/nat_dmz_bypass.sh  ovpn_reload

	nums=$(iptables -w -nvL openvpn_forwarding_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -D openvpn_forwarding_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -nvL openvpn_forwarding_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done
	
	nums=$(iptables -w -t nat -nvL openvpn_prerouting_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t nat -D openvpn_prerouting_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL openvpn_prerouting_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done
	
	nums=$(iptables -w  -nvL zone_wan_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w  -D zone_wan_openvpn ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -nvL zone_wan_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

	nums=$(iptables -w -t nat -nvL postrouting_rule_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t nat -D postrouting_rule_openvpn ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL postrouting_rule_openvpn --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

	nbptc down ${IFNAME} "ovpn"

	tables="${OPENVPN_FORWARD_TABLE} ${OPENVPN_ROUTING_TABLE}"
	for tbl in $tables
	do
		nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
		while [ -n "${nums}" ]; do
			for num in $nums;
			do
				iptables -w -t mangle -D ${tbl} ${num}
				break; # just delete the first once.
			done
			nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
		done
	done

	ippool_set="ovpn_${IFNAME}"
	ipset destroy $ippool_set 

}

openvpn_server_down_hook
exit 0