#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/nat/nat_core.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
OPENVPN_FWMARK=0x20000
ANTI_OPENVPN_FWMARK=0xfffdffff
OPENVPN_FORWARD_TABLE=OPENVPN_FORWARD_MARK
OPENVPN_ROUTING_TABLE=OPENVPN_ROUTING_MARK

IFNAME=$1
VLOCALIP=$4

route_table_init()
{
	ret=`ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		iptables -w -t mangle -F ${OPENVPN_FORWARD_TABLE}
		iptables -w -t mangle -F ${OPENVPN_ROUTING_TABLE}
		ip rule add pref $VPN_RULE_NUM fwmark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} table $VPN_RULE_NUM
	fi

	ret=`vnete ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		vnete ip rule add pref $VPN_RULE_NUM table $VPN_RULE_NUM
	fi
}

openvpn_server_up_hook()
{
	local indexname=
    local proto=
    local port=
	local local_network=
	local ippool=
	local pool_network=
	local pool_prefix=
	local wans=
	local ifname=
	local device=
	
	indexname=$(echo $IFNAME | tr -cd "[0-9]")
	echo "openvpn_server_up ${IFNAME} ${indexname}" >  /dev/console

	uci set openvpn.server${indexname}.vlocalip=${VLOCALIP}
	uci commit openvpn
	
	config_load 'openvpn'	
	config_get port "server$indexname" port
	config_get proto "server$indexname" proto
	config_get local_network "server$indexname" local_network
	config_get ippool "server$indexname" server
	config_get wans "server$indexname" wan
	config_get ifname "server$indexname" dev

	#update V zone groups
	. /lib/zone/zone_api.sh
	for wan in ${wans}
	do
		zone_add_vgroup "$IFNAME" "$wan"
		local devices=$(zone_get_wan_devices $wan)
		for device in ${devices}
		do
			fw add 4 f zone_wan_openvpn ACCEPT { "-i $device -p $proto -m $proto --dport $port -m comment --comment $IFNAME" }
		done
	done

	#FORWORDING
	fw add 4 n openvpn_prerouting_rule ACCEPT { "-p $proto -m $proto --dport $port -m comment --comment $IFNAME" }	
    fw add 4 f openvpn_forwarding_rule ACCEPT { "-o $IFNAME -m conntrack --ctstate RELATED,ESTABLISHED" }
	for local in ${local_network} 
	do 
		fw add 4 f openvpn_forwarding_rule ACCEPT { "-i br-lan -o $IFNAME -s $local" }
		fw add 4 f openvpn_forwarding_rule ACCEPT { "-i $IFNAME -o br-lan -d $local" }
	done
	
	nbptc up ${IFNAME} "ovpn"


#	export > /dev/console
#	vnete sysctl -w /proc/sys/net/ipv4/conf/all/proxy_arp=1

	pool_network=$(ipcalc -n $ippool | cut -d '=' -f2)
	pool_prefix=$(ipcalc -p $ippool | cut -d '=' -f2)
	ippool=${pool_network}/${pool_prefix}

	ippool_set="ovpn_${IFNAME}"
	ipset create $ippool_set hash:net
	ipset flush $ippool_set
	for local in ${local_network} 
	do 
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -i br-lan -s ${local} -m set --match-set ${ippool_set} dst -m comment --comment "if=${IFNAME}" -j MARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -i br-lan -s ${local} -m set --match-set ${ippool_set} dst  -m comment --comment "if=${IFNAME}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_ROUTING_TABLE} -i "${IFNAME}" -m set --match-set ${ippool_set} src -m set --match-set $(zone_allwan_group) dst -m comment --comment "if=${IFNAME},vnet=${local}" -j CONNMARK --or-mark ${OPENVPN_FWMARK}
		iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${OPENVPN_FWMARK}/${OPENVPN_FWMARK} -m comment --comment "if=${IFNAME},vnet=${local}" -j MARK --or-mark ${OPENVPN_FWMARK}
		if [ ${local} != "0.0.0.0/0" ]; then
			iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -s ${local} ! -i br-lan -m conntrack --ctdir REPLY -m comment --comment "if=${IFNAME}" -j MARK --or-mark ${OPENVPN_FWMARK}
			iptables -w -t mangle -A ${OPENVPN_FORWARD_TABLE} -s ${local} -m set ! --match-set ${ippool_set} dst -m policy --dir in --pol ipsec ! -i br-lan -m conntrack --ctdir REPLY -m comment --comment "if=${IFNAME}" -j MARK --and-mark ${ANTI_OPENVPN_FWMARK}
		fi
	done 

	#添加网络代理功能
	nat_postrouting_add openvpn "-A" "-m set --match-set $ippool_set src -o br-lan -m comment --comment "if=${IFNAME}" -j RETURN"
	nat_postrouting_add openvpn "-A" "-m set --match-set $ippool_set src -m comment --comment "if=${IFNAME}" -j MASQUERADE" 

	#添加dmz相关规则
	/lib/nat/nat_dmz_bypass.sh ovpn_reload

}



route_table_init
openvpn_server_up_hook
exit 0
