#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/firewall/core_tpfirewall.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
WIREGUARD_FWMARK=0x20000
WIREGUARD_FORWARD_TABLE=WIREGUARD_FORWARD_MARK
WIREGUARD_ROUTING_TABLE=WIREGUARD_ROUTING_MARK
WIREGUARD_OUTPUT_TABLE=WIREGUARD_OUTPUT_RULE
WIREGUARD_INPUT_RULE=wireguard_input_rule
WIREGUARD_PRESERVICE_RULE=wireguard_preservice_rule
WIREGUARD_SERVICE_RULE=wireguard_service_rule

METHOD=$1
IFNAME=$2
ALLOW_IP=$3
LISTEN_PORT=$3

do_service_wg_down()
{
	nums=$(iptables -w -t nat -nvL $WIREGUARD_SERVICE_RULE --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t nat -D $WIREGUARD_SERVICE_RULE ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL $WIREGUARD_SERVICE_RULE --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

	nums=$(iptables -w -t raw -nvL $WIREGUARD_PRESERVICE_RULE --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
		while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t raw -D $WIREGUARD_PRESERVICE_RULE ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t raw -nvL $WIREGUARD_PRESERVICE_RULE --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done
}

wireguard_down_hook()
{

	#CHAIN:manage_devs
	fw_event_manage_dev "${IFNAME}:down"
	
	nums=$(iptables -w -nvL wireguard_forwarding_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -D wireguard_forwarding_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -nvL wireguard_forwarding_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done
	

	nums=$(iptables -w -t nat -nvL wireguard_prerouting_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t nat -D wireguard_prerouting_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL wireguard_prerouting_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done
	
	nums=$(iptables -w -t nat -nvL wireguard_postrouting_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t nat -D wireguard_postrouting_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t nat -nvL wireguard_postrouting_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

	tables="${WIREGUARD_FORWARD_TABLE} ${WIREGUARD_ROUTING_TABLE}"
	for tbl in $tables
	do
		nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
		while [ -n "${nums}" ]; do
			for num in $nums;
			do
				iptables -w -t mangle -D ${tbl} ${num}
				break; # just delete the first once.
			done
			nums=$(iptables -w -t mangle -nL ${tbl} --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
		done
	done

	##del filter rules
	nums=$(iptables -w -t filter -nL ${WIREGUARD_INPUT_RULE} --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -w -t filter -D ${WIREGUARD_INPUT_RULE} ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t filter -nL ${WIREGUARD_INPUT_RULE} --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

    nums=$(iptables -w -t filter -nvL input_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	while [ -n "${nums}" ]; do
		for num in $nums;
		do
			iptables -D input_rule ${num}
			break; # just delete the first once.
		done
		nums=$(iptables -w -t filter -nvL input_rule --line-numbers | grep "\b${IFNAME}\b" | cut -d ' ' -f 1)
	done

	##delete interface in dnsproxy
	[ -n "$IFNAME" ] && /lib/dnsproxy/api.sh delete "${IFNAME}"

	do_service_wg_down

	local chain=zone_vpn_${IFNAME}_MSSFIX
	fw flush 4 m ${chain}
	fw del 4 m zone_MSSFIX ${chain} ^ { -o ${IFNAME} }
	fw del 4 m zone_MSSFIX ${chain} ^ { -i ${IFNAME} }
	fw del 4 m ${chain}

	port=$(uci get wireguard_interface.$IFNAME.listen_port)
	/lib/nat/nat_dmz_bypass.sh del udp $port all_wan
    

}


peer_down_hook()
{	
	#FORWORDING

	fw del 4 f wireguard_forwarding_rule ACCEPT { "-i br-lan -o $IFNAME -d $ALLOW_IP" }
	fw del 4 f wireguard_forwarding_rule ACCEPT { "-i $IFNAME -o br-lan -s $ALLOW_IP" }

	#fw add 4 n openvpn_postrouting_rule MASQUERADE { "-d $ALLOW_IP -o $IFNAME" }
#	export > /dev/console

	if [ "${ALLOW_IP}" != "0.0.0.0/0" ]; then
		iptables -w -t nat -D wireguard_postrouting_rule  -o br-lan -s ${ALLOW_IP} -m comment --comment "if=${IFNAME}" -j RETURN
		iptables -w -t nat -D wireguard_postrouting_rule  ! -o ${IFNAME} -s ${ALLOW_IP} -m comment --comment "if=${IFNAME}" -j MASQUERADE
		ip route del  ${ALLOW_IP} dev ${IFNAME}
	else
		iptables -w -t nat -D wireguard_postrouting_rule  -o br-lan  -m comment --comment "if=${IFNAME}" -j RETURN
		iptables -w -t nat -D wireguard_postrouting_rule  -o ${IFNAME}  -m comment --comment "if=${IFNAME}" -j MASQUERADE
	fi
	iptables -w -t mangle -D ${WIREGUARD_FORWARD_TABLE} -i br-lan  -d ${ALLOW_IP} -m comment --comment "if=${IFNAME}" -j MARK --or-mark ${WIREGUARD_FWMARK}
	iptables -w -t mangle -D ${WIREGUARD_FORWARD_TABLE} -i br-lan  -d ${ALLOW_IP} -m comment --comment "if=${IFNAME}" -j CONNMARK --or-mark ${WIREGUARD_FWMARK}

	
	ip route del  ${ALLOW_IP} dev ${IFNAME} table $VPN_RULE_NUM
}

if [ "${METHOD}" = "interface" ]; then
    wireguard_down_hook
fi
if [ "${METHOD}" = "peer" ]; then
    peer_down_hook
fi

