#!/bin/sh
. /lib/functions.sh
. /lib/functions/service.sh
. /lib/openvpn/openvpn-common.sh
. /lib/zone/zone_api.sh
. /lib/functions/network.sh
. /lib/firewall/fw.sh
. /lib/firewall/core_tpfirewall.sh

#VPN相关策略路由条目都在table id为25的路由表中，其中内核arp.c中有相关修改，若是需要修改优先级，请同时修改arp.c的相关代码
VPN_RULE_NUM=25
WIREGUARD_FWMARK=0x20000
WIREGUARD_FORWARD_TABLE=WIREGUARD_FORWARD_MARK
WIREGUARD_ROUTING_TABLE=WIREGUARD_ROUTING_MARK
WIREGUARD_OUTPUT_TABLE=WIREGUARD_OUTPUT_RULE
WIREGUARD_INPUT_RULE=wireguard_input_rule
WIREGUARD_PRESERVICE_RULE=wireguard_preservice_rule
WIREGUARD_SERVICE_RULE=wireguard_service_rule

METHOD=$1
IFNAME=$2
ALLOW_IP=$3
LISTEN_PORT=$3
WG_MTU=$4


add_wg_service()
{
	local ifconfigall=$(ifconfig $IFNAME)
	local iplocal=${ifconfigall#*inet addr:}
	iplocal=${iplocal%% *}
	[ -z "$iplocal" ] && return
	iptables -w -t raw -I $WIREGUARD_PRESERVICE_RULE -i $IFNAME -p udp --dport 53 -d $1 -j CT --notrack -m comment --comment "if=${IFNAME}"
	iptables -w -t raw -I $WIREGUARD_PRESERVICE_RULE -i $IFNAME -p tcp --dport 53 -d $1 -j CT --notrack -m comment --comment "if=${IFNAME}"
	iptables -w -t nat -A $WIREGUARD_SERVICE_RULE -o $IFNAME -p udp --sport 53 -s $iplocal -j SNAT --to-source $1 -m comment --comment "if=${IFNAME}"
	iptables -w -t nat -A $WIREGUARD_SERVICE_RULE -o $IFNAME -p tcp --sport 53 -s $iplocal -j SNAT --to-source $1 -m comment --comment "if=${IFNAME}"
}

find_lanip()
{
	local section=$1
	local system=`uci get vnetwork.$section.system`
	[ "$system" == "1" ] && lanip=`uci get vnetwork.$section.ipaddr`
}


do_service_wg_up()
{
	#echo "dns1: $dns1 dns2 $dns2" > /dev/console
	local lanip="169.254.11.22"
	config_load vnetwork
	config_foreach find_lanip vinterface
	[ "$lanip" == "169.254.11.22" ] && return
	[ -n "$lanip" ] && add_wg_service "$lanip" 
}


route_table_init()
{
	ret=`ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		iptables -w -t mangle -F ${WIREGUARD_FORWARD_TABLE}
		iptables -w -t mangle -F ${WIREGUARD_ROUTING_TABLE}
		ip rule add pref $VPN_RULE_NUM fwmark ${WIREGUARD_FWMARK}/${WIREGUARD_FWMARK} table $VPN_RULE_NUM
	fi

	ret=`vnete ip rule list | grep $VPN_RULE_NUM:`
	if [ -z "$ret" ]; then
		vnete ip rule add pref $VPN_RULE_NUM table $VPN_RULE_NUM
	fi
	if ! iptables -w -t nat -C POSTROUTING -j postrouting_rule 2>/dev/null; then
		iptables -w -t nat -N  postrouting_rule 
		iptables -w -t nat -I POSTROUTING -j postrouting_rule 
	fi
	if ! iptables -w -t nat -C postrouting_rule -j wireguard_postrouting_rule 2>/dev/null; then
		iptables -w -t nat -A postrouting_rule -j wireguard_postrouting_rule
	fi
}

wireguard_up_hook()
{	
	#CHAIN:manage_devs
	fw_event_manage_dev "${IFNAME}:up"
	
	iptables -w -t mangle -A ${WIREGUARD_ROUTING_TABLE} -i "${IFNAME}"  -m set --match-set $(zone_allwan_group) dst -m comment --comment "if=${IFNAME}" -j CONNMARK --or-mark ${WIREGUARD_FWMARK}
	iptables -w -t mangle -A ${WIREGUARD_FORWARD_TABLE} -m set --match-set $(zone_allwan_group) src -m conntrack --ctdir REPLY -m connmark --mark ${WIREGUARD_FWMARK}/${WIREGUARD_FWMARK} -m comment --comment "if=${IFNAME}"  -j MARK --or-mark ${WIREGUARD_FWMARK}
	iptables -w -t nat -A wireguard_prerouting_rule  -p udp -m udp --dport "${LISTEN_PORT}" -m comment --comment "if=${IFNAME}" -j ACCEPT
	iptables -w  -A wireguard_forwarding_rule  -p udp -m udp --dport "${LISTEN_PORT}" -m comment --comment "if=${IFNAME}" -j ACCEPT
	iptables -w -t mangle -I ${WIREGUARD_OUTPUT_TABLE} -p udp -m udp --sport "${LISTEN_PORT}" -m comment --comment "if=${IFNAME}" -j RETURN
	iptables -w -t filter -A ${WIREGUARD_INPUT_RULE} -i "${IFNAME}" -m set --match-set VMe dst -m comment --comment "if=${IFNAME}" -j ACCEPT
    iptables -w -t filter -A input_rule -p udp --dport "${LISTEN_PORT}" -m comment --comment "if=${IFNAME}" -j ACCEPT

	local chain=zone_vpn_${IFNAME}_MSSFIX
	local cur_mss=$(($WG_MTU-40))
	local rule="-p tcp -m tcp --tcp-flags SYN,RST SYN"
	local target_size="TCPMSS --set-mss ${cur_mss}"
	local target_mtu="TCPMSS --clamp-mss-to-pmtu"
	fw add 4 m ${chain}
	fw add 4 m zone_MSSFIX ${chain} ^ { -o ${IFNAME} }
	fw add 4 m zone_MSSFIX ${chain} ^ { -i ${IFNAME} }
	fw add 4 m ${chain} "$target_size" $ { "$rule" }
	fw add 4 m ${chain} "$target_mtu" $ { "$rule" }

	##add interface into dnsproxy
	[ -n "$IFNAME" ] && /lib/dnsproxy/api.sh add "$IFNAME"
	do_service_wg_up

	/lib/nat/nat_dmz_bypass.sh add udp $LISTEN_PORT all_wan
}

peer_up_hook()
{	
	#FORWORDING

	fw add 4 f wireguard_forwarding_rule ACCEPT { "-i br-lan -o $IFNAME -d $ALLOW_IP" }
	fw add 4 f wireguard_forwarding_rule ACCEPT { "-i $IFNAME -o br-lan -s $ALLOW_IP" }

	#fw add 4 n openvpn_postrouting_rule MASQUERADE { "-d $ALLOW_IP -o $IFNAME" }
#	export > /dev/console

	if [ "${ALLOW_IP}" != "0.0.0.0/0" ]; then
		iptables -w -t nat -A wireguard_postrouting_rule  -o br-lan -s ${ALLOW_IP} -m comment --comment "if=${IFNAME}" -j RETURN
		iptables -w -t nat -A wireguard_postrouting_rule  ! -o ${IFNAME} -s ${ALLOW_IP} -m comment --comment "if=${IFNAME}" -j MASQUERADE
		#ip route add  ${ALLOW_IP} dev ${IFNAME}
	else
		iptables -w -t nat -A wireguard_postrouting_rule  -o br-lan  -m comment --comment "if=${IFNAME}" -j RETURN
		iptables -w -t nat -A wireguard_postrouting_rule  -o ${IFNAME}  -m comment --comment "if=${IFNAME}" -j MASQUERADE
	fi

	iptables -w -t mangle -A ${WIREGUARD_FORWARD_TABLE} -i br-lan  -d ${ALLOW_IP} -m comment --comment "if=${IFNAME}" -j MARK --or-mark ${WIREGUARD_FWMARK}
	iptables -w -t mangle -A ${WIREGUARD_FORWARD_TABLE} -i br-lan  -d ${ALLOW_IP} -m comment --comment "if=${IFNAME}" -j CONNMARK --or-mark ${WIREGUARD_FWMARK}

}

route_table_init
if [ "${METHOD}" = "interface" ]; then
    wireguard_up_hook
fi
if [ "${METHOD}" = "peer" ]; then
    peer_up_hook
fi
