--[[
Author: Xie zhiyong
Date: 2021-01-11 20:04:31
LastEditTime: 2022-01-18 20:45:48
LastEditors: Please set LastEditors
Description: In User Settings Edit
--]]
local uci    = require "luci.model.uci"
local dbg    = require "luci.tools.debug"
local sys    = require "luci.sys"
local socket = require "socket"
local err = require("luci.tools.error")
local form = require "luci.tools.form"
local uci_r = uci.cursor()
local ip = require("luci.ip")
form = form.Form(uci_r, {"name"})

WG_CONFIG_PATH = "/etc/wireguard/"
WG_CONFIG = "wireguard_interface"
WG_TYPE = "wg"
PEER_CONFIG = "wireguard_peers"
PEER_TYPE = "peer"
function get_name_from_id(config,config_type,id)
	local ret
	uci_r:foreach(config,config_type,
		function(section)
	
			if section.id == tostring(id) then

				ret=section[".name"]

				return
				
			end
		end
	)
	return ret
   

end
--example wg config file
--[[
[Interface]
PrivateKey = eHif98RgFrBj2LQzl0+xfvcn9mR74xlRcUlspxcIpls=
ListenPort = 51820
Address = 192.168.0.78/32

[Peer]
PublicKey = b+ue+VBkDu6Lj3pUmUlE9IYjpxRd7k8Y6i5uRLnDUAg=
AllowedIPs = 192.168.1.1/24
Endpoint = 10.10.10.20:51820
PersistentKeepalive = 25
]]
function generate_wg_conf(secname)
    dbg("WG_CONFIG_PATH..secname "..WG_CONFIG_PATH..secname)
    local wg_conf_file = io.open(WG_CONFIG_PATH..secname..".conf" ,"w+");
    local wg_conf = uci_r:get_all(WG_CONFIG,secname)
    wg_conf_file:write("[Interface] \n")
    wg_conf_file:write("PrivateKey = "..wg_conf.private_key.."\n")
    wg_conf_file:write("ListenPort = "..wg_conf.listen_port.."\n")
    
    uci_r:foreach(PEER_CONFIG,PEER_TYPE,
        function(section)
        
            if section.interface == wg_conf.id and section.status=="on" then
                wg_conf_file:write("[Peer] \n")

                if section.endpoint_port and section.endpoint then
                    wg_conf_file:write("Endpoint = "..section.endpoint..":"..section.endpoint_port .."\n")
                end

                if section.public_key  then
                    wg_conf_file:write("PublicKey = "..section.public_key.."\n")
                end

                if section.allowed_address  then
                    section.allowed_address = (type( section.allowed_address)=="table") and  section.allowed_address or { section.allowed_address}
                    wg_conf_file:write("AllowedIPs = ")
                    for i,j in ipairs(section.allowed_address) do
                        if i ~= #section.allowed_address then
                            wg_conf_file:write(j..", ")
                        else
                            wg_conf_file:write(j)
                        end

                    end
                    wg_conf_file:write("\n")
                end

                if section.alive_time  then
                    wg_conf_file:write("PersistentKeepalive  = "..section.alive_time.."\n")
                end

                if section.preshared_key then
                    wg_conf_file:write("PresharedKey   = "..section.preshared_key.."\n")
                end

                
                
            end
        end
    )
    wg_conf_file:close()


end

function generate_allwg_conf()
    uci_r:foreach(WG_CONFIG,WG_TYPE,

        function(section)
            generate_wg_conf(section[".name"])
        end

    )
end
--ip link add dev wg${s} type wireguard
--ip address add dev wg0 192.168.10.1/32
--wg setconf wg0 /usr/sbin/wg0.conf 
--ip link set up dev wg0
function wireguard_interface_up(wg_name)
    dbg("wgup"..wg_name)
    local wg_conf = uci_r:get_all(WG_CONFIG,wg_name)
    if wg_conf.status == "off" then
       return 
    end
    local local_ip_address = uci_r:get(WG_CONFIG,wg_name,"local_ipaddr")
    local mtu = uci_r:get(WG_CONFIG,wg_name,"mtu")
    sys.fork_call("ip link add dev "..wg_name.." type wireguard")
    sys.fork_call("ip address add dev "..wg_name.." "..local_ip_address.."/32")
    sys.fork_call("wg syncconf "..wg_name.." "..WG_CONFIG_PATH..wg_name..".conf")
    sys.fork_call("ip link set up dev "..wg_name.." mtu "..mtu)
    sys.fork_call("/lib/wireguard/wireguard-up.sh ".." interface "..wg_name.." "..wg_conf.listen_port.." "..mtu)
    uci_r:foreach(PEER_CONFIG,PEER_TYPE,
        function(section)
        
            if section.interface == wg_conf.id and section.status=="on" then
                if section.allowed_address  then
                    for i,j in ipairs(section.allowed_address) do
                        sys.fork_call("/lib/wireguard/wireguard-up.sh ".." peer "..wg_name.." "..j)
                    end
                end
                
            end
        end
    )  

end
function wireguard_interface_down(wg_name)

    local wg_conf = uci_r:get_all(WG_CONFIG,wg_name)
    sys.fork_call("ip link del dev "..wg_name.." type wireguard")
    sys.fork_call("rm -f "..WG_CONFIG_PATH..wg_name..".conf")
    

    sys.fork_call("/lib/wireguard/wireguard-down.sh interface "..wg_name)


end
function wireguard_peer_up(wg_name,peer_name)

    local wg_conf = uci_r:get_all(WG_CONFIG,wg_name)
    if wg_conf.status == "off" then
       return 
    end
    
    sys.fork_call("wg syncconf "..wg_name.." "..WG_CONFIG_PATH..wg_name..".conf")

    local peer_conf = uci_r:get_all(PEER_CONFIG,peer_name)
    if peer_conf.status == "off" then
        return 
     end
    if peer_conf.allowed_address then
        for i,j in ipairs(peer_conf.allowed_address) do
            sys.fork_call("/lib/wireguard/wireguard-up.sh ".." peer "..wg_name.." "..j)

        end
    end


end
function wireguard_peer_down(wg_name,peer_name)

    local wg_conf = uci_r:get_all(WG_CONFIG,wg_name)
    if wg_conf.status == "off" then
       return 
    end
    sys.fork_call("wg syncconf "..wg_name.." "..WG_CONFIG_PATH..wg_name..".conf")

    local peer_conf = uci_r:get_all(PEER_CONFIG,peer_name)
    if peer_conf.allowed_address then
        for i,j in ipairs(peer_conf.allowed_address) do
            sys.fork_call("/lib/wireguard/wireguard-down.sh ".." peer "..wg_name.." "..j)

        end
    end


end
function wg_init_start()
    generate_allwg_conf()
    uci_r:foreach(WG_CONFIG,WG_TYPE,
    function(section)
    
        if section.status=="on" then
      
            wireguard_interface_up(section[".name"])

        end

    end
    )
end

function wg_init_stop()
    generate_allwg_conf()
    uci_r:foreach(WG_CONFIG,WG_TYPE,
    function(section)
    
        if section.status=="on" then
      
            wireguard_interface_down(section[".name"])

        end

    end
    )
end

function check_key(key)

    local cmd = "echo \""..key.."\" |wg pubkey"
    dbg("cmd"..cmd)
    local wg_status = sys.fork_call(cmd)
    dbg("wg_status "..wg_status)

    return 
end
local arg_method = arg[1] 
local secname = arg[2] 
local peer_name = arg[3]
if arg_method == "generate_conf" then 
    dbg("gen conf")
    generate_wg_conf(secname)
end
if arg_method == "wg_up" then
    generate_wg_conf(secname) 
    wireguard_interface_up(secname)
end
if arg_method == "wg_down" then 
    generate_wg_conf(secname) 
    wireguard_interface_down(secname)
end
if arg_method == "peer_up" then 
    generate_wg_conf(secname) 
    wireguard_peer_up(secname,peer_name)
end
if arg_method == "peer_down" then 
    generate_wg_conf(secname) 
    wireguard_peer_down(secname,peer_name)
end
if arg_method == "wg_reload" then 
    wireguard_interface_down(secname)
    generate_wg_conf(secname)
    wireguard_interface_up(secname)
end
if arg_method == "start" then 
    wg_init_start()
end
if arg_method == "stop" then 
    wg_init_stop()
end
if arg_method == "check_key" then 
    check_key(secname)
end