/** @file      pace2_applications.h
  * @brief     PACE 2 application definitions
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_APPLICATIONS_H
#define PACE2_APPLICATIONS_H

/* Includes *********************************************************************/
#include "pace2_basic_def.h"
#include "pace2_attributes_bitmask.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Public Types *****************************************************************/

/**
 * @brief Enum defining all available applications.
**/

typedef enum pace2_applications {
    PACE2_APPLICATION_UNKNOWN = 0,
    PACE2_APPLICATION_FACEBOOK,
    PACE2_APPLICATION_MAGICJACK,
    PACE2_APPLICATION_ITUNES,
    PACE2_APPLICATION_MYSPACE,
    PACE2_APPLICATION_FACETIME,
    PACE2_APPLICATION_TRUPHONE,
    PACE2_APPLICATION_TWITTER,
    PACE2_APPLICATION_XBOX,
    PACE2_APPLICATION_RES_9,
    PACE2_APPLICATION_GOOGLE_MAIL,
    PACE2_APPLICATION_RES_11,
    PACE2_APPLICATION_ICLOUD,
    PACE2_APPLICATION_RES_13,
    PACE2_APPLICATION_DROPBOX,
    PACE2_APPLICATION_RES_15,
    PACE2_APPLICATION_IMESSAGE,
    PACE2_APPLICATION_GOTOMYPC,
    PACE2_APPLICATION_GOTOMEETING,
    PACE2_APPLICATION_RES_19,
    PACE2_APPLICATION_AMAZON_CLOUD,
    PACE2_APPLICATION_DAILYMOTION,
    PACE2_APPLICATION_DEEZER,
    PACE2_APPLICATION_RES_23,
    PACE2_APPLICATION_RES_24,
    PACE2_APPLICATION_OFFICE365,
    PACE2_APPLICATION_CNTV,
    PACE2_APPLICATION_RES_27,
    PACE2_APPLICATION_YOUTUBE,
    PACE2_APPLICATION_VOIP_SWITCH,
    PACE2_APPLICATION_ZYNGA,
    PACE2_APPLICATION_CRIMECITY,
    PACE2_APPLICATION_MODERNWAR,
    PACE2_APPLICATION_VIPPIE,
    PACE2_APPLICATION_BOX,
    PACE2_APPLICATION_RES_35,
    PACE2_APPLICATION_ADOBE_CREATIVE_CLOUD,
    PACE2_APPLICATION_LINKEDIN,
    PACE2_APPLICATION_RES_38,
    PACE2_APPLICATION_HI5,
    PACE2_APPLICATION_SORIBADA,
    PACE2_APPLICATION_ZOHO_WORK_ONLINE,
    PACE2_APPLICATION_RES_42,
    PACE2_APPLICATION_INSTAGRAM,
    PACE2_APPLICATION_SUGARSYNC,
    PACE2_APPLICATION_WICKR,
    PACE2_APPLICATION_RES_46,
    PACE2_APPLICATION_RES_47,
    PACE2_APPLICATION_RES_48,
    PACE2_APPLICATION_VOXER,
    PACE2_APPLICATION_RES_50,
    PACE2_APPLICATION_RES_51,
    PACE2_APPLICATION_BURNER,
    PACE2_APPLICATION_FLICKR,
    PACE2_APPLICATION_RES_54,
    PACE2_APPLICATION_RES_55,
    PACE2_APPLICATION_REDTUBE,
    PACE2_APPLICATION_EVERNOTE,
    PACE2_APPLICATION_SPREED,
    PACE2_APPLICATION_TEXTPLUS,
    PACE2_APPLICATION_XVIDEOS,
    PACE2_APPLICATION_SNAPCHAT,
    PACE2_APPLICATION_RES_62,
    PACE2_APPLICATION_MYMUSIC_TW,
    PACE2_APPLICATION_RENREN,
    PACE2_APPLICATION_YELP,
    PACE2_APPLICATION_AMAZON_SHOP,
    PACE2_APPLICATION_YOUPORN,
    PACE2_APPLICATION_EBAY,
    PACE2_APPLICATION_PORNHUB,
    PACE2_APPLICATION_SPIDEROAK,
    PACE2_APPLICATION_ADRIVE,
    PACE2_APPLICATION_TUMBLR,
    PACE2_APPLICATION_XING,
    PACE2_APPLICATION_GOOGLE_PLAY_MUSIC,
    PACE2_APPLICATION_VIMEO,
    PACE2_APPLICATION_SALESFORCE,
    PACE2_APPLICATION_RES_77,
    PACE2_APPLICATION_SINA_WEIBO,
    PACE2_APPLICATION_RES_79,
    PACE2_APPLICATION_LIVEJASMIN,
    PACE2_APPLICATION_JINGDONG,
    PACE2_APPLICATION_LASTFM,
    PACE2_APPLICATION_IMS,
    PACE2_APPLICATION_BAIDU,
    PACE2_APPLICATION_TAOBAO,
    PACE2_APPLICATION_RES_86,
    PACE2_APPLICATION_TTPOD,
    PACE2_APPLICATION_RES_88,
    PACE2_APPLICATION_PINTEREST,
    PACE2_APPLICATION_RES_90,
    PACE2_APPLICATION_NET2PHONE,
    PACE2_APPLICATION_FOURSQUARE,
    PACE2_APPLICATION_RES_93,
    PACE2_APPLICATION_MEGA,
    PACE2_APPLICATION_RES_95,
    PACE2_APPLICATION_VBULLETIN,
    PACE2_APPLICATION_CLOUDME,
    PACE2_APPLICATION_YIKYAK,
    PACE2_APPLICATION_REDMINE,
    PACE2_APPLICATION_IHEART_RADIO,
    PACE2_APPLICATION_AVIRA_ANTIVIRUS,
    PACE2_APPLICATION_RES_102,
    PACE2_APPLICATION_KASPERSKY_ANTIVIRUS,
    PACE2_APPLICATION_AVG_ANTIVIRUS,
    PACE2_APPLICATION_SOUNDCLOUD,
    PACE2_APPLICATION_RES_106,
    PACE2_APPLICATION_AMAZON_PRIME_MUSIC,
    PACE2_APPLICATION_RUNKEEPER,
    PACE2_APPLICATION_MYBOOK,
    PACE2_APPLICATION_NIKEPLUS,
    PACE2_APPLICATION_MAGV,
    PACE2_APPLICATION_MAGV_KIDS,
    PACE2_APPLICATION_RES_113,
    PACE2_APPLICATION_LETV,
    PACE2_APPLICATION_MY_FITNESS_PAL,
    PACE2_APPLICATION_SOHU,
    PACE2_APPLICATION_1KXUN,
    PACE2_APPLICATION_TVBS,
    PACE2_APPLICATION_XNXX,
    PACE2_APPLICATION_CTITV,
    PACE2_APPLICATION_RES_121,
    PACE2_APPLICATION_RES_122,
    PACE2_APPLICATION_WORKOUT_TRAINER,
    PACE2_APPLICATION_RES_124,
    PACE2_APPLICATION_RES_125,
    PACE2_APPLICATION_RES_126,
    PACE2_APPLICATION_BITTORRENT,
    PACE2_APPLICATION_EDONKEY,
    PACE2_APPLICATION_RES_129,
    PACE2_APPLICATION_GNUTELLA,
    PACE2_APPLICATION_DIRECTCONNECT,
    PACE2_APPLICATION_APPLEJUICE,
    PACE2_APPLICATION_SOULSEEK,
    PACE2_APPLICATION_ARES,
    PACE2_APPLICATION_FREENET,
    PACE2_APPLICATION_RES_136,
    PACE2_APPLICATION_SKYPE,
    PACE2_APPLICATION_RES_138,
    PACE2_APPLICATION_YAHOO,
    PACE2_APPLICATION_VONAGE,
    PACE2_APPLICATION_RES_141,
    PACE2_APPLICATION_GADUGADU,
    PACE2_APPLICATION_MPLUS,
    PACE2_APPLICATION_RES_144,
    PACE2_APPLICATION_RES_145,
    PACE2_APPLICATION_RES_146,
    PACE2_APPLICATION_DDL_4SHARED_COM,
    PACE2_APPLICATION_RES_148,
    PACE2_APPLICATION_RES_149,
    PACE2_APPLICATION_RES_150,
    PACE2_APPLICATION_RES_151,
    PACE2_APPLICATION_RES_152,
    PACE2_APPLICATION_RES_153,
    PACE2_APPLICATION_DDL_DATA_HU,
    PACE2_APPLICATION_DDL_DEPOSITFILES_COM,
    PACE2_APPLICATION_RES_156,
    PACE2_APPLICATION_RES_157,
    PACE2_APPLICATION_RES_158,
    PACE2_APPLICATION_DDL_FILE_UPLOAD_NET,
    PACE2_APPLICATION_RES_160,
    PACE2_APPLICATION_DDL_FILEARCHIV_RU,
    PACE2_APPLICATION_RES_162,
    PACE2_APPLICATION_RES_163,
    PACE2_APPLICATION_DDL_FILEFACTORY_COM,
    PACE2_APPLICATION_RES_165,
    PACE2_APPLICATION_RES_166,
    PACE2_APPLICATION_RES_167,
    PACE2_APPLICATION_RES_168,
    PACE2_APPLICATION_DDL_FILER_NET,
    PACE2_APPLICATION_RES_170,
    PACE2_APPLICATION_RES_171,
    PACE2_APPLICATION_RES_172,
    PACE2_APPLICATION_RES_173,
    PACE2_APPLICATION_RES_174,
    PACE2_APPLICATION_RES_175,
    PACE2_APPLICATION_DDL_GIGASIZE_COM,
    PACE2_APPLICATION_RES_177,
    PACE2_APPLICATION_RES_178,
    PACE2_APPLICATION_RES_179,
    PACE2_APPLICATION_RES_180,
    PACE2_APPLICATION_RES_181,
    PACE2_APPLICATION_DDL_LETECKAPOSTA_CZ,
    PACE2_APPLICATION_RES_183,
    PACE2_APPLICATION_RES_184,
    PACE2_APPLICATION_DDL_LOAD_TO,
    PACE2_APPLICATION_RES_186,
    PACE2_APPLICATION_DDL_MEDIAFIRE_COM,
    PACE2_APPLICATION_RES_188,
    PACE2_APPLICATION_DDL_MEGASHARE_COM,
    PACE2_APPLICATION_RES_190,
    PACE2_APPLICATION_RES_191,
    PACE2_APPLICATION_RES_192,
    PACE2_APPLICATION_RES_193,
    PACE2_APPLICATION_RES_194,
    PACE2_APPLICATION_RES_195,
    PACE2_APPLICATION_RES_196,
    PACE2_APPLICATION_RES_197,
    PACE2_APPLICATION_RES_198,
    PACE2_APPLICATION_RES_199,
    PACE2_APPLICATION_RES_200,
    PACE2_APPLICATION_RES_201,
    PACE2_APPLICATION_RES_202,
    PACE2_APPLICATION_DDL_SENDSPACE_COM,
    PACE2_APPLICATION_RES_204,
    PACE2_APPLICATION_RES_205,
    PACE2_APPLICATION_RES_206,
    PACE2_APPLICATION_RES_207,
    PACE2_APPLICATION_RES_208,
    PACE2_APPLICATION_RES_209,
    PACE2_APPLICATION_RES_210,
    PACE2_APPLICATION_RES_211,
    PACE2_APPLICATION_RES_212,
    PACE2_APPLICATION_RES_213,
    PACE2_APPLICATION_RES_214,
    PACE2_APPLICATION_RES_215,
    PACE2_APPLICATION_RES_216,
    PACE2_APPLICATION_RES_217,
    PACE2_APPLICATION_RES_218,
    PACE2_APPLICATION_RES_219,
    PACE2_APPLICATION_RES_220,
    PACE2_APPLICATION_RES_221,
    PACE2_APPLICATION_RES_222,
    PACE2_APPLICATION_RES_223,
    PACE2_APPLICATION_RES_224,
    PACE2_APPLICATION_RES_225,
    PACE2_APPLICATION_RES_226,
    PACE2_APPLICATION_RES_227,
    PACE2_APPLICATION_RES_228,
    PACE2_APPLICATION_RES_229,
    PACE2_APPLICATION_RES_230,
    PACE2_APPLICATION_DDL_1FICHIER_COM,
    PACE2_APPLICATION_RES_232,
    PACE2_APPLICATION_RES_233,
    PACE2_APPLICATION_RES_234,
    PACE2_APPLICATION_RES_235,
    PACE2_APPLICATION_RES_236,
    PACE2_APPLICATION_RES_237,
    PACE2_APPLICATION_RES_238,
    PACE2_APPLICATION_RES_239,
    PACE2_APPLICATION_RES_240,
    PACE2_APPLICATION_RES_241,
    PACE2_APPLICATION_RES_242,
    PACE2_APPLICATION_RES_243,
    PACE2_APPLICATION_RES_244,
    PACE2_APPLICATION_RES_245,
    PACE2_APPLICATION_RES_246,
    PACE2_APPLICATION_DDL_EASYBYTEZ_COM,
    PACE2_APPLICATION_RES_248,
    PACE2_APPLICATION_RES_249,
    PACE2_APPLICATION_RES_250,
    PACE2_APPLICATION_RES_251,
    PACE2_APPLICATION_DDL_FASTFILESHARE_COM_AR,
    PACE2_APPLICATION_RES_253,
    PACE2_APPLICATION_RES_254,
    PACE2_APPLICATION_RES_255,
    PACE2_APPLICATION_RES_256,
    PACE2_APPLICATION_RES_257,
    PACE2_APPLICATION_RES_258,
    PACE2_APPLICATION_RES_259,
    PACE2_APPLICATION_RES_260,
    PACE2_APPLICATION_RES_261,
    PACE2_APPLICATION_RES_262,
    PACE2_APPLICATION_DDL_FILESMONSTER_COM,
    PACE2_APPLICATION_RES_264,
    PACE2_APPLICATION_RES_265,
    PACE2_APPLICATION_RES_266,
    PACE2_APPLICATION_DDL_FSHARE_VN,
    PACE2_APPLICATION_RES_268,
    PACE2_APPLICATION_DDL_GIGAPETA_COM,
    PACE2_APPLICATION_RES_270,
    PACE2_APPLICATION_RES_271,
    PACE2_APPLICATION_DDL_HITFILE_NET,
    PACE2_APPLICATION_DDL_HULKSHARE_COM,
    PACE2_APPLICATION_DDL_HYPERFILESHARE_COM,
    PACE2_APPLICATION_RES_275,
    PACE2_APPLICATION_RES_276,
    PACE2_APPLICATION_RES_277,
    PACE2_APPLICATION_RES_278,
    PACE2_APPLICATION_RES_279,
    PACE2_APPLICATION_RES_280,
    PACE2_APPLICATION_RES_281,
    PACE2_APPLICATION_RES_282,
    PACE2_APPLICATION_RES_283,
    PACE2_APPLICATION_DDL_MULTISHARE_CZ,
    PACE2_APPLICATION_RES_285,
    PACE2_APPLICATION_RES_286,
    PACE2_APPLICATION_RES_287,
    PACE2_APPLICATION_RES_288,
    PACE2_APPLICATION_DDL_QSHARE_COM,
    PACE2_APPLICATION_RES_290,
    PACE2_APPLICATION_RES_291,
    PACE2_APPLICATION_DDL_SCRIBD_COM,
    PACE2_APPLICATION_RES_293,
    PACE2_APPLICATION_RES_294,
    PACE2_APPLICATION_RES_295,
    PACE2_APPLICATION_RES_296,
    PACE2_APPLICATION_RES_297,
    PACE2_APPLICATION_RES_298,
    PACE2_APPLICATION_DDL_TURBOBIT_NET,
    PACE2_APPLICATION_RES_300,
    PACE2_APPLICATION_DDL_ULOZ_TO,
    PACE2_APPLICATION_RES_302,
    PACE2_APPLICATION_RES_303,
    PACE2_APPLICATION_DDL_UP_4SHARE_VN,
    PACE2_APPLICATION_RES_305,
    PACE2_APPLICATION_RES_306,
    PACE2_APPLICATION_RES_307,
    PACE2_APPLICATION_RES_308,
    PACE2_APPLICATION_RES_309,
    PACE2_APPLICATION_RES_310,
    PACE2_APPLICATION_RES_311,
    PACE2_APPLICATION_RES_312,
    PACE2_APPLICATION_RES_313,
    PACE2_APPLICATION_RES_314,
    PACE2_APPLICATION_RES_315,
    PACE2_APPLICATION_RES_316,
    PACE2_APPLICATION_DDL_ZIDDU_COM,
    PACE2_APPLICATION_RES_318,
    PACE2_APPLICATION_RES_319,
    PACE2_APPLICATION_RES_320,
    PACE2_APPLICATION_RES_321,
    PACE2_APPLICATION_RES_322,
    PACE2_APPLICATION_DDL_2SHARED_COM,
    PACE2_APPLICATION_RES_324,
    PACE2_APPLICATION_RES_325,
    PACE2_APPLICATION_RES_326,
    PACE2_APPLICATION_RES_327,
    PACE2_APPLICATION_RES_328,
    PACE2_APPLICATION_RES_329,
    PACE2_APPLICATION_RES_330,
    PACE2_APPLICATION_RES_331,
    PACE2_APPLICATION_RES_332,
    PACE2_APPLICATION_RES_333,
    PACE2_APPLICATION_RES_334,
    PACE2_APPLICATION_RES_335,
    PACE2_APPLICATION_RES_336,
    PACE2_APPLICATION_RES_337,
    PACE2_APPLICATION_RES_338,
    PACE2_APPLICATION_RES_339,
    PACE2_APPLICATION_RES_340,
    PACE2_APPLICATION_RES_341,
    PACE2_APPLICATION_RES_342,
    PACE2_APPLICATION_RES_343,
    PACE2_APPLICATION_RES_344,
    PACE2_APPLICATION_RES_345,
    PACE2_APPLICATION_RES_346,
    PACE2_APPLICATION_RES_347,
    PACE2_APPLICATION_RES_348,
    PACE2_APPLICATION_RES_349,
    PACE2_APPLICATION_RES_350,
    PACE2_APPLICATION_RES_351,
    PACE2_APPLICATION_DDL_UPTOBOX_COM,
    PACE2_APPLICATION_RES_353,
    PACE2_APPLICATION_DDL_GAMEFRONT_COM,
    PACE2_APPLICATION_RES_355,
    PACE2_APPLICATION_RES_356,
    PACE2_APPLICATION_DDL_TURBOBIT_RU,
    PACE2_APPLICATION_USENET,
    PACE2_APPLICATION_RES_359,
    PACE2_APPLICATION_FILETOPIA,
    PACE2_APPLICATION_RES_361,
    PACE2_APPLICATION_IMESH,
    PACE2_APPLICATION_RES_363,
    PACE2_APPLICATION_RES_364,
    PACE2_APPLICATION_RES_365,
    PACE2_APPLICATION_RES_366,
    PACE2_APPLICATION_RES_367,
    PACE2_APPLICATION_ZATTOO,
    PACE2_APPLICATION_QQ,
    PACE2_APPLICATION_QQLIVE,
    PACE2_APPLICATION_RES_371,
    PACE2_APPLICATION_POPO,
    PACE2_APPLICATION_THUNDER,
    PACE2_APPLICATION_UUSEE,
    PACE2_APPLICATION_VNC,
    PACE2_APPLICATION_STEAM,
    PACE2_APPLICATION_HALFLIFE2,
    PACE2_APPLICATION_SOFTETHER,
    PACE2_APPLICATION_TELNET,
    PACE2_APPLICATION_OFF,
    PACE2_APPLICATION_TOR,
    PACE2_APPLICATION_WORLDOFWARCRAFT,
    PACE2_APPLICATION_HAMACHI_VPN,
    PACE2_APPLICATION_SKINNY,
    PACE2_APPLICATION_CITRIX,
    PACE2_APPLICATION_PALTALK,
    PACE2_APPLICATION_QUAKE,
    PACE2_APPLICATION_SECONDLIFE,
    PACE2_APPLICATION_BATTLEFIELD,
    PACE2_APPLICATION_RES_390,
    PACE2_APPLICATION_ICECAST,
    PACE2_APPLICATION_SHOUTCAST,
    PACE2_APPLICATION_QQGAME,
    PACE2_APPLICATION_VEOHTV,
    PACE2_APPLICATION_RES_395,
    PACE2_APPLICATION_PANDORA,
    PACE2_APPLICATION_IPP,
    PACE2_APPLICATION_WARCRAFT3,
    PACE2_APPLICATION_RES_399,
    PACE2_APPLICATION_RES_400,
    PACE2_APPLICATION_RES_401,
    PACE2_APPLICATION_RES_402,
    PACE2_APPLICATION_IPTV,
    PACE2_APPLICATION_YOURFREEDOM,
    PACE2_APPLICATION_VOIPSWITCH_VOIPTUNNEL,
    PACE2_APPLICATION_RES_406,
    PACE2_APPLICATION_FUNSHION,
    PACE2_APPLICATION_AIMINI,
    PACE2_APPLICATION_RES_409,
    PACE2_APPLICATION_FLORENSIA,
    PACE2_APPLICATION_TEAMSPEAK,
    PACE2_APPLICATION_MAPLESTORY,
    PACE2_APPLICATION_RES_413,
    PACE2_APPLICATION_RES_414,
    PACE2_APPLICATION_DOFUS,
    PACE2_APPLICATION_FIESTA,
    PACE2_APPLICATION_SPLASHFIGHTER,
    PACE2_APPLICATION_ACTIVESYNC,
    PACE2_APPLICATION_RES_419,
    PACE2_APPLICATION_CROSSFIRE,
    PACE2_APPLICATION_RES_421,
    PACE2_APPLICATION_GUILDWARS,
    PACE2_APPLICATION_ARMAGETRON,
    PACE2_APPLICATION_RFACTOR,
    PACE2_APPLICATION_TEAMVIEWER,
    PACE2_APPLICATION_GAMEKIT,
    PACE2_APPLICATION_ULTRABAC,
    PACE2_APPLICATION_SPOTIFY,
    PACE2_APPLICATION_IMPLUS,
    PACE2_APPLICATION_RES_430,
    PACE2_APPLICATION_ULTRASURF,
    PACE2_APPLICATION_VIBER,
    PACE2_APPLICATION_TANGO,
    PACE2_APPLICATION_IPLAYER,
    PACE2_APPLICATION_OPERAMINI,
    PACE2_APPLICATION_BOLT,
    PACE2_APPLICATION_RES_437,
    PACE2_APPLICATION_WHATSAPP,
    PACE2_APPLICATION_WEBEX,
    PACE2_APPLICATION_NETFLIX,
    PACE2_APPLICATION_RES_441,
    PACE2_APPLICATION_IMO,
    PACE2_APPLICATION_CITRIX_GOTO,
    PACE2_APPLICATION_RES_444,
    PACE2_APPLICATION_RES_445,
    PACE2_APPLICATION_VENTRILO,
    PACE2_APPLICATION_RES_447,
    PACE2_APPLICATION_EBUDDY,
    PACE2_APPLICATION_ADOBE_CONNECT,
    PACE2_APPLICATION_RES_450,
    PACE2_APPLICATION_RES_451,
    PACE2_APPLICATION_WEBDAV,
    PACE2_APPLICATION_RES_453,
    PACE2_APPLICATION_RES_454,
    PACE2_APPLICATION_RES_455,
    PACE2_APPLICATION_RES_456,
    PACE2_APPLICATION_RES_457,
    PACE2_APPLICATION_RES_458,
    PACE2_APPLICATION_PDPROXY,
    PACE2_APPLICATION_CYBERGHOST,
    PACE2_APPLICATION_GOOGLE_CLOUD,
    PACE2_APPLICATION_GOOGLE_DRIVE,
    PACE2_APPLICATION_GOOGLE_DOCS,
    PACE2_APPLICATION_RES_464,
    PACE2_APPLICATION_ORACLE_DB,
    PACE2_APPLICATION_ITV,
    PACE2_APPLICATION_RES_467,
    PACE2_APPLICATION_RES_468,
    PACE2_APPLICATION_CHANNEL_4OD,
    PACE2_APPLICATION_RES_470,
    PACE2_APPLICATION_POISON_IVY,
    PACE2_APPLICATION_HIDRIVE,
    PACE2_APPLICATION_WECHAT,
    PACE2_APPLICATION_RES_474,
    PACE2_APPLICATION_KIK_MESSENGER,
    PACE2_APPLICATION_LINE,
    PACE2_APPLICATION_IPERF,
    PACE2_APPLICATION_OS_UPDATE,
    PACE2_APPLICATION_HIKE_MESSENGER,
    PACE2_APPLICATION_RES_480,
    PACE2_APPLICATION_IDRIVE,
    PACE2_APPLICATION_RES_482,
    PACE2_APPLICATION_RES_483,
    PACE2_APPLICATION_DOTA2,
    PACE2_APPLICATION_QVOD,
    PACE2_APPLICATION_HULU,
    PACE2_APPLICATION_MSEXCHANGE,
    PACE2_APPLICATION_MSNAV,
    PACE2_APPLICATION_FUZE,
    PACE2_APPLICATION_KAKAOTALK,
    PACE2_APPLICATION_DESTINY,
    PACE2_APPLICATION_VYPR_VPN,
    PACE2_APPLICATION_THREEMA,
    PACE2_APPLICATION_RES_494,
    PACE2_APPLICATION_TWITCH,
    PACE2_APPLICATION_WOWZA,
    PACE2_APPLICATION_RUNTASTIC,
    PACE2_APPLICATION_TELEGRAM,
    PACE2_APPLICATION_STREETVOICE,
    PACE2_APPLICATION_DUOKAN,
    PACE2_APPLICATION_RES_501,
    PACE2_APPLICATION_SKOUT,
    PACE2_APPLICATION_RES_503,
    PACE2_APPLICATION_GAME_OF_WAR,
    PACE2_APPLICATION_FETNET,
    PACE2_APPLICATION_FBS,
    PACE2_APPLICATION_RES_507,
    PACE2_APPLICATION_PINKFONG,
    PACE2_APPLICATION_KOLLECT_BOOK,
    PACE2_APPLICATION_SPTG_TV,
    PACE2_APPLICATION_RES_511,
    PACE2_APPLICATION_RES_512,
    PACE2_APPLICATION_RES_513,
    PACE2_APPLICATION_RES_514,
    PACE2_APPLICATION_AOWE,
    PACE2_APPLICATION_IWOW_SYSTEX,
    PACE2_APPLICATION_9KU,
    PACE2_APPLICATION_NEXTTV,
    PACE2_APPLICATION_SYNCPLICITY,
    PACE2_APPLICATION_RES_520,
    PACE2_APPLICATION_YUANTA_WEB,
    PACE2_APPLICATION_HINET,
    PACE2_APPLICATION_MYCARD,
    PACE2_APPLICATION_KONO,
    PACE2_APPLICATION_RES_525,
    PACE2_APPLICATION_DUO_DUO,
    PACE2_APPLICATION_WEALTH_GOD,
    PACE2_APPLICATION_ILEADER,
    PACE2_APPLICATION_THREE_BAMBOO,
    PACE2_APPLICATION_TAAZE,
    PACE2_APPLICATION_AUDIKO,
    PACE2_APPLICATION_PUBU_BOOKBUFFET,
    PACE2_APPLICATION_MITAKE,
    PACE2_APPLICATION_HYREAD,
    PACE2_APPLICATION_RES_535,
    PACE2_APPLICATION_QUALYS,
    PACE2_APPLICATION_CONSTANT_CONTACT,
    PACE2_APPLICATION_RALLY,
    PACE2_APPLICATION_RES_539,
    PACE2_APPLICATION_INTRALINKS,
    PACE2_APPLICATION_LIVEPERSON,
    PACE2_APPLICATION_SPEEDTEST,
    PACE2_APPLICATION_BLUEJEANS,
    PACE2_APPLICATION_APPLE_MUSIC,
    PACE2_APPLICATION_NAVER,
    PACE2_APPLICATION_BOOKING_COM,
    PACE2_APPLICATION_CNN,
    PACE2_APPLICATION_RES_548,
    PACE2_APPLICATION_PAYEASY,
    PACE2_APPLICATION_LINE_RANGERS,
    PACE2_APPLICATION_IOS_APP_STORE,
    PACE2_APPLICATION_MYVIDEO_TW,
    PACE2_APPLICATION_KING_GAMING,
    PACE2_APPLICATION_CANDY_CRUSH,
    PACE2_APPLICATION_CANDY_CRUSH_SODA,
    PACE2_APPLICATION_BUBBLE_WITCH,
    PACE2_APPLICATION_FARM_HERO,
    PACE2_APPLICATION_PLAYSTATION,
    PACE2_APPLICATION_GOOGLE_PLAY,
    PACE2_APPLICATION_SUPERCELL,
    PACE2_APPLICATION_CLASH_OF_CLANS,
    PACE2_APPLICATION_BOOM_BEACH,
    PACE2_APPLICATION_HAY_DAY,
    PACE2_APPLICATION_WINDOWS_STORE,
    PACE2_APPLICATION_SKYPE_FOR_BUSINESS,
    PACE2_APPLICATION_MINECRAFT,
    PACE2_APPLICATION_RES_567,
    PACE2_APPLICATION_ROVIO_GAMING,
    PACE2_APPLICATION_ANGRY_BIRDS,
    PACE2_APPLICATION_CITRIX_XENAPP,
    PACE2_APPLICATION_REDDIT,
    PACE2_APPLICATION_LEAGUE_OF_LEGENDS,
    PACE2_APPLICATION_BATTLENET,
    PACE2_APPLICATION_PAYPAL,
    PACE2_APPLICATION_PLANTS_VS_ZOMBIES_2,
    PACE2_APPLICATION_HALFBRICK_STUDIOS,
    PACE2_APPLICATION_GITHUB,
    PACE2_APPLICATION_HAMIVIDEO,
    PACE2_APPLICATION_RES_579,
    PACE2_APPLICATION_NICONICO,
    PACE2_APPLICATION_ALIPAY,
    PACE2_APPLICATION_RES_582,
    PACE2_APPLICATION_ZALO,
    PACE2_APPLICATION_NTT_DOCOMO,
    PACE2_APPLICATION_SOFTBANK_ONLINESHOP,
    PACE2_APPLICATION_GUILDWARS2,
    PACE2_APPLICATION_TRENDMICRO_WORRYFREE,
    PACE2_APPLICATION_WIKIPEDIA,
    PACE2_APPLICATION_UBER,
    PACE2_APPLICATION_YANDEX,
    PACE2_APPLICATION_VK,
    PACE2_APPLICATION_ODNOKLASSNIKI,
    PACE2_APPLICATION_RES_593,
    PACE2_APPLICATION_ALIBABA,
    PACE2_APPLICATION_ALIEXPRESS,
    PACE2_APPLICATION_TMALL,
    PACE2_APPLICATION_EA_ORIGIN,
    PACE2_APPLICATION_NORTON_SECURITY,
    PACE2_APPLICATION_TIBIA,
    PACE2_APPLICATION_POCHTA_ROSSII,
    PACE2_APPLICATION_RES_601,
    PACE2_APPLICATION_DRIVELOCK,
    PACE2_APPLICATION_RES_603,
    PACE2_APPLICATION_RES_604,
    PACE2_APPLICATION_EA_GAMES,
    PACE2_APPLICATION_RES_606,
    PACE2_APPLICATION_POPCAP,
    PACE2_APPLICATION_SURVEY_MONKEY,
    PACE2_APPLICATION_NEW_RELIC,
    PACE2_APPLICATION_HUDDLE,
    PACE2_APPLICATION_ZENDESK,
    PACE2_APPLICATION_RES_612,
    PACE2_APPLICATION_RES_613,
    PACE2_APPLICATION_DENA,
    PACE2_APPLICATION_BACKBLAZE,
    PACE2_APPLICATION_DOCUSIGN,
    PACE2_APPLICATION_RES_617,
    PACE2_APPLICATION_RES_618,
    PACE2_APPLICATION_ANGHAMI,
    PACE2_APPLICATION_PATH_OF_EXILE,
    PACE2_APPLICATION_SLACK,
    PACE2_APPLICATION_POKEMON_GO,
    PACE2_APPLICATION_NIANTIC_LABS,
    PACE2_APPLICATION_2GIS,
    PACE2_APPLICATION_RUTUBE,
    PACE2_APPLICATION_RES_626,
    PACE2_APPLICATION_PRISMA,
    PACE2_APPLICATION_RES_628,
    PACE2_APPLICATION_RES_629,
    PACE2_APPLICATION_AMAZON_PRIME_VIDEO,
    PACE2_APPLICATION_CLASH_ROYALE,
    PACE2_APPLICATION_GOOGLE_SHARED_SERVICES,
    PACE2_APPLICATION_SIGNAL,
    PACE2_APPLICATION_RES_634,
    PACE2_APPLICATION_RES_635,
    PACE2_APPLICATION_RES_636,
    PACE2_APPLICATION_RES_637,
    PACE2_APPLICATION_DDL_EDISK,
    PACE2_APPLICATION_RES_639,
    PACE2_APPLICATION_AZAR,
    PACE2_APPLICATION_SKY_GO,
    PACE2_APPLICATION_RES_642,
    PACE2_APPLICATION_MAIL_RU,
    PACE2_APPLICATION_RES_644,
    PACE2_APPLICATION_RES_645,
    PACE2_APPLICATION_KINOPOISK,
    PACE2_APPLICATION_IVI_RU,
    PACE2_APPLICATION_TINDER,
    PACE2_APPLICATION_ANYDESK,
    PACE2_APPLICATION_ICQ,
    PACE2_APPLICATION_TENOR,
    PACE2_APPLICATION_GOOGLE_MAPS,
    PACE2_APPLICATION_GOOGLE_EARTH,
    PACE2_APPLICATION_CHANNEL_ONE,
    PACE2_APPLICATION_GIPHY,
    PACE2_APPLICATION_VPN_X,
    PACE2_APPLICATION_IMGUR,
    PACE2_APPLICATION_ONEDRIVE,
    PACE2_APPLICATION_NINTENDO_NETWORK,
    PACE2_APPLICATION_AMAZON_CHIME,
    PACE2_APPLICATION_IFLIX,
    PACE2_APPLICATION_PPFILM,
    PACE2_APPLICATION_APPLE_SERVICES,
    PACE2_APPLICATION_MICROSOFT_SERVICES,
    PACE2_APPLICATION_ICFLIX,
    PACE2_APPLICATION_LOGMEIN,
    PACE2_APPLICATION_PSIPHON,
    PACE2_APPLICATION_WAZE,
    PACE2_APPLICATION_GOOGLE_FCM,
    PACE2_APPLICATION_APNS,
    PACE2_APPLICATION_DIGICEL_PLAYGO,
    PACE2_APPLICATION_SHAZAM,
    PACE2_APPLICATION_RES_673,
    PACE2_APPLICATION_DIGICEL_TOPUP,
    PACE2_APPLICATION_TUNEIN_RADIO,
    PACE2_APPLICATION_RES_676,
    PACE2_APPLICATION_SYNOLOGY,
    PACE2_APPLICATION_MY_DIGICEL,
    PACE2_APPLICATION_RES_679,
    PACE2_APPLICATION_RES_680,
    PACE2_APPLICATION_BADOO,
    PACE2_APPLICATION_AIRBNB,
    PACE2_APPLICATION_HIGHTAIL,
    PACE2_APPLICATION_EASY_TAXI,
    PACE2_APPLICATION_TOMTOM,
    PACE2_APPLICATION_TRIPADVISOR,
    PACE2_APPLICATION_ESPN,
    PACE2_APPLICATION_DAZN,
    PACE2_APPLICATION_MOBILE_STRIKE,
    PACE2_APPLICATION_KING_OF_AVALON,
    PACE2_APPLICATION_LINEAGE_2,
    PACE2_APPLICATION_LYFT,
    PACE2_APPLICATION_TRULIA,
    PACE2_APPLICATION_AKAMAI_CLOUD,
    PACE2_APPLICATION_RES_695,
    PACE2_APPLICATION_NICKELODEON_PLAY,
    PACE2_APPLICATION_HERE,
    PACE2_APPLICATION_SIMCITY_BUILDIT,
    PACE2_APPLICATION_STARWARS_GOH,
    PACE2_APPLICATION_FASTLY,
    PACE2_APPLICATION_SAP,
    PACE2_APPLICATION_WORLD_OF_KUNG_FU,
    PACE2_APPLICATION_CLOUDFLARE,
    PACE2_APPLICATION_LIFE_CHURCH,
    PACE2_APPLICATION_RES_705,
    PACE2_APPLICATION_GRINDR,
    PACE2_APPLICATION_APPLE_MAPS,
    PACE2_APPLICATION_APPLE_GEOLOCATION,
    PACE2_APPLICATION_DDL_UPLOADED,
    PACE2_APPLICATION_DDL_EASYPASTE_ORG,
    PACE2_APPLICATION_RES_711,
    PACE2_APPLICATION_YOUKU_TUDOU,
    PACE2_APPLICATION_MEETIC_GROUP,
    PACE2_APPLICATION_RES_714,
    PACE2_APPLICATION_VUDU,
    PACE2_APPLICATION_WISH,
    PACE2_APPLICATION_AUTH0,
    PACE2_APPLICATION_SLINGTV,
    PACE2_APPLICATION_GOOGLE_APIS,
    PACE2_APPLICATION_RACKSPACE_CLOUD,
    PACE2_APPLICATION_GOOGLE_CALENDAR,
    PACE2_APPLICATION_GOOGLE_ANALYTICS,
    PACE2_APPLICATION_STACKPATH_CLOUD,
    PACE2_APPLICATION_RES_724,
    PACE2_APPLICATION_LEARNINGHUB_ONLINE,
    PACE2_APPLICATION_GOOGLE_ADS,
    PACE2_APPLICATION_FOX_SPORTS,
    PACE2_APPLICATION_AMAZON_SERVICES,
    PACE2_APPLICATION_AMAZON_DRIVE,
    PACE2_APPLICATION_CRASHLYTICS,
    PACE2_APPLICATION_FACEBOOK_CLOUD,
    PACE2_APPLICATION_360TOTAL,
    PACE2_APPLICATION_TURNER,
    PACE2_APPLICATION_RES_734,
    PACE2_APPLICATION_LOOP_CARIBBEAN_LOCAL_NEWS,
    PACE2_APPLICATION_LOOP_PACIFIC_LOCAL_NEWS,
    PACE2_APPLICATION_UNITY,
    PACE2_APPLICATION_FUNPLUS,
    PACE2_APPLICATION_MAPBOX,
    PACE2_APPLICATION_CEDEXIS,
    PACE2_APPLICATION_ZENMATE_VPN,
    PACE2_APPLICATION_SMULE,
    PACE2_APPLICATION_PIA,
    PACE2_APPLICATION_RES_744,
    PACE2_APPLICATION_RES_745,
    PACE2_APPLICATION_IPVANISH,
    PACE2_APPLICATION_TUNNELBEAR,
    PACE2_APPLICATION_GOOGLE_APP_ENGINE,
    PACE2_APPLICATION_WIRE_MESSENGER,
    PACE2_APPLICATION_EPIC_BROWSER_PROXY,
    PACE2_APPLICATION_RES_751,
    PACE2_APPLICATION_RES_752,
    PACE2_APPLICATION_TIGER_VPN,
    PACE2_APPLICATION_EPIC_GAMES,
    PACE2_APPLICATION_AVIRA_PHANTOM_VPN,
    PACE2_APPLICATION_TURBO_VPN,
    PACE2_APPLICATION_IQIYI,
    PACE2_APPLICATION_RES_758,
    PACE2_APPLICATION_PPTV,
    PACE2_APPLICATION_SNAP_VPN,
    PACE2_APPLICATION_RES_761,
    PACE2_APPLICATION_VPN_UNLIMITED,
    PACE2_APPLICATION_RES_763,
    PACE2_APPLICATION_AVAST_SERVICES,
    PACE2_APPLICATION_NORD_VPN,
    PACE2_APPLICATION_PROTON_VPN,
    PACE2_APPLICATION_SAFERVPN,
    PACE2_APPLICATION_HIDE_ME_VPN,
    PACE2_APPLICATION_EXPRESSVPN,
    PACE2_APPLICATION_TUNNELGURU,
    PACE2_APPLICATION_YOUTUBE_MUSIC,
    PACE2_APPLICATION_WINDSCRIBE_VPN,
    PACE2_APPLICATION_IVPN,
    PACE2_APPLICATION_DISCORD,
    PACE2_APPLICATION_FIREVPN,
    PACE2_APPLICATION_BOTIM,
    PACE2_APPLICATION_HI_VPN,
    PACE2_APPLICATION_VPN_PROXY_MASTER,
    PACE2_APPLICATION_ZOOGVPN,
    PACE2_APPLICATION_IVACY_VPN,
    PACE2_APPLICATION_PUREVPN,
    PACE2_APPLICATION_MULLVAD_VPN,
    PACE2_APPLICATION_FREEDOME_VPN,
    PACE2_APPLICATION_RES_784,
    PACE2_APPLICATION_STRONGVPN,
    PACE2_APPLICATION_RES_786,
    PACE2_APPLICATION_WARGAMING_NET,
    PACE2_APPLICATION_YANDEX_MUSIC,
    PACE2_APPLICATION_VPN_AREA,
    PACE2_APPLICATION_STEGANOS_ONLINE_SHIELD_VPN,
    PACE2_APPLICATION_SUPERVPN,
    PACE2_APPLICATION_STEGANOS_SERVICES,
    PACE2_APPLICATION_FROOTVPN,
    PACE2_APPLICATION_RES_794,
    PACE2_APPLICATION_RES_795,
    PACE2_APPLICATION_CME,
    PACE2_APPLICATION_YANDEX_NAVIGATOR,
    PACE2_APPLICATION_YANDEX_DISK,
    PACE2_APPLICATION_YANDEX_MAIL,
    PACE2_APPLICATION_TIKTOK,
    PACE2_APPLICATION_YANDEX_MAPS,
    PACE2_APPLICATION_RES_802,
    PACE2_APPLICATION_GAANA,
    PACE2_APPLICATION_XCUM_COM,
    PACE2_APPLICATION_LIFE_RU,
    PACE2_APPLICATION_RES_806,
    PACE2_APPLICATION_RES_807,
    PACE2_APPLICATION_RES_808,
    PACE2_APPLICATION_YEECALL,
    PACE2_APPLICATION_SHELLFIRE_VPN,
    PACE2_APPLICATION_RES_811,
    PACE2_APPLICATION_REN_TV,
    PACE2_APPLICATION_HOLA_VPN,
    PACE2_APPLICATION_HOLA_SERVICES,
    PACE2_APPLICATION_CDNVIDEO,
    PACE2_APPLICATION_RES_816,
    PACE2_APPLICATION_ARKVPN,
    PACE2_APPLICATION_FORTNITE,
    PACE2_APPLICATION_AMAZE_VPN,
    PACE2_APPLICATION_VUNGLE,
    PACE2_APPLICATION_VIVOX,
    PACE2_APPLICATION_MEGOGO,
    PACE2_APPLICATION_FINCH_VPN,
    PACE2_APPLICATION_FAST_SECURE_VPN,
    PACE2_APPLICATION_TVPLAYER,
    PACE2_APPLICATION_WORLD_OF_TANKS,
    PACE2_APPLICATION_RES_827,
    PACE2_APPLICATION_RES_828,
    PACE2_APPLICATION_APPVPN,
    PACE2_APPLICATION_X_VPN,
    PACE2_APPLICATION_DOTVPN,
    PACE2_APPLICATION_ENCRYPT_ME,
    PACE2_APPLICATION_RES_833,
    PACE2_APPLICATION_DROID_VPN,
    PACE2_APPLICATION_BROWSEC_VPN,
    PACE2_APPLICATION_RES_836,
    PACE2_APPLICATION_RES_837,
    PACE2_APPLICATION_APEX_LEGENDS,
    PACE2_APPLICATION_VPN_365,
    PACE2_APPLICATION_COUNTERSTRIKE_GO,
    PACE2_APPLICATION_VPN_TOR_BROWSER,
    PACE2_APPLICATION_SUPERVPN_BY_CHENG,
    PACE2_APPLICATION_ACT_VPN,
    PACE2_APPLICATION_BRIGHTCOVE,
    PACE2_APPLICATION_SPORTBOX_RU,
    PACE2_APPLICATION_JUST_PROXY_VPN,
    PACE2_APPLICATION_VEEPN,
    PACE2_APPLICATION_OOKLA,
    PACE2_APPLICATION_EAGLE_VPN,
    PACE2_APPLICATION_BRIA_MOBILE,
    PACE2_APPLICATION_VPN_GATE,
    PACE2_APPLICATION_ANCHORFREE_SERVICES,
    PACE2_APPLICATION_BRIA_TEAMS,
    PACE2_APPLICATION_SHADOWSOCKS,
    PACE2_APPLICATION_ASPHALT9,
    PACE2_APPLICATION_GAMELOFT_SERVICES,
    PACE2_APPLICATION_ACTMOBILE_SERVICES,
    PACE2_APPLICATION_NOVATECH_CORE,
    PACE2_APPLICATION_SECURE_VPN,
    PACE2_APPLICATION_APPSFLYER,
    PACE2_APPLICATION_ADJUST,
    PACE2_APPLICATION_GENIEVPN,
    PACE2_APPLICATION_RES_863,
    PACE2_APPLICATION_HELPSHIFT,
    PACE2_APPLICATION_FAST_VPN_MASTER,
    PACE2_APPLICATION_VPN_UNLIMITED_PROXY,
    PACE2_APPLICATION_HIGHSPEED_VPN,
    PACE2_APPLICATION_VPN_MASTER_BY_ANDRO_TEAM,
    PACE2_APPLICATION_RES_869,
    PACE2_APPLICATION_TENCENT_GAMES,
    PACE2_APPLICATION_RES_871,
    PACE2_APPLICATION_RES_872,
    PACE2_APPLICATION_DOUYU,
    PACE2_APPLICATION_INGRESS_PRIME,
    PACE2_APPLICATION_ELECTRONIC_ARTS,
    PACE2_APPLICATION_RADIOPLAYER,
    PACE2_APPLICATION_MOPUB,
    PACE2_APPLICATION_PUBG_MOBILE,
    PACE2_APPLICATION_RES_879,
    PACE2_APPLICATION_SPIEGEL_TV,
    PACE2_APPLICATION_SPIEGEL_SERVICES,
    PACE2_APPLICATION_RES_882,
    PACE2_APPLICATION_FORGE_OF_EMPIRES,
    PACE2_APPLICATION_INNOGAMES,
    PACE2_APPLICATION_RECEIVER_RADIO,
    PACE2_APPLICATION_GOODGAME_STUDIOS,
    PACE2_APPLICATION_RES_887,
    PACE2_APPLICATION_RADIO_DE,
    PACE2_APPLICATION_BBC_NEWS,
    PACE2_APPLICATION_BBC_SERVICES,
    PACE2_APPLICATION_UNREAL_ENGINE,
    PACE2_APPLICATION_ZOOM,
    PACE2_APPLICATION_ADOBE_SERVICES,
    PACE2_APPLICATION_ATT_SERVICES,
    PACE2_APPLICATION_FUNK,
    PACE2_APPLICATION_NEXX_TV,
    PACE2_APPLICATION_UNBLOCK_SITES,
    PACE2_APPLICATION_BRAWL_STARS,
    PACE2_APPLICATION_COUNTERPATH_SERVICES,
    PACE2_APPLICATION_SNAP_MASTER_VPN,
    PACE2_APPLICATION_JOYN,
    PACE2_APPLICATION_PRO7SAT1_SERVICES,
    PACE2_APPLICATION_DEAD_RIVALS,
    PACE2_APPLICATION_TAPJOY,
    PACE2_APPLICATION_IGG,
    PACE2_APPLICATION_LORDS_MOBILE,
    PACE2_APPLICATION_MUDFISH,
    PACE2_APPLICATION_RES_908,
    PACE2_APPLICATION_MIXPANEL,
    PACE2_APPLICATION_ADCOLONY,
    PACE2_APPLICATION_RES_911,
    PACE2_APPLICATION_BIG_FARM,
    PACE2_APPLICATION_FACEAPP,
    PACE2_APPLICATION_ASPHALT8,
    PACE2_APPLICATION_MOAT,
    PACE2_APPLICATION_ADSAFEPROTECTED,
    PACE2_APPLICATION_ADFORM,
    PACE2_APPLICATION_FREEWHEEL,
    PACE2_APPLICATION_SPOTX,
    PACE2_APPLICATION_IRONSOURCE,
    PACE2_APPLICATION_TRAFFICJUNKY,
    PACE2_APPLICATION_TRAFFICFACTORY,
    PACE2_APPLICATION_HATCH_CLOUD_GAMING,
    PACE2_APPLICATION_APPLOVIN,
    PACE2_APPLICATION_DELTADNA,
    PACE2_APPLICATION_ADTIMING,
    PACE2_APPLICATION_FLURRY,
    PACE2_APPLICATION_OPTIMIZELY,
    PACE2_APPLICATION_SMARTADSERVER,
    PACE2_APPLICATION_EXACTAG,
    PACE2_APPLICATION_SCORECARDRESEARCH,
    PACE2_APPLICATION_CONVIVA,
    PACE2_APPLICATION_BAMTECH_MEDIA,
    PACE2_APPLICATION_BEBI,
    PACE2_APPLICATION_HBO_SERVICES,
    PACE2_APPLICATION_RES_936,
    PACE2_APPLICATION_CRITEO,
    PACE2_APPLICATION_SWRVE,
    PACE2_APPLICATION_CHARTBOOST,
    PACE2_APPLICATION_RES_940,
    PACE2_APPLICATION_DIRECTV,
    PACE2_APPLICATION_VPNSECURE,
    PACE2_APPLICATION_LIFTOFF,
    PACE2_APPLICATION_RES_944,
    PACE2_APPLICATION_DISNEY_SERVICES,
    PACE2_APPLICATION_MY5,
    PACE2_APPLICATION_SOURCE_ENGINE,
    PACE2_APPLICATION_GEOCACHING,
    PACE2_APPLICATION_FOX_NEWS,
    PACE2_APPLICATION_KALTURA,
    PACE2_APPLICATION_RES_951,
    PACE2_APPLICATION_RES_952,
    PACE2_APPLICATION_FOX_SERVICES,
    PACE2_APPLICATION_BRANCH,
    PACE2_APPLICATION_VIKI,
    PACE2_APPLICATION_WORDPRESS,
    PACE2_APPLICATION_GRAVATAR,
    PACE2_APPLICATION_PARSELY,
    PACE2_APPLICATION_RES_959,
    PACE2_APPLICATION_PBS,
    PACE2_APPLICATION_CRACKLE,
    PACE2_APPLICATION_OPENX,
    PACE2_APPLICATION_RES_963,
    PACE2_APPLICATION_DISNEY_RADIO,
    PACE2_APPLICATION_KOCHAVA,
    PACE2_APPLICATION_ABC,
    PACE2_APPLICATION_RES_967,
    PACE2_APPLICATION_APTELIGENT,
    PACE2_APPLICATION_APPLE_NEWS,
    PACE2_APPLICATION_RES_970,
    PACE2_APPLICATION_SEGMENT,
    PACE2_APPLICATION_TUNE,
    PACE2_APPLICATION_APPLE_TV,
    PACE2_APPLICATION_APPTENTIVE,
    PACE2_APPLICATION_AIRABLE,
    PACE2_APPLICATION_PLUTO_TV,
    PACE2_APPLICATION_RES_977,
    PACE2_APPLICATION_RES_978,
    PACE2_APPLICATION_SONY_PICTURES,
    PACE2_APPLICATION_NFL_SUNDAY_TICKET,
    PACE2_APPLICATION_PORNHUB_NETWORK,
    PACE2_APPLICATION_PDANET,
    PACE2_APPLICATION_MLB,
    PACE2_APPLICATION_MLB_AT_BAT,
    PACE2_APPLICATION_TUBI,
    PACE2_APPLICATION_WATCH_TBS,
    PACE2_APPLICATION_DISNEY_PLUS,
    PACE2_APPLICATION_OPENSIGNAL,
    PACE2_APPLICATION_SHOWTIME,
    PACE2_APPLICATION_QUANTUM_METRIC,
    PACE2_APPLICATION_MPARTICLE,
    PACE2_APPLICATION_SHOWTIME_ANYTIME,
    PACE2_APPLICATION_CANAL_PLUS,
    PACE2_APPLICATION_AUDIBLE,
    PACE2_APPLICATION_TEALIUM,
    PACE2_APPLICATION_NUANCE,
    PACE2_APPLICATION_CBS_SERVICES,
    PACE2_APPLICATION_NBC_SPORTS,
    PACE2_APPLICATION_EUROSPORT_SERVICES,
    PACE2_APPLICATION_ENDEAVOR_STREAMING,
    PACE2_APPLICATION_TOTOK,
    PACE2_APPLICATION_FYBER,
    PACE2_APPLICATION_UNIVISION_NOW,
    PACE2_APPLICATION_TABOOLA,
    PACE2_APPLICATION_RES_1005,
    PACE2_APPLICATION_SKY_SERVICES,
    PACE2_APPLICATION_USERVOICE,
    PACE2_APPLICATION_BIGO_SERVICES,
    PACE2_APPLICATION_BITTORRENT_NOW,
    PACE2_APPLICATION_MARIO_KART_TOUR,
    PACE2_APPLICATION_TURKCELL_SERVICES,
    PACE2_APPLICATION_SKY_SPORTS,
    PACE2_APPLICATION_BIP_MESSENGER,
    PACE2_APPLICATION_SWOOGO,
    PACE2_APPLICATION_NBC_SERVICES,
    PACE2_APPLICATION_ETISALAT_SERVICES,
    PACE2_APPLICATION_SWITCH_TV,
    PACE2_APPLICATION_MICROSOFT_TEAMS,
    PACE2_APPLICATION_FLASHTALKING,
    PACE2_APPLICATION_RES_1020,
    PACE2_APPLICATION_RES_1021,
    PACE2_APPLICATION_NIELSEN_SERVICES,
    PACE2_APPLICATION_FOX_BUSINESS,
    PACE2_APPLICATION_PRIVATEVPN,
    PACE2_APPLICATION_SURFSHARK_VPN,
    PACE2_APPLICATION_YOSPACE,
    PACE2_APPLICATION_SAPLING_GROWTH_VPN,
    PACE2_APPLICATION_NFL_SERVICES,
    PACE2_APPLICATION_XUNLEI_SERVICES,
    PACE2_APPLICATION_THE_NBC_APP,
    PACE2_APPLICATION_FIZY,
    PACE2_APPLICATION_CLOUDINARY,
    PACE2_APPLICATION_TENCENT_SERVICES,
    PACE2_APPLICATION_ULTRAVPN,
    PACE2_APPLICATION_RES_1035,
    PACE2_APPLICATION_STARZ,
    PACE2_APPLICATION_WECHAT_WORK,
    PACE2_APPLICATION_RESILIO_SYNC,
    PACE2_APPLICATION_IMGIX,
    PACE2_APPLICATION_RES_1040,
    PACE2_APPLICATION_KAKAO_SERVICES,
    PACE2_APPLICATION_SOLVVY,
    PACE2_APPLICATION_JW_PLAYER,
    PACE2_APPLICATION_STACKLA,
    PACE2_APPLICATION_MUX,
    PACE2_APPLICATION_FRESHDESK,
    PACE2_APPLICATION_FRESHWORKS,
    PACE2_APPLICATION_AMPLITUDE,
    PACE2_APPLICATION_RES_1049,
    PACE2_APPLICATION_UNDER_ARMOUR_SERVICES,
    PACE2_APPLICATION_MAP_MY_FITNESS,
    PACE2_APPLICATION_TRUSTARC,
    PACE2_APPLICATION_MICROSOFT_EXCEL,
    PACE2_APPLICATION_MICROSOFT_WORD,
    PACE2_APPLICATION_MICROSOFT_POWERPOINT,
    PACE2_APPLICATION_MICROSOFT_OUTLOOK,
    PACE2_APPLICATION_SNAP_SERVICES,
    PACE2_APPLICATION_BOOTSTRAPCDN,
    PACE2_APPLICATION_KAKAOSTORY,
    PACE2_APPLICATION_RES_1060,
    PACE2_APPLICATION_SPORTSMAX,
    PACE2_APPLICATION_KAKAOTV_LIVE,
    PACE2_APPLICATION_APPLICASTER,
    PACE2_APPLICATION_MICROSOFT_SHAREPOINT,
    PACE2_APPLICATION_RES_1065,
    PACE2_APPLICATION_INDEX_EXCHANGE,
    PACE2_APPLICATION_NBC_NEWS,
    PACE2_APPLICATION_FUBOTV,
    PACE2_APPLICATION_APPDYNAMICS,
    PACE2_APPLICATION_TOUR_TRACKER,
    PACE2_APPLICATION_BLACKBOARD,
    PACE2_APPLICATION_CHARTBEAT,
    PACE2_APPLICATION_EBOT7,
    PACE2_APPLICATION_MICROSOFT_ONENOTE,
    PACE2_APPLICATION_HBO_GO,
    PACE2_APPLICATION_MEDIAMATH,
    PACE2_APPLICATION_VILYNX,
    PACE2_APPLICATION_WWE,
    PACE2_APPLICATION_RES_1079,
    PACE2_APPLICATION_RES_1080,
    PACE2_APPLICATION_SENTRY,
    PACE2_APPLICATION_YUPPTV,
    PACE2_APPLICATION_PERIMETERX,
    PACE2_APPLICATION_NBA,
    PACE2_APPLICATION_PLAYRIX,
    PACE2_APPLICATION_APP_BAZAAR,
    PACE2_APPLICATION_HOTJAR,
    PACE2_APPLICATION_TAPLYTICS,
    PACE2_APPLICATION_RTVE_ALACARTA,
    PACE2_APPLICATION_HAYU,
    PACE2_APPLICATION_GIGYA,
    PACE2_APPLICATION_TNT,
    PACE2_APPLICATION_SHOWMAX,
    PACE2_APPLICATION_ATRESMEDIA,
    PACE2_APPLICATION_CDNETWORKS,
    PACE2_APPLICATION_GOOGLE_CLASSROOM,
    PACE2_APPLICATION_PEER5,
    PACE2_APPLICATION_SATISFI_LABS,
    PACE2_APPLICATION_AYOBA,
    PACE2_APPLICATION_ROBLOX,
    PACE2_APPLICATION_NHL,
    PACE2_APPLICATION_INNOVATIVE_CONNECTING,
    PACE2_APPLICATION_OUTBRAIN,
    PACE2_APPLICATION_STAN,
    PACE2_APPLICATION_VPN_MONSTER,
    PACE2_APPLICATION_MEDIACORP_SERVICES,
    PACE2_APPLICATION_ATRESPLAYER,
    PACE2_APPLICATION_AIRSHIP,
    PACE2_APPLICATION_DIDI_SERVICES,
    PACE2_APPLICATION_MEDIASET_ESPANA,
    PACE2_APPLICATION_MITELE,
    PACE2_APPLICATION_MEWATCH,
    PACE2_APPLICATION_RES_1113,
    PACE2_APPLICATION_RES_1114,
    PACE2_APPLICATION_NIKE_SERVICES,
    PACE2_APPLICATION_ACCEDO,
    PACE2_APPLICATION_99TAXIS,
    PACE2_APPLICATION_CHARTER_COMMUNICATIONS,
    PACE2_APPLICATION_VIU,
    PACE2_APPLICATION_PCCW,
    PACE2_APPLICATION_ALIBABA_CLOUD,
    PACE2_APPLICATION_LOCALYTICS,
    PACE2_APPLICATION_TIDAL,
    PACE2_APPLICATION_RES_1124,
    PACE2_APPLICATION_YOUBORA,
    PACE2_APPLICATION_GOOGLE_MEET,
    PACE2_APPLICATION_COINBASE,
    PACE2_APPLICATION_WIDEORBIT,
    PACE2_APPLICATION_MEETRICS,
    PACE2_APPLICATION_CONDUCTRICS,
    PACE2_APPLICATION_LAUNCHDARKLY,
    PACE2_APPLICATION_ART19,
    PACE2_APPLICATION_HIVEMQ,
    PACE2_APPLICATION_HANDELSBLATT,
    PACE2_APPLICATION_PODIGEE,
    PACE2_APPLICATION_CLOUD_MAIL_RU,
    PACE2_APPLICATION_INNOVID,
    PACE2_APPLICATION_MOVISTAR_SERVICES,
    PACE2_APPLICATION_UCWEB,
    PACE2_APPLICATION_WIZROCKET,
    PACE2_APPLICATION_BATCH,
    PACE2_APPLICATION_VIZBEE,
    PACE2_APPLICATION_WIRTSCHAFTSWOCHE,
    PACE2_APPLICATION_HANDELSBLATT_SERVICES,
    PACE2_APPLICATION_IQ_DIGITAL,
    PACE2_APPLICATION_MEET_GROUP_SERVICES,
    PACE2_APPLICATION_MOVISTAR_PLUS,
    PACE2_APPLICATION_WIZZEE,
    PACE2_APPLICATION_INTERCOM,
    PACE2_APPLICATION_NETEASE,
    PACE2_APPLICATION_PENTHERA,
    PACE2_APPLICATION_RIOT_GAMES,
    PACE2_APPLICATION_BYTEDANCE,
    PACE2_APPLICATION_RES_1154,
    PACE2_APPLICATION_TAGGED,
    PACE2_APPLICATION_WEBTOON,
    PACE2_APPLICATION_MEETME,
    PACE2_APPLICATION_DYNAMIC_YIELD,
    PACE2_APPLICATION_ELEX_TECHNOLOGY,
    PACE2_APPLICATION_CLASH_OF_KINGS,
    PACE2_APPLICATION_IMDB,
    PACE2_APPLICATION_BILLO,
    PACE2_APPLICATION_AIHELP,
    PACE2_APPLICATION_CRUNCHYROLL,
    PACE2_APPLICATION_SHEIN,
    PACE2_APPLICATION_AVIRA_SERVICES,
    PACE2_APPLICATION_SAMSUNG_SERVICES,
    PACE2_APPLICATION_XIGUA_VIDEO,
    PACE2_APPLICATION_TOUTIAO,
    PACE2_APPLICATION_BUZZSPROUT,
    PACE2_APPLICATION_MEGAPHONE,
    PACE2_APPLICATION_BUGSNAG,
    PACE2_APPLICATION_KNOTCH,
    PACE2_APPLICATION_WURL,
    PACE2_APPLICATION_LIBSYN,
    PACE2_APPLICATION_TRITON_DIGITAL,
    PACE2_APPLICATION_VNG_CORPORATION,
    PACE2_APPLICATION_BAIDU_MAPS,
    PACE2_APPLICATION_MOONTON,
    PACE2_APPLICATION_AMAGI,
    PACE2_APPLICATION_KUAISHOU,
    PACE2_APPLICATION_MARKMONITOR,
    PACE2_APPLICATION_MEDIUM,
    PACE2_APPLICATION_SIMPLECAST,
    PACE2_APPLICATION_WALMART,
    PACE2_APPLICATION_GRUBHUB,
    PACE2_APPLICATION_KWAI,
    PACE2_APPLICATION_BRAZE,
    PACE2_APPLICATION_KEEPSOLID_SERVICES,
    PACE2_APPLICATION_VPN_LITE,
    PACE2_APPLICATION_SPRINGSERVE,
    PACE2_APPLICATION_RES_1192,
    PACE2_APPLICATION_WORKPLACE,
    PACE2_APPLICATION_YAANI,
    PACE2_APPLICATION_WASHINGTON_POST,
    PACE2_APPLICATION_RES_1196,
    PACE2_APPLICATION_ROMWE,
    PACE2_APPLICATION_TARGET_SIGNATURE,
    PACE2_APPLICATION_PLEX,
    PACE2_APPLICATION_AMAZON_ALEXA,
    PACE2_APPLICATION_BUMBLE,
    PACE2_APPLICATION_WEATHER_CHANNEL,
    PACE2_APPLICATION_REUTERS_NEWS,
    PACE2_APPLICATION_CRYPTOTRADER,
    PACE2_APPLICATION_RES_1205,
    PACE2_APPLICATION_CLUB_FACTORY,
    PACE2_APPLICATION_BOMBORA,
    PACE2_APPLICATION_MOOV,
    PACE2_APPLICATION_COLLECTD,
    PACE2_APPLICATION_ORACLE_SERVICES,
    PACE2_APPLICATION_COMCAST,
    PACE2_APPLICATION_SKRED_MESSENGER,
    PACE2_APPLICATION_ACAST,
    PACE2_APPLICATION_THE_CW,
    PACE2_APPLICATION_CW_SEED,
    PACE2_APPLICATION_FACEBOOK_LITE,
    PACE2_APPLICATION_OPERA_SERVICES,
    PACE2_APPLICATION_THE_WALL_STREET_JOURNAL,
    PACE2_APPLICATION_TEXTNOW,
    PACE2_APPLICATION_BLOOMBERG,
    PACE2_APPLICATION_UDEMY,
    PACE2_APPLICATION_THE_ECONOMIST,
    PACE2_APPLICATION_INVESTING_COM_SERVICES,
    PACE2_APPLICATION_CRYPTOWATCH,
    PACE2_APPLICATION_AMONG_US,
    PACE2_APPLICATION_COOKIEPRO,
    PACE2_APPLICATION_BEIN_SPORTS,
    PACE2_APPLICATION_BEIN_MEDIA_SERVICES,
    PACE2_APPLICATION_ERSTREAM,
    PACE2_APPLICATION_WE_MEET,
    PACE2_APPLICATION_NEWSDOG,
    PACE2_APPLICATION_PUGPIG,
    PACE2_APPLICATION_DIGITURK_PLAY,
    PACE2_APPLICATION_DIGICEL_SERVICES,
    PACE2_APPLICATION_DOORDASH,
    PACE2_APPLICATION_PHOTO_WONDER,
    PACE2_APPLICATION_QUANTCAST,
    PACE2_APPLICATION_JIOSAAVN,
    PACE2_APPLICATION_OPERA_NEWS,
    PACE2_APPLICATION_XHAMSTER,
    PACE2_APPLICATION_SHAHID,
    PACE2_APPLICATION_MBC_SERVICES,
    PACE2_APPLICATION_QQMUSIC,
    PACE2_APPLICATION_SEAMLESS,
    PACE2_APPLICATION_CIRES21,
    PACE2_APPLICATION_SPLUNK,
    PACE2_APPLICATION_COINCAP,
    PACE2_APPLICATION_SHAPESHIFT_SERVICES,
    PACE2_APPLICATION_AL_JAZEERA_SERVICES,
    PACE2_APPLICATION_AL_JAZEERA_ENGLISH,
    PACE2_APPLICATION_DO_GLOBAL_SERVICES,
    PACE2_APPLICATION_WEBFLOW,
    PACE2_APPLICATION_PAYBACK,
    PACE2_APPLICATION_MEITU_SERVICES,
    PACE2_APPLICATION_CAMSCANNER,
    PACE2_APPLICATION_INTSIG_SERVICES,
    PACE2_APPLICATION_JAUMO,
    PACE2_APPLICATION_PUBMATIC,
    PACE2_APPLICATION_JIO_SERVICES,
    PACE2_APPLICATION_GROUPON,
    PACE2_APPLICATION_RES_1261,
    PACE2_APPLICATION_VERIZON_MEDIA_SERVICES,
    PACE2_APPLICATION_ACTIVISION_BLIZZARD,
    PACE2_APPLICATION_AUDIOMACK,
    PACE2_APPLICATION_KAKAO_MAP,
    PACE2_APPLICATION_OSN,
    PACE2_APPLICATION_GSMA,
    PACE2_APPLICATION_GOOGLE_JIBE,
    PACE2_APPLICATION_BEAUTYPLUS,
    PACE2_APPLICATION_VIVAVIDEO,
    PACE2_APPLICATION_CALL_OF_DUTY,
    PACE2_APPLICATION_MICOWORLD_SERVICES,
    PACE2_APPLICATION_HAGO,
    PACE2_APPLICATION_DAUM_CAFE,
    PACE2_APPLICATION_TENCENT_NEWS,
    PACE2_APPLICATION_FLIPBOARD,
    PACE2_APPLICATION_VIACOM18_SERVICES,
    PACE2_APPLICATION_VOOT,
    PACE2_APPLICATION_SELFIECITY,
    PACE2_APPLICATION_MEITU,
    PACE2_APPLICATION_KASPERSKY_SERVICES,
    PACE2_APPLICATION_HAKUNA,
    PACE2_APPLICATION_BERMUDA_VIDEO_CHAT,
    PACE2_APPLICATION_XIAOMI_SERVICES,
    PACE2_APPLICATION_OPERA_NEWS_LITE,
    PACE2_APPLICATION_GAIJIN_ENTERTAINMENT,
    PACE2_APPLICATION_SCOPELY_SERVICES,
    PACE2_APPLICATION_MARVEL_STRIKE_FORCE,
    PACE2_APPLICATION_SKY_NEWS,
    PACE2_APPLICATION_SYSTEX_SERVICES,
    PACE2_APPLICATION_PERFECT_CORP_SERVICES,
    PACE2_APPLICATION_YOUCAM_MAKEUP,
    PACE2_APPLICATION_DTAG_SERVICES,
    PACE2_APPLICATION_PUBG,
    PACE2_APPLICATION_GOOGLE_VOICE,
    PACE2_APPLICATION_THE_SCORE,
    PACE2_APPLICATION_PANTAYA,
    PACE2_APPLICATION_DAUM_MAIL,
    PACE2_APPLICATION_FACEBOOK_BROADCASTING,
    PACE2_APPLICATION_WAR_THUNDER,
    PACE2_APPLICATION_APUS_SERVICES,
    PACE2_APPLICATION_UFOTO_SERVICES,
    PACE2_APPLICATION_PEAK_GAMES_SERVICES,
    PACE2_APPLICATION_TOON_BLAST,
    PACE2_APPLICATION_EXPO_FRAMEWORK,
    PACE2_APPLICATION_THE_NEW_YORK_TIMES,
    PACE2_APPLICATION_DU_RECORDER,
    PACE2_APPLICATION_RES_1308,
    PACE2_APPLICATION_MEIZU_SERVICES,
    PACE2_APPLICATION_JOYY_SERVICES,
    PACE2_APPLICATION_MOBVISTA,
    PACE2_APPLICATION_STARTIMES_ON,
    PACE2_APPLICATION_VODAFONE_SERVICES,
    PACE2_APPLICATION_THE_GUARDIAN,
    PACE2_APPLICATION_GUARDIAN_MEDIA_SERVICES,
    PACE2_APPLICATION_MOZILLA_SERVICES,
    PACE2_APPLICATION_MAGNITE,
    PACE2_APPLICATION_DC_UNIVERSE,
    PACE2_APPLICATION_RAKUTEN_SERVICES,
    PACE2_APPLICATION_PLAYDEMIC_SERVICES,
    PACE2_APPLICATION_GOLF_CLASH,
    PACE2_APPLICATION_ONETRUST,
    PACE2_APPLICATION_LIGHTSTEP,
    PACE2_APPLICATION_RES_1324,
    PACE2_APPLICATION_VS_MEDIA,
    PACE2_APPLICATION_EXARING_SERVICES,
    PACE2_APPLICATION_WAIPU_TV,
    PACE2_APPLICATION_GOOGLE_NEWS,
    PACE2_APPLICATION_RING_CENTRAL,
    PACE2_APPLICATION_MORTAL_KOMBAT,
    PACE2_APPLICATION_VPN_VALLEY,
    PACE2_APPLICATION_BEIN_CONNECT,
    PACE2_APPLICATION_BOOMPLAY_MUSIC,
    PACE2_APPLICATION_KONGREGATE,
    PACE2_APPLICATION_NINJA_KIWI_SERVICES,
    PACE2_APPLICATION_HUAWEI_SERVICES,
    PACE2_APPLICATION_XMRIG,
    PACE2_APPLICATION_LOL_WILD_RIFT,
    PACE2_APPLICATION_TINKOFF,
    PACE2_APPLICATION_YANDEX_MARKET,
    PACE2_APPLICATION_SWEET_SELFIE,
    PACE2_APPLICATION_SWEET_SELFIE_LITE,
    PACE2_APPLICATION_DIGICERT,
    PACE2_APPLICATION_SWITCHERRY_VPN,
    PACE2_APPLICATION_XENDER,
    PACE2_APPLICATION_NCSOFT,
    PACE2_APPLICATION_ORNET,
    PACE2_APPLICATION_COINGECKO,
    PACE2_APPLICATION_UMG,
    PACE2_APPLICATION_MALWAREBYTES,
    PACE2_APPLICATION_REALNETWORKS,
    PACE2_APPLICATION_KEYCDN,
    PACE2_APPLICATION_RILEY_CILLIAN,
    PACE2_APPLICATION_STREAMLIVE_TO,
    PACE2_APPLICATION_WILDLIFE_STUDIOS,
    PACE2_APPLICATION_BLOCK_CRAFT_3D,
    PACE2_APPLICATION_RES_1357,
    PACE2_APPLICATION_LOVOO,
    PACE2_APPLICATION_CLUBHOUSE,
    PACE2_APPLICATION_SNIPER_3D,
    PACE2_APPLICATION_COMODO_SERVICES,
    PACE2_APPLICATION_RIBBON_COMMUNICATIONS,
    PACE2_APPLICATION_BLACKBERRY_WORK,
    PACE2_APPLICATION_JOOM,
    PACE2_APPLICATION_BLACKBERRY_SERVICES,
    PACE2_APPLICATION_GOOGLE_PAY,
    PACE2_APPLICATION_GEEKFLARE,
    PACE2_APPLICATION_WAR_MACHINES,
    PACE2_APPLICATION_RAWVOICE,
    PACE2_APPLICATION_PODTRAC,
    PACE2_APPLICATION_WISE,
    PACE2_APPLICATION_GOOGLE_PODCASTS,
    PACE2_APPLICATION_SHALLTRY_GROUP,
    PACE2_APPLICATION_LIVERAMP,
    PACE2_APPLICATION_EURONEWS,
    PACE2_APPLICATION_LEAP_FITNESS_GROUP,
    PACE2_APPLICATION_HEXAGLOBE,
    PACE2_APPLICATION_ROCKET_LEAGUE,
    PACE2_APPLICATION_MAGINE_SERVICES,
    PACE2_APPLICATION_PAY_RW,
    PACE2_APPLICATION_VUBA_VUBA,
    PACE2_APPLICATION_YANDEX_GO,
    PACE2_APPLICATION_YANDEX_TELEMOST,
    PACE2_APPLICATION_NEXON_CORPORATION,
    PACE2_APPLICATION_AGODA,
    PACE2_APPLICATION_GRAB,
    PACE2_APPLICATION_NYPR,
    PACE2_APPLICATION_PRX_SERVICES,
    PACE2_APPLICATION_DOVETAIL,
    PACE2_APPLICATION_HUYA,
    PACE2_APPLICATION_SHAREIT,
    PACE2_APPLICATION_EVERSEND,
    PACE2_APPLICATION_5CHANNEL,
    PACE2_APPLICATION_PET_RESCUE_SAGA,
    PACE2_APPLICATION_CLASSDOJO,
    PACE2_APPLICATION_PHOTOMATH,
    PACE2_APPLICATION_MIHOYO_GAMES,
    PACE2_APPLICATION_GENSHIN_IMPACT,
    PACE2_APPLICATION_HONKAI_IMPACT_3RD,
    PACE2_APPLICATION_KHAN_ACADEMY,
    PACE2_APPLICATION_HOMER,
    PACE2_APPLICATION_CASTBOX,
    PACE2_APPLICATION_OPENTEXT_SERVICES,
    PACE2_APPLICATION_EDX,
    PACE2_APPLICATION_SOCRATIC,
    PACE2_APPLICATION_TELIA_SERVICES,
    PACE2_APPLICATION_TELIA_TV,
    PACE2_APPLICATION_PEACOCK_TV,
    PACE2_APPLICATION_JIO_ADS,
    PACE2_APPLICATION_DISCOVERY_PLUS,
    PACE2_APPLICATION_DUOLINGO,
    PACE2_APPLICATION_RES_1412,
    PACE2_APPLICATION_BET_PLUS,
    PACE2_APPLICATION_PARAMOUNT_PLUS,
    PACE2_APPLICATION_HUBSPOT,
    PACE2_APPLICATION_PHILO,
    PACE2_APPLICATION_DNA_TV,
    PACE2_APPLICATION_XIAOMI_WEATHER,
    PACE2_APPLICATION_JIOCHAT,
    PACE2_APPLICATION_REMIND,
    PACE2_APPLICATION_ELISA_SERVICES,
    PACE2_APPLICATION_ELISA_VIIHDE,
    PACE2_APPLICATION_PUBNUB,
    PACE2_APPLICATION_FUNIMATION,
    PACE2_APPLICATION_DRIFT,
    PACE2_APPLICATION_IMG,
    PACE2_APPLICATION_ENDEAVOR_SERVICES,
    PACE2_APPLICATION_SOFTBANK_SERVICES,
    PACE2_APPLICATION_REFACE,
    PACE2_APPLICATION_MTV_KATSOMO,
    PACE2_APPLICATION_AMAZON_ADS,
    PACE2_APPLICATION_PRENDE_TV,
    PACE2_APPLICATION_UNIVISION_COMMUNICATIONS,
    PACE2_APPLICATION_ANCHORFM,
    PACE2_APPLICATION_VEVE,
    PACE2_APPLICATION_RES_1436,
    PACE2_APPLICATION_QUALTRICS,
    PACE2_APPLICATION_WANGSU_CDN,
    PACE2_APPLICATION_ALGOLIA,
    PACE2_APPLICATION_MAXIVISION,
    PACE2_APPLICATION_G_CORE_SERVICES,
    PACE2_APPLICATION_NORDIC_ENTERTAINMENT_GROUP,
    PACE2_APPLICATION_VIAFREE,
    PACE2_APPLICATION_NETMERA,
    PACE2_APPLICATION_GRACENOTE,
    PACE2_APPLICATION_STORYFM_CN,
    PACE2_APPLICATION_YLE_AREENA,
    PACE2_APPLICATION_STREAMABLE,
    PACE2_APPLICATION_SPEAKOL,
    PACE2_APPLICATION_YALLAKORA,
    PACE2_APPLICATION_LINKONLINE,
    PACE2_APPLICATION_GEMINI_MEDIA_EGYPT,
    PACE2_APPLICATION_SANOMA_SERVICES,
    PACE2_APPLICATION_RUUTU,
    PACE2_APPLICATION_FINNPANEL,
    PACE2_APPLICATION_NOON_VPN,
    PACE2_APPLICATION_MASRAWY,
    PACE2_APPLICATION_KTPLAY,
    PACE2_APPLICATION_TEAM_FORTRESS_2,
    PACE2_APPLICATION_TRUST_WALLET,
    PACE2_APPLICATION_CANAL_PLUS_AFRIQUE,
    PACE2_APPLICATION_SARMADY_SERVICES,
    PACE2_APPLICATION_FILGOAL,
    PACE2_APPLICATION_AMC_THEATRES,
    PACE2_APPLICATION_YOUM7,
    PACE2_APPLICATION_VALVE_CORPORATION,
    PACE2_APPLICATION_RES_1467,
    PACE2_APPLICATION_XOOM,
    PACE2_APPLICATION_IBM,
    PACE2_APPLICATION_MIDOKI_SERVICES,
    PACE2_APPLICATION_VPN_MASTER_X,
    PACE2_APPLICATION_QUALAROO,
    PACE2_APPLICATION_MTS_SERVICES,
    PACE2_APPLICATION_EXPEDIA,
    PACE2_APPLICATION_GLOBALSIGN,
    PACE2_APPLICATION_LETSENCRYPT,
    PACE2_APPLICATION_YIELDMO,
    PACE2_APPLICATION_IDENTRUST,
    PACE2_APPLICATION_GODADDY,
    PACE2_APPLICATION_GOOGLE_TRUST_SERVICES,
    PACE2_APPLICATION_WATCH_IT,
    PACE2_APPLICATION_AMAZON_TRUST_SERVICES,
    PACE2_APPLICATION_ENTRUST,
    PACE2_APPLICATION_BINANCE,
    PACE2_APPLICATION_ELFSIGHT,
    PACE2_APPLICATION_SAILTHRU,
    PACE2_APPLICATION_CERTUM,
    PACE2_APPLICATION_MEDIANOVA_CDN,
    PACE2_APPLICATION_NAPSTER,
    PACE2_APPLICATION_SECTIGO,
    PACE2_APPLICATION_BAZAARVOICE,
    PACE2_APPLICATION_SUNING_SERVICES,
    PACE2_APPLICATION_INDIGITALL,
    PACE2_APPLICATION_ADA_SUPPORT,
    PACE2_APPLICATION_PENGUIN_SPORTS,
    PACE2_APPLICATION_PUSHER,
    PACE2_APPLICATION_CSG_MEDIA,
    PACE2_APPLICATION_GRAPESHOT,
    PACE2_APPLICATION_KNIGHTHOOD,
    PACE2_APPLICATION_BLOSSOM_BLAST_SAGA,
    PACE2_APPLICATION_CYBERTRUST_CA,
    PACE2_APPLICATION_USERCENTRICS,
    PACE2_APPLICATION_IDVIU,
    PACE2_APPLICATION_IAPPS_TECHNOLOGIES,
    PACE2_APPLICATION_CALM,
    PACE2_APPLICATION_PLAYSTATION_NOW,
    PACE2_APPLICATION_MICROSOFT_AZURE,
    PACE2_APPLICATION_ZEGO,
    PACE2_APPLICATION_DISH_NETWORK,
    PACE2_APPLICATION_JQUERY,
    PACE2_APPLICATION_MOX_TV,
    PACE2_APPLICATION_STATS_PERFORM,
    PACE2_APPLICATION_CONTENTFUL,
    PACE2_APPLICATION_WETV,
    PACE2_APPLICATION_ADEX,
    PACE2_APPLICATION_ARLO,
    PACE2_APPLICATION_JIOHEALTHHUB,
    PACE2_APPLICATION_GOG,
    PACE2_APPLICATION_RING,
    PACE2_APPLICATION_FITBIT,
    PACE2_APPLICATION_GOOGLE_FIT,
    PACE2_APPLICATION_DIAMOND_DIARIES_SAGA,
    PACE2_APPLICATION_DELUXE_MEDIA,
    PACE2_APPLICATION_THE_TRADE_DESK,
    PACE2_APPLICATION_JIOCLOUD,
    PACE2_APPLICATION_TRIVAGO,
    PACE2_APPLICATION_MINUTELY,
    PACE2_APPLICATION_GARENA_SERVICES,
    PACE2_APPLICATION_GARENA_FREE_FIRE,
    PACE2_APPLICATION_IMPACT_SERVICES,
    PACE2_APPLICATION_SAMSUNG_MAX_PRIVACY_VPN,
    PACE2_APPLICATION_PRIVATE_TUNNEL_VPN,
    PACE2_APPLICATION_TRIPLELIFT,
    PACE2_APPLICATION_RES_1534,
    PACE2_APPLICATION_MAILCHIMP,
    PACE2_APPLICATION_A1_BELARUS,
    PACE2_APPLICATION_VOKA_TV,
    PACE2_APPLICATION_SPB_TV,
    PACE2_APPLICATION_SMART_MEDIA_SOLUTIONS,
    PACE2_APPLICATION_SMAATO,
    PACE2_APPLICATION_TEADS,
    PACE2_APPLICATION_PYRAMID_SOLITAIRE_SAGA,
    PACE2_APPLICATION_RISKIFIED,
    PACE2_APPLICATION_RES_1544,
    PACE2_APPLICATION_RES_1545,
    PACE2_APPLICATION_LEANPLUM,
    PACE2_APPLICATION_TAPAD,
    PACE2_APPLICATION_SQUARESPACE,
    PACE2_APPLICATION_BRIGHTLINE,
    PACE2_APPLICATION_EXTREME_REACH,
    PACE2_APPLICATION_ROCKETVPN,
    PACE2_APPLICATION_MOOVIT,
    PACE2_APPLICATION_COINBOX,
    PACE2_APPLICATION_SYBO_GAMES,
    PACE2_APPLICATION_SUBWAY_SURFERS,
    PACE2_APPLICATION_NPR,
    PACE2_APPLICATION_ESET_SERVICES,
    PACE2_APPLICATION_MAKEUPPLUS,
    PACE2_APPLICATION_KAKAO_WEBTOON,
    PACE2_APPLICATION_ZEE5,
    PACE2_APPLICATION_JUSPAY,
    PACE2_APPLICATION_POKEMON_UNITE,
    PACE2_APPLICATION_ZOMATO,
    PACE2_APPLICATION_PLYMEDIA,
    PACE2_APPLICATION_BITPAY,
    PACE2_APPLICATION_PHYCAN_VPN_SERVICES,
    PACE2_APPLICATION_INTERNET_ARCHIVE,
    PACE2_APPLICATION_CLOUDFLARE_WARP_VPN,
    PACE2_APPLICATION_BRD_WALLET,
    PACE2_APPLICATION_EXODUS_WALLET,
    PACE2_APPLICATION_BEAUTYCAM,
    PACE2_APPLICATION_GAB,
    PACE2_APPLICATION_MEASUREMENT_LAB,
    PACE2_APPLICATION_GLOVO,
    PACE2_APPLICATION_SAMSUNG_MUSIC,
    PACE2_APPLICATION_SWEET_TV,
    PACE2_APPLICATION_UKLON,
    PACE2_APPLICATION_TOCA_BOCA,
    PACE2_APPLICATION_TAVR_MEDIA,
    PACE2_APPLICATION_LIGHTSAIL_VPN,
    PACE2_APPLICATION_IBM_CLOUD_VIDEO,
    PACE2_APPLICATION_AGORA_SERVICES,
    PACE2_APPLICATION_SUN_FM_UKRAINE,
    PACE2_APPLICATION_OLL_TV,
    PACE2_APPLICATION_LANET_TV,
    PACE2_APPLICATION_TIGO_SERVICES,
    PACE2_APPLICATION_UBER_EATS,
    PACE2_APPLICATION_BOLT_TECHNOLOGY,
    PACE2_APPLICATION_STRIPE,
    PACE2_APPLICATION_MEDALLIA,
    PACE2_APPLICATION_ONEPLUSONE_MEDIA_SERVICES,
    PACE2_APPLICATION_ONEPLUSONE_VIDEO,
    PACE2_APPLICATION_VERISIGN,
    PACE2_APPLICATION_RES_1594,
    PACE2_APPLICATION_ADMIXER,
    PACE2_APPLICATION_DINGTALK,
    PACE2_APPLICATION_ONLINE_RADIO_BOX,
    PACE2_APPLICATION_MOLOCO,
    PACE2_APPLICATION_RES_1599,
    PACE2_APPLICATION_PROSTREAM_RU,
    PACE2_APPLICATION_BLOCKCHAIN_COM,
    PACE2_APPLICATION_QUALCOMM,
    PACE2_APPLICATION_JIOMART,
    PACE2_APPLICATION_JIOMEET,
    PACE2_APPLICATION_HCL_NOTES_AND_DOMINO,
    PACE2_APPLICATION_MI_FIT,
    PACE2_APPLICATION_PAXFUL_WALLET,
    PACE2_APPLICATION_SCHILDBACH_WALLET,
    PACE2_APPLICATION_CAKE,
    PACE2_APPLICATION_TIGERCONNECT,
    PACE2_APPLICATION_ICLOUD_PRIVATE_RELAY,
    PACE2_APPLICATION_LUDO_KING,
    PACE2_APPLICATION_TWILIO_SERVICES,
    PACE2_APPLICATION_ANKAMA_GAMES,
    PACE2_APPLICATION_META_SERVICES,
    PACE2_APPLICATION_HUAWEI_HEALTH,
    PACE2_APPLICATION_YOUDAO,
    PACE2_APPLICATION_XINHUA_NEWS,
    PACE2_APPLICATION_RAYGUN,
    PACE2_APPLICATION_GOOGLE_CHAT,
    PACE2_APPLICATION_DOUBLEVERIFY,
    PACE2_APPLICATION_EXTOLE,
    PACE2_APPLICATION_STARMAKER,
    PACE2_APPLICATION_MX_PLAYER,
    PACE2_APPLICATION_ORIGINS_DIGITAL,
    PACE2_APPLICATION_SYGIC_SERVICES,
    PACE2_APPLICATION_BUDGE_STUDIOS,
    PACE2_APPLICATION_MTS_TV,
    PACE2_APPLICATION_SMADEX,
    PACE2_APPLICATION_AMAZON_GAMES,
    PACE2_APPLICATION_NEW_WORLD,
    PACE2_APPLICATION_AARKI,
    PACE2_APPLICATION_XIAOMI_SHAREME,
    PACE2_APPLICATION_GOODBARBER,
    PACE2_APPLICATION_LITRES_RU,
    PACE2_APPLICATION_QWILT,
    PACE2_APPLICATION_START_RU,
    PACE2_APPLICATION_POU,
    PACE2_APPLICATION_MUMBLE,
    PACE2_APPLICATION_AMEX_SERVICES,
    PACE2_APPLICATION_WIZARDS_OF_THE_COAST,
    PACE2_APPLICATION_MAGIC_THE_GATHERING_ARENA,
    PACE2_APPLICATION_MOZILLA_VPN,
    PACE2_APPLICATION_DEUTSCHE_KREDITBANK,
    PACE2_APPLICATION_DEUTSCHE_BANK,
    PACE2_APPLICATION_NORTONLIFELOCK_SERVICES,
    PACE2_APPLICATION_NORTONLIFELOCK_VPN_SERVICES,
    PACE2_APPLICATION_ZAPYA,
    PACE2_APPLICATION_AMOBEE,
    PACE2_APPLICATION_UBITUS_SERVICES,
    PACE2_APPLICATION_NINTENDO_SWITCH_CLOUD_GAMING,
    PACE2_APPLICATION_SPREAKER,
    PACE2_APPLICATION_JIOCINEMA,
    PACE2_APPLICATION_EPSILON_CONVERSANT,
    PACE2_APPLICATION_NVIDIA_SERVICES,
    PACE2_APPLICATION_WINK,
    PACE2_APPLICATION_ROSTELECOM,
    PACE2_APPLICATION_NGENIX,
    PACE2_APPLICATION_INTEL_SERVICES,
    PACE2_APPLICATION_AMD_SERVICES,
    PACE2_APPLICATION_PIKMIN_BLOOM,
    PACE2_APPLICATION_123VPN,
    PACE2_APPLICATION_PLAYTIKA_SERVICES,
    PACE2_APPLICATION_AVG_SERVICES,
    PACE2_APPLICATION_NETFLIX_SERVICES,
    PACE2_APPLICATION_NETFLIX_GAMES,
    PACE2_APPLICATION_HERO_WARS,
    PACE2_APPLICATION_SWIFTY_BEAVER,
    PACE2_APPLICATION_BITDEFENDER_SERVICES,
    PACE2_APPLICATION_IHEART_MEDIA_SERVICES,
    PACE2_APPLICATION_SOPHOS_SERVICES,
    PACE2_APPLICATION_MCAFEE_SERVICES,
    PACE2_APPLICATION_EUROSPORT_NEWS,
    PACE2_APPLICATION_OLA,
    PACE2_APPLICATION_LG_ELECTRONICS_SERVICES,
    PACE2_APPLICATION_ERICSSON_SERVICES,
    PACE2_APPLICATION_SHARECHAT,
    PACE2_APPLICATION_VALORANT,
    PACE2_APPLICATION_INDEED,
    PACE2_APPLICATION_NVIDIA_GEFORCE_NOW,
    PACE2_APPLICATION_TRUEKEY,
    PACE2_APPLICATION_MOON_ACTIVE,
    PACE2_APPLICATION_SAFE_FAMILY,
    PACE2_APPLICATION_LOTAME,
    PACE2_APPLICATION_FIREFOX_PRIVATE_NETWORK,
    PACE2_APPLICATION_MEDIA_NET,
    PACE2_APPLICATION_JSDELIVR,
    PACE2_APPLICATION_UBISOFT_SERVICES,
    PACE2_APPLICATION_UBISOFT_CONNECT,
    PACE2_APPLICATION_BLITZER_DE,
    PACE2_APPLICATION_SAMSUNG_PAY,
    PACE2_APPLICATION_GOOGLE_PHOTOS,
    PACE2_APPLICATION_STELLAR,
    PACE2_APPLICATION_TRACKMANIA,
    PACE2_APPLICATION_WANT_WANT_SERVICES,
    PACE2_APPLICATION_UNRULY,
    PACE2_APPLICATION_TOTALAV,
    PACE2_APPLICATION_ATUDO,
    PACE2_APPLICATION_BID_SWITCH,
    PACE2_APPLICATION_ING_BANKING,
    PACE2_APPLICATION_FEEDAD,
    PACE2_APPLICATION_COMMERZBANK_BANKING,
    PACE2_APPLICATION_VR_BANKING,
    PACE2_APPLICATION_ANZU_VR,
    PACE2_APPLICATION_BEESWAX,
    PACE2_APPLICATION_DAUNTLESS,
    PACE2_APPLICATION_CATCHMEDIA,
    PACE2_APPLICATION_PIANO,
    PACE2_APPLICATION_SHOPIFY,
    PACE2_APPLICATION_PC_PROTECT,
    PACE2_APPLICATION_CISCO_SYSTEMS_SERVICES,
    PACE2_APPLICATION_SPARKASSE_BANKING,
    PACE2_APPLICATION_HOLASPARK,
    PACE2_APPLICATION_MICROSOFT_BING,
    PACE2_APPLICATION_TESTMY_NET,
    PACE2_APPLICATION_FUELIO,
    PACE2_APPLICATION_SPEEDOF_ME,
    PACE2_APPLICATION_XFINITY,
    PACE2_APPLICATION_XFINITY_SPEEDTEST,
    PACE2_APPLICATION_PAYCENTIVE,
    PACE2_APPLICATION_RAINBOW_SIX_SIEGE,
    PACE2_APPLICATION_JPMORGAN_CHASE,
    PACE2_APPLICATION_MULTI_MEDIA_LLC,
    PACE2_APPLICATION_CAPITALONE,
    PACE2_APPLICATION_SYGIC_TRAVEL_MAPS,
    PACE2_APPLICATION_GLASSBOX,
    PACE2_APPLICATION_STARK_GAMES,
    PACE2_APPLICATION_WELLS_FARGO,
    PACE2_APPLICATION_BLINKIST,
    PACE2_APPLICATION_GAMES_WORKSHOP,
    PACE2_APPLICATION_WARHAMMER_GAMING,
    PACE2_APPLICATION_REDFAST,
    PACE2_APPLICATION_SAMSUNG_HEALTH,
    PACE2_APPLICATION_SPEEDSMART,
    PACE2_APPLICATION_SYGIC_GPS_NAVIGATION,
    PACE2_APPLICATION_COMCAST_TECH_SOLUTIONS,
    PACE2_APPLICATION_VIUTV,
    PACE2_APPLICATION_DEMANDBASE,
    PACE2_APPLICATION_RAINBOW_SIX_EXTRACTION,
    PACE2_APPLICATION_WARHAMMER_TV,
    PACE2_APPLICATION_DISNEY_HOTSTAR,
    PACE2_APPLICATION_WUNDERKIND,
    PACE2_APPLICATION_MIGHT_AND_MAGIC_CHESS_ROYALE,
    PACE2_APPLICATION_PULSE_INSIGHTS,
    PACE2_APPLICATION_RADARBOT,
    PACE2_APPLICATION_ADSWIZZ,
    PACE2_APPLICATION_BRIGHT_DATA,
    PACE2_APPLICATION_WAKANIM,
    PACE2_APPLICATION_ETSY,
    PACE2_APPLICATION_VANILLA,
    PACE2_APPLICATION_CITADEL_COLOUR,
    PACE2_APPLICATION_BEHAVIOUR_SERVICES,
    PACE2_APPLICATION_DEAD_BY_DAYLIGHT,
    PACE2_APPLICATION_BLACK_LIBRARY,
    PACE2_APPLICATION_DIGITAL_TURBINE,
    PACE2_APPLICATION_DUCKDUCKGO,
    PACE2_APPLICATION_LIVEONE,
    PACE2_APPLICATION_JOSH,
    PACE2_APPLICATION_BABE,
    PACE2_APPLICATION_WIKIMEDIA_FOUNDATION,
    PACE2_APPLICATION_AVAST_VPN_SERVICES,
    PACE2_APPLICATION_OPENWEB,
    PACE2_APPLICATION_STREAMIO,
    PACE2_APPLICATION_TMT_BROADCASTING,
    PACE2_APPLICATION_UNO,
    PACE2_APPLICATION_VERVE_GROUP,
    PACE2_APPLICATION_MCAFEE_SECURITY,
    PACE2_APPLICATION_LOKALISE,
    PACE2_APPLICATION_BRAINLY,
    PACE2_APPLICATION_ZONER_ANTIVIRUS,
    PACE2_APPLICATION_ZONER_SERVICES,
    PACE2_APPLICATION_MAGENTA_TV,
    PACE2_APPLICATION_LARK,
    PACE2_APPLICATION_CAPCUT,
    PACE2_APPLICATION_PODBEAN,
    PACE2_APPLICATION_PERMUTIVE,
    PACE2_APPLICATION_WISTIA,
    PACE2_APPLICATION_RAPPI,
    PACE2_APPLICATION_KION,
    PACE2_APPLICATION_OKKO,
    PACE2_APPLICATION_PICSART,
    PACE2_APPLICATION_MX_TAKATAK,
    PACE2_APPLICATION_SENDER,
    PACE2_APPLICATION_F_SECURE,
    PACE2_APPLICATION_AAROGYA_SETU,
    PACE2_APPLICATION_ANIVIEW,
    PACE2_APPLICATION_PINDUODUO,
    PACE2_APPLICATION_GEARBOX_ENTERTAINMENT_SERVICES,
    PACE2_APPLICATION_GEARBOX_SHIFT,
    PACE2_APPLICATION_AGE_OF_EMPIRES,
    PACE2_APPLICATION_RELIC_ENTERTAINMENT,
    PACE2_APPLICATION_XBOX_GAME_STUDIOS,
    PACE2_APPLICATION_SPLIT,
    PACE2_APPLICATION_TRUTH_SOCIAL,
    PACE2_APPLICATION_RUMBLE,
    PACE2_APPLICATION_FANDOM,
    PACE2_APPLICATION_USCREEN,
    PACE2_APPLICATION_HYPERCONNECT,
    PACE2_APPLICATION_FONT_AWESOME,
    PACE2_APPLICATION_RUBY_GAME_STUDIO,
    PACE2_APPLICATION_MOENGAGE,
    PACE2_APPLICATION_LOST_ARK,
    PACE2_APPLICATION_MIRO,
    PACE2_APPLICATION_GOOGLE_TV,
    PACE2_APPLICATION_CADENT,
    PACE2_APPLICATION_PANGLE,
    PACE2_APPLICATION_EMBRACE,
    PACE2_APPLICATION_STREAMGUYS,
    PACE2_APPLICATION_HOLLER,
    PACE2_APPLICATION_EASYBRAIN,
    PACE2_APPLICATION_EASYBRAIN_MUSIC_MAKER,
    PACE2_APPLICATION_CREATIVE_ASSEMBLY,
    PACE2_APPLICATION_SEGA,
    PACE2_APPLICATION_MATCH_GROUP,
    PACE2_APPLICATION_TOTAL_WAR,
    PACE2_APPLICATION_FERAL_INTERACTIVE,
    PACE2_APPLICATION_LIVEBOARD,
    PACE2_APPLICATION_NARAKA_BLADEPOINT,
    PACE2_APPLICATION_NETEASE_GAMES,
    PACE2_APPLICATION_PATREON,
    PACE2_APPLICATION_USPS,
    PACE2_APPLICATION_IAB_EUROPE,
    PACE2_APPLICATION_UPS,
    PACE2_APPLICATION_Q4,
    PACE2_APPLICATION_TIDIO,
    PACE2_APPLICATION_WARNER_BROS_DISCOVERY_SERVICES,
    PACE2_APPLICATION_LAZADA,
    PACE2_APPLICATION_ONLYFANS,
    PACE2_APPLICATION_PRIMIS,
    PACE2_APPLICATION_BLUE_BILLYWIG,
    PACE2_APPLICATION_GEOLOCATION_TECHNOLOGY,
    PACE2_APPLICATION_LUMEN_TECHNOLOGIES,
    PACE2_APPLICATION_FRUITSHAKE,
    PACE2_APPLICATION_IMPROVE_DIGITAL,
    PACE2_APPLICATION_TERMINUS,
    PACE2_APPLICATION_TABLEAU,
    PACE2_APPLICATION_TINY_TINAS_WONDERLANDS,
    PACE2_APPLICATION_AG_PROJECTS_SERVICES,
    PACE2_APPLICATION_DISPLAY_IO,
    PACE2_APPLICATION_BEETALK,
    PACE2_APPLICATION_ZILLOW,
    PACE2_APPLICATION_PLURK,
    PACE2_APPLICATION_INMOBI,
    PACE2_APPLICATION_ARC_XP,
    PACE2_APPLICATION_EMBRACER_GROUP,
    PACE2_APPLICATION_SPIN_THE_BOTTLE,
    PACE2_APPLICATION_XANDR,
    PACE2_APPLICATION_MAPP,
    PACE2_APPLICATION_9GAG,
    PACE2_APPLICATION_SQUARE_ENIX,
    PACE2_APPLICATION_FINAL_FANTASY_XIV,
    PACE2_APPLICATION_NAVER_WORKS,
    PACE2_APPLICATION_NAVER_CORPORATION,
    PACE2_APPLICATION_INTUIT,
    PACE2_APPLICATION_GAZPROM_MEDIA,
    PACE2_APPLICATION_RES_1856,
    PACE2_APPLICATION_RADANCY,
    PACE2_APPLICATION_RAMBLER,
    PACE2_APPLICATION_CRYPTO_COM,
    PACE2_APPLICATION_FIFA_SOCCER,
    PACE2_APPLICATION_KUSTOMER,
    PACE2_APPLICATION_LOGROCKET,
    PACE2_APPLICATION_SPOTHERO,
    PACE2_APPLICATION_ADOBE_EXPERIENCE_CLOUD,
    PACE2_APPLICATION_ICF_TECHNOLOGY,
    PACE2_APPLICATION_TRICOLOR_TV,
    PACE2_APPLICATION_REDFIN,
    PACE2_APPLICATION_MATTERPORT,
    PACE2_APPLICATION_NAVER_CLOUD,
    PACE2_APPLICATION_ROGUE_COMPANY,
    PACE2_APPLICATION_WOW_TV,
    PACE2_APPLICATION_QUORA,
    PACE2_APPLICATION_INFOBIP,
    PACE2_APPLICATION_EDGIO,
    PACE2_APPLICATION_FLIPKART,
    PACE2_APPLICATION_NEAR,
    PACE2_APPLICATION_AVO,
    PACE2_APPLICATION_CONTENTSQUARE,
    PACE2_APPLICATION_FALL_GUYS,
    PACE2_APPLICATION_YAHOO_AD_TECH,
    PACE2_APPLICATION_MEESHO,
    PACE2_APPLICATION_TOWER_OF_FANTASY,
    PACE2_APPLICATION_NPERF,
    PACE2_APPLICATION_NBCUNIVERSAL,
    PACE2_APPLICATION_NAVER_MAP,
    PACE2_APPLICATION_SNAPDEAL,
    PACE2_APPLICATION_SMARTCLIP,
    PACE2_APPLICATION_DIGITALOCEAN,
    PACE2_APPLICATION_HAPPN,
    PACE2_APPLICATION_2K,
    PACE2_APPLICATION_MICROSOFT_FAMILY_SAFETY,
    PACE2_APPLICATION_RUMBLEVERSE,
    PACE2_APPLICATION_WB_GAMES,
    PACE2_APPLICATION_MICROSOFT_DEFENDER,
    PACE2_APPLICATION_ADOBE_FONTS,
    PACE2_APPLICATION_VIMOND,
    PACE2_APPLICATION_AIONCLOUD,
    PACE2_APPLICATION_AMC_NETWORKS,
    PACE2_APPLICATION_AMC_PLUS,
    PACE2_APPLICATION_CABIFY,
    PACE2_APPLICATION_PAYTM,
    PACE2_APPLICATION_LENOVO_SERVICES,
    PACE2_APPLICATION_UPLIVE,
    PACE2_APPLICATION_NAVER_MAIL,
    PACE2_APPLICATION_AKAMAI_TECHNOLOGIES,
    PACE2_APPLICATION_STACKADAPT,
    PACE2_APPLICATION_HIREZ_STUDIOS,
    PACE2_APPLICATION_VIX,
    PACE2_APPLICATION_TREMOR_VIDEO,
    PACE2_APPLICATION_BLOCK_INC,
    PACE2_APPLICATION_CASH_APP,
    PACE2_APPLICATION_OUTFIT7,
    PACE2_APPLICATION_MYTHIC_LEGENDS,
    PACE2_APPLICATION_UI_AG_SERVICES,
    PACE2_APPLICATION_GMX_MAIL,
    PACE2_APPLICATION_INSHORTS,
    PACE2_APPLICATION_REVOLUT,
    PACE2_APPLICATION_ZEPETO,
    PACE2_APPLICATION_DIABLO_IMMORTAL,
    PACE2_APPLICATION_LIFE360,
    PACE2_APPLICATION_SESSION,
    PACE2_APPLICATION_PREMIER,
    PACE2_APPLICATION_TENCENT_CLOUD,
    PACE2_APPLICATION_PAYPAY,
    PACE2_APPLICATION_6SENSE,
    PACE2_APPLICATION_VIMEO_OTT,
    PACE2_APPLICATION_ZEUS_NETWORK,
    PACE2_APPLICATION_VIDAZOO,
    PACE2_APPLICATION_HIDIVE,
    PACE2_APPLICATION_GRUPO_GLOBO_SERVICES,
    PACE2_APPLICATION_GLOBOPLAY,
    PACE2_APPLICATION_SHUDDER,
    PACE2_APPLICATION_KINGSOFT_SERVICES,
    PACE2_APPLICATION_KINGSOFT_CLOUD,
    PACE2_APPLICATION_KOBO,
    PACE2_APPLICATION_FOR_HONOR,
    PACE2_APPLICATION_TREMOR_INTERNATIONAL,
    PACE2_APPLICATION_FINANCIAL_TIMES,
    PACE2_APPLICATION_KAPE_SERVICES,
    PACE2_APPLICATION_SQUARE,
    PACE2_APPLICATION_HCAPTCHA,
    PACE2_APPLICATION_AMC_SVOD,
    PACE2_APPLICATION_SUNDANCE_NOW,
    PACE2_APPLICATION_NAVER_NOW,
    PACE2_APPLICATION_AUSTRALIA_POST,
    PACE2_APPLICATION_BUNGIE_GAMES,
    PACE2_APPLICATION_ORA_TV,
    PACE2_APPLICATION_CCP_GAMES,
    PACE2_APPLICATION_EVE_ONLINE,
    PACE2_APPLICATION_ZYPE,
    PACE2_APPLICATION_UMENG_PLUS,
    PACE2_APPLICATION_BYJUS,
    PACE2_APPLICATION_TILDA,
    PACE2_APPLICATION_CDN77,
    PACE2_APPLICATION_TAPCLAP,
    PACE2_APPLICATION_PIRATE_TREASURES,
    PACE2_APPLICATION_PRISMIC,
    PACE2_APPLICATION_TENCENT_MUSIC,
    PACE2_APPLICATION_HTTPOOL,
    PACE2_APPLICATION_CACHEFLY,
    PACE2_APPLICATION_ALIBABA_GROUP_SERVICES,
    PACE2_APPLICATION_QANDA,
    PACE2_APPLICATION_VERCEL,
    PACE2_APPLICATION_STREAMOTION,
    PACE2_APPLICATION_DRAFTKINGS_SERVICES,
    PACE2_APPLICATION_DRAFTKINGS_CASINO,
    PACE2_APPLICATION_DRAFTKINGS_SPORTSBOOK,
    PACE2_APPLICATION_WIX,
    PACE2_APPLICATION_FANDUEL_SERVICES,
    PACE2_APPLICATION_FANDUEL_CASINO,
    PACE2_APPLICATION_FANDUEL_SPORTSBOOK,
    PACE2_APPLICATION_CRYPTIC_STUDIOS,
    PACE2_APPLICATION_NEVERWINTER,
    PACE2_APPLICATION_RESSO_MUSIC,
    PACE2_APPLICATION_XUMO,
    PACE2_APPLICATION_REALM_ROYALE,
    PACE2_APPLICATION_VIKINGARD,
    PACE2_APPLICATION_BADLANDERS,
    PACE2_APPLICATION_INFINITE_LAGRANGE,
    PACE2_APPLICATION_VIEWLIFT,
    PACE2_APPLICATION_TVNZ,
    PACE2_APPLICATION_WYNK_MUSIC,
    PACE2_APPLICATION_GEARBOX_PUBLISHING,
    PACE2_APPLICATION_VIJU,
    PACE2_APPLICATION_VIASAT_RU,
    PACE2_APPLICATION_LILITHGAMES,
    PACE2_APPLICATION_RISE_OF_KINGDOMS,
    PACE2_APPLICATION_DECA_GAMES,
    PACE2_APPLICATION_GODS_AND_GLORY,
    PACE2_APPLICATION_PODCAST_ADDICT,
    PACE2_APPLICATION_EVE_ECHOES,
    PACE2_APPLICATION_LAZY_AUDIO,
    PACE2_APPLICATION_AMEDIATEKA,
    PACE2_APPLICATION_MIXI,
    PACE2_APPLICATION_MIXI_SERVICES,
    PACE2_APPLICATION_LOTR_WAR,
    PACE2_APPLICATION_GOOGLE_TRANSLATE,
    PACE2_APPLICATION_NTT_SERVICES,
    PACE2_APPLICATION_RADIKO,
    PACE2_APPLICATION_CYBERAGENT_SERVICES,
    PACE2_APPLICATION_AMEBA,
    PACE2_APPLICATION_RED_HOT_PIE,
    PACE2_APPLICATION_CLICKTOUCH_GAMES,
    PACE2_APPLICATION_ADTECH_STUDIO,
    PACE2_APPLICATION_MEDIALAB_AI_SERVICES,
    PACE2_APPLICATION_MITENE,
    PACE2_APPLICATION_TRANSPERTH,
    PACE2_APPLICATION_HINGE,
    PACE2_APPLICATION_SUPERAWESOME,
    PACE2_APPLICATION_YANDEX_TRANSLATE,
    PACE2_APPLICATION_CASHI,
    PACE2_APPLICATION_BOOKING_HOLDINGS_SERVICES,
    PACE2_APPLICATION_BANCO_AZTECA,
    PACE2_APPLICATION_GSTATIC,
    PACE2_APPLICATION_DIDI_FOOD,
    PACE2_APPLICATION_DIDI_RIDER,
    PACE2_APPLICATION_FACEBOOK_MESSENGER,
    PACE2_APPLICATION_COPPEL,
    PACE2_APPLICATION_UNCLASSIFIED,
    PACE2_APPLICATION_OKCUPID,
    PACE2_APPLICATION_MERCADO_LIBRE,
    PACE2_APPLICATION_BAND,
    PACE2_APPLICATION_MERCADO_PAGO,
    PACE2_APPLICATION_SMILEGATE,
    PACE2_APPLICATION_BANCOPPEL,
    PACE2_APPLICATION_V4,
    PACE2_APPLICATION_VLMEDIA_SERVICES,
    PACE2_APPLICATION_MICROSOFT_TRANSLATOR,
    PACE2_APPLICATION_CDNJS,
    PACE2_APPLICATION_TELEMETRO,
    PACE2_APPLICATION_CORPORACION_MEDCOM,
    PACE2_APPLICATION_HERE_TV,
    PACE2_APPLICATION_OVERWATCH,
    PACE2_APPLICATION_SNOW_SERVICES,
    PACE2_APPLICATION_MASTODON,
    PACE2_APPLICATION_GLU_MOBILE_SERVICES,
    PACE2_APPLICATION_KARTRIDER_RUSH,
    PACE2_APPLICATION_GROUPM,
    PACE2_APPLICATION_ULTRAVIEWER,
    PACE2_APPLICATION_KINGDOM_AGE,
    PACE2_APPLICATION_STAR_TREK_ONLINE,
    PACE2_APPLICATION_WEB_DE,
    PACE2_APPLICATION_MAIL_COM,
    PACE2_APPLICATION_ARTE,
    PACE2_APPLICATION_VITA,
    PACE2_APPLICATION_PARAMOUNT_SERVICES,
    PACE2_APPLICATION_BLUTV,
    PACE2_APPLICATION_XIAOHONGSHU,
    PACE2_APPLICATION_RAKUTEN_TV,
    PACE2_APPLICATION_APPLE_WEATHER,
    PACE2_APPLICATION_MUBI,
    PACE2_APPLICATION_QUANTUMCAST,
    PACE2_APPLICATION_KNIGHTS_AND_DRAGONS,
    PACE2_APPLICATION_SMITE,
    PACE2_APPLICATION_TELEPORT_MEDIA,
    PACE2_APPLICATION_CBS_NEWS,
    PACE2_APPLICATION_CBS_SPORTS,
    PACE2_APPLICATION_RBC_NEWS,
    PACE2_APPLICATION_CANONICAL_SERVICES,
    PACE2_APPLICATION_EXXEN,
    PACE2_APPLICATION_SONY_GROUP_SERVICES,
    PACE2_APPLICATION_SOUL_HUNTERS,
    PACE2_APPLICATION_ADTELLIGENT,
    PACE2_APPLICATION_DATADOG,
    PACE2_APPLICATION_SEGMENTO_RU,
    PACE2_APPLICATION_POLYFILL_IO,
    PACE2_APPLICATION_B612,
    PACE2_APPLICATION_WARPATH_ACE_SHOOTER,
    PACE2_APPLICATION_TOWER_OF_SAVIORS,
    PACE2_APPLICATION_LEMON8,
    PACE2_APPLICATION_COVEO,
    PACE2_APPLICATION_HYPIC,
    PACE2_APPLICATION_ESCAPE_FROM_TARKOV,
    PACE2_APPLICATION_POCKET,
    PACE2_APPLICATION_MARVEL_SNAP,
    PACE2_APPLICATION_ADOBE_DOCUMENT_CLOUD,
    PACE2_APPLICATION_ZANGI_MESSENGER,
    PACE2_APPLICATION_BEAMABLE,
    PACE2_APPLICATION_BUNNY_NET,
    PACE2_APPLICATION_BCE_SERVICES,
    PACE2_APPLICATION_CRAVE,
    PACE2_APPLICATION_BYTEPLUS,
    PACE2_APPLICATION_PLARIUM,
    PACE2_APPLICATION_SPARTA_WAR_OF_EMPIRES,
    PACE2_APPLICATION_SBER,
    PACE2_APPLICATION_JIVOCHAT,
    PACE2_APPLICATION_YOUTUBE_KIDS,
    PACE2_APPLICATION_OPPO_MOBILE_SERVICES,
    PACE2_APPLICATION_HEYTAP,
    PACE2_APPLICATION_MAX,
    PACE2_APPLICATION_WORLD_OF_WARSHIPS,
    PACE2_APPLICATION_YOUTUBE_TV,
    PACE2_APPLICATION_GOOGLE_FAMILY_LINK,
    PACE2_APPLICATION_CELTIC_HEROES,
    PACE2_APPLICATION_HERO_HUNTERS,
    PACE2_APPLICATION_TIMES_INTERNET_SERVICES,
    PACE2_APPLICATION_SERVICENOW,
    PACE2_APPLICATION_HONKAI_STAR_RAIL,
    PACE2_APPLICATION_ELDER_SCROLLS_ONLINE,
    PACE2_APPLICATION_NUVERSE,
    PACE2_APPLICATION_TVPLUS,
    PACE2_APPLICATION_NASDAQ_SERVICES,
    PACE2_APPLICATION_SESISOFT_SERVICES,
    PACE2_APPLICATION_TIVIBU_GO,
    PACE2_APPLICATION_DUNGEON_FIGHTER_ONLINE,
    PACE2_APPLICATION_5GMARK,
    PACE2_APPLICATION_KILL_SHOT_BRAVO,
    PACE2_APPLICATION_PUHUTV,
    PACE2_APPLICATION_DOGUS_SERVICES,
    PACE2_APPLICATION_OPENAI,
    PACE2_APPLICATION_QOBUZ,
    PACE2_APPLICATION_S_SPORT_PLUS,
    PACE2_APPLICATION_WORLD_OF_TANKS_BLITZ,

    PACE2_APPLICATIONS_COUNT

} PACE2_applications;

typedef PACE2_DEPRECATED("Application is marked as deprecated and should not be used anymore.") PACE2_applications PACE2_deprecated_applications;
#define PACE2_APPLICATION_REALMEDIA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_9)
#define PACE2_APPLICATION_GOOBER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_11)
#define PACE2_APPLICATION_UBUNTUONE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_13)
#define PACE2_APPLICATION_YUILOP ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_15)
#define PACE2_APPLICATION_WINDOWS_AZURE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_19)
#define PACE2_APPLICATION_GROOVESHARK ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_23)
#define PACE2_APPLICATION_SUDAPHONE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_24)
#define PACE2_APPLICATION_SINATV ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_27)
#define PACE2_APPLICATION_SKYDRIVE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_35)
#define PACE2_APPLICATION_ORKUT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_38)
#define PACE2_APPLICATION_HOTMAIL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_42)
#define PACE2_APPLICATION_TIGERTEXT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_46)
#define PACE2_APPLICATION_VIBE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_47)
#define PACE2_APPLICATION_ENLEGION ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_48)
#define PACE2_APPLICATION_LOVEFILM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_50)
#define PACE2_APPLICATION_FRIENDSTER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_51)
#define PACE2_APPLICATION_LIVE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_54)
#define PACE2_APPLICATION_ICQ_WEBCHAT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_55)
#define PACE2_APPLICATION_YOUKU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_62)
#define PACE2_APPLICATION_ITUNES_RADIO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_77)
#define PACE2_APPLICATION_FORFONE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_79)
#define PACE2_APPLICATION_WATCHEVER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_86)
#define PACE2_APPLICATION_360MOBILE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_88)
#define PACE2_APPLICATION_VINE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_90)
#define PACE2_APPLICATION_ILIVE_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_93)
#define PACE2_APPLICATION_DOOOK ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_95)
#define PACE2_APPLICATION_NORTON_ANTIVIRUS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_102)
#define PACE2_APPLICATION_RDIO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_106)
#define PACE2_APPLICATION_DOTA_LEGEND ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_113)
#define PACE2_APPLICATION_ENDOMONDO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_121)
#define PACE2_APPLICATION_TIMELY_TV ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_122)
#define PACE2_APPLICATION_MAP_MY_RUN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_124)
#define PACE2_APPLICATION_KING_OF_PIRATES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_125)
#define PACE2_APPLICATION_TUDOU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_126)
#define PACE2_APPLICATION_FASTTRACK ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_129)
#define PACE2_APPLICATION_MUTE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_136)
#define PACE2_APPLICATION_MSN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_138)
#define PACE2_APPLICATION_OSCAR ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_141)
#define PACE2_APPLICATION_IRC ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_144)
#define PACE2_APPLICATION_SLINGBOX ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_145)
#define PACE2_APPLICATION_VCAST ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_146)
#define PACE2_APPLICATION_DDL_ADRIVE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_148)
#define PACE2_APPLICATION_DDL_BADONGO_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_149)
#define PACE2_APPLICATION_DDL_BADONGO_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_150)
#define PACE2_APPLICATION_DDL_BESTSHARING_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_151)
#define PACE2_APPLICATION_DDL_BIGFILEZ_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_152)
#define PACE2_APPLICATION_DDL_CHINAMOFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_153)
#define PACE2_APPLICATION_DDL_EASY_SHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_156)
#define PACE2_APPLICATION_DDL_FAST_LOAD_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_157)
#define PACE2_APPLICATION_DDL_FAST_SHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_158)
#define PACE2_APPLICATION_DDL_FILE_UPLOAD_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_160)
#define PACE2_APPLICATION_DDL_FILEBASE_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_162)
#define PACE2_APPLICATION_DDL_FILECLOUD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_163)
#define PACE2_APPLICATION_DDL_FILEFRONT_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_165)
#define PACE2_APPLICATION_DDL_FILEHO_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_166)
#define PACE2_APPLICATION_DDL_FILEHOST_TV ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_167)
#define PACE2_APPLICATION_DDL_FILEPOST_RU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_168)
#define PACE2_APPLICATION_DDL_FILES_UPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_170)
#define PACE2_APPLICATION_DDL_FILES_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_171)
#define PACE2_APPLICATION_DDL_FILESAFE_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_172)
#define PACE2_APPLICATION_DDL_FILESEND_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_173)
#define PACE2_APPLICATION_DDL_FILEUPYOURS_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_174)
#define PACE2_APPLICATION_DDL_FTP2SHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_175)
#define PACE2_APPLICATION_DDL_GIGESHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_177)
#define PACE2_APPLICATION_DDL_HOSTGGG_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_178)
#define PACE2_APPLICATION_DDL_HOTFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_179)
#define PACE2_APPLICATION_DDL_IFOLDER_RU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_180)
#define PACE2_APPLICATION_DDL_KEEPMYFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_181)
#define PACE2_APPLICATION_DDL_LIVE_SHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_183)
#define PACE2_APPLICATION_DDL_LIVEDEPOT_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_184)
#define PACE2_APPLICATION_DDL_MASSMIRROR_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_186)
#define PACE2_APPLICATION_DDL_MEGAROTIC_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_188)
#define PACE2_APPLICATION_DDL_MEGASHARES_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_190)
#define PACE2_APPLICATION_DDL_MEGAUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_191)
#define PACE2_APPLICATION_DDL_MOFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_192)
#define PACE2_APPLICATION_DDL_MOFILE_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_193)
#define PACE2_APPLICATION_DDL_MYTEMPDIR_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_194)
#define PACE2_APPLICATION_DDL_NETLOAD_IN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_195)
#define PACE2_APPLICATION_DDL_ODSIEBIE_NAJLEPSZE_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_196)
#define PACE2_APPLICATION_DDL_QUICKSHARING_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_197)
#define PACE2_APPLICATION_DDL_RAPIDSHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_198)
#define PACE2_APPLICATION_DDL_RAPIDSHARE_DE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_199)
#define PACE2_APPLICATION_DDL_RAPIDUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_200)
#define PACE2_APPLICATION_DDL_SAVEFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_201)
#define PACE2_APPLICATION_DDL_SENDMEFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_202)
#define PACE2_APPLICATION_DDL_SHAREBASE_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_204)
#define PACE2_APPLICATION_DDL_SHAREBEE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_205)
#define PACE2_APPLICATION_DDL_SHAREBIG_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_206)
#define PACE2_APPLICATION_DDL_SHAREBIGFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_207)
#define PACE2_APPLICATION_DDL_SHAREBIGFLIE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_208)
#define PACE2_APPLICATION_DDL_SIMPLEUPLOAD_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_209)
#define PACE2_APPLICATION_DDL_SPEEDSHARE_ORG ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_210)
#define PACE2_APPLICATION_DDL_TURBOUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_211)
#define PACE2_APPLICATION_DDL_ULTRASHARE_DE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_212)
#define PACE2_APPLICATION_DDL_UP_FILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_213)
#define PACE2_APPLICATION_DDL_UPLOADED_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_214)
#define PACE2_APPLICATION_DDL_UPLOADING_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_215)
#define PACE2_APPLICATION_DDL_UPLOADINGIT_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_216)
#define PACE2_APPLICATION_DDL_UPLOADPOWER_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_217)
#define PACE2_APPLICATION_DDL_UPLOADYOURFILES_DE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_218)
#define PACE2_APPLICATION_DDL_VIP_FILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_219)
#define PACE2_APPLICATION_DDL_WIIUPLOAD_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_220)
#define PACE2_APPLICATION_DDL_YOURFILEHOST_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_221)
#define PACE2_APPLICATION_DDL_YOURFILES_BIZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_222)
#define PACE2_APPLICATION_DDL_YOUTUBE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_223)
#define PACE2_APPLICATION_DDL_ZSHARE_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_224)
#define PACE2_APPLICATION_DDL_4SHARED_VN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_225)
#define PACE2_APPLICATION_DDL_MEGA_1280_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_226)
#define PACE2_APPLICATION_DDL_MEGASHARE_VN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_227)
#define PACE2_APPLICATION_DDL_FILESERVE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_228)
#define PACE2_APPLICATION_DDL_FILESONIC_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_229)
#define PACE2_APPLICATION_DDL_1_UPLOAD_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_230)
#define PACE2_APPLICATION_DDL_4FASTFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_232)
#define PACE2_APPLICATION_DDL_10UPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_233)
#define PACE2_APPLICATION_DDL_1CLICKSHARE_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_234)
#define PACE2_APPLICATION_DDL_FILEDUDE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_235)
#define PACE2_APPLICATION_DDL_ASIXFILES_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_236)
#define PACE2_APPLICATION_DDL_BANASHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_237)
#define PACE2_APPLICATION_DDL_BIGGERUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_238)
#define PACE2_APPLICATION_DDL_BITSHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_239)
#define PACE2_APPLICATION_DDL_CASH_FILE_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_240)
#define PACE2_APPLICATION_DDL_CRAMIT_IN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_241)
#define PACE2_APPLICATION_DDL_CZSHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_242)
#define PACE2_APPLICATION_DDL_DATA_LOADING_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_243)
#define PACE2_APPLICATION_DDL_DATAPORT_CZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_244)
#define PACE2_APPLICATION_DDL_DATEI_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_245)
#define PACE2_APPLICATION_DDL_DOWNUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_246)
#define PACE2_APPLICATION_DDL_EDISK_CZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_248)
#define PACE2_APPLICATION_DDL_ENTERUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_249)
#define PACE2_APPLICATION_DDL_EUROSHARE_EU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_250)
#define PACE2_APPLICATION_DDL_EXTABIT_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_251)
#define PACE2_APPLICATION_DDL_FIK1_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_253)
#define PACE2_APPLICATION_DDL_FILE2UPLOAD_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_254)
#define PACE2_APPLICATION_DDL_FILEAPE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_255)
#define PACE2_APPLICATION_DDL_FILEFLYER_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_256)
#define PACE2_APPLICATION_DDL_FILEFROG_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_257)
#define PACE2_APPLICATION_DDL_FILEHOOK_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_258)
#define PACE2_APPLICATION_DDL_FILEOVER_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_259)
#define PACE2_APPLICATION_DDL_FILEPOST_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_260)
#define PACE2_APPLICATION_DDL_FILES_MAIL_RU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_261)
#define PACE2_APPLICATION_DDL_FILESHARE_IN_UA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_262)
#define PACE2_APPLICATION_DDL_FILEVO_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_264)
#define PACE2_APPLICATION_DDL_FILEZUP_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_265)
#define PACE2_APPLICATION_DDL_FREAKSHARE_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_266)
#define PACE2_APPLICATION_DDL_GAIAFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_268)
#define PACE2_APPLICATION_DDL_GOTOUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_270)
#define PACE2_APPLICATION_DDL_HELLSHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_271)
#define PACE2_APPLICATION_DDL_I_FILEZ_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_275)
#define PACE2_APPLICATION_DDL_IFILE_IT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_276)
#define PACE2_APPLICATION_DDL_KEWLSHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_277)
#define PACE2_APPLICATION_DDL_LETITBIT_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_278)
#define PACE2_APPLICATION_DDL_LOADFILES_IN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_279)
#define PACE2_APPLICATION_DDL_MEGAFREE_KZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_280)
#define PACE2_APPLICATION_DDL_MEGARAPID_EU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_281)
#define PACE2_APPLICATION_DDL_MEGASHARE_VNN_VN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_282)
#define PACE2_APPLICATION_DDL_MIDUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_283)
#define PACE2_APPLICATION_DDL_NETUPLOADED_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_285)
#define PACE2_APPLICATION_DDL_ORON_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_286)
#define PACE2_APPLICATION_DDL_PRZEKLEJ_PL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_287)
#define PACE2_APPLICATION_DDL_PUTSHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_288)
#define PACE2_APPLICATION_DDL_QUICKSHARE_CZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_290)
#define PACE2_APPLICATION_DDL_RAPIDSHARK_PL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_291)
#define PACE2_APPLICATION_DDL_SHARE_ONLINE_BZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_293)
#define PACE2_APPLICATION_DDL_SHARE_RAPID_CZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_294)
#define PACE2_APPLICATION_DDL_SHRAGLE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_295)
#define PACE2_APPLICATION_DDL_STORAGE_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_296)
#define PACE2_APPLICATION_DDL_TERAFILES_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_297)
#define PACE2_APPLICATION_DDL_TUNABOX_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_298)
#define PACE2_APPLICATION_DDL_UGOTFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_300)
#define PACE2_APPLICATION_DDL_UNIBYTES_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_302)
#define PACE2_APPLICATION_DDL_UNLIMIT_CO_IL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_303)
#define PACE2_APPLICATION_DDL_UPLOAD_COM_UA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_305)
#define PACE2_APPLICATION_DDL_UPLOADBOX_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_306)
#define PACE2_APPLICATION_DDL_UPLOADER_PL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_307)
#define PACE2_APPLICATION_DDL_UPLOADFLOOR_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_308)
#define PACE2_APPLICATION_DDL_UPLOADSTATION_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_309)
#define PACE2_APPLICATION_DDL_UPNITO_SK ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_310)
#define PACE2_APPLICATION_DDL_USERSHARE_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_311)
#define PACE2_APPLICATION_DDL_VIDEOBB_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_312)
#define PACE2_APPLICATION_DDL_WUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_313)
#define PACE2_APPLICATION_DDL_X7_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_314)
#define PACE2_APPLICATION_DDL_YUNFILE_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_315)
#define PACE2_APPLICATION_DDL_ZOMGUPLOAD_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_316)
#define PACE2_APPLICATION_DDL_1_UPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_318)
#define PACE2_APPLICATION_DDL_1_CLICKSHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_319)
#define PACE2_APPLICATION_DDL_123UPLOAD_PL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_320)
#define PACE2_APPLICATION_DDL_139PAN_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_321)
#define PACE2_APPLICATION_DDL_163PAN_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_322)
#define PACE2_APPLICATION_DDL_4SHARE_WS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_324)
#define PACE2_APPLICATION_DDL_4US_TO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_325)
#define PACE2_APPLICATION_DDL_ALLDRIVES_GE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_326)
#define PACE2_APPLICATION_DDL_ALLSHARES_GE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_327)
#define PACE2_APPLICATION_DDL_ANNONHOST_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_328)
#define PACE2_APPLICATION_DDL_BOOSTERKING_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_329)
#define PACE2_APPLICATION_DDL_COBRASHARE_SK ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_330)
#define PACE2_APPLICATION_DDL_COOLSHARE_CZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_331)
#define PACE2_APPLICATION_DDL_CROCKO_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_332)
#define PACE2_APPLICATION_DDL_CUM_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_333)
#define PACE2_APPLICATION_DDL_EYVX_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_334)
#define PACE2_APPLICATION_DDL_FILEARN_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_335)
#define PACE2_APPLICATION_DDL_FILEJUNGLE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_336)
#define PACE2_APPLICATION_DDL_FILEMAZE_WS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_337)
#define PACE2_APPLICATION_DDL_FILESFLASH_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_338)
#define PACE2_APPLICATION_DDL_FREESPACE_BY ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_339)
#define PACE2_APPLICATION_DDL_FSX_HU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_340)
#define PACE2_APPLICATION_DDL_JAKFILE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_341)
#define PACE2_APPLICATION_DDL_JUMBOFILES_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_342)
#define PACE2_APPLICATION_DDL_KICKLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_343)
#define PACE2_APPLICATION_DDL_MISSUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_344)
#define PACE2_APPLICATION_DDL_MOVREEL_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_345)
#define PACE2_APPLICATION_DDL_PLATINSHARE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_346)
#define PACE2_APPLICATION_DDL_SANUPLOAD_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_347)
#define PACE2_APPLICATION_DDL_SHAREFLARE_NET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_348)
#define PACE2_APPLICATION_DDL_U_115_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_349)
#define PACE2_APPLICATION_DDL_UPLOADC_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_350)
#define PACE2_APPLICATION_DDL_UPLOADKING_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_351)
#define PACE2_APPLICATION_DDL_FILESTUBE_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_353)
#define PACE2_APPLICATION_DDL_MEGAPORN_COM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_355)
#define PACE2_APPLICATION_DDL_FILESONIC_IN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_356)
#define PACE2_APPLICATION_FTP ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_359)
#define PACE2_APPLICATION_MANOLITO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_361)
#define PACE2_APPLICATION_PANDO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_363)
#define PACE2_APPLICATION_SOPCAST ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_364)
#define PACE2_APPLICATION_TVUPLAYER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_365)
#define PACE2_APPLICATION_PPSTREAM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_366)
#define PACE2_APPLICATION_PPLIVE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_367)
#define PACE2_APPLICATION_FEIDIAN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_371)
#define PACE2_APPLICATION_PCANYWHERE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_390)
#define PACE2_APPLICATION_OPENFT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_395)
#define PACE2_APPLICATION_OOVOO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_399)
#define PACE2_APPLICATION_GOOGLE_HANGOUTS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_400)
#define PACE2_APPLICATION_ISKOOT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_401)
#define PACE2_APPLICATION_FRING ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_402)
#define PACE2_APPLICATION_MEEBO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_406)
#define PACE2_APPLICATION_CLUBPENGUIN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_409)
#define PACE2_APPLICATION_OSH ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_413)
#define PACE2_APPLICATION_PS3 ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_414)
#define PACE2_APPLICATION_JABBER_APPLICATION_NIMBUZZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_419)
#define PACE2_APPLICATION_WAP_WSP ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_421)
#define PACE2_APPLICATION_ANTSP2P ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_430)
#define PACE2_APPLICATION_SCYDO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_437)
#define PACE2_APPLICATION_MOJO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_441)
#define PACE2_APPLICATION_FICALL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_444)
#define PACE2_APPLICATION_COMODOUNITE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_445)
#define PACE2_APPLICATION_MYPEOPLE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_447)
#define PACE2_APPLICATION_JAP ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_450)
#define PACE2_APPLICATION_WUALA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_451)
#define PACE2_APPLICATION_LYNC ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_453)
#define PACE2_APPLICATION_AUDIOGALAXY ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_454)
#define PACE2_APPLICATION_MIG33 ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_455)
#define PACE2_APPLICATION_HTTP_TUNNEL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_456)
#define PACE2_APPLICATION_RHAPSODY ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_457)
#define PACE2_APPLICATION_WEBQQ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_458)
#define PACE2_APPLICATION_GOOGLE_PLUS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_464)
#define PACE2_APPLICATION_DEMAND5 ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_467)
#define PACE2_APPLICATION_RSVP ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_468)
#define PACE2_APPLICATION_LOTUS_NOTES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_470)
#define PACE2_APPLICATION_MYVIDEO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_474)
#define PACE2_APPLICATION_DIDI ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_480)
#define PACE2_APPLICATION_SPREECAST ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_482)
#define PACE2_APPLICATION_SIMFY ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_483)
#define PACE2_APPLICATION_NTLM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_494)
#define PACE2_APPLICATION_MEI_LU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_501)
#define PACE2_APPLICATION_PLAYTALES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_503)
#define PACE2_APPLICATION_NATIONAL_BASEBALL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_507)
#define PACE2_APPLICATION_CIAOMOBILE_HDTV ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_511)
#define PACE2_APPLICATION_SKYKING ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_512)
#define PACE2_APPLICATION_MOVES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_513)
#define PACE2_APPLICATION_PATH ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_514)
#define PACE2_APPLICATION_REALMA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_520)
#define PACE2_APPLICATION_SPEAKABOOS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_525)
#define PACE2_APPLICATION_DISNEY_STORYTIME ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_535)
#define PACE2_APPLICATION_MAGINE_TV ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_539)
#define PACE2_APPLICATION_PERISCOPE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_548)
#define PACE2_APPLICATION_MEERKAT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_567)
#define PACE2_APPLICATION_SLACKER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_579)
#define PACE2_APPLICATION_USTREAM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_582)
#define PACE2_APPLICATION_CRASHPLAN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_593)
#define PACE2_APPLICATION_AFTER_SCHOOL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_601)
#define PACE2_APPLICATION_QIP_RU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_603)
#define PACE2_APPLICATION_MUSICAL_LY ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_604)
#define PACE2_APPLICATION_ITUNESU ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_606)
#define PACE2_APPLICATION_OUTLOOK_MOBILE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_612)
#define PACE2_APPLICATION_CHATCUBE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_613)
#define PACE2_APPLICATION_BETTERNET ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_617)
#define PACE2_APPLICATION_GOOD ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_618)
#define PACE2_APPLICATION_2CHANNEL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_626)
#define PACE2_APPLICATION_CLASH_ROYAL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_628)
#define PACE2_APPLICATION_UPPTALK ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_629)
#define PACE2_APPLICATION_KANKAN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_634)
#define PACE2_APPLICATION_DDL_SHARE_ONLINE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_635)
#define PACE2_APPLICATION_DDL_ULTRASHARE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_636)
#define PACE2_APPLICATION_DDL_YUNFILE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_637)
#define PACE2_APPLICATION_DDL_123UPLOAD ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_639)
#define PACE2_APPLICATION_IO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_642)
#define PACE2_APPLICATION_ICRYPT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_644)
#define PACE2_APPLICATION_AIM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_645)
#define PACE2_APPLICATION_DIGICEL_MUSIC ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_673)
#define PACE2_APPLICATION_DUBSMASH ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_676)
#define PACE2_APPLICATION_GOOGLE_ALLO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_679)
#define PACE2_APPLICATION_DDL_RUSFOLDER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_680)
#define PACE2_APPLICATION_LIMELIGHT_CLOUD ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_695)
#define PACE2_APPLICATION_VEVO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_705)
#define PACE2_APPLICATION_GOOGLE_DUO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_711)
#define PACE2_APPLICATION_SARAHAH ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_714)
#define PACE2_APPLICATION_HOUSEPARTY ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_724)
#define PACE2_APPLICATION_LOOP_TRADE_CLASSIFIEDS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_734)
#define PACE2_APPLICATION_SYMANTEC ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_744)
#define PACE2_APPLICATION_MAXDOME ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_745)
#define PACE2_APPLICATION_ZERO_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_751)
#define PACE2_APPLICATION_SURFEASY ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_752)
#define PACE2_APPLICATION_VPN_MASTER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_758)
#define PACE2_APPLICATION_TOKBOX ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_761)
#define PACE2_APPLICATION_HMA_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_763)
#define PACE2_APPLICATION_EASY_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_784)
#define PACE2_APPLICATION_BUFFERED_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_786)
#define PACE2_APPLICATION_BBM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_794)
#define PACE2_APPLICATION_SPEED_VPN_INVENTIVE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_795)
#define PACE2_APPLICATION_TOUCH_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_802)
#define PACE2_APPLICATION_VPN_IN_TOUCH ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_806)
#define PACE2_APPLICATION_SPEED_VPN_MAD ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_807)
#define PACE2_APPLICATION_DASH_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_808)
#define PACE2_APPLICATION_IBVPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_811)
#define PACE2_APPLICATION_ATOM_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_816)
#define PACE2_APPLICATION_VPN_EASY ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_827)
#define PACE2_APPLICATION_ZPN_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_828)
#define PACE2_APPLICATION_LINKVPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_833)
#define PACE2_APPLICATION_ENMASSE_GAMES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_836)
#define PACE2_APPLICATION_HOTSPOT_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_837)
#define PACE2_APPLICATION_TUNVPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_863)
#define PACE2_APPLICATION_MIXER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_869)
#define PACE2_APPLICATION_OPEN_SUPER_HOTSPOT_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_871)
#define PACE2_APPLICATION_VPN_PROXY_MASTER_UNLIMITED ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_872)
#define PACE2_APPLICATION_7TV ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_879)
#define PACE2_APPLICATION_EUROSPORT_PLAYER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_882)
#define PACE2_APPLICATION_WIZARDS_UNITE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_887)
#define PACE2_APPLICATION_INVENTIVE_VPN_SERVICES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_908)
#define PACE2_APPLICATION_SOMA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_911)
#define PACE2_APPLICATION_HEYZAP ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_936)
#define PACE2_APPLICATION_ACCOUNTKIT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_940)
#define PACE2_APPLICATION_LEVEL3_CLOUD ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_944)
#define PACE2_APPLICATION_VERIZON_DIGITAL_MEDIA_SERVICES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_951)
#define PACE2_APPLICATION_STREAMCRAFT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_952)
#define PACE2_APPLICATION_WEGAMERS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_959)
#define PACE2_APPLICATION_RUBICON_PROJECT ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_963)
#define PACE2_APPLICATION_STREAMTHEWORLD ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_967)
#define PACE2_APPLICATION_HBO_NOW ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_970)
#define PACE2_APPLICATION_DISNEY_CHANNEL ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_977)
#define PACE2_APPLICATION_DISCOVERY_COMMUNICATIONS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_978)
#define PACE2_APPLICATION_ADRISE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1005)
#define PACE2_APPLICATION_YOUTUBE_GO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1020)
#define PACE2_APPLICATION_SYMANTEC_VPN_SERVICES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1021)
#define PACE2_APPLICATION_PLADFORM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1035)
#define PACE2_APPLICATION_AMAZON_PRIME_NOW ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1040)
#define PACE2_APPLICATION_APPNEXUS ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1049)
#define PACE2_APPLICATION_PLAYSTATION_VIDEO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1060)
#define PACE2_APPLICATION_CISCO_SERVICES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1065)
#define PACE2_APPLICATION_TOPBUZZ ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1079)
#define PACE2_APPLICATION_NBC_SPORTS_SCORES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1080)
#define PACE2_APPLICATION_HBO_MAX ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1113)
#define PACE2_APPLICATION_WARNERMEDIA_SERVICES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1114)
#define PACE2_APPLICATION_SPOT_IM ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1124)
#define PACE2_APPLICATION_APPSEE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1154)
#define PACE2_APPLICATION_ZYNN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1192)
#define PACE2_APPLICATION_CURALATE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1196)
#define PACE2_APPLICATION_CXENSE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1205)
#define PACE2_APPLICATION_FYEO ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1261)
#define PACE2_APPLICATION_LUMINATI ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1308)
#define PACE2_APPLICATION_ARC_PUBLISHING ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1324)
#define PACE2_APPLICATION_LUMEN_TECH ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1357)
#define PACE2_APPLICATION_VIACOM_CBS_SERVICES ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1412)
#define PACE2_APPLICATION_VRV ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1436)
#define PACE2_APPLICATION_SNAIL_VPN ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1467)
#define PACE2_APPLICATION_ALPHABETTY_SAGA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1534)
#define PACE2_APPLICATION_PAPA_PEAR_SAGA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1544)
#define PACE2_APPLICATION_DIAMOND_DIGGER_SAGA ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1545)
#define PACE2_APPLICATION_DIVANTV ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1594)
#define PACE2_APPLICATION_LIVEXLIVE ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1599)
#define PACE2_APPLICATION_TNT_PREMIER ((PACE2_deprecated_applications) PACE2_APPLICATION_RES_1856)

/**
 * @brief Struct representing all available applications as a bitmask.
 *
 * @deprecated The usage of this struct is deprecated, because of Dynamic Upgrade incompatibility.
**/
typedef struct {
#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 ukn:1;                      //!< unknown
    u32 facebook:1;                 //!< Facebook
    u32 magicjack:1;                //!< magicJack
    u32 itunes:1;                   //!< iTunes
    u32 myspace:1;                  //!< MySpace
    u32 facetime:1;                 //!< FaceTime
    u32 truphone:1;                 //!< Truphone
    u32 twitter:1;                  //!< Twitter
    u32 xbox:1;                     //!< XBOX gaming console
    PACE2_DEPRECATED("Application RealMedia is marked as deprecated and should not be used anymore.")
    u32 realmedia:1;                //!< RealMedia
    u32 google_mail:1;              //!< Google Mail
    PACE2_DEPRECATED("Application Goober is marked as deprecated and should not be used anymore.")
    u32 goober:1;                   //!< Goober
    u32 icloud:1;                   //!< iCloud
    PACE2_DEPRECATED("Application Ubuntu One is marked as deprecated and should not be used anymore.")
    u32 ubuntuone:1;                //!< Ubuntu One
    u32 dropbox:1;                  //!< Dropbox
    PACE2_DEPRECATED("Application Yuilop is marked as deprecated and should not be used anymore.")
    u32 yuilop:1;                   //!< Yuilop
    PACE2_DEPRECATED("Application iMessage is marked as deprecated and should not be used anymore.")
    u32 imessage:1;                 //!< iMessage
    u32 gotomypc:1;                 //!< GoToMyPC
    u32 gotomeeting:1;              //!< GoToMeeting
    u32 windows_azure:1;            //!< Windows Azure
    u32 amazon_cloud:1;             //!< Amazon Cloud
    u32 dailymotion:1;              //!< Dailymotion
    u32 deezer:1;                   //!< Deezer
    u32 grooveshark:1;              //!< Grooveshark
    u32 sudaphone:1;                //!< SUDA Phone
    u32 office365:1;                //!< Office 365
    u32 cntv:1;                     //!< CNTV
    PACE2_DEPRECATED("Application SINATV is marked as deprecated and should not be used anymore.")
    u32 sinatv:1;                   //!< SINATV
    u32 youtube:1;                  //!< YouTube
    u32 voipswitch:1;               //!< VoipSwitch
    u32 zynga:1;                    //!< Zynga
    u32 crimecity:1;                //!< Crime City
    u32 modernwar:1;                //!< Modern War
    u32 vippie:1;                   //!< Vippie
    u32 box:1;                      //!< Box
    PACE2_DEPRECATED("Application Skydrive is marked as deprecated and should not be used anymore.")
    u32 skydrive:1;                 //!< Skydrive
    u32 adobe_creative_cloud:1;     //!< Adobe Creative Cloud
    u32 linkedin:1;                 //!< LinkedIn
    u32 orkut:1;                    //!< Orkut
    u32 hi5:1;                      //!< hi5
    u32 soribada:1;                 //!< Soribada
    u32 zoho_work_online:1;         //!< Zoho Work Online
    u32 hotmail_webmail:1;          //!< Hotmail
    u32 instagram:1;                //!< Instagram
    u32 sugarsync:1;                //!< SugarSync
    u32 wickr:1;                    //!< Wickr
    u32 tigertext:1;                //!< TigerText
    u32 vibe:1;                     //!< Vibe
    u32 enlegion:1;                 //!< enLegion
    u32 voxer:1;                    //!< Voxer
    PACE2_DEPRECATED("Application LOVEFiLM is marked as deprecated and should not be used anymore.")
    u32 lovefilm:1;                 //!< LOVEFiLM
    PACE2_DEPRECATED("Application Friendster is marked as deprecated and should not be used anymore.")
    u32 friendster:1;               //!< Friendster
    u32 burner:1;                   //!< Burner
    u32 flickr:1;                   //!< Flickr
    PACE2_DEPRECATED("Application live-com is marked as deprecated and should not be used anymore.")
    u32 live_com:1;                 //!< live-com
    PACE2_DEPRECATED("Application ICQ Webchat is marked as deprecated and should not be used anymore.")
    u32 icq_webchat:1;              //!< ICQ Webchat
    u32 redtube:1;                  //!< RedTube
    u32 evernote:1;                 //!< Evernote
    u32 spreed:1;                   //!< Spreed
    u32 textplus:1;                 //!< textPlus
    u32 xvideos:1;                  //!< Xvideos
    u32 snapchat:1;                 //!< Snapchat
    PACE2_DEPRECATED("Application Youku is marked as deprecated and should not be used anymore.")
    u32 youku:1;                    //!< Youku
    u32 mymusic_tw:1;               //!< MyMusic
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 mymusic_tw:1;               //!< MyMusic
    PACE2_DEPRECATED("Application Youku is marked as deprecated and should not be used anymore.")
    u32 youku:1;                    //!< Youku
    u32 snapchat:1;                 //!< Snapchat
    u32 xvideos:1;                  //!< Xvideos
    u32 textplus:1;                 //!< textPlus
    u32 spreed:1;                   //!< Spreed
    u32 evernote:1;                 //!< Evernote
    u32 redtube:1;                  //!< RedTube
    PACE2_DEPRECATED("Application ICQ Webchat is marked as deprecated and should not be used anymore.")
    u32 icq_webchat:1;              //!< ICQ Webchat
    PACE2_DEPRECATED("Application live-com is marked as deprecated and should not be used anymore.")
    u32 live_com:1;                 //!< live-com
    u32 flickr:1;                   //!< Flickr
    u32 burner:1;                   //!< Burner
    PACE2_DEPRECATED("Application Friendster is marked as deprecated and should not be used anymore.")
    u32 friendster:1;               //!< Friendster
    PACE2_DEPRECATED("Application LOVEFiLM is marked as deprecated and should not be used anymore.")
    u32 lovefilm:1;                 //!< LOVEFiLM
    u32 voxer:1;                    //!< Voxer
    u32 enlegion:1;                 //!< enLegion
    u32 vibe:1;                     //!< Vibe
    u32 tigertext:1;                //!< TigerText
    u32 wickr:1;                    //!< Wickr
    u32 sugarsync:1;                //!< SugarSync
    u32 instagram:1;                //!< Instagram
    u32 hotmail_webmail:1;          //!< Hotmail
    u32 zoho_work_online:1;         //!< Zoho Work Online
    u32 soribada:1;                 //!< Soribada
    u32 hi5:1;                      //!< hi5
    u32 orkut:1;                    //!< Orkut
    u32 linkedin:1;                 //!< LinkedIn
    u32 adobe_creative_cloud:1;     //!< Adobe Creative Cloud
    PACE2_DEPRECATED("Application Skydrive is marked as deprecated and should not be used anymore.")
    u32 skydrive:1;                 //!< Skydrive
    u32 box:1;                      //!< Box
    u32 vippie:1;                   //!< Vippie
    u32 modernwar:1;                //!< Modern War
    u32 crimecity:1;                //!< Crime City
    u32 zynga:1;                    //!< Zynga
    u32 voipswitch:1;               //!< VoipSwitch
    u32 youtube:1;                  //!< YouTube
    PACE2_DEPRECATED("Application SINATV is marked as deprecated and should not be used anymore.")
    u32 sinatv:1;                   //!< SINATV
    u32 cntv:1;                     //!< CNTV
    u32 office365:1;                //!< Office 365
    u32 sudaphone:1;                //!< SUDA Phone
    u32 grooveshark:1;              //!< Grooveshark
    u32 deezer:1;                   //!< Deezer
    u32 dailymotion:1;              //!< Dailymotion
    u32 amazon_cloud:1;             //!< Amazon Cloud
    u32 windows_azure:1;            //!< Windows Azure
    u32 gotomeeting:1;              //!< GoToMeeting
    u32 gotomypc:1;                 //!< GoToMyPC
    PACE2_DEPRECATED("Application iMessage is marked as deprecated and should not be used anymore.")
    u32 imessage:1;                 //!< iMessage
    PACE2_DEPRECATED("Application Yuilop is marked as deprecated and should not be used anymore.")
    u32 yuilop:1;                   //!< Yuilop
    u32 dropbox:1;                  //!< Dropbox
    PACE2_DEPRECATED("Application Ubuntu One is marked as deprecated and should not be used anymore.")
    u32 ubuntuone:1;                //!< Ubuntu One
    u32 icloud:1;                   //!< iCloud
    PACE2_DEPRECATED("Application Goober is marked as deprecated and should not be used anymore.")
    u32 goober:1;                   //!< Goober
    u32 google_mail:1;              //!< Google Mail
    PACE2_DEPRECATED("Application RealMedia is marked as deprecated and should not be used anymore.")
    u32 realmedia:1;                //!< RealMedia
    u32 xbox:1;                     //!< XBOX gaming console
    u32 twitter:1;                  //!< Twitter
    u32 truphone:1;                 //!< Truphone
    u32 facetime:1;                 //!< FaceTime
    u32 myspace:1;                  //!< MySpace
    u32 itunes:1;                   //!< iTunes
    u32 magicjack:1;                //!< magicJack
    u32 facebook:1;                 //!< Facebook
    u32 ukn:1;                      //!< unknown
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 renren:1;                   //!< Renren
    u32 yelp:1;                     //!< Yelp
    u32 amazon_shop:1;              //!< Amazon shopping
    u32 youporn:1;                  //!< YouPorn
    u32 ebay:1;                     //!< eBay
    u32 pornhub:1;                  //!< PornHub
    u32 spideroak:1;                //!< Spideroak
    u32 adrive:1;                   //!< Adrive
    u32 tumblr:1;                   //!< Tumblr
    u32 xing:1;                     //!< XING
    u32 google_play_music:1;        //!< Google Play Music
    u32 vimeo:1;                    //!< Vimeo
    u32 salesforce:1;               //!< Salesforce
    u32 itunes_radio:1;             //!< iTunes Radio
    u32 sina_weibo:1;               //!< Sina Weibo
    u32 forfone:1;                  //!< Forfone
    u32 livejasmin:1;               //!< LiveJasmin
    u32 jingdong:1;                 //!< Jingdong
    u32 lastfm:1;                   //!< Last-fm
    u32 ims:1;                      //!< IP Multimedia Subsystem
    u32 baidu:1;                    //!< Baidu
    u32 taobao:1;                   //!< Taobao
    u32 watchever:1;                //!< Watchever
    u32 ttpod:1;                    //!< TTPod
    PACE2_DEPRECATED("Application 360 Mobile Security is marked as deprecated and should not be used anymore.")
    u32 _360mobile:1;               //!< 360 Mobile Security
    u32 pinterest:1;                //!< Pinterest
    u32 vine:1;                     //!< Vine
    u32 net2phone:1;                //!< Net2Phone
    u32 foursquare:1;               //!< Foursquare
    PACE2_DEPRECATED("Application iLive-to is marked as deprecated and should not be used anymore.")
    u32 ilive_to:1;                 //!< iLive-to
    u32 mega:1;                     //!< MEGA
    u32 doook:1;                    //!< Doook
    u32 vbulletin:1;                //!< vBulletin
    u32 cloudme:1;                  //!< CloudMe
    u32 yikyak:1;                   //!< Yik Yak
    u32 redmine:1;                  //!< Redmine
    u32 iheartradio:1;              //!< iHeartRadio
    u32 avira_antivir:1;            //!< Avira Antivirus
    PACE2_DEPRECATED("Application Norton Antivirus is marked as deprecated and should not be used anymore.")
    u32 norton_antivir:1;           //!< Norton Antivirus
    u32 kaspersky_antivir:1;        //!< Kaspersky Antivirus
    u32 avg_antivir:1;              //!< AVG Antivirus
    u32 soundcloud:1;               //!< SoundCloud
    PACE2_DEPRECATED("Application Rdio is marked as deprecated and should not be used anymore.")
    u32 rdio:1;                     //!< Rdio
    u32 amazon_prime_music:1;       //!< Amazon Prime Music
    u32 runkeeper:1;                //!< RunKeeper
    u32 mybook:1;                   //!< myBook
    u32 nikeplus:1;                 //!< Nike Plus
    u32 magv:1;                     //!< MagV
    u32 magv_kids:1;                //!< MagV Kids
    u32 dota_legend:1;              //!< Dota Legend
    u32 letv:1;                     //!< LeTV
    u32 my_fitness_pal:1;           //!< MyFitnessPal
    u32 sohu:1;                     //!< Sohu
    u32 _1kxun:1;                   //!< 1kxun
    u32 tvbs:1;                     //!< TVBS
    u32 xnxx:1;                     //!< XNXX
    u32 ctitv:1;                    //!< CtiTV
    u32 endomondo:1;                //!< Endomondo
    u32 timely_tv:1;                //!< Timely TV
    u32 workout_trainer:1;          //!< Workout Trainer
    u32 mapmyrun:1;                 //!< MapMyRun
    u32 king_of_pirates:1;          //!< King of Pirates
    PACE2_DEPRECATED("Application Tudou is marked as deprecated and should not be used anymore.")
    u32 tudou:1;                    //!< Tudou
    u32 bit:1;                      //!< BitTorrent
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 bit:1;                      //!< BitTorrent
    PACE2_DEPRECATED("Application Tudou is marked as deprecated and should not be used anymore.")
    u32 tudou:1;                    //!< Tudou
    u32 king_of_pirates:1;          //!< King of Pirates
    u32 mapmyrun:1;                 //!< MapMyRun
    u32 workout_trainer:1;          //!< Workout Trainer
    u32 timely_tv:1;                //!< Timely TV
    u32 endomondo:1;                //!< Endomondo
    u32 ctitv:1;                    //!< CtiTV
    u32 xnxx:1;                     //!< XNXX
    u32 tvbs:1;                     //!< TVBS
    u32 _1kxun:1;                   //!< 1kxun
    u32 sohu:1;                     //!< Sohu
    u32 my_fitness_pal:1;           //!< MyFitnessPal
    u32 letv:1;                     //!< LeTV
    u32 dota_legend:1;              //!< Dota Legend
    u32 magv_kids:1;                //!< MagV Kids
    u32 magv:1;                     //!< MagV
    u32 nikeplus:1;                 //!< Nike Plus
    u32 mybook:1;                   //!< myBook
    u32 runkeeper:1;                //!< RunKeeper
    u32 amazon_prime_music:1;       //!< Amazon Prime Music
    PACE2_DEPRECATED("Application Rdio is marked as deprecated and should not be used anymore.")
    u32 rdio:1;                     //!< Rdio
    u32 soundcloud:1;               //!< SoundCloud
    u32 avg_antivir:1;              //!< AVG Antivirus
    u32 kaspersky_antivir:1;        //!< Kaspersky Antivirus
    PACE2_DEPRECATED("Application Norton Antivirus is marked as deprecated and should not be used anymore.")
    u32 norton_antivir:1;           //!< Norton Antivirus
    u32 avira_antivir:1;            //!< Avira Antivirus
    u32 iheartradio:1;              //!< iHeartRadio
    u32 redmine:1;                  //!< Redmine
    u32 yikyak:1;                   //!< Yik Yak
    u32 cloudme:1;                  //!< CloudMe
    u32 vbulletin:1;                //!< vBulletin
    u32 doook:1;                    //!< Doook
    u32 mega:1;                     //!< MEGA
    PACE2_DEPRECATED("Application iLive-to is marked as deprecated and should not be used anymore.")
    u32 ilive_to:1;                 //!< iLive-to
    u32 foursquare:1;               //!< Foursquare
    u32 net2phone:1;                //!< Net2Phone
    u32 vine:1;                     //!< Vine
    u32 pinterest:1;                //!< Pinterest
    PACE2_DEPRECATED("Application 360 Mobile Security is marked as deprecated and should not be used anymore.")
    u32 _360mobile:1;               //!< 360 Mobile Security
    u32 ttpod:1;                    //!< TTPod
    u32 watchever:1;                //!< Watchever
    u32 taobao:1;                   //!< Taobao
    u32 baidu:1;                    //!< Baidu
    u32 ims:1;                      //!< IP Multimedia Subsystem
    u32 lastfm:1;                   //!< Last-fm
    u32 jingdong:1;                 //!< Jingdong
    u32 livejasmin:1;               //!< LiveJasmin
    u32 forfone:1;                  //!< Forfone
    u32 sina_weibo:1;               //!< Sina Weibo
    u32 itunes_radio:1;             //!< iTunes Radio
    u32 salesforce:1;               //!< Salesforce
    u32 vimeo:1;                    //!< Vimeo
    u32 google_play_music:1;        //!< Google Play Music
    u32 xing:1;                     //!< XING
    u32 tumblr:1;                   //!< Tumblr
    u32 adrive:1;                   //!< Adrive
    u32 spideroak:1;                //!< Spideroak
    u32 pornhub:1;                  //!< PornHub
    u32 ebay:1;                     //!< eBay
    u32 youporn:1;                  //!< YouPorn
    u32 amazon_shop:1;              //!< Amazon shopping
    u32 yelp:1;                     //!< Yelp
    u32 renren:1;                   //!< Renren
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 edk:1;                      //!< eDonkey
    u32 kazaa:1;                    //!< KaZaa/Fasttrack
    u32 gnu:1;                      //!< Gnutella
    u32 directconnect:1;            //!< DirectConnect
    u32 apple:1;                    //!< AppleJuice
    u32 soul:1;                     //!< Soulseek
    u32 ares:1;                     //!< Ares
    u32 freenet:1;                  //!< Freenet
    u32 mute:1;                     //!< Mute
    u32 skype:1;                    //!< Skype
    u32 msn:1;                      //!< MSN
    u32 yahoo:1;                    //!< Yahoo
    u32 vonage:1;                   //!< vonage
    u32 oscar:1;                    //!< Oscar
    u32 gadugadu:1;                 //!< Gadu-Gadu
    u32 mplus:1;                    //!< MPlus
    PACE2_DEPRECATED("Application IRC is marked as deprecated and should not be used anymore.")
    u32 irc:1;                      //!< IRC
    u32 sling:1;                    //!< Slingbox
    u32 vcast:1;                    //!< Vcast
    u32 _4shared_com:1;             //!< 4shared-com
    PACE2_DEPRECATED("Application adrive-com is marked as deprecated and should not be used anymore.")
    u32 adrive_com:1;               //!< adrive-com
    u32 badongo_com:1;              //!< badongo-com
    PACE2_DEPRECATED("Application badongo-net is marked as deprecated and should not be used anymore.")
    u32 badongo_net:1;              //!< badongo-net
    u32 bestsharing_com:1;          //!< bestsharing-com
    PACE2_DEPRECATED("Application bigfilez-com is marked as deprecated and should not be used anymore.")
    u32 bigfilez_com:1;             //!< bigfilez-com
    PACE2_DEPRECATED("Application chinamofile-com is marked as deprecated and should not be used anymore.")
    u32 chinamofile_com:1;          //!< chinamofile-com
    u32 data_hu:1;                  //!< data-hu
    u32 depositfiles_com:1;         //!< depositfiles-com
    u32 easy_share_com:1;           //!< easy-share-com
    PACE2_DEPRECATED("Application fast-load-net is marked as deprecated and should not be used anymore.")
    u32 fast_load_net:1;            //!< fast-load-net
    PACE2_DEPRECATED("Application fast-share-com is marked as deprecated and should not be used anymore.")
    u32 fast_share_com:1;           //!< fast-share-com
    u32 file_upload_net:1;          //!< file-upload-net
    PACE2_DEPRECATED("Application file-upload-to is marked as deprecated and should not be used anymore.")
    u32 file_upload_to:1;           //!< file-upload-to
    u32 filearchiv_ru:1;            //!< filearchiv-ru
    PACE2_DEPRECATED("Application filebase-to is marked as deprecated and should not be used anymore.")
    u32 filebase_to:1;              //!< filebase-to
    PACE2_DEPRECATED("Application filecloud-com is marked as deprecated and should not be used anymore.")
    u32 filecloud_com:1;            //!< filecloud-com
    u32 filefactory_com:1;          //!< filefactory-com
    PACE2_DEPRECATED("Application filefront-com is marked as deprecated and should not be used anymore.")
    u32 filefront_com:1;            //!< filefront-com
    PACE2_DEPRECATED("Application fileho-com is marked as deprecated and should not be used anymore.")
    u32 fileho_com:1;               //!< fileho-com
    u32 filehost_tv:1;              //!< filehost-tv
    u32 filepost_ru:1;              //!< filepost-ru
    u32 filer_net:1;                //!< filer-net
    PACE2_DEPRECATED("Application files-upload-com is marked as deprecated and should not be used anymore.")
    u32 files_upload_com:1;         //!< files-upload-com
    u32 files_to:1;                 //!< files-to
    PACE2_DEPRECATED("Application filesafe-to is marked as deprecated and should not be used anymore.")
    u32 filesafe_to:1;              //!< filesafe-to
    u32 filesend_net:1;             //!< filesend-net
    u32 fileupyours_com:1;          //!< fileupyours-com
    PACE2_DEPRECATED("Application ftp2share-com is marked as deprecated and should not be used anymore.")
    u32 ftp2share_com:1;            //!< ftp2share-com
    u32 gigasize_com:1;             //!< gigasize-com
    PACE2_DEPRECATED("Application gigeshare-com is marked as deprecated and should not be used anymore.")
    u32 gigeshare_com:1;            //!< gigeshare-com
    PACE2_DEPRECATED("Application hostggg-com is marked as deprecated and should not be used anymore.")
    u32 hostggg_com:1;              //!< hostggg-com
    PACE2_DEPRECATED("Application hotfile-com is marked as deprecated and should not be used anymore.")
    u32 hotfile_com:1;              //!< hotfile-com
    PACE2_DEPRECATED("Application ifolder-ru is marked as deprecated and should not be used anymore.")
    u32 ifolder_ru:1;               //!< ifolder-ru
    PACE2_DEPRECATED("Application keepmyfile-com is marked as deprecated and should not be used anymore.")
    u32 keepmyfile_com:1;           //!< keepmyfile-com
    u32 leteckaposta_cz:1;          //!< leteckaposta-cz
    PACE2_DEPRECATED("Application live-share-com is marked as deprecated and should not be used anymore.")
    u32 live_share_com:1;           //!< live-share-com
    PACE2_DEPRECATED("Application livedepot-net is marked as deprecated and should not be used anymore.")
    u32 livedepot_net:1;            //!< livedepot-net
    u32 load_to:1;                  //!< load-to
    PACE2_DEPRECATED("Application massmirror-com is marked as deprecated and should not be used anymore.")
    u32 massmirror_com:1;           //!< massmirror-com
    u32 mediafire_com:1;            //!< mediafire-com
    PACE2_DEPRECATED("Application megarotic-com is marked as deprecated and should not be used anymore.")
    u32 megarotic_com:1;            //!< megarotic-com
    u32 megashare_com:1;            //!< megashare-com
    u32 megashares_com:1;           //!< megashares-com
    u32 megaupload_com:1;           //!< megaupload-com
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 megaupload_com:1;           //!< megaupload-com
    u32 megashares_com:1;           //!< megashares-com
    u32 megashare_com:1;            //!< megashare-com
    PACE2_DEPRECATED("Application megarotic-com is marked as deprecated and should not be used anymore.")
    u32 megarotic_com:1;            //!< megarotic-com
    u32 mediafire_com:1;            //!< mediafire-com
    PACE2_DEPRECATED("Application massmirror-com is marked as deprecated and should not be used anymore.")
    u32 massmirror_com:1;           //!< massmirror-com
    u32 load_to:1;                  //!< load-to
    PACE2_DEPRECATED("Application livedepot-net is marked as deprecated and should not be used anymore.")
    u32 livedepot_net:1;            //!< livedepot-net
    PACE2_DEPRECATED("Application live-share-com is marked as deprecated and should not be used anymore.")
    u32 live_share_com:1;           //!< live-share-com
    u32 leteckaposta_cz:1;          //!< leteckaposta-cz
    PACE2_DEPRECATED("Application keepmyfile-com is marked as deprecated and should not be used anymore.")
    u32 keepmyfile_com:1;           //!< keepmyfile-com
    PACE2_DEPRECATED("Application ifolder-ru is marked as deprecated and should not be used anymore.")
    u32 ifolder_ru:1;               //!< ifolder-ru
    PACE2_DEPRECATED("Application hotfile-com is marked as deprecated and should not be used anymore.")
    u32 hotfile_com:1;              //!< hotfile-com
    PACE2_DEPRECATED("Application hostggg-com is marked as deprecated and should not be used anymore.")
    u32 hostggg_com:1;              //!< hostggg-com
    PACE2_DEPRECATED("Application gigeshare-com is marked as deprecated and should not be used anymore.")
    u32 gigeshare_com:1;            //!< gigeshare-com
    u32 gigasize_com:1;             //!< gigasize-com
    PACE2_DEPRECATED("Application ftp2share-com is marked as deprecated and should not be used anymore.")
    u32 ftp2share_com:1;            //!< ftp2share-com
    u32 fileupyours_com:1;          //!< fileupyours-com
    u32 filesend_net:1;             //!< filesend-net
    PACE2_DEPRECATED("Application filesafe-to is marked as deprecated and should not be used anymore.")
    u32 filesafe_to:1;              //!< filesafe-to
    u32 files_to:1;                 //!< files-to
    PACE2_DEPRECATED("Application files-upload-com is marked as deprecated and should not be used anymore.")
    u32 files_upload_com:1;         //!< files-upload-com
    u32 filer_net:1;                //!< filer-net
    u32 filepost_ru:1;              //!< filepost-ru
    u32 filehost_tv:1;              //!< filehost-tv
    PACE2_DEPRECATED("Application fileho-com is marked as deprecated and should not be used anymore.")
    u32 fileho_com:1;               //!< fileho-com
    PACE2_DEPRECATED("Application filefront-com is marked as deprecated and should not be used anymore.")
    u32 filefront_com:1;            //!< filefront-com
    u32 filefactory_com:1;          //!< filefactory-com
    PACE2_DEPRECATED("Application filecloud-com is marked as deprecated and should not be used anymore.")
    u32 filecloud_com:1;            //!< filecloud-com
    PACE2_DEPRECATED("Application filebase-to is marked as deprecated and should not be used anymore.")
    u32 filebase_to:1;              //!< filebase-to
    u32 filearchiv_ru:1;            //!< filearchiv-ru
    PACE2_DEPRECATED("Application file-upload-to is marked as deprecated and should not be used anymore.")
    u32 file_upload_to:1;           //!< file-upload-to
    u32 file_upload_net:1;          //!< file-upload-net
    PACE2_DEPRECATED("Application fast-share-com is marked as deprecated and should not be used anymore.")
    u32 fast_share_com:1;           //!< fast-share-com
    PACE2_DEPRECATED("Application fast-load-net is marked as deprecated and should not be used anymore.")
    u32 fast_load_net:1;            //!< fast-load-net
    u32 easy_share_com:1;           //!< easy-share-com
    u32 depositfiles_com:1;         //!< depositfiles-com
    u32 data_hu:1;                  //!< data-hu
    PACE2_DEPRECATED("Application chinamofile-com is marked as deprecated and should not be used anymore.")
    u32 chinamofile_com:1;          //!< chinamofile-com
    PACE2_DEPRECATED("Application bigfilez-com is marked as deprecated and should not be used anymore.")
    u32 bigfilez_com:1;             //!< bigfilez-com
    u32 bestsharing_com:1;          //!< bestsharing-com
    PACE2_DEPRECATED("Application badongo-net is marked as deprecated and should not be used anymore.")
    u32 badongo_net:1;              //!< badongo-net
    u32 badongo_com:1;              //!< badongo-com
    PACE2_DEPRECATED("Application adrive-com is marked as deprecated and should not be used anymore.")
    u32 adrive_com:1;               //!< adrive-com
    u32 _4shared_com:1;             //!< 4shared-com
    u32 vcast:1;                    //!< Vcast
    u32 sling:1;                    //!< Slingbox
    PACE2_DEPRECATED("Application IRC is marked as deprecated and should not be used anymore.")
    u32 irc:1;                      //!< IRC
    u32 mplus:1;                    //!< MPlus
    u32 gadugadu:1;                 //!< Gadu-Gadu
    u32 oscar:1;                    //!< Oscar
    u32 vonage:1;                   //!< vonage
    u32 yahoo:1;                    //!< Yahoo
    u32 msn:1;                      //!< MSN
    u32 skype:1;                    //!< Skype
    u32 mute:1;                     //!< Mute
    u32 freenet:1;                  //!< Freenet
    u32 ares:1;                     //!< Ares
    u32 soul:1;                     //!< Soulseek
    u32 apple:1;                    //!< AppleJuice
    u32 directconnect:1;            //!< DirectConnect
    u32 gnu:1;                      //!< Gnutella
    u32 kazaa:1;                    //!< KaZaa/Fasttrack
    u32 edk:1;                      //!< eDonkey
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    PACE2_DEPRECATED("Application mofile-com is marked as deprecated and should not be used anymore.")
    u32 mofile_com:1;               //!< mofile-com
    PACE2_DEPRECATED("Application mofile-net is marked as deprecated and should not be used anymore.")
    u32 mofile_net:1;               //!< mofile-net
    PACE2_DEPRECATED("Application mytempdir-com is marked as deprecated and should not be used anymore.")
    u32 mytempdir_com:1;            //!< mytempdir-com
    u32 netload_in:1;               //!< netload-in
    u32 odsiebie_najlepsze_net:1;   //!< odsiebie-najlepsze-net
    PACE2_DEPRECATED("Application quicksharing-com is marked as deprecated and should not be used anymore.")
    u32 quicksharing_com:1;         //!< quicksharing-com
    u32 rapidshare_com:1;           //!< rapidshare-com
    PACE2_DEPRECATED("Application rapidshare-de is marked as deprecated and should not be used anymore.")
    u32 rapidshare_de:1;            //!< rapidshare-de
    PACE2_DEPRECATED("Application rapidupload-com is marked as deprecated and should not be used anymore.")
    u32 rapidupload_com:1;          //!< rapidupload-com
    u32 savefile_com:1;             //!< savefile-com
    PACE2_DEPRECATED("Application sendmefile-com is marked as deprecated and should not be used anymore.")
    u32 sendmefile_com:1;           //!< sendmefile-com
    u32 sendspace_com:1;            //!< sendspace-com
    PACE2_DEPRECATED("Application sharebase-to is marked as deprecated and should not be used anymore.")
    u32 sharebase_to:1;             //!< sharebase-to
    PACE2_DEPRECATED("Application sharebee-com is marked as deprecated and should not be used anymore.")
    u32 sharebee_com:1;             //!< sharebee-com
    PACE2_DEPRECATED("Application sharebig-com is marked as deprecated and should not be used anymore.")
    u32 sharebig_com:1;             //!< sharebig-com
    u32 sharebigfile_com:1;         //!< sharebigfile-com
    PACE2_DEPRECATED("Application sharebigflie-com is marked as deprecated and should not be used anymore.")
    u32 sharebigflie_com:1;         //!< sharebigflie-com
    u32 simpleupload_net:1;         //!< simpleupload-net
    u32 speedshare_org:1;           //!< speedshare-org
    u32 turboupload_com:1;          //!< turboupload-com
    PACE2_DEPRECATED("Application ultrashare-de is marked as deprecated and should not be used anymore.")
    u32 ultrashare_de:1;            //!< ultrashare-de
    u32 up_file_com:1;              //!< up-file-com
    PACE2_DEPRECATED("Application uploaded-to is marked as deprecated and should not be used anymore.")
    u32 uploaded_to:1;              //!< uploaded-to
    u32 uploading_com:1;            //!< uploading-com
    u32 uploadingit_com:1;          //!< uploadingit-com
    PACE2_DEPRECATED("Application uploadpower-com is marked as deprecated and should not be used anymore.")
    u32 uploadpower_com:1;          //!< uploadpower-com
    PACE2_DEPRECATED("Application uploadyourfiles-de is marked as deprecated and should not be used anymore.")
    u32 uploadyourfiles_de:1;       //!< uploadyourfiles-de
    PACE2_DEPRECATED("Application vip-file-com is marked as deprecated and should not be used anymore.")
    u32 vip_file_com:1;             //!< vip-file-com
    PACE2_DEPRECATED("Application wiiupload-net is marked as deprecated and should not be used anymore.")
    u32 wiiupload_net:1;            //!< wiiupload-net
    u32 yourfilehost_com:1;         //!< yourfilehost-com
    PACE2_DEPRECATED("Application yourfiles-biz is marked as deprecated and should not be used anymore.")
    u32 yourfiles_biz:1;            //!< yourfiles-biz
    PACE2_DEPRECATED("Application youtube-com is marked as deprecated and should not be used anymore.")
    u32 youtube_com:1;              //!< youtube-com
    u32 zshare_net:1;               //!< zshare-net
    PACE2_DEPRECATED("Application 4shared-vn is marked as deprecated and should not be used anymore.")
    u32 _4shared_vn:1;              //!< 4shared-vn
    PACE2_DEPRECATED("Application mega-1280-com is marked as deprecated and should not be used anymore.")
    u32 mega_1280_com:1;            //!< mega-1280-com
    PACE2_DEPRECATED("Application megashare-vn is marked as deprecated and should not be used anymore.")
    u32 megashare_vn:1;             //!< megashare-vn
    u32 fileserve_com:1;            //!< fileserve-com
    u32 filesonic_com:1;            //!< filesonic-com
    PACE2_DEPRECATED("Application 1-upload-to is marked as deprecated and should not be used anymore.")
    u32 _1_upload_to:1;             //!< 1-upload-to
    u32 _1fichier_com:1;            //!< 1fichier-com
    u32 _4fastfile_com:1;           //!< 4fastfile-com
    u32 _10upload_com:1;            //!< 10upload-com
    u32 _1clickshare_net:1;         //!< 1clickshare-net
    u32 filedude_com:1;             //!< filedude-com
    u32 asixfiles_com:1;            //!< asixfiles-com
    u32 banashare_com:1;            //!< banashare-com
    u32 biggerupload_com:1;         //!< biggerupload-com
    u32 bitshare_com:1;             //!< bitshare-com
    PACE2_DEPRECATED("Application cash-file-net is marked as deprecated and should not be used anymore.")
    u32 cash_file_net:1;            //!< cash-file-net
    u32 cramit_in:1;                //!< cramit-in
    u32 czshare_com:1;              //!< czshare-com
    u32 data_loading_com:1;         //!< data-loading-com
    u32 dataport_cz:1;              //!< dataport-cz
    u32 datei_to:1;                 //!< datei-to
    u32 downupload_com:1;           //!< downupload-com
    u32 easybytez_com:1;            //!< easybytez-com
    PACE2_DEPRECATED("Application edisk-cz is marked as deprecated and should not be used anymore.")
    u32 edisk_cz:1;                 //!< edisk-cz
    u32 enterupload_com:1;          //!< enterupload-com
    u32 euroshare_eu:1;             //!< euroshare-eu
    u32 extabit_com:1;              //!< extabit-com
    u32 fastfileshare_com_ar:1;     //!< fastfileshare-com-ar
    u32 fik1_com:1;                 //!< fik1-com
    u32 file2upload_net:1;          //!< file2upload-net
    u32 fileape_com:1;              //!< fileape-com
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 fileape_com:1;              //!< fileape-com
    u32 file2upload_net:1;          //!< file2upload-net
    u32 fik1_com:1;                 //!< fik1-com
    u32 fastfileshare_com_ar:1;     //!< fastfileshare-com-ar
    u32 extabit_com:1;              //!< extabit-com
    u32 euroshare_eu:1;             //!< euroshare-eu
    u32 enterupload_com:1;          //!< enterupload-com
    PACE2_DEPRECATED("Application edisk-cz is marked as deprecated and should not be used anymore.")
    u32 edisk_cz:1;                 //!< edisk-cz
    u32 easybytez_com:1;            //!< easybytez-com
    u32 downupload_com:1;           //!< downupload-com
    u32 datei_to:1;                 //!< datei-to
    u32 dataport_cz:1;              //!< dataport-cz
    u32 data_loading_com:1;         //!< data-loading-com
    u32 czshare_com:1;              //!< czshare-com
    u32 cramit_in:1;                //!< cramit-in
    PACE2_DEPRECATED("Application cash-file-net is marked as deprecated and should not be used anymore.")
    u32 cash_file_net:1;            //!< cash-file-net
    u32 bitshare_com:1;             //!< bitshare-com
    u32 biggerupload_com:1;         //!< biggerupload-com
    u32 banashare_com:1;            //!< banashare-com
    u32 asixfiles_com:1;            //!< asixfiles-com
    u32 filedude_com:1;             //!< filedude-com
    u32 _1clickshare_net:1;         //!< 1clickshare-net
    u32 _10upload_com:1;            //!< 10upload-com
    u32 _4fastfile_com:1;           //!< 4fastfile-com
    u32 _1fichier_com:1;            //!< 1fichier-com
    PACE2_DEPRECATED("Application 1-upload-to is marked as deprecated and should not be used anymore.")
    u32 _1_upload_to:1;             //!< 1-upload-to
    u32 filesonic_com:1;            //!< filesonic-com
    u32 fileserve_com:1;            //!< fileserve-com
    PACE2_DEPRECATED("Application megashare-vn is marked as deprecated and should not be used anymore.")
    u32 megashare_vn:1;             //!< megashare-vn
    PACE2_DEPRECATED("Application mega-1280-com is marked as deprecated and should not be used anymore.")
    u32 mega_1280_com:1;            //!< mega-1280-com
    PACE2_DEPRECATED("Application 4shared-vn is marked as deprecated and should not be used anymore.")
    u32 _4shared_vn:1;              //!< 4shared-vn
    u32 zshare_net:1;               //!< zshare-net
    PACE2_DEPRECATED("Application youtube-com is marked as deprecated and should not be used anymore.")
    u32 youtube_com:1;              //!< youtube-com
    PACE2_DEPRECATED("Application yourfiles-biz is marked as deprecated and should not be used anymore.")
    u32 yourfiles_biz:1;            //!< yourfiles-biz
    u32 yourfilehost_com:1;         //!< yourfilehost-com
    PACE2_DEPRECATED("Application wiiupload-net is marked as deprecated and should not be used anymore.")
    u32 wiiupload_net:1;            //!< wiiupload-net
    PACE2_DEPRECATED("Application vip-file-com is marked as deprecated and should not be used anymore.")
    u32 vip_file_com:1;             //!< vip-file-com
    PACE2_DEPRECATED("Application uploadyourfiles-de is marked as deprecated and should not be used anymore.")
    u32 uploadyourfiles_de:1;       //!< uploadyourfiles-de
    PACE2_DEPRECATED("Application uploadpower-com is marked as deprecated and should not be used anymore.")
    u32 uploadpower_com:1;          //!< uploadpower-com
    u32 uploadingit_com:1;          //!< uploadingit-com
    u32 uploading_com:1;            //!< uploading-com
    PACE2_DEPRECATED("Application uploaded-to is marked as deprecated and should not be used anymore.")
    u32 uploaded_to:1;              //!< uploaded-to
    u32 up_file_com:1;              //!< up-file-com
    PACE2_DEPRECATED("Application ultrashare-de is marked as deprecated and should not be used anymore.")
    u32 ultrashare_de:1;            //!< ultrashare-de
    u32 turboupload_com:1;          //!< turboupload-com
    u32 speedshare_org:1;           //!< speedshare-org
    u32 simpleupload_net:1;         //!< simpleupload-net
    PACE2_DEPRECATED("Application sharebigflie-com is marked as deprecated and should not be used anymore.")
    u32 sharebigflie_com:1;         //!< sharebigflie-com
    u32 sharebigfile_com:1;         //!< sharebigfile-com
    PACE2_DEPRECATED("Application sharebig-com is marked as deprecated and should not be used anymore.")
    u32 sharebig_com:1;             //!< sharebig-com
    PACE2_DEPRECATED("Application sharebee-com is marked as deprecated and should not be used anymore.")
    u32 sharebee_com:1;             //!< sharebee-com
    PACE2_DEPRECATED("Application sharebase-to is marked as deprecated and should not be used anymore.")
    u32 sharebase_to:1;             //!< sharebase-to
    u32 sendspace_com:1;            //!< sendspace-com
    PACE2_DEPRECATED("Application sendmefile-com is marked as deprecated and should not be used anymore.")
    u32 sendmefile_com:1;           //!< sendmefile-com
    u32 savefile_com:1;             //!< savefile-com
    PACE2_DEPRECATED("Application rapidupload-com is marked as deprecated and should not be used anymore.")
    u32 rapidupload_com:1;          //!< rapidupload-com
    PACE2_DEPRECATED("Application rapidshare-de is marked as deprecated and should not be used anymore.")
    u32 rapidshare_de:1;            //!< rapidshare-de
    u32 rapidshare_com:1;           //!< rapidshare-com
    PACE2_DEPRECATED("Application quicksharing-com is marked as deprecated and should not be used anymore.")
    u32 quicksharing_com:1;         //!< quicksharing-com
    u32 odsiebie_najlepsze_net:1;   //!< odsiebie-najlepsze-net
    u32 netload_in:1;               //!< netload-in
    PACE2_DEPRECATED("Application mytempdir-com is marked as deprecated and should not be used anymore.")
    u32 mytempdir_com:1;            //!< mytempdir-com
    PACE2_DEPRECATED("Application mofile-net is marked as deprecated and should not be used anymore.")
    u32 mofile_net:1;               //!< mofile-net
    PACE2_DEPRECATED("Application mofile-com is marked as deprecated and should not be used anymore.")
    u32 mofile_com:1;               //!< mofile-com
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    PACE2_DEPRECATED("Application fileflyer-com is marked as deprecated and should not be used anymore.")
    u32 fileflyer_com:1;            //!< fileflyer-com
    PACE2_DEPRECATED("Application filefrog-com is marked as deprecated and should not be used anymore.")
    u32 filefrog_com:1;             //!< filefrog-com
    u32 filehook_com:1;             //!< filehook-com
    u32 fileover_net:1;             //!< fileover-net
    u32 filepost_com:1;             //!< filepost-com
    u32 files_mail_ru:1;            //!< files-mail-ru
    u32 fileshare_in_ua:1;          //!< fileshare-in-ua
    u32 filesmonster_com:1;         //!< filesmonster-com
    u32 filevo_com:1;               //!< filevo-com
    PACE2_DEPRECATED("Application filezup-com is marked as deprecated and should not be used anymore.")
    u32 filezup_com:1;              //!< filezup-com
    u32 freakshare_net:1;           //!< freakshare-net
    u32 fshare_vn:1;                //!< fshare-vn
    u32 gaiafile_com:1;             //!< gaiafile-com
    u32 gigapeta_com:1;             //!< gigapeta-com
    u32 gotoupload_com:1;           //!< gotoupload-com
    u32 hellshare_com:1;            //!< hellshare-com
    u32 hitfile_net:1;              //!< hitfile-net
    u32 hulkshare_com:1;            //!< hulkshare-com
    u32 hyperfileshare_com:1;       //!< hyperfileshare-com
    u32 i_filez_com:1;              //!< i-filez-com
    u32 ifile_it:1;                 //!< ifile-it
    u32 kewlshare_com:1;            //!< kewlshare-com
    u32 letitbit_net:1;             //!< letitbit-net
    u32 loadfiles_in:1;             //!< loadfiles-in
    u32 megafree_kz:1;              //!< megafree-kz
    PACE2_DEPRECATED("Application megarapid-eu is marked as deprecated and should not be used anymore.")
    u32 megarapid_eu:1;             //!< megarapid-eu
    u32 megashare_vnn_vn:1;         //!< megashare-vnn-vn
    u32 midupload_com:1;            //!< midupload-com
    u32 multishare_cz:1;            //!< multishare-cz
    u32 netuploaded_com:1;          //!< netuploaded-com
    u32 oron_com:1;                 //!< oron-com
    PACE2_DEPRECATED("Application przeklej-pl is marked as deprecated and should not be used anymore.")
    u32 przeklej_pl:1;              //!< przeklej-pl
    u32 putshare_com:1;             //!< putshare-com
    u32 qshare_com:1;               //!< qshare-com
    u32 quickshare_cz:1;            //!< quickshare-cz
    u32 rapidshark_pl:1;            //!< rapidshark-pl
    u32 scribd_com:1;               //!< scribd-com
    PACE2_DEPRECATED("Application share-online-bz is marked as deprecated and should not be used anymore.")
    u32 share_online_bz:1;          //!< share-online-bz
    u32 share_rapid_cz:1;           //!< share-rapid-cz
    u32 shragle_com:1;              //!< shragle-com
    u32 storage_to:1;               //!< storage-to
    u32 terafiles_net:1;            //!< terafiles-net
    u32 tunabox_net:1;              //!< tunabox-net
    u32 turbobit_net:1;             //!< turbobit-net
    u32 ugotfile_com:1;             //!< ugotfile-com
    u32 uloz_to:1;                  //!< uloz-to
    u32 unibytes_com:1;             //!< unibytes-com
    u32 unlimit_co_il:1;            //!< unlimit-co-il
    u32 up_4share_vn:1;             //!< up-4share-vn
    u32 upload_com_ua:1;            //!< upload-com-ua
    u32 uploadbox_com:1;            //!< uploadbox-com
    u32 uploader_pl:1;              //!< uploader-pl
    u32 uploadfloor_com:1;          //!< uploadfloor-com
    u32 uploadstation_com:1;        //!< uploadstation-com
    u32 upnito_sk:1;                //!< upnito-sk
    u32 usershare_net:1;            //!< usershare-net
    u32 videobb_com:1;              //!< videobb-com
    u32 wupload_com:1;              //!< wupload-com
    u32 x7_to:1;                    //!< x7-to
    PACE2_DEPRECATED("Application yunfile-to is marked as deprecated and should not be used anymore.")
    u32 yunfile_to:1;               //!< yunfile-to
    PACE2_DEPRECATED("Application zomgupload-to is marked as deprecated and should not be used anymore.")
    u32 zomgupload_to:1;            //!< zomgupload-to
    u32 ziddu_com:1;                //!< ziddu-com
    u32 _1_upload_com:1;            //!< 1-upload-com
    PACE2_DEPRECATED("Application 1-clickshare-com is marked as deprecated and should not be used anymore.")
    u32 _1_clickshare_com:1;        //!< 1-clickshare-com
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    PACE2_DEPRECATED("Application 1-clickshare-com is marked as deprecated and should not be used anymore.")
    u32 _1_clickshare_com:1;        //!< 1-clickshare-com
    u32 _1_upload_com:1;            //!< 1-upload-com
    u32 ziddu_com:1;                //!< ziddu-com
    PACE2_DEPRECATED("Application zomgupload-to is marked as deprecated and should not be used anymore.")
    u32 zomgupload_to:1;            //!< zomgupload-to
    PACE2_DEPRECATED("Application yunfile-to is marked as deprecated and should not be used anymore.")
    u32 yunfile_to:1;               //!< yunfile-to
    u32 x7_to:1;                    //!< x7-to
    u32 wupload_com:1;              //!< wupload-com
    u32 videobb_com:1;              //!< videobb-com
    u32 usershare_net:1;            //!< usershare-net
    u32 upnito_sk:1;                //!< upnito-sk
    u32 uploadstation_com:1;        //!< uploadstation-com
    u32 uploadfloor_com:1;          //!< uploadfloor-com
    u32 uploader_pl:1;              //!< uploader-pl
    u32 uploadbox_com:1;            //!< uploadbox-com
    u32 upload_com_ua:1;            //!< upload-com-ua
    u32 up_4share_vn:1;             //!< up-4share-vn
    u32 unlimit_co_il:1;            //!< unlimit-co-il
    u32 unibytes_com:1;             //!< unibytes-com
    u32 uloz_to:1;                  //!< uloz-to
    u32 ugotfile_com:1;             //!< ugotfile-com
    u32 turbobit_net:1;             //!< turbobit-net
    u32 tunabox_net:1;              //!< tunabox-net
    u32 terafiles_net:1;            //!< terafiles-net
    u32 storage_to:1;               //!< storage-to
    u32 shragle_com:1;              //!< shragle-com
    u32 share_rapid_cz:1;           //!< share-rapid-cz
    PACE2_DEPRECATED("Application share-online-bz is marked as deprecated and should not be used anymore.")
    u32 share_online_bz:1;          //!< share-online-bz
    u32 scribd_com:1;               //!< scribd-com
    u32 rapidshark_pl:1;            //!< rapidshark-pl
    u32 quickshare_cz:1;            //!< quickshare-cz
    u32 qshare_com:1;               //!< qshare-com
    u32 putshare_com:1;             //!< putshare-com
    PACE2_DEPRECATED("Application przeklej-pl is marked as deprecated and should not be used anymore.")
    u32 przeklej_pl:1;              //!< przeklej-pl
    u32 oron_com:1;                 //!< oron-com
    u32 netuploaded_com:1;          //!< netuploaded-com
    u32 multishare_cz:1;            //!< multishare-cz
    u32 midupload_com:1;            //!< midupload-com
    u32 megashare_vnn_vn:1;         //!< megashare-vnn-vn
    PACE2_DEPRECATED("Application megarapid-eu is marked as deprecated and should not be used anymore.")
    u32 megarapid_eu:1;             //!< megarapid-eu
    u32 megafree_kz:1;              //!< megafree-kz
    u32 loadfiles_in:1;             //!< loadfiles-in
    u32 letitbit_net:1;             //!< letitbit-net
    u32 kewlshare_com:1;            //!< kewlshare-com
    u32 ifile_it:1;                 //!< ifile-it
    u32 i_filez_com:1;              //!< i-filez-com
    u32 hyperfileshare_com:1;       //!< hyperfileshare-com
    u32 hulkshare_com:1;            //!< hulkshare-com
    u32 hitfile_net:1;              //!< hitfile-net
    u32 hellshare_com:1;            //!< hellshare-com
    u32 gotoupload_com:1;           //!< gotoupload-com
    u32 gigapeta_com:1;             //!< gigapeta-com
    u32 gaiafile_com:1;             //!< gaiafile-com
    u32 fshare_vn:1;                //!< fshare-vn
    u32 freakshare_net:1;           //!< freakshare-net
    PACE2_DEPRECATED("Application filezup-com is marked as deprecated and should not be used anymore.")
    u32 filezup_com:1;              //!< filezup-com
    u32 filevo_com:1;               //!< filevo-com
    u32 filesmonster_com:1;         //!< filesmonster-com
    u32 fileshare_in_ua:1;          //!< fileshare-in-ua
    u32 files_mail_ru:1;            //!< files-mail-ru
    u32 filepost_com:1;             //!< filepost-com
    u32 fileover_net:1;             //!< fileover-net
    u32 filehook_com:1;             //!< filehook-com
    PACE2_DEPRECATED("Application filefrog-com is marked as deprecated and should not be used anymore.")
    u32 filefrog_com:1;             //!< filefrog-com
    PACE2_DEPRECATED("Application fileflyer-com is marked as deprecated and should not be used anymore.")
    u32 fileflyer_com:1;            //!< fileflyer-com
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    PACE2_DEPRECATED("Application 123upload-pl is marked as deprecated and should not be used anymore.")
    u32 _123upload_pl:1;            //!< 123upload-pl
    u32 _139pan_com:1;              //!< 139pan-com
    u32 _163pan_com:1;              //!< 163pan-com
    u32 _2shared_com:1;             //!< 2shared-com
    u32 _4share_ws:1;               //!< 4share-ws
    u32 _4us_to:1;                  //!< 4us-to
    u32 alldrives_ge:1;             //!< alldrives-ge
    u32 allshares_ge:1;             //!< allshares-ge
    u32 annonhost_net:1;            //!< annonhost-net
    u32 boosterking_com:1;          //!< boosterking-com
    u32 cobrashare_sk:1;            //!< cobrashare-sk
    u32 coolshare_cz:1;             //!< coolshare-cz
    u32 crocko_com:1;               //!< crocko-com
    PACE2_DEPRECATED("Application cum-com is marked as deprecated and should not be used anymore.")
    u32 cum_com:1;                  //!< cum-com
    u32 eyvx_com:1;                 //!< eyvx-com
    u32 filearn_com:1;              //!< filearn-com
    u32 filejungle_com:1;           //!< filejungle-com
    u32 filemaze_ws:1;              //!< filemaze-ws
    u32 filesflash_com:1;           //!< filesflash-com
    u32 freespace_by:1;             //!< freespace-by
    u32 fsx_hu:1;                   //!< fsx-hu
    u32 jakfile_com:1;              //!< jakfile-com
    u32 jumbofiles_com:1;           //!< jumbofiles-com
    u32 kickload_com:1;             //!< kickload-com
    u32 missupload_com:1;           //!< missupload-com
    u32 movreel_com:1;              //!< movreel-com
    u32 platinshare_com:1;          //!< platinshare-com
    u32 sanupload_com:1;            //!< sanupload-com
    u32 shareflare_net:1;           //!< shareflare-net
    u32 u_115_com:1;                //!< u-115-com
    u32 uploadc_com:1;              //!< uploadc-com
    u32 uploadking_com:1;           //!< uploadking-com
    u32 uptobox_com:1;              //!< uptobox-com
    u32 filestube_com:1;            //!< filestube-com
    u32 gamefront_com:1;            //!< gamefront-com
    u32 megaporn_com:1;             //!< megaporn-com
    u32 filesonic_in:1;             //!< filesonic-in
    u32 turbobit_ru:1;              //!< turbobit-ru
    u32 usenet:1;                   //!< Usenet
    PACE2_DEPRECATED("Application FTP is marked as deprecated and should not be used anymore.")
    u32 ftp:1;                      //!< FTP
    u32 filetopia:1;                //!< Filetopia
    u32 manolito:1;                 //!< Manolito
    u32 imesh:1;                    //!< iMesh
    u32 pando:1;                    //!< Pando
    u32 sopcast:1;                  //!< SopCast
    u32 tvuplayer:1;                //!< TVUPlayer
    PACE2_DEPRECATED("Application PPStream is marked as deprecated and should not be used anymore.")
    u32 ppstream:1;                 //!< PPStream
    PACE2_DEPRECATED("Application PPLive is marked as deprecated and should not be used anymore.")
    u32 pplive:1;                   //!< PPLive
    u32 zattoo:1;                   //!< Zattoo
    u32 qq:1;                       //!< QQ
    u32 qqlive:1;                   //!< QQLive
    u32 feidian:1;                  //!< Feidian
    u32 popo:1;                     //!< PoPo
    u32 thunder:1;                  //!< Thunder/Webthunder
    u32 uusee:1;                    //!< UUSee
    u32 vnc:1;                      //!< VNC
    u32 steam:1;                    //!< Steam
    u32 hl2:1;                      //!< Half-Life 2
    u32 softether:1;                //!< SoftEthernet
    u32 telnet:1;                   //!< Telnet
    u32 off:1;                      //!< OFF
    u32 tor:1;                      //!< TOR
    u32 worldofwarcraft:1;          //!< World of Warcraft
    u32 hamachi:1;                  //!< HamachiVPN
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 hamachi:1;                  //!< HamachiVPN
    u32 worldofwarcraft:1;          //!< World of Warcraft
    u32 tor:1;                      //!< TOR
    u32 off:1;                      //!< OFF
    u32 telnet:1;                   //!< Telnet
    u32 softether:1;                //!< SoftEthernet
    u32 hl2:1;                      //!< Half-Life 2
    u32 steam:1;                    //!< Steam
    u32 vnc:1;                      //!< VNC
    u32 uusee:1;                    //!< UUSee
    u32 thunder:1;                  //!< Thunder/Webthunder
    u32 popo:1;                     //!< PoPo
    u32 feidian:1;                  //!< Feidian
    u32 qqlive:1;                   //!< QQLive
    u32 qq:1;                       //!< QQ
    u32 zattoo:1;                   //!< Zattoo
    PACE2_DEPRECATED("Application PPLive is marked as deprecated and should not be used anymore.")
    u32 pplive:1;                   //!< PPLive
    PACE2_DEPRECATED("Application PPStream is marked as deprecated and should not be used anymore.")
    u32 ppstream:1;                 //!< PPStream
    u32 tvuplayer:1;                //!< TVUPlayer
    u32 sopcast:1;                  //!< SopCast
    u32 pando:1;                    //!< Pando
    u32 imesh:1;                    //!< iMesh
    u32 manolito:1;                 //!< Manolito
    u32 filetopia:1;                //!< Filetopia
    PACE2_DEPRECATED("Application FTP is marked as deprecated and should not be used anymore.")
    u32 ftp:1;                      //!< FTP
    u32 usenet:1;                   //!< Usenet
    u32 turbobit_ru:1;              //!< turbobit-ru
    u32 filesonic_in:1;             //!< filesonic-in
    u32 megaporn_com:1;             //!< megaporn-com
    u32 gamefront_com:1;            //!< gamefront-com
    u32 filestube_com:1;            //!< filestube-com
    u32 uptobox_com:1;              //!< uptobox-com
    u32 uploadking_com:1;           //!< uploadking-com
    u32 uploadc_com:1;              //!< uploadc-com
    u32 u_115_com:1;                //!< u-115-com
    u32 shareflare_net:1;           //!< shareflare-net
    u32 sanupload_com:1;            //!< sanupload-com
    u32 platinshare_com:1;          //!< platinshare-com
    u32 movreel_com:1;              //!< movreel-com
    u32 missupload_com:1;           //!< missupload-com
    u32 kickload_com:1;             //!< kickload-com
    u32 jumbofiles_com:1;           //!< jumbofiles-com
    u32 jakfile_com:1;              //!< jakfile-com
    u32 fsx_hu:1;                   //!< fsx-hu
    u32 freespace_by:1;             //!< freespace-by
    u32 filesflash_com:1;           //!< filesflash-com
    u32 filemaze_ws:1;              //!< filemaze-ws
    u32 filejungle_com:1;           //!< filejungle-com
    u32 filearn_com:1;              //!< filearn-com
    u32 eyvx_com:1;                 //!< eyvx-com
    PACE2_DEPRECATED("Application cum-com is marked as deprecated and should not be used anymore.")
    u32 cum_com:1;                  //!< cum-com
    u32 crocko_com:1;               //!< crocko-com
    u32 coolshare_cz:1;             //!< coolshare-cz
    u32 cobrashare_sk:1;            //!< cobrashare-sk
    u32 boosterking_com:1;          //!< boosterking-com
    u32 annonhost_net:1;            //!< annonhost-net
    u32 allshares_ge:1;             //!< allshares-ge
    u32 alldrives_ge:1;             //!< alldrives-ge
    u32 _4us_to:1;                  //!< 4us-to
    u32 _4share_ws:1;               //!< 4share-ws
    u32 _2shared_com:1;             //!< 2shared-com
    u32 _163pan_com:1;              //!< 163pan-com
    u32 _139pan_com:1;              //!< 139pan-com
    PACE2_DEPRECATED("Application 123upload-pl is marked as deprecated and should not be used anymore.")
    u32 _123upload_pl:1;            //!< 123upload-pl
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 skinny:1;                   //!< Skinny
    u32 citrix:1;                   //!< Citrix
    u32 paltalk:1;                  //!< Paltalk
    u32 quake:1;                    //!< Quake
    u32 secondlife:1;               //!< Second Life
    u32 battlefield:1;              //!< Battlefield
    u32 pcanywhere:1;               //!< PCAnywhere
    u32 icecast:1;                  //!< Icecast
    u32 shoutcast:1;                //!< Shoutcast
    u32 qqgame:1;                   //!< QQGame
    u32 veohtv:1;                   //!< VeohTV
    u32 openft:1;                   //!< OpenFT
    u32 pandora:1;                  //!< Pandora
    u32 ipp:1;                      //!< IPP
    u32 warcraft3:1;                //!< Warcraft 3
    PACE2_DEPRECATED("Application ooVoo is marked as deprecated and should not be used anymore.")
    u32 oovoo:1;                    //!< ooVoo
    u32 google_hangouts:1;          //!< Google Hangouts
    u32 iskoot:1;                   //!< Iskoot
    u32 fring:1;                    //!< Fring
    u32 iptv:1;                     //!< IPTV
    u32 yourfreedom:1;              //!< YourFreedom Tunnel
    u32 voipswitchvoiptunnel:1;     //!< VoipSwitch VoIP Tunnel
    u32 meebo:1;                    //!< Meebo
    u32 funshion:1;                 //!< Funshion
    u32 aimini:1;                   //!< Aimini
    PACE2_DEPRECATED("Application clubpenguin is marked as deprecated and should not be used anymore.")
    u32 clubpenguin:1;              //!< ClubPenguin
    u32 florensia:1;                //!< Florensia
    u32 teamspeak:1;                //!< TeamSpeak
    u32 maplestory:1;               //!< MapleStory
    u32 osh:1;                      //!< Octoshape
    PACE2_DEPRECATED("Application PS3 is marked as deprecated and should not be used anymore.")
    u32 ps3:1;                      //!< PS3
    u32 dofus:1;                    //!< Dofus
    u32 fiesta:1;                   //!< Fiesta
    u32 splashfighter:1;            //!< SplashFighter
    u32 activesync:1;               //!< Activesync
    u32 nimbuzz:1;                  //!< Nimbuzz
    u32 crossfire:1;                //!< CrossFire
    PACE2_DEPRECATED("Application WAP-WSP is marked as deprecated and should not be used anymore.")
    u32 wapwsp:1;                   //!< WAP-WSP
    u32 guildwars:1;                //!< Guildwars
    u32 armagetron:1;               //!< Armagetron
    u32 rfactor:1;                  //!< rFactor
    u32 teamviewer:1;               //!< TeamViewer
    u32 gamekit:1;                  //!< GameKit
    u32 ultrabac:1;                 //!< UltraBac
    u32 spotify:1;                  //!< Spotify
    u32 implus:1;                   //!< IMplus
    u32 antsp2p:1;                  //!< ANtsP2P
    u32 ultrasurf:1;                //!< UltraSurf
    u32 viber:1;                    //!< Viber
    u32 tango:1;                    //!< Tango
    u32 iplayer:1;                  //!< iPlayer
    u32 operamini:1;                //!< Opera Mini
    u32 bolt:1;                     //!< Bolt
    u32 scydo:1;                    //!< Scydo
    u32 whatsapp:1;                 //!< WhatsApp
    u32 webex:1;                    //!< Webex
    u32 netflix:1;                  //!< NetFlix
    u32 mojo:1;                     //!< Mojo
    u32 imo:1;                      //!< IMO
    u32 citrixgoto:1;               //!< CitrixGoTo
    u32 ficall:1;                   //!< FiCall
    u32 comodounite:1;              //!< ComodoUnite
    u32 ventrilo:1;                 //!< Ventrilo
    PACE2_DEPRECATED("Application MyPeople is marked as deprecated and should not be used anymore.")
    u32 mypeople:1;                 //!< MyPeople
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    PACE2_DEPRECATED("Application MyPeople is marked as deprecated and should not be used anymore.")
    u32 mypeople:1;                 //!< MyPeople
    u32 ventrilo:1;                 //!< Ventrilo
    u32 comodounite:1;              //!< ComodoUnite
    u32 ficall:1;                   //!< FiCall
    u32 citrixgoto:1;               //!< CitrixGoTo
    u32 imo:1;                      //!< IMO
    u32 mojo:1;                     //!< Mojo
    u32 netflix:1;                  //!< NetFlix
    u32 webex:1;                    //!< Webex
    u32 whatsapp:1;                 //!< WhatsApp
    u32 scydo:1;                    //!< Scydo
    u32 bolt:1;                     //!< Bolt
    u32 operamini:1;                //!< Opera Mini
    u32 iplayer:1;                  //!< iPlayer
    u32 tango:1;                    //!< Tango
    u32 viber:1;                    //!< Viber
    u32 ultrasurf:1;                //!< UltraSurf
    u32 antsp2p:1;                  //!< ANtsP2P
    u32 implus:1;                   //!< IMplus
    u32 spotify:1;                  //!< Spotify
    u32 ultrabac:1;                 //!< UltraBac
    u32 gamekit:1;                  //!< GameKit
    u32 teamviewer:1;               //!< TeamViewer
    u32 rfactor:1;                  //!< rFactor
    u32 armagetron:1;               //!< Armagetron
    u32 guildwars:1;                //!< Guildwars
    PACE2_DEPRECATED("Application WAP-WSP is marked as deprecated and should not be used anymore.")
    u32 wapwsp:1;                   //!< WAP-WSP
    u32 crossfire:1;                //!< CrossFire
    u32 nimbuzz:1;                  //!< Nimbuzz
    u32 activesync:1;               //!< Activesync
    u32 splashfighter:1;            //!< SplashFighter
    u32 fiesta:1;                   //!< Fiesta
    u32 dofus:1;                    //!< Dofus
    PACE2_DEPRECATED("Application PS3 is marked as deprecated and should not be used anymore.")
    u32 ps3:1;                      //!< PS3
    u32 osh:1;                      //!< Octoshape
    u32 maplestory:1;               //!< MapleStory
    u32 teamspeak:1;                //!< TeamSpeak
    u32 florensia:1;                //!< Florensia
    u32 clubpenguin:1;              //!< ClubPenguin
    u32 aimini:1;                   //!< Aimini
    u32 funshion:1;                 //!< Funshion
    u32 meebo:1;                    //!< Meebo
    u32 voipswitchvoiptunnel:1;     //!< VoipSwitch VoIP Tunnel
    u32 yourfreedom:1;              //!< YourFreedom Tunnel
    u32 iptv:1;                     //!< IPTV
    u32 fring:1;                    //!< Fring
    u32 iskoot:1;                   //!< Iskoot
    u32 google_hangouts:1;          //!< Google Hangouts
    PACE2_DEPRECATED("Application ooVoo is marked as deprecated and should not be used anymore.")
    u32 oovoo:1;                    //!< ooVoo
    u32 warcraft3:1;                //!< Warcraft 3
    u32 ipp:1;                      //!< IPP
    u32 pandora:1;                  //!< Pandora
    u32 openft:1;                   //!< OpenFT
    u32 veohtv:1;                   //!< VeohTV
    u32 qqgame:1;                   //!< QQGame
    u32 shoutcast:1;                //!< Shoutcast
    u32 icecast:1;                  //!< Icecast
    u32 pcanywhere:1;               //!< PCAnywhere
    u32 battlefield:1;              //!< Battlefield
    u32 secondlife:1;               //!< Second Life
    u32 quake:1;                    //!< Quake
    u32 paltalk:1;                  //!< Paltalk
    u32 citrix:1;                   //!< Citrix
    u32 skinny:1;                   //!< Skinny
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 ebuddy:1;                   //!< eBuddy
    u32 adobeconnect:1;             //!< AdobeConnect
    u32 jap:1;                      //!< JAP
    u32 wuala:1;                    //!< Wuala
    u32 webdav:1;                   //!< WebDAV
    PACE2_DEPRECATED("Application Lync is marked as deprecated and should not be used anymore.")
    u32 lync:1;                     //!< Lync
    u32 audiogalaxy:1;              //!< Audiogalaxy
    PACE2_DEPRECATED("Application Mig33 is marked as deprecated and should not be used anymore.")
    u32 mig33:1;                    //!< Mig33
    u32 httptunnel:1;               //!< HTTPTunnel
    u32 rhapsody:1;                 //!< Rhapsody
    u32 webqq:1;                    //!< WebQQ
    u32 pdproxy:1;                  //!< PDProxy
    u32 cyberghost:1;               //!< CyberGhost
    u32 google_cloud:1;             //!< Google Cloud
    u32 google_drive:1;             //!< Google Drive
    u32 google_docs:1;              //!< Google Docs
    u32 google_plus:1;              //!< Google Plus
    u32 oracledb:1;                 //!< Oracle Database
    u32 itv:1;                      //!< ITV
    u32 demand5:1;                  //!< Demand 5
    PACE2_DEPRECATED("Application RSVP is marked as deprecated and should not be used anymore.")
    u32 rsvp:1;                     //!< RSVP
    u32 channel4od:1;               //!< Channel 4oD
    u32 lotusnotes:1;               //!< Lotus Notes
    u32 poisonivy:1;                //!< Poison Ivy
    u32 hidrive:1;                  //!< HiDrive
    u32 wechat:1;                   //!< WeChat
    PACE2_DEPRECATED("Application MyVideo is marked as deprecated and should not be used anymore.")
    u32 myvideo:1;                  //!< MyVideo
    u32 kik:1;                      //!< Kik Messenger
    u32 line:1;                     //!< Line
    u32 iperf:1;                    //!< Iperf
    u32 osupdate:1;                 //!< OS update
    u32 hike:1;                     //!< Hike Messenger
    u32 didi:1;                     //!< DiDi
    u32 idrive:1;                   //!< IDrive
    u32 spreecast:1;                //!< Spreecast
    u32 simfy:1;                    //!< Simfy
    u32 dota2:1;                    //!< Dota 2
    u32 qvod:1;                     //!< QVOD
    u32 hulu:1;                     //!< Hulu
    u32 msexchange:1;               //!< Microsoft Exchange
    u32 msnav:1;                    //!< Microsoft Dynamics NAV
    u32 fuze:1;                     //!< Fuze Meeting
    u32 kakaotalk:1;                //!< Kakaotalk
    u32 destiny:1;                  //!< Destiny
    u32 vyprvpn:1;                  //!< VyprVPN
    u32 threema:1;                  //!< Threema
    PACE2_DEPRECATED("Application NTLM is marked as deprecated and should not be used anymore.")
    u32 ntlm:1;                     //!< NTLM
    u32 twitch:1;                   //!< Twitch
    u32 wowza:1;                    //!< Wowza
    u32 runtastic:1;                //!< Runtastic
    u32 telegram:1;                 //!< Telegram
    u32 streetvoice:1;              //!< StreetVoice
    u32 duokan:1;                   //!< Duokan
    u32 mei_lu:1;                   //!< Mei Lu
    u32 skout:1;                    //!< Skout
    u32 playtales:1;                //!< PlayTales
    u32 gameofwar:1;                //!< Game of War
    u32 fetnet:1;                   //!< FETnet
    u32 fbs:1;                      //!< Fubon E-Broker
    u32 national_baseball:1;        //!< National Baseball
    u32 pinkfong:1;                 //!< PinkFong
    u32 kollect_book:1;             //!< Kollect Book
    u32 sptg_tv:1;                  //!< SPTG TV
    u32 ciaomobile_hdtv:1;          //!< Ciaomobile HDTV
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 ciaomobile_hdtv:1;          //!< Ciaomobile HDTV
    u32 sptg_tv:1;                  //!< SPTG TV
    u32 kollect_book:1;             //!< Kollect Book
    u32 pinkfong:1;                 //!< PinkFong
    u32 national_baseball:1;        //!< National Baseball
    u32 fbs:1;                      //!< Fubon E-Broker
    u32 fetnet:1;                   //!< FETnet
    u32 gameofwar:1;                //!< Game of War
    u32 playtales:1;                //!< PlayTales
    u32 skout:1;                    //!< Skout
    u32 mei_lu:1;                   //!< Mei Lu
    u32 duokan:1;                   //!< Duokan
    u32 streetvoice:1;              //!< StreetVoice
    u32 telegram:1;                 //!< Telegram
    u32 runtastic:1;                //!< Runtastic
    u32 wowza:1;                    //!< Wowza
    u32 twitch:1;                   //!< Twitch
    PACE2_DEPRECATED("Application NTLM is marked as deprecated and should not be used anymore.")
    u32 ntlm:1;                     //!< NTLM
    u32 threema:1;                  //!< Threema
    u32 vyprvpn:1;                  //!< VyprVPN
    u32 destiny:1;                  //!< Destiny
    u32 kakaotalk:1;                //!< Kakaotalk
    u32 fuze:1;                     //!< Fuze Meeting
    u32 msnav:1;                    //!< Microsoft Dynamics NAV
    u32 msexchange:1;               //!< Microsoft Exchange
    u32 hulu:1;                     //!< Hulu
    u32 qvod:1;                     //!< QVOD
    u32 dota2:1;                    //!< Dota 2
    u32 simfy:1;                    //!< Simfy
    u32 spreecast:1;                //!< Spreecast
    u32 idrive:1;                   //!< IDrive
    u32 didi:1;                     //!< DiDi
    u32 hike:1;                     //!< Hike Messenger
    u32 osupdate:1;                 //!< OS update
    u32 iperf:1;                    //!< Iperf
    u32 line:1;                     //!< Line
    u32 kik:1;                      //!< Kik Messenger
    PACE2_DEPRECATED("Application MyVideo is marked as deprecated and should not be used anymore.")
    u32 myvideo:1;                  //!< MyVideo
    u32 wechat:1;                   //!< WeChat
    u32 hidrive:1;                  //!< HiDrive
    u32 poisonivy:1;                //!< Poison Ivy
    u32 lotusnotes:1;               //!< Lotus Notes
    u32 channel4od:1;               //!< Channel 4oD
    PACE2_DEPRECATED("Application RSVP is marked as deprecated and should not be used anymore.")
    u32 rsvp:1;                     //!< RSVP
    u32 demand5:1;                  //!< Demand 5
    u32 itv:1;                      //!< ITV
    u32 oracledb:1;                 //!< Oracle Database
    u32 google_plus:1;              //!< Google Plus
    u32 google_docs:1;              //!< Google Docs
    u32 google_drive:1;             //!< Google Drive
    u32 google_cloud:1;             //!< Google Cloud
    u32 cyberghost:1;               //!< CyberGhost
    u32 pdproxy:1;                  //!< PDProxy
    u32 webqq:1;                    //!< WebQQ
    u32 rhapsody:1;                 //!< Rhapsody
    u32 httptunnel:1;               //!< HTTPTunnel
    PACE2_DEPRECATED("Application Mig33 is marked as deprecated and should not be used anymore.")
    u32 mig33:1;                    //!< Mig33
    u32 audiogalaxy:1;              //!< Audiogalaxy
    PACE2_DEPRECATED("Application Lync is marked as deprecated and should not be used anymore.")
    u32 lync:1;                     //!< Lync
    u32 webdav:1;                   //!< WebDAV
    u32 wuala:1;                    //!< Wuala
    u32 jap:1;                      //!< JAP
    u32 adobeconnect:1;             //!< AdobeConnect
    u32 ebuddy:1;                   //!< eBuddy
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 skyking:1;                  //!< Skyking
    u32 moves:1;                    //!< Moves
    u32 path:1;                     //!< Path
    u32 aowe:1;                     //!< Age of Warring Empire
    u32 iwow_systex:1;              //!< iWow systex
    u32 _9ku:1;                     //!< 9ku
    u32 nexttv:1;                   //!< Next TV
    u32 syncplicity:1;              //!< Syncplicity
    u32 realma:1;                   //!< RealMa
    u32 yuanta_web:1;               //!< Yuanta Web
    u32 hinet:1;                    //!< HiNet
    u32 mycard:1;                   //!< MyCard
    u32 kono:1;                     //!< Kono
    u32 speakaboos:1;               //!< Speakaboos
    u32 duo_duo:1;                  //!< Duo Duo
    u32 wealth_god:1;               //!< Wealth God
    u32 ileader:1;                  //!< iLeader
    u32 three_bamboo:1;             //!< Three Bamboo
    u32 taaze:1;                    //!< Taaze
    u32 audiko:1;                   //!< Audiko
    u32 pubu_bookbuffet:1;          //!< pubu bookbuffet
    u32 mitake:1;                   //!< Mitake
    u32 hyread:1;                   //!< Hyread
    u32 disney_storytime:1;         //!< Disney Storytime
    u32 qualys:1;                   //!< Qualys
    u32 constant_contact:1;         //!< Constant Contact
    u32 rally:1;                    //!< Rally
    u32 maginetv:1;                 //!< Magine TV
    u32 intralinks:1;               //!< IntraLinks
    u32 liveperson:1;               //!< LivePerson
    u32 speedtest:1;                //!< Speedtest
    u32 bluejeans:1;                //!< BlueJeans
    u32 apple_music:1;              //!< Apple Music
    u32 naver:1;                    //!< Naver
    u32 booking_com:1;              //!< Booking-com
    u32 cnn:1;                      //!< CNN
    u32 periscope:1;                //!< Periscope
    u32 payeasy:1;                  //!< PayEasy
    u32 line_rangers:1;             //!< Line Rangers
    u32 ios_app_store:1;            //!< iOS App Store
    u32 myvideo_tw:1;               //!< MyVideo TW
    u32 king_gaming:1;              //!< King Gaming
    u32 candy_crush:1;              //!< Candy Crush
    u32 candy_crush_soda:1;         //!< Candy Crush Soda
    u32 bubble_witch:1;             //!< Bubble Witch
    u32 farm_hero:1;                //!< Farm Hero
    u32 playstation:1;              //!< PlayStation Network
    u32 google_play:1;              //!< Google Play
    u32 supercell:1;                //!< Supercell
    u32 clash_of_clans:1;           //!< Clash of Clans
    u32 boom_beach:1;               //!< Boom Beach
    u32 hay_day:1;                  //!< Hay Day
    u32 windows_store:1;            //!< Windows Store
    u32 skype_for_business:1;       //!< Skype for Business
    u32 minecraft:1;                //!< Minecraft
    PACE2_DEPRECATED("Application Meerkat is marked as deprecated and should not be used anymore.")
    u32 meerkat:1;                  //!< Meerkat
    u32 rovio_gaming:1;             //!< Rovio Gaming
    u32 angry_birds:1;              //!< Angry Birds
    u32 citrix_xenapp:1;            //!< Citrix XenApp
    u32 reddit:1;                   //!< reddit
    u32 league_of_legends:1;        //!< League of Legends
    u32 battle_net:1;               //!< BattleNet
    u32 paypal:1;                   //!< PayPal
    u32 plants_vs_zombies_2:1;      //!< Plants vs Zombies 2
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 plants_vs_zombies_2:1;      //!< Plants vs Zombies 2
    u32 paypal:1;                   //!< PayPal
    u32 battle_net:1;               //!< BattleNet
    u32 league_of_legends:1;        //!< League of Legends
    u32 reddit:1;                   //!< reddit
    u32 citrix_xenapp:1;            //!< Citrix XenApp
    u32 angry_birds:1;              //!< Angry Birds
    u32 rovio_gaming:1;             //!< Rovio Gaming
    PACE2_DEPRECATED("Application Meerkat is marked as deprecated and should not be used anymore.")
    u32 meerkat:1;                  //!< Meerkat
    u32 minecraft:1;                //!< Minecraft
    u32 skype_for_business:1;       //!< Skype for Business
    u32 windows_store:1;            //!< Windows Store
    u32 hay_day:1;                  //!< Hay Day
    u32 boom_beach:1;               //!< Boom Beach
    u32 clash_of_clans:1;           //!< Clash of Clans
    u32 supercell:1;                //!< Supercell
    u32 google_play:1;              //!< Google Play
    u32 playstation:1;              //!< PlayStation Network
    u32 farm_hero:1;                //!< Farm Hero
    u32 bubble_witch:1;             //!< Bubble Witch
    u32 candy_crush_soda:1;         //!< Candy Crush Soda
    u32 candy_crush:1;              //!< Candy Crush
    u32 king_gaming:1;              //!< King Gaming
    u32 myvideo_tw:1;               //!< MyVideo TW
    u32 ios_app_store:1;            //!< iOS App Store
    u32 line_rangers:1;             //!< Line Rangers
    u32 payeasy:1;                  //!< PayEasy
    u32 periscope:1;                //!< Periscope
    u32 cnn:1;                      //!< CNN
    u32 booking_com:1;              //!< Booking-com
    u32 naver:1;                    //!< Naver
    u32 apple_music:1;              //!< Apple Music
    u32 bluejeans:1;                //!< BlueJeans
    u32 speedtest:1;                //!< Speedtest
    u32 liveperson:1;               //!< LivePerson
    u32 intralinks:1;               //!< IntraLinks
    u32 maginetv:1;                 //!< Magine TV
    u32 rally:1;                    //!< Rally
    u32 constant_contact:1;         //!< Constant Contact
    u32 qualys:1;                   //!< Qualys
    u32 disney_storytime:1;         //!< Disney Storytime
    u32 hyread:1;                   //!< Hyread
    u32 mitake:1;                   //!< Mitake
    u32 pubu_bookbuffet:1;          //!< pubu bookbuffet
    u32 audiko:1;                   //!< Audiko
    u32 taaze:1;                    //!< Taaze
    u32 three_bamboo:1;             //!< Three Bamboo
    u32 ileader:1;                  //!< iLeader
    u32 wealth_god:1;               //!< Wealth God
    u32 duo_duo:1;                  //!< Duo Duo
    u32 speakaboos:1;               //!< Speakaboos
    u32 kono:1;                     //!< Kono
    u32 mycard:1;                   //!< MyCard
    u32 hinet:1;                    //!< HiNet
    u32 yuanta_web:1;               //!< Yuanta Web
    u32 realma:1;                   //!< RealMa
    u32 syncplicity:1;              //!< Syncplicity
    u32 nexttv:1;                   //!< Next TV
    u32 _9ku:1;                     //!< 9ku
    u32 iwow_systex:1;              //!< iWow systex
    u32 aowe:1;                     //!< Age of Warring Empire
    u32 path:1;                     //!< Path
    u32 moves:1;                    //!< Moves
    u32 skyking:1;                  //!< Skyking
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 halfbrick_studios:1;        //!< Halfbrick Studios
    u32 github:1;                   //!< GitHub
    u32 hamivideo:1;                //!< HamiVideo
    u32 slacker:1;                  //!< Slacker
    u32 niconico:1;                 //!< Niconico
    u32 alipay:1;                   //!< AliPay
    u32 ustream:1;                  //!< Ustream
    u32 zalo:1;                     //!< Zalo
    u32 docomo:1;                   //!< NTT Docomo
    u32 softbank:1;                 //!< Softbank Onlineshop
    u32 guildwars2:1;               //!< Guild Wars 2
    u32 trendmicro_worryfree:1;     //!< Trend Micro Worry-Free
    u32 wikipedia:1;                //!< Wikipedia
    u32 uber:1;                     //!< Uber
    u32 yandex:1;                   //!< Yandex
    u32 vk:1;                       //!< VK
    u32 odnoklassniki:1;            //!< Odnoklassniki
    u32 crashplan:1;                //!< CrashPlan
    u32 alibaba:1;                  //!< Alibaba
    u32 aliexpress:1;               //!< AliExpress
    u32 tmall:1;                    //!< TMall
    u32 ea_origin:1;                //!< EA Origin
    u32 norton_security:1;          //!< Norton Security
    u32 tibia:1;                    //!< Tibia
    u32 pochta_rossii:1;            //!< Pochta Rossii
    u32 after_school:1;             //!< After School
    u32 drivelock:1;                //!< DriveLock
    u32 qip_ru:1;                   //!< QIP-RU
    PACE2_DEPRECATED("Application musical_ly is marked as deprecated and should not be used anymore.")
    u32 musical_ly:1;               //!< musical_ly
    u32 ea_games:1;                 //!< EA Games
    u32 itunesu:1;                  //!< iTunesU
    u32 popcap:1;                   //!< PopCap
    u32 surveymonkey:1;             //!< SurveyMonkey
    u32 new_relic:1;                //!< New Relic
    u32 huddle:1;                   //!< Huddle
    u32 zendesk:1;                  //!< ZenDesk
    u32 outlookmobile:1;            //!< OutlookMobile
    u32 chatcube:1;                 //!< ChatCube
    u32 dena:1;                     //!< DeNA
    u32 backblaze:1;                //!< BackBlaze
    u32 docusign:1;                 //!< DocuSign
    u32 betternet:1;                //!< Betternet
    u32 good:1;                     //!< Good
    u32 anghami:1;                  //!< Anghami
    u32 path_of_exile:1;            //!< Path of Exile
    u32 slack:1;                    //!< Slack
    u32 pokemon_go:1;               //!< Pokemon Go
    u32 niantic_labs:1;             //!< Niantic Labs
    u32 _2gis:1;                    //!< 2Gis
    u32 rutube:1;                   //!< Rutube
    u32 _2channel:1;                //!< 2channel
    u32 prisma:1;                   //!< Prisma
    PACE2_DEPRECATED("Application Clash Royal is marked as deprecated and should not be used anymore.")
    u32 clash_royal:1;              //!< Clash Royal
    u32 upptalk:1;                  //!< Upptalk
    u32 amazon_prime_video:1;       //!< Amazon Prime Video
    u32 clash_royale:1;             //!< Clash Royale
    u32 google_shared_services:1;   //!< Google Shared Services
    u32 signal:1;                   //!< Signal
    u32 kankan:1;                   //!< KanKan
    u32 share_online:1;             //!< share-online
    u32 ultrashare:1;               //!< Ultrashare
    u32 yunfile:1;                  //!< Yunfile
    u32 edisk:1;                    //!< eDisk
    u32 _123upload:1;               //!< 123upload
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 _123upload:1;               //!< 123upload
    u32 edisk:1;                    //!< eDisk
    u32 yunfile:1;                  //!< Yunfile
    u32 ultrashare:1;               //!< Ultrashare
    u32 share_online:1;             //!< share-online
    u32 kankan:1;                   //!< KanKan
    u32 signal:1;                   //!< Signal
    u32 google_shared_services:1;   //!< Google Shared Services
    u32 clash_royale:1;             //!< Clash Royale
    u32 amazon_prime_video:1;       //!< Amazon Prime Video
    u32 upptalk:1;                  //!< Upptalk
    PACE2_DEPRECATED("Application Clash Royal is marked as deprecated and should not be used anymore.")
    u32 clash_royal:1;              //!< Clash Royal
    u32 prisma:1;                   //!< Prisma
    u32 _2channel:1;                //!< 2channel
    u32 rutube:1;                   //!< Rutube
    u32 _2gis:1;                    //!< 2Gis
    u32 niantic_labs:1;             //!< Niantic Labs
    u32 pokemon_go:1;               //!< Pokemon Go
    u32 slack:1;                    //!< Slack
    u32 path_of_exile:1;            //!< Path of Exile
    u32 anghami:1;                  //!< Anghami
    u32 good:1;                     //!< Good
    u32 betternet:1;                //!< Betternet
    u32 docusign:1;                 //!< DocuSign
    u32 backblaze:1;                //!< BackBlaze
    u32 dena:1;                     //!< DeNA
    u32 chatcube:1;                 //!< ChatCube
    u32 outlookmobile:1;            //!< OutlookMobile
    u32 zendesk:1;                  //!< ZenDesk
    u32 huddle:1;                   //!< Huddle
    u32 new_relic:1;                //!< New Relic
    u32 surveymonkey:1;             //!< SurveyMonkey
    u32 popcap:1;                   //!< PopCap
    u32 itunesu:1;                  //!< iTunesU
    u32 ea_games:1;                 //!< EA Games
    PACE2_DEPRECATED("Application musical_ly is marked as deprecated and should not be used anymore.")
    u32 musical_ly:1;               //!< musical_ly
    u32 qip_ru:1;                   //!< QIP-RU
    u32 drivelock:1;                //!< DriveLock
    u32 after_school:1;             //!< After School
    u32 pochta_rossii:1;            //!< Pochta Rossii
    u32 tibia:1;                    //!< Tibia
    u32 norton_security:1;          //!< Norton Security
    u32 ea_origin:1;                //!< EA Origin
    u32 tmall:1;                    //!< TMall
    u32 aliexpress:1;               //!< AliExpress
    u32 alibaba:1;                  //!< Alibaba
    u32 crashplan:1;                //!< CrashPlan
    u32 odnoklassniki:1;            //!< Odnoklassniki
    u32 vk:1;                       //!< VK
    u32 yandex:1;                   //!< Yandex
    u32 uber:1;                     //!< Uber
    u32 wikipedia:1;                //!< Wikipedia
    u32 trendmicro_worryfree:1;     //!< Trend Micro Worry-Free
    u32 guildwars2:1;               //!< Guild Wars 2
    u32 softbank:1;                 //!< Softbank Onlineshop
    u32 docomo:1;                   //!< NTT Docomo
    u32 zalo:1;                     //!< Zalo
    u32 ustream:1;                  //!< Ustream
    u32 alipay:1;                   //!< AliPay
    u32 niconico:1;                 //!< Niconico
    u32 slacker:1;                  //!< Slacker
    u32 hamivideo:1;                //!< HamiVideo
    u32 github:1;                   //!< GitHub
    u32 halfbrick_studios:1;        //!< Halfbrick Studios
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 azar:1;                     //!< Azar
    u32 sky_go:1;                   //!< Sky Go
    u32 io:1;                       //!< iO
    u32 mail_ru:1;                  //!< Mail-Ru
    u32 icrypt:1;                   //!< iCrypt
    u32 aim:1;                      //!< AIM
    u32 kinopoisk:1;                //!< Kinopoisk
    u32 ivi_ru:1;                   //!< Ivi-Ru
    u32 tinder:1;                   //!< Tinder
    u32 anydesk:1;                  //!< AnyDesk
    u32 icq:1;                      //!< ICQ
    u32 tenor:1;                    //!< Tenor
    u32 google_maps:1;              //!< Google Maps
    u32 google_earth:1;             //!< Google Earth
    u32 channel_one:1;              //!< Channel One
    u32 giphy:1;                    //!< Giphy
    u32 vpn_x:1;                    //!< VPN-X
    u32 imgur:1;                    //!< Imgur
    u32 onedrive:1;                 //!< OneDrive
    u32 nintendo_network:1;         //!< Nintendo Network
    u32 amazon_chime:1;             //!< Amazon Chime
    u32 iflix:1;                    //!< iFlix
    u32 pipifilm:1;                 //!< PPFilm
    u32 apple_services:1;           //!< Apple Services
    u32 microsoft_services:1;       //!< Microsoft Services
    u32 icflix:1;                   //!< IcFlix
    u32 logmein:1;                  //!< LogMeIn
    u32 psiphon:1;                  //!< Psiphon
    u32 waze:1;                     //!< Waze
    u32 google_fcm:1;               //!< Google FCM
    u32 apns:1;                     //!< APNS
    u32 digicel_playgo:1;           //!< Digicel PlayGo
    u32 shazam:1;                   //!< Shazam
    u32 digicel_music:1;            //!< Digicel Music
    u32 digicel_topup:1;            //!< Digicel TopUp
    u32 tunein_radio:1;             //!< TuneIn Radio
    u32 dubsmash:1;                 //!< Dubsmash
    u32 synology:1;                 //!< Synology
    u32 my_digicel:1;               //!< My Digicel
    u32 google_allo:1;              //!< Google Allo
    u32 rusfolder:1;                //!< rusfolder
    u32 badoo:1;                    //!< Badoo
    u32 airbnb:1;                   //!< AirBnB
    u32 hightail:1;                 //!< Hightail
    u32 easy_taxi:1;                //!< Easy Taxi
    u32 tomtom:1;                   //!< TomTom
    u32 tripadvisor:1;              //!< Tripadvisor
    u32 espn:1;                     //!< ESPN
    u32 dazn:1;                     //!< DAZN
    u32 mobile_strike:1;            //!< Mobile Strike
    u32 king_of_avalon:1;           //!< King of Avalon
    u32 lineage_2:1;                //!< Lineage 2
    u32 lyft:1;                     //!< Lyft
    u32 trulia:1;                   //!< Trulia
    u32 akamai_cloud:1;             //!< Akamai Cloud
    u32 limelight_cloud:1;          //!< Limelight Cloud
    u32 nickelodeon_play:1;         //!< Nickelodeon Play
    u32 here:1;                     //!< Here
    u32 simcity_buildit:1;          //!< SimCity BuildIt
    u32 starwars_goh:1;             //!< StarWars Galaxy of Heroes
    u32 fastly:1;                   //!< Fastly
    u32 sap:1;                      //!< SAP
    u32 wokf:1;                     //!< World of Kung Fu
    u32 cloudflare:1;               //!< Cloudflare
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 cloudflare:1;               //!< Cloudflare
    u32 wokf:1;                     //!< World of Kung Fu
    u32 sap:1;                      //!< SAP
    u32 fastly:1;                   //!< Fastly
    u32 starwars_goh:1;             //!< StarWars Galaxy of Heroes
    u32 simcity_buildit:1;          //!< SimCity BuildIt
    u32 here:1;                     //!< Here
    u32 nickelodeon_play:1;         //!< Nickelodeon Play
    u32 limelight_cloud:1;          //!< Limelight Cloud
    u32 akamai_cloud:1;             //!< Akamai Cloud
    u32 trulia:1;                   //!< Trulia
    u32 lyft:1;                     //!< Lyft
    u32 lineage_2:1;                //!< Lineage 2
    u32 king_of_avalon:1;           //!< King of Avalon
    u32 mobile_strike:1;            //!< Mobile Strike
    u32 dazn:1;                     //!< DAZN
    u32 espn:1;                     //!< ESPN
    u32 tripadvisor:1;              //!< Tripadvisor
    u32 tomtom:1;                   //!< TomTom
    u32 easy_taxi:1;                //!< Easy Taxi
    u32 hightail:1;                 //!< Hightail
    u32 airbnb:1;                   //!< AirBnB
    u32 badoo:1;                    //!< Badoo
    u32 rusfolder:1;                //!< rusfolder
    u32 google_allo:1;              //!< Google Allo
    u32 my_digicel:1;               //!< My Digicel
    u32 synology:1;                 //!< Synology
    u32 dubsmash:1;                 //!< Dubsmash
    u32 tunein_radio:1;             //!< TuneIn Radio
    u32 digicel_topup:1;            //!< Digicel TopUp
    u32 digicel_music:1;            //!< Digicel Music
    u32 shazam:1;                   //!< Shazam
    u32 digicel_playgo:1;           //!< Digicel PlayGo
    u32 apns:1;                     //!< APNS
    u32 google_fcm:1;               //!< Google FCM
    u32 waze:1;                     //!< Waze
    u32 psiphon:1;                  //!< Psiphon
    u32 logmein:1;                  //!< LogMeIn
    u32 icflix:1;                   //!< IcFlix
    u32 microsoft_services:1;       //!< Microsoft Services
    u32 apple_services:1;           //!< Apple Services
    u32 pipifilm:1;                 //!< PPFilm
    u32 iflix:1;                    //!< iFlix
    u32 amazon_chime:1;             //!< Amazon Chime
    u32 nintendo_network:1;         //!< Nintendo Network
    u32 onedrive:1;                 //!< OneDrive
    u32 imgur:1;                    //!< Imgur
    u32 vpn_x:1;                    //!< VPN-X
    u32 giphy:1;                    //!< Giphy
    u32 channel_one:1;              //!< Channel One
    u32 google_earth:1;             //!< Google Earth
    u32 google_maps:1;              //!< Google Maps
    u32 tenor:1;                    //!< Tenor
    u32 icq:1;                      //!< ICQ
    u32 anydesk:1;                  //!< AnyDesk
    u32 tinder:1;                   //!< Tinder
    u32 ivi_ru:1;                   //!< Ivi-Ru
    u32 kinopoisk:1;                //!< Kinopoisk
    u32 aim:1;                      //!< AIM
    u32 icrypt:1;                   //!< iCrypt
    u32 mail_ru:1;                  //!< Mail-Ru
    u32 io:1;                       //!< iO
    u32 sky_go:1;                   //!< Sky Go
    u32 azar:1;                     //!< Azar
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 life_church:1;              //!< Life Church
    u32 vevo:1;                     //!< Vevo
    u32 grindr:1;                   //!< Grindr
    u32 apple_maps:1;               //!< Apple Maps
    u32 apple_geolocation:1;        //!< Apple Geolocation
    u32 uploaded:1;                 //!< uploaded
    u32 easypaste_org:1;            //!< easypaste-org
    u32 google_duo:1;               //!< Google Duo
    u32 youku_tudou:1;              //!< Youku Tudou
    u32 meetic_group:1;             //!< Meetic Group
    u32 sarahah:1;                  //!< Sarahah
    u32 vudu:1;                     //!< Vudu
    u32 wish:1;                     //!< Wish
    u32 auth0:1;                    //!< Auth0
    u32 slingtv:1;                  //!< Sling TV
    u32 google_apis:1;              //!< Google APIs
    u32 rackspace_cloud:1;          //!< Rackspace Cloud
    u32 google_calendar:1;          //!< Google Calendar
    u32 google_analytics:1;         //!< Google Analytics
    u32 stackpath_cloud:1;          //!< StackPath Cloud
    u32 houseparty:1;               //!< Houseparty
    u32 learninghub_online:1;       //!< LearningHub-online
    u32 google_ads:1;               //!< Google Ads
    u32 fox_sports:1;               //!< Fox Sports
    u32 amazon_services:1;          //!< Amazon Services
    u32 amazon_drive:1;             //!< Amazon Drive
    u32 crashlytics:1;              //!< Crashlytics
    u32 facebook_cloud:1;           //!< Facebook Cloud
    u32 _360total:1;                //!< 360 Total Security
    u32 turner:1;                   //!< Turner
    u32 loop_trade_classifieds:1;   //!< Loop Trade Classifieds
    u32 loop_caribbean_local_news:1;//!< Loop Caribbean Local News
    u32 loop_pacific_local_news:1;  //!< Loop Pacific Local News
    u32 unity:1;                    //!< Unity
    u32 funplus:1;                  //!< FunPlus
    u32 mapbox:1;                   //!< Mapbox
    u32 cedexis:1;                  //!< Cedexis
    u32 zenmate_vpn:1;              //!< Zenmate VPN
    u32 smule:1;                    //!< Smule
    u32 pia_vpn:1;                  //!< Private Internet Access
    u32 symantec:1;                 //!< Symantec
    u32 maxdome:1;                  //!< Maxdome
    u32 ipvanish:1;                 //!< IPVanish
    u32 tunnelbear:1;               //!< TunnelBear
    u32 google_app_engine:1;        //!< Google App Engine
    u32 wire_messenger:1;           //!< Wire Messenger
    u32 epic_browser_proxy:1;       //!< Epic Browser Proxy
    u32 zero_vpn:1;                 //!< Zero VPN
    u32 surfeasy:1;                 //!< Surfeasy
    u32 tiger_vpn:1;                //!< Tiger VPN
    u32 epic_games:1;               //!< Epic Games
    u32 avira_phantom_vpn:1;        //!< Avira Phantom VPN
    u32 turbo_vpn:1;                //!< Turbo VPN
    u32 iqiyi:1;                    //!< Iqiyi
    PACE2_DEPRECATED("Application vpn_master is marked as deprecated and should not be used anymore.")
    u32 vpn_master:1;               //!< VPN Master
    u32 pptv:1;                     //!< PPTV
    u32 snap_vpn:1;                 //!< Snap VPN
    u32 tokbox:1;                   //!< TokBox
    u32 vpn_unlimited:1;            //!< VPN Unlimited
    u32 hma_vpn:1;                  //!< HMA VPN
    u32 avast_services:1;           //!< Avast Services
    u32 nord_vpn:1;                 //!< Nord VPN
    u32 proton_vpn:1;               //!< Proton VPN
    u32 safer_vpn:1;                //!< SaferVPN
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 safer_vpn:1;                //!< SaferVPN
    u32 proton_vpn:1;               //!< Proton VPN
    u32 nord_vpn:1;                 //!< Nord VPN
    u32 avast_services:1;           //!< Avast Services
    u32 hma_vpn:1;                  //!< HMA VPN
    u32 vpn_unlimited:1;            //!< VPN Unlimited
    u32 tokbox:1;                   //!< TokBox
    u32 snap_vpn:1;                 //!< Snap VPN
    u32 pptv:1;                     //!< PPTV
    PACE2_DEPRECATED("Application vpn_master is marked as deprecated and should not be used anymore.")
    u32 vpn_master:1;               //!< VPN Master
    u32 iqiyi:1;                    //!< Iqiyi
    u32 turbo_vpn:1;                //!< Turbo VPN
    u32 avira_phantom_vpn:1;        //!< Avira Phantom VPN
    u32 epic_games:1;               //!< Epic Games
    u32 tiger_vpn:1;                //!< Tiger VPN
    u32 surfeasy:1;                 //!< Surfeasy
    u32 zero_vpn:1;                 //!< Zero VPN
    u32 epic_browser_proxy:1;       //!< Epic Browser Proxy
    u32 wire_messenger:1;           //!< Wire Messenger
    u32 google_app_engine:1;        //!< Google App Engine
    u32 tunnelbear:1;               //!< TunnelBear
    u32 ipvanish:1;                 //!< IPVanish
    u32 maxdome:1;                  //!< Maxdome
    u32 symantec:1;                 //!< Symantec
    u32 pia_vpn:1;                  //!< Private Internet Access
    u32 smule:1;                    //!< Smule
    u32 zenmate_vpn:1;              //!< Zenmate VPN
    u32 cedexis:1;                  //!< Cedexis
    u32 mapbox:1;                   //!< Mapbox
    u32 funplus:1;                  //!< FunPlus
    u32 unity:1;                    //!< Unity
    u32 loop_pacific_local_news:1;  //!< Loop Pacific Local News
    u32 loop_caribbean_local_news:1;//!< Loop Caribbean Local News
    u32 loop_trade_classifieds:1;   //!< Loop Trade Classifieds
    u32 turner:1;                   //!< Turner
    u32 _360total:1;                //!< 360 Total Security
    u32 facebook_cloud:1;           //!< Facebook Cloud
    u32 crashlytics:1;              //!< Crashlytics
    u32 amazon_drive:1;             //!< Amazon Drive
    u32 amazon_services:1;          //!< Amazon Services
    u32 fox_sports:1;               //!< Fox Sports
    u32 google_ads:1;               //!< Google Ads
    u32 learninghub_online:1;       //!< LearningHub-online
    u32 houseparty:1;               //!< Houseparty
    u32 stackpath_cloud:1;          //!< StackPath Cloud
    u32 google_analytics:1;         //!< Google Analytics
    u32 google_calendar:1;          //!< Google Calendar
    u32 rackspace_cloud:1;          //!< Rackspace Cloud
    u32 google_apis:1;              //!< Google APIs
    u32 slingtv:1;                  //!< Sling TV
    u32 auth0:1;                    //!< Auth0
    u32 wish:1;                     //!< Wish
    u32 vudu:1;                     //!< Vudu
    u32 sarahah:1;                  //!< Sarahah
    u32 meetic_group:1;             //!< Meetic Group
    u32 youku_tudou:1;              //!< Youku Tudou
    u32 google_duo:1;               //!< Google Duo
    u32 easypaste_org:1;            //!< easypaste-org
    u32 uploaded:1;                 //!< uploaded
    u32 apple_geolocation:1;        //!< Apple Geolocation
    u32 apple_maps:1;               //!< Apple Maps
    u32 grindr:1;                   //!< Grindr
    u32 vevo:1;                     //!< Vevo
    u32 life_church:1;              //!< Life Church
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u32 hide_me_vpn:1;              //!< HideMe VPN
    u32 express_vpn:1;              //!< ExpressVPN
    u32 tunnelguru:1;               //!< TunnelGuru
    u32 youtube_music:1;            //!< Youtube Music
    u32 windscribe_vpn:1;           //!< Windscribe VPN
    u32 ivpn:1;                     //!< IVPN
    u32 discord:1;                  //!< Discord
    u32 firevpn:1;                  //!< FireVPN
    u32 botim:1;                    //!< BOTIM
    u32 hi_vpn:1;                   //!< Hi VPN
    u32 vpn_proxy_master:1;         //!< VPN Proxy Master
    u32 zoogvpn:1;                  //!< ZOOGVPN
    u32 ivacy_vpn:1;                //!< Ivacy VPN
    u32 purevpn:1;                  //!< PureVPN
    u32 mullvad_vpn:1;              //!< Mullvad VPN
    u32 freedome_vpn:1;             //!< Freedome VPN
    u32 easy_vpn:1;                 //!< Easy VPN
    u32 strong_vpn:1;               //!< StrongVPN
    u32 buffered_vpn:1;             //!< Buffered VPN
    u32 wargaming_net:1;            //!< Wargaming-net
    u32 yandex_music:1;             //!< Yandex Music
    u32 vpn_area:1;                 //!< VPN Area
    u32 steganos_online_shield_vpn:1;//!< Steganos Online Shield VPN
    u32 supervpn:1;                 //!< SuperVPN
    u32 steganos_services:1;        //!< Steganos Services
    u32 froot_vpn:1;                //!< FrootVPN
    u32 bbm:1;                      //!< BlackBerry Messenger
    u32 speed_vpn_inventive:1;      //!< Speed VPN Inventive
    u32 cme:1;                      //!< CME
    u32 yandex_navigator:1;         //!< Yandex Navigator
    u32 yandex_disk:1;              //!< Yandex Disk
    u32 yandex_mail:1;              //!< Yandex Mail
    u32 tiktok:1;                   //!< TikTok
    u32 yandex_maps:1;              //!< Yandex Maps
    u32 touch_vpn:1;                //!< Touch VPN
    u32 gaana:1;                    //!< Gaana
    u32 xcum_com:1;                 //!< xCum-com
    u32 life_ru:1;                  //!< Life-Ru
    u32 vpn_in_touch:1;             //!< VPN in Touch
    u32 speed_vpn_mad:1;            //!< Speed VPN MobileAppDev
    u32 dash_vpn:1;                 //!< Dash VPN
    u32 yeecall:1;                  //!< YeeCall
    u32 shellfire_vpn:1;            //!< Shellfire VPN
    u32 ibvpn:1;                    //!< IbVPN
    u32 ren_tv:1;                   //!< Ren-Tv
    u32 hola_vpn:1;                 //!< Hola VPN
    u32 hola_services:1;            //!< Hola Services
    u32 cdnvideo:1;                 //!< CDNvideo
    u32 atom_vpn:1;                 //!< Atom VPN
    u32 arkvpn:1;                   //!< ArkVPN
    u32 fortnite:1;                 //!< Fortnite
    u32 amaze_vpn:1;                //!< Amaze VPN
    u32 vungle:1;                   //!< Vungle
    u32 vivox:1;                    //!< Vivox
    u32 megogo:1;                   //!< Megogo
    u32 finch_vpn:1;                //!< Finch VPN
    u32 fast_secure_vpn:1;          //!< Fast Secure VPN
    u32 tvplayer:1;                 //!< TVPlayer
    u32 world_of_tanks:1;           //!< World Of Tanks
    u32 vpn_easy:1;                 //!< VPN Easy
    u32 zpn_vpn:1;                  //!< ZPN VPN
    u32 appvpn:1;                   //!< AppVPN
    u32 x_vpn:1;                    //!< X-VPN
    u32 dot_vpn:1;                  //!< DotVPN
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u32 dot_vpn:1;                  //!< DotVPN
    u32 x_vpn:1;                    //!< X-VPN
    u32 appvpn:1;                   //!< AppVPN
    u32 zpn_vpn:1;                  //!< ZPN VPN
    u32 vpn_easy:1;                 //!< VPN Easy
    u32 world_of_tanks:1;           //!< World Of Tanks
    u32 tvplayer:1;                 //!< TVPlayer
    u32 fast_secure_vpn:1;          //!< Fast Secure VPN
    u32 finch_vpn:1;                //!< Finch VPN
    u32 megogo:1;                   //!< Megogo
    u32 vivox:1;                    //!< Vivox
    u32 vungle:1;                   //!< Vungle
    u32 amaze_vpn:1;                //!< Amaze VPN
    u32 fortnite:1;                 //!< Fortnite
    u32 arkvpn:1;                   //!< ArkVPN
    u32 atom_vpn:1;                 //!< Atom VPN
    u32 cdnvideo:1;                 //!< CDNvideo
    u32 hola_services:1;            //!< Hola Services
    u32 hola_vpn:1;                 //!< Hola VPN
    u32 ren_tv:1;                   //!< Ren-Tv
    u32 ibvpn:1;                    //!< IbVPN
    u32 shellfire_vpn:1;            //!< Shellfire VPN
    u32 yeecall:1;                  //!< YeeCall
    u32 dash_vpn:1;                 //!< Dash VPN
    u32 speed_vpn_mad:1;            //!< Speed VPN MobileAppDev
    u32 vpn_in_touch:1;             //!< VPN in Touch
    u32 life_ru:1;                  //!< Life-Ru
    u32 xcum_com:1;                 //!< xCum-com
    u32 gaana:1;                    //!< Gaana
    u32 touch_vpn:1;                //!< Touch VPN
    u32 yandex_maps:1;              //!< Yandex Maps
    u32 tiktok:1;                   //!< TikTok
    u32 yandex_mail:1;              //!< Yandex Mail
    u32 yandex_disk:1;              //!< Yandex Disk
    u32 yandex_navigator:1;         //!< Yandex Navigator
    u32 cme:1;                      //!< CME
    u32 speed_vpn_inventive:1;      //!< Speed VPN Inventive
    u32 bbm:1;                      //!< BlackBerry Messenger
    u32 froot_vpn:1;                //!< FrootVPN
    u32 steganos_services:1;        //!< Steganos Services
    u32 supervpn:1;                 //!< SuperVPN
    u32 steganos_online_shield_vpn:1;//!< Steganos Online Shield VPN
    u32 vpn_area:1;                 //!< VPN Area
    u32 yandex_music:1;             //!< Yandex Music
    u32 wargaming_net:1;            //!< Wargaming-net
    u32 buffered_vpn:1;             //!< Buffered VPN
    u32 strong_vpn:1;               //!< StrongVPN
    u32 easy_vpn:1;                 //!< Easy VPN
    u32 freedome_vpn:1;             //!< Freedome VPN
    u32 mullvad_vpn:1;              //!< Mullvad VPN
    u32 purevpn:1;                  //!< PureVPN
    u32 ivacy_vpn:1;                //!< Ivacy VPN
    u32 zoogvpn:1;                  //!< ZOOGVPN
    u32 vpn_proxy_master:1;         //!< VPN Proxy Master
    u32 hi_vpn:1;                   //!< Hi VPN
    u32 botim:1;                    //!< BOTIM
    u32 firevpn:1;                  //!< FireVPN
    u32 discord:1;                  //!< Discord
    u32 ivpn:1;                     //!< IVPN
    u32 windscribe_vpn:1;           //!< Windscribe VPN
    u32 youtube_music:1;            //!< Youtube Music
    u32 tunnelguru:1;               //!< TunnelGuru
    u32 express_vpn:1;              //!< ExpressVPN
    u32 hide_me_vpn:1;              //!< HideMe VPN
#endif

// END OF PACE2_applications_struct

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u64 padding_14;
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u64 padding_14;
#endif

#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
    u64 padding_15;
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
    u64 padding_15;
#endif

} PACE2_applications_struct;

/**
 * List of strings, each identifying the appropriate application.
 *
 * Items do not change between releases. Only for new applications strings are added.
 */
#define PACE2_APPLICATIONS_SHORT_STRS \
    "ukn",\
    "facebook",\
    "magicjack",\
    "itunes",\
    "myspace",\
    "facetime",\
    "truphone",\
    "twitter",\
    "xbox",\
    "realmedia",\
    "google-mail",\
    "goober",\
    "icloud",\
    "ubuntuone",\
    "dropbox",\
    "yuilop",\
    "imessage",\
    "gotomypc",\
    "gotomeeting",\
    "windows-azure",\
    "amazon-cloud",\
    "dailymotion",\
    "deezer",\
    "grooveshark",\
    "sudaphone",\
    "office365",\
    "cntv",\
    "sinatv",\
    "youtube",\
    "voipswitch",\
    "zynga",\
    "crimecity",\
    "modernwar",\
    "vippie",\
    "box",\
    "skydrive",\
    "adobe-creative-cloud",\
    "linkedin",\
    "orkut",\
    "hi5",\
    "soribada",\
    "zoho-work-online",\
    "hotmail-webmail",\
    "instagram",\
    "sugarsync",\
    "wickr",\
    "tigertext",\
    "vibe",\
    "enlegion",\
    "voxer",\
    "lovefilm",\
    "friendster",\
    "burner",\
    "flickr",\
    "live-com",\
    "icq-webchat",\
    "redtube",\
    "evernote",\
    "spreed",\
    "textplus",\
    "xvideos",\
    "snapchat",\
    "youku",\
    "mymusic-tw",\
    "renren",\
    "yelp",\
    "amazon-shop",\
    "youporn",\
    "ebay",\
    "pornhub",\
    "spideroak",\
    "adrive",\
    "tumblr",\
    "xing",\
    "google-play-music",\
    "vimeo",\
    "salesforce",\
    "itunes-radio",\
    "sina-weibo",\
    "forfone",\
    "livejasmin",\
    "jingdong",\
    "lastfm",\
    "ims",\
    "baidu",\
    "taobao",\
    "watchever",\
    "ttpod",\
    "360mobile",\
    "pinterest",\
    "vine",\
    "net2phone",\
    "foursquare",\
    "ilive-to",\
    "mega",\
    "doook",\
    "vbulletin",\
    "cloudme",\
    "yikyak",\
    "redmine",\
    "iheartradio",\
    "avira-antivir",\
    "norton-antivir",\
    "kaspersky-antivir",\
    "avg-antivir",\
    "soundcloud",\
    "rdio",\
    "amazon-prime-music",\
    "runkeeper",\
    "mybook",\
    "nikeplus",\
    "magv",\
    "magv-kids",\
    "dota-legend",\
    "letv",\
    "my-fitness-pal",\
    "sohu",\
    "1kxun",\
    "tvbs",\
    "xnxx",\
    "ctitv",\
    "endomondo",\
    "timely-tv",\
    "workout-trainer",\
    "mapmyrun",\
    "king-of-pirates",\
    "tudou",\
    "bit",\
    "edk",\
    "kazaa",\
    "gnu",\
    "directconnect",\
    "apple",\
    "soul",\
    "ares",\
    "freenet",\
    "mute",\
    "skype",\
    "msn",\
    "yahoo",\
    "vonage",\
    "oscar",\
    "gadugadu",\
    "mplus",\
    "irc",\
    "sling",\
    "vcast",\
    "4shared-com",\
    "adrive-com",\
    "badongo-com",\
    "badongo-net",\
    "bestsharing-com",\
    "bigfilez-com",\
    "chinamofile-com",\
    "data-hu",\
    "depositfiles-com",\
    "easy-share-com",\
    "fast-load-net",\
    "fast-share-com",\
    "file-upload-net",\
    "file-upload-to",\
    "filearchiv-ru",\
    "filebase-to",\
    "filecloud-com",\
    "filefactory-com",\
    "filefront-com",\
    "fileho-com",\
    "filehost-tv",\
    "filepost-ru",\
    "filer-net",\
    "files-upload-com",\
    "files-to",\
    "filesafe-to",\
    "filesend-net",\
    "fileupyours-com",\
    "ftp2share-com",\
    "gigasize-com",\
    "gigeshare-com",\
    "hostggg-com",\
    "hotfile-com",\
    "ifolder-ru",\
    "keepmyfile-com",\
    "leteckaposta-cz",\
    "live-share-com",\
    "livedepot-net",\
    "load-to",\
    "massmirror-com",\
    "mediafire-com",\
    "megarotic-com",\
    "megashare-com",\
    "megashares-com",\
    "megaupload-com",\
    "mofile-com",\
    "mofile-net",\
    "mytempdir-com",\
    "netload-in",\
    "odsiebie-najlepsze-net",\
    "quicksharing-com",\
    "rapidshare-com",\
    "rapidshare-de",\
    "rapidupload-com",\
    "savefile-com",\
    "sendmefile-com",\
    "sendspace-com",\
    "sharebase-to",\
    "sharebee-com",\
    "sharebig-com",\
    "sharebigfile-com",\
    "sharebigflie-com",\
    "simpleupload-net",\
    "speedshare-org",\
    "turboupload-com",\
    "ultrashare-de",\
    "up-file-com",\
    "uploaded-to",\
    "uploading-com",\
    "uploadingit-com",\
    "uploadpower-com",\
    "uploadyourfiles-de",\
    "vip-file-com",\
    "wiiupload-net",\
    "yourfilehost-com",\
    "yourfiles-biz",\
    "youtube-com",\
    "zshare-net",\
    "4shared-vn",\
    "mega-1280-com",\
    "megashare-vn",\
    "fileserve-com",\
    "filesonic-com",\
    "1-upload-to",\
    "1fichier-com",\
    "4fastfile-com",\
    "10upload-com",\
    "1clickshare-net",\
    "filedude-com",\
    "asixfiles-com",\
    "banashare-com",\
    "biggerupload-com",\
    "bitshare-com",\
    "cash-file-net",\
    "cramit-in",\
    "czshare-com",\
    "data-loading-com",\
    "dataport-cz",\
    "datei-to",\
    "downupload-com",\
    "easybytez-com",\
    "edisk-cz",\
    "enterupload-com",\
    "euroshare-eu",\
    "extabit-com",\
    "fastfileshare-com-ar",\
    "fik1-com",\
    "file2upload-net",\
    "fileape-com",\
    "fileflyer-com",\
    "filefrog-com",\
    "filehook-com",\
    "fileover-net",\
    "filepost-com",\
    "files-mail-ru",\
    "fileshare-in-ua",\
    "filesmonster-com",\
    "filevo-com",\
    "filezup-com",\
    "freakshare-net",\
    "fshare-vn",\
    "gaiafile-com",\
    "gigapeta-com",\
    "gotoupload-com",\
    "hellshare-com",\
    "hitfile-net",\
    "hulkshare-com",\
    "hyperfileshare-com",\
    "i-filez-com",\
    "ifile-it",\
    "kewlshare-com",\
    "letitbit-net",\
    "loadfiles-in",\
    "megafree-kz",\
    "megarapid-eu",\
    "megashare-vnn-vn",\
    "midupload-com",\
    "multishare-cz",\
    "netuploaded-com",\
    "oron-com",\
    "przeklej-pl",\
    "putshare-com",\
    "qshare-com",\
    "quickshare-cz",\
    "rapidshark-pl",\
    "scribd-com",\
    "share-online-bz",\
    "share-rapid-cz",\
    "shragle-com",\
    "storage-to",\
    "terafiles-net",\
    "tunabox-net",\
    "turbobit-net",\
    "ugotfile-com",\
    "uloz-to",\
    "unibytes-com",\
    "unlimit-co-il",\
    "up-4share-vn",\
    "upload-com-ua",\
    "uploadbox-com",\
    "uploader-pl",\
    "uploadfloor-com",\
    "uploadstation-com",\
    "upnito-sk",\
    "usershare-net",\
    "videobb-com",\
    "wupload-com",\
    "x7-to",\
    "yunfile-to",\
    "zomgupload-to",\
    "ziddu-com",\
    "1-upload-com",\
    "1-clickshare-com",\
    "123upload-pl",\
    "139pan-com",\
    "163pan-com",\
    "2shared-com",\
    "4share-ws",\
    "4us-to",\
    "alldrives-ge",\
    "allshares-ge",\
    "annonhost-net",\
    "boosterking-com",\
    "cobrashare-sk",\
    "coolshare-cz",\
    "crocko-com",\
    "cum-com",\
    "eyvx-com",\
    "filearn-com",\
    "filejungle-com",\
    "filemaze-ws",\
    "filesflash-com",\
    "freespace-by",\
    "fsx-hu",\
    "jakfile-com",\
    "jumbofiles-com",\
    "kickload-com",\
    "missupload-com",\
    "movreel-com",\
    "platinshare-com",\
    "sanupload-com",\
    "shareflare-net",\
    "u-115-com",\
    "uploadc-com",\
    "uploadking-com",\
    "uptobox-com",\
    "filestube-com",\
    "gamefront-com",\
    "megaporn-com",\
    "filesonic-in",\
    "turbobit-ru",\
    "usenet",\
    "ftp",\
    "filetopia",\
    "manolito",\
    "imesh",\
    "pando",\
    "sopcast",\
    "tvuplayer",\
    "ppstream",\
    "pplive",\
    "zattoo",\
    "qq",\
    "qqlive",\
    "feidian",\
    "popo",\
    "thunder",\
    "uusee",\
    "vnc",\
    "steam",\
    "hl2",\
    "softether",\
    "telnet",\
    "off",\
    "tor",\
    "worldofwarcraft",\
    "hamachi",\
    "skinny",\
    "citrix",\
    "paltalk",\
    "quake",\
    "secondlife",\
    "battlefield",\
    "pcanywhere",\
    "icecast",\
    "shoutcast",\
    "qqgame",\
    "veohtv",\
    "openft",\
    "pandora",\
    "ipp",\
    "warcraft3",\
    "oovoo",\
    "google-hangouts",\
    "iskoot",\
    "fring",\
    "iptv",\
    "yourfreedom",\
    "voipswitchvoiptunnel",\
    "meebo",\
    "funshion",\
    "aimini",\
    "clubpenguin",\
    "florensia",\
    "teamspeak",\
    "maplestory",\
    "osh",\
    "ps3",\
    "dofus",\
    "fiesta",\
    "splashfighter",\
    "activesync",\
    "nimbuzz",\
    "crossfire",\
    "wapwsp",\
    "guildwars",\
    "armagetron",\
    "rfactor",\
    "teamviewer",\
    "gamekit",\
    "ultrabac",\
    "spotify",\
    "implus",\
    "antsp2p",\
    "ultrasurf",\
    "viber",\
    "tango",\
    "iplayer",\
    "operamini",\
    "bolt",\
    "scydo",\
    "whatsapp",\
    "webex",\
    "netflix",\
    "mojo",\
    "imo",\
    "citrixgoto",\
    "ficall",\
    "comodounite",\
    "ventrilo",\
    "mypeople",\
    "ebuddy",\
    "adobeconnect",\
    "jap",\
    "wuala",\
    "webdav",\
    "lync",\
    "audiogalaxy",\
    "mig33",\
    "httptunnel",\
    "rhapsody",\
    "webqq",\
    "pdproxy",\
    "cyberghost",\
    "google-cloud",\
    "google-drive",\
    "google-docs",\
    "google-plus",\
    "oracledb",\
    "itv",\
    "demand5",\
    "rsvp",\
    "channel4od",\
    "lotusnotes",\
    "poisonivy",\
    "hidrive",\
    "wechat",\
    "myvideo",\
    "kik",\
    "line",\
    "iperf",\
    "osupdate",\
    "hike",\
    "didi",\
    "idrive",\
    "spreecast",\
    "simfy",\
    "dota2",\
    "qvod",\
    "hulu",\
    "msexchange",\
    "msnav",\
    "fuze",\
    "kakaotalk",\
    "destiny",\
    "vyprvpn",\
    "threema",\
    "ntlm",\
    "twitch",\
    "wowza",\
    "runtastic",\
    "telegram",\
    "streetvoice",\
    "duokan",\
    "mei-lu",\
    "skout",\
    "playtales",\
    "gameofwar",\
    "fetnet",\
    "fbs",\
    "national-baseball",\
    "pinkfong",\
    "kollect-book",\
    "sptg-tv",\
    "ciaomobile-hdtv",\
    "skyking",\
    "moves",\
    "path",\
    "aowe",\
    "iwow-systex",\
    "9ku",\
    "nexttv",\
    "syncplicity",\
    "realma",\
    "yuanta-web",\
    "hinet",\
    "mycard",\
    "kono",\
    "speakaboos",\
    "duo-duo",\
    "wealth-god",\
    "ileader",\
    "three-bamboo",\
    "taaze",\
    "audiko",\
    "pubu-bookbuffet",\
    "mitake",\
    "hyread",\
    "disney-storytime",\
    "qualys",\
    "constant-contact",\
    "rally",\
    "maginetv",\
    "intralinks",\
    "liveperson",\
    "speedtest",\
    "bluejeans",\
    "apple-music",\
    "naver",\
    "booking-com",\
    "cnn",\
    "periscope",\
    "payeasy",\
    "line-rangers",\
    "ios-app-store",\
    "myvideo-tw",\
    "king-gaming",\
    "candy-crush",\
    "candy-crush-soda",\
    "bubble-witch",\
    "farm-hero",\
    "playstation",\
    "google-play",\
    "supercell",\
    "clash-of-clans",\
    "boom-beach",\
    "hay-day",\
    "windows-store",\
    "skype-for-business",\
    "minecraft",\
    "meerkat",\
    "rovio-gaming",\
    "angry-birds",\
    "citrix-xenapp",\
    "reddit",\
    "league-of-legends",\
    "battle-net",\
    "paypal",\
    "plants-vs-zombies-2",\
    "halfbrick-studios",\
    "github",\
    "hamivideo",\
    "slacker",\
    "niconico",\
    "alipay",\
    "ustream",\
    "zalo",\
    "docomo",\
    "softbank-onlineshop",\
    "guildwars2",\
    "trendmicro-worryfree",\
    "wikipedia",\
    "uber",\
    "yandex",\
    "vk",\
    "odnoklassniki",\
    "crashplan",\
    "alibaba",\
    "aliexpress",\
    "tmall",\
    "ea-origin",\
    "norton-security",\
    "tibia",\
    "pochta-rossii",\
    "after-school",\
    "drivelock",\
    "qip-ru",\
    "musical-ly",\
    "ea-games",\
    "itunesu",\
    "popcap",\
    "surveymonkey",\
    "new-relic",\
    "huddle",\
    "zendesk",\
    "outlookmobile",\
    "chatcube",\
    "dena",\
    "backblaze",\
    "docusign",\
    "betternet",\
    "good",\
    "anghami",\
    "path-of-exile",\
    "slack",\
    "pokemon-go",\
    "niantic-labs",\
    "2gis",\
    "rutube",\
    "2channel",\
    "prisma",\
    "clash-royal",\
    "upptalk",\
    "amazon-prime-video",\
    "clash-royale",\
    "google-services",\
    "signal",\
    "kankan",\
    "share-online",\
    "ultrashare",\
    "yunfile",\
    "edisk",\
    "123upload",\
    "azar",\
    "sky-go",\
    "io",\
    "mail-ru",\
    "icrypt",\
    "aim",\
    "kinopoisk",\
    "ivi-ru",\
    "tinder",\
    "anydesk",\
    "icq",\
    "tenor",\
    "google-maps",\
    "google-earth",\
    "channel-one",\
    "giphy",\
    "vpn-x",\
    "imgur",\
    "onedrive",\
    "nintendo-network",\
    "amazon-chime",\
    "iflix",\
    "ppfilm",\
    "apple-services",\
    "microsoft-services",\
    "icflix",\
    "logmein",\
    "psiphon",\
    "waze",\
    "google-fcm",\
    "apns",\
    "digicel-playgo",\
    "shazam",\
    "digicel-music",\
    "digicel-topup",\
    "tunein-radio",\
    "dubsmash",\
    "synology",\
    "my-digicel",\
    "google-allo",\
    "rusfolder",\
    "badoo",\
    "airbnb",\
    "hightail",\
    "easy-taxi",\
    "tomtom",\
    "tripadvisor",\
    "espn",\
    "dazn",\
    "mobile-strike",\
    "king-of-avalon",\
    "lineage-2",\
    "lyft",\
    "trulia",\
    "akamai-cloud",\
    "limelight-cloud",\
    "nickelodeon-play",\
    "here",\
    "simcity-buildit",\
    "starwars-goh",\
    "fastly",\
    "sap",\
    "wokf",\
    "cloudflare",\
    "life-church",\
    "vevo",\
    "grindr",\
    "apple-maps",\
    "apple-geolocation",\
    "uploaded",\
    "easypaste-org",\
    "google-duo",\
    "youku-tudou",\
    "meetic-group",\
    "sarahah",\
    "vudu",\
    "wish",\
    "auth0",\
    "slingtv",\
    "google-apis",\
    "rackspace-cloud",\
    "google-calendar",\
    "google-analytics",\
    "stackpath-cloud",\
    "houseparty",\
    "learninghub-online",\
    "google-ads",\
    "fox-sports",\
    "amazon-services",\
    "amazon-drive",\
    "crashlytics",\
    "facebook-cloud",\
    "360total",\
    "turner",\
    "loop-trade-classifieds",\
    "loop-caribbean-local-news",\
    "loop-pacific-local-news",\
    "unity",\
    "funplus",\
    "mapbox",\
    "cedexis",\
    "zenmate-vpn",\
    "smule",\
    "pia-vpn",\
    "symantec",\
    "maxdome",\
    "ipvanish",\
    "tunnelbear",\
    "google-app-engine",\
    "wire-messenger",\
    "epic-browser-proxy",\
    "zero-vpn",\
    "surfeasy",\
    "tiger-vpn",\
    "epic-games",\
    "avira-phantom-vpn",\
    "turbo-vpn",\
    "iqiyi",\
    "vpn-master",\
    "pptv",\
    "snap-vpn",\
    "tokbox",\
    "vpn-unlimited",\
    "hma-vpn",\
    "avast-services",\
    "nord-vpn",\
    "proton-vpn",\
    "safer-vpn",\
    "hide-me-vpn",\
    "express-vpn",\
    "tunnelguru",\
    "youtube-music",\
    "windscribe-vpn",\
    "ivpn",\
    "discord",\
    "firevpn",\
    "botim",\
    "hi-vpn",\
    "vpn-proxy-master",\
    "zoogvpn",\
    "ivacy-vpn",\
    "purevpn",\
    "mullvad-vpn",\
    "freedome-vpn",\
    "easy-vpn",\
    "strong-vpn",\
    "buffered-vpn",\
    "wargaming-net",\
    "yandex-music",\
    "vpn-area",\
    "steganos-online-shield-vpn",\
    "supervpn",\
    "steganos-services",\
    "froot-vpn",\
    "bbm",\
    "speed-vpn-inventive",\
    "cme",\
    "yandex-navigator",\
    "yandex-disk",\
    "yandex-mail",\
    "tiktok",\
    "yandex-maps",\
    "touch-vpn",\
    "gaana",\
    "xcum-com",\
    "life-ru",\
    "vpn-in-touch",\
    "speed-vpn-mad",\
    "dash-vpn",\
    "yeecall",\
    "shellfire-vpn",\
    "ibvpn",\
    "ren-tv",\
    "hola-vpn",\
    "hola-services",\
    "cdnvideo",\
    "atom-vpn",\
    "arkvpn",\
    "fortnite",\
    "amaze-vpn",\
    "vungle",\
    "vivox",\
    "megogo",\
    "finch-vpn",\
    "fast-secure-vpn",\
    "tvplayer",\
    "world-of-tanks",\
    "vpn-easy",\
    "zpn-vpn",\
    "appvpn",\
    "x-vpn",\
    "dot-vpn",\
    "encrypt-me",\
    "linkvpn",\
    "droid-vpn",\
    "browsec-vpn",\
    "enmasse-games",\
    "hotspot-vpn",\
    "apex-legends",\
    "vpn-365",\
    "counterstrike-go",\
    "vpn-tor-browser",\
    "supervpn-by-cheng",\
    "act-vpn",\
    "brightcove",\
    "sportbox-ru",\
    "just-proxy-vpn",\
    "veepn",\
    "ookla",\
    "eagle-vpn",\
    "bria-mobile",\
    "vpn-gate",\
    "anchorfree-services",\
    "bria-teams",\
    "shadowsocks",\
    "asphalt9",\
    "gameloft-services",\
    "actmobile-services",\
    "novatech-core",\
    "secure-vpn",\
    "appsflyer",\
    "adjust",\
    "genievpn",\
    "tunvpn",\
    "helpshift",\
    "fastvpn-master",\
    "vpn-unlimited-proxy",\
    "highspeed-vpn",\
    "vpn-master-andro",\
    "mixer",\
    "tencent-games",\
    "open-super-hotspot-vpn",\
    "vpn-proxy-master-unlimited",\
    "douyu",\
    "ingress-prime",\
    "electronic-arts",\
    "radioplayer",\
    "mopub",\
    "pubg-mobile",\
    "7tv",\
    "spiegel-tv",\
    "spiegel-services",\
    "eurosport-player",\
    "forge-of-empires",\
    "innogames",\
    "receiver-radio",\
    "goodgame-studios",\
    "wizards-unite",\
    "radio-de",\
    "bbc-news",\
    "bbc-services",\
    "unreal-engine",\
    "zoom",\
    "adobe-services",\
    "att-services",\
    "funk",\
    "nexx-tv",\
    "unblock-sites",\
    "brawl-stars",\
    "counterpath-services",\
    "snap-master-vpn",\
    "joyn",\
    "pro7sat1-services",\
    "dead-rivals",\
    "tapjoy",\
    "igg",\
    "lords-mobile",\
    "mudfish",\
    "inventive-vpn-services",\
    "mixpanel",\
    "adcolony",\
    "soma",\
    "big-farm",\
    "faceapp",\
    "asphalt8",\
    "moat",\
    "adsafeprotected",\
    "adform",\
    "freewheel",\
    "spotx",\
    "ironsource",\
    "trafficjunky",\
    "trafficfactory",\
    "hatch-cloud-gaming",\
    "applovin",\
    "deltadna",\
    "adtiming",\
    "flurry",\
    "optimizely",\
    "smartadserver",\
    "exactag",\
    "scorecardresearch",\
    "conviva",\
    "bamtech-media",\
    "bebi",\
    "hbo-services",\
    "heyzap",\
    "criteo",\
    "swrve",\
    "chartboost",\
    "accountkit",\
    "directv",\
    "vpnsecure",\
    "liftoff",\
    "level3-cloud",\
    "disney-services",\
    "my5",\
    "source-engine",\
    "geocaching",\
    "fox-news",\
    "kaltura",\
    "vdms",\
    "streamcraft",\
    "fox-services",\
    "branch",\
    "viki",\
    "wordpress",\
    "gravatar",\
    "parsely",\
    "wegamers",\
    "pbs",\
    "crackle",\
    "openx",\
    "rubicon-project",\
    "disney-radio",\
    "kochava",\
    "abc",\
    "streamtheworld",\
    "apteligent",\
    "apple-news",\
    "hbo-now",\
    "segment",\
    "tune",\
    "apple-tv",\
    "apptentive",\
    "airable",\
    "pluto-tv",\
    "disney-channel",\
    "discovery-communications",\
    "sony-pictures",\
    "nfl-sunday-ticket",\
    "pornhub-network",\
    "pdanet",\
    "mlb",\
    "mlb-at-bat",\
    "tubi",\
    "watch-tbs",\
    "disney-plus",\
    "opensignal",\
    "showtime",\
    "quantum-metric",\
    "mparticle",\
    "showtime-anytime",\
    "canal-plus",\
    "audible",\
    "tealium",\
    "nuance",\
    "cbs-services",\
    "nbc-sports",\
    "eurosport-services",\
    "endeavor-streaming",\
    "totok",\
    "fyber",\
    "univision-now",\
    "taboola",\
    "adrise",\
    "sky-services",\
    "uservoice",\
    "bigo-services",\
    "bittorrent-now",\
    "mario-kart-tour",\
    "turkcell-services",\
    "sky-sports",\
    "bip-messenger",\
    "swoogo",\
    "nbc-services",\
    "etisalat-services",\
    "switch-tv",\
    "microsoft-teams",\
    "flashtalking",\
    "youtube-go",\
    "symantec-vpn-services",\
    "nielsen-services",\
    "fox-business",\
    "privatevpn",\
    "surfshark-vpn",\
    "yospace",\
    "sapling-growth-vpn",\
    "nfl-services",\
    "xunlei-services",\
    "the-nbc-app",\
    "fizy",\
    "cloudinary",\
    "tencent-services",\
    "ultravpn",\
    "pladform",\
    "starz",\
    "wechat-work",\
    "resilio-sync",\
    "imgix",\
    "amazon-prime-now",\
    "kakao-services",\
    "solvvy",\
    "jwplayer",\
    "stackla",\
    "mux",\
    "freshdesk",\
    "freshworks",\
    "amplitude",\
    "appnexus",\
    "under-armour-services",\
    "mapmyfitness",\
    "trustarc",\
    "microsoft-excel",\
    "microsoft-word",\
    "microsoft-powerpoint",\
    "microsoft-outlook",\
    "snap-services",\
    "bootstrapcdn",\
    "kakaostory",\
    "playstation-video",\
    "sportsmax",\
    "kakaotv-live",\
    "applicaster",\
    "microsoft-sharepoint",\
    "cicso-services",\
    "index-exchange",\
    "nbc-news",\
    "fubo-tv",\
    "appdynamics",\
    "tour-tracker-grand-tours",\
    "blackboard",\
    "chartbeat",\
    "e-bot7",\
    "microsoft-onenote",\
    "hbo-go",\
    "mediamath",\
    "vilynx",\
    "wwe",\
    "topbuzz",\
    "nbc-sports-scores",\
    "sentry",\
    "yupptv",\
    "perimeterx",\
    "nba",\
    "playrix",\
    "app-bazaar",\
    "hotjar",\
    "taplytics",\
    "rtve-alacarta",\
    "hayu",\
    "gigya",\
    "tnt",\
    "showmax",\
    "atresmedia",\
    "cdnetworks",\
    "google-classroom",\
    "peer5",\
    "satisfi-labs",\
    "ayoba",\
    "roblox",\
    "nhl",\
    "innovative-connecting",\
    "outbrain",\
    "stan",\
    "vpn-monster",\
    "mediacorp-services",\
    "atresplayer",\
    "airship",\
    "didi-services",\
    "mediaset-espana",\
    "mitele",\
    "mewatch",\
    "hbo-max",\
    "warnermedia-services",\
    "nike-services",\
    "accedo",\
    "99taxis",\
    "charter-communications",\
    "viu",\
    "pccw",\
    "alibaba-cloud",\
    "localytics",\
    "tidal",\
    "spot-im",\
    "youbora",\
    "google-meet",\
    "coinbase",\
    "wide-orbit",\
    "meetrics",\
    "conductrics",\
    "launchdarkly",\
    "art19",\
    "hivemq",\
    "handelsblatt",\
    "podigee",\
    "cloud-mail-ru",\
    "innovid",\
    "movistar-services",\
    "ucweb",\
    "wizrocket",\
    "batch",\
    "vizbee",\
    "wirtschaftswoche",\
    "handelsblatt-services",\
    "iq-digital",\
    "meet-group-services",\
    "movistar-plus",\
    "wizzee",\
    "intercom",\
    "netease",\
    "penthera",\
    "riot-games",\
    "bytedance",\
    "appsee",\
    "tagged",\
    "webtoon",\
    "meetme",\
    "dynamic-yield",\
    "elex-technology",\
    "clash-of-kings",\
    "imdb",\
    "billo",\
    "aihelp",\
    "crunchyroll",\
    "shein",\
    "avira-services",\
    "samsung-services",\
    "xigua-video",\
    "toutiao",\
    "buzzsprout",\
    "megaphone",\
    "bugsnag",\
    "knotch",\
    "wurl",\
    "libsyn",\
    "triton-digital",\
    "vng-corporation",\
    "baidu-maps",\
    "moonton",\
    "amagi",\
    "kuaishou",\
    "markmonitor",\
    "medium",\
    "simplecast",\
    "walmart",\
    "grubhub",\
    "kwai",\
    "braze",\
    "keepsolid-services",\
    "vpn-lite",\
    "springserve",\
    "zynn",\
    "workplace",\
    "yaani",\
    "washington-post",\
    "curalate",\
    "romwe",\
    "target-signature",\
    "plex",\
    "amazon-alexa",\
    "bumble",\
    "weather-channel",\
    "reuters-news",\
    "cryptotrader",\
    "cxense",\
    "club-factory",\
    "bombora",\
    "moov",\
    "collectd",\
    "oracle-services",\
    "comcast",\
    "skred-messenger",\
    "acast",\
    "the-cw",\
    "cw-seed",\
    "facebook-lite",\
    "opera-services",\
    "the-wall-street-journal",\
    "textnow",\
    "bloomberg",\
    "udemy",\
    "the-economist",\
    "investing-com-services",\
    "cryptowatch",\
    "among-us",\
    "cookiepro",\
    "bein-sports",\
    "bein-media-services",\
    "erstream",\
    "we-meet",\
    "newsdog",\
    "pugpig",\
    "digiturk-play",\
    "digicel-services",\
    "doordash",\
    "photo-wonder",\
    "quantcast",\
    "jiosaavn",\
    "opera-news",\
    "xhamster",\
    "shahid",\
    "mbc-services",\
    "qqmusic",\
    "seamless",\
    "cires21",\
    "splunk",\
    "coincap",\
    "shapeshift-services",\
    "al-jazeera-services",\
    "al-jazeera-english",\
    "do-global-services",\
    "webflow",\
    "payback",\
    "meitu-services",\
    "camscanner",\
    "intsig-services",\
    "jaumo",\
    "pubmatic",\
    "jio-services",\
    "groupon",\
    "fyeo",\
    "verizon-media-services",\
    "activision-blizzard",\
    "audiomack",\
    "kakao-map",\
    "osn",\
    "gsma",\
    "google-jibe",\
    "beautyplus",\
    "vivavideo",\
    "call-of-duty",\
    "micoworld-services",\
    "hago",\
    "daum-cafe",\
    "tencent-news",\
    "flipboard",\
    "viacom18-services",\
    "voot",\
    "selfiecity",\
    "meitu",\
    "kaspersky-services",\
    "hakuna",\
    "bermuda-video-chat",\
    "xiaomi-services",\
    "opera-news-lite",\
    "gaijin-entertainment",\
    "scopely-services",\
    "marvel-strike-force",\
    "sky-news",\
    "systex-services",\
    "perfect-corp-services",\
    "youcam-makeup",\
    "dtag-services",\
    "pubg",\
    "google-voice",\
    "the-score",\
    "pantaya",\
    "daum-mail",\
    "facebook-broadcasting",\
    "war-thunder",\
    "apus-services",\
    "ufoto-services",\
    "peak-games-services",\
    "toon-blast",\
    "expo-framework",\
    "the-new-york-times",\
    "du-recorder",\
    "luminati",\
    "meizu-services",\
    "joyy-services",\
    "mobvista",\
    "startimes-on",\
    "vodafone-services",\
    "the-guardian",\
    "guardian-media-services",\
    "mozilla-services",\
    "magnite",\
    "dc-universe",\
    "rakuten-services",\
    "playdemic-services",\
    "golf-clash",\
    "onetrust",\
    "lightstep",\
    "arc-publishing",\
    "vs-media",\
    "exaring-services",\
    "waipu-tv",\
    "google-news",\
    "ring-central",\
    "mortal-kombat",\
    "vpn-valley",\
    "bein-connect",\
    "boomplay-music",\
    "kongregate",\
    "ninja-kiwi-services",\
    "huawei-services",\
    "xmrig",\
    "lol-wild-rift",\
    "tinkoff",\
    "yandex-market",\
    "sweet-selfie",\
    "sweet-selfie-lite",\
    "digicert",\
    "switcherry-vpn",\
    "xender",\
    "ncsoft",\
    "ornet",\
    "coingecko",\
    "umg",\
    "malwarebytes",\
    "realnetworks",\
    "keycdn",\
    "riley-cillian",\
    "streamlive-to",\
    "wildlife-studios",\
    "block-craft-3d",\
    "lumen-tech",\
    "lovoo",\
    "clubhouse",\
    "sniper-3d",\
    "comodo-services",\
    "ribbon-communications",\
    "blackberry-work",\
    "joom",\
    "blackberry-services",\
    "google-pay",\
    "geekflare",\
    "war-machines",\
    "rawvoice",\
    "podtrac",\
    "wise",\
    "google-podcasts",\
    "shalltry-group",\
    "liveramp",\
    "euronews",\
    "leap-fitness-group",\
    "hexaglobe",\
    "rocket-league",\
    "magine-services",\
    "pay-rw",\
    "vuba-vuba",\
    "yandex-go",\
    "yandex-telemost",\
    "nexon-corporation",\
    "agoda",\
    "grab",\
    "nypr",\
    "prx-services",\
    "dovetail",\
    "huya",\
    "shareit",\
    "eversend",\
    "5channel",\
    "pet-rescue-saga",\
    "classdojo",\
    "photomath",\
    "mihoyo-games",\
    "genshin-impact",\
    "honkai-impact-3rd",\
    "khan-academy",\
    "homer",\
    "castbox",\
    "opentext-services",\
    "edx",\
    "socratic",\
    "telia-services",\
    "telia-tv",\
    "peacock-tv",\
    "jio-ads",\
    "discovery-plus",\
    "duolingo",\
    "viacom-cbs-services",\
    "bet-plus",\
    "paramount-plus",\
    "hubspot",\
    "philo",\
    "dna-tv",\
    "xiaomi-weather",\
    "jiochat",\
    "remind",\
    "elisa-services",\
    "elisa-viihde",\
    "pubnub",\
    "funimation",\
    "drift",\
    "img",\
    "endeavor-services",\
    "softbank-services",\
    "reface",\
    "mtv-katsomo",\
    "amazon-ads",\
    "prende-tv",\
    "univision-communications",\
    "anchorfm",\
    "veve",\
    "vrv",\
    "qualtrics",\
    "wangsu-cdn",\
    "algolia",\
    "maxivision",\
    "g-core-services",\
    "nordic-entertainment-group",\
    "viafree",\
    "netmera",\
    "gracenote",\
    "storyfm-cn",\
    "yle-areena",\
    "streamable",\
    "speakol",\
    "yallakora",\
    "linkonline",\
    "gemini-media-egypt",\
    "sanoma-services",\
    "ruutu",\
    "finnpanel",\
    "noon-vpn",\
    "masrawy",\
    "ktplay",\
    "team-fortress-2",\
    "trust-wallet",\
    "canal-plus-afrique",\
    "sarmady-services",\
    "filgoal",\
    "amc-theatres",\
    "youm7",\
    "valve-corporation",\
    "snail-vpn",\
    "xoom",\
    "ibm",\
    "midoki-services",\
    "vpn-master-x",\
    "qualaroo",\
    "mts-services",\
    "expedia",\
    "globalsign",\
    "letsencrypt",\
    "yieldmo",\
    "identrust",\
    "godaddy",\
    "google-trust-services",\
    "watch-it",\
    "amazon-trust-services",\
    "entrust",\
    "binance",\
    "elfsight",\
    "sailthru",\
    "certum",\
    "medianova-cdn",\
    "napster",\
    "sectigo",\
    "bazaarvoice",\
    "suning-services",\
    "indigitall",\
    "ada-support",\
    "penguin-sports",\
    "pusher",\
    "csg-media",\
    "grapeshot",\
    "knighthood",\
    "blossom-blast-saga",\
    "cybertrust-ca",\
    "usercentrics",\
    "idviu",\
    "iapps-technologies",\
    "calm",\
    "playstation-now",\
    "microsoft-azure",\
    "zego",\
    "dish-network",\
    "jquery",\
    "mox-tv",\
    "stats-perform",\
    "contentful",\
    "wetv",\
    "adex",\
    "arlo",\
    "jiohealthhub",\
    "gog",\
    "ring",\
    "fitbit",\
    "google-fit",\
    "diamond-diaries-saga",\
    "deluxe-media",\
    "the-trade-desk",\
    "jiocloud",\
    "trivago",\
    "minutely",\
    "garena-services",\
    "garena-free-fire",\
    "impact-services",\
    "samsung-max-privacy-vpn",\
    "private-tunnel-vpn",\
    "triple-lift",\
    "alphabetty-saga",\
    "mailchimp",\
    "a1-belarus",\
    "voka-tv",\
    "spb-tv",\
    "smart-media-solutions",\
    "smaato",\
    "teads",\
    "pyramid-solitaire-saga",\
    "riskified",\
    "papa-pear-saga",\
    "diamond-digger-saga",\
    "leanplum",\
    "tapad",\
    "squarespace",\
    "brightline",\
    "extreme-reach",\
    "rocketvpn",\
    "moovit",\
    "coinbox",\
    "sybo-games",\
    "subway-surfers",\
    "npr",\
    "eset-services",\
    "makeupplus",\
    "kakao-webtoon",\
    "zee5",\
    "juspay",\
    "pokemon-unite",\
    "zomato",\
    "plymedia",\
    "bitpay",\
    "phycan-vpn-services",\
    "internet-archive",\
    "cloudflare-warp-vpn",\
    "brd-wallet",\
    "exodus-wallet",\
    "beautycam",\
    "gab",\
    "measurement-lab",\
    "glovo",\
    "samsung-music",\
    "sweet-tv",\
    "uklon",\
    "toca-boca",\
    "tavr-media",\
    "lightsail-vpn",\
    "ibm-cloud-video",\
    "agora-services",\
    "sun-fm-ukraine",\
    "oll-tv",\
    "lanet-tv",\
    "tigo-services",\
    "uber-eats",\
    "bolt-technology",\
    "stripe",\
    "medallia",\
    "oneplusone-media-services",\
    "oneplusone-video",\
    "verisign",\
    "divantv",\
    "admixer",\
    "dingtalk",\
    "online-radio-box",\
    "moloco",\
    "livexlive",\
    "prostream-ru",\
    "blockchain-com",\
    "qualcomm",\
    "jiomart",\
    "jiomeet",\
    "hcl-notes-and-domino",\
    "mi-fit",\
    "paxful-wallet",\
    "schildbach-wallet",\
    "cake",\
    "tigerconnect",\
    "icloud-private-relay",\
    "ludo-king",\
    "twilio-services",\
    "ankama-games",\
    "meta-services",\
    "huawei-health",\
    "youdao",\
    "xinhua-news",\
    "raygun",\
    "google-chat",\
    "doubleverify",\
    "extole",\
    "starmaker",\
    "mx-player",\
    "origins-digital",\
    "sygic-services",\
    "budge-studios",\
    "mts-tv",\
    "smadex",\
    "amazon-games",\
    "new-world",\
    "aarki",\
    "xiaomi-shareme",\
    "goodbarber",\
    "litres-ru",\
    "qwilt",\
    "start-ru",\
    "pou",\
    "mumble",\
    "amex-services",\
    "wotc",\
    "mtg-arena",\
    "mozilla-vpn",\
    "dkb",\
    "deutsche-bank",\
    "nortonlifelock-services",\
    "nortonlifelock-vpn-services",\
    "zapya",\
    "amobee",\
    "ubitus-services",\
    "nintendo-switch-cloud-gaming",\
    "spreaker",\
    "jiocinema",\
    "epsilon-conversant",\
    "nvidia-services",\
    "wink",\
    "rostelecom",\
    "ngenix",\
    "intel-services",\
    "amd-services",\
    "pikmin-bloom",\
    "123vpn",\
    "playtika-services",\
    "avg-services",\
    "netflix-services",\
    "netflix-games",\
    "hero-wars",\
    "swifty-beaver",\
    "bitdefender-services",\
    "iheartmedia-services",\
    "sophos-services",\
    "mcafee-services",\
    "eurosport-news",\
    "ola",\
    "lg-electronics-services",\
    "ericsson-services",\
    "sharechat",\
    "valorant",\
    "indeed",\
    "nvidia-geforce-now",\
    "true-key",\
    "moon-active",\
    "safe-family",\
    "lotame",\
    "firefox-private-network",\
    "media-net",\
    "jsdelivr",\
    "ubisoft-services",\
    "ubisoft-connect",\
    "blitzer-de",\
    "samsung-pay",\
    "google-photos",\
    "stellar",\
    "trackmania",\
    "want-want-services",\
    "unruly",\
    "totalav",\
    "atudo",\
    "bid-switch",\
    "ing-banking",\
    "feedad",\
    "commerzbank-banking",\
    "vr-banking",\
    "anzu-vr",\
    "beeswax",\
    "dauntless",\
    "catchmedia",\
    "piano",\
    "shopify",\
    "pc-protect",\
    "cisco-systems-services",\
    "sparkasse-banking",\
    "holaspark",\
    "microsoft-bing",\
    "testmy-net",\
    "fuelio",\
    "speedof-me",\
    "xfinity",\
    "xfinity-speedtest",\
    "paycentive",\
    "rainbow-six-siege",\
    "jpmorgan-chase",\
    "multi-media-llc",\
    "capital-one",\
    "sygic-travel-maps",\
    "glassbox",\
    "stark-games",\
    "wells-fargo",\
    "blinkist",\
    "games-workshop",\
    "warhammer-gaming",\
    "redfast",\
    "samsung-health",\
    "speedsmart",\
    "sygic-gps-navigation",\
    "comcast-tech-solutions",\
    "viu-tv",\
    "demandbase",\
    "rainbow-six-extraction",\
    "warhammer-tv",\
    "disney-hotstar",\
    "wunderkind",\
    "might-and-magic-chess-royale",\
    "pulse-insights",\
    "radarbot",\
    "adswizz",\
    "bright-data",\
    "wakanim",\
    "etsy",\
    "vanilla",\
    "citadel-colour",\
    "behaviour-services",\
    "dbd",\
    "black-library",\
    "digital-turbine",\
    "duckduckgo",\
    "liveone",\
    "josh",\
    "babe",\
    "wikimedia-foundation",\
    "avast-vpn-services",\
    "open-web",\
    "streamio",\
    "tmt-broadcasting",\
    "uno",\
    "verve-group",\
    "mcafee-security",\
    "lokalise",\
    "brainly",\
    "zoner-antivirus",\
    "zoner-services",\
    "magenta-tv",\
    "lark",\
    "capcut",\
    "podbean",\
    "permutive",\
    "wistia",\
    "rappi",\
    "kion",\
    "okko",\
    "picsart",\
    "mx-takatak",\
    "sender",\
    "f-secure",\
    "aarogya-setu",\
    "aniview",\
    "pinduoduo",\
    "gearbox-entertainment-services",\
    "gearbox-shift",\
    "age-of-empires",\
    "relic-entertainment",\
    "xbox-game-studios",\
    "split",\
    "truth-social",\
    "rumble",\
    "fandom",\
    "uscreen",\
    "hyperconnect",\
    "font-awesome",\
    "ruby-game-studio",\
    "moengage",\
    "lost-ark",\
    "miro",\
    "google-tv",\
    "cadent",\
    "pangle",\
    "embrace",\
    "streamguys",\
    "holler",\
    "easybrain",\
    "easybrain-music-maker",\
    "creative-assembly",\
    "sega",\
    "match-group",\
    "total-war",\
    "feral-interactive",\
    "liveboard",\
    "naraka-bladepoint",\
    "netease-games",\
    "patreon",\
    "usps",\
    "iab-europe",\
    "ups",\
    "q4",\
    "tidio",\
    "warner-bros-discovery-services",\
    "lazada",\
    "onlyfans",\
    "primis",\
    "blue-billywig",\
    "geolocation-technology",\
    "lumen-technologies",\
    "fruitshake",\
    "improve-digital",\
    "terminus",\
    "tableau",\
    "tiny-tinas-wonderlands",\
    "ag-projects-services",\
    "display-io",\
    "beetalk",\
    "zillow",\
    "plurk",\
    "inmobi",\
    "arc-xp",\
    "embracer-group",\
    "spin-the-bottle",\
    "xandr",\
    "mapp",\
    "9gag",\
    "square-enix",\
    "final-fantasy-xiv",\
    "naver-works",\
    "naver-corporation",\
    "intuit",\
    "gazprom-media",\
    "tnt-premier",\
    "radancy",\
    "rambler",\
    "crypto-com",\
    "fifa-soccer",\
    "kustomer",\
    "logrocket",\
    "spothero",\
    "adobe-experience-cloud",\
    "icf-technology",\
    "tricolor-tv",\
    "redfin",\
    "matterport",\
    "naver-cloud",\
    "rogue-company",\
    "wow-tv",\
    "quora",\
    "infobip",\
    "edgio",\
    "flipkart",\
    "near",\
    "avo",\
    "contentsquare",\
    "fall-guys",\
    "yahoo-ad-tech",\
    "meesho",\
    "tower-of-fantasy",\
    "nperf",\
    "nbc-universal",\
    "naver-map",\
    "snapdeal",\
    "smartclip",\
    "digitalocean",\
    "happn",\
    "2k",\
    "microsoft-family-safety",\
    "rumbleverse",\
    "wb-games",\
    "microsoft-defender",\
    "adobe-fonts",\
    "vimond",\
    "aioncloud",\
    "amc-networks",\
    "amc-plus",\
    "cabify",\
    "paytm",\
    "lenovo-services",\
    "uplive",\
    "naver-mail",\
    "akamai-technologies",\
    "stackadapt",\
    "hi-rez-studios",\
    "vix",\
    "tremor-video",\
    "block-inc",\
    "cash-app",\
    "outfit7",\
    "mythic-legends",\
    "ui-ag-services",\
    "gmx-mail",\
    "inshorts",\
    "revolut",\
    "zepeto",\
    "diablo-immortal",\
    "life360",\
    "session",\
    "premier",\
    "tencent-cloud",\
    "paypay",\
    "6sense",\
    "vimeo-ott",\
    "zeus-network",\
    "vidazoo",\
    "hidive",\
    "grupo-globo-services",\
    "globoplay",\
    "shudder",\
    "kingsoft-services",\
    "kingsoft-cloud",\
    "kobo",\
    "for-honor",\
    "tremor-international",\
    "financial-times",\
    "kape-services",\
    "square",\
    "hcaptcha",\
    "amc-svod",\
    "sundance-now",\
    "naver-now",\
    "australia-post",\
    "bungie-games",\
    "ora-tv",\
    "ccp-games",\
    "eve-online",\
    "zype",\
    "umeng-plus",\
    "byjus",\
    "tilda",\
    "cdn77",\
    "tapclap",\
    "pirate-treasures",\
    "prismic",\
    "tencent-music",\
    "httpool",\
    "cachefly",\
    "alibaba-group-services",\
    "qanda",\
    "vercel",\
    "streamotion",\
    "draftkings-services",\
    "draftkings-casino",\
    "draftkings-sportsbook",\
    "wix",\
    "fanduel-services",\
    "fanduel-casino",\
    "fanduel-sportsbook",\
    "cryptic-studios",\
    "neverwinter",\
    "resso-music",\
    "xumo",\
    "realm-royale",\
    "vikingard",\
    "badlanders",\
    "infinite-lagrange",\
    "viewlift",\
    "tvnz",\
    "wynk-music",\
    "gearbox-publishing",\
    "viju",\
    "viasat-ru",\
    "lilithgames",\
    "rise-of-kingdoms",\
    "deca-games",\
    "gods-and-glory",\
    "podcast-addict",\
    "eve-echoes",\
    "lazy-audio",\
    "amediateka",\
    "mixi",\
    "mixi-services",\
    "lotr-war",\
    "google-translate",\
    "ntt-services",\
    "radiko",\
    "cyberagent-services",\
    "ameba",\
    "redhotpie",\
    "clicktouch-games",\
    "adtech-studio",\
    "medialab-ai-services",\
    "mitene",\
    "transperth",\
    "hinge",\
    "superawesome",\
    "yandex-translate",\
    "cashi",\
    "booking-holdings-services",\
    "banco-azteca",\
    "gstatic",\
    "didi-food",\
    "didi-rider",\
    "facebook-messenger",\
    "coppel",\
    "unclassified",\
    "okcupid",\
    "mercado-libre",\
    "band",\
    "mercado-pago",\
    "smilegate",\
    "bancoppel",\
    "v4",\
    "vlmedia-services",\
    "microsoft-translator",\
    "cdnjs",\
    "telemetro",\
    "corporacion-medcom",\
    "here-tv",\
    "overwatch",\
    "snow-services",\
    "mastodon",\
    "glu-mobile-services",\
    "kartrider-rush",\
    "groupm",\
    "ultraviewer",\
    "kingdom-age",\
    "star-trek-online",\
    "web-de",\
    "mail-com",\
    "arte",\
    "vita",\
    "paramount-services",\
    "blutv",\
    "xiaohongshu",\
    "rakuten-tv",\
    "apple-weather",\
    "mubi",\
    "quantumcast",\
    "knights-and-dragons",\
    "smite",\
    "teleport-media",\
    "cbs-news",\
    "cbs-sports",\
    "rbc-news",\
    "canonical-services",\
    "exxen",\
    "sony-group-services",\
    "soul-hunters",\
    "adtelligent",\
    "datadog",\
    "segmento-ru",\
    "polyfill-io",\
    "b612",\
    "warpath-ace-shooter",\
    "tower-of-saviors",\
    "lemon8",\
    "coveo",\
    "hypic",\
    "escape-from-tarkov",\
    "pocket",\
    "marvel-snap",\
    "adobe-document-cloud",\
    "zangi-messenger",\
    "beamable",\
    "bunny-net",\
    "bce-services",\
    "crave",\
    "byteplus",\
    "plarium",\
    "sparta-war-of-empires",\
    "sber",\
    "jivochat",\
    "youtube-kids",\
    "oppo-mobile-services",\
    "heytap",\
    "max",\
    "world-of-warships",\
    "youtube-tv",\
    "google-family-link",\
    "celtic-heroes",\
    "hero-hunters",\
    "times-internet-services",\
    "servicenow",\
    "honkai-star-rail",\
    "elder-scrolls-online",\
    "nuverse",\
    "tvplus",\
    "nasdaq-services",\
    "sesisoft-services",\
    "tivibu-go",\
    "dungeon-fighter-online",\
    "5gmark",\
    "kill-shot-bravo",\
    "puhutv",\
    "dogus-services",\
    "openai",\
    "qobuz",\
    "s-sport-plus",\
    "world-of-tanks-blitz"

/**
 * List of strings, each identifying the appropriate application in a more human readable way.
 *
 * Items may be changed between releases.
 */
#define PACE2_APPLICATIONS_LONG_STRS \
    "unknown",\
    "Facebook",\
    "magicJack",\
    "iTunes",\
    "MySpace",\
    "FaceTime",\
    "Truphone",\
    "Twitter",\
    "XBOX gaming console",\
    "RealMedia",\
    "Google Mail",\
    "Goober",\
    "iCloud",\
    "Ubuntu One",\
    "Dropbox",\
    "Yuilop",\
    "iMessage",\
    "GoToMyPC",\
    "GoToMeeting",\
    "Windows Azure",\
    "Amazon Cloud",\
    "Dailymotion",\
    "Deezer",\
    "Grooveshark",\
    "SUDA Phone",\
    "Office 365",\
    "CNTV",\
    "SINATV",\
    "YouTube",\
    "VoipSwitch",\
    "Zynga",\
    "Crime City",\
    "Modern War",\
    "Vippie",\
    "Box",\
    "Skydrive",\
    "Adobe Creative Cloud",\
    "LinkedIn",\
    "Orkut",\
    "hi5",\
    "Soribada",\
    "Zoho Work Online",\
    "Hotmail",\
    "Instagram",\
    "SugarSync",\
    "Wickr",\
    "TigerText",\
    "Vibe",\
    "enLegion",\
    "Voxer",\
    "LOVEFiLM",\
    "Friendster",\
    "Burner",\
    "Flickr",\
    "live-com",\
    "ICQ Webchat",\
    "RedTube",\
    "Evernote",\
    "Spreed",\
    "textPlus",\
    "Xvideos",\
    "Snapchat",\
    "Youku",\
    "MyMusic",\
    "Renren",\
    "Yelp",\
    "Amazon shopping",\
    "YouPorn",\
    "eBay",\
    "PornHub",\
    "Spideroak",\
    "Adrive",\
    "Tumblr",\
    "XING",\
    "Google Play Music",\
    "Vimeo",\
    "Salesforce",\
    "iTunes Radio",\
    "Sina Weibo",\
    "Forfone",\
    "LiveJasmin",\
    "Jingdong",\
    "Last-fm",\
    "IP Multimedia Subsystem",\
    "Baidu",\
    "Taobao",\
    "Watchever",\
    "TTPod",\
    "360 Mobile Security",\
    "Pinterest",\
    "Vine",\
    "Net2Phone",\
    "Foursquare",\
    "iLive-to",\
    "MEGA",\
    "Doook",\
    "vBulletin",\
    "CloudMe",\
    "Yik Yak",\
    "Redmine",\
    "iHeartRadio",\
    "Avira Antivirus",\
    "Norton Antivirus",\
    "Kaspersky Antivirus",\
    "AVG Antivirus",\
    "SoundCloud",\
    "Rdio",\
    "Amazon Prime Music",\
    "RunKeeper",\
    "myBook",\
    "Nike Plus",\
    "MagV",\
    "MagV Kids",\
    "Dota Legend",\
    "LeTV",\
    "MyFitnessPal",\
    "Sohu",\
    "1kxun",\
    "TVBS",\
    "XNXX",\
    "CtiTV",\
    "Endomondo",\
    "Timely TV",\
    "Workout Trainer",\
    "MapMyRun",\
    "King of Pirates",\
    "Tudou",\
    "BitTorrent",\
    "eDonkey",\
    "KaZaa/Fasttrack",\
    "Gnutella",\
    "DirectConnect",\
    "AppleJuice",\
    "Soulseek",\
    "Ares",\
    "Freenet",\
    "Mute",\
    "Skype",\
    "MSN",\
    "Yahoo",\
    "vonage",\
    "Oscar",\
    "Gadu-Gadu",\
    "MPlus",\
    "IRC",\
    "Slingbox",\
    "Vcast",\
    "4shared-com",\
    "adrive-com",\
    "badongo-com",\
    "badongo-net",\
    "bestsharing-com",\
    "bigfilez-com",\
    "chinamofile-com",\
    "data-hu",\
    "depositfiles-com",\
    "easy-share-com",\
    "fast-load-net",\
    "fast-share-com",\
    "file-upload-net",\
    "file-upload-to",\
    "filearchiv-ru",\
    "filebase-to",\
    "filecloud-com",\
    "filefactory-com",\
    "filefront-com",\
    "fileho-com",\
    "filehost-tv",\
    "filepost-ru",\
    "filer-net",\
    "files-upload-com",\
    "files-to",\
    "filesafe-to",\
    "filesend-net",\
    "fileupyours-com",\
    "ftp2share-com",\
    "gigasize-com",\
    "gigeshare-com",\
    "hostggg-com",\
    "hotfile-com",\
    "ifolder-ru",\
    "keepmyfile-com",\
    "leteckaposta-cz",\
    "live-share-com",\
    "livedepot-net",\
    "load-to",\
    "massmirror-com",\
    "mediafire-com",\
    "megarotic-com",\
    "megashare-com",\
    "megashares-com",\
    "megaupload-com",\
    "mofile-com",\
    "mofile-net",\
    "mytempdir-com",\
    "netload-in",\
    "odsiebie-najlepsze-net",\
    "quicksharing-com",\
    "rapidshare-com",\
    "rapidshare-de",\
    "rapidupload-com",\
    "savefile-com",\
    "sendmefile-com",\
    "sendspace-com",\
    "sharebase-to",\
    "sharebee-com",\
    "sharebig-com",\
    "sharebigfile-com",\
    "sharebigflie-com",\
    "simpleupload-net",\
    "speedshare-org",\
    "turboupload-com",\
    "ultrashare-de",\
    "up-file-com",\
    "uploaded-to",\
    "uploading-com",\
    "uploadingit-com",\
    "uploadpower-com",\
    "uploadyourfiles-de",\
    "vip-file-com",\
    "wiiupload-net",\
    "yourfilehost-com",\
    "yourfiles-biz",\
    "youtube-com",\
    "zshare-net",\
    "4shared-vn",\
    "mega-1280-com",\
    "megashare-vn",\
    "fileserve-com",\
    "filesonic-com",\
    "1-upload-to",\
    "1fichier-com",\
    "4fastfile-com",\
    "10upload-com",\
    "1clickshare-net",\
    "filedude-com",\
    "asixfiles-com",\
    "banashare-com",\
    "biggerupload-com",\
    "bitshare-com",\
    "cash-file-net",\
    "cramit-in",\
    "czshare-com",\
    "data-loading-com",\
    "dataport-cz",\
    "datei-to",\
    "downupload-com",\
    "easybytez-com",\
    "edisk-cz",\
    "enterupload-com",\
    "euroshare-eu",\
    "extabit-com",\
    "fastfileshare-com-ar",\
    "fik1-com",\
    "file2upload-net",\
    "fileape-com",\
    "fileflyer-com",\
    "filefrog-com",\
    "filehook-com",\
    "fileover-net",\
    "filepost-com",\
    "files-mail-ru",\
    "fileshare-in-ua",\
    "filesmonster-com",\
    "filevo-com",\
    "filezup-com",\
    "freakshare-net",\
    "fshare-vn",\
    "gaiafile-com",\
    "gigapeta-com",\
    "gotoupload-com",\
    "hellshare-com",\
    "hitfile-net",\
    "hulkshare-com",\
    "hyperfileshare-com",\
    "i-filez-com",\
    "ifile-it",\
    "kewlshare-com",\
    "letitbit-net",\
    "loadfiles-in",\
    "megafree-kz",\
    "megarapid-eu",\
    "megashare-vnn-vn",\
    "midupload-com",\
    "multishare-cz",\
    "netuploaded-com",\
    "oron-com",\
    "przeklej-pl",\
    "putshare-com",\
    "qshare-com",\
    "quickshare-cz",\
    "rapidshark-pl",\
    "scribd-com",\
    "share-online-bz",\
    "share-rapid-cz",\
    "shragle-com",\
    "storage-to",\
    "terafiles-net",\
    "tunabox-net",\
    "turbobit-net",\
    "ugotfile-com",\
    "uloz-to",\
    "unibytes-com",\
    "unlimit-co-il",\
    "up-4share-vn",\
    "upload-com-ua",\
    "uploadbox-com",\
    "uploader-pl",\
    "uploadfloor-com",\
    "uploadstation-com",\
    "upnito-sk",\
    "usershare-net",\
    "videobb-com",\
    "wupload-com",\
    "x7-to",\
    "yunfile-to",\
    "zomgupload-to",\
    "ziddu-com",\
    "1-upload-com",\
    "1-clickshare-com",\
    "123upload-pl",\
    "139pan-com",\
    "163pan-com",\
    "2shared-com",\
    "4share-ws",\
    "4us-to",\
    "alldrives-ge",\
    "allshares-ge",\
    "annonhost-net",\
    "boosterking-com",\
    "cobrashare-sk",\
    "coolshare-cz",\
    "crocko-com",\
    "cum-com",\
    "eyvx-com",\
    "filearn-com",\
    "filejungle-com",\
    "filemaze-ws",\
    "filesflash-com",\
    "freespace-by",\
    "fsx-hu",\
    "jakfile-com",\
    "jumbofiles-com",\
    "kickload-com",\
    "missupload-com",\
    "movreel-com",\
    "platinshare-com",\
    "sanupload-com",\
    "shareflare-net",\
    "u-115-com",\
    "uploadc-com",\
    "uploadking-com",\
    "uptobox-com",\
    "filestube-com",\
    "gamefront-com",\
    "megaporn-com",\
    "filesonic-in",\
    "turbobit-ru",\
    "Usenet",\
    "FTP",\
    "Filetopia",\
    "Manolito",\
    "iMesh",\
    "Pando",\
    "SopCast",\
    "TVUPlayer",\
    "PPStream",\
    "PPLive",\
    "Zattoo",\
    "QQ",\
    "QQLive",\
    "Feidian",\
    "PoPo",\
    "Thunder/Webthunder",\
    "UUSee",\
    "VNC",\
    "Steam",\
    "Half-Life 2",\
    "SoftEthernet",\
    "Telnet",\
    "OFF",\
    "TOR",\
    "World of Warcraft",\
    "HamachiVPN",\
    "Skinny",\
    "Citrix",\
    "Paltalk",\
    "Quake",\
    "Second Life",\
    "Battlefield",\
    "PCAnywhere",\
    "Icecast",\
    "SHOUTcast",\
    "QQGame",\
    "VeohTV",\
    "OpenFT",\
    "Pandora",\
    "IPP",\
    "Warcraft 3",\
    "ooVoo",\
    "Google Hangouts",\
    "Iskoot",\
    "Fring",\
    "IPTV",\
    "YourFreedom Tunnel",\
    "VoipSwitch VoIP Tunnel",\
    "Meebo",\
    "Funshion",\
    "Aimini",\
    "ClubPenguin",\
    "Florensia",\
    "TeamSpeak",\
    "MapleStory",\
    "Octoshape",\
    "PS3",\
    "Dofus",\
    "Fiesta",\
    "SplashFighter",\
    "Activesync",\
    "Nimbuzz",\
    "CrossFire",\
    "WAP-WSP",\
    "Guildwars",\
    "Armagetron",\
    "rFactor",\
    "TeamViewer",\
    "GameKit",\
    "UltraBac",\
    "Spotify",\
    "IMplus",\
    "ANtsP2P",\
    "UltraSurf",\
    "Viber",\
    "Tango",\
    "iPlayer",\
    "Opera Mini",\
    "Bolt",\
    "Scydo",\
    "WhatsApp",\
    "Webex",\
    "NetFlix",\
    "Mojo",\
    "IMO",\
    "CitrixGoTo",\
    "FiCall",\
    "ComodoUnite",\
    "Ventrilo",\
    "MyPeople",\
    "eBuddy",\
    "AdobeConnect",\
    "JAP",\
    "Wuala",\
    "WebDAV",\
    "Lync",\
    "Audiogalaxy",\
    "Mig33",\
    "HTTPTunnel",\
    "Rhapsody",\
    "WebQQ",\
    "PDProxy",\
    "CyberGhost",\
    "Google Cloud",\
    "Google Drive",\
    "Google Docs",\
    "Google Plus",\
    "Oracle Database",\
    "ITV",\
    "Demand 5",\
    "RSVP",\
    "Channel 4oD",\
    "Lotus Notes",\
    "Poison Ivy",\
    "HiDrive",\
    "WeChat",\
    "MyVideo",\
    "Kik Messenger",\
    "Line",\
    "Iperf",\
    "OS update",\
    "Hike Messenger",\
    "DiDi",\
    "IDrive",\
    "Spreecast",\
    "Simfy",\
    "Dota 2",\
    "QVOD",\
    "Hulu",\
    "Microsoft Exchange",\
    "Microsoft Dynamics NAV",\
    "Fuze Meeting",\
    "Kakaotalk",\
    "Destiny",\
    "VyprVPN",\
    "Threema",\
    "NTLM",\
    "Twitch",\
    "Wowza",\
    "Runtastic",\
    "Telegram",\
    "StreetVoice",\
    "Duokan",\
    "Mei Lu",\
    "Skout",\
    "PlayTales",\
    "Game of War",\
    "FETnet",\
    "Fubon E-Broker",\
    "National Baseball",\
    "PinkFong",\
    "Kollect Book",\
    "SPTG TV",\
    "Ciaomobile HDTV",\
    "Skyking",\
    "Moves",\
    "Path",\
    "Age of Warring Empire",\
    "iWow systex",\
    "9ku",\
    "Next TV",\
    "Syncplicity",\
    "RealMa",\
    "Yuanta Web",\
    "HiNet",\
    "MyCard",\
    "Kono",\
    "Speakaboos",\
    "Duo Duo",\
    "Wealth God",\
    "iLeader",\
    "Three Bamboo",\
    "Taaze",\
    "Audiko",\
    "pubu bookbuffet",\
    "Mitake",\
    "Hyread",\
    "Disney Storytime",\
    "Qualys",\
    "Constant Contact",\
    "Rally",\
    "Magine TV",\
    "IntraLinks",\
    "LivePerson",\
    "Speedtest",\
    "BlueJeans",\
    "Apple Music",\
    "Naver",\
    "Booking-com",\
    "CNN",\
    "Periscope",\
    "PayEasy",\
    "Line Rangers",\
    "iOS App Store",\
    "MyVideo TW",\
    "King Gaming",\
    "Candy Crush",\
    "Candy Crush Soda",\
    "Bubble Witch",\
    "Farm Hero",\
    "PlayStation Network",\
    "Google Play",\
    "Supercell",\
    "Clash of Clans",\
    "Boom Beach",\
    "Hay Day",\
    "Windows Store",\
    "Skype for Business",\
    "Minecraft",\
    "Meerkat",\
    "Rovio Gaming",\
    "Angry Birds",\
    "Citrix XenApp",\
    "reddit",\
    "League of Legends",\
    "BattleNet",\
    "PayPal",\
    "Plants vs Zombies 2",\
    "Halfbrick Studios",\
    "GitHub",\
    "HamiVideo",\
    "Slacker",\
    "Niconico",\
    "AliPay",\
    "Ustream",\
    "Zalo",\
    "NTT Docomo",\
    "Softbank Onlineshop",\
    "Guild Wars 2",\
    "Trend Micro Worry-Free",\
    "Wikipedia",\
    "Uber",\
    "Yandex",\
    "VK",\
    "Odnoklassniki",\
    "CrashPlan",\
    "Alibaba",\
    "AliExpress",\
    "TMall",\
    "EA Origin",\
    "Norton Security",\
    "Tibia",\
    "Pochta Rossii",\
    "After School",\
    "DriveLock",\
    "QIP-RU",\
    "musical_ly",\
    "EA Games",\
    "iTunesU",\
    "PopCap",\
    "SurveyMonkey",\
    "New Relic",\
    "Huddle",\
    "ZenDesk",\
    "OutlookMobile",\
    "ChatCube",\
    "DeNA",\
    "BackBlaze",\
    "DocuSign",\
    "Betternet",\
    "Good",\
    "Anghami",\
    "Path of Exile",\
    "Slack",\
    "Pokemon Go",\
    "Niantic Labs",\
    "2Gis",\
    "Rutube",\
    "2channel",\
    "Prisma",\
    "Clash Royal",\
    "Upptalk",\
    "Amazon Prime Video",\
    "Clash Royale",\
    "Google Shared Services",\
    "Signal",\
    "KanKan",\
    "share-online",\
    "Ultrashare",\
    "Yunfile",\
    "eDisk",\
    "123upload",\
    "Azar",\
    "Sky Go",\
    "iO",\
    "Mail-Ru",\
    "iCrypt",\
    "AIM",\
    "Kinopoisk",\
    "Ivi-Ru",\
    "Tinder",\
    "AnyDesk",\
    "ICQ",\
    "Tenor",\
    "Google Maps",\
    "Google Earth",\
    "Channel One",\
    "Giphy",\
    "VPN-X",\
    "Imgur",\
    "OneDrive",\
    "Nintendo Network",\
    "Amazon Chime",\
    "iFlix",\
    "PPFilm",\
    "Apple Services",\
    "Microsoft Services",\
    "IcFlix",\
    "LogMeIn",\
    "Psiphon",\
    "Waze",\
    "Google FCM",\
    "APNS",\
    "Digicel PlayGo",\
    "Shazam",\
    "Digicel Music",\
    "Digicel TopUp",\
    "TuneIn Radio",\
    "Dubsmash",\
    "Synology",\
    "My Digicel",\
    "Google Allo",\
    "rusfolder",\
    "Badoo",\
    "AirBnB",\
    "Hightail",\
    "Easy Taxi",\
    "TomTom",\
    "Tripadvisor",\
    "ESPN",\
    "DAZN",\
    "Mobile Strike",\
    "King of Avalon",\
    "Lineage 2",\
    "Lyft",\
    "Trulia",\
    "Akamai Cloud",\
    "Limelight Cloud",\
    "Nickelodeon Play",\
    "Here",\
    "SimCity BuildIt",\
    "StarWars Galaxy of Heroes",\
    "Fastly",\
    "SAP",\
    "World of Kung Fu",\
    "Cloudflare",\
    "Life Church",\
    "Vevo",\
    "Grindr",\
    "Apple Maps",\
    "Apple Geolocation",\
    "uploaded",\
    "easypaste-org",\
    "Google Duo",\
    "Youku Tudou",\
    "Meetic Group",\
    "Sarahah",\
    "Vudu",\
    "Wish",\
    "Auth0",\
    "Sling TV",\
    "Google APIs",\
    "Rackspace Cloud",\
    "Google Calendar",\
    "Google Analytics",\
    "StackPath Cloud",\
    "Houseparty",\
    "LearningHub-online",\
    "Google Ads",\
    "Fox Sports",\
    "Amazon Services",\
    "Amazon Drive",\
    "Crashlytics",\
    "Facebook Cloud",\
    "360 Total Security",\
    "Turner",\
    "Loop Trade Classifieds",\
    "Loop Caribbean Local News",\
    "Loop Pacific Local News",\
    "Unity",\
    "FunPlus",\
    "Mapbox",\
    "Cedexis",\
    "Zenmate VPN",\
    "Smule",\
    "Private Internet Access",\
    "Symantec",\
    "Maxdome",\
    "IPVanish",\
    "TunnelBear",\
    "Google App Engine",\
    "Wire Messenger",\
    "Epic Browser Proxy",\
    "Zero VPN",\
    "Surfeasy",\
    "Tiger VPN",\
    "Epic Games",\
    "Avira Phantom VPN",\
    "Turbo VPN",\
    "Iqiyi",\
    "VPN Master",\
    "PPTV",\
    "Snap VPN",\
    "TokBox",\
    "VPN Unlimited",\
    "HMA VPN",\
    "Avast Services",\
    "Nord VPN",\
    "Proton VPN",\
    "SaferVPN",\
    "HideMe VPN",\
    "ExpressVPN",\
    "TunnelGuru",\
    "Youtube Music",\
    "Windscribe VPN",\
    "IVPN",\
    "Discord",\
    "FireVPN",\
    "BOTIM",\
    "Hi VPN",\
    "VPN Proxy Master",\
    "ZOOGVPN",\
    "Ivacy VPN",\
    "PureVPN",\
    "Mullvad VPN",\
    "Freedome VPN",\
    "Easy VPN",\
    "StrongVPN",\
    "Buffered VPN",\
    "Wargaming-net",\
    "Yandex Music",\
    "VPN Area",\
    "Steganos Online Shield VPN",\
    "SuperVPN",\
    "Steganos Services",\
    "FrootVPN",\
    "BlackBerry Messenger",\
    "Speed VPN Inventive",\
    "CME",\
    "Yandex Navigator",\
    "Yandex Disk",\
    "Yandex Mail",\
    "TikTok",\
    "Yandex Maps",\
    "Touch VPN",\
    "Gaana",\
    "xCum-com",\
    "Life-Ru",\
    "VPN in Touch",\
    "Speed VPN MobileAppDev",\
    "Dash VPN",\
    "YeeCall",\
    "Shellfire VPN",\
    "IbVPN",\
    "Ren-Tv",\
    "Hola VPN",\
    "Hola Services",\
    "CDNvideo",\
    "Atom VPN",\
    "ArkVPN",\
    "Fortnite",\
    "Amaze VPN",\
    "Vungle",\
    "Vivox",\
    "Megogo",\
    "Finch VPN",\
    "Fast Secure VPN",\
    "TVPlayer",\
    "World Of Tanks",\
    "VPN Easy",\
    "ZPN VPN",\
    "AppVPN",\
    "X-VPN",\
    "DotVPN",\
    "Encrypt-me",\
    "LinkVPN",\
    "Droid VPN",\
    "Browsec VPN",\
    "Enmasse Games",\
    "Hotspot VPN",\
    "Apex Legends",\
    "VPN 365",\
    "CounterStrikeGO",\
    "VPN Tor Browser",\
    "SuperVPN by Cheng",\
    "Act VPN",\
    "Brightcove",\
    "Sportbox-Ru",\
    "Just Proxy VPN",\
    "VeePN",\
    "Ookla",\
    "Eagle VPN",\
    "Bria Mobile",\
    "VPN Gate",\
    "AnchorFree Services",\
    "Bria Teams",\
    "Shadowsocks",\
    "Asphalt9",\
    "Gameloft-Services",\
    "Actmobile Services",\
    "Novatech Core",\
    "SecureVPN",\
    "AppsFlyer",\
    "Adjust",\
    "GenieVPN",\
    "TunVPN",\
    "Helpshift",\
    "FastVPN Master",\
    "VPN Unlimited Proxy",\
    "HighSpeedVPN",\
    "VPN Master by Andro Team",\
    "Mixer",\
    "Tencent Games",\
    "Open Super Hotspot VPN",\
    "VPN Proxy Master Unlimited",\
    "Douyu",\
    "Ingress Prime",\
    "Electronic Arts",\
    "Radioplayer",\
    "MoPub",\
    "PUBG Mobile",\
    "7TV",\
    "Spiegel TV",\
    "Spiegel Services",\
    "Eurosport Player",\
    "Forge of Empires",\
    "InnoGames",\
    "Receiver Radio",\
    "Goodgame Studios",\
    "Wizards Unite",\
    "Radio-De",\
    "BBC News",\
    "BBC Services",\
    "Unreal Engine",\
    "Zoom",\
    "Adobe Services",\
    "ATT Services",\
    "Funk",\
    "Nexx TV",\
    "Unblock Sites",\
    "Brawl Stars",\
    "Counterpath Services",\
    "Snap Master VPN",\
    "Joyn",\
    "Pro7Sat1 Services",\
    "Dead Rivals",\
    "Tapjoy",\
    "IGG",\
    "Lords Mobile",\
    "Mudfish",\
    "Inventive VPN Services",\
    "Mixpanel",\
    "Adcolony",\
    "SOMA",\
    "Big Farm",\
    "FaceApp",\
    "Asphalt8",\
    "Moat",\
    "AdSafeProtected",\
    "AdForm",\
    "FreeWheel",\
    "SpotX",\
    "IronSource",\
    "TrafficJunky",\
    "TrafficFactory",\
    "Hatch Cloud Gaming",\
    "AppLovin",\
    "DeltaDNA",\
    "AdTiming",\
    "Flurry",\
    "Optimizely",\
    "SmartAdServer",\
    "Exactag",\
    "ScorecardResearch",\
    "Conviva",\
    "BAMTech Media",\
    "Bebi",\
    "HBO Services",\
    "Heyzap",\
    "Criteo",\
    "Swrve",\
    "Chartboost",\
    "Accountkit",\
    "DirecTV",\
    "VPNSecure",\
    "Liftoff",\
    "Level3 Cloud",\
    "Disney Services",\
    "My5",\
    "Source Engine",\
    "Geocaching",\
    "Fox News",\
    "Kaltura",\
    "Verizon Digital Media Services",\
    "Streamcraft",\
    "Fox Services",\
    "Branch",\
    "Viki",\
    "Wordpress",\
    "Gravatar",\
    "Parsely",\
    "WeGamers",\
    "PBS",\
    "Crackle",\
    "OpenX",\
    "Rubicon Project",\
    "Disney Radio",\
    "Kochava",\
    "American Broadcasting Company",\
    "StreamTheWorld",\
    "Apteligent",\
    "Apple News",\
    "HBO Now",\
    "Segment",\
    "TUNE",\
    "Apple TV",\
    "Apptentive",\
    "Airable",\
    "Pluto TV",\
    "Disney Channel",\
    "Discovery Communications",\
    "Sony Pictures",\
    "NFL Sunday Ticket",\
    "Pornhub Network",\
    "PdaNet",\
    "MLB",\
    "MLB At Bat",\
    "Tubi",\
    "Watch TBS",\
    "Disney Plus",\
    "OpenSignal",\
    "Showtime",\
    "Quantum Metric",\
    "mParticle",\
    "Showtime Anytime",\
    "Canal Plus",\
    "Audible",\
    "Tealium",\
    "Nuance",\
    "CBS Services",\
    "NBC Sports",\
    "Eurosport Services",\
    "Endeavor Streaming",\
    "ToTok",\
    "Fyber",\
    "Univision Now",\
    "Taboola",\
    "adRise",\
    "Sky Services",\
    "UserVoice",\
    "BIGO Services",\
    "BitTorrent Now",\
    "Mario Kart Tour",\
    "Turkcell Services",\
    "Sky Sports",\
    "BiP Messenger",\
    "Swoogo",\
    "NBC Services",\
    "Etisalat Services",\
    "Switch TV",\
    "Microsoft Teams",\
    "Flashtalking",\
    "Youtube Go",\
    "Symantec VPN Services",\
    "Nielsen Services",\
    "Fox Business",\
    "PrivateVPN",\
    "Surfshark VPN",\
    "Yospace",\
    "Sapling Growth VPN Services",\
    "NFL Services",\
    "Xunlei Services",\
    "The NBC App",\
    "Fizy by Turkcell",\
    "Cloudinary",\
    "Tencent Services",\
    "UltraVPN",\
    "Pladform",\
    "Starz",\
    "WeChat Work",\
    "Resilio Sync",\
    "ImgIX",\
    "Amazon Prime Now",\
    "Kakao Services",\
    "Solvvy",\
    "JW Player",\
    "Stackla",\
    "Mux",\
    "Freshdesk",\
    "Freshworks",\
    "Amplitude",\
    "AppNexus",\
    "Under Armour Services",\
    "MapMyFitness",\
    "TrustArc",\
    "Microsoft Excel",\
    "Microsoft Word",\
    "Microsoft PowerpPoint",\
    "Microsoft Outlook",\
    "Snap Services",\
    "BootstrapCDN",\
    "KakaoStory",\
    "PlayStation Video",\
    "SportsMax",\
    "KakaoTV Live",\
    "Applicaster",\
    "Microsoft SharePoint",\
    "Cisco Services",\
    "Index Exchange",\
    "NBC News",\
    "fuboTV",\
    "AppDynamics",\
    "Tour Tracker Grand Tours",\
    "Blackboard",\
    "Chartbeat",\
    "e-bot7",\
    "Microsoft OneNote",\
    "HBO Go",\
    "MediaMath",\
    "Vilynx",\
    "WWE",\
    "TopBuzz",\
    "NBC Sports Scores",\
    "Sentry Functional Software",\
    "YuppTV",\
    "PerimeterX",\
    "NBA",\
    "Playrix",\
    "App Bazaar",\
    "Hotjar",\
    "Taplytics",\
    "RTVE alacarta",\
    "hayu",\
    "Gigya",\
    "TNT",\
    "Showmax",\
    "Atresmedia",\
    "CDNetworks",\
    "Google Classroom",\
    "Peer5",\
    "Satisfi Labs",\
    "Ayoba",\
    "Roblox",\
    "NHL",\
    "Innovative Connecting",\
    "Outbrain",\
    "Stan",\
    "VPN Monster",\
    "Mediacorp Services",\
    "ATRESplayer",\
    "Airship",\
    "DiDi Services",\
    "Mediaset Espana",\
    "Mitele",\
    "meWATCH",\
    "HBO Max",\
    "WarnerMedia Services",\
    "Nike Services",\
    "Accedo",\
    "99Taxis",\
    "Charter Communications",\
    "Viu",\
    "PCCW",\
    "Alibaba Cloud",\
    "Localytics",\
    "Tidal",\
    "Spot-Im",\
    "Youbora",\
    "Google Meet",\
    "Coinbase",\
    "WideOrbit",\
    "Meetrics",\
    "Conductrics",\
    "LaunchDarkly",\
    "ART19",\
    "HiveMQ",\
    "Handelsblatt",\
    "Podigee",\
    "Cloud-Mail-Ru",\
    "Innovid",\
    "Movistar Services",\
    "UCWeb",\
    "WizRocket",\
    "Batch",\
    "Vizbee",\
    "Wirtschaftswoche",\
    "Handelsblatt Services",\
    "IQ Digital",\
    "Meet Group Services",\
    "Movistar Plus",\
    "Wizzee",\
    "Intercom",\
    "NetEase",\
    "Penthera",\
    "Riot Games",\
    "ByteDance",\
    "Appsee",\
    "Tagged",\
    "Webtoon",\
    "MeetMe",\
    "Dynamic Yield",\
    "Elex Technology",\
    "Clash of Kings",\
    "IMDb",\
    "Billo",\
    "AIHelp",\
    "Crunchyroll",\
    "Shein",\
    "Avira Services",\
    "Samsung Services",\
    "Xigua Video",\
    "Toutiao",\
    "Buzzsprout",\
    "Megaphone",\
    "Bugsnag",\
    "Knotch",\
    "Wurl",\
    "Libsyn",\
    "Triton Digital",\
    "VNG Corporation",\
    "Baidu Maps",\
    "Moonton",\
    "Amagi",\
    "Kuaishou",\
    "MarkMonitor",\
    "Medium",\
    "Simplecast",\
    "Walmart",\
    "Grubhub",\
    "Kwai",\
    "Braze",\
    "KeepSolid Services",\
    "VPN Lite",\
    "SpringServe",\
    "Zynn",\
    "Workplace",\
    "Yaani",\
    "Washington Post",\
    "Curalate",\
    "ROMWE",\
    "Target Signature",\
    "Plex",\
    "Amazon Alexa",\
    "Bumble",\
    "Weather Channel",\
    "Reuters News",\
    "CryptoTrader",\
    "Cxense",\
    "Club Factory",\
    "Bombora",\
    "MOOV",\
    "collectd",\
    "Oracle Services",\
    "Comcast",\
    "Skred Messenger",\
    "Acast",\
    "The CW",\
    "CW Seed",\
    "Facebook Lite",\
    "Opera Services",\
    "The Wall Street Journal",\
    "TextNow",\
    "Bloomberg",\
    "Udemy",\
    "The Economist",\
    "Investing-com Services",\
    "CryptoWatch",\
    "Among Us",\
    "CookiePro",\
    "beIN SPORTS",\
    "beIN Media Services",\
    "Erstream",\
    "We Meet",\
    "NewsDog",\
    "Pugpig",\
    "Digiturk Play",\
    "Digicel Services",\
    "DoorDash",\
    "Photo Wonder",\
    "Quantcast",\
    "JioSaavn",\
    "Opera News",\
    "xHamster",\
    "Shahid",\
    "MBC Services",\
    "QQMusic",\
    "Seamless",\
    "Cires21",\
    "Splunk",\
    "CoinCap",\
    "ShapeShift Services",\
    "Al Jazeera Services",\
    "Al Jazeera English",\
    "DO Global Services",\
    "Webflow",\
    "Payback",\
    "Meitu Services",\
    "CamScanner",\
    "Intsig Services",\
    "JAUMO",\
    "PubMatic",\
    "Jio Services",\
    "Groupon",\
    "FYEO",\
    "Verizon Media Services",\
    "Activision Blizzard",\
    "Audiomack",\
    "Kakao Map",\
    "OSN",\
    "GSM Association",\
    "Google Jibe",\
    "BeautyPlus",\
    "VivaVideo",\
    "Call of Duty",\
    "MicoWorld Services",\
    "Hago",\
    "Daum Cafe",\
    "Tencent News",\
    "Flipboard",\
    "Viacom18 Services",\
    "Voot",\
    "SelfieCity",\
    "Meitu",\
    "Kaspersky Services",\
    "Hakuna",\
    "Bermuda Video Chat",\
    "Xiaomi Services",\
    "Opera News Lite",\
    "Gaijin Entertainment",\
    "Scopely Services",\
    "MARVEL Strike Force",\
    "Sky News",\
    "Systex Services",\
    "Perfect Corp Services",\
    "YouCam Makeup",\
    "Deutsche Telekom AG Services",\
    "PlayerUnknowns Battlegrounds",\
    "Google Voice",\
    "theScore",\
    "Pantaya",\
    "Daum Mail",\
    "Facebook Broadcasting",\
    "War Thunder",\
    "APUS Services",\
    "Ufoto Services",\
    "Peak Games Services",\
    "Toon Blast",\
    "Expo Framework",\
    "The New York Times",\
    "DU Recorder",\
    "Luminati",\
    "Meizu Services",\
    "JOYY Services",\
    "Mobvista",\
    "StarTimes ON",\
    "Vodafone Services",\
    "The Guardian",\
    "Guardian Media Services",\
    "Mozilla Services",\
    "Magnite",\
    "DC Universe",\
    "Rakuten Services",\
    "Playdemic Services",\
    "Golf Clash",\
    "OneTrust",\
    "Lightstep",\
    "Arc Publishing",\
    "VS Media",\
    "Exaring Services",\
    "waipu-tv",\
    "Google News",\
    "RingCentral",\
    "Mortal Kombat",\
    "VPN Valley",\
    "beIN CONNECT",\
    "Boomplay Music",\
    "Kongregate",\
    "Ninja Kiwi Services",\
    "Huawei Services",\
    "XMRig",\
    "LoL Wild Rift",\
    "Tinkoff",\
    "Yandex Market",\
    "Sweet Selfie",\
    "Sweet Selfie Lite",\
    "DigiCert",\
    "Switcherry VPN",\
    "Xender",\
    "NCSoft",\
    "OrNET",\
    "CoinGecko",\
    "Universal Music Group",\
    "Malwarebytes",\
    "RealNetworks",\
    "KeyCDN",\
    "Riley Cillian",\
    "Streamlive-to",\
    "Wildlife Studios",\
    "Block Craft 3D",\
    "Lumen Technoloies",\
    "LOVOO",\
    "Clubhouse",\
    "Sniper 3D",\
    "Comodo Services",\
    "Ribbon Communications",\
    "BlackBerry Work",\
    "Joom",\
    "BlackBerry Services",\
    "Google Pay",\
    "Geekflare",\
    "War Machines",\
    "RawVoice",\
    "Podtrac",\
    "Wise",\
    "Google Podcasts",\
    "Shalltry Group",\
    "LiveRamp",\
    "Euronews",\
    "Leap Fitness Group",\
    "Hexaglobe",\
    "Rocket League",\
    "Magine Services",\
    "Pay-Rw",\
    "Vuba Vuba",\
    "Yandex Go",\
    "Yandex Telemost",\
    "Nexon Corporation",\
    "Agoda",\
    "Grab",\
    "New York Public Radio",\
    "PRX Services",\
    "Dovetail",\
    "Huya",\
    "SHAREit",\
    "Eversend",\
    "5channel",\
    "Pet Rescue Saga",\
    "ClassDojo",\
    "Photomath",\
    "miHoYo Games",\
    "Genshin Impact",\
    "Honkai Impact 3rd",\
    "Khan Academy",\
    "HOMER",\
    "Castbox",\
    "OpenText Services",\
    "edX",\
    "Socratic",\
    "Telia Services",\
    "Telia TV",\
    "Peacock TV",\
    "Jio Ads",\
    "Discovery Plus",\
    "Duolingo",\
    "ViacomCBS Services",\
    "BET Plus",\
    "Paramount Plus",\
    "HubSpot",\
    "Philo",\
    "DNA TV",\
    "Xiaomi Weather",\
    "JioChat",\
    "Remind",\
    "Elisa Services",\
    "Elisa Viihde",\
    "PubNub",\
    "Funimation",\
    "Drift",\
    "International Management Group",\
    "Endeavor Services",\
    "Softbank Services",\
    "Reface",\
    "MTV Katsomo",\
    "Amazon Ads",\
    "Prende TV",\
    "Univision Communications",\
    "Anchorfm",\
    "VeVe",\
    "VRV",\
    "Qualtrics",\
    "Wangsu CDN",\
    "Algolia",\
    "Maxivision",\
    "G-Core Services",\
    "Nordic Entertainment Group",\
    "Viafree",\
    "Netmera",\
    "Gracenote",\
    "storyfm-cn",\
    "Yle Areena",\
    "Streamable",\
    "Speakol",\
    "Yallakora",\
    "LINKonLINE",\
    "Gemini Media Egypt",\
    "Sanoma Services",\
    "Ruutu",\
    "Finnpanel",\
    "Noon VPN",\
    "Masrawy",\
    "KTPlay",\
    "Team Fortress 2",\
    "Trust Wallet",\
    "Canal Plus Afrique",\
    "Sarmady Services",\
    "FilGoal",\
    "AMC Theatres",\
    "Youm7",\
    "Valve Corporation",\
    "Snail VPN",\
    "Xoom",\
    "IBM",\
    "Midoki Services",\
    "VPN Master X",\
    "Qualaroo",\
    "MTS Services",\
    "Expedia",\
    "GlobalSign",\
    "LetsEncrypt",\
    "Yieldmo",\
    "IdenTrust",\
    "GoDaddy",\
    "Google Trust Services",\
    "Watch iT",\
    "Amazon Trust Services",\
    "Entrust",\
    "Binance",\
    "Elfsight",\
    "Sailthru",\
    "Certum",\
    "MediaNova CDN",\
    "Napster",\
    "Sectigo",\
    "Bazaarvoice",\
    "Suning Services",\
    "Indigitall",\
    "Ada Support",\
    "Penguin Sports",\
    "Pusher",\
    "CSG Media",\
    "Grapeshot",\
    "Knighthood",\
    "Blossom Blast Saga",\
    "Cybertrust CA",\
    "Usercentrics",\
    "Idviu",\
    "iApps Technologies",\
    "Calm",\
    "PlayStation Now",\
    "Microsoft Azure",\
    "Zego",\
    "Dish Network",\
    "jQuery",\
    "Mox TV",\
    "Stats Perform",\
    "Contentful",\
    "WeTV",\
    "Adex",\
    "Arlo",\
    "JioHealthHub",\
    "GOG",\
    "Ring",\
    "Fitbit",\
    "Google Fit",\
    "Diamond Diaries Saga",\
    "Deluxe Media",\
    "The Trade Desk",\
    "JioCloud",\
    "Trivago",\
    "Minutely",\
    "Garena Services",\
    "Garena Free Fire",\
    "Impact Services",\
    "Samsung Max Privacy VPN",\
    "Private Tunnel VPN",\
    "TripleLift",\
    "AlphaBetty Saga",\
    "Mailchimp",\
    "A1 Belarus",\
    "VOKA TV",\
    "SPB TV",\
    "Smart Media Solutions",\
    "Smaato",\
    "Teads",\
    "Pyramid Solitaire Saga",\
    "Riskified",\
    "Papa Pear Saga",\
    "Diamond Digger Saga",\
    "Leanplum",\
    "Tapad",\
    "Squarespace",\
    "BrightLine",\
    "Extreme Reach",\
    "RocketVPN",\
    "Moovit",\
    "Coinbox",\
    "SYBO Games",\
    "Subway Surfers",\
    "National Public Radio",\
    "ESET Services",\
    "MakeupPlus",\
    "Kakao Webtoon",\
    "ZEE5",\
    "Juspay",\
    "Pokemon Unite",\
    "Zomato",\
    "PLYmedia",\
    "BitPay",\
    "Phycan VPN Services",\
    "Internet Archive",\
    "Cloudflare WARP VPN",\
    "BRD Wallet",\
    "Exodus Wallet",\
    "BeautyCam",\
    "Gab",\
    "Measurement Lab",\
    "Glovo",\
    "Samsung Music",\
    "Sweet TV",\
    "Uklon",\
    "Toca Boca",\
    "TAVR Media",\
    "Lightsail VPN",\
    "IBM Cloud Video",\
    "Agora Services",\
    "Sun FM Ukraine",\
    "OLL TV",\
    "Lanet TV",\
    "Tigo Services",\
    "Uber Eats",\
    "Bolt Technology",\
    "Stripe",\
    "Medallia",\
    "OneplusOne Media Services",\
    "OneplusOne Video",\
    "Verisign",\
    "DivanTV",\
    "Admixer",\
    "DingTalk",\
    "Online Radio Box",\
    "MOLOCO",\
    "LiveXLive",\
    "ProStream-Ru",\
    "Blockchain-com",\
    "Qualcomm",\
    "JioMart",\
    "JioMeet",\
    "HCL Notes and Domino",\
    "Mi Fit",\
    "Paxful Wallet",\
    "Schildbach-Wallet",\
    "Cake",\
    "TigerConnect",\
    "iCloud Private Relay",\
    "Ludo King",\
    "Twilio Services",\
    "Ankama Games",\
    "Meta Services",\
    "Huawei Health",\
    "Youdao",\
    "Xinhua News",\
    "Raygun",\
    "Google Chat",\
    "DoubleVerify",\
    "Extole",\
    "StarMaker",\
    "MX Player",\
    "Origins Digital",\
    "Sygic Services",\
    "Budge Studios",\
    "MTS TV",\
    "Smadex",\
    "Amazon Games",\
    "New World",\
    "Aarki",\
    "Xiaomi ShareMe",\
    "GoodBarber",\
    "LitRes-Ru",\
    "Qwilt",\
    "Start-Ru",\
    "Pou",\
    "Mumble",\
    "AmEx Services",\
    "Wizards of the Coast",\
    "Magic The Gathering Arena",\
    "Mozilla VPN",\
    "Deutsche Kreditbank",\
    "Deutsche Bank",\
    "NortonLifeLock Services",\
    "NortonLifeLock VPN Services",\
    "Zapya",\
    "Amobee",\
    "Ubitus Services",\
    "Nintendo Switch Cloud Gaming",\
    "Spreaker",\
    "JioCinema",\
    "Epsilon-Conversant",\
    "Nvidia Services",\
    "Wink",\
    "Rostelecom",\
    "NGenix",\
    "Intel Services",\
    "Advanced Micro Devices Services",\
    "Pikmin Bloom",\
    "123VPN",\
    "Playtika Services",\
    "AVG Services",\
    "Netflix Services",\
    "Netflix Games",\
    "Hero Wars",\
    "SwiftyBeaver",\
    "Bitdefender Services",\
    "iHeartMedia Services",\
    "Sophos Services",\
    "McAfee Services",\
    "Eurosport News",\
    "Ola",\
    "LG Electronics Services",\
    "Ericsson Services",\
    "ShareChat",\
    "Valorant",\
    "Indeed",\
    "Nvidia GeForce NOW",\
    "True Key",\
    "Moon Active",\
    "Safe Family",\
    "Lotame",\
    "Firefox Private Network",\
    "Media-net",\
    "JSDelivr",\
    "Ubisoft Services",\
    "Ubisoft Connect",\
    "Blitzer-de",\
    "Samsung Pay",\
    "Google Photos",\
    "Stellar",\
    "TrackMania",\
    "Want Want Services",\
    "Unruly",\
    "TotalAV",\
    "atudo",\
    "BidSwitch",\
    "ING Banking",\
    "FeedAd",\
    "Commerzbank Banking",\
    "VR Banking",\
    "Anzu VR",\
    "Beeswax",\
    "Dauntless",\
    "CatchMedia",\
    "Piano",\
    "Shopify",\
    "PC Protect",\
    "Cisco Systems Services",\
    "Sparkasse Banking",\
    "HolaSpark",\
    "Microsoft Bing",\
    "TestMy-net",\
    "Fuelio",\
    "SpeedOf-Me",\
    "Xfinity",\
    "Xfinity Speedtest",\
    "Paycentive",\
    "Rainbow Six Siege",\
    "JPMorgan Chase",\
    "Multi Media LLC",\
    "Capital One",\
    "Sygic Travel Maps",\
    "Glassbox",\
    "Stark Games",\
    "Wells Fargo",\
    "Blinkist",\
    "Games Workshop",\
    "Warhammer Gaming",\
    "Redfast",\
    "Samsung Health",\
    "SpeedSmart",\
    "Sygic GPS Navigation",\
    "Comcast Technology Solutions",\
    "ViuTV",\
    "Demandbase",\
    "Rainbow Six Extraction",\
    "Warhammer TV",\
    "Disney Hotstar",\
    "Wunderkind",\
    "Might And Magic Chess Royale",\
    "Pulse Insights",\
    "Radarbot",\
    "AdsWizz",\
    "Bright Data",\
    "Wakanim",\
    "Etsy",\
    "Vanilla",\
    "Citadel Colour",\
    "Behaviour Services",\
    "Dead by Daylight",\
    "Black Library",\
    "Digital Turbine",\
    "DuckDuckGo",\
    "LiveOne",\
    "Josh",\
    "BaBe",\
    "Wikimedia Foundation",\
    "Avast VPN Services",\
    "OpenWeb",\
    "Streamio",\
    "TMT Broadcasting",\
    "UNO",\
    "Verve Group",\
    "McAfee Security",\
    "Lokalise",\
    "Brainly",\
    "ZONER AntiVirus",\
    "ZONER Services",\
    "Magenta TV",\
    "Lark",\
    "CapCut",\
    "Podbean",\
    "Permutive",\
    "Wistia",\
    "Rappi",\
    "KION",\
    "Okko",\
    "Picsart",\
    "MX TakaTak",\
    "Sender",\
    "F-Secure",\
    "Aarogya Setu",\
    "Aniview",\
    "Pinduoduo",\
    "Gearbox Entertainment Services",\
    "Gearbox Shift",\
    "Age of Empires",\
    "Relic Entertainment",\
    "Xbox Game Studios",\
    "Split",\
    "Truth Social",\
    "Rumble",\
    "Fandom",\
    "Uscreen",\
    "Hyperconnect",\
    "Font Awesome",\
    "Ruby Game Studio",\
    "MoEngage",\
    "Lost Ark",\
    "Miro",\
    "Google TV",\
    "Cadent",\
    "Pangle",\
    "Embrace",\
    "StreamGuys",\
    "Holler",\
    "Easybrain",\
    "Easybrain Music Maker",\
    "Creative Assembly",\
    "Sega",\
    "Match Group",\
    "Total War",\
    "Feral Interactive",\
    "LiveBoard",\
    "Naraka Bladepoint",\
    "NetEase Games",\
    "Patreon",\
    "United States Postal Service",\
    "IAB Europe",\
    "United Parcel Service",\
    "Q4",\
    "Tidio",\
    "Warner Bros Discovery Services",\
    "Lazada",\
    "OnlyFans",\
    "Primis",\
    "Blue Billywig",\
    "Geolocation Technology",\
    "Lumen Technologies",\
    "Fruitshake",\
    "Improve Digital",\
    "Terminus",\
    "Tableau",\
    "Tiny Tinas Wonderlands",\
    "AG Projects Services",\
    "Display-io",\
    "BeeTalk",\
    "Zillow",\
    "Plurk",\
    "InMobi",\
    "Arc XP",\
    "Embracer Group",\
    "Spin The Bottle",\
    "Xandr",\
    "Mapp",\
    "9GAG",\
    "Square Enix",\
    "Final Fantasy XIV",\
    "Naver Works",\
    "Naver Corporation",\
    "Intuit",\
    "Gazprom-Media",\
    "TNT-Premier",\
    "Radancy",\
    "Rambler",\
    "Crypto-com",\
    "FIFA Soccer",\
    "Kustomer",\
    "LogRocket",\
    "SpotHero",\
    "Adobe Experience Cloud",\
    "ICF Technology",\
    "Tricolor TV",\
    "Redfin",\
    "Matterport",\
    "Naver Cloud",\
    "Rogue Company",\
    "WOW TV",\
    "Quora",\
    "Infobip",\
    "Edgio",\
    "Flipkart",\
    "Near",\
    "Avo",\
    "Contentsquare",\
    "Fall Guys",\
    "Yahoo Ad Tech",\
    "Meesho",\
    "Tower of Fantasy",\
    "nPerf",\
    "NBCUniversal",\
    "Naver Map",\
    "Snapdeal",\
    "Smartclip",\
    "DigitalOcean",\
    "happn",\
    "2K",\
    "Microsoft Family Safety",\
    "Rumbleverse",\
    "WB Games",\
    "Microsoft Defender",\
    "Adobe Fonts",\
    "Vimond",\
    "AIONCLOUD",\
    "AMC Networks",\
    "AMC Plus",\
    "Cabify",\
    "Paytm",\
    "Lenovo Services",\
    "Uplive",\
    "Naver Mail",\
    "Akamai Technologies",\
    "StackAdapt",\
    "Hi-Rez Studios",\
    "ViX",\
    "Tremor Video",\
    "Block Inc",\
    "Cash App",\
    "Outfit7",\
    "Mythic Legends",\
    "United Internet AG Services",\
    "GMX Mail",\
    "Inshorts",\
    "Revolut",\
    "Zepeto",\
    "Diablo Immortal",\
    "Life360",\
    "Session",\
    "Premier",\
    "Tencent Cloud",\
    "PayPay",\
    "6Sense",\
    "Vimeo OTT",\
    "Zeus Network",\
    "Vidazoo",\
    "HIDIVE",\
    "Grupo Globo Services",\
    "Globoplay",\
    "Shudder",\
    "Kingsoft Services",\
    "Kingsoft Cloud",\
    "Kobo",\
    "For Honor",\
    "Tremor International",\
    "Financial Times",\
    "Kape Services",\
    "Square",\
    "hCaptcha",\
    "AMC SVOD",\
    "Sundance Now",\
    "Naver NOW",\
    "Australia Post",\
    "Bungie Games",\
    "Ora TV",\
    "CCP Games",\
    "EVE Online",\
    "Zype",\
    "Umeng Plus",\
    "BYJUS",\
    "Tilda",\
    "CDN77",\
    "Tapclap",\
    "Pirate Treasures",\
    "Prismic",\
    "Tencent Music",\
    "Httpool",\
    "CacheFly",\
    "Alibaba Group Services",\
    "QANDA",\
    "Vercel",\
    "Streamotion",\
    "DraftKings Services",\
    "DraftKings Casino",\
    "DraftKings Sportsbook",\
    "Wix",\
    "FanDuel Services",\
    "FanDuel Casino",\
    "FanDuel Sportsbook",\
    "Cryptic Studios",\
    "Neverwinter",\
    "Resso Music",\
    "Xumo",\
    "Realm Royale",\
    "Vikingard",\
    "Badlanders",\
    "Infinite Lagrange",\
    "ViewLift",\
    "TVNZ",\
    "Wynk Music",\
    "Gearbox Publishing",\
    "Viju",\
    "Viasat Ru",\
    "LilithGames",\
    "Rise of Kingdoms",\
    "DECA Games",\
    "Gods and Glory",\
    "Podcast Addict",\
    "EVE Echoes",\
    "Lazy Audio",\
    "Amediateka",\
    "Mixi",\
    "Mixi Services",\
    "Lord of the Rings War",\
    "Google Translate",\
    "NTT Services",\
    "Radiko",\
    "CyberAgent Services",\
    "Ameba",\
    "RedHotPie",\
    "Clicktouch Games",\
    "Adtech Studio",\
    "MediaLab AI Services",\
    "Mitene",\
    "Transperth",\
    "Hinge",\
    "SuperAwesome",\
    "Yandex Translate",\
    "Cashi",\
    "Booking Holdings Services",\
    "Banco Azteca",\
    "GStatic",\
    "DiDi Food",\
    "DiDi Rider",\
    "Facebook Messenger",\
    "Coppel",\
    "unclassified",\
    "OkCupid",\
    "Mercado Libre",\
    "BAND",\
    "Mercado Pago",\
    "Smilegate",\
    "BanCoppel",\
    "V4",\
    "VLMedia Services",\
    "Microsoft Translator",\
    "CDNJS",\
    "Telemetro",\
    "Corporacion Medcom",\
    "Here TV",\
    "Overwatch",\
    "Snow Services",\
    "Mastodon",\
    "Glu Mobile Services",\
    "KartRider Rush",\
    "GroupM",\
    "UltraViewer",\
    "Kingdom Age",\
    "Star Trek Online",\
    "WEB-DE",\
    "mail-com",\
    "ARTE",\
    "VITA",\
    "Paramount Services",\
    "BluTV",\
    "Xiaohongshu",\
    "Rakuten TV",\
    "Apple Weather",\
    "MUBI",\
    "QuantumCast",\
    "Knights and Dragons",\
    "Smite",\
    "Teleport Media",\
    "CBS News",\
    "CBS Sports",\
    "RBC News",\
    "Canonical Services",\
    "Exxen",\
    "Sony Group Services",\
    "Soul Hunters",\
    "Adtelligent",\
    "Datadog",\
    "Segmento-ru",\
    "Polyfill-io",\
    "B612",\
    "Warpath Ace Shooter",\
    "Tower of Saviors",\
    "Lemon8",\
    "Coveo",\
    "Hypic",\
    "Escape from Tarkov",\
    "Pocket",\
    "Marvel Snap",\
    "Adobe Document Cloud",\
    "Zangi Messenger",\
    "Beamable",\
    "bunny-net",\
    "BCE Services",\
    "Crave",\
    "BytePlus",\
    "Plarium",\
    "Sparta War of Empires",\
    "Sber",\
    "JivoChat",\
    "Youtube Kids",\
    "Oppo Mobile Services",\
    "HeyTap",\
    "Max",\
    "World Of Warships",\
    "Youtube TV",\
    "Google Family Link",\
    "Celtic Heroes",\
    "Hero Hunters",\
    "Times Internet Services",\
    "ServiceNow",\
    "Honkai Star Rail",\
    "Elder Scrolls Online",\
    "Nuverse",\
    "TVPlus",\
    "Nasdaq Services",\
    "SESISOFT Services",\
    "Tivibu GO",\
    "Dungeon Fighter Online",\
    "5GMARK",\
    "Kill Shot Bravo",\
    "puhutv",\
    "Dogus Services",\
    "OpenAI",\
    "Qobuz",\
    "S Sport Plus",\
    "World Of Tanks Blitz"

/**
 * List of strings, each describing the appropriate application.
 *
 * Items may be changed between releases.
 */
#define PACE2_APPLICATIONS_DESCRIPTION \
    "No application was detected for this flow and the application classification has stopped.",\
    "Facebook is the most popular social platform on the web. Detected traffic also includes the messenger application.",\
    "MagicJack is a VoIP application which can also use a regular landline with a usb adapter.",\
    "iTunes is a media streaming and management software from Apple.",\
    "Myspace is a social networking site with a focus on music and movies.",\
    "FaceTime is a free VoIP application with video chat functionality for Apple platforms.",\
    "Truphone is an application that finds the best cellular networks for travellers.",\
    "Twitter is a popular micro blogging service.",\
    "All traffic generated by the XBOX gaming console and its variants.",\
    "Removed. Reason: Was falsely implemented as an application. RealMedia is implemented as a protocol (transferred content) instead.",\
    "Google Mail is a free mail service provided by Google using HTTPS.",\
    "Goober was a VoIP messenger application that was available on many platforms. It was renamed into ChatCube in 2014. Therefore, all Goober related traffic will be classified as ChatCube.",\
    "Apple iCloud allows synchronization of files, mails and contacts on iOS, Mac and Windows platforms.",\
    "Ubuntu One was a filetransfer and synchronization service. Cloud storage and synchronization features were shut down at the end of July 2014, therefore the Ubuntu One application was deprecated.",\
    "DropBox is a cloud based file storage service.",\
    "Yuilop was a messenger application for smartphones. The application was renamed to Upptalk with version 2. Therefore, all Yuilop related traffic will be classified as Upptalk.",\
    "iMessage is a messaging service for Apple platforms.",\
    "GoToMyPC is a remote control software provided by Citrix.",\
    "GoToMeeting is a web conferencing application designed to broadcast the desktop view of a host computer to a group of computers connected to the host through the Internet.",\
    "Windows Azure was renamed into Microsoft Azure, a cloud service from Microsoft providing data storage and media streaming services. Please use instead Microsoft Azure application.",\
    "Amazon Cloud consists of various web services which can be used for load balancing, content distribution or cpu intensive tasks.",\
    "Dailymotion is a popular video streaming portal.",\
    "Deezer is a popular music-on-demand streaming service.",\
    "Grooveshark was an online music streaming service which allowed users to provide their own music content. It was removed from the stores in April 2015 because of copyright infringement lawsuits. Since the ownership of the service and website was transferred to the label(s) behind the lawsuits, all Grooveshark-related traffic will be classified as Universal Music Group.",\
    "SUDA Phone was a Korean voice over IP application for smartphones and tablets. It no longer exists, therefore it was deprecated.",\
    "Office 365 is a browser based office suite provided by Microsoft.",\
    "CNTV (China Network Television), since 2017 also CCTV (China Central Television), is a Chinese broadcaster with a network of 50 channels, providing online streaming services.",\
    "SINATV was a chinese video streaming application and website. It was removed from the stores and the related website does no longer exist, therefore it was deprecated. Related traffic will be classified as Sina Weibo.",\
    "Youtube is a popular video streaming website.",\
    "VoipSwitch (VoIP Tunnel) is a tunneling software for VoIP traffic.",\
    "Various mobile and browser based games by Zynga.",\
    "Crime City is a game with the goal to build a criminal empire and fight against other players.",\
    "Modern War is a game with the goal to build a global military empire and fight against other players.",\
    "Vippie is a VoIP and messenger application for smartphones.",\
    "Box is a cloud based file storage service similar to DropBox.",\
    "Skydrive was an online file storage service provided by Microsoft. It was renamed to OneDrive in February 2014. Therefore, this application was deprecated and all related traffic will be handled by the OneDrive application classification.",\
    "Adobe Creative Cloud provides access to Adobe's creative tools for a monthly fee, this also includes a cloud storage for user data.",\
    "Linkedin is a social networking website and application with a focus on business contacts.",\
    "Orkut was a social networking website provided by Google. It was removed from the stores in September 2014 and was therefore deprecated. All related traffic will be handled by the Google Shared Services application classification.",\
    "hi5 is a social network website.",\
    "Soribada is a korean music and video streaming website.",\
    "Zoho Work Online is a browser based office suite providing services for most business related tasks.",\
    "Hotmail was a popular free mail service provided by Microsoft. Microsoft phased out Hotmail in October 2011, relaunching the service as Outlook.com in 2012. Therefore, the application was deprecated and all related traffic will be handled by the Microsoft Outlook application classification.",\
    "Instagram is a image editing and management software for Android and iOS.",\
    "SugarSync is an online file synchronisation and storage service.",\
    "Wickr is an instant messaging application for smartphones which gives the user control over the accessibility of messages and files.",\
    "TigerText was an instant messenger service where all messages are kept on the server and can be deleted by the sender after a certain time. Their brand has been changed to TigerConnect on March 1, 2018. Therefore the application was deprecated and all related traffic will be handled by TigerConnect.",\
    "Vibe was a geo-social media sharing application for iOS. It supported sharing of text, images and videos within a certain distance. It is no longer available on the app store and was therefore deprecated.",\
    "EnLegion was a location based instant messaging application for iOS. It was removed from the stores in September 2012 and was therefore deprecated.",\
    "Voxer is an VoIP application with push-to-talk functionality.",\
    "LOVEFiLM was a movie streaming portal provided by Amazon. In February 2014, Amazon announced that Lovefilm's streaming service would be folded into its Instant Video service. Therefore, this application was deprecated and all related traffic will be handled by the Amazon Prime Video application classification.",\
    "Friendster was a web based social network and gaming site. In June 2018 it closed down as a company and eventually dissolved. Therefore it was deprecated.",\
    "Burner is an VoIP application that is available on iOS and Android.",\
    "Flickr is an imagehoster and sharing portal for photos and short videos.",\
    "live-com was a web platform which joined different Microsoft products, such as e-mail and instant messaging. The brand phased out during August 2012, therefore the application was deprecated and all related traffic is now handled by the Microsoft Services application classification.",\
    "Removed. Reason: The name of the application was not fitting. Instead the application ICQ should be used.",\
    "RedTube is a video streaming site specialised on pornographic content. Videos are uploaded either by users or by the publishers themselves for advertising purposes.",\
    "Evernote is a service for making notes online.",\
    "Spreed is a browser based web conferencing service.",\
    "textPlus is a free texting and calling smartphone application.",\
    "Xvideos is a streaming website for pornographic content.",\
    "Snapchat is a messaging application used for sharing photos, videos, text and drawings.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from Tudou, instead the application Youku Tudou should be used.",\
    "MyMusic is a taiwanese music streaming service.",\
    "RenRen is a popular social networking website in China. Various services such as photo sharing, games and music streaming are also provided.",\
    "Yelp is a social network with the purpose to share local reviews and events.",\
    "Amazon shopping is the online shop provided by Amazon in various countries.",\
    "YouPorn is a streaming website for pornographic content.",\
    "EBay is an online auction and shopping website.",\
    "PornHub is a video streaming site specialised on pornographic content. Videos are uploaded either by users or by the publishers themselves for advertising purposes.",\
    "SpiderOak is an online backup service to share, sync, access, store and back up data.",\
    "ADrive is an online storage, backup and cloud drive application which is available on the web and mobile devices.",\
    "Tumblr is a blogging platform currently owned by Yahoo. Besides uploading their own content users may also embed blogs of other users into their own.",\
    "XING is a web-based social network for professional occupations. AMIANDO, an online event management company, is since 2011 part of XING, and will be identified as application XING.",\
    "Google Play Music is a music streaming service by Google.",\
    "Vimeo is a video-sharing website where users can upload, share and view videos.",\
    "Salesforce provides business related cloud services such as customer relationship management.",\
    "iTunes Radio is an internet radio streaming service from Apple. Since 2015 it's a part of Apple Music and will be identified as application Apple Music.",\
    "Sina Weibo is a popular micro blogging service in China.",\
    "Forfone was a free VoIP application that could also send SMS, current location as well as photos. It was removed from the stores in 2015 and therefore the application was deprecated.",\
    "LiveJasmin is a video chatting site specialised on pornographic content.",\
    "JingDong is a electronic commerce website in China.",\
    "Last-fm is a free music streaming service.",\
    "The IP Multimedia Subsystem is a set of specifications from 3GPP which aim to provide a standardized way of accessing services (VOIP, presence information) from different network types.",\
    "Baidu is an application used for provisioning of all services related to Baidu, Inc., a Chinese multinational technology company.",\
    "Taobao is Chinas largest C2C online shopping platform.",\
    "Watchever was a streaming website for movies and TV shows. It was removed from the stores in December 2016 and therefore the application was deprecated.",\
    "TTPod is a audio and video streaming platform in China.",\
    "Removed. Reason: The name of the application was not fitting. Instead the application 360 Total Security should be used.",\
    "Pinterest is a social network for sharing photos, locations and websites.",\
    "Vine was a social network for creating and sharing short video clips. It was officially discontinued in April 2019 and was therefore deprecated.",\
    "Net2Phone is a SIP based VoIP application for calls to landline.",\
    "Foursquare is a location-based social network with the purpose to share locations, reviews and events.",\
    "iLive.to was a streaming website where users were able to publish their own TV channels. The domain moved to streamlive.to in November 2014. Therefore the application was deprecated and all related traffic will be handled by the Streamlive.to application classification.",\
    "MEGA is a popular secure file storage and communication service.",\
    "Doook was a Java-based chat application. Doook was discontinued, therefore it was deprecated.",\
    "vBulletin is a popular forum software.",\
    "CloudMe is an file storage service that offers cloud storage and file synchronization.",\
    "Yik Yak is an anonymous social media application that allows users to microblog to other users nearby.",\
    "Redmine is a free and open source, web-based project management and issue tracking tool.",\
    "iHeartRadio is an internet radio platform owned by the iHeartMedia, Inc.",\
    "Avira Antivirus is an antivirus solution.",\
    "Removed. Reason: The name of the application was not fitting. Instead the application Norton Security should be used.",\
    "Kaspersky Antivirus is a Russian antivirus solution.",\
    "AVG Antivirus is an antivirus solution.",\
    "SoundCloud is a swedish online audio distribution platform. It allows to upload, record, promote and share music.",\
    "Rdio was an online music streaming service with social media functions. In November 2015, Rdio filed for bankruptcy and announced the sale of their intellectual property to Pandora Radio. Therefore, the application was deprecated and all related traffic will be handled by Pandora.",\
    "Amazon Prime Music is an online music store from Amazon. This classification does also cover Amazon Music and Amazon Music Unlimited.",\
    "RunKeeper is a social network application for tracking fitness statistics.",\
    "myBook is a Taiwanese book store application.",\
    "Nike Plus is a fitness tracking application.",\
    "MagV is an online magazin store.",\
    "MagV Kids is an online magazin store for kids.",\
    "Dota Legend was a taiwanese round based RPG (Role-Playing-Game). The application is no longer accessible. Therefore, it was deprecated.",\
    "LeTV (also known as le.com or Leshi Video) is a Chinese platform for news, movies, video streaming and general entertainment.",\
    "MyFitnessPal is a free smartphone app and website that tracks diet and exercise to determine optimal nutrients and caloric intake for the users goals.",\
    "Sohu is a Chinese news aggregator, search engine, advertising and gaming portal.",\
    "1kxun is a free application for streaming audio and video content from external service providers.",\
    "TVBS is an taiwanese television channel with an online platform.",\
    "XNXX is a streaming website for pornographic content.",\
    "CtiTV is a nationwide television network based in Taiwan.",\
    "Endomondo was a social application for tracking fitness statistics. In December 2020, Endomondo was acquired by Under Armour, but then completely shut down end of 2021. Therefore, the application was deprecated.",\
    "Timely TV was a taiwan streaming website and application for movies and TV shows. It was no longer in use and was therefore deprecated",\
    "Workout Trainer is a fitness application for mobile devices that creates a training routine for the user and allows users to share progress with their friends.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application MapMyFitness should be used.",\
    "King of Pirates was a taiwanese Beat 'em up game. In July 2021, the game shut down all services. Therefore it was deprecated.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from Youku, instead the application Youku Tudou should be used.",\
    "BitTorrent is the general application for the corresponding P2P file sharing network.",\
    "eDonkey is a well-known P2P network.",\
    "KaZaa/Fasttrack was a P2P file sharing client. Since August 2012, the Kazaa website was no longer active. Therefore it was deprecated.",\
    "Gnutella is a P2P file sharing network and uses the corresponding protocol with the same name.",\
    "DirectConnect is a P2P network.",\
    "AppleJuice is a P2P file sharing network.",\
    "Soulseek is a P2P filesharing network developed by Nir Arbel. Similar to Napster it uses a central server.",\
    "Ares is a P2P network.",\
    "Freenet is a network for storing decentralized encrypted data with the aim of anonymity.",\
    "Mute was a P2P file-sharing program providing anonymity. The last release was in May 2009 and the project has been abandoned ever since.",\
    "Skype is an application for voice and video telephony. It also allows calls to normal phone networks.",\
    "MSN (rebranded as Windows Live Messenger in 2005), was an instant messaging app developed by Microsoft. It was discontinued in 2013 and was therefore deprecated. All related traffic is handled by the Microsoft Services application classification.",\
    "Yahoo is an American web services provider, that offers a web portal, a search engine, and related services.",\
    "vonage is a collaboration app mainly used in business contexts for calling, texting or video conferencing. It is also a Company providing VoIP API's for integration into other services. Vonage was acquired by Ericsson in 2021.",\
    "Removed. Reason: Was falsely implemented as an application. Oscar is implemented as a layer 7 protocol instead.",\
    "Gadu-Gadu is a Polish instant messenger with the capability for file-transfer, voice and video chat.",\
    "MPlus (alias M+ Messenger) is a chinese mobile instant messaging application published by Taiwan Kuro Times Ltd.",\
    "Removed. Reason: Was falsely implemented as an application. IRC is implemented as a layer 7 protocol instead.",\
    "Slingbox was an application that allowed television streaming over IP. In November 2022, Slingbox servers were permanently taken offline. Therefore, it was deprecated.",\
    "Vcast was an application for streaming current video contents like sports, news and entertainment from mobile devices. It was developed by Verizon Wireless and shut down in December 2015. Therefore, the application was deprecated.",\
    "The application 4shared-com is used for traffic from the hoster with the same name.",\
    "The application adrive-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application badongo-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application badongo-net was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application bestsharing-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application bigfilez-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application chinamofile-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application data-hu is used for traffic from the hoster with the same name.",\
    "The application depositfiles-com is used for traffic from the hoster with the same name.",\
    "The application easy-share-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fast-load-net was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fast-share-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application file-upload-net is used for traffic from the hoster with the same name.",\
    "The application file-upload-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filearchiv-ru is used for traffic from the hoster with the same name.",\
    "The application filebase-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filecloud-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filefactory-com is used for traffic from the hoster with the same name.",\
    "The application filefront-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable. All related traffic will be handled by the gamefront-com application classification.",\
    "The application fileho-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filehost-tv was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filepost-ru is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filer-net is used for traffic from the hoster with the same name.",\
    "The application files-upload-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application files-to was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filesafe-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filesend-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fileupyours-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application ftp2share-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application gigasize-com is used for traffic from the hoster with the same name.",\
    "The application gigeshare-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application hostggg-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application hotfile-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application ifolder-ru was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable. All related traffic will be handled by the rusfolder application classification.",\
    "The application keepmyfile-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application leteckaposta-cz is used for traffic from the hoster with the same name.",\
    "The application live-share-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application livedepot-net was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application load-to is used for traffic from the hoster with the same name.",\
    "The application massmirror-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application mediafire-com is used for traffic from the hoster with the same name.",\
    "The application megarotic-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application megashare-com is used for traffic from the hoster with the same name.",\
    "The application megashares-com is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable and it now redirects to tabtight-com.",\
    "The application megaupload-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application chinamofile-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application mofile-net was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application mytempdir-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application netload-in is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application odsiebie-najlepsze-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application quicksharing-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application rapidshare-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application rapidshare-de was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application rapidupload-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application savefile-com is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable and the domain is up for sale.",\
    "The application sendmefile-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application sendspace-com is used for traffic from the hoster with the same name.",\
    "The application sharebase-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application sharebee-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application sharebig-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application sharebigfile-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer a fileshare website.",\
    "The application sharebigflie-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application simpleupload-net was used for traffic from the hoster with the same name. It now redirects to file-upload-net",\
    "The application speedshare-org is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application turboupload-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable and it now redirects to transferbigfiles-com.",\
    "The application ultrashare-de was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable. All related traffic will be handled by the ultrashare-net application classification.",\
    "The application up-file-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploaded-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploading-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploadingit-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploadpower-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploadyourfiles-de was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application vip-file-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application wiiupload-net was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application yourfilehost-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application yourfiles-biz was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application ddl-youtube-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application zshare-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 4shared-vn was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application mega-1280-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application megashare-vn was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fileserve-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filesonic-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 1-upload-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 1fichier-com is used for traffic from the hoster with the same name.",\
    "The application 4fastfile-com was deprecated and should not be used anymore.",\
    "The application 10upload-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 1clickshare-net was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filedude-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application asixfiles-com was deprecated and should not be used anymore.",\
    "The application banashare-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application biggerupload-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application bitshare-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application cash-file-net was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application cramit-in is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application czshare-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application data-loading-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application dataport-cz was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application datei-to was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application downupload-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application easybytez-com is used for traffic from the hoster with the same name.",\
    "The application edisk-cz was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application enterupload-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application euroshare-eu is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application extabit-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fastfileshare-com-ar is used for traffic from the hoster with the same name.",\
    "The application fik1-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application file2upload-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fileape-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fileflyer-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filefrog-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filehook-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fileover-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filepost-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application files-mail-ru was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable. All related traffic will be handled by the Cloud-Mail-Ru application classification.",\
    "The application fileshare-in-ua is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filesmonster-com is used for traffic from the hoster with the same name.",\
    "The application filevo-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filezup-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application freakshare-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fshare-vn is used for traffic from the hoster with the same name.",\
    "The application gaiafile-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application gigapeta-com is used for traffic from the hoster with the same name.",\
    "The application gotoupload-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application hellshare-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application hitfile-net is used for traffic from the hoster with the same name.",\
    "The application hulkshare-com is used for traffic from the hoster with the same name.",\
    "The application hyperfileshare-com is used for traffic from the hoster with the same name.",\
    "The application i-filez-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application ifile-it is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application kewlshare-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application letitbit-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application loadfiles-in is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application megafree-kz was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application megarapid-eu was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application megashare-vnn-vn is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application midupload-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application multishare-cz is used for traffic from the hoster with the same name.",\
    "The application netuploaded-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application oron-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application przeklej-pl was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable. All related traffic will be handled by the easypaste-org application classification.",\
    "The application putshare-com is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable and the domain is up for sale.",\
    "The application qshare-com is used for traffic from the hoster with the same name.",\
    "The application quickshare-cz was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application rapidshark-pl was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application scribd-com is used for traffic from the hoster with the same name.",\
    "The application share-online-bz was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable. All related traffic will be handled by the share-online application classification.",\
    "The application share-rapid-cz was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application shragle-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application storage-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application terafiles-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application tunabox-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application turbobit-net is used for traffic from the hoster with the same name. Hitfile is using turbobit servers.",\
    "The application ugotfile-com is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uloz-to is used for traffic from the hoster with the same name.",\
    "The application unibytes-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application unlimit-co-il is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application up-4share-vn is used for traffic from the hoster with the same name.",\
    "The application upload-com-ua is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploadbox-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploader-pl was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploadfloor-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploadstation-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application upnito-sk is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application usershare-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application videobb-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application wupload-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application x7-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application yunfile-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable. All related traffic will be handled by the yunfile application classification.",\
    "The application zomgupload-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application ziddu-com is used for traffic from the hoster with the same name.",\
    "The application 1-upload-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 1-clickshare-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 123upload-pl was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 139pan-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 163pan-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 2shared-com is used for traffic from the hoster with the same name.",\
    "The application 4share-ws was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application 4us-to was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application alldrives-ge was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application allshares-ge was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application annonhost-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application boosterking-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application cobrashare-sk is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application coolshare-cz was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application crocko-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application cum-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable. All related traffic will be handled by the xcum-com application classification.",\
    "The application eyvx-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filearn-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filejungle-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filemaze-ws was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filesflash-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application freespace-by was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application fsx-hu is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application jakfile-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application jumbofiles-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application kickload-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application missupload-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application movreel-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application platinshare-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application sanupload-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application shareflare-net was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application u-115-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploadc-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uploadking-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application uptobox-com is used for traffic from the hoster with the same name.",\
    "The application filestube-com was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application gamefront-com is used for traffic from the hoster with the same name.",\
    "The application megaporn-com was used to classify traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application filesonic-in is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application turbobit-ru is used for traffic from the hoster with the same name.",\
    "The Usenet consists of various newsgroups in which users can post text messages similar to a web forum. Some servers allow the user to attach binary files which sometimes is used to share copyrighted files.",\
    "Removed. Reason: Was falsely implemented as an application. FTP is implemented as a layer 7 protocol instead.",\
    "Filetopia is a free peer-to-peer file sharing client and networking tool.",\
    "Manolito (MP2PN) was a P2P filesharing network. The application is no longer reachable and was discontinued after its last software update in 2010. Therefore it was deprecated.",\
    "iMesh is a centralized P2P file sharing client for video and music.",\
    "Pando was a free p2p file sharing software with integration for various messengers and social media sites. In August 2013 Pando ceased its business and shutdown all servers. It was acquired by Microsoft, therefore the application was deprecated and all related traffic will be handled by the Microsoft Services application classification.",\
    "SopCast was a peer to peer tv streaming application. Users were able to view and record already provided channels or start their own channel. The service is no longer available, therefore it was deprecated.",\
    "TVUPlayer was a P2P television streaming application. TVU Networks shut down the TVUPlayer service in February, 2013. Therefore this application was deprecated.",\
    "PPStream was an online-TV service that provided many free movies and videos and was popular in the Asian market. Their brand has been changed to Iqiyi in May 2013. Therefore the application was deprecated and all related traffic will be handled by the Iqiyi application classification.",\
    "PPLive was a Chinese online TV service. It changed its brand to PPTV in 2013. Therefore, the application was deprecated and all related traffic will be handled by the PPTV application classification.",\
    "Zattoo is a television and radio streaming software.",\
    "QQ is a popular instant messenger in China that was developed by Tencent.",\
    "QQLive is a Vietnamese application which allows users to have video-chats or watch erotic live-streams of Asian girls.",\
    "Feidian was a Chinese P2PTV program similar to PPLive. It was deprecated because the application is not reachable and cannot be located on the internet anymore.",\
    "PoPo is an instant messaging application popular in China.",\
    "Thunder/Webthunder is a P2P filesharing application mainly used in China.",\
    "UUsee is a peer to peer television streaming application.",\
    "VNC is a free remote desktop application which allows the user to remotely control the desktop of an other computer.",\
    "Steam is a game distribution platform developed by Valve.",\
    "Half-Life 2 is a first-person shooter game published and developed by Valve. It is using the Source Engine and is distributed via Steam.",\
    "SoftEther is a VPN solution developed by the japanese company with the same name. It was renamed to PacketiX VPN with version 2.0.",\
    "Telnet is an application based on opensource Putty Telnet as its backend library.",\
    "The Owner-Free File System (OFF System, or OFF for short) is a peer-to-peer distributed file system through which all shared files are represented by randomized multi-used data blocks.",\
    "Tor is a anonymization network and is supposed to hide the users identity by passing his traffic through various servers.",\
    "World of Warcraft is the most popular online roleplaying game worldwide.",\
    "Hamachi is a zero configuration VPN.",\
    "Skinny (SCCP) is a session signaling protocol developed by Selsius Systems and now owned by Cisco. It is mostly used in Cisco IP phone products as communication protocol between the phone and the Cisco Communications Manager.",\
    "Citrix is an application used for provisioning of all services related to Citrix Systems, a American multinational software company.",\
    "Paltalk is a free instant messaging, voip and video chat application where users may create their own channels.",\
    "Quake is a well known ego-shooter released 1996 by id Software. The source code was published 1999 which led to many new projects using the Quake engine.",\
    "Second Life is a virtual world in which all players can create their own objects and freely trade those. The virtual currency used in the game can be exchanged for real money.",\
    "Battlefield is a series of first-person shooter games.",\
    "PCAnywhere was a remote desktop solution developed by Symantec. Since Symantec discontinued the product in May 2014, the application was deprecated.",\
    "Icecast is an open source streaming project.",\
    "SHOUTcast is a free audio and video streaming server developed by Nullsoft.",\
    "QQGame is an online platform for gaming and community interaction.",\
    "VeohTV is a video distribution website.",\
    "OpenFT (giFT) was a frontend for various file sharing protocols. The web servers are no longer reachable. Therefore, the application was deprecated.",\
    "Pandora is an internet radio application.",\
    "Internet Printing Protocol (IPP) is used to send commands to remote printers.",\
    "WarCraft 3 is a strategy game developed by Blizzard Entertainment. While the game is quite old it's multiplayer mode still is quite popular even today.",\
    "ooVoo was a video chat and a messaging application. It was discontinued in November 2017. Therefore the application was deprecated.",\
    "Google Hangouts was an instant messenger with audio, video and file-transfer capabilities. Google shut it down in November 2022, therefore it was deprecated.",\
    "iSkoot was a voice over IP client for mobile devices. In October, 2010, iSkoot was acquired by Qualcomm. Therefore, the application was deprecated and all related traffic will be handled by Qualcomm.",\
    "Fring was a mobile application for audio calls and video chats with messaging capabilities. In September 2013, GENBAND announced it has acquired this service. In 2017, GENBAND and Sonus Networks merged into Ribbon Communications. Therefore, this application was deprecated and all related traffic will be handled by the Ribbon Communications application classification.",\
    "IPTV is a service for television over IP.",\
    "YourFreedom is a web tunneling software developed to bypass firewalls and provide anonymization. It offers various connection modes which generate different kind of traffic.",\
    "VoipSwitch VoIP Tunnel aims to bypass firewalls blocking VoIP traffic by tunneling all the VoIP traffic over a single port.",\
    "Meebo was an instant messaging and social networking application. It was removed from the stores in June 2012 and acquired by Google. Therefore, the application was deprecated and all related traffic will be handled by the Google Shared Services application classification.",\
    "Funshion is a chinese p2p streaming application.",\
    "Aimini is a P2P software program used for file-transfer, audio/video chat and online TV.",\
    "ClubPenguin was a massively multiplayer online game (MMO) with cartoon penguin-avatars. It was discontinued in March 2017. Therefore, the application was deprecated.",\
    "Florensia is a massive multiplayer online role playing game with a 3D anime style.",\
    "TeamSpeak is a popular voice over ip application primarily designed for online gaming.",\
    "MapleStory is a free-to-play massive multiplayer online role play game.",\
    "Octoshape is a multimedia application that provides on-demand audio and video streaming using peer-to-peer grid technology. It was acquired by Akamai in April 2015. Therefore, all Octoshape related traffic will be classified as Akamai Cloud.",\
    "Removed. Reason: The name of the application was not fitting. Instead the application PlayStation Network should be used.",\
    "Dofus is a massive multiplayer online role playing game that takes place in a heroic-fantasy universe.",\
    "Fiesta is a massive multiplayer online role play game with anime style characters.",\
    "SplashFighters is a 3D action multiplayer game developed by CyberStep.",\
    "ActiveSync is a program that synchronizes Microsoft applications.",\
    "Nimbuzz was an XMPP-based instant messenger. The Nimbuzz site and servers went offline in mid 2019 and remain unavailable, the apps were removed from the stores in April 2019. Therefore, this application was deprecated.",\
    "CrossFire is a free-to-play first-person shooter game for Windows developed by Z8 Games.",\
    "Removed. Reason: Was falsely implemented as an application. WAP-WSP is implemented as a layer 7 protocol instead.",\
    "Guildwars is a popular online role-playing game series.",\
    "Armagetron is a 3D multiplayer game based on the light cycle sequence from the film Tron.",\
    "rFactor is a racing simulation developed by Image Space Incorporated (ISI).",\
    "TeamViewer is a desktop sharing application designed to be easy to use. For private use its free.",\
    "Gamekit is a cross-platform development tool.",\
    "UltraBac is a client-server based backup software.",\
    "Spotify is a music streaming and media services provider.",\
    "IMplus is a social media browser and instant messaging aggregator.",\
    "ANtsP2P was a peer-to-peer file sharing software with full encryption. The last beta version was released in 2013. The application was discontinued, therefore it was deprecated.",\
    "UltraSurf is a web tunneling software designed to bypass firewalls and other blocking/censorship attempts.",\
    "Viber is a voice over ip application which allows free calls over a network connection if the software is installed on both devices.",\
    "Tango is a live video streaming and social media platform.",\
    "iPlayer is a video-tv streaming service.",\
    "Opera Mini is the mobile version of the Opera browser. It uses a proxy server which reformats and compresses data before forwarding it to the client.",\
    "Bolt is a free mobile Web Browser.",\
    "Scydo was a mobile VoIP application. It was last time updated in 2013 and removed from all stores. Therefore, the application was deprecated.",\
    "WhatsApp is a very popular instant messaging software for smartphones.",\
    "Webex is an online conferencing application developed by Cisco.",\
    "Netflix is a well-known streaming service for movies and series.",\
    "Mojo was a P2P client to share iTunes libraries. The software did not receive an update since 2009, therefore it was deprecated.",\
    "IMO is a multi protocol instant messenger.",\
    "CitrixGoTo is the generic traffic used by different CitrixGoTo applications.",\
    "Ficall was a mobile application for instant messaging, voice calls and watching IPTV. Ficall was discontinued in 2011, therefore it was deprecated.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application Comodo Services should be used.",\
    "Ventrilo is a voice over ip application primarily used by online gamers.",\
    "MyPeople was a korean voice over IP and IM application provided by Kakao. It is no longer available in the stores and last updated in 2015. Therefore, the app was deprecated and all related traffic will be handled by the Kakao Services application classification.",\
    "eBuddy is an instant messenger for web and mobile devices.",\
    "Adobe Connect is a business software for web conferencing, presentations and shared desktops.",\
    "JAP (Java Anon Proxy) was an internet anonymizer also known as JonDo, that was developed by the University of Dresden in Germany. In August 2021, JAP stopped service. Therefore it was deprecated.",\
    "Wuala was an online storage service. The service was shutdown in August 2015. Therefore the application was deprecated.",\
    "WebDAV is an HTTP extension for file management which allows using a webserver as an online storage.",\
    "Lync was an enterprise software application for instant messaging and videotelephony developed by Microsoft. In 2015, the software was rebranded from Lync to Skype for Business. Therefore, this application was deprecated and all related traffic will be handled by the Skype for Business application classification.",\
    "Audiogalaxy was a music streaming service for own music. Until 2002 it was a music file-sharing software. It was officially discontinued in January 2013 and was therefore deprecated.",\
    "Mig33 (also known as migme Limited) was a mobile social network that supported instant messages, blogs and minigames. The site and app service went offline in August 2017. Therefore, the application was deprecated.",\
    "HTTPTunnel was an application to tunnel traffic using the HTTP protocol. Connecting to the servers is not possible anymore and the software did not receive an update since 2010, therefore it was deprecated.",\
    "Rhapsody was an on-demand music subscription service that offered unlimited access to a large library of digital music. It was rebranded as Napster in June 2016. Therefore, this application was deprecated and all related traffic will be handled by the Napster application classification.",\
    "WebQQ provided an entire desktop including various widgets and applications inside a Browser. The main functionality was the QQ instant messenger. The service was shutdown in January 2019. Therefore, the application was deprecated and all related traffic will be handled by the QQ application classification.",\
    "PD-Proxy is a VPN Tunneling Software for Windows offering anonymous and secure internet access. A premium account is required to be able to use all servers.",\
    "Cyberghost is a well-known VPN application provided by a company with the same name.",\
    "Google Cloud covers generic traffic generated by different cloud applications like Google Photos.",\
    "Google Drive is a cloud storage application provided by Google.",\
    "Google Docs is a cloud application to share and create office documents.",\
    "Google Plus was a social network provided by Google. It was discontinued in April 2019 and was therefore deprecated.",\
    "Oracle Database is a commonly known database management system produced and marketed by Oracle Corporation.",\
    "ITV is a british commercial televison broadcaster.",\
    "Demand 5 (previously Five Download) was a television network providing streams of various shows. From December 2018, the Demand 5 app was retired from all platforms and replaced by My5. Therefore the application was deprecated and all related traffic is handled by the My5 application classification.",\
    "Removed. Reason: Was falsely implemented as an application. RSVP is implemented as a layer 4 protocol instead.",\
    "Channel 4oD is a British puplic-service television broadcaster.",\
    "Lotus Notes, later IBM Notes, was a collaborative client-server software platform, that provided business collaboration functions, such as email, calendars and to-do lists. In July 2019, due to the acquisition of several products by HCL Technologies, Lotus Notes was renamed to HCL Notes and Domino.",\
    "Poison Ivy is a remote control software which is primarily used as malware. The server can be injected into executable files and installs itself on the system when run. It allows file-transfers, registry editing, webcam activation, screen capture, audio capture and other features usually found in this kind of application.",\
    "HiDrive is a cloud based file storage offered by Strato.",\
    "WeChat is a voice and text messaging application for different smartphone operating systems.",\
    "MyVideo was a movie streaming platform similar to YouTube that also offered commercial content and free streaming of complete movies. MyVideo has been acquired by Maxdome in April 2016. Therefore, the app was deprecated and all related traffic will be handled by the Maxdome application classification.",\
    "Kik Messenger is a free IM application for mobile devices with a focus on teenage users.",\
    "Line is a VoIP and messaging application.",\
    "Iperf is a tool to measure network performance by generating traffic.",\
    "OS update classifies traffic generated by the update process of operating system updates.",\
    "Hike Messenger is a free instant messaging application.",\
    "DiDi was a voice over IP and instant messaging application. The related Android/iOS apps are not available anymore and domains were shut down. Therefore, the application was deprecated.",\
    "IDrive is a cloud based storage and backup service.",\
    "Spreecast was a streaming service for private and commercial videos. The site was shut down on July 2016, therefore the application was deprecated.",\
    "Simfy was a music streaming service for desktop and mobile devices. It has been taken over by Deezer in 2015. Therefore the app was deprecated and all related traffic is handled by the Deezer application classification.",\
    "Dota 2 is a free to play online multiplayer game.",\
    "QVoD is a chinese all-in-one media player.",\
    "Hulu is an ad-supported on-demand video streaming service.",\
    "Microsoft Exchange is a groupware server to manage mails, contacts and calendars.",\
    "Microsoft Dynamics NAV is an enterprise resource planning software product.",\
    "Fuze is a free online meeting service.",\
    "Kakaotalk is a free mobile instant messaging application with text and call features.",\
    "Destiny (or Destiny 2) is a Massive Multiplayer Online First-Person Shooter (MMOFPS).",\
    "VyprVPN is a personal VPN solution.",\
    "Threema is an encrypted instant messaging application.",\
    "Removed. Reason: Was falsely implemented as an application. NTLM is implemented as a layer 7 protocol instead.",\
    "Twitch is a live streaming platform which is focused on video gaming and e-sport videos.",\
    "Wowza is a B2B live and on-demand video streaming platform.",\
    "Runtastic is a fitness tracking application for mobile devices with social networking features like tracking and sharing runs.",\
    "Telegram is an encrypted instant messaging application.",\
    "StreetVoice is a social network for sharing self made music.",\
    "Duokan is a Chinese application for reading eBooks.",\
    "Mei Lu (full name Mei Lu Ke Monogatari) was an Asian RPG tower defense strategy game by Happy Elements Technology (Beijing) Limited. It was removed from all App-Stores and the last update was made in 2017. Therefore, the application was deprecated.",\
    "Skout is a location-based social network and dating application.",\
    "PlayTales, formerly TouchyBooks, was an app for kids to read and play with interactive ebooks. It was last updated in 2016 and the service is no longer available. Therefore, the application was deprecated.",\
    "Game Of War is a mobile game developed and published by Machine Zone.",\
    "The FETnet application covers all traffic generated by websites or applications by Far Eastone Telecommunications Co., Ltd., a leading company in Taiwan which provides telecommunications and digital application services.",\
    "Fubon E-Broker (FBS) is a Chinese stock finance application.",\
    "National Baseball was a mobile sports game developed by Netmarble Joybomb Inc. It was removed from all App-Stores and was last updated in 2016. Therefore the application was deprecated.",\
    "PinkFong is a label which offers different apps for kids (music, videos).",\
    "Kollect Book is a Taiwanese book store. This also covers the application Hami Book.",\
    "SPTG TV is a nationwide television network based in taiwan.",\
    "Ciaomobile HDTV was a provider of livestreams based in Taiwan. The website is no longer reachable and was therefore deprecated.",\
    "Skyking was a popular video streaming application and website in China. The website is no longer accessible and the mobile applications do no longer connect and was therefore deprecated.",\
    "Moves was an application for tracking the daily sports activities of users. It was acquired by Facebook in 2014 and has been discontinued since July 2018. Therefore, the application was deprecated and all related traffic will be handled by the Facebook application classification.",\
    "Path was a social networking service for mobile devices to share photos and messages. In 2015, Path announced it had been acquired for an undisclosed amount by Kakao. It was removed from the stores in October 2018. Therefore, the application was deprecated and all related traffic will be handled by the Kakao Services application classification.",\
    "Age of Warring Empire is a RPG game.",\
    "iWow systex allows to check the stock exchange quotation with the iPhone or iPad.",\
    "9ku is a website for streaming and downloading music and ringtones.",\
    "Next TV is a nation wide chinese televison network.",\
    "Syncplicity is a file sharing and synchronization service which supports synchronizations between other online services like Google Docs and Facebook.",\
    "RealMa was a nationwide television network based in taiwan. It no longer exists, therefore it was deprecated.",\
    "Yuanta Web, also known as Yuanta Securities, is a Taiwanese provider of stock exchange and investment information applications.",\
    "HiNet is an application used for provisioning of all services related to HiNet, also known as Chunghwa Telecom Company, Ltd., the largest telecommunications company in Taiwan.",\
    "MyCard is a service mostly used in China and other Asian countries. It provides a prepaid system to purchase different digital products or in-game virtual goods.",\
    "Kono is a free application for reading articles from chinese newspapers and magazines.",\
    "Speakaboos was an app which offered kids stories with animation, music and sound effects. The Speakaboos company acquired HOMER in 2017 and the products where merged under the HOMER brand. Therefore, the application was deprecated and all related traffic will be handled by the HOMER application.",\
    "Duo Duo, also known as Dudu Reader, is a free application for reading and downloading Chinese ebooks.",\
    "Wealth God is a chinese stock exchange application.",\
    "iLeader is a chinese stock exchange application.",\
    "Three Bamboo is chinese stock exchange application.",\
    "Taaze is a free application for downloading and reading chinese ebooks and online magazines.",\
    "Audiko is a free application for cutting and downloading ringtones.",\
    "pubu bookbuffet is a free application for reading and downloading chinese ebooks and magazines.",\
    "Mitake is an application used for provisioning of all services related to Mitake Information Corporation, a Taiwanese company that researches and develops banking and stock exchange applications.",\
    "HyRead is a free application for reading and downloading chinese ebooks and magazines.",\
    "Disney Storytime was an application that offered Disney stories with animation, music and sound effects. It was removed from all stores and last updated in August 2019. Therefore, the application was deprecated.",\
    "Qualys is a provider of information security and compliance cloud solutions.",\
    "Constant Contact is an email marketing solution.",\
    "Rally is an Agile Development Software for Business Agility.",\
    "Magine TV was a TV streaming platform. It was removed from the stores in February 2019. Therefore, the application was deprecated and the related traffic will be handled by the Magine Services application classification.",\
    "IntraLinks is a global technology provider of inter-enterprise content management and collaboration solutions.",\
    "LivePerson is a website with live chat software for companies.",\
    "Speedtest is an internet connection quality measurement tool.",\
    "BlueJeans is an instant messaging software for smartphones. It also allows voice and video communication.",\
    "Apple Music is a music streaming service from Apple.",\
    "Naver is a sponsored advertisement portal in South Korea, that uses its own proprietary search engine.",\
    "Booking-com is an international online booking website and app.",\
    "CNN is an US based cable and satellite television channel. The website and app provide worldwide news.",\
    "Periscope was an application for streaming live videos. Twitter announced it would be discontinuing the service on March 31, 2021. Therefore, the application was deprecated and all related traffic is now handled by the Twitter application classification.",\
    "PayEasy is a business-to-client (B2C) platform in China.",\
    "Line Rangers is an RPG battle game.",\
    "This application covers all traffic generated by the iOS App Store.",\
    "MyVideo TW is a taiwanese video streaming service.",\
    "King Gaming is an application used for provisioning of all services related to King Digital Entertainment (also known as King.com), a video game developer specialised in the creation of social games.",\
    "Candy Crush is a mobile puzzle game series centered around differently colored candies.",\
    "Candy Crush Soda is a mobile puzzle game centered around differently colored candies. It is a variation from the original Candy Crush game, but with additional unique candies and more divine matching combinations.",\
    "Bubble Witch is a mobile puzzle game centered around bubbles in different colors.",\
    "Farm Hero is a mobile puzzle game centered around different fruits and vegetables.",\
    "This application covers all traffic generated by the PlayStation Network gaming console and its variants.",\
    "Google Play is the app store from Google providing application content.",\
    "Supercell is an application used for provisioning of all services related to Supercell, a Finnish online gaming company.",\
    "Clash of Clans is a fantasy-themed free mobile strategy game.",\
    "Boom Beach is a free real-time combat strategy game.",\
    "Hay Day is a free mobile farming game.",\
    "This application covers all traffic generated by the WindowsPhone Store or the Microsoft Store.",\
    "Skype for Business is the Microsoft instant messenger for corporations.",\
    "Minecraft is a popular game from the development studio Mojang and is written in Java.",\
    "Meerkat was a video-sharing service where users could stream their content. In October 2016, Meerkat was shut down and has been replaced by Houseparty. Therefore, the application was deprecated and all related traffic will be classified by the Housparty application classification.",\
    "Rovio Gaming is an application used for provisioning of all services related to Rovio Entertainment Oyj, a Finnish video game developer.",\
    "Angry Birds is an action-based game focused on multi-colored birds that try to save their eggs from green-colored pigs.",\
    "Citrix XenApp is a service which provides access to virtual applications from any device.",\
    "Reddit is an online bulletin board system where members can submit content.",\
    "League of Legends is a multiplayer online battle arena game.",\
    "BattleNet is an online gaming platform by Activision Blizzard. All traffic that is produced by the games that are accessed through this platform, like Hearthstone or Overwatch, will be classified as BattleNet.",\
    "PayPal is a worldwide inline payment system.",\
    "Plants vs Zombies 2 is a sequel to the game Plants vs Zombies released by PopCap Games.",\
    "Halfbrick Studios is a developer for video games. The most popular game provided by Halfbrick Studios is Fruit Ninja.",\
    "GitHub is a Web-based git repository hosting service.",\
    "HamiVideo is an application for streaming chinese online videos.",\
    "Slacker, also called Slacker Radio, was renamed into LiveXLive, an online music streaming service.",\
    "Niconico is a popular video sharing website in Japan managed by Dwango.",\
    "AliPay is an online payment platform launched in China.",\
    "Ustream was a video streaming service based in San Francisco. In April 2018 it changed its name to IBM Cloud Video. Therefore, all related traffic will be handled by the IBM Cloud Video application.",\
    "Zalo is a Vietnamese mobile messaging app.",\
    "NTT Docomo is a mobile phone operator in Japan. This classification covers the website and webstore of NTT Docomo.",\
    "Softbank Onlineshop is the online shop provided by Softbank Services.",\
    "Guild Wars 2 is a popular worldwide MMORPG and the sequel to the original Guildwars.",\
    "Trend Micro Worry-Free is an security solution of Trendmicro for middle-class companies.",\
    "Wikipedia is an online encyclopedia.",\
    "Uber is an online service for personal transportation.",\
    "Yandex is an application used for provisioning of all services related to Yandex, a search engine mostly used in Russia.",\
    "VK is a popular social platform in Russia.",\
    "Odnoklassniki is a Russian social networking platform.",\
    "CrashPlan was an American backup software solution. The application was retired as a supported version in May 2018. Therefore, the application was deprecated.",\
    "Alibaba is the largest B2B marketplace in China and one of the largest worldwide. The website and application allow users to buy directly from manufacturers for relatively low prices.",\
    "AliExpress is a B2C online retail service based in China that allows customers worldwide to purchase items from Chinese companies.",\
    "TMall is an online shopping platform for brands and retailers in China.",\
    "EA Origin is an online gaming, digital distribution and digital rights management platform.",\
    "Norton Security, also known as Norton 360, is an antivirus and security solution.",\
    "Tibia is an online role playing game developed by CipSoft.",\
    "Pochta Rossii is the official Russian Post application. It exists both as a web application and mobile applications for various operating systems.",\
    "After School was an application for communication about schools. While the service has not been formally discontinued, the website is offline and the applications are no longer available. Therefore, the application was deprecated.",\
    "DriveLock is a disk encryption and application control solution.",\
    "QIP-RU was a Russian Internet portal offering various kinds of news, games, and e-mail services. The website is no longer reachable and was therefore deprecated",\
    "musical-ly was a music video sharing application. In August 2018, TikTok merged with musical-ly, which ended the service. Therefore, all musical-ly related traffic will be classified as TikTok.",\
    "EA Games is an application used for provisioning of all services related to EA Games, a collection of mobile and sport games for all platforms.",\
    "iTunesU was a virtual classroom and grading application. It was discontinued by the provider starting 1 January 2022 and was therefore deprecated.",\
    "PopCap is an application used for provisioning of all services related to PopCap Games, Inc., an American video game developer.",\
    "SurveyMonkey is a cloud based online survey service.",\
    "New Relic is a web and mobile application monitoring software as a service.",\
    "Huddle is a privately held cloud-based collaboration software company.",\
    "ZenDesk is a software company providing a cloud-based customer service platform.",\
    "Removed. Reason: The name of the application was not fitting. Instead the application Microsoft Outlook should be used.",\
    "ChatCube (formerly Goober) was a VoIP instant messenger. It was deprecated because it disappeared from all stores and was not updated anymore since 2015.",\
    "DeNA is a provider of mobile portal and e-commerce websites based in Japan.",\
    "BackBlaze is an online backup tool that allow users to back up their data to an offsite data center.",\
    "DocuSign provides electronic signature technology and Digital Transaction Management services.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application AnchorFree Services should be used.",\
    "Good Technology made products for managing and securing mobile devices in a business environment. In 2017 all Good Technology products were rebranded under the BlackBerry Software brand. Therefore, the application was deprecated and all related traffic will be handled by BlackBerry Work.",\
    "Anghami is a legal music streaming platform and digital distribution company in the MENA region.",\
    "Path of Exile is a free to play online action role-playing game.",\
    "Slack is a cloud-based team collaboration tool.",\
    "Pokemon Go is a free-to-play augmented reality mobile game.",\
    "Niantic Labs is an application used for provisioning of all services related to Niantic, Inc., an American software development company focused on augmented reality mobile games.",\
    "2Gis is a Russian geolocation platform.",\
    "Rutube is a Russian video streaming portal.",\
    "2channel was a Japanese textboard. In 2017 2channel began redirecting to 5channel due to domain and trademark conflicts with the original owner. Therefore the application was deprecated and all related traffic will be handled by 5channel.",\
    "Prisma is an application that transforms photos into works of art using the styles of famous artists.",\
    "Removed. Reason: The name of the application was not fitting. Instead the application Clash Royale should be used.",\
    "Upptalk (formerly known as Yuilop) was a messenger application for mobile phones. The company behind Upptalk entered insolvency procedures in 2014 and the service was discontinued in 2017. Therefore it was deprecated.",\
    "Amazon Prime Video is an Internet video on demand service.",\
    "Clash Royale is a real-time multiplayer game in a fantasy setting.",\
    "Google Shared Services is an application used for provision of all services related to Google, an American multinational technology company focused on Internet-related services and products.",\
    "Signal is an encrypted instant messaging and voice calling application.",\
    "KanKan was a video-on-demand service published by Xunlei. It was sold in 2015 to Beijing Nesound International Media Corp., Ltd., removed from the stores and was therefore deprecated.",\
    "The application share-online is used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application ultrashare was used for web hosting, webspace and webstore of the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "The application yunfile was used for file hosting and sharing. It was deprecated because the website is no longer reachable.",\
    "The application eDisk is used for online storage for back up, sharing and search for pictures, videos, music and other files.",\
    "The application 123upload was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "Azar is an app for random video conversations with other users.",\
    "Sky Go is an online television service from Sky for watching live and on demand video content.",\
    "iO was an instant messaging application for making audio and video calls, sending messages, pictures and videos. It was provided by the Swisscom AG and was discontinued in August 2017. Therefore, the application was deprecated.",\
    "Mail-Ru is a Russian web portal, which provides e-mail, social network, and searching services.",\
    "iCrypt was a messaging app which allowed to send and receive text and multimedia messages securely. iCrypt was discontinued, therefore it was deprecated.",\
    "AIM (AOL Instant Messenger) was an instant messaging program, which used (among other things) the protocol Oscar. It was removed from the stores in December 2017. Since AOL is a subsidiary of Verizon Communications, all AIM-related traffic will be classified as Verizon Media Services.",\
    "Kinopoisk is a Russian website and application that provides information and content about movies as well as trailers and options to watch on-demand movies and shows.",\
    "Ivi-Ru is a Russian online video streaming service, previously known as digitalaccess-Ru (LTD. Digital Access).",\
    "Tinder is a location-based social app for dating and finding friends.",\
    "AnyDesk is an application that allows remote access between computers with different operating systems.",\
    "ICQ is an popular instant messaging application that also allows to make calls and send stickers.",\
    "Tenor is an application that provides a wide variety of GIFs. Tenor also provides GIFs for other apps and websites.",\
    "Google Maps is a navigation and map service provided by Google.",\
    "Google Earth is a virtual globe application provided by Google.",\
    "Channel One is a russian television channel.",\
    "Giphy is a provider for picture- and gif-content for multiple messengers.",\
    "VPN-X is a cross-platform P2P/SSL/TLS software VPN solution developed by birdssoft.",\
    "Imgur is an online image hosting and sharing platform",\
    "Microsoft OneDrive (previously known as Skydrive) is an online file storage service.",\
    "Nintendo Network is an application used for provisioning of all services related to Nintendo. It provides online functionality for the Nintendo customers, gaming consoles and related services.",\
    "Amazon Chime is a call service and communication software for IM, VoIP and online meetings provided by Amazon.",\
    "iFlix is a video-on-demand streaming platform for browser and mobile apps.",\
    "PPFilm is a chinese video and streaming-portal.",\
    "Apple Services is an application used for provisioning of all services related to Apple Inc., an American technology company famous for its computer software and consumer electronics.",\
    "Microsoft Services is an application used for provisioning of all services related to Microsoft, an American technology company focused on computer software, consumer electronics and personal computers.",\
    "IcFlix is a popular arabian video streaming website.",\
    "LogMeIn is a remote connectivity and access service.",\
    "Psiphon is an open-source tool that uses a combination of secure communication and obfuscation technologies (VPN, SSH, and HTTP Proxy). It is specifically designed for Internet censorship circumvention.",\
    "Waze is a GPS navigation software, which actively uses crowd-sourcing techniques to estimate the fastest route.",\
    "Google FCM (Firebase Cloud Messaging) is a notification platform provided by Google. It is also known as GCM (Google Cloud Messaging).",\
    "APNS (Apple Push Notification service) is a widely used notification platform.",\
    "Digicel PlayGo is an online streaming service.",\
    "Shazam is a media identifying service application for music, movies, advertising and television shows.",\
    "Digicel Music was an online music download and streaming service. It was decommissioned on July 29, 2022. Therefore, the application was deprecated.",\
    "Digicel TopUp (also known as Digicel International) is a payment recharge service.",\
    "TuneIn Radio is an online streaming service for music, radio, podcasts and audio books.",\
    "Dubsmash is a social app for posting videos in combination with sound in a selfie like manner. It was shut down in February 2022, therefore the application was deprecated.",\
    "Synology provides network attached storage drives with the ability to synchronize with several online clouds.",\
    "My Digicel is an application for managing a Digicel account.",\
    "Google Allo was an instant messaging service provided by Google. It was suspended in March 2019. Therefore, the application was deprecated. All related traffic is handled within the application Google Duo as well as other generic Google-related services like Google APIs, Google Shared Services and Google Cloud.",\
    "The application rusfolder was used for traffic from the hoster with the same name. It was deprecated because the website is no longer reachable.",\
    "Badoo is a dating focused social network.",\
    "AirBnB provides an online marktplace for hospitality services.",\
    "Hightail is a cloud service to send/receive and digitally sign files.",\
    "Easy Taxi is an E-hailing application for booking and tracking taxis in real-time, mostly available in Latin America and Europe.",\
    "TomTom is a brand of various products and services related to traffic, navigation, and mapping.",\
    "Tripadvisor is a touristic service for planning vacation trips.",\
    "ESPN is a sports network containing different related apps for news and media.",\
    "DAZN is a dedicated live sports streaming service.",\
    "Mobile Strike is a mobile game published by Machine Zone.",\
    "King of Avalon is a mobile online multiplayer strategy game.",\
    "Lineage 2 is a popular medieval fantasy MMORPG.",\
    "Lyft is a mobile application for car transportation in the United States.",\
    "Trulia is an online residential real estate site for home buyers, sellers, renters and real estate professionals in the United States.",\
    "Akamai Cloud is a CDN and cloud service provided by Akamai Technologies.",\
    "Limelight Cloud was a CDN and cloud service provided by Limelight Networks. In June 2022, Limelight Networks acquired Edgecast and rebranded itself to Edgio. Therefore, all Limelight Cloud related traffic will be classified as Edgio.",\
    "Nickelodeon Play is a streaming app for kids.",\
    "Here (WeGo) is an app-provider for map and navigation services.",\
    "SimCity BuildIt is a mobile game of the popular SimCity series.",\
    "StarWars Galaxy of Heroes is a free mobile game.",\
    "Fastly is a CDN and cloud service providing secure online storage and delivery service.",\
    "SAP is a enterprise software to manage business operations and customer relations. The classification also covers traffic of SAP SuccessFactors, a cloud-based Software as a Service.",\
    "World of Kung Fu is a multiplayer online role-playing game.",\
    "Cloudflare is a CDN and cloud service providing secure online storage and delivery service.",\
    "Life Church is an application that allows users to watch or listen to preachings and video messages from pastors, to request prayers and submit questions about Christian topics.",\
    "Vevo was a delivery service for music videos. It was removed from the stores in May 2018. Therefore, the application was deprecated.",\
    "Grindr is a dating mobile app for gay and bisexual men.",\
    "Apple Maps is a navigation and map service provided by Apple.",\
    "Apple Geolocation is a service used to gather information based on the current location of the user.",\
    "The application uploaded is used for traffic from the hoster with the same name.",\
    "The application easypaste-org is used for traffic from the hoster with the same name.",\
    "Google Duo was a video telephony service. It was merged with Google Meet in August 2022. Therefore, the application was moved and all Google Duo-related traffic will be classified as Google Meet.",\
    "Youku Tudou, the merging of Youku and Tudou, is a popular video streaming website in China.",\
    "The Meetic Group provides a variety of dating services, like neu.de, match.com and more.",\
    "Sarahah was a social network platform for anonymous feedback. It was removed from the stores in December 2021 and was therefore deprecated.",\
    "Vudu is an American content and video delivery service.",\
    "Wish is an online shopping website.",\
    "Auth0 is an account and identification managing service.",\
    "Sling TV s an US-american internet video and television service provider.",\
    "Google APIs is the communication interface with Google Services.",\
    "Rackspace Cloud is a CDN and cloud service providing secure online storage and delivery service.",\
    "Google Calendar is an online calendar service.",\
    "Google Analytics is a traffic analysis service for websites.",\
    "StackPath Cloud is a CDN and cloud service providing secure online storage and delivery service. In 2016, StackPath acquired NetDNA (formerly known as MaxCDN), therefore traffic related to the CDN services of NetDNA will also be classified as StackPath Cloud.",\
    "Houseparty (formerly known as Meerkat) was a group video chat application. It was officially discontinued in October 2021 and was therefore deprecated.",\
    "LearningHub-online is a digital platform for accessing learning materials and tutorials for various topics.",\
    "Google Ads is the service Google uses for its advertisement traffic.",\
    "Fox Sports is a live stream service for sport events.",\
    "Amazon Services is an application used for provision of all services related to Amazon, a multinational technology company.",\
    "Amazon Drive is an cloud application used for photo and video storage.",\
    "Crashlytics is a Google owned company providing application logging or crash reporting of iOS and Android applications.",\
    "Facebook Cloud is a CDN and cloud service providing online storage and content delivery. This classification contains fbcdn and akamai traffic and is mostly used for static content like photos or videos.",\
    "360 Total Security is an anti virus and system cleaning software.",\
    "Turner is an application used for provisioning of all services related to the Turner Broadcasting System, a division within the WarnerMedia Inc.",\
    "Loop Trade Classifieds was a small e-commercial application with a focus on the caribbean region. The backend services of the application do not serve content anymore. Therefore it was deprecated.",\
    "Loop Caribbean Local News is a content, radio and news feed application with a focus on the caribbean region.",\
    "Loop Pacific Local News is a content, radio and news feed application with a focus on the pacific region.",\
    "The Unity classification covers all traffic related to Unity Technologies, a video game software development company, most famous for developing the gaming engine Unity, primarily used to develop video games and simulations. Applifier, a cross-promotional network of social game publishers, was also acquired by Unity.",\
    "FunPlus is an application used for provisioning of all services related to FunPlus (also known as DianDian or Century Games), a chinese mobile game development company.",\
    "Mapbox is a custom online map provider for websites and applications.",\
    "Cedexis is an Application Delivery Platform (ADP) and mostly used in conjunction with other CDN services.",\
    "Zenmate VPN is a VPN service provided by the German company Zenguard GmbH.",\
    "Smule is a provider of social apps connected to music. Detected apps include Magic Piano, Sing, Auto Rap, Guitar, Ocarina, and Songify.",\
    "Private Internet Access is a VPN service with servers all over the world.",\
    "Symantec was an application used for provisioning of all services related to the Symantec Corporation, an American software company. In November 2019, Broadcom Inc. acquired the Enterprise Security software division of Symantec. The private customer business of the Norton brand and LifeLock remained with Symantec Corporation, which then renamed itself into NortonLifeLock Inc. Therefore the application was deprecated and all related traffic will be handled by NortonLifeLock Services.",\
    "Maxdome was a german video-on-demand portal which acquired the MyVideo online video portal. It was acquired by Joyn in 2020. Therefore, the application was deprecated and all related traffic will be handled by Joyn.",\
    "IPVanish is a VPN service provider located in the USA.",\
    "TunnelBear is a VPN service provided by TunnelBear Inc.",\
    "Google App Engine is a platform as a service system to create and manage your web application and related cloud storage.",\
    "Wire Messenger is a messenger app of Wire Swiss GmbH.",\
    "Epic Browser Proxy is an integrated proxy service of the Epic Privacy Browser",\
    "Zero VPN was a dedicated VPN service for Android. The application did not receive an update since 2018 and the backend services are not reachable anymore. Therefore, it was deprecated.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application Symantec VPN Services should be used.",\
    "Tiger VPN is a VPN service founded in Slovakia.",\
    "Epic Games is an application used for provisioning of all services related to Epic Games, a popular video gaming and software development company.",\
    "Avira Phantom VPN is a VPN service provided by Avira.",\
    "Turbo VPN is a VPN service with thousands of global servers.",\
    "Iqiyi, formerly known as PPStream, is a Chinese p2p video streaming network providing a variety of channels.",\
    "Removed. Reason: The name of the application was not fitting. Instead the application Snap Master VPN should be used.",\
    "PPTV is a Chinese video streaming website and application. It was formerly named PPLive.",\
    "Snap VPN is a free VPN service specific to Android.",\
    "TokBox is an interactive messaging and screen sharing service. It was acquired by vonage in August 2018.",\
    "VPN Unlimited is a VPN service for mobile devices.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application Avast VPN Services should be used.",\
    "Avast Services is an application used for provisioning of all services related to the globally operating Cybersecurity company Avast, which provides Antivirus and VPN services.",\
    "Nord VPN is a VPN service provided by the company Tefincom, located in Panama.",\
    "Proton VPN is a security focused FREE VPN service, developed by CERN and MIT scientists.",\
    "SaferVPN is a VPN service, developed by Perimeter 81.",\
    "Hide.me VPN is a VPN service provided by a company located in Malaysia.",\
    "ExpressVPN is a VPN service with servers in nearly 100 different countries.",\
    "TunnelGuru is a company providing multiple VPN and tunneling services. Related mobile applications are HammerVPN, TroidVPN and Webtunnel.",\
    "Youtube Music is a dedicated youtube audio streaming service.",\
    "Windscribe VPN is a free VPN application.",\
    "IVPN is a VPN service provided by Privatus Limited. They are an organizational member of the EFF.",\
    "Discord is a freeware chat and VoIP application designed for gaming communities.",\
    "FireVPN is a free VPN service provided by the company FireVPN.",\
    "BOTIM is an instant messaging, video and voice call software, developed by Algento, an American technology company.",\
    "Hi VPN is a free VPN service provided by Hi Security Lab.",\
    "VPN Proxy Master is a mobile VPN service.",\
    "ZoogVPN is a VPN service legally registered in Greece.",\
    "Ivacy VPN is a VPN service provider based in Hong Kong.",\
    "PureVPN is a commercial VPN service based in Hong Kong and operated by GZ Systems.",\
    "Mullvad VPN is a VPN service that helps keeping the users online activity, identity and location private.",\
    "Freedome VPN is a VPN service provided by F-Secure.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application Sapling Growth VPN Services should be used.",\
    "StrongVPN is a VPN application established in 1995, and therefore one of the oldest names in the VPN industry.",\
    "Buffered VPN was a Hungary-based VPN provider that maintained a small network of about 40 servers with unlimited bandwidth as standard. Buffered VPN was taken over by Encrypt.me in 2019. Therefore, the application was deprecated and all related traffic will be handled by the Encrypt.me application classification.",\
    "Wargaming.net is an application used for provisioning of all services related to Wargaming Group Limited, a Belarusian video game company and provider of free-to-play online action games.",\
    "Yandex Music is a Russian music streaming service.",\
    "VPN Area is a small, Bulgarian based company that provides VPN applications for mobile devices.",\
    "Steganos Online Shield VPN is a VPN application available for mobile devices.",\
    "SuperVPN is a VPN service developed by SuperSoftTech.",\
    "Steganos Services is an application used for provisioning of all services related to the german based Cybersecurity company Steganos, which provides password-manager, privacy suite, online-banking and VPN services.",\
    "FrootVPN is a VPN service with several servers all over the world, provided by the company FrootVPN located in Sweden.",\
    "BlackBerry Messenger, also known as BBM, was an encrypted instant messaging, voice and video calling application. BBM Consumer for Android and iOS was shut down in May 2019. Therefore, the application was deprecated and related traffic will be handled by the BlackBerry Services application classification.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application Inventive VPN Services should be used.",\
    "CME is an instant messenger developed by Technology Services LLC. It provides group chat, voice and video calls.",\
    "Yandex Navigator is a Russian online maps service application.",\
    "Yandex Disk is a Russian cloud application used for file storage and sharing.",\
    "Yandex Mail is a Russian mail service that is able to access all inboxes in one application.",\
    "Tiktok is a music video sharing application which replaced the app musical.ly.",\
    "Yandex Maps is a Russian map service similar to Google Maps.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application AnchorFree Services should be used.",\
    "Gaana is an Indian music-on-demand streaming service.",\
    "xCum-com is a streaming website for pornographic content.",\
    "Life-Ru is a Russian news application and website.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application AnchorFree Services should be used.",\
    "Speed VPN MobileAppDev was a VPN service developed by Vendor MobileAppDev. It was deprecated because the backend service are not reachable anymore.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application Actmobile Services should be used.",\
    "YeeCall is a free social networking application for video calls provided by Yeecall Network Limited.",\
    "Shellfire VPN is a VPN service from the german based company Shellfire Gattung u. Behr GbR.",\
    "IbVPN was a free VPN application provided by Amplusnet SRL. It was discontinued in March 2020. Therefore it was deprecated.",\
    "Ren-Tv is a Russian channel provider.",\
    "Hola VPN is a VPN service through a peer-to-peer network. It also uses peer-to-peer caching.",\
    "Hola Services provides several products based on sharing the idle resources of its users.",\
    "CDNvideo is a Russian content delivery network.",\
    "Atom VPN was a VPN service provided by Atomvpn.com. Since July 2022, backend service have not been reachable. Therefore it was deprecated.",\
    "ArkVPN is a VPN service provided by CloudArk.net.",\
    "Fortnite is an online survival video game.",\
    "Amaze VPN is a free VPN service for Android.",\
    "Vungle Inc. provides in-app video advertising technology.",\
    "Vivox is a VoIP provider for video games and online communities.",\
    "Megogo is a Russian online movie and television provider.",\
    "Finch VPN is a free VPN service provided by Finch Technology Enterprise.",\
    "Fast Secure VPN is a free VPN service for Android, provided by BCC Network.",\
    "TVPlayer is a streaming service provided by TVPlayer Limited.",\
    "World Of Tanks is an online multiplayer game that is built upon a freemium model.",\
    "VPN Easy was a free VPN application provided by ZPN APPS. It was deprecated because the backend services are not reachable anymore.",\
    "ZPN VPN was a free VPN application provided by ZPN APPS. Since March 2021, the backend services have not been reachable. Therefore it was deprecated.",\
    "AppVPN is a VPN application provided by Wcomes Technologies and with servers located in the USA, UK, Germany, Japan and Singapore.",\
    "X-VPN is a VPN service provided by Free Connected Limited.",\
    "DotVPN is a free VPN service provided by Smart Security Ltd.",\
    "Encrypt.me is a free VPN service provided by Cloak Holdings LLC.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications that use the same core library, instead the application Novatech Core should be used.",\
    "DroidVPN is a free VPN service provided by DroidVPN Inc.",\
    "Browsec VPN is a free VPN application provided by Browsec, LLC.",\
    "Enmasse Games was a game developer specializing in online games like Breach, Tera, Kritika Online and Closers. On August 17, 2020, En Masse announced that it had begun ceasing operations. Therefore it was deprecated",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application AnchorFree Services should be used.",\
    "Apex Legends is a free-to-play battle royale video game first released in 2019 and developed by Respawn Entertainment.",\
    "VPN 365 is a free VPN service provided by Fans up entertainment limited.",\
    "CounterStrikeGO is a free to play first-person shooter initially released in 2012.",\
    "VPN Tor Browser is a free VPN service developed by Bending Spoons.",\
    "SuperVPN by Cheng is a VPN application developed by Cheng Cheng.",\
    "Act VPN is a free VPN application.",\
    "Brightcove is a CDN service that provides the possibility to integrate videos into other cloud services.",\
    "Sportbox-Ru is a Russian sport information website.",\
    "Just Proxy VPN is a free VPN application for Android.",\
    "VeePN is an app developing company with VPN focus, also known for their proprietary protocol SmartVeePN.",\
    "Ookla is an application used for provisioning of all services related to Ookla, an internet testing, data and analysis company based in Seattle, WA.",\
    "Eagle VPN is a free VPN service provided by Eagle VPN proxy.",\
    "Bria Mobile is a VoIP/SIP Softphone and message system for business use case.",\
    "VPN Gate is a public VPN relay Server system. Multiple available VPN services download and use this dedicated service.",\
    "AnchorFree Services is an application used for provisioning of all services related to AnchorFree GmbH, a VPN app developer that has been active since 2005.",\
    "Bria Teams is a team communication system for business use cases.",\
    "Shadowsocks is the basic application for all Shadowsocks protocol traffic.",\
    "Asphalt9 is a mobile car racing game.",\
    "Gameloft Services is an application used for provisioning of all services related to Gameloft, a French video game publisher.",\
    "Actmobile Services is an application used for provisioning of all services related to Actmobile, an app developing company with VPN focus.",\
    "Novatech Core is a core library that is used by different VPN applications.",\
    "SecureVPN is a VPN service developed by SecureVPN.com Inc.",\
    "AppsFlyer is a mobile marketing analytics and attribution platform.",\
    "Adjust is a mobile marketing analytics platform.",\
    "GenieVPN is a free VPN application provided by Hoho Technology Limited.",\
    "TunVPN was a free VPN service developed by ConnectEasy Technology LTD. It was rebranded as RocketVPN in July 20, 2020. Therefore, this application was deprecated and all related traffic will be handled by the RocketVPN application classification.",\
    "Helpshift is a mobile messaging service platform.",\
    "FastVPN Master is a VPN service developed by Super VPN Master for iPhone Turbo Co, .Ltd.",\
    "VPN Unlimited Proxy is a free VPN application developed by Laofu Wang.",\
    "HighSpeedVPN is a VPN service developed by KSystems from Ukraine.",\
    "VPN Master by Andro Team is a VPN service developed by Andro Team in the USA.",\
    "Mixer was an interactive video game live steaming platform. The service was shut down in July 2020. Therefore the application was deprecated.",\
    "Tencent Games is an application used for provisioning of all services related to Tencent Games, a service of Tencent Holding Limited, which publishes video games.",\
    "Open Super Hotspot VPN was a free VPN service for iOS developed by Tech Electrical Control Limited.Open Super Hotspot VPN was discontinued, therefore it was deprecated.",\
    "Removed. Reason: The name of the application was not fitting. Because it is not distinguishable from other applications by the same provider, instead the application Sapling Growth VPN Services should be used.",\
    "Douyu is a chinese live streaming platform which offers services like video sharing and live broadcast.",\
    "Ingress Prime is a location-based free-to-play augmented-reality mobile game.",\
    "Electronic Arts is an application used for provisioning of all services related to Electronic Arts Inc. (EA), a world wide organization for publishing and developing a wide range of games and services around games.",\
    "Radioplayer is a german radio service with a lot of radio stations.",\
    "MoPub provides advertising technology services for publishers of mobile applications.",\
    "PUBG Mobile (Player Unknown Battlegrounds Mobile) is a mobile action game.",\
    "7TV was a free-TV streaming application for ProSiebenSat.1 television channels. It was renamed into Joyn in 2019. Therefore, the application was deprecated and all related traffic will be handled within the Joyn application classification.",\
    "Spiegel TV is a Web-TV service related to the German weekly magazin Der Spiegel.",\
    "Spiegel Services is an application used for provisioning of all services related to Der Spiegel, a German weekly news magazine.",\
    "Eurosport Player was a live sport coverage application. In January 2021, it was replaced by Discovery Plus. Therefore, all related traffic will be classified as Discovery Plus.",\
    "Forge of Empires is a turn-based strategy game.",\
    "InnoGames is an application used for provisioning of all services related to InnoGames GmbH, a video game developer and publisher located in Germany.",\
    "Receiver Radio is an internet radio streaming application.",\
    "Goodgame Studios is an application used for provisioning of all services related to Goodgame Studios, a big online games developer and publisher located in Germany.",\
    "Wizards Unite was a mobile Harry Potter game. It was discontinued and removed from the Google Play Store and Apple App Store on 06 December 2021. Therefore the application was deprecated.",\
    "Radio-De is a german radio service with a lot of radio stations. Their most important brands are RadioMe, radio.de and radio.net.",\
    "BBC News is a breaking news broadcasting service and video/radio streaming platform.",\
    "BBC Services is an application used for provisioning of all services related to the British Broadcasting Corporation, the largest broadcaster in the world.",\
    "Unreal Engine is a popular gaming engine that supports a wide range of platforms.",\
    "Zoom is a video conferencing application developed by Zoom Video Communications, Inc.",\
    "Adobe Services is an application used for provision of all services related to Adobe, an American computer software company.",\
    "ATT Services is an application used for provisioning of all services related to the American Telephone and Telegraph Company, a multinational entertainment and mass media company.",\
    "Funk is an application to stream contents from the Funk network.",\
    "Nexx TV is a video content delivery network.",\
    "Unblock Sites is a free VPN application developed by ExtraApps.",\
    "Brawl Stars is a free mobile battle game with variety of different game modes.",\
    "Counterpath Services is an application used for provisioning of all services related to the Counterpath Corporation, a Canadian software company.",\
    "Snap Master VPN is a mobile VPN service.",\
    "Joyn is a free-tv streaming application for Pro7 and Sat1 television channels.",\
    "Pro7Sat1 Services is an application used for provision of all services related to ProSiebenSat.1 Media SE, a European mass media company, based in Germany.",\
    "Dead Rivals is a mobile Zombie MMORPG.",\
    "Tapjoy is an advertisement system that has customers all over the world.",\
    "IGG is an application used for provisioning of all services related to I Got Games Inc., a developer of several mobile games.",\
    "Lords Mobile is a popular strategy MMO game.",\
    "Mudfish is a VPN and Proxy service focused on regional logged games.",\
    "Inventive VPN Services was an application used for provisioning of all services related to Inventive Apps Ltd. The VPN application no longer exists, therefore it was deprecated.",\
    "Mixpanel is an application monetization and user behavior analysis suite.",\
    "Adcolony is a mobile advertising and marketing platform.",\
    "SOMA was a free instant messaging application, that also allowed voice and video calls. While the service has not been formally discontinued, the website is offline and the applications are no longer available. Therefore, the application was deprecated.",\
    "Big Farm is a free mobile farming game.",\
    "FaceApp is a free portrait editing application published by FaceApp Inc.",\
    "Asphalt8 is a mobile car racing game.",\
    "Moat is a company that provides analytics and advertising technology services for publishers of mobile applications.",\
    "AdSafeProtected is an potentially unwanted adware application.",\
    "AdForm is an integrated advertising platform.",\
    "FreeWheel is a provider of TV and video advertisement technology for publishers, advertisers, and media buyers.",\
    "SpotX is an online video advertisement platform.",\
    "IronSource is a provider of solutions for app monetization and distribution. It is one of the biggest ad networks for casual games, especially known for its in-game advertising and mobile game advertising strategies like rewarded video ads.",\
    "TrafficJunky is an integrated advertising platform.",\
    "TrafficFactory is an integrated advertising platform.",\
    "Hatch Cloud Gaming is a gaming service with more then 100 casual games.",\
    "AppLovin is a mobile technology company with a focus on mobile advertising and analytics platforms as well as casual mobile games.",\
    "DeltaDNA develops an analytics and marketing platform for games.",\
    "AdTiming is a global marketing platform.",\
    "Flurry is an American mobile analytics, monetization and advertising company.",\
    "Optimizely is a provider of customer experience optimization software.",\
    "SmartAdServer is a remote advertising system.",\
    "Exactag is the leading provider of marketing attribution in germany.",\
    "ScorecardResearch collects internet web browsing data.",\
    "Conviva is a provider of video optimization and analytics services.",\
    "BAMTech Media is an American internet video provider.",\
    "Bebi is a big advertisement network and provides account management and performance monitoring.",\
    "HBO Services is an application used for provisioning of all services related to Home Box Office, an American television network.",\
    "Heyzap was a mobile ad network. It was acquired by Fyber in 2016 and stopped serving ads in 2020. Therefore, all related traffic is now handled by the Fyber application classification.",\
    "Criteo is an online marketing company.",\
    "Swrve is a marketing and customer engagement platform.",\
    "Chartboost is an in-app monetization and mobile advertising platform.",\
    "Accountkit provided services for customers to quickly register for apps without a password. In March 2020, the company stopped offering the service; therefore, the application was deprecated and all related traffic is now handled by the Meta Services application classification.",\
    "DirecTV is an American pay television provider.",\
    "VPNSecure is a free VPN service by VPNSecure Pty Ltd.",\
    "Liftoff is an online marketing company.",\
    "Level3 Cloud was a Tier1 network with CDN and cloud service provided by Level 3 Communications. In October 2016, CenturyLink (later renamed to Lumen Technologies) acquired Level 3 Communications. Therefore, the application was deprecated and all related traffic will be handled by the Lumen Technologies application classification.",\
    "Disney Services is an application used for provisioning of all services related to Walt Disney Television (formerly known as ABC Group), a subsidiary of Disney Media Networks.",\
    "My5 (formerly known as Demand 5) is a television network providing streams of various shows.",\
    "Source Engine is a gaming engine created by Valve.",\
    "Geocaching is an application for the treasure hunting game with the same name.",\
    "Fox News is a news app most famous in the US.",\
    "Kaltura is a provider of on-demand and live video cloud solutions.",\
    "Verizon Digital Media Services (VDMS) was the video streaming and cdn delivery service of Verizon Media. In September 2021 it was rebranded as Edgecast Networks, which was merged in June 2022 with Limelight Networks into Edgio. Therefore, all Limelight Cloud related traffic will be classified as Edgio.",\
    "Streamcraft was an online streaming platform for gamers. It was shut down in October 2019. Therefore the application was deprecated.",\
    "Fox Services is an application used for provisioning of all services related to Fox, an entertainment service most famous in the US.",\
    "Branch is an advertisement monetization provider.",\
    "Viki (also known as Rakuten Viki) is an American video streaming website and application with a special focus on content from Asian countries, especially Korea and Japan.",\
    "Wordpress is a CMS System for multiple apps and websites. It also provides additional data over network connections. The classification includes also traffic that originates from the WP Engine, a WordPress Digital Experience Platform.",\
    "Gravatar is a service for providing avatars. It is integrated in the WordPress blogging platform.",\
    "Parsely is a marketing and content analytics service.",\
    "WeGamers was a popular communication platform for mobile games. The website and the iOS store link have been taken down. Therefore, the application was deprecated.",\
    "PBS (Public Broadcasting Service) is a public television provider.",\
    "Crackle is an application that allows the user to stream free Hollywood movies, TV series and other shows.",\
    "OpenX is an advertisement and big data provider.",\
    "Rubicon Project was an advertisement monetization and ad collaboration provider. In 2020, Rubicon Project and Telaria merged into Magnite. Therefore, this application was deprecated and all related traffic will be handled by the Magnite application classification.",\
    "Disney Radio is a video and music streaming app provided by the Walt Disney Company.",\
    "Kochava is a mobile app analytics platform.",\
    "ABC (American Broadcasting Company) is an American radio and television provider.",\
    "StreamTheWorld was a streaming technology company, which provided services for radio and tv broadcasters. In June 2010, the company was acquired by Triton Digital. Therefore, the application was deprecated and all related traffic will be handled by Triton Digital.",\
    "Apteligent is a mobile application analytics platform.",\
    "Apple News is a news service provided by Apple.",\
    "HBO Now was a stand alone streaming service for cable or satellite TV. In May 2020, it was succeeded by HBO Max and formally taken off in December 2020. Therefore, the application was deprecated and all related traffic will be handled by HBO Max.",\
    "Segment is a data analytics platform for marketing purposes.",\
    "TUNE, formerly known as HasOffer, is an advertisement monetization provider with a mobile app tracking service.",\
    "Apple TV is a streaming service provided by Apple.",\
    "Apptentive is a advertisement monetization provider.",\
    "Airable is a white label internet-media-service for online radio and audio on demand shows as well as videos and TV shows.",\
    "Pluto TV is a video streaming app and internet television service that provides advertiser-supported video on demand content.",\
    "Disney Channel was a video and music streaming app provided by the Walt Disney Company. The application was discontinued in March 2020, and therefore it was deprecated. All related traffic will be handled by the Disney Services application.",\
    "Discovery Communications was an application used for provisioning of all services related to Discovery, Inc., the publisher of several applications which stream Discovery Channel content. It was acquired by Warner Bros. Discovery, Inc. in April 2022. Therefore, all Discovery Communications related traffic will be classified as Warner Bros Discovery Services.",\
    "Sony Pictures is an application used for provisioning of all services related to Sony Pictures Entertainment Inc., an American company focused on filmed entertainment like movies and television programs.",\
    "NFL Sunday Ticket is a video content delivering network for NFL games.",\
    "Pornhub Network is an application used for provisioning of all services related to the Pornhub Network, a network of websites for sharing pornographic video contents owned by MindGeek.",\
    "PdaNet is an application to share network access without tethering. The FoxFi application has become part of PdaNet.",\
    "MLB is an application used for provisioning of all services related to Major League Baseball, an American baseball organization and publisher of different streaming applications for live Baseball content.",\
    "MLB At Bat is an application for streaming live Major League Baseball content.",\
    "Tubi is a movie and TV streaming application.",\
    "Watch TBS is a streaming application for different shows, live sports and movies.",\
    "Disney Plus is an application provided by Walt Disney Company, which allows the user to stream disney movies.",\
    "OpenSignal is a company focused on mobile analytics for network measurement and performance quantification.",\
    "Showtime is an application used for provisioning of all services related to Showtime Networks Inc., an American television network and streaming service.",\
    "Quantum Metric is an advertisement monetization provider.",\
    "mParticle is is a CDP (Customer Data Platform) that unifies customer data and simplifies partner integrations.",\
    "Showtime Anytime is an application provided by Showtime Networks, which allows the user to stream free movies, TV series and other shows.",\
    "Canal Plus is an application provided by the French television group with the same name.",\
    "Audible is a company for selling digital audio books.",\
    "Tealium is an American company that provides a tag management system and marketing software.",\
    "Nuance is an company focused on Enterprise Business. The portfolio ranges from customer service to speech recognition.",\
    "CBS Services is an application used for provisioning of all services related to Columbia Broadcasting System, a broadcasting television service in the US.",\
    "NBC Sports is a personalized service that delivers sports related news.",\
    "Eurosport Services is an application used for provisioning of all services related to Eurosport, a televison sports network.",\
    "Endeavor Streaming is a company providing digital video technology for different devices. It is best known for the NeuLion Digital Platform, an end-to-end video streaming platform provided by them as well as its subsidiary Dice Technology Ltd., a content deliverer of video and sports data.",\
    "ToTok is a free messaging application.",\
    "Fyber is an advertising technology company that helps developers earn money from advertisements.",\
    "Univision Now is an American streaming and video-on-demand service for Spanish-language content.",\
    "Taboola is an advertising company that recommends sponsored web content.",\
    "adRise was an internet TV ad platform that distributes and monetizes TV shows by partnering with content owners. The company rebranded as Tubi, Inc. Therefore, the application was deprecated and all related traffic will be handled by Tubi.",\
    "Sky Services is an application used for provisioning of all services related to Sky, a European pay tv provider.",\
    "UserVoice is a service tool to collect, analyze and close feedback from customers.",\
    "BIGO Services is an application used for provisioning of all services related to BIGO, a Singapore-based company focused on video broadcasting and voice over IP related products and services.",\
    "BitTorrent Now is a music and video streaming platform.",\
    "Mario Kart Tour is a game from the publisher Nintendo Co which is available for iOS and Android.",\
    "Turkcell Services is an application used for provisioning of all services related to Turkcell, the leading mobile phone operator of Turkey.",\
    "Sky Sports is a service that delivers sports related news.",\
    "BiP Messenger is an instant messaging application.",\
    "Swoogo is an event marketing web application.",\
    "NBC Services is an application used for provisioning of all services related to the National Broadcasting Company, an American radio and television network.",\
    "Etisalat Services is an application used for provisioning of all services related to Etisalat, a multinational Emirati based telecommunications services provider.",\
    "Switch TV is an online television service from Etisalat for watching live and on demand video content.",\
    "Microsoft Teams offers a shared workspace for team communication and collaboration.",\
    "Flashtalking is a advertisement monetization provider.",\
    "Youtube Go was a dedicated youtube service to download, watch and share videos. It was discontinued in August 2022. Therefore, the application was deprecated.",\
    "Symantec VPN Services was an application that covered the traffic of different VPN Services provided by the Symantec Corporation. Since Symantec renamed itself to NortonLifeLock Inc. in November 2019, the application was deprecated and all related traffic will be handled by NortonLifeLock VPN Services.",\
    "Nielsen Services is an application used for provisioning of all services related to The Nielsen Company, especially Nielsen Online (also known as imrworldwide.com), a content analytics and measurement service.",\
    "Fox Business is an application which provides business news stories and videos.",\
    "PrivateVPN is a commercial VPN service developed by Privat Kommunikation Sverige AB.",\
    "Surfshark VPN is a VPN service provided by Surfshark Ltd.",\
    "Yospace is a digital video distribution company.",\
    "Sapling Growth VPN Services is an application used for provisioning of all services related to Sapling Growth Tech Co., a company that provides several mobile VPN applications.",\
    "NFL Services is an application used for provisioning of all services related to the National Football League and the company NFL Enterprises LLC, which operates and promotes professional football athletic clubs and events.",\
    "Xunlei Services is an application used for provisioning of all services related to Xunlei, a chinese technology company and service provider, mostly known for the Thunder P2P application.",\
    "The NBC App is a mobile streaming app for TV shows.",\
    "Fizy by Turkcell is a mobile music streaming application provided by Turkcell.",\
    "Cloudinary is a content delivery network for images and videos.",\
    "Tencent Services is an application used for provisioning of all services related to Tencent Holding Limited, a Chinese company focused on Internet-related services and technology.",\
    "UltraVPN is a VPN application developed by Wirelab IO.",\
    "Pladform was a Russian digital distribution system for licensed video content. It was closed in 2021 and was therefore deprecated.",\
    "Starz is a mobile movie and TV show streaming application.",\
    "WeChat Work, formerly WeChat Enterprise Accounts, is a business communication and office collaboration tool developed by Tencent.",\
    "Resilio Sync is a peer-to-peer data synchronization and sharing solution.",\
    "ImgIX is a picture content provider.",\
    "Amazon Prime Now was an Amazon application used by Amazon Prime members for fast delivery of products. It was discontinued by the provider starting 1 January 2021 and was therefore deprecated.",\
    "Kakao Services is an application used for provisioning of all services related to Kakao, a company providing communication products and services.",\
    "Solvvy is a platform helping companies to improve customer experience through automation and machine learning.",\
    "JW Player is an online video advertisement service.",\
    "Stackla is a visual content engine helping marketers to discover, manage and display content across their platforms.",\
    "Mux is a video technology company which offers real-time performance monitoring and analytics for video streaming.",\
    "Freshdesk is a helpdesk software from Freshworks.",\
    "Freshworks is a cloud based customer engagement company.",\
    "Amplitude is a real time mobile analytics platform.",\
    "AppNexus was a mobile advertising and marketing platform. It was renamed into Xandr in June 2018. Therefore, all AppNexus related traffic will be classified as Xandr.",\
    "Under Armour Services is an application used for provisioning of all services related to Under Armour Inc., an American apparel company.",\
    "MapMyFitness is a brand of several fitness tracking applications for mobile devices with social networking features.",\
    "TrustArc is a privacy compliance technology company which provides software and services for privacy management processes.",\
    "Microsoft Excel is a spreadsheet developed by Microsoft and is part of Office 365.",\
    "Microsoft Word is a word processor developed by Microsoft and is part of Office 365.",\
    "Microsoft PowerPoint is a presentation program developed by Microsoft and is part of Office 365.",\
    "Microsoft Outlook is a personal information manager developed by Microsoft and is part of Office 365.",\
    "Snap Services is an application used for provisioning of all services related to Snap Inc., an American social media company.",\
    "BootstrapCDN is a content delivery network for Bootstrap, Font Awesome and Bootswatch.",\
    "KakaoStory is an application used for sharing day-to-day life stories similar to Instagram.",\
    "PlayStation Video was an application used for streaming TV shows and movies from the users video library, which were bought or rented from the PlayStation Store. PlayStation Video was shut down in August 2021 and was therefore deprecated.",\
    "SportsMax is a Caribbean sports channel.",\
    "KakaoTV Live is a live streaming application published by Kakao Corporation.",\
    "Applicaster is a software as a service (SaaS) for media and TV app development.",\
    "Microsoft SharePoint is a document management and storage program developed by Microsoft and is part of Office 365.",\
    "Removed. Reason: The short name had a spelling mistake. The application Cisco Systems Services should be used instead.",\
    "Index Exchange (formerly known as Casale Media) is an online advertising network headquartered in New York and Toronto.",\
    "NBC News is an application provided by NBC which can be used for reading, listening and watching news and live content from all over the world.",\
    "fuboTV is an entertainment application that allows the user to watch TV shows, movies, live news, and sports.",\
    "AppDynamics is an application performance management company and is a subsidiary of Cisco.",\
    "Tour Tracker Grand Tours is an application for professional cycling which combines live reports and race commentary with functionalities like GPS race tracking and interactive route maps.",\
    "Blackboard is an education platform founded by Blackboard Inc.",\
    "Chartbeat is a real-time analytics service for website owners to monitor and optimize their content.",\
    "e-bot7 is an AI based analytical service to automate customer services.",\
    "Microsoft OneNote is an information gathering and multi-user collaboration program developed by Microsoft and is part of Office 365.",\
    "HBO Go is a mobile application for streaming TV and movies.",\
    "MediaMath offers digital advertising media and data management technologies",\
    "Vilynx Inc. develops AI-based tools for media companies like optimized motion video previews and a recommendation and metadata engine.",\
    "WWE is the official app of World Wrestling Entertainment Inc. and offers different kinds of WWE content like videos, news, photos and Wrestler profiles.",\
    "TopBuzz was a free AI-based news aggregator that delivered a personalized news feed. ByteDance shut down the application in 2020, therefore it was deprecated and most of its related traffic will be handled by the BaBe application classification, which was its successor.",\
    "NBC Sports Scores was a personalized service that delivered up-to-date scores, notifications and news. It was discontinued by the provider starting September 2020 and was therefore deprecated.",\
    "Sentry (also known as Functional Software Inc.) provides real-time error tracking for web apps, mobile apps and games.",\
    "YuppTV is a content provider for South Asian films, shows, and live TV.",\
    "PerimeterX is a web security provider based on analytical services.",\
    "NBA is an application for streaming NBA games.",\
    "Playrix is a company which develops several free-to-play mobile games.",\
    "App Bazaar is an alternative Android app store made primarily for Indian users.",\
    "Hotjar is a behavior analytics company that builds research and optimization tools for web businesses.",\
    "Taplytics is a product optimization and customer experience platform with features like A/B testing, push notification management and marketing automation.",\
    "RTVE alacarta is an application for streaming of Spanish TV and National Radio content.",\
    "hayu is an application for streaming reality TV shows on demand.",\
    "Gigya, Inc. is a tech company that offers customer identity management services like managing profiles, preferences and consent settings.",\
    "TNT is an American subscription television network with focus on drama.",\
    "Showmax is a South African video streaming service.",\
    "Atresmedia is an application used for provisioning of all services related to Atresmedia, a Spanish media group, present in television, radio and cinema industries.",\
    "CDNetworks is a large Content Delivery Network founded in 2000 with focus on emerging Internet markets such as India and China.",\
    "Google Classroom is an education platform founded by Google.",\
    "Peer5 is a peer-to-peer Content Delivery Network based on WebRTC.",\
    "Satisfi Labs is a knowledge management platform for conversational search and commerce.",\
    "Ayoba is an encrypted instant messaging application.",\
    "Roblox is an online game creation system that allows users to design their own games and play games created by other users.",\
    "NHL is the official app of the National Hockey League and offers different kinds of Hockey content like videos, news, photos and player profiles.",\
    "Innovative Connecting is an application used for provisioning of all services related to Innovative Connecting, a company which publishes several mobile VPN services.",\
    "Outbrain is an advertising platform that displays boxes with links to the sites' pages among sponsored content.",\
    "Stan is an entertainment application that allows the user to watch TV shows and movies.",\
    "VPN Monster is an Android-specific VPN service provided by Innovative Connecting.",\
    "Mediacorp Services is an application used for provisioning of all services related to Mediacorp, the national public broadcaster of Singapore.",\
    "ATRESplayer is a TV show and movie streaming platform with Spanish language content.",\
    "Airship is a marketing and customer engagement platform.",\
    "DiDi Services is an application used for provisioning of all services related to DiDi, a Chinese company that provides app-based transportation services.",\
    "Mediaset Espana is an application used for provisioning of all services related to Mediaset Espana, a Spanish television network and media production company.",\
    "Mitele is a Spanish video-on-demand and live broadcasting platform.",\
    "meWATCH (formerly known as Toggle) is a streaming application from Singapore which allows the user to stream live and on-demand movies, news, sports, and other content.",\
    "HBO Max was a mobile application for streaming TV and movies. In May 2023, HBO Max was renamed into Max. Therefore, the application was deprecated and all related traffic will be handled by Max.",\
    "WarnerMedia Services was an application used for provisioning of all services related to WarnerMedia, LLC, a multinational media and entertainment company located in New York. It was acquired by Warner Bros. Discovery, Inc. in April 2022. Therefore, all Discovery Communications related traffic will be classified as Warner Bros Discovery Services.",\
    "Nike Services is an application used for provisioning of all services related to Nike Inc., an American apparel and accessories company.",\
    "Accedo is a OTT video streaming service.",\
    "99Taxis is a brazilian E-hailing application for booking and tracking taxis in real-time.",\
    "Charter Communications, Inc., is an American telecommunications and mass media company. Since it acquired Time Warner Cable and Bright House Networks in 2016, it is the third-largest pay TV service in the US.",\
    "Viu is an application for streaming Asian entertainment.",\
    "PCCW is an application used for provisioning of all services related to Pacific Century Cyberworks, a chinese telecommunication and internet services company.",\
    "Alibaba Cloud, also known as Aliyun, is a Chinese provider of cloud services.",\
    "Localytics offers a mobile engagement platform for targeted advertisement.",\
    "Tidal is a music streaming service with focus on high quality audio.",\
    "Spot-Im was a digital platform which added social networking functionality to websites. The service and the company were renamed to OpenWeb in June 2020. Therefore, this application is deprecated and all related traffic will be handled by the OpenWeb application classification.",\
    "Youbora Suite is a service provided by NPAW (Nice People At Work). It provides a composement of independent products, such as analytics data visualization, CDN, ad performance reporting and user session monitoring.",\
    "Google Meet is a video-communication service developed by Google.",\
    "Coinbase is a digital currency wallet and platform where businesses and consumers can do transactions in digital currencies.",\
    "WideOrbit is the leader in premium broadcast technology, especially focused on managing, monetizing and measuring the reach of audio content like podcasts.",\
    "Meetrics provides analytical services and quality measuring in the fields of digital advertising.",\
    "Conductrics is the provider of an optimization platform for marketers and developers.",\
    "LaunchDarkly is a Feature Management Platform for software developers.",\
    "ART19 is a leading company for publishers and advertisers of podcasts, located in California. It provides tools for hosting, monetization, and distribution of podcasts.",\
    "HiveMQ is a MQTT broker and a client to move data to and from IoT devices.",\
    "The Handelsblatt application is a news app provided by one of the leading German-language business newspapers Handelsblatt.",\
    "Podigee is a provider of podcast-hosting and analytics services for German speaking countries.",\
    "The application Cloud-Mail-Ru (formerly known as files-mail-ru) is a cloud storage service owned by the Mail.Ru Group.",\
    "Innovid is a big online advertising and video marketing company.",\
    "Movistar Services is an application used for provisioning of all services related to Movistar, a Spanish telecommunications service provider.",\
    "UCWeb is an application used for provisioning of all services related to the chinese telecommunication and internet services company.",\
    "WizRocket is a SaaS based mobile marketing company which provides mobile app analytics and user engagement products.",\
    "Batch is a mobile customer management system and push notification system provider.",\
    "Vizbee Inc. is a company focused on software solutions for video streaming, located in New York.",\
    "The Wirtschaftswoche application is a news app with a focus on finance and economics.",\
    "Handelsblatt Services is an application used for provisioning of all services related to the Handelsblatt Media Group, a German group of media companies with a focus on finance and economy.",\
    "The IQ Digital media marketing gmbh is one of the leading marketing and sales organizations for German media.",\
    "The Meet Group Services is an application used for provisioning of all services related to The Meet Group, Inc., an American company that owns several mobile social networking applications.",\
    "Movistar Plus is a Spanish video on demand (VOD) provider.",\
    "Wizzee is a mobile network company providing data plan services.",\
    "Intercom provides business related cloud services such as customer relationship management.",\
    "The NetEase application is a Chinese Internet technology company providing online services centered on content, community, communications and commerce.",\
    "Penthera is a company that focuses on optimizing video streams and offers various services related to video streaming.",\
    "Riot Games is an application used for provisioning of all services related to Riot Games, Inc., an American video game developer and organizer of esports events located in California.",\
    "ByteDance is an application used for provisioning of all services related to the Beijing Bytedance Technology Co., Ltd. a Chinese company that owns several mobile applications.",\
    "Appsee was a visual mobile app analytics platform that offered analytics tools for tracking and optimization. The company was acquired by ServiceNow in 2019. Therefore, the application was deprecated and all related traffic is now handled by the ServiceNow application classification.",\
    "Tagged is a social search app.",\
    "Webtoon is an application for reading shared webtoons. In additions to this, users can create and publish their own stories.",\
    "MeetMe is a location-based chat and video application for meeting new people.",\
    "Dynamic Yield is an AI-based personalization and data management platform.",\
    "Elex Technology is an application used for provisioning of all services related to the Elex Technology Co., Ltd., a Chinese gaming company headquartered in Beijing.",\
    "Clash of Kings is a Chinese online multiplayer strategy game.",\
    "The IMDb application is an application based on the Internet Movie Database, an online database which collects all kinds of information related to films, television programs, games and streaming content.",\
    "Billo is an online video platform for user-created content.",\
    "AIHelp is a customer service platform which applies AI and machine learning techniques for an enhanced user experience.",\
    "Crunchyroll is an US American video on demand service provider.",\
    "Shein is an international online fashion shop.",\
    "Avira Services is an application used for provisioning of all services related to Avira Operations GmbH.",\
    "Samsung Services is an application used for provisioning of all services related to Samsung, a South Korean multi-industry company.",\
    "Xigua Video is a short form video streaming platform.",\
    "Toutiao is an chinese news site and service.",\
    "Buzzsprout provides tools for hosting, monetization, and distribution of podcasts.",\
    "Megaphone is a provider of podcast-hosting and advertising services.",\
    "Bugsnag is an analytics technology company which provides services for monitoring and optimization.",\
    "Knotch is an analytics technology company which provides services for content intelligence.",\
    "Wurl is a provider of video streaming and advertising services.",\
    "Libsyn (Liberated Syndication Inc.) provides tools for hosting, monetization, and distribution of podcasts.",\
    "Triton Digital provides tools for hosting, monetization, and distribution of podcasts.",\
    "VNG Corporation (VNG) is a Vietnamese technology company, specialized in digital content and online entertainment, social networking, and e-commerce.",\
    "Baidu Maps is a map service from Baidu. It is similar to Google Maps.",\
    "Moonton, formerly YoungJoy Technology, is a Chinese video game developer and publisher most famous for its Mobile Legends games for smartphones.",\
    "Amagi is a provider of cloud managed broadcast and monetization services.",\
    "Kuaishou is an application used for provisioning of all services related to Beijing Kuaishou Technology Co., Ltd, a Chinese technology company, specialized in digital video content.",\
    "MarkMonitor is an American software company focused on domain management services.",\
    "The Medium application is based on the online publishing platform with the same name. Medium is an example of social journalism and is regarded as a blog host.",\
    "Simplecast is a provider of podcast-hosting and advertising services.",\
    "Walmart is an online shopping platform.",\
    "Grubhub Inc. is an American food ordering and delivery application based in Chicago.",\
    "Kwai is a Chinese video sharing service from Kuaishou.",\
    "Braze is an American software company (formerly known as Appboy Inc.) focused on business related cloud services such as customer relationship management.",\
    "KeepSolid Services is an application used for provisioning of all services related to KeepSolid Inc., an American software company that offers online security and productivity solutions.",\
    "VPN Lite is a free VPN application with a minimalistic interface.",\
    "SpringServe is an online video advertisement service.",\
    "Zynn was a short-form video platform similar to TikTok, that enables users to create and share videos with others. It was discontinued and removed from the Google Play Store on 20 August 2021. Therefore the application was deprecated.",\
    "Workplace is a video-communication and chat service developed by Facebook.",\
    "Yaani is a search engine and web browser.",\
    "The Washington Post application is a news app provided by one of the leading American daily newspapers.",\
    "Curalate was a provider of services which combine social networks and e-commerce. In July 2020, it was acquired by Bazaarvoice. Therefore the application was deprecated and all related traffic will be handled by Bazaarvoice.",\
    "ROMWE is an international online fashion shop.",\
    "Target Signature is an online shopping platform.",\
    "Plex is a mobile streaming app for live TV, movies and shows.",\
    "Amazon Alexa is a virtual intelligent voice-assistant for shopping, playing music or news and control capable home automation devices.",\
    "Bumble is a dating focused social network.",\
    "Weather Channel (official name The Weather Channel or TWC) is a mobile weather application that provides live radar and weather alerts like storm warnings. The Weather Channel application also covers all traffic related to The Weather Company, the owner of TWC as well as Weather Underground.",\
    "Reuters News is a news application provided by Reuters, an international news organization.",\
    "CryptoTrader is an application that helps to track the crypto market by providing real-time charts,news and price alerts.",\
    "Cxense was an AI-based data management platform which provided services to analyze, visualize and monetize business data. It was acquired by Piano in October 2019. Hence, the application was deprecated and all related traffic will be handled by the Piano application classification.",\
    "Club Factory is an online shopping platform from India.",\
    "Bombora is a marketing and engagement platform targeted on business-to-business customers.",\
    "MOOV is a streaming application popular in China.",\
    "collectd is a daemon which collects system and application performance metrics periodically and provides mechanisms to store the data in a variety of ways.",\
    "Oracle Services is an application used for provisioning of all services related to Oracle Corporation, an American multinational computer technology corporation.",\
    "Comcast is an application used for provisioning of all services related to the Comcast Corporation, an American telecommunications conglomerate that supports multiple services.",\
    "Skred Messenger is peer to peer instant messaging application which works without using an email address or phone number.",\
    "Acast is a provider of podcast-hosting for publishers and advertisers.",\
    "The CW is an entertainment application that allows the user to watch shows.",\
    "CW Seed is a streaming application which can be used to watch comedy shows and content from The CW.",\
    "Facebook Lite is an alternative Facebook application that uses less data, loads more quickly and also works under suboptimal network conditions.",\
    "Opera Services is an application used for provisioning of all services related to Opera, a Norwegian software company that specializes in web browsers.",\
    "The The Wall Street Journal application is a news app provided by its corresponding international daily newspaper.",\
    "TextNow is a free texting and calling smartphone application.",\
    "Bloomberg L.P. is a software, media, financial and data company. The website and mobile apps provide global business, stock market and finance news, monitoring of personal portfolios and video and audio content in Global Finance, Business, Economics and Investment.",\
    "Udemy is an online learning platform featuring video courses.",\
    "The The Economist is a news application provided by its corresponding weekly newspaper from London which reports about economical and political news.",\
    "Investing-com Services covers the traffic caused by the financial markets platform with the same name, which provides real-time data, charts, financial tools and analysis across 250 exchanges around the world.",\
    "CryptoWatch offers a real-time WebSocket API for streaming normalized cryptocurrency market data.",\
    "Among Us is a deduction based multiplayer online game.",\
    "CookiePro is a cookie consent tool that offers products for cookie compliance and mobile app compliance.",\
    "beIN SPORTS is a streaming application that provides sports news, videos, highlights, scores and standings.",\
    "beIN Media Services is an application used for provisioning of all services related to the beIN Media Group, a global sport and entertainment network headquartered in Qatar.",\
    "Erstream is a Turkish content and video delivery service.",\
    "We Meet is a video conference and meetings mobile application.",\
    "NewsDog is a news app popular in India which brings together the latest protests news, videos as well as other news.",\
    "Pugpig is a publishing platform used by leading media brands.",\
    "Digiturk Play is a turkish mobile application to watch live TV or episodes of favourite series.",\
    "Digicel Services is an application used for provisioning of all services related to Digicel, a Caribbean mobile phone network and home entertainment provider.",\
    "DoorDash is an application for food delivery from local restaurants.",\
    "Photo Wonder is a photo editing app with a focus on manipulating photos of persons and effects used in social networks.",\
    "Quantcast provides AI-based audience measurement services.",\
    "JioSaavn is a popular indian music streaming service.",\
    "Opera News is an international news app owned by Opera.",\
    "xHamster is a streaming website for pornographic content.",\
    "Shahid is a popular arabian video on demand streaming service.",\
    "MBC Services is an application used for provisioning of all services related to the Middle East Broadcasting Center, a saudi arabic media group.",\
    "QQMusic is a chinese freemium music streaming service owned by Tencent Technology (Shenzhen) Company Ltd.",\
    "Seamless is a food ordering and delivery service.",\
    "Cires21 is a streaming platform that allows optimized live video distribution.",\
    "Splunk Inc. is an American corporation, that produces software for searching, monitoring and analyzing big data via a Web-style interface.",\
    "CoinCap is a provider of real-time market status of cryptocurrency with tracking features.",\
    "ShapeShift Services is an application used for provisioning of all services related to ShapeShift, a company that offers global trading of a variety of digital assets via web and mobile platforms.",\
    "Al Jazeera Services is an application used for provisioning of all services related to Al Jazeera Media Network, owner of the broadcaster Al Jazeera.",\
    "Al Jazeera English is an application for english news channel owned by Al Jazeera Media Network.",\
    "DO Global Services is an application used for provisioning of all services related to DO Global, a Chinese software company.",\
    "Webflow Inc. provides services for building and hosting websites with a built-in editor.",\
    "Payback is an application for contactless payment, coupon activation and point collection for the Payback bonus system.",\
    "Meitu Services is an application used for provisioning of all services related to Meitu Inc., a Chinese technology company headquartered in Xiamen, Fujian which makes image-and-video processing applications. This application also covers all traffic related to the Meitu subsidiary Pixocial, Inc.",\
    "CamScanner is a Chinese mobile app owned by Intsig that allows iOS and Android devices to be used as image scanners.",\
    "Intsig Services is an application used for provisioning of all services related to Intsig, a company that develops mobile applications specializing in pattern recognition and image processing.",\
    "JAUMO is a flirting application that lets the user come into contact with local singles.",\
    "PubMatic is an online advertising and digital publishing service.",\
    "Jio Services is an application used for provisioning of all services related to Jio Platforms and all it's subsidiaries. Jio Platforms is an Indian digital services company.",\
    "Groupon is a shopping application that offers discounts for goods and services by local merchants.",\
    "FYEO (For Your Ears Only) was a podcast streaming service. It was discontinued due to market decision in October 2021 and was therefore deprecated",\
    "Verizon Media Services is an application used for provisioning of all services related to Verizon Media, a division of Verizon Communications that focuses on media and online business.",\
    "Activision Blizzard is an application used for provisioning of all services related to Activision Blizzard, Inc.. The containing business units are: Activision Publishing, Blizzard Entertainment, King Gaming, Major League Gaming, and Activision Blizzard Studios.",\
    "Audiomack is a music streaming service with the possibility to download music to play it offline.",\
    "Kakao Map is a navigation and map service popular in South Korea.",\
    "OSN (Orbit Showtime Network) is an Arabic streaming application named after the satellite provider with the same name. It allows users to watch series, movies, documentaries, and kids content.",\
    "GSM Association, also known as Global System for Mobile Communications Association, is an organisation for mobile network operators worldwide.",\
    "Google Jibe is a RCS platform which provides services for the native communication over mobile phone networks similar to WhatsApp which is intended to replace the SMS and Multimedia Messaging Services.",\
    "BeautyPlus is a Chinese photo editing mobile application mainly used by social media users.",\
    "VivaVideo is a video editing application provided by QuVideo Inc.",\
    "Call of Duty is a popular first-person-shooter gaming franchise available on a variety of platforms and as mobile applications.",\
    "MicoWorld Services is an application used for provisioning of all services related to MicoWorld Limited, a Chinese company headquartered in Beijing, that provides several social chat apps.",\
    "Hago application is a free online gaming platform with a focus on social interaction of its users.",\
    "Daum Cafe is a Korean social networking application that provides so-called Fancafes, essentially message boards where fans are able to interact with artists and each other.",\
    "Tencent News, also known as QQNews, is a news application popular in China which brings together the latest news.",\
    "Flipboard is an international news aggregator application.",\
    "Viacom18 Services is an application used for provisioning of all services related to Viacom 18 Media Pvt. Limited, an Indian joint venture operating in the television industry.",\
    "Voot is an Indian video on demand service available as mobile application, and as a website for desktop consumption.",\
    "SelfieCity is an application which provides editing and capturing of selfie photos and videos.",\
    "Meitu is a photo editing application to enhance facial photos and to add a virtual look.",\
    "Kaspersky Services is an application used for provisioning of all services related to Kaspersky Lab, an anti-virus and cybersecurity provider headquartered in Moscow.",\
    "Hakuna is a social networking application with live streaming and chat functionality.",\
    "Bermuda Video Chat is a dating focused social network with integrated video chat.",\
    "Xiaomi Services is an application used for provisioning of all services related to Xiaomi Inc, a Chinese electronic and software company.",\
    "Opera News Lite is the lighter version of the Opera News application designed for limited mobile networks.",\
    "Gaijin Entertainment is an application used for provisioning of all services related to the company Gaijin Entertainment, a European video game developer and publisher mostly known for their multiplayer games.",\
    "Scopely Services is an application used for provisioning of all services related to Scopely, an American mobile games developer and publisher.",\
    "MARVEL Strike Force is a free-to-play combat/role-playing game with Marvel characters.",\
    "Sky News is a British international news application.",\
    "Systex Services is an application used for provisioning of all services related to SYSTEX Corporation, a Taiwan-based IT services provider.",\
    "Perfect Corp Services is an application used for provisioning of all services related to Perfect Corp, also known as Perfect Mobile Corp, a beauty tech solutions provider and subsidiary of CyberLink.",\
    "YouCam Makeup is a Chinese photo editing application that lets users apply different effects on their pictures.",\
    "Deutsche Telekom AG Services is an application used for provisioning of all services related to Deutsche Telekom AG, an Telecommunication company from Germany.",\
    "PlayerUnknowns Battlegrounds is an online battle royale video game.",\
    "Google Voice is a messaging application with calling and voicemail functionality.",\
    "theScore is a sports news application that offers real-time game stats, videos, social content and betting.",\
    "Pantaya is a mobile movie and TV show streaming application.",\
    "Daum Mail is a Korean email collector that gathers email messages from different services in a combined client.",\
    "Facebook Broadcasting covers all traffic caused by broadcasting videos via Facebook.",\
    "War Thunder is a vehicular combat multiplayer video game.",\
    "APUS Services is an application used for provisioning of all services related to APUS, a Chinese mobile technology company.",\
    "Ufoto Services is an application used for provisioning of all services related to Ufoto (Hangzhou Gexiang Technology), a mobile internet technology company which does selfie and photo-sharing applications.",\
    "Peak Games Services is an application used for provisioning of all services related to Peak Games, a Turkish mobile games developer and publisher.",\
    "Toon Blast is an online puzzle game application based on cartoons.",\
    "Expo Framework is an open-source set of tools used to build React applications.",\
    "The New York Times is an American newspaper application.",\
    "DU Recorder is an application that records and edits videos that are displayed on the screen.",\
    "Luminati was an proxy and data collection services. It was rebranded into new service Bright Data. Hence, the application was deprecated and all related traffic will be handled by the Bright Data application classification.",\
    "Meizu Services is an application used for provisioning of all services related to Meizu Technology Co., Ltd., a Chinese consumer electronics company.",\
    "JOYY Services is an application used for provisioning of all services related to JOYY Inc. (formerly YY Inc.), a major Chinese video-based social media platform.",\
    "Mobvista is a technology platform that provides mobile advertising and mobile analytics services to app developers.",\
    "StarTimes ON is Chinese video streaming service for watching live TV and entertainment, it has a strong presence in Sub-Saharan Africa.",\
    "Vodafone Services is an application used for provisioning of all services related to Vodafone Group plc, a British multinational telecommunications company.",\
    "The Guardian is a British international news application.",\
    "Guardian Media Services is an application used for provisioning of all services related to Guardian News and Media Ltd, a company that develops mobile applications around the British newspaper, The Guardian.",\
    "Mozilla Services is an application used for provisioning of all services related to Mozilla Foundation, a nonprofit organization.",\
    "Magnite is the largest independent omnichannel sell-side platform worldwide. It offers web publishers services to manage their advertising inventory, fill it with ads, and receive revenue.",\
    "DC Universe is a digital subscription service that offers access to movies, TV series, comics and merchandise related to DC Comics, Inc., an American comic book publisher.",\
    "Rakuten Services is an application used for provisioning of all services related to Rakuten, Inc., a Japanese E-commerce, fintech, digital content and communication services company.",\
    "Playdemic Services is an application used for provisioning of all services related to Playdemic, a mobile games developer and publisher based in the UK.",\
    "Golf Clash is a mobile multiplayer golf game.",\
    "OneTrust is the leading privacy management software platform to operationalize data privacy compliance and privacy by design.",\
    "Lightstep provides an observability platform with different tools like a correlation engine, error analysis, and service diagrams for developers.",\
    "Arc Publishing was a software-as-a-service (SaaS) platform that was one of the leading services for publishers and media companies worldwide which offered news-oriented content management. It was rebranded as Arc XP in 2021. Therefore, all Arc Publishing related traffic will be classified as Arc XP.",\
    "VS Media, Inc., is an American platform provider specializing in scalable live video chat solutions. Its services are mostly used by pornographic webcam applications or websites.",\
    "Exaring Services is an application used for provisioning of all services related to Exaring AG, a German streaming services provider.",\
    "waipu.tv is a German internet TV and video-on-demand service.",\
    "Google News is an application that aggregates news from different sources.",\
    "RingCentral is a business communications platform with messaging, video conferencing and phone functionality.",\
    "Mortal Kombat is a popular arcade fighting game centered around martial arts.",\
    "VPN Valley is a free VPN application developed by Mona Enterprise Inc.",\
    "beIN CONNECT is a streaming application with a focus on sports, that also provides some movie and kids channels.",\
    "Boomplay Music is a music streaming application with focus on the African market.",\
    "Kongregate is a publisher and hoster for many free-to-play online games.",\
    "Ninja Kiwi Services is a mobile games developer and publisher based in New Zealand.",\
    "Huawei Services is an application used for provisioning of all services related to Huawei Technologies Co., Ltd., a Chinese multinational technology company.",\
    "XMRig is an application which can be used for mining the Monero cryptocurrency.",\
    "LoL Wild Rift is a multiplayer online battle arena game based on League of Legends.",\
    "Tinkoff is a banking application offered by Tinkoff Bank, a Russian online bank.",\
    "Yandex Market is a Russian online marketplace and aggregator of online shops.",\
    "Sweet Selfie is an image and video editing application.",\
    "Sweet Selfie Lite is the lighter version of the Sweet Selfie application.",\
    "DigiCert, Inc. is an American technology company focused on digital security. As a certificate authority and trusted third party, DigiCert provides the public key infrastructure and validation required for issuing digital certificates or TLS certificates.",\
    "Switcherry VPN is a mobile VPN application that exists as a freemium and a premium version.",\
    "Xender is a file transfer and sharing application.",\
    "NCSoft is an application used for provisioning of all services related to the NCSoft Corporation, an international video game developer and publisher known for the distribution of MMORPGs.",\
    "OrNET is a mobile web browser for the Tor network.",\
    "CoinGecko is a data analytics platform for tracking cryptocurrencies and blockchain assets.",\
    "Universal Music Group (UMG) Recordings, Inc. is an American global music corporation that provides mobile applications aimed at artists and music fans.",\
    "Malwarebytes is an anti-malware solution developed by the corresponding American digital security company Malwarebytes Inc..",\
    "RealNetworks, Inc. is an American provider of Internet streaming media delivery services.",\
    "KeyCDN is a European CDN, provided by prionity LLC, a privately funded company headquartered in Switzerland.",\
    "Riley Cillian Hong Kong Limited, also known as RC Platform, is a Chinese app developer famous for its social networking applications Tumile, LivU and Yaar.",\
    "Streamlive-to (formerly known as iLive.to) is a streaming website to watch live TV, movies and TV series on demand.",\
    "Wildlife Studios is an application used for provisioning of all services related to Wildlife Studios a Brazilian mobile games publisher. It was formerly known as TFG, Top Free Games and Fun Games For Free.",\
    "Block Craft 3D is a free city building mobile game similar to Minecraft published by Wildlife Studios.",\
    "Removed. Reason: The name of the application was not fitting. Instead the application Lumen Technologies should be used.",\
    "LOVOO is a location-based social app for dating and finding friends.",\
    "Clubhouse is a social networking application focusing on audio content that helps create virtual rooms with conversations on diverse topics such as music, networking, dating or political discussions.",\
    "Sniper 3D is a free mobile multiplayer FPS game published by Wildlife Studios.",\
    "Comodo Services is an application used for provisioning of all services related to Comodo Security Solutions, Inc., an American cybersecurity company mainly focused on security solutions.",\
    "Ribbon Communications US LLC is a company focused on IP-based real time communications security and software solutions.",\
    "BlackBerry Work (formerly known as Good) is an application for organizing work including email, scheduling and contact management functionality.",\
    "Joom is a an online shopping application and website.",\
    "BlackBerry Services is an application used for provisioning of all services related to BlackBerry Limited, a Canadian enterprise software and services company.",\
    "Google Pay is a payment and money exchange application developed by Google.",\
    "Geekflare provides services for web- and application-engineers like website performance, domain security, and whois hosting.",\
    "War Machines is a free tank battle mobile game published by Wildlife Studios.",\
    "RawVoice, the parent company of Blubrry, provides a suite of tools and services for podcasting, that includes hosting, distribution, and monetization of podcasts.",\
    "Podtrac provides industry-standard podcast measurement data for podcasts.",\
    "Wise (formerly TransferWise) is a worldwide funds transfer application based in London.",\
    "Google Podcasts is a podcast application developed by Google.",\
    "Shalltry Group is an application used for provisioning of all services related to the company Shalltry Group, a Chinese company specialized on security and device cleaning applications.",\
    "LiveRamp Holdings, Inc. is a SaaS company that offers a data connectivity platform whose services include data onboarding and the transfer of offline data online for marketing purposes.",\
    "Euronews is a pan-European television news network based in France. It is also a provider of live streaming world news.",\
    "Leap Fitness Group is a company that develops fitness and health mobile applications.",\
    "Hexaglobe is an on-demand video content delivery service based in Paris.",\
    "Rocket League is a popular video game in which the user plays soccer, but with rocket-powered cars.",\
    "Magine Services is an application used for provisioning of all services related to Magine Holding AB, a Swedish company that provides OTT media services.",\
    "Pay-Rw is an application for bill payment provided by Esicia Ltd.",\
    "Vuba Vuba is an application for food ordering and delivery provided by Vuba Vuba Africa.",\
    "Yandex Go is an application that allows users to request rides, order food from restaurants or send items across town.",\
    "Yandex Telemost is a video conferencing tool that lets the user set up video meetings by link.",\
    "Nexon Corporation is a South Korean-Japanese technology company, specialized in online games for PC and mobile.",\
    "Agoda is an application for searching and booking accommodation and flights. It also offers discounts.",\
    "Grab is an application that offers ride-hailing, food-delivery and cashless payment services.",\
    "New York Public Radio (NYPR) is a not-for-profit corporation, including several radio stations that provide radio journalism, podcasts, and classical music. The two most famous channels are WNYC and WQXR.",\
    "PRX Services is an application used for provisioning of all services related to PRX, a non-profit media company specializing in audio journalism and storytelling. Its most important services are PRX Exchange, an open audio distribution marketplace and Dovetail, a content management platform for media contents.",\
    "Dovetail is a content management and podcast publishing platform that offers services like scalable dynamic ad serving, monetization and user analytics.",\
    "Huya is a live streaming platform which is focused on video gaming and e-sport based in China.",\
    "SHAREit is a file transfer and sharing application.",\
    "Eversend is a multi-currency wallets platform that provides currency exchange, merchant payments, and other financial services.",\
    "5channel (formerly 2channel) is a Japanese textboard. It is one of Japans most popular online communities.",\
    "Pet Rescue Saga is a free-to-play puzzle game with the goal to rescue animals from animal thieves.",\
    "ClassDojo is an online learning platform that connects teachers with students and parents.",\
    "Photomath is a mobile app which uses the camera of the smartphone to scan and then solve mathematical problems.",\
    "miHoYo Games is an application used for provisioning of all services related to miHoYo Co., Ltd., a Chinese video game developer based in Shanghai.",\
    "Genshin Impact is an Anime-influenced action role-playing game that features an open-world environment and action-based battle system.",\
    "Honkai Impact 3rd is an Anime-influenced free-to-play 3D action role-playing mobile game notable for incorporating a variety of genres.",\
    "Khan Academy is a digital platform with the goal of creating a set of online tools that help educate students.",\
    "HOMER is an early literacy learning platform with different games and stories for children.",\
    "Castbox is an application that distributes free podcasts, it also produces its own shows.",\
    "OpenText Services is an application used for provisioning of all services related to the OpenText Corporation, a Canadian company that develops and sells enterprise information management software.",\
    "edX is an education platform which hosts online university-level courses in a wide range of disciplines.",\
    "Socratic is an online learning platform developed by Google LLC.",\
    "Telia Services is an application used for provisioning of all services related to Telia, a Swedish telecommunications company.",\
    "Telia TV is a service for televison over IP in Sweden.",\
    "Peacock TV is an application that allows access to the video streaming service Peacock, that primarily shows content from NBCUniversal studios and other content providers, including television series, films, news, and sports.",\
    "Jio Ads is the service Jio Platforms uses for its advertisement traffic.",\
    "Discovery Plus is an application provided by Discovery, Inc., which allows the user to stream programs which are operated by Discovery, Inc. like HGTV or Animal Planet.",\
    "Duolingo is an American language-learning platform as well as a digital language proficiency assessment exam.",\
    "ViacomCBS Services was an application used for provisioning of all services related to ViacomCBS Inc., an American mass media conglomerate. In 2022, ViacomCBS Inc. changed its name to Paramount Global. Therefore, the application was deprecated and all related traffic will be handled by Paramount Services.",\
    "BET Plus is an American OTT video streaming service.",\
    "Paramount Plus is a mobile application for streaming shows from CBS, Nickelodeon, Comedy Central and MTV, live sporting events like UEFA Champions League and the NFL, as well as exclusive shows and originals.",\
    "HubSpot offers an engagement platform for marketing, sales and customer service.",\
    "Philo (formerly Tivli) is an application that provides access to the OTT streaming television service of the American internet television company with the same name.",\
    "DNA TV is a streaming television service provided by DNA Ltd.",\
    "Xiaomi Weather is a popular mobile weather application.",\
    "JioChat is a video conferencing, voice calling and messaging application from India.",\
    "The application Remind is a communication platform in education designed to involve parents. It is mainly for schools from kindergarten to 12th grade.",\
    "Elisa Services is an application used for provisioning of all services related to Elisa, a Finnish telecommunications company.",\
    "Elisa Viihde is a Finnish entertainment service by the company Elisa, which offers video-on-demand services.",\
    "PubNub provides a Realtime Communication Platform and APIs as the foundation for online chat, live events, geolocation, remote control, and live updates.",\
    "Funimation is an American streaming service specialized on dubbed and subtitled Anime. It is also an American entertainment company that specializes in the dubbing and distribution of East Asian media.",\
    "Drift is a sales and marketing platform which uses chat bots to provide personalized conversations to the visitors and customers of website or application including it.",\
    "The International Management Group is a global sports, events and talent management company that distributes multimedia content to global broadcasters. They are best known for their content management platforms, the Digital Video Exchange (DVE) and the Digital Gaming Exchange (DGE).",\
    "Endeavor Services is an application used for provisioning of all services related to the Endeavor Group Holdings, Inc., (formerly known as William Morris Endeavor Entertainment, WME), an American company for talent and media agencies that represents artists as well as the NFL and NHL.",\
    "Softbank Services is an application used for provisioning of all services related to Softbank Corp., a Japanese multinational telecommunications and Internet cooperation.",\
    "Reface is an application that swap faces in videos, GIFs and memes.",\
    "MTV Katsomo is a Finnish application for streaming of different video content like live TV, series and news related to the channel MTV3.",\
    "Amazon Ads covers all Amazon-related advertisement traffic, like traffic caused by the Sizmek Ad Suite, the Amazon Marketing Cloud or the Amazon Demand Side Platform (DSP).",\
    "Prende TV is an American streaming app for Spanish-language TV and Movies.",\
    "Univision Communications is an application used for provisioning of all services related to Univision Communications Inc., an American media company focused on Spanish language television content.",\
    "Anchorfm is an application that distributes free podcasts. It also produces its own shows.",\
    "VeVe is an E-Commerce application specialized on 3D collectibles, which can also be displayed by collectors in virtual showrooms.",\
    "VRV was a video streaming application that provided content, mainly Sci-fi and Anime. In May 2023, VRV merged with Crunchyroll. Therefore, the application was deprecated and all related traffic is now handled by the Crunchyroll application classification.",\
    "Qualtrics is a platform for creating and distributing surveys.",\
    "Wangsu CDN is an application used for provisioning of CDN services related to Shanghai Wangsu Science and Technology Co., a China-based company that provides CDN and Internet data center (IDC) services.",\
    "Algolia is a search as a service platform that find results across the web, mobile and voice.",\
    "Maxivision is a Finnish internet TV service.",\
    "G-Core Services is a secure edge and cloud platform solution provider.",\
    "Nordic Entertainment Group is an application used for provisioning of all services related to Nordic Entertainment Group, a media and entertainment company headquartered in Stockholm.",\
    "Viafree is a Finnish video streaming service that offers free content with ads.",\
    "Netmera is a mobile application engagement platform for campaigns with data analysis features.",\
    "Gracenote is a digital provider of media metadata and automatic content recognition.",\
    "storyfm-cn is a chinese application that distributes free podcasts.",\
    "Yle Areena is a Finnish streaming application that offers various audio and video contents like TV series, movies, podcasts and webradio.",\
    "Streamable is a hosting and streaming services provider based in the US.",\
    "Speakol is a native advertising platform with AI-based recommendation functionalities.",\
    "Yallakora is an Egyptian soccer news and streaming application.",\
    "LINKonLINE is an application used for provisioning of all services related to LINKonLINE, an Egyptian provider for online services and online content.",\
    "Gemini Media Egypt is an Egyptian media agency which provides several media solutions to advertisers.",\
    "Sanoma Services is an application used for provisioning of all services related to the Sanoma Corporation, Finlands largest media group.",\
    "Ruutu is a Finnish video streaming service.",\
    "Finnpanel is a Finnish analytics service that measures TV, Internet TV and radio consumption.",\
    "Noon VPN is a VPN service provided by the company Alzahim Digital Services Ltd.",\
    "Masrawy is an Arabic Egyptian news web portal which presents Arabic-language news in the Middle East and wider Arabic-speaking community.",\
    "KTPlay is an integrable platform which enables in-app user-to-user interactions in gaming applications.",\
    "Team Fortress 2 is a first-person shooter game published and developed by Valve. It is using the Source Engine and is distributed via Steam.",\
    "Trust Wallet is a digital currency wallet and platform where businesses and consumers can do transactions in digital currencies.",\
    "Canal Plus Afrique is an application provided by the French television group with the same name in the African region.",\
    "Sarmady Services is an application used for provisioning of all services related to Sarmady, a subsidiary of Vodafone Egypt.",\
    "FilGoal is an Egyptian sports application publishing mainly football news.",\
    "AMC Theatres is an American movie theater chain. The AMC Theatres application can be used to buy tickets, get movie information and find theaters close to the current location.",\
    "Youm7 is a popular daily newspaper published in Egypt.",\
    "Valve Corporation is a american software and hardware company, specialized in games and related infrastructure.",\
    "Snail VPN was a mobile VPN application that existed as a freemium and a premium version. In March 2022, Super vpn (snail vpn) stopped service. Therefore it was deprecated.",\
    "Xoom is an application for money transfer and utility bills payment.",\
    "IBM is an application used for provisioning of all services related to the International Business Machines Corporation.",\
    "Midoki Services is an application used for provisioning of all services related to Midoki, a British gaming studio.",\
    "VPN Master X is a mobile VPN service.",\
    "Qualaroo is a platform for creating and distributing surveys.",\
    "MTS Services is an application used for provisioning of all services related to MTS (Mobile TeleSystems), the leading telecommunications group in Russia and the CIS.",\
    "Expedia is an American online travel shopping company for consumer and small business travel.",\
    "GlobalSign is a technology company focused on digital security. As a certificate authority and trusted third party, GlobalSign provides the public key infrastructure and validation required for issuing digital certificates or TLS certificates.",\
    "LetsEncrypt, is a certificate authority run by Internet Security Research Group (ISRG) that provides X.509 certificates for Transport Layer Security (TLS) encryption.",\
    "Yieldmo is an American technology company focused on mobile advertising.",\
    "IdenTrust is a public key certificate authority that provides digital certificates to financial institutions, healthcare providers, government agencies and enterprises.",\
    "GoDaddy, Inc. is an American technology company focused on digital security, domain registration and web hosting. As a certificate authority and trusted third party, GoDaddy provides the public key infrastructure and validation required for issuing digital certificates or TLS certificates.",\
    "Google Trust Services, is a certificate authority and provides the public key infrastructure and validation required for digital or TLS certificates.",\
    "Watch iT is an Arabic video streaming service based in Egypt.",\
    "Amazon Trust Services, is a certificate authority and provides the public key infrastructure and validation required for digital or TLS certificates.",\
    "Entrust Corp., provides financial institutions, national governments, corporate enterprises and other organizations with technologies to establish trusted identities and conduct secure transactions.",\
    "Binance is a cryptocurrency exchange that provides a platform for trading various cryptocurrencies.",\
    "Elfsight provides website widgets for marketing and customer engagement.",\
    "Sailthru offers technology for email, web, and advertising content. Since Carnival Mobile was acquired by Sailthru, it is also covered by this application.",\
    "Certum is a Polish technology company focused on digital security. As a certificate authority and trusted third party, Certum provides the public key infrastructure and validation required for issuing digital certificates or TLS certificates.",\
    "MediaNova CDN is an application used for provisioning of CDN services related to MediaNova, a company that provides encoding, streaming and small/large object delivery CDN services with its cloud and private ( on-prem or managed) solutions.",\
    "Napster (formerly known as Rhapsody) is an on-demand music subscription service and online music store based in Seattle.",\
    "Sectigo (formerly known as Comodo CA) is a UK based digital certificate authority that issues SSL and other digital certificates.",\
    "Bazaarvoice, Inc. is an American technology company that provides user-generated content (UGC) like product ratings, reviews and FAQs for e-commerce websites.",\
    "Suning Services is an application used for provisioning of all services related to the Suning Holdings Group Co., Ltd., a large Chinese company.",\
    "Indigitall is a Spanish company that provides marketing automation services.",\
    "Ada Support provides a personalized customer service AI-powered chatbot.",\
    "Penguin Sports, also known as Penguin Live or QQ Live, is a professional internet sports platform with interactive live broadcast and event services.",\
    "Pusher povides realtime features like charts, notifications, location tracking or in-app chats for mobile applications and websites.",\
    "CSG Media is a company that provides computer related services like revenue management, digital monetization and customer engagement solutions.",\
    "Grapeshot is a globally working company that provides contextual intelligence solutions.",\
    "Knighthood is a role-playing game, in which the player fights as a Rage Knight in a fantasy world.",\
    "Blossom Blast Saga is a mobile game centered around linking flower buds of the same colors.",\
    "Cybertrust CA is a Japan based digital certificate authority that issues SSL and other digital certificates.",\
    "Usercentrics provides consent management services for providers and advertisers.",\
    "Idviu is a provider of video management, processing, protection and distribution.",\
    "iApps Technologies GmbH develops software solutions for the publishing of newspapers and magazines, the managements of paid content, and user analysis.",\
    "Calm is an application that offers guided meditations and sleep aids.",\
    "PlayStation Now (also known as PS Now) is a video game streaming service used to provide PlayStation gaming content.",\
    "Microsoft Azure is a cloud service from Microsoft providing data storage and media streaming services.",\
    "Zego provides cloud streaming services as well as real-time network quality monitoring and comprehensive streaming quality reporting services.",\
    "Dish Network is an application used for provisioning of all services related to the Dish Network Corporation, an American television provider.",\
    "jQuery is a JavaScript library for HTML document traversal and manipulation, event handling, animation, and Ajax.",\
    "Mox TV allows video publishers and advertisers to distribute and monetize their video content.",\
    "Stats Perform is a sports data business that collects and distributes time-stamped, contextual and live sports data via an API.",\
    "Contentful is the API-first content management platform to create, manage and publish content on any digital channel.",\
    "WeTV (also known as Tencent Video) is a Chinese video streaming service that provides movies, dramas, and shows.",\
    "Adex is a globally operating specialist in data management solutions and real-time advertising.",\
    "Arlo is a Smart Home Security application that allows the user to stream live video and play recorded clips from wireless surveillance cameras as well as easily arm and disarm the Smart Home system.",\
    "JioHealthHub connects users with Healthcare service providers across India.",\
    "GOG (formerly Good Old Games) is a digital distribution platform for video games.",\
    "Ring is a Smart Home Security application that allows users to answer notifications and check on their home in real time as well as getting real time crime and safety alerts from their neighbors and local law enforcement.",\
    "Fitbit is a fitness application that offers workout routines, nutrition programs, meditation advice and health status tracking.",\
    "Google Fit is a fitness tracking application for mobile devices and can be integrated with some other devices and fitness apps.",\
    "Diamond Diaries Saga is a puzzle linker game based around items of jewelry in different colors.",\
    "Deluxe Media is an American company that provides OTT media distribution services.",\
    "The Trade Desk is a provider of an independent demand-side platform that allows cross-device advertisement delivery and real-time optimizations to improve the performance of advertisements.",\
    "JioCloud is a cloud storage application, provided by Jio, for files of any type and size.",\
    "Trivago is an online hotel search site which also provide price comparison between hotels.",\
    "Minutely provides AI-powered video recommendation and thumbnail generation services.",\
    "Garena Services is an application used for provisioning of all services related to Garena, an online game developer based in Singapore.",\
    "Garena Free Fire is a survival shooter online game.",\
    "Impact Services is an application used for provisioning of all services related to Impact Tech, Inc., an American organization that helps businesses manage, automate and optimize all types of partnerships.",\
    "Samsung Max Privacy VPN provides VPN services and other privacy measures exclusively for Samsung devices.",\
    "Private Tunnel VPN is a VPN application provided by OpenVPN, Inc.",\
    "TripleLift is a technology company that provides programmatic advertising, in-feed advertising, branded content, and OTT.",\
    "AlphaBetty Saga was a social mobile word game. It was discontinued by the provider starting December 15, 2021 and was therefore deprecated.",\
    "Mailchimp is an American all-in-one marketing and advertisement platform.",\
    "A1 Belarus is an application used for provisioning of all services related to A1, a Belarusian telecommunication company.",\
    "VOKA TV is a Belarusian video service.",\
    "SPB TV allows video publishers and advertisers to distribute and monetize their video content.",\
    "Smart Media Solutions is an Ukrainian company that provides CDN and OTT media distribution services.",\
    "Smaato is a real-time advertising platform for mobile publishers and app developers focused on the monetization of mobile apps and connecting advertisers to in-app ad inventory.",\
    "Teads is a marketing platform for outstream advertising that enables publishers to monetize their editorial offers and advertisers to place their videos in specific content environments.",\
    "Pyramid Solitaire Saga is a solitaire strategy puzzle game located in an ancient world.",\
    "Riskified provides AI-based revenue protection and fraud prevention services for e-commerce platforms.",\
    "Papa Pear Saga was a free-to-play puzzle game based around fruits and vegetables. It was discontinued by the provider starting 14 January 2022 and was therefore deprecated.",\
    "Diamond Digger Saga was a puzzle linker game based around differently colored diamonds. It was discontinued in January 2022. Therefore the application was deprecated.",\
    "Leanplum is a mobile customer engagement platform that offers different services for app-developers that enable them to send timely and relevant push notifications, in-app messages, and emails to better serve their customers.",\
    "Tapad is an advertisement service provided by Tapad Inc., a company that develops and markets software and services for cross-device advertising and content delivery.",\
    "Squarespace provides software as a service for website building and hosting, and allows users to use pre-built website templates.",\
    "BrightLine is a multimedia advertisement service provided by Brightline iTV, a company that provides interactive television solutions for the entertainment and advertising sectors.",\
    "Extreme Reach is a tech platform for video ad campaign workflow that delivers ads to TV screens and mobile devices.",\
    "RocketVPN is a free VPN service developed by ConnectEasy Technology LTD.",\
    "Moovit provides public transportation information such as departure and arrival times for train and bus, maps and digital payment services for tickets.",\
    "Coinbox is a cryptocurrency wallet and platform where businesses and consumers can buy and sell cryptocurrencies.",\
    "SYBO Games is an application used for provisioning of all services related to SYBO ApS, a Danish mobile game developer mainly known for the mobile game Subway Surfers.",\
    "Subway Surfers is the most downloaded mobile runner game.",\
    "National Public Radio (NPR) is an American non-profit media organization that produces and distributes news and cultural programming.",\
    "ESET Services is an application used for provisioning of all services related to ESET GmbH, an anti-virus and cybersecurity provider headquartered in Bratislava, Slovenia.",\
    "MakeupPlus is a photo and video editing application for virtual makeup try-on.",\
    "Kakao Webtoon is a digital comics application which is popular in South Korea.",\
    "ZEE5 is an Indian OTT streaming service that launched in 2018.",\
    "Juspay is an Indian online payment solution provider.",\
    "Pokemon Unite is a free-to-play game based on the popular Pokemon franchise.",\
    "Zomato is an Indian multinational restaurant aggregator and food delivery application.",\
    "PLYmedia is the developer of SaaS solutions for ad-tech, marketing and media industries.",\
    "BitPay is a cryptocurrency wallet and platform where businesses and consumers can buy and sell cryptocurrencies.",\
    "Phycan VPN Services is an application used for provisioning of all services related to Phycan Technology Limited, a Chinese developer of free VPN applications.",\
    "Internet Archive is a non-profit online library of books, movies, software, music and websites. It also provides audio content as a b2b service.",\
    "Cloudflare WARP VPN, also known as 1.1.1.1 Faster Internet, is a VPN service provided by Cloudflare, Inc.",\
    "BRD Wallet is a simple and secure wallet for multiple cryptocurrencies.",\
    "Exodus Wallet is a digital currency wallet for secure management and exchange of cryptocurrencies like Bitcoin, Ethereum, Ripple and more.",\
    "BeautyCam is a Chinese photo editing mobile application mainly used by social media users.",\
    "Gab is a social network that champions free speech, individual liberty and the free flow of information online.",\
    "Measurement Lab (M-Lab) provides a large collection of open Internet performance data. Their Speed Test provides advanced diagnostics of the performance of the users broadband connection through quick measurements.",\
    "Glovo is an on-demand courier service mainly offering food delivery but also delivery of pharmaceuticals and general goods.",\
    "Samsung Music is a music streaming service provided by Samsung.",\
    "Sweet TV is an Ukrainian streaming platform which provides access to TV channels and movies for the whole family.",\
    "Uklon is a Ukrainian ride-hailing service.",\
    "Toca Boca is an application used to cover applications by Toca Boca, a Swedish app development studio focused on child-friendly applications for tablets and smartphones.",\
    "TAVR Media is an Ukrainian company responsible for several radio stations and applications.",\
    "Lightsail VPN is a free mobile VPN application available on Android smartphones.",\
    "IBM Cloud Video (formerly known as Ustream) is an American live video streaming service.",\
    "Agora Services is an application used for provisioning of all services related to Agora, a video, voice, and live interactive streaming platform, providing services for embedded voice and video chat, real-time recording, interactive live streaming, and real-time messaging.",\
    "Sun FM Ukraine is an online Ukrainian music streaming radio.",\
    "OLL TV is a Ukrainian streaming platform which provides access to TV series and movies.",\
    "Lanet TV is a Ukrainian internet TV application.",\
    "Tigo Services is an application used for provisioning of all services related to Tigo, a mobile phone service provider company owned by the international company Millicom.",\
    "Uber Eats is a food ordering and delivery application, where users can read menus, reviews and ratings, order, and pay for food from participating restaurants.",\
    "Bolt Technology is an application used for provisioning of all services related to Bolt Technology, a transportation and delivery service provider.",\
    "Stripe is an Irish-American financial services and software as a service (SaaS) company.",\
    "Medallia is a platform that provides software-as-a-service (SaaS) customer experience management (CEM) and employee experience management.",\
    "OneplusOne Media Services is an application used for provisioning of all services related to 1+1 Media Group, a Ukrainian media holding which joins together several information and entertainment channels.",\
    "OneplusOne Video is a Ukrainian video streaming application.",\
    "Verisign is an american company that operates network infrastructure and the authority to register for multiple top-level domains.",\
    "DivanTV was a Ukrainian online TV service and streaming application. It ceased operations at the end of 2021, therefore it was deprecated.",\
    "Admixer is an independent adtech company that develops digital advertising products and services for publishers and app developers.",\
    "DingTalk is an enterprise communication and collaboration platform developed by Alibaba Group.",\
    "Online Radio Box is a service that allows the user to create an application for a radio station.",\
    "MOLOCO is a global Ad-company headquartered in Silicon Valley that offers a programmatic advertising platform powered by machine learning.",\
    "LiveXLive pronounced Live by Live was a music streaming platform. It was rebranded into LiveOne in October 2021. Therefore, the application was deprecated and all related traffic will be handled by the LiveOne application classification.",\
    "Pro-Stream-Ru LLC is a provider of quality services for video content.",\
    "Blockchain-com provides cryptocurrency related services, such as a wallet, an exchange, a block explorer and services for institutional investors.",\
    "Qualcomm is a US based chip manufacturer.",\
    "JioMart is an Indian online shopping application, which focuses on online groceries sales and other product categories such as fashion, home essentials,and lifestyle products.",\
    "JioMeet is a video conferencing application from India that can host up to 100 participants.",\
    "HCL Notes and Domino is a client-server cross-platform runtime environment for several business collaboration tools, such as email, calendars and instant messaging.",\
    "Mi Fit is a fitness application that tracks physical activity, analyzes sleep and evaluates workouts.",\
    "Paxful Wallet is a peer-to-peer application for buying and selling digital currencies.",\
    "Schildbach-Wallet is a decentralized peer-to-peer Bitcoin wallet.",\
    "Cake is a English-learning application that provides quizzes, audio and video content.",\
    "TigerConnect is a instant messenger service where all messages are kept on the server and can be deleted by the sender after a certain time.",\
    "iCloud Privat-Relay is a dedicated feature for iCloud+ subscription to ensure secure only web surfing.",\
    "Ludo King is a cross-platform multiplayer game designed after classic board games.",\
    "Twilio provides APIs for software developers to embed voice, messaging, video and email capabilities into their applications.",\
    "Ankama Games is an application used for provisioning of all services related to Ankama, a French company focused on the design of MMORPGs.",\
    "Meta Services is an application used for provisioning of all services related to Meta Platforms, Inc. (formerly Facebook, Inc.), the parent company of Facebook, Instagram and WhatsApp.",\
    "Huawei Health is an application used for monitoring health and physical activity of users.",\
    "Youdao is a Chinese search engine, data storage provider and Chinese-to-English translation service.",\
    "Xinhua News is the English-language mobile app of the Xinhua News Agency, the state-run press agency of the People's Republic of China.",\
    "Raygun provides services and tools to proactively find, diagnose and fix software errors and performance problems in web and mobile applications.",\
    "Google Chat is a direct messaging and team chat software developed by Google. It replaces Google Hangouts.",\
    "DoubleVerify is a leading software platform for digital media measurement and analytics.",\
    "Extole, Inc. provides social media marketing services with a focus on referral marketing solutions.",\
    "StarMaker is an online video network and application that allows users to sing, record and share their favorite songs and battle with friends and other singers from all over the world.",\
    "MX Player is an Indian video streaming and video on demand platform.",\
    "Origins Digital provides broadcasting and media solutions for sports events.",\
    "Sygic Services is an application used for provisioning of all services related to Sygic, a Slovak company that provides automotive navigation applications.",\
    "Budge Studios is a Canadian video game company focused on entertaining and educating children. Their application portfolio consists of original and branded properties, including My Little Pony, Barbie, Hello Kitty, and many more.",\
    "MTS TV is a pay-TV service and streaming application provided by MTS Belarus. It allows users to watch Russian TV channels as well as movies on demand.",\
    "Smadex is a mobile-first user acquisition platform.",\
    "Amazon Games is an American video game company and division of Amazon that focuses on publishing video games.",\
    "New World is a buy-to-play MMORPG set in the mid-seventeenth century on the fictional Aeternum Island.",\
    "Aarki is an AI-enabled mobile marketing platform that uses machine learning and big data.",\
    "Xiaomi ShareMe is a file transfer and sharing application.",\
    "GoodBarber is a mobile app building service including eCommerce and content management functionality.",\
    "LitRes-Ru is an ebook store offering ebooks and audiobooks for PCs, and mobile devices.",\
    "Qwilt develops video delivery technology for the online entertainment market. In collaboration with Cisco they are building a CDN and edge architecture.",\
    "Start-Ru is a Russian subscription-based streaming service with several in-house productions that is available worldwide.",\
    "Pou is a virtual pet game offered by Zakeh Limited.",\
    "Mumble is a voice over IP (VoIP) application primarily designed to be used by gamers.",\
    "AmEx Services is an application used for provisioning of all services related to the American Express Company (AmEx), a multinational corporation specialized in financial services, such as credit cards and consumer travel services as well as marketing products, and services for merchants.",\
    "Wizards of the Coast is an application used for provisioning of all services related to Wizards of the Coast LLC, an american company focused on games mostly based on fantasy and science fiction.",\
    "Magic The Gathering Arena is a digital collectible card game based on the physical card game Magic The Gathering.",\
    "Mozilla VPN is a subscription-based Virtual Private Network that runs on a global network of servers.",\
    "Deutsche Kreditbank is the second largest direct bank in Germany.",\
    "Deutsche Bank is the online banking app of Deutsche Bank, one of Germanys largest Banks.",\
    "NortonLifeLock Services is an application used for provisioning of all services related to the American software company NortonLifeLock Inc. (formerly known as Symantec Corporation) that provides cybersecurity software and services.",\
    "NortonLifeLock VPN Services covers the traffic of different VPN Services provided by NortonLifeLock Inc.",\
    "Zapya is a file transfer and sharing application.",\
    "Amobee is an advertising technology company, which helps companies to optimize results by advertisement and customer engagement.",\
    "Ubitus Services is an application used for provisioning of all services related to Ubitus K.K., a provider of cloud game technology.",\
    "Nintendo Switch Cloud Gaming is a video game feature that allows players to play Nintendo Switch games via an external server.",\
    "Spreaker develops a web and mobile based podcasting platform designed to create, upload and distribute content throughout the web.",\
    "JioCinema is an Indian video streaming service for movies and TV shows.",\
    "Epsilon-Conversant, a merge between the two businesses Epsilon and Conversant, is a global leader in interaction management marketing.",\
    "Nvidia Services is an American multinational company. The main focus of business is graphics processing units (GPUs) or system on chip units (SoCs). It is an application used for provisioning of all services related to Nvidia Corporation.",\
    "Wink is an application developed by Rostelecom to watch television channels, films and TV programs.",\
    "Rostelecom is an application used for provisioning of all services related to Rostelecom, a Russian digital service provider.",\
    "NGenix is an application used for provisioning of CDN services related to NGenix, a Russian CDN and cloud provider.",\
    "Intel Services is an American chip fabricator company. The main focus of business is microprocessors but also delivers wifi or network cards. It is an application used for provisioning of all services related to Intel Coropration.",\
    "Advanced Micro Devices Services is an application used for provisioning of all services related to Advanced Micro Devices, Inc. (AMD), an American company producing mostly chips for central processing units (CPUs) or graphic processing units (GPUs).",\
    "Pikmin Bloom, is an augmented reality mobile game similar to Pokemon Go.",\
    "123VPN is a free VPN application provided by Amplusnet SRL.",\
    "Playtika Services is an application used for provisioning of all services related to Playtika Holding Corp., an Israel-based free-to-play mobile game development and publishing company.",\
    "AVG Services is an application used for provisioning of all services related to AVG Technologies, a Czech developer of cybersecurity, privacy and utility applications.",\
    "Netflix Services is an application used for provisioning of all services related to Netflix, Inc., an American media provider.",\
    "Netflix Games is an application used to cover traffic of different games that are published and distributed by Netflix.",\
    "Hero Wars is an online fantasy roleplaying game.",\
    "SwiftyBeaver is a logging development framework and that includes cloud storage for logs.",\
    "Bitdefender Services is an application used for provisioning of all services related to the globally operating Cybersecurity company Bitdefender, which provides Antivirus and VPN services.",\
    "iHeartMedia Services is an application used for provisioning of all services related to iHeartMedia, Inc., an American mass media corporation.",\
    "Sophos Services is an application used for provisioning of all services related to Sophos Group plc, a British security software and hardware company.",\
    "McAfee Services is an application used for provisioning of all services related to McAfee, LLC, an American computer security software company.",\
    "Eurosport News provides sports related news, scores and sport event livestreams.",\
    "Ola is a ridesharing service available in Australia, India, New Zealand and the UK.",\
    "LG Electronics Inc. is a South Korean multinational electronics company.",\
    "Ericsson Services is a Swedish multinational company. The main focus of business are infrastructure, software, and services for information and communication technology It is an application used for provisioning of all services related to Ericsson.",\
    "ShareChat is an Indian social media and social networking service. ShareChat also launched the application Moj, that emulates the features of TikTok and is India's number one short video app.",\
    "Valorant is a team-based first-person shooter game developed by Riot Games.",\
    "Indeed is an American worldwide employment application and website for job listings.",\
    "Nvidia GeForce NOW is a cloud gaming service provided by Nvidia Corporation that allows users to access a virtual computer to play on.",\
    "True Key is an application used for securing passwords. It performs the functions of a password manager.",\
    "Moon Active is an application used for provisioning of all services related to Moon Active Ltd., a game development company focused on mobile games such as Coin Master.",\
    "Safe Family provides parents with the visibility and controls needed to improve web, app and digital safety for children.",\
    "Lotame is a data management platform to make smarter marketing, product and business decisions.",\
    "Firefox Private Network is a Firefox (Beta) feature that enables secure web browsing.",\
    "Media-net is a company from India that develops innovative products for advertisers and publishers. It is one of the largest ad tech companies worldwide.",\
    "JSDelivr is a free CDN for open-source projects serving files directly from npm or GitHub.",\
    "Ubisoft Services is an application used for provisioning of all services related to Ubisoft Entertainment SA, a French video game company.",\
    "Ubisoft Connect is a dedicated customer service from Ubisoft, to provide additional content for all games. It is the successor of the Uplay service.",\
    "Blitzer-de is an application which helps to receive speed camera alerts and warnings against all fixed speed cameras.",\
    "Samsung Pay is a mobile payment and digital wallet application service provided by Samsung Electronics.",\
    "Google Photos is an online service by Google to store pictures or videos.",\
    "Stellar is an open-source platform for currency exchange and payments.",\
    "TrackMania is a series of racing games where players can create their own tracks.",\
    "Want Want Services is an application used for provisioning of all services related to Want Want Holdings Limited, a food manufacturer from Taiwan that is also the parent of some news- and television- related companies like China Times and China Television.",\
    "Unruly is an advertising platform which allows users to maximize their ad revenue across web and mobile platforms.",\
    "TotalAV is an antivirus and mobile security solution.",\
    "atudo is a navigation and map service provided by atudo GmbH.",\
    "BidSwitch is a middleware that allows connected programmatic partners to seamlessly access new platforms and services to optimize their performance.",\
    "The ING Banking application is a finance app for mobile banking provided by the ING Group, a Dutch multinational banking and financial services corporation.",\
    "FeedAd is a video advertisement SDK, that connects advertisers and publishers.",\
    "Commerzbank Banking is the online banking app of Commerzbank, one of Germanys largest banks.",\
    "VR Banking is the online banking app of Volksbank Raiffeisenbank, one of Germanys largest banks.",\
    "Anzu VR (Anzu Virtual Reality Ltd.) is a provider of in-game advertising that brings real-world brand ads into video games and esports, for example on virtual billboards or team-shirts in sports games.",\
    "Beeswax is a service that allows customers to buy adverts space on websites, mobile apps, video players, and other devices.",\
    "Dauntless is a free-to-play action video game that takes place in a fantasy setting, where the player has to fight against monstrous creatures.",\
    "CatchMedia is an application provided by Catch Media, Inc. that enables its users to provide and monetize digital content across different devices. It provides context-aware promotions, messaging and content delivery.",\
    "Piano is a data management platform which provides services to understand and influence customer behavior.",\
    "Shopify is a service that allows customers to integrate a proprietary e-commerce platform as their online stores.",\
    "PC Protect is a security application that combines protection against malicious threats along with an easy to use optimization tool.",\
    "Cisco Systems Services is an application used for provisioning of all services related to Cisco Systems, Inc., an American technology company headquartered in the center of Silicon Valley.",\
    "The Sparkasse Banking application is the online banking app of Sparkassen- Finanzgruppe, a German group of banks, public insurance providers, and financial service providers.",\
    "HolaSpark is a service which makes video content dynamic, for example from static thumbnails to dynamic video previews, for integrated applications.",\
    "Microsoft Bing is a web search engine provided by Microsoft Corporation.",\
    "TestMy-net is an internet connection quality measurement tool.",\
    "Fuelio is an application used to track mileage, gas consumption and gas costs of vehicles.",\
    "SpeedOf-Me is an internet connection quality measurement tool.",\
    "Xfinity is an application used for provisioning of all services related to the Comcast Cable Communications, LLC, a subsidiary of Comcast Corporation.",\
    "Xfinity Speedtest is an internet connection quality measurement tool by Xfinity.",\
    "Paycentive is a loyalty solutions provider, which enables debit or credit cards to be used as loyalty cards.",\
    "Rainbow Six Siege (full name: Tom Clancy's Rainbow Six Siege) is an online video game published by Ubisoft. It is a tactical shooter game that puts high emphasis on environmental destruction and cooperation between players.",\
    "JPMorgan Chase is an American company that provides banking and financial investment services.",\
    "Multi Media LLC is a full-service provider of live streaming video platforms and online community forums. Their most famous Trademark is Chaturbate, a commercial adult webcam site.",\
    "The Capital One application is a top used banking website in the american region provided by the Capital One Group, a North American banking and financial services corporation.",\
    "Sygic Travel Maps is a trip planner application designed for travelers.",\
    "Glassbox is an Israeli software company, that sells session-replay analytics software and services.",\
    "Stark Games Stark Games is a free-to-play game development studio located in Minsk, Belarus. Its two most important games are Edges of Reality and Magic Seasons.",\
    "Wells Fargo is a multinational financial service provider from the US and one of the worlds largest banks.",\
    "The Blinkist application is a book-summarizing service that provides text or audio content.",\
    "Games Workshop is an application used for provisioning of all services related to Games Workshop Group, a British manufacturer of miniature wargames.",\
    "Warhammer Gaming is an application used for provisioning of all services related to Warhammer, a series of games by Games Workshop.",\
    "Redfast is an active user personalization system for subscription and engagement businesses.",\
    "Samsung Health is an application used to track various aspects of daily life contributing to well being such as physical activity, diet and sleep.",\
    "SpeedSmart provides a real-world internet speed test that covers download, upload, ping and jitter tests and contains real-time graphs to show internet connection consistency.",\
    "Sygic GPS Navigation is an application that provides offline navigation for its users.",\
    "Comcast Technology Solutions is a unit of Comcast that targets multiscreen video distribution and ad-tech services.",\
    "ViuTV is an application for streaming live and on demand video content of the free TV station based in Hong Kong with the same name.",\
    "Demandbase is a cloud-based ad targeting and web personalization platform. Its API returns firmographic data for a specific IP address or cookie.",\
    "Rainbow Six Extraction is a multiplayer tactical shooter online video game published by Ubisoft.",\
    "Warhammer TV is a video streaming service with videos about Warhammer, a series of games by Games Workshop.",\
    "Disney Hotstar is a mobile video streaming application for Indian entertainment, TV shows, live cricket, news, and movies.",\
    "Wunderkind (formerly BounceX) is an online marketing analytics platform that enables its clients to track web traffic as they convert new customers.",\
    "Might And Magic Chess Royale is a massive battle royale game published by Ubisoft.",\
    "Pulse Insights provides a cloud-based microsurvey platform that helps clients to get unique insights to maximize customer experience and conversion rates.",\
    "Radarbot is an application that provides GPS navigation and receive real-time speed camera alerts.",\
    "AdsWizz provides effective monetization and personalization services of digital audio content by providing a complete suite of advertising and analytics solutions.",\
    "Bright Data is a business proxy network and web data platform.",\
    "Wakanim is a video-on-demand streaming service that specialises in the online streaming of Japanese anime series.",\
    "Etsy is an American e-commerce company.",\
    "Vanilla is a provider of branded community software.",\
    "Citadel Colour is an application by Games Workshop, that provides painting guides for Warhammer figurines.",\
    "Behaviour Services is an application used for provisioning of all services related to Behaviour Interactive Inc, a Canadian video game company.",\
    "Dead by Daylight is an asymmetric survival horror game published by Behaviour Interactive.",\
    "Black Library is an application by Games Workshop that provides books and audiobooks about Warhammer.",\
    "Digital Turbine is an application used for provisioning of all services related to Digital Turbine, Inc. and its subsidiaries.",\
    "DuckDuckGo is a privacy focused web search engine.",\
    "LiveOne is a music streaming platform which allows users to listen to music, tune into podcasts, and watch video and live streaming performances.",\
    "Josh is an Indian photo and video creation and editing application.",\
    "BaBe (Baca Berita) is an Indonesian news recommendation platform.",\
    "Wikimedia Foundation, Inc. is a nonprofit organization that owns and operates the Wikimedia projects.",\
    "Avast VPN Services is an application used for provisioning of all services related to VPN applications developed and published by Avast.",\
    "OpenWeb is a social engagement platform that monitors and filters online comment sections.",\
    "Streamio is an online video platform that handles everything around publishing and distributing video and live stream.",\
    "TMT Broadcasting is a German company that provides services for digital audio and video transmission, as well as broadcast processing.",\
    "UNO is a card matching game published by Ubisoft.",\
    "Verve Group is an omnichannel ad platform offering programmatic solutions that connect advertisers and publishers to people in real-time.",\
    "McAfee Security provides security services such as VPN and system scans.",\
    "Lokalise is a localization and translation management service.",\
    "Brainly is a peer-to-peer learning application where users can ask and answer homework questions.",\
    "ZONER AntiVirus is a Czech antivirus solution.",\
    "ZONER Services is an application used for provisioning of all services related to ZONER software a.s., a Czech software company which provides internet services, photographic software, online security, and publishing solutions.",\
    "Magenta TV is a video streaming service provided by Deutsche Telekom AG.",\
    "Lark is a collaboration tool, that integrates messaging, video conferencing, calendars, email, and other applications.",\
    "CapCut is a video editing application.",\
    "Podbean is a service for hosting, distribution and monetization of podcasts.",\
    "Permutive provides a data management platform for publishers, that allows the monetization of advertisements in digital media.",\
    "Wistia is a video platform that helps businesses to create, host, manage, share and measure videos.",\
    "Rappi is a consumer tech company that is specialized in providing online delivery services.",\
    "KION is a popular Russian subscription-based streaming service.",\
    "Okko is one of the largest video-on-demand online services in Russia.",\
    "Picsart is an editor application for photos and videos, that allows users to create and share collages and own designs with filters, effects and stickers.",\
    "MX TakaTak is a short video and social networking application similar to tiktok.",\
    "Sender is a tool for businesses to automate the sending of messages.",\
    "F-Secure is an application used for provisioning of all services related to the F-Secure Corporation, a global cyber security and privacy company.",\
    "Aarogya Setu is an Indian COVID-19 contact tracing, syndromic mapping, and self-assessment mobile application.",\
    "Aniview provides end-to-end video advertising and monetization solutions.",\
    "Pinduoduo is an online marketplace for agricultural products in China.",\
    "Gearbox Entertainment Services is an application used for provisioning of all services related to the Gearbox Entertainment Company, LLC., a creator and distributor of transmedia entertainment.",\
    "Gearbox Shift is a customer service system for gamers that provides a store, an achievement system and connection to other gamers.",\
    "Age of Empires is a series of real-time strategy video games in historical settings.",\
    "Relic Entertainment is a Canadian video game developer specialized in real-time strategy games.",\
    "Xbox Game Studios (formerly known as Microsoft Game Studios) is an American video game publisher best known for the development and publishing of video games for Microsoft Windows.",\
    "Split is a feature delivery platform that powers feature flag management, software experimentation, and continuous delivery.",\
    "Truth Social is a microblogging service and social media platform by Trump Media and Technology Group.",\
    "Rumble is an online video platform.",\
    "Fandom (formerly known as Wikia) is a wiki hosting service designed for fans of games, movies, and entertainers.",\
    "Uscreen is a platform for online video monetization and distribution.",\
    "Hyperconnect is a South Korean social discovery and video technology company.",\
    "Font Awesome is a font and icon toolkit used by web designers and developers.",\
    "Ruby Game Studio is an application used for provisioning of all services related to Ruby Game Studio, a Turkish mobile game developer and publisher.",\
    "MoEngage is a customer engagement platform that analyzes users behavior and enables personalized communication.",\
    "Lost Ark is a fantasy massively multiplayer online action role-playing game (MMORPG).",\
    "Miro is a collaborative online whiteboard.",\
    "Google TV is a streaming service and entertainment hub that is available on a range of smart devices.",\
    "Cadent is a platform that provides advanced TV advertising solutions.",\
    "Pangle is a mobile app advertisement service and related to TikTok for Business.",\
    "Embrace is an automated bug tracking and performance monitoring platform for mobile applications.",\
    "StreamGuys is a platform that provides several multimedia-related services like monitoring, hosting, monetization and live-streaming of podcasts and other media.",\
    "Holler is an application that provides animated content like emojis, stickers and GIFs for messaging.",\
    "Easybrain is an application used for provisioning of all services related to Easybrain, a Cypriot mobile game developer.",\
    "Easybrain Music Maker is an application used for provisioning of all services related to mobile music creation applications by Easybrain.",\
    "Creative Assembly is a British video game developer, that is most famous for developing games of the Total War series.",\
    "Sega is a Japanese video game and entertainment company.",\
    "Match Group is a US company that owns the worlds largest portfolio of online dating services.",\
    "Total War is a series of turn-based and real-time tactics video games in historical or fictional settings.",\
    "Feral Interacitve is a British video game developer, that specializes in porting games to different platforms. It is most famous for developing games of the Total War Series, and the Batman Arkham Series.",\
    "LiveBoard is an online whiteboard application where users can simultaneously edit lesson documents.",\
    "Naraka Bladepoint is a battle royale game with martial arts inspired melee combat between up to 60 players.",\
    "NetEase Games is a provider of self-developed mobile and PC-games.",\
    "Patreon is a US membership platform that allows subscribers to support creators financially in exchange for perks and rewards.",\
    "United States Postal Service is an organization that provides postal services in the USA. It is an independent agency of the US federal government.",\
    "IAB Europe (Interactive Advertising Bureau Europe) is the European-level trade association for the digital marketing and advertising ecosystem.",\
    "United Parcel Service (UPS) is an American company that provides shipping and receiving postal services worldwide.",\
    "Q4 Inc. is a platform for capital markets communication.",\
    "Tidio is a chat application that allows companies to connect with customers.",\
    "Warner Bros Discovery Services is an application used for provisioning of all services related to Warner Bros. Discovery, Inc., an American multinational media and entertainment company.",\
    "Lazada is the most used e-commerce platform in Southeast Asia.",\
    "OnlyFans is a membership platform that allows subscribers to financially support creators. The service is based in London.",\
    "Primis is a global video distribution platform that increases revenue for publishers.",\
    "Blue Billywig is an online video platform from the Netherlands that enables management, hosting, publishing and analysis of video content.",\
    "Geolocation Technology, Inc. offer web services like providing intellectual property geolocation, intellectual property address transfer, web content production, web marketing, and other services.",\
    "Lumen Technologies (formerly CenturyLink) is an American company that provides network services, security, cloud solutions, and managed services.",\
    "Fruitshake is a Russian company that develops free-to-play HTML5 games for social networks like VK and Odnoklassniki as well as mobile devices.",\
    "Improve Digital is a company that provides advertisement and monetization solutions for video, web, mobile and game publishers.",\
    "Terminus is a marketing platform used to target companies, engage decision-makers on their terms, and accelerate pipeline velocity.",\
    "Tableau is a data visualization, analytics and business intelligence platform.",\
    "Tiny Tinas Wonderlands is an action role-playing video game and is part of the Borderlands game series.",\
    "AG Projects Services is an application used for provisioning of all services related to AG Projects, a provider of real-time communication systems based on SIP protocol.",\
    "Display-io is an in-app advertising monetization platform providing advertising solutions for publishers and advertisers.",\
    "BeeTalk is a mobile dating and friendship app that allows users to connect to other people who are near them.",\
    "Zillow is a real estate application that allows users to get information about properties and make appointments to buy, sell or rent a home.",\
    "Plurk is a Taiwanese social networking and micro-blogging service with a horizontal timeline.",\
    "InMobi is an Indian mobile advertisement technology company.",\
    "Arc XP is a software-as-a-service (SaaS) platform which offers news-oriented content management for publishers and media companies worldwide.",\
    "Embracer Group AB is a media and video game holding company. The headquarter of this company is located in Sweden.",\
    "Spin The Bottle is a mobile dating game application in which users can kiss each other, send gifts, share music, videos and use text chat for social communication.",\
    "Xandr (formerly AppNexus) is an advertising and analytics platform for buying and selling consumer-centric digital advertising.",\
    "Mapp is a provider of cloud-based digital marketing solutions.",\
    "9GAG is a social media platform for sharing internet memes, GIFs and videos.",\
    "Square Enix is a Japanese video game developer and publisher.",\
    "Final Fantasy XIV is a popular online roleplaying game, that is based on the related Final Fantasy video game series.",\
    "Naver Works is a full suite of services, including mail, calendar, drive, and others needed for work, with a special focus on calls and messaging.",\
    "Naver Corporation is a South Korean internet conglomerate.",\
    "Intuit is a company that offers financial management software.",\
    "Gazprom-Media Holding is the Russian market leader in television, film, production and distribution, digital media, radio, press and advertising.",\
    "TNT-Premier was a Russian streaming service provider with exclusive content in various genres and formats. It was rebranded as Premier in 2019. Therefore, all TNT-Premier related traffic will be classified as Premier.",\
    "Radancy, formerly known as TMP Worldwide Advertising and Communications, LLC, is a company that provides SaaS for recruitment and talent acquisition.",\
    "Rambler is a Russian search engine and one of the biggest Russian web portals, offering e-mail, news aggregation, e-commerce and other services.",\
    "Crypto-com is a cryptocurrency exchange that provides a platform for trading various cryptocurrencies.",\
    "FIFA Soccer is a mobile multiplayer soccer game that is part of the popular FIFA video game series.",\
    "Kustomer is a Software-as-a-Service (SaaS) CRM platform specializing in customer service and support.",\
    "LogRocket is a service that records videos of user sessions with logs and network data, identifying problems and revealing the root cause of bugs.",\
    "SpotHero is a digital marketplace that connects drivers looking to reserve and pay for parking spaces with parking lots and similar service providers.",\
    "Adobe Experience Cloud is a collection of marketing and analytics products by Adobe, such as Adobe Analytics, Adobe Advertising, Adobe Commerce, Marketo or the Adobe Experience Manager.",\
    "ICF Technology provides live streaming and processing services to multiple platforms for broadcasting globally.",\
    "Tricolor TV is a popular Russian subscription-based streaming service.",\
    "Redfin is a US real estate and house finder application that allows users to book a free tour with a Redfin agent to see objects of interest in a short amount of time.",\
    "Matterport is a company that sells 3D capturing technology, targeted towards the real estate industry.",\
    "Naver Cloud is a South Korean corporation providing IT infrastructure and enterprise solutions for Naver and its subsidiaries.",\
    "Rogue Company is a free-to-play multiplayer tactical third-person hero shooter video game.",\
    "WOW TV is a subscription-based streaming service for movies and series.",\
    "Quora is a platform where users can ask questions and get answers from other users as well as sharing their own knowledge.",\
    "Infobip is a customer engagement and support service.",\
    "Edgio, Inc. (a merger of the two former companies Limelight Networks and Edgecast) is a provider of CDN services used for delivery of digital media content and software as well as video streaming services.",\
    "Flipkart is India's leading e-commerce marketplace.",\
    "Near delivers dynamic advertising solutions for companies across retail, automotive, entertainment, and consumer markets.",\
    "Avo is a data governance platform that error-proofs analytics events and speeds up implementation.",\
    "Contentsquare provides an experience analytics platform that tracks and visualizes digital behaviors, delivering intelligent recommendations.",\
    "Fall Guys is a cross-platform battle royale game in which players compete against each other in a series of mini-games.",\
    "Yahoo Ad Tech is a global media and advertising company. It offers advertisement, content publishing, campaigning, monetization, and resource information services.",\
    "Meesho is an Indian e-commerce platform.",\
    "Tower of Fantasy is an Anime-influenced shared open world MMORPG.",\
    "nPerf is a speedmeter and network coverage testing service.",\
    "NBCUniversal is an application used for provisioning of all services related to NBCUniversal Media, LLC, one of the world's leading media and entertainment companies.",\
    "Naver Map is a South Korean GPS navigation application.",\
    "Snapdeal is an Indian e-commerce company for fashion, technology, home and general merchandise.",\
    "Smartclip is a multi-screen and brand advertising company based in Germany that distributes video ad formats across all devices and platforms.",\
    "DigitalOcean provides a cloud platform to deploy, manage, and scale applications of any size.",\
    "happn is a location-based online dating application.",\
    "2K is an American video game publisher.",\
    "Microsoft Family Safety is an application that helps parents to create a safe space for their kids online and to block inappropriate content.",\
    "Rumbleverse is a free-to-play battle royale wrestling action game.",\
    "WB Games (formerly Agora Games, Inc.), is a video game development company.",\
    "Microsoft-Defender is an antivirus solution included in Windows by default.",\
    "Adobe Fonts is an online font library that can be used directly on websites or synced via Adobe Creative Cloud to applications.",\
    "Vimond Media Solutions is a provider of an OTT streaming platform with scalable backend APIs.",\
    "AIONCLOUD is a platform that provides global cloud security services.",\
    "AMC Networks is an American entertainment company.",\
    "AMC Plus is an American video on demand service with subscription.",\
    "Cabify is a transportation and mobility application that can be used to hire vehicles, order private car rides, or send things from one area of the city to the other.",\
    "Paytm is an Indian online payment platform.",\
    "Lenovo Services is an application used for provisioning of all services related to Lenovo, a multinational technology company specializing in designing, manufacturing, and marketing consumer electronics, personal computers, software, business solutions, and related services.",\
    "Uplive is a social media platform that offers video livestreaming and casual games.",\
    "Naver Mail is one of the most popular email services in Korea.",\
    "Akamai Technologies is an application used for provisioning of all services related to Akamai Technologies, Inc., an American company providing web services like CDN, cloud and cybersecurity.",\
    "StackAdapt provides a self-serve advertising platform that specializes in multi-channel solutions including video, audio, and display ads.",\
    "Hi-Rez Studios is an American game developing corporation.",\
    "ViX is a Spanish-language OTT video streaming service.",\
    "Tremor Video is a video advertising company with expanded offerings in Data-Driven TV and All-Screen Video.",\
    "Block Inc., is an American multinational technology conglomerate.",\
    "Cash App is a mobile payment and digital wallet application.",\
    "Outfit7 is an application used for provisioning of all services related to the slowenian video game development company.",\
    "Mythic Legends is a competitive auto-battler strategy game.",\
    "United Internet AG Services is an application used for provisioning of all services related to the United Internet AG, a global internet services company headquartered in Germany.",\
    "GMX (Global Message eXchange) Mail is a free advertising-supported email service.",\
    "Inshorts is a news application that selects news from multiple sources and summarises them to present in a short and 60 words or less format.",\
    "Revolut is a British financial company offering a lot of digital banking services.",\
    "Zepeto is a Korean chat and meet application that uses 3D avatars.",\
    "Diablo Immortal is a free-to-play massively multiplayer online action role-playing game (MMORPG) developed by Blizzard Entertainment and NetEase.",\
    "Life360 is an American provider for location-based services. They also have their own social networking application.",\
    "Session is an end-to-end encrypted messenger with VoIP functionalities that minimizes sensitive metadata.",\
    "Premier is a Russian streaming service provider with exclusive content in various genres and formats.",\
    "Tencent Cloud, also known as QCloud, is a Chinese provider of secure, reliable and high-performance cloud compute services.",\
    "PayPay is a Japanese mobile payment application.",\
    "6Sense Insights Inc. is a software company that provides data analytics and marketing services through its own predictive B2B intelligence engine.",\
    "Vimeo OTT is a service for providers of video content to easily reach their audience, including distribution, analytics and different monetization models.",\
    "Zeus Network (also known as Zeus) is an American video streaming service.",\
    "Vidazoo is a video content and yield management platform that offers video monetization solutions for advertisers and publishers.",\
    "HIDIVE is a subscription-based video on demand service specialized in streaming anime.",\
    "Grupo Globo Services is an application used for provisioning of all services related to Grupo Globo, a Brazilian mass media conglomerate.",\
    "Globoplay is a subscription-based video on-demand OTT streaming service.",\
    "Shudder is an American video on demand service with subscription for horror movies and series.",\
    "Kingsoft Services is an application used for provisioning of all services related to Kingsoft Corporation, a Chinese company based in Beijing.",\
    "Kingsoft Cloud, is the largest independent cloud service provider in China.",\
    "Kobo (anagram of book) is a website and application for buying ebooks, audiobooks, e-readers, and tablets.",\
    "For Honor is a fighting game that allows players to play the roles of historical forms of soldiers and warriors such as knights, samurais, and vikings.",\
    "Tremor International is an application used for provisioning of all services related to Tremor International Ltd., a publicly traded advertising-technology company.",\
    "Financial Times is a British daily newspaper that focuses on business and economic current affairs.",\
    "Kape Services is an application used for provisioning of all services related to Kape Technologies, a digital security software provider.",\
    "Square is a payments platform for small and medium businesses that enables them to accept credit card payments.",\
    "hCaptcha is an anti-bot solution that protects user privacy and rewards websites.",\
    "AMC SVOD is an application used for provisioning of all services related to AMCs branded subscription streaming video services.",\
    "Sundance Now is an American subscription-based streaming service specialized in streaming movies related to the Sundance Film Festival.",\
    "Naver NOW (previously Naver TV) is a South Korean video streaming service that provides live as well as on-demand content.",\
    "Australia Post is an application that helps users in Australia to track and manage their deliveries, sign up for Parcel Lockers and provides other post-related services.",\
    "The Bungie Games application covers all traffic related to Bungie, Inc., an American video game company.",\
    "Ora TV is an onDemand video network that offers free television shows streaming online.",\
    "CCP Games is an application used for provisioning of all services related to the Icelandic video game development company.",\
    "EVE Online is a space and science fiction based free-to-play massively multiplayer online role-playing game (MMORPG).",\
    "Zype provides a cloud-based platform to manage and distribute video content.",\
    "Umeng Plus is an online statistical analysis platform for mobile applications.",\
    "BYJUS is an Indian multinational educational technology company that offers various educational web-courses and applications for students of different ages.",\
    "Tilda is a website building tool and hoster.",\
    "CDN77 is a british CDN and cloud service providing high demand website and app access.",\
    "Tapclap is an application used for provisioning of all services related to Tapclap, a mobile game developer.",\
    "Pirate Treasures is an episode-based treasure hunt match 3 mobile game.",\
    "Prismic offers a headless content management system for editing online content.",\
    "Tencent Music is an application used for provisioning of all services related to Tencent Music, a provider of music streaming services for the Chinese market.",\
    "Httpool is a provider of integrated digital advertising solutions and marketing services.",\
    "CacheFly is a secure video delivery and cloud solution provider.",\
    "Alibaba Group Services is an application used for provisioning of all services related to the Alibaba Group Holding Limited, a Chinese multinational technology company that specializes in E-commerce and retail, cloud computing, as well as entertainment.",\
    "QANDA is a learning platform that scans math problems and provides step-by-step solutions.",\
    "Vercel is a frontend development and delivery tool.",\
    "Streamotion is an Australian streaming service provider. Their portfolio includes the entertainment streaming service BINGE, the sports streaming service Kayo Sports and the news streaming service Flash.",\
    "DraftKings Services is an application used for provisioning of all services related to DraftKings Inc., an American sports betting and fantasy sports company.",\
    "DraftKings Casino is an application and website on which users can play various virtual casino games like blackjack, poker, roulette and more to win real money.",\
    "DraftKings Sportsbook is a sports betting application and website that allows users to wager on all kinds of different teams, sports and events.",\
    "Wix provides cloud-based web development services that allow users to create websites and mobile sites.",\
    "FanDuel Services is an application used for provisioning of all services related to the FanDuel Group, an American gambling company offering sportsbook and fantasy sports services.",\
    "FanDuel Casino is an application that allows users to play various online casino games like poker or roulette in a legal and regulated environment for real money.",\
    "FanDuel Sportsbook is an application that allows users to place bets on various teams, players or matches in different major leagues like the NBA, NHL, MLB and NFL.",\
    "Cryptic Studios is an American video game developer specializing in MMORPGs.",\
    "Neverwinter is a free-to-play fantasy MMORPG.",\
    "Resso Music is an Indian music streaming platform that has some additional social features like sharing lyrics, comment on titles or streaming own content.",\
    "Xumo is an American OTT Service.",\
    "Realm Royale is a free-to-play third-person shooter battle royale game.",\
    "Vikingard is a role-playing game (RPG) with a mix of base-building and kingdom management elements.",\
    "Badlanders is a third-person immense battlefield action game.",\
    "Infinite Lagrange is a space strategy mobile game developed by NetEase Games.",\
    "ViewLift is an OTT streaming service and cloud based CMS provider.",\
    "TVNZ (Television New Zealand) is a television network that is broadcast throughout New Zealand and parts of the Pacific region.",\
    "Wynk Music is a free indian based music and podcast streaming service.",\
    "Gearbox Publishing is an application used for provisioning of all games services published by itself.",\
    "Viju is a russian movie and series streaming service.",\
    "Viasat Ru is an application used for provisioning of all services related to the Viasat Global LLC, a Russian media company.",\
    "LilithGames is a Chinese mobile game developing corporation.",\
    "Rise of Kingdoms is a free-to-play strategy game.",\
    "DECA Games is a German game publisher focused on acquiring older free-to-play games and afterwards ensure operation continues.",\
    "Gods and Glory is a game with the goal to build a fantasy empire and fight against other players.",\
    "Podcast Addict is an application that distributes podcasts, live radio, RSS feeds, and audio books.",\
    "EVE Echoes is a spaceship massively multiplayer online game.",\
    "Lazy Audio is a Chinese audiobook platform.",\
    "Amediateka is a Russian streaming service. The service offers exclusive access to HBO content.",\
    "Mixi is an online Japanese social networking service where users can talk about their hobbies in various types of communities such as music, games or actors.",\
    "Mixi Services is an application used for provisioning of all services related to Mixi Inc., a Japanese company mainly engaged in the digital entertainment, sports and lifestyle space.",\
    "Lord of the Rings War (also known as Rise to War) is a mobile game in which players must build their armies and strategically expand their territory in pursuit of the One Ring.",\
    "Google Translate is an application for multilingual text, photo and video translation.",\
    "NTT Services is an application used for provisioning of all services related to The Nippon Telegraph and Telephone Corporation, a Japanese telecommunications company.",\
    "Radiko is a Japanese IP simultaneous radio service.",\
    "CyberAgent Services is an application used for provisioning of all services related to CyberAgent Inc., a Japanese digital advertising company.",\
    "Ameba is a Japanese blogging and social networking application.",\
    "RedHotPie is an application used for dating and chatting.",\
    "The Clicktouch Games application covers all traffic related to Clicktouch Co., Ltd., (also known as Dianchu Technology), a Chinese mobile game company.",\
    "Adtech Studio is a Japanese provider of advertisement technology.",\
    "MediaLab AI Services is an application used for provisioning of all services related to MediaLab AI Inc., a US-based company that designs and develops mainly messaging application software.",\
    "Mitene is a brand by Mixi that offers several apps for parents and families, the most important being FamilyAlbum.",\
    "Transperth provides public transport information for Perth, Australia.",\
    "Hinge is a dating application which aims towards a younger demographic.",\
    "SuperAwesome is a company that provides different tools and services for a safe digital engagement with kids and teens, like delivering responsible digital marketing campaigns for young people or providing a parent verification and consent platform.",\
    "Yandex Translate is a Russian application for multilingual text, image, and voice translation.",\
    "Cashi is a dedicated reward and payment system for Mexican Walmart subsidiaries.",\
    "Booking Holdings Services is an application used for provisioning of all services related to Booking Holdings Inc., an American travel technology company owning several travel aggregators and metasearch engines.",\
    "Banco Azteca is a Mexican bank chain.",\
    "GStatic is a service by Google used to load and retrieve static content from their Content Delivery Network.",\
    "DiDi Food is a food ordering and delivery application.",\
    "DiDi Rider (also DiDi Passenger) is an application that provides private transportation services for travel and parcel delivery.",\
    "Facebook Messenger is an instant messaging app and platform used to send messages and exchange photos, videos and other media.",\
    "Coppel is a Mexican department store chain and online retailer.",\
    "The engine finished processing this flow without a classification result. This can also be triggered by reaching the packet limit or the packet timeout before establishing a reliable classification. The packet limit and timeout can be configured via stage 3 configuration options.",\
    "OkCupid is a free dating app that features multiple-choice questions to match members.",\
    "Mercado Libre is an online marketplace for e-commerce and auctions in Latin America.",\
    "BAND is a mobile community application for group communication.",\
    "Mercado Pago is a payment platform for online sales in Latin America.",\
    "Smilegate is a Korean video game developer and publisher.",\
    "BanCoppel is a Mexican bank offering full banking services.",\
    "V4 is a fantasy massively multiplayer online role-playing game (MMORPG).",\
    "VLMedia Services is an application used for provisioning of all services related to VLMedia, a Turkey-based company that designs and develops social applications and games.",\
    "Microsoft Translator is an application for multilingual text, voice, photo and video translation.",\
    "CDNJS is a CDN service hosting Javascript, CSS libraries and other free and open-source software.",\
    "Telemetro is a television network from Panama.",\
    "Corporacion Medcom is an application used for provisioning of all services related to Corporacion Medcom, a mass media company from Panama.",\
    "Here TV is an American television network targeting lesbian, gay, bisexual, and transgender (LGBT) audiences.",\
    "Overwatch is a series of online multiplayer first-person shooter (FPS) video games that feature hero-based combat between two teams of players.",\
    "Snow Services is an application used for provisioning of all services related to the SNOW Corporation, a South Korean company that produces photo and video editing software.",\
    "Mastodon is a decentralized social network.",\
    "Glu Mobile Services is an application used for provisioning of all services related to Glu Mobile LLC, an American gaming company specializing in video games for mobile phones and tablet computers.",\
    "KartRider Rush is a mobile kart racing game.",\
    "GroupM is an application used for provisioning of all services related to GroupM, a global media investment management company comprised of different media agencies.",\
    "UltraViewer is a remote desktop application for computer remote control.",\
    "Kingdom Age is a mobile game where players can create their own fantasy world, full of castles and competitive siege warfare.",\
    "Star Trek Online is a free-to-play sci-fi online MMORPG.",\
    "WEB-DE is Germany's leading communications platform, covering search, cloud and entertainment services as well as its own e-mail service called FreeMail.",\
    "mail-com is a web portal and e-mail service mainly addressing international users. It is supplemented by a search engine, editorial news and cloud applications.",\
    "ARTE is an European public service channel dedicated to culture.",\
    "VITA is a video editing application.",\
    "Paramount Services is an application used for provisioning of all services related to Paramount Global, an American multinational mass media and entertainment conglomerate.",\
    "BluTV is a subscriber video on-demand entertainment platform for Turkish and Arabic-speaking audiences.",\
    "Xiaohongshu is a Chinese social media application similar to Instagram.",\
    "Rakuten TV is a video-on-demand (VOD) streaming service, offering movies and TV series for subscription, rental and purchase.",\
    "Apple Weather is a weather forecast service provided by Apple.",\
    "MUBI is a global streaming platform for curated films.",\
    "QuantumCast is an online broadcasting system for audio program creation and streaming.",\
    "Knights and Dragons is an action role-playing game featuring battles against mythical creatures.",\
    "Smite is a free-to-play multiplayer online battle arena game.",\
    "Teleport Media is a P2P CDN for online TV and VOD broadcasters.",\
    "CBS News is a website and application provided by CBS used for watching live and on-demand news content from all over the world.",\
    "CBS Sports is a website and application provided by CBS used for reading and watching live sports news and scores. In CBS Sports Fantasy users can create and manage their own imaginary sports teams.",\
    "RBC News is a Russian online news portal.",\
    "Canonical Services is an application used for provisioning of all services related to Canonical Ltd., a UK-based company which supports, secures and manages Ubuntu services and infrastructure.",\
    "Exxen is a Turkish subscription streaming service that produces its own content, as well as owning various streaming rights of foreign productions and sports events.",\
    "Sony Group Services is an application used for provisioning of all services related to Sony Group Corporation, a Japanese multinational conglomerate corporation doing business in consumer and professional electronics, gaming, entertainment, and financial services.",\
    "Soul Hunters is a free-to-play action-strategic RPG.",\
    "Adtelligent Inc. is an advertising technology company providing solutions for publishers and advertisers.",\
    "Datadog is a monitoring service for cloud applications through a SaaS-based analytics platform.",\
    "Segmento-ru is a Russian platform which employs machine learning to target digital advertising.",\
    "Polyfill-io is a service which accepts a request for a set of browser features and returns only the polyfills that are needed by the requesting browser.",\
    "B612 is camera application with photo and video editing capabilities.",\
    "Warpath Ace Shooter is a free-to-play strategy game.",\
    "Tower of Saviors is a mobile game that combines match 3 and RPG game mechanics with characters from different mythologies.",\
    "Lemon8 is a social media application similar to TikTok.",\
    "Coveo is an application that uses AI-driven analysis to increase the relevance of search, content recommendations and personalized experiences for its enterprise clients.",\
    "Hypic is a photo editing and AI art application.",\
    "Escape from Tarkov is an online multiplayer tactical first-person shooter game.",\
    "Pocket is a social bookmarking service for storing, sharing and discovering web bookmarks.",\
    "Marvel Snap is a digital card game based on the Marvel universe.",\
    "Adobe Document Cloud is a service from Adobe used to store PDF files in the cloud and to access them remotely.",\
    "Zangi Messenger is a free messaging application that allows users to perform voice and video calls, instant messaging, group chats and file sharing.",\
    "Beamable provides implementation services of Games as a Service features within the Unity game engine.",\
    "bunny-net is a CDN that specializes in delivering content with high-speed around the world.",\
    "BCE Services is an application used for provisioning of all services related to Bell Canada Enterprises Inc., a Canadian media provider.",\
    "Crave is a Canadian-based streaming service for movies and series.",\
    "BytePlus is an application that offers content delivery, data intelligence, personalization and video solutions.",\
    "Plarium is an application used for provisioning of all services related to Plarium Global Ltd., a video game developer and publisher based in Israel.",\
    "Sparta War of Empires is a massive multiplayer strategy video game set in 5th Century BCE Greece.",\
    "Sber is a Russian banking and financial services provider.",\
    "Jivochat is an omnichannel business messenger that is designed for e-commerce websites to talk to their customers in real-time.",\
    "Youtube Kids is a dedicated YouTube streaming service for Kids with parental control.",\
    "Oppo Mobile Services is used for provisioning of all services related to Oppo, a Chinese consumer electronics manufacturer.",\
    "HeyTap is an internet service platform with products including an app market, game center and browser.",\
    "Max is a mobile application for streaming TV and movies.",\
    "World Of Warships is an online multiplayer game that is naval warfare-themed.",\
    "Youtube TV is a streaming television service that enables users to watch live and on-demand TV channels.",\
    "Google Family Link is a family parental controls service that allows parents to approve or disapprove apps, set screen times, and restrict content.",\
    "Celtic Heroes is a warcraft and adventure MMORPG.",\
    "Hero Hunters is a team-based online multiplayer action role-playing game.",\
    "Times Internet Services is an application used for provisioning of all services related to Times Internet, an Indian internet technology company.",\
    "ServiceNow is a technology company that provides a cloud computing platform, allowing other businesses to digitalize their work more easily.",\
    "Honkai Star Rail is an Anime-influenced space fantasy role-playing gacha video game.",\
    "Elder Scrolls Online is a massively multiplayer online role-playing game developed by Zenimax Media Inc., a subsidiary of Xbox Game Studios.",\
    "Nuverse is an application used for provisioning of all services related to Nuverse Ltd., a Hong Kong-based game development and publishing company.",\
    "TVPlus (alias TV+) is a digital TV platform launched by Turkcell.",\
    "Nasdaq Services is an application used for provisioning of all services related to Nasdaq, an American stock exchange.",\
    "SESISOFT Services is an application used for provisioning of all services related to SESISOFT Co.,Ltd, a South Korean video game developer.",\
    "Tivibu GO is a video watching portal for Turkish-speaking audiences.",\
    "Dungeon Fighter Online is a free to play online multiplayer 2D action role-playing game.",\
    "5GMARK provides an internet speed test that allows comparision of smartphones, networks or places on 2G, 3G, 4G, 5G and WiFi. It also contains real-time graphs to show internet connection consistency.",\
    "Kill Shot Bravo is a free online first-person-shooter mobile game",\
    "puhutv is a Turkish video streaming service.",\
    "Dogus Services is an application used for provisioning of all services related to Dogus Yayin Grubu, a Turkish mass media conglomorate.",\
    "OpenAI is a service utilizing artificial intelligence to provide chatbot, image-generation, transcription and translation solutions.",\
    "Qobuz is a French music streaming and downloading service.",\
    "S Sport Plus is a Turkish video-on-demand and streaming application for sport events.",\
    "World Of Tanks Blitz is an online multiplayer tank shooter game."

/* Public Prototypes ************************************************************/

#include "pace2_protocols.h"

/**
 * @brief This function provides the used attributes for the given application.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @deprecated The usage of this function is deprecated. Please use
 *             @ref pace2_get_number_of_application_attributes_of_application and
 *             @ref pace2_get_application_attribute_of_application_by_index instead.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   application_id          The application ID.
 * @param[out]  application_attributes  Pointer to bitmask of type @ref PACE2_application_attributes_bitmask "PACE2_application_attributes_bitmask".
 * @retval      PACE2_SUCCESS           @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_PARAM_MISSING     @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval      PACE2_FAILURE           @copydoc PACE2_return_state::PACE2_FAILURE
 *                                      The given application id does not exist.
 */
PACE2_DEPRECATED(
    "Deprecated, Please use pace2_get_number_of_application_attributes_of_application and "
    "pace2_get_application_attribute_of_application_by_index instead.")
PACE2_return_state pace2_get_application_attributes_of_application(PACE2_module * const pace2_module,
                                                                   int const thread_ID,
                                                                   PACE2_applications const application_id,
                                                                   PACE2_application_attributes_mask * const application_attributes);

/**
 * @brief This function provides the short string for the given application.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module                    Constant pointer to the PACE 2 module.
 *                                              Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID                       The ID of the thread currently processing.
 *                                              Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   application_id                  The application ID.
 * @returns     The applications short string or NULL when one of the input parameters is invalid.
 */
const char * pace2_get_application_short_string(PACE2_module * const pace2_module, int const thread_ID,
                                                PACE2_applications const application_id);

/**
 * @brief This function provides the long string for the given application.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   application_id          The application ID.
 * @returns     The applications long string or NULL when one of the input parameters is invalid.
 */
const char * pace2_get_application_long_string(PACE2_module * const pace2_module, int const thread_ID,
                                               PACE2_applications const application_id);

/**
 * @brief This function provides the description for the given application.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   application_id          The application ID.
 * @returns     The applications description or NULL when one of the input parameters is invalid.
 */
const char * pace2_get_application_description(PACE2_module * const pace2_module, int const thread_ID,
                                               PACE2_applications const application_id);

/**
 * @brief This function provides the used applications by the given protocol.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   protocol_id             The protocol ID.
 * @param[out]  applications            Pointer to applications of type @ref PACE2_applications "PACE2_applications".
 * @param[out]  number_of_applications  Pointer to number_of_applications of type @ref u32.
 * @retval      PACE2_SUCCESS           @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_FAILURE           @copydoc PACE2_return_state::PACE2_FAILURE
 *                                      The given protocol id does not exist.
 */
PACE2_return_state pace2_get_used_applications_of_protocol(PACE2_module * const pace2_module,
                                                           int const thread_ID,
                                                           PACE2_protocols const protocol_id,
                                                           PACE2_applications const ** applications,
                                                           u32 * number_of_applications);

/**
 * @brief This function provides the number of available application attributes for the given application ID.
 *
 * @param[in]  pace2_module         Constant pointer to the PACE 2 module.
 *                                  Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]  thread_ID            The ID of the thread currently processing.
 *                                  Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]  application_id       The application ID.
 * @param[out] number_of_attributes The number of available application attributes for @p application_id.
 *
 * @retval     PACE2_SUCCESS        @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval     PACE2_FAILURE        @copydoc PACE2_return_state::PACE2_FAILURE
 */
PACE2_return_state pace2_get_number_of_application_attributes_of_application(PACE2_module * const pace2_module,
                                                                             int const thread_ID,
                                                                             PACE2_applications const application_id,
                                                                             u16 * const number_of_attributes);

/**
 * @brief This function can be used to iterate over the number of application attributes of a specific application. The
 *        number of attributes for this application has to be determined with
 *        @ref pace2_get_number_of_application_attributes_of_application beforehand.
 *
 * @param[in] pace2_module      Constant pointer to the PACE 2 module.
 *                              Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in] thread_ID         The ID of the thread currently processing.
 *                              Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in] application_id    The application ID.
 * @param[in] attribute_index   Index of the attribute.
 *
 * @return The attribute of type enum pace2_application_attributes for the given @p application_id and
 *         @p attribute_index on success. @ref PACE2_APPLICATION_ATTRIBUTE_NONE otherwise and in all error cases.
 */
enum pace2_application_attributes pace2_get_application_attribute_of_application_by_index(
    PACE2_module * const pace2_module,
    int const thread_ID,
    PACE2_applications const application_id,
    u16 attribute_index);

/**
 * @brief This function provides the child applications of the given application.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   application_id          The application ID.
 * @param[out]  applications            Pointer to applications of type @ref PACE2_applications "PACE2_applications".
 * @param[out]  number_of_applications  Pointer to number_of_applications of type @ref u32.
 * @retval      PACE2_SUCCESS           @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_PARAM_MISSING     @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval      PACE2_FAILURE           @copydoc PACE2_return_state::PACE2_FAILURE
 *                                      The given application id does not exist.
 */
PACE2_return_state pace2_get_child_applications_of_application(PACE2_module * const pace2_module,
                                                               int const thread_ID,
                                                               PACE2_applications const application_id,
                                                               PACE2_applications const ** applications,
                                                               u32 * number_of_applications);

/**
 * @brief This function provides the parent application of the given application.
 *
 * The function supports the dynamic upgrade feature. If the library is upgraded
 * to a newer version the function implicitly calls it's upgrade variant.
 *
 * @param[in]   pace2_module            Constant pointer to the PACE 2 module.
 *                                      Only used in case of dynamic upgrade otherwise it can be NULL.
 * @param[in]   thread_ID               The ID of the thread currently processing.
 *                                      Only used in case of dynamic upgrade otherwise it can be 0.
 * @param[in]   application_id          The application ID.
 * @param[out]  parent_application      Pointer to the parent application of type @ref PACE2_applications
 *                                      "PACE2_applications.
 * @retval      PACE2_SUCCESS           @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval      PACE2_PARAM_MISSING     @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval      PACE2_FAILURE           @copydoc PACE2_return_state::PACE2_FAILURE
 *                                      The given application id does not exist.
 */
PACE2_return_state pace2_get_parent_application_of_application(PACE2_module * const pace2_module,
                                                               int const thread_ID,
                                                               PACE2_applications const application_id,
                                                               PACE2_applications * parent_application);

#ifdef __cplusplus
}
#endif
#endif /* PACE2_APPLICATIONS_H */

/*********************************************************************************
 ** EOF
 ********************************************************************************/
