/** @file      pace2_pa_tags.h
  * @brief     PACE 2 component programming interface of the grouping functionality for protocols and applications.
  * @copyright Copyright (C) 2014 by ipoque GmbH, a Rohde & Schwarz company. All rights reserved.
  *            See also the DISCLAIMER file that is part of your delivery. */

#ifndef PACE2_PA_TAG_FUNC_H
#define PACE2_PA_TAG_FUNC_H

#include "pace2_basic_def.h"
#include "pace2_protocols.h"

#ifdef __cplusplus
extern "C" {
#endif

#define PACE2_PA_TAGS_CUSTOM_COUNT  2048
#define PACE2_PA_TAGS_DEFAULT_COUNT 128
#define PACE2_NUMBER_OF_INTERNAL_GROUP_TAG_TYPES 48

#define PACE2_PA_GET_MAIN_TAG_FAILURE 0xFFFF
#define PACE2_PA_GET_SUB_TAG_FAILURE 0xFFFF

#define PACE2_PA_INTERNAL_TAGS(X) \
        X(PACE2_PA_TAG_GENERIC = PACE2_PA_TAGS_CUSTOM_COUNT,    Generic,                                generic)\
        X(PACE2_PA_TAG_P2P,                                     Peer to Peer,                           peer-to-peer)\
        X(PACE2_PA_TAG_GAMING,                                  Gaming,                                 gaming)\
        X(PACE2_PA_TAG_TUNNEL,                                  Tunnel,                                 tunnel)\
        X(PACE2_PA_TAG_BUSINESS,                                Business,                               business)\
        X(PACE2_PA_TAG_E_COMMERCE,                              E-Commerce,                             e-commerce)\
        X(PACE2_PA_TAG_VOIP,                                    Voice over IP,                          voice-over-ip)\
        X(PACE2_PA_TAG_MESSAGING,                               Messaging,                              messaging)\
        X(PACE2_PA_TAG_STREAMING,                               Streaming,                              streaming)\
        X(PACE2_PA_TAG_MOBILE,                                  Mobile,                                 mobile)\
        X(PACE2_PA_TAG_REMOTE_CONTROL,                          Remote Control,                         remote-control)\
        X(PACE2_PA_TAG_MAIL,                                    Mail,                                   mail)\
        X(PACE2_PA_TAG_NETWORK_MANAGEMENT,                      Network Management,                     network-management)\
        X(PACE2_PA_TAG_DATABASE,                                Database,                               database)\
        X(PACE2_PA_TAG_FILETRANSFER,                            Filetransfer,                           filetransfer)\
        X(PACE2_PA_TAG_WEB,                                     Web,                                    web)\
        X(PACE2_PA_TAG_CONFERENCE,                              Conference,                             conference)\
        X(PACE2_PA_TAG_SOCIAL,                                  Social,                                 social)\
        X(PACE2_PA_TAG_SHAREHOSTING,                            Sharehosting,                           sharehosting)\
        X(PACE2_PA_TAG_DEPRECATED,                              Deprecated,                             deprecated)\
        X(PACE2_PA_TAG_INDUSTRIAL,                              Industrial,                             industrial)\
        X(PACE2_PA_TAG_ENCRYPTED,                               Encrypted,                              encrypted)\
        X(PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES,         Advertisement and Analytic Services,    advertisement-and-analytic-services)\
        X(PACE2_PA_TAG_NEWS,                                    News,                                   news)\
        X(PACE2_PA_TAG_HEALTH_FITNESS,                          Health and Fitness,                     health-and-fitness)\
        X(PACE2_PA_TAG_CLOUD_CDN_SERVICES,                      Cloud and CDN Services,                 cloud-and-cdn-services)\
        X(PACE2_PA_TAG_NAVIGATION,                              Navigation,                             navigation)\
        X(PACE2_PA_TAG_FINANCE,                                 Finance,                                finance)\
        X(PACE2_PA_TAG_TRAVEL_TRANSPORTATION,                   Travel and Transportation,              travel-and-transportation)\
        X(PACE2_PA_TAG_PORNOGRAPHY,                             Pornography,                            pornography)\
        X(PACE2_PA_TAG_BOOKS_MAGAZINES,                         Books and Magazines,                    books-and-magazines)\
        X(PACE2_PA_TAG_AUDIO_ENTERTAINMENT,                     Audio Entertainment,                    audio-entertainment)\
        X(PACE2_PA_TAG_EDUCATION,                               Education,                              education)\
        X(PACE2_PA_TAG_M2M_IOT,                                 M2M and IoT,                            m2m-and-iot)\
        X(PACE2_PA_TAG_DEVICE_SECURITY,                         Device Security,                        device-security)\
        X(PACE2_PA_TAG_MULTIMEDIA_SERVICES,                     Multimedia Service Providers,           multimedia-service-providers)\
        X(PACE2_PA_TAG_ORGANIZERS,                              Organizers,                             organizers)\
        X(PACE2_PA_TAG_ENTERPRISE_SERVICES,                     Enterprise Services,                    enterprise-services)\
        X(PACE2_PA_TAG_APPSTORES_UPDATES,                       App-Stores and OS Updates,              app-stores-and-os-updates)\
        X(PACE2_PA_TAG_BROWSERS,                                Browsers,                               browsers)\
        X(PACE2_PA_TAG_REMOVED,                                 Removed,                                removed)\
        X(PACE2_PA_TAG_MOVED,                                   Moved,                                  moved)\
        X(PACE2_PA_TAG_WALLET,                                  Wallet,                                 wallet)\
        X(PACE2_PA_TAG_CERTIFICATE_AUTHORITY,                   Certificate Authority,                  certificate-authority)\
        X(PACE2_PA_TAG_VR_AR,                                   Virtual and Augmented Reality,          virtual-and-augmented-reality)\
        X(PACE2_PA_TAG_SMART_HOME,                              Smart Home,                             smart-home)\
        X(PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES,         Development Tools and Services,         development-tools-and-services)\
        X(PACE2_PA_TAG_SEARCH_ENGINE,                           Search Engine,                          search-engine)

enum PACE2_pa_internal_tags {
    PACE2_PA_INTERNAL_TAGS(GENERATE_ENUM)
};

#define PACE2_PA_DEFAULT_PROTOCOL_TAGS(X) \
        X(PACE2_PROTOCOL_UNKNOWN,                       PACE2_PA_TAG_GENERIC)\
        X(PACE2_PROTOCOL_BITTORRENT,                    PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_EDONKEY,                       PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_GNUTELLA,                      PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_WINMX,                         PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_DIRECTCONNECT,                 PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_APPLEJUICE,                    PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_SOULSEEK,                      PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_ARES,                          PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_FREENET,                       PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_RES_10,                        PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_XDCC,                          PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_SKYPE,                         PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_SIP,                           PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_RES_14,                        PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_YAHOO,                         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_OSCAR,                         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_IRC,                           PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_XMPP,                          PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_HTTP,                          PACE2_PA_TAG_WEB)\
        X(PACE2_PROTOCOL_POP,                           PACE2_PA_TAG_MAIL)\
        X(PACE2_PROTOCOL_SMTP,                          PACE2_PA_TAG_MAIL)\
        X(PACE2_PROTOCOL_IMAP,                          PACE2_PA_TAG_MAIL)\
        X(PACE2_PROTOCOL_RES_23,                        PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_FTP,                           PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_DNS,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_RTSP,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_FILETOPIA,                     PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_RES_28,                        PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_IMESH,                         PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_SSL,                           PACE2_PA_TAG_ENCRYPTED)\
        X(PACE2_PROTOCOL_FLASH,                         PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_MMS,                           PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_MPEG,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_QUICKTIME,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_35,                        PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_OPENVPN,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_IPSEC,                         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_GRE,                           PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_NOT_DETECTED,                  PACE2_PA_TAG_GENERIC)\
        X(PACE2_PROTOCOL_H323,                          PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_WINDOWSMEDIA,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_REALMEDIA,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_TVANTS,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_44,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_45,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_46,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_PPLIVE,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_IAX,                           PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_VTUN,                          PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_MGCP,                          PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_GADUGADU,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_ZATTOO,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_WINNY,                         PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_QQ,                            PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_QQLIVE,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_56,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_SSH,                           PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_POPO,                          PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_THUNDER,                       PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_UUSEE,                         PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_VNC,                           PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_ICMP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_DHCP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_IGMP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_EGP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_RES_66,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_67,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_XBOX,                          PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_SOFTETHER,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_SCTP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_SMB,                           PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_TELNET,                        PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_NTP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_NFS,                           PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_RTP,                           PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_TOR,                           PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_SSDP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_WORLDOFWARCRAFT,               PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_AVI,                           PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_VPN_X,                         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_HAMACHI_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_SKINNY,                        PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_RES_83,                        PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_OGG,                           PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_PALTALK,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_MOVE,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_POSTGRES,                      PACE2_PA_TAG_DATABASE)\
        X(PACE2_PROTOCOL_OSPF,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_BGP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_QUAKE,                         PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_SECONDLIFE,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_MYSQL,                         PACE2_PA_TAG_DATABASE)\
        X(PACE2_PROTOCOL_BATTLEFIELD,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_94,                        PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_RDP,                           PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_SNMP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_KONTIKI,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_VEOHTV,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_KERBEROS,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_IP_IN_IP,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_GTP,                           PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RADIUS,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_STUN,                          PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_SYSLOG,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_TDS,                           PACE2_PA_TAG_DATABASE)\
        X(PACE2_PROTOCOL_NETBIOS,                       PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_MDNS,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_IPP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_LDAP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_WARCRAFT3,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_XDMCP,                         PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_RES_112,                       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_PROTOCOL_RES_113,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_MAPI,                          PACE2_PA_TAG_MAIL)\
        X(PACE2_PROTOCOL_L2TP,                          PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_ISAKMP,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RTCP,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_TFTP,                          PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_RES_119,                       PACE2_PA_TAG_DATABASE)\
        X(PACE2_PROTOCOL_PPP,                           PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_PPTP,                          PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_STEALTHNET,                    PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_SAP,                           PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_ICMPV6,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_YOURFREEDOM,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_VOIPSWITCH_VOIPTUNNEL,         PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_DHCPV6,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_MEEBO,                         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_FUNSHION,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_AFP,                           PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_AIMINI,                        PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_RES_132,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_FLORENSIA,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_TEAMSPEAK,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_MAPLESTORY,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_136,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_137,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_DOFUS,                         PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_139,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_WORLD_OF_KUNG_FU,              PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_FIESTA,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_SPLASHFIGHTER,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_CROSSFIRE,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_WAP_WSP,                       PACE2_PA_TAG_MOBILE)\
        X(PACE2_PROTOCOL_WAP_WTP_WSP,                   PACE2_PA_TAG_MOBILE)\
        X(PACE2_PROTOCOL_WAP_WTLS,                      PACE2_PA_TAG_MOBILE)\
        X(PACE2_PROTOCOL_RES_147,                       PACE2_PA_TAG_MOBILE)\
        X(PACE2_PROTOCOL_GUILDWARS,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_ARMAGETRON,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RIM_PROPRIETARY,               PACE2_PA_TAG_MOBILE)\
        X(PACE2_PROTOCOL_RFACTOR,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RDT,                           PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_TEAMVIEWER,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_GAMEKIT,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_NETMOTION,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_TEREDO,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_SPOTIFY,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_158,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_ULTRASURF,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_VIBER,                         PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_TANGO,                         PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_RES_162,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_JBK3K,                         PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_OPERAMINI,                     PACE2_PA_TAG_MOBILE)\
        X(PACE2_PROTOCOL_BOLT,                          PACE2_PA_TAG_MOBILE)\
        X(PACE2_PROTOCOL_RES_166,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_WHATSAPP,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_WEBEX,                         PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_PROTOCOL_RES_169,                       PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_IMO,                           PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_CITRIX_GOTO,                   PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_PROTOCOL_COMODOUNITE,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_173,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_VENTRILO,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_RES_175,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_WEBSOCKET,                     PACE2_PA_TAG_WEB)\
        X(PACE2_PROTOCOL_EBUDDY,                        PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_SSTP,                          PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_179,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_RES_180,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_LDP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_RES_182,                       PACE2_PA_TAG_WEB)\
        X(PACE2_PROTOCOL_MSRP,                          PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_RES_184,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_SOCKS,                         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_186,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_187,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_188,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_189,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_RES_190,                       PACE2_PA_TAG_WEB)\
        X(PACE2_PROTOCOL_PDPROXY,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_192,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_193,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_FIX,                           PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_ORACLE_DB,                     PACE2_PA_TAG_DATABASE)\
        X(PACE2_PROTOCOL_RSVP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_LOTUS_NOTES,                   PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_PROTOCOL_POISON_IVY,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_NETFLOW,                       PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_WECHAT,                        PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_SOAP,                          PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_KIK_MESSENGER,                 PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_MPLUS,                         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_LINE,                          PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_IPERF,                         PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_DIAMETER,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_HIKE_MESSENGER,                PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_MQTT,                          PACE2_PA_TAG_M2M_IOT)\
        X(PACE2_PROTOCOL_DCE_RPC,                       PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_IDRIVE,                        PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_DOTA2,                         PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_QVOD,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_213,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_MSNAV,                         PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_TIBIA,                         PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_KAKAO,                         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_DESTINY,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_VYPR_VPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_TACACS,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_THREEMA,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_NTLM,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_RES_222,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_WHOIS,                         PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_LAYER3_IPV4,                   PACE2_PA_TAG_GENERIC)\
        X(PACE2_PROTOCOL_LAYER3_IPV6,                   PACE2_PA_TAG_GENERIC)\
        X(PACE2_PROTOCOL_LAYER4_TCP,                    PACE2_PA_TAG_GENERIC)\
        X(PACE2_PROTOCOL_LAYER4_UDP,                    PACE2_PA_TAG_GENERIC)\
        X(PACE2_PROTOCOL_RTMP,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_NNTP,                          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_FLUTE,                         PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_MEGACO,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_RES_232,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_QUIC,                          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_FUZE,                          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_RES_235,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_236,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_SPEEDTEST,                     PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_IEC61850,                      PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_SCADA,                         PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_SHOUTCAST,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_CASSANDRA_QUERY_LANGUAGE,      PACE2_PA_TAG_DATABASE)\
        X(PACE2_PROTOCOL_CASSANDRA_INTERNODE_COMMUNICATION, PACE2_PA_TAG_DATABASE)\
        X(PACE2_PROTOCOL_THRIFT,                        PACE2_PA_TAG_GENERIC)\
        X(PACE2_PROTOCOL_PLAYSTATION,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_SUPERCELL,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_246,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_T3,                            PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_SRTP,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_SRTCP,                         PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_LEAGUE_OF_LEGENDS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_FACEBOOK,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_BATTLENET,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_WOWZA,                         PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_ZALO,                          PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_GUILDWARS2,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_256,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_EA_ORIGIN,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_258,                       PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_ICECAST,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_260,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_GOOD,                          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_PATH_OF_EXILE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_PC_OVER_IP,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_OCSP,                          PACE2_PA_TAG_WEB)\
        X(PACE2_PROTOCOL_MODBUS,                        PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_FASP,                          PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_ICA,                           PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_OPC_UA,                        PACE2_PA_TAG_M2M_IOT)\
        X(PACE2_PROTOCOL_OPENFLOW,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_IPMI,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_AMQP,                          PACE2_PA_TAG_M2M_IOT)\
        X(PACE2_PROTOCOL_S7COMM,                        PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_OFF,                           PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_RSYNC,                         PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_ZEROMQ,                        PACE2_PA_TAG_GENERIC)\
        X(PACE2_PROTOCOL_DNP3,                          PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_BACNET,                        PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_NINTENDO_NETWORK,              PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_PPFILM,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_BITCOIN,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_LETV,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_PSIPHON,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_ZERO,                          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_SYNOLOGY,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_COAP,                          PACE2_PA_TAG_M2M_IOT)\
        X(PACE2_PROTOCOL_LINEAGE_2,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_IPFIX,                         PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_MINECRAFT,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_STRATUM_MINING,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_LOCAL_PEER_DISCOVERY,          PACE2_PA_TAG_P2P)\
        X(PACE2_PROTOCOL_RES_291,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_292,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_293,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_IQIYI,                         PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_295,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_VPN_UNLIMITED,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_EXPRESSVPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_TUNNELGURU,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_BOTIM,                         PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_HI_VPN,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_301,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_302,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_YEECALL,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_RES_304,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_305,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_WIREGUARD,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_307,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_FAST_SECURE_VPN,               PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_309,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_310,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_APPVPN,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_X_VPN,                         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_DROID_VPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_DTLS,                          PACE2_PA_TAG_ENCRYPTED)\
        X(PACE2_PROTOCOL_SMART_VEEPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_APEX_LEGENDS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_VPN_365,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_318,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_319,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_ACT_VPN,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_321,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_MTPROTO,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_SHADOWSOCKS,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_324,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_ACTMOBILE_SERVICES,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_ZRTP,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_PUBG_MOBILE,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_GAMELOFT_SERVICES,             PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_WICKR,                         PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_ANCHORFREE_SERVICES,           PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_331,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_UNREAL_ENGINE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_ZOOM,                          PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_PROTOCOL_RES_334,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_335,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_IGG,                           PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_MUDFISH,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RES_338,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_HATCH_CLOUD_GAMING,            PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_SOURCE_ENGINE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_341,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_YOUKU_TUDOU,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_PDANET,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_HTTP2,                         PACE2_PA_TAG_WEB)\
        X(PACE2_PROTOCOL_TOTOK,                         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_BIGO_SERVICES,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_WEBM,                          PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_RES_348,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_IPVANISH,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_MICROSOFT_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_RIP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_PRIVATEVPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_SPONGYCASTLE,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_SURFSHARK_VPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_SAPLING_GROWTH_VPN,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_IEC13818,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_TENCENT,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_ULTRAVPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_WECHAT_WORK,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_ALIBABA,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_ISIS,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_RAKNET,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_INNOVATIVE_CONNECTING,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_VIPNET,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RESP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_NOE,                           PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_AGORA,                         PACE2_PA_TAG_STREAMING)\
        X(PACE2_PROTOCOL_MIOP,                          PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_GIOP,                          PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_ELEX_TECHNOLOGY,               PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_JSON_RPC,                      PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_ZIOP,                          PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_UFTP,                          PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_MOONTON,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RTPS,                          PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_ETHEREUM,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_RES_377,                       PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_PROTOCOL_SFLOW,                         PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_COLLECTD,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_HL7,                           PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_AMONG_US,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_ETHERNET_IP,                   PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_DNSCRYPT,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_ONC_RPC,                       PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_VXI_11,                        PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_HISLIP,                        PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_IEEE_C37118,                   PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_IEC_62056,                     PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_IEEE_1703,                     PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_GAIJIN_ENTERTAINMENT,          PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_MICOWORLD_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_PHOTON,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_NINJA_KIWI_SERVICES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_KASPERSKY_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_CALL_OF_DUTY_MOBILE,           PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_ACTIVISION_SERVICES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_CAMSCANNER,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_PROTOCOL_LOL_WILD_RIFT,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_TASE_2,                        PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_PROTOCOL_RES_400,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_PROTOCOL_LITECOIN,                      PACE2_PA_TAG_FINANCE)\
        X(PACE2_PROTOCOL_PORTMAP,                       PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_HAGO,                          PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_MIHOYO_GAMES,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_405,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_SUPERVPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_ALLJOYN,                       PACE2_PA_TAG_M2M_IOT)\
        X(PACE2_PROTOCOL_NOON_VPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_RIOT_GAMES,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_GARENA_FREE_FIRE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_STP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_NORD_VPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_LIGHTSAIL_VPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_PROTOCOL_LLDP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_LLC,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_DINGTALK,                      PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_PROTOCOL_MANUFACTURING_MESSAGING_SPECIFICATION, PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_STOMP,                         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_ETHERNET_POWERLINK,            PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_UBITUS,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_ETHERCAT,                      PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_ARP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_PROFINET,                      PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_OUROBOROS,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_PROTOCOL_EIGRP,                         PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_CIP,                           PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_CDP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_VALORANT,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_VTP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_MMSE,                          PACE2_PA_TAG_MESSAGING)\
        X(PACE2_PROTOCOL_DTP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_NVIDIA,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_UBISOFT,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_VRRP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_PPPOE,                         PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_VXLAN,                         PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_CLOUDFLARE,                    PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_PROTOCOL_FINS,                          PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_PFCP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_LOST_ARK,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_POLKADOT,                      PACE2_PA_TAG_FINANCE)\
        X(PACE2_PROTOCOL_MUMBLE,                        PACE2_PA_TAG_SOCIAL)\
        X(PACE2_PROTOCOL_NARAKA_BLADEPOINT,             PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_FINAL_FANTASY_XIV,             PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_ETHERNET,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_NHRP,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_PINDUODUO,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_PROTOCOL_CISCO_METADATA,                PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_IEEE_8021Q,                    PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_IEEE_8021AD,                   PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_MPLS,                          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_FALL_GUYS,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_RES_453,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_IEC_61850_GOOSE,               PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_IEC_61850_SV,                  PACE2_PA_TAG_INDUSTRIAL)\
        X(PACE2_PROTOCOL_IEEE_8021AH,                   PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_NETEASE,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_H245,                          PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_H225,                          PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_T120,                          PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_HIREZ_SERVICES,                PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_CRYPTIC_STUDIOS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_CCP_GAMES,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_WARGAMING_NET,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_KAFKA,                         PACE2_PA_TAG_M2M_IOT)\
        X(PACE2_PROTOCOL_DIDI_SERVICES,                 PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_PROTOCOL_V4,                            PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_PTP,                           PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_PROTOCOL_ESCAPE_FROM_TARKOV,            PACE2_PA_TAG_GAMING)\
        X(PACE2_PROTOCOL_ZANGI,                         PACE2_PA_TAG_VOIP)\
        X(PACE2_PROTOCOL_NOMACHINE_NX,                  PACE2_PA_TAG_REMOTE_CONTROL)

#define PACE2_PA_DEFAULT_APPLICATION_TAGS(X)\
        X(PACE2_APPLICATION_UNKNOWN,                    PACE2_PA_TAG_GENERIC)\
        X(PACE2_APPLICATION_FACEBOOK,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MAGICJACK,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ITUNES,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MYSPACE,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_FACETIME,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_TRUPHONE,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_TWITTER,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_XBOX,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_9,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GOOGLE_MAIL,                PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_RES_11,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ICLOUD,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_13,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_DROPBOX,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_15,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_IMESSAGE,                   PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_GOTOMYPC,                   PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_GOTOMEETING,                PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_19,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_AMAZON_CLOUD,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_DAILYMOTION,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DEEZER,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_23,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_24,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_OFFICE365,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CNTV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_27,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_YOUTUBE,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VOIP_SWITCH,                PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ZYNGA,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CRIMECITY,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MODERNWAR,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VIPPIE,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_BOX,                        PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_35,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_ADOBE_CREATIVE_CLOUD,       PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_LINKEDIN,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_38,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_HI5,                        PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SORIBADA,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ZOHO_WORK_ONLINE,           PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_42,                     PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_INSTAGRAM,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SUGARSYNC,                  PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_WICKR,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_46,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_47,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_48,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_VOXER,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_50,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_51,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_BURNER,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_FLICKR,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_54,                     PACE2_PA_TAG_WEB)\
        X(PACE2_APPLICATION_RES_55,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_REDTUBE,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_EVERNOTE,                   PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_SPREED,                     PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_TEXTPLUS,                   PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_XVIDEOS,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SNAPCHAT,                   PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_62,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MYMUSIC_TW,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RENREN,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_YELP,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_AMAZON_SHOP,                PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_YOUPORN,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_EBAY,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PORNHUB,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SPIDEROAK,                  PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_ADRIVE,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_TUMBLR,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_XING,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_GOOGLE_PLAY_MUSIC,          PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_VIMEO,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SALESFORCE,                 PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_RES_77,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SINA_WEIBO,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_79,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_LIVEJASMIN,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_JINGDONG,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_LASTFM,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_IMS,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_BAIDU,                      PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_TAOBAO,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_RES_86,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TTPOD,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_88,                     PACE2_PA_TAG_MOBILE)\
        X(PACE2_APPLICATION_PINTEREST,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_90,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_NET2PHONE,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_FOURSQUARE,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_93,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MEGA,                       PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_95,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_VBULLETIN,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_CLOUDME,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_YIKYAK,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_REDMINE,                    PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_IHEART_RADIO,               PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_AVIRA_ANTIVIRUS,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_RES_102,                    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_KASPERSKY_ANTIVIRUS,        PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_AVG_ANTIVIRUS,              PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_SOUNDCLOUD,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_106,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_PRIME_MUSIC,         PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RUNKEEPER,                  PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_MYBOOK,                     PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_NIKEPLUS,                   PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_MAGV,                       PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_MAGV_KIDS,                  PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_RES_113,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LETV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MY_FITNESS_PAL,             PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_SOHU,                       PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_1KXUN,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TVBS,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_XNXX,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CTITV,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_121,                    PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_RES_122,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORKOUT_TRAINER,            PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_RES_124,                    PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_RES_125,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_126,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BITTORRENT,                 PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_EDONKEY,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_129,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_GNUTELLA,                   PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_DIRECTCONNECT,              PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_APPLEJUICE,                 PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_SOULSEEK,                   PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_ARES,                       PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_FREENET,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_136,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_SKYPE,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_138,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_YAHOO,                      PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_VONAGE,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_141,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_GADUGADU,                   PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_MPLUS,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_144,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_145,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_146,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DDL_4SHARED_COM,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_148,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_149,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_150,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_151,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_152,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_153,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_DATA_HU,                PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_DEPOSITFILES_COM,       PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_156,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_157,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_158,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILE_UPLOAD_NET,        PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_160,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILEARCHIV_RU,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_162,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_163,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILEFACTORY_COM,        PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_165,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_166,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_167,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_168,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILER_NET,              PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_170,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_171,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_172,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_173,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_174,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_175,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_GIGASIZE_COM,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_177,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_178,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_179,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_180,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_181,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_LETECKAPOSTA_CZ,        PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_183,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_184,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_LOAD_TO,                PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_186,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_MEDIAFIRE_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_188,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_MEGASHARE_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_190,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_191,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_192,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_193,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_194,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_195,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_196,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_197,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_198,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_199,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_200,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_201,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_202,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_SENDSPACE_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_204,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_205,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_206,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_207,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_208,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_209,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_210,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_211,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_212,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_213,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_214,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_215,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_216,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_217,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_218,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_219,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_220,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_221,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_222,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_223,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_224,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_225,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_226,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_227,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_228,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_229,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_230,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_1FICHIER_COM,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_232,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_233,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_234,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_235,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_236,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_237,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_238,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_239,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_240,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_241,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_242,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_243,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_244,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_245,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_246,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_EASYBYTEZ_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_248,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_249,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_250,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_251,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FASTFILESHARE_COM_AR,   PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_253,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_254,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_255,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_256,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_257,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_258,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_259,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_260,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_261,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_262,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILESMONSTER_COM,       PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_264,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_265,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_266,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FSHARE_VN,              PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_268,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_GIGAPETA_COM,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_270,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_271,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_HITFILE_NET,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_HULKSHARE_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_HYPERFILESHARE_COM,     PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_275,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_276,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_277,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_278,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_279,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_280,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_281,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_282,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_283,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_MULTISHARE_CZ,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_285,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_286,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_287,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_288,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_QSHARE_COM,             PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_290,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_291,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_SCRIBD_COM,             PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_293,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_294,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_295,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_296,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_297,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_298,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_TURBOBIT_NET,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_300,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_ULOZ_TO,                PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_302,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_303,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_UP_4SHARE_VN,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_305,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_306,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_307,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_308,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_309,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_310,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_311,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_312,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_313,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_314,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_315,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_316,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_ZIDDU_COM,              PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_318,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_319,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_320,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_321,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_322,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_2SHARED_COM,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_324,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_325,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_326,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_327,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_328,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_329,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_330,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_331,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_332,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_333,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_334,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_335,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_336,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_337,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_338,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_339,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_340,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_341,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_342,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_343,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_344,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_345,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_346,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_347,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_348,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_349,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_350,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_351,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_UPTOBOX_COM,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_353,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_GAMEFRONT_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_355,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_356,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_TURBOBIT_RU,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_USENET,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_359,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_FILETOPIA,                  PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_361,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_IMESH,                      PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_363,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_364,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_365,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_366,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_367,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ZATTOO,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QQ,                         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_QQLIVE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_371,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_POPO,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_THUNDER,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_UUSEE,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VNC,                        PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_STEAM,                      PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_HALFLIFE2,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SOFTETHER,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TELNET,                     PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_OFF,                        PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_TOR,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_WORLDOFWARCRAFT,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HAMACHI_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SKINNY,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_CITRIX,                     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PALTALK,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_QUAKE,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SECONDLIFE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BATTLEFIELD,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_390,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_ICECAST,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHOUTCAST,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QQGAME,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VEOHTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_395,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_PANDORA,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_IPP,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_WARCRAFT3,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_399,                    PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_400,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_401,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_402,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_IPTV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_YOURFREEDOM,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VOIPSWITCH_VOIPTUNNEL,      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_406,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_FUNSHION,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AIMINI,                     PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_409,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FLORENSIA,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TEAMSPEAK,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_MAPLESTORY,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_413,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_414,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DOFUS,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FIESTA,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SPLASHFIGHTER,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ACTIVESYNC,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_419,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_CROSSFIRE,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_421,                    PACE2_PA_TAG_MOBILE)\
        X(PACE2_APPLICATION_GUILDWARS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ARMAGETRON,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RFACTOR,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TEAMVIEWER,                 PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_GAMEKIT,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ULTRABAC,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SPOTIFY,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_IMPLUS,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_430,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_ULTRASURF,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VIBER,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_TANGO,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_IPLAYER,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OPERAMINI,                  PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_BOLT,                       PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_RES_437,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_WHATSAPP,                   PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_WEBEX,                      PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_NETFLIX,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_441,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_IMO,                        PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_CITRIX_GOTO,                PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_444,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_445,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VENTRILO,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_447,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_EBUDDY,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_ADOBE_CONNECT,              PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_450,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_451,                    PACE2_PA_TAG_WEB)\
        X(PACE2_APPLICATION_WEBDAV,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_453,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_454,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_455,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_456,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_457,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_458,                    PACE2_PA_TAG_WEB)\
        X(PACE2_APPLICATION_PDPROXY,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_CYBERGHOST,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_GOOGLE_CLOUD,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_DRIVE,               PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_GOOGLE_DOCS,                PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_RES_464,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ORACLE_DB,                  PACE2_PA_TAG_DATABASE)\
        X(PACE2_APPLICATION_ITV,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_467,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_468,                    PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_CHANNEL_4OD,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_470,                    PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_POISON_IVY,                 PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_HIDRIVE,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_WECHAT,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_474,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KIK_MESSENGER,              PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_LINE,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_IPERF,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_OS_UPDATE,                  PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_HIKE_MESSENGER,             PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_480,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_IDRIVE,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_482,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_483,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DOTA2,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_QVOD,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HULU,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MSEXCHANGE,                 PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_MSNAV,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_FUZE,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_KAKAOTALK,                  PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_DESTINY,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VYPR_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_THREEMA,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_494,                    PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_TWITCH,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WOWZA,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RUNTASTIC,                  PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_TELEGRAM,                   PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_STREETVOICE,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DUOKAN,                     PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_RES_501,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SKOUT,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_503,                    PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_GAME_OF_WAR,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FETNET,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FBS,                        PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_RES_507,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PINKFONG,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KOLLECT_BOOK,               PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_SPTG_TV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_511,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_512,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_513,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_514,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_AOWE,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IWOW_SYSTEX,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_9KU,                        PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_NEXTTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SYNCPLICITY,                PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_520,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_YUANTA_WEB,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_HINET,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MYCARD,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_KONO,                       PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_RES_525,                    PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_DUO_DUO,                    PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_WEALTH_GOD,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_ILEADER,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_THREE_BAMBOO,               PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_TAAZE,                      PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_AUDIKO,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PUBU_BOOKBUFFET,            PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_MITAKE,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_HYREAD,                     PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_RES_535,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUALYS,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CONSTANT_CONTACT,           PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RALLY,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_539,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_INTRALINKS,                 PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_LIVEPERSON,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SPEEDTEST,                  PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_BLUEJEANS,                  PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_APPLE_MUSIC,                PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_NAVER,                      PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_BOOKING_COM,                PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_CNN,                        PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_548,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PAYEASY,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_LINE_RANGERS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IOS_APP_STORE,              PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_MYVIDEO_TW,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KING_GAMING,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CANDY_CRUSH,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CANDY_CRUSH_SODA,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BUBBLE_WITCH,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FARM_HERO,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PLAYSTATION,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOOGLE_PLAY,                PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_SUPERCELL,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CLASH_OF_CLANS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BOOM_BEACH,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HAY_DAY,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WINDOWS_STORE,              PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_SKYPE_FOR_BUSINESS,         PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MINECRAFT,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_567,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ROVIO_GAMING,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ANGRY_BIRDS,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CITRIX_XENAPP,              PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_REDDIT,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_LEAGUE_OF_LEGENDS,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BATTLENET,                  PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_PAYPAL,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_PLANTS_VS_ZOMBIES_2,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HALFBRICK_STUDIOS,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GITHUB,                     PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_HAMIVIDEO,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_579,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_NICONICO,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ALIPAY,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_RES_582,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ZALO,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_NTT_DOCOMO,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SOFTBANK_ONLINESHOP,        PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_GUILDWARS2,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TRENDMICRO_WORRYFREE,       PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_WIKIPEDIA,                  PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_UBER,                       PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_YANDEX,                     PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_VK,                         PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ODNOKLASSNIKI,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_593,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ALIBABA,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_ALIEXPRESS,                 PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_TMALL,                      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_EA_ORIGIN,                  PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_NORTON_SECURITY,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_TIBIA,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_POCHTA_ROSSII,              PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_601,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DRIVELOCK,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_603,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_604,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_EA_GAMES,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_606,                    PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_POPCAP,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SURVEY_MONKEY,              PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_NEW_RELIC,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_HUDDLE,                     PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_ZENDESK,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_RES_612,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_613,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_DENA,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BACKBLAZE,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_DOCUSIGN,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_617,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_618,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ANGHAMI,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PATH_OF_EXILE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SLACK,                      PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_POKEMON_GO,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NIANTIC_LABS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_2GIS,                       PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_RUTUBE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_626,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PRISMA,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_628,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_629,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_AMAZON_PRIME_VIDEO,         PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CLASH_ROYALE,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOOGLE_SHARED_SERVICES,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SIGNAL,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_634,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_635,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_636,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_637,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_EDISK,                  PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_639,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_AZAR,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SKY_GO,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_642,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_MAIL_RU,                    PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_RES_644,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_645,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_KINOPOISK,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_IVI_RU,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TINDER,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ANYDESK,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_ICQ,                        PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_TENOR,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_GOOGLE_MAPS,                PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_GOOGLE_EARTH,               PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_CHANNEL_ONE,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GIPHY,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_VPN_X,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IMGUR,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ONEDRIVE,                   PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_NINTENDO_NETWORK,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AMAZON_CHIME,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_IFLIX,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PPFILM,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPLE_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ICFLIX,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LOGMEIN,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_PSIPHON,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_WAZE,                       PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_GOOGLE_FCM,                 PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_APNS,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_DIGICEL_PLAYGO,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHAZAM,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_673,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_DIGICEL_TOPUP,              PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_TUNEIN_RADIO,               PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_676,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SYNOLOGY,                   PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_MY_DIGICEL,                 PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_RES_679,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_680,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_BADOO,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_AIRBNB,                     PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_HIGHTAIL,                   PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_EASY_TAXI,                  PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_TOMTOM,                     PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_TRIPADVISOR,                PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_ESPN,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DAZN,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MOBILE_STRIKE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_KING_OF_AVALON,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LINEAGE_2,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LYFT,                       PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_TRULIA,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_AKAMAI_CLOUD,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_RES_695,                    PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_NICKELODEON_PLAY,           PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HERE,                       PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_SIMCITY_BUILDIT,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_STARWARS_GOH,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FASTLY,                     PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_SAP,                        PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WORLD_OF_KUNG_FU,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CLOUDFLARE,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_LIFE_CHURCH,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_705,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GRINDR,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_APPLE_MAPS,                 PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_APPLE_GEOLOCATION,          PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_DDL_UPLOADED,               PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_EASYPASTE_ORG,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_711,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_YOUKU_TUDOU,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MEETIC_GROUP,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_714,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_VUDU,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WISH,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_AUTH0,                      PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_SLINGTV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GOOGLE_APIS,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RACKSPACE_CLOUD,            PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_CALENDAR,            PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_GOOGLE_ANALYTICS,           PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STACKPATH_CLOUD,            PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_RES_724,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_LEARNINGHUB_ONLINE,         PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_GOOGLE_ADS,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FOX_SPORTS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AMAZON_DRIVE,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CRASHLYTICS,                PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_FACEBOOK_CLOUD,             PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_360TOTAL,                   PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_TURNER,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_734,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_LOOP_CARIBBEAN_LOCAL_NEWS,  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_LOOP_PACIFIC_LOCAL_NEWS,    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_UNITY,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FUNPLUS,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MAPBOX,                     PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_CEDEXIS,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_ZENMATE_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SMULE,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PIA,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_744,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_745,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_IPVANISH,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TUNNELBEAR,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_GOOGLE_APP_ENGINE,          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WIRE_MESSENGER,             PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_EPIC_BROWSER_PROXY,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_751,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_752,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TIGER_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_EPIC_GAMES,                 PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_AVIRA_PHANTOM_VPN,          PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TURBO_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IQIYI,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_758,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PPTV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SNAP_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_761,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_VPN_UNLIMITED,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_763,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_AVAST_SERVICES,             PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_NORD_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PROTON_VPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SAFERVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_HIDE_ME_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_EXPRESSVPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TUNNELGURU,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_YOUTUBE_MUSIC,              PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_WINDSCRIBE_VPN,             PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IVPN,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DISCORD,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_FIREVPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BOTIM,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_HI_VPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VPN_PROXY_MASTER,           PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ZOOGVPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IVACY_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PUREVPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MULLVAD_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_FREEDOME_VPN,               PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_784,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_STRONGVPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_786,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_WARGAMING_NET,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_YANDEX_MUSIC,               PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_VPN_AREA,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_STEGANOS_ONLINE_SHIELD_VPN, PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SUPERVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_STEGANOS_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_FROOTVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_794,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_795,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_CME,                        PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_YANDEX_NAVIGATOR,           PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_YANDEX_DISK,                PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_YANDEX_MAIL,                PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_TIKTOK,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_YANDEX_MAPS,                PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_RES_802,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_GAANA,                      PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_XCUM_COM,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LIFE_RU,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_806,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_807,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_808,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_YEECALL,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_SHELLFIRE_VPN,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_811,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_REN_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HOLA_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_HOLA_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CDNVIDEO,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_RES_816,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ARKVPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_FORTNITE,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AMAZE_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VUNGLE,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_VIVOX,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_MEGOGO,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FINCH_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_FAST_SECURE_VPN,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TVPLAYER,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORLD_OF_TANKS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_827,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_828,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_APPVPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_X_VPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DOTVPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ENCRYPT_ME,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_833,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DROID_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BROWSEC_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_836,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_837,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_APEX_LEGENDS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VPN_365,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_COUNTERSTRIKE_GO,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VPN_TOR_BROWSER,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SUPERVPN_BY_CHENG,          PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ACT_VPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRIGHTCOVE,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_SPORTBOX_RU,                PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_JUST_PROXY_VPN,             PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VEEPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_OOKLA,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_EAGLE_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRIA_MOBILE,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_VPN_GATE,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ANCHORFREE_SERVICES,        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRIA_TEAMS,                 PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_SHADOWSOCKS,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ASPHALT9,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GAMELOFT_SERVICES,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ACTMOBILE_SERVICES,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_NOVATECH_CORE,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SECURE_VPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_APPSFLYER,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADJUST,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_GENIEVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_863,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_HELPSHIFT,                  PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_FAST_VPN_MASTER,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VPN_UNLIMITED_PROXY,        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_HIGHSPEED_VPN,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VPN_MASTER_BY_ANDRO_TEAM,   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_869,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TENCENT_GAMES,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_871,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_872,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DOUYU,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_INGRESS_PRIME,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ELECTRONIC_ARTS,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RADIOPLAYER,                PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_MOPUB,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PUBG_MOBILE,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_879,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SPIEGEL_TV,                 PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_SPIEGEL_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_882,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FORGE_OF_EMPIRES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_INNOGAMES,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RECEIVER_RADIO,             PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_GOODGAME_STUDIOS,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_887,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RADIO_DE,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_BBC_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_BBC_SERVICES,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_UNREAL_ENGINE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ZOOM,                       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_ADOBE_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ATT_SERVICES,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FUNK,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NEXX_TV,                    PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_UNBLOCK_SITES,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRAWL_STARS,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_COUNTERPATH_SERVICES,       PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_SNAP_MASTER_VPN,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_JOYN,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PRO7SAT1_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_DEAD_RIVALS,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TAPJOY,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_IGG,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LORDS_MOBILE,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MUDFISH,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_908,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MIXPANEL,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADCOLONY,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_911,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_BIG_FARM,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FACEAPP,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ASPHALT8,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MOAT,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADSAFEPROTECTED,            PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADFORM,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FREEWHEEL,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SPOTX,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_IRONSOURCE,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TRAFFICJUNKY,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TRAFFICFACTORY,             PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HATCH_CLOUD_GAMING,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_APPLOVIN,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DELTADNA,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADTIMING,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FLURRY,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_OPTIMIZELY,                 PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SMARTADSERVER,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EXACTAG,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SCORECARDRESEARCH,          PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CONVIVA,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BAMTECH_MEDIA,              PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BEBI,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HBO_SERVICES,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_936,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CRITEO,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SWRVE,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CHARTBOOST,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_940,                    PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_DIRECTV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VPNSECURE,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_LIFTOFF,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_944,                    PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_DISNEY_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MY5,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SOURCE_ENGINE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GEOCACHING,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FOX_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_KALTURA,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RES_951,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RES_952,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FOX_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BRANCH,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_VIKI,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORDPRESS,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_GRAVATAR,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_PARSELY,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_959,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PBS,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CRACKLE,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OPENX,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_963,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DISNEY_RADIO,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KOCHAVA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ABC,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_967,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_APTELIGENT,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_APPLE_NEWS,                 PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_970,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SEGMENT,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TUNE,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_APPLE_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPTENTIVE,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_AIRABLE,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_PLUTO_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_977,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_978,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SONY_PICTURES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NFL_SUNDAY_TICKET,          PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PORNHUB_NETWORK,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PDANET,                     PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_MLB,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MLB_AT_BAT,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TUBI,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WATCH_TBS,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DISNEY_PLUS,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OPENSIGNAL,                 PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_SHOWTIME,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUANTUM_METRIC,             PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MPARTICLE,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SHOWTIME_ANYTIME,           PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CANAL_PLUS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AUDIBLE,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_TEALIUM,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_NUANCE,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_CBS_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NBC_SPORTS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_EUROSPORT_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ENDEAVOR_STREAMING,         PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_TOTOK,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_FYBER,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_UNIVISION_NOW,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TABOOLA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1005,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SKY_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_USERVOICE,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_BIGO_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BITTORRENT_NOW,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MARIO_KART_TOUR,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TURKCELL_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SKY_SPORTS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BIP_MESSENGER,              PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_SWOOGO,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NBC_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ETISALAT_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SWITCH_TV,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MICROSOFT_TEAMS,            PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_FLASHTALKING,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1020,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1021,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_NIELSEN_SERVICES,           PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FOX_BUSINESS,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_PRIVATEVPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SURFSHARK_VPN,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_YOSPACE,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SAPLING_GROWTH_VPN,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_NFL_SERVICES,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_XUNLEI_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_THE_NBC_APP,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FIZY,                       PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_CLOUDINARY,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_TENCENT_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ULTRAVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_1035,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_STARZ,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WECHAT_WORK,                PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RESILIO_SYNC,               PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_IMGIX,                      PACE2_PA_TAG_DATABASE)\
        X(PACE2_APPLICATION_RES_1040,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_KAKAO_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SOLVVY,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_JW_PLAYER,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_STACKLA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MUX,                        PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_FRESHDESK,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_FRESHWORKS,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AMPLITUDE,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1049,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_UNDER_ARMOUR_SERVICES,      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_MAP_MY_FITNESS,             PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_TRUSTARC,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_EXCEL,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MICROSOFT_WORD,             PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MICROSOFT_POWERPOINT,       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MICROSOFT_OUTLOOK,          PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_SNAP_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BOOTSTRAPCDN,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_KAKAOSTORY,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_1060,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SPORTSMAX,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KAKAOTV_LIVE,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPLICASTER,                PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_SHAREPOINT,       PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_RES_1065,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_INDEX_EXCHANGE,             PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NBC_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_FUBOTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPDYNAMICS,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TOUR_TRACKER,               PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_BLACKBOARD,                 PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CHARTBEAT,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EBOT7,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_ONENOTE,          PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_HBO_GO,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MEDIAMATH,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_VILYNX,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_WWE,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1079,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_1080,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_SENTRY,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_YUPPTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PERIMETERX,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NBA,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PLAYRIX,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_APP_BAZAAR,                 PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_HOTJAR,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TAPLYTICS,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_RTVE_ALACARTA,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HAYU,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GIGYA,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_TNT,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHOWMAX,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ATRESMEDIA,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CDNETWORKS,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_CLASSROOM,           PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_PEER5,                      PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_SATISFI_LABS,               PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_AYOBA,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_ROBLOX,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NHL,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_INNOVATIVE_CONNECTING,      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_OUTBRAIN,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STAN,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VPN_MONSTER,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MEDIACORP_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ATRESPLAYER,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AIRSHIP,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DIDI_SERVICES,              PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_MEDIASET_ESPANA,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MITELE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MEWATCH,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1113,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1114,                   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NIKE_SERVICES,              PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_ACCEDO,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_99TAXIS,                    PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_CHARTER_COMMUNICATIONS,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VIU,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PCCW,                       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ALIBABA_CLOUD,              PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_LOCALYTICS,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TIDAL,                      PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_1124,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_YOUBORA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_MEET,                PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_COINBASE,                   PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_WIDEORBIT,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_MEETRICS,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CONDUCTRICS,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_LAUNCHDARKLY,               PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_ART19,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_HIVEMQ,                     PACE2_PA_TAG_M2M_IOT)\
        X(PACE2_APPLICATION_HANDELSBLATT,               PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_PODIGEE,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_CLOUD_MAIL_RU,              PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_INNOVID,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MOVISTAR_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_UCWEB,                      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_WIZROCKET,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BATCH,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_VIZBEE,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_WIRTSCHAFTSWOCHE,           PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_HANDELSBLATT_SERVICES,      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_IQ_DIGITAL,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MEET_GROUP_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MOVISTAR_PLUS,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WIZZEE,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_INTERCOM,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_NETEASE,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PENTHERA,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RIOT_GAMES,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BYTEDANCE,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_1154,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TAGGED,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WEBTOON,                    PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_MEETME,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DYNAMIC_YIELD,              PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_ELEX_TECHNOLOGY,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CLASH_OF_KINGS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IMDB,                       PACE2_PA_TAG_DATABASE)\
        X(PACE2_APPLICATION_BILLO,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AIHELP,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_CRUNCHYROLL,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHEIN,                      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_AVIRA_SERVICES,             PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_SAMSUNG_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_XIGUA_VIDEO,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TOUTIAO,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_BUZZSPROUT,                 PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_MEGAPHONE,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BUGSNAG,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_KNOTCH,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_WURL,                       PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_LIBSYN,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_TRITON_DIGITAL,             PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_VNG_CORPORATION,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BAIDU_MAPS,                 PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_MOONTON,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AMAGI,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_KUAISHOU,                   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MARKMONITOR,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MEDIUM,                     PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_SIMPLECAST,                 PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_WALMART,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_GRUBHUB,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_KWAI,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_BRAZE,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_KEEPSOLID_SERVICES,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VPN_LITE,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SPRINGSERVE,                PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RES_1192,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WORKPLACE,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_YAANI,                      PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_WASHINGTON_POST,            PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_1196,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ROMWE,                      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_TARGET_SIGNATURE,           PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PLEX,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_ALEXA,               PACE2_PA_TAG_SMART_HOME)\
        X(PACE2_APPLICATION_BUMBLE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WEATHER_CHANNEL,            PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_REUTERS_NEWS,               PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CRYPTOTRADER,               PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_RES_1205,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CLUB_FACTORY,               PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_BOMBORA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MOOV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_COLLECTD,                   PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_ORACLE_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_COMCAST,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SKRED_MESSENGER,            PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_ACAST,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_THE_CW,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CW_SEED,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FACEBOOK_LITE,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_OPERA_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_THE_WALL_STREET_JOURNAL,    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_TEXTNOW,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_BLOOMBERG,                  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_UDEMY,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_THE_ECONOMIST,              PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_INVESTING_COM_SERVICES,     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_CRYPTOWATCH,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_AMONG_US,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_COOKIEPRO,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_BEIN_SPORTS,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BEIN_MEDIA_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ERSTREAM,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_WE_MEET,                    PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_NEWSDOG,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_PUGPIG,                     PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_DIGITURK_PLAY,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DIGICEL_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_DOORDASH,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PHOTO_WONDER,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_QUANTCAST,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_JIOSAAVN,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_OPERA_NEWS,                 PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_XHAMSTER,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHAHID,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MBC_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_QQMUSIC,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SEAMLESS,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_CIRES21,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SPLUNK,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_COINCAP,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_SHAPESHIFT_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AL_JAZEERA_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AL_JAZEERA_ENGLISH,         PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_DO_GLOBAL_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_WEBFLOW,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_PAYBACK,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_MEITU_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CAMSCANNER,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_INTSIG_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_JAUMO,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PUBMATIC,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_JIO_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GROUPON,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_RES_1261,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_VERIZON_MEDIA_SERVICES,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ACTIVISION_BLIZZARD,        PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_AUDIOMACK,                  PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_KAKAO_MAP,                  PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_OSN,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GSMA,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_GOOGLE_JIBE,                PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_BEAUTYPLUS,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_VIVAVIDEO,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CALL_OF_DUTY,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MICOWORLD_SERVICES,         PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_HAGO,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DAUM_CAFE,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_TENCENT_NEWS,               PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_FLIPBOARD,                  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_VIACOM18_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VOOT,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SELFIECITY,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MEITU,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_KASPERSKY_SERVICES,         PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_HAKUNA,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_BERMUDA_VIDEO_CHAT,         PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_XIAOMI_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_OPERA_NEWS_LITE,            PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_GAIJIN_ENTERTAINMENT,       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SCOPELY_SERVICES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MARVEL_STRIKE_FORCE,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SKY_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_SYSTEX_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PERFECT_CORP_SERVICES,      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_YOUCAM_MAKEUP,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DTAG_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PUBG,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOOGLE_VOICE,               PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_THE_SCORE,                  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_PANTAYA,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DAUM_MAIL,                  PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_FACEBOOK_BROADCASTING,      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WAR_THUNDER,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_APUS_SERVICES,              PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_UFOTO_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PEAK_GAMES_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_TOON_BLAST,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_EXPO_FRAMEWORK,             PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_THE_NEW_YORK_TIMES,         PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_DU_RECORDER,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1308,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MEIZU_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_JOYY_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MOBVISTA,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STARTIMES_ON,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VODAFONE_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_THE_GUARDIAN,               PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_GUARDIAN_MEDIA_SERVICES,    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MOZILLA_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MAGNITE,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DC_UNIVERSE,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RAKUTEN_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PLAYDEMIC_SERVICES,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOLF_CLASH,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ONETRUST,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_LIGHTSTEP,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1324,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_VS_MEDIA,                   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_EXARING_SERVICES,           PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WAIPU_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GOOGLE_NEWS,                PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RING_CENTRAL,               PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_MORTAL_KOMBAT,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VPN_VALLEY,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BEIN_CONNECT,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BOOMPLAY_MUSIC,             PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_KONGREGATE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NINJA_KIWI_SERVICES,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HUAWEI_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_XMRIG,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_LOL_WILD_RIFT,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TINKOFF,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_YANDEX_MARKET,              PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_SWEET_SELFIE,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SWEET_SELFIE_LITE,          PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DIGICERT,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SWITCHERRY_VPN,             PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_XENDER,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_NCSOFT,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ORNET,                      PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_COINGECKO,                  PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_UMG,                        PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_MALWAREBYTES,               PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_REALNETWORKS,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_KEYCDN,                     PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_RILEY_CILLIAN,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_STREAMLIVE_TO,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WILDLIFE_STUDIOS,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BLOCK_CRAFT_3D,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_1357,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_LOVOO,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_CLUBHOUSE,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SNIPER_3D,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_COMODO_SERVICES,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_RIBBON_COMMUNICATIONS,      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BLACKBERRY_WORK,            PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_JOOM,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_BLACKBERRY_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_PAY,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_GEEKFLARE,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_WAR_MACHINES,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RAWVOICE,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_PODTRAC,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_WISE,                       PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_GOOGLE_PODCASTS,            PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SHALLTRY_GROUP,             PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_LIVERAMP,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EURONEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_LEAP_FITNESS_GROUP,         PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_HEXAGLOBE,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ROCKET_LEAGUE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MAGINE_SERVICES,            PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_PAY_RW,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_VUBA_VUBA,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_YANDEX_GO,                  PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_YANDEX_TELEMOST,            PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_NEXON_CORPORATION,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AGODA,                      PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_GRAB,                       PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_NYPR,                       PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PRX_SERVICES,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_DOVETAIL,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_HUYA,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHAREIT,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_EVERSEND,                   PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_5CHANNEL,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PET_RESCUE_SAGA,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CLASSDOJO,                  PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_PHOTOMATH,                  PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_MIHOYO_GAMES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GENSHIN_IMPACT,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HONKAI_IMPACT_3RD,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_KHAN_ACADEMY,               PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_HOMER,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CASTBOX,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_OPENTEXT_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_EDX,                        PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_SOCRATIC,                   PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_TELIA_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_TELIA_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PEACOCK_TV,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_JIO_ADS,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DISCOVERY_PLUS,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DUOLINGO,                   PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_RES_1412,                   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BET_PLUS,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PARAMOUNT_PLUS,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HUBSPOT,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PHILO,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DNA_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_XIAOMI_WEATHER,             PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_JIOCHAT,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_REMIND,                     PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_ELISA_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ELISA_VIIHDE,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PUBNUB,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_FUNIMATION,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DRIFT,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_IMG,                        PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ENDEAVOR_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SOFTBANK_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_REFACE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MTV_KATSOMO,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_ADS,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PRENDE_TV,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_UNIVISION_COMMUNICATIONS,   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ANCHORFM,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_VEVE,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_RES_1436,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUALTRICS,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_WANGSU_CDN,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_ALGOLIA,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_MAXIVISION,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_G_CORE_SERVICES,            PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_NORDIC_ENTERTAINMENT_GROUP, PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_VIAFREE,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NETMERA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_GRACENOTE,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_STORYFM_CN,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_YLE_AREENA,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_STREAMABLE,                 PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SPEAKOL,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_YALLAKORA,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LINKONLINE,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GEMINI_MEDIA_EGYPT,         PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SANOMA_SERVICES,            PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RUUTU,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FINNPANEL,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NOON_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MASRAWY,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_KTPLAY,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_TEAM_FORTRESS_2,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TRUST_WALLET,               PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_CANAL_PLUS_AFRIQUE,         PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SARMADY_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_FILGOAL,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_AMC_THEATRES,               PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_YOUM7,                      PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_VALVE_CORPORATION,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_1467,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_XOOM,                       PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_IBM,                        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MIDOKI_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VPN_MASTER_X,               PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_QUALAROO,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_MTS_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_EXPEDIA,                    PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_GLOBALSIGN,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_LETSENCRYPT,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_YIELDMO,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_IDENTRUST,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_GODADDY,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_GOOGLE_TRUST_SERVICES,      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WATCH_IT,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_TRUST_SERVICES,      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ENTRUST,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_BINANCE,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_ELFSIGHT,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SAILTHRU,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CERTUM,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MEDIANOVA_CDN,              PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_NAPSTER,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SECTIGO,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_BAZAARVOICE,                PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_SUNING_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_INDIGITALL,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADA_SUPPORT,                PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_PENGUIN_SPORTS,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PUSHER,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_CSG_MEDIA,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_GRAPESHOT,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_KNIGHTHOOD,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BLOSSOM_BLAST_SAGA,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CYBERTRUST_CA,              PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_USERCENTRICS,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_IDVIU,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_IAPPS_TECHNOLOGIES,         PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CALM,                       PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_PLAYSTATION_NOW,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MICROSOFT_AZURE,            PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_ZEGO,                       PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_DISH_NETWORK,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_JQUERY,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_MOX_TV,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_STATS_PERFORM,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CONTENTFUL,                 PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_WETV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ADEX,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ARLO,                       PACE2_PA_TAG_SMART_HOME)\
        X(PACE2_APPLICATION_JIOHEALTHHUB,               PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_GOG,                        PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_RING,                       PACE2_PA_TAG_SMART_HOME)\
        X(PACE2_APPLICATION_FITBIT,                     PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_GOOGLE_FIT,                 PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_DIAMOND_DIARIES_SAGA,       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DELUXE_MEDIA,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_THE_TRADE_DESK,             PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_JIOCLOUD,                   PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_TRIVAGO,                    PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_MINUTELY,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_GARENA_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GARENA_FREE_FIRE,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IMPACT_SERVICES,            PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SAMSUNG_MAX_PRIVACY_VPN,    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PRIVATE_TUNNEL_VPN,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TRIPLELIFT,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1534,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MAILCHIMP,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_A1_BELARUS,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VOKA_TV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SPB_TV,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SMART_MEDIA_SOLUTIONS,      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SMAATO,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TEADS,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PYRAMID_SOLITAIRE_SAGA,     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RISKIFIED,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_1544,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_1545,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LEANPLUM,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_TAPAD,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SQUARESPACE,                PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_BRIGHTLINE,                 PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_EXTREME_REACH,              PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ROCKETVPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MOOVIT,                     PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_COINBOX,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_SYBO_GAMES,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SUBWAY_SURFERS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NPR,                        PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_ESET_SERVICES,              PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_MAKEUPPLUS,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_KAKAO_WEBTOON,              PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_ZEE5,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_JUSPAY,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_POKEMON_UNITE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ZOMATO,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PLYMEDIA,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BITPAY,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_PHYCAN_VPN_SERVICES,        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_INTERNET_ARCHIVE,           PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CLOUDFLARE_WARP_VPN,        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRD_WALLET,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_EXODUS_WALLET,              PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_BEAUTYCAM,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_GAB,                        PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MEASUREMENT_LAB,            PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_GLOVO,                      PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_SAMSUNG_MUSIC,              PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SWEET_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_UKLON,                      PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_TOCA_BOCA,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TAVR_MEDIA,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_LIGHTSAIL_VPN,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IBM_CLOUD_VIDEO,            PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AGORA_SERVICES,             PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SUN_FM_UKRAINE,             PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_OLL_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LANET_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TIGO_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_UBER_EATS,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_BOLT_TECHNOLOGY,            PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_STRIPE,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_MEDALLIA,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_ONEPLUSONE_MEDIA_SERVICES,  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ONEPLUSONE_VIDEO,           PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VERISIGN,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_1594,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ADMIXER,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DINGTALK,                   PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_ONLINE_RADIO_BOX,           PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_MOLOCO,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1599,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PROSTREAM_RU,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BLOCKCHAIN_COM,             PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_QUALCOMM,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_JIOMART,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_JIOMEET,                    PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_HCL_NOTES_AND_DOMINO,       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_MI_FIT,                     PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_PAXFUL_WALLET,              PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_SCHILDBACH_WALLET,          PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_CAKE,                       PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_TIGERCONNECT,               PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_ICLOUD_PRIVATE_RELAY,       PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_LUDO_KING,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TWILIO_SERVICES,            PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_ANKAMA_GAMES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_META_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_HUAWEI_HEALTH,              PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_YOUDAO,                     PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_XINHUA_NEWS,                PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RAYGUN,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_CHAT,                PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_DOUBLEVERIFY,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EXTOLE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STARMAKER,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MX_PLAYER,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ORIGINS_DIGITAL,            PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SYGIC_SERVICES,             PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_BUDGE_STUDIOS,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MTS_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SMADEX,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_AMAZON_GAMES,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NEW_WORLD,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AARKI,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_XIAOMI_SHAREME,             PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_GOODBARBER,                 PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_LITRES_RU,                  PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_QWILT,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_START_RU,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_POU,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MUMBLE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_AMEX_SERVICES,              PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_WIZARDS_OF_THE_COAST,       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MAGIC_THE_GATHERING_ARENA,  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MOZILLA_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DEUTSCHE_KREDITBANK,        PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_DEUTSCHE_BANK,              PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_NORTONLIFELOCK_SERVICES,    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NORTONLIFELOCK_VPN_SERVICES, PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ZAPYA,                      PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_AMOBEE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_UBITUS_SERVICES,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NINTENDO_SWITCH_CLOUD_GAMING, PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SPREAKER,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_JIOCINEMA,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_EPSILON_CONVERSANT,         PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NVIDIA_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WINK,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ROSTELECOM,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NGENIX,                     PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_INTEL_SERVICES,             PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_AMD_SERVICES,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_PIKMIN_BLOOM,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_123VPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PLAYTIKA_SERVICES,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AVG_SERVICES,               PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_NETFLIX_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NETFLIX_GAMES,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HERO_WARS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SWIFTY_BEAVER,              PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_BITDEFENDER_SERVICES,       PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_IHEART_MEDIA_SERVICES,      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SOPHOS_SERVICES,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_MCAFEE_SERVICES,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_EUROSPORT_NEWS,             PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_OLA,                        PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_LG_ELECTRONICS_SERVICES,    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ERICSSON_SERVICES,          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SHARECHAT,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_VALORANT,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_INDEED,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_NVIDIA_GEFORCE_NOW,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TRUEKEY,                    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_MOON_ACTIVE,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SAFE_FAMILY,                PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_LOTAME,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FIREFOX_PRIVATE_NETWORK,    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MEDIA_NET,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_JSDELIVR,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_UBISOFT_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_UBISOFT_CONNECT,            PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_BLITZER_DE,                 PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_SAMSUNG_PAY,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_GOOGLE_PHOTOS,              PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_STELLAR,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_TRACKMANIA,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WANT_WANT_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_UNRULY,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TOTALAV,                    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_ATUDO,                      PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_BID_SWITCH,                 PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_ING_BANKING,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_FEEDAD,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_COMMERZBANK_BANKING,        PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_VR_BANKING,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_ANZU_VR,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BEESWAX,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DAUNTLESS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CATCHMEDIA,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PIANO,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SHOPIFY,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PC_PROTECT,                 PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_CISCO_SYSTEMS_SERVICES,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SPARKASSE_BANKING,          PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_HOLASPARK,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_BING,             PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_TESTMY_NET,                 PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_FUELIO,                     PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_SPEEDOF_ME,                 PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_XFINITY,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_XFINITY_SPEEDTEST,          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_PAYCENTIVE,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_RAINBOW_SIX_SIEGE,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_JPMORGAN_CHASE,             PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_MULTI_MEDIA_LLC,            PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CAPITALONE,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_SYGIC_TRAVEL_MAPS,          PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_GLASSBOX,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STARK_GAMES,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WELLS_FARGO,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_BLINKIST,                   PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_GAMES_WORKSHOP,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WARHAMMER_GAMING,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_REDFAST,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SAMSUNG_HEALTH,             PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_SPEEDSMART,                 PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_SYGIC_GPS_NAVIGATION,       PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_COMCAST_TECH_SOLUTIONS,     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_VIUTV,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DEMANDBASE,                 PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_RAINBOW_SIX_EXTRACTION,     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WARHAMMER_TV,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DISNEY_HOTSTAR,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WUNDERKIND,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MIGHT_AND_MAGIC_CHESS_ROYALE, PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PULSE_INSIGHTS,             PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_RADARBOT,                   PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_ADSWIZZ,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BRIGHT_DATA,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_WAKANIM,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ETSY,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_VANILLA,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_CITADEL_COLOUR,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BEHAVIOUR_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_DEAD_BY_DAYLIGHT,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BLACK_LIBRARY,              PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_DIGITAL_TURBINE,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_DUCKDUCKGO,                 PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_LIVEONE,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_JOSH,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BABE,                       PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_WIKIMEDIA_FOUNDATION,       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AVAST_VPN_SERVICES,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_OPENWEB,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_STREAMIO,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_TMT_BROADCASTING,           PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_UNO,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VERVE_GROUP,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MCAFEE_SECURITY,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_LOKALISE,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_BRAINLY,                    PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_ZONER_ANTIVIRUS,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_ZONER_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MAGENTA_TV,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LARK,                       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_CAPCUT,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PODBEAN,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PERMUTIVE,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_WISTIA,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RAPPI,                      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_KION,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OKKO,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PICSART,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MX_TAKATAK,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SENDER,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_F_SECURE,                   PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_AAROGYA_SETU,               PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_ANIVIEW,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PINDUODUO,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_GEARBOX_ENTERTAINMENT_SERVICES, PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GEARBOX_SHIFT,              PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_AGE_OF_EMPIRES,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RELIC_ENTERTAINMENT,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_XBOX_GAME_STUDIOS,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SPLIT,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_TRUTH_SOCIAL,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RUMBLE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FANDOM,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_USCREEN,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_HYPERCONNECT,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_FONT_AWESOME,               PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_RUBY_GAME_STUDIO,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MOENGAGE,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_LOST_ARK,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MIRO,                       PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_GOOGLE_TV,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CADENT,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_PANGLE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EMBRACE,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STREAMGUYS,                 PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_HOLLER,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_EASYBRAIN,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_EASYBRAIN_MUSIC_MAKER,      PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_CREATIVE_ASSEMBLY,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SEGA,                       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MATCH_GROUP,                PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_TOTAL_WAR,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FERAL_INTERACTIVE,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LIVEBOARD,                  PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_NARAKA_BLADEPOINT,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NETEASE_GAMES,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PATREON,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_USPS,                       PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_IAB_EUROPE,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_UPS,                        PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_Q4,                         PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_TIDIO,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_WARNER_BROS_DISCOVERY_SERVICES, PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_LAZADA,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_ONLYFANS,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PRIMIS,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BLUE_BILLYWIG,              PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_GEOLOCATION_TECHNOLOGY,     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_LUMEN_TECHNOLOGIES,         PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_FRUITSHAKE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IMPROVE_DIGITAL,            PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TERMINUS,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TABLEAU,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_TINY_TINAS_WONDERLANDS,     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AG_PROJECTS_SERVICES,       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_DISPLAY_IO,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BEETALK,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ZILLOW,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PLURK,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_INMOBI,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ARC_XP,                     PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_EMBRACER_GROUP,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SPIN_THE_BOTTLE,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_XANDR,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MAPP,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_9GAG,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SQUARE_ENIX,                PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_FINAL_FANTASY_XIV,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NAVER_WORKS,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_NAVER_CORPORATION,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_INTUIT,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_GAZPROM_MEDIA,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_1856,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RADANCY,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RAMBLER,                    PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_CRYPTO_COM,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_FIFA_SOCCER,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_KUSTOMER,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_LOGROCKET,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SPOTHERO,                   PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_ADOBE_EXPERIENCE_CLOUD,     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ICF_TECHNOLOGY,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TRICOLOR_TV,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_REDFIN,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_MATTERPORT,                 PACE2_PA_TAG_VR_AR)\
        X(PACE2_APPLICATION_NAVER_CLOUD,                PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_ROGUE_COMPANY,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WOW_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUORA,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_INFOBIP,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_EDGIO,                      PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_FLIPKART,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_NEAR,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_AVO,                        PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CONTENTSQUARE,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FALL_GUYS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_YAHOO_AD_TECH,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MEESHO,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_TOWER_OF_FANTASY,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NPERF,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_NBCUNIVERSAL,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NAVER_MAP,                  PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_SNAPDEAL,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_SMARTCLIP,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_DIGITALOCEAN,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_HAPPN,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_2K,                         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MICROSOFT_FAMILY_SAFETY,    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_RUMBLEVERSE,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WB_GAMES,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MICROSOFT_DEFENDER,         PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_ADOBE_FONTS,                PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_VIMOND,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_AIONCLOUD,                  PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_AMC_NETWORKS,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMC_PLUS,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CABIFY,                     PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_PAYTM,                      PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_LENOVO_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_UPLIVE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_NAVER_MAIL,                 PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_AKAMAI_TECHNOLOGIES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_STACKADAPT,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HIREZ_STUDIOS,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VIX,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TREMOR_VIDEO,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BLOCK_INC,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CASH_APP,                   PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_OUTFIT7,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MYTHIC_LEGENDS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_UI_AG_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GMX_MAIL,                   PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_INSHORTS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_REVOLUT,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_ZEPETO,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_DIABLO_IMMORTAL,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LIFE360,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SESSION,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_PREMIER,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TENCENT_CLOUD,              PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_PAYPAY,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_6SENSE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_VIMEO_OTT,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ZEUS_NETWORK,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VIDAZOO,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_HIDIVE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GRUPO_GLOBO_SERVICES,       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GLOBOPLAY,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHUDDER,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KINGSOFT_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_KINGSOFT_CLOUD,             PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_KOBO,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_FOR_HONOR,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TREMOR_INTERNATIONAL,       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FINANCIAL_TIMES,            PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_KAPE_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SQUARE,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_HCAPTCHA,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_AMC_SVOD,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SUNDANCE_NOW,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NAVER_NOW,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AUSTRALIA_POST,             PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_BUNGIE_GAMES,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ORA_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CCP_GAMES,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_EVE_ONLINE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ZYPE,                       PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_UMENG_PLUS,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BYJUS,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_TILDA,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_CDN77,                      PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_TAPCLAP,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PIRATE_TREASURES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PRISMIC,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_TENCENT_MUSIC,              PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_HTTPOOL,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CACHEFLY,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_ALIBABA_GROUP_SERVICES,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_QANDA,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_VERCEL,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_STREAMOTION,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DRAFTKINGS_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_DRAFTKINGS_CASINO,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DRAFTKINGS_SPORTSBOOK,      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WIX,                        PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_FANDUEL_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_FANDUEL_CASINO,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FANDUEL_SPORTSBOOK,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CRYPTIC_STUDIOS,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NEVERWINTER,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RESSO_MUSIC,                PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_XUMO,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_REALM_ROYALE,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VIKINGARD,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BADLANDERS,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_INFINITE_LAGRANGE,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VIEWLIFT,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_TVNZ,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WYNK_MUSIC,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_GEARBOX_PUBLISHING,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VIJU,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VIASAT_RU,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_LILITHGAMES,                PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RISE_OF_KINGDOMS,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DECA_GAMES,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GODS_AND_GLORY,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PODCAST_ADDICT,             PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_EVE_ECHOES,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LAZY_AUDIO,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_AMEDIATEKA,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MIXI,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MIXI_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_LOTR_WAR,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOOGLE_TRANSLATE,           PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_NTT_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RADIKO,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_CYBERAGENT_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AMEBA,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RED_HOT_PIE,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_CLICKTOUCH_GAMES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ADTECH_STUDIO,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MEDIALAB_AI_SERVICES,       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MITENE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_TRANSPERTH,                 PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_HINGE,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SUPERAWESOME,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_YANDEX_TRANSLATE,           PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CASHI,                      PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_BOOKING_HOLDINGS_SERVICES,  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BANCO_AZTECA,               PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_GSTATIC,                    PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_DIDI_FOOD,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_DIDI_RIDER,                 PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_FACEBOOK_MESSENGER,         PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_COPPEL,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_UNCLASSIFIED,               PACE2_PA_TAG_GENERIC)\
        X(PACE2_APPLICATION_OKCUPID,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MERCADO_LIBRE,              PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_BAND,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MERCADO_PAGO,               PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_SMILEGATE,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BANCOPPEL,                  PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_V4,                         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VLMEDIA_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_TRANSLATOR,       PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CDNJS,                      PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_TELEMETRO,                  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CORPORACION_MEDCOM,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_HERE_TV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OVERWATCH,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SNOW_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MASTODON,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_GLU_MOBILE_SERVICES,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_KARTRIDER_RUSH,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GROUPM,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ULTRAVIEWER,                PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_KINGDOM_AGE,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_STAR_TREK_ONLINE,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WEB_DE,                     PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_MAIL_COM,                   PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_ARTE,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VITA,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PARAMOUNT_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BLUTV,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_XIAOHONGSHU,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RAKUTEN_TV,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPLE_WEATHER,              PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_MUBI,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUANTUMCAST,                PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_KNIGHTS_AND_DRAGONS,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SMITE,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TELEPORT_MEDIA,             PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_CBS_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CBS_SPORTS,                 PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RBC_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CANONICAL_SERVICES,         PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_EXXEN,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SONY_GROUP_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SOUL_HUNTERS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ADTELLIGENT,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DATADOG,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SEGMENTO_RU,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_POLYFILL_IO,                PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_B612,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WARPATH_ACE_SHOOTER,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TOWER_OF_SAVIORS,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LEMON8,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_COVEO,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HYPIC,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ESCAPE_FROM_TARKOV,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_POCKET,                     PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_MARVEL_SNAP,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ADOBE_DOCUMENT_CLOUD,       PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_ZANGI_MESSENGER,            PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_BEAMABLE,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_BUNNY_NET,                  PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_BCE_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CRAVE,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BYTEPLUS,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_PLARIUM,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SPARTA_WAR_OF_EMPIRES,      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SBER,                       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_JIVOCHAT,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_YOUTUBE_KIDS,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OPPO_MOBILE_SERVICES,       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_HEYTAP,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_MAX,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORLD_OF_WARSHIPS,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_YOUTUBE_TV,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GOOGLE_FAMILY_LINK,         PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_CELTIC_HEROES,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HERO_HUNTERS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TIMES_INTERNET_SERVICES,    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SERVICENOW,                 PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_HONKAI_STAR_RAIL,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ELDER_SCROLLS_ONLINE,       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NUVERSE,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_TVPLUS,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NASDAQ_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SESISOFT_SERVICES,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TIVIBU_GO,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DUNGEON_FIGHTER_ONLINE,     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_5GMARK,                     PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_KILL_SHOT_BRAVO,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PUHUTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DOGUS_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_OPENAI,                     PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_QOBUZ,                      PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_S_SPORT_PLUS,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORLD_OF_TANKS_BLITZ,       PACE2_PA_TAG_GAMING)

#define PACE2_PA_DEFAULT_APPLICATION_SUB_TAGS(X)\
        X(PACE2_APPLICATION_UNKNOWN,                    PACE2_PA_TAG_GENERIC)\
        X(PACE2_APPLICATION_FACEBOOK,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_MAGICJACK,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ITUNES,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MYSPACE,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_FACETIME,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_TRUPHONE,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_TWITTER,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_XBOX,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_9,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GOOGLE_MAIL,                PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_RES_11,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ICLOUD,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_13,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_DROPBOX,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_15,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_IMESSAGE,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_GOTOMYPC,                   PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_GOTOMEETING,                PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_19,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_AMAZON_CLOUD,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_DAILYMOTION,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DEEZER,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_23,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_24,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_OFFICE365,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CNTV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_27,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_YOUTUBE,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VOIP_SWITCH,                PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ZYNGA,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CRIMECITY,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MODERNWAR,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VIPPIE,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_BOX,                        PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_35,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_ADOBE_CREATIVE_CLOUD,       PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_LINKEDIN,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_38,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_HI5,                        PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SORIBADA,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ZOHO_WORK_ONLINE,           PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_42,                     PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_INSTAGRAM,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SUGARSYNC,                  PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_WICKR,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_46,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_47,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_48,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_VOXER,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_50,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_51,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_BURNER,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_FLICKR,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_54,                     PACE2_PA_TAG_WEB)\
        X(PACE2_APPLICATION_RES_55,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_REDTUBE,                    PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_EVERNOTE,                   PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_SPREED,                     PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_TEXTPLUS,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_XVIDEOS,                    PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_SNAPCHAT,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_62,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MYMUSIC_TW,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RENREN,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_YELP,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_AMAZON_SHOP,                PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_YOUPORN,                    PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_EBAY,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PORNHUB,                    PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_SPIDEROAK,                  PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_ADRIVE,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_TUMBLR,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_XING,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_GOOGLE_PLAY_MUSIC,          PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_VIMEO,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SALESFORCE,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_77,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SINA_WEIBO,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_79,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_LIVEJASMIN,                 PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_JINGDONG,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_LASTFM,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_IMS,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_BAIDU,                      PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_TAOBAO,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_RES_86,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TTPOD,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_88,                     PACE2_PA_TAG_MOBILE)\
        X(PACE2_APPLICATION_PINTEREST,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_90,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_NET2PHONE,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_FOURSQUARE,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_93,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MEGA,                       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_95,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_VBULLETIN,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_CLOUDME,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_YIKYAK,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_REDMINE,                    PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_IHEART_RADIO,               PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_AVIRA_ANTIVIRUS,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_RES_102,                    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_KASPERSKY_ANTIVIRUS,        PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_AVG_ANTIVIRUS,              PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_SOUNDCLOUD,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_106,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_PRIME_MUSIC,         PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RUNKEEPER,                  PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_MYBOOK,                     PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_NIKEPLUS,                   PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_MAGV,                       PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_MAGV_KIDS,                  PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_RES_113,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LETV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MY_FITNESS_PAL,             PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_SOHU,                       PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_1KXUN,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TVBS,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_XNXX,                       PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_CTITV,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_121,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_122,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORKOUT_TRAINER,            PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_RES_124,                    PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_RES_125,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_126,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BITTORRENT,                 PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_EDONKEY,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_129,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_GNUTELLA,                   PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_DIRECTCONNECT,              PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_APPLEJUICE,                 PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_SOULSEEK,                   PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_ARES,                       PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_FREENET,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_136,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_SKYPE,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_138,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_YAHOO,                      PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_VONAGE,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_141,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_GADUGADU,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_MPLUS,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_144,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_145,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_146,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DDL_4SHARED_COM,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_148,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_149,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_150,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_151,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_152,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_153,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_DATA_HU,                PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_DEPOSITFILES_COM,       PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_156,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_157,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_158,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILE_UPLOAD_NET,        PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_160,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILEARCHIV_RU,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_162,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_163,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILEFACTORY_COM,        PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_165,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_166,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_167,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_168,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILER_NET,              PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_170,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_171,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_172,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_173,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_174,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_175,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_GIGASIZE_COM,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_177,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_178,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_179,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_180,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_181,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_LETECKAPOSTA_CZ,        PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_183,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_184,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_LOAD_TO,                PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_186,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_MEDIAFIRE_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_188,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_MEGASHARE_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_190,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_191,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_192,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_193,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_194,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_195,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_196,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_197,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_198,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_199,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_200,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_201,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_202,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_SENDSPACE_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_204,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_205,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_206,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_207,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_208,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_209,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_210,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_211,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_212,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_213,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_214,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_215,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_216,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_217,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_218,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_219,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_220,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_221,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_222,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_223,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_224,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_225,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_226,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_227,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_228,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_229,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_230,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_1FICHIER_COM,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_232,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_233,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_234,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_235,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_236,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_237,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_238,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_239,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_240,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_241,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_242,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_243,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_244,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_245,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_246,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_EASYBYTEZ_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_248,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_249,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_250,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_251,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FASTFILESHARE_COM_AR,   PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_253,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_254,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_255,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_256,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_257,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_258,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_259,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_260,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_261,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_262,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FILESMONSTER_COM,       PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_264,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_265,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_266,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_FSHARE_VN,              PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_268,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_GIGAPETA_COM,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_270,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_271,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_HITFILE_NET,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_HULKSHARE_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_HYPERFILESHARE_COM,     PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_275,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_276,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_277,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_278,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_279,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_280,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_281,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_282,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_283,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_MULTISHARE_CZ,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_285,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_286,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_287,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_288,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_QSHARE_COM,             PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_290,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_291,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_SCRIBD_COM,             PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_293,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_294,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_295,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_296,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_297,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_298,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_TURBOBIT_NET,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_300,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_ULOZ_TO,                PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_302,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_303,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_UP_4SHARE_VN,           PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_305,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_306,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_307,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_308,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_309,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_310,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_311,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_312,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_313,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_314,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_315,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_316,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_ZIDDU_COM,              PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_318,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_319,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_320,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_321,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_322,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_2SHARED_COM,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_324,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_325,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_326,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_327,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_328,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_329,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_330,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_331,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_332,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_333,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_334,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_335,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_336,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_337,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_338,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_339,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_340,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_341,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_342,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_343,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_344,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_345,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_346,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_347,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_348,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_349,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_350,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_351,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_UPTOBOX_COM,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_353,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_GAMEFRONT_COM,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_355,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_356,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_TURBOBIT_RU,            PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_USENET,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_359,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_FILETOPIA,                  PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_361,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_IMESH,                      PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_363,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_364,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_365,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_366,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_367,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ZATTOO,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QQ,                         PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_QQLIVE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_371,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_POPO,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_THUNDER,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_UUSEE,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VNC,                        PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_STEAM,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HALFLIFE2,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SOFTETHER,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TELNET,                     PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_OFF,                        PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_TOR,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_WORLDOFWARCRAFT,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HAMACHI_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SKINNY,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_CITRIX,                     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PALTALK,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_QUAKE,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SECONDLIFE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BATTLEFIELD,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_390,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_ICECAST,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHOUTCAST,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QQGAME,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VEOHTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_395,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_PANDORA,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_IPP,                        PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_WARCRAFT3,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_399,                    PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_400,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_401,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_402,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_IPTV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_YOURFREEDOM,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VOIPSWITCH_VOIPTUNNEL,      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_406,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_FUNSHION,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AIMINI,                     PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_RES_409,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FLORENSIA,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TEAMSPEAK,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_MAPLESTORY,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_413,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_414,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DOFUS,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FIESTA,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SPLASHFIGHTER,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ACTIVESYNC,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_419,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_CROSSFIRE,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_421,                    PACE2_PA_TAG_MOBILE)\
        X(PACE2_APPLICATION_GUILDWARS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ARMAGETRON,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RFACTOR,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TEAMVIEWER,                 PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_GAMEKIT,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ULTRABAC,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SPOTIFY,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_IMPLUS,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_430,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_ULTRASURF,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VIBER,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_TANGO,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_IPLAYER,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OPERAMINI,                  PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_BOLT,                       PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_RES_437,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_WHATSAPP,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_WEBEX,                      PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_NETFLIX,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_441,                    PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_IMO,                        PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_CITRIX_GOTO,                PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_444,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_445,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VENTRILO,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_447,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_EBUDDY,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ADOBE_CONNECT,              PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RES_450,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_451,                    PACE2_PA_TAG_WEB)\
        X(PACE2_APPLICATION_WEBDAV,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_453,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_454,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_455,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_456,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_457,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_458,                    PACE2_PA_TAG_WEB)\
        X(PACE2_APPLICATION_PDPROXY,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_CYBERGHOST,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_GOOGLE_CLOUD,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_DRIVE,               PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_GOOGLE_DOCS,                PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_RES_464,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ORACLE_DB,                  PACE2_PA_TAG_DATABASE)\
        X(PACE2_APPLICATION_ITV,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_467,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_468,                    PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_CHANNEL_4OD,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_470,                    PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_POISON_IVY,                 PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_HIDRIVE,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_WECHAT,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_474,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KIK_MESSENGER,              PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_LINE,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_IPERF,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_OS_UPDATE,                  PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_HIKE_MESSENGER,             PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_480,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_IDRIVE,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_482,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_483,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DOTA2,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_QVOD,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HULU,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MSEXCHANGE,                 PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_MSNAV,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_FUZE,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_KAKAOTALK,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_DESTINY,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VYPR_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_THREEMA,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_494,                    PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_TWITCH,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WOWZA,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RUNTASTIC,                  PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_TELEGRAM,                   PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_STREETVOICE,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DUOKAN,                     PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_RES_501,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SKOUT,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_503,                    PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_GAME_OF_WAR,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FETNET,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FBS,                        PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_RES_507,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PINKFONG,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KOLLECT_BOOK,               PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_SPTG_TV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_511,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_512,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_513,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_514,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_AOWE,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IWOW_SYSTEX,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_9KU,                        PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_NEXTTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SYNCPLICITY,                PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_RES_520,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_YUANTA_WEB,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_HINET,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MYCARD,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_KONO,                       PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_RES_525,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DUO_DUO,                    PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_WEALTH_GOD,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_ILEADER,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_THREE_BAMBOO,               PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_TAAZE,                      PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_AUDIKO,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PUBU_BOOKBUFFET,            PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_MITAKE,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_HYREAD,                     PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_RES_535,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUALYS,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CONSTANT_CONTACT,           PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RALLY,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_539,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_INTRALINKS,                 PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_LIVEPERSON,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SPEEDTEST,                  PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_BLUEJEANS,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_APPLE_MUSIC,                PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_NAVER,                      PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_BOOKING_COM,                PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_CNN,                        PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_548,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PAYEASY,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_LINE_RANGERS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IOS_APP_STORE,              PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_MYVIDEO_TW,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KING_GAMING,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CANDY_CRUSH,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CANDY_CRUSH_SODA,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BUBBLE_WITCH,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FARM_HERO,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PLAYSTATION,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOOGLE_PLAY,                PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_SUPERCELL,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CLASH_OF_CLANS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BOOM_BEACH,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HAY_DAY,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WINDOWS_STORE,              PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_SKYPE_FOR_BUSINESS,         PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MINECRAFT,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_567,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ROVIO_GAMING,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ANGRY_BIRDS,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CITRIX_XENAPP,              PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_REDDIT,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_LEAGUE_OF_LEGENDS,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BATTLENET,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PAYPAL,                     PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_PLANTS_VS_ZOMBIES_2,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HALFBRICK_STUDIOS,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GITHUB,                     PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_HAMIVIDEO,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_579,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_NICONICO,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ALIPAY,                     PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_RES_582,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ZALO,                       PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_NTT_DOCOMO,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SOFTBANK_ONLINESHOP,        PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_GUILDWARS2,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TRENDMICRO_WORRYFREE,       PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_WIKIPEDIA,                  PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_UBER,                       PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_YANDEX,                     PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_VK,                         PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ODNOKLASSNIKI,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_593,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ALIBABA,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_ALIEXPRESS,                 PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_TMALL,                      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_EA_ORIGIN,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NORTON_SECURITY,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_TIBIA,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_POCHTA_ROSSII,              PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_601,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DRIVELOCK,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_603,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_604,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_EA_GAMES,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_606,                    PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_POPCAP,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SURVEY_MONKEY,              PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_NEW_RELIC,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_HUDDLE,                     PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_ZENDESK,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_612,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_613,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_DENA,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BACKBLAZE,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_DOCUSIGN,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RES_617,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_618,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ANGHAMI,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PATH_OF_EXILE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SLACK,                      PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_POKEMON_GO,                 PACE2_PA_TAG_VR_AR)\
        X(PACE2_APPLICATION_NIANTIC_LABS,               PACE2_PA_TAG_VR_AR)\
        X(PACE2_APPLICATION_2GIS,                       PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_RUTUBE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_626,                    PACE2_PA_TAG_WEB)\
        X(PACE2_APPLICATION_PRISMA,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_628,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_629,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_AMAZON_PRIME_VIDEO,         PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CLASH_ROYALE,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOOGLE_SHARED_SERVICES,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SIGNAL,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_634,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_635,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_636,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_637,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_EDISK,                  PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_639,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_AZAR,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SKY_GO,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_642,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_MAIL_RU,                    PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_RES_644,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_645,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_KINOPOISK,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_IVI_RU,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TINDER,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ANYDESK,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_ICQ,                        PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_TENOR,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_GOOGLE_MAPS,                PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_GOOGLE_EARTH,               PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_CHANNEL_ONE,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GIPHY,                      PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_VPN_X,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IMGUR,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ONEDRIVE,                   PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_NINTENDO_NETWORK,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AMAZON_CHIME,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_IFLIX,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PPFILM,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPLE_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ICFLIX,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LOGMEIN,                    PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_PSIPHON,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_WAZE,                       PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_GOOGLE_FCM,                 PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_APNS,                       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_DIGICEL_PLAYGO,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHAZAM,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_673,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_DIGICEL_TOPUP,              PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_TUNEIN_RADIO,               PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_676,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SYNOLOGY,                   PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_MY_DIGICEL,                 PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_RES_679,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_RES_680,                    PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_BADOO,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_AIRBNB,                     PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_HIGHTAIL,                   PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_EASY_TAXI,                  PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_TOMTOM,                     PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_TRIPADVISOR,                PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_ESPN,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DAZN,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MOBILE_STRIKE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_KING_OF_AVALON,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LINEAGE_2,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LYFT,                       PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_TRULIA,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_AKAMAI_CLOUD,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_RES_695,                    PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_NICKELODEON_PLAY,           PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HERE,                       PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_SIMCITY_BUILDIT,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_STARWARS_GOH,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FASTLY,                     PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_SAP,                        PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WORLD_OF_KUNG_FU,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CLOUDFLARE,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_LIFE_CHURCH,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_705,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GRINDR,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_APPLE_MAPS,                 PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_APPLE_GEOLOCATION,          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_DDL_UPLOADED,               PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_DDL_EASYPASTE_ORG,          PACE2_PA_TAG_SHAREHOSTING)\
        X(PACE2_APPLICATION_RES_711,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_YOUKU_TUDOU,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MEETIC_GROUP,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_714,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_VUDU,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WISH,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_AUTH0,                      PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_SLINGTV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GOOGLE_APIS,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RACKSPACE_CLOUD,            PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_CALENDAR,            PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_GOOGLE_ANALYTICS,           PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STACKPATH_CLOUD,            PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_RES_724,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_LEARNINGHUB_ONLINE,         PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_GOOGLE_ADS,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FOX_SPORTS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AMAZON_DRIVE,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CRASHLYTICS,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_FACEBOOK_CLOUD,             PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_360TOTAL,                   PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_TURNER,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_734,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_LOOP_CARIBBEAN_LOCAL_NEWS,  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_LOOP_PACIFIC_LOCAL_NEWS,    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_UNITY,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FUNPLUS,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MAPBOX,                     PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_CEDEXIS,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ZENMATE_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SMULE,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PIA,                        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_744,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_745,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_IPVANISH,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TUNNELBEAR,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_GOOGLE_APP_ENGINE,          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WIRE_MESSENGER,             PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_EPIC_BROWSER_PROXY,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_751,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_752,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TIGER_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_EPIC_GAMES,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AVIRA_PHANTOM_VPN,          PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TURBO_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IQIYI,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_758,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PPTV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SNAP_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_761,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_VPN_UNLIMITED,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_763,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_AVAST_SERVICES,             PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_NORD_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PROTON_VPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SAFERVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_HIDE_ME_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_EXPRESSVPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TUNNELGURU,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_YOUTUBE_MUSIC,              PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_WINDSCRIBE_VPN,             PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IVPN,                       PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DISCORD,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_FIREVPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BOTIM,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_HI_VPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VPN_PROXY_MASTER,           PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ZOOGVPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IVACY_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PUREVPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MULLVAD_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_FREEDOME_VPN,               PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_784,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_STRONGVPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_786,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_WARGAMING_NET,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_YANDEX_MUSIC,               PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_VPN_AREA,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_STEGANOS_ONLINE_SHIELD_VPN, PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SUPERVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_STEGANOS_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_FROOTVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_794,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_RES_795,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_CME,                        PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_YANDEX_NAVIGATOR,           PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_YANDEX_DISK,                PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_YANDEX_MAIL,                PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_TIKTOK,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_YANDEX_MAPS,                PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_RES_802,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_GAANA,                      PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_XCUM_COM,                   PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_LIFE_RU,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_806,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_807,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_808,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_YEECALL,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_SHELLFIRE_VPN,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_811,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_REN_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HOLA_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_HOLA_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CDNVIDEO,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_RES_816,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ARKVPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_FORTNITE,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AMAZE_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VUNGLE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_VIVOX,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_MEGOGO,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FINCH_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_FAST_SECURE_VPN,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TVPLAYER,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORLD_OF_TANKS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_827,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_828,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_APPVPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_X_VPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DOTVPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ENCRYPT_ME,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_833,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DROID_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BROWSEC_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_836,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_837,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_APEX_LEGENDS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VPN_365,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_COUNTERSTRIKE_GO,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VPN_TOR_BROWSER,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SUPERVPN_BY_CHENG,          PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ACT_VPN,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRIGHTCOVE,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_SPORTBOX_RU,                PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_JUST_PROXY_VPN,             PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VEEPN,                      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_OOKLA,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_EAGLE_VPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRIA_MOBILE,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_VPN_GATE,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ANCHORFREE_SERVICES,        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRIA_TEAMS,                 PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_SHADOWSOCKS,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ASPHALT9,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GAMELOFT_SERVICES,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ACTMOBILE_SERVICES,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_NOVATECH_CORE,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SECURE_VPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_APPSFLYER,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADJUST,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_GENIEVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_863,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_HELPSHIFT,                  PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_FAST_VPN_MASTER,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VPN_UNLIMITED_PROXY,        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_HIGHSPEED_VPN,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VPN_MASTER_BY_ANDRO_TEAM,   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_869,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TENCENT_GAMES,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_871,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_872,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DOUYU,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_INGRESS_PRIME,              PACE2_PA_TAG_VR_AR)\
        X(PACE2_APPLICATION_ELECTRONIC_ARTS,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RADIOPLAYER,                PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_MOPUB,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PUBG_MOBILE,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_879,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SPIEGEL_TV,                 PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_SPIEGEL_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_882,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FORGE_OF_EMPIRES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_INNOGAMES,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RECEIVER_RADIO,             PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_GOODGAME_STUDIOS,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_887,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RADIO_DE,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_BBC_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_BBC_SERVICES,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_UNREAL_ENGINE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ZOOM,                       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_ADOBE_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ATT_SERVICES,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FUNK,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NEXX_TV,                    PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_UNBLOCK_SITES,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRAWL_STARS,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_COUNTERPATH_SERVICES,       PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_SNAP_MASTER_VPN,            PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_JOYN,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PRO7SAT1_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_DEAD_RIVALS,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TAPJOY,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_IGG,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LORDS_MOBILE,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MUDFISH,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_908,                    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MIXPANEL,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADCOLONY,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_911,                    PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_BIG_FARM,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FACEAPP,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ASPHALT8,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MOAT,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADSAFEPROTECTED,            PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADFORM,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FREEWHEEL,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SPOTX,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_IRONSOURCE,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TRAFFICJUNKY,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TRAFFICFACTORY,             PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HATCH_CLOUD_GAMING,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_APPLOVIN,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DELTADNA,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADTIMING,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FLURRY,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_OPTIMIZELY,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SMARTADSERVER,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EXACTAG,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SCORECARDRESEARCH,          PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CONVIVA,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BAMTECH_MEDIA,              PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BEBI,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HBO_SERVICES,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_936,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CRITEO,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SWRVE,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CHARTBOOST,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_940,                    PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_DIRECTV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VPNSECURE,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_LIFTOFF,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_944,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_DISNEY_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MY5,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SOURCE_ENGINE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GEOCACHING,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FOX_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_KALTURA,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RES_951,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RES_952,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FOX_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BRANCH,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_VIKI,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORDPRESS,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_GRAVATAR,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PARSELY,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_959,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PBS,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CRACKLE,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OPENX,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_963,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DISNEY_RADIO,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KOCHAVA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ABC,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_967,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_APTELIGENT,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_APPLE_NEWS,                 PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_970,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SEGMENT,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TUNE,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_APPLE_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPTENTIVE,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_AIRABLE,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_PLUTO_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_977,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_978,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SONY_PICTURES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NFL_SUNDAY_TICKET,          PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PORNHUB_NETWORK,            PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_PDANET,                     PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_MLB,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MLB_AT_BAT,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TUBI,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WATCH_TBS,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DISNEY_PLUS,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OPENSIGNAL,                 PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_SHOWTIME,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUANTUM_METRIC,             PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MPARTICLE,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SHOWTIME_ANYTIME,           PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CANAL_PLUS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AUDIBLE,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_TEALIUM,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_NUANCE,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CBS_SERVICES,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NBC_SPORTS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_EUROSPORT_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ENDEAVOR_STREAMING,         PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_TOTOK,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_FYBER,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_UNIVISION_NOW,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TABOOLA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1005,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SKY_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_USERVOICE,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_BIGO_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BITTORRENT_NOW,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MARIO_KART_TOUR,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TURKCELL_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SKY_SPORTS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BIP_MESSENGER,              PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_SWOOGO,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NBC_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ETISALAT_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SWITCH_TV,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MICROSOFT_TEAMS,            PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_FLASHTALKING,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1020,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1021,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_NIELSEN_SERVICES,           PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FOX_BUSINESS,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_PRIVATEVPN,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SURFSHARK_VPN,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_YOSPACE,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SAPLING_GROWTH_VPN,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_NFL_SERVICES,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_XUNLEI_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_THE_NBC_APP,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FIZY,                       PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_CLOUDINARY,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_TENCENT_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ULTRAVPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_RES_1035,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_STARZ,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WECHAT_WORK,                PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_RESILIO_SYNC,               PACE2_PA_TAG_P2P)\
        X(PACE2_APPLICATION_IMGIX,                      PACE2_PA_TAG_DATABASE)\
        X(PACE2_APPLICATION_RES_1040,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_KAKAO_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SOLVVY,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_JW_PLAYER,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STACKLA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MUX,                        PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_FRESHDESK,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_FRESHWORKS,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AMPLITUDE,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1049,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_UNDER_ARMOUR_SERVICES,      PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_MAP_MY_FITNESS,             PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_TRUSTARC,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MICROSOFT_EXCEL,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MICROSOFT_WORD,             PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MICROSOFT_POWERPOINT,       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MICROSOFT_OUTLOOK,          PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_SNAP_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BOOTSTRAPCDN,               PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_KAKAOSTORY,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RES_1060,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SPORTSMAX,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KAKAOTV_LIVE,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPLICASTER,                PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_SHAREPOINT,       PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_RES_1065,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_INDEX_EXCHANGE,             PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NBC_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_FUBOTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPDYNAMICS,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TOUR_TRACKER,               PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_BLACKBOARD,                 PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CHARTBEAT,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EBOT7,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_ONENOTE,          PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_HBO_GO,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MEDIAMATH,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_VILYNX,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_WWE,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1079,                   PACE2_PA_TAG_WEB)\
        X(PACE2_APPLICATION_RES_1080,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_SENTRY,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_YUPPTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PERIMETERX,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NBA,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PLAYRIX,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_APP_BAZAAR,                 PACE2_PA_TAG_APPSTORES_UPDATES)\
        X(PACE2_APPLICATION_HOTJAR,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TAPLYTICS,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RTVE_ALACARTA,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HAYU,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GIGYA,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_TNT,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHOWMAX,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ATRESMEDIA,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CDNETWORKS,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_CLASSROOM,           PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_PEER5,                      PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_SATISFI_LABS,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_AYOBA,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ROBLOX,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NHL,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_INNOVATIVE_CONNECTING,      PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_OUTBRAIN,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STAN,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VPN_MONSTER,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MEDIACORP_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ATRESPLAYER,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AIRSHIP,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DIDI_SERVICES,              PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_MEDIASET_ESPANA,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MITELE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MEWATCH,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1113,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1114,                   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NIKE_SERVICES,              PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_ACCEDO,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_99TAXIS,                    PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_CHARTER_COMMUNICATIONS,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VIU,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PCCW,                       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ALIBABA_CLOUD,              PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_LOCALYTICS,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TIDAL,                      PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_RES_1124,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_YOUBORA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_MEET,                PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_COINBASE,                   PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_WIDEORBIT,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_MEETRICS,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CONDUCTRICS,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_LAUNCHDARKLY,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ART19,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_HIVEMQ,                     PACE2_PA_TAG_M2M_IOT)\
        X(PACE2_APPLICATION_HANDELSBLATT,               PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_PODIGEE,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_CLOUD_MAIL_RU,              PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_INNOVID,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MOVISTAR_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_UCWEB,                      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_WIZROCKET,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BATCH,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_VIZBEE,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_WIRTSCHAFTSWOCHE,           PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_HANDELSBLATT_SERVICES,      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_IQ_DIGITAL,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MEET_GROUP_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MOVISTAR_PLUS,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WIZZEE,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_INTERCOM,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_NETEASE,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PENTHERA,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RIOT_GAMES,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BYTEDANCE,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_1154,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TAGGED,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WEBTOON,                    PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_MEETME,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_DYNAMIC_YIELD,              PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ELEX_TECHNOLOGY,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CLASH_OF_KINGS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IMDB,                       PACE2_PA_TAG_DATABASE)\
        X(PACE2_APPLICATION_BILLO,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AIHELP,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CRUNCHYROLL,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHEIN,                      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_AVIRA_SERVICES,             PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_SAMSUNG_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_XIGUA_VIDEO,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TOUTIAO,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_BUZZSPROUT,                 PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_MEGAPHONE,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BUGSNAG,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_KNOTCH,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_WURL,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_LIBSYN,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_TRITON_DIGITAL,             PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_VNG_CORPORATION,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BAIDU_MAPS,                 PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_MOONTON,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AMAGI,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_KUAISHOU,                   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MARKMONITOR,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MEDIUM,                     PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_SIMPLECAST,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_WALMART,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_GRUBHUB,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_KWAI,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_BRAZE,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_KEEPSOLID_SERVICES,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_VPN_LITE,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_SPRINGSERVE,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1192,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WORKPLACE,                  PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_YAANI,                      PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_WASHINGTON_POST,            PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RES_1196,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ROMWE,                      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_TARGET_SIGNATURE,           PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PLEX,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_ALEXA,               PACE2_PA_TAG_SMART_HOME)\
        X(PACE2_APPLICATION_BUMBLE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WEATHER_CHANNEL,            PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_REUTERS_NEWS,               PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CRYPTOTRADER,               PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_RES_1205,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CLUB_FACTORY,               PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_BOMBORA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MOOV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_COLLECTD,                   PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_ORACLE_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_COMCAST,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SKRED_MESSENGER,            PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ACAST,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_THE_CW,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CW_SEED,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FACEBOOK_LITE,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_OPERA_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_THE_WALL_STREET_JOURNAL,    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_TEXTNOW,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_BLOOMBERG,                  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_UDEMY,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_THE_ECONOMIST,              PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_INVESTING_COM_SERVICES,     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_CRYPTOWATCH,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_AMONG_US,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_COOKIEPRO,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_BEIN_SPORTS,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BEIN_MEDIA_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ERSTREAM,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_WE_MEET,                    PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_NEWSDOG,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_PUGPIG,                     PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_DIGITURK_PLAY,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DIGICEL_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_DOORDASH,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PHOTO_WONDER,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_QUANTCAST,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_JIOSAAVN,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_OPERA_NEWS,                 PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_XHAMSTER,                   PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_SHAHID,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MBC_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_QQMUSIC,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SEAMLESS,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_CIRES21,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SPLUNK,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_COINCAP,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_SHAPESHIFT_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AL_JAZEERA_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AL_JAZEERA_ENGLISH,         PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_DO_GLOBAL_SERVICES,         PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_WEBFLOW,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_PAYBACK,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_MEITU_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CAMSCANNER,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_INTSIG_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_JAUMO,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PUBMATIC,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_JIO_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GROUPON,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_RES_1261,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_VERIZON_MEDIA_SERVICES,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ACTIVISION_BLIZZARD,        PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_AUDIOMACK,                  PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_KAKAO_MAP,                  PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_OSN,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GSMA,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_GOOGLE_JIBE,                PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_BEAUTYPLUS,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_VIVAVIDEO,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CALL_OF_DUTY,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MICOWORLD_SERVICES,         PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_HAGO,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DAUM_CAFE,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_TENCENT_NEWS,               PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_FLIPBOARD,                  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_VIACOM18_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VOOT,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SELFIECITY,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MEITU,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_KASPERSKY_SERVICES,         PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_HAKUNA,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_BERMUDA_VIDEO_CHAT,         PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_XIAOMI_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_OPERA_NEWS_LITE,            PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_GAIJIN_ENTERTAINMENT,       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SCOPELY_SERVICES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MARVEL_STRIKE_FORCE,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SKY_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_SYSTEX_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PERFECT_CORP_SERVICES,      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_YOUCAM_MAKEUP,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DTAG_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PUBG,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOOGLE_VOICE,               PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_THE_SCORE,                  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_PANTAYA,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DAUM_MAIL,                  PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_FACEBOOK_BROADCASTING,      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WAR_THUNDER,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_APUS_SERVICES,              PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_UFOTO_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PEAK_GAMES_SERVICES,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TOON_BLAST,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_EXPO_FRAMEWORK,             PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_THE_NEW_YORK_TIMES,         PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_DU_RECORDER,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RES_1308,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MEIZU_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_JOYY_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MOBVISTA,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STARTIMES_ON,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VODAFONE_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_THE_GUARDIAN,               PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_GUARDIAN_MEDIA_SERVICES,    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MOZILLA_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MAGNITE,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DC_UNIVERSE,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RAKUTEN_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_PLAYDEMIC_SERVICES,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOLF_CLASH,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ONETRUST,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_LIGHTSTEP,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1324,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_VS_MEDIA,                   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_EXARING_SERVICES,           PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WAIPU_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GOOGLE_NEWS,                PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RING_CENTRAL,               PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_MORTAL_KOMBAT,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VPN_VALLEY,                 PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BEIN_CONNECT,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BOOMPLAY_MUSIC,             PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_KONGREGATE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NINJA_KIWI_SERVICES,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HUAWEI_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_XMRIG,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_LOL_WILD_RIFT,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TINKOFF,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_YANDEX_MARKET,              PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_SWEET_SELFIE,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SWEET_SELFIE_LITE,          PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_DIGICERT,                   PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_SWITCHERRY_VPN,             PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_XENDER,                     PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_NCSOFT,                     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ORNET,                      PACE2_PA_TAG_BROWSERS)\
        X(PACE2_APPLICATION_COINGECKO,                  PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_UMG,                        PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_MALWAREBYTES,               PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_REALNETWORKS,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_KEYCDN,                     PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_RILEY_CILLIAN,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_STREAMLIVE_TO,              PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WILDLIFE_STUDIOS,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BLOCK_CRAFT_3D,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_1357,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_LOVOO,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_CLUBHOUSE,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SNIPER_3D,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_COMODO_SERVICES,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_RIBBON_COMMUNICATIONS,      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BLACKBERRY_WORK,            PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_JOOM,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_BLACKBERRY_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GOOGLE_PAY,                 PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_GEEKFLARE,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WAR_MACHINES,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RAWVOICE,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_PODTRAC,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_WISE,                       PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_GOOGLE_PODCASTS,            PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SHALLTRY_GROUP,             PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_LIVERAMP,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EURONEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_LEAP_FITNESS_GROUP,         PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_HEXAGLOBE,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ROCKET_LEAGUE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MAGINE_SERVICES,            PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_PAY_RW,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_VUBA_VUBA,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_YANDEX_GO,                  PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_YANDEX_TELEMOST,            PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_NEXON_CORPORATION,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AGODA,                      PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_GRAB,                       PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_NYPR,                       PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PRX_SERVICES,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_DOVETAIL,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HUYA,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHAREIT,                    PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_EVERSEND,                   PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_5CHANNEL,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PET_RESCUE_SAGA,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CLASSDOJO,                  PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_PHOTOMATH,                  PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_MIHOYO_GAMES,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GENSHIN_IMPACT,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HONKAI_IMPACT_3RD,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_KHAN_ACADEMY,               PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_HOMER,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CASTBOX,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_OPENTEXT_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_EDX,                        PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_SOCRATIC,                   PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_TELIA_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_TELIA_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PEACOCK_TV,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_JIO_ADS,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DISCOVERY_PLUS,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DUOLINGO,                   PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_RES_1412,                   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_BET_PLUS,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PARAMOUNT_PLUS,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_HUBSPOT,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PHILO,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DNA_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_XIAOMI_WEATHER,             PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_JIOCHAT,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_REMIND,                     PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_ELISA_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ELISA_VIIHDE,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PUBNUB,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_FUNIMATION,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DRIFT,                      PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_IMG,                        PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ENDEAVOR_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SOFTBANK_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_REFACE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MTV_KATSOMO,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_ADS,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PRENDE_TV,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_UNIVISION_COMMUNICATIONS,   PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_ANCHORFM,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_VEVE,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_RES_1436,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUALTRICS,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WANGSU_CDN,                 PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_ALGOLIA,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MAXIVISION,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_G_CORE_SERVICES,            PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_NORDIC_ENTERTAINMENT_GROUP, PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_VIAFREE,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NETMERA,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_GRACENOTE,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_STORYFM_CN,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_YLE_AREENA,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_STREAMABLE,                 PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SPEAKOL,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_YALLAKORA,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LINKONLINE,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GEMINI_MEDIA_EGYPT,         PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SANOMA_SERVICES,            PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RUUTU,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FINNPANEL,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NOON_VPN,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MASRAWY,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_KTPLAY,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_TEAM_FORTRESS_2,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TRUST_WALLET,               PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_CANAL_PLUS_AFRIQUE,         PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SARMADY_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_FILGOAL,                    PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_AMC_THEATRES,               PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_YOUM7,                      PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_VALVE_CORPORATION,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_1467,                   PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_XOOM,                       PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_IBM,                        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MIDOKI_SERVICES,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VPN_MASTER_X,               PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_QUALAROO,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MTS_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_EXPEDIA,                    PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_GLOBALSIGN,                 PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_LETSENCRYPT,                PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_YIELDMO,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_IDENTRUST,                  PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_GODADDY,                    PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_GOOGLE_TRUST_SERVICES,      PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_WATCH_IT,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AMAZON_TRUST_SERVICES,      PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_ENTRUST,                    PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_BINANCE,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_ELFSIGHT,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SAILTHRU,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CERTUM,                     PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_MEDIANOVA_CDN,              PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_NAPSTER,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SECTIGO,                    PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_BAZAARVOICE,                PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_SUNING_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_INDIGITALL,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ADA_SUPPORT,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_PENGUIN_SPORTS,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PUSHER,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_CSG_MEDIA,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_GRAPESHOT,                  PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_KNIGHTHOOD,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BLOSSOM_BLAST_SAGA,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CYBERTRUST_CA,              PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_USERCENTRICS,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_IDVIU,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_IAPPS_TECHNOLOGIES,         PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CALM,                       PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_PLAYSTATION_NOW,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MICROSOFT_AZURE,            PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_ZEGO,                       PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_DISH_NETWORK,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_JQUERY,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MOX_TV,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STATS_PERFORM,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CONTENTFUL,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WETV,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ADEX,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ARLO,                       PACE2_PA_TAG_SMART_HOME)\
        X(PACE2_APPLICATION_JIOHEALTHHUB,               PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_GOG,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RING,                       PACE2_PA_TAG_SMART_HOME)\
        X(PACE2_APPLICATION_FITBIT,                     PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_GOOGLE_FIT,                 PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_DIAMOND_DIARIES_SAGA,       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DELUXE_MEDIA,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_THE_TRADE_DESK,             PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_JIOCLOUD,                   PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_TRIVAGO,                    PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_MINUTELY,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_GARENA_SERVICES,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GARENA_FREE_FIRE,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IMPACT_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SAMSUNG_MAX_PRIVACY_VPN,    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PRIVATE_TUNNEL_VPN,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_TRIPLELIFT,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1534,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MAILCHIMP,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_A1_BELARUS,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_VOKA_TV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SPB_TV,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SMART_MEDIA_SOLUTIONS,      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SMAATO,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TEADS,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PYRAMID_SOLITAIRE_SAGA,     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RISKIFIED,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_RES_1544,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RES_1545,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LEANPLUM,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_TAPAD,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SQUARESPACE,                PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_BRIGHTLINE,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EXTREME_REACH,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ROCKETVPN,                  PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MOOVIT,                     PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_COINBOX,                    PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_SYBO_GAMES,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SUBWAY_SURFERS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NPR,                        PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_ESET_SERVICES,              PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_MAKEUPPLUS,                 PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_KAKAO_WEBTOON,              PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_ZEE5,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_JUSPAY,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_POKEMON_UNITE,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ZOMATO,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PLYMEDIA,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BITPAY,                     PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_PHYCAN_VPN_SERVICES,        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_INTERNET_ARCHIVE,           PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CLOUDFLARE_WARP_VPN,        PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_BRD_WALLET,                 PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_EXODUS_WALLET,              PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_BEAUTYCAM,                  PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_GAB,                        PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MEASUREMENT_LAB,            PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_GLOVO,                      PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_SAMSUNG_MUSIC,              PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_SWEET_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_UKLON,                      PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_TOCA_BOCA,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TAVR_MEDIA,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_LIGHTSAIL_VPN,              PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_IBM_CLOUD_VIDEO,            PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AGORA_SERVICES,             PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SUN_FM_UKRAINE,             PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_OLL_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LANET_TV,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TIGO_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_UBER_EATS,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_BOLT_TECHNOLOGY,            PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_STRIPE,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_MEDALLIA,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ONEPLUSONE_MEDIA_SERVICES,  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ONEPLUSONE_VIDEO,           PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VERISIGN,                   PACE2_PA_TAG_CERTIFICATE_AUTHORITY)\
        X(PACE2_APPLICATION_RES_1594,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ADMIXER,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DINGTALK,                   PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_ONLINE_RADIO_BOX,           PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_MOLOCO,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RES_1599,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PROSTREAM_RU,               PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BLOCKCHAIN_COM,             PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_QUALCOMM,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_JIOMART,                    PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_JIOMEET,                    PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_HCL_NOTES_AND_DOMINO,       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_MI_FIT,                     PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_PAXFUL_WALLET,              PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_SCHILDBACH_WALLET,          PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_CAKE,                       PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_TIGERCONNECT,               PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_ICLOUD_PRIVATE_RELAY,       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_LUDO_KING,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TWILIO_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_ANKAMA_GAMES,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_META_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_HUAWEI_HEALTH,              PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_YOUDAO,                     PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_XINHUA_NEWS,                PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RAYGUN,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_GOOGLE_CHAT,                PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_DOUBLEVERIFY,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EXTOLE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STARMAKER,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MX_PLAYER,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ORIGINS_DIGITAL,            PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_SYGIC_SERVICES,             PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_BUDGE_STUDIOS,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MTS_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SMADEX,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_AMAZON_GAMES,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NEW_WORLD,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AARKI,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_XIAOMI_SHAREME,             PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_GOODBARBER,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_LITRES_RU,                  PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_QWILT,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_START_RU,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_POU,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MUMBLE,                     PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_AMEX_SERVICES,              PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_WIZARDS_OF_THE_COAST,       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MAGIC_THE_GATHERING_ARENA,  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MOZILLA_VPN,                PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_DEUTSCHE_KREDITBANK,        PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_DEUTSCHE_BANK,              PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_NORTONLIFELOCK_SERVICES,    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_NORTONLIFELOCK_VPN_SERVICES, PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_ZAPYA,                      PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_AMOBEE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_UBITUS_SERVICES,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NINTENDO_SWITCH_CLOUD_GAMING, PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SPREAKER,                   PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_JIOCINEMA,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_EPSILON_CONVERSANT,         PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_NVIDIA_SERVICES,            PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WINK,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ROSTELECOM,                 PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NGENIX,                     PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_INTEL_SERVICES,             PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_AMD_SERVICES,               PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_PIKMIN_BLOOM,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_123VPN,                     PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_PLAYTIKA_SERVICES,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AVG_SERVICES,               PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_NETFLIX_SERVICES,           PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_NETFLIX_GAMES,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HERO_WARS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SWIFTY_BEAVER,              PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_BITDEFENDER_SERVICES,       PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_IHEART_MEDIA_SERVICES,      PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SOPHOS_SERVICES,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_MCAFEE_SERVICES,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_EUROSPORT_NEWS,             PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_OLA,                        PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_LG_ELECTRONICS_SERVICES,    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ERICSSON_SERVICES,          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_SHARECHAT,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VALORANT,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_INDEED,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_NVIDIA_GEFORCE_NOW,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TRUEKEY,                    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_MOON_ACTIVE,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SAFE_FAMILY,                PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_LOTAME,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FIREFOX_PRIVATE_NETWORK,    PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_MEDIA_NET,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_JSDELIVR,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_UBISOFT_SERVICES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_UBISOFT_CONNECT,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BLITZER_DE,                 PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_SAMSUNG_PAY,                PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_GOOGLE_PHOTOS,              PACE2_PA_TAG_FILETRANSFER)\
        X(PACE2_APPLICATION_STELLAR,                    PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_TRACKMANIA,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WANT_WANT_SERVICES,         PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_UNRULY,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TOTALAV,                    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_ATUDO,                      PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_BID_SWITCH,                 PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_ING_BANKING,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_FEEDAD,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_COMMERZBANK_BANKING,        PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_VR_BANKING,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_ANZU_VR,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BEESWAX,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DAUNTLESS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CATCHMEDIA,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PIANO,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SHOPIFY,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_PC_PROTECT,                 PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_CISCO_SYSTEMS_SERVICES,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SPARKASSE_BANKING,          PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_HOLASPARK,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_MICROSOFT_BING,             PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_TESTMY_NET,                 PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_FUELIO,                     PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_SPEEDOF_ME,                 PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_XFINITY,                    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_XFINITY_SPEEDTEST,          PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_PAYCENTIVE,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_RAINBOW_SIX_SIEGE,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_JPMORGAN_CHASE,             PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_MULTI_MEDIA_LLC,            PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_CAPITALONE,                 PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_SYGIC_TRAVEL_MAPS,          PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_GLASSBOX,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STARK_GAMES,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WELLS_FARGO,                PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_BLINKIST,                   PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_GAMES_WORKSHOP,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WARHAMMER_GAMING,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_REDFAST,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_SAMSUNG_HEALTH,             PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_SPEEDSMART,                 PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_SYGIC_GPS_NAVIGATION,       PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_COMCAST_TECH_SOLUTIONS,     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_VIUTV,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DEMANDBASE,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RAINBOW_SIX_EXTRACTION,     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WARHAMMER_TV,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DISNEY_HOTSTAR,             PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WUNDERKIND,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MIGHT_AND_MAGIC_CHESS_ROYALE, PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PULSE_INSIGHTS,             PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_RADARBOT,                   PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_ADSWIZZ,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BRIGHT_DATA,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_WAKANIM,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_ETSY,                       PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_VANILLA,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_CITADEL_COLOUR,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BEHAVIOUR_SERVICES,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DEAD_BY_DAYLIGHT,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BLACK_LIBRARY,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DIGITAL_TURBINE,            PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DUCKDUCKGO,                 PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_LIVEONE,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_JOSH,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BABE,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WIKIMEDIA_FOUNDATION,       PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_AVAST_VPN_SERVICES,         PACE2_PA_TAG_TUNNEL)\
        X(PACE2_APPLICATION_OPENWEB,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_STREAMIO,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_TMT_BROADCASTING,           PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_UNO,                        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VERVE_GROUP,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MCAFEE_SECURITY,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_LOKALISE,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_BRAINLY,                    PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_ZONER_ANTIVIRUS,            PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_ZONER_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_MAGENTA_TV,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_LARK,                       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_CAPCUT,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PODBEAN,                    PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_PERMUTIVE,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_WISTIA,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_RAPPI,                      PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_KION,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OKKO,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_PICSART,                    PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_MX_TAKATAK,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SENDER,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_F_SECURE,                   PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_AAROGYA_SETU,               PACE2_PA_TAG_HEALTH_FITNESS)\
        X(PACE2_APPLICATION_ANIVIEW,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PINDUODUO,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_GEARBOX_ENTERTAINMENT_SERVICES, PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GEARBOX_SHIFT,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AGE_OF_EMPIRES,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RELIC_ENTERTAINMENT,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_XBOX_GAME_STUDIOS,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SPLIT,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_TRUTH_SOCIAL,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RUMBLE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_FANDOM,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_USCREEN,                    PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_HYPERCONNECT,               PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_FONT_AWESOME,               PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_RUBY_GAME_STUDIO,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MOENGAGE,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_LOST_ARK,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MIRO,                       PACE2_PA_TAG_ORGANIZERS)\
        X(PACE2_APPLICATION_GOOGLE_TV,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CADENT,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_PANGLE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EMBRACE,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_STREAMGUYS,                 PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_HOLLER,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_EASYBRAIN,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_EASYBRAIN_MUSIC_MAKER,      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CREATIVE_ASSEMBLY,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SEGA,                       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MATCH_GROUP,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_TOTAL_WAR,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FERAL_INTERACTIVE,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LIVEBOARD,                  PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_NARAKA_BLADEPOINT,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NETEASE_GAMES,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PATREON,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_USPS,                       PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_IAB_EUROPE,                 PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_UPS,                        PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_Q4,                         PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_TIDIO,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_WARNER_BROS_DISCOVERY_SERVICES, PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_LAZADA,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_ONLYFANS,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PRIMIS,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_BLUE_BILLYWIG,              PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_GEOLOCATION_TECHNOLOGY,     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_LUMEN_TECHNOLOGIES,         PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_FRUITSHAKE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_IMPROVE_DIGITAL,            PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TERMINUS,                   PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_TABLEAU,                    PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_TINY_TINAS_WONDERLANDS,     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_AG_PROJECTS_SERVICES,       PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_DISPLAY_IO,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BEETALK,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ZILLOW,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_PLURK,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_INMOBI,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ARC_XP,                     PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_EMBRACER_GROUP,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SPIN_THE_BOTTLE,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_XANDR,                      PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_MAPP,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_9GAG,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SQUARE_ENIX,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FINAL_FANTASY_XIV,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NAVER_WORKS,                PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_NAVER_CORPORATION,          PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_INTUIT,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_GAZPROM_MEDIA,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RES_1856,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RADANCY,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_RAMBLER,                    PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_CRYPTO_COM,                 PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_FIFA_SOCCER,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_KUSTOMER,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_LOGROCKET,                  PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SPOTHERO,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_ADOBE_EXPERIENCE_CLOUD,     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_ICF_TECHNOLOGY,             PACE2_PA_TAG_PORNOGRAPHY)\
        X(PACE2_APPLICATION_TRICOLOR_TV,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_REDFIN,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_MATTERPORT,                 PACE2_PA_TAG_VR_AR)\
        X(PACE2_APPLICATION_NAVER_CLOUD,                PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_ROGUE_COMPANY,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WOW_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUORA,                      PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_INFOBIP,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_EDGIO,                      PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_FLIPKART,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_NEAR,                       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_AVO,                        PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CONTENTSQUARE,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FALL_GUYS,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_YAHOO_AD_TECH,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MEESHO,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_TOWER_OF_FANTASY,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NPERF,                      PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_NBCUNIVERSAL,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NAVER_MAP,                  PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_SNAPDEAL,                   PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_SMARTCLIP,                  PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DIGITALOCEAN,               PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_HAPPN,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_2K,                         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MICROSOFT_FAMILY_SAFETY,    PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_RUMBLEVERSE,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WB_GAMES,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MICROSOFT_DEFENDER,         PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_ADOBE_FONTS,                PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_VIMOND,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_AIONCLOUD,                  PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_AMC_NETWORKS,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AMC_PLUS,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CABIFY,                     PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_PAYTM,                      PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_LENOVO_SERVICES,            PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_UPLIVE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_NAVER_MAIL,                 PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_AKAMAI_TECHNOLOGIES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_STACKADAPT,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HIREZ_STUDIOS,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VIX,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TREMOR_VIDEO,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BLOCK_INC,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CASH_APP,                   PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_OUTFIT7,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_MYTHIC_LEGENDS,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_UI_AG_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GMX_MAIL,                   PACE2_PA_TAG_MAIL)\
        X(PACE2_APPLICATION_INSHORTS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_REVOLUT,                    PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_ZEPETO,                     PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_DIABLO_IMMORTAL,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LIFE360,                    PACE2_PA_TAG_NAVIGATION)\
        X(PACE2_APPLICATION_SESSION,                    PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_PREMIER,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_TENCENT_CLOUD,              PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_PAYPAY,                     PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_6SENSE,                     PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_VIMEO_OTT,                  PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ZEUS_NETWORK,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VIDAZOO,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HIDIVE,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GRUPO_GLOBO_SERVICES,       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_GLOBOPLAY,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SHUDDER,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_KINGSOFT_SERVICES,          PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_KINGSOFT_CLOUD,             PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_KOBO,                       PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_FOR_HONOR,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TREMOR_INTERNATIONAL,       PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_FINANCIAL_TIMES,            PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_KAPE_SERVICES,              PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SQUARE,                     PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_HCAPTCHA,                   PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_AMC_SVOD,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SUNDANCE_NOW,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NAVER_NOW,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_AUSTRALIA_POST,             PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_BUNGIE_GAMES,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ORA_TV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CCP_GAMES,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_EVE_ONLINE,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ZYPE,                       PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_UMENG_PLUS,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_BYJUS,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_TILDA,                      PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_CDN77,                      PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_TAPCLAP,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PIRATE_TREASURES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PRISMIC,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_TENCENT_MUSIC,              PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_HTTPOOL,                    PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_CACHEFLY,                   PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_ALIBABA_GROUP_SERVICES,     PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_QANDA,                      PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_VERCEL,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_STREAMOTION,                PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DRAFTKINGS_SERVICES,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DRAFTKINGS_CASINO,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DRAFTKINGS_SPORTSBOOK,      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WIX,                        PACE2_PA_TAG_BUSINESS)\
        X(PACE2_APPLICATION_FANDUEL_SERVICES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FANDUEL_CASINO,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_FANDUEL_SPORTSBOOK,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_CRYPTIC_STUDIOS,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NEVERWINTER,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RESSO_MUSIC,                PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_XUMO,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_REALM_ROYALE,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VIKINGARD,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BADLANDERS,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_INFINITE_LAGRANGE,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VIEWLIFT,                   PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_TVNZ,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WYNK_MUSIC,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_GEARBOX_PUBLISHING,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VIJU,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VIASAT_RU,                  PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_LILITHGAMES,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_RISE_OF_KINGDOMS,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_DECA_GAMES,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GODS_AND_GLORY,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PODCAST_ADDICT,             PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_EVE_ECHOES,                 PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LAZY_AUDIO,                 PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_AMEDIATEKA,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_MIXI,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MIXI_SERVICES,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_LOTR_WAR,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GOOGLE_TRANSLATE,           PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_NTT_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_RADIKO,                     PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_CYBERAGENT_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_AMEBA,                      PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_RED_HOT_PIE,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_CLICKTOUCH_GAMES,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ADTECH_STUDIO,              PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_MEDIALAB_AI_SERVICES,       PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_MITENE,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_TRANSPERTH,                 PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_HINGE,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_SUPERAWESOME,               PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_YANDEX_TRANSLATE,           PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CASHI,                      PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_BOOKING_HOLDINGS_SERVICES,  PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_BANCO_AZTECA,               PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_GSTATIC,                    PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_DIDI_FOOD,                  PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_DIDI_RIDER,                 PACE2_PA_TAG_TRAVEL_TRANSPORTATION)\
        X(PACE2_APPLICATION_FACEBOOK_MESSENGER,         PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_COPPEL,                     PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_UNCLASSIFIED,               PACE2_PA_TAG_GENERIC)\
        X(PACE2_APPLICATION_OKCUPID,                    PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MERCADO_LIBRE,              PACE2_PA_TAG_E_COMMERCE)\
        X(PACE2_APPLICATION_BAND,                       PACE2_PA_TAG_CONFERENCE)\
        X(PACE2_APPLICATION_MERCADO_PAGO,               PACE2_PA_TAG_WALLET)\
        X(PACE2_APPLICATION_SMILEGATE,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BANCOPPEL,                  PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_V4,                         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_VLMEDIA_SERVICES,           PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MICROSOFT_TRANSLATOR,       PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_CDNJS,                      PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_TELEMETRO,                  PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CORPORACION_MEDCOM,         PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_HERE_TV,                    PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OVERWATCH,                  PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SNOW_SERVICES,              PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_MASTODON,                   PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_GLU_MOBILE_SERVICES,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_KARTRIDER_RUSH,             PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_GROUPM,                     PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_ULTRAVIEWER,                PACE2_PA_TAG_REMOTE_CONTROL)\
        X(PACE2_APPLICATION_KINGDOM_AGE,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_STAR_TREK_ONLINE,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_WEB_DE,                     PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_MAIL_COM,                   PACE2_PA_TAG_SEARCH_ENGINE)\
        X(PACE2_APPLICATION_ARTE,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_VITA,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_PARAMOUNT_SERVICES,         PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BLUTV,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_XIAOHONGSHU,                PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_RAKUTEN_TV,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_APPLE_WEATHER,              PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_MUBI,                       PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_QUANTUMCAST,                PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_KNIGHTS_AND_DRAGONS,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SMITE,                      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TELEPORT_MEDIA,             PACE2_PA_TAG_MULTIMEDIA_SERVICES)\
        X(PACE2_APPLICATION_CBS_NEWS,                   PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_CBS_SPORTS,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_RBC_NEWS,                   PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_CANONICAL_SERVICES,         PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_EXXEN,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_SONY_GROUP_SERVICES,        PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SOUL_HUNTERS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ADTELLIGENT,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_DATADOG,                    PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_SEGMENTO_RU,                PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_POLYFILL_IO,                PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_B612,                       PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_WARPATH_ACE_SHOOTER,        PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TOWER_OF_SAVIORS,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_LEMON8,                     PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_COVEO,                      PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HYPIC,                      PACE2_PA_TAG_SOCIAL)\
        X(PACE2_APPLICATION_ESCAPE_FROM_TARKOV,         PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_POCKET,                     PACE2_PA_TAG_NEWS)\
        X(PACE2_APPLICATION_MARVEL_SNAP,                PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ADOBE_DOCUMENT_CLOUD,       PACE2_PA_TAG_BOOKS_MAGAZINES)\
        X(PACE2_APPLICATION_ZANGI_MESSENGER,            PACE2_PA_TAG_VOIP)\
        X(PACE2_APPLICATION_BEAMABLE,                   PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_BUNNY_NET,                  PACE2_PA_TAG_CLOUD_CDN_SERVICES)\
        X(PACE2_APPLICATION_BCE_SERVICES,               PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_CRAVE,                      PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_BYTEPLUS,                   PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_PLARIUM,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SPARTA_WAR_OF_EMPIRES,      PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_SBER,                       PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_JIVOCHAT,                   PACE2_PA_TAG_MESSAGING)\
        X(PACE2_APPLICATION_YOUTUBE_KIDS,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_OPPO_MOBILE_SERVICES,       PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_HEYTAP,                     PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES)\
        X(PACE2_APPLICATION_MAX,                        PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORLD_OF_WARSHIPS,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_YOUTUBE_TV,                 PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_GOOGLE_FAMILY_LINK,         PACE2_PA_TAG_DEVICE_SECURITY)\
        X(PACE2_APPLICATION_CELTIC_HEROES,              PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_HERO_HUNTERS,               PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TIMES_INTERNET_SERVICES,    PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_SERVICENOW,                 PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES)\
        X(PACE2_APPLICATION_HONKAI_STAR_RAIL,           PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_ELDER_SCROLLS_ONLINE,       PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_NUVERSE,                    PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TVPLUS,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_NASDAQ_SERVICES,            PACE2_PA_TAG_FINANCE)\
        X(PACE2_APPLICATION_SESISOFT_SERVICES,          PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_TIVIBU_GO,                  PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DUNGEON_FIGHTER_ONLINE,     PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_5GMARK,                     PACE2_PA_TAG_NETWORK_MANAGEMENT)\
        X(PACE2_APPLICATION_KILL_SHOT_BRAVO,            PACE2_PA_TAG_GAMING)\
        X(PACE2_APPLICATION_PUHUTV,                     PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_DOGUS_SERVICES,             PACE2_PA_TAG_ENTERPRISE_SERVICES)\
        X(PACE2_APPLICATION_OPENAI,                     PACE2_PA_TAG_EDUCATION)\
        X(PACE2_APPLICATION_QOBUZ,                      PACE2_PA_TAG_AUDIO_ENTERTAINMENT)\
        X(PACE2_APPLICATION_S_SPORT_PLUS,               PACE2_PA_TAG_STREAMING)\
        X(PACE2_APPLICATION_WORLD_OF_TANKS_BLITZ,       PACE2_PA_TAG_GAMING)

#define PACE2_PA_GENERATE_DEFAULT_TAG_ARRAY(pa, group)     group,

/**
 * List of strings, each describes the appropriate group.
 *
 * Items are subject to change.
 */
#define PACE2_PA_DEFAULT_TAG_DESCRIPTIONS(X) \
    X(PACE2_PA_TAG_GENERIC = PACE2_PA_TAGS_CUSTOM_COUNT,    General group for all not yet assigned protocols and applications.)\
    X(PACE2_PA_TAG_P2P,                                     Group for Peer 2 Peer related protocols and applications.)\
    X(PACE2_PA_TAG_GAMING,                                  Group for gaming related protocols and applications.)\
    X(PACE2_PA_TAG_TUNNEL,                                  Group for tunnel related protocols and applications.)\
    X(PACE2_PA_TAG_BUSINESS,                                Group for business related protocols and applications.)\
    X(PACE2_PA_TAG_E_COMMERCE,                              Group for E-Commerce related protocols and applications.)\
    X(PACE2_PA_TAG_VOIP,                                    Group for VoIP related protocols and applications.)\
    X(PACE2_PA_TAG_MESSAGING,                               Group for instant messaging related protocols and applications.)\
    X(PACE2_PA_TAG_STREAMING,                               Group for video-streaming related protocols and applications.)\
    X(PACE2_PA_TAG_MOBILE,                                  Group for mobile network related protocols and applications.)\
    X(PACE2_PA_TAG_REMOTE_CONTROL,                          Group for remote control related protocols and applications.)\
    X(PACE2_PA_TAG_MAIL,                                    Group for mail service related protocols and applications.)\
    X(PACE2_PA_TAG_NETWORK_MANAGEMENT,                      Group for network management related protocols and applications.)\
    X(PACE2_PA_TAG_DATABASE,                                Group for database related protocols and applications.)\
    X(PACE2_PA_TAG_FILETRANSFER,                            Group for file transfer related protocols and applications.)\
    X(PACE2_PA_TAG_WEB,                                     Group for web surfing related protocols and applications.)\
    X(PACE2_PA_TAG_CONFERENCE,                              Group for conferencing and team collaboration related protocols and applications.)\
    X(PACE2_PA_TAG_SOCIAL,                                  Group for social media related protocols and applications.)\
    X(PACE2_PA_TAG_SHAREHOSTING,                            Group for Sharehoster related protocols and applications.)\
    X(PACE2_PA_TAG_DEPRECATED,                              Group for all deprecated protocols and applications.)\
    X(PACE2_PA_TAG_INDUSTRIAL,                              Group for industrial related protocols and applications.)\
    X(PACE2_PA_TAG_ENCRYPTED,                               Group for encrypted protocols and applications.)\
    X(PACE2_PA_TAG_ADVERTISEMENT_ANALYTIC_SERVICES,         Group for protocols and applications which are related to advertisement and analytic services.)\
    X(PACE2_PA_TAG_NEWS,                                    Group for protocols and applications which are related to news reporting and online journalism.)\
    X(PACE2_PA_TAG_HEALTH_FITNESS,                          Group for protocols and applications which provide fitness and diet tracking as well as sports routines.)\
    X(PACE2_PA_TAG_CLOUD_CDN_SERVICES,                      Group for protocols and applications which provide cloud services or Content Delivery Networks.)\
    X(PACE2_PA_TAG_NAVIGATION,                              Group for protocols and applications which provide maps and navigation services as well as location information.)\
    X(PACE2_PA_TAG_FINANCE,                                 Group for protocols and applications which provide stock exchange and cryptocurrency information as well as online money transactions.)\
    X(PACE2_PA_TAG_TRAVEL_TRANSPORTATION,                   Group for protocols and applications which are related to travelling and transportation.)\
    X(PACE2_PA_TAG_PORNOGRAPHY,                             Group for protocols and applications which provide sexual subject content.)\
    X(PACE2_PA_TAG_BOOKS_MAGAZINES,                         Group for protocols and applications which allows the user to download and read ebooks, comics and magazines.)\
    X(PACE2_PA_TAG_AUDIO_ENTERTAINMENT,                     Group for protocols and applications which provide audiobooks, music, ringtones, podcasts or webradio.)\
    X(PACE2_PA_TAG_EDUCATION,                               Group for protocols and applications which provide educational content or reference.)\
    X(PACE2_PA_TAG_M2M_IOT,                                 Group for Machine to Machine and Internet of Things related protocols and applications.)\
    X(PACE2_PA_TAG_DEVICE_SECURITY,                         Group for protocols and applications which deliver Antivirus, Anti-Malware or firewall solutions.)\
    X(PACE2_PA_TAG_MULTIMEDIA_SERVICES,                     Group for protocols and applications which provide services like optimization, monetization, hosting or distribution for multimedia contents.)\
    X(PACE2_PA_TAG_ORGANIZERS,                              Group for protocols and applications which help users to organize documents, tasks, pieces of work, and schedules as well as passwords and accounts.)\
    X(PACE2_PA_TAG_ENTERPRISE_SERVICES,                     Group for protocols and applications which are used to provide all services related to a particular company.)\
    X(PACE2_PA_TAG_APPSTORES_UPDATES,                       Group for protocols and applications which cover traffic caused by App-Stores, digital distribution platforms for games, and OS Updates.)\
    X(PACE2_PA_TAG_BROWSERS,                                Group for protocols and applications that cover traffic produced by Webbrowsers.)\
    X(PACE2_PA_TAG_REMOVED,                                 Group for protocols and applications that are removed from the classification and documentation.)\
    X(PACE2_PA_TAG_MOVED,                                   Group for protocols and applications that were either renamed or taken over by another provider.)\
    X(PACE2_PA_TAG_WALLET,                                  Group for protocols and applications which provide wallet services for financial applications.)\
    X(PACE2_PA_TAG_CERTIFICATE_AUTHORITY,                   Group for protocols and applications which provide SSL and TLS verification.)\
    X(PACE2_PA_TAG_VR_AR,                                   Group for protocols and applications which are related to Virtual and Augmented Reality.)\
    X(PACE2_PA_TAG_SMART_HOME,                              Group for protocols and applications which provide home automation and security.)\
    X(PACE2_PA_TAG_PROFESSIONAL_DEV_TOOLS_SERVICES,         Group for protocols and applications which provide technologies and services for improving websites and applications with features like customer relationship management, traffic management or specific functionalities.)\
    X(PACE2_PA_TAG_SEARCH_ENGINE,                           Group for protocols and applications which are designed to carry out web searches.)

/**
 * @brief Sets the main tag of a specified protocol.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] proto_id                      ID of the protocol.
 * @param[in] main_tag                      Tag to assign to the protocol.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_OUT_OF_MEMORY              @copydoc PACE2_return_state::PACE2_OUT_OF_MEMORY
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 */
PACE2_return_state pace2_set_protocol_main_tag(PACE2_module * const pace2_module, PACE2_protocols const proto_id,
                                               u16 const main_tag);
/**
 * @brief Sets the main tag of a specified application.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @param[in] main_tag                      Tag to assign to the application.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_OUT_OF_MEMORY              @copydoc PACE2_return_state::PACE2_OUT_OF_MEMORY
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 */
PACE2_return_state pace2_set_application_main_tag(PACE2_module * const pace2_module, PACE2_applications const app_id,
                                                  u16 const main_tag);

/**
 * @brief Sets the sub tag of a specified application.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @param[in] sub_tag                       Tag to assign to the application.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_OUT_OF_MEMORY              @copydoc PACE2_return_state::PACE2_OUT_OF_MEMORY
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 */
PACE2_return_state pace2_set_application_sub_tag(PACE2_module * const pace2_module, PACE2_applications const app_id,
                                                 u16 const sub_tag);

/**
 * @brief Returns the main tag of a specified protocol or 0xFFFF if something is wrong.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] proto_id                      ID of the protocol.
 * @return                                  Main tag of the specified protocol or 0xFFFF for failure.
 */
u16 pace2_get_protocol_main_tag(PACE2_module * const pace2_module, PACE2_protocols const proto_id);

/**
 * @brief Returns the main tag of a specified application or 0xFFFF if something is wrong.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @return                                  Main tag of the specified application or 0xFFFF for failure.
 */
u16 pace2_get_application_main_tag(PACE2_module * const pace2_module, PACE2_applications const app_id);

/**
 * @brief Returns the sub tag of a specified application or 0xFFFF if something is wrong.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @return                                  Sub tag of the specified application or 0xFFFF for failure.
 */
u16 pace2_get_application_sub_tag(PACE2_module * const pace2_module, PACE2_applications const app_id);

/**
 * @brief Returns a string representation of a specified protocol's main tag.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] proto_id                      ID of the protocol.
 * @return                                  a string representation of the specified protocol's main tag.
 */
char const * pace2_get_protocol_main_tag_string(PACE2_module * const pace2_module, PACE2_protocols const proto_id);

/**
 * @brief Returns a string representation of a specified application's main tag.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @return                                  a string representation of the specified application's main tag.
 *                                          In case of invalid parameters NULL is returned, when no string is set for
 *                                          this ID this function returns an empty string.
 */
char const * pace2_get_application_main_tag_string(PACE2_module * const pace2_module, PACE2_applications const app_id);

/**
 * @brief Returns a string representation of a specified application's sub tag.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @return                                  a string representation of the specified application's sub tag.
 *                                          In case of invalid parameters NULL is returned, when no string is set for
 *                                          this ID this function returns an empty string.
 */
char const * pace2_get_application_sub_tag_string(PACE2_module * const pace2_module, PACE2_applications const app_id);

/**
 * @brief Returns a string representation of a specified tag.
 *
 * @deprecated Deprecated due to missing thread_ID parameter.
 *             Instead, use @ref pace2_get_pa_tag_str.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] tag                           ID of the tag.
 * @return                                  a string representation of the specified tag.
 */
PACE2_DEPRECATED("Please use 'pace2_get_pa_tag_str' instead.")
char const * pace2_get_pa_tag_string(PACE2_module * const pace2_module, u16 const tag);

/**
 * @brief Returns a string representation of a specified tag.
 *
 * This function supports DU and is equivalent to @ref pace2_get_pa_tag_string.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] thread_ID                     The ID of the thread currently processing.
 * @param[in] tag                           ID of the tag.
 * @return                                  a string representation of the specified tag.
 */
char const * pace2_get_pa_tag_str(PACE2_module const * const pace2_module, int const thread_ID, u16 const tag);

/**
 * @brief Returns a short form string representation of a specified tag.
 *
 * This function supports DU
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] thread_ID                     The ID of the thread currently processing.
 * @param[in] tag                           ID of the tag.
 * @return                                  a short form string representation of the specified tag.
 */
char const * pace2_get_pa_tag_short_string(PACE2_module const * const pace2_module, int const thread_ID, u16 const tag);

/**
 * @brief Adds a tag to a specified protocol.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] proto_id                      ID of the protocol.
 * @param[in] tag                           ID of the tag.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_INVALID_TAG                @copydoc PACE2_return_state::PACE2_INVALID_TAG
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 *                                          The given proto_id is bigger than the amount of protocol IDs.
 */
PACE2_return_state pace2_add_protocol_tag(PACE2_module * const pace2_module, PACE2_protocols const proto_id,
                                          u16 const tag);

/**
 * @brief Adds a tag to a specified application.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @param[in] tag                           ID of the tag.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_INVALID_TAG                @copydoc PACE2_return_state::PACE2_INVALID_TAG
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 *                                          The given app_id is bigger than the amount of application IDs.
 */
PACE2_return_state pace2_add_application_tag(PACE2_module * const pace2_module, PACE2_applications const app_id,
                                             u16 const tag);

/**
 * @brief Removes a tag from a specified protocol.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] proto_id                      ID of the protocol.
 * @param[in] tag                           ID of the tag.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_INVALID_TAG                @copydoc PACE2_return_state::PACE2_INVALID_TAG
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 *                                          The given proto_id is bigger than the amount of protocol IDs.
 */
PACE2_return_state pace2_remove_protocol_tag(PACE2_module * const pace2_module, PACE2_protocols const proto_id,
                                             u16 const tag);

/**
 * @brief Removes a tag from a specified application.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @param[in] tag                           ID of the tag.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_INVALID_TAG                @copydoc PACE2_return_state::PACE2_INVALID_TAG
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 *                                          The given proto_id is bigger than the amount of application IDs.
 */
PACE2_return_state pace2_remove_application_tag(PACE2_module * const pace2_module, PACE2_applications const app_id,
                                                u16 const tag);
/**
 * @brief Clears all tags of a specified protocol.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] proto_id                      ID of the protocol.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 *                                          The given proto_id is bigger than the amount of protocol IDs.
 */
PACE2_return_state pace2_clear_protocol_tags(PACE2_module * const pace2_module, PACE2_protocols const proto_id);

/**
 * @brief Clears all tags of a specified application.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @retval PACE2_SUCCESS                    @copydoc PACE2_return_state::PACE2_SUCCESS
 * @retval PACE2_PARAM_MISSING              @copydoc PACE2_return_state::PACE2_PARAM_MISSING
 * @retval PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS @copydoc PACE2_return_state::PACE2_INDEX_OUTSIDE_ARRAY_BOUNDS
 *                                          The given proto_id is bigger than the amount of application IDs.
 */
PACE2_return_state pace2_clear_application_tags(PACE2_module * const pace2_module, PACE2_applications const app_id);

/**
 * @brief Checks whether a specified protocol has a specific tag.
 *
 * This function supports DU. Old protocols keep their pre-DU tags, new ones are initialized
 * with default values
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] proto_id                      ID of the protocol.
 * @param[in] tag                           ID of the tag.
 * @retval IPQ_TRUE                         Tag is set for the protocol.
 * @retval IPQ_FALSE                        Tag is not set for the protocol.
 */
u8 pace2_protocol_has_tag(PACE2_module * const pace2_module, PACE2_protocols const proto_id, u16 const tag);

/**
 * @brief Checks whether a specified application has a specific tag.
 *
 * This function supports DU. Old applications keep their pre-DU tags, new ones are initialized
 * with default values
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] app_id                        ID of the application.
 * @param[in] tag                           ID of the tag.
 * @retval IPQ_TRUE                         Tag is set for the application.
 * @retval IPQ_FALSE                        Tag is not set for the application.
 */
u8 pace2_application_has_tag(PACE2_module * const pace2_module, PACE2_applications const app_id, u16 const tag);

/**
 * @brief Returns the description of a specified default tag.
 *
 * This function supports DU.
 *
 * @param[in] pace2_module                  Pointer to the pace 2 module.
 * @param[in] tag_id                        ID of the tag.
 * @return                                  a string representation of the tag description.
 *
 */
char const * pace2_get_pa_tag_description_string(PACE2_module * const pace2_module, u16 const tag_id);

#ifdef __cplusplus
}
#endif
#endif // PACE2_PA_TAG_FUNC_H

/*********************************************************************************
 ** EOF
 ********************************************************************************/
